/*===========================================================================

FILE: addrbook.c

SERVICES: Sample applet illustrating use of BREW Address Book Interface

DESCRIPTION
  This file contains usage examples of IAddrBook and IAddrRec interfaces in BREW

PUBLIC CLASSES:  
   N/A


   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/

/*===============================================================================
                     INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "AEEModGen.h"
#include "AEEAppGen.h"
#include "AEEMenu.h"
#include "AEEStdLib.h"
#include "AEEAddrBook.h"
#include "addrbook.bid"

/*===========================================================================

                      PUBLIC DATA DECLARATIONS

===========================================================================*/

/*-------------------------------------------------------------------
            Type Declarations
-------------------------------------------------------------------*/
// AddrBook app struct. This is the main struct for this applet. This will hold 
// all the data members that needs to be remembered throughout the life of
// the applet. 
// THE FIRST DATA MEMBER OF THIS STRUCT MUST BE AN AEEApplet OBJECT. 
// This is critical requirement which is related to an applet being
// dynamically loaded. The app developers should add their data members
// following the AEEApplet data member.
typedef struct _CAddrBkApp
{
	AEEApplet	a;        // Mandatory first AEEApplet data member
   IMenuCtl *  m_pIMenu;

   // Device parameters
   int             m_nLineHeight;
   AEEDeviceInfo   m_dInfo;

}CAddrBkApp;

/*-------------------------------------------------------------------
            Function Prototypes
-------------------------------------------------------------------*/
// App Handle Event function
static boolean AddrBkApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);

// App specific data alloc-init/free functions
static boolean AddrBk_InitAppData(IApplet* pMe);
static void AddrBk_FreeAppData(IApplet* pMe);

static void BuildMainMenu(CAddrBkApp *pMe);
static boolean AddrBkUsage (CAddrBkApp * pMe, uint16 wParam);
static void DisplayEvent (CAddrBkApp *pMe, uint16 wParam);
static void DisplayOutput(CAddrBkApp * pMe, int nline, char *pszStr);

static int  AddrBkTest(CAddrBkApp *pme);

/*-------------------------------------------------------------------
            Global Constant Definitions
-------------------------------------------------------------------*/
// App Resource File
#define  APP_RES_FILE      "addrbkuse.bar"

// App specific constants
#define ADDRBK_CREATE_REC           101
#define ADDRBK_ADD_FIELD            102
#define ADDRBK_REMOVE_FIELD         103
#define ADDRBK_UPDATE_FIELD         104
#define ADDRBK_UPDATE_ALL_FIELDS    105
#define ADDRBK_REMOVE_REC           106 
#define ADDRBK_GET_FIELD_COUNT      107
#define ADDRBK_COUNT_RECS           108
#define ADDRBK_ENUM_CATEGORIES      109
#define ADDRBK_ENUM_FIELDS          110





/*===============================================================================
                     FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to IDB_InitAppData(). 
   This is done as follows: InitAppData() is called to initialize AppletData 
   instance. It is app developers responsibility to fill-in app data initialization 
   code of InitAppData(). App developer is also responsible to release memory 
   allocated for data contained in AppletData -- this can be done in 
   IDB_FreeAppData().

PROTOTYPE:
   int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell interface. 

	pIModule: pin]: Contains pointer to the IModule interface to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid IApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the app needs to be loaded and if AEEApplet_New() invocation was
     successful
  EFAILED: If the app does not need to be loaded or if errors occurred in 
     AEEApplet_New(). If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
{
   *ppObj = NULL;
		
   // We want to load this App. So, invoke AEEApplet_New().To it, pass the
   // address of the app-specific handle event function.
   if(ClsId == AEECLSID_ADDRBOOK_BID)
   {
      if(AEEApplet_New(sizeof(CAddrBkApp), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)AddrBkApp_HandleEvent,(PFNFREEAPPDATA)AddrBk_FreeAppData)
         == TRUE)
      {
         if (AddrBk_InitAppData((IApplet*)*ppObj) == TRUE)
         {
            return(AEE_SUCCESS);
         }
      }
   }
	return (EFAILED);
}

/*===========================================================================

FUNCTION AddrBkApp_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean AddrBkApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

	ecode: Specifies the Event sent to this applet

   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/
static boolean AddrBkApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
   CAddrBkApp * pMe = (CAddrBkApp*)pi;

   switch (eCode) 
	{
      case EVT_APP_START:
         // Create the IMenu interface object. When the applet is started the 
         // main menu is set up. When the user selects a menu item from the main 
         // menu, an EVT_COMMAND event is received by this applet. The processing for 
         // EVT_COMMAND event is found below.
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, (void **)&pMe->m_pIMenu)
            != SUCCESS)
         {
				return FALSE;
         }

         // Build the main menu
			BuildMainMenu(pMe);
			return(TRUE);
      case EVT_APP_STOP:
         return(TRUE);
      case EVT_KEY: 
         // We come here when a key is pressed on the handset keypad. Call 
         // the IMenuCtl handle event function to handle this key press.
         // If this key press corressponds to an Up/Down arrow 
         // key press, this event will be handled by the IMenuCtl object
         // in moving to the next menu item (in the given direction). 
         // If the key is neither Up/Down, the IMenuCtl the key press 
         // is not handled, hence, return FALSE.
			if(pMe->m_pIMenu != NULL && IMENUCTL_HandleEvent(pMe->m_pIMenu, EVT_KEY, wParam, 0))
				return TRUE;
         else
            return FALSE;
		case EVT_COMMAND:
         // When a menu item is selected this applet receives the
         // EVT_COMMAND event in the wParam field.
          return AddrBkUsage (pMe, wParam);
      default:
         break;
   }
   return FALSE;
}



/*===========================================================================

FUNCTION AddrBk_InitAppData

DESCRIPTION
   This function initializes app specific data allocates memory for app data 
   (AppletData) and sets it to pAppData of AEEApplet. It also function . 
   App developer needs to fill-in the initialization code.

PROTOTYPE:
	boolean AddrBk_InitAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains 
   information specific to this applet. It was initialized during the 
   AEEClsCreateInstance().

DEPENDENCIES
   Assumes pi is not NULL

RETURN VALUE
  TRUE: If the app has app data is allocated and initialized successfully
  FALSE: Either app data could not be allocated or initialzied

SIDE EFFECTS
  none
===========================================================================*/
static boolean AddrBk_InitAppData(IApplet* pi)
{
   int pnAscent;        // Stores the ascent in number of pixels
   int pnDescent;       // Stores the descent in number of pixels

   CAddrBkApp * pMe = (CAddrBkApp*)pi;

   // Initialize the MenuCtl pointer to NULL
   pMe->m_pIMenu = NULL;

   // Get the font metrics info
   pMe->m_nLineHeight = IDISPLAY_GetFontMetrics (pMe->a.m_pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&pMe->m_dInfo);

   return TRUE;
}

/*===========================================================================

FUNCTION AddrBk_FreeAppData

DESCRIPTION
   Frees data contained in app data and memory for app data itself.
   App developer needs to free data contained in AppletData.

PROTOTYPE:
	void AddrBk_FreeAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains 
   information specific to this applet. It was initialized during the 
   AEEClsCreateInstance().

DEPENDENCIES
   Assumes pi is not NULL

RETURN VALUE
   None

SIDE EFFECTS
   None
===========================================================================*/
static void AddrBk_FreeAppData(IApplet* pi)
{
   CAddrBkApp * pMe = (CAddrBkApp*)pi;

   // Release main menu
   if (pMe->m_pIMenu != NULL)
   {
      IMENUCTL_Release (pMe->m_pIMenu);
      pMe->m_pIMenu = NULL;
   }   
}

/*===============================================================================
                     APP HELPER FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: AddrBkUsage

DESCRIPTION
	This function encompasses all the usage examples of all the functions
   in code blocks switched using the BREW API function Id passed into this
   function. The code blocks will include almost all the variable declarations
   needed to demonstrate the usage of a given function. Within the code block
   all the needed interfaces are instantiated and released subsequent to their
   use.

   There are some exceptions to the above general rule, such as call-back functions,
   common helper function, or common display functions called from within the
   code block which are defined elsewhere in this file.

   There are some instances where an object instantiated within a code block is
   not released at the end of the code block, to be released at a later point
   (for example in a call-back function, in this case pointers to such objects
   are stored in the AEEApplet object to be released later).
   
PROTOTYPE:
   void AddrBkUsage (CAddrBkApp * pMe, uint16 wParam)

PARAMETERS:
	pMe: [in]: Contains a pointer to the CAddrBkApp struct.
   wParam: [in]: Contains the Id of the function type to be used (contains the
      Id corresponding to the menu item selected on the emulator).

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the functions call was successful. FALSE otherwise

SIDE EFFECTS
  none
===========================================================================*/
static boolean AddrBkUsage (CAddrBkApp * pme, uint16 wParam)
{
   IShell *pIShell;
   char *szWorkNum,*szHomeNum, *szName, *szEmail, *szURL, *szAddr,*szNotes;
   AECHAR *aszWorkNum,*aszHomeNum, *aszName, *aszEmail, *aszURL, *aszAddr,*aszNotes;
   AEEAddrField field[7];
   boolean bRet = TRUE;
   uint16   nSize;   


   szWorkNum = (char *)MALLOC(20);
   szHomeNum = (char *)MALLOC(20);
   szName = (char *)MALLOC(30);
   szEmail = (char *)MALLOC(30);
   szURL = (char *)MALLOC(30);
   szAddr = (char *)MALLOC(30);
   szNotes = (char *)MALLOC(30);

   aszWorkNum = (AECHAR *)MALLOC(40);
   aszHomeNum = (AECHAR *)MALLOC(40);
   aszName = (AECHAR *)MALLOC(60);
   aszEmail = (AECHAR *)MALLOC(60);
   aszURL = (AECHAR *)MALLOC(60);
   aszAddr = (AECHAR *)MALLOC(60);
   aszNotes = (AECHAR *)MALLOC(60);

   if(!szWorkNum || !szHomeNum ||  !szName ||  !szEmail ||  !szURL ||  !szAddr || !szNotes || 
      !aszWorkNum || !aszHomeNum ||  !aszName ||  !aszEmail ||  !aszURL ||  !aszAddr || 
      !aszNotes)
   {
      return(FALSE);
   }


   pIShell = pme->a.m_pIShell;

   // Erase screen first for display output purposes.
   IDISPLAY_ClearScreen (pme->a.m_pIDisplay);

   switch (wParam)
   {


      case ADDRBK_CREATE_REC:
      {

           IAddrBook *pAddrBook;

           //Work Num
           field[0].fID = AEE_ADDRFIELD_PHONE_WORK;
           field[0].fType = AEEDB_FT_STRING;
           STRCPY(szWorkNum,"8589996666");
           STRTOWSTR(szWorkNum,aszWorkNum,40);
           field[0].pBuffer= aszWorkNum;
           field[0].wDataLen = (WSTRLEN(aszWorkNum)+1)*sizeof(AECHAR);

           //HOME Num
           field[1].fID = AEE_ADDRFIELD_PHONE_HOME;
           field[1].fType = AEEDB_FT_STRING;
           STRCPY(szHomeNum,"8581112222");
           STRTOWSTR(szHomeNum,aszHomeNum,40);
           field[1].pBuffer= aszHomeNum;
           field[1].wDataLen = (WSTRLEN(aszHomeNum)+1)*sizeof(AECHAR);

           //Name
           field[2].fID = AEE_ADDRFIELD_NAME;
           field[2].fType = AEEDB_FT_STRING;
           STRCPY(szName,"My BREW");
           STRTOWSTR(szName,aszName,60);
           field[2].pBuffer= aszName;
           field[2].wDataLen = (WSTRLEN(aszName)+1)*sizeof(AECHAR);

           //Email
           field[3].fID = AEE_ADDRFIELD_EMAIL;
           field[3].fType = AEEDB_FT_STRING;
           STRCPY(szEmail,"me@brew.com");
           STRTOWSTR(szEmail,aszEmail,60);
           field[3].pBuffer= aszEmail;
           field[3].wDataLen = (WSTRLEN(aszEmail)+1)*sizeof(AECHAR);


           //URL
           field[4].fID = AEE_ADDRFIELD_URL;
           field[4].fType = AEEDB_FT_STRING;
           STRCPY(szURL,"www.brew.com");
           STRTOWSTR(szURL,aszURL,60);
           field[4].pBuffer= aszURL;
           field[4].wDataLen = (WSTRLEN(aszURL)+1)*sizeof(AECHAR);

           //Address
           field[5].fID = AEE_ADDRFIELD_ADDRESS;
           field[5].fType = AEEDB_FT_STRING;
           STRCPY(szAddr,"21, Park Avenue");
           STRTOWSTR(szAddr,aszAddr,60);
           field[5].pBuffer= aszAddr;
           field[5].wDataLen = (WSTRLEN(aszAddr)+1)*sizeof(AECHAR);


           //Notes
           field[6].fID = AEE_ADDRFIELD_NOTES;
           field[6].fType = AEEDB_FT_STRING;
           STRCPY(szNotes,"Test Notes.Hoping.");
           STRTOWSTR(szNotes,aszNotes,60);
           field[6].pBuffer= aszNotes;
           field[6].wDataLen = (WSTRLEN(aszNotes)+1)*sizeof(AECHAR);



           if(!ISHELL_CreateInstance(pme->a.m_pIShell, AEECLSID_ADDRBOOK, (void **)&pAddrBook))
           {
               IAddrRec *pRec;

               pRec = IADDRBOOK_CreateRec(pAddrBook,AEE_ADDR_CAT_PERSONAL, (AEEAddrField*)field, 7);

               if(pRec)
               {
                  DisplayOutput (pme, -1, "MyBREW Record Create Successful");
                  IADDRREC_Release(pRec);
               }
               else
                  DisplayOutput (pme, -1, "MyBREW Record Create Failed");


               IADDRBOOK_Release(pAddrBook);
           }
           else
               DisplayOutput (pme, -1, "Could not create IAddrBook");

         }
         break;




      case ADDRBK_ADD_FIELD:
         {

            //Add Field. Add field to the MyBREW record

            IAddrRec *pR;
            IAddrBook *pb;
            int nRet;

            STRCPY(szName,"My BREW");
            STRTOWSTR(szName,aszName,60);


            if(!ISHELL_CreateInstance(pme->a.m_pIShell, AEECLSID_ADDRBOOK, (void **)&pb))
            {
            
               nSize = (WSTRLEN(aszName)+1) *sizeof(AECHAR);

               nRet = IADDRBOOK_EnumRecInit(pb,  AEE_ADDR_CAT_NONE, AEE_ADDRFIELD_NAME, (void *)aszName, nSize);
               if(nRet == AEE_SUCCESS)
               {
                  pR = IADDRBOOK_EnumNextRec(pb);
                  if(pR)
                  {

                    //Fax Num
                    field[0].fID = AEE_ADDRFIELD_PHONE_FAX;
                    field[0].fType = AEEDB_FT_STRING;
                    STRCPY(szHomeNum,"9998886666");
                    STRTOWSTR(szHomeNum,aszHomeNum,40);
                    field[0].pBuffer= aszHomeNum;
                    field[0].wDataLen = (WSTRLEN(aszHomeNum)+1)*sizeof(AECHAR);
                  
                    if(IADDRREC_AddField(pR,(AEEAddrField*)(&field[0])) == AEE_SUCCESS)
                        DisplayOutput (pme, -1, "Added FaxNum to MyBrew");
                    else
                        DisplayOutput (pme, -1, "Failed to Add FaxNum field to MyBrew");

                    IADDRREC_Release(pR);
                  }
                  else
                     DisplayOutput (pme, -1, "Failed to Locate MyBrew");
               }
               else
                  DisplayOutput (pme, -1, "Failed to Initialize Search for MyBrew");

               IADDRBOOK_Release(pb);
            }
            else
               DisplayOutput (pme, -1, "Could not create IAddrBook");
         }
         break;

      case ADDRBK_REMOVE_FIELD:
      {

            //Add Field. Remove field from the MyBREW record

            IAddrRec *pR;
            IAddrBook *pb;
            int nRet,nCount;

            STRCPY(szName,"My BREW");
            STRTOWSTR(szName,aszName,60);

            if(!ISHELL_CreateInstance(pme->a.m_pIShell, AEECLSID_ADDRBOOK, (void **)&pb))
            {
            
               nSize = (WSTRLEN(aszName)+1) *sizeof(AECHAR);
               nRet = IADDRBOOK_EnumRecInit(pb,  AEE_ADDR_CAT_NONE, AEE_ADDRFIELD_NAME, (void *)aszName, nSize);
               if(nRet == AEE_SUCCESS)
               {
                  pR = IADDRBOOK_EnumNextRec(pb);
                  if(pR)
                  {

                     nCount = IADDRREC_GetFieldCount(pR);

                     if(IADDRREC_RemoveField(pR,nCount-1) == AEE_SUCCESS)
                        DisplayOutput (pme, -1, "Removed last field from MyBrew");
                    else
                        DisplayOutput (pme, -1, "Failed to Remove field from MyBrew");

                    IADDRREC_Release(pR);
                  }
                  else
                     DisplayOutput (pme, -1, "Failed to Locate MyBrew");
               }
               else
                  DisplayOutput (pme, -1, "Failed to Initialize Search for MyBrew");

               IADDRBOOK_Release(pb);
            }
            else
               DisplayOutput (pme, -1, "Could not create IAddrBook");

      }
      break;

      case ADDRBK_UPDATE_FIELD:
      {
            //Update field in the MyBREW record

            IAddrRec *pR;
            IAddrBook *pb;
            int nRet;

            STRCPY(szName,"My BREW");
            STRTOWSTR(szName,aszName,60);

            if(!ISHELL_CreateInstance(pme->a.m_pIShell, AEECLSID_ADDRBOOK, (void **)&pb))
            {
            
               nSize = (WSTRLEN(aszName)+1) *sizeof(AECHAR);
               nRet = IADDRBOOK_EnumRecInit(pb,  AEE_ADDR_CAT_NONE, AEE_ADDRFIELD_NAME, (void *)aszName, nSize);
               if(nRet == AEE_SUCCESS)
               {
                  pR = IADDRBOOK_EnumNextRec(pb);
                  if(pR)
                  {

                     //Change Work Phone Number
                     AEEAddrField newF, *ptr = NULL;
                     int i,nFields = IADDRREC_GetFieldCount(pR);


                     for(i = 0; i < nFields; i++)
                     {
                        ptr = IADDRREC_GetField(pR,i);
                        if(ptr->fID == AEE_ADDRFIELD_PHONE_WORK)
                           break;
                     }
                     if(ptr)
                     {
                        newF.fID = ptr->fID;
                        newF.fType = ptr->fType;
                        STRCPY(szWorkNum,"9142356789");
                        STRTOWSTR(szWorkNum,aszWorkNum,40);
                        newF.pBuffer = aszWorkNum;
                        newF.wDataLen = (WSTRLEN(aszWorkNum)+1)*sizeof(AECHAR);
                        
                        if(IADDRREC_UpdateField(pR,i,&newF) == AEE_SUCCESS)
                           DisplayOutput (pme, -1, "Updated WorkNum in MyBrew");
                        else
                           DisplayOutput (pme, -1, "Failed to Update WorkNum in MyBrew");
                     }
                     else
                        DisplayOutput (pme, -1, "Failed to Locate WorkNum in MyBrew");

                    IADDRREC_Release(pR);
                  }
                  else
                     DisplayOutput (pme, -1, "Failed to Locate MyBrew");
               }
               else
                  DisplayOutput (pme, -1, "Failed to Initialize Search for MyBrew");

               IADDRBOOK_Release(pb);
            }
            else
               DisplayOutput (pme, -1, "Could not create IAddrBook");

      }
      break;

      case ADDRBK_UPDATE_ALL_FIELDS:
      {
            //Update all fields in the MyBREW record

            IAddrRec *pR;
            IAddrBook *pb;
            int nRet;

            STRCPY(szName,"My BREW");
            STRTOWSTR(szName,aszName,60);

            if(!ISHELL_CreateInstance(pme->a.m_pIShell, AEECLSID_ADDRBOOK, (void **)&pb))
            {
            
               nSize = (WSTRLEN(aszName)+1) *sizeof(AECHAR);
               nRet = IADDRBOOK_EnumRecInit(pb,  AEE_ADDR_CAT_NONE, AEE_ADDRFIELD_NAME, (void *)aszName, nSize);
               if(nRet == AEE_SUCCESS)
               {
                  pR = IADDRBOOK_EnumNextRec(pb);
                  if(pR)
                  {

                    //Work Num
                    field[0].fID = AEE_ADDRFIELD_PHONE_WORK;
                    field[0].fType = AEEDB_FT_STRING;
                    STRCPY(szWorkNum,"1111111111");
                    STRTOWSTR(szWorkNum,aszWorkNum,40);
                    field[0].pBuffer= aszWorkNum;
                    field[0].wDataLen = (WSTRLEN(aszWorkNum)+1)*sizeof(AECHAR);

                    //HOME Num
                    field[1].fID = AEE_ADDRFIELD_PHONE_HOME;
                    field[1].fType = AEEDB_FT_STRING;
                    STRCPY(szHomeNum,"2222222222");
                    STRTOWSTR(szHomeNum,aszHomeNum,40);
                    field[1].pBuffer= aszHomeNum;
                    field[1].wDataLen = (WSTRLEN(aszHomeNum)+1)*sizeof(AECHAR);

                    //Name
                    field[2].fID = AEE_ADDRFIELD_NAME;
                    field[2].fType = AEEDB_FT_STRING;
                    STRCPY(szName,"My BREW");
                    STRTOWSTR(szName,aszName,60);
                    field[2].pBuffer= aszName;
                    field[2].wDataLen = (WSTRLEN(aszName)+1)*sizeof(AECHAR);

                    //Email
                    field[3].fID = AEE_ADDRFIELD_EMAIL;
                    field[3].fType = AEEDB_FT_STRING;
                    STRCPY(szEmail,"new@new.com");
                    STRTOWSTR(szEmail,aszEmail,60);
                    field[3].pBuffer= aszEmail;
                    field[3].wDataLen = (WSTRLEN(aszEmail)+1)*sizeof(AECHAR);
                        
                    if(IADDRREC_UpdateAllFields(pR,field,4) == AEE_SUCCESS)
                      DisplayOutput (pme, -1, "Updated all fields in MyBrew");
                    else
                      DisplayOutput (pme, -1, "Failed to Update all fields in MyBrew");

                    IADDRREC_Release(pR);
                  }
                  else
                   DisplayOutput (pme, -1, "Failed to Locate MyBrew");
                 
                }
                else
                  DisplayOutput (pme, -1, "Failed to Initialize Search for MyBrew");

               IADDRBOOK_Release(pb);
            }
            else
               DisplayOutput (pme, -1, "Could not create IAddrBook");

      }
      break;

      case ADDRBK_REMOVE_REC:
      {
            //Remove MyBREW record

            IAddrRec *pR;
            IAddrBook *pb;
            int nRet;

            STRCPY(szName,"My BREW");
            STRTOWSTR(szName,aszName,60);

            if(!ISHELL_CreateInstance(pme->a.m_pIShell, AEECLSID_ADDRBOOK, (void **)&pb))
            {
            
               nSize = (WSTRLEN(aszName)+1) *sizeof(AECHAR);
               nRet = IADDRBOOK_EnumRecInit(pb,  AEE_ADDR_CAT_NONE, AEE_ADDRFIELD_NAME, (void *)aszName, nSize);
               if(nRet == AEE_SUCCESS)
               {
                  pR = IADDRBOOK_EnumNextRec(pb);
                  if(pR)
                  {

                     if(IADDRREC_RemoveRec(pR) == AEE_SUCCESS)
                        DisplayOutput (pme, -1, "MyBrew Removed");
                     else
                        DisplayOutput (pme, -1, "Failed to remove MyBrew");
                  }
                  else
                     DisplayOutput (pme, -1, "Failed to Locate MyBrew");
               }
               else
                  DisplayOutput (pme, -1, "Failed to Initialize Search for MyBrew");

               IADDRBOOK_Release(pb);
            }
            else
               DisplayOutput (pme, -1, "Could not create IAddrBook");

      }
      break;

      case ADDRBK_GET_FIELD_COUNT:
      {
            //Get Field Count

            IAddrRec *pR;
            IAddrBook *pb;
            int nRet,nCount;

            STRCPY(szName,"My BREW");
            STRTOWSTR(szName,aszName,60);

            if(!ISHELL_CreateInstance(pme->a.m_pIShell, AEECLSID_ADDRBOOK, (void **)&pb))
            {
            
               nSize = (WSTRLEN(aszName)+1) *sizeof(AECHAR);
               nRet = IADDRBOOK_EnumRecInit(pb,  AEE_ADDR_CAT_NONE, AEE_ADDRFIELD_NAME, (void *)aszName, nSize);
               if(nRet == AEE_SUCCESS)
               {
                  pR = IADDRBOOK_EnumNextRec(pb);
                  if(pR)
                  {

                     nCount = IADDRREC_GetFieldCount(pR);
                     SPRINTF(szName,"MyBREW has %d fields",nCount);

                     DisplayOutput (pme, -1, szName);
                     IADDRREC_Release(pR);
                  }
                  else
                     DisplayOutput (pme, -1, "Failed to Locate MyBrew");
               }
               else
                  DisplayOutput (pme, -1, "Failed to Initialize Search for MyBrew");

               IADDRBOOK_Release(pb);
            }
            else
               DisplayOutput (pme, -1, "Could not create IAddrBook");

      }
      break;

      case ADDRBK_COUNT_RECS:
      {
            //Count Recs in phone book

            IAddrBook *pb;
            int nCount;


            if(!ISHELL_CreateInstance(pme->a.m_pIShell, AEECLSID_ADDRBOOK, (void **)&pb))
            {
            
               nCount = IADDRBOOK_GetNumRecs(pb);

               SPRINTF(szName,"Address Book has %d records",nCount);

               DisplayOutput (pme, -1, szName);

               IADDRBOOK_Release(pb);
            }
            else
               DisplayOutput (pme, -1, "Could not create IAddrBook");

      }
      break;

      case ADDRBK_ENUM_CATEGORIES:
      {
            //Enum Categories

            IAddrBook *pb;
            int nCount;


            if(!ISHELL_CreateInstance(pme->a.m_pIShell, AEECLSID_ADDRBOOK, (void **)&pb))
            {
            
                  if(IADDRBOOK_EnumCategoryInit(pb) == AEE_SUCCESS)
                  {
                     AEEAddrCat cat;
                     nCount = 0;
                     while(IADDRBOOK_EnumNextCategory(pb,&cat))
                        nCount++;
      
                     SPRINTF(szName,"%d Address Categories supported",nCount);

                     DisplayOutput (pme, -1, szName);
                  }
                  else
                     DisplayOutput (pme, -1, "Error EnumCategoryInit");
      
                  IADDRBOOK_Release(pb);
            }
            else
               DisplayOutput (pme, -1, "Could not create IAddrBook");

      }
      break;

      case ADDRBK_ENUM_FIELDS:
      {
            //Enum Fields

            IAddrBook *pb;
            int nCount;


            if(!ISHELL_CreateInstance(pme->a.m_pIShell, AEECLSID_ADDRBOOK, (void **)&pb))
            {
            
                  if(IADDRBOOK_EnumFieldsInfoInit(pb,AEE_ADDR_CAT_PERSONAL) == AEE_SUCCESS)
                  {
                     AEEAddrFieldInfo fInfo;
                     nCount = 0;
                     while(IADDRBOOK_EnumNextFieldsInfo(pb,&fInfo))
                        nCount++;
      
                     SPRINTF(szName,"%d Address Fields supported",nCount);

                     DisplayOutput (pme, -1, szName);
                  }
                  else
                     DisplayOutput (pme, -1, "Error EnumFieldsInfoInit");
      
                  IADDRBOOK_Release(pb);
            }
            else
               DisplayOutput (pme, -1, "Could not create IAddrBook");

      }
      break;

      default:
         bRet = FALSE;
   }
   
   // Display above event. 
   DisplayEvent (pme, wParam);

   //Cleanup
   FREE(szWorkNum);
   FREE(szHomeNum);
   FREE(szName); 
   FREE(szEmail); 
   FREE(szURL); 
   FREE(szAddr);
   FREE(szNotes);
   FREE(aszWorkNum);
   FREE(aszHomeNum); 
   FREE(aszName); 
   FREE(aszEmail); 
   FREE(aszURL); 
   FREE(aszAddr);
   FREE(aszNotes);

   return (bRet);
}

/*===========================================================================

FUNCTION: DisplayEvent

DESCRIPTION
    This function is a display function which sets up the initial screen
    for the display output for a given interface function. This displays
    the function name at the top of the screen and displays the
    strings "Press Up/Down" and "keys to continue.." at the bottom of 
    the screen. In the case of functions that have no output it also 
    displays the string "No Output..." in the middile of the screen.

PROTOTYPE:
   void DisplayEvent (CAddrBkApp *pMe, uint16 wParam)

PARAMETERS:
   pMe:    [in]: Pointer to CAddrBkApp interface object
   wParam: [in]: API function type Id selected by user

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void DisplayEvent (CAddrBkApp *pMe, uint16 wParam)
{
   AECHAR szBuf[100];
   AEERect rc;

   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
      return;

   // Set display rectangle
   SETAEERECT (&rc, 0, 0, pMe->m_dInfo.cxScreen, pMe->m_dInfo.cyScreen);

   if (pMe->a.m_pIDisplay)
   {

      STR_TO_WSTR("Press Up/Down", szBuf, sizeof(szBuf));
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,5, rc.dy - 
         2*pMe->m_nLineHeight, &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
      STR_TO_WSTR("keys to continue..", szBuf, sizeof(szBuf));
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,5, rc.dy - 
         pMe->m_nLineHeight, &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
		IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);
   }
}

/*===========================================================================

FUNCTION: BuildMainMenu

DESCRIPTION
    This function builds the main menu when this applet is started.
    It displays a simple menu with menu items corresponding to 
    each of the interface functions.
    
PROTOTYPE:
   void BuildMainMenu(CAddrBkApp *pMe)

PARAMETERS:
   pMe: [in]: Pointer to AEEApplet struct

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void BuildMainMenu(CAddrBkApp *pMe)
{
	AEERect rc;
   AECHAR szBuf[40];

   if (pMe->m_pIMenu == NULL || pMe->a.m_pIShell == NULL)
      return;

	// Set Title
   STR_TO_WSTR("AddrBook Examples", szBuf, sizeof(szBuf));
	IMENUCTL_SetTitle(pMe->m_pIMenu, NULL, 0, szBuf);

	//Set the Rectangle for the Menu
   SETAEERECT (&rc, 0, 0, pMe->m_dInfo.cxScreen, pMe->m_dInfo.cyScreen);

	IMENUCTL_SetRect(pMe->m_pIMenu, &rc);  

	//Add invidual entries to the Menu


   STR_TO_WSTR("Create Record", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, ADDRBK_CREATE_REC, szBuf, 0);


   STR_TO_WSTR("Add Field", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, ADDRBK_ADD_FIELD, szBuf, 0);

   
   STR_TO_WSTR("Remove Field", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, ADDRBK_REMOVE_FIELD, szBuf, 0);
			
   STR_TO_WSTR("Update Field", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, ADDRBK_UPDATE_FIELD, szBuf, 0);

   
   STR_TO_WSTR("Update All Fields", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, ADDRBK_UPDATE_ALL_FIELDS, szBuf, 0);

   STR_TO_WSTR("Get Field Count", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, ADDRBK_GET_FIELD_COUNT, szBuf, 0);

   STR_TO_WSTR("Remove Record", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, ADDRBK_REMOVE_REC, szBuf, 0);


   STR_TO_WSTR("Count Records", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, ADDRBK_COUNT_RECS, szBuf, 0);	


   STR_TO_WSTR("Enum Categories", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, ADDRBK_ENUM_CATEGORIES, szBuf, 0);	

   
   STR_TO_WSTR("Enum Fields", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, ADDRBK_ENUM_FIELDS, szBuf, 0);	


   // Activate menu
	IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);
}

/*===========================================================================

FUNCTION: DisplayOutput

DESCRIPTION
    This function displays an output string at a given line number on the
    screen. If the nline parameter is a negative value (-1) the string
    is displayed in the middle of the screen. If the "nline" value is larger
    than or equal to zero the "nline" value is multiplied by 15 and the 
    resulting value in pixels is set to the y-coordinate of the start of 
    the string display on the screen. If the string does not fit on one line
    the string wraps around to the next line (spaced rougly 10-15 pixels apart).
    By default 5 is used as the starting the x-coordinate of a displayed 
    string.

    How many characters that fit on one line is calculated for each line 
    that is wrapped around to the next line.

    Note: depending on the phone screen size and the fonts used for characters 
          the output might differ on different handsets (devices). Where some 
          handsets will have a smaller screen and large default fonts which will 
          cause partial overlapping of lines. This function does not try to address
          these issues (this is meant as a simple display function).
    
PROTOTYPE:
   void DisplayOutput(CAddrBkApp * pMe, int nline, char *pszStr)

PARAMETERS:
   pMe:     [in]: Contains a pointer to the CAddrBkApp class.
   nline:  [in]: Contains the line number to start displaying the text. 
        The line numbers are by default spaced 15 pixels apart along the y-axis.
   pszStr: [in]: The character string to be displayed on the screen.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void DisplayOutput(CAddrBkApp * pMe, int nline, char *pszStr)
{
   AECHAR * pszBuf = NULL;
   AECHAR * psz = NULL;
   int pixelWidth;
   AEEFont font = AEE_FONT_NORMAL;
   int pnFits = 0, dy;
   int totalCh = 0;

   if (pMe == NULL)
      return;

   // Allocate buffer to hold string.
   if ((pszBuf = (AECHAR *)MALLOC(200)) == NULL)
      return;

   // Convert to wide string (unicode)
   STR_TO_WSTR ((char *)pszStr, pszBuf, 200);

   // If nlines is zero then print this string starting around the middle of 
   // the screen. Or else multiply nlines by 10 to decide the y coordinate of
   // the start of the string.
   if (nline < 0) {
      dy = pMe->m_dInfo.cyScreen*2/5;
   }
   else{
      dy = nline * pMe->m_nLineHeight;
   }

   // psz keeps track of the point from which to write from the string buffer
   // in case the string does not fit one line and needs to wrap around in the
   // next line.
   psz = pszBuf;
      
   // Need to calculate the lotal string length to decide if any wrapping
   // around is needed.
   totalCh = STRLEN ((char *)pszStr);

   // Keep displaying text string on multiple lines if the string can't be displayed
   // on one single line. Lines are spaced 15 pixels apart.
   while ((totalCh > 0) && (*psz != NULL))
   { 
      // Get information on how many characters will fit in a line.
      // Give the pointer to the buffer to be displayed, and the number of
      // pixels along the x axis you want to display the string in (max number)
      // pnFits will have the max number of chars that will fit in the maxWidth
      // number of pixels (given string can't fit in one line), or the number of 
      // chars in the string (if it does fit in one line). pnWidth gives the
      // number of pixels that will be used to display pnFits number of chars.
      pixelWidth = IDISPLAY_MeasureTextEx(pMe->a.m_pIDisplay,
                      font, 
                      (AECHAR *) psz,  // Start of the buffer to display,
                      -1,
                      pMe->m_dInfo.cxScreen - 5, // maxWidth
                      &pnFits);         // Number of chars that will fit a line

      // If pnFits is zero there is something wrong in the input to above function. 
      // Normally this scenario should not occur. But, have the check anyway.
      if (pnFits == 0)
      {
         FREE (pszBuf);
         return;
      }

      IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, psz, pnFits, 5 /*start dx*/, 
         dy, 0 /* use default rectangle coordinates */, 0);

      psz += pnFits;      // move pointer to the next segment to be displayed
      totalCh -= pnFits;  // reduce the total number of characters to still display
      dy += pMe->m_nLineHeight; // Place next line charHeight pixels below the 
                                // previous line. 
      IDISPLAY_Update(pMe->a.m_pIDisplay);
      if (totalCh < pnFits)
         pnFits = totalCh;  // if total number is less than pnFits, adjust pnFits
   }

   FREE (pszBuf);
   return;   
} // End of DisplayOutput









