#include "cppapp.h"
#include "cppapp.bid"
#include "appclasses.h"
#include "AEEStdlib.h"

//-------------------------------------------------------------------------
// This application serves as a sample C++ application. The C++ capabilities
// being demonstrated are inheritance, operator overloading, function overloading, 
// new/delete overloading and polymorphism.
// CPPApp is the applet class that derives from AEEApplet. This class uses
// two helper classes, Writer and SmartWriter to display messages on device
// screen.
// Writer class writes messages on device screen using ITextCtl. SmartWriter
// extends Writer class and writes messages in bold using IStatic. Writer
// class provides WriteLine() method for writiting any message to device display.
// This method is specialised by derived class SmartWriter to write message
// in bold. Writer class has overloaded DisplayOutput() methods to support
// WriteLine() methods of base as well as derived class.
// Both of the classes have overloaded new, delete and stream out operators.
//-------------------------------------------------------------------------
// App specific constants
#define USAGE_BASE_CLASS                    100
#define USAGE_DERIVED_CLASS                 101
#define USAGE_BASE_CLASS_OVERLOADEDOP       102
#define USAGE_DERIVED_CLASS_OVERLOADEDOP    103
#define USAGE_BASE_CLASS_NEWDELETE          104
#define USAGE_DERIVED_CLASS_NEWDELETE       105
#define USAGE_POLYMORPHISM                  106

#define APP_RES_FILE    "cppapp.bar"

//--------------------------------------------------------------------------
// All static functions of CPPApp class.
//--------------------------------------------------------------------------
extern "C" 
int AEEClsCreateInstance(AEECLSID clsID, IShell* pIShell, IModule* pIModule, void **ppobj)
{
	if(clsID == AEECLSID_CPPAPP)
	{
		// We want to load this App. So, invoke AEEApplet_New().To it, pass the
		// address of the app-specific handle event function.
		if(!AEEApplet_New(sizeof(CPPApp), clsID, pIShell, pIModule, (IApplet**)ppobj,(AEEHANDLER)CPPApp::HandleEvent,
			(PFNFREEAPPDATA)CPPApp::freeAppData))
			return EFAILED;

		// Allocate and initialize app specific data
		if (!CPPApp::InitAppData((IApplet *)*ppobj))
			return EFAILED;

		return SUCCESS;
	}
	return EFAILED;
}

boolean  CPPApp::InitAppData(IApplet *pIApplet)
{
	CPPApp* pCPPApp = (CPPApp*)pIApplet;
	return pCPPApp->OnAppInitData();
}

void  CPPApp::freeAppData(CPPApp *pCPPApp)
{
	pCPPApp->OnAppfreeData();
}

boolean  CPPApp::HandleEvent(CPPApp *pCPPApp, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{   
	return pCPPApp->OnEvent(eCode, wParam, dwParam);
}

//--------------------------------------------------------------------------
// All member functions of CPPApp class.
//--------------------------------------------------------------------------
boolean CPPApp::OnAppInitData()
{
    m_pIMenuCtl = NULL;
    return TRUE;
}

void CPPApp::OnAppfreeData()
{
    if (m_pIMenuCtl)
        IMENUCTL_Release(m_pIMenuCtl);
}

boolean CPPApp::OnEvent(AEEEvent eCode, uint16 wParam, uint32 dwParam)
{
	switch (eCode) 
	{
	    case EVT_APP_START:			
			// When this applet is started, try to create the main menu
			// that allows the user to select a usage example to execute.
			// If we cannot create an instance of the menu class, exit and
			// indicate that we have handled the event by returning TRUE
			if(ISHELL_CreateInstance(m_pIShell, AEECLSID_MENUCTL, 
				  (void **)&m_pIMenuCtl) != SUCCESS)
				return TRUE;

			// Succeeded in obtaining a menu instance pointer, so
			// construct the menu and display it on the screen
			BuildMainMenu();
			return(TRUE);

	    case EVT_KEY:
            IMENUCTL_SetActive(m_pIMenuCtl, TRUE);
            if (wParam == AVK_CLR)
                return TRUE;
			else 
                return IMENUCTL_HandleEvent(m_pIMenuCtl, EVT_KEY, wParam, dwParam);
		
	    case EVT_COMMAND:
            switch(wParam)
			{
				// The following commands are generated by user selections
				// from the main usage app menu.
			    case USAGE_BASE_CLASS:
                case USAGE_DERIVED_CLASS:
                case USAGE_BASE_CLASS_OVERLOADEDOP:
                case USAGE_DERIVED_CLASS_OVERLOADEDOP:
                case USAGE_BASE_CLASS_NEWDELETE:
                case USAGE_DERIVED_CLASS_NEWDELETE:  
                case USAGE_POLYMORPHISM:
				   // Deactivate main menu to turn off horizontal scrolling
				   // of long menu items
				   IMENUCTL_SetActive(m_pIMenuCtl, FALSE);

				   // Execute the usage example the user has selected
                   CPPUsage (wParam);
			       return TRUE;
			    default:
				    return FALSE;
			}
		
	    case EVT_APP_STOP:
		    return(TRUE);
			
	    default:
		    return(FALSE);	   
	}
}

bool CPPApp::CPPUsage(uint16 wParam)
{
    char* szGreeting = (char*) MALLOC(sizeof("Hello, World!"));
    STRCPY(szGreeting, "Hello, World!");
    switch (wParam)
    {
        case USAGE_BASE_CLASS:
        {

            Writer writer(m_pIShell);
            writer.WriteLine(szGreeting);
            break;
        }
        case USAGE_DERIVED_CLASS:
        {
            SmartWriter smartwriter(m_pIShell);
            smartwriter.WriteLine(szGreeting);
            break;
        }
        case USAGE_BASE_CLASS_OVERLOADEDOP:
        {
            Writer writer(m_pIShell);
            writer << szGreeting;
            break;
        }
        case USAGE_DERIVED_CLASS_OVERLOADEDOP:
        {
            SmartWriter smartwriter(m_pIShell);
            smartwriter << szGreeting;
            break;
        }
        case USAGE_BASE_CLASS_NEWDELETE:
        {
            Writer* pwriter = new Writer(m_pIShell);
            pwriter->WriteLine(szGreeting);
            delete pwriter;
            break;
        }
        case USAGE_DERIVED_CLASS_NEWDELETE:
        {
            SmartWriter* psmartwriter = new SmartWriter(m_pIShell);
            psmartwriter->WriteLine(szGreeting);
            delete psmartwriter;
            break;
        }
        case USAGE_POLYMORPHISM:
        {
            Writer* pwriter = new SmartWriter(m_pIShell);
            pwriter->WriteLine(szGreeting);
            delete pwriter;
            break;
        }
        default:
            FREE(szGreeting);
            return FALSE;
    }
    FREE(szGreeting);
    return TRUE;
}

/*===========================================================================

FUNCTION: BuildMainMenu

DESCRIPTION
    This function constructs the menu control that allows the user to
	select a usage example to execute.  The command ID for each menu item
	is passed to the usage example function above when the user selects the
	menu item for that example.
    
PROTOTYPE:
    void CPPApp::BuildMainMenu()

PARAMETERS:
    None  

DEPENDENCIES
    None

RETURN VALUE
    None

SIDE EFFECTS
    None

===========================================================================*/
void CPPApp::BuildMainMenu()
{
	AEERect qrc;
	AEEDeviceInfo di;
    AECHAR* szBuf;
    char* charBuf;

	// Make sure the pointers we'll be using are valid
    if (m_pIShell == NULL || m_pIMenuCtl == NULL)
	   return;

    if ((szBuf = (AECHAR *) MALLOC(TEXT_BUFFER_SIZE)) == NULL)
	   return;

    if ((charBuf = (char *) MALLOC(TEXT_BUFFER_SIZE/sizeof(AECHAR))) == NULL)
    {
       FREE(szBuf);
	    return;
    }

	// Set Title
    STRCPY(charBuf, "C++ Examples");
    STR_TO_WSTR(charBuf, szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_SetTitle(m_pIMenuCtl, NULL, 0, szBuf);

	//Set the Rectangle for the Menu
	ISHELL_GetDeviceInfo(m_pIShell,&di);
	qrc.x = 0;
	qrc.y = 0;
	qrc.dx = di.cxScreen;
	qrc.dy = di.cyScreen;
	IMENUCTL_SetRect(m_pIMenuCtl, &qrc);  

	//Add individual entries to the Menu

    // Add Base Class Usage to menu
    STRCPY(charBuf, "1. Base Class Usage");
    STR_TO_WSTR(charBuf, szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(m_pIMenuCtl, 0, 0, USAGE_BASE_CLASS, szBuf, 0);

    // Add Derived Class Usage to menu
    STRCPY(charBuf, "2. Derived Class Usage");
    STR_TO_WSTR(charBuf, szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(m_pIMenuCtl, 0, 0, USAGE_DERIVED_CLASS, szBuf, 0);

    // Add Base Class Op Overloading to menu
    STRCPY(charBuf, "3. Base Class Op Overloading");
    STR_TO_WSTR(charBuf, szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(m_pIMenuCtl, 0, 0, USAGE_BASE_CLASS_OVERLOADEDOP, szBuf, 0);

    // Add Derived Class Op Overloading to menu
    STRCPY(charBuf, "4. Derived Class Op Overloading");
    STR_TO_WSTR(charBuf, szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(m_pIMenuCtl, 0, 0, USAGE_DERIVED_CLASS_OVERLOADEDOP, szBuf, 0);

    // Add Base Class new/delete to menu
    STRCPY(charBuf, "5. Base Class new/delete");
    STR_TO_WSTR(charBuf, szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(m_pIMenuCtl, 0, 0, USAGE_BASE_CLASS_NEWDELETE, szBuf, 0);

    // Add Derived Class new/delete to menu
    STRCPY(charBuf, "6. Derived Class new/delete");
    STR_TO_WSTR(charBuf, szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(m_pIMenuCtl, 0, 0, USAGE_DERIVED_CLASS_NEWDELETE, szBuf, 0);

    // Add Polymorphism to menu
    STRCPY(charBuf, "7. Polymorphism");
    STR_TO_WSTR(charBuf, szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(m_pIMenuCtl, 0, 0, USAGE_POLYMORPHISM, szBuf, 0);

    // Activate menu
	IMENUCTL_SetActive(m_pIMenuCtl,TRUE);

	FREE(szBuf);
    FREE(charBuf);
}
