/*======================================================
FILE:  QCOMCalApp.c

SERVICES:  Sample Calendar App

GENERAL DESCRIPTION:
	Sample calendar app running under AEE

PUBLIC CLASSES AND STATIC FUNCTIONS:
	List the class(es) and static functions included in this file

INITIALIZATION & SEQUENCING REQUIREMENTS:

	See Exported Routines

        Copyright  1999-2001 QUALCOMM Incorporated.
               All Rights Reserved.
            QUALCOMM Proprietary/GTDR
=====================================================*/

//===========================================================================
//
//                       PLEASE NOTE THE FOLLOWING
//
// This sample application was developed before some BREW APIs were available.
// As a result, it contains code that may not be optimal.  The main areas 
// of note related to this issue are:
//
// - Dialogs
// - Messages
// - Simple Prompts
//
// BREW now provides much better support for these operation
// 
// In future revisions, this code will be modified to support these new APIs.
//
//
//===========================================================================

#if defined(BREW_STATIC_APP)
#include <stdlib.h>              // Required for ATOI
#endif

#include "AEEShell.h"
#include "AEEDisp.h"
#include "AEEText.h"
#include "AEEHeap.h"
#include "AEEMenu.h"
#include "AEEDB.h"
#include "AEEDate.h"
#include "AEETime.h"
#include "AEEModTable.h"
#include "AEEStdLib.h"
#include "AEEModGen.h"
#include "AEEAppGen.h"
#include "AEESampleAppIDs.h"

#include "CalApp_res.h"
#include "CalApp.h"


/*===========================================================================

                      PUBLIC DATA DECLARATIONS

===========================================================================*/

static boolean       Cal_HandleEvent(CCalApp * pme, AEEEvent eCode, uint16 wParam, uint32 dwParam);
static void          Cal_Free(CCalApp * pme);
static void          Cal_ClearScreen(CCalApp * pme);
static void          Cal_ResetControls(CCalApp * pme);
static int           Cal_ChangeState(CCalApp * pme, int nState, boolean bRevert);
static void          Cal_ForceState(CCalApp * pme, int nState);
static void          Cal_InitEventData(CCalApp * pme, CalEvent * pce);
static CalEvent *    Cal_FindFirst(CCalApp * pme, uint32 lDateTime);
static CalEvent *    Cal_FindPrev(CCalApp * pme, uint32 lDateTime);
static uint32        Cal_GetCurrentDateTime(CCalApp * pme);
static CalEvent *    Cal_FindRecID(CCalApp * pme, uint16 nRecID);
static void          Cal_SetFirstAlarm(CCalApp * pme);
static void          Cal_ShowAlert(CCalApp * pme);
static void          Cal_SoundAlarm(CCalApp * pme);
static void          Cal_DeactivateAlarm(CCalApp * pme);
static boolean       Cal_OpenDatabase(CCalApp * pme);
static void          Cal_ReadDatabase(CCalApp * pme);
static void          Cal_ReadRecords(CCalApp * pme);
static boolean       Cal_InitClasses(CCalApp * pme);
static void          Cal_Suspend(CCalApp * pme);
static void          Cal_Resume(CCalApp * pme);
static void          Cal_DetermineDayMask(CCalApp * pme);
static void          Cal_FillAlarmList(CCalApp * pme, IMenuCtl * pm, uint16 wTitleID, int16 nCurr);
static void          Cal_SetShellAlarm(CCalApp * pme, uint32 lAlarm, uint16 wID);
static void          Cal_FillResMenu(CCalApp * pme, IMenuCtl * pm, uint16 wID, uint16 wTitleID, int16 nCurr);
static AECHAR *      Cal_WriteTime(CCalApp * pme, AECHAR * pszBuff, long lTime, int * pnRemaining);

static AECHAR *      Cal_LoadResString(CCalApp * pme, int nIDStr, AECHAR * pszBuf, int nBufSize, int * pnCopied);
static boolean       Cal_DisplayString(CCalApp * pme, AECHAR * psz, int nLen, int x, int y, int nFont, boolean bRightAlign);
static boolean       Cal_DisplayResString(CCalApp * pme, int nIDStr, int x, int y, int nFont, boolean bRightAlign);
static void          Cal_SetupTimeCtl(CCalApp * pme, ITimeCtl * pt, uint16 wID, AEERect * prc, uint32 lTime);
static void          Cal_ShowMessage(CCalApp * pme, uint16 wID, int nReturnState);
static void          Cal_CreateUpdateRecord(CCalApp * pme, CalEvent * pCalEvent,const AECHAR * pszDescription,const AECHAR * pszNote,CalBaseFields * pb);
static void          Cal_HandleAlarm(CCalApp * pme, uint16 wID);

static boolean       Cal_BeginState(CCalApp * pme,int nState);
static boolean       Cal_EndState(CCalApp * pme,int nState);
static void          Cal_MainMenu(CCalApp * pme);
static void          Cal_ViewDay(CCalApp * pme);
static void          Cal_ViewMonth(CCalApp * pme);
static void          Cal_ViewEvent(CCalApp * pme);
static void          Cal_ViewEventRecord(CCalApp * pme, uint16 wID, boolean bAlert);
static void          Cal_EditEvent(CCalApp * pme);
static void          Cal_AddEvent(CCalApp * pme);
static void          Cal_EnterName(CCalApp * pme);
static void          Cal_EnterNote(CCalApp * pme);
static void          Cal_EventAdded(CCalApp * pme);
static void          Cal_Return(CCalApp * pme);
static void          Cal_Options(CCalApp * pme);
static void          Cal_EraseEvent(CCalApp * pme);
static void          Cal_AlarmTime(CCalApp * pme);
static void          Cal_OptAutoDelete(CCalApp * pme);
static void          Cal_EndEnterName(CCalApp * pme);
static void          Cal_EndEnterNote(CCalApp * pme);
static void          Cal_EndViewMonth(CCalApp * pme);
static void          Cal_EraseConfirm(CCalApp * pme);
static void          Cal_EndAlarmTime(CCalApp * pme);
static void          Cal_EndOptAutoDelete(CCalApp * pme);
static void          Cal_OptAlarm(CCalApp * pme);
static void          Cal_EndOptAlarm(CCalApp * pme);
static void          Cal_OptHours(CCalApp * pme);
static void          Cal_EndOptHours(CCalApp * pme);
static boolean       Cal_EnumDays(CCalApp * pme, uint32 lDate, PFNENUMDAYCB pfnCB);
static void          Cal_MenuAddState(CCalApp * pme, IMenuCtl * pIMenuCtl, int nState,AECHAR * pString, int16 nIDStr, uint32 lData);
static boolean       Cal_Busy(CCalApp * pme, uint16 wState);


static void          dllInsert(dlistnode ** pphead, dlistnode * pref, dlistnode * pnew);
static dlistnode *   dllRemove(dlistnode ** pphead, dlistnode * p);
static dlistnode *   dllScan(dlistnode * pstart, dlistnode * pend,PFNCBDLLFIND pfnCB, uint32 data);
static boolean       CalReadRec(IDBRecord * prec, CalDBRecFields * pf);
static void          InitStrField(AEEDBField * pFields, int name, AECHAR * psz);
static void          InitBinaryField(AEEDBField * pFields, int name, void * pBuff, int nSize);
static void          VCenterBlt(IImage * pImage, int x, int y, int dy, int cyImage);
static AECHAR *      FindNextToken(AECHAR * psz, AECHAR ch);
static boolean       FindFirstCB(CalEvent * pCalEvent, uint32 lDateTime);
static boolean       FindRecIDCB(CalEvent * pCalEvent, uint32 nRecID);

#define DLL_INSERT(pph,p,pn)        dllInsert((dlistnode**)(pph), (dlistnode*)(p), (dlistnode*)(pn))
#define DLL_REMOVE(pph,p)           dllRemove((dlistnode**)(pph), (dlistnode*)(p))
#define DLL_SCAN(ps,pe,pfn,d)       dllScan((dlistnode*)(ps),(dlistnode*)(pe),(PFNCBDLLFIND)(pfn),(uint32)(d))


//
// Constant Data
//

static const int16 garIDs[] = {
   0,
   IDS_VIEWDAY,
   IDS_VIEWMONTH,
   0,
   IDS_ADDEVENT,
   0,
   IDS_NEXT,
   IDS_NEXT,
   IDS_NEXT,
   IDS_AUTODELETE,
   0,
   IDS_OPTIONS,
   IDS_DAYTIME,
   IDS_DEFALARM,
   0,
   IDS_ERASE,
   0,
   IDS_VIEWEVENT
};

//
// Static Applet Functions - These functions need only be declared
// when an applet is being linked by the Manufacturer during their
// main build process.  They are not required for dynamically downloaded
// applets.
//
#if defined(BREW_STATIC_APP)

static int     CCalMod_CreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj);
static int     CCalMod_Load(IShell *ps, void * pHelpers, IModule ** pMod);

static const AEEAppInfo  gaiCal = {AEECLSID_CAL,CALAPP_RES_FILE,IDS_CALENDAR, IDB_ICON,IDB_CAL,0,0,AFLAG_PIM};

/*===========================================================================

                      PUBLIC FUNCTION DECLARATIONS

===========================================================================*/

//===========================================================================
//
// Returns static module information
//
//===========================================================================
PFNMODENTRY Cal_GetModInfo(IShell * ps,AEECLSID ** ppClasses, AEEAppInfo ** pApps, uint16 * pnApps,uint16 * pwMinPriv)
{
   *pApps = (AEEAppInfo *)&gaiCal;
   *pnApps = sizeof(gaiCal) / sizeof(AEEAppInfo);
   return((PFNMODENTRY)CCalMod_Load);
}

//===========================================================================
//
// Called by BREW to initialize the static module.
//
//===========================================================================
static int CCalMod_Load(IShell *ps, void * pHelpers, IModule ** pMod)
{
   return(AEEStaticMod_New((int16)(sizeof(AEEMod)),ps,pHelpers,pMod,(PFNMODCREATEINST)CCalMod_CreateInstance,(PFNFREEMODDATA)NULL));
}

#endif

//===========================================================================
//
// 
//
//===========================================================================
#if defined(BREW_STATIC_APP)
static int CCalMod_CreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
#else
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
#endif
{
   CCalApp *      pme = NULL;

   if(ClsId != AEECLSID_CAL)
      return(EBADCLASS);

   *ppObj = NULL;

   if(!AEEApplet_New(sizeof(CCalApp), ClsId, pIShell,po,(IApplet**)ppObj,(AEEHANDLER)Cal_HandleEvent,(PFNFREEAPPDATA)Cal_Free))
      return(ENOMEMORY);

   pme = (CCalApp *)*ppObj;

   // Needed for Cal_GetCurrentDataTime

   if(ISHELL_CreateInstance(pIShell, AEECLSID_DATECTL, (void **)&pme->m_pIDate)){
abort:
      IAPPLET_Release((IApplet *)pme);
      *ppObj = NULL;
      return(EFAILED);
   }
   if(ISHELL_CreateInstance(pIShell, AEECLSID_DBMGR, (void **)&pme->m_pIDBMgr))
      goto abort;

   if(ISHELL_GetPrefs(pIShell, AEECLSID_CAL, CAL_CFG_VERSION, &pme->m_cfg, sizeof(pme->m_cfg))){
      pme->m_cfg.nAutoDelete = DEFAULT_AUTO_DELETE;
      pme->m_cfg.wDayStart = 7 * MINUTES_PER_HOUR;
      pme->m_cfg.wDayEnd = 18 * MINUTES_PER_HOUR;
      pme->m_cfg.nAlarmTime = 0;
   }

   return(AEE_SUCCESS);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_Free(CCalApp * pme)
{
   CalEvent * pce = NULL;

   while (NULL != (pce = (CalEvent*)DLL_REMOVE(&pme->m_pEventList, NULL)))
      FREE(pce);

   ISHELL_CancelTimer(pme->a.m_pIShell, NULL, pme);

   if (pme->m_pIMenu)
      IMENUCTL_Release(pme->m_pIMenu);

   if (pme->m_pISoftKey)
      IMENUCTL_Release(pme->m_pISoftKey);

   if (pme->m_pIDatePick)
      IDATECTL_Release(pme->m_pIDatePick);

   if (pme->m_pIDate)
      IDATECTL_Release(pme->m_pIDate);

   if (pme->m_pIText)
      ITEXTCTL_Release(pme->m_pIText);

   if (pme->m_pITimeStart)
      ITIMECTL_Release(pme->m_pITimeStart);

   if (pme->m_pITimeEnd)
      ITIMECTL_Release(pme->m_pITimeEnd);

   if (pme->m_pIDuration)
      ITIMECTL_Release(pme->m_pIDuration);

   if (pme->m_pIList)
      IMENUCTL_Release(pme->m_pIList);

   if (pme->m_pIDatabase)
      while(IDATABASE_Release(pme->m_pIDatabase) > 0);

   if (pme->m_pIDBMgr)
      IDBMGR_Release(pme->m_pIDBMgr);

}

//===========================================================================
//
// 
//
//===========================================================================
static boolean Cal_HandleEvent(CCalApp * pme, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{
   switch (eCode) {
      case EVT_APP_SUSPEND:
         Cal_Suspend(pme);
         return(TRUE);

      case EVT_COPYRIGHT_END:
      case EVT_APP_RESUME:
         Cal_Resume(pme);
         return(TRUE);

      case EVT_APP_START:
         {
         uint32   lDateTime;

      // Do not allow the app to start if there is no valid timestamp...

         if(ISHELL_GetSeconds(pme->a.m_pIShell) < ONE_YEAR_SECS){
            AEEAppStart * pas = (AEEAppStart *)dwParam;
            pas->error = EINVALIDTIME;
            return(FALSE);
         }

         Cal_ReadDatabase(pme);
   
         if(!pme->m_bInit)
            Cal_InitClasses(pme);

         lDateTime = Cal_GetCurrentDateTime(pme);
         pme->m_lCurrentDay = GETDATE(lDateTime);

      // If there is a pending alarm, show it...

         if(pme->m_wPendingAlarmID)
            return(Cal_ChangeState(pme, S_ALERT, FALSE));

         pme->m_nCurrentState = S_MAINMENU;
         ISHELL_ShowCopyright(pme->a.m_pIShell);
         return(TRUE);
         }

      case EVT_ALARM:
         Cal_HandleAlarm(pme, wParam);
         return(TRUE);

      case EVT_CTL_TAB:
         {
         int   i;
         

         if(pme->m_nCtlsOnScreen <= 1)
            return(FALSE);

         for(i = 0; i < pme->m_nCtlsOnScreen; i++){
            if(dwParam == (uint32)pme->m_pCtl[i]){

               ICONTROL_SetActive((IControl *)dwParam, FALSE);
               ICONTROL_Redraw((IControl *)dwParam);
               if(wParam){
                  i++;
                  if(i == pme->m_nCtlsOnScreen)
                     i = 0;
               }
               else{
                  i--;
                  if(i < 0)
                     i = pme->m_nCtlsOnScreen - 1;
               }

               dwParam = (uint32)pme->m_pCtl[i];

      // For time controls make sure we start on the correct field...

               if(dwParam == (uint32)pme->m_pITimeStart || dwParam == (uint32)pme->m_pIDuration || dwParam == (uint32)pme->m_pITimeEnd)
                  ITIMECTL_SetEditField((ITimeCtl *)dwParam, (ITField)(wParam ? ITF_HOUR : ITF_MIN));

               ICONTROL_SetActive((IControl *)dwParam, TRUE);
               ICONTROL_Redraw((IControl *)dwParam);
               break;
            }
         }
         return(TRUE);               
         }
                     
      case EVT_CTL_CHANGING:
         {
         CtlValChange * pv;
         uint16         w1, w2;

         pv = (CtlValChange *)dwParam;

         if(pv){

            w1 = (uint16)(pv->dwParam / MSECS_PER_MINUTE);

            switch(pme->m_nCurrentState){
               case S_OPTHOURS:
                  {
                  boolean  bStart;

                  bStart = (((ITimeCtl *)pv->pc == pme->m_pITimeStart) ? TRUE : FALSE);
                  w2 = (uint16)(ITIMECTL_GetTime((bStart ? pme->m_pITimeEnd : pme->m_pITimeStart)) / MSECS_PER_MINUTE);
                  if(bStart){
                     if(w1 >= w2)
                        pv->bValid = FALSE;
                  }
                  else{
                     if(w1 <= w2)
                        pv->bValid = FALSE;
                  }
                  break;
                  }

               case S_ALARMTIME:
                  if(pv->pc == (IControl *)pme->m_pITimeStart){
                     w1 = (uint16)(pv->dwParam / MSECS_PER_MINUTE);
                     w2 = (uint16)(ITIMECTL_GetTime(pme->m_pIDuration) / MSECS_PER_MINUTE);
                  }
                  else{
                     w1 = (uint16)(ITIMECTL_GetTime(pme->m_pITimeStart) / MSECS_PER_MINUTE);
                     w2 = (uint16)(pv->dwParam / MSECS_PER_MINUTE);
                  }
                  if(w1 > MAXTIME - w2)
                     pv->bValid = FALSE;
                  break;
            }
         }
         return(TRUE);
         }
         
      case EVT_COMMAND: 

         // Handle T9, etc.

         if(ITEXTCTL_HandleEvent(pme->m_pIText, eCode, wParam, dwParam))
            return(TRUE);

         // save the current item wParam if EVT_COMMAND came from the menu control
         // and current state >= S_VIEWEVENT

         switch(pme->m_nCurrentState){

            case S_MAINMENU:
               if((wParam == S_VIEWDAY || wParam == S_VIEWMONTH) && Cal_Busy(pme, wParam))
                  return(TRUE);
               break;

         // See if they are asking us to re-calc the month active mask...

            case S_VIEWMONTH:
               if(wParam != S_VIEWDAY){
                  Cal_DetermineDayMask(pme);
                  return(TRUE);
               }                  
               break;

            case S_OPTALARMTIME:
            case S_OPTHOURS:
            case S_OPTAUTODELETE:
               wParam = S_MAINMENU;
               break;

            case S_VIEWDAY:
               if(wParam > S_VIEWEVENT) {
                  IMENUCTL_GetItemData(pme->m_pIMenu, (int16)wParam, (uint32 *)(&pme->m_pceCurrent));
                  if(pme->m_pceCurrent)
                     wParam = S_VIEWEVENT;
                  else{
                     uint16 wDur;

                     pme->m_lStartTime = (uint32)IMENUCTL_GetItemTime(pme->m_pIMenu, (int16)wParam, &wDur);
                     pme->m_lDuration = (uint32)wDur;
                     wParam = S_ADDEVENT;
                  }
               }
               break;

            case S_ALARMTIME:
               wParam = S_ENTERNOTE;
               break;

            case S_ALERT:
               switch(wParam){
                  case S_MAINMENU:
                     if(pme->m_bAlarmActivated){
                        ISHELL_CloseApplet(pme->a.m_pIShell, FALSE);
                        return(TRUE);
                     }
                     break;

                  case S_VIEWEVENT:
                     {
                     uint32   lID;

                     if(Cal_Busy(pme, wParam))
                        return(TRUE);

                     if(IMENUCTL_GetItemData(pme->m_pISoftKey, (int16)wParam, &lID))
                        pme->m_pceCurrent = Cal_FindRecID(pme, (uint16)lID);
                     break;
                     }
               }
               break;
         }
            
         return Cal_ChangeState(pme, wParam, FALSE);

      case EVT_KEY_HELD:
      case EVT_KEY_PRESS:
      case EVT_KEY_RELEASE:
         if(ITEXTCTL_HandleEvent(pme->m_pIText, eCode, wParam, dwParam))
            return(TRUE);
         break;

      case EVT_KEY:
         {

   // If we are waiting for the database to complete, then dont process any keys...

         if(pme->m_nPendingState)
            return((boolean)(wParam != AVK_END && wParam != AVK_CLR));

         if(pme->m_nCurrentState == S_MESSAGE && wParam != AVK_END){
            Cal_Return(pme);
            return(TRUE);
         }

         if (IMENUCTL_HandleEvent(pme->m_pIMenu, eCode, wParam, dwParam))
            return TRUE;

         if(IMENUCTL_HandleEvent(pme->m_pISoftKey, eCode, wParam, dwParam))
            return TRUE;

         if(IDATECTL_HandleEvent(pme->m_pIDatePick, eCode, wParam, dwParam))
            return TRUE;

         if(IDATECTL_HandleEvent(pme->m_pIDate, eCode, wParam, dwParam))
            return TRUE;

         if (ITEXTCTL_HandleEvent(pme->m_pIText, eCode, wParam, dwParam))
            return TRUE;

         if (ITIMECTL_HandleEvent(pme->m_pITimeStart, eCode, wParam, dwParam))
            return TRUE;

         if (ITIMECTL_HandleEvent(pme->m_pITimeEnd, eCode, wParam, dwParam))
            return TRUE;

         if (ITIMECTL_HandleEvent(pme->m_pIDuration, eCode, wParam, dwParam))
            return TRUE;

         if (IMENUCTL_HandleEvent(pme->m_pIList, eCode, wParam, dwParam))
            return TRUE;

         if (pme->m_pfnKeyHandler && (*pme->m_pfnKeyHandler)(pme, wParam))
            return TRUE;

         switch (wParam) {
            
            case AVK_END:
               pme->m_wPendingAlarmID = 0;
               break;

            case AVK_CLR:
               pme->m_wPendingAlarmID = 0;
               if(pme->m_nCurrentState == S_MAINMENU || (pme->m_nCurrentState == S_ALERT && pme->m_bAlarmActivated))
                  return(FALSE);
               return Cal_ChangeState(pme, 0, TRUE);
         }
      } break;
   }
   
   return 0;
}

//===========================================================================
//
// 
//
//===========================================================================
static IDBRecord * Cal_UpdateRecord(CCalApp * pme, IDBRecord * pidb, 
                               const AECHAR * pszDescription, 
                               const AECHAR * pszNote, 
                               CalBaseFields * pb)
{
   AEEDBField     arFields[NUMFLDS];
   int            nFields;

   InitBinaryField(arFields,FLD_ALARM,(void *)pb, sizeof(CalBaseFields));

   nFields = 1;

   if(pszDescription && *pszDescription){
      InitStrField(arFields,FLD_DESC, (AECHAR*)pszDescription);
      nFields++;
   }

   if(pszNote && *pszNote){
      InitStrField(arFields,FLD_NOTE, (AECHAR*)pszNote);
      nFields++;
   }

   if (NULL == pidb)
      pidb = IDATABASE_CreateRecord(pme->m_pIDatabase, arFields, nFields);
   else{
      if(IDBRECORD_Update(pidb, arFields, nFields)){
         IDBRECORD_Release(pidb);
         pidb = NULL;
      }
   }
   
   if(pidb == NULL)
      ISHELL_MessageBox(pme->a.m_pIShell,NULL,0,ENOMEMORY);

   return pidb;
}

//===========================================================================
//
// 
//
//===========================================================================
static AECHAR * Cal_WriteTime(CCalApp * pme, AECHAR * pszBuff, long lTime, int * pnRemaining)
{
   long nHour;
   long nMinute;
   AECHAR * psz = pszBuff;

   nHour = GET_HOUR(lTime);
   if(nHour == 0)
      nHour = 12;
   else
   if(nHour > 12)
      nHour = nHour - 12;

   nMinute = GET_MINUTE(lTime);
   
   psz = WWRITELONGEX(psz, nHour, 0, pnRemaining);
   if (*pnRemaining) {
      *(psz++) = ':';
      (*pnRemaining)--;
      psz = WWRITELONGEX(psz, nMinute, 2, pnRemaining);
   }
   if (*pnRemaining){
      *(psz++) = IDISPLAY_GetSymbol(pme->a.m_pIDisplay,(AEESymbol)(IS_PM(lTime) ? AEE_SYM_PM : AEE_SYM_AM),AEE_FONT_NORMAL);
      (*pnRemaining)--;
      *psz = 0;
   }
   return psz;
}


//===========================================================================
//
// 
//
//===========================================================================
static void Cal_HandleAlarm(CCalApp * pme, uint16 wID)
{
   IDBRecord   *  pRec;
   CalDBRecFields f;
   uint32         lAlarmTime;

   pme->m_wPendingAlarmID = 0;

   if(!Cal_OpenDatabase(pme))
      return;

   pRec = IDATABASE_GetRecordByID(pme->m_pIDatabase, wID);

   if(!pRec)
      return;

   // Clear all of the fields...

   MEMSET(&f, 0, sizeof(CalDBRecFields));

   if(!CalReadRec(pRec, &f))
      return;

   IDBRECORD_Release(pRec);

   lAlarmTime = f.b.dwTime - f.b.wAlarmTime;

   if(Cal_GetCurrentDateTime(pme) < lAlarmTime)
      Cal_SetShellAlarm(pme, lAlarmTime, wID);
   else{
      pme->m_wPendingAlarmID = wID;
      if(ISHELL_ActiveApplet(pme->a.m_pIShell) == AEECLSID_CAL)
         Cal_ChangeState(pme, S_ALERT, FALSE);
      else{
         pme->m_bAlarmActivated = TRUE;
         ISHELL_StartApplet(pme->a.m_pIShell, AEECLSID_CAL);
      }
   }
}


//===========================================================================
//
// 
//
//===========================================================================
static boolean Cal_InitClasses(CCalApp * pme)
{
   IShell * pIShell = pme->a.m_pIShell;

   // get our global IDisplay object

   if(ISHELL_CreateInstance(pIShell, AEECLSID_DISPLAY, (void **)&pme->a.m_pIDisplay))
      return(FALSE);
   
   // get the IMenuCtl menu object

   if(ISHELL_CreateInstance(pIShell, AEECLSID_MENUCTL, (void **)&pme->m_pIMenu))
      return(FALSE);
   
   // get the IMenuCtl soft-key object

   if(ISHELL_CreateInstance(pIShell, AEECLSID_SOFTKEYCTL, (void **)&pme->m_pISoftKey))
      return(FALSE);
   
   // get the date pick object

   if(ISHELL_CreateInstance(pIShell, AEECLSID_DATEPICKCTL, (void **)&pme->m_pIDatePick))
      return(FALSE);
   
   if(ISHELL_CreateInstance(pIShell, AEECLSID_TEXTCTL, (void **)&pme->m_pIText))
      return(FALSE);

   ITEXTCTL_SetMaxSize(pme->m_pIText, MAXTEXT);

   if(ISHELL_CreateInstance(pIShell, AEECLSID_CLOCKCTL, (void **)&pme->m_pITimeStart))
      return(FALSE);

   if(ISHELL_CreateInstance(pIShell, AEECLSID_CLOCKCTL, (void **)&pme->m_pITimeEnd))
      return(FALSE);

   ITIMECTL_SetProperties(pme->m_pITimeStart, TP_NOEDIT_AMPM);
   ITIMECTL_SetProperties(pme->m_pITimeEnd, TP_NOEDIT_AMPM);

   ISHELL_CreateInstance(pIShell, AEECLSID_COUNTDOWNCTL, (void **)&pme->m_pIDuration);
   if (!pme->m_pIDuration)
      return FALSE;

   ITIMECTL_SetProperties(pme->m_pIDuration, TP_NO_SECONDS);
   ITIMECTL_SetIncrement(pme->m_pIDuration, DURATION_INCREMENT);
   ITIMECTL_SetIncrement(pme->m_pITimeStart, DURATION_INCREMENT);
   ITIMECTL_SetIncrement(pme->m_pITimeEnd, DURATION_INCREMENT);

   ISHELL_CreateInstance(pIShell, AEECLSID_LISTCTL, (void **)&pme->m_pIList);
   if (!pme->m_pIList)
      return FALSE;

   // get the screen dimensions   
   if (pme->a.m_pIDisplay) {
      AEEDeviceInfo  dm;

      ISHELL_GetDeviceInfo(pIShell, &dm);
      pme->m_cx = dm.cxScreen;
      pme->m_cy = dm.cyScreen;

      // get the font metrics

      pme->m_nFontHeight = IDISPLAY_GetFontMetrics(pme->a.m_pIDisplay, AEE_FONT_BOLD, NULL, NULL);
   }

   pme->m_bInit = TRUE;

   return TRUE;
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_ReadDatabase(CCalApp * pme)
{
   pme->m_bDBReady = FALSE;
   pme->m_bSetAlarm = TRUE;
   ISHELL_SetTimer(pme->a.m_pIShell, DB_READ_DELAY, (PFNNOTIFY)(Cal_ReadRecords), pme);
}

//===========================================================================
//
// 
//
//===========================================================================
static boolean Cal_OpenDatabase(CCalApp * pme)
{
   if(!pme->m_pIDatabase){
      pme->m_pIDatabase = IDBMGR_OpenDatabaseEx(pme->m_pIDBMgr, CAL_DATABASE_NAME, 1, SCHEDULER_MIN_SIZE, SCHEDULER_MIN_RECS);
      if (!pme->m_pIDatabase)
         return(FALSE);
   }
   return(TRUE);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_ReadRecords(CCalApp * pme)
{
   int            i;
   uint32         lDelBefore;
   CalEvent *     pce;
   CalEvent *     pcePrev;
   uint32         lDateTime, lAlarmTime;
   IDBRecord *    prec;
   CalDBRecFields f;

   if(!Cal_OpenDatabase(pme))
      return;

   if(pme->m_cfg.nAutoDelete){
      lDelBefore = Cal_GetCurrentDateTime(pme);
      if(lDelBefore)
         lDelBefore = MAKECOMPTIME((GETDATE(lDelBefore) - pme->m_cfg.nAutoDelete), 0);
   }
   else
      lDelBefore = 0;

   lDateTime = Cal_GetCurrentDateTime(pme);

   for(i = 0; i < DB_READ_RECS; i++){

      pce = pcePrev = NULL;

      prec = IDATABASE_GetNextRecord(pme->m_pIDatabase);

      if(prec == NULL)
         break;

   // Clear all of the fields...

      MEMSET(&f, 0, sizeof(CalDBRecFields));

      if(!CalReadRec(prec, &f))
         continue;

   // See if we are auto-deleting the record...

      if(lDelBefore && f.b.dwTime <= lDelBefore){
         IDBRECORD_Remove(prec);
         continue;
      }

      IDBRECORD_Release(prec);

   // Add the record into the list...

      if((pce = (CalEvent *)MALLOC(sizeof(CalEvent))) != NULL){

         MEMCPY(&pce->m_b, &f.b, sizeof(CalBaseFields));
         pce->m_nRecID = f.nRecID;

         if(f.b.wAlarmTime){
            lAlarmTime = f.b.dwTime - f.b.wAlarmTime;
            if(lDateTime < lAlarmTime)
               pce->m_lAlarmDateTime = lAlarmTime;
            else
               pce->m_b.wAlarmTime = 0;
         }

         // scan list for insertion position

         pcePrev = Cal_FindPrev(pme, pce->m_b.dwTime);

         if (pcePrev && pcePrev->m_b.dwTime > pce->m_b.dwTime)
            pcePrev = NULL;      // insert at head
         // insert after pcePrev or at head if pcePrev == NULL
         DLL_INSERT(&pme->m_pEventList, pcePrev, pce);
      }
   }

   if(!prec){

      if(pme->m_bSetAlarm)      
         Cal_SetFirstAlarm(pme);

      pme->m_bDBReady = TRUE;

      if(pme->m_nPendingState)
         Cal_ChangeState(pme, pme->m_nPendingState, FALSE);
   }
   else
      ISHELL_SetTimer(pme->a.m_pIShell, DB_READ_DELAY, (PFNNOTIFY)(Cal_ReadRecords), pme);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_Suspend(CCalApp * pme)
{
   Cal_ResetControls(pme);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_Resume(CCalApp * pme)
{
   int   nState = pme->m_nCurrentState;

   Cal_ClearScreen(pme);
   Cal_ResetControls(pme);
   
   if (nState >= S_MAINMENU && nState < NUMSTATES)
      Cal_BeginState(pme,nState);
}

//===========================================================================
//
// 
//
//===========================================================================
static boolean Cal_BeginState(CCalApp * pme,int nState)
{
   // 
   // NOTE:  This is done in this manner (rather than in a table) because of 
   // ARM relocation issues.  Placing the functions in a static table causes a linkage error
   //

   switch(nState){
      case S_MAINMENU:
         Cal_MainMenu(pme);
         break;
      case S_VIEWDAY:
         Cal_ViewDay(pme);
         break;
      case S_VIEWMONTH:
         Cal_ViewMonth(pme);
         break;
      case S_EDITEVENT:
         Cal_EditEvent(pme);
         break;
      case S_ADDEVENT:
         Cal_AddEvent(pme);
         break;
      case S_ENTERNAME:
         Cal_EnterName(pme);
         break;
      case S_ENTERNOTE:
         Cal_EnterNote(pme);
         break;
      case S_EVENTADDED:
         Cal_EventAdded(pme);
         break;
      case S_ALARMTIME:
         Cal_AlarmTime(pme);
         break;
      case S_OPTAUTODELETE:
         Cal_OptAutoDelete(pme);
         break;
      case S_RETURN:
         Cal_Return(pme);
         break;
      case S_OPTIONS:
         Cal_Options(pme);
         break;
      case S_OPTHOURS:
         Cal_OptHours(pme);
         break;
      case S_OPTALARMTIME:
         Cal_OptAlarm(pme);
         break;
      case S_ERASECONFIRM:
         Cal_EraseConfirm(pme);
         break;
      case S_ERASEEVENT:
         Cal_EraseEvent(pme);
         break;
      case S_ALERT:
         Cal_ShowAlert(pme);
         break;
      case S_VIEWEVENT:
         Cal_ViewEvent(pme);
         break;
      default:
         return(FALSE);
   }
   return(TRUE);
}

//===========================================================================
//
// 
//
//===========================================================================
static boolean Cal_EndState(CCalApp * pme,int nState)
{
   // 
   // NOTE:  This is done in this manner (rather than in a table) because of 
   // ARM relocation issues.  Placing the functions in a static table causes a linkage error
   //

   switch(nState){
      case S_VIEWMONTH:
         Cal_EndViewMonth(pme);
         break;
      case S_ENTERNAME:
         Cal_EndEnterName(pme);
         break;
      case S_ENTERNOTE:
         Cal_EndEnterNote(pme);
         break;
      case S_ALARMTIME:
         Cal_EndAlarmTime(pme);
         break;
      case S_OPTAUTODELETE:
         Cal_EndOptAutoDelete(pme);
         break;
      case S_OPTHOURS:
         Cal_EndOptHours(pme);
         break;
      case S_OPTALARMTIME:
         Cal_EndOptAlarm(pme);
         break;
      default:
         return(FALSE);
   }
   return(TRUE);
}

//===========================================================================
//
// 
//
//===========================================================================
static int Cal_ChangeState(CCalApp * pme, int nState, boolean bRevert)
{
   // if we are backing up to previous history item...

   pme->m_nPendingState = 0;

   if (bRevert) {
      if (pme->m_nHistoryCount > 0)
         nState = pme->m_arHistory[--pme->m_nHistoryCount];
      else
         nState = S_MAINMENU;
   }
   else {
      int nCurState = pme->m_nCurrentState;
      // execute "End State" function
      if (nCurState >= S_MAINMENU && nCurState < NUMSTATES)
         Cal_EndState(pme,nCurState);

      // returning to main menu flushes the history stack
      if (nState == S_MAINMENU)
         pme->m_nHistoryCount = 0;
      else {
         // save the new state in the stack
         if (pme->m_nHistoryCount < MAXHISTORY) {
            pme->m_arHistory[pme->m_nHistoryCount] = pme->m_nCurrentState;
            pme->m_nHistoryCount++;
         }
      }
   }

   pme->m_nCurrentState = nState;

   // clear the screen
   Cal_ClearScreen(pme);

   // deactivate all other controls
   Cal_ResetControls(pme);

   // execute "Begin State" function
   if (nState >= S_MAINMENU && nState < NUMSTATES) {
      if (Cal_BeginState(pme,nState))
         return 1;
   }
   return 0;
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_ForceState(CCalApp * pme, int nState)
{
   // force state to new state

   pme->m_nCurrentState = nState;

   // goto new "Begin State" function

   if (nState >= S_MAINMENU && nState < NUMSTATES){
      Cal_ResetControls(pme);
      Cal_BeginState(pme,nState);
   }
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_ResetControls(CCalApp * pme)
{
   AEERect rc;

   ISHELL_CancelTimer(pme->a.m_pIShell, (PFNNOTIFY)(Cal_Return), pme);
   ISHELL_CancelTimer(pme->a.m_pIShell, (PFNNOTIFY)(Cal_DeactivateAlarm), pme);

   pme->m_pfnKeyHandler = NULL;
   pme->m_nCtlsOnScreen = 0;
   pme->m_pCtl[0] = NULL;

   // deactivate all other controls
   IDATECTL_Reset(pme->m_pIDatePick);
   IDATECTL_Reset(pme->m_pIDate);
   IMENUCTL_Reset(pme->m_pIMenu);
   IMENUCTL_SetProperties(pme->m_pIMenu, 0);
   IMENUCTL_Reset(pme->m_pISoftKey);
   ITEXTCTL_Reset(pme->m_pIText);

   ITIMECTL_Reset(pme->m_pITimeStart);
   ITIMECTL_Reset(pme->m_pITimeEnd);
   ITIMECTL_Reset(pme->m_pIDuration);

   // time controls doesn't properly reset...
   ITIMECTL_SetActive(pme->m_pITimeStart, 0);
   ITIMECTL_SetActive(pme->m_pITimeEnd, 0);

   IMENUCTL_Reset(pme->m_pIList);

   // this control occasionally changes it's rectangle
   SETAEERECT(&rc, 0, 0, pme->m_cx, pme->m_cy);
   IMENUCTL_SetRect(pme->m_pIMenu, &rc);
   // and sometimes disables commands
   IMENUCTL_EnableCommand(pme->m_pIMenu, 1);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_MenuAddState(CCalApp * pme, IMenuCtl * pIMenuCtl, int nState, 
                            AECHAR * pString, int16 nIDStr, uint32 lData)
{
   const char * pszResource = CALAPP_RES_FILE;

   if (pString) {
      nIDStr = 0;
      pszResource = NULL;
   }
   else     // !pString
   if (0 == nIDStr && nState >= 0 && nState < NUMSTATES)
      nIDStr = garIDs[nState];

   if (pString || (nIDStr && pszResource))
      IMENUCTL_AddItem(pIMenuCtl, pszResource, nIDStr, (int16)nState, pString, lData);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_DisplayCurrentDay(CCalApp * pme)
{
   AEEImageInfo   bi;
   IImage *       pLeft = NULL;
   IImage *       pRight = NULL;
   AECHAR         buffer[100];
   int            nChars;
   int            x = 1;

   // open the resource file and get the arrow bitmaps

   pLeft  = ISHELL_LoadResImage(pme->a.m_pIShell, CALAPP_RES_FILE, IDB_LEFTARROW);
   pRight = ISHELL_LoadResImage(pme->a.m_pIShell, CALAPP_RES_FILE, IDB_RIGHTARROW);

	if(!pRight || !pLeft)
		return;

   // erase the previous date
   IDISPLAY_EraseRgn(pme->a.m_pIDisplay, 0, 0, pme->m_cx, LINEHEIGHT);

   IIMAGE_GetInfo(pRight, &bi);

	// Display left arrow...

   VCenterBlt(pLeft, 0, 0, LINEHEIGHT,bi.cy);
   x = bi.cx + 2;
   IIMAGE_Release(pLeft);

   // the date-pick control can be used as a date conversion object
   IDATECTL_SetJulianDay(pme->m_pIDatePick, pme->m_lCurrentDay);

   IDATECTL_GetDayString(pme->m_pIDatePick, buffer, sizeof(buffer), &nChars);
   Cal_DisplayString(pme, buffer, ABBREV_LEN, x, 2, AEE_FONT_BOLD, 0);
      
   IDATECTL_GetDateString(pme->m_pIDatePick, buffer, sizeof(buffer), &nChars, DFMT_DD_MON_YY);
   x = (pme->m_cx - 3) - bi.cx;
   Cal_DisplayString(pme, buffer, nChars, x, 2, AEE_FONT_BOLD, 1);

   // Display right arrow

   VCenterBlt(pRight, pme->m_cx - bi.cx - 1, 0, LINEHEIGHT, bi.cy);
   IIMAGE_Release(pRight);

   IDISPLAY_Update(pme->a.m_pIDisplay);
}

//===========================================================================
//
// 
//
//===========================================================================
static AECHAR * Cal_LoadResString(CCalApp * pme, int nIDStr, AECHAR * pszBuf, int nBufSize, int * pnCopied)
{
   int nCopied;

   nCopied = ISHELL_LoadResString(pme->a.m_pIShell, CALAPP_RES_FILE, (int16)nIDStr, pszBuf, nBufSize);

   if(pnCopied)
      *pnCopied = nCopied;

   return(pszBuf + nCopied);
}

//===========================================================================
//
// 
//
//===========================================================================
static boolean Cal_DisplayString(CCalApp * pme, AECHAR * psz, int nLen, int x, int y, int nFont, boolean bRightAlign)
{
   uint32   dwFlags = 0;

   if (nLen < 0)
      nLen = WSTRLEN(psz);

   if (nLen > 0) {
      if (bRightAlign)
         x -= IDISPLAY_MeasureTextEx(pme->a.m_pIDisplay, (AEEFont)nFont, psz, nLen, -1, NULL);

      if(x == -1)
         dwFlags |= IDF_ALIGN_CENTER;

      if(y == -1)
         dwFlags |= IDF_ALIGN_MIDDLE;

      IDISPLAY_DrawText(pme->a.m_pIDisplay, (AEEFont)nFont, psz, nLen, x, y, NULL,dwFlags);
      return TRUE;
   }
   return FALSE;
}

//===========================================================================
//
// 
//
//===========================================================================
static boolean Cal_DisplayResString(CCalApp * pme, int nIDStr, int x, int y, int nFont, boolean bRightAlign)
{
   AECHAR buffer[100];
   int nLen;

   Cal_LoadResString(pme, nIDStr, buffer, sizeof(buffer)/sizeof(AECHAR), &nLen);
   return Cal_DisplayString(pme, buffer, nLen, x, y, nFont, bRightAlign);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_MainMenu(CCalApp * pme)
{
   // no current event 
   pme->m_pceCurrent = NULL;

   IMENUCTL_SetTitle(pme->m_pIMenu, CALAPP_RES_FILE, IDS_CALENDAR, NULL);

   pme->m_lDuration = pme->m_lStartTime = 0;

   Cal_MenuAddState(pme, pme->m_pIMenu, S_VIEWDAY,       NULL, 0, 0);
   Cal_MenuAddState(pme, pme->m_pIMenu, S_VIEWMONTH,     NULL, 0, 0);
   Cal_MenuAddState(pme, pme->m_pIMenu, S_ADDEVENT,      NULL, 0, 0);
   Cal_MenuAddState(pme, pme->m_pIMenu, S_OPTIONS,       NULL, 0, 0);

   pme->m_nReturnState = S_MAINMENU;

   IMENUCTL_SetActive(pme->m_pIMenu, 1);
}


//===========================================================================
//
// 
//
//===========================================================================
static boolean Cal_ViewDayKeyHandler(CCalApp * pme, int nKeyCode)
{
   switch (nKeyCode) {
      
      case AVK_RIGHT:
         pme->m_lCurrentDay++;
         Cal_ResetControls(pme);
         Cal_ClearScreen(pme);
         Cal_ViewDay(pme);
         return TRUE;
      
      case AVK_LEFT:
         pme->m_lCurrentDay--;
         Cal_ResetControls(pme);
         Cal_ClearScreen(pme);
         Cal_ViewDay(pme);
         return TRUE;
   }      
   return FALSE;
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_FillEmpty(CCalApp * pme, uint16 wStart, uint16 wEnd, uint16  * pwID)
{
   uint16      wDur, wID, wHourMin;
   AECHAR  szBuff[1];
   int16       nDurTotal;

   *szBuff = 0;

   wID = *pwID;

   if(wStart == 0){
      wStart = pme->m_cfg.wDayStart;
      if(wStart >= wEnd)
         return;
   }

   if(wEnd == MAXTIME){
      wEnd = pme->m_cfg.wDayEnd;
      if(wEnd <= wStart)
         return;
   }

   nDurTotal = (int16)(wEnd - wStart);

   while(nDurTotal > 0){
      wHourMin = ((wStart / 60) + 1) * 60;
      if(wHourMin <= wEnd)
         wDur = wHourMin - wStart;
      else
         wDur = wEnd - wStart;

      Cal_MenuAddState(pme, pme->m_pIMenu, wID, szBuff, 0, (uint32)0);
      IMENUCTL_SetItemTime(pme->m_pIMenu, wID++, wStart, wDur);
      wStart += wDur;
      nDurTotal -= (int16)wDur;
   }

   *pwID = wID;
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_ViewDay(CCalApp * pme)
{
   CalEvent *        pce;
   CalEvent *        pcePrev;
   CalEvent *        pceFirst;
   IDBRecord  *      pRec;
   AECHAR        szBuff[32];
   uint16            wMin,wID,wEndLast;
   CalDBRecFields    f;
   AEERect             rc;
   uint16            wEnd,nFirstSel = 0;
   uint32            lDateTime,dwProps;

   pme->m_pfnKeyHandler = Cal_ViewDayKeyHandler;

   // no current event 
   pme->m_pceCurrent = NULL;

   dwProps = MP_CALENDAR;

   lDateTime = Cal_GetCurrentDateTime(pme);

   if(lDateTime && (long)GETDATE(lDateTime) == pme->m_lCurrentDay)
      dwProps |= MP_AUTOSCROLLTIME;

   IMENUCTL_SetProperties(pme->m_pIMenu, dwProps);

   Cal_DisplayCurrentDay(pme);
  
   pme->m_nReturnState = S_VIEWDAY;

   // for all events found that match pme->m_lCurrentDay, add to menu

   SETAEERECT(&rc, 0, LINEHEIGHT, pme->m_cx, pme->m_cy-LINEHEIGHT);
   IMENUCTL_SetRect(pme->m_pIMenu, &rc);

   // For each hour...
   //
   // - Check to see if there are event(s)
   // - Add them in order
   // - If no events, then add the empty time...

   wID      = S_VIEWEVENT + 1;
   pcePrev  = NULL;
   pce      = Cal_FindFirst(pme, MAKECOMPTIME(pme->m_lCurrentDay,0));
   wMin     = wEndLast = 0;

   for(pceFirst = NULL; pce != pceFirst ; pce = pce->pNext){

      if((uint32)GETDATE(pce->m_b.dwTime) != (uint32)pme->m_lCurrentDay)
         break;

      if(!pceFirst)
         pceFirst = pce;
      
      wMin = (uint16)GETTIME(pce->m_b.dwTime);

      if(pcePrev){
         wEnd = (uint16)GETTIME(pcePrev->m_b.dwTime) + pcePrev->m_b.wDuration;
         if(wEnd > wEndLast)
            wEndLast = wEnd;
      }
      else
         wEndLast = 0;
   
      if(wMin > wEndLast)
         Cal_FillEmpty(pme, wEndLast, wMin, &wID);

      pRec = IDATABASE_GetRecordByID(pme->m_pIDatabase, pce->m_nRecID);

      if(!pRec)
         break;

      MEMSET(&f, 0, sizeof(CalDBRecFields));
      f.pszDesc = szBuff;
      f.nDescLen = sizeof(szBuff) / sizeof(AECHAR);
      CalReadRec(pRec, &f);
      IDBRECORD_Release(pRec);
      Cal_MenuAddState(pme, pme->m_pIMenu, wID, szBuff, 0, (uint32)pce);
      IMENUCTL_SetItemTime(pme->m_pIMenu, wID, wMin, pce->m_b.wDuration);

      if(!nFirstSel)
         nFirstSel = wID;

      wID++;
      pcePrev = pce;
   }

   if(pcePrev)
      wMin += pcePrev->m_b.wDuration;
   
   if(wMin < MAXTIME)
      Cal_FillEmpty(pme, wMin, MAXTIME, &wID);

   // If it's not gonna auto-scroll, then select the first event...

   if(!(dwProps & MP_AUTOSCROLLTIME) && nFirstSel)
      IMENUCTL_SetSel(pme->m_pIMenu, nFirstSel);

   // Automatically sets selection if current day...

   IMENUCTL_SetActive(pme->m_pIMenu, 1);
}

//===========================================================================
//
// 
//
//===========================================================================
static boolean Cal_ViewMonthCB(CalEvent * pce, ENUMDAYDATA * pEnumData)
{
   CCalApp * pme = pEnumData->pme;
   int         ny, nm, nd, ny2, nm2, nd2;

   IDATECTL_SetJulianDay(pme->m_pIDatePick, GETDATE(pce->m_b.dwTime));
   IDATECTL_GetDate(pme->m_pIDatePick, &ny, &nm, &nd);

   IDATECTL_SetJulianDay(pme->m_pIDatePick, pme->m_lCurrentDay);
   IDATECTL_GetDate(pme->m_pIDatePick, &ny2, &nm2, &nd2);

   if(ny == ny2 && nm == nm2)
      pme->m_dwMask |= 1 << (nd - 1);
   return FALSE;
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_ViewMonth(CCalApp * pme)
{
   // no current event 
   pme->m_pceCurrent = NULL;

   Cal_DetermineDayMask(pme);

   IDATECTL_SetJulianDay(pme->m_pIDatePick, pme->m_lCurrentDay);
   IDATECTL_EnableCommand(pme->m_pIDatePick, TRUE, S_VIEWDAY);
   IDATECTL_SetActive(pme->m_pIDatePick, TRUE);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_DetermineDayMask(CCalApp * pme)
{
   uint32   lSave = 0, lJDate = 0;
   int      nMonth, nYear, nDay;
   IDateCtl * pd = pme->m_pIDatePick;

   // Get the current day and use it to get the julian date for the fist day
   // of the month.  

   lSave = pme->m_lCurrentDay;
   lJDate = IDATECTL_GetJulianDay(pd);
   IDATECTL_GetDate(pd, &nYear, &nMonth, &nDay);
   IDATECTL_SetDate(pd, nYear, nMonth, 1);
   pme->m_lCurrentDay = IDATECTL_GetJulianDay(pd);

   pme->m_dwMask = 0;
   Cal_EnumDays(pme, pme->m_lCurrentDay, Cal_ViewMonthCB);
   IDATECTL_SetActiveDayMask(pd, pme->m_dwMask);;

   // Restore the date...

   IDATECTL_SetJulianDay(pd, lJDate);
   pme->m_lCurrentDay = lSave;
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_EndViewMonth(CCalApp * pme)
{
   pme->m_lCurrentDay = IDATECTL_GetJulianDay(pme->m_pIDatePick);
}


//===========================================================================
//
// 
//
//===========================================================================
static void Cal_ViewEvent(CCalApp * pme)
{
   if(!pme->m_pceCurrent)
      Cal_ChangeState(pme, S_MAINMENU, TRUE);
   else
      Cal_ViewEventRecord(pme, pme->m_pceCurrent->m_nRecID, FALSE);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_ViewEventRecord(CCalApp * pme, uint16 wID, boolean bAlert)
{
   AECHAR     szBuff[32];
   AECHAR *   psz;
   long           lTime;
   int nRemaining = sizeof(szBuff) / sizeof(AECHAR);
   IDBRecord *    pIDBRecord = NULL;
   AEERect          rc;
   CalDBRecFields f;
   AECHAR     *pszText;
   uint16         wDescLen, wLen;

   // set the return state
   pme->m_nReturnState = S_VIEWEVENT;

   // get the IDBRecord
   pIDBRecord = IDATABASE_GetRecordByID(pme->m_pIDatabase, wID);

   MEMSET(&f, 0, sizeof(CalDBRecFields));

   f.pszDesc = pme->m_szEventName;
   f.nDescLen = sizeof(pme->m_szEventName);

   f.pszNote = pme->m_szNote;
   f.nNoteLen = sizeof(pme->m_szNote) / sizeof(AECHAR);

   if(pIDBRecord && CalReadRec(pIDBRecord, &f)){

      IDBRECORD_Release(pIDBRecord);

      IMENUCTL_GetRect(pme->m_pISoftKey, &rc);
      SETAEERECT(&rc, 0, 0, pme->m_cx, pme->m_cy-rc.dy);
      ITEXTCTL_SetRect(pme->m_pIText, &rc);
      ITEXTCTL_SetProperties(pme->m_pIText, TP_FRAME|TP_MULTILINE);

   // format the event start & end time

      lTime = GETTIME(f.b.dwTime);
      psz = Cal_WriteTime(pme, szBuff, lTime, &nRemaining);
      if (nRemaining > 3) {
         *(psz++) = ' ';
         *(psz++) = '-';
         *(psz++) = ' ';
         nRemaining = psz - szBuff;
      }
      lTime += f.b.wDuration;
      psz = Cal_WriteTime(pme, psz, lTime, &nRemaining);
      ITEXTCTL_SetTitle(pme->m_pIText, NULL, 0, szBuff);

      wDescLen = WSTRLEN(f.pszDesc);
      wLen = (wDescLen + WSTRLEN(f.pszNote) + 2);
      if((pszText = (AECHAR *)MALLOC(wLen * sizeof(AECHAR))) != NULL){
         MEMCPY(pszText, f.pszDesc, wDescLen * sizeof(AECHAR));
         pszText[wDescLen] = (AECHAR)'\n';
         MEMCPY(pszText + wDescLen + 1, f.pszNote, WSTRLEN(f.pszNote) * sizeof(AECHAR));
         ITEXTCTL_SetText(pme->m_pIText, pszText, -1);
         FREE(pszText);
      }
   }
   
   if(!bAlert){
      Cal_MenuAddState(pme, pme->m_pISoftKey, S_EDITEVENT, NULL, IDS_EDIT, 0);
      Cal_MenuAddState(pme, pme->m_pISoftKey, S_ERASECONFIRM, NULL, IDS_ERASE, 0);
      Cal_MenuAddState(pme, pme->m_pISoftKey, S_ADDEVENT, NULL, IDS_NEW, 0);
   }
   else{
      Cal_MenuAddState(pme, pme->m_pISoftKey, S_MAINMENU, NULL, IDC_OK, 0);
      ISHELL_SetTimer(pme->a.m_pIShell, MAX_ALARM_MSECS, (PFNNOTIFY)(Cal_DeactivateAlarm), pme);
      Cal_SoundAlarm(pme);
   }

   IMENUCTL_SetActive(pme->m_pISoftKey, 1);

   // Force an update - in case SetActive did nothing (already active)

   IDISPLAY_Update(pme->a.m_pIDisplay);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_SoundAlarm(CCalApp * pme)
{
   if(!ISHELL_Beep(pme->a.m_pIShell, BEEP_ALERT, TRUE))
      ISHELL_Beep(pme->a.m_pIShell, BEEP_VIBRATE_ALERT, FALSE);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_DeactivateAlarm(CCalApp * pme)
{
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_Options(CCalApp * pme)
{
   IMENUCTL_SetTitle(pme->m_pIMenu, CALAPP_RES_FILE, IDS_OPTIONS, NULL);
   Cal_MenuAddState(pme, pme->m_pIMenu, S_OPTHOURS, NULL, 0, 0);
   Cal_MenuAddState(pme, pme->m_pIMenu, S_OPTAUTODELETE, NULL, 0, 0);
   Cal_MenuAddState(pme, pme->m_pIMenu, S_OPTALARMTIME, NULL, 0, 0);
   IMENUCTL_SetActive(pme->m_pIMenu, 1);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_OptHours(CCalApp * pme)
{
   AEERect          qrc,rcm;
   int            nPad, nExtra, nOccupied,nLineHeight;
   boolean        bSoftKey;

   // Determine the layout of the controls...

   Cal_DisplayResString(pme, IDS_DAYTIME, 0, 0, AEE_FONT_BOLD, 0);

   nLineHeight = LINEHEIGHT;
   pme->m_nCtlsOnScreen = 2;

   nOccupied = (nLineHeight * 3) + 1;
   nExtra = pme->m_cy - nOccupied;

   IMENUCTL_GetRect(pme->m_pISoftKey, &rcm);

   bSoftKey = FALSE;
   nPad  = 0;

   if(nExtra > 3){
      if(nExtra >= rcm.dy){
         nExtra -= rcm.dy;
         bSoftKey = TRUE;
      }
      nPad = nExtra / 3;
   }
      
   SETAEERECT(&qrc, 0, nLineHeight + nPad, pme->m_cx, nLineHeight);

   Cal_SetupTimeCtl(pme, pme->m_pITimeStart, IDS_START, &qrc, (uint32)pme->m_cfg.wDayStart);
   pme->m_pCtl[0] = (IControl *)pme->m_pITimeStart;

   qrc.y += qrc.dy + nPad;

   Cal_SetupTimeCtl(pme, pme->m_pITimeEnd, IDS_END, &qrc, (uint32)pme->m_cfg.wDayEnd);
   pme->m_pCtl[1] = (IControl *)pme->m_pITimeEnd;

   // Set the default control to be the start time...

   ITIMECTL_SetActive(pme->m_pITimeStart, TRUE);

   ITIMECTL_Redraw(pme->m_pITimeStart);
   ITIMECTL_Redraw(pme->m_pITimeEnd);

   if(bSoftKey){
      Cal_MenuAddState(pme, pme->m_pISoftKey, S_RETURN, NULL, IDS_OK, 0);
      IMENUCTL_SetActive(pme->m_pISoftKey, 1);
   }

   IDISPLAY_Update(pme->a.m_pIDisplay);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_EndOptHours(CCalApp * pme)
{
   uint16 wStart, wEnd;

   wStart  = (uint16)(ITIMECTL_GetTime(pme->m_pITimeStart) / MSECS_PER_MINUTE);
   wEnd    = (uint16)(ITIMECTL_GetTime(pme->m_pITimeEnd) / MSECS_PER_MINUTE);

   if(wStart < wEnd){
      pme->m_cfg.wDayStart = wStart;
      pme->m_cfg.wDayEnd = wEnd;
      ISHELL_SetPrefs(pme->a.m_pIShell, AEECLSID_CAL, CAL_CFG_VERSION, &pme->m_cfg, sizeof(pme->m_cfg));
   }
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_OptAutoDelete(CCalApp * pme)
{
   Cal_FillResMenu(pme, pme->m_pIMenu,IDS_AUTODELS, IDS_AUTODELETE, (int16)pme->m_cfg.nAutoDelete);
   IMENUCTL_SetActive(pme->m_pIMenu, 1);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_EndOptAutoDelete(CCalApp * pme)
{
   uint32   lData;

   IMENUCTL_GetItemData(pme->m_pIMenu, IMENUCTL_GetSel(pme->m_pIMenu), &lData);

   pme->m_cfg.nAutoDelete = (uint16)lData;

   ISHELL_SetPrefs(pme->a.m_pIShell, AEECLSID_CAL, CAL_CFG_VERSION, &pme->m_cfg, sizeof(pme->m_cfg));
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_OptAlarm(CCalApp * pme)
{
   Cal_FillAlarmList(pme, pme->m_pIMenu, IDS_DEFALARM, pme->m_cfg.nAlarmTime);
   IMENUCTL_SetActive(pme->m_pIMenu, 1);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_EndOptAlarm(CCalApp * pme)
{
   uint16   nSel;
   uint32   dwAlarm;

   nSel = IMENUCTL_GetSel(pme->m_pIMenu);

   if(nSel != -1 && IMENUCTL_GetItemData(pme->m_pIMenu, nSel, &dwAlarm)){
      pme->m_cfg.nAlarmTime = (int16)dwAlarm;
      ISHELL_SetPrefs(pme->a.m_pIShell, AEECLSID_CAL, CAL_CFG_VERSION, &pme->m_cfg, sizeof(pme->m_cfg));
   }
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_EraseConfirm(CCalApp * pme)
{
   Cal_DisplayResString(pme, IDS_ERASECONFIRM, 0,  ((pme->m_cy - (LINEHEIGHT * 2)) / 2), AEE_FONT_BOLD, 0);

   Cal_MenuAddState(pme, pme->m_pISoftKey, S_ERASEEVENT, NULL,IDS_YES , 0);
   Cal_MenuAddState(pme, pme->m_pISoftKey, S_RETURN, NULL, IDS_NO, 0);

   IMENUCTL_SetActive(pme->m_pISoftKey, 1);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_EraseEvent(CCalApp * pme)
{
   CalEvent * pce = pme->m_pceCurrent;

   if (pce) {
      IDBRecord * pidb = IDATABASE_GetRecordByID(pme->m_pIDatabase, pce->m_nRecID);
      IDBRECORD_Remove(pidb);
      DLL_REMOVE(&pme->m_pEventList, pce);
      FREE(pce);
   }

   Cal_ShowMessage(pme, IDS_ERASED, S_VIEWDAY);
}


//===========================================================================
//
// 
//
//===========================================================================
static void Cal_EditEvent(CCalApp * pme)
{
   Cal_InitEventData(pme, pme->m_pceCurrent);
   Cal_ForceState(pme, S_ENTERNAME);   
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_AddEvent(CCalApp * pme)
{
   pme->m_pceCurrent = NULL;
   Cal_InitEventData(pme, NULL);
   Cal_ForceState(pme, S_ENTERNAME);   
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_InitEventData(CCalApp * pme, CalEvent * pce)
{
   if (pce) {
      IDBRecord * pIDBRecord = NULL;
      CalDBRecFields f;

      // get the IDBRecord
      pIDBRecord = IDATABASE_GetRecordByID(pme->m_pIDatabase, pce->m_nRecID);
      // reset the dbrecord enum position

      MEMSET(&f, 0, sizeof(CalDBRecFields));
      f.pszDesc = pme->m_szEventName;
      f.nDescLen = sizeof(pme->m_szEventName);
      f.pszNote = pme->m_szNote;
      f.nNoteLen = sizeof(pme->m_szNote) / sizeof(AECHAR);

      if(!CalReadRec(pIDBRecord, &f))
         return;

      IDBRECORD_Release(pIDBRecord);

      pme->m_lEventDay     = GETDATE(f.b.dwTime);
      pme->m_lStartTime    = GETTIME(f.b.dwTime);
      pme->m_lDuration     = f.b.wDuration;
      pme->m_nAlarmTime    = f.b.wAlarmTime;
      pme->m_bSetAlarm     = (f.b.wAlarmTime != 0 ? TRUE : FALSE);
     
   }
   else {

      long   lDateTime;

      if(!pme->m_lDuration){
         lDateTime = Cal_GetCurrentDateTime(pme);
         if(lDateTime && (long)GETDATE(lDateTime) == pme->m_lCurrentDay)
            pme->m_lStartTime = ((GETTIME(lDateTime) + (MINUTES_PER_HOUR - 1)) / MINUTES_PER_HOUR) * MINUTES_PER_HOUR;
         else
            pme->m_lStartTime = 8 * MINUTES_PER_HOUR;
         pme->m_lDuration = MINUTES_PER_HOUR;
      }

      pme->m_szEventName[0]   = 0;
      pme->m_szNote[0]        = 0;
      pme->m_lEventDay        = pme->m_lCurrentDay;
      pme->m_nAlarmTime       = pme->m_cfg.nAlarmTime;
      pme->m_bSetAlarm        = (pme->m_cfg.nAlarmTime != 0 ? TRUE : FALSE);
   }
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_EnterName(CCalApp * pme)
{
   AEERect rc;

   // get the softkey rectangle so we can set the text control's rectangle

   IMENUCTL_GetRect(pme->m_pISoftKey, &rc);

   SETAEERECT(&rc, 0, 0, pme->m_cx, pme->m_cy-rc.dy);
   ITEXTCTL_SetRect(pme->m_pIText, &rc);
   ITEXTCTL_SetProperties(pme->m_pIText, TP_FRAME|TP_T9_MODE|TP_MULTILINE);
   ITEXTCTL_SetText(pme->m_pIText, pme->m_szEventName, -1);
   ITEXTCTL_SetTitle(pme->m_pIText, CALAPP_RES_FILE, (uint16)(pme->m_pceCurrent ? IDS_EVENTNAME : IDS_NEWNAME), NULL);

   Cal_MenuAddState(pme, pme->m_pISoftKey, S_ALARMTIME, NULL, 0, 0);

   ITEXTCTL_SetSoftKeyMenu(pme->m_pIText, pme->m_pISoftKey);
   ITEXTCTL_SetActive(pme->m_pIText, 1);
   IMENUCTL_Redraw(pme->m_pISoftKey);
}


//===========================================================================
//
// 
//
//===========================================================================
static void Cal_EndEnterName(CCalApp * pme)
{
   ITEXTCTL_GetText(pme->m_pIText, pme->m_szEventName, MAXTEXT);

   if(!pme->m_szEventName[0])
      ISHELL_LoadResString(pme->a.m_pIShell, CALAPP_RES_FILE,IDS_UNKNOWN,pme->m_szEventName, sizeof(pme->m_szEventName) / sizeof(AECHAR));
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_SetupTimeCtl(CCalApp * pme, ITimeCtl * pt, uint16 wID, AEERect * prc, uint32 lTime)
{
   AECHAR  szBuff[16];

   Cal_LoadResString(pme, wID, szBuff, sizeof(szBuff),NULL);
   IDISPLAY_DrawText(pme->a.m_pIDisplay, AEE_FONT_NORMAL, (const AECHAR *)szBuff, -1, 0,prc->y,NULL,0);
   prc->x = IDISPLAY_MeasureText(pme->a.m_pIDisplay, AEE_FONT_NORMAL, szBuff) + 5;
   prc->dx = pme->m_cx - prc->x;
   ITIMECTL_SetRect(pt, prc);
   ITIMECTL_SetTime(pt, lTime * MSECS_PER_MINUTE);
   ITIMECTL_EnableCommand(pt, TRUE, S_ENTERNOTE);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_AlarmTime(CCalApp * pme)
{
   AEERect          qrc,rcm;
   int            nPad, nExtra, nOccupied,nLineHeight;
   boolean        bSoftKey;
   boolean        bActive;

   // Determine the layout of the controls...

   nLineHeight = LINEHEIGHT;
   pme->m_nCtlsOnScreen = 3;

   nOccupied = (nLineHeight * 3) + 1;
   nExtra = pme->m_cy - nOccupied;

   IMENUCTL_GetRect(pme->m_pISoftKey, &rcm);

   bSoftKey = FALSE;
   nPad  = 0;

   if(nExtra > 3){
      if(nExtra >= rcm.dy){
         nExtra -= rcm.dy;
         bSoftKey = TRUE;
      }
      nPad = nExtra / 3;
   }
      
   SETAEERECT(&qrc, 0, 0, pme->m_cx, nLineHeight);

   Cal_SetupTimeCtl(pme, pme->m_pITimeStart, IDS_START, &qrc, pme->m_lStartTime);
   pme->m_pCtl[0] = (IControl *)pme->m_pITimeStart;

   qrc.y += qrc.dy + nPad;

   Cal_SetupTimeCtl(pme, pme->m_pIDuration, IDS_DURATION, &qrc, pme->m_lDuration);
   pme->m_pCtl[1] = (IControl *)pme->m_pIDuration;

   qrc.x = 0;
   qrc.dx = pme->m_cx - 2;
   qrc.y += qrc.dy + nPad;
   IMENUCTL_SetRect(pme->m_pIList, &qrc);

   // The menu should be a standard menu that's the size of the screen...

   bActive = TRUE;

   Cal_FillAlarmList(pme, pme->m_pIList, IDS_ALARMTIME, pme->m_nAlarmTime);

   pme->m_pCtl[2] = (IControl *)pme->m_pIList;

   // Set the default control to be the start time...

   ITIMECTL_SetActive(pme->m_pIDuration, TRUE);

   ITIMECTL_Redraw(pme->m_pITimeStart);
   ITIMECTL_Redraw(pme->m_pIDuration);
   IMENUCTL_Redraw(pme->m_pIList);

   if(bSoftKey){
      Cal_MenuAddState(pme, pme->m_pISoftKey, S_ENTERNOTE, NULL, 0, 0);
      IMENUCTL_SetActive(pme->m_pISoftKey, 1);
   }

   IDISPLAY_Update(pme->a.m_pIDisplay);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_FillAlarmList(CCalApp * pme, IMenuCtl * pm, uint16 wTitleID, int16 nCurr)
{
   Cal_FillResMenu(pme, pm, IDS_ALARMS, wTitleID,nCurr);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_FillResMenu(CCalApp * pme, IMenuCtl * pm, uint16 wID, uint16 wTitleID, int16 nCurr)
{
   int16          nSel = 0;
   int16          i;
   int            nTime;
   AECHAR *       pszList;
   AECHAR *       psz;
   AECHAR *       pszNum;
   AECHAR *       pszNext;
   int            nSize;
   char           szNum[20];

   if(wTitleID)
      IMENUCTL_SetTitle(pm, CALAPP_RES_FILE, wTitleID, NULL);
   
   nSize = 200;
   pszList = (AECHAR *)MALLOC(nSize);
   if(!pszList)
      return;

   ISHELL_LoadResString(pme->a.m_pIShell, CALAPP_RES_FILE, wID, pszList, nSize);

   for(i = 0, psz = pszList; psz != NULL;i++){
      pszNum = FindNextToken(psz, (AECHAR)',');
      if(!pszNum)
         break;
      *pszNum = 0;
      pszNum++;
      pszNext = FindNextToken(pszNum, (AECHAR)'|');
      if(pszNext){
         *pszNext = 0;
         pszNext++;
      }

      WSTR_TO_STR(pszNum, szNum, sizeof(szNum));
      nTime = ATOI(szNum);

      IMENUCTL_AddItem(pm, NULL, 0, i, psz, (uint32)nTime);

      if(nTime == nCurr)
         nSel = i;
      i++;
      psz = pszNext;
   }

   IMENUCTL_SetSel(pm, nSel);
   ISHELL_FreeResData(pme->a.m_pIShell, pszList);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_EndAlarmTime(CCalApp * pme)
{
   uint32   lData = 0;
   long     lNowDay, lDateTime;

   pme->m_lStartTime = (ITIMECTL_GetTime(pme->m_pITimeStart) / MSECS_PER_MINUTE);
   pme->m_lDuration = (ITIMECTL_GetTime(pme->m_pIDuration) / MSECS_PER_MINUTE);
   IMENUCTL_GetItemData(pme->m_pIList, IMENUCTL_GetSel(pme->m_pIList), &lData);

   // Do not set an alarm unless it's appropriate

   lDateTime = Cal_GetCurrentDateTime(pme);
   lNowDay = GETDATE(lDateTime);

   if(lNowDay > (long)pme->m_lEventDay || (lNowDay == (long)pme->m_lEventDay && (long)GETTIME(lDateTime) >= (long)pme->m_lStartTime)){
      pme->m_bSetAlarm = FALSE;
      pme->m_nAlarmTime = 0;
   }
   else{
      pme->m_bSetAlarm = (lData == 0 ? FALSE : TRUE);
      pme->m_nAlarmTime = (uint16)lData;
   }
}


//===========================================================================
//
// 
//
//===========================================================================
static void Cal_EnterNote(CCalApp * pme)
{
   AEERect rc;
   
   ITEXTCTL_SetTitle(pme->m_pIText, CALAPP_RES_FILE, IDS_NOTE, NULL);
   IMENUCTL_GetRect(pme->m_pISoftKey, &rc);
   SETAEERECT(&rc, 0, 0, pme->m_cx, pme->m_cy-rc.dy);
   ITEXTCTL_SetRect(pme->m_pIText, &rc);
   ITEXTCTL_SetProperties(pme->m_pIText, TP_FRAME|TP_MULTILINE|TP_T9_MODE);
   //ITEXTCTL_SetMaxLength...
   ITEXTCTL_SetText(pme->m_pIText, pme->m_szNote, -1);
   Cal_MenuAddState(pme, pme->m_pISoftKey, S_EVENTADDED, NULL, 0, 0);
   ITEXTCTL_SetSoftKeyMenu(pme->m_pIText, pme->m_pISoftKey);
   ITEXTCTL_SetActive(pme->m_pIText, 1);
   IMENUCTL_Redraw(pme->m_pISoftKey);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_EndEnterNote(CCalApp * pme)
{
   CalBaseFields b;

   ITEXTCTL_GetText(pme->m_pIText, pme->m_szNote, MAXTEXT);

   b.wDuration = (uint16)pme->m_lDuration;
   b.dwTime = MAKECOMPTIME(pme->m_lEventDay,pme->m_lStartTime);
   b.wAlarmTime = pme->m_nAlarmTime;

#if defined(ALARM_TEST)
   b.dwTime = Cal_GetCurrentDateTime(pme) + 2;
   b.wAlarmTime = 1;
#endif

   Cal_CreateUpdateRecord(pme, pme->m_pceCurrent, pme->m_szEventName, pme->m_szNote,&b);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_EventAdded(CCalApp * pme)
{
   Cal_ShowMessage(pme, (uint16)((pme->m_pceCurrent == NULL) ? IDS_EVENTADDED : IDS_EVENTCHANGED), S_VIEWDAY);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_ShowMessage(CCalApp * pme, uint16 wID, int nReturnState)
{
   Cal_DisplayResString(pme, wID, -1, -1, AEE_FONT_NORMAL, 0);

   pme->m_pceCurrent  = NULL;
   pme->m_nReturnState     = nReturnState;
   pme->m_nCurrentState    = S_MESSAGE;
   ISHELL_SetTimer(pme->a.m_pIShell, MESSAGE_TIMEOUT, (PFNNOTIFY)(Cal_Return), pme);

   IDISPLAY_Update(pme->a.m_pIDisplay);
}


//===========================================================================
//
// 
//
//===========================================================================
static void Cal_Return(CCalApp * pme)
{
   while(pme->m_nHistoryCount--)
      if (pme->m_arHistory[pme->m_nHistoryCount] == pme->m_nReturnState)
         break;

   if (pme->m_nHistoryCount == 0)
      pme->m_nReturnState = S_MAINMENU;

   Cal_ForceState(pme, pme->m_nReturnState);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_ShowAlert(CCalApp * pme)
{
   if (pme->m_wPendingAlarmID)
      Cal_ViewEventRecord(pme, pme->m_wPendingAlarmID, TRUE);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_ClearScreen(CCalApp * pme)
{
   IDISPLAY_EraseRgn(pme->a.m_pIDisplay, 0, 0, pme->m_cx, pme->m_cy);
}

//===========================================================================
//
// 
//
//===========================================================================
static CalEvent * Cal_FindFirst(CCalApp * pme, uint32 lDateTime)
{
   return (CalEvent*)DLL_SCAN(pme->m_pEventList, NULL, FindFirstCB, lDateTime);
}

//===========================================================================
//
// 
//
//===========================================================================
static CalEvent * Cal_FindPrev(CCalApp * pme, uint32 lDateTime)
{
   CalEvent * p = Cal_FindFirst(pme, lDateTime);

   if (!p) 
      p = pme->m_pEventList;

   if (p)
      p = p->pPrev;

   return p;
}

//===========================================================================
//
// 
//
//===========================================================================
static CalEvent * Cal_FindRecID(CCalApp * pme, uint16 nRecID)
{
   return (CalEvent*)DLL_SCAN(pme->m_pEventList, NULL, FindRecIDCB, (uint32)nRecID);
}

//===========================================================================
//
// 
//
//===========================================================================
static boolean Cal_EnumDays(CCalApp * pme, uint32 lDate, PFNENUMDAYCB pfnCB)
{
   CalEvent * pce = NULL;

   ENUMDAYDATA  enumdata;
   enumdata.pme      = pme;
   enumdata.lDate    = lDate;
   enumdata.bFound   = 0;
   enumdata.nCounter = 0;
   
   // for all events found that match pme->m_lCurrentDay, add to menu
   pce = Cal_FindFirst(pme, MAKECOMPTIME(lDate,0));
   DLL_SCAN(pce, NULL, pfnCB, &enumdata);
   return enumdata.bFound;
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_CreateUpdateRecord(CCalApp * pme, CalEvent * pCalEvent,
                               const AECHAR * pszDescription, 
                               const AECHAR * pszNote, 
                               CalBaseFields * pb)
{
   CalEvent *  pcePrev = NULL;
   boolean     bInsert = FALSE;
   IDBRecord * pIDBRecord = NULL;
   uint16      wMax;
   uint32      lAlarmTime;

   wMax = (uint16)(MAXTIME - (uint16)GETTIME(pb->dwTime));
   if(pb->wDuration > wMax)
      pb->wDuration = wMax;
   
   if (!pCalEvent) {
      if (NULL != (pCalEvent = (CalEvent *)MALLOC(sizeof(CalEvent)))) {
         // create a new record
         pIDBRecord = Cal_UpdateRecord(pme, NULL, pszDescription, pszNote,pb);

         if(pIDBRecord != NULL){
            pCalEvent->m_nRecID = IDBRECORD_GetID(pIDBRecord);
            IDBRECORD_Release(pIDBRecord);
            bInsert = TRUE;
         }
         else{
            FREE(pCalEvent);
            pCalEvent = NULL;
         }
      }
   }
   else {
      if (pCalEvent->m_b.dwTime != pb->dwTime) {
         // because date/time may have been changed, check if current 
         // position is still ok
         if (!CalEvent_IsInPosition(pCalEvent)) {
            // not in position, remove
            DLL_REMOVE(&pme->m_pEventList, pCalEvent);
            bInsert = TRUE;
         }
      }

      // update the existing record

      pIDBRecord = IDATABASE_GetRecordByID(pme->m_pIDatabase, pCalEvent->m_nRecID);
      if(pIDBRecord != NULL){
         pIDBRecord = Cal_UpdateRecord(pme, pIDBRecord, pszDescription,pszNote, pb);
         if(pIDBRecord)
            IDBRECORD_Release(pIDBRecord);
         else
            pCalEvent = NULL;
      }
      else
         pCalEvent = NULL;
   }


   // set the event date/time in the list node

   if (pCalEvent) {

      MEMCPY(&pCalEvent->m_b, pb, sizeof(CalBaseFields));

      if (pb->wAlarmTime){
         lAlarmTime = pb->dwTime - pb->wAlarmTime;
         if(lAlarmTime <= Cal_GetCurrentDateTime(pme))
            lAlarmTime = 0;
      }
      else
         lAlarmTime = 0;
      pCalEvent->m_lAlarmDateTime = lAlarmTime;
   
      // insert the CalEvent into list sorted order
      if (bInsert) {
         // scan list for insertion position
         pcePrev = Cal_FindPrev(pme, pb->dwTime);
         if (pcePrev && pcePrev->m_b.dwTime > pb->dwTime)
            pcePrev = NULL;      // insert at head
   
         // insert after pcePrev or at head if pcePrev == NULL
         DLL_INSERT(&pme->m_pEventList, pcePrev, pCalEvent);
      }
   
      if (pCalEvent->m_lAlarmDateTime)
         Cal_SetFirstAlarm(pme);
   }
}

//===========================================================================
//
// 
//
//===========================================================================
static uint32 Cal_GetCurrentDateTime(CCalApp * pme)
{
   uint32 lDate;
   uint32 lMin;
   uint32 lSeconds = ISHELL_GetSeconds(pme->a.m_pIShell);
   JulianType jdate;

   if(lSeconds < ONE_YEAR_SECS)
      return(0);

   ISHELL_GetJulianDate(pme->a.m_pIShell, lSeconds, &jdate);
   // convert to date time
   IDATECTL_SetDate(pme->m_pIDate, jdate.wYear, jdate.wMonth, jdate.wDay);
   lDate = IDATECTL_GetJulianDay(pme->m_pIDate);
   lMin = (jdate.wHour * 60) + jdate.wMinute;
   // convert to composite date/time
   return MAKECOMPTIME(lDate,lMin);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_SetShellAlarm(CCalApp * pme, uint32 lAlarm, uint16 wID)
{
   uint16   nMinutes = 0,nMins, nMinsAlarm;
   uint16   nDay, nDayAlarm,nDaysDiff;
   uint32   lDateTime;

   ISHELL_CancelAlarm(pme->a.m_pIShell, AEECLSID_CAL, 0);

   lDateTime = Cal_GetCurrentDateTime(pme);

   if(lAlarm < lDateTime)
      return;

   nDay = (uint16)GETDATE(lDateTime);
   nDayAlarm = (uint16)GETDATE(lAlarm);
   nMins = (uint16)GETTIME(lDateTime);
   nMinsAlarm = (uint16)GETTIME(lAlarm);

   // Determine the correct number of minutes...

   if(nDayAlarm == nDay)
      nMinutes = nMinsAlarm - nMins;
   else{
      nMinutes = (ONE_DAY_MINS - nMins) + nMinsAlarm;
      nDaysDiff = (nDayAlarm - nDay) - 1;
      if(nDaysDiff >= MAX_ALARM_DAYS)
         nDaysDiff = 7;

      nMinutes += (nDaysDiff * ONE_DAY_MINS);
   }
   ISHELL_SetAlarm(pme->a.m_pIShell, AEECLSID_CAL, wID, nMinutes);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Cal_SetFirstAlarm(CCalApp * pme)
{
   uint32      lDateTime;
   CalEvent *  pce = pme->m_pEventList;
   CalEvent *  pceFirst = NULL;
   CalEvent *  pceAlarm = NULL;
   uint32      lAlarmDateTime = 0xFFFFFFFF;

   lDateTime = Cal_GetCurrentDateTime(pme);

   while (pce != pceFirst) {

      if (!pceFirst)
         pceFirst = pce;

      if (pce->m_lAlarmDateTime > lDateTime && pce->m_lAlarmDateTime < lAlarmDateTime) {
         lAlarmDateTime = pce->m_lAlarmDateTime;
         pceAlarm = pce;
      }
      pce = pce->pNext;
   }

   ISHELL_CancelAlarm(pme->a.m_pIShell, AEECLSID_CAL, 0);

   if(pceAlarm)
      Cal_SetShellAlarm(pme, pceAlarm->m_lAlarmDateTime, pceAlarm->m_nRecID);
}

//===========================================================================
//
// 
//
//===========================================================================
static boolean Cal_Busy(CCalApp * pme, uint16 wState)
{
   AECHAR  szBuff[32];
   AEERect       rc;

   if(pme->m_bDBReady)
      return(FALSE);

   pme->m_nPendingState = wState;
   ISHELL_LoadResString(pme->a.m_pIShell, CALAPP_RES_FILE, IDS_WAIT, szBuff,sizeof(szBuff) / sizeof(AECHAR));
   rc.dx = IDISPLAY_MeasureText(pme->a.m_pIDisplay, AEE_FONT_BOLD, szBuff) + 10;
   rc.dy = (pme->m_nFontHeight + 10);
   rc.x = (pme->m_cx - rc.dx) / 2;
   rc.y = (pme->m_cy - rc.dy) / 2;
   IDISPLAY_DrawText(pme->a.m_pIDisplay, AEE_FONT_BOLD, szBuff, -1, -1, -1, &rc,(IDF_RECT_FRAME|IDF_RECT_FILL|IDF_ALIGN_CENTER|IDF_ALIGN_MIDDLE));
   IDISPLAY_Update(pme->a.m_pIDisplay);
   return(TRUE);
}

//***************************************************************************
//
// Local Helper Routines
//
//
//***************************************************************************

//===========================================================================
//
// 
//
//===========================================================================
static boolean CalReadRec(IDBRecord * prec, CalDBRecFields * pf)
{
   boolean        bValid = TRUE;
   AEEDBFieldType fType;
   AEEDBFieldName dbfname;
   uint16         len;

   pf->nRecID     = 0;
   MEMSET(&pf->b, 0, sizeof(CalBaseFields));
   if(pf->pszDesc)
      *pf->pszDesc = 0;
   if(pf->pszNote)
      *pf->pszNote = 0;
   
  IDBRECORD_Reset(prec);

   // Iterate all of the fields...

   while((fType = IDBRECORD_NextField(prec, &dbfname, &len)) != AEEDB_FT_NONE){
      switch(dbfname){
         case FLD_ALARM:
            if(fType == AEEDB_FT_BINARY){
               CalBaseFields * pb = (CalBaseFields *)IDBRECORD_GetField(prec, &dbfname, &fType, &len);
               if(pb && len == sizeof(CalBaseFields))
                  MEMCPY(&pf->b, pb, sizeof(CalBaseFields));
            }
            break;

         case FLD_DESC:
            if(pf->pszDesc && fType == AEEDB_FT_STRING)
               WSTRNCOPY(pf->pszDesc, pf->nDescLen, IDBRECORD_GetFieldString(prec));
            break;

         case FLD_NOTE:
            if(pf->pszNote && fType == AEEDB_FT_STRING)
               WSTRNCOPY(pf->pszNote, pf->nNoteLen, IDBRECORD_GetFieldString(prec));
            break;
      }

   }

   if(!pf->b.dwTime){
      IDBRECORD_Remove(prec);
      bValid = FALSE;
   }
   else
      pf->nRecID = IDBRECORD_GetID(prec);

   return(bValid);
}


//===========================================================================
//
// 
//
//===========================================================================
static void InitStrField(AEEDBField * pFields, int name, AECHAR * psz)
{
   AEEDBField * pf = &pFields[name - FLD_NAME_BASE];

   (pf)->fType    = AEEDB_FT_STRING; 
   (pf)->fName    = (AEEDBFieldName)(name); 
   (pf)->wDataLen = WSTRSIZE(psz);
   (pf)->pBuffer  = (void*)(psz);
}


//===========================================================================
//
// 
//
//===========================================================================
static void InitBinaryField(AEEDBField * pFields, int name, void * pBuff, int nSize)
{
   AEEDBField * pf = &pFields[name - FLD_NAME_BASE];

  (pf)->fType    = AEEDB_FT_BINARY; 
  (pf)->fName    = (AEEDBFieldName)(name); 
  (pf)->wDataLen = nSize;
  (pf)->pBuffer  = pBuff;
}

//===========================================================================
//
// 
//
//===========================================================================
static void VCenterBlt(IImage * pImage, int x, int y, int dy, int cyImage)
{
   int   y1;

   if (pImage) {
      y1 = y + ((dy - cyImage) / 2);
      IIMAGE_Draw(pImage, x, y1);
   }
}

//===========================================================================
//
// 
//
//===========================================================================
static AECHAR * FindNextToken(AECHAR * psz, AECHAR ch)
{
   AECHAR   chn;

   while((chn = *psz) != (AECHAR)0){
      if(chn == ch)
         return(psz);
      psz++;
   }
   return(NULL);
}

//===========================================================================
//
// 
//
//===========================================================================
static boolean FindFirstCB(CalEvent * pCalEvent, uint32 lDateTime)
{ 
   // stop scan if node time is greater than or equal to search target

   return (pCalEvent->m_b.dwTime >= lDateTime); 
}

//===========================================================================
//
// 
//
//===========================================================================
static boolean FindRecIDCB(CalEvent * pCalEvent, uint32 nRecID)
{
   // stop scan if recID matches
   return (pCalEvent->m_nRecID == (uint16)nRecID);
}


//===========================================================================
//
// 
//
//===========================================================================
static void dllInsert(dlistnode ** pphead, dlistnode * pref, dlistnode * pnew)
{
   if (pnew) {
      // node points to itself
      pnew->pnext = pnew;
      pnew->pprev = pnew;
      // deal with head node?
      if (pphead) {
         if (!*pphead)           // if head pointer NULL, set it to new node
            *pphead = pnew;
         if (!pref) {            // if no reference node, insert at beginning of list
            pref = (*pphead)->pprev;
            *pphead = pnew;
         }
      }
      // insert after reference node
      if (pref) {
         pnew->pnext = pref->pnext;
         pnew->pprev = pref;
         pnew->pnext->pprev = pnew;
         pref->pnext = pnew;
      }
   }
}


//===========================================================================
//
// 
//
//===========================================================================
static dlistnode * dllRemove(dlistnode ** pphead, dlistnode * p)
{
   if (pphead && *pphead) {
      if (!p)
         p = *pphead;
      if (p) {
         // deal with head node?
         if (*pphead == p) {      // fix head node if we are removing it
            if (p == p->pnext)
               *pphead = NULL;
            else
               *pphead = p->pnext;
         }
         p->pprev->pnext = p->pnext;
         p->pnext->pprev = p->pprev;
         p->pprev = p;
         p->pnext = p;
      }
   }
   return p;
}

//===========================================================================
//
// 
//
//===========================================================================
static dlistnode * dllScan(dlistnode * pstart, dlistnode * pend,PFNCBDLLFIND pfnCB, uint32 data)
{
   dlistnode * p = pstart;
   boolean bfound = 0;

   if (!pstart || !pfnCB)
      return NULL;

   if (!pend)
      pend = pstart;

   do {
      if (pfnCB(p, data)) {
         bfound = 1;
         break;
      }
      p = p->pnext;
   } while (p != pend);
   
   return bfound ? p : NULL;
}


