/*===========================================================================

                  Calendar/Scheduler Applet Header

DESCRIPTION
  This file contains the calendar/scheduler applet implementation,


        Copyright  1999-2001 QUALCOMM Incorporated.
               All Rights Reserved.
            QUALCOMM Proprietary/GTDR
===========================================================================*/
#if !defined(CALAPP_H)
#define CALAPP_H 1

typedef struct CalEvent    CalEvent;
typedef struct _CCalApp    CCalApp;
typedef struct dlistnode   dlistnode;


//
// Database Fields and Structures...
//

#define CAL_DATABASE_NAME           "Calendar"

#define SCHEDULER_MIN_SIZE          1000
#define SCHEDULER_MIN_RECS          20
#define DB_READ_RECS                5
#define DB_READ_DELAY               100

typedef struct _CalBaseFields
{
   uint32   dwTime;     
   uint16   wDuration;  // Minutes
   uint16   wAlarmTime; // Minutes 
} CalBaseFields;

#define FLD_NAME_BASE               0x80

enum {
  FLD_ALARM = FLD_NAME_BASE,  // UIDB_FT_BINARY
  FLD_DESC,                   // UIDB_FT_STRING
  FLD_NOTE,                   // UIDB_FT_STRING
  FLD_LAST   
};

#define NUMFLDS (FLD_LAST - FLD_NAME_BASE)


//
// Calendar App Configuration...
//

#define CAL_CFG_VERSION             0x1001

typedef struct _CalCfg 
{
   uint16      nAutoDelete;
   uint16      wDayStart;
   uint16      wDayEnd;
   int16       nAlarmTime;
} CalCfg;

//
// Helpful Macros
//

// Time Macros

#define MAKECOMPTIME(jdate,time)    (((jdate)<<16)|(time))
#define GETDATE(comptime)           (((comptime)>>16) & 0x0000FFFF)
#define GETTIME(comptime)           ((comptime) & 0x0000FFFF)
#define GET_HOUR(t)                 ((t)/MINUTES_PER_HOUR)
#define GET_MINUTE(t)               ((t)%MINUTES_PER_HOUR)
#define IS_PM(t)                    ((t)>=(MAXTIME/2))
                                    
#define ARRAYSIZE(a)                (sizeof(a)/sizeof(a[0]))

// Time macros

#define SECONDS_PER_HOUR            (3600)
#define MINUTES_PER_HOUR            (60)
#define SECONDS_PER_DAY             (SECONDS_PER_HOUR * 24l)
#define MSECS_PER_MINUTE            (1000l * 60l)
#define MAXTIME                     (ONE_DAY_MINS)
#define ALARM_ICON_FLASH_MSECS      (500l)
#define ONE_DAY_MINS                (24 * 60)
#define MAX_ALARM_DAYS              (32000 / ONE_DAY_MINS)
#define MAX_ALARM_MSECS             (15l * 1000l)
#define MESSAGE_TIMEOUT             (4000)

#define DURATION_INCREMENT 15

#define AUTO_DELETE_NEVER  0
#define AUTO_DELETE_DAY    1
#define AUTO_DELETE_WEEK   7
#define AUTO_DELETE_MONTH  31
   
#define DEFAULT_AUTO_DELETE   AUTO_DELETE_MONTH  // Auto-Delete After 1 Month....

typedef void      (*PFNSTATEPROC)   (CCalApp * pme);
typedef boolean   (*PFNKEYHANDLER)  (CCalApp * pme, int nKeyCode);

//
//   For compactness and ease of sorting, the CalEvent struct stores 
//   the date/time in a single long variable which is a composite 
//   of the julian date and the starting time in minutes.
//      
//      composite_time = (julian_date << 16) | start_time_in_minutes;
//

struct CalEvent
{
   CalEvent    *  pNext;
   CalEvent    *  pPrev;
   uint16         m_nRecID;
   CalBaseFields  m_b;
   uint32         m_lAlarmDateTime;
};

typedef struct _CalDBRecFields
{
   uint16         nRecID;
   CalBaseFields  b;
   AECHAR *       pszDesc;
   int            nDescLen;
   AECHAR *       pszNote;
   int            nNoteLen;
} CalDBRecFields;


#define MAXHISTORY      20
#define MAXTEXT         64
#define MAX_SCREEN_CTLS 5

struct _CCalApp
{
   AEEApplet         a;

   // calendar events list (circular, doubly-linked)
   CalEvent *        m_pEventList;
   
   // Interface pointers
   IDBMgr      *     m_pIDBMgr;
   IDatabase   *     m_pIDatabase;

   // QCOM UI controls
   IMenuCtl *        m_pIMenu;
   IMenuCtl *        m_pISoftKey;
   IMenuCtl *        m_pIList;
   IDateCtl *        m_pIDatePick;
   IDateCtl *        m_pIDate;
   ITextCtl *        m_pIText;
   ITimeCtl *        m_pITimeStart;
   ITimeCtl *        m_pITimeEnd;
   ITimeCtl *        m_pIDuration;      // Not used unless 7GP

   // screen and font dimensions
   int               m_cx;
   int               m_cy;
   int               m_nFontHeight;
   CalCfg            m_cfg;
   boolean           m_bInit;

   // current date displayed
   long              m_lCurrentDay;

   // installable per-state key handler 
   PFNKEYHANDLER     m_pfnKeyHandler;

   // history stack
   int               m_arHistory[MAXHISTORY];
   int               m_nHistoryCount;
   int               m_nCurrentState;
   int               m_nReturnState;

   // event input variables
   AECHAR            m_szEventName[MAXTEXT];
   AECHAR            m_szNote[MAXTEXT];
   long              m_lEventDay;
   uint32            m_lStartTime;
   uint32            m_lDuration;
   uint16            m_nAlarmTime;
   int               m_bSetAlarm: 1;
   int               m_bDBReady:1;
   // Icon stuff...
   int               m_bAlarmActivated:1;

   // Complex Dialog Stuff

   IControl *        m_pCtl[MAX_SCREEN_CTLS];
   int               m_nCtlsOnScreen;

   // EVT_COMMAND data

   CalEvent *        m_pceCurrent;

   uint16            m_wPendingAlarmID;
   uint32            m_dwMask;
   uint16            m_nPendingState;

};

#define LINEHEIGHT   (pme->m_nFontHeight+2)

//
// Link-list stuff...
//

struct dlistnode
{
   dlistnode * pnext;
   dlistnode * pprev;
};

typedef boolean (*PFNCBDLLFIND)(dlistnode *pnode, uint32 data);

// Callback prototype for EnumDays

typedef struct
{
   CCalApp * pme;
   long        lDate;         // day that we are enumerating
   boolean     bFound;        // signals that an event exists for this day
   int         nCounter;      // variable may be used as a counter inside callback
} ENUMDAYDATA;

typedef boolean (*PFNENUMDAYCB)  (CalEvent * pCalEvent, ENUMDAYDATA * pEnumData);

// State Definitions

enum {
   S_MAINMENU = 0,
   S_VIEWDAY,
   S_VIEWMONTH,
   S_EDITEVENT,
   S_ADDEVENT,
   S_ENTERNAME,
   S_ENTERNOTE,
   S_EVENTADDED,
   S_ALARMTIME,
   S_OPTAUTODELETE,
   S_RETURN, 
   S_OPTIONS,
   S_OPTHOURS,
   S_OPTALARMTIME,
   S_ERASECONFIRM,
   S_ERASEEVENT,
   S_ALERT,
   S_VIEWEVENT,       // state stored in menu as S_VIEWEVENT + n
   S_MESSAGE
};

#define NUMSTATES    ((int)S_MESSAGE)

#define CalEvent_IsInPosition(p) \
   ((p)->pPrev->m_b.dwTime <= (p)->m_b.dwTime && \
    (p)->pNext->m_b.dwTime >= (p)->m_b.dwTime)



#endif
