/*======================================================
FILE:  CCalcApp.c

SERVICES:  AEE Sample Calculator and Tip Calculator

GENERAL DESCRIPTION:

PUBLIC CLASSES AND STATIC FUNCTIONS:
	List the class(es) and static functions included in this file

INITIALIZATION & SEQUENCING REQUIREMENTS:

	See Exported Routines

        Copyright  1999-2001 QUALCOMM Incorporated.
               All Rights Reserved.
            QUALCOMM Proprietary/GTDR
=====================================================*/

//===========================================================================
//
//                       PLEASE NOTE THE FOLLOWING
//
// This sample application was developed before some BREW APIs were available.
// As a result, it contains code that may not be optimal.  The main areas 
// of note related to this issue are:
//
// - Dialogs
// - Messages
// - Simple Prompts
//
// BREW now provides much better support for these operation
// 
// In future revisions, this code will be modified to support these new APIs.
//
//
//===========================================================================
#include "AEEShell.h"
#include "AEEStdLib.h"
#include "AEEDisp.h"
#include "AEEHeap.h"
#include "AEEMenu.h"
#include "AEEModGen.h"
#include "AEEAppGen.h"
#include "CalcApp_res.h"
#include "AEESampleAppIDs.h"

#define TSTATE_AMOUNT         0
#define TSTATE_RATE           1
#define TSTATE_SPECIAL_RATE   2
#define TSTATE_SHOWBILL       3
#define TSTATE_SPLIT          4
#define TSTATE_SHOWSPLIT      5

#define NTYPE_FLOAT           0
#define NTYPE_CURRENCY        1
#define NTYPE_DECIMAL         2

// 
// Calc information stored on disk...
//

#define CALC_CFG_VERSION      0x1000

typedef struct _CalcCfg 
{
   uint16      wVersion;
   double      vMem;
} CalcCfg;

typedef struct _calcnum calcnum;

struct _calcnum
{
   calcnum *   pNext;
   uint16      wOp;
   double      v;
};

#define MAX_CALC_TEXT         13  /* including decimal point */
#define MAX_TIP_TEXT          9  // Max tip calc is on $9999.99

typedef struct _CCalcApp                                    // or use the OBJECT() macro (QCOMBASE.H)
{
   AEEApplet         a;

   IMenuCtl *        m_pMenu;
   AECHAR            m_szText[MAX_CALC_TEXT + 1];
   AEERect           m_rc;
   calcnum *         m_pValList;
   int16             m_cxInd;
   int16             m_cy;
   int16             m_cx;
   AEECLSID          m_cls;
   int               m_nState;
   int               m_bClearLast:1;
   uint16            m_nType;
   double            m_nAmount;
   double            m_nTip;
   double            m_nGuests;
   uint16            m_wLastOperator;

   AECHAR            m_szMemAnnun[2];
   AECHAR            m_szNums[11];
   CalcCfg           m_cfg;
} CCalcApp;


#define IsCurrency(pme)    ((pme)->m_nType == NTYPE_CURRENCY)
#define IsFloat(pme)       ((pme)->m_nType == NTYPE_FLOAT)
#define IsDecimal(pme)     ((pme)->m_nType == NTYPE_DECIMAL)

static boolean Calc_HandleEvent(CCalcApp * pme, AEEEvent eCode, uint16 wParam, uint32 dwParam);
static void    Calc_Free(CCalcApp * pme);

static void    Calc_ToggleSign(CCalcApp * pme);
static void    Calc_Startup(CCalcApp * pme);
static void    Calc_DrawScreen(CCalcApp * pme);
static int     Calc_NumText(CCalcApp * pme,int x, RGBVAL clr, boolean bDraw, boolean bAllSmall);
static boolean Calc_AddChar(CCalcApp * pme, AECHAR chAdd, boolean bUnique);
static void    Calc_SetVal(CCalcApp * pme,double v,boolean bUpdate);
static void    Calc_DrawNum(CCalcApp * pme, boolean bUpdate);
static void    Calc_ShowAnnun(CCalcApp * pme, uint16 wOp, boolean bUpdate);
static void    TipApp_DrawScreen(CCalcApp * pme, int nState);
static int     TipApp_ShowBillText(CCalcApp * pme, IDisplay * pd, int y, double n, uint16 wID, boolean bUseExist);
static void    Calc_PushVal(CCalcApp * pme,uint16 wOp);
static void    Calc_RemoveNum(CCalcApp * pme, calcnum * pn, calcnum * pnPrev);
static void    Calc_ClearVals(CCalcApp * pme);
static void    Calc_SetupRect(CCalcApp * pme);
static double  Calc_GetVal(CCalcApp * pme);

static double HACK_Float(const char * psz);

#define MAKE_FLOAT(x)  HACK_Float(#x)

//
// Constant Data
//

static const int16         gMenuIDs[] = { IDC_DIV,
                                          IDC_SUB,
                                          IDC_EQUAL,
                                          IDC_ADD,
                                          IDC_MUL,
                                          IDC_DOT,
                                          IDC_CLR,
                                          IDC_MEM,
                                          IDC_MEMADD,
                                          IDC_MEMCLR,
                                          IDC_SIGN,
                                          0};

static const uint16       gStdPcts[] = {IDC_RATE15,
                                       IDC_RATE18,
                                       IDC_RATE20,
                                       IDC_RATE10,
                                       IDC_RATE5,
                                       IDC_RATEOTHER,0};

//
// Static Applet Functions - These functions need only be declared
// when an applet is being linked by the Manufacturer during their
// main build process.  They are not required for dynamically downloaded
// applets.
//
#if defined(BREW_STATIC_APP)

static int     CalcMod_CreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj);
static int     CalcMod_Load(IShell *ps, void * pHelpers, IModule ** pMod);

static const AEEAppInfo    gaiCalc[] = {{AEECLSID_CALC,CALCAPP_RES_FILE,IDS_CALCTITLE, IDI_CALC,0,0,0,AFLAG_TOOL},
                                       {AEECLSID_TIPCALC,CALCAPP_RES_FILE,IDS_TIPTITLE, IDI_TIPCALC,0,0,0,AFLAG_TOOL}};

/*===========================================================================

                      PUBLIC FUNCTION DECLARATIONS

===========================================================================*/

//===========================================================================
//
// 
//
//===========================================================================
PFNMODENTRY Calc_GetModInfo(IShell * ps,AEECLSID ** ppClasses, AEEAppInfo ** pApps, uint16 * pnApps,uint16 * pwMinPriv)
{
   *pApps = (AEEAppInfo *)gaiCalc;
   *pnApps = sizeof(gaiCalc) / sizeof(AEEAppInfo);
   return((PFNMODENTRY)CalcMod_Load);
}

//===========================================================================
//
// 
//
//===========================================================================
static int CalcMod_Load(IShell *ps, void * pHelpers, IModule ** pMod)
{
   return(AEEStaticMod_New((int16)(sizeof(AEEMod)),ps,pHelpers,pMod,(PFNMODCREATEINST)CalcMod_CreateInstance,(PFNFREEMODDATA)NULL));
}

#endif

//===========================================================================
//
// Called by AEEModGen when the applet has been requested by BREW
//
//===========================================================================
#if defined(BREW_STATIC_APP)
static int CalcMod_CreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
#else
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
#endif
{
   CCalcApp *     pme;
   AEEDeviceInfo  dm;

   *ppObj = NULL;

   if(!AEEApplet_New(sizeof(CCalcApp), ClsId, pIShell,po,(IApplet**)ppObj,(AEEHANDLER)Calc_HandleEvent,(PFNFREEAPPDATA)Calc_Free))
      return(ENOMEMORY);

   pme = (CCalcApp *)*ppObj;

   pme->m_cls = ClsId;

   ISHELL_LoadResString(pIShell, CALCAPP_RES_FILE, IDS_NUMS, pme->m_szNums, sizeof(pme->m_szNums));

   if(ClsId == AEECLSID_CALC) 
      ISHELL_GetPrefs(pIShell, AEECLSID_CALC, CALC_CFG_VERSION, &pme->m_cfg, sizeof(pme->m_cfg));

   ISHELL_GetDeviceInfo(pIShell, &dm);
   pme->m_rc.dx = pme->m_cx = dm.cxScreen;
   pme->m_rc.dy = pme->m_cy = dm.cyScreen;
   return(AEE_SUCCESS);
}

//===========================================================================
//
// 
//
//===========================================================================
boolean  Calc_HandleEvent(CCalcApp * pme, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{
   switch(eCode){

      case EVT_APP_START:
         Calc_Startup(pme);
         return(TRUE);

      case EVT_APP_STOP:
         ISHELL_SetPrefs(pme->a.m_pIShell, AEECLSID_CALC, CALC_CFG_VERSION, &pme->m_cfg, sizeof(CalcCfg));
         return(TRUE);

      case EVT_COMMAND:
         switch(wParam){
            case IDC_RATEOTHER:
               TipApp_DrawScreen(pme, TSTATE_SPECIAL_RATE);
               break;

            case IDC_RATE5:
            case IDC_RATE10:
            case IDC_RATE15:
            case IDC_RATE18:
            case IDC_RATE20:
               {
               double   v;

               switch(wParam){
                  case IDC_RATE5:
                     v = MAKE_FLOAT(0.05);
                     break;
                  case IDC_RATE10:
                     v = MAKE_FLOAT(0.10);
                     break;
                  case IDC_RATE15:
                     v = MAKE_FLOAT(0.15);
                     break;
                  case IDC_RATE18:
                     v = MAKE_FLOAT(0.18);
                     break;
                  case IDC_RATE20:
                     v = MAKE_FLOAT(0.20);
                     break;
                  default:
                     return(FALSE);
               }
               pme->m_nTip = FMUL(v,pme->m_nAmount);
               TipApp_DrawScreen(pme, TSTATE_SHOWBILL);
               break;
               }

            case IDC_SPLIT:
               TipApp_DrawScreen(pme,TSTATE_SPLIT);
               break;

            case IDC_OK:
            case IDC_NEXT:
            case IDC_DONE:
               switch(pme->m_nState){
                  case TSTATE_SPLIT:
                     pme->m_nGuests = Calc_GetVal(pme);
                     if(FCMP_E(pme->m_nGuests,0.0))
                        pme->m_nGuests = MAKE_FLOAT(1.0);
                     TipApp_DrawScreen(pme, TSTATE_SHOWSPLIT);
                     break;

                  case TSTATE_AMOUNT:
                     pme->m_nAmount = Calc_GetVal(pme);
                     TipApp_DrawScreen(pme, TSTATE_RATE);
                     break;

                  case TSTATE_SHOWSPLIT:
                  case TSTATE_SHOWBILL:
                     ISHELL_CloseApplet(pme->a.m_pIShell,FALSE);
                     break;

                  case TSTATE_SPECIAL_RATE:
                     pme->m_nTip = Calc_GetVal(pme);
                     TipApp_DrawScreen(pme, TSTATE_SHOWBILL);
                     break;
               }
               break;
            case IDC_ADD:
            case IDC_SUB:
            case IDC_DIV:
            case IDC_MUL:
            case IDC_EQUAL:
               Calc_PushVal(pme, wParam);
               return(TRUE);
            
            case IDC_SIGN:
               Calc_ToggleSign(pme);
               break;

            case IDC_MEMADD:
               pme->m_cfg.vMem = Calc_GetVal(pme);
               Calc_ShowAnnun(pme,0,TRUE);
               break;
               
            case IDC_MEM:
               {
               uint16   wOp;

               Calc_SetVal(pme, pme->m_cfg.vMem,TRUE);
               pme->m_bClearLast = FALSE;
               wOp = pme->m_wLastOperator;
               pme->m_wLastOperator = 0;
               if(wOp && wOp != IDC_EQUAL){
                  IMENUCTL_SetSel(pme->m_pMenu, IDC_EQUAL);
                  return(TRUE);
               }
               break;
               }

            case IDC_CLR:
               Calc_ClearVals(pme);
               Calc_SetVal(pme, 0.0,FALSE);
               Calc_ShowAnnun(pme, 0, FALSE);
               Calc_DrawNum(pme,TRUE);
               break;
                              
            case IDC_MEMCLR:
               pme->m_cfg.vMem = 0.0;
               Calc_ShowAnnun(pme,0,TRUE);
               break;

            case IDC_DOT:
               Calc_AddChar(pme,(AECHAR)'.',TRUE);
               break;

#if defined(FEATURE_UI_FLIP)
            case IDC_EXIT:
               ISHELL_CloseApplet(pme->a.m_pIShell);
               break;
#endif
         }

         if(pme->m_pMenu)
            IMENUCTL_SetSel(pme->m_pMenu, IDC_EQUAL);

         return(TRUE);

      case EVT_KEY_HELD:
      case EVT_KEY:
         {

         boolean  bClrHeld;

         bClrHeld = (eCode == EVT_KEY_HELD);

         if(bClrHeld && wParam != AVK_CLR)
            return(FALSE);
            

         // This is used in case the AlarmClock Enabled timer is running.  If they hit any 
         // key, the clock automatically goes to activated...
   
         if(pme->m_pMenu && IMENUCTL_HandleEvent(pme->m_pMenu, eCode, wParam, dwParam))
            return(TRUE);

         switch(wParam){

         // Tip Order AMOUNT -> RATE -> [SPECIAL RATE] -> SPLIT -> TOTAL

            case AVK_CLR:
               {
               int   nState = pme->m_nState;

               switch(nState){
                  case TSTATE_SHOWBILL:
                     if(bClrHeld)
                        break;
                     TipApp_DrawScreen(pme, TSTATE_RATE);   // Go back to RATE...
                     break;

                  case TSTATE_SHOWSPLIT:
                     if(bClrHeld)
                        break;
                     TipApp_DrawScreen(pme,TSTATE_SPLIT);   // Go back to SPLIT...
                     break;

                  case TSTATE_RATE:
                     if(bClrHeld)
                        break;
                     TipApp_DrawScreen(pme, TSTATE_AMOUNT); // Go back to AMOUNT...
                     break;
                     
                  case TSTATE_AMOUNT:        // Clear the amount or exit...
                  case TSTATE_SPLIT:         // Clear the amount or go back to RATE...
                  case TSTATE_SPECIAL_RATE:  // Clear the amount or go back to RATE...
                     if(!FCMP_E(Calc_GetVal(pme),0.0)){
                        if(bClrHeld || (nState == TSTATE_AMOUNT && pme->m_bClearLast))
                           Calc_HandleEvent(pme, EVT_COMMAND, IDC_CLR, 0);
                        else
                           Calc_AddChar(pme, 0, FALSE);
                        return(TRUE);
                     }

                     if(nState == TSTATE_AMOUNT)
                        return(FALSE);

                     TipApp_DrawScreen(pme, TSTATE_RATE);
                     break;

                  default:
                     return(FALSE);
               }
               return(TRUE);
               }

            case AVK_UP:
            case AVK_DOWN:
               {
               double   v,v1,vMin,vMax;

               if(pme->m_nState != TSTATE_SPLIT)
                  break;

               if(wParam == AVK_UP)
                  v1 = MAKE_FLOAT(1.0);
               else
                  v1 = MAKE_FLOAT(-1.0);
               vMin = MAKE_FLOAT(1.0);
               vMax = MAKE_FLOAT(99.0);
               
               v = FADD(Calc_GetVal(pme),v1);
               if(FCMP_L(v,vMin))
                  v = vMin;
               else{
                  if(FCMP_G(v,vMax))
                     v = vMax;
               }
               Calc_SetVal(pme, v, TRUE);
               }
               return(TRUE);
     
            case AVK_STAR:
               Calc_ToggleSign(pme);
               return(TRUE);

            case AVK_POUND:
               Calc_PushVal(pme, IDC_EQUAL);
               return(TRUE);

            default:
               if(wParam < AVK_0 || wParam > AVK_9)
                  break;

         // If the last key was an operator, clear the last displayed number...

               Calc_AddChar(pme,pme->m_szNums[wParam - AVK_0], FALSE);

               return(TRUE);
         }
         break;
         }
   }
   return(FALSE);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Calc_Free(CCalcApp * pme)
{
   Calc_ClearVals(pme);

   if(pme->m_pMenu != NULL){
      IMENUCTL_Release(pme->m_pMenu);
      pme->m_pMenu = NULL;
   }
}

//===========================================================================
//
// 
//
//===========================================================================
static void Calc_ClearVals(CCalcApp * pme)
{
   if(pme->m_pValList != NULL){
      while(pme->m_pValList != NULL)
         Calc_RemoveNum(pme, pme->m_pValList,NULL);
   }

   *pme->m_szText = 0;
   pme->m_wLastOperator = 0;
   pme->m_bClearLast = TRUE;
}

//===========================================================================
//
// 
//
//===========================================================================
static void Calc_SetupRect(CCalcApp * pme)
{
   AEERect rc;
   int   cyAvail, cyFont,y;

   if(pme->m_pMenu){

   // Attempt to center the text vertically...

      IMENUCTL_GetRect(pme->m_pMenu, &rc);
      y = IDISPLAY_GetFontMetrics(pme->a.m_pIDisplay, AEE_FONT_BOLD, NULL, NULL) + 1;

      cyAvail = rc.y - y;
      cyFont = IDISPLAY_GetFontMetrics(pme->a.m_pIDisplay, AEE_FONT_LARGE, NULL, NULL) + 2;
      if(cyAvail >= cyFont)
         y += (cyAvail - cyFont) / 2;

      pme->m_rc.y    = y;
      pme->m_rc.dy   = cyFont;
   }
}

//===========================================================================
//
// 
//
//===========================================================================
void Calc_Startup(CCalcApp * pme)
{
   AEERect rc;

   SETAEERECT(&rc, 0, 0, pme->m_cx, pme->m_cy);
   IDISPLAY_FillRect(pme->a.m_pIDisplay, &rc, RGB_WHITE);

   pme->m_bClearLast = TRUE;
   pme->m_nTip = pme->m_nAmount = 0.0;

   if(pme->m_cls == AEECLSID_TIPCALC)
      TipApp_DrawScreen(pme,TSTATE_AMOUNT);
   else
      Calc_DrawScreen(pme);
}

//===========================================================================
//
// 
//
//===========================================================================
void Calc_DrawScreen(CCalcApp * pme)
{
   IMenuCtl *     pm;
   AECHAR     szBuff[32];
   int            i;
   int16          wID;

   pme->m_nType = NTYPE_FLOAT;
   pme->m_nState = TSTATE_AMOUNT;

   if((pm = pme->m_pMenu) == NULL){
      if(ISHELL_CreateInstance(pme->a.m_pIShell, AEECLSID_SOFTKEYCTL, (void **) (&pm)))
         return;

      pme->m_pMenu = pm;

#if 0
      IMENUCTL_SetProperties(pm, MP_MAXSOFTKEYITEMS|MP_NO_FRAMES);
#else
      IMENUCTL_SetProperties(pm, MP_MAXSOFTKEYITEMS);
#endif
      for(i = 0; (wID = gMenuIDs[i]) != 0;i++)
         IMENUCTL_AddItem(pm, CALCAPP_RES_FILE,wID, wID,NULL,0);

      IMENUCTL_SetSel(pm, IDC_ADD);
   }

   Calc_SetupRect(pme);

   ISHELL_LoadResString(pme->a.m_pIShell, CALCAPP_RES_FILE, IDS_MEMIND, pme->m_szMemAnnun, sizeof(pme->m_szMemAnnun));
   pme->m_cxInd = IDISPLAY_MeasureText(pme->a.m_pIDisplay,AEE_FONT_BOLD,(const AECHAR *)pme->m_szMemAnnun);

   ISHELL_LoadResString(pme->a.m_pIShell, CALCAPP_RES_FILE, IDS_CALCTITLE, szBuff, sizeof(szBuff));
   IDISPLAY_DrawText(pme->a.m_pIDisplay, AEE_FONT_BOLD, szBuff, -1, 2, 0, NULL,0);
   Calc_SetVal(pme, 0.0,FALSE);
   Calc_ShowAnnun(pme, 0, FALSE);

   IMENUCTL_Redraw(pm);
   IMENUCTL_SetActive(pm,TRUE);

   IDISPLAY_Update(pme->a.m_pIDisplay);
}

//===========================================================================
//
// 
//
//===========================================================================
static int TipApp_ShowBillText(CCalcApp * pme, IDisplay * pd, int y, double n, uint16 wID, boolean bUseExist)
{
   AECHAR  szBuff[32];
   AECHAR *psz;
   int16       nWidth;

   // Draw the description text...
     
   ISHELL_LoadResString(pme->a.m_pIShell, CALCAPP_RES_FILE, wID, szBuff, sizeof(szBuff));
   IDISPLAY_DrawText(pd, AEE_FONT_NORMAL, szBuff, -1, 2, y, NULL,0);

   // Draw the value - right justified...

   if(bUseExist)
      psz = pme->m_szText;
   else{

      szBuff[0] = (AECHAR)'$';
      FLOAT_TO_WSTR(n,szBuff + 1, sizeof(szBuff));

   // Clip trailing 0s...

      for(psz = szBuff; *psz != 0 &&  *psz != '.'; psz++);
      if(*psz != 0)
         psz[3] = 0;
      psz = szBuff;
   }

   nWidth = IDISPLAY_MeasureText(pd, AEE_FONT_NORMAL, (const AECHAR *)psz);

   IDISPLAY_DrawText(pd, AEE_FONT_NORMAL, psz, -1, pme->m_rc.dx - (nWidth + 2), y, NULL,0);

   return(y + IDISPLAY_GetFontMetrics(pd, AEE_FONT_NORMAL, NULL, NULL) + 2);
}

//===========================================================================
//
// 
//
//===========================================================================
static void TipApp_DrawScreen(CCalcApp * pme, int nState)
{
   AECHAR  szBuff[32];
   AEERect       rc;
   IDisplay *  pd = pme->a.m_pIDisplay;
   IMenuCtl *  pm = pme->m_pMenu;

   // Close the current menu...

   if(pm){
      IMENUCTL_Release(pm);
      pme->m_pMenu = NULL;
   }

   if(ISHELL_CreateInstance(pme->a.m_pIShell, (nState == TSTATE_RATE ? AEECLSID_MENUCTL : AEECLSID_SOFTKEYCTL), (void **) (&pm)))
      return;

   pme->m_pMenu = pm;

   Calc_SetupRect(pme);

   // Clear the screen...

   SETAEERECT(&rc, 0,0,pme->m_cx, pme->m_cy);
   IDISPLAY_FillRect(pd, &rc, RGB_WHITE);

   *szBuff = 0;

   pme->m_nState = nState;
   pme->m_cxInd = 0;

   switch(nState){   
      case TSTATE_SPLIT:
         pme->m_nType = NTYPE_DECIMAL;
         ISHELL_LoadResString(pme->a.m_pIShell, CALCAPP_RES_FILE, IDS_ZEROTIP, pme->m_szText, sizeof(pme->m_szText));
         ISHELL_LoadResString(pme->a.m_pIShell, CALCAPP_RES_FILE, (uint16)IDS_GUESTS, szBuff, sizeof(szBuff));
         IMENUCTL_AddItem(pm, CALCAPP_RES_FILE, IDC_NEXT, IDC_NEXT ,NULL,0);
#if 0
         Calc_SetVal(pme, 2.0,FALSE);
#else
         STR_TO_WSTR("2",pme->m_szText,sizeof(pme->m_szText));
#endif

         Calc_DrawNum(pme,FALSE);
         break;

      case TSTATE_AMOUNT:
      case TSTATE_SPECIAL_RATE:
         pme->m_nType = NTYPE_CURRENCY;
         ISHELL_LoadResString(pme->a.m_pIShell, CALCAPP_RES_FILE, IDS_ZEROTIP, pme->m_szText, sizeof(pme->m_szText));
         ISHELL_LoadResString(pme->a.m_pIShell, CALCAPP_RES_FILE, (uint16)(nState == TSTATE_AMOUNT ? IDS_AMOUNT : IDS_TIP), szBuff, sizeof(szBuff));
         IMENUCTL_AddItem(pm, CALCAPP_RES_FILE, IDC_NEXT, IDC_NEXT ,NULL,0);
         Calc_DrawNum(pme,FALSE);
         break;

      case TSTATE_RATE:
         {
         uint16   * pw;

         IMENUCTL_SetProperties(pm, 0);
         ISHELL_LoadResString(pme->a.m_pIShell, CALCAPP_RES_FILE, IDS_RATE, szBuff, sizeof(szBuff));
         for(pw = (uint16 *)gStdPcts; *pw != 0; pw++)
            IMENUCTL_AddItem(pm, CALCAPP_RES_FILE, *pw, *pw ,NULL,0);
         break;
         }

      case TSTATE_SHOWSPLIT:
         {
         int      y;
         double   nBill,nTotal;

         nTotal = FADD(pme->m_nAmount,pme->m_nTip);
         nBill = FDIV(nTotal,pme->m_nGuests);
      
         // If the total amount is clipped too low, simply increment it by a penny...

         if(FCMP_L(FMUL(nBill,pme->m_nGuests),nTotal))
            nBill = FADD(nBill,MAKE_FLOAT(1.0));
         y = TipApp_ShowBillText(pme, pd, 0, nTotal, IDS_TOTAL,0);
         y = TipApp_ShowBillText(pme, pd, y, 0.0, IDS_GUESTSDESC,TRUE);
         TipApp_ShowBillText(pme, pd, y, nBill, IDS_SPLITBILL,0);
         IMENUCTL_AddItem(pm, CALCAPP_RES_FILE, IDC_DONE, IDC_DONE ,NULL,0);
         }
         break;
         
      case TSTATE_SHOWBILL:
         {
         int      y;

         y = TipApp_ShowBillText(pme, pd, 0, pme->m_nAmount, IDS_BILL,0);
         y = TipApp_ShowBillText(pme, pd, y, pme->m_nTip, IDS_TIPDESC,0);
         TipApp_ShowBillText(pme, pd, y, FADD(pme->m_nTip,pme->m_nAmount), IDS_TOTAL,0);

         IMENUCTL_AddItem(pm, CALCAPP_RES_FILE, IDC_DONE, IDC_DONE ,NULL,0);
         IMENUCTL_AddItem(pm, CALCAPP_RES_FILE, IDC_SPLIT, IDC_SPLIT ,NULL,0);
         break;
         }
         
   }

   if(*szBuff)
      IDISPLAY_DrawText(pd, AEE_FONT_BOLD, szBuff, -1, 2, 0, NULL, 0);

   IMENUCTL_SetActive(pm, TRUE);   
   IMENUCTL_Redraw(pm);

   IDISPLAY_Update(pd);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Calc_RemoveNum(CCalcApp * pme, calcnum * pn, calcnum * pnPrev)
{
   if(pnPrev)
      pnPrev->pNext = pn->pNext;
   else
      pme->m_pValList = pn->pNext;

   FREE(pn);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Calc_PushVal(CCalcApp * pme, uint16 wOp)
{
   calcnum     *pn, *pnNext, *pnPrev;
   double      v = 0.0;

   if(pme->m_cls != AEECLSID_CALC)
      return;

   for(pnPrev = pme->m_pValList; pnPrev != NULL; pnPrev = pnNext){
      pnNext = pnPrev->pNext;
      if(pnNext == NULL)
         break;
   }

   // Save the operator state.  This allows us to move the menu selection automatically...

   pme->m_wLastOperator = wOp;

   // It's an operator following an operator, just replace it...

   if(pme->m_bClearLast){
      if(pnPrev){
         pnPrev->wOp = wOp;
         Calc_ShowAnnun(pme,wOp,TRUE); 
         return;
      }
   }   

   pn = (calcnum *)MALLOC(sizeof(calcnum));

   if(pn == NULL)
      return;

   if(pnPrev)
      pnPrev->pNext = pn;
   else
      pme->m_pValList = pn;

   pme->m_bClearLast = TRUE;

   pn->wOp = wOp;
   pn->v = Calc_GetVal(pme);

   // First resolve any MUL/DIV operations...

   for(pn = pme->m_pValList, pnPrev = NULL; pn != NULL; pn = pnNext){

      pnNext = pn->pNext;

   // Nothing more to do...

      if(pnNext == NULL)
         break;

   // We can compute the incremental value if the next operation is of lower or equal precedence
   // to the current...

      if((pn->wOp & (~1)) >= (pnNext->wOp & (~1))){

         switch(pn->wOp){
            case IDC_MUL:
               pnNext->v = FMUL(pnNext->v,pn->v);
               break;

            case IDC_ADD:
               pnNext->v = FADD(pnNext->v,pn->v);
               break;
   
            case IDC_DIV:

     // Deal with Divide by Zero...

               if(FCMP_E(pnNext->v,0.0)){ 
                  AECHAR     szError[64];
                  AEERect    rc;

                  rc = pme->m_rc;
                  Calc_ClearVals(pme);
                  wOp = 0;
                  ISHELL_LoadResString(pme->a.m_pIShell, CALCAPP_RES_FILE, IDS_DIV_0, szError, sizeof(szError));
                  IDISPLAY_DrawText(pme->a.m_pIDisplay,AEE_FONT_BOLD,szError,-1,rc.x,rc.y,&rc,IDF_RECT_FILL|IDF_ALIGN_CENTER|IDF_ALIGN_BOTTOM);
                  goto done;
               }
               pnNext->v = FDIV(pn->v,pnNext->v);
               break;

            case IDC_SUB:
               pnNext->v = FSUB(pn->v,pnNext->v);
               break;
         }
         Calc_RemoveNum(pme, pn, pnPrev);
      }
      else
         pnPrev = pn;
   }

   if(wOp == IDC_EQUAL){

   // Start at the beginning of the list and accumulate the total...

      v = 0.0;

      while((pn = pme->m_pValList) != NULL){

         pnNext = pn->pNext;
         v = pn->v;
         wOp = pn->wOp;
         Calc_RemoveNum(pme, pn, NULL);

         if(!pnNext)
            break;

         if(wOp == IDC_ADD)
            pnNext->v = FADD(pnNext->v,v);
         else
            pnNext->v = FSUB(pnNext->v,v);
      }
      Calc_SetVal(pme, v, FALSE);
   }
   else{
      if(pme->m_pValList->pNext == NULL && (wOp == IDC_ADD || wOp == IDC_SUB))
         Calc_SetVal(pme, pme->m_pValList->v, FALSE);
   }

done:

   Calc_ShowAnnun(pme,wOp,TRUE); 
}

//===========================================================================
//
// 
//
//===========================================================================
static boolean Calc_AddChar(CCalcApp * pme, AECHAR chAdd, boolean bUnique)
{
   AECHAR *   psz, ch;
   int        i;
   AECHAR *   pszBase = pme->m_szText;
   boolean    bMenuSel, bTip;

   bTip = IsCurrency(pme);
   bMenuSel = FALSE;

   if(pme->m_wLastOperator){
      if(!bTip && pme->m_wLastOperator != IDC_EQUAL)
         bMenuSel = TRUE;
      pme->m_wLastOperator = 0;
   }
   
   if(pme->m_nState != TSTATE_AMOUNT && pme->m_nState != TSTATE_SPECIAL_RATE && pme->m_nState != TSTATE_SPLIT)
      return(FALSE);

   if(pme->m_bClearLast){
      pme->m_bClearLast = FALSE;
      if(bTip)
         ISHELL_LoadResString(pme->a.m_pIShell, CALCAPP_RES_FILE, IDS_ZEROTIP, pszBase, sizeof(pme->m_szText));
      else
         pszBase[0] = 0;
   }

   for(psz = pszBase, i = 0; (ch = *psz) != 0; i++, psz++){
      if(bUnique && ch == chAdd)
         return(FALSE);
   }


   // If we are removing a character...

   if(!chAdd){

      psz--;

   // If it's the tip calculator, we are dealing with currency. In that case, pull the values from 
   // left to right and pre-pend a 0.  We will then strip leading zeros below...

      if(bTip){
         AECHAR * pszSrc;

         for(pszSrc = psz - 1; pszSrc >= pszBase + 1; pszSrc--, psz--){
            ch = *pszSrc;
            if(ch == (AECHAR)'.'){
               pszSrc--;
               ch = *pszSrc;
            }   
            if(*psz == '.')
               psz--;
            *psz = ch;
         }
         if(*psz == (AECHAR)'.')
            psz--;
         *psz = (AECHAR)'0';
      }
      else{
         if(!i)
            return(FALSE);
         *psz = 0;
         if(!*pszBase){
            *pszBase = (AECHAR)'0';
            pszBase[1] = 0;
            pme->m_bClearLast = TRUE;     // Force the 0 to be removed the next time...
         }
      }
   }
   else{

   // If it's the tip calculator, move the decimal point...

      if(bTip){
         if(i >= MAX_TIP_TEXT - 1)
            return(FALSE);
         psz[-3] = psz[-2];
         psz[-2] = (AECHAR)'.';
      }         
      else{
         if(i >= MAX_CALC_TEXT - 1)
            return(FALSE);
      }
      *psz = chAdd;
      psz[1] = 0;
   }


   // Remove the leading zeros on the tip calculator...

   if(bTip && i >= 5){

      AECHAR * pszSrc, * pszDst;

      pszDst = pszBase + 1;
      pszSrc = pszBase + 2;
      if(*pszDst == (AECHAR)'0' && *pszSrc != (AECHAR)'.'){
         while(1){
            ch = *pszSrc;
            *pszDst = ch;
            if(ch == 0)
               break;
            pszSrc++;
            pszDst++;
         }
      }
   }      

   // Auto-highlight the equals sign...

   if(bMenuSel)
      IMENUCTL_SetSel(pme->m_pMenu, IDC_EQUAL);

   Calc_DrawNum(pme,TRUE);

   return(TRUE);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Calc_ToggleSign(CCalcApp * pme)
{
   AECHAR  * pszDst;
   AECHAR  * pszSrc;

   if(pme->m_cls == AEECLSID_CALC){
      pszDst = pme->m_szText;
   
      if(*pszDst == (AECHAR)'-'){
         pszSrc = pszDst + 1;
         while(1){
            if((*pszDst = *pszSrc) == 0)
               break;
            pszSrc++;
            pszDst++;
         }
      }
      else{
         while(*pszDst)
            pszDst++;
         pszSrc = pszDst - 1;
         pszDst[1] = 0;
         while(pszSrc >= pme->m_szText){
            *pszDst = *pszSrc;
            pszSrc--;
            pszDst--;
         }
         *pszDst = (AECHAR)'-';
      }
      Calc_DrawNum(pme,TRUE);
   }
}         

//===========================================================================
//
// 
//
//===========================================================================
static void Calc_ShowAnnun(CCalcApp * pme, uint16 wOp, boolean bUpdate)
{
   AEERect     rc;
   int         nHeight;
   IDisplay *  pd = pme->a.m_pIDisplay;
   calcnum     *pn;
   AECHAR      szBuff[2];
      
   nHeight = IDISPLAY_GetFontMetrics(pd, AEE_FONT_BOLD, NULL,NULL);

   SETAEERECT(&rc, 0, (pme->m_cy - (nHeight * 2)) / 2, pme->m_cxInd + 2, nHeight * 2);
   IDISPLAY_FillRect(pd,&rc, RGB_WHITE);

   rc.x = 1;
   rc.dx -= 2;
   
   rc.dy = nHeight;
   if(!FCMP_E(pme->m_cfg.vMem,0.0))
      IDISPLAY_DrawText(pd,AEE_FONT_BOLD,pme->m_szMemAnnun,1,1,rc.y,&rc,IDF_RECT_FILL|IDF_TEXT_INVERTED);

   if(!wOp){
      for(pn = pme->m_pValList; pn != NULL ; pn = pn->pNext)
         wOp = pn->wOp;
   }
   
   if(wOp){
      rc.y += nHeight;
      ISHELL_LoadResString(pme->a.m_pIShell, CALCAPP_RES_FILE, wOp, szBuff, sizeof(szBuff));
      IDISPLAY_DrawText(pd,AEE_FONT_BOLD,szBuff, 1,1,rc.y,NULL,0);
   }

   if(bUpdate)
      IDISPLAY_Update(pd);
}

//===========================================================================
//
// 
//
//===========================================================================
static int Calc_NumText(CCalcApp * pme,int x, RGBVAL clr, boolean bDraw, boolean bAllSmall)
{
   IDisplay    *  pd = pme->a.m_pIDisplay;
   int16          nWidth, nTotalWidth;
   AECHAR  *      pch,ch;
   int            nLHeight, nHeight,i, ySmall, yLarge, yDot, y, ySymbol;
   AEEFont        fnt;
  
   nTotalWidth = 0;

   // Y for small characters is up from bottom of y for large characters...

   nHeight = IDISPLAY_GetFontMetrics(pd, AEE_FONT_BOLD, NULL,NULL);

   // Determine size and starting y for large characters...

   nLHeight = IDISPLAY_GetFontMetrics(pd, AEE_FONT_LARGE, NULL,NULL);

   // Set normal (AM/PM, etc.) up from the bottom of the large numbers and 
   // colons in the middle...

   if(pme->m_rc.dy > nLHeight){
      yLarge = pme->m_rc.y + ((pme->m_rc.dy - nLHeight) / 2);
      ySmall   = (yLarge + nLHeight) - nHeight;
      if(bAllSmall){
         yDot = ySmall;
         ySymbol = ySmall;
      }
      else{
         yDot  = yLarge + nLHeight - nHeight;
         ySymbol = yLarge + ((nLHeight - nHeight) / 2);
      }
   }
   else{
      bAllSmall = TRUE;
      ySmall = yDot = ySymbol = yLarge = (pme->m_rc.y + 1);
   }


   // Draw or measure each character...

   for(i = 0, pch = pme->m_szText; (ch = *pch) != 0; pch++, i++){

      if(!bAllSmall && ((ch <= (AECHAR)'9' && ch >= (AECHAR)'0') || ch == (AECHAR)'.' || ch == (AECHAR)'-')){
         fnt = AEE_FONT_LARGE;
         y = yLarge;
      }
      else{
         fnt = AEE_FONT_BOLD;
         if(ch == (AECHAR)'.')
            y = yDot;
         else{
            if(ch == (AECHAR)'$')
               y = ySymbol;
            else
               y = ySmall;
         }
      }

      nWidth = IDISPLAY_MeasureTextEx(pd,fnt,(const AECHAR *)pch,1,-1,NULL);

      if(bDraw){
         clr = IDISPLAY_SetColor(pd, CLR_USER_TEXT, clr);
         IDISPLAY_DrawText(pd,fnt,pch,1,x,y,NULL,0);
         IDISPLAY_SetColor(pd, CLR_USER_TEXT, clr);
      }

      nTotalWidth += nWidth;
      x += nWidth;
   }

   if(!IsFloat(pme))
      nTotalWidth += 2;
   return(nTotalWidth);
}

//===========================================================================
//
// 
//
//===========================================================================
static void Calc_DrawNum(CCalcApp * pme, boolean bUpdate)
{
   boolean     bSmall;
   int         x;
   IDisplay *  pd;
   int16       nWidth;
   AEERect     rc;

   pd = pme->a.m_pIDisplay;

   rc = pme->m_rc;

   if(!IsFloat(pme))
      IDISPLAY_FrameSolidRect(pd, &rc);
   else{
      rc.x = pme->m_cxInd + 2;
      IDISPLAY_FillRect(pd, &rc, RGB_WHITE);   
   }

   nWidth = Calc_NumText(pme, -1, RGB_WHITE, FALSE, (bSmall = FALSE));
   if(nWidth >= (pme->m_rc.dx - (pme->m_cxInd + 2)))
      nWidth = Calc_NumText(pme, -1, RGB_WHITE, FALSE, (bSmall = TRUE));

   x = pme->m_rc.dx - nWidth;
   Calc_NumText(pme, x, RGB_NONE, TRUE, bSmall);

   if(bUpdate)
      IDISPLAY_Update(pd);
}


//===========================================================================
//
// 
//
//===========================================================================
static void Calc_SetVal(CCalcApp * pme, double v, boolean bUpdate)
{
   AECHAR *   psz, ch, *pszLastZero, * pszSrc, *pszDst;
   boolean    bDot;

   FLOAT_TO_WSTR(v, pme->m_szText, sizeof(pme->m_szText));

   // Clip leading spaces...

   for(pszDst = pszSrc = pme->m_szText; (ch = *pszSrc) == ' '; pszSrc++);

   if(pszSrc != pszDst){
      while(1){
         ch = *pszSrc;
         *pszDst = ch;
         if(!ch)
            break;
         pszDst++;
         pszSrc++;
      }
   }

   // Clip trailing zeros...

   for(bDot = FALSE, pszLastZero = NULL, psz = pme->m_szText; (ch = *psz) != 0; psz++){
      if(ch == (AECHAR)'.')
         bDot = TRUE;
      else{
         if(bDot){
            if(ch == (AECHAR)'0'){
               if(!pszLastZero)
                  pszLastZero = psz;
            }
            else
               pszLastZero = NULL;
         }
      }
   }

   // If the trailing zero followed the decimal point, clip them both...

   if(pszLastZero){
      if(pszLastZero[-1] == (AECHAR)'.')
         pszLastZero--;
      *pszLastZero = 0;
   }
           
   Calc_DrawNum(pme,bUpdate);
}


//===========================================================================
//
// 
//
//===========================================================================
static double Calc_GetVal(CCalcApp * pme)
{
   AECHAR * psz = pme->m_szText;

   if(*psz == (AECHAR)'$')
      psz++;
   return(WSTR_TO_FLOAT(psz));
}

//===========================================================================
//
// 
//
//===========================================================================
static double HACK_Float(const char * psz)
{
   AECHAR   sz[20];

   STR_TO_WSTR((char *)psz,sz,sizeof(sz));
   return(WSTR_TO_FLOAT(sz));
}

