/*======================================================
FILE:  ClockApps.c

SERVICES:  AEE Sample Alarm Clock, Countdown, Stopwatch Apps

GENERAL DESCRIPTION:

PUBLIC CLASSES AND STATIC FUNCTIONS:
	List the class(es) and static functions included in this file

INITIALIZATION & SEQUENCING REQUIREMENTS:

	See Exported Routines
 
        Copyright  1999-2001 QUALCOMM Incorporated.
               All Rights Reserved.
            QUALCOMM Proprietary/GTDR
=====================================================*/

//===========================================================================
//
//                       PLEASE NOTE THE FOLLOWING
//
// This sample application was developed before some BREW APIs were available.
// As a result, it contains code that may not be optimal.  The main areas 
// of note related to this issue are:
//
// - Dialogs
// - Messages
// - Simple Prompts
//
// BREW now provides much better support for these operation
// 
// In future revisions, this code will be modified to support these new APIs.
//
//
//===========================================================================

#include "AEEShell.h"
#include "AEEAppGen.h"
#include "AEEModGen.h"
#include "AEEStdLib.h"
#include "AEEDisp.h"
#include "AEEHeap.h"
#include "AEEMenu.h"
#include "AEETime.h"
#include "AEEModTable.h"
#include "AEESampleAppIDs.h"
#include "ClockApps_res.h"

/*===========================================================================

                      PUBLIC DATA DECLARATIONS

===========================================================================*/

#define ALARM_ON_TIMER           4000

#define ONE_SECOND               1000l
#define ELIPSES_TIMER_MSECS      500l

#define RUNNING_UPDATE_MSECS     ONE_SECOND
#define ALARM_TIMER_MSECS        500l
#define MAX_ALARM_MSECS          (15l * 1000l)
#define MSECS_PER_SEC            1000l
#define SECS_PER_HOUR            (60l * 60l)
#define SECS_PER_DAY             (SECS_PER_HOUR * 24l)
#define MSECS_PER_DAY            (SECS_PER_DAY * 1000l)
#define SNOOZE_MSECS             (10l * 60l * 1000l)  // 10 minutes...
#define VALID_TIME_POLL          (30l * MSECS_PER_SEC)
#define DEFAULT_ALARM_TIME       (MSECS_PER_SEC * SECS_PER_HOUR * 8l) // 8:00 am

typedef enum { CS_IDLE =0,
               CS_STOPPED,
               AC_ALARM_OFF,
               AC_ALARM_ON,
               AC_SET_ENABLED,
               CS_RUNNING,
               CA_EXPIRED} ClockAppState;

typedef struct _ClockMod   ClockMod;
typedef struct _ClockApp ClockApp;

// 
// Clock information stored on disk...
//

#define CLOCK_APP_CFG_FILE   "ClockApps.cfg"

#define CLOCK_CFG_VERSION  0x1002

typedef struct _ClockAppCfg 
{
   uint16   wVersion;
   uint32   dwSWTime;         // Last time of stopwatch...
   uint32   dwLastAlarm;
   uint32   dwAlarmTime;
} ClockAppCfg;
   
struct _ClockApp                                    // or use the OBJECT() macro (QCOMBASE.H)
{
   AEEApplet         a;
   AEERect           m_rcDisp;
   IMenuCtl *        m_pMenu;
   ITimeCtl *        m_pTime;
   ClockAppState     m_nState;
   uint32            m_dwDispTime;
   int               m_bSettingTime:1;
   int               m_bActive:1;
   int               m_bHideTime:1;
   int               m_bTimerActive:1;
   int               m_nTimeIdx;    // Character being edited in the time field...
   uint32            m_nTicks;
   ClockAppCfg       m_cfg;
};

#define IsStopWatch(pme)   ((boolean)(pme->a.clsID == AEECLSID_STOPWATCH))
#define IsCountDown(pme)   ((boolean)(pme->a.clsID == AEECLSID_COUNTDOWN))
#define IsAlarmClock(pme)  ((boolean)(pme->a.clsID == AEECLSID_ALARMCLOCK))

static void    ClockApp_Free(IApplet * po);
static boolean ClockApp_HandleEvent(IApplet * po, AEEEvent eCode, uint16 wParam, uint32 dwParam);

static void    CASetTimer(ClockApp * pme, uint32 dwTime);
static void    CACancelTimer(ClockApp * pme);
static void    CASaveSettings(ClockApp * pme);
static void    CARedraw(ClockApp * pme);
static void    CASetupMenu(ClockApp * pme);
static void    CAActivate(ClockApp * pme);
static void    CADeactivate(ClockApp *pme);
static void    CAShutdown(ClockApp * pme);
static void    CAStartup(ClockApp * pme);
static void    CAAlarm(ClockApp * pme);
static boolean CAExpired(ClockApp *pme);
static void    CAResetCounters(ClockApp * pme);

static void    CSStop(ClockApp * pme);
static void    CSActiveTimerCallback(ClockApp * pme);
static boolean CSRunning(ClockApp * pme, int32 nTicks);

static void    ACActivateTimerCallback(ClockApp * pme);
static void    ACSetTimer(ClockApp * pme);
static uint32  ACGetTime(ClockApp * pme);
static void    ACSnooze(ClockApp * pme);

static void    AddMenuItem(IMenuCtl * pm, int nID);

#if defined(BREW_STATIC_APP)

static int     ClockMod_CreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj);
static int     ClockMod_Load(IShell *ps, void * pHelpers, IModule ** pMod);

static const AEEAppInfo    gaiClock[]   = {{AEECLSID_STOPWATCH,CLOCKAPPS_RES_FILE, IDS_STOPWATCH_TITLE, IDB_STOPWATCH_ICON,0,0,0,AFLAG_TOOL},
                                          {AEECLSID_ALARMCLOCK,CLOCKAPPS_RES_FILE, IDS_ALARMCLOCK_TITLE, IDB_ALARMCLOCK_ICON,0,0,0,AFLAG_TOOL},
                                          {AEECLSID_COUNTDOWN,CLOCKAPPS_RES_FILE, IDS_COUNTDOWN_TITLE, IDB_COUNTDOWN_ICON,0,0,0,AFLAG_TOOL}};

/*===========================================================================

                      PUBLIC FUNCTION DECLARATIONS

===========================================================================*/

//===========================================================================
//
// 
//
//===========================================================================
PFNMODENTRY Clock_GetModInfo(IShell * ps,AEECLSID ** ppClasses, AEEAppInfo ** pApps, uint16 * pnApps,uint16 * pwMinPriv)
{
   *pApps = (AEEAppInfo *)gaiClock;
   *pnApps = sizeof(gaiClock) / sizeof(AEEAppInfo);
   return((PFNMODENTRY)ClockMod_Load);
}

//===========================================================================
//
// 
//
//===========================================================================
static int ClockMod_Load(IShell *ps, void * pHelpers, IModule ** pMod)
{
   return(AEEStaticMod_New((int16)(sizeof(AEEMod)),ps,pHelpers,pMod,(PFNMODCREATEINST)ClockMod_CreateInstance,(PFNFREEMODDATA)NULL));
}

#endif

//===========================================================================
//
// 
//
//===========================================================================
#if defined(BREW_STATIC_APP)
static int ClockMod_CreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
#else
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
#endif
{
   ClockApp    *pme;

   pme = NULL;

   *ppObj = NULL;

   if(!AEEApplet_New(sizeof(ClockApp), ClsId, pIShell,po,(IApplet**)ppObj,(AEEHANDLER)ClockApp_HandleEvent,(PFNFREEAPPDATA)ClockApp_Free))
      return(EFAILED);

   pme = (ClockApp *)*ppObj;

   // First time we've run/loaded.  Setup the necessary fields in the
   // class.  This includes the vtable, etc.  After this point, we will
   // only reset the IShell pointer to indicate that we have been created...

   ISHELL_GetPrefs(pme->a.m_pIShell, AEECLSID_ALARMCLOCK, CLOCK_CFG_VERSION, &pme->m_cfg, sizeof(ClockAppCfg));
   pme->a.clsID = ClsId;
      
   switch(ClsId){

      case AEECLSID_ALARMCLOCK:
         pme->m_dwDispTime = pme->m_cfg.dwAlarmTime;
         if(pme->m_dwDispTime)
            pme->m_nState = AC_ALARM_ON;
         else{
            pme->m_nState = AC_ALARM_OFF;
            pme->m_dwDispTime = pme->m_cfg.dwLastAlarm;
            if(!pme->m_dwDispTime)
               pme->m_dwDispTime = DEFAULT_ALARM_TIME;
         }
         break;

      case AEECLSID_STOPWATCH:
         pme->m_dwDispTime = pme->m_cfg.dwSWTime;
         if(pme->m_cfg.dwSWTime){
            pme->m_nState = CS_STOPPED;
            break;
         }

      // Fall thru...

      default:
         pme->m_nState = CS_IDLE;
         break;
   }
   
   pme->a.m_pIShell = pIShell;

   *ppObj = pme;
   return(AEE_SUCCESS);
}

/*===========================================================================

                      APPLET CLASS

===========================================================================*/

//===========================================================================
//
// 
//
//===========================================================================
static void ClockApp_Free(IApplet * po)
{
}

//===========================================================================
//
// 
//
//===========================================================================
static boolean  ClockApp_HandleEvent(IApplet * po, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{
   ClockApp *     pme = (ClockApp *)po;
   ITimeCtl   * pt;

   pt = pme->m_pTime;

   switch(eCode){

      case EVT_APP_START:
         if(pme->m_nState != CA_EXPIRED){
            ISHELL_ShowCopyright(pme->a.m_pIShell);
            return(TRUE);
         }

   // Fall Thru!!!

      case EVT_APP_RESUME:
      case EVT_COPYRIGHT_END:
         CAStartup(pme);
         return(TRUE);

      case EVT_ALARM:
         CAExpired(pme);
         return(TRUE);

      case EVT_APP_SUSPEND:
      case EVT_APP_STOP:
         CAShutdown(pme);
         return(TRUE);

      case EVT_COMMAND:

         if(pme->m_nState == CA_EXPIRED)
           CADeactivate(pme);
            
         switch(wParam){

            case IDC_RESUME:     // Countdown
               pme->m_nState = CS_RUNNING;
               CARedraw(pme);
               break;

            case IDC_STOP:       // StopWatch/Countdown
               CSStop(pme);
               break;
               
            case IDC_DONE:
            case IDC_START:      // All
               if(pme->m_bSettingTime){
                  ITIMECTL_SetActive(pt,FALSE);
                  pme->m_bSettingTime = FALSE;
               }
               pme->m_dwDispTime = ITIMECTL_GetTime(pt);

               if(IsAlarmClock(pme))
                  pme->m_cfg.dwLastAlarm = pme->m_dwDispTime;
               else{

      // SAS - 05/22/00 - Do not allow them to activate the countdown timer when it's set to 0...

                  if(IsCountDown(pme) && !pme->m_dwDispTime){
                     CSStop(pme);
                     break;
                  }
               }
               CAActivate(pme);
               break;

            case IDC_SNOOZE:
               ACSnooze(pme);
               break;

            case IDC_RESET:      // Countdown/Stopwatch
               CAResetCounters(pme);
               if(IsStopWatch(pme)){
                  CARedraw(pme);
                  break;
               }

            case IDC_SET:        // Countdown/AlarmClock
               if(!ACGetTime(pme)){
                  ISHELL_MessageBox(pme->a.m_pIShell, NULL, 0, EINVALIDTIME);
                  break;
               }
            case IDC_CANCEL:
               ISHELL_CancelTimer(pme->a.m_pIShell, NULL, pme);
               pme->m_bSettingTime = !pme->m_bSettingTime;
               ITIMECTL_SetActive(pt,(boolean)pme->m_bSettingTime);
               CARedraw(pme);
               break;

            case IDC_ENABLE:     // AlarmClock
               CAActivate(pme);
               break;

            case IDC_DISABLE:    // AlarmClock
               CADeactivate(pme);
               break;

            case IDC_OFF:
               ISHELL_CloseApplet(pme->a.m_pIShell,FALSE);
               break;
         }
         return(TRUE);

      case EVT_KEY:
         {

         // This is used in case the AlarmClock Enabled timer is running.  If they hit any 
         // key, the clock automatically goes to activated...
   
         if(pme->m_nState == AC_SET_ENABLED){
            ACActivateTimerCallback(pme);
            break;
         }

         if(pt && ITIMECTL_HandleEvent(pt, eCode, wParam, dwParam))
            return(TRUE);

         if(pme->m_pMenu && IMENUCTL_HandleEvent(pme->m_pMenu, eCode, wParam, dwParam))
            return(TRUE);

         // SNOOZE or ANY KEY will turn the alarm clock off...

         if(pme->m_nState == CA_EXPIRED){
            ACSnooze(pme);
            return(FALSE);
         }

         if(pme->m_bSettingTime && (wParam == AVK_END || wParam == AVK_CLR)){
            ITIMECTL_SetActive(pt, FALSE);
            pme->m_bSettingTime = FALSE;
            CAResetCounters(pme);
            if(IsCountDown(pme)){
               CARedraw(pme);
               return(TRUE);
            }
            return(FALSE);
         }
         break;
         }
   }
   return(FALSE);
}

// 
// STATIC ROUTINES SHARED BY STOPWATCH AND COUNTDOWN
//

//===========================================================================
//
// 
//
//===========================================================================
static void CASaveSettings(ClockApp * pme)
{
   ISHELL_SetPrefs(pme->a.m_pIShell, AEECLSID_ALARMCLOCK, CLOCK_CFG_VERSION, &pme->m_cfg, sizeof(ClockAppCfg));
}

//===========================================================================
//
// 
//
//===========================================================================
static void CSStop(ClockApp * pme)
{
   ISHELL_CancelTimer(pme->a.m_pIShell, (PFNNOTIFY)(CSActiveTimerCallback), pme);
   if(IsStopWatch(pme))
      pme->m_dwDispTime = ISHELL_GetTimeMS(pme->a.m_pIShell) - pme->m_nTicks;
   pme->m_nState = CS_STOPPED;
   CARedraw(pme);
}

//===========================================================================
//
// 
//
//===========================================================================
static boolean CSRunning(ClockApp * pme, int32 nTicks)
{
   int32          nDiff;
   ITimeCtl       *pt;

   pt = pme->m_pTime;

   // Deal with the time...

   if(IsStopWatch(pme)){
      pme->m_dwDispTime = (nTicks - pme->m_nTicks);
      ITIMECTL_SetTimeEx(pt, pme->m_dwDispTime,TRUE);
   }
   else{

      nDiff = pme->m_nTicks - nTicks;
      if(nDiff <= 0)
         nDiff = 0;
      pme->m_dwDispTime = nDiff;
      ITIMECTL_SetTimeEx(pt, pme->m_dwDispTime,TRUE);

   // If the timer expired, then switch to displaying the alarm...

      if(!nDiff)
         return(CAExpired(pme));
   }

   return(TRUE);
}

//===========================================================================
//
// 
//
//===========================================================================
static void CSActiveTimerCallback(ClockApp * pme)
{
   uint32   nMS;

   if(pme->m_nState == CA_EXPIRED){
      CAAlarm(pme);
      nMS = ALARM_TIMER_MSECS;
   }
   else{

      nMS = ISHELL_GetTimeMS(pme->a.m_pIShell);

      if(!CSRunning(pme, (int32)nMS))
         return;

   // Hit the next second as close as possible...

      nMS = (RUNNING_UPDATE_MSECS - (ISHELL_GetTimeMS(pme->a.m_pIShell) - nMS)) + 1;
   }

   ISHELL_SetTimer(pme->a.m_pIShell, nMS, (PFNNOTIFY)(CSActiveTimerCallback), pme);
}

//
// STATIC ROUTINES SHARED BY CLOCKAPPS
//

//===========================================================================
//
// 
//
//===========================================================================
static void CARedraw(ClockApp * pme)
{
   AECHAR            szBuff[32];
   IDisplay *        pd = pme->a.m_pIDisplay;
   ClockAppState     nState;
   int16             wTitle = 0, wDefTitle;
   ITimeCtl          *pt;
   AEERect           rc, mrc;
   int               nHeight;
   uint32            dwProps;
   AEECLSID          cls;

   // Set some defaults...

   cls = 0;
   wDefTitle = 0;

   switch(pme->a.clsID){
      case AEECLSID_STOPWATCH:
         wDefTitle = IDS_STOPWATCH_TITLE;
         cls = AEECLSID_STOPWATCHCTL;
         break;
      case AEECLSID_COUNTDOWN:
         wDefTitle = IDS_COUNTDOWN_TITLE;
         cls = AEECLSID_COUNTDOWNCTL;
         break;
      case AEECLSID_ALARMCLOCK:
         cls = AEECLSID_CLOCKCTL;
         break;
   }

   pme->m_bHideTime = FALSE;

   IDISPLAY_FillRect(pd, &pme->m_rcDisp, RGB_WHITE);

   nState = pme->m_nState;

   dwProps = TP_AUTOREDRAW;

   if(nState == AC_SET_ENABLED){
      ISHELL_LoadResString(pme->a.m_pIShell, CLOCKAPPS_RES_FILE, IDS_ALARM_ON_MSG, szBuff, sizeof(szBuff));
      IDISPLAY_DrawText(pd,AEE_FONT_BOLD,szBuff,-1,2,0,NULL,0);
   }
   else{
      CASetupMenu(pme);

      if(pme->m_bSettingTime)
         wTitle = (IsAlarmClock(pme) ? IDS_ALARM_TIME : IDS_SETTIMER);
      else{
         switch(pme->m_nState){
            case AC_ALARM_ON:
               wTitle = IDS_ALARM_ON;
               break;
            case AC_ALARM_OFF:
               wTitle = IDS_ALARM_OFF;
               break;
            case CS_STOPPED:
               wTitle = IDS_STOPPED;
               break;
            case CS_RUNNING:
               wTitle = IDS_RUNNING;
               dwProps |= TP_NO_MSECONDS;
               break;
            case CA_EXPIRED:
               wTitle = (IsAlarmClock(pme) ? IDS_ALARMCLOCK_TITLE : IDS_EXPIRED);
               IDISPLAY_Backlight(pd, TRUE);
               break;
            default:
               wTitle = wDefTitle;
               break;
         }
      }
       
      ISHELL_LoadResString(pme->a.m_pIShell, CLOCKAPPS_RES_FILE, wTitle, szBuff, sizeof(szBuff));
      IDISPLAY_DrawText(pd,AEE_FONT_BOLD,szBuff,-1,2,1,NULL,0);
   }

   if((pt = pme->m_pTime) == NULL){
      if(ISHELL_CreateInstance(pme->a.m_pIShell, cls, (void **) &pt))
         return;
      pme->m_pTime = pt;
      ITIMECTL_SetProperties(pt, dwProps);
   }
   else{
      if(IsStopWatch(pme))
         ITIMECTL_SetProperties(pt, dwProps);
   }

   nHeight = IDISPLAY_GetFontMetrics(pd, AEE_FONT_BOLD, NULL, NULL) + 2;
   if(nState == AC_SET_ENABLED)
      SETAEERECT(&rc, 0, nHeight, pme->m_rcDisp.dx, pme->m_rcDisp.dy - nHeight);
   else{
      IMENUCTL_GetRect(pme->m_pMenu, &mrc);
      SETAEERECT(&rc, 0, nHeight, pme->m_rcDisp.dx, (mrc.y - nHeight));
   }

   ITIMECTL_SetRect(pt,&rc);
   ITIMECTL_SetTime(pt,pme->m_dwDispTime);
   IDISPLAY_UpdateEx(pd, TRUE);
}

//===========================================================================
//
// 
//
//===========================================================================
static void CASetupMenu(ClockApp * pme)
{
   IMenuCtl *     pm;
   ClockAppState  nState;

   if((pm = pme->m_pMenu) == NULL){

      if(ISHELL_CreateInstance(pme->a.m_pIShell, AEECLSID_SOFTKEYCTL, (void **) &pm))
         return;

      pme->m_pMenu = pm;
   }
   IMENUCTL_DeleteAll(pm);

   nState = pme->m_nState;

   if(pme->m_bSettingTime)
      AddMenuItem(pm,(IsCountDown(pme) ? IDC_START : IDC_DONE));
   else{   
      switch(pme->a.clsID){
         case AEECLSID_STOPWATCH:
            if(nState == CS_RUNNING)
               AddMenuItem(pm, IDC_STOP);
            else{
               AddMenuItem(pm, IDC_START);

      // Only show the reset option if it's already running...

               if(pme->m_dwDispTime)
                  AddMenuItem(pm, IDC_RESET);
            }
            break;

         case AEECLSID_COUNTDOWN:
            switch(nState){

               case CS_RUNNING:
                  AddMenuItem(pm,IDC_STOP);
                  break;

               case CA_EXPIRED:
                  AddMenuItem(pm, IDC_OFF);
                  AddMenuItem(pm, IDC_RESET);
                  break;

               case CS_IDLE:
               case CS_STOPPED:
                  if(pme->m_dwDispTime){
                     AddMenuItem(pm, IDC_START);
                     AddMenuItem(pm, IDC_RESET);
                  }
                  else
                     AddMenuItem(pm, IDC_SET);
                  break;
            }                  
            break;

         case AEECLSID_ALARMCLOCK:
            if(nState == CA_EXPIRED){
               AddMenuItem(pm, IDC_OFF);
               AddMenuItem(pm, IDC_SNOOZE);
            }
            else{
               if(nState == AC_ALARM_ON)
                  AddMenuItem(pm,IDC_DISABLE);
               else{
                  if(pme->m_dwDispTime)
                     AddMenuItem(pm, IDC_ENABLE);
               }
               AddMenuItem(pm,IDC_SET);
            }
            break;
      }
#if defined(FEATURE_UI_FLIP)
      AddMenuItem(pm,IDC_EXIT);
#endif
   }
   IMENUCTL_Redraw(pm);
   IMENUCTL_SetActive(pm,TRUE);
}

//===========================================================================
//
// 
//
//===========================================================================
static void CAActivate(ClockApp * pme)
{
   uint32   dwTimeNow;

   pme->m_nState = (ClockAppState)(IsAlarmClock(pme) ? AC_SET_ENABLED : CS_RUNNING);
   pme->m_bSettingTime = FALSE;

   // Shows the current value of m_dwDispTime...

   CARedraw(pme);

   if(pme->m_nState == CS_RUNNING){

   // Note that we now use m_nTicks to show the updated time.  If it's the stopwatch or
   // the m_nTicks time is 0, we must get it..

      dwTimeNow = ISHELL_GetTimeMS(pme->a.m_pIShell);

      if(IsStopWatch(pme))
         pme->m_nTicks = dwTimeNow - pme->m_dwDispTime;
      else
         pme->m_nTicks = dwTimeNow;

      if(IsCountDown(pme))
         pme->m_nTicks += pme->m_dwDispTime;

      CSActiveTimerCallback(pme);
   }
   else
      ISHELL_SetTimer(pme->a.m_pIShell, ALARM_ON_TIMER, (PFNNOTIFY)(ACActivateTimerCallback), pme);
}

//===========================================================================
//
// 
//
//===========================================================================
static void CADeactivate(ClockApp * pme)
{
   ISHELL_Beep(pme->a.m_pIShell, BEEP_OFF, FALSE);

   CACancelTimer(pme);
   ISHELL_CancelTimer(pme->a.m_pIShell, NULL, pme);

   if(IsAlarmClock(pme))
      pme->m_nState = AC_ALARM_OFF;
   else
      pme->m_nState = CS_IDLE;

   pme->m_bSettingTime = FALSE;
   CARedraw(pme);
}

//===========================================================================
//
// 
//
//===========================================================================
static boolean CAExpired(ClockApp *pme)
{
   pme->m_nState = CA_EXPIRED;

   if(IsCountDown(pme))
      CAResetCounters(pme);

   if(!pme->m_bActive){
      ISHELL_StartApplet(pme->a.m_pIShell, pme->a.clsID);
      return(TRUE);
   }

   CACancelTimer(pme);
   CARedraw(pme);
   ISHELL_SetTimer(pme->a.m_pIShell, MAX_ALARM_MSECS, (PFNNOTIFY)(CADeactivate), pme);

   // Use sound/vib to alert them...

   if(!ISHELL_Beep(pme->a.m_pIShell, BEEP_ALERT, TRUE))
      ISHELL_Beep(pme->a.m_pIShell, BEEP_VIBRATE_ALERT, FALSE);

   CSActiveTimerCallback(pme);
   return(TRUE);
}

//===========================================================================
//
// 
//
//===========================================================================
static void CAAlarm(ClockApp * pme)
{
   ITimeCtl  * pt = pme->m_pTime;
   AEERect     rc;
   IDisplay *  pd = pme->a.m_pIDisplay;

   pme->m_bHideTime = !pme->m_bHideTime;

   if(pme->m_bHideTime){
      ITIMECTL_GetRect(pt, &rc);
      IDISPLAY_FillRect(pd, &rc, RGB_WHITE);
      IDISPLAY_Update(pd);
   }
   else
      ITIMECTL_Redraw(pt);
}

//===========================================================================
//
// 
//
//===========================================================================
static void CAStartup(ClockApp * pme)
{
   AEEDeviceInfo     dm;
   ClockAppState     nState = pme->m_nState;
   IDisplay *        pd;

   // Get the display interface...

   pme->m_bActive = TRUE;

   pd = pme->a.m_pIDisplay;

   ISHELL_GetDeviceInfo(pme->a.m_pIShell, &dm);
   pme->m_rcDisp.dx = dm.cxScreen;
   pme->m_rcDisp.dy = dm.cyScreen;

   // If we were running the countdown timer in the background, cancel the    
   // expiration timer.

   if(IsCountDown(pme))
      CACancelTimer(pme);
   
   // If the countdown timer is running or the alarm clock has expired, start
   // the alarm...

   CARedraw(pme);

   switch(nState){

      case CS_RUNNING:
         CSActiveTimerCallback(pme);
         break;

      case CA_EXPIRED:
         CAExpired(pme);
         break;
   }
}

//===========================================================================
//
// 
//
//===========================================================================
static void CACancelTimer(ClockApp * pme)
{
   if(IsAlarmClock(pme))
      ISHELL_CancelAlarm(pme->a.m_pIShell, AEECLSID_ALARMCLOCK, 0);

   if(pme->m_cfg.dwAlarmTime){
      pme->m_cfg.dwAlarmTime = 0;
      CASaveSettings(pme);
   }
   ISHELL_CancelTimer(pme->a.m_pIShell, (PFNNOTIFY)(CAExpired), pme);
   if(pme->m_bTimerActive){
      pme->m_bTimerActive = FALSE;
      IAPPLET_Release((IApplet *)pme);
   }
}

//===========================================================================
//
// 
//
//===========================================================================
static void CASetTimer(ClockApp * pme, uint32 dwTime)
{
   if(!pme->m_bTimerActive){
      IAPPLET_AddRef((IApplet *)pme);
      pme->m_bTimerActive = TRUE;
   }
   ISHELL_SetTimer(pme->a.m_pIShell, pme->m_dwDispTime, (PFNNOTIFY)(CAExpired), pme);
}

//===========================================================================
//
// 
//
//===========================================================================
static void CAShutdown(ClockApp * pme)
{
   // Cancel any foreground timers...

   pme->m_bActive = FALSE;

   ISHELL_CancelTimer(pme->a.m_pIShell, (PFNNOTIFY)(CADeactivate), pme);
   ISHELL_CancelTimer(pme->a.m_pIShell, (PFNNOTIFY)(ACActivateTimerCallback),pme);
   ISHELL_CancelTimer(pme->a.m_pIShell, (PFNNOTIFY)(CSActiveTimerCallback), pme);

   if(IsAlarmClock(pme)){
      if(pme->m_nState == AC_ALARM_OFF)
         CAResetCounters(pme);
   }
   else{
      if(IsCountDown(pme) && pme->m_dwDispTime && pme->m_nState == CS_RUNNING)
         CASetTimer(pme, pme->m_dwDispTime);
   }

   if(pme->m_pMenu != NULL){
      IMENUCTL_Release(pme->m_pMenu);
      pme->m_pMenu = NULL;
   }

   if(pme->m_pTime != NULL){
      ITIMECTL_Release(pme->m_pTime);
      pme->m_pTime = NULL;
   }
}

//===========================================================================
//
// 
//
//===========================================================================
static void CAResetCounters(ClockApp * pme)
{
   pme->m_dwDispTime = 0;
   pme->m_nTicks = 0;

   // Clear the persistent stopwatch time...

   if(pme->m_cfg.dwSWTime){
      pme->m_cfg.dwSWTime = 0;
      CASaveSettings(pme);
   }   
}


//
// STATIC ALARM CLOCK SPECIFIC ROUTINES
//

//===========================================================================
//
// 
//
//===========================================================================
static uint32 ACGetTime(ClockApp * pme)
{
   uint32   nSecs;

   nSecs = ISHELL_GetSeconds(pme->a.m_pIShell);

   if(nSecs < ONE_YEAR_SECS)
      return(0);

   nSecs = (nSecs % SECS_PER_DAY) * 1000l;

   if(!nSecs)
      nSecs++;

   return(nSecs);
}

//===========================================================================
//
// 
//
//===========================================================================
static void ACActivateTimerCallback(ClockApp * pme)
{
   ISHELL_CancelTimer(pme->a.m_pIShell, (PFNNOTIFY)(ACActivateTimerCallback), pme);
   pme->m_nState = AC_ALARM_ON;
   ACSetTimer(pme);
   ISHELL_CloseApplet(pme->a.m_pIShell,FALSE);
}

//===========================================================================
//
// 
//
//===========================================================================
static void ACSetTimer(ClockApp * pme)
{
   uint32      dwNow, dwAlarm, dwMSecs, nMinsNow, nMinsAlarm;

   pme->m_nState = AC_ALARM_ON;

   // Calculate the timeout...

   dwAlarm = pme->m_dwDispTime;

   // Get the actual time since midnight for today...

   dwNow = ACGetTime(pme);

   // Set the timer.  If the current time is showing as invalid, we will
   // set it to 30 seconds.  The timer will then expire and we will check
   // it again...

   if(dwNow){

      nMinsNow = dwNow / (60l * 1000l);
      nMinsAlarm = dwAlarm / (60l * 1000l);

   // Here is the rule:
   // - If the alarm is set ahead of the current time, calc the difference.
   // - If the alarm matches the current time, set it for 5 seconds
   // - If the alarm is less then the current time, set it to wrap...

      if(nMinsNow < nMinsAlarm)
         dwMSecs = dwAlarm - dwNow;
      else{
         if(nMinsNow == nMinsAlarm)
            dwMSecs = 5000l;
         else
            dwMSecs = (MSECS_PER_DAY - dwNow) + dwAlarm;
      }
   }
   else
      dwMSecs = VALID_TIME_POLL;

   pme->m_cfg.dwAlarmTime = dwAlarm;
   CASaveSettings(pme);
   ISHELL_SetAlarm(pme->a.m_pIShell, AEECLSID_ALARMCLOCK, 0, (uint32)(dwMSecs / (1000l * 60l)));
#if 0
   CASetTimer(pme, dwMSecs);
#endif
}

//===========================================================================
//
// 
//
//===========================================================================
static void ACSnooze(ClockApp * pme)
{
   CADeactivate(pme);
   if(IsAlarmClock(pme)){
      pme->m_dwDispTime = ACGetTime(pme) + SNOOZE_MSECS;
      CAActivate(pme);
   }
}

//
// STATIC UTILITY ROUTINES
//

//===========================================================================
//
// 
//
//===========================================================================
static void AddMenuItem(IMenuCtl * pm, int nID)
{
   IMENUCTL_AddItem(pm, CLOCKAPPS_RES_FILE,(int16)nID,(int16)nID,NULL,0);
}

