/*===========================================================================

FILE: Extension.c
  
SERVICES: Sample class exported through IModule methods
    
DESCRIPTION
   This is to demonstrate class functionality. This will be invoked by 
   the ExtUse applet.
      
PUBLIC CLASSES:  
   IExtension - AEECLSID_EXTENSION_CLS 0x01009021
        
INITIALIZATION AND SEQUENCING REQUIREMENTS:
   The following explanation applies to this sample containing one applet which serves
   as a base for app developers to create their own classes using IModule Services:
   

   In the applet source file (like this one), include AEEAppGen.h.

   Mandatory Sections in Applet Source (this file):
   -----------------------------------------------
   Following Mandatory Sections are required for each applet source file. 
   (Search for "Mandatory" to identify these sections)

   Includes:     
      Copy this section as-is from the sample applet. It contains:
      AEEAppGen.h: For AEEApplet declaration

   Type Declarations:
		A data structure must be defined to hold the app specific data (class members
		and VTables. In this structure, the first elements must be your VTable.

   Functions: (For more details, see corresponding function description in this applet)

      AEEClsCreateInstance(): creates an instance of the class and initializes it. 
                              It is called by AEEModGen when class is being created.

   
   Important Notes:
   ---------------
   1. DO NOT use any "static data" in the applet. Always use the functions exported by
	   AEEStdlib or by IHeap services to dynamically allocate data and make it a member of 
		the applet structure. 
   2. DO NOT include and link "standard C library". Use AEE Memory Services (in AEEHeap.h) and Standard Library macros(in AEEStdLib.h).
      For example, use MALLOC() to allocate memory, WSTRCPY() to make a copy of Unicode (wide) string.
   3. BREW is Unicode(wide string) compliant ONLY (no ISOLATIN1/ANSI) except for file names which are ISOLATIN1/ANSI.
      ALWAYS USE AECHAR instead of "char". Use string services provided in AEEStdLib.h for string manipulation.
   4. It is always a good idea to DEFINE RESOURCES using BREW ResourceEditor. Make Strings, Bitmaps, Dialogs, etc. 
      as resources. ResourceEditor saves resources as .bri file, generates resource header file 
      and compiles .bri into a .bar binary file, which can be used by the applet.

   Miscellanoeus Notes:
   -------------------
   1. Make sure that the class ID used for the app is the same as that defined corresponding in .MIF file
   2. Always make sure that compiled resource (.bar) file and corresponding
      resource header (a) reside in app directory and (b) are included in the applet code.
      Define a constant APP_RES_FILE containing the name of the compiled resource file (with .bar extension).

   More than one applet:
   --------------------
   If more than one applet needs to be defined, then do the following
   (1) Follow the above description for each applet




   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/
/*===============================================================================
INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "Extension.h"		// Our header to distribute for our class

/*===============================================================================
CLASS DEFINITIONS
- If you have more than one class in this module, you should list them here.
=============================================================================== */
typedef struct _ExtensionCls
{
	// Declare our VTable
	// by doing so, we have placed function pointers
	// in the begininng of this structure
	DECLARE_VTBL(IExtensionCls)

	// Class member variables
	uint32		m_nRefs;		// References to us
	IShell *	   m_pIShell;		// copy of Shell pointer
	IDisplay *	m_pIDisplay;	// Display interface pointer
	IModule *	m_pIModule;		// IModule interface pointer
}ExtensionCls;

// Exported AEEClsCreateInstance
extern int			AEEClsCreateInstance(AEECLSID, IShell*, IModule*, void**);

/*-------------------------------------------------------------------
Function Prototypes
-------------------------------------------------------------------*/
static uint32		ExtensionCls_AddRef(IExtensionCls * po);
static uint32		ExtensionCls_Release(IExtensionCls * po);
static int			ExtensionCls_DrawSmiley(IExtensionCls * po, AEEPoint ptCenter, int nRadius);
static int			ExtensionCls_DrawHello(IExtensionCls * po, AEEPoint ptXYOffset);

/*===========================================================================

Function:  ExtensionCls_New()

Description:
		This function is called from AEEClsCreateInstance. This helps 
		with construction of the class and initialization of it's members.	

Prototype:
	int ExtensionCls_New(int16 nSize, IShell *pIShell, IModule* pIModule, IModule ** ppMod)

Parameters:
	nSize: Specifies the size (in bytes) of the structure to be allocated for the module
	Typically, this is the size of the structure (or class)
	pIShell: Pointer to IShell interface
	pIModule: Pointer to the IModule interface
	ppMod: [ou]: On return, *ppMod contains a valid pointer to the ExtensionCls structure 
	(or class)

Return Value:
	AEE_SUCCESS: If successful
	Error code: IF failed

Comments:  None

Side Effects: None

==============================================================================*/
int ExtensionCls_New(int16 nSize, IShell *pIShell, IModule* pIModule, IModule ** ppMod)
{
	ExtensionCls *			   pMe = NULL;
	VTBL(IExtensionCls) *	modFuncs;

	if( !ppMod || !pIShell || !pIModule )
		return EFAILED;

	*ppMod = NULL;

	// Allocate memory for the ExtensionCls object
	if( nSize < sizeof(ExtensionCls) )
		nSize += sizeof(ExtensionCls);

	if( (pMe = (ExtensionCls *)MALLOC(nSize + sizeof(IExtensionClsVtbl))) == NULL )
		return ENOMEMORY;

	// Allocate the vtbl and initialize it. Note that the modules and apps must not
	// have any static data. Hence, we need to allocate the vtbl as well.
	modFuncs = (IExtensionClsVtbl *)((byte *)pMe + nSize);
	//Initialize individual entries in the VTBL
	modFuncs->AddRef			= ExtensionCls_AddRef;
	modFuncs->Release			= ExtensionCls_Release;
	modFuncs->DrawSmiley		= ExtensionCls_DrawSmiley;
	modFuncs->DrawHello			= ExtensionCls_DrawHello;

	// initialize the vtable
	INIT_VTBL(pMe, IModule, *modFuncs);

	// initialize the data members
	pMe->m_nRefs      = 1;
	pMe->m_pIShell    = pIShell;
	pMe->m_pIModule   = pIModule;

	// Add References and get IDisplay
	ISHELL_AddRef(pIShell);
	IMODULE_AddRef(pIModule);
	if( ISHELL_CreateInstance(pIShell, AEECLSID_DISPLAY, (void **)&pMe->m_pIDisplay) != SUCCESS )
		return EFAILED;

	// Set the pointer in the parameter
	*ppMod = (IModule*)pMe;

	return AEE_SUCCESS;
}

/*===========================================================================

Function:  ExtensionCls_AddRef()

Description:
		This function increases the reference count for the ExtensionCls object

Prototype:
	uint32 ExtensionCls_AddRef(IExtensionCls * po)

Parameters:
   po: Pointer to IExtensionCls interface whos reference count is to be incremented

Return Value:
	The updated reference count

Comments:  None

Side Effects: None

==============================================================================*/
static uint32 ExtensionCls_AddRef(IExtensionCls * po)
{
   return (++((ExtensionCls *)po)->m_nRefs);
}
/*===========================================================================

Function:  ExtensionCls_Release()

Description:
		This function decreases the reference count for the ExtensionCls object. If
		the reference count reaches zero, it does the required cleanup

Prototype:
	uint32 ExtensionCls_Release(IExtensionCls* po)

Parameters:
   po: Pointer to the IExtensionCls interface whose reference count needs to be
	decremented.

Return Value:
   The updated reference count

Comments:  None

Side Effects: None

==============================================================================*/
static uint32 ExtensionCls_Release(IExtensionCls * po)
{
	ExtensionCls *		pMe = (ExtensionCls *)po;

	if( --pMe->m_nRefs != 0 )
		return pMe->m_nRefs;

	// Ref count is zero. So, release memory associated with this object.

	if( pMe->m_pIDisplay )
		IDISPLAY_Release(pMe->m_pIDisplay);

	// Release interfaces
	ISHELL_Release(pMe->m_pIShell);
	IMODULE_Release(pMe->m_pIModule);
	//Free the object itself
	FREE_VTBL(pMe, IModule);
	FREE( pMe );

	return 0;
}


/*===========================================================================

Function:  AEEClsCreateInstance()

Description:
		This function decreases the reference count for the ExtensionCls object. If
		the reference count reaches zero, it does the required cleanup.
		Calls a helper function to initialize the class members and VTable.

Prototype:
	uint32 AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

Parameters:
   ClsId: Clsid passed in
   pIShell: IShell pointer
   po: IModule pointer
   ppObj[out]: Pointer to a pointer of the class object to fill

Return Value:
   AEE_SUCCESS on pass
   Error code on failure
Comments:  None

Side Effects: None

==============================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
{
	*ppObj = NULL;
		
	if( ClsId == AEECLSID_EXTENSION_CLS )
	{
		if( ExtensionCls_New(sizeof(ExtensionCls), pIShell, po, (IModule **)ppObj) == SUCCESS )
			return AEE_SUCCESS;
	}
	return EFAILED;

}
/*===========================================================================

Function:  ExtensionCls_DrawSmiley()

Description:
		This function Draws a square face on the screen at the location given
      of the specified size, with a minimum radius being 9 pixels.
      This is the implementation of an exported function for the IExtensionCls

Prototype:
	int ExtensionCls_DrawSmiley(IExtensionCls * po, AEEPoint ptCenter, int nRadius)

Parameters:
   po:         Pointer to IExtensionCls interface whose reference count is to be incremented
   ptCenter:   Point of the center of Face
   nRadius:    Size of face's radius, so face will occupy 2 * nRadius

Return Value:
	SUCCESS for successful execution
   EFAILED for error

Comments:  Caller must Update the display

Side Effects: None

==============================================================================*/
static int	ExtensionCls_DrawSmiley(IExtensionCls * po, AEEPoint ptCenter, int nRadius)
{
	ExtensionCls *		pMe = (ExtensionCls *) po;
	AEEPoint		      ptTmp;

	if( !pMe || !pMe->m_pIDisplay || nRadius < 9 )
		return EFAILED;

	// Top and Left side of the box
	ptTmp.x = ptCenter.x - nRadius;
	ptTmp.y = ptCenter.y - nRadius;


	IDISPLAY_DrawHLine(pMe->m_pIDisplay, ptTmp.x, ptTmp.y, nRadius * 2); 
	IDISPLAY_DrawVLine(pMe->m_pIDisplay, ptTmp.x, ptTmp.y, nRadius * 2); 

	// Bottom side
	ptTmp.y += 2 * nRadius;
	IDISPLAY_DrawHLine(pMe->m_pIDisplay, ptTmp.x, ptTmp.y, nRadius * 2); 

	// Right Side
	ptTmp.x += 2 * nRadius;
	ptTmp.y -= 2 * nRadius;
	IDISPLAY_DrawVLine(pMe->m_pIDisplay, ptTmp.x, ptTmp.y, nRadius * 2); 

	// eyes
	ptTmp.x = ptCenter.x - (nRadius / 2);
	ptTmp.y = ptCenter.y - (nRadius / 2);
	IDISPLAY_DrawVLine(pMe->m_pIDisplay, ptTmp.x, ptTmp.y, nRadius / 3 ); 

	ptTmp.x += nRadius;
	IDISPLAY_DrawVLine(pMe->m_pIDisplay, ptTmp.x, ptTmp.y, nRadius / 3 ); 

	// mouth
	ptTmp.x = ptCenter.x - (nRadius * 2 / 3);
	ptTmp.y = ptCenter.y + (nRadius * 2 / 3);
	IDISPLAY_DrawHLine(pMe->m_pIDisplay, ptTmp.x, ptTmp.y, nRadius * 4 / 3); 

	return SUCCESS;
}
/*===========================================================================

Function:  ExtensionCls_DrawHello()

Description:
		This function Draws the word HELLO on the screen at the location given
      This is the implementation of an exported function for the IExtensionCls

Prototype:
	int ExtensionCls_DrawHello(IExtensionCls * po, AEEPoint ptXYOffset)

Parameters:
   po:            Pointer to IExtensionCls interface whose reference count is to be incremented
   ptXYOffset:    XY Point of the Text to draw

Return Value:
	SUCCESS for successful execution
   EFAILED for error

Comments:  Caller must Update the display

Side Effects: None

==============================================================================*/
static int	ExtensionCls_DrawHello(IExtensionCls * po, AEEPoint ptXYOffset)
{
	ExtensionCls *	   pMe = (ExtensionCls *) po;
	AECHAR		      szbuff[] = {'H','E','L','L','O','\0'};

	if( !pMe || !pMe->m_pIDisplay )
		return EFAILED;

	IDISPLAY_DrawText(pMe->m_pIDisplay, AEE_FONT_BOLD, szbuff, -1, ptXYOffset.x, ptXYOffset.y, NULL, 0);

	return SUCCESS;

}

/*========================End of File ================================================*/

