//        Copyright  1999-2001 QUALCOMM Incorporated.
//               All Rights Reserved.
//            QUALCOMM Proprietary/GTDR
#ifndef __EXTENSIONCLS_H__
#define __EXTENSIONCLS_H__

#include "AEE.h"			// Standard AEE definitions
#include "AEEShell.h"	// AEE Shell Services
#include "AEEAppgen.h"		// Madatory include
#include "AEEModgen.h"		// Mandatory include
#include "AEEHeap.h"		// Memory interface
#include "AEEStdlib.h"		// BREW's standard library
#include "AEEDisp.h"		// Display interface
#include "AEEUsageAppIDs.h"

// Extension Class typedef
typedef struct _IExtensionCls IExtensionCls;

//********************************************************************************************************************************
//
// IExtensionCls Interface Definition
//
//********************************************************************************************************************************
QINTERFACE(IExtensionCls)
{
	DECLARE_IBASE(IExtensionCls)
	int		(*DrawSmiley)(IExtensionCls * po, AEEPoint ptCenter, int nRadius);
	int		(*DrawHello)(IExtensionCls * po, AEEPoint ptXYOffset);
};

#define IEXTCLS_AddRef(p)				      	GET_PVTBL(p,IExtensionCls)->AddRef(p)
#define IEXTCLS_Release(p)					      GET_PVTBL(p,IExtensionCls)->Release(p)
#define IEXTCLS_DrawSmiley(p,ptC,nR)			GET_PVTBL(p,IExtensionCls)->DrawSmiley(p,ptC,nR)
#define IEXTCLS_DrawHello(p,ptC)				   GET_PVTBL(p,IExtensionCls)->DrawHello(p,ptC)


/*=====================================================================
  INTERFACES   DOCUMENTATION
======================================================================= 

Interface Name: IExtensionCls

Description: 
   IExtensionCls is a Class made to show an exampl eof an extension
   class under BREW. ExtUse applet is complementary to this class
   since it dynamically creates and instance of this class and uses
   its exported methods.

======================================================================= 

Function:  IEXTCLS_AddRef()

Description:
		This function increases the reference count for the ExtensionCls object

Prototype:
	uint32 IEXTCLS_AddRef(IExtensionCls * po)

Parameters:
   po: Pointer to IExtensionCls interface whos reference count is to be incremented

Return Value:
	The updated reference count

Comments:  None

Side Effects: None

======================================================================= 

Function:  IEXTCLS_Release()

Description:
		This function decreases the reference count for the ExtensionCls object. If
		the reference count reaches zero, it does the required cleanup

Prototype:
	uint32 IEXTCLS_Release(IExtensionCls* po)

Parameters:
   po: Pointer to the IExtensionCls interface whose reference count needs to be
	decremented.

Return Value:
   The updated reference count

Comments:  None

Side Effects: None

======================================================================= 

Function:  IEXTCLS_DrawSmiley()

Description:
		This function Draws a square face on the screen at the location given
      of the specified size, with a minimum radius being 9 pixels.
      This is the implementation of an exported function for the IExtensionCls

Prototype:
	int IEXTCLS_DrawSmiley(IExtensionCls * po, AEEPoint ptCenter, int nRadius)

Parameters:
   po:         Pointer to IExtensionCls interface whose reference count is to be incremented
   ptCenter:   Point of the center of Face
   nRadius:    Size of face's radius, so face will occupy 2 * nRadius

Return Value:
	SUCCESS for successful execution
   EFAILED for error

Comments:  Caller must Update the display

Side Effects: None

======================================================================= 

Function:  IEXTCLS_DrawHello()

Description:
		This function Draws the word HELLO on the screen at the location given
      This is the implementation of an exported function for the IExtensionCls

Prototype:
	int IEXTCLS_DrawHello(IExtensionCls * po, AEEPoint ptXYOffset)

Parameters:
   po:            Pointer to IExtensionCls interface whose reference count is to be incremented
   ptXYOffset:    XY Point of the Text to draw

Return Value:
	SUCCESS for successful execution
   EFAILED for error

Comments:  Caller must Update the display

Side Effects: None

=======================================================================*/

#endif  //__EXTENSIONCLS_H__


