/*===========================================================================

FILE: ExtUse.c
  
SERVICES: Sample applet using AEE dynamic class libraries
    
DESCRIPTION
   This file contains the implementation of a simple example using an
   AEE dynamic class libraries.
   This is to demonstrate loading a dynamic class module and calling functions
   from it.
      
PUBLIC CLASSES:  
   N/A
        
INITIALIZATION AND SEQUENCING REQUIREMENTS:
   The following explanation applies to this sample containing one applet which serves
   as a base for app developers to create their own applets using AEE Services:
   

   In the applet source file (like this one), include AEEAppGen.h.

   Mandatory Sections in Applet Source (this file):
   -----------------------------------------------
   Following Mandatory Sections are required for each applet source file. 
   (Search for "Mandatory" to identify these sections)

   Includes:     
      Copy this section as-is from the sample applet. It contains:
      AEEAppGen.h: For AEEApplet declaration

   Type Declarations:
		A data structure must be defined to hold the app specific data. In this structure,
		the first element must be of type AEEApplet.

   Functions: (For more details, see corresponding function description in this applet)

      App_HandleEvent(): is the Event Handler to this applet.
                         Copy the function outline from the sample applet and add app specific code.

      AEEClsCreateInstance(): creates an instance of the applet and initializes it. 
                              It is called by AEEModGen when applet is being created.

   
   Important Notes:
   ---------------
   1. DO NOT use any "static data" in the applet. Always use the functions exported by
	   AEEStdlib or by IHeap services to dynamically allocate data and make it a member of 
		the applet structure. 
   2. DO NOT include and link "standard C library". Use AEE Memory Services (in AEEHeap.h) and Standard Library macros(in AEEStdLib.h).
      For example, use MALLOC() to allocate memory, WSTRCPY() to make a copy of Unicode (wide) string.
   3. BREW is Unicode(wide string) compliant ONLY (no ISOLATIN1/ANSI) except for file names which are ISOLATIN1/ANSI.
      ALWAYS USE AECHAR instead of "char". Use string services provided in AEEStdLib.h for string manipulation.
   4. It is always a good idea to DEFINE RESOURCES using BREW ResourceEditor. Make Strings, Bitmaps, Dialogs, etc. 
      as resources. ResourceEditor saves resources as .bri file, generates resource header file 
      and compiles .bri into a .bar binary file, which can be used by the applet.

   Miscellanoeus Notes:
   -------------------
   1. Make sure that the class ID used for the app is the same as that defined corresponding in .MIF file
   2. Always make sure that compiled resource (.bar) file and corresponding
      resource header (a) reside in app directory and (b) are included in the applet code.
      Define a constant APP_RES_FILE containing the name of the compiled resource file (with .bar extension).

   More than one applet:
   --------------------
   If more than one applet needs to be defined, then do the following
   (1) Follow the above description for each applet




   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/


/*===============================================================================
INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "AEEModGen.h"                    // Module interface definitions
#include "AEEAppGen.h"                    // Applet interface definitions
#include "AEEShell.h"                     // Shell interface definitions
#include "AEEDisp.h"                      // Display interface defintions
#include "AEEFile.h"		                  //	AEEFile Services
#include "..\extension\extension.h"       // Extension Class header

/*-------------------------------------------------------------------
Function Prototypes
-------------------------------------------------------------------*/
static boolean ExtUse_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);

/*===============================================================================
FUNCTION DEFINITIONS
=============================================================================== */
/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to InitAppData(). This is done as follows:
   InitAppData() is called to initialize AppletData instance. It is app developers 
   responsibility to fill-in app data initialization code of InitAppData(). App developer
   is also responsible to release memory allocated for data contained in AppletData -- this can be
   done in FreeAppData().

PROTOTYPE:
	int AEEAppCreateInstance(AEECLSID clsID, IShell* pIShell, IModule* pIModule, 
										  IApplet** ppApplet)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell interface. 

	pIModule: pin]: Contains pointer to the IModule interface to the current module to which
	this app belongs

	ppApplet: [out]: On return, *ppApplet must point to a valid AEEApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  SUCCESS: If the app needs to be loaded and if AEEApplet_New() invocation was successful
  EFAILED: If the app does not need to be loaded or if errors occurred in AEEApplet_New().
  If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
{
   *ppObj = NULL;
		
   if( ClsId == AEECLSID_EXTUSE_APP )
   {
      if( AEEApplet_New(sizeof(AEEApplet), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)ExtUse_HandleEvent,NULL )
         == TRUE)
      {
         return AEE_SUCCESS;
      }
   }
	return EFAILED;
}

/*===========================================================================

FUNCTION ExtUse_HandleEvent
  
DESCRIPTION
   This is the EventHandler for this app. All events to this app are handled in this
   function. All APPs must supply an Event Handler.
    
PROTOTYPE:
   boolean ExtUse_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
      
PARAMETERS:
   pi: Pointer to the AEEApplet structure. This structure contains information specific
   to this applet. It was initialized during the AppCreateInstance() function.
        
   ecode: Specifies the Event sent to this applet
          
   wParam, dwParam: Event specific data.
            
DEPENDENCIES
   none
              
RETURN VALUE
   TRUE: If the app has processed the event
   FALSE: If the app did not process the event
                
SIDE EFFECTS
   none
===========================================================================*/
static boolean ExtUse_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
   AEEDeviceInfo	   di;
   AECHAR			   szBuf[] = {'P','r','e','s','s',' ','1','o', 'r', '2', '\0'};
   IExtensionCls *	pExtCls = NULL;
   AEEPoint			   ptSmile;
   AEEPoint			   ptHello;
   AEEApplet *       pMe = (AEEApplet*)pi;

   if( !pMe )
      return FALSE;

   ptSmile.x = 25;
   ptSmile.y = 25;
   ptHello.x = 20;
   ptHello.y = 44;

   switch (eCode) 
	{
      case EVT_APP_START:                        
         ISHELL_GetDeviceInfo(pMe->m_pIShell, &di);
         IDISPLAY_ClearScreen(pMe->m_pIDisplay);  // Clear whole screen

         // Display string on screen
         IDISPLAY_DrawText(pMe->m_pIDisplay, AEE_FONT_BOLD, szBuf, -1, 0, 0, 
            0, IDF_ALIGN_CENTER | IDF_ALIGN_MIDDLE);

         IDISPLAY_Update (pMe->m_pIDisplay);
			return TRUE;
      case EVT_APP_STOP:
         return TRUE;
	  case EVT_KEY_PRESS:
			switch( wParam )
			{
			case AVK_1:
				if( ISHELL_CreateInstance(pMe->m_pIShell, AEECLSID_EXTENSION_CLS, (void **)&pExtCls) )
					return FALSE;
				IDISPLAY_ClearScreen(pMe->m_pIDisplay);  
				IEXTCLS_DrawSmiley(pExtCls, ptSmile, 15);
				IDISPLAY_Update(pMe->m_pIDisplay);
				IEXTCLS_Release(pExtCls);
				
				return TRUE;
			case AVK_2:
				if( ISHELL_CreateInstance(pMe->m_pIShell, AEECLSID_EXTENSION_CLS, (void **)&pExtCls) )
					return FALSE;
				IDISPLAY_ClearScreen(pMe->m_pIDisplay);
				IEXTCLS_DrawHello(pExtCls,ptHello);
				IDISPLAY_Update(pMe->m_pIDisplay);
				IEXTCLS_Release(pExtCls);				
				return TRUE;

			}
		  return FALSE;

      default:
         break;
   }

   return FALSE;
}


