/*===========================================================================

FILE: I411Viewer.c
  
SERVICES: Sample applet using AEE IIMAGE Base Classe, and registering a viewer
    
DESCRIPTION
    This file contains a class based on IIMAGE
	The nature of the image format is not to contain animation, nor different sizes
	they are a specific height and width(64X48) and do not change.
	The other Class here is a regular applet to allow the user, to register the 
	viewer class, or to unregiste it. Also shown is a method for showing an example image of 
	the type the image viewer handles, to show that it worked.
	AEECLSID_REG_411 - Registers/Unregisters our viewer
	AEECLSID_VIEW_411 - Implements the IIMAGE interface
PUBLIC CLASSES:  
   N/A
        
INITIALIZATION AND SEQUENCING REQUIREMENTS:



   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/


/*===============================================================================
INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
// Mandatory includes
#include "AEEAppGen.h"	// AEEApplet Declaration
#include "AEE.h"			// Standard AEE Declarations
#include "AEEShell.h"	// AEE Shell Services
#include "AEEDisp.h"		// AEE Display Services
#include "AEEHeap.h"		// AEE Memory Allocation Services
#include "AEEStdLib.h"  // AEE StdLib Services
#include "AEEUsageAppIDs.h" // Usage app ID

// App developer code BEGIN
//
// Services used by app
#include "I411Viewer.h" // Our Own
// App resource header generated by BREW ResourceEditor.
#include "I411Viewer_res.h" // res stuff

/*===========================================================================

  PUBLIC DATA DECLARATIONS
  
===========================================================================*/

/*-------------------------------------------------------------------
Type Declarations
-------------------------------------------------------------------*/

/*-------------------------------------------------------------------
      Type Declarations
-------------------------------------------------------------------*/
// Mandatory App specific data. 
// IMPORTANT NOTE: DO NOT CHANGE THE NAME OF THE FOLLOWING STRUCTURES

// Our Registration Application has just a menu
typedef struct _i411RegApp i411RegApp;
struct _i411RegApp 
{
	AEEApplet      a;
	IMenuCtl *     imenu;
   int            nColorDepth;
};

/*-------------------------------------------------------------------
Function Prototypes
-------------------------------------------------------------------*/
// Our prototypes for implementation of IIMAGE methods
static uint32        CI411Image_AddRef(IImage * po);
static uint32        CI411Image_Release(IImage * po);
static void          CI411Image_Draw(IImage * po, int x, int y);
static void          CI411Image_GetInfo(IImage * po, AEEImageInfo * pi);
static boolean       CI411Image_SetData(IImage * po, void * pData);
static void          CI411Image_SetParm(IImage * po, int nParm, int n1, int n2);
static void          CI411Image_DrawFrame(IImage * po,int nFrame, int x, int y);
static void          CI411Image_Start(IImage * po, int x, int y);
static void          CI411Image_Stop(IImage * po);
// Our prototypes for our own methods to assist in carrying out IIMAGE methods
static void          CI411Image_Animate(CI411Image * po);
static void	*		   CI411Image_SetupBMPImage(void * pImageData, AEEImageInfo * m_info);
static void          CI411Image_SetStream(IImage * po, IAStream * ps);
static void          CI411Image_Notify(IImage * po, PFNIMAGEINFO pfn, void * pUser);
static boolean       CI411Image_HandleEvent(IImage * po, AEEEvent e, uint16 w, uint32 dw);
static void			   CI411Image_ReadableCB(CI411Image * pme);
static boolean		   CI411Image_Realloc(CI411Image * pme);


// Mandatory function for our Applet to register our viewer:
// App Handle Event functions
static boolean i411RegApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);
static void    DisplayOutput(i411RegApp * pMe, int nline, char *pszStr);

// Mandatory App specific data alloc-init/free functions
static boolean InitAppData(IApplet* pMe);
static void FreeAppData(IApplet* pMe);


/*-------------------------------------------------------------------
Global Constant Definitions
-------------------------------------------------------------------*/
// Size of buffer used to hold text strings
#define TEXT_BUFFER_SIZE (100 * sizeof(AECHAR))

/*-------------------------------------------------------------------
Global Data Definitions
-------------------------------------------------------------------*/


/*-------------------------------------------------------------------
Static variable Definitions
-------------------------------------------------------------------*/
// no static variables

/*===============================================================================
FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

Function:  CI411Image_AddRef()

Description:
		This function increases the reference count for the CI411Image object

Prototype:
	uint32  CI411Image_AddRef(IImage * po)

Parameters:
   po: Pointer to IImage interface whos reference count is to be incremented

Return Value:
	The updated reference count

Comments:  None

Side Effects: None

==============================================================================*/
static uint32  CI411Image_AddRef(IImage * po)
{
	CI411Image *pme = (CI411Image *)po;
   return ++((pme)->m_nRefs);
}
/*===========================================================================

Function:  CI411Image_Release()

Description:
		This function decreases the referecne count for the CI411Image object. If
		the reference count reaches zero, it does the required cleanup

Prototype:
	uint32	CI411Image_Release(IApplet * po)

Parameters:
   po: Pointer to the IImage interface whose reference count needs to be
	decremented.

Return Value:
   The updated reference count

Comments:  None

Side Effects: None

==============================================================================*/
static uint32  CI411Image_Release(IImage * po)
{
	IShell * pIShell = NULL;

	CI411Image * pme = (CI411Image *)po;

	if (--pme->m_nRefs)
		return(pme->m_nRefs);

	//Release the interfaces created in AEEClsCreateInstance()
	if (pme->m_pDisplay)
	  IDISPLAY_Release(pme->m_pDisplay);   


	pIShell = pme->m_pShell;

	// Release our device specific Bitmap
	if( pme->m_pBmp )
		SYSFREE(pme->m_pBmp);

	// Release the data that was passed to us in SetData
	if( pme->m_pData )
		FREE(pme->m_pData);

	// Free our VTable functions
	FREE(pme->m_pvt);

	// delete this CI411Image object
	FREE(pme);   

	ISHELL_Release(pIShell);   // Release the Shell

	return 0;
}
/*===========================================================================

FUNCTION COEMImage_GetInfo

DESCRIPTION
   Send a copy of the image info
  
DEPENDENCIES
  none

RETURN VALUE
  none

SIDE EFFECTS
  none
===========================================================================*/
static void CI411Image_GetInfo(IImage * po, AEEImageInfo * pi)
{
   CI411Image * pme = (CI411Image *)po;

   MEMCPY(pi, &pme->m_info, sizeof(AEEImageInfo));
}

/*===========================================================================

FUNCTION COEMImage_SetData

DESCRIPTION
   Sets our data for the bitmap. This is the data we need to transform to
   a displayable bitmap.
   Steps:
   - Change our data to a 256-color bitmap
   - Convert to a vendor specific bitmap

DEPENDENCIES
  none

RETURN VALUE
  TRUE/FALSE

SIDE EFFECTS
  We have to free pData hen we are done with it
===========================================================================*/
static boolean CI411Image_SetData(IImage * po, void * pData)
{
   CI411Image *    pme = (CI411Image *)po;
   void  *         pBmp = NULL;  // Convert to 256-color bitmap
   void  *         pNBmp = NULL; // Convert to device bitmap
   void *          pImageData = NULL; // pointer to actual image data
   boolean         bRealloc = FALSE; // Check to see if mem was reallocated


	// Free the old data...
	if(pme->m_pData)
	{
		FREE(pme->m_pData);
		pme->m_pData = NULL;
	}
	if( pme->m_pBmp )
	{
		SYSFREE(pme->m_pBmp);
		pme->m_pBmp = NULL;
	}

	pImageData = pData;

   // oh-oh
   if( !pImageData )
	   return(FALSE);

   // Set us up the WINBMP
   if((pBmp = CI411Image_SetupBMPImage(pImageData, &pme->m_info)) != NULL)
   {
	   // Set us up the device bitmap
		pNBmp = CONVERTBMP( pBmp, &(pme->m_info), &bRealloc );


		if( !pNBmp )
		{
			pme->m_pData   = pBmp;  // To Free in Release
			return(FALSE);
		}

		if( bRealloc )
		{
			FREE(pBmp);
			pBmp = pNBmp;
		}

		// Set some data up
		pme->m_pBmp    = pNBmp;  // To Free
		pme->m_pData   = pData;  // To Free
		pme->m_rc.dx   = pme->m_info.cx;
		pme->m_rc.dy   = pme->m_info.cy;
		return(TRUE);
   }
   return(FALSE);
}

/*===========================================================================

===========================================================================*/
static void * CI411Image_SetupBMPImage(void * pImageData, AEEImageInfo * m_info)
{
	uint8 *pBmpData = NULL;
	byte *pPalette = NULL;
	char *pImageDt = (char *) pImageData;
	int i = 0, j = 0, k = 0;
   uint8 BMP256Header[] = {(0x42) , (0x4D) , (0x36) , (0x10) , (0x0) , 
					   (0x0) , (0x0) , (0x0) , (0x0) , (0x0) , (0x36) , (0x04) , 
					   (0x0) , (0x0) , (0x28) , (0x0) , (0x0) , (0x0) , (0x40) , 
					   (0x0) , (0x0) , (0x0) , (0x30) ,(0x0) , (0x0) , (0x0) , (0x1)
					   , (0x0) , (0x08) , (0x0) , (0x0) , (0x0) , (0x0) , (0x0) , 
					   (0x0) , (0x0C) , (0x0) , (0x0) , (0xC4) , (0x0E) , (0x0) ,
					   (0x0) , (0xC4) , (0x0E) , (0x0) , (0x0) , (0x0) , 
					   (0x0) , (0x0) , (0x0) , (0x0) , (0x0) , (0x0) , (0x0)};
	byte y[4]; // pixels 1-4
	int8 cb, cr; // changeblue, changered
	int r, g, b; // red green blue
	int it = 0, ita = 0; // Keep track of where we are in pImageData
	int mod, pal, idx;
	// Validate Param
	if( !pImageData || !m_info )
		return NULL;

   // Hardcoded, because of the nature of a 411 file
   m_info->cx = 64;
   m_info->cy = 48;
   m_info->nColors = 8;
   m_info->bAnimated = FALSE;
   m_info->cxFrame = 64;

   // Allocate a palette 
	pPalette = MALLOC(1024);
	if( !pPalette )
		return NULL;

	// Allocate space for WinBmp
	pBmpData = MALLOC(sizeof(BMP256Header) + 1024 + (48) * 64);
	if( !pBmpData )
	{
		FREE(pPalette);
		return NULL;
	}

	// Set Header, again hardcoded above cuz nature of the image format for 411
	for( i = 0; i < sizeof(BMP256Header); i++)
		MEMSET(&(pBmpData[i]), BMP256Header[i], sizeof(byte));

	// yeah zeroes
	it = i = j = k = 0;
	// Make our palette
	do
	 {
		 pPalette[it] = (byte)k;//blue
		 pPalette[it+1] = (byte)j;//green
		 pPalette[it+2] = (byte)i;//red
		 pPalette[it+3] = 0;//nullbyte
		 it +=4;
		 k += 0x33;

		 if( k > 0xFF )
		 {
			 k = 0x00;
			 j += 0x33;
			 if( j > 0xFF )
			 {
				 j = 0x00;
				 i += 0x33;
				 if( i > 0xFF )
					 break;
			 }

		 }
	 }while(TRUE); // we leave after last palette entry (0xFF,0xFF,0xFF) #216

	 // fill the rest with black, if you want better looking images
	 // then fill the rest with colors you might wanna map
	for(; it < 1024; it+=4)
	{
		pPalette[it] = 0xff;
		pPalette[it+1] = 0xff;
		pPalette[it+2] = 0xff;
		pPalette[it+3] = 0;
	}
	// Add the palette to our bitmap header
	for( i = 0; i < 1024; i++)
		MEMSET(&(pBmpData[sizeof(BMP256Header) + i]), pPalette[i], sizeof(byte));

	// free
	FREE(pPalette);
	pPalette = NULL;

	// Do Magic
	// Get our data from source and translate and place into bitmap
	it = 0;
	for( k = 0; k < 48; k++)
	{
		ita = sizeof(BMP256Header) + (48-k-1) * 64;

		for( j = 0; j < 16; j++)
		{
			
			y[0] = pImageDt[it];
			y[1] = pImageDt[it + 1];
			y[2] = pImageDt[it + 2];
			y[3] = pImageDt[it + 3];
			cb = pImageDt[it + 4];
			cr = pImageDt[it + 5];

			// supposed to wrap
			cb -= (int8)128;
			cr -= (int8)128;

			it += 6; // Read in 6, so bump

			// map colors downward below
			for( i = 0; i < 4; i++)
			{
				r = ((14020 * cr) / 10000);
				r = y[i] + r;
				g = ((y[i] * 100000) - 34414 * cb - 71414 * cr) / 100000;
				b = ((y[i] * 1000) + 1772 * cb) / 1000;
				if (r < 0) r = 0;
				if (g < 0) g = 0;
				if (b < 0) b = 0;

				if (r > 255) r = 255;
				if (g > 255) g = 255;
				if (b > 255) b = 255;


				idx = b / 51;
				mod = b % 51;
				if( mod > 25 && idx != 6)
					idx++;
				pal = idx;

				idx = g / 51;
				mod = g % 51;
				if( mod > 25 && idx != 6)
					idx++;
				pal += idx * 6;

				idx = r / 51;
				mod = r % 51;
				if( mod > 25 && idx != 6 )
					idx++;
				pal += idx * 36;

				
				if( ita < (sizeof(BMP256Header) + (48) * 64) )
				{
					pBmpData[1024 + ita++] = (uint8)pal;
				}
			}
		}
	}

	return (void *) pBmpData;

}
/*===========================================================================

FUNCTION CI411Image_SetData

DESCRIPTION
  Sets data about our image, shouldn't be used in this format of an image

DEPENDENCIES
  none

RETURN VALUE
  none

SIDE EFFECTS
  none
===========================================================================*/
static void CI411Image_SetParm(IImage * po, int nParm, int n1, int n2)
{
	CI411Image * pme = (CI411Image *)po;

	switch(nParm)
	{
	  case IPARM_SIZE:
		 pme->m_rc.dx = (int16)n1;
		 pme->m_rc.dy = (int16)n2;
		 break;

	  case IPARM_OFFSET:
		 pme->m_xOffset = (int16)n1;
		 pme->m_yOffset = (int16)n2;
		 break;

	  case IPARM_NFRAMES:
		 if(n1 < 0)
		 {
			if(!pme->m_info.cy)
			   return;
			n1 = pme->m_info.cx / pme->m_info.cy;
		 }

		 pme->m_nFrames = (short)n1;
		 pme->m_nFrame = 0;
		 break;

	  case IPARM_CXFRAME:
		 if(n1 <= 0)
		 {
			n1 = pme->m_info.cy;
			if(!n1)
			   return;
		 }
		 pme->m_nFrames = (int16)(pme->m_info.cx / n1);
		 pme->m_nFrame = 0;
		 break;

	  case IPARM_ROP:
		 pme->m_rop = (AEERasterOp)n1;
		 break;

	  case IPARM_RATE:
		 if(n1 == 0)
			CI411Image_Stop(po);
		 else
		 {
			if(n1 < 0)
			   n1 = 1501;
		 }
		 pme->m_nRate = (short)n1;
		 break;

	}

	if(pme->m_nFrames)
	{
	  pme->m_info.cxFrame = (int8)(pme->m_info.cx / pme->m_nFrames);
	  pme->m_info.bAnimated = TRUE;
	}
}
/*===========================================================================

FUNCTION CI411Image_Draw

DESCRIPTION
   Draw our image, used often

DEPENDENCIES
  none

RETURN VALUE
  none

SIDE EFFECTS
  none
===========================================================================*/
static void CI411Image_Draw(IImage * po, int x, int y)
{
   CI411Image * pme = (CI411Image *)po;

   IDISPLAY_BitBlt(pme->m_pDisplay, x, y, pme->m_rc.dx, pme->m_rc.dy , pme->m_pBmp, pme->m_xOffset, pme->m_yOffset, pme->m_rop);
}
/*===========================================================================

FUNCTION CI411Image_DrawFrame

DESCRIPTION
  Just draw the image, usually you would draw the frame if supporting frames

DEPENDENCIES
  none

RETURN VALUE
  none

SIDE EFFECTS
  none
===========================================================================*/
static void CI411Image_DrawFrame(IImage * po,int nFrame, int x, int y)
{
   CI411Image * pme = (CI411Image *)po;

   IDISPLAY_BitBlt(pme->m_pDisplay, x, y, pme->m_rc.dx, pme->m_rc.dy , pme->m_pBmp, pme->m_xOffset, pme->m_yOffset, pme->m_rop);
}

/*===========================================================================

FUNCTION CI411Image_Start

DESCRIPTION
  Call Animate helper function to Start our animation

DEPENDENCIES
  none

RETURN VALUE
  none

SIDE EFFECTS
  none
===========================================================================*/
static void CI411Image_Start(IImage * po, int x, int y)
{
   CI411Image * pme = (CI411Image *)po;

   pme->m_rc.x = (short)x;
   pme->m_rc.y = (short)y;
   pme->m_nFrame = 0;
   CI411Image_Animate(pme);
}

/*===========================================================================

FUNCTION CI411Image_Stop

DESCRIPTION
  Cancel timer associated with Start()

DEPENDENCIES
  none

RETURN VALUE
  none

SIDE EFFECTS
  none
===========================================================================*/
static void CI411Image_Stop(IImage * po)
{
   CI411Image * pme = (CI411Image *)po;

   ISHELL_CancelTimer((IShell *)pme->m_pShell, (PFNNOTIFY)(CI411Image_Animate), pme);
}

/*===========================================================================

FUNCTION CI411Image_Animate

DESCRIPTION
  This basically will just redraw same image over and over
  If your format supports animation, then you will want to do different here
DEPENDENCIES
  none

RETURN VALUE
  none

SIDE EFFECTS
  none
===========================================================================*/
static void CI411Image_Animate(CI411Image * po)
{
   CI411Image_DrawFrame((IImage *)po, po->m_nFrame, po->m_rc.x, po->m_rc.y);
   IDISPLAY_Update(po->m_pDisplay);
   
   if(++po->m_nFrame >= po->m_nFrames)
      po->m_nFrame = 0;
   ISHELL_SetTimer(po->m_pShell, (uint32)po->m_nRate, (PFNNOTIFY)(CI411Image_Animate),po);
}
static void CI411Image_SetStream(IImage * po, IAStream * ps)
{
	CI411Image * pme = (CI411Image *)po;
	int32       nSize;

	// Free the old data...
	if(pme->m_pData)
	{
		FREE(pme->m_pData);
		pme->m_pData = NULL;
	}
	if( pme->m_pBmp )
	{
		SYSFREE(pme->m_pBmp);
		pme->m_pBmp = NULL;
	}
	if(pme->m_pStream)
	{
		IASTREAM_Release(pme->m_pStream);
		pme->m_pStream = NULL;
	}

	nSize = IASTREAM_Read(ps, NULL, 0);
	if(nSize > 0)
	{
		nSize++;
		pme->m_pData = MALLOC( nSize );   
		if( !pme->m_pData )
			return;
		pme->m_nBytes = nSize;
	}

	pme->m_pStream = ps;
	IASTREAM_AddRef(ps);
	CI411Image_ReadableCB(pme);
}

/*===========================================================================

===========================================================================*/
static void CI411Image_ReadableCB(CI411Image * pme)
{
	int32    nRead,nAvail;
	byte *   pRead;

	while( TRUE )
	{
		nAvail = pme->m_nBytes - pme->m_nRead;
		if( !nAvail )
		{
			if( !CI411Image_Realloc(pme) )
				break;
			nAvail = 500;
		}   
		pRead = (byte *)pme->m_pData + pme->m_nRead;
		nRead = IASTREAM_Read(pme->m_pStream,pRead,nAvail);

		// In Progress?

		if( nRead == AEE_STREAM_WOULDBLOCK )
			break;

		// Finished?

		if( !nRead )
		{
			pRead = (byte *)pme->m_pData;
			pme->m_pData = NULL;
			CI411Image_SetData((IImage *)pme, (void *)pRead);
			return;
		}
		pme->m_nRead += nRead;
	}
	IASTREAM_Readable(pme->m_pStream, (PFNNOTIFY)CI411Image_ReadableCB, pme);
}

/*===========================================================================

===========================================================================*/
static boolean CI411Image_Realloc(CI411Image * pme)
{
   pme->m_nBytes += 500;
   pme->m_pData = REALLOC(pme->m_pData, pme->m_nBytes);   
   return (pme->m_pData != NULL);
}

/*===========================================================================

===========================================================================*/
static void CI411Image_Notify(IImage * po, PFNIMAGEINFO pfn, void * pUser)
{
   CI411Image * pme = (CI411Image *)po;

   pme->m_pfnNotify = pfn;
   pme->m_pUser = pUser;
}

/*===========================================================================

===========================================================================*/
static boolean CI411Image_HandleEvent(IImage * po, AEEEvent e, uint16 w, uint32 dw)
{
	return(FALSE);
}

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time.

  Also, this is where we create our viewer class and hand it back to the caller. No AEEApplet_New
  needs to be called in that case

PROTOTYPE:
	int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell interface. 

	pIModule: pin]: Contains pointer to the IModule interface to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppApplet must point to a valid IBase object. 
	If the classID	represnts an applet, then ppObj must point to a valid AEEApplet structure.Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the class/app creation was successful. 
  EFAILED: Error occurred while creating the class/app. In this case, the app/class will
  not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID clsID, IShell* pIShell, IModule* pIModule, void** ppApplet)
{
	AEEApplet* pMe = 0;
	CI411Image *pImage;
	VTBL(IImage) *appFuncs;

	*ppApplet = NULL;

   if(clsID == AEECLSID_REG_411)
   {
		// We want to load this App. So, invoke AEEApplet_New().To it, pass the
		// address of the app-specific handle event function.
      if ( !AEEApplet_New(sizeof(i411RegApp),clsID, pIShell, pIModule, (IApplet**)ppApplet, (AEEHANDLER) i411RegApp_HandleEvent, (PFNFREEAPPDATA)FreeAppData) )
         return EFAILED;

      pMe = (AEEApplet*)(*ppApplet);

      if ( !pMe )
         return EFAILED;

      // App developer code BEGIN
      //
      // Allocate and initialize app specific data
      if ( !InitAppData((IApplet*)pMe) )
         return EFAILED;
      //
      // App developer code END

      return SUCCESS;
   }
   else if(clsID == AEECLSID_VIEW_411)
   {
		//Validate parameters
		if (!pIShell || !pIModule)
		 return EFAILED;

		*ppApplet = NULL;
		// Create room for our Image object
		if (NULL == (pImage = (CI411Image *)MALLOC(sizeof(CI411Image))))
		  return EFAILED;

		// Allocate and Initialize the VTBL
		appFuncs = (IImageVtbl*) MALLOC(sizeof(IImageVtbl));
		if(appFuncs == NULL)
		{
			FREE(pImage);
			return EFAILED;
		}         

		// Initialize the individual entries in the VTBL
		appFuncs->AddRef		= CI411Image_AddRef;
		appFuncs->Release		= CI411Image_Release;
		appFuncs->Draw			= CI411Image_Draw;
		appFuncs->DrawFrame		= CI411Image_DrawFrame;
		appFuncs->GetInfo		= CI411Image_GetInfo;
		appFuncs->SetParm		= CI411Image_SetParm;
		appFuncs->Start			= CI411Image_Start;
		appFuncs->Stop			= CI411Image_Stop;
		appFuncs->SetStream		= CI411Image_SetStream;
		appFuncs->Notify		= CI411Image_Notify;
		appFuncs->HandleEvent	= CI411Image_HandleEvent;

		INIT_VTBL(pImage, IImage, *appFuncs);   // Initialize the VTBL

		// Initialize the 'class' data members
		pImage->m_nRefs			= 1;
		pImage->m_pShell		= pIShell;
		pImage->m_pDisplay		= NULL;
		pImage->m_cls			= clsID;
		pImage->m_pBmp			= NULL;
		pImage->m_pData			= NULL;
		pImage->m_rop           = AEE_RO_COPY;
		pImage->m_nFrames       = 1;
		pImage->m_nRate         = 1501;
		pImage->m_pUser			= NULL;
		pImage->m_pfnNotify		= NULL;

		ISHELL_AddRef(pIShell);

		IMODULE_AddRef(pIModule);

		// Create an instance of our display
		ISHELL_CreateInstance(pIShell, AEECLSID_DISPLAY, (void **) &pImage->m_pDisplay);
		if (!pImage->m_pDisplay)
		{
			//Cleanup
			FREE(appFuncs);
			FREE(pImage);
			return EFAILED;
		}
		
		// Keep our VTable around
		pImage->m_pvt = appFuncs;

		// Give caller back our instance
		*ppApplet = (IApplet*)pImage;

      return SUCCESS;
   }

   return EFAILED;
}

/*===========================================================================

===========================================================================*/
static boolean InitAppData(IApplet* pMe)
{
	i411RegApp * pme = (i411RegApp * ) pMe;

	// Make menu to register class
	if ( ISHELL_CreateInstance(pme->a.m_pIShell, AEECLSID_MENUCTL, (void **)&pme->imenu) )
		return(FALSE);

	return(TRUE);

}

/*===========================================================================

===========================================================================*/
static void FreeAppData(IApplet* pMe)
{
	i411RegApp * pme = (i411RegApp * ) pMe;

	// release it
	if( pme->imenu )
		IMENUCTL_Release(pme->imenu);

	return;

}

/*===========================================================================

===========================================================================*/
static boolean i411RegApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{
	i411RegApp *   pMe = (i411RegApp *) pi;
	AECHAR         szText[32];
	char           szTxt[32];
	IImage *       pImage = NULL;
	int            err;
   AEERect        rct;
   int            charHeight  = 0;

   if( pMe->imenu && IMENUCTL_HandleEvent(pMe->imenu, eCode, wParam, dwParam))
      return(TRUE);

	switch (eCode) {

		case EVT_APP_START:
         {
      	AEEDeviceInfo	dm;

      	ISHELL_GetDeviceInfo(pMe->a.m_pIShell, &dm);
         pMe->nColorDepth = dm.nColorDepth;

         charHeight = IDISPLAY_GetFontMetrics (pMe->a.m_pIDisplay, AEE_FONT_NORMAL,NULL, NULL);
			// Build menu
         SETAEERECT(&rct, 0, charHeight, dm.cxScreen, dm.cyScreen -charHeight);
         IMENUCTL_SetRect(pMe->imenu, &rct);
			ISHELL_LoadResString(pMe->a.m_pIShell, I411VIEWER_RES_FILE, IDS_REGISTER, szText, sizeof(szText));
			IMENUCTL_AddItem((pMe->imenu), I411VIEWER_RES_FILE, IDS_REGISTER, IDS_REGISTER, szText, 0);

			ISHELL_LoadResString(pMe->a.m_pIShell, I411VIEWER_RES_FILE, IDS_UNREGISTER, szText, sizeof(szText));
			IMENUCTL_AddItem((pMe->imenu), I411VIEWER_RES_FILE, IDS_UNREGISTER, IDS_UNREGISTER, szText, 0);

			ISHELL_LoadResString(pMe->a.m_pIShell, I411VIEWER_RES_FILE, IDS_SHOW_IMAGE, szText, sizeof(szText));
			IMENUCTL_AddItem((pMe->imenu), I411VIEWER_RES_FILE, IDS_SHOW_IMAGE, IDS_SHOW_IMAGE, szText, 0);
			IMENUCTL_SetActive(pMe->imenu, TRUE);
			return(TRUE);
         }

		case EVT_COMMAND:
			switch( wParam ){
   			case IDS_REGISTER:
	   			// Get MIME type we handle, Unregister existing handler for it
		   		// Register ours
			   	err = ISHELL_LoadResString(pMe->a.m_pIShell, I411VIEWER_RES_FILE, IDS_MIME_HANDLER, szText, sizeof(szText));
				   WSTR_TO_STR(szText, szTxt, sizeof(szText));
				   ISHELL_RegisterHandler(pMe->a.m_pIShell, HTYPE_VIEWER, szTxt, 0);
				   err = ISHELL_RegisterHandler(pMe->a.m_pIShell, HTYPE_VIEWER, szTxt, AEECLSID_VIEW_411);
               if( !err )
                  DisplayOutput(pMe, 0, "Registered!");
               else
                  DisplayOutput(pMe, 0, "Register Failed!");

			   	return(TRUE);

   			case IDS_UNREGISTER:
		   		// Get MIME type we handle
	   			// Unregister it
				   err = ISHELL_LoadResString(pMe->a.m_pIShell, I411VIEWER_RES_FILE, IDS_MIME_HANDLER, szText, sizeof(szText));
				   WSTR_TO_STR(szText, szTxt, sizeof(szText));
				   err = ISHELL_RegisterHandler(pMe->a.m_pIShell, HTYPE_VIEWER, szTxt, 0);
               if( !err )
                  DisplayOutput(pMe, 0, "UnRegistered!");
               else
                  DisplayOutput(pMe, 0, "Unregister Failed!");
			   	return(TRUE);

   			case IDS_SHOW_IMAGE:

               if(pMe->nColorDepth < 8 ){
                  DisplayOutput(pMe, 0, "Need 8bit color!");
                  return(TRUE);
               }

				// Use the generic IIMAGE to test our class
   				pImage = ISHELL_LoadImage(pMe->a.m_pIShell, "b.411");
	   			if( pImage ){
                  AEEImageInfo   ii;

               // clean
                  IMENUCTL_SetActive(pMe->imenu, FALSE);
                  IDISPLAY_ClearScreen(pMe->a.m_pIDisplay);
               //measure
                  IIMAGE_GetInfo(pImage, &ii);
                  charHeight = IDISPLAY_GetFontMetrics (pMe->a.m_pIDisplay, AEE_FONT_NORMAL,NULL, NULL);
               // Format
                  SPRINTF(szTxt, "h: %d, w: %d", ii.cy, ii.cx);
               // Show
                  DisplayOutput(pMe, 0, "Press 1 for Menu");
                  DisplayOutput(pMe, 1, szTxt);
			   		IIMAGE_Draw(pImage, 0, charHeight * 2);
					   IDISPLAY_Update(pMe->a.m_pIDisplay);
               // Kill
					   IIMAGE_Release(pImage);
				   }
               else
                  DisplayOutput(pMe, 0, "Unable to Load Image");
   				return(TRUE);

		   	default:
	   			return(TRUE);
			}
         break;

		case EVT_KEY:
         if(wParam == AVK_1){
             IDISPLAY_ClearScreen(pMe->a.m_pIDisplay);
             IMENUCTL_SetActive(pMe->imenu, TRUE);
             return(TRUE);
			}
			return(FALSE);
			

		default:
			return(FALSE);
	}
	return(FALSE);
}

/*===========================================================================

===========================================================================*/
static void DisplayOutput(i411RegApp * pMe, int nline, char *pszStr)
{
   AEEDeviceInfo di; // Device Info
   AECHAR * szBuf;      
   AECHAR * psz = NULL;
   int charHeight = 0;      // Stores the char height in pixels for given font
   int pnAscent = 0;        // Stores the ascent in number of pixels
   int pnDescent = 0;       // Stores the descent in number of pixels
   int pixelWidth;
   AEEFont font = AEE_FONT_NORMAL;
   int pnFits = 0, dy;
   int totalCh = 0;
   IShell *pIShell = pMe->a.m_pIShell;
   IDisplay *pIDisplay = pMe->a.m_pIDisplay;
   AEERect  rct;

   // Make sure the pointers we'll be using are valid
   if (pIShell == NULL || pIDisplay == NULL)
	   return;

   if ((szBuf = (AECHAR *) MALLOC(TEXT_BUFFER_SIZE * sizeof(AECHAR))) == NULL)
	   return;

   // Get device information
   ISHELL_GetDeviceInfo(pIShell,&di);

   // Get the font metrics info
   charHeight = IDISPLAY_GetFontMetrics (pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   // Convert to wide string (unicode)
   STR_TO_WSTR ((char *)pszStr, szBuf, TEXT_BUFFER_SIZE * sizeof(AECHAR));

   // If nlines is zero then print this string starting around the middle of 
   // the screen. Or else multiply nlines by charheight to decide the y coordinate of
   // the start of the string.
   if (nline < 0) {
      dy = di.cyScreen*2/5;
      SETAEERECT(&rct, 0, dy, di.cxScreen, charHeight);
      IDISPLAY_FillRect(pMe->a.m_pIDisplay, &rct, RGB_NONE);
   }
   else{
      dy = nline * charHeight;
      SETAEERECT(&rct, 0, dy, di.cxScreen, charHeight);
      IDISPLAY_FillRect(pMe->a.m_pIDisplay, &rct, RGB_NONE);
   }

   // psz keeps track of the point from which to write from the string buffer
   // in case the string does not fit one line and needs to wrap around in the
   // next line.
   psz = szBuf;
      
   // Need to calculate the lotal string length to decide if any wrapping
   // around is needed.
   totalCh = STRLEN ((char *)pszStr);

   // Keep displaying text string on multiple lines if the string can't be displayed
   // on one single line. Lines are spaced 15 pixels apart.
   while ((totalCh > 0) && (*psz != NULL))
   { 
      // Get information on how many characters will fit in a line.
      // Give the pointer to the buffer to be displayed, and the number of
      // pixels along the x axis you want to display the string in (max number)
      // pnFits will have the max number of chars that will fit in the maxWidth
      // number of pixels (given string can't fit in one line), or the number of 
      // chars in the string (if it does fit in one line). pnWidth gives the
      // number of pixels that will be used to display pnFits number of chars.
      pixelWidth = IDISPLAY_MeasureTextEx(pIDisplay,
                      font, 
                      (AECHAR *) psz,  // Start of the buffer to display,
                      -1,
                      di.cxScreen - 5, // maxWidth
                      &pnFits);         // Number of chars that will fit a line

      // If pnFits is zero there is something wrong in the input to above function. 
      // Normally this scenario should not occur. But, have the check anyway.
      if (pnFits == 0)
	  {
		 FREE(szBuf);
         return;
      }

      IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, psz, pnFits, 5 /*start dx*/, 
         dy, 0 /* use default rectangle coordinates */, 0);

      psz += pnFits;      // move pointer to the next segment to be displayed
      totalCh -= pnFits;  // reduce the total number of characters to still display
      dy += charHeight;   // Place next line charHeight pixels below the 
                          // previous line.
      IDISPLAY_Update(pIDisplay); //, TRUE);
      if (totalCh < pnFits)
         pnFits = totalCh;  // if total number is less than pnFits, adjust pnFits
      // Do not wrap string, remove next line to allow wrapping
      totalCh = 0;
   }
   FREE(szBuf);
   return;   
} // End of DisplayOutput
