/*///////////////////////////////////////////////////////////////////////////////
||
|| FILE: IHtmlViewer.c
||
|| SERVICES: Sample applet using AEE
||
|| DESCRIPTION
||    This file contains the implementation of a simple example using the
||    AEE services.
||
|| PUBLIC CLASSES:
||    N/A
||
|| INITIALIZATION AND SEQUENCING REQUIREMENTS:
||   The following explanation applies to this sample containing one applet which serves
||   as a base for app developers to create their own applets using AEE Services:
||   
||
||   In the applet source file (like this one), include AEEAppGen.h.
||
||   Mandatory Sections in Applet Source (this file):
||   -----------------------------------------------
||   Following Mandatory Sections are required for each applet source file. 
||   (Search for "Mandatory" to identify these sections)
||
||   Includes:     
||      Copy this section as-is from the sample applet. It contains:
||      AEEAppGen.h: For AEEApplet declaration
||
||   Type Declarations:
||    A data structure must be defined to hold the app specific data. In this structure,
||    the first element must be of type AEEApplet.
||
||   Functions: (For more details, see corresponding function description in this applet)
||
||      App_HandleEvent(): is the Event Handler to this applet.
||                         Copy the function outline from the sample applet and add app specific code.
||
||      AEEClsCreateInstance(): creates an instance of the applet and initializes it. 
||                              It is called by AEEModGen when applet is being created.
||
||   
||
||    Important Notes:
||    ---------------
||    1. DO NOT use any "static data" in the applet. Always use the
||       functions exported by AEEStdlib or by IHeap services to
||       dynamically allocate data and make it a member of the applet
||       structure.
||
||    2. DO NOT include and link "standard C library". Use AEE Memory
||       Services (in AEEHeap.h) and Standard Library macros(in
||       AEEStdLib.h).  For example, use MALLOC() to allocate memory,
||       WSTRCPY() to make a copy of Unicode (wide) string.
||
||    3. BREW is Unicode(wide string) compliant ONLY (no ISOLATIN1/ANSI)
||       except for file names which are ISOLATIN1/ANSI.  ALWAYS USE
||       AECHAR instead of "char". Use string services provided in
||       AEEStdLib.h for string manipulation.
||
||    4. It is always a good idea to DEFINE RESOURCES using BREW
||       ResourceEditor. Make Strings, Bitmaps, Dialogs, etc.  as
||       resources. ResourceEditor saves resources as .bri file,
||       generates resource header file and compiles .bri into a .bar
||       binary file, which can be used by the applet.
||
||    Miscellanoeus Notes:
||    -------------------
||    1. Make sure that the class ID used for the app is the same as that
||       defined corresponding in .MIF file
||    2. Always make sure that compiled resource (.bar) file and corresponding
||       resource header (a) reside in app directory and (b) are included in
||       the applet code. Define a constant APP_RES_FILE containing the name
||       of the compiled resource file (with .bar extension).
||
||    More than one applet:
||    --------------------
||    If more than one applet needs to be defined, then do the following
||    (1) Follow the above description for each applet
||
||
||       Copyright  2000-2001 QUALCOMM Incorporated.
||                   All Rights Reserved.
||                    QUALCOMM Proprietary/GTDR
||
///////////////////////////////////////////////////////////////////////////////// */
#include "AEE.h"              // Standard AEE Declarations
#include "AEEShell.h"         // AEE Shell Services
#include "AEEDisp.h"          // AEE Display Services
#include "AEEStdLib.h"        // AEE StdLib Services
#include "AEEAppGen.h"        // AEEApplet structure and Applet services
#include "AEESampleAppIDs.h"  // Sample App ID

// Services used by app
#include "AEEHtmlViewer.h"    // IHtmlViewer interface
#include "AEEFile.h"
#include "AEEMenu.h"


// forward decls
typedef struct HViewerApp HViewerApp;

// forward decls (vtable parts)
//static uint32   HViewerApp_AddRef      (IApplet *pif);
//static uint32   HViewerApp_Release     (IApplet *pif);
static boolean  HViewerApp_HandleEvent (IApplet *pif, AEEEvent eCode, uint16 wParam, uint32 dwParam);

// forward decls (helpers)
static void     HViewerApp_FrameViewer (HViewerApp *pme);
static void     HViewerApp_DrawPict    (HViewerApp *pme);


struct HViewerApp {
   //First element of this structure must be AEEApplet.
   AEEApplet         a ;
   
   IShell *          pIShell;
   IModule *         pIModule;
   IDisplay *        pIDisplay;
   IHtmlViewer *     pIHtmlViewer;

   AEEDeviceInfo     di;
   AEERect           rc;
   byte              byHist[999];
   int               ndxHist;

   int               nCurrentPage;
   int               bViewerShown;
};



#ifndef SETAEERECT
#define SETAEERECT(prc,l,t,w,h)   (prc)->x=(int16)(l),(prc)->y=(int16)(t),(prc)->dx=(int16)(w),(prc)->dy=(int16)(h)
#endif

#define INFLATERECT(prc,nx,ny) \
   (prc)->x-=(nx),(prc)->y-=(ny),(prc)->dx+=((nx)*2),(prc)->dy+=((ny)*2)


#define IDC_OPTION1     1
#define IDC_OPTION2     2
#define IDC_OPTION3     3


#define BORDER 4

const char gszError[] = "<p align=center>Cannot Find File: \"htmltest.htm\"</p>";


const char gszTests[] =
(
   // Intro
   "<h1 align=center>HTML Sample</h1>"
   "<a href=test:1>Help</a><br>"
   "<a href=test:2>Block Elements</a><br>"
   "<a href=test:3>Hidden Elements</a><br>"
   "<a href=test:4>Text Styles</a><br>"
   "<a href=test:5>Forms</a><br>"
   "<a href=test:90>Hide Viewer</a><br>"
   "\0"

   // 1: Help
   "<h1 align=center>Help</h1>"
   "CLR = previous page<br>"
   "1 = toggle scroll bar<br>"
   "2 = Load Source<br>"
   "3 = Load File<br>"
   "< > = change width<br>"
   "\0"

   // 2: Block-level elements
   "<P>P at top</p>"
   "<P ALIGN=RIGHT>Right-aligned paragraph</p>"
   "This text follows &lt;/P&gt;"
   "<blockquote>BLOCKQUOTED text a b c d e f g h i j k l m n o p q r s t u v w x y z</blockquote>"
   "Did you ever read what they call Science Fiction? It's a scream. It is written like this: "
   "&quot;I check out with K 19 on Aldabaran III, and stepped out through the crummalite hatch "
   "on my 22 Model Sirus Hardtop. I cocked the timejector in secondary and waded through the "
   "bright blue manda grass.&quot;"
   "\0"

   // 3: Hidden Elements
   "<title>This is the Title</title>"
   "<!-- This should be omitted -->"
   "<style>H1 {background: yellow}</style>"
   "<script>function f() { return 1; }</script>"
   "<!-- This should be displayed -->"
   "<body>Only this text should be displayed</body>"
   "\0"


   // 4: Styles
   "<h1 align=center>Header 1</h1>"
   "<font color=maroon>Here is <a href='test:0'>a link</a> "
   "and some <big>BIG</big> text.</font>"
   "<p>This is a <b>long paragraph </b>that should wrap.</p>"
   "<strong>Strong</strong> "
   "<b>bold</b> "
   "<font color=red>RED</font> <font color=maroon>MAROON</font> "
   "<font color=purple>PURPLE</font>"
   "<p>Here is <a href='test:1'>a single link that will span more than one line</a>. "
   "<p><a href='test:1'><b>This</b> one</a> has a nested B element."
   "\0"

   
   // 5: Forms
   "<h1>Forms</h1>"
   "<p align=right>"
   "Checkbox: "
   "<input type=checkbox checked><br>"
   "Text:"
   "<input type=text value='A Form Edit Field' size=8><br>"
   "Another:"
   "<input type=checkbox checked></p>"
   "<a href='test:6'>Another form test</a>"
   "<p>More text to scroll...</p>"
   "<p>B More text to scroll...</p>"
   "<p>C More text...</p>"
   "<h1>This is H1</h1>"
   "<p>D More text...</p>"
   "<input type=submit>"
   "<p>E More text...</p>"
   "<a href='test:1'>Help</a>"
   "<p>F More text...</p>"
   "<blockquote>This is a blockquote, and it should wrap.</blockquote>"
   "<p>H More text to scroll...</p>"
   "<p>I More text to scroll...</p>"
   "<p>J More text to scroll...</p>"
   "<p>K More text to scroll...</p>"
   "<input type=radio name=dataset value=1 checked>"
   "<input type=radio name=dataset value=2>"
   "<input type=radio name=dataset value=3>"
   "<p>L More text...</p>"
   "<P>END</p>"
   "\0"
   

   // 6: Simple form test
   "<form action='http://www.yahoo.com'>"
   "Text control"
   "<input type=text name=TextCtl value='Two Words' size=50>"
   "Checkbox"
   "<input type=checkbox name=ChkBox value=Option1 checked >"
   "<input type=checkbox name=ChkBox value=Option2 >"
   "<input type=submit>"
   "</form>"
   "<p>END</p>"
   "\0"
);


#define RELEASEIF(pi)      { if (pi) { IBASE_Release((IBase*)(pi)); (pi)=0; }}

void ReleaseI(void **ppif)
{
  if (*ppif) {
    IBASE_Release((IBase*)*ppif);
    *ppif = 0;
  }
}


void HViewerApp_FrameViewer(HViewerApp *pme)
{
   AEERect rc;

   SETAEERECT(&rc, 0, 0, pme->di.cxScreen, pme->di.cyScreen);
   IDISPLAY_FillRect(pme->pIDisplay, &rc, MAKE_RGB(0xc0,0xc0,0xc0));

   rc = pme->rc;
   
   // one-pixel black frame surrounding...
   IDISPLAY_FillRect(pme->pIDisplay, &rc, RGB_BLACK);
   INFLATERECT(&rc, -1, -1);
            
   // one-pixel white frame surrounding the viewer
   IDISPLAY_FillRect(pme->pIDisplay, &rc, RGB_WHITE);
   INFLATERECT(&rc, -1, -1);
   
   IHTMLVIEWER_SetRect(pme->pIHtmlViewer, &rc);

   pme->bViewerShown = TRUE;
}   


void HViewerApp_DrawPict(HViewerApp *pme)
{
   AEERect rc;

   SETAEERECT(&rc, 0, 0, pme->di.cxScreen, pme->di.cyScreen);
   IDISPLAY_FillRect(pme->pIDisplay, &rc, MAKE_RGB(0xc0,0xc0,0xc0));

   // Hide viewer
   rc.dy = 0;
   IHTMLVIEWER_SetRect(pme->pIHtmlViewer, &rc);


   pme->bViewerShown = FALSE;
}


// Find test page in gszTests[].
//
// If out of range, returns pointer to empty string.
//
const char *GetPage(int nPage)
{
   const char *psz = gszTests;
   
   while (nPage && *psz) {
      psz += STRLEN(psz) + 1;
      --nPage;
   }

   return psz;
}


void HViewerApp_GoTo(HViewerApp *pme, int nPage)
{
   pme->nCurrentPage = nPage;
   pme->byHist[pme->ndxHist++] = nPage;

   if (nPage == 90) {
      // Hide HTML viewer; draw graphic
      if (pme->bViewerShown) {
         HViewerApp_DrawPict(pme);
      }

   } else {
      // Show HTML viewer
      if (!pme->bViewerShown) {
         HViewerApp_FrameViewer(pme);
      }

      IHTMLVIEWER_ParseBuffer(pme->pIHtmlViewer, GetPage(nPage));
   }
}



/*///////////////////////////////////////////////////////////////////////////////
|| Handle events that the Shell sends our way
///////////////////////////////////////////////////////////////////////////////
*/
static boolean HViewerApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
   HViewerApp *pme = (HViewerApp *)pi;

   if (pme->pIHtmlViewer && IHTMLVIEWER_HandleEvent(pme->pIHtmlViewer, eCode, wParam, dwParam))
      return TRUE;

   switch (eCode) {
      
      // ////////////////////////
      // 
      case EVT_APP_START:
         if (pme->pIHtmlViewer) {

            SETAEERECT(&pme->rc, 0, 0, pme->di.cxScreen, pme->di.cyScreen);
            // shrink the viewers rect to allow for border
            INFLATERECT(&pme->rc, -BORDER, -BORDER);

            HViewerApp_FrameViewer(pme);
            HViewerApp_GoTo(pme, 0);
         }
         return TRUE;

      // ////////////////////////
      // 
      case EVT_APP_STOP:
         return TRUE;   

      // ////////////////////////
      // 
      case EVT_KEY:

         if (wParam == AVK_CLR) {
            if (pme->ndxHist < 2)
               return FALSE;
            pme->ndxHist -= 2;
            HViewerApp_GoTo(pme, pme->byHist[pme->ndxHist]);
            return TRUE;
         } 
         else if (wParam == AVK_1) {

            // Toggle scroll bar
            
            uint32 props = IHTMLVIEWER_GetProperties(pme->pIHtmlViewer);

            props ^= HVP_SCROLLBAR;

            IHTMLVIEWER_SetProperties(pme->pIHtmlViewer, props);
            return TRUE;

         } 
         else if (wParam == AVK_2) {

            ISourceUtil * pisu = NULL;
            ISource * pISource = NULL;

            if (SUCCESS == ISHELL_CreateInstance(pme->pIShell, AEECLSID_SOURCEUTIL, (void**)&pisu)) {
               if (SUCCESS == ISOURCEUTIL_SourceFromMemory(pisu, GetPage(4), STRLEN(GetPage(4)),
                                                           NULL, 0, &pISource)) {
                  
                  // This AddRef's pISource
                  IHTMLVIEWER_LoadSource(pme->pIHtmlViewer, pISource);
               }
            }
            RELEASEIF(pisu);
            RELEASEIF(pISource);
            return TRUE;
         } 
         else if (wParam == AVK_3) {

            IFileMgr * pifm;
            if (SUCCESS == ISHELL_CreateInstance(pme->pIShell, AEECLSID_FILEMGR, (void**)&pifm)) {
               IFile * pif = IFILEMGR_OpenFile(pifm, "htmltest.htm", _OFM_READ);
               
               // save current page in history
               pme->byHist[pme->ndxHist++] = pme->nCurrentPage;

               if (pif) {
                  IHTMLVIEWER_LoadStream(pme->pIHtmlViewer, (IAStream*)pif);
                  RELEASEIF(pif);
               }
               else {
                  // cannot find "htmltest.htm"
                  IHTMLVIEWER_ParseBuffer(pme->pIHtmlViewer, gszError);
               }
               RELEASEIF(pifm);
            }
            return TRUE;
         } 

         else if (wParam == AVK_LEFT) {
             pme->rc.dx -= 1;
             if (pme->bViewerShown)
                HViewerApp_FrameViewer(pme);
         } 
         else if (wParam == AVK_RIGHT) {
             pme->rc.dx += 1;
             if (pme->bViewerShown)
                HViewerApp_FrameViewer(pme);
         }

         break;

   }  // switch(eCode)

   return FALSE;
}



static void HViewerApp_NotifyCB(HViewerApp *pme, HViewNotify *phvn)
{
   switch (phvn->code) {
      case HVN_REDRAW_SCREEN:
         IDISPLAY_ClearScreen(pme->pIDisplay);
         if (pme->bViewerShown) {
            HViewerApp_FrameViewer(pme);
            IHTMLVIEWER_Redraw(pme->pIHtmlViewer);
         } else {
            HViewerApp_DrawPict(pme);
         }
         break;
      case HVN_JUMP:
         if (phvn->u.jump.pszURL && STRNICMP(phvn->u.jump.pszURL, "test:", 5) == 0) {
            HViewerApp_GoTo(pme, ATOI(phvn->u.jump.pszURL+5));
         }
         break;
      case HVN_SUBMIT:
         // See phvn->u.submit for information on form submission (URL, method, etc.)
         DBGPRINTF("Submitted!");
         break;
   }

}

/*===========================================================================
  Ctor/Dtor
===========================================================================*/

static boolean HViewerApp_Init(HViewerApp *pme, IModule * pIModule, IShell * pIShell)
{
   pme->pIShell = pIShell;
   ISHELL_AddRef(pIShell);

   pme->pIModule = pIModule;
   IMODULE_AddRef(pIModule);

   if (SUCCESS != ISHELL_CreateInstance(pIShell, AEECLSID_HTML, (void**)&pme->pIHtmlViewer))
      return 0;

   if (SUCCESS != ISHELL_CreateInstance(pIShell, AEECLSID_DISPLAY, (void**)&pme->pIDisplay))
      return 0;

   ISHELL_GetDeviceInfo(pme->pIShell, &pme->di);

   IHTMLVIEWER_SetNotifyFn(pme->pIHtmlViewer, (PFNHVIEWNOTIFY)HViewerApp_NotifyCB, pme);
//   IHTMLVIEWER_SetProperties(pme->pIHtmlViewer, HVP_INLINE_EDIT);
   return 1;
}


static void HViewerApp_Dtor(HViewerApp *pme)
{
   RELEASEIF(pme->pIShell);
   RELEASEIF(pme->pIModule);
   RELEASEIF(pme->pIHtmlViewer);
   RELEASEIF(pme->pIDisplay);
}

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
   This function is invoked while the app is being loaded. All Modules must provide this 
   function. Ensure to retain the same name and parameters for this function.
   In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
   that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time.

PROTOTYPE:
   int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
   clsID: [in]: Specifies the ClassID of the applet which is being loaded

   pIShell: [in]: Contains pointer to the IShell interface. 

   pIModule: pin]: Contains pointer to the IModule interface to the current module to which
   this app belongs

   ppObj: [out]: On return, *ppApplet must point to a valid IBase object. 
   If the classID represnts an applet, then ppObj must point to a valid AEEApplet structure.Allocation
   of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the class/app creation was successful. 
  EFAILED: Error occurred while creating the class/app. In this case, the app/class will
  not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
//Mandatory Function
int AEEClsCreateInstance(AEECLSID clsid, IShell * pIShell, IModule * pIModule, void ** ppObj)
{
   HViewerApp * pme = NULL;
   
   *ppObj = NULL;

   if (clsid != AEECLSID_HTMLVIEWERAPP)
      return ECLASSNOTSUPPORT;

   //Create the applet
   if (NULL == AEEApplet_New(sizeof(HViewerApp), clsid, pIShell, pIModule, (IApplet**)&pme, 
                    (AEEHANDLER)HViewerApp_HandleEvent, (PFNFREEAPPDATA)HViewerApp_Dtor))
      return ENOMEMORY;

   //Initialize applet data
   if (HViewerApp_Init(pme, pIModule, pIShell))
      *ppObj = pme;
   else
      IAPPLET_Release((IApplet*)pme);

   return (*ppObj ? AEE_SUCCESS : EFAILED);
}



