/*===========================================================================

FILE: KeyPad.c

SERVICES:

DESCRIPTION
  This file contains the implementation of a simple applet that allows testing
  the configuration of keypad

PUBLIC CLASSES:  
   N/A

INITIALIZATION AND SEQUENCING REQUIREMENTS:
   The following explanation applies to this sample containing one applet which serves
   as a base for app developers to create their own applets using AEE Services:
   

   In the applet source file (like this one), include AEEAppGen.h.

   Mandatory Sections in Applet Source (this file):
   -----------------------------------------------
   Following Mandatory Sections are required for each applet source file. 
   (Search for "Mandatory" to identify these sections)

   Includes:     
      Copy this section as-is from the sample applet. It contains:
      AEEAppGen.h: For AEEApplet declaration

   Type Declarations:
		A data structure must be defined to hold the app specific data. In this structure,
		the first element must be of type AEEApplet.

   Functions: (For more details, see corresponding function description in this applet)

      App_HandleEvent(): is the Event Handler to this applet.
                         Copy the function outline from the sample applet and add app specific code.

      AEEClsCreateInstance(): creates an instance of the applet and initializes it. 
                              It is called by AEEModGen when applet is being created.

   
   Important Notes:
   ---------------
   1. DO NOT use any "static data" in the applet. Always use the functions exported by
	   AEEStdlib or by IHeap services to dynamically allocate data and make it a member of 
		the applet structure. 
   2. DO NOT include and link "standard C library". Use AEE Memory Services (in AEEHeap.h) and Standard Library macros(in AEEStdLib.h).
      For example, use MALLOC() to allocate memory, WSTRCPY() to make a copy of Unicode (wide) string.
   3. BREW is Unicode(wide string) compliant ONLY (no ISOLATIN1/ANSI) except for file names which are ISOLATIN1/ANSI.
      ALWAYS USE AECHAR instead of "char". Use string services provided in AEEStdLib.h for string manipulation.
   4. It is always a good idea to DEFINE RESOURCES using BREW ResourceEditor. Make Strings, Bitmaps, Dialogs, etc. 
      as resources. ResourceEditor saves resources as .bri file, generates resource header file 
      and compiles .bri into a .bar binary file, which can be used by the applet.

   Miscellanoeus Notes:
   -------------------
   1. Make sure that the class ID used for the app is the same as that defined corresponding in .MIF file
   2. Always make sure that compiled resource (.bar) file and corresponding
      resource header (a) reside in app directory and (b) are included in the applet code.
      Define a constant APP_RES_FILE containing the name of the compiled resource file (with .bar extension).

   More than one applet:
   --------------------
   If more than one applet needs to be defined, then do the following
   (1) Follow the above description for each applet

   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/


/*===============================================================================
                     INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "AEE.h"			// Standard AEE Declarations
#include "AEEShell.h"	// AEE Shell Services
#include "AEEDisp.h"		// AEE Display Services
#include "AEEStdLib.h"  // AEE StdLib Services
#include "AEEAppGen.h"	// AEEApplet structure and Applet services
#include "AEESampleAppIDs.h" // Sample App ID

// Services used by app
#include "TextWin.h"

// App resource header generated by BREW ResourceEditor.
#include "KeyPad_res.h" // Applet resources

/*===========================================================================
                      PUBLIC DATA DECLARATIONS
===========================================================================*/

/*-------------------------------------------------------------------
            Type Declarations
-------------------------------------------------------------------*/
//Mandatory structure.
//Structure to contain app data. This is the Applet structure that will be allocated
// for the app. Define this structure if the app needs to store any data that needs
// to be remembered across different events or callbacks. The first element of this
// structure MUST be of type AEEApplet. This is extremely important. Following this
// element, store the app-specific data. Note that AEEApplet structure already exposes
// the most commonly needed interface pointers such as Shell, Display, etc. 
typedef struct _CKeyPadApp CKeyPadApp;
struct _CKeyPadApp
{
   AEEApplet   a;

	int      m_cxScreen;
	int      m_cyScreen;

   AEEEvent m_eCode;
   uint16   m_wParam;

   TextWin* m_pText;
   boolean  m_bReady;
   uint16   m_wTestID;
};

/*-------------------------------------------------------------------
            Function Prototypes
-------------------------------------------------------------------*/
// Mandatory function:
// App Handle Event function
static boolean KeyPadApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);

// App specific data alloc-init/free functions
static boolean KeyPadApp_InitAppData(IApplet* po);
static void KeyPadApp_FreeAppData(IApplet* po);

// App Helper Functions
static void ProcessKeyEvent(CKeyPadApp * pMe);
static void DrawAppTitle(CKeyPadApp * pMe);
static void AppendTitle(CKeyPadApp * pMe, uint16 wResID, AEERect rect);
static void AppendStringRes(CKeyPadApp * pMe, uint16 wResID, boolean bIndent);
static void AppendString(CKeyPadApp * pMe, AECHAR * szBuf, boolean bIndent);

/*-------------------------------------------------------------------
            Global Constant Definitions
-------------------------------------------------------------------*/
// App Resource File
#define  APP_RES_FILE      "keypad.bar"

// App specific constants
#define  KEY_BASE          (uint16)AVK_FIRST
#define  MAX_STRLEN        100
#define  MAX_INDENT        1
#define  LINEHEIGHT        13


/*-------------------------------------------------------------------
            Global Data Definitions
-------------------------------------------------------------------*/


/*-------------------------------------------------------------------
            Static variable Definitions
-------------------------------------------------------------------*/



/*===============================================================================
                     FUNCTION DEFINITIONS
=============================================================================== */
/*===========================================================================

Function:  KeyPadApp_InitAppData()

Description:
		This function initializes the app specific data.

Prototype:
	void KeyPadApp_InitAppData(IApplet* po)

Parameters:
   IApplet* po: Pointer to the applet structure. This is of the type defined
	to store applet data. (CMIDIApp*, in this case).

Return Value:
	TRUE: If successful
	FALSE: IF failed

Comments:  None

Side Effects: None

==============================================================================*/
boolean KeyPadApp_InitAppData(IApplet* po)
{
	CKeyPadApp *pMe = (CKeyPadApp*)po;

   AEEDeviceInfo  di;
   AEERect rect;

   // Get screen pixel count
   ISHELL_GetDeviceInfo(pMe->a.m_pIShell, &di);
   pMe->m_cxScreen = di.cxScreen;
	pMe->m_cyScreen = di.cyScreen;

   // Create the text window
   rect.x = 0;
   rect.dx = pMe->m_cxScreen;
   rect.y = 3*LINEHEIGHT;
   rect.dy = pMe->m_cyScreen - rect.y;
   pMe->m_pText = TextWin_Create((AEEApplet*)pMe, rect, AEE_FONT_NORMAL);

   if ( !pMe->m_pText )
   {
      return FALSE;
   }

   pMe->m_bReady = FALSE;
   pMe->m_wTestID = 0;
 
   return TRUE;
}

/*===========================================================================

Function:  KeyPadApp_FreeAppData()

Description:
	This function frees the app data. This function is registered with the
	applet framework when the applet is created (inside AEEClsCreateInstance() function).
	This function is called by the app framework when the reference count of the 
	applet reaches zero. This function must free all the app data that has been
	allocated by this app. For ex: if their is data that was remembered when
	the app is suspended and resumed, those data must be freed here.

Prototype:
	void KeyPadApp_FreeAppData(IApplet* po)

Parameters:
   IApplet* po: Pointer to the applet structure. This is of the type defined
	to store applet data. (CMIDIApp*, in this case).

Return Value:
	None

Comments:  None

Side Effects: None

==============================================================================*/
void KeyPadApp_FreeAppData(IApplet* po)
{
	CKeyPadApp *pMe = (CKeyPadApp*)po;

   if ( pMe->m_pText )
      TextWin_Release(pMe->m_pText);

}

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time.

PROTOTYPE:
	int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell interface. 

	pIModule: pin]: Contains pointer to the IModule interface to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppApplet must point to a valid IBase object. 
	If the classID	represnts an applet, then ppObj must point to a valid AEEApplet structure.Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the class/app creation was successful. 
  EFAILED: Error occurred while creating the class/app. In this case, the app/class will
  not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
//Mandatory Function
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
{
	*ppObj = NULL;

   if(ClsId == AEECLSID_KEYPADAPP)
	{
		//Create the applet
      if(AEEApplet_New(sizeof(CKeyPadApp), ClsId, pIShell,po,(IApplet**)ppObj,(AEEHANDLER)KeyPadApp_HandleEvent,(PFNFREEAPPDATA)KeyPadApp_FreeAppData))
		{
			//Initialize applet data
         if(KeyPadApp_InitAppData((IApplet*)*ppObj))
			{
				//Data initialized successfully
				return(AEE_SUCCESS);
			}
			else
			{
				//Release the applet. This will free the memory allocated for the applet when
				// AEEApplet_New was called.
				IAPPLET_Release((IApplet*)*ppObj);
				return EFAILED;
			}

      }//AEEApplet_New

   }// ClsId == AEECLSID_KEYPADAPP

   return(EFAILED);
}

/*===========================================================================

FUNCTION KeyPadApp_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean KeyPadApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AppClsCreateInstance() function.

	ecode: Specifies the Event sent to this applet

   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/
//Mandatory Function
static boolean KeyPadApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{
   CKeyPadApp * pMe = (CKeyPadApp *)pi;
	 
   switch ( eCode ) 
   {   
      case EVT_APP_START:  // Process Start event
         DrawAppTitle(pMe);
         return TRUE;
         break;

      case EVT_APP_STOP:   // process STOP key press
         return TRUE;
         break;

      case EVT_KEY:	      // process Key Action event
         pMe->m_eCode = eCode;
         pMe->m_wParam = wParam;
         switch ( (AVKType)wParam )
         {
            case AVK_SELECT:
               if ( !pMe->m_bReady )
               {
                  pMe->m_bReady = TRUE;
                  DrawAppTitle(pMe);
               }
               else
               {
                  ProcessKeyEvent(pMe);
               }
               return TRUE;

            case AVK_UP:
               if ( !pMe->m_bReady )
               {
                  TextWin_Scroll(pMe->m_pText, -1);
               }
               else
               {
                  ProcessKeyEvent(pMe);
               }
               return TRUE;

            case AVK_DOWN:
               if ( !pMe->m_bReady )
               {
                  TextWin_Scroll(pMe->m_pText, 1);
               }
               else
               {
                  ProcessKeyEvent(pMe);
               }
               return TRUE;

            default:
               if ( pMe->m_bReady )
               {
                  ProcessKeyEvent(pMe);
               }
               return TRUE;
         } // switch wParam

      default:
         return TRUE;

    }

    return FALSE;
}

/*===============================================================================
                     APP HELPER FUNCTION DEFINITIONS
=============================================================================== */
static void DrawAppTitle(CKeyPadApp * pMe)
{
   uint16 cy = 3*LINEHEIGHT;
   AEERect rect;

   SETAEERECT(&rect,  0, 0, pMe->m_cxScreen, cy );

   IDISPLAY_EraseRect(pMe->a.m_pIDisplay, &rect);

   rect.dy = LINEHEIGHT;
   AppendTitle(pMe, IDS_APP_TITLE, rect);

   rect.y = LINEHEIGHT;
   if ( !pMe->m_bReady )
   {
      AppendTitle(pMe, IDS_NOT_READY1, rect);
      rect.y = 2*LINEHEIGHT;
      AppendTitle(pMe, IDS_NOT_READY2, rect);
   }
   else
   {
      AppendTitle(pMe, IDS_READY, rect);
   }
   IDISPLAY_Update(pMe->a.m_pIDisplay);
}

static void AppendTitle(CKeyPadApp * pMe, uint16 wResID, AEERect rect)
{
   AECHAR szBuf[MAX_STRLEN];

   ISHELL_LoadResString(pMe->a.m_pIShell, APP_RES_FILE, wResID, (AECHAR*)szBuf, MAX_STRLEN);
   IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_BOLD,
                     szBuf, -1, 0, rect.y, &rect, 0 );
}

static void AppendStringRes(CKeyPadApp * pMe, uint16 wResID, boolean bIndent)
{
   AECHAR szBuf[MAX_STRLEN];
   AECHAR szIndent[MAX_STRLEN];
   AECHAR* szString = (AECHAR*) szBuf;

   ISHELL_LoadResString(pMe->a.m_pIShell, APP_RES_FILE, wResID, szBuf, MAX_STRLEN);

   if ( bIndent )
   {
      ISHELL_LoadResString(pMe->a.m_pIShell, APP_RES_FILE, IDS_BLANK, (AECHAR*)szIndent, MAX_STRLEN);
      WSTRCAT(szIndent, szBuf);
      szString = (AECHAR*) szIndent;
   }

   TextWin_Append(pMe->m_pText, szString);
}

static void AppendString(CKeyPadApp * pMe, AECHAR* szBuf, boolean bIndent)
{
   AECHAR szIndent[MAX_STRLEN];
   AECHAR* szString = (AECHAR*) szBuf;

   if ( bIndent )
   {
      ISHELL_LoadResString(pMe->a.m_pIShell, APP_RES_FILE, IDS_BLANK, (AECHAR*)szIndent, MAX_STRLEN);
      WSTRCAT(szIndent, szBuf);
      szString = (AECHAR*) szIndent;
   }

   TextWin_Append(pMe->m_pText, szString);
}

static void ProcessKeyEvent(CKeyPadApp * pMe)
{
   // Display: EVT_KEY received
   AppendStringRes(pMe, IDS_EVT_KEY, FALSE);

   // wParam contains the primary code
   AppendStringRes(pMe, IDS_PRIMARY_CODE, FALSE);
   AppendStringRes(pMe, (uint16)(pMe->m_wParam - KEY_BASE), TRUE);

   pMe->m_bReady = FALSE;
   DrawAppTitle(pMe);
}

/*========================End of File ================================================*/