/*======================================================
FILE:  MemoApp.c

SERVICES:  Sample Memo Pad Application

GENERAL DESCRIPTION:
	Sample memo pad app running under AEE

PUBLIC CLASSES AND STATIC FUNCTIONS:
	List the class(es) and static functions included in this file

INITIALIZATION & SEQUENCING REQUIREMENTS:

	See Exported Routines

        Copyright  1999-2001 QUALCOMM Incorporated.
               All Rights Reserved.
            QUALCOMM Proprietary/GTDR
=====================================================*/

//#define TEST_MENUS

//===========================================================================
//
//                       PLEASE NOTE THE FOLLOWING
//
// This sample application was developed before some BREW APIs were available.
// As a result, it contains code that may not be optimal.  The main areas 
// of note related to this issue are:
//
// - Dialogs
// - Messages
// - Simple Prompts
//
// BREW now provides much better support for these operation
// 
// In future revisions, this code will be modified to support these new APIs.
//
//
//===========================================================================

#include "AEEShell.h"
#include "AEEStdLib.h"
#include "AEEDisp.h"
#include "AEEText.h"
#include "AEEHeap.h"
#include "AEEMenu.h"
#include "AEEDB.h"
#include "MemoApp_res.h"
#include "AEEModGen.h"
#include "AEEAppGen.h"
#include "AEESampleAppIDs.h"

#include "AEEControls_res.h"

/*===========================================================================

                      PUBLIC DATA DECLARATIONS

===========================================================================*/

#define MEMO_MIN_DB                 1000
#define MEMO_MIN_RECS               20

#define SAVED_TIMEOUT               (3000)
#define MEMOPAD_DATABASE_NAME       "MemoPad"
#define MAX_MEMO_TEXT               128
#define MAX_MEMO_TITLE              32
#define REC_NONE                    0xffff

typedef struct _CMemoApp
{
   AEEApplet      a;

   IDBMgr      *  m_pIDBMgr;
   IDatabase   *  m_pIDatabase;

   // QCOM UI controls

   IMenuCtl *     m_pIMenu;
   IMenuCtl *     m_pISoftKey;
   ITextCtl *     m_pIText;

   // screen and font dimensions
   int            m_cx;
   int            m_cy;
   int            m_nFontHeight;

   int16          m_nState;
   uint16         m_wRecID;

   AECHAR         m_szTitle[MAX_MEMO_TITLE + 1];
   AECHAR         m_szText[MAX_MEMO_TEXT + 1];

   AECHAR         m_szEmpty[32];
} CMemoApp;

static boolean    CMemoApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);
static void       CMemoApp_Free(CMemoApp * pme);
static AECHAR *   CMemoApp_ClipString(CMemoApp * pme, AECHAR *pszIn, boolean bAlloc);
static void       CMemoApp_SetState(CMemoApp * pme, int16 nState);
static void       CMemoApp_MainScreen(CMemoApp * pme);
static void       CMemoApp_EnterText(CMemoApp * pme);
static void       CMemoApp_EnterTextDone(CMemoApp * pme);
static void       CMemoApp_EnterTitle(CMemoApp * pme);
static void       CMemoApp_EnterTitleDone(CMemoApp * pme);
static void       CMemoApp_DeletedMessage(CMemoApp * pme);
static void       CMemoApp_Message(CMemoApp * pme, uint16 wID, AECHAR * pszTitle);
static void       CMemoApp_View(CMemoApp * pme);
static void       CMemoApp_DeleteMemo(CMemoApp * pme);
static void       CMemoApp_DeleteMemoDone(CMemoApp * pme);
static void       CMemoApp_SetupTextField(CMemoApp * pme, AEEDBFieldName nField, uint16 wTitleID, boolean bMultiLine, uint16 nNextState);
static void       CMemoApp_Save(CMemoApp * pme);
static void       CMemoApp_ReturnToMainScreen(CMemoApp * pme);
static AECHAR *   CMemoApp_GetTitle(CMemoApp * pme, IDBRecord * pRec, boolean * pbFromText, boolean bViewOnly);
static AECHAR *   CMemoApp_GetText(CMemoApp * pme, IDBRecord * pRec);
static void       CMemoApp_Reset(CMemoApp * pme);

enum {MS_MAIN_SCREEN,
      MS_ENTER_TEXT,
      MS_ENTER_TITLE,
      MS_SAVED,
      MS_VIEW,
      MS_DELETE,
      MS_DELETED,
      MS_MAX};
         
//
// Static Data
//

static const int gBackIDs[] = {-1,
                               MS_MAIN_SCREEN,
                               MS_ENTER_TEXT,
                               MS_MAIN_SCREEN,
                               MS_MAIN_SCREEN,
                               MS_VIEW,
                               MS_MAIN_SCREEN};


//
// Static Applet Functions - These functions need only be declared
// when an applet is being linked by the Manufacturer during their
// main build process.  They are not required for dynamically downloaded
// applets.
//
#if defined(BREW_STATIC_APP)

static int     MemoMod_CreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj);
static int     MemoMod_Load(IShell *ps, void * pHelpers, IModule ** pMod);

static const AEEAppInfo    gaiMemoPad = {AEECLSID_MEMO,MEMOAPP_RES_FILE, IDS_APP_TITLE, IDI_MEMOPAD,0,0,0,AFLAG_PIM};

/*===========================================================================

                      PUBLIC FUNCTION DECLARATIONS

===========================================================================*/

//===========================================================================
//
// Returns static module information
//
//===========================================================================
PFNMODENTRY MemoPad_GetModInfo(IShell * ps,AEECLSID ** ppClasses, AEEAppInfo ** pApps, uint16 * pnApps,uint16 * pwMinPriv)
{
   *pApps = (AEEAppInfo *)&gaiMemoPad;
   *pnApps = sizeof(gaiMemoPad) / sizeof(AEEAppInfo);
   return((PFNMODENTRY)MemoMod_Load);
}

//===========================================================================
//
// Called by BREW to initialize the static module.
//
//===========================================================================
static int MemoMod_Load(IShell *ps, void * pHelpers, IModule ** pMod)
{
   return(AEEStaticMod_New((int16)(sizeof(AEEMod)),ps,pHelpers,pMod,(PFNMODCREATEINST)MemoMod_CreateInstance,(PFNFREEMODDATA)NULL));
}

#endif

//===========================================================================
//
// Called by AEEModGen when the applet has been requested by BREW
//
//===========================================================================
#if defined(BREW_STATIC_APP)
static int MemoMod_CreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
#else
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
#endif
{
   CMemoApp *     pme = NULL;
   AEEDeviceInfo  dm;

   if(ClsId != AEECLSID_MEMO)
      return(EBADCLASS);

   *ppObj = NULL;

   if(!AEEApplet_New(sizeof(CMemoApp), ClsId, pIShell,po,(IApplet**)ppObj,(AEEHANDLER)CMemoApp_HandleEvent,(PFNFREEAPPDATA)CMemoApp_Free))
      return(ENOMEMORY);

   pme = (CMemoApp *)*ppObj;

   pme->m_wRecID     = REC_NONE;

   ISHELL_LoadResString(pIShell, MEMOAPP_RES_FILE, IDS_UNTITLED, pme->m_szEmpty, sizeof(pme->m_szEmpty));

   if(ISHELL_CreateInstance(pIShell, AEECLSID_MENUCTL, (void **)&pme->m_pIMenu)){
abort:
      IAPPLET_Release((IApplet *)pme);
      *ppObj = NULL;
      return(EFAILED);
   }

   if(ISHELL_CreateInstance(pIShell, AEECLSID_SOFTKEYCTL, (void **)&pme->m_pISoftKey))
      goto abort;

   if(ISHELL_CreateInstance(pIShell, AEECLSID_TEXTCTL, (void **)&pme->m_pIText))
      goto abort;

   ISHELL_GetDeviceInfo(pIShell, &dm);
   pme->m_cx = dm.cxScreen;
   pme->m_cy = dm.cyScreen;
   pme->m_nFontHeight = IDISPLAY_GetFontMetrics(pme->a.m_pIDisplay, AEE_FONT_BOLD, NULL, NULL);

   if(ISHELL_CreateInstance((IShell*)pme->a.m_pIShell, AEECLSID_DBMGR, (void **)&pme->m_pIDBMgr))
      goto abort;

   pme->m_pIDatabase = IDBMGR_OpenDatabaseEx(pme->m_pIDBMgr, MEMOPAD_DATABASE_NAME, 1, MEMO_MIN_DB, MEMO_MIN_RECS);

   if (!pme->m_pIDatabase)
      goto abort;

   pme->m_nState = -1;
   return(AEE_SUCCESS);
}

/*===========================================================================

                      APPLET CLASS

===========================================================================*/

//===========================================================================
//
// 
//
//===========================================================================
static boolean CMemoApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{
  
   CMemoApp * pme = (CMemoApp*)pi;
   
   switch (eCode) {

      case EVT_DIALOG_INIT:
      case EVT_DIALOG_START:
         return(TRUE);

      case EVT_APP_SUSPEND:
         CMemoApp_Reset(pme);
         return(TRUE);

      case EVT_COPYRIGHT_END:
      case EVT_APP_RESUME:
      case EVT_DIALOG_END:
         CMemoApp_SetState(pme, pme->m_nState);
         return(TRUE);

      case EVT_APP_START:
         pme->m_nState = MS_MAIN_SCREEN;
         ISHELL_ShowCopyright(pme->a.m_pIShell);
         return(TRUE);

      case EVT_APP_STOP:
         return(TRUE);

      case EVT_COMMAND: 

   // This allows us to handle the T9 stuff with little mods...

         if(!ITEXTCTL_HandleEvent(pme->m_pIText, eCode, wParam, dwParam))
            CMemoApp_SetState(pme, (int16)wParam);
         return(TRUE);

      case EVT_KEY_HELD:
      case EVT_KEY_PRESS:
      case EVT_KEY_RELEASE:
         if(ITEXTCTL_HandleEvent(pme->m_pIText, eCode, wParam, dwParam))
            return(TRUE);
         break;

      case EVT_KEY: 
         
         if(wParam == AVK_CLR && ISHELL_GetActiveDialog(pme->a.m_pIShell)){
            ISHELL_EndDialog(pme->a.m_pIShell);
            return(TRUE);
         }

         if(IMENUCTL_HandleEvent(pme->m_pIMenu, eCode, wParam, dwParam))
            return(TRUE);

         if(IMENUCTL_HandleEvent(pme->m_pISoftKey, eCode, wParam, dwParam))
            return(TRUE);

         if(ITEXTCTL_HandleEvent(pme->m_pIText, eCode, wParam, dwParam))
            return(TRUE);

         if(pme->m_nState > 0 && wParam == AVK_CLR){
            CMemoApp_SetState(pme, -1);
            return(TRUE);
         }
         return(FALSE);
   }
   
   return(FALSE);
}

//===========================================================================
//
// 
//
//===========================================================================
static void CMemoApp_Free(CMemoApp * pme)
{
   ISHELL_CancelTimer(pme->a.m_pIShell, NULL, pme);

   if (pme->m_pIMenu)
      IMENUCTL_Release(pme->m_pIMenu);

   if (pme->m_pISoftKey)
      IMENUCTL_Release(pme->m_pISoftKey);

   if (pme->m_pIText)
      ITEXTCTL_Release(pme->m_pIText);

   if (pme->m_pIDatabase)
      IDATABASE_Release(pme->m_pIDatabase);

   if (pme->m_pIDBMgr)
      IDBMGR_Release(pme->m_pIDBMgr);

}

//===========================================================================
//
// 
//
//===========================================================================
static void    CMemoApp_SetState(CMemoApp * pme, int16 nState)
{
   int16   nStateLast = pme->m_nState;
   AEERect rc;

   if(nState == -1){
      if(nStateLast == -1)
         nStateLast = MS_MAIN_SCREEN;
      nState = gBackIDs[nStateLast];
   }
   else{

   // 
   // NOTE:  This is done in this manner (rather than in a table) because of 
   // ARM relocation issues.  Placing the functions in a static table causes a linkage error
   //

      switch(nStateLast){
         case MS_ENTER_TEXT:
            CMemoApp_EnterTextDone(pme);
            break;
         case MS_ENTER_TITLE:
            CMemoApp_EnterTitleDone(pme);
            break;
         case MS_DELETE:
            CMemoApp_DeleteMemoDone(pme);
            break;
      }

      if(nState >= MS_MAX){
         pme->m_wRecID = (uint16)(nState - MS_MAX);
         nState = MS_VIEW;
      }
   }

   pme->m_nState = nState;

   CMemoApp_Reset(pme);
   
   SETAEERECT(&rc, 0, 0,pme->m_cx, pme->m_cy);
   IDISPLAY_FillRect(pme->a.m_pIDisplay, &rc, RGB_WHITE);

   // 
   // NOTE:  This is done in this manner (rather than in a table) because of 
   // ARM relocation issues.  Placing the functions in a static table causes a linkage error
   //

   switch(pme->m_nState){
      case MS_MAIN_SCREEN:
         CMemoApp_MainScreen(pme);
         break;
      case MS_ENTER_TEXT:
         CMemoApp_EnterText(pme);
         break;
      case MS_ENTER_TITLE:
         CMemoApp_EnterTitle(pme);
         break;
      case MS_SAVED:
         CMemoApp_Save(pme);
         break;
      case MS_VIEW:
         CMemoApp_View(pme);
         break;
      case MS_DELETE:
         CMemoApp_DeleteMemo(pme);
         break;
      case MS_DELETED:
         CMemoApp_DeletedMessage(pme);
         break;
   }
}

//===========================================================================
//
// 
//
//===========================================================================
static void CMemoApp_Reset(CMemoApp * pme)
{
   ITEXTCTL_Reset(pme->m_pIText);
   IMENUCTL_Reset(pme->m_pIMenu);
   IMENUCTL_Reset(pme->m_pISoftKey);
}

//===========================================================================
//
// 
//
//===========================================================================
static void    CMemoApp_MainScreen(CMemoApp * pme)
{
   IDBRecord   *pRec;
   int         i;
   AECHAR  * psz;


   // Cleanup any pending stuff...

   pme->m_wRecID = REC_NONE;
   pme->m_szText[0] = pme->m_szTitle[0] = 0;

   // Setup the main screen...

   IMENUCTL_SetTitle(pme->m_pIMenu, MEMOAPP_RES_FILE, IDS_APP_TITLE, NULL);

   IDATABASE_Reset(pme->m_pIDatabase);
   
   i = MS_MAX;

   while((pRec = IDATABASE_GetNextRecord(pme->m_pIDatabase)) != NULL){

      if((psz = CMemoApp_GetTitle(pme, pRec, NULL, TRUE)) != NULL)
         IMENUCTL_AddItem(pme->m_pIMenu, NULL, 0, (uint16)(i + IDBRECORD_GetID(pRec)), psz,0);

      IDBRECORD_Release(pRec);
   }

#if !defined(TEST_MENUS)
   IMENUCTL_AddItem(pme->m_pIMenu, MEMOAPP_RES_FILE, IDC_ADD, MS_ENTER_TEXT, NULL, 0);
#else
   {
   CtlAddItem  t;
   AECHAR      szTemp[32];
   int         i;

   ISHELL_LoadResString(pme->a.m_pIShell, MEMOAPP_RES_FILE, IDC_ADD, szTemp, sizeof(szTemp));
   MEMSET(&t, 0, sizeof(t));
            t.pText = szTemp;
            t.pszResImage = AEECONTROLS_RES_FILE;
            t.wImage = IDB_GAME_THUMB;
            t.wItemID = MS_ENTER_TEXT;
            IMENUCTL_AddItemEx(pme->m_pIMenu, &t);

   t.wItemID = MS_ENTER_TEXT + 100;
   t.pszResText = MEMOAPP_RES_FILE;
   t.wText = IDS_REALLY_LONG;
   t.pText = NULL;
   IMENUCTL_AddItemEx(pme->m_pIMenu, &t);

   for(i = 0; i < 50; i++){
      t.pText = szTemp;
      t.wImage = IDB_TOOL_THUMB;
      t.wItemID++;
      IMENUCTL_AddItemEx(pme->m_pIMenu, &t);
   }
   }
#endif


   IMENUCTL_SetActive(pme->m_pIMenu, TRUE);
}

//===========================================================================
//
// 
//
//===========================================================================
static void    CMemoApp_View(CMemoApp * pme)
{
   IDBRecord * pRec;
   AEERect       rcm, rc;
   AEEDBFieldName fn;
   uint16      wLen,wSize;
   AECHAR  * pszMsg, * psz, * pszDest;
   boolean     bFromText;

   if((pRec = IDATABASE_GetRecordByID(pme->m_pIDatabase, pme->m_wRecID)) == NULL){
      CMemoApp_SetState(pme, MS_MAIN_SCREEN);
      return;
   }

   // Add the menu items and set the softkey menu active...

#if !defined(TEST_MENUS)
   IMENUCTL_AddItem(pme->m_pISoftKey, MEMOAPP_RES_FILE, IDC_EDIT, MS_ENTER_TEXT, NULL, 0);
#else
   {
   AECHAR   szTemp[32];
   CtlAddItem  t;

   ISHELL_LoadResString(pme->a.m_pIShell, MEMOAPP_RES_FILE, IDC_EDIT, szTemp, sizeof(szTemp));
   MEMSET(&t, 0, sizeof(t));
            t.pText = szTemp;
            t.pszResText = AEECONTROLS_RES_FILE;
            t.wImage = IDB_GAME_THUMB;
            t.wItemID = MS_ENTER_TEXT;
            IMENUCTL_AddItemEx(pme->m_pISoftKey, &t);
   }
#endif

   IMENUCTL_AddItem(pme->m_pISoftKey, MEMOAPP_RES_FILE, IDC_DELETE, MS_DELETE, NULL, 0);
   IMENUCTL_SetActive(pme->m_pISoftKey, TRUE);

   // Setup the Text control to be as large as the remainder of the screen...

   IMENUCTL_GetRect(pme->m_pISoftKey, &rcm);
   SETAEERECT(&rc, 0, 0, rcm.dx, rcm.y);
   ITEXTCTL_SetProperties(pme->m_pIText, TP_FRAME|TP_MULTILINE);
   ITEXTCTL_SetRect(pme->m_pIText, &rc);

   wSize = 0;

   if((psz = CMemoApp_GetTitle(pme, pRec, &bFromText, FALSE)) != NULL && !bFromText)
      wSize += WSTRLEN(psz) + 1;

   if((psz = CMemoApp_GetText(pme, pRec)) != NULL)
      wSize += WSTRLEN(psz) + 1;

   pszMsg = (AECHAR *)MALLOC(wSize * sizeof(AECHAR));

   if(pszMsg != NULL){

      pszDest = pszMsg;

      IDBRECORD_Reset(pRec);
      if(IDBRECORD_NextField(pRec, &fn, &wLen) == AEEDB_FT_STRING){
         psz = IDBRECORD_GetFieldString(pRec);
         if(psz != NULL){
            wLen = WSTRLEN(psz);
            MEMCPY(pszDest, psz, wLen * sizeof(AECHAR));
            if(fn != AEEDBFIELD_TEXT){
               pszDest += wLen;
               *pszDest = (AECHAR)'\n';
               pszDest++;

               if(IDBRECORD_NextField(pRec, &fn, &wLen) == AEEDB_FT_STRING){
                  psz = IDBRECORD_GetFieldString(pRec);
                  if(psz != NULL)
                     MEMCPY(pszDest, psz, WSTRLEN(psz) * sizeof(AECHAR));
               }
            }
         }
      }

      ITEXTCTL_SetMaxSize(pme->m_pIText, (uint16)WSTRLEN(pszMsg));
      ITEXTCTL_SetText(pme->m_pIText, pszMsg, -1);
      IMENUCTL_Redraw(pme->m_pISoftKey);

      FREE(pszMsg);
   }

   IDBRECORD_Release(pRec);
}

//===========================================================================
//
// 
//
//===========================================================================
static void    CMemoApp_EnterText(CMemoApp * pme)
{
   CMemoApp_SetupTextField(pme, AEEDBFIELD_TEXT, IDS_ENTER_TEXT, TRUE, MS_ENTER_TITLE);
}

//===========================================================================
//
// 
//
//===========================================================================
static void    CMemoApp_EnterTitle(CMemoApp * pme)
{
   CMemoApp_SetupTextField(pme, AEEDBFIELD_TITLE, IDS_ENTER_TITLE, FALSE, MS_SAVED);
}

//===========================================================================
//
// 
//
//===========================================================================
static void CMemoApp_SetupTextField(CMemoApp * pme, AEEDBFieldName nField, uint16 wTitleID, boolean bMultiLine, uint16 nNextState)
{
   IDBRecord   *  pRec;
   AECHAR  *       psz;
   AEERect        rc,rcm;
   uint32         dwFlags;
   AECHAR          szEmpty[1];
   boolean        bFree;

   psz = NULL;

   bFree = FALSE;

   if(pme->m_wRecID != REC_NONE && (pRec = IDATABASE_GetRecordByID(pme->m_pIDatabase, pme->m_wRecID)) != NULL){

      if(nField == AEEDBFIELD_TITLE)
         psz = CMemoApp_GetTitle(pme, pRec, NULL, FALSE);
      else
         psz = CMemoApp_GetText(pme, pRec);
   }
   else{

   // If we are editing a new record, use the text entered...

      if(nField == AEEDBFIELD_TITLE){
         psz = CMemoApp_ClipString(pme, pme->m_szText, TRUE);
         bFree = TRUE;
      }
      pRec = NULL;
   }

   if(!psz){
      psz = szEmpty;
      *psz = (AECHAR)0;
      bFree = FALSE;
   }

   IMENUCTL_AddItem(pme->m_pISoftKey, MEMOAPP_RES_FILE, IDC_OK, nNextState, NULL, 0);

   ITEXTCTL_SetTitle(pme->m_pIText, MEMOAPP_RES_FILE, wTitleID, NULL);
   IMENUCTL_GetRect(pme->m_pISoftKey, &rcm);
   SETAEERECT(&rc, 0, 0, rcm.dx, rcm.y);

   dwFlags = TP_FRAME|TP_T9_MODE;
   if(bMultiLine)
      dwFlags |= TP_MULTILINE;

   ITEXTCTL_SetMaxSize(pme->m_pIText, (uint16)(bMultiLine ? MAX_MEMO_TEXT : MAX_MEMO_TITLE));

   ITEXTCTL_SetProperties(pme->m_pIText, dwFlags);
   ITEXTCTL_SetRect(pme->m_pIText, &rc);

   ITEXTCTL_SetSoftKeyMenu(pme->m_pIText, pme->m_pISoftKey);
   ITEXTCTL_SetText(pme->m_pIText, psz, -1);

   if(bFree)
      FREE(psz);

   if(nField == AEEDBFIELD_TEXT || *psz == (AECHAR)0){
      ITEXTCTL_SetActive(pme->m_pIText, TRUE);
      IMENUCTL_Redraw(pme->m_pISoftKey);
   }
   else{
      IMENUCTL_SetActive(pme->m_pISoftKey, TRUE);
      ITEXTCTL_Redraw(pme->m_pIText);
   }
   if(pRec)
      IDBRECORD_Release(pRec);

}

//===========================================================================
//
// 
//
//===========================================================================
static void    CMemoApp_EnterTextDone(CMemoApp * pme)
{
   AECHAR * psz;

   psz = ITEXTCTL_GetTextPtr(pme->m_pIText);

   if(psz)
      WSTRNCOPY(pme->m_szText, MAX_MEMO_TEXT,psz);
   else
      *pme->m_szText = 0;
}

//===========================================================================
//
// 
//
//===========================================================================
static void    CMemoApp_EnterTitleDone(CMemoApp * pme)
{
   AECHAR *   psz, *pszT, *pszCurr,ch,chT = 0;
   IDBRecord *    pRec;
   boolean        bFromText;

   MEMSET(pme->m_szTitle, 0, sizeof(pme->m_szTitle));

   psz = ITEXTCTL_GetTextPtr(pme->m_pIText);

   // If the title did not change and it was generated from the text, just ignore it...

   if(pme->m_wRecID != REC_NONE && (pRec = IDATABASE_GetRecordByID(pme->m_pIDatabase, pme->m_wRecID)) != NULL){
      pszCurr = CMemoApp_GetTitle(pme, pRec, &bFromText, FALSE);
      if(!bFromText)
         pszCurr = NULL;
   }
   else{

   // If it was a new record, compare it to the text...

      pRec = NULL;
      pszCurr = pme->m_szText;
   }

   if(pszCurr){
      for(pszT = psz; ; pszCurr++, pszT++){
		   ch = *pszCurr;
         chT = *pszT;
		   if(!ch)
			   break;
         if(ch != chT)
            break;
      }
      if(!ch && !chT)
         psz = NULL;
   }

   if(pRec)
      IDBRECORD_Release(pRec);

   if(psz)
      WSTRNCOPY(pme->m_szTitle, MAX_MEMO_TITLE, psz);
}

//===========================================================================
//
// 
//
//===========================================================================
static void CMemoApp_Save(CMemoApp * pme)
{  
   IDBRecord * pRec;
   AEEDBField     f[2];
   int         nIdx;
   AECHAR * psz;

   nIdx = 0;

   psz = pme->m_szTitle;

   if(*psz){
      f[0].fName = AEEDBFIELD_TITLE;
      f[0].fType = AEEDB_FT_STRING;
      f[0].pBuffer = psz;
      f[0].wDataLen = WSTRSIZE(psz);
      nIdx++;
   }

   f[nIdx].fName = AEEDBFIELD_TEXT;
   f[nIdx].fType = AEEDB_FT_STRING;
   f[nIdx].pBuffer = pme->m_szText;
   f[nIdx].wDataLen = WSTRSIZE(pme->m_szText);

   if(pme->m_wRecID != REC_NONE && (pRec = IDATABASE_GetRecordByID(pme->m_pIDatabase, pme->m_wRecID)) != NULL)
      IDBRECORD_Update(pRec, f, nIdx + 1);
   else
      pRec = IDATABASE_CreateRecord(pme->m_pIDatabase, f, nIdx + 1);

   if(!pRec)
      CMemoApp_Message(pme, IDS_BAD_SAVE, NULL);
   else{      
      if(!*psz)
         psz = CMemoApp_GetTitle(pme, pRec, NULL, TRUE);

      CMemoApp_Message(pme, IDS_SAVED, (*psz ? psz : NULL));

      IDBRECORD_Release(pRec);
   }

   *pme->m_szText = *pme->m_szTitle = 0;
}

//===========================================================================
//
// 
//
//===========================================================================
static void CMemoApp_DeletedMessage(CMemoApp * pme)
{
   CMemoApp_Message(pme, IDS_DELETED, NULL);
}

//===========================================================================
//
// 
//
//===========================================================================
static void CMemoApp_Message(CMemoApp * pme, uint16 wID, AECHAR * pszTitle)
{
   int      y;
   AECHAR   szBuff[32];
   AEERect  rc;

   IMENUCTL_GetRect(pme->m_pISoftKey, &rc);

   if(pszTitle != NULL){
      y = (rc.y - (pme->m_nFontHeight * 2)) / 2;
      IDISPLAY_DrawText(pme->a.m_pIDisplay,AEE_FONT_NORMAL,pszTitle,-1,-1,y,NULL,IDF_ALIGN_CENTER);
      y += pme->m_nFontHeight;
   }
   else
      y = (rc.y - pme->m_nFontHeight) / 2;

   ISHELL_LoadResString(pme->a.m_pIShell, MEMOAPP_RES_FILE, wID, szBuff, sizeof(szBuff));
   IDISPLAY_DrawText(pme->a.m_pIDisplay,AEE_FONT_NORMAL,szBuff,-1,-1,y,NULL,IDF_ALIGN_CENTER);

   IDISPLAY_Update(pme->a.m_pIDisplay);

   ISHELL_SetTimer(pme->a.m_pIShell, SAVED_TIMEOUT, (PFNNOTIFY)(CMemoApp_ReturnToMainScreen), pme);
}

//===========================================================================
//
// 
//
//===========================================================================
static void CMemoApp_ReturnToMainScreen(CMemoApp * pme)
{
   CMemoApp_SetState(pme, MS_MAIN_SCREEN);
}

//===========================================================================
//
// 
//
//===========================================================================
static void    CMemoApp_DeleteMemo(CMemoApp * pme)
{
   AECHAR  szBuff[32];
   AEERect       rc;

   IMENUCTL_GetRect(pme->m_pISoftKey, &rc);

   ISHELL_LoadResString(pme->a.m_pIShell, MEMOAPP_RES_FILE, IDS_DELETEQ, szBuff, sizeof(szBuff));
   IDISPLAY_DrawText(pme->a.m_pIDisplay,AEE_FONT_NORMAL,szBuff,-1,-1, (rc.y - pme->m_nFontHeight) / 2,NULL,IDF_ALIGN_CENTER);

   // Add the menu items and set the softkey menu active...

   IMENUCTL_AddItem(pme->m_pISoftKey, MEMOAPP_RES_FILE, IDC_YES, MS_DELETED, NULL, 0);
   IMENUCTL_AddItem(pme->m_pISoftKey, MEMOAPP_RES_FILE, IDC_NO, (uint16)-1, NULL, 0);
   IMENUCTL_SetActive(pme->m_pISoftKey, TRUE);

}

//===========================================================================
//
// 
//
//===========================================================================
static void    CMemoApp_DeleteMemoDone(CMemoApp * pme)
{
   IDBRecord * pRec;
   
   if(pme->m_wRecID != REC_NONE && (pRec = IDATABASE_GetRecordByID(pme->m_pIDatabase, pme->m_wRecID)) != NULL)
      IDBRECORD_Remove(pRec);
}

//===========================================================================
//
// 
//
//===========================================================================
static AECHAR * CMemoApp_GetText(CMemoApp * pme, IDBRecord * pRec)
{
   AEEDBFieldName fn;
   uint16      wLen;
   AECHAR * pszRet;

   IDBRECORD_Reset(pRec);
   while(IDBRECORD_NextField(pRec, &fn, &wLen) == AEEDB_FT_STRING){
      if(fn == AEEDBFIELD_TEXT && (pszRet = IDBRECORD_GetFieldString(pRec)) != NULL)
         return(pszRet);
   }
   return(NULL);
}

//===========================================================================
//
// 
//
//===========================================================================
static AECHAR * CMemoApp_GetTitle(CMemoApp * pme, IDBRecord * pRec, boolean * pbFromText,boolean bViewOnly)
{
   AEEDBFieldName    fn;
   AECHAR  *  pszRet;
   uint16         wLen;
   boolean        bText = FALSE;

   // Look for the title...

   if(pbFromText)
      *pbFromText = FALSE;

   pszRet = NULL;

   IDBRECORD_Reset(pRec);

   while(IDBRECORD_NextField(pRec, &fn, &wLen) == AEEDB_FT_STRING){
      if(fn == AEEDBFIELD_TITLE && (pszRet = IDBRECORD_GetFieldString(pRec)) != NULL)
         break;
   }         

   // There is no title.  Build a title from the text...

   if(pszRet == NULL){

      pszRet = CMemoApp_GetText(pme, pRec);

   // If there is a text field, see if it fits...

      if(pszRet && *pszRet){
         bText = TRUE;
         CMemoApp_ClipString(pme, pszRet, FALSE);
      }
   }

   if(bViewOnly && (!pszRet || !*pszRet))
      pszRet = pme->m_szEmpty;
      
   if(pbFromText)
      *pbFromText = bText;

   return(pszRet);
}


//===========================================================================
//
// 
//
//===========================================================================
static AECHAR * CMemoApp_ClipString(CMemoApp * pme, AECHAR *pszIn, boolean bAlloc)
{
   AECHAR *       pszRet, * pszEnd,ch;
   AEERect        rc;
   int16          wWidth;

   if(bAlloc){
      pszRet = WSTRDUP(pszIn);
      if(!pszRet)
         return(NULL);
   }
   else
      pszRet = pszIn;

   pszEnd = pszRet + WSTRLEN(pszRet) - 1;

   ch = IDISPLAY_GetSymbol(pme->a.m_pIDisplay, AEE_SYM_ELLIPSES, AEE_FONT_NORMAL);

   ITEXTCTL_GetRect(pme->m_pIText, &rc);

   while(pszEnd > pszRet){

      wWidth = IDISPLAY_MeasureText(pme->a.m_pIDisplay, AEE_FONT_NORMAL, pszRet);

      if(wWidth <= rc.dx - 8)
         break;

      --pszEnd;
      *pszEnd = 0;
      --pszEnd;
      *pszEnd = ch;
   }
   return(pszRet);
}

