/*===========================================================================

FILE: Twinapp.c
  
SERVICES: Sample applet using AEE
    
DESCRIPTION
   This file contains the implementation of a simple example using the
   AEE services.
   This is to demonstrate presence of two applets within the same module.
   The module is called "TwinApp1" and the two applets within this module
   are "TwinApp1" and "TwinApp2".
   It also demonstartes the use of IDateCtl, IGraphic, ITimeCtl, ITextCtl interfaces
      
PUBLIC CLASSES:  
   N/A
        
INITIALIZATION AND SEQUENCING REQUIREMENTS:
   The following explanation applies to this sample containing one applet which serves
   as a base for app developers to create their own applets using AEE Services:
   

   In the applet source file (like this one), include AEEAppGen.h.

   Mandatory Sections in Applet Source (this file):
   -----------------------------------------------
   Following Mandatory Sections are required for each applet source file. 
   (Search for "Mandatory" to identify these sections)

   Includes:     
      Copy this section as-is from the sample applet. It contains:
      AEEAppGen.h: For AEEApplet declaration

   Type Declarations:
		A data structure must be defined to hold the app specific data. In this structure,
		the first element must be of type AEEApplet.

   Functions: (For more details, see corresponding function description in this applet)

      App_HandleEvent(): is the Event Handler to this applet.
                         Copy the function outline from the sample applet and add app specific code.

      AEEClsCreateInstance(): creates an instance of the applet and initializes it. 
                              It is called by AEEModGen when applet is being created.

   
   Important Notes:
   ---------------
   1. DO NOT use any "static data" in the applet. Always use the functions exported by
	   AEEStdlib or by IHeap services to dynamically allocate data and make it a member of 
		the applet structure. 
   2. DO NOT include and link "standard C library". Use AEE Memory Services (in AEEHeap.h) and Standard Library macros(in AEEStdLib.h).
      For example, use MALLOC() to allocate memory, WSTRCPY() to make a copy of Unicode (wide) string.
   3. BREW is Unicode(wide string) compliant ONLY (no ISOLATIN1/ANSI) except for file names which are ISOLATIN1/ANSI.
      ALWAYS USE AECHAR instead of "char". Use string services provided in AEEStdLib.h for string manipulation.
   4. It is always a good idea to DEFINE RESOURCES using BREW ResourceEditor. Make Strings, Bitmaps, Dialogs, etc. 
      as resources. ResourceEditor saves resources as .bri file, generates resource header file 
      and compiles .bri into a .bar binary file, which can be used by the applet.

   Miscellanoeus Notes:
   -------------------
   1. Make sure that the class ID used for the app is the same as that defined corresponding in .MIF file
   2. Always make sure that compiled resource (.bar) file and corresponding
      resource header (a) reside in app directory and (b) are included in the applet code.
      Define a constant APP_RES_FILE containing the name of the compiled resource file (with .bar extension).

   More than one applet:
   --------------------
   If more than one applet needs to be defined, then do the following
   (1) Follow the above description for each applet




   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/


/*===============================================================================
INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
// Mandatory includes
#include "AEEAppGen.h"	// AEEApplet Declaration

#include "AEE.h"			// Standard AEE Declarations
#include "AEEShell.h"	// AEE Shell Services
#include "AEEDisp.h"		// AEE Display Services
#include "AEEHeap.h"		// AEE Memory Allocation Services
#include "AEEStdLib.h"  // AEE StdLib Services
#include "Twinapp1.bid" // App ID for Twinapp1
#include "Twinapp2.bid" // App ID for Twinapp2

// App developer code BEGIN
//

/*===========================================================================

  PUBLIC DATA DECLARATIONS
  
===========================================================================*/

/*-------------------------------------------------------------------
Type Declarations
-------------------------------------------------------------------*/

/*-------------------------------------------------------------------
      Type Declarations
-------------------------------------------------------------------*/
// Mandatory App specific data. 
// IMPORTANT NOTE: DO NOT CHANGE THE NAME OF THE FOLLOWING STRUCTURES


typedef struct _TwinApp1 TwinApp1;

struct _TwinApp1
{
	AEEApplet a;
};

typedef struct _TwinApp2 TwinApp2;
struct _TwinApp2
{
	AEEApplet a;
   // App developer code BEGIN
   //

   //
   // App developer code END
};








/*-------------------------------------------------------------------
Function Prototypes
-------------------------------------------------------------------*/
// App Handle Event functions
static boolean TwinApp1_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);
static boolean TwinApp2_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);



/*-------------------------------------------------------------------
Global Constant Definitions
-------------------------------------------------------------------*/


/*-------------------------------------------------------------------
Global Data Definitions
-------------------------------------------------------------------*/
// App specific constants
#define  MAX_STRLEN        100

/*-------------------------------------------------------------------
Static variable Definitions
-------------------------------------------------------------------*/


/*===============================================================================
FUNCTION DEFINITIONS
=============================================================================== */




/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time.

PROTOTYPE:
	int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell interface. 

	pIModule: pin]: Contains pointer to the IModule interface to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppApplet must point to a valid IBase object. 
	If the classID	represnts an applet, then ppObj must point to a valid AEEApplet structure.Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the class/app creation was successful. 
  EFAILED: Error occurred while creating the class/app. In this case, the app/class will
  not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID clsID, IShell* pIShell, IModule* pIModule, void** ppApplet)
{
   AEEApplet* pMe = 0;
   *ppApplet = NULL;
 

   if(clsID == AEECLSID_TWINAPP1)
   {
		// We want to load this App. So, invoke AEEApplet_New().To it, pass the
		// address of the app-specific handle event function.
      if ( !AEEApplet_New(sizeof(TwinApp1),clsID, pIShell, pIModule, (IApplet**)ppApplet, (AEEHANDLER) TwinApp1_HandleEvent, 0) )
         return EFAILED;

      pMe = (AEEApplet*)(*ppApplet);

      if ( !pMe )
         return EFAILED;

      // App developer code BEGIN
      //
 
      //
      // App developer code END

      return SUCCESS;
   }
   else if(clsID == AEECLSID_TWINAPP2)
   {
		// We want to load this App. So, invoke AEEApplet_New().To it, pass the
		// address of the app-specific handle event function.
      if ( !AEEApplet_New(sizeof(AEEApplet),clsID, pIShell, pIModule, (IApplet**)ppApplet, (AEEHANDLER)TwinApp2_HandleEvent, 0) )
         return EFAILED;

      pMe = (AEEApplet*)(*ppApplet);

      if ( !pMe )
         return EFAILED;

      // App developer code BEGIN
      //
      // Allocate and initialize app specific data
      //
      // App developer code END

      return SUCCESS;
   }

   return EFAILED;
}

/*===========================================================================

FUNCTION TwinApp1_HandleEvent
  
DESCRIPTION
   This is the EventHandler for this app. All events to this app are handled in this
   function. All APPs must supply an Event Handler.
    
PROTOTYPE:
   boolean TwinApp1_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
      
PARAMETERS:
   pi: Pointer to the AEEApplet structure. This structure contains information specific
   to this applet. It was initialized during the AppCreateInstance() function.
        
   ecode: Specifies the Event sent to this applet
          
   wParam, dwParam: Event specific data.
            
DEPENDENCIES
   none
              
RETURN VALUE
   TRUE: If the app has processed the event
   FALSE: If the app did not process the event
                
SIDE EFFECTS
   none
===========================================================================*/
static boolean TwinApp1_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{	
	AEEDeviceInfo di;
	AECHAR textBuf[] = {'T', 'w', 'i','n',' ','A','p','p',' ','1', '\0'};
	TwinApp1 * pMe = (TwinApp1*)pi;
	AEERect qrc;
	
	
	//Get the device info and set the screen rect so that it can be used
	// by all tests here.
	ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);
	qrc.x = 0;
	qrc.y = 0;
	qrc.dx = di.cxScreen;
	qrc.dy = di.cyScreen;   
	
	
	
	switch (eCode) 
	{
	case EVT_APP_START:
		
		
		IDISPLAY_EraseRect(pMe->a.m_pIDisplay,&qrc);
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, textBuf, -1,5, 15, 0,0);
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_BOLD, textBuf, -1,5, 45,0,0);
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_LARGE, textBuf, -1,5, 75,0,0);     
		
		IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);
		
		
		return(TRUE);
		
	case EVT_APP_STOP:
		
	
		
		return(TRUE);
		
	default:
		
		return FALSE;	
		
		
    }
	
	return(FALSE);
}

/*===========================================================================

FUNCTION TwinApp2_HandleEvent

DESCRIPTION
   This is the EventHandler for this app. All events to this app are handled in this
   function. All APPs must supply an Event Handler.
    
PROTOTYPE:
   boolean TwinApp2_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
      
PARAMETERS:
   pi: Pointer to the AEEApplet structure. This structure contains information specific
   to this applet. It was initialized during the AppCreateInstance() function.
        
   ecode: Specifies the Event sent to this applet
          
   wParam, dwParam: Event specific data.
            
DEPENDENCIES
   none
              
RETURN VALUE
   TRUE: If the app has processed the event
   FALSE: If the app did not process the event
                
SIDE EFFECTS
   none
===========================================================================*/
static boolean TwinApp2_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{
   
   TwinApp2 * pMe = (TwinApp2*)pi;
   AEEDeviceInfo di;
   AEERect qrc;
   AECHAR szBuf[MAX_STRLEN];
   
   //Get the device info and set the screen rect so that it can be used
   // by all tests here.
   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);
   qrc.x = 0;
   qrc.y = 0;
   qrc.dx = di.cxScreen;
   qrc.dy = di.cyScreen;
   
   switch (eCode) 
   {
      
   case EVT_APP_START:
      IDISPLAY_EraseRect(pMe->a.m_pIDisplay,&qrc);  //Clear the display
      STR_TO_WSTR("Twin App 2 Start", szBuf, sizeof(szBuf));
      IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_BOLD, szBuf, -1,5, 15, 0,0);
      IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);
      return(TRUE);
      
   case EVT_APP_STOP:
      IDISPLAY_EraseRect(pMe->a.m_pIDisplay,&qrc);  //Clear the display
      STR_TO_WSTR("Twin App 2 Stop", szBuf, sizeof(szBuf));
      IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_BOLD, szBuf, -1,5, 15, 0,0);
      IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);
      return(TRUE);
      
   case EVT_KEY: 
      switch(wParam)
      {
      case AVK_1:
         //Usage of Drawtext
         IDISPLAY_EraseRect(pMe->a.m_pIDisplay,&qrc);  //Clear the display
         STR_TO_WSTR("Twin App 2 Key 1", szBuf, sizeof(szBuf));
         IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_BOLD, szBuf, -1,5, 15, 0,0);
         IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);
         return(TRUE);
		}
   }
   
   return(FALSE);
}

/*===============================================================================
                     APP DATA ALLOC-INIT/FREE FUNCTION DEFINITIONS
=============================================================================== */




/*===============================================================================
                     APP HELPER FUNCTION DEFINITIONS
=============================================================================== */

 
  
/*========================End of File ================================================*/