/*===========================================================================

FILE: GraphicsBM.c

SERVICES: Sample GraphicsBM applet showing usage of AEE ISound interface

DESCRIPTION
  This file contains the implementation of a Sound Applet applet using the
  AEE ISound interface services.

  PUBLIC CLASSES:  
   N/A

INITIALIZATION AND SEQUENCING REQUIREMENTS:
   The following explanation applies to this sample containing one applet which serves
   as a base for app developers to create their own applets using AEE Services:
   

   In the applet source file (like this one), include AEEAppGen.h.

   Mandatory Sections in Applet Source (this file):
   -----------------------------------------------
   Following Mandatory Sections are required for each applet source file. 
   (Search for "Mandatory" to identify these sections)

   Includes:     
      Copy this section as-is from the sample applet. It contains:
      AEEAppGen.h: For AEEApplet declaration

   Type Declarations:
		A data structure must be defined to hold the app specific data. In this structure,
		the first element must be of type AEEApplet.

   Functions: (For more details, see corresponding function description in this applet)

      App_HandleEvent(): is the Event Handler to this applet.
                         Copy the function outline from the sample applet and add app specific code.

      AEEClsCreateInstance(): creates an instance of the applet and initializes it. 
                              It is called by AEEModGen when applet is being created.

   
   Important Notes:
   ---------------
   1. DO NOT use any "static data" in the applet. Always use the functions exported by
	   AEEStdlib or by IHeap services to dynamically allocate data and make it a member of 
		the applet structure. 
   2. DO NOT include and link "standard C library". Use AEE Memory Services (in AEEHeap.h) and Standard Library macros(in AEEStdLib.h).
      For example, use MALLOC() to allocate memory, WSTRCPY() to make a copy of Unicode (wide) string.
   3. BREW is Unicode(wide string) compliant ONLY (no ISOLATIN1/ANSI) except for file names which are ISOLATIN1/ANSI.
      ALWAYS USE AECHAR instead of "char". Use string services provided in AEEStdLib.h for string manipulation.
   4. It is always a good idea to DEFINE RESOURCES using BREW ResourceEditor. Make Strings, Bitmaps, Dialogs, etc. 
      as resources. ResourceEditor saves resources as .bri file, generates resource header file 
      and compiles .bri into a .bar binary file, which can be used by the applet.

   Miscellanoeus Notes:
   -------------------
   1. Make sure that the class ID used for the app is the same as that defined corresponding in .MIF file
   2. Always make sure that compiled resource (.bar) file and corresponding
      resource header (a) reside in app directory and (b) are included in the applet code.
      Define a constant DISPSPEEDTUNE_RES_FILE containing the name of the compiled resource file (with .bar extension).

   More than one applet:
   --------------------
   If more than one applet needs to be defined, then do the following
   (1) Follow the above description for each applet

   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR

===========================================================================*/

/*===============================================================================
                     INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "AEE.h"			// Standard AEE Declarations
#include "AEEShell.h"	// AEE Shell Services
#include "AEEDisp.h"		// AEE Display Services
#include "AEEStdLib.h"  // AEE StdLib Services
#include "AEEAppGen.h"	// AEEApplet structure and Applet services
#include "AEESampleAppIDs.h" // Sample App ID

// Services used by app
#include "AEEMenu.h"          // AEE MenuCtl Services
#include "AEEGraphics.h"      // AEE 2D Graphics Services
#include "AEEFile.h"          // AEE File Services

#include "AEESpeedTune.h"
#include "dispspeedtune_res.h"

/*===========================================================================
                      PUBLIC DATA DECLARATIONS
===========================================================================*/
/*-------------------------------------------------------------------
            Global Constant Definitions
-------------------------------------------------------------------*/
// App specific constants
#define DST_MAX_STRLEN           100

#define DST_LARGE_RECT_DX        80
#define DST_LARGE_RECT_DY        80

#define DST_MAX_DURATION         300   //2000
#define DST_AVG_VALUE            1     //4

#define DST_TEST_DIR             "testdir"
#define DST_TEST_READ_FILE       "testfile.rd"
#define DST_TEST_WRITE_FILE      "testfile.wr"
#define DST_FILE_BUF_SIZE        1024
#define DST_NUM_TEST_DIRS        100
#define DST_NUM_TEST_FILES       40

#define MAKE_FLOAT(x)            HACK_Float(#x)

/*-------------------------------------------------------------------
            Type Declarations
-------------------------------------------------------------------*/
typedef enum
{
   DST_OP_NONE,
   DST_OP_START,
   DST_OP_STOP
} DSTOperation;

//Mandatory structure.
//Structure to contain app data. This is the Applet structure that will be allocated
// for the app. Define this structure if the app needs to store any data that needs
// to be remembered across different events or callbacks. The first element of this
// structure MUST be of type AEEApplet. This is extremely important. Following this
// element, store the app-specific data. Note that AEEApplet structure already exposes
// the most commonly needed interface pointers such as Shell, Display, etc. 
typedef struct _CDispSpeedTune CDispSpeedTune;
struct _CDispSpeedTune
{
   AEEApplet      a;

	int            m_cxWidth;
	int            m_cyHeight;
   int            m_nCharHeight;
   uint16         m_wPercent;            
   boolean        m_bDone;

   IFileMgr *     m_pIFileMgr;
   IMenuCtl *     m_pSKMenu;
   AEERect        m_rectSKMenu;

   DSTOperation   m_eOp;
   DSTParam       m_eTest;
   PFNNOTIFY      m_pfnTestHandler[(int)DST_PARAM_LAST];
   AECHAR         m_TestName[(int)DST_PARAM_LAST][30];
   DSTParamInfo   m_TestRes;

   AEECallback    m_cb;

   IStatic *      m_pIStatic;
   IGraphics *    m_pIGraphics;

   AECHAR         m_szBuf[DST_MAX_STRLEN];
   AECHAR         m_szBuf2[DST_MAX_STRLEN];

   double         m_AvgArray[DST_AVG_VALUE]; // Array used to average results
   int            m_nAvgIndex;   // Current index into results array

   uint32         m_dwLargeBmpSize;

   uint16         m_wMsg;        // Status string to display
   uint16         m_wMsgExtra;   // Extra status string info
   boolean        m_bTestError;  // Test error has occurred -- abort.

   uint16         m_wDiv;

//   AECHAR         m_szTemp[DST_MAX_STRLEN];

#ifdef AEE_SIMULATOR
   PFNNOTIFY      m_pfnResult;
   PFNNOTIFY      m_pfnProgress;
#endif
};/*-------------------------------------------------------------------
            Global Constant Definitions
-------------------------------------------------------------------*/

/*-------------------------------------------------------------------
            Type Declarations
-------------------------------------------------------------------*/

/*-------------------------------------------------------------------
            Function Prototypes
-------------------------------------------------------------------*/
// Mandatory function:
// App Handle Event function
static boolean CDispSpeedTune_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);

// App specific data alloc-init/free functions
static boolean CDispSpeedTune_InitAppData(IApplet* po);
static void CDispSpeedTune_FreeAppData(IApplet* po);

// App Helper Functions
static void CDispSpeedTune_DrawTextStatus( CDispSpeedTune* pMe );
static void CDispSpeedTune_BuildUI( CDispSpeedTune* pMe );

static void CDispSpeedTune_StartBenchmarkTests(CDispSpeedTune * pMe);
static void CDispSpeedTune_StopBenchmarkTests(CDispSpeedTune * pMe);

static void CDispSpeedTune_TestHandlerCb1( void * pUser );
static void CDispSpeedTune_TestHandlerCb2( void * pUser );
static void CDispSpeedTune_TestHandlerCd( void * pUser );
static void CDispSpeedTune_TestHandlerCf1( void * pUser );
static void CDispSpeedTune_TestHandlerCf2( void * pUser );
static void CDispSpeedTune_TestHandlerCu( void * pUser );
static void CDispSpeedTune_TestHandlerIdle( void * pUser );
static void CDispSpeedTune_TestHandlerCFEnumInit( void * pUser );
static void CDispSpeedTune_TestHandlerCFEnumNext( void * pUser );
static void CDispSpeedTune_TestHandlerCFRead1( void * pUser );
static void CDispSpeedTune_TestHandlerCFRead2( void * pUser );
static void CDispSpeedTune_TestHandlerCFWrite1( void * pUser );
static void CDispSpeedTune_TestHandlerCFWrite2( void * pUser );
static void CDispSpeedTune_TestHandlerCFMkDir( void * pUser );
static void CDispSpeedTune_TestHandlerCFRmDir( void * pUser );

static void CDispSpeedTune_CallbackNotify(void * pUser);
static void CDispSpeedTune_ResetData( CDispSpeedTune* pMe );
static double CDispSpeedTune_ComputeAvg( CDispSpeedTune* pMe );
static void CDispSpeedTune_Deschedule( CDispSpeedTune* pMe );
static void CDispSpeedTune_UpdateProgress( CDispSpeedTune* pMe );
static void CDispSpeedTune_CalibrateResults( CDispSpeedTune* pMe );
static double CDispSpeedTune_Calibrate(CDispSpeedTune* pMe, double fRes, double fInit, uint32 dwValue);
static void CDispSpeedTune_SaveResults( CDispSpeedTune* pMe );

static void DrawHist(IDisplay * pd, AEERect * prc, uint16 wPct);
static double HACK_Float(const char * psz);
static double DW_To_F(uint32 dwVal);
static uint32 	RemoveDeepDir(CDispSpeedTune * pMe, const char *szPath);
static uint32 fs_MultiFileCreate(CDispSpeedTune * pMe);

/*-------------------------------------------------------------------
            Global Data Definitions
-------------------------------------------------------------------*/
#define CDispSpeedTune_TOTAL_PCT  (DST_AVG_VALUE * (uint16)DST_PARAM_LAST)
/*-------------------------------------------------------------------
            Static variable Definitions
-------------------------------------------------------------------*/

/*===============================================================================
                     FUNCTION DEFINITIONS
=============================================================================== */
/*===========================================================================

Function:  GraphicsBM_InitAppData()

Description:
		This function initializes the app specific data.

Prototype:
	void GraphicsBM_InitAppData(IApplet* po)

Parameters:
   IApplet* po: Pointer to the applet structure. This is of the type defined
	to store applet data. (CDispSpeedTune*, in this case).

Return Value:
	TRUE: If successful
	FALSE: IF failed

Comments:  None

Side Effects: None

==============================================================================*/
boolean CDispSpeedTune_InitAppData(IApplet* po)
{
	CDispSpeedTune *  pMe = (CDispSpeedTune*)po;
   AEEDeviceInfo     di;
   int               i;
   
   // Get screen pixel count
   ISHELL_GetDeviceInfo(pMe->a.m_pIShell, &di);
   pMe->m_cxWidth = di.cxScreen;
	pMe->m_cyHeight = di.cyScreen;
   pMe->m_nCharHeight = IDISPLAY_GetFontMetrics(pMe->a.m_pIDisplay, AEE_FONT_BOLD, NULL, NULL);

   for (i = 0; i < (int)DST_PARAM_LAST; i++)
      ISHELL_LoadResString(pMe->a.m_pIShell, DISPSPEEDTUNE_RES_FILE, (uint16)(i + IDS_TEST_BASE), pMe->m_TestName[i], sizeof(pMe->m_TestName[i]));

	//Create softkey menu
   if ( ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR, (void **)&pMe->m_pIFileMgr) )
      return FALSE;

   if (ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOFTKEYCTL, (void **)&pMe->m_pSKMenu))
	   return FALSE;

	if (ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_STATIC, (void **)&pMe->m_pIStatic))
	   return FALSE;

   if (ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_GRAPHICS, (void **)&pMe->m_pIGraphics))
      return FALSE;

   // Initialize test handlers
   // Display benchmark
   pMe->m_pfnTestHandler[(int)DST_PARAM_Cb1] = CDispSpeedTune_TestHandlerCb1;
   pMe->m_pfnTestHandler[(int)DST_PARAM_Cb2] = CDispSpeedTune_TestHandlerCb2;
   pMe->m_pfnTestHandler[(int)DST_PARAM_Cd] = CDispSpeedTune_TestHandlerCd;
   pMe->m_pfnTestHandler[(int)DST_PARAM_Cf1] = CDispSpeedTune_TestHandlerCf1;
   pMe->m_pfnTestHandler[(int)DST_PARAM_Cf2] = CDispSpeedTune_TestHandlerCf2;
   pMe->m_pfnTestHandler[(int)DST_PARAM_Cu] = CDispSpeedTune_TestHandlerCu;
   pMe->m_pfnTestHandler[(int)DST_PARAM_IDLE] = CDispSpeedTune_TestHandlerIdle;

   // File System benchmark
   pMe->m_pfnTestHandler[(int)DST_PARAM_CFEnumInitFile] = CDispSpeedTune_TestHandlerCFEnumInit;
   pMe->m_pfnTestHandler[(int)DST_PARAM_CFEnumInitDir] = CDispSpeedTune_TestHandlerCFEnumInit;
   pMe->m_pfnTestHandler[(int)DST_PARAM_CFEnumNextFile] = CDispSpeedTune_TestHandlerCFEnumNext;
   pMe->m_pfnTestHandler[(int)DST_PARAM_CFEnumNextDir] = CDispSpeedTune_TestHandlerCFEnumNext;
   pMe->m_pfnTestHandler[(int)DST_PARAM_CFRead1] = CDispSpeedTune_TestHandlerCFRead1;
   pMe->m_pfnTestHandler[(int)DST_PARAM_CFRead2] = CDispSpeedTune_TestHandlerCFRead2;
   pMe->m_pfnTestHandler[(int)DST_PARAM_CFWrite1] = CDispSpeedTune_TestHandlerCFWrite1;
   pMe->m_pfnTestHandler[(int)DST_PARAM_CFWrite2] = CDispSpeedTune_TestHandlerCFWrite2;
   pMe->m_pfnTestHandler[(int)DST_PARAM_CFMkDir] = CDispSpeedTune_TestHandlerCFMkDir;
   pMe->m_pfnTestHandler[(int)DST_PARAM_CFRmDir] = CDispSpeedTune_TestHandlerCFRmDir;

   CDispSpeedTune_ResetData(pMe);

#ifdef AEE_SIMULATOR
   pMe->m_pfnResult = NULL;
   pMe->m_pfnProgress = NULL;
#endif

   return TRUE;
}

static void CDispSpeedTune_FreeAppData(IApplet* po)
{	
   CDispSpeedTune *pMe = (CDispSpeedTune*)po;

//   RemoveDeepDir(pMe, DST_TEST_DIR);

   if (pMe->m_pIFileMgr)
      IFILEMGR_Release(pMe->m_pIFileMgr);

   if (pMe->m_pSKMenu)
      IMENUCTL_Release(pMe->m_pSKMenu);

   if (pMe->m_pIStatic)
      ISTATIC_Release(pMe->m_pIStatic);

   if (pMe->m_pIGraphics)
      IGRAPHICS_Release(pMe->m_pIGraphics);
}

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time.

PROTOTYPE:
	int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell interface. 

	pIModule: pin]: Contains pointer to the IModule interface to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppApplet must point to a valid IBase object. 
	If the classID	represnts an applet, then ppObj must point to a valid AEEApplet structure.Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the class/app creation was successful. 
  EFAILED: Error occurred while creating the class/app. In this case, the app/class will
  not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
//Mandatory Function
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
{
	*ppObj = NULL;

   if(ClsId == AEECLSID_DSTAPP)
	{
		//Create the applet
      if(AEEApplet_New(sizeof(CDispSpeedTune), ClsId, pIShell,po,(IApplet**)ppObj,(AEEHANDLER)CDispSpeedTune_HandleEvent,(PFNFREEAPPDATA)CDispSpeedTune_FreeAppData))
		{
			//Initialize applet data
         if(CDispSpeedTune_InitAppData((IApplet*)*ppObj))
			{
				//Data initialized successfully
				return(AEE_SUCCESS);
			}
			else
			{
				//Release the applet. This will free the memory allocated for the applet when
				// AEEApplet_New was called.
				IAPPLET_Release((IApplet*)*ppObj);
				return EFAILED;
			}

      }//AEEApplet_New

   }// ClsId == AEECLSID_DSTAPP

   return(EFAILED);
}

/*===========================================================================

FUNCTION GraphicsBM_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean GraphicsBM_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AppClsCreateInstance() function.

	ecode: Specifies the Event sent to this applet

   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/
//Mandatory Function
static boolean CDispSpeedTune_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{
   CDispSpeedTune * pMe = (CDispSpeedTune *)pi;

   switch ( eCode ) 
   {   
      case EVT_APP_START:   // Process Start event
         CDispSpeedTune_BuildUI(pMe);
         CDispSpeedTune_DrawTextStatus(pMe);
         return TRUE;  

      case EVT_APP_STOP:        // process STOP event
         IFILEMGR_Remove(pMe->m_pIFileMgr, DST_TEST_WRITE_FILE);
         RemoveDeepDir(pMe, DST_TEST_DIR);
         return (TRUE);

      case EVT_KEY:
      {
         switch ( (AVKType)wParam )
         {
#ifdef AEE_SIMULATOR
            case AVK_FUNCTION1:
               pMe->m_pfnResult = (PFNNOTIFY)dwParam;
               return TRUE;
               break;

            case AVK_FUNCTION2:
               pMe->m_pfnProgress = (PFNNOTIFY)dwParam;
               return TRUE;
               break;
#endif
#ifdef CDISPSPEEDTUNE_TESTING
               // To test read and write for various buffer sizes.
            case AVK_1: pMe->m_wDiv = 1; return TRUE;
            case AVK_2: pMe->m_wDiv = 2; return TRUE;
            case AVK_4: pMe->m_wDiv = 4; return TRUE;
            case AVK_8: pMe->m_wDiv = 8; return TRUE;
            case AVK_9: pMe->m_wDiv = 16; return TRUE;
#endif
            case AVK_UP:
            case AVK_DOWN:
               return ISTATIC_HandleEvent(pMe->m_pIStatic, EVT_KEY, wParam, dwParam);
               break;

            default:
               return IMENUCTL_HandleEvent(pMe->m_pSKMenu, EVT_KEY, wParam, dwParam);
               break;
         } // switch
      } // case

      case EVT_COMMAND:
      {
         switch ( (DSTOperation)wParam )
         {
            case DST_OP_START:
               if (pMe->m_eOp != DST_OP_START)
                  CDispSpeedTune_StartBenchmarkTests(pMe);
               return TRUE;
               break;

            case DST_OP_STOP:
               if (pMe->m_eOp == DST_OP_START)
               {
                  pMe->m_bTestError = TRUE;
                  if (!pMe->m_bDone)
                     pMe->m_wMsgExtra = IDS_ABORTED;
               }
               CDispSpeedTune_StopBenchmarkTests(pMe);
               return TRUE;
               break;

            default:
               pMe->m_bTestError = TRUE;
               pMe->m_wMsgExtra = IDS_UNKNOWN;
               CDispSpeedTune_StopBenchmarkTests(pMe);
               return TRUE;
               break;
         } // switch

      } // case
   } // switch (eCode)

   return FALSE;
}

/*===============================================================================
                     APP HELPER FUNCTION DEFINITIONS
=============================================================================== */
static void CDispSpeedTune_DrawTextStatus( CDispSpeedTune* pMe )
{
   AECHAR * psz = NULL;
   uint16   wStrSize = DST_MAX_STRLEN * sizeof(AECHAR);

   psz = MALLOC(wStrSize);
   if (!psz)
      return;

   IDISPLAY_ClearScreen(pMe->a.m_pIDisplay);

#ifdef CDISPSPEEDTUNE_TESTING
   {
      int i;

      psz[0] = 0;
      for (i = 0; i < (int)DST_PARAM_LAST; i++)
      {
         WSTRCAT(psz, pMe->m_TestName[i]);
         FLOAT_TO_WSTR(pMe->m_TestRes.fParams[i], pMe->m_szBuf, sizeof(pMe->m_szBuf));
         WSTRCAT(psz, pMe->m_szBuf);
         WSTRCAT(psz, (AECHAR *)"\n");
      }
   }
#endif

   ISHELL_LoadResString(pMe->a.m_pIShell, DISPSPEEDTUNE_RES_FILE, (uint16)IDS_APP_TITLE, pMe->m_szBuf, sizeof(pMe->m_szBuf));
   ISHELL_LoadResString(pMe->a.m_pIShell, DISPSPEEDTUNE_RES_FILE, pMe->m_wMsg, psz, wStrSize);
   if (pMe->m_wMsgExtra)
   {
      ISHELL_LoadResString(pMe->a.m_pIShell, DISPSPEEDTUNE_RES_FILE, pMe->m_wMsgExtra, pMe->m_szBuf2, sizeof(pMe->m_szBuf2));
      WSTRCAT(psz, pMe->m_szBuf2);
   }
   ISTATIC_SetText(pMe->m_pIStatic, pMe->m_szBuf, psz, AEE_FONT_BOLD, AEE_FONT_NORMAL);

   // Display the text control
   ISTATIC_Redraw(pMe->m_pIStatic);
   IMENUCTL_Redraw(pMe->m_pSKMenu);

   if (psz)
      FREE(psz);
}

static void CDispSpeedTune_BuildUI( CDispSpeedTune* pMe )
{
   AEERect  rect;

   IMENUCTL_AddItem(pMe->m_pSKMenu, DISPSPEEDTUNE_RES_FILE, IDS_START, (uint16)DST_OP_START, NULL, (uint32)pMe);
   IMENUCTL_AddItem(pMe->m_pSKMenu, DISPSPEEDTUNE_RES_FILE, IDS_STOP, (uint16)DST_OP_STOP, NULL, (uint32)pMe);
   IMENUCTL_GetRect(pMe->m_pSKMenu, &pMe->m_rectSKMenu);
   IMENUCTL_SetActive(pMe->m_pSKMenu, TRUE);
   IMENUCTL_Redraw(pMe->m_pSKMenu);

   SETAEERECT(&rect, 0, 0, pMe->m_cxWidth, pMe->m_cyHeight - pMe->m_rectSKMenu.dy);
   ISTATIC_SetRect(pMe->m_pIStatic, &rect);
   ISTATIC_SetProperties(pMe->m_pIStatic, ST_CENTERTITLE);
}

static void CDispSpeedTune_StartBenchmarkTests(CDispSpeedTune * pMe)
{
   CDispSpeedTune_ResetData(pMe);
   pMe->m_eOp = DST_OP_START;
   CDispSpeedTune_UpdateProgress(pMe);

   (pMe->m_pfnTestHandler[(int)pMe->m_eTest])(pMe);
}

static void CDispSpeedTune_StopBenchmarkTests(CDispSpeedTune * pMe)
{
   if (pMe->m_eOp == DST_OP_START)
   {
      pMe->m_eOp = DST_OP_STOP;
      pMe->m_wMsg = pMe->m_bTestError ? IDS_ERROR : IDS_DONE;
      // Extra msg is set by caller in pMe->m_wMsgExtra
   }

   if (pMe->m_bTestError || pMe->m_eOp != DST_OP_START)
   {
      CDispSpeedTune_DrawTextStatus(pMe);
      CDispSpeedTune_ResetData(pMe);
   }
   else
      CDispSpeedTune_DrawTextStatus(pMe);
}

static void CDispSpeedTune_TestHandlerIdle( void * pUser )
{
   CDispSpeedTune * pMe = (CDispSpeedTune *)pUser;
   uint32   dwStartTime = 0;
   uint32   dwCount = 0;
   uint32   dwDuration = DST_MAX_DURATION;
   uint32   dwPrev = 0, dwCur = 0;

   if (!pMe)
      return;

   // Compute time based on running the function for a fixed amout of time
   dwStartTime = ISHELL_GetUpTimeMS(pMe->a.m_pIShell);
   while ( (dwCur = ISHELL_GetUpTimeMS(pMe->a.m_pIShell) - dwStartTime) < dwDuration )
   {
      dwCount++;
      dwPrev = dwCur;
   }

   if (dwCount > 1)
      pMe->m_AvgArray[pMe->m_nAvgIndex] = FDIV(DW_To_F(dwPrev), DW_To_F(dwCount - 1));

   CDispSpeedTune_Deschedule(pMe);
}

static void CDispSpeedTune_TestHandlerCb1( void * pUser )
{
   CDispSpeedTune *  pMe = (CDispSpeedTune *)pUser;
   uint32            dwStartTime = 0;
   uint32            dwCount = 0;
   uint32            dwDuration = DST_MAX_DURATION;
   uint32            dwPrev = 0, dwCur = 0;
   IImage *          pIImage = NULL;

   if (!pMe)
      return;

   pIImage = ISHELL_LoadResImage(pMe->a.m_pIShell, DISPSPEEDTUNE_RES_FILE, IDB_01X01);
   if (!pIImage)
   {
      pMe->m_bTestError = TRUE;
      pMe->m_wMsgExtra = IDS_MEM_ERROR;
      CDispSpeedTune_StopBenchmarkTests(pMe);
      return;
   }

   // Compute time based on running the function for a fixed amout of time
   dwStartTime = ISHELL_GetUpTimeMS(pMe->a.m_pIShell);
   while ( (dwCur = ISHELL_GetUpTimeMS(pMe->a.m_pIShell) - dwStartTime) < dwDuration )
   {
      IIMAGE_Draw(pIImage, 0, 0);
      dwCount++;
      dwPrev = dwCur;
   }

   if (dwCount > 1)
      pMe->m_AvgArray[pMe->m_nAvgIndex] = FDIV(DW_To_F(dwPrev), DW_To_F(dwCount - 1));

   if (pIImage)
      IIMAGE_Release(pIImage);

   CDispSpeedTune_Deschedule(pMe);
}

static void CDispSpeedTune_TestHandlerCb2( void * pUser )
{
   CDispSpeedTune *  pMe = (CDispSpeedTune *)pUser;
   uint32            dwStartTime = 0;
   uint32            dwCount = 0;
   uint32            dwDuration = DST_MAX_DURATION;
   uint32            dwPrev = 0, dwCur = 0;
   IImage *          pIImage = NULL;
   AEEImageInfo      imgInfo;
   double            fRes = MAKE_FLOAT(0.0);

   if (!pMe) 
      return;

   pIImage = ISHELL_LoadResImage(pMe->a.m_pIShell, DISPSPEEDTUNE_RES_FILE, IDB_LARGE);
   if (!pIImage)
   {
      pMe->m_bTestError = TRUE;
      pMe->m_wMsgExtra = IDS_MEM_ERROR;
      CDispSpeedTune_StopBenchmarkTests(pMe);
      return;
   }

   IIMAGE_GetInfo(pIImage, &imgInfo);
   pMe->m_dwLargeBmpSize = (uint32) imgInfo.cx * imgInfo.cy;
   if (pMe->m_dwLargeBmpSize == 0)
   {
      pMe->m_bTestError = TRUE;
      pMe->m_wMsgExtra = IDS_CB2_ERROR;
      CDispSpeedTune_StopBenchmarkTests(pMe);
      return;
   }

   // Compute time based on running the function for a fixed amout of time
   dwStartTime = ISHELL_GetUpTimeMS(pMe->a.m_pIShell);
   while ( (dwCur = ISHELL_GetUpTimeMS(pMe->a.m_pIShell) - dwStartTime) < dwDuration )
   {
      IIMAGE_Draw(pIImage, 0, 0);
      dwCount++;
      dwPrev = dwCur;
   }

   if (dwCount > 1)
      fRes = FDIV( DW_To_F(dwPrev), DW_To_F(dwCount - 1) );
   pMe->m_AvgArray[pMe->m_nAvgIndex] = CDispSpeedTune_Calibrate(pMe, fRes, pMe->m_TestRes.fParams[(int)DST_PARAM_Cb1], pMe->m_dwLargeBmpSize);

   if (pIImage)
      IIMAGE_Release(pIImage);

   CDispSpeedTune_Deschedule(pMe);
}

static void CDispSpeedTune_TestHandlerCd( void * pUser )
{
   CDispSpeedTune * pMe = (CDispSpeedTune *)pUser;
   uint32   dwStartTime = 0;
   uint32   dwCount = 0;
   uint32   dwDuration = DST_MAX_DURATION;
   uint32   dwPrev = 0, dwCur = 0;
   AEEPoint p;

   if (!pMe)
      return;

   p.x = 10;
   p.y = 10;

   // Compute time based on running the function for a fixed amout of time
   dwStartTime = ISHELL_GetUpTimeMS(pMe->a.m_pIShell);
   while ( (dwCur = ISHELL_GetUpTimeMS(pMe->a.m_pIShell) - dwStartTime) < dwDuration )
   {
      IGRAPHICS_DrawPoint(pMe->m_pIGraphics, &p);
      dwCount++;
      dwPrev = dwCur;
   }

   if (dwCount > 1)
      pMe->m_AvgArray[pMe->m_nAvgIndex] = FDIV(DW_To_F(dwPrev), DW_To_F(dwCount - 1));

   CDispSpeedTune_Deschedule(pMe);
}

static void CDispSpeedTune_TestHandlerCf1( void * pUser )
{
   CDispSpeedTune * pMe = (CDispSpeedTune *)pUser;
   uint32   dwStartTime = 0;
   uint32   dwCount = 0;
   uint32   dwDuration = DST_MAX_DURATION;
   uint32   dwPrev = 0, dwCur = 0;
   AEERect  rect;

   if (!pMe)
      return;

   SETAEERECT(&rect, 0, 0, 1, 1);

   // Compute time based on running the function for a fixed amout of time
   dwStartTime = ISHELL_GetUpTimeMS(pMe->a.m_pIShell);
   while ( (dwCur = ISHELL_GetUpTimeMS(pMe->a.m_pIShell) - dwStartTime) < dwDuration )
   {
      IDISPLAY_FillRect(pMe->a.m_pIDisplay, &rect, MAKE_RGB(0, 192, 192));
      dwCount++;
      dwPrev = dwCur;
   }

   if (dwCount > 1)
      pMe->m_AvgArray[pMe->m_nAvgIndex] = FDIV(DW_To_F(dwPrev), DW_To_F(dwCount - 1));

   CDispSpeedTune_Deschedule(pMe);
}

static void CDispSpeedTune_TestHandlerCf2( void * pUser )
{
   CDispSpeedTune * pMe = (CDispSpeedTune *)pUser;
   uint32   dwStartTime = 0;
   uint32   dwCount = 0;
   uint32   dwDuration = DST_MAX_DURATION;
   uint32   dwPrev = 0, dwCur = 0;
   AEERect  rect;
   double   fRes = MAKE_FLOAT(0.0);

   if (!pMe)
      return;

   SETAEERECT(&rect, 0, 0, DST_LARGE_RECT_DX, DST_LARGE_RECT_DY);

   // Compute time based on running the function for a fixed amout of time
   dwStartTime = ISHELL_GetUpTimeMS(pMe->a.m_pIShell);
   while ( (dwCur = ISHELL_GetUpTimeMS(pMe->a.m_pIShell) - dwStartTime) < dwDuration )
   {
      IDISPLAY_FillRect(pMe->a.m_pIDisplay, &rect, MAKE_RGB(0, 192, 192));
      dwCount++;
      dwPrev = dwCur;
   }

   if (dwCount > 1)
      fRes = FDIV( DW_To_F(dwPrev), DW_To_F(dwCount - 1) );
   pMe->m_AvgArray[pMe->m_nAvgIndex] = CDispSpeedTune_Calibrate(pMe, fRes, pMe->m_TestRes.fParams[(int)DST_PARAM_Cf1], DST_LARGE_RECT_DX + DST_LARGE_RECT_DY);

   CDispSpeedTune_Deschedule(pMe);
}

static void CDispSpeedTune_TestHandlerCu( void * pUser )
{
   CDispSpeedTune * pMe = (CDispSpeedTune *)pUser;
   uint32   dwStartTime = 0;
   uint32   dwCount = 0;
   uint32   dwDuration = DST_MAX_DURATION;
   uint32   dwPrev = 0, dwCur = 0;

   if (!pMe)
      return;

   // Compute time based on running the function for a fixed amout of time
   dwStartTime = ISHELL_GetUpTimeMS(pMe->a.m_pIShell);
   while ( (dwCur = ISHELL_GetUpTimeMS(pMe->a.m_pIShell) - dwStartTime) < dwDuration )
   {
      IDISPLAY_UpdateEx(pMe->a.m_pIDisplay, FALSE);
      dwCount++;
      dwPrev = dwCur;
   }

   if (dwCount > 1)
      pMe->m_AvgArray[pMe->m_nAvgIndex] = FDIV(DW_To_F(dwPrev), DW_To_F(dwCount - 1));

   CDispSpeedTune_Deschedule(pMe);
}

static void CDispSpeedTune_TestHandlerCFEnumInit( void * pUser )
{
   CDispSpeedTune * pMe = (CDispSpeedTune *)pUser;
   uint32   dwStartTime = 0;
   uint32   dwCount = 0;
   uint32   dwDuration = DST_MAX_DURATION;
   uint32   dwPrev = 0, dwCur = 0;
   double   fRes = MAKE_FLOAT(0.0);
   boolean  bDirs;

   if (!pMe)
      return;

   // Do this only once
   if (pMe->m_eTest == DST_PARAM_CFEnumInitFile)
   {
      if (!fs_MultiFileCreate(pMe))
      {
         pMe->m_bTestError = TRUE;
         pMe->m_wMsgExtra = IDS_FCREATE_ERROR;
         CDispSpeedTune_StopBenchmarkTests(pMe);
         return;
      }
   }

   bDirs = pMe->m_eTest == DST_PARAM_CFEnumInitDir ? TRUE : FALSE;

   // Compute CFRead1
   dwStartTime = ISHELL_GetUpTimeMS(pMe->a.m_pIShell);
   while ( (dwCur = ISHELL_GetUpTimeMS(pMe->a.m_pIShell) - dwStartTime) < dwDuration )
   {
      IFILEMGR_EnumInit(pMe->m_pIFileMgr, DST_TEST_DIR, bDirs);
      dwCount++;
      dwPrev = dwCur;
   }

   if (dwCount > 1)   
      fRes = FDIV(DW_To_F(dwPrev), DW_To_F(dwCount - 1));
   pMe->m_AvgArray[pMe->m_nAvgIndex] = CDispSpeedTune_Calibrate(pMe, fRes, 0, 0);

   CDispSpeedTune_Deschedule(pMe);
}

static void CDispSpeedTune_TestHandlerCFEnumNext( void * pUser )
{
   CDispSpeedTune * pMe = (CDispSpeedTune *)pUser;
   uint32   dwStartTime = 0;
   uint32   dwCount = 0;
   uint32   dwDuration = DST_MAX_DURATION;
   uint32   dwPrev = 0, dwCur = 0;
   double   fRes = MAKE_FLOAT(0.0);
   FileInfo fi;
   int      nEnums = 0;
   double   fEnumDelay = 0;
   boolean  bDirs;

   if (!pMe)
      return;

   bDirs = pMe->m_eTest == DST_PARAM_CFEnumNextDir ? TRUE : FALSE;

   // Compute CFRead1
   IFILEMGR_EnumInit(pMe->m_pIFileMgr, DST_TEST_DIR, bDirs);
   dwStartTime = ISHELL_GetUpTimeMS(pMe->a.m_pIShell);
   while ( (dwCur = ISHELL_GetUpTimeMS(pMe->a.m_pIShell) - dwStartTime) < dwDuration )
   {
      if ( !IFILEMGR_EnumNext(pMe->m_pIFileMgr, &fi) )
      {
         nEnums++;
         IFILEMGR_EnumInit(pMe->m_pIFileMgr, DST_TEST_DIR, bDirs);
      }
      dwCount++;
      dwPrev = dwCur;
   }

   if (nEnums && dwCount > 1)
   {
      fEnumDelay = FMUL( pMe->m_TestRes.fParams[(int)pMe->m_eTest - 2], DW_To_F((uint32)nEnums) );
      fEnumDelay = FDIV( fEnumDelay, DW_To_F(dwCount - 1) );
      fRes = FSUB( FDIV(DW_To_F(dwPrev), DW_To_F(dwCount - 1)), fEnumDelay);
   }

   pMe->m_AvgArray[pMe->m_nAvgIndex] = CDispSpeedTune_Calibrate(pMe, fRes, 0, 0);

   CDispSpeedTune_Deschedule(pMe);
}

static void CDispSpeedTune_TestHandlerCFRead1( void * pUser )
{
   CDispSpeedTune * pMe = (CDispSpeedTune *)pUser;
   uint32   dwStartTime = 0;
   uint32   dwCount = 0;
   uint32   dwDuration = DST_MAX_DURATION;
   uint32   dwPrev = 0, dwCur = 0;
   IFile *     pIFile;
   uint8    buffer;
   uint32   dwBufSize = 1;
   double   fRes = MAKE_FLOAT(0.0);

   if (!pMe)
      return;

   pIFile = IFILEMGR_OpenFile(pMe->m_pIFileMgr, DST_TEST_READ_FILE, _OFM_READWRITE);
   if (!pIFile)
   {
      pMe->m_bTestError = TRUE;
      pMe->m_wMsgExtra = IDS_CFREAD_ERROR;
      CDispSpeedTune_StopBenchmarkTests(pMe);
      return;
   }

   // Compute CFRead1
   dwStartTime = ISHELL_GetUpTimeMS(pMe->a.m_pIShell);
   while ( (dwCur = ISHELL_GetUpTimeMS(pMe->a.m_pIShell) - dwStartTime) < dwDuration )
   {
      if (0 == IFILE_Read(pIFile, &buffer, dwBufSize))
      {
         IFILE_Seek(pIFile, _SEEK_START, 0);
         IFILE_Read(pIFile, &buffer, dwBufSize);
      }
      dwCount++;
      dwPrev = dwCur;
   }
   if (dwCount > 1)
      fRes = FDIV(DW_To_F(dwPrev), DW_To_F(dwCount - 1));
   pMe->m_AvgArray[pMe->m_nAvgIndex] = CDispSpeedTune_Calibrate(pMe, fRes, 0, 0);

   IFILE_Release(pIFile);

   CDispSpeedTune_Deschedule(pMe);
}

static void CDispSpeedTune_TestHandlerCFRead2( void * pUser )
{
   CDispSpeedTune * pMe = (CDispSpeedTune *)pUser;
   uint32   dwStartTime = 0;
   uint32   dwCount = 0;
   uint32   dwDuration = DST_MAX_DURATION;
   uint32   dwPrev = 0, dwCur = 0;
   IFile *     pIFile;
   uint8 *  pBuffer = NULL;
   uint32   dwBufSize = DST_FILE_BUF_SIZE/pMe->m_wDiv;
   int      nCount;
   FileInfo fi;
   double   fRes = MAKE_FLOAT(0.0);

   if (!pMe)
      return;

   IFILEMGR_GetInfo(pMe->m_pIFileMgr, DST_TEST_READ_FILE, &fi);
   nCount = fi.dwSize / dwBufSize;
   pBuffer = MALLOC(dwBufSize);
   if (!pBuffer)
   {
      pMe->m_bTestError = TRUE;
      pMe->m_wMsgExtra = IDS_MEM_ERROR;
      CDispSpeedTune_StopBenchmarkTests(pMe);
      return;
   }

   pIFile = IFILEMGR_OpenFile(pMe->m_pIFileMgr, DST_TEST_READ_FILE, _OFM_READWRITE);
   if (!pIFile)
   {
      pMe->m_bTestError = TRUE;
      pMe->m_wMsgExtra = IDS_CFREAD_ERROR;
      CDispSpeedTune_StopBenchmarkTests(pMe);
      return;
   }

   // Compute CFRead2
   dwStartTime = ISHELL_GetUpTimeMS(pMe->a.m_pIShell);
   while ( (dwCur = ISHELL_GetUpTimeMS(pMe->a.m_pIShell) - dwStartTime) < dwDuration )
   {
      if (0 == IFILE_Read(pIFile, pBuffer, dwBufSize))
      {
         IFILE_Seek(pIFile, _SEEK_START, 0);
         IFILE_Read(pIFile, pBuffer, dwBufSize);
      }
      dwCount++;
      dwPrev = dwCur;
   }

   if (dwCount > 1)
      fRes = FDIV(DW_To_F(dwPrev), DW_To_F(dwCount - 1));
   pMe->m_AvgArray[pMe->m_nAvgIndex] = CDispSpeedTune_Calibrate(pMe, fRes, pMe->m_TestRes.fParams[(int)DST_PARAM_CFRead1], dwBufSize);

   IFILE_Release(pIFile);
   if (pBuffer)
      FREE(pBuffer);

   CDispSpeedTune_Deschedule(pMe);
}

static void CDispSpeedTune_TestHandlerCFWrite1( void * pUser )
{
   CDispSpeedTune * pMe = (CDispSpeedTune *)pUser;
   uint32   dwStartTime = 0;
   uint32   dwCount = 0;
   uint32   dwDuration = DST_MAX_DURATION;
   uint32   dwPrev = 0, dwCur = 0;
   IFile *     pIFile;
   uint8    buffer;
   uint32   dwBufSize = 1;
   int      nCount = 1024;
   int      i;
   double   fRes = MAKE_FLOAT(0.0);

   if (!pMe)
      return;

   IFILEMGR_Remove(pMe->m_pIFileMgr, DST_TEST_WRITE_FILE);
   pIFile = IFILEMGR_OpenFile(pMe->m_pIFileMgr, DST_TEST_WRITE_FILE, _OFM_CREATE);
   if (!pIFile)
   {
      pMe->m_bTestError = TRUE;
      pMe->m_wMsgExtra = IDS_CFWRITE_ERROR;
      CDispSpeedTune_StopBenchmarkTests(pMe);
      return;
   }

   i = 0;
   dwStartTime = ISHELL_GetUpTimeMS(pMe->a.m_pIShell);
   while ( (dwCur = ISHELL_GetUpTimeMS(pMe->a.m_pIShell) - dwStartTime) < dwDuration )
   {
      IFILE_Write(pIFile, &buffer, dwBufSize);
      if (++i >= nCount)
      {
         i = 0;
         IFILE_Seek(pIFile, _SEEK_START, 0);
      }

      dwCount++;
      dwPrev = dwCur;
   }

   if (dwCount > 1)
      fRes = FDIV(DW_To_F(dwPrev), DW_To_F(dwCount - 1));
   pMe->m_AvgArray[pMe->m_nAvgIndex] = CDispSpeedTune_Calibrate(pMe, fRes, 0, 0);

   IFILE_Release(pIFile);

   CDispSpeedTune_Deschedule(pMe);
}

static void CDispSpeedTune_TestHandlerCFWrite2( void * pUser )
{
   CDispSpeedTune * pMe = (CDispSpeedTune *)pUser;
   uint32   dwStartTime = 0;
   uint32   dwCount = 0;
   uint32   dwDuration = DST_MAX_DURATION;
   uint32   dwPrev = 0, dwCur = 0;
   IFile *     pIFile;
   uint8 *  pBuffer = NULL;
   uint32   dwBufSize = DST_FILE_BUF_SIZE/pMe->m_wDiv;
   int      nCount = 8;
   int      i;
   double   fRes = MAKE_FLOAT(0.0);

   if (!pMe)
      return;

   pBuffer = MALLOC(dwBufSize);
   if (!pBuffer)
   {
      pMe->m_bTestError = TRUE;
      pMe->m_wMsgExtra = IDS_MEM_ERROR;
      CDispSpeedTune_StopBenchmarkTests(pMe);
      return;
   }

   IFILEMGR_Remove(pMe->m_pIFileMgr, DST_TEST_WRITE_FILE);
   pIFile = IFILEMGR_OpenFile(pMe->m_pIFileMgr, DST_TEST_WRITE_FILE, _OFM_CREATE);
   if (!pIFile)
   {
      pMe->m_bTestError = TRUE;
      pMe->m_wMsgExtra = IDS_CFWRITE_ERROR;
      CDispSpeedTune_StopBenchmarkTests(pMe);
      return;
   }

   // Compute CFWrite2
   i = 0;
   dwStartTime = ISHELL_GetUpTimeMS(pMe->a.m_pIShell);
   while ( (dwCur = ISHELL_GetUpTimeMS(pMe->a.m_pIShell) - dwStartTime) < dwDuration )
   {
      IFILE_Write(pIFile, pBuffer, dwBufSize);
      if (++i >= nCount)
      {
         i = 0;
         IFILE_Seek(pIFile, _SEEK_START, 0);
      }

      dwCount++;
      dwPrev = dwCur;
   }

   if (dwCount > 1)
      fRes = FDIV(DW_To_F(dwPrev), DW_To_F(dwCount - 1));
   pMe->m_AvgArray[pMe->m_nAvgIndex] = CDispSpeedTune_Calibrate(pMe, fRes, pMe->m_TestRes.fParams[(int)DST_PARAM_CFWrite1], dwBufSize);

   IFILE_Release(pIFile);
   if (pBuffer)
      FREE(pBuffer);

   CDispSpeedTune_Deschedule(pMe);
}

static void CDispSpeedTune_TestHandlerCFMkDir( void * pUser )
{
   CDispSpeedTune * pMe = (CDispSpeedTune *)pUser;
   uint32   dwStartTime = 0;
   uint32   dwCount = 0;
   uint32   dwDuration = DST_MAX_DURATION;
   uint32   dwPrev = 0, dwCur = 0;
   double   fRes = MAKE_FLOAT(0.0);
	char *   szDir = MALLOC(MAX_FILE_NAME);

   if (!pMe)
      return;

   if (!szDir)
   {
      pMe->m_bTestError = TRUE;
      pMe->m_wMsgExtra = IDS_MEM_ERROR;
      CDispSpeedTune_StopBenchmarkTests(pMe);
      return;
   }

   if (!RemoveDeepDir(pMe, DST_TEST_DIR))
   {
      pMe->m_bTestError = TRUE;
      pMe->m_wMsgExtra = IDS_FREMOVE_ERROR;
      CDispSpeedTune_StopBenchmarkTests(pMe);
      return;
   }

	IFILEMGR_MkDir(pMe->m_pIFileMgr, DST_TEST_DIR);

   // Compute CFMkDir
   dwStartTime = ISHELL_GetUpTimeMS(pMe->a.m_pIShell);
   while ( (dwCur = ISHELL_GetUpTimeMS(pMe->a.m_pIShell) - dwStartTime) < dwDuration )
   {
      SPRINTF(szDir,"%s\\%d.dir", DST_TEST_DIR, (int)dwCount);
      IFILEMGR_MkDir(pMe->m_pIFileMgr, szDir);
      dwCount++;
      dwPrev = dwCur;
      if (dwCount == DST_NUM_TEST_DIRS)
      {
         dwCount++; // Since it is decremented by one, later during computation
         ISHELL_Beep(pMe->a.m_pIShell, BEEP_ALERT, FALSE);
         break;
      }
   }

   if (dwCount > 1)
      fRes = FDIV(DW_To_F(dwPrev), DW_To_F(dwCount - 1));
   pMe->m_AvgArray[pMe->m_nAvgIndex] = CDispSpeedTune_Calibrate(pMe, fRes, 0, 0);

	FREE(szDir);

   CDispSpeedTune_Deschedule(pMe);
}

static void CDispSpeedTune_TestHandlerCFRmDir( void * pUser )
{
   CDispSpeedTune * pMe = (CDispSpeedTune *)pUser;
   uint32   dwStartTime = 0;
   uint32   dwCount = 0;
   uint32   dwDuration = DST_MAX_DURATION;
   uint32   dwPrev = 0, dwCur = 0;
   double   fRes = MAKE_FLOAT(0.0);
	char *   szDir = MALLOC(MAX_FILE_NAME);

   if (!pMe)
      return;

   if (!szDir)
   {
      pMe->m_bTestError = TRUE;
      pMe->m_wMsgExtra = IDS_MEM_ERROR;
      CDispSpeedTune_StopBenchmarkTests(pMe);
      return;
   }

   if (!fs_MultiFileCreate(pMe))
   {
      pMe->m_bTestError = TRUE;
      pMe->m_wMsgExtra = IDS_FCREATE_ERROR;
      CDispSpeedTune_StopBenchmarkTests(pMe);
      return;
   }

   // Compute CFRmDir
   dwStartTime = ISHELL_GetUpTimeMS(pMe->a.m_pIShell);
   while ( (dwCur = ISHELL_GetUpTimeMS(pMe->a.m_pIShell) - dwStartTime) < dwDuration )
   {
      SPRINTF(szDir,"%s\\%d.dir", DST_TEST_DIR, (int)dwCount);
      IFILEMGR_RmDir(pMe->m_pIFileMgr, szDir);
      dwCount++;
      dwPrev = dwCur;
      if (dwCount == DST_NUM_TEST_DIRS)
      {
         dwCount++; // Since it is decremented by one, later during computation
         ISHELL_Beep(pMe->a.m_pIShell, BEEP_ALERT, FALSE);
         break;
      }
   }

   if (dwCount > 1)
      fRes = FDIV(DW_To_F(dwPrev), DW_To_F(dwCount - 1));
   pMe->m_AvgArray[pMe->m_nAvgIndex] = CDispSpeedTune_Calibrate(pMe, fRes, 0, 0);

   FREE(szDir);

   CDispSpeedTune_Deschedule(pMe);
}

static void CDispSpeedTune_CallbackNotify(void * pUser)
{
   CDispSpeedTune * pMe = (CDispSpeedTune *)pUser;
   
   if (pMe->m_eOp != DST_OP_START)
   {
      pMe->m_bTestError = TRUE;
      pMe->m_wMsgExtra = IDS_ABORTED;
      CDispSpeedTune_StopBenchmarkTests(pMe);
      return;
   }

   if (++pMe->m_nAvgIndex >= DST_AVG_VALUE)
   {
      pMe->m_TestRes.fParams[(int)pMe->m_eTest] = CDispSpeedTune_ComputeAvg(pMe);
      pMe->m_nAvgIndex = 0;
      if ( ++pMe->m_eTest >= DST_PARAM_LAST )
      {
         pMe->m_bDone = TRUE;
         CDispSpeedTune_CalibrateResults(pMe);
         CDispSpeedTune_SaveResults(pMe);
#ifdef AEE_SIMULATOR
         if (pMe->m_pfnResult)
            pMe->m_pfnResult(pMe->m_TestRes.fParams);
#endif
         CDispSpeedTune_StopBenchmarkTests(pMe);
         return;
      }
   }

   (pMe->m_pfnTestHandler[(int)pMe->m_eTest])(pMe);
}

static void CDispSpeedTune_ResetData( CDispSpeedTune* pMe )
{
   int   i;
   double fZero;

   ISHELL_CancelTimer(pMe->a.m_pIShell, CDispSpeedTune_CallbackNotify, pMe);
   if (pMe->m_bTestError)
      IFILEMGR_Remove(pMe->m_pIFileMgr, DST_CONFIG_FILE);

   pMe->m_eOp = DST_OP_NONE;
   pMe->m_eTest = DST_PARAM_FIRST;
   pMe->m_wPercent = 0;
   pMe->m_bDone = FALSE;

   fZero = MAKE_FLOAT(0.0);
   for (i = 0; i < (int)DST_PARAM_LAST; i++)
   {
      pMe->m_TestRes.fParams[i] = fZero;
   }

   for (i = 0; i < DST_AVG_VALUE; i++)
      pMe->m_AvgArray[i] = fZero;

   pMe->m_dwLargeBmpSize = 1;
   pMe->m_wDiv = 1;

   pMe->m_nAvgIndex = 0;

   pMe->m_wMsg = IDS_START_MSG;
   pMe->m_wMsgExtra = 0;
   pMe->m_bTestError = FALSE;

//#ifdef AEE_SIMULATOR
//   pMe->m_pfnResult = NULL;
//   pMe->m_pfnProgress = NULL;
//#endif

}

static double CDispSpeedTune_ComputeAvg( CDispSpeedTune* pMe )
{
   double   d;
   double   fAvg;
   int      i;

   d  = fAvg = MAKE_FLOAT(0.0);

   for (i = 0; i < DST_AVG_VALUE; i++)
      d = FADD(d, pMe->m_AvgArray[i]);

   fAvg = FDIV(d, DW_To_F((uint32)DST_AVG_VALUE));

   return fAvg;
}

static void CDispSpeedTune_Deschedule( CDispSpeedTune* pMe )
{
#if 0
   pMe->m_cb.pfnNotify     = CDispSpeedTune_CallbackNotify;
   pMe->m_cb.pNotifyData   = pMe;

   // Signal AEE
   ISHELL_Resume(pMe->a.m_pIShell, &pMe->m_cb);
#endif
   pMe->m_wPercent++;
   CDispSpeedTune_UpdateProgress(pMe);

   ISHELL_SetTimer(pMe->a.m_pIShell, 1000, CDispSpeedTune_CallbackNotify, pMe);
}

static void CDispSpeedTune_UpdateProgress( CDispSpeedTune* pMe )
{
   AEERect  rect;
   AEERect  rcHist;
   uint16   wPct;

   ISTATIC_GetRect(pMe->m_pIStatic, &rect);
   IDISPLAY_EraseRect(pMe->a.m_pIDisplay, &rect);

   ISHELL_LoadResString(pMe->a.m_pIShell, DISPSPEEDTUNE_RES_FILE, (uint16)IDS_APP_TITLE, pMe->m_szBuf, sizeof(pMe->m_szBuf));
   ISHELL_LoadResString(pMe->a.m_pIShell, DISPSPEEDTUNE_RES_FILE, (uint16)IDS_PROG, pMe->m_szBuf2, sizeof(pMe->m_szBuf2));
   ISTATIC_SetText(pMe->m_pIStatic, pMe->m_szBuf, pMe->m_szBuf2, AEE_FONT_BOLD, AEE_FONT_NORMAL);
   ISTATIC_Redraw(pMe->m_pIStatic);

   SETAEERECT(&rcHist, 10, (rect.dy*2)/3, rect.dx - 20, 15);
   wPct = (uint16)(100 * pMe->m_wPercent)/CDispSpeedTune_TOTAL_PCT;
   DrawHist(pMe->a.m_pIDisplay, &rcHist, wPct);
#ifdef AEE_SIMULATOR
   if (pMe->m_pfnProgress)
      (pMe->m_pfnProgress)(&wPct);
#endif
   IDISPLAY_UpdateEx(pMe->a.m_pIDisplay, FALSE);
}

static void CDispSpeedTune_CalibrateResults( CDispSpeedTune* pMe )
{
   double   fc = pMe->m_TestRes.fParams[(int)DST_PARAM_IDLE];
   double   fZero = MAKE_FLOAT(0.0);
   double   ft;

   // Cb1:
   ft = FSUB(pMe->m_TestRes.fParams[(int)DST_PARAM_Cb1], fc);
   pMe->m_TestRes.fParams[(int)DST_PARAM_Cb1] = FCMP_L(ft, fZero) ? fZero : ft;

   //Cb2:
   ft = FSUB( pMe->m_TestRes.fParams[(int)DST_PARAM_Cb2], FDIV(fc, DW_To_F(pMe->m_dwLargeBmpSize)) );
   pMe->m_TestRes.fParams[(int)DST_PARAM_Cb2] = FCMP_L(ft, fZero) ? fZero : ft;

   // Cd:
   ft = FSUB(pMe->m_TestRes.fParams[(int)DST_PARAM_Cd], fc);
   pMe->m_TestRes.fParams[(int)DST_PARAM_Cd] = FCMP_L(ft, fZero) ? fZero : ft;

   // Cf1:
   ft = FSUB(pMe->m_TestRes.fParams[(int)DST_PARAM_Cf1], fc);
   pMe->m_TestRes.fParams[(int)DST_PARAM_Cf1] = FCMP_L(ft, fZero) ? fZero : ft;

   // Cf2:
   ft = FSUB( pMe->m_TestRes.fParams[(int)DST_PARAM_Cf2], FDIV(fc, DW_To_F((uint32)DST_LARGE_RECT_DX + DST_LARGE_RECT_DY)) );
   pMe->m_TestRes.fParams[(int)DST_PARAM_Cf2] = FCMP_L(ft, fZero) ? fZero : ft;

   // Cu:
   ft = FSUB(pMe->m_TestRes.fParams[(int)DST_PARAM_Cu], fc);
   pMe->m_TestRes.fParams[(int)DST_PARAM_Cu] = FCMP_L(ft, fZero) ? fZero : ft;
}

static void CDispSpeedTune_SaveResults( CDispSpeedTune* pMe )
{
   IFile *     pIFile;
   char *      pszFileStr;
   AECHAR      szCrLf[3];
   uint16      wStrSize = DST_MAX_STRLEN * sizeof(char);

   szCrLf[0] = '\r';
   szCrLf[1] = '\n';
   szCrLf[2] = 0;

   pszFileStr = MALLOC(wStrSize);
   if (!pszFileStr)
      return;

   IFILEMGR_Remove(pMe->m_pIFileMgr, DST_CONFIG_FILE);
   pIFile = IFILEMGR_OpenFile(pMe->m_pIFileMgr, DST_CONFIG_FILE, _OFM_CREATE);

   if (pIFile)
   {
      int      i;
      boolean  bErr = FALSE;

      for (i = 0; i < (int)DST_PARAM_LAST && !bErr; i++)
      {
         pMe->m_szBuf2[0] = 0;
         WSTRCAT(pMe->m_szBuf2, pMe->m_TestName[i]);
         FLOAT_TO_WSTR(pMe->m_TestRes.fParams[i], pMe->m_szBuf, sizeof(pMe->m_szBuf));
         WSTRCAT(pMe->m_szBuf2, pMe->m_szBuf);
         WSTRCAT(pMe->m_szBuf2, szCrLf);
         WSTR_TO_STR(pMe->m_szBuf2, pszFileStr, wStrSize);

         bErr = !IFILE_Write(pIFile, pszFileStr, STRLEN(pszFileStr) );
      }

      IFILE_Release(pIFile);
   }

   FREE(pszFileStr);
}

static double CDispSpeedTune_Calibrate(CDispSpeedTune* pMe, double fRes, double fInit, uint32 dwValue)
{
   double   fIdle = pMe->m_TestRes.fParams[(int)DST_PARAM_IDLE];
   double   fZero = MAKE_FLOAT(0.0);
   double   ft;

   ft = FSUB(fRes, fInit);
   if (dwValue)
   {
      ft = FDIV(ft, DW_To_F(dwValue));
      ft = FSUB( ft, FDIV(fIdle, DW_To_F(dwValue)) );
   }
   else
      ft = FSUB( ft, fIdle );

   return FCMP_L(ft, fZero) ? fZero : ft;
}

static void DrawHist(IDisplay * pd, AEERect * prc, uint16 wPct)
{
   AEERect  rc = *prc;

   if(wPct > 100)
      wPct = 100;

   IDISPLAY_DrawFrame(pd, &rc, AEE_FT_INDENT, CLR_SYS_SCROLLBAR);
   rc.dx = (rc.dx * wPct) / 100;
   IDISPLAY_FillRect(pd, &rc, CLR_SYS_SCROLLBAR_FILL);
}

static double HACK_Float(const char * psz)
{
   AECHAR   sz[20];

   STR_TO_WSTR((char *)psz,sz,sizeof(sz));
   return(WSTR_TO_FLOAT(sz));
}

static double DW_To_F(uint32 dwVal)
{
   AECHAR   sz[20];
   char     s[20];

   SPRINTF(s, "%lu", dwVal);
   STR_TO_WSTR((char *)s,sz,sizeof(sz));
   return(WSTR_TO_FLOAT(sz));
}

/****************************************************************************
* FUNCTION:      RemoveDeepDir
*
* DESCRIPTION:   This function removes a directory after
* deleting all the files in the directory.
*
* PARAMETERS:
*    Brewstone  *pMe  - Pointer to the applet instance.
*    szPath           - Name of directory to be delete
*
* RETURN VALUE:  None
*
******************************************************************************/
static uint32 RemoveDeepDir(CDispSpeedTune * pMe, const char *szPath)
{
   FileInfo fi;
   uint32 start,stop;

   if(!szPath)
      return 0;

   start = ISHELL_GetTimeMS(pMe->a.m_pIShell);

   //Directory not found
   if(IFILEMGR_Test(pMe->m_pIFileMgr,szPath))
      return 0;

   IFILEMGR_EnumInit(pMe->m_pIFileMgr,szPath, FALSE);
   while(IFILEMGR_EnumNext(pMe->m_pIFileMgr, &fi))
   {
      IFILEMGR_Remove(pMe->m_pIFileMgr,fi.szName);
   }

   IFILEMGR_EnumInit(pMe->m_pIFileMgr,szPath, TRUE);
   while(IFILEMGR_EnumNext(pMe->m_pIFileMgr, &fi))
   {
      IFILEMGR_RmDir(pMe->m_pIFileMgr,fi.szName);
   }
   
   //Delete directory
   IFILEMGR_RmDir(pMe->m_pIFileMgr,szPath);

   stop = ISHELL_GetTimeMS(pMe->a.m_pIShell);

   return(stop - start);
}

/****************************************************************************
* FUNCTION:      fs_MultiFileCreate
*
* DESCRIPTION:   This benchmark test creates multiple files in the app's directory. 
* It creates a sub-directory within the app's directory and popuates it
* with 20 files. This test must be called first
*
* PARAMETERS:
*    Brewstone  *pMe  - Pointer to the applet instance.
*
* RETURN VALUE:  None
*
******************************************************************************/
static uint32 fs_MultiFileCreate(CDispSpeedTune * pMe)
{

	char *szBuf;
	char *szFile = MALLOC(MAX_FILE_NAME);
	int i;
	IFile * pIfile;
	uint32	dwStart,dwStop;

	if(!szFile)
		return(0);

	//Remove the directory
	RemoveDeepDir(pMe, DST_TEST_DIR);

	//Create the directory
	if(IFILEMGR_MkDir(pMe->m_pIFileMgr,DST_TEST_DIR))
		return 0;

	szBuf = MALLOC(40);
	if(!szBuf)
		return(0);

   STRCPY(szBuf,"filetest__");

   dwStart = ISHELL_GetTimeMS(pMe->a.m_pIShell);

	//Add a  bunch of files to this directory
	for(i = 0; i < DST_NUM_TEST_FILES; i++)
	{
		MEMSET(szFile,0,sizeof(szFile));

      //Create a unique file name
      SPRINTF(szFile,"%s\\%d.dat",DST_TEST_DIR,i);
      pIfile = IFILEMGR_OpenFile(pMe->m_pIFileMgr, szFile, _OFM_CREATE);
		if(pIfile)
		{
			IFILE_Write(pIfile, szBuf, STRLEN(szBuf));
			IFILE_Release(pIfile);
		}

      //Create a unique dir name
      SPRINTF(szFile,"%s\\%d.dir",DST_TEST_DIR,i);
	   IFILEMGR_MkDir(pMe->m_pIFileMgr,szFile);

	}
	dwStop = ISHELL_GetTimeMS(pMe->a.m_pIShell);

	FREE(szBuf);
	FREE(szFile);

	return(dwStop - dwStart);
}


/*========================End of File ================================================*/

