/*===========================================================================

FILE: iMemStreamusage.c

SERVICES: Sample applet using AEE

DESCRIPTION
  This file contains usage examples of IMemStream interface. IMemStream interface
  functions allow the user to create/operate on IMemStream objects. IMemStream 
  interface offers basic sound services.

PUBLIC CLASSES:  
   N/A


   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/

/*===============================================================================
                     INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "AEEModGen.h"
#include "AEEAppGen.h"
#include "AEEShell.h"
#include "AEEStdLib.h"
#include "imemstream.bid"
#include "imemstream_res.h"
#include "AEEImage.h"
#include "AEEFile.h"

/*===========================================================================

                      PUBLIC DATA DECLARATIONS

===========================================================================*/

/*-------------------------------------------------------------------
            Type Declarations
-------------------------------------------------------------------*/
// MemStream app struct. This is the main struct for this applet. This will hold 
// all the data members that needs to be remembered throughout the life of
// the applet. 
// THE FIRST DATA MEMBER OF THIS STRUCT MUST BE AN AEEApplet OBJECT. 
// This is critical requirement which is related to an applet being
// dynamically loaded. The app developers should add their data members
// following the AEEApplet data member.
typedef struct _CIMemStreamApp
{
	AEEApplet	a;        // Mandatory first AEEApplet data member
   boolean     bOn;
   boolean     bgColor;

   // Device parameters
   int             m_nLineHeight;
   AEEDeviceInfo   m_dInfo;

}CIMemStreamApp;


/*-------------------------------------------------------------------
            Function Prototypes
-------------------------------------------------------------------*/
// App Handle Event function
static boolean MemStreamApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);

// App specific data alloc-init/free functions
static boolean IMemStream_InitAppData(IApplet* pMe);
static void IMemStream_FreeAppData(IApplet* pMe);

static void IMemStreamUsage (CIMemStreamApp * pMe, uint16 wParam);

/*-------------------------------------------------------------------
            Global Constant Definitions
-------------------------------------------------------------------*/
// App Resource File
#define  APP_RES_FILE      "imemstream.bar"

// App specific constants
#define USAGE_BASIC_USAGE      100


/*===============================================================================
                     FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to IMemStream_InitAppData(). 
   This is done as follows: InitAppData() is called to initialize AppletData 
   instance. It is app developers responsibility to fill-in app data initialization 
   code of InitAppData(). App developer is also responsible to release memory 
   allocated for data contained in AppletData -- this can be done in 
   IMemStream_FreeAppData().

PROTOTYPE:
   int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell object. 

	pIModule: pin]: Contains pointer to the IModule object to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid IApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the app needs to be loaded and if AEEApplet_New() invocation was
     successful
  EFAILED: If the app does not need to be loaded or if errors occurred in 
     AEEApplet_New(). If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
{
   *ppObj = NULL;
		
   // We want to load this App. So, invoke AEEApplet_New().To it, pass the
   // address of the app-specific handle event function.
   if(ClsId == AEECLSID_MEMSTREAM_APP){
      if(AEEApplet_New(sizeof(CIMemStreamApp), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)MemStreamApp_HandleEvent,(PFNFREEAPPDATA)IMemStream_FreeAppData)
         == TRUE)
      {
         if (IMemStream_InitAppData((IApplet*)*ppObj) == TRUE)
         {
            return(AEE_SUCCESS);
         }
      }
   }
	return (EFAILED);
}

/*===========================================================================

FUNCTION MemStreamApp_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean MemStreamApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

	ecode: Specifies the Event sent to this applet

   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/
static boolean MemStreamApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
   CIMemStreamApp * pMe = (CIMemStreamApp*)pi;

   switch (eCode) 
	{
       case EVT_APP_START:
		   IMemStreamUsage (pMe, wParam);

			return(TRUE);
      case EVT_APP_STOP:
         return(TRUE);

      default:
         break;
   }
   return FALSE;
}

/*===========================================================================

FUNCTION IMemStream_InitAppData

DESCRIPTION
   This function initializes app specific data allocates memory for app data 
   (AppletData) and sets it to pAppData of AEEApplet. It also function . 
   App developer needs to fill-in the initialization code.

PROTOTYPE:
	boolean IMemStream_InitAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains 
   information specific to this applet. It was initialized during the 
   AEEClsCreateInstance().

DEPENDENCIES
   Assumes pi is not NULL

RETURN VALUE
  TRUE: If the app has app data is allocated and initialized successfully
  FALSE: Either app data could not be allocated or initialzied

SIDE EFFECTS
  none
===========================================================================*/
static boolean IMemStream_InitAppData(IApplet* pi)
{
   CIMemStreamApp * pMe = (CIMemStreamApp*)pi;

   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&pMe->m_dInfo);

   return TRUE;
}

/*===========================================================================

FUNCTION IMemStream_FreeAppData

DESCRIPTION
   Frees data contained in app data and memory for app data itself.
   App developer needs to free data contained in AppletData.

PROTOTYPE:
	void IMemStream_FreeAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains 
   information specific to this applet. It was initialized during the 
   AEEClsCreateInstance().

DEPENDENCIES
   Assumes pi is not NULL

RETURN VALUE
   None

SIDE EFFECTS
   None
===========================================================================*/
static void IMemStream_FreeAppData(IApplet* pi)
{
   

}

/*===============================================================================
                     APP HELPER FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: IMemStreamUsage

DESCRIPTION
   
PROTOTYPE:
   void IMemStreamUsage (CIMemStreamApp * pMe, uint16 wParam)

PARAMETERS:

DEPENDENCIES
  none

RETURN VALUE
  none

SIDE EFFECTS
  none
===========================================================================*/
static void IMemStreamUsage (CIMemStreamApp * pMe, uint16 wParam)
{
  
   FileInfo fi;
   IFile * pIFile;
   IFileMgr * pIFileMgr = NULL;
   byte *      pBuff;
   IImage *pIImage;
   AEECLSID cls;
   AEEImageInfo aii;
   uint16 imgx = 0, imgy = 0;
   AEEDeviceInfo di; // Device Info
   IMemAStream *pMemStream = NULL;



   // Get screen size information for display purposes.
   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);

   if (0 == (cls = ISHELL_GetHandler(pMe->a.m_pIShell, HTYPE_VIEWER, "image/bmp")))
				return;

   ISHELL_CreateInstance(pMe->a.m_pIShell, cls, (void **)(&pIImage));
   if(NULL == pIImage)
		return;

			// Create an instance of IFileMgr so we can access the .bmp file
   ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR, (void **)(&pIFileMgr));
   if (NULL == pIFileMgr)
	{
		IIMAGE_Release(pIImage);
		return;
    }

			// Open the browser icon bitmap file for reading
   if (NULL == (pIFile = IFILEMGR_OpenFile(pIFileMgr, "BrowserIcon.bmp", _OFM_READ)))
	{
		IIMAGE_Release(pIImage);
		IFILEMGR_Release(pIFileMgr);
		return;
     }

	// Determine the size of the bitmap file and allocate memory
	// to store its contents
	IFILE_GetInfo(pIFile, &fi);
	if (NULL == (pBuff = (byte *) MALLOC(fi.dwSize)))
	{
		IIMAGE_Release(pIImage);
		IFILE_Release(pIFile);
		IFILEMGR_Release(pIFileMgr);
		return;
	}

	// Read the contents of the .bmp file into the
	// area allocated in memory for it
	if (IFILE_Read(pIFile, (void *)pBuff, fi.dwSize) == 0)
	{
		IIMAGE_Release(pIImage);
		FREE(pBuff);
		IFILE_Release(pIFile);
		IFILEMGR_Release(pIFileMgr);
		return;
	}	
	
	ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MEMASTREAM, (void **)(&pMemStream));
	if (NULL == pMemStream)
	{
		IIMAGE_Release(pIImage);
		FREE(pBuff);
		IFILE_Release(pIFile);
		IFILEMGR_Release(pIFileMgr);
      return;
	}


	IMEMASTREAM_Set(pMemStream,pBuff, fi.dwSize, 0, 0);
	// Use SetData to associate the .bmp file read into memory
	// with the IImage instance we've created
	// (NOTE: the third parameter to SetData indicates if the .bmp
	// file has MIME header info, FALSE in this case)
	IIMAGE_SetStream(pIImage, (IAStream*)pMemStream);

	{
		// Retrieve bitmap information, including its pixel dimensions
		// (these will be stored in aii.cx and aii.cy)
		IIMAGE_GetInfo(pIImage, &aii);

		// Calculate coordinates for bitmap -- 
		// we'll display it centered on the screen
		imgx = (di.cxScreen - aii.cx) / 2;
		imgy = (di.cyScreen - aii.cy) / 2;

		// Now draw the image
		IIMAGE_Draw(pIImage,imgx,imgy);
	}
	IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);

	// Don't need the bitmap any more, so release the interface 
	// (this will also free the data allocated by MALLOC).
	IIMAGE_Release (pIImage);
	IFILE_Release(pIFile);
	IFILEMGR_Release(pIFileMgr);
 
   //We should not explicitly free the buffer. This will automatically be done
   //when the MemAStream object is released.
	//FREE(pBuff);
   IMEMASTREAM_Release(pMemStream);
    
}