/*===========================================================================

FILE: idatectlusage.c

SERVICES: Sample applet using AEE

DESCRIPTION
  This file contains usage examples of IDATECTL interface. IDATECTL interface
  functions allow the user to create/operate on IDateCtl objects. IDateCtl
  objects are used to invoke Calender date related functionality.

  The date control interface has two modes. One is the DATE mode and the 
  other is the DATE-PICK mode. In the date mode the date object contains 
  the day of week, month, day, and year.

  In the date-pick mode a calendar view is present.

PUBLIC CLASSES:  
   N/A


   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/

/*===============================================================================
                     INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "AEEModGen.h"
#include "AEEAppGen.h"
#include "AEEMenu.h"
#include "AEEStdLib.h"
#include "AEEUsageAppIDs.h"
#include "AEEDate.h"

/*===========================================================================

                      PUBLIC DATA DECLARATIONS

===========================================================================*/

/*-------------------------------------------------------------------
            Type Declarations
-------------------------------------------------------------------*/
// DateCtl struct. This is the main struct for this applet. This will hold 
// all the data members that needs to be remembered throughout the life of
// the applet. 
// THE FIRST DATA MEMBER OF THIS STRUCT MUST BE AN AEEApplet OBJECT. 
// This is critical requirement which is related to an applet being
// dynamically loaded. The app developers should add their data members
// following the AEEApplet data member.
typedef struct _CIDateCtlApp
{
	AEEApplet	a;        // Mandatory first AEEApplet data member
   IMenuCtl *  m_pIMenu;
   IDateCtl *  m_pIDateCtl;
   IDateCtl *  m_pIDatePickCtl;

   // Device parameters
   int             m_nLineHeight;
   AEEDeviceInfo   m_dInfo;

}CIDateCtlApp;

/*-------------------------------------------------------------------
            Function Prototypes
-------------------------------------------------------------------*/
// Mandatory function:
// App Handle Event function
static boolean DateCtlApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);

// App specific data alloc-init/free functions
static boolean IDateCtl_InitAppData(IApplet* pMe);
static void IDateCtl_FreeAppData(IApplet* pMe);

static void BuildMainMenu(CIDateCtlApp *pMe);
static void IDateCtlUsage (CIDateCtlApp * pMe, uint16 wParam);
static void DisplayEvent (CIDateCtlApp *pMe, uint16 wParam);
static void DisplayOutput(CIDateCtlApp * pMe, int nline, char *pszStr);

/*-------------------------------------------------------------------
            Global Constant Definitions
-------------------------------------------------------------------*/

// App specific constants
#define USAGE_BASIC_USAGE_DATE       100
#define USAGE_BASIC_USAGE_DATEPICK   101
#define USAGE_HANDLEEVENT_DATE       102
#define USAGE_HANDLEEVENT_DATEPICK   103
#define USAGE_REDRAW                 104
#define USAGE_SETACTIVE_DATE         105
#define USAGE_SETACTIVE_DATEPICK     106
#define USAGE_ISACTIVE               107
#define USAGE_SETRECT                108
#define USAGE_GETRECT                109
#define USAGE_SETPROPERTIES          110
#define USAGE_GETPROPERTIES          111
#define USAGE_RESET                  112
#define USAGE_SETTITLE               113
#define USAGE_SETJULIANDAY           114
#define USAGE_GETJULIANDAY           115
#define USAGE_SETDATE                116
#define USAGE_GETDATE                117
#define USAGE_GETDAYOFWEEK           118
#define USAGE_GETDAYSTRING           119
#define USAGE_GETMONTHSTRING         120
#define USAGE_GETDATESTRING          121
#define USAGE_ENABLECOMMAND          122
#define USAGE_SETACTIVEDAYMASK       123

/*===============================================================================
                     FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to IDateCtl_InitAppData(). 
   This is done as follows: InitAppData() is called to initialize AppletData 
   instance. It is app developers responsibility to fill-in app data initialization 
   code of InitAppData(). App developer is also responsible to release memory 
   allocated for data contained in AppletData -- this can be done in 
   IDateCtl_FreeAppData().

PROTOTYPE:
   int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell interface. 

	pIModule: pin]: Contains pointer to the IModule interface to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid IApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the app needs to be loaded and if AEEApplet_New() invocation was
     successful
  EFAILED: If the app does not need to be loaded or if errors occurred in 
     AEEApplet_New(). If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
{
   *ppObj = NULL;
		
   // We want to load this App. So, invoke AEEApplet_New().To it, pass the
   // address of the app-specific handle event function.
   if(ClsId == AEECLSID_DATECTL_APP){
      if(AEEApplet_New(sizeof(CIDateCtlApp), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)DateCtlApp_HandleEvent,(PFNFREEAPPDATA)IDateCtl_FreeAppData)
         == TRUE)
      {
         if (IDateCtl_InitAppData((IApplet*)*ppObj) == TRUE)
         {
            return(AEE_SUCCESS);
         }
      }
   }
	return (EFAILED);
}

/*===========================================================================

FUNCTION DateCtlApp_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean App_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

	ecode: Specifies the Event sent to this applet

   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/

static boolean DateCtlApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
   CIDateCtlApp * pMe = (CIDateCtlApp*)pi;

   switch (eCode) 
	{
      case EVT_APP_START:
         // Create the IMenu interface object. When the applet is started the 
         // main menu is set up. When the user selects a menu item from the main 
         // menu, an EVT_COMMAND event is received by this applet. The processing for 
         // EVT_COMMAND event is found below.
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, (void **)&pMe->m_pIMenu)
            != SUCCESS)
         {
				return FALSE;
         }

         // Build the main menu
			BuildMainMenu(pMe);
			return(TRUE);
      case EVT_APP_STOP:
         return(TRUE);
      case EVT_KEY: 
         // Call the DateCtl HandleEvent to see if this event would be 
         // handled by the DateCtl object. When the Up or Down key is 
         // pressed we will fail the check below, release the IDateCtl object
         // and pass the controls to the menu control object.
         // First, call handler of the regular date control object
         if (pMe->m_pIDateCtl != NULL
            && IDATECTL_HandleEvent(pMe->m_pIDateCtl, EVT_KEY, wParam, 0))
         {
            // This event was handled by DateCtl. Return here since
            // this was handled by the time control object.
            return TRUE;
         }

         // Call the handler of the Date-Pick object
         if (pMe->m_pIDatePickCtl != NULL
            && IDATECTL_HandleEvent(pMe->m_pIDatePickCtl, EVT_KEY, wParam, 0))
         {
            // This event was handled by Date-Pick control. Return here since
            // this was handled by the date-pick control object.
            return TRUE;
         }

         // Release the IDateCtl object if it is not NULL.
         if (pMe->m_pIDateCtl != NULL)
         {
            IDATECTL_Release (pMe->m_pIDateCtl);
            pMe->m_pIDateCtl = NULL;
         }

         // Release the Count-Down object if it is not NULL.
         if (pMe->m_pIDatePickCtl != NULL)
         {
            IDATECTL_Release (pMe->m_pIDatePickCtl);
            pMe->m_pIDatePickCtl = NULL;
         }

         // We come here when a key is pressed on the handset keypad. Call 
         // the IMenuCtl handle event function to handle this key press.
         // If this key press corressponds to an Up/Down arrow 
         // key press, this event will be handled by the IMenuCtl object
         // in moving to the next menu item (in the given direction). 
			if(pMe->m_pIMenu != NULL && IMENUCTL_HandleEvent(pMe->m_pIMenu, EVT_KEY, wParam, 0))
				return TRUE;
         else
         {
            // We would have come here because the pressed key was not handled 
            // by either the DateCtl or the MenuCtl HandleEvent functions (the 
            // keys handled by the above controls are Up/Down/Select/Left/Right).
            //
            // Hence, we have reached here since 0-9, *, #, or CLR keys were
            // pressed. If the key is "CLR" then redraw the main applet menu.
				// Or just return TRUE and ignore the key press
            if (wParam == AVK_CLR)
            {
					IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);
					IMENUCTL_Redraw (pMe->m_pIMenu);
            }
            return TRUE;
         }
		case EVT_COMMAND:
         // When a menu item is selected this applet receives the
         // EVT_COMMAND event in the wParam field.
  		   switch(wParam)
         {
            case USAGE_BASIC_USAGE_DATE:
            case USAGE_BASIC_USAGE_DATEPICK:
            case USAGE_HANDLEEVENT_DATE:
            case USAGE_HANDLEEVENT_DATEPICK:
            case USAGE_REDRAW:
            case USAGE_SETACTIVE_DATE:
            case USAGE_SETACTIVE_DATEPICK:
            case USAGE_ISACTIVE:
            case USAGE_SETRECT:
            case USAGE_GETRECT:
            case USAGE_SETPROPERTIES:
            case USAGE_GETPROPERTIES:
            case USAGE_RESET:
            case USAGE_SETTITLE:
            case USAGE_SETJULIANDAY:
            case USAGE_GETJULIANDAY:
            case USAGE_SETDATE:
            case USAGE_GETDATE:
            case USAGE_GETDAYOFWEEK:
            case USAGE_GETDAYSTRING:
            case USAGE_GETMONTHSTRING:
            case USAGE_GETDATESTRING:
            case USAGE_ENABLECOMMAND:
            case USAGE_SETACTIVEDAYMASK:
               // Call the usage function that exercises all the 
               // interface functions.
               IDateCtlUsage (pMe, wParam);
               return TRUE;
		   	default:
			   	return FALSE;
         }
      default:
         break;
   }
   return FALSE;
}



/*===========================================================================

FUNCTION IDateCtl_InitAppData

DESCRIPTION
   This function initializes app specific data allocates memory for app data 
   (AppletData) and sets it to pAppData of AEEApplet. It also function . 
   App developer needs to fill-in the initialization code.

PROTOTYPE:
	boolean IDateCtl_InitAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains 
   information specific to this applet. It was initialized during the 
   AEEClsCreateInstance().

DEPENDENCIES
   Assumes pi is not NULL

RETURN VALUE
  TRUE: If the app has app data is allocated and initialized successfully
  FALSE: Either app data could not be allocated or initialzied

SIDE EFFECTS
  none
===========================================================================*/
static boolean IDateCtl_InitAppData(IApplet* pi)
{
   int pnAscent;        // Stores the ascent in number of pixels
   int pnDescent;       // Stores the descent in number of pixels

   CIDateCtlApp * pMe = (CIDateCtlApp*)pi;

   // Initialize the MenuCtl pointer to NULL
   pMe->m_pIMenu = NULL;
   pMe->m_pIDateCtl = NULL;
   pMe->m_pIDatePickCtl = NULL;

   // Get the font metrics info
   pMe->m_nLineHeight = IDISPLAY_GetFontMetrics (pMe->a.m_pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&pMe->m_dInfo);

   return TRUE;
}

/*===========================================================================

FUNCTION IDateCtl_FreeAppData

DESCRIPTION
   Frees data contained in app data and memory for app data itself.
   App developer needs to free data contained in AppletData.

PROTOTYPE:
	void IDateCtl_FreeAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains 
   information specific to this applet. It was initialized during the 
   AEEClsCreateInstance().

DEPENDENCIES
   Assumes pi is not NULL

RETURN VALUE
   None

SIDE EFFECTS
   None
===========================================================================*/
static void IDateCtl_FreeAppData(IApplet* pi)
{
   CIDateCtlApp * pMe = (CIDateCtlApp*)pi;

   if (pMe->m_pIMenu)
   {
      IMENUCTL_Release (pMe->m_pIMenu);
      pMe->m_pIMenu = NULL;
   }

   if (pMe->m_pIDateCtl)
   {
      IDATECTL_Release (pMe->m_pIDateCtl);
      pMe->m_pIDateCtl = NULL;
   }

   if (pMe->m_pIDatePickCtl)
   {
      IDATECTL_Release (pMe->m_pIDatePickCtl);
      pMe->m_pIDatePickCtl = NULL;
   }

}

/*===============================================================================
                     APP HELPER FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: IDateCtlUsage

DESCRIPTION
	This function encompasses all the usage examples of all the functions
   in code blocks switched using the BREW API function Id passed into this
   function. The code blocks will include almost all the variable declarations
   needed to demonstrate the usage of a given function. Within the code block
   all the needed interfaces are instantiated and released subsequent to their
   use.

   There are some exceptions to the above general rule, such as call-back functions,
   common helper function, or common display functions called from within the
   code block which are defined elsewhere in this file.

   There are some instances where an object instantiated within a code block is
   not released at the end of the code block, to be released at a later point
   (for example in a call-back function, in this case pointers to such objects
   are stored in the AEEApplet object to be released later).
   
PROTOTYPE:
   void IDateCtlUsage (CIDateCtlApp * pMe, uint16 wParam)

PARAMETERS:
	pMe: [in]: Contains a pointer to the CIDateCtlApp struct.
   wParam: [in]: Contains the Id of the function type to be used (contains the
      Id corresponding to the menu item selected on the emulator).

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the functions call was successful. FALSE otherwise

SIDE EFFECTS
  none
===========================================================================*/
static void IDateCtlUsage (CIDateCtlApp * pMe, uint16 wParam)
{
   AEEDeviceInfo *di;        // Device Info
   int lineHeight = 0;       // Stores the char height in pixels for given font
   IShell *pIShell = pMe->a.m_pIShell;

   lineHeight = pMe->m_nLineHeight;
   di = &pMe->m_dInfo;

   if (pMe == NULL || pIShell == NULL || pMe->a.m_pIDisplay == NULL || di == NULL)
       return;

   // Erase screen first for display output purposes.
   IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
	IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
	IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);

   switch (wParam)
   {
      case USAGE_BASIC_USAGE_DATE:
         {
            // 1. Usage: Basic Date Ctl Usage
            IDateCtl * pIDateCtl = NULL;
            AEERect    rct;
            uint32     currentDay = 0;
            uint32     currentJulianDate = 0;
            int        nMonth, nYear, nDay;
            int        nLength = 0;
            AECHAR     dayString[20];
            AECHAR     dateString[20];
            char szBuf1 [20] = {0};
            char szBuf2 [50] = {0};
        
            // Create IDateCtl object. This step needs to be done only if
            // the IDateCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_DATECTL, (void **)&pIDateCtl);

            if (pIDateCtl)
            {
               // Set rectangle coordinates of the date control object.
               rct.x = 10;
               rct.y = lineHeight + 3;     // Start on the second line
               rct.dx = di->cxScreen;
               rct.dy = lineHeight*2 + 5;  // Accomodate for 2 lines to be drawn

               IDATECTL_SetRect(pIDateCtl, &rct);

               // DateCtl can be used as a date conversion object
               currentJulianDate = 10456;
               IDATECTL_SetJulianDay(pIDateCtl, currentJulianDate);

	 	         // Get Day string
               if (IDATECTL_GetDayString(pIDateCtl, dayString, 
                  sizeof (dayString), &nLength))
               {
                  WSTR_TO_STR (dayString, szBuf1, sizeof(szBuf1));
                  STRCPY (szBuf2, szBuf1);
               }               
               
               // Get Date String
               if (IDATECTL_GetDateString(pIDateCtl, dateString,
                  sizeof(dateString), &nLength, DFMT_MONTH_DD_YYYY))
               {
                  STRCAT (szBuf2, ", ");
                  WSTR_TO_STR (dateString, szBuf1, sizeof(szBuf1));
                  STRCAT (szBuf2, szBuf1);
               }

               // Display string buffer on line 1.
               DisplayOutput (pMe, 1, szBuf2);

               // Get month, day and year from Julian date. But these are not 
               // used as display output.
               currentJulianDate = 11000;
               IDATECTL_SetJulianDay(pIDateCtl, currentJulianDate);
               IDATECTL_GetDate (pIDateCtl, &nYear, &nMonth, &nDay);

               // Display month/day/year retrieved from the above IDATECTL_GetDate 
               // function on line 2.
               SPRINTF (szBuf2, "Mon/Day/Yr: %i, %i, %i", nMonth, nDay, nYear);
               DisplayOutput (pMe, 2, szBuf2);

               // Set new rectangle coordinates
               rct.x = 5;
               rct.y = 3*lineHeight; // Change only the y axis (rectangle shifted down)
               IDATECTL_SetRect(pIDateCtl, &rct);

               // Redraw newly set date at new rect coordinate.
               IDATECTL_Redraw (pIDateCtl);

               // Set new rectangle coordinates. Change the x-axis value so that 
               // the rectangle is 20 pixel displaced to the right. Change the
               // y-axis toso that the rectangle shifts down.
               rct.x = 20;       
               rct.y = 6*lineHeight;
               IDATECTL_SetRect(pIDateCtl, &rct);

               // Get the current day and use it to get the julian date 
               // for the fist day of the month.  
               IDATECTL_GetDate(pIDateCtl, &nYear, &nMonth, &nDay);
               IDATECTL_SetDate(pIDateCtl, nYear, nMonth, 1);
               currentDay = IDATECTL_GetJulianDay(pIDateCtl);

               // Restore the date...
               IDATECTL_SetJulianDay(pIDateCtl, currentJulianDate);

               // Redraw newly set date at new rect coordinate.
               //IDATECTL_Redraw (pIDateCtl);
               IDATECTL_SetActive (pIDateCtl, TRUE);
               
               pMe->m_pIDateCtl  = pIDateCtl;
               // The date object is NOT released at this point. The date
               // object is released in the HandleEvents function under EVT_KEY. 
               // This gives the user the ability to edit date on an active 
               // date object on the screen by pressing the up/down/left/right
               // keys. The date object is released when the user 
               // presses any key other than UP/DOWN/LEFT/RIGHT or SELECT.
            }
         }
         break;
      case USAGE_BASIC_USAGE_DATEPICK:
         {
            // 2. Usage: Basic Date-Pick Ctl Usage
            AEERect rc;
            uint32     dwMask = 0xf0f0f0;
            uint32     currentJulianDate = 0;
            int        nMonth, nYear, nDay;
            int        nLength = 0;
            AECHAR     dayString[20];
            AECHAR     dateString[20];
            char szBuf1 [20] = {0};
            char szBuf2 [50] = {0};

            // The goal of the basic usage section is to show most of the
            // date-pick control interface functions being used in one block.

            // Create IDateCtl object. This step needs to be done only if
            // the IDateCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_DATEPICKCTL, (void **)&pMe->m_pIDatePickCtl);

            if (pMe->m_pIDatePickCtl)
            {
               // Set date and rectangle for display purposes.
               IDATECTL_SetDate(pMe->m_pIDatePickCtl, 2002, 10, 10);
               IDATECTL_SetRect(pMe->m_pIDatePickCtl, &rc);

               // The Julian Day set will overwrite the date set in the
               // IDATECTL_SetDate function invoke above.
               currentJulianDate = 11333;
               IDATECTL_SetJulianDay(pMe->m_pIDatePickCtl, currentJulianDate);

	 	         // Get Day string
               if (IDATECTL_GetDayString(pMe->m_pIDatePickCtl, dayString, 
                  sizeof (dayString), &nLength))
               {
                  WSTR_TO_STR (dayString, szBuf1, sizeof(szBuf1));
                  STRCPY (szBuf2, szBuf1);
               }               
               
               // Get Date String
               if (IDATECTL_GetDateString(pMe->m_pIDatePickCtl, dateString,
                  sizeof(dateString), &nLength, DFMT_MONTH_DD_YYYY))
               {
                  STRCAT (szBuf2, ", ");
                  WSTR_TO_STR (dateString, szBuf1, sizeof(szBuf1));
                  STRCAT (szBuf2, szBuf1);
               }

               // Display string buffer on line 1.
               DisplayOutput (pMe, 1, szBuf2);

               // Get month, day and year from Julian date. But these are not 
               // used as display output.
               currentJulianDate = 12123;
               IDATECTL_SetJulianDay(pMe->m_pIDatePickCtl, currentJulianDate);
               IDATECTL_GetDate (pMe->m_pIDatePickCtl, &nYear, &nMonth, &nDay);

               // Display month/day/year retrieved from the above IDATECTL_GetDate 
               // function on line 2.
               SPRINTF (szBuf2, "Mon/Day/Yr: %i, %i, %i", nMonth, nDay, nYear);
               DisplayOutput (pMe, 2, szBuf2);

               // Initialize a new rectangle and set the new rectangle.
               SETAEERECT (&rc, 0, 3*lineHeight, di->cxScreen, di->cyScreen*2/3);
               IDATECTL_SetRect(pMe->m_pIDatePickCtl, &rc);
               // Set a new date
               IDATECTL_SetDate(pMe->m_pIDatePickCtl, 1999, 9, 21);

               // Sets a new active day mask. Active day mask is 32 bit integer 
               // in which each of 0 to 30 bits specifies active status of a day. 
               // An active day, having bit corresponding to it set, is drawn in 
               // reverse video.
               IDATECTL_SetActiveDayMask (pMe->m_pIDatePickCtl, dwMask);

               // Set the date-pick object active. This gives the user the
               // ability to use the arrow keys to change the day/month/year.
               // The user can exit by pressing any of the other keys.
               IDATECTL_SetActive (pMe->m_pIDatePickCtl, TRUE);

               // The date object is NOT released at this point. The date
               // object is released in the HandleEvents function under EVT_KEY. 
               // This gives the user the ability to edit date on an active 
               // date object on the screen by pressing the up/down/left/right
               // keys. The date object is released when the user 
               // presses any key other than UP/DOWN/LEFT/RIGHT or SELECT.
            }
         }
         break;
      case USAGE_HANDLEEVENT_DATE:
         {
            // 3. Usage: IDATECTL_HandleEvent
            AEERect      rct;
            AEEEvent     evt = EVT_KEY;       // DateCtl only supports this event
            uint16       wParam  = AVK_LEFT;  // Supports SELECT, LEFT, RIGHT, UP, DOWN
            uint32       dwParam = 0;

            // Create IDateCtl object. This step needs to be done only if
            // the IDateCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_DATECTL, (void **)&pMe->m_pIDateCtl);

            if (pMe->m_pIDateCtl)
            {
               // Initialize a rectangle
               SETAEERECT (&rct, 0, lineHeight *2, di->cxScreen, di->cyScreen - 3* lineHeight);
               // SetRect/SetDate used for output display purposes only. For more 
               // details see IDATECTL_SetDate/IDATECTL_SetRect usage. 
               IDATECTL_SetRect(pMe->m_pIDateCtl, &rct);
               IDATECTL_SetDate(pMe->m_pIDateCtl, 2000, 1, 1);

               // The date control needs to be active for the handle event function
               // to succeed. See IDATECTL_SetActive usage for more information.
               IDATECTL_SetActive (pMe->m_pIDateCtl, TRUE);

               // DATECTL_HandleEvent: is used to handle the events received by 
               // date control object. If the date control object is in non 
               // edit mode, it processes only set title, set date and press of 
               // UP and DOWN key events. In date edit mode (active), it processes 
               // various events like key up, down, left, and right.

               // Invoke HandleEvent. 
               // The default selected field is the Month field (First field). By 
               // invoking HandleEvents function with EVT_KEY event and the key value 
               // of AVK_LEFT the field selection will be moved to the third field 
               // (Year field) on the date control object.
               //
               // Note: a more practical or typical use of IDATECTL_HandleEvent is 
               // found in the HandleEvent function of this applet under EVT_KEY event.

               if ((IDATECTL_HandleEvent(pMe->m_pIDateCtl, evt, wParam, dwParam)) == FALSE)
               {
                  // Release Date control. This step only needs to be done
                  // if no further use of the IDateCtl object is needed.
                  IDATECTL_Release (pMe->m_pIDateCtl);
                  return;
               }

               // The date object is NOT released at this point. The date
               // object is released in the HandleEvents function under EVT_KEY. 
               // This gives the user the ability to edit date on an active 
               // date object on the screen by pressing the up/down/left/right
               // keys. The date object is released when the user 
               // presses any key other than UP/DOWN/LEFT/RIGHT or SELECT.
            }
         }
         break;
      case USAGE_HANDLEEVENT_DATEPICK:
         {
            // 4. Usage: IDATECTL_HandleEvent (Date Pick mode)
            AEERect      rct;
            AEEEvent     evt = EVT_KEY;       // DateCtl only supports this event
            uint16       wParam  = AVK_LEFT;  // Supports SELECT, LEFT, RIGHT, UP, DOWN
            uint32       dwParam = 0;

            // Create IDateCtl object. This step needs to be done only if
            // the IDateCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_DATEPICKCTL, (void **)&pMe->m_pIDatePickCtl);

            if (pMe->m_pIDatePickCtl)
            {
               // Initialize a rectangle
               SETAEERECT (&rct, 0, di->cyScreen/5, di->cxScreen, di->cyScreen*3/4);
               // SetRect/SetDate used for output display purposes only. For more 
               // details see IDATECTL_SetDate/IDATECTL_SetRect usage. 
               IDATECTL_SetRect(pMe->m_pIDatePickCtl, &rct);
               IDATECTL_SetDate(pMe->m_pIDatePickCtl, 2000, 1, 1);

               // The date control needs to be active for the handle event function
               // to succeed. See IDATECTL_SetActive usage for more information.
               IDATECTL_SetActive (pMe->m_pIDatePickCtl, TRUE);

               // DATECTL_HandleEvent: is used to handle the events received by 
               // date control object. If the date control object is in non 
               // edit mode, it processes only set title, set date and press of 
               // UP and DOWN key events. In date edit mode (active), it processes 
               // various events like key up, down, left, and right.

               // Invoke HandleEvent. 
               // The default selected field is the day-of-month. By invoking HandleEvents 
               // function with EVT_KEY event and the key value of AVK_LEFT the field 
               // selection will be moved to the previous day-of-month (since the date 
               // that is set is Jan, 1, 2000, the previous day is Dec, 31 1999).
               //
               // Note: a more typical use of IDATECTL_HandleEvent is found in 
               // the HandleEvent function of this applet (under EVT_KEY event).
               if ((IDATECTL_HandleEvent(pMe->m_pIDatePickCtl, evt, wParam, dwParam)) == FALSE)
               {
                  // Release Date control. This step only needs to be done
                  // if no further use of the IDateCtl object is needed.
                  IDATECTL_Release (pMe->m_pIDatePickCtl);
                  return;
               }

               // The date object is NOT released at this point. The date
               // object is released in the HandleEvents function under EVT_KEY. 
               // This gives the user the ability to edit date on an active 
               // date object on the screen by pressing the up/down/left/right
               // keys. The date object is released when the user 
               // presses any key other than UP/DOWN/LEFT/RIGHT or SELECT.
            }
         }
         break;
      case USAGE_REDRAW:
         {
            // 5. Usage: IDATECTL_Redraw
            IDateCtl * pIDateCtl = NULL;
            IDateCtl * pIDatePickCtl = NULL;
            AEERect rct;
            boolean bRetVal = FALSE;

            // The output of this example will result in the date control object
            // being drawn in the top half of the screen and the date-pick
            // contorl object drawn in the bottom half of the screen.
            //
            // Create IDateCtl object. This step needs to be done only if
            // the IDateCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_DATECTL, (void **)&pIDateCtl);
            ISHELL_CreateInstance(pIShell, AEECLSID_DATEPICKCTL, (void **)&pIDatePickCtl);

            if (pIDateCtl && pIDatePickCtl)
            {
               // Set Date/Rect for output display purposes only. 
               // See IDATECTL_SetRect/IDATECTL_SetDate usage for more details.
               SETAEERECT (&rct, 0, di->cyScreen/8, di->cxScreen, di->cyScreen/4); 
               IDATECTL_SetRect(pIDateCtl, &rct);

               SETAEERECT (&rct, 0, di->cyScreen/3, di->cxScreen, di->cyScreen*3/4); 
               IDATECTL_SetRect(pIDatePickCtl, &rct);

               IDATECTL_SetDate(pIDateCtl, 2000, 2, 2);
               IDATECTL_SetDate(pIDatePickCtl, 2000, 2, 2);

               // IDATECTL_Redraw: instructs the date control object to redraw 
               // it's contents. The Date control object does not redraw its 
               // contents every time the underlying data of the date control 
               // changes. This allows several data updates to occur while minimizing 
               // screen flashes. 
               bRetVal = IDATECTL_Redraw (pIDateCtl);
               bRetVal = IDATECTL_Redraw (pIDatePickCtl);

               // We do not need to check the return values here since we are going
               // release both the date objects here. The output will just redraw the
               // date objects on the screen.

               // Release both date object
               IDATECTL_Release (pIDateCtl);
               IDATECTL_Release (pIDatePickCtl);
            }
            else
            {
               // Atleast one of the objects did not get created. Release the
               // other one if it did get created.
               if (pIDateCtl != NULL)
                  IDATECTL_Release (pIDateCtl);
               if (pIDatePickCtl != NULL)
                  IDATECTL_Release (pIDatePickCtl);
            }
         }
         break;
      case USAGE_SETACTIVE_DATE:
         {
            // 6. Usage: IDATECTL_SetActive	
            AEERect rct;

            //Create IDateCtl object. This step needs to be done only if
            //the IDateCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_DATECTL, (void **)&pMe->m_pIDateCtl);
	
            if (pMe->m_pIDateCtl)
            {	
               // Initialize a rectangle
               SETAEERECT (&rct, 0, lineHeight*2, di->cxScreen, di->cyScreen - 5* lineHeight);
 
               // Set Date/Rect for output display purposes. 
               // See IDATECTL_SetRect/IDATECTL_SetDate usage for more details.
               IDATECTL_SetRect(pMe->m_pIDateCtl, &rct);
               IDATECTL_SetDate(pMe->m_pIDateCtl, 2000, 3, 3);

               // IDATECTL_SetActive: is used to make a date control object 
               // active. Only an active date control object handles the event 
               // sent to it. Inactive date control object just ignores the events.
               IDATECTL_SetActive (pMe->m_pIDateCtl, TRUE);

               // The date object is NOT released at this point. The date
               // object is released in the HandleEvents function under EVT_KEY. 
               // This gives the user the ability to edit date on an active 
               // date object on the screen by pressing the up/down/left/right
               // keys. The date object is released when the user 
               // presses any key other than UP/DOWN/LEFT/RIGHT or SELECT.
            }
         }
         break;
      case USAGE_SETACTIVE_DATEPICK:
         {
            // 7. Usage: IDATECTL_SetActive
            AEERect rct;

            //Create IDateCtl object. This step needs to be done only if
            //the IDateCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_DATEPICKCTL, (void **)&pMe->m_pIDatePickCtl);
	
            if (pMe->m_pIDatePickCtl)
            {	
               // Initialize a rectangle
               SETAEERECT (&rct, 0, lineHeight*2, di->cxScreen, di->cyScreen - 5* lineHeight);

               // Set Date/Rect for output display purposes. 
               // See IDATECTL_SetRect/IDATECTL_SetDate usage for more details.
               IDATECTL_SetRect(pMe->m_pIDatePickCtl, &rct);
               IDATECTL_SetDate(pMe->m_pIDatePickCtl, 2000, 3, 3);

               // IDATECTL_SetActive: is used to make a date control object 
               // active. Only an active date control object handles the event 
               // sent to it. Inactive date control object just ignores the events.
               IDATECTL_SetActive (pMe->m_pIDatePickCtl, TRUE);

               // The date object is NOT released at this point. The date
               // object is released in the HandleEvents function under EVT_KEY. 
               // This gives the user the ability to edit date on an active 
               // date object on the screen by pressing the up/down/left/right
               // keys. The date object is released when the user 
               // presses any key other than UP/DOWN/LEFT/RIGHT or SELECT.
            }
         }
         break;
      case USAGE_ISACTIVE:
         {
            // 8. Usage: IDATECTL_IsActive
            AEERect    rct;
            IDateCtl * pIDateCtl = NULL;
            IDateCtl * pIDatePickCtl = NULL;
            boolean    bRetVal;
		
            // Create IDateCtl object. This step needs to be done only if
            // the IDateCtl object has not been created already.
            ISHELL_CreateInstance(pIShell, AEECLSID_DATECTL, (void **)&pIDateCtl);
            ISHELL_CreateInstance(pIShell, AEECLSID_DATEPICKCTL, (void **)&pIDatePickCtl);

            if (pIDateCtl && pIDatePickCtl)
            {
               SETAEERECT (&rct, 0, di->cyScreen/8, di->cxScreen, di->cyScreen/4); 
               IDATECTL_SetRect(pIDateCtl, &rct);
               SETAEERECT (&rct, 0, di->cyScreen/3, di->cxScreen, di->cyScreen*3/4); 

               IDATECTL_SetRect(pIDatePickCtl, &rct);
               IDATECTL_SetDate(pIDateCtl, 2000, 4, 4);
               IDATECTL_SetDate(pIDatePickCtl, 2000, 4, 4);

               // Activate both the objects
               IDATECTL_SetActive (pIDateCtl, TRUE);
               IDATECTL_SetActive (pIDatePickCtl, TRUE);

               // IDATECTL_IsActive: returns whether the date control 
               // object is active or not. Active state is indicated 
               // by return value of TRUE where as inactive state is 
               // indicated by return value of FALSE.
               bRetVal = IDATECTL_IsActive (pIDateCtl);
               bRetVal = IDATECTL_IsActive (pIDatePickCtl);

               //
               // This example does not have display output...
               //

               // Release IDateCtl objects. This step needs to be done
               // only if no further use of the IDateCtl objects are needed.
               IDATECTL_Release (pIDateCtl);
               IDATECTL_Release (pIDatePickCtl);
            }
            else
            {
               // Atleast one of the objects did not get created. Release the
               // other one if it did get created.
               if (pIDateCtl != NULL)
                  IDATECTL_Release (pIDateCtl);
               if (pIDatePickCtl != NULL)
                  IDATECTL_Release (pIDatePickCtl);
            }
         }
         break;
      case USAGE_SETRECT:
         {
            // 9. Usage: IDATECTL_SetRect 
            IDateCtl * pIDateCtl = NULL;
            IDateCtl * pIDatePickCtl = NULL;
            AEEDeviceInfo dInfo; // Device Info
            AEERect rctDate;
            AEERect rctDatePick;

            // In this example we initialize two rectangles and 
            // and set these rectangles as date and date-pick control
            // object rectangles. Then we activate these two objects, 
            // which results in the objects being drawn on the screen.
            // Subsequent to drawing the date objects are released.
            // The resulting output has a date control object being drawn
            // on the top half of the screen and a date-pick object 
            // being drawn in the bottom half of the screen.

            // Get screen size information for display purposes.
            ISHELL_GetDeviceInfo(pIShell,&dInfo);

            // Give location values to place the date and date-pick
            // control objects on the screen.
            rctDate.x = dInfo.cxScreen/4;
            rctDate.y = dInfo.cyScreen/8;
            rctDate.dx = dInfo.cxScreen;
            rctDate.dy = dInfo.cyScreen/4;

            rctDatePick.x = dInfo.cxScreen/8;
            rctDatePick.y = dInfo.cyScreen/3;
            rctDatePick.dx = dInfo.cxScreen*3/4;
            rctDatePick.dy = dInfo.cyScreen*2/3;

            // Create IDateCtl object. This step needs to be done only if
            // the IDateCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_DATECTL, (void **)&pIDateCtl); 
            ISHELL_CreateInstance(pIShell, AEECLSID_DATEPICKCTL, (void **)&pIDatePickCtl); 

            if (pIDateCtl && pIDatePickCtl)
            {
               // IDATECTL_SetRect: can be used to set the specified rectangle 
               // as bounding rectangle of the date control object. If the 
               // date control object is in month view, specified rectangle 
               // is also used to determine grid used for drawing days.
               IDATECTL_SetRect(pIDateCtl, &rctDate);
               IDATECTL_SetRect(pIDatePickCtl, &rctDatePick);

               // Set Date/Active to display date info in new rectangle coordinates.
               // See IDATECTL_SetRect/IDATECTL_SetActive usage for more details.
               IDATECTL_SetDate(pIDateCtl, 2000, 5, 5);
               IDATECTL_SetActive (pIDateCtl, TRUE);
               IDATECTL_SetDate(pIDatePickCtl, 2000, 5, 5);
               IDATECTL_SetActive (pIDatePickCtl, TRUE);

               // Release IDateCtl objects. This step needs to be done
               // only if no further use of the IDateCtl objects are needed.
               IDATECTL_Release (pIDateCtl);
               IDATECTL_Release (pIDatePickCtl);
            }
            else
            {
               // Atleast one of the objects did not get created. Release the
               // other one if it did get created.
               if (pIDateCtl != NULL)
                  IDATECTL_Release (pIDateCtl);
               if (pIDatePickCtl != NULL)
                  IDATECTL_Release (pIDatePickCtl);
            }
         }
         break;
      case USAGE_GETRECT:
         {
            // 10. Usage: IDATECTL_GetRect
            IDateCtl * pIDateCtl = NULL;
            IDateCtl * pIDatePickCtl = NULL;
            AEERect      rct;
 
            // Create IDateCtl objects. This step needs to be done only if
            // the IDateCtl objects has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_DATECTL, (void **)&pIDateCtl); 
            ISHELL_CreateInstance(pIShell, AEECLSID_DATEPICKCTL, (void **)&pIDatePickCtl); 

            if (pIDateCtl && pIDatePickCtl)
            {
               // IDATECTL_GetRect: returns whether the date control object 
               // is active or not. Active state is indicated by return value 
               // of TRUE where as inactive state is indicated by return value of FALSE.
               IDATECTL_GetRect(pIDateCtl, &rct);
               IDATECTL_GetRect(pIDatePickCtl, &rct);

               //
               // This example does not have output...
               //

               // Release IDateCtl objects. This step needs to be done
               // only if no further use of the IDateCtl objects are needed.
               IDATECTL_Release (pIDateCtl);
               IDATECTL_Release (pIDatePickCtl);
            }
            else
            {
               // Atleast one of the objects did not get created. Release the
               // other one if it did get created.
               if (pIDateCtl != NULL)
                  IDATECTL_Release (pIDateCtl);
               if (pIDatePickCtl != NULL)
                  IDATECTL_Release (pIDatePickCtl);
            }
         }
         break;      
      case USAGE_SETPROPERTIES:
      case USAGE_GETPROPERTIES:
         {
            // 11. Usage:IDATECTL_SetProperties/IDATECTL_GetProperties
            IDateCtl * pIDateCtl = NULL;
            IDateCtl * pIDatePickCtl = NULL;
            AEERect    rct;
            uint32     nProperties = CP_3D_BORDER;    // Control has 3D border
            uint32     nPropDate = 0;                 // Test value
            uint32     nPropDatePick = 0;
	
            // Create IDateCtl object. This step needs to be done only if
            // the IDateCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_DATECTL, (void **)&pIDateCtl); 
            ISHELL_CreateInstance(pIShell, AEECLSID_DATEPICKCTL, (void **)&pIDatePickCtl); 

            if (pIDateCtl && pIDatePickCtl)
            {
      		   // IDATECTL_SetProperties: sets date control specific properties 
               // or flags. Presently there are no date control specific properties 
               // or flags to be set. 
	      	   IDATECTL_SetProperties(pIDateCtl, nProperties);
	      	   IDATECTL_SetProperties(pIDatePickCtl, nProperties);

               // IDATECTL_GetProperties: fills given pointer to AEERect with 
               // the cordinates of the current bounding rectangle of the date 
               // control object.  This is particularly useful after a control 
               // is created to determine it's optimal/default size and position.
               //
               // Presently there are no date control specific properties 
               // or flags to be get.
               nPropDate = IDATECTL_GetProperties(pIDateCtl);
               nPropDatePick = IDATECTL_GetProperties(pIDatePickCtl);

               // Set Date/Rect/Redraw for output display purposes only. 
               // See IDATECTL_SetRect/IDATECTL_SetDate/IDATECTL_Redraw usage 
               // for more details.
               SETAEERECT (&rct, 0, di->cyScreen/8, di->cxScreen, di->cyScreen/4); 
               IDATECTL_SetRect(pIDateCtl, &rct);
               IDATECTL_SetDate(pIDateCtl, 2000, 2, 2);
               IDATECTL_Redraw (pIDateCtl);

               SETAEERECT (&rct, 0, di->cyScreen/3, di->cxScreen, di->cyScreen*3/4); 
               IDATECTL_SetRect(pIDatePickCtl, &rct);
               IDATECTL_SetDate(pIDatePickCtl, 2000, 2, 2);
               IDATECTL_Redraw (pIDatePickCtl);

               // Release IDateCtl objects. This step needs to be done
               // only if no further use of the IDateCtl objects are needed.
               IDATECTL_Release (pIDateCtl);
               IDATECTL_Release (pIDatePickCtl);
            }
            else
            {
               // Atleast one of the objects did not get created. Release the
               // other one if it did get created.
               if (pIDateCtl != NULL)
                  IDATECTL_Release (pIDateCtl);
               if (pIDatePickCtl != NULL)
                  IDATECTL_Release (pIDatePickCtl);
            }
         }
         break;
      case USAGE_RESET:
         {
            // 12. Usage: IDATECTL_Reset
            AECHAR    szBuf [30] = {0};
            AEERect   rct;

            // In this section we create date and date-pick control
            // objects and demonstrate the use of IDATECTL_Reset
            // function in both the date and date-pick modes. The key steps 
            // of the following section are given below.
            //
            // Step 1. Initialize a rectangle object to cover the top part of 
            //         the screen and use it to set the rectangle of the date
            //         control object.
            // Step 2. Initialize the rectangle object to cover the bottom 3/4
            //         of the screen. Use this rectangle to set the rectangle of
            //         the date-pick control object.
            // Step 3. Set the date of both the date and date-pick control objects
            //         to Jul, 7, 2000. 
            // Step 4. Set the Title of both the date and date-pick control objects.
            // Step 5. Set both date and date-pick objects active. This will draw
            //         both the objects on the screen, including the titles.
            // Step 6. Reset the date and date-pick objects. This causes the 
            //         the titles of the objects to be deleted and the object
            //         made inactive.
            // Step 7. Redraw both the objects. The Titles of both the objects will
            //         not be present since they were deleted in the previous step.
            //
            // At this point the screen will have both date control objects drawn 
            // on the screen without their Titles. But, they are both inactive.
            // When the user presses the Up/Down/Left/Right or any other 
            // keys they are not handled by either objects HandleEvent functions. 
            // Hence, both the date and date-pick control objects will be released
            // and the main menu will be displayed.

           
            // Create IDateCtl object. This step needs to be done only if
            // the IDateCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_DATECTL, (void **)&pMe->m_pIDateCtl); 
            ISHELL_CreateInstance(pIShell, AEECLSID_DATEPICKCTL, (void **)&pMe->m_pIDatePickCtl); 

            if (pMe->m_pIDateCtl && pMe->m_pIDatePickCtl)
            {
               // Step 1. Initialize rectangle object and SetRect of the date  
               //         control object to this rectangle
               SETAEERECT (&rct, 0, di->cyScreen/8, di->cxScreen, di->cyScreen/4); 
               IDATECTL_SetRect(pMe->m_pIDateCtl, &rct);

               // Step 2. Initialize rectangle object and SetRect of the date-pick  
               //         control object to this rectangle
               SETAEERECT (&rct, 0, di->cyScreen/3, di->cxScreen, di->cyScreen*3/4); 
               IDATECTL_SetRect(pMe->m_pIDatePickCtl, &rct);

               // Step 3. Set date of both the date and date-pick objects to 
               //         7/7/2000.
               IDATECTL_SetDate(pMe->m_pIDateCtl, 2000, 7, 7);
               IDATECTL_SetDate(pMe->m_pIDatePickCtl, 2000, 7, 7);

               // Step 4. Set the Title of both the date and date-pick 
               //         control objects to "Date Control Title"..
               STR_TO_WSTR("Date Control Title", szBuf, sizeof(szBuf));
               IDATECTL_SetTitle(pMe->m_pIDateCtl, NULL, NULL, szBuf);
               IDATECTL_SetTitle(pMe->m_pIDatePickCtl, NULL, NULL, szBuf);

               // Step 5. Set both objects active.
               IDATECTL_SetActive (pMe->m_pIDateCtl, TRUE); 
               IDATECTL_SetActive (pMe->m_pIDatePickCtl, TRUE); 

               // Step 6. Reset the date and date-pick objects.
               // 
		         // IDATECTL_Reset: instructs the date control to reset (free/delete) 
               // its title as well as to immediately leave active/focus mode.
		         IDATECTL_Reset (pMe->m_pIDateCtl);
		         IDATECTL_Reset (pMe->m_pIDatePickCtl);

               // Step 7. Redraw both the objects
               IDATECTL_Redraw (pMe->m_pIDateCtl);
               IDATECTL_Redraw (pMe->m_pIDatePickCtl);

               // The date objects are NOT released at this point. The date
               // objects are released in the HandleEvents function under EVT_KEY.
               // Since both objects are not active the HandleEvent functions
               // for both under EVT_KEY will not handle any key press events.
               // Hence, the objects will be released and the control will be
               // pass the main menu when the user presses any key (except the END
               // key where the applet is exited). In other words the date objects 
               // will be released and the main menu will be redrawn.
            }
            else
            {
               // Atleast one of the objects did not get created. Release the
               // other one if it did get created.
               if (pMe->m_pIDateCtl != NULL)
                  IDATECTL_Release (pMe->m_pIDateCtl);
               if (pMe->m_pIDatePickCtl != NULL)
                  IDATECTL_Release (pMe->m_pIDatePickCtl);
            }
         }
         break;
      case USAGE_SETTITLE:
         // 13. Usage: IDATECTL_SetTitle
         {
            IDateCtl * pIDateCtl = NULL;
            IDateCtl * pIDatePickCtl = NULL;
            boolean    bRetVal = FALSE;
            AECHAR szBuf [100];
            AEERect rct;
		     
            // In this section we create date and date-pick control
            // objects and demonstrate the use of IDATECTL_SetTitle
            // function in both the date and date-pick modes. The key steps 
            // of the following section are given below.
            //
            // Note: The Date-Pick object does not have a Title associated with 
            //       it. The IDATECTL_SetTitle function can be used to set
            //       the title of a date-pick control object, but this title
            //       will not appear when a date-pick object is drawn.
            //
            // Step 1. Create a date control and a date-pick control object.
            // Step 2. Set title of the date control object
            // Step 3. Set title of the date-pick control object
            // Step 4. Set rectangle for the date control object.
            // Step 5. Set rectangle for the date-pick control object.
            // Step 6. Set date for both control objects.
            // Step 7. Redraw both the control objects.
            // Step 8. Release date and date-pick control objects.

            // Step 1: Create a date control and a date-pick control object.
            ISHELL_CreateInstance(pIShell, AEECLSID_DATECTL, (void **)&pIDateCtl);
            ISHELL_CreateInstance(pIShell, AEECLSID_DATEPICKCTL, (void **)&pIDatePickCtl); 

            if (pIDateCtl && pIDatePickCtl)
            {
               // Step 2: Set title of date control
               STR_TO_WSTR("Date Control ", szBuf, sizeof(szBuf));
               if (!(bRetVal = IDATECTL_SetTitle(pIDateCtl, NULL, NULL, szBuf)))
               {
                  // Release Date control. This step only needs to be done
                  // if no further use of the IDateCtl object is needed.
                  IDATECTL_Release (pIDateCtl);
                  IDATECTL_Release (pIDatePickCtl);
                  return;
               }

               // Step 3: Set title of date-pick control
               STR_TO_WSTR("Date-Pick Control ", szBuf, sizeof(szBuf));
               if ((bRetVal = IDATECTL_SetTitle(pIDatePickCtl, NULL, NULL, szBuf))
                   == FALSE)
               {
                  // Release Date control. This step only needs to be done
                  // if no further use of the IDateCtl object is needed.
                  IDATECTL_Release (pIDateCtl);
                  IDATECTL_Release (pIDatePickCtl);
                  return;
               }

               // Step 4. Set rectangle for the date control object.
               SETAEERECT (&rct, 0, di->cyScreen/10, di->cxScreen, di->cyScreen/3); 
               IDATECTL_SetRect(pIDateCtl, &rct);

               // Step 5. Set rectangle for the date-pick control object.
               SETAEERECT (&rct, 0, di->cyScreen*2/5, di->cxScreen, di->cyScreen*3/4); 
               IDATECTL_SetRect(pIDatePickCtl, &rct);

               // Step 6: Set dates for both the date and date-pick objects.
               IDATECTL_SetDate(pIDateCtl, 2000, 8, 8);
               IDATECTL_SetDate(pIDatePickCtl, 2000, 8, 8);

               // Step 7: Redraw both the objects. This will draw the respective
               //         objects on the screen at the coordinates and dimensions 
               //         set previously using IDATECTL_SetRect.
               IDATECTL_Redraw (pIDateCtl);
               IDATECTL_Redraw (pIDatePickCtl);

               //
               // Note: The date control object will be draw in the top 
               //       half of the screen. It will contain a Title specified above in 
               //       Step 2.
               //       The date-pick control object will be drawn in the bottom
               //       2/3 of the screen. But it will NOT HAVE a title drawn for it.
               //

               // Step 8: Release IDateCtl objects. This step needs to be done
               //         only if no further use of the IDateCtl objects are needed.
               IDATECTL_Release (pIDateCtl);
               IDATECTL_Release (pIDatePickCtl);
            }
            else
            {
               // Atleast one of the objects did not get created. Release the
               // other one if it did get created.
               if (pIDateCtl != NULL)
                  IDATECTL_Release (pIDateCtl);
               if (pIDatePickCtl != NULL)
                  IDATECTL_Release (pIDatePickCtl);
            }
         }
         break;
      case USAGE_SETJULIANDAY:
         {
            // 14. Usage: IDATECTL_SetJulianDay
            IDateCtl * pIDateCtl = NULL;
            IDateCtl * pIDatePickCtl = NULL;
            boolean    boolRetVal = FALSE;	   
            int32      julianDate = 1000;       // Initialize Julian date
            AEERect    rct;

            // In this section we create date and date-pick control
            // objects and demonstrate the use of IDATECTL_SetJulianDay
            // function in both the date and date-pick modes. The key steps 
            // of the following section are given below.
            //
            // Step 1. Create a date control and a date-pick control object.
            // Step 2. Set Julian date for the date control object. The date set
            //         is 1000, which corresponds to Saturday, September 23, 1972
            // Step 3. Set Julian date for the date-pick control object. The date set
            //         is the same as above
            // Step 3. Set title of the date-pick control object
            // Step 4. Set rectangle for the date control object.
            // Step 5. Set rectangle for the date-pick control object.
            // Step 6. Redraw both the control objects.
            // Step 7. Release date and date-pick control objects.

            // Step 1: Create a date control and a date-pick control object.
            ISHELL_CreateInstance(pIShell, AEECLSID_DATECTL, (void **)&pIDateCtl);
            ISHELL_CreateInstance(pIShell, AEECLSID_DATEPICKCTL, (void **)&pIDatePickCtl); 

            if (pIDateCtl && pIDatePickCtl)
            {
               // Step 2: Set Julian date for the date control object
               if ((boolRetVal = IDATECTL_SetJulianDay(pIDateCtl, julianDate))
                  == FALSE)
               {
                  // Release both objects
                  IDATECTL_Release (pIDateCtl);
                  IDATECTL_Release (pIDatePickCtl);
                  return;
               }

               // Step 3: Set Julian date for the date-pick control object
               if ((boolRetVal = IDATECTL_SetJulianDay(pIDatePickCtl, julianDate))
                  == FALSE)
               {
                  // Release both objects
                  IDATECTL_Release (pIDateCtl);
                  IDATECTL_Release (pIDatePickCtl);
                  return;
               }

               // Step 4. Set rectangle for the date control object.
               SETAEERECT (&rct, 0, di->cyScreen/10, di->cxScreen, di->cyScreen/3); 
               IDATECTL_SetRect(pIDateCtl, &rct);

               // Step 5. Set rectangle for the date-pick control object.
               SETAEERECT (&rct, 0, di->cyScreen*2/5, di->cxScreen, di->cyScreen*3/4); 
               IDATECTL_SetRect(pIDatePickCtl, &rct);

               // Step 6: Redraw both the objects. This will draw the respective
               //         objects on the screen at the coordinates and dimensions 
               //         set previously using IDATECTL_SetRect.
               IDATECTL_Redraw (pIDateCtl);
               IDATECTL_Redraw (pIDatePickCtl);

               //
               // Above redraw will cause both the date objects to be drawn on the
               // screen, with the date set to Saturday, September 23, 1972.
               //

               // Step 7: Release IDateCtl objects. This step needs to be done
               //         only if no further use of the IDateCtl objects are needed.
               IDATECTL_Release (pIDateCtl);
               IDATECTL_Release (pIDatePickCtl);
            }
            else
            {
               // Atleast one of the objects did not get created. Release the
               // other one if it did get created.
               if (pIDateCtl != NULL)
                  IDATECTL_Release (pIDateCtl);
               if (pIDatePickCtl != NULL)
                  IDATECTL_Release (pIDatePickCtl);
            }
         }
         break;
      case USAGE_GETJULIANDAY:
         {
            // 15. Usage: IDATECTL_GetJulianDay
            IDateCtl * pIDateCtl = NULL;
            IDateCtl * pIDatePickCtl = NULL;
            int32      julianDate = 0;
            char       szBuf [30] = {0};

            // In this section we create date and date-pick control
            // objects and demonstrate the use of IDATECTL_GetJulianDay
            // function in both the date and date-pick modes. The key steps 
            // of the following section are given below.
            //
            // Step 1. Create a date control and a date-pick control object.
            // Step 2. Set Julian date for the date control object. The date set
            //         is 12345.
            // Step 3. Set Julian date for the date-pick control object. The date set
            //         is 54321
            // Step 3. Get Julian date fron of the date control object
            // Step 4. Display the Julian date retrieved from date 
            //         control object. DisplayOutput function used to display
            //         date is general purpose helper function. See 
            //         function description for more information.
            // Step 5. Get the julian date for the date-pick control object.
            // Step 6. Display the Julian date retrieved from date-pick 
            //         control object.
            // Step 7. Release date and date-pick control objects.


            // Step 1: Create a date control and a date-pick control object.
            ISHELL_CreateInstance(pIShell, AEECLSID_DATECTL, (void **)&pIDateCtl);
            ISHELL_CreateInstance(pIShell, AEECLSID_DATEPICKCTL, (void **)&pIDatePickCtl); 

            if (pIDateCtl && pIDatePickCtl)
            {
               // Step 2: Set the julian dates for both date controls
               //         so they can be retrieved using IDATECTL_GetJulianDay.
               IDATECTL_SetJulianDay(pIDateCtl, 12345);
               IDATECTL_SetJulianDay(pIDatePickCtl, 54321);

               // Step 3: Get Julian date
               //
               // IDATECTL_GetJulianDay: is used to get julian day value of 
               // the specified date control object.
               julianDate = IDATECTL_GetJulianDay(pIDateCtl);

               // Step 4: Display the Julian date retrieved from date object.
               SPRINTF (szBuf, "Julian Date: %i", julianDate);
               DisplayOutput (pMe, 2, szBuf);

               // Step 5: Get the julian date for the date-pick control object.
               julianDate = IDATECTL_GetJulianDay(pIDatePickCtl);

               // Step 6: Display the Julian date retrieved from the date-pick object.
               SPRINTF (szBuf, "DatePick JulDate: %i", julianDate);
               DisplayOutput (pMe, 4, szBuf);

               // Step 7: Release IDateCtl objects. This step needs to be done
               //         only if no further use of the IDateCtl objects are needed.
               IDATECTL_Release (pIDateCtl);
               IDATECTL_Release (pIDatePickCtl);
            }
            else
            {
               // Atleast one of the objects did not get created. Release the
               // other one if it did get created.
               if (pIDateCtl != NULL)
                  IDATECTL_Release (pIDateCtl);
               if (pIDatePickCtl != NULL)
                  IDATECTL_Release (pIDatePickCtl);
            }
         }
         break;
      case USAGE_SETDATE:
         {
            // 16. Usage: IDATECTL_SetDate
            IDateCtl * pIDateCtl = NULL;
            IDateCtl * pIDatePickCtl = NULL;
            boolean    bRetVal = FALSE;
            int        nYear  = 2001;
            int        nMonth = 9;
            int        nDay   = 10;
            AEERect    rct;

            // In this section we create date and date-pick control
            // objects and demonstrate the use of IDATECTL_SetDate
            // function in both the date and date-pick modes. The key steps 
            // of the following section are given below.
            //
            // Step 1. Create a date control and a date-pick control object.
            // Step 2. Set date for the date control object. The date set
            //         is Sept, 10, 2001.
            // Step 3. Set date for the date-pick control object. The date set
            //         is the same as in the previous step.
            // Step 4. Set rectangle and redraw the date control object.
            // Step 5. Set rectangle and redraw the date-pick control object.
            // Step 6. Release date and date-pick control objects.

            // Step 1: Create a date control and a date-pick control object.
            ISHELL_CreateInstance(pIShell, AEECLSID_DATECTL, (void **)&pIDateCtl);
            ISHELL_CreateInstance(pIShell, AEECLSID_DATEPICKCTL, (void **)&pIDatePickCtl); 

            if (pIDateCtl && pIDatePickCtl)
            {
               // Step 2: Set Date of date object.
               if ((bRetVal = IDATECTL_SetDate(pIDateCtl, nYear, nMonth, nDay))
                  == FALSE)
               {
                  // Release date controls
                  IDATECTL_Release (pIDateCtl);
                  IDATECTL_Release (pIDatePickCtl);
                  return;
               }

               // Step 3: Set date of date-pick object.
               if ((bRetVal = IDATECTL_SetDate(pIDatePickCtl, nYear, nMonth, nDay))
                  == FALSE)
               {
                  // Release Date controls.
                  IDATECTL_Release (pIDateCtl);
                  IDATECTL_Release (pIDatePickCtl);
                  return;
               }

               // Step 4. Set rectangle and redraw the date control object.
               SETAEERECT (&rct, 0, di->cyScreen/8, di->cxScreen, di->cyScreen/4); 
               IDATECTL_SetRect(pIDateCtl, &rct);
               IDATECTL_Redraw (pIDateCtl);

               // Step 5: Set rectangle and redraw the date-pick control object.
               SETAEERECT (&rct, 0, di->cyScreen/3, di->cxScreen, di->cyScreen*3/4); 
               IDATECTL_SetRect(pIDatePickCtl, &rct);
               IDATECTL_Redraw (pIDatePickCtl);

               //
               // At this point the newly set values of both the objects will
               // be displayed on the screen. The date control object will be 
               // displayed in the top section of the screen and the date-pick
               // object will be displayed below the date contorl object.
               //

               // Step 6: Release IDateCtl objects. This step needs to be done
               //         only if no further use of the IDateCtl objects are needed.
               IDATECTL_Release (pIDateCtl);
               IDATECTL_Release (pIDatePickCtl);
            }
            else
            {
               // Atleast one of the objects did not get created. Release the
               // other one if it did get created.
               if (pIDateCtl != NULL)
                  IDATECTL_Release (pIDateCtl);
               if (pIDatePickCtl != NULL)
                  IDATECTL_Release (pIDatePickCtl);
            }
         }
         break;
      case USAGE_GETDATE:
         {
            // 17. Usage: IDATECTL_GetDate
            IDateCtl * pIDateCtl = NULL;
            IDateCtl * pIDatePickCtl = NULL;
            boolean    bRetVal = FALSE;
            int        nYear  = 2001;
            int        nMonth = 9;
            int        nDay   = 10;
            char       szBuf [30] = {0};

            // In this section we create date and date-pick control
            // objects and demonstrate the use of IDATECTL_GetDate
            // function in both the date and date-pick modes. The key steps 
            // of the following section are given below.
            //
            // Step 1. Create a date control and a date-pick control object.
            // Step 2. Set date for the date control object. The dates set
            //         are 5/23/2002 and 6/24/2003 for date and date-pick 
            //         respectively.
            // Step 3. Get date of the date control object.
            // Step 4. Display the dates retrieved in the previous step.
            // Step 5. Get date of the date-pick control object.
            // Step 6. Display the dates retrieved in the previous step.
            // Step 7. Release the date control objects.

            // Step 1: Create a date control and a date-pick control object.
            ISHELL_CreateInstance(pIShell, AEECLSID_DATECTL, (void **)&pIDateCtl);
            ISHELL_CreateInstance(pIShell, AEECLSID_DATEPICKCTL, (void **)&pIDatePickCtl); 

            if (pIDateCtl && pIDatePickCtl)
            {
               // Step 2: Set dates of date and date-pick objects.
               IDATECTL_SetDate(pIDateCtl, 2002, 5, 23);
               IDATECTL_SetDate(pIDatePickCtl, 2003, 6, 24);

               // Step 3: Get Date of date object.
               if ((bRetVal = IDATECTL_GetDate(pIDateCtl, &nYear, &nMonth, &nDay))
                  == FALSE)
               {
                  // Release date controls
                  IDATECTL_Release (pIDateCtl);
                  IDATECTL_Release (pIDatePickCtl);
                  return;
               }

               // Step 4. Display the dates retrieved in the previous step.
               SPRINTF (szBuf, "Date: %i/%i/%i", nMonth, nDay, nYear);
               DisplayOutput (pMe, 2, szBuf);

               // Step 5: Get date of date-pick object.
               if ((bRetVal = IDATECTL_GetDate(pIDatePickCtl, &nYear, &nMonth, &nDay))
                  == FALSE)
               {
                  // Release Date controls.
                  IDATECTL_Release (pIDateCtl);
                  IDATECTL_Release (pIDatePickCtl);
                  return;
               }

               // Step 6. Display the dates retrieved in the previous step.
               SPRINTF (szBuf, "DatePick: %i/%i/%i", nMonth, nDay, nYear);
               DisplayOutput (pMe, 4, szBuf);

               // Step 7: Release IDateCtl objects. This step needs to be done
               //         only if no further use of the IDateCtl objects are needed.
               IDATECTL_Release (pIDateCtl);
               IDATECTL_Release (pIDatePickCtl);
            }
            else
            {
               // Atleast one of the objects did not get created. Release the
               // other one if it did get created.
               if (pIDateCtl != NULL)
                  IDATECTL_Release (pIDateCtl);
               if (pIDatePickCtl != NULL)
                  IDATECTL_Release (pIDatePickCtl);
            }
         }
         break;
      case USAGE_GETDAYOFWEEK:
         {
            // 18. Usage: IDATECTL_GetDayOfWeek
            IDateCtl * pIDateCtl = NULL;
            IDateCtl * pIDatePickCtl = NULL;
            uint16     dayOfWeek = 0;
            char       szBuf[30] = {0};

            // In this section we create date and date-pick control
            // objects and demonstrate the use of IDATECTL_GetDate
            // function in both the date and date-pick modes. The key 
            // steps of the following section are given below.
            //
            // Step 1. Create a date control and a date-pick control object.
            // Step 2. Get DayOfWeek value from the date control object. The
            //         value retrieved will be the default value set when the
            //         object was created.
            // Step 3. Display DOW of date control object
            // Step 4. Get DayOfWeek value from the date-pick object
            // Step 5. Display the DOW of the  date-pick object value
            //         on the screen
            // Step 6. Release the date control objects.

            // Step 1: Create a date control and a date-pick control object.
            ISHELL_CreateInstance(pIShell, AEECLSID_DATECTL, (void **)&pIDateCtl);
            ISHELL_CreateInstance(pIShell, AEECLSID_DATEPICKCTL, (void **)&pIDatePickCtl); 

            if (pIDateCtl && pIDatePickCtl)
            {
               // Step 2. 
               //
               // IDATECTL_GetDayOfWeek: is used to get day of week from the 
               // date control object. In this case the default DayOfWeek 
               // will be the value that will be returned.
               dayOfWeek = IDATECTL_GetDayOfWeek(pIDateCtl);

               // Step 3: Display DOW of date control object.
               SPRINTF (szBuf, "DayOfWeek (Date): %u", dayOfWeek);
               DisplayOutput (pMe, 2, szBuf);

               // Step 4: Get DayOfWeek value from the date-pick object.
               dayOfWeek = IDATECTL_GetDayOfWeek(pIDateCtl);

               // Step 5: Display the DOW of the  date-pick object value on the screen
               SPRINTF (szBuf, "DayofWeek (DatePick): %u", dayOfWeek);
               DisplayOutput (pMe, 4, szBuf);

               // Step 6: Release IDateCtl objects. This step needs to be done
               //         only if no further use of the IDateCtl objects are needed.
               IDATECTL_Release (pIDateCtl);
               IDATECTL_Release (pIDatePickCtl);
            }
            else
            {
               // Atleast one of the objects did not get created. Release the
               // other one if it did get created.
               if (pIDateCtl != NULL)
                  IDATECTL_Release (pIDateCtl);
               if (pIDatePickCtl != NULL)
                  IDATECTL_Release (pIDatePickCtl);
            }
         }
         break;
      case USAGE_GETDAYSTRING:
      case USAGE_GETMONTHSTRING:
         {
            // 19. Usage: IDATECTL_GetDayString
            IDateCtl * pIDateCtl = NULL;
            IDateCtl * pIDatePickCtl = NULL;
            int        nLength = 0;
            AECHAR     dayString[20] = {0};
            char       szBuf1 [20] = {0};
            char       szBuf2 [40] = {0};
            AECHAR   * pAECh; 

            // In this section we create date and date-pick control
            // objects and demonstrate the use of IDATECTL_GetDayString
            // and IDATECTL_GetMonthString functions, in both the date and 
            // date-pick modes. The key steps of the following section are 
            // given below.
            //
            // Step 1. Create a date control and a date-pick control object.
            // Step 2. Set dates of both the objects.
            // Step 3. Invoke IDATECTL_GetDayString function using the date
            //         control object.
            // Step 4. Convert date string to short string format and prepare
            //         a string to display. The wide string to short string 
            //         conversion (WSTR_TO_STR) is done for convenience only. 
            //         Here DisplayOutput function used to display the string, 
            //         takes short string as input. 
            // Note: Ordinarily the string conversion is not needed since 
            //       strings are displayed using IDISPLAY_DrawText which 
            //       takes wide string as input. In this case the string
            //       conversion is done for convenience.
            // Step 5. Display the day string
            // Step 6. Invoke IDATECTL_GetDayString function to get the day 
            //         string of the date-pick control object.
            // Step 7. Display output similar to Step 4 and 5.
            // Step 8. Get month string of the date control object using
            //         the IDATECTL_GetMonthString function..
            // Step 9. Display month string of the date control object.
            // Step 10. Get month string of the date-pick control object using
            //          the IDATECTL_GetMonthString function.
            // Step 11. Display month string of the date-pick control object.

            // Step 12. Release both objects.

            // Step 1: Create a date control and a date-pick control object.
            ISHELL_CreateInstance(pIShell, AEECLSID_DATECTL, (void **)&pIDateCtl);
            ISHELL_CreateInstance(pIShell, AEECLSID_DATEPICKCTL, (void **)&pIDatePickCtl); 

            if (pIDateCtl && pIDatePickCtl)
            {
               // Step 2: Set dates for both the date and date-pick objects.
               IDATECTL_SetDate(pIDateCtl, 2000, 11, 11);
               IDATECTL_SetDate(pIDatePickCtl, 2000, 12, 12);

               //======================================
               // GetDayString: 
               //======================================
               //
               // Step 3. GetDayString for date control object.
               //
               // IDATECTL_GetDayString: is used to get the name of the day 
               // corresponding to the date control object's current date.
               if ((pAECh =IDATECTL_GetDayString(pIDateCtl, dayString, 
                  sizeof (dayString), &nLength)) == NULL)
               {
                  // Release both objects
                  IDATECTL_Release (pIDateCtl);
                  IDATECTL_Release (pIDatePickCtl);
                  return;
               }

               // Step 4. Convert day string to short string format for display. 
               //         The wide string to short string conversion (WSTR_TO_STR) 
               //         is done for convenience only. Here DisplayOutput function 
               //         used to display the string, takes short string as input. 
               // Note: Ordinarily the string conversion is not needed since 
               //       strings are displayed using IDISPLAY_DrawText which 
               //       takes wide string as input. In this case the string
               //       conversion is done for convenience.
               STRCPY (szBuf2, "Date (D): ");
               WSTR_TO_STR (dayString, szBuf1, sizeof(szBuf1));
               STRCAT (szBuf2, szBuf1);

               // Step 5. Display string buffer.
               DisplayOutput (pMe, 2, szBuf2);

               // Step 6. Get day string for date-pick control
               if ((pAECh = IDATECTL_GetDayString(pIDatePickCtl, dayString, 
                  sizeof (dayString), &nLength)) == NULL)
               {
                  // Release both objects
                  IDATECTL_Release (pIDateCtl);
                  IDATECTL_Release (pIDatePickCtl);
                  return;
               }

               // Step 7. Display output similar to Step 4 and 5.
               STRCPY (szBuf2, "DatePick (D): ");
               WSTR_TO_STR (dayString, szBuf1, sizeof(szBuf1));
               STRCAT (szBuf2, szBuf1);
               DisplayOutput (pMe, 3, szBuf2);

               //======================================
               // GetMonthString
               //======================================
               //
               // Step 8. Get month string of the date control object.
               // IDATECTL_GetMonthString: is used to get the name of the 
               // month of the date control object's current date.
               if ((pAECh = IDATECTL_GetMonthString(pIDateCtl, dayString,
                  sizeof (dayString), &nLength)) == NULL)
               {
                  // Release both objects
                  IDATECTL_Release (pIDateCtl);
                  IDATECTL_Release (pIDatePickCtl);
                  return;
               }

               // Step 9. Display month string of the date control object.
               //         Use conversions as in Steps 4 and 5.
               STRCPY (szBuf2, "Date (M): ");
               WSTR_TO_STR (dayString, szBuf1, sizeof(szBuf1));
               STRCAT (szBuf2, szBuf1);
               DisplayOutput (pMe, 4, szBuf2);

               // Step 10. Get Month string of the date-pick object.
               if ((pAECh = IDATECTL_GetMonthString(pIDatePickCtl, dayString,
                  sizeof (dayString), &nLength)) == NULL)
               {
                  // Release both objects
                  IDATECTL_Release (pIDateCtl);
                  IDATECTL_Release (pIDatePickCtl);
                  return;
               }

               // Step 11. Display the month string of the date-pick object.
               //         Use conversions as in Steps 4 and 5.
               STRCPY (szBuf2, "DatePick (M): ");
               WSTR_TO_STR (dayString, szBuf1, sizeof(szBuf1));
               STRCAT (szBuf2, szBuf1);
               DisplayOutput (pMe, 5, szBuf2);

               // Step 12: Release IDateCtl objects. This step needs to be done
               //         only if no further use of the IDateCtl objects are needed.
               IDATECTL_Release (pIDateCtl);
               IDATECTL_Release (pIDatePickCtl);
            }
            else
            {
               // Atleast one of the objects did not get created. Release the
               // other one if it did get created.
               if (pIDateCtl != NULL)
                  IDATECTL_Release (pIDateCtl);
               if (pIDatePickCtl != NULL)
                  IDATECTL_Release (pIDatePickCtl);
            }
         }
         break;
      case USAGE_GETDATESTRING:
         {
            // 20. Usage: IDATECTL_GetDateString
            IDateCtl * pIDateCtl = NULL;
            int        nLength = 0;
            AECHAR     dateString[20] = {0};
            char       szBuf1 [20] = {0};
            char       szBuf2 [40] = {0};
            boolean    bRetValue = FALSE;
 
            // In this section we create date control object and 
            // demonstrate the use of IDATECTL_GetDateString
            // function in the date control mode. The key 
            // steps of the following section are given below.
            // 
            // Note: The date string returned are the same for both
            //       DATECTL and DATEPICKCTL modes. Hence, only the 
            //       date control is used in the example below.
            //
            // Step 1. Create a date control  object.
            // Step 2: Set date for the date control object.
            // Step 3. Invoke IDATECTL_GetDateString function using the date
            //         control object. Here, the format of the date string
            //         will be in the DFMT_DD_MONTH_YYYY format.
            // Step 4. Convert date string to short string format and prepare
            //         a string to display. The wide string to short string 
            //         conversion (WSTR_TO_STR) is done for convenience only. 
            //         Here DisplayOutput function used to display the string, 
            //         takes short string as input. 
            // Note: Ordinarily the string conversion is not needed since 
            //       strings are displayed using IDISPLAY_DrawText which 
            //       takes wide string as input. In this case the string
            //       conversion is done for convenience.
            // Step 5. Display the date string
            // Step 6. Get date string again using the DFMT_DD_MON_YYYY 
            //         flag. The date returned is in the "18 Jul 2000" format.
            // Step 7.  Display the date string on the next line on the screen.
            // Step 8. Get date string in the DFMT_MONTH_DD_YYYY format. 
            //         The date returned is in the "July 18, 2000" format.
            // Step 9. Display date string on the screen.
            // Step 10. Get date string again using the DFMT_MON_DD_YY 
            //         flag. The date returned is in the "Jul 18, '00" format.
            // Step 11. Display date string in the DFMT_MON_DD_YY format.
            // Step 12. Release the date control object.

            // Step 1: Create a date control and a date-pick control object.
            ISHELL_CreateInstance(pIShell, AEECLSID_DATECTL, (void **)&pIDateCtl);

            if (pIDateCtl)
            {
               // Step 2: Set date for the date control object.
               IDATECTL_SetDate(pIDateCtl, 2001, 7, 11);

               //======================================
               // GetDateString
               //======================================
               //
               // Step 3. Get Date String
               // IDATECTL_GetDateString: is used to get date string in specified 
               // format. The format specified here is DFMT_DD_MONTH_YYYY which 
               // returns the date string in "18 July 2000" format.
               if ((bRetValue = IDATECTL_GetDateString(pIDateCtl, dateString,
                  sizeof(dateString), &nLength, DFMT_DD_MONTH_YYYY)) == NULL)
               {
                  IDATECTL_Release (pIDateCtl);
                  return;
               }

               // Step 4. Display the date string.
               STRCPY (szBuf2, "Date: ");
               WSTR_TO_STR (dateString, szBuf1, sizeof(szBuf1));
               STRCAT (szBuf2, szBuf1);

               // Step 5. Display date string in the "18 July 2000" format.
               DisplayOutput (pMe, 2, szBuf2);

               // Step 6. Get date string in the DFMT_DD_MON_YYYY format. 
               //         The date returned is in the "18 Jul 2000" format.
               if ((bRetValue =IDATECTL_GetDateString(pIDateCtl, dateString,
                  sizeof(dateString), &nLength, DFMT_DD_MON_YY)) == NULL)
               {
                  IDATECTL_Release (pIDateCtl);
                  return;
               }

               // Step 7.  Display the date string on the next line on the screen.
               STRCPY (szBuf2, "Date: ");
               WSTR_TO_STR (dateString, szBuf1, sizeof(szBuf1));
               STRCAT (szBuf2, szBuf1);
               DisplayOutput (pMe, 3, szBuf2);

               // Step 8. Get date string in the DFMT_MONTH_DD_YYYY format. 
               //         The date returned is in the "July 18, 2000" format.
               if ((bRetValue =IDATECTL_GetDateString(pIDateCtl, dateString,
                  sizeof(dateString), &nLength, DFMT_MONTH_DD_YYYY)) == NULL)
               {
                  IDATECTL_Release (pIDateCtl);
                  return;
               }

               // Step 9. Display date string on the screen.
               STRCPY (szBuf2, "Date: ");
               WSTR_TO_STR (dateString, szBuf1, sizeof(szBuf1));
               STRCAT (szBuf2, szBuf1);
               DisplayOutput (pMe, 4, szBuf2);

               // Step 10. Get date string again using the DFMT_MON_DD_YY 
               //         flag. The date returned is in the "Jul 18, '00" format.
               if ((bRetValue =IDATECTL_GetDateString(pIDateCtl, dateString,
                  sizeof(dateString), &nLength, DFMT_MON_DD_YY)) == NULL)
               {
                  IDATECTL_Release (pIDateCtl);
                  return;
               }

               // Step 11. Display date string in the DFMT_MON_DD_YY format.
               STRCPY (szBuf2, "Date: ");
               WSTR_TO_STR (dateString, szBuf1, sizeof(szBuf1));
               STRCAT (szBuf2, szBuf1);
               DisplayOutput (pMe, 5, szBuf2);

               // Step 12. Release date control object.
               IDATECTL_Release (pIDateCtl);
            }
         }
         break;
      case USAGE_ENABLECOMMAND:
         {
            // 21. Usage: IDATECTL_EnableCommand
            IDateCtl * pIDateCtl = NULL;
            IDateCtl * pIDatePickCtl = NULL;
            uint16     nCmdId = 0;       // User defined command.
 
            // In this section we create date and date-pick control 
            // objects and demonstrate the use of IDATECTL_EnableCommand
            // function.

            //Create IDateCtl object. This step needs to be done only if
            //the IDateCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_DATECTL, (void **)&pIDateCtl);
            ISHELL_CreateInstance(pIShell, AEECLSID_DATEPICKCTL, (void **)&pIDatePickCtl); 

            if (pIDateCtl && pIDatePickCtl)
            {
               // Invoke EnableCommand. This method is used to enable the date 
               // control object to send a user defined command to the active 
               // applet. If bEnable is TRUE, upon receiving the event generated 
               // by press of select key, the date control object sends nCmdId 
               // as EVT_COMMAND to the active applet. Handling of this event is 
               // applet specific and is user defined.
               IDATECTL_EnableCommand (pIDateCtl, TRUE, nCmdId);
               IDATECTL_EnableCommand (pIDatePickCtl, TRUE, nCmdId);

               // No display output...

               // Release IDateCtl objects. This step needs to be done
               // only if no further use of the IDateCtl objects are needed.
               IDATECTL_Release (pIDateCtl);
               IDATECTL_Release (pIDatePickCtl);
            }
         }
         break;
      case USAGE_SETACTIVEDAYMASK:
         {
            // 22. Usage: IDATECTL_SetActiveDayMask
            IDateCtl * pIDatePickCtl = NULL;
            uint32     dwMask = 0xf0f0f0;
            AEERect    rc;
 
            // In this section we create date-pick control object and 
            // demonstrate the use of IDATECTL_SetActiveDayMask
            // function in the date-pick mode. 

            // Note: This function has NO EFFECT in the date control mode.
            //       Hence, no date control example will be provided.
            //       
            // In this example we set a mask of 0xf0f0f0 which correspond to
            // the days 5-8, 13-16, and 21-24 being drawn in reverse video.
            // The date and the recatangle is set in the date-pick object.
            // The IDATECTL_SetActiveDayMask function is invoked with the 
            // mask given above. The date-pick object is redrawn so that 
            // the date-pick object is drawn on the screen with the dates
            // indicated above in reverse video. Finally, the date-pick 
            // object is released.

            //Create IDatePickCtl object. This step needs to be done only if
            //the IDatePickCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_DATEPICKCTL, (void **)&pIDatePickCtl); 
	 
            if (pIDatePickCtl)
            {	
               // Set date and rectangle for display purposes.
               IDATECTL_SetDate(pIDatePickCtl, 2000, 9, 9);
               SETAEERECT (&rc, 0, lineHeight, di->cxScreen, di->cyScreen - 2*lineHeight);
               IDATECTL_SetRect(pIDatePickCtl, &rc);

               // IDATECTL_SetActiveDayMask: sets a new active day mask. Active 
               // day mask is 32 bit integer in which each of 0 to 30 bits 
               // specifies active status of a day. An active day, having bit 
               // corresponding to it set, is drawn in reverse video.
               IDATECTL_SetActiveDayMask (pIDatePickCtl, dwMask);

               // Set object active.
               IDATECTL_Redraw (pIDatePickCtl);

               // Release IDateCtl object. This step needs to be done
               // only if no further use of the IDateCtl object is needed.
               IDATECTL_Release (pIDatePickCtl);
            }	
         }
         break;
      default:
         return;
   }
   
   // Display above event. 
   DisplayEvent (pMe, wParam);

   return;
}

/*===========================================================================

FUNCTION: DisplayEvent

DESCRIPTION
    This function is a display function which sets up the initial screen
    for the display output for a given interface function. This displays
    the function name at the top of the screen and displays the
    strings "Press CLR to continue.." at the bottom of 
    the screen. In the case of functions that have no output it also 
    displays the string "No Output..." in the middile of the screen.

PROTOTYPE:
   void DisplayEvent (CIDateCtlApp *pMe, uint16 wParam)

PARAMETERS:
   pMe:    [in]: Pointer to CIDateCtlApp interface object
   wParam: [in]: API function type Id selected by user

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void DisplayEvent (CIDateCtlApp *pMe, uint16 wParam)
{
   AECHAR szBuf[30];
   AEERect rc; 

   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
      return;

   // Set display rectangle
   SETAEERECT (&rc, 0, 0, pMe->m_dInfo.cxScreen, pMe->m_dInfo.cyScreen);
   
   if (pMe->a.m_pIDisplay)
   {
      switch (wParam)
      {
         case USAGE_BASIC_USAGE_DATE:
            STR_TO_WSTR("Basic Usage (DateCtl):", szBuf, sizeof(szBuf));
            break;
         case USAGE_BASIC_USAGE_DATEPICK:
            STR_TO_WSTR("Basic Usage (DatePick):", szBuf, sizeof(szBuf));
            break;
         case USAGE_HANDLEEVENT_DATE:
            STR_TO_WSTR("HandleEvent (Date)", szBuf, sizeof(szBuf));
            break;
         case USAGE_HANDLEEVENT_DATEPICK:
            STR_TO_WSTR("HandleEvent (Date Pick)", szBuf, sizeof(szBuf));
            break;
         case USAGE_REDRAW:
            STR_TO_WSTR("Redraw", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETACTIVE_DATE:
            STR_TO_WSTR("SetActive (date)", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETACTIVE_DATEPICK:
            STR_TO_WSTR("SetActive (date-pick)", szBuf, sizeof(szBuf));
            break;
         case USAGE_ISACTIVE:
            STR_TO_WSTR("IsActive", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETRECT:
            STR_TO_WSTR("SetRect", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETRECT:
            STR_TO_WSTR("GetRect", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETPROPERTIES:
            STR_TO_WSTR("SetProperties", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETPROPERTIES:
            STR_TO_WSTR("GetProperties", szBuf, sizeof(szBuf));
            break;
         case USAGE_RESET:
            STR_TO_WSTR("Reset", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETTITLE:
            STR_TO_WSTR("SetTitle", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETJULIANDAY:
            STR_TO_WSTR("SetJulianDay", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETJULIANDAY:
            STR_TO_WSTR("GetJulianDay", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETDATE:
            STR_TO_WSTR("SetDate", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETDATE:
            STR_TO_WSTR("GetDate", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETDAYOFWEEK:
            STR_TO_WSTR("GetDayofWeek", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETDAYSTRING:
            STR_TO_WSTR("GetDayString", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETMONTHSTRING:
            STR_TO_WSTR("GetMonthString", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETDATESTRING:
            STR_TO_WSTR("GetDateString", szBuf, sizeof(szBuf));
            break;
         case USAGE_ENABLECOMMAND:
            STR_TO_WSTR("EnableCommand", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETACTIVEDAYMASK:
            STR_TO_WSTR("SetActiveDayMask", szBuf, sizeof(szBuf));
            break;
         default:
            STR_TO_WSTR("Unsupported fn", szBuf, sizeof(szBuf));
      }
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,5, 0, &rc,
		   IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);

      // Functions with no output
      switch (wParam)
      {
         case USAGE_GETRECT:
         case USAGE_ENABLECOMMAND:
            // Place in the middle of screen
            STR_TO_WSTR("No Output...", szBuf, sizeof(szBuf));
 		      IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,10, 
               rc.dy/2 - pMe->m_nLineHeight, &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
         default:
            break;
      }
      STR_TO_WSTR("Press CLR to continue..", szBuf, sizeof(szBuf));
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,5, rc.dy - 
         pMe->m_nLineHeight, &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
		IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);
   }
}

/*===========================================================================

FUNCTION: BuildMainMenu

DESCRIPTION
    This function builds the main menu when this applet is started.
    It displays a simple menu with menu items corresponding to 
    each of the interface functions.
    
PROTOTYPE:
   void BuildMainMenu(CIDateCtlApp *pMe)

PARAMETERS:
   pMe: [in]: Pointer to AEEApplet struct

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void BuildMainMenu(CIDateCtlApp *pMe)
{
	AEERect rc;
   AECHAR szBuf[50];

   if (pMe->m_pIMenu == NULL || pMe->a.m_pIShell == NULL)
      return;

   // Set display rectangle
   SETAEERECT (&rc, 0, 0, pMe->m_dInfo.cxScreen, pMe->m_dInfo.cyScreen);

	// Set Title
   STR_TO_WSTR("IDateCtl Examples", szBuf, sizeof(szBuf));
	IMENUCTL_SetTitle(pMe->m_pIMenu, NULL, 0, szBuf);

	//Set the Rectangle for the Menu
	IMENUCTL_SetRect(pMe->m_pIMenu, &rc);  

	//Add invidual entries to the Menu

   // Add Basic Usage to menu
   STR_TO_WSTR("1. Basic Use(Date)", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_BASIC_USAGE_DATE, szBuf, 0);

   // Add Basic Usage to menu
   STR_TO_WSTR("2. Basic Use(DatePick)", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_BASIC_USAGE_DATEPICK, szBuf, 0);

   // Add Handle Event to menu
   STR_TO_WSTR("3. HandleEvent (Date)", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_HANDLEEVENT_DATE, szBuf, 0);

   // Add Handle Event to menu
   STR_TO_WSTR("4. HandleEvent (DatePick)", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_HANDLEEVENT_DATEPICK, szBuf, 0);

   // Add Redraw to menu
   STR_TO_WSTR("5. Redraw", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_REDRAW, szBuf, 0);
			
   // Add SetActive to menu
   STR_TO_WSTR("6. SetActive (Date)", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETACTIVE_DATE, szBuf, 0);

   // Add SetActive to menu
   STR_TO_WSTR("7. SetActive (DatePick)", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETACTIVE_DATEPICK, szBuf, 0);

   // Add IsActive to menu
   STR_TO_WSTR("8. IsActive", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_ISACTIVE, szBuf, 0);

	// Add SetRect to menu
   STR_TO_WSTR("9. SetRect", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETRECT, szBuf, 0);

	// Add GetRect to menu
   STR_TO_WSTR("10. GetRect", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETRECT, szBuf, 0);	

	// Add Set Properties to menu
   STR_TO_WSTR("11. SetProperties", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPROPERTIES, szBuf, 0);	

	// Add GetProperties to menu
   STR_TO_WSTR("12. GetProperties", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETPROPERTIES, szBuf, 0);	

	// Add Reset to menu
   STR_TO_WSTR("13. Reset", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_RESET, szBuf, 0);	

   // Add SetTitle to menu
   STR_TO_WSTR("14. SetTitle", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETTITLE, szBuf, 0);	

   // Add SetJulianDate to menu
   STR_TO_WSTR("15. SetJulianDay", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETJULIANDAY, szBuf, 0);	

   // Add GetJulianDate to menu
   STR_TO_WSTR("16. GetJulianDay", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETJULIANDAY, szBuf, 0);	

	// Add SetDate to menu
   STR_TO_WSTR("17. SetDate", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETDATE, szBuf, 0);	

	// Add GetDate to menu
   STR_TO_WSTR("18. GetDate", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETDATE, szBuf, 0);	

	// Add GetDayOfWeek to menu
   STR_TO_WSTR("19. GetDayOfWeek", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETDAYOFWEEK, szBuf, 0);	

	// Add GetDayString to menu
   STR_TO_WSTR("20. GetDayString", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETDAYSTRING, szBuf, 0);	

	// Add GetMonthString to menu
   STR_TO_WSTR("21. GetMonthString", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETMONTHSTRING, szBuf, 0);	

	// Add GetDateString to menu
   STR_TO_WSTR("22. GetDateString", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETDATESTRING, szBuf, 0);	

	// Add EnableCommand to menu
   STR_TO_WSTR("23. EnableCommand", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_ENABLECOMMAND, szBuf, 0);	

	// Add SetActiveDayMask to menu
   STR_TO_WSTR("24. SetActiveDayMask", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETACTIVEDAYMASK, szBuf, 0);	

   // Activate menu
	IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);
}

/*===========================================================================

FUNCTION: DisplayOutput

DESCRIPTION
    This function displays an output string at a given line number on the
    screen. If the nline parameter is a negative value (-1) the string
    is displayed in the middle of the screen. If the "nline" value is larger
    than or equal to zero the "nline" value is multiplied by 15 and the 
    resulting value in pixels is set to the y-coordinate of the start of 
    the string display on the screen. If the string does not fit on one line
    the string wraps around to the next line (spaced rougly 10-15 pixels apart).
    By default 5 is used as the starting the x-coordinate of a displayed 
    string.

    How many characters that fit on one line is calculated for each line 
    that is wrapped around to the next line.

    Note: depending on the phone screen size and the fonts used for characters 
          the output might differ on different handsets (devices). Where some 
          handsets will have a smaller screen and large default fonts which will 
          cause partial overlapping of lines. This function does not try to address
          these issues (this is meant as a simple display function).
    
PROTOTYPE:
   void DisplayOutput(CIDateCtlApp * pi, int nline, char *pszStr)

PARAMETERS:
   pi:     [in]: Contains a pointer to the IApplet class.
   nline:  [in]: Contains the line number to start displaying the text. 
        The line numbers are by default spaced 15 pixels apart along the y-axis.
   pszStr: [in]: The character string to be displayed on the screen.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void DisplayOutput(CIDateCtlApp * pMe, int nline, char *pszStr)
{
   AECHAR * pszBuf = NULL;
   AECHAR * psz = NULL;
   int pixelWidth;
   AEEFont font = AEE_FONT_NORMAL;
   int pnFits = 0, dy;
   int totalCh = 0;

   if (pMe == NULL)
      return;

   // Allocate buffer to hold string.
   if ((pszBuf = (AECHAR *)MALLOC(200)) == NULL)
      return;

   // Convert to wide string (unicode)
   STR_TO_WSTR ((char *)pszStr, pszBuf, 200);

   // If nlines is zero then print this string starting around the middle of 
   // the screen. Or else multiply nlines by 10 to decide the y coordinate of
   // the start of the string.
   if (nline < 0) {
      dy = pMe->m_dInfo.cyScreen*2/5;
   }
   else{
      dy = nline * pMe->m_nLineHeight;
   }

   // psz keeps track of the point from which to write from the string buffer
   // in case the string does not fit one line and needs to wrap around in the
   // next line.
   psz = pszBuf;
      
   // Need to calculate the lotal string length to decide if any wrapping
   // around is needed.
   totalCh = STRLEN ((char *)pszStr);

   // Keep displaying text string on multiple lines if the string can't be displayed
   // on one single line. Lines are spaced 15 pixels apart.
   while ((totalCh > 0) && (*psz != NULL))
   { 
      // Get information on how many characters will fit in a line.
      // Give the pointer to the buffer to be displayed, and the number of
      // pixels along the x axis you want to display the string in (max number)
      // pnFits will have the max number of chars that will fit in the maxWidth
      // number of pixels (given string can't fit in one line), or the number of 
      // chars in the string (if it does fit in one line). pnWidth gives the
      // number of pixels that will be used to display pnFits number of chars.
      pixelWidth = IDISPLAY_MeasureTextEx(pMe->a.m_pIDisplay,
                      font, 
                      (AECHAR *) psz,  // Start of the buffer to display,
                      -1,
                      pMe->m_dInfo.cxScreen - 5, // maxWidth
                      &pnFits);         // Number of chars that will fit a line

      // If pnFits is zero there is something wrong in the input to above function. 
      // Normally this scenario should not occur. But, have the check anyway.
      if (pnFits == 0)
      {
         FREE (pszBuf);
         return;
      }

      IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, psz, pnFits, 5 /*start dx*/, 
         dy, 0 /* use default rectangle coordinates */, 0);

      psz += pnFits;      // move pointer to the next segment to be displayed
      totalCh -= pnFits;  // reduce the total number of characters to still display
      dy += pMe->m_nLineHeight; // Place next line charHeight pixels below the 
                                // previous line. 
      IDISPLAY_Update(pMe->a.m_pIDisplay);
      if (totalCh < pnFits)
         pnFits = totalCh;  // if total number is less than pnFits, adjust pnFits
   }

   FREE (pszBuf);
   return;   
} // End of DisplayOutput

