/*===========================================================================

FILE: idbusage.c

SERVICES: Sample applet using AEE

DESCRIPTION
  This file contains usage examples of IDBMgr/IDatabase/IDBRecord interfaces. 
  IDBMgr/IDatabase/IDBRecord interface functions allow the user to create/operate 
  on Database objects. 

PUBLIC CLASSES:  
   N/A


   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/

/*===============================================================================
                     INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "AEEModGen.h"
#include "AEEAppGen.h"
#include "AEEMenu.h"
#include "AEEStdLib.h"
#include "AEEUsageAppIDs.h"
#include "AEEDB.h"

/*===========================================================================

                      PUBLIC DATA DECLARATIONS

===========================================================================*/

/*-------------------------------------------------------------------
            Type Declarations
-------------------------------------------------------------------*/
// Database app struct. This is the main struct for this applet. This will hold 
// all the data members that needs to be remembered throughout the life of
// the applet. 
// THE FIRST DATA MEMBER OF THIS STRUCT MUST BE AN AEEApplet OBJECT. 
// This is critical requirement which is related to an applet being
// dynamically loaded. The app developers should add their data members
// following the AEEApplet data member.
typedef struct _CIDBApp
{
	AEEApplet	a;        // Mandatory first AEEApplet data member
   IMenuCtl *  m_pIMenu;

   // Device parameters
   int             m_nLineHeight;
   AEEDeviceInfo   m_dInfo;

}CIDBApp;

/*-------------------------------------------------------------------
            Function Prototypes
-------------------------------------------------------------------*/
// App Handle Event function
static boolean DBApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);

// App specific data alloc-init/free functions
static boolean IDB_InitAppData(IApplet* pMe);
static void IDB_FreeAppData(IApplet* pMe);

static void BuildMainMenu(CIDBApp *pMe);
static void IDBUsage (CIDBApp * pMe, uint16 wParam);
static void DisplayEvent (CIDBApp *pMe, uint16 wParam);
static void DisplayOutput(CIDBApp * pMe, int nline, char *pszStr);

/*-------------------------------------------------------------------
            Global Constant Definitions
-------------------------------------------------------------------*/
// App Resource File
#define  APP_RES_FILE      "idbusage.bar"

// App specific constants
#define USAGE_BASIC_USAGE          100
#define USAGE_DBMGR_OPENDATABASE   101
#define USAGE_DBMGR_OPENDATABASEEX 102
#define USAGE_DBMGR_REMOVE         103
#define USAGE_DB_CREATERECORD      104
#define USAGE_DB_GETRECORDCOUNT    105
#define USAGE_DB_RESET             106
#define USAGE_DB_GETNEXTRECORD     107
#define USAGE_DB_GETRECORDBYID     108
#define USAGE_DBREC_RESET          109
#define USAGE_DBREC_UPDATE         110
#define USAGE_DBREC_REMOVE         111
#define USAGE_DBREC_GETID          112
#define USAGE_DBREC_NEXTFIELD      113
#define USAGE_DBREC_GETFIELD       114
#define USAGE_DBREC_GETFIELDWORD   115
#define USAGE_DBREC_GETFIELDDWORD  116
#define USAGE_DBREC_GETFIELDSTRING 117


/*===============================================================================
                     FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to IDB_InitAppData(). 
   This is done as follows: InitAppData() is called to initialize AppletData 
   instance. It is app developers responsibility to fill-in app data initialization 
   code of InitAppData(). App developer is also responsible to release memory 
   allocated for data contained in AppletData -- this can be done in 
   IDB_FreeAppData().

PROTOTYPE:
   int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell interface. 

	pIModule: pin]: Contains pointer to the IModule interface to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid IApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the app needs to be loaded and if AEEApplet_New() invocation was
     successful
  EFAILED: If the app does not need to be loaded or if errors occurred in 
     AEEApplet_New(). If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
{
   *ppObj = NULL;
		
   // We want to load this App. So, invoke AEEApplet_New().To it, pass the
   // address of the app-specific handle event function.
   if(ClsId == AEECLSID_DATABASE_APP)
   {
      if(AEEApplet_New(sizeof(CIDBApp), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)DBApp_HandleEvent,(PFNFREEAPPDATA)IDB_FreeAppData)
         == TRUE)
      {
         if (IDB_InitAppData((IApplet*)*ppObj) == TRUE)
         {
            return(AEE_SUCCESS);
         }
      }
   }
	return (EFAILED);
}

/*===========================================================================

FUNCTION DBApp_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean DBApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

	ecode: Specifies the Event sent to this applet

   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/
static boolean DBApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
   CIDBApp * pMe = (CIDBApp*)pi;

   switch (eCode) 
	{
      case EVT_APP_START:
         // Create the IMenu interface object. When the applet is started the 
         // main menu is set up. When the user selects a menu item from the main 
         // menu, an EVT_COMMAND event is received by this applet. The processing for 
         // EVT_COMMAND event is found below.
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, (void **)&pMe->m_pIMenu)
            != SUCCESS)
         {
				return FALSE;
         }

         // Build the main menu
			BuildMainMenu(pMe);
			return(TRUE);
      case EVT_APP_STOP:
         return(TRUE);
      case EVT_KEY: 
         // We come here when a key is pressed on the handset keypad. Call 
         // the IMenuCtl handle event function to handle this key press.
			if(pMe->m_pIMenu != NULL)
			{
				if ((IMENUCTL_IsActive(pMe->m_pIMenu) == FALSE) && ((wParam == AVK_UP) || (wParam == AVK_DOWN)))
				{
					IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);
					IMENUCTL_Redraw(pMe->m_pIMenu);
				}
				return IMENUCTL_HandleEvent(pMe->m_pIMenu, EVT_KEY, wParam, 0);
			}
         else
            return FALSE;
		case EVT_COMMAND:
         // When a menu item is selected this applet receives the
         // EVT_COMMAND event in the wParam field.
  		   switch(wParam)
         {
            case USAGE_BASIC_USAGE:
            case USAGE_DBMGR_OPENDATABASE:
            case USAGE_DBMGR_OPENDATABASEEX:
            case USAGE_DBMGR_REMOVE:
            case USAGE_DB_CREATERECORD:
            case USAGE_DB_GETRECORDCOUNT:
            case USAGE_DB_RESET:
            case USAGE_DB_GETNEXTRECORD:
            case USAGE_DB_GETRECORDBYID:
            case USAGE_DBREC_RESET:
            case USAGE_DBREC_UPDATE:
            case USAGE_DBREC_REMOVE:
            case USAGE_DBREC_GETID:
            case USAGE_DBREC_NEXTFIELD:
            case USAGE_DBREC_GETFIELD:
            case USAGE_DBREC_GETFIELDWORD:
            case USAGE_DBREC_GETFIELDDWORD:
            case USAGE_DBREC_GETFIELDSTRING:
               // Call the usage function that exercises all the 
               // interface functions.
               IDBUsage (pMe, wParam);
               return TRUE;
		   	default:
			   	return FALSE;
         }
      default:
         break;
   }
   return FALSE;
}



/*===========================================================================

FUNCTION IDB_InitAppData

DESCRIPTION
   This function initializes app specific data allocates memory for app data 
   (AppletData) and sets it to pAppData of AEEApplet. It also function . 
   App developer needs to fill-in the initialization code.

PROTOTYPE:
	boolean IDB_InitAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains 
   information specific to this applet. It was initialized during the 
   AEEClsCreateInstance().

DEPENDENCIES
   Assumes pi is not NULL

RETURN VALUE
  TRUE: If the app has app data is allocated and initialized successfully
  FALSE: Either app data could not be allocated or initialzied

SIDE EFFECTS
  none
===========================================================================*/
static boolean IDB_InitAppData(IApplet* pi)
{
   int pnAscent;        // Stores the ascent in number of pixels
   int pnDescent;       // Stores the descent in number of pixels

   CIDBApp * pMe = (CIDBApp*)pi;

   // Initialize the MenuCtl pointer to NULL
   pMe->m_pIMenu = NULL;

   // Get the font metrics info
   pMe->m_nLineHeight = IDISPLAY_GetFontMetrics (pMe->a.m_pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&pMe->m_dInfo);

   return TRUE;
}

/*===========================================================================

FUNCTION IDB_FreeAppData

DESCRIPTION
   Frees data contained in app data and memory for app data itself.
   App developer needs to free data contained in AppletData.

PROTOTYPE:
	void IDB_FreeAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains 
   information specific to this applet. It was initialized during the 
   AEEClsCreateInstance().

DEPENDENCIES
   Assumes pi is not NULL

RETURN VALUE
   None

SIDE EFFECTS
   None
===========================================================================*/
static void IDB_FreeAppData(IApplet* pi)
{
   CIDBApp * pMe = (CIDBApp*)pi;

   // Release main menu
   if (pMe->m_pIMenu != NULL)
   {
      IMENUCTL_Release (pMe->m_pIMenu);
      pMe->m_pIMenu = NULL;
   }   
}

/*===============================================================================
                     APP HELPER FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: IDBUsage

DESCRIPTION
	This function encompasses all the usage examples of all the functions
   in code blocks switched using the BREW API function Id passed into this
   function. The code blocks will include almost all the variable declarations
   needed to demonstrate the usage of a given function. Within the code block
   all the needed interfaces are instantiated and released subsequent to their
   use.

   There are some exceptions to the above general rule, such as call-back functions,
   common helper function, or common display functions called from within the
   code block which are defined elsewhere in this file.

   There are some instances where an object instantiated within a code block is
   not released at the end of the code block, to be released at a later point
   (for example in a call-back function, in this case pointers to such objects
   are stored in the AEEApplet object to be released later).
   
PROTOTYPE:
   void IDBUsage (CIDBApp * pMe, uint16 wParam)

PARAMETERS:
	pMe: [in]: Contains a pointer to the CIDBApp struct.
   wParam: [in]: Contains the Id of the function type to be used (contains the
      Id corresponding to the menu item selected on the emulator).

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the functions call was successful. FALSE otherwise

SIDE EFFECTS
  none
===========================================================================*/
static void IDBUsage (CIDBApp * pMe, uint16 wParam)
{
   IShell *pIShell = pMe->a.m_pIShell;

   // Erase screen first for display output purposes.
   IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
	IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
	IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);

   switch (wParam)
   {
      case USAGE_BASIC_USAGE:
         {
            // 1. Usage: Basic IDB Usage
            // This section tries to exercise each of the interface function
            // all IDBMgr, IDatabase, IDBRecord interfaces atleast once.
            // All the variables used in this example are defined within this
            // code block.
            IDBMgr    * pIDBMgr = NULL;
            IDatabase * pIDatabase = NULL;
            IDBRecord * pIDBRecord1 = NULL;
            IDBRecord * pIDBRecord2 = NULL;
            IDBRecord * pIDBRec1 = NULL;
            IDBRecord * pIDBRec2 = NULL;
            int nRetVal = EBADFILENAME;
            AEEDBField dbField1[2];      // data for record 1
            AEEDBField dbField2[3];      // data for record 2
            uint32 recordCount = 0;      // number of records in the database
            const char firstName [] = "John";    
            const char lastName  [] = "Smith";   
            const char fullName  [] = "Jane Smith";
            uint16 category  = 123;       // Category value
            uint32 homePhone = 7654321;  // Home phone number
            char szBuf[30]   = {0};      // temp buffer for printing 
            AEEDBFieldType fieldType;    // stores field type
            AEEDBFieldName fieldName;    // stores field name
            uint16  fieldLen, recId = 0;  
            byte  * data = NULL;
            uint16  wordValue = 0;
            uint32  dwordValue = 0;
            boolean bRetValue = FALSE;
            AECHAR * psz = NULL;

            // Data fill values used to create database records. 
            // The parameter dbField1 is a two item array of type 
            // AEEDBField. Each array item corresponds to two fields 
            // in a record, respectively. The parameter dbField2 is 
            // a three item array of type AEEDBField. Each array item 
            // corresponds to the three fileds in the record, respectively.
            //
            // The first record will contain one feild for a first
            // name and a second field for a last name of a person.
            // The second record will contain one field for a last
            // name, a second field for a category, and a third 
            // field for a home phone number.
            // 
            // Data fill record 1 field 1
            dbField1[0].fName = AEEDBFIELD_FIRSTNAME;
            dbField1[0].fType = AEEDB_FT_STRING;
            dbField1[0].pBuffer = (void *)firstName;
            dbField1[0].wDataLen = STRLEN (firstName);
            // Data fill record 1 field 2
            dbField1[1].fName = AEEDBFIELD_LASTNAME;
            dbField1[1].fType = AEEDB_FT_STRING;
            dbField1[1].pBuffer = (void *)lastName;
            dbField1[1].wDataLen = STRLEN (lastName);

            // Data fill record 2 field 1 
            dbField2[0].fName = AEEDBFIELD_LASTNAME;     
            dbField2[0].fType = AEEDB_FT_STRING;          
            dbField2[0].pBuffer = (void *)&fullName; 
            dbField2[0].wDataLen = STRLEN (fullName);
            // Data fill record 2 field 2 
            dbField2[1].fName = AEEDBFIELD_CATEGORY;
            dbField2[1].fType = AEEDB_FT_WORD;
            dbField2[1].pBuffer = (void *)&category;
            dbField2[1].wDataLen = sizeof (uint16);
            // Data fill record 2 field 3
            dbField2[2].fName = AEEDBFIELD_HOME_PHONE;
            dbField2[2].fType = AEEDB_FT_DWORD;
            dbField2[2].pBuffer = (void *)&homePhone;
            dbField2[2].wDataLen = sizeof (uint32);

            // Create IDBMgr object. This step needs to be done only if
            // the IDBMgr object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_DBMGR, (void **)&pIDBMgr);

            if (pIDBMgr == NULL)
               return;

            // Create a database.
            if ((pIDatabase = IDBMGR_OpenDatabase (pIDBMgr, "db", TRUE)) == NULL)
            {
               // Release IDBMgr Interface and return. 
               IDBMGR_Release (pIDBMgr);
               return;
            }

            // Create two records. The first record has two fields
            // and the second record has three fields.
            pIDBRecord1 = IDATABASE_CreateRecord (pIDatabase, dbField1, 2); 
            pIDBRecord2 = IDATABASE_CreateRecord (pIDatabase, dbField2, 3); 

            // If anyone of the above records were not created, clean up and return.
            if (pIDBRecord1 == NULL || pIDBRecord2 == NULL)
            {
               // Release any record that did get created.
               if (pIDBRecord1 != NULL)
                  IDBRECORD_Release (pIDBRecord1);
               if (pIDBRecord2 != NULL)
                  IDBRECORD_Release (pIDBRecord2);

               // IDATABASE_Release releases any database records that are 
               // still in the database, closes the open database files, and 
               // frees any memory associated with the database.
               IDATABASE_Release (pIDatabase);
               IDBMGR_Remove (pIDBMgr, "db");   // Remove DB file
               IDBMGR_Release (pIDBMgr);        // Release the DB manager
               return;
            }

            // Since we reached here we have been able to create two 
            // records in the database. We should release the DBRecord
            // interface pointers here since we don't plan to use them 
            // anymore.
            //
            // Note: Even though we release the DBRecord interface 
            //       pointers we still have access to the DBRecords
            //       using the IDatabase pointer. Note the difference 
            //       between a DBRecord interface and a DBRecord itself.
            IDBRECORD_Release (pIDBRecord1);
            IDBRECORD_Release (pIDBRecord2);
            
            // Get the reocrd count in the database.
            recordCount = IDATABASE_GetRecordCount (pIDatabase);

            // Display the database record count record count at 
            // line 1 on the screen.
            SPRINTF (szBuf, "Record Count: %u", recordCount);
            DisplayOutput(pMe, 1, szBuf);

            // This will reset the record Index to 0. Hence,
            // IDATABASE_GetNextRecord function retrieves the
            // first record of the database.
            IDATABASE_Reset (pIDatabase);

            // Get the first record of the database.
            pIDBRec1 = IDATABASE_GetNextRecord (pIDatabase);

            if (pIDBRec1 != NULL)
            {
               // Get the field information of the first field in the record.
               // Since this is the first field operation IDBRECORD_NextField
               // gets the first field on the record.
               if ((fieldType = IDBRECORD_NextField (pIDBRec1, &fieldName, 
                  &fieldLen)) == AEEDB_FT_STRING)
               {
                  // Get the data of the field
                  if ((data = IDBRECORD_GetField (pIDBRec1, &fieldName, 
                     &fieldType, &fieldLen)) != NULL)
                  {
                     // Print the data on the screen at line 2.
                     SPRINTF (szBuf, "First Name: %s", (char *)data);
                     DisplayOutput (pMe, 2, szBuf);
                  }
               }

               // Get the field information of the second field in the record.
               if ((fieldType = IDBRECORD_NextField (pIDBRec1, &fieldName, 
                  &fieldLen)) == AEEDB_FT_STRING)
               {
                   // Get data of the field
                  if ((data = IDBRECORD_GetField (pIDBRec1, &fieldName, 
                     &fieldType, &fieldLen)) != NULL)
                  {
                     // Print the data on the screen at line 3.
                     SPRINTF (szBuf, "Last Name: %s", (char *)data);
                     DisplayOutput (pMe, 3, szBuf);
                  }
               }

               // Release the record since we don't plan to use it any more.
               IDBRECORD_Release(pIDBRec1);
            }

            // It should be noted that when the IDATABASE_GetNextRecord
            // function is used to retrieve a record, this record needs
            // to be released prior to another record being retrieved 
            // from the database. In effect only one database record should
            // be retrieved from the database at anygiven time.
            //
            // This makes sure that the reference counter of the database
            // record will be properly incremented and decremented, as 
            // a result releasing all the memory allocated for each 
            // record. 
            // Note: the IDBRECORD_Release function will decrement
            //       the reference count. If the reference count
            //       is zero it releases all the resources allocated 
            //       to it. For this mechanism to work the user needs
            //       to maintain only one outstanding db record.

            // Get the second record from the database.
            pIDBRec2 = IDATABASE_GetNextRecord (pIDatabase);

            if (pIDBRec2 != NULL)
            {
               IDBRecord * pRec = NULL;

               // Here we are using an alternative way to retrieve a record by
               // using a record ID.
               // Use the record pointer to get the record ID.
               recId = IDBRECORD_GetID (pIDBRec2);

               // We should release pIDBRec2 since we are not going
               // use it again, and we will use another way to retrieve
               // the same record.
               IDBRECORD_Release(pIDBRec2);

               // Print the record ID on the screen at line 4.
               SPRINTF (szBuf, "Record 2 ID: %u", recId);
               DisplayOutput (pMe, 4, szBuf);

               // Now get the record using the ID.
               pRec = IDATABASE_GetRecordByID (pIDatabase, recId);

               // The field index of the record retrieved is zero by default.
               // Hence, the three operations of IDBRECORD_NextField
               // move the field index to the last field. We reset the
               // index by back to zero by invoking IDBRECORD_Reset() at the end.
               fieldType = IDBRECORD_NextField(pRec, &fieldName, &fieldLen);
               fieldType = IDBRECORD_NextField(pRec, &fieldName, &fieldLen);
               fieldType = IDBRECORD_NextField(pRec, &fieldName, &fieldLen);

               // Reset the field index to 0.
               IDBRECORD_Reset (pRec);

               // Get the first three fields and their contents, and print the 
               // data of the field by retriving it using IDBRECORD_GetFieldString
               // IDBRECORD_GetFieldWord or IDBRECORD_GetFieldDWord functions.

               // Get the field information of the first field.
               if ((fieldType = IDBRECORD_NextField(pRec, &fieldName, &fieldLen))
                  == AEEDB_FT_STRING)
               {
                  // Get the raw data of the field
                  if ((psz = IDBRECORD_GetFieldString (pRec)) != NULL)
                  {
                     // Print the data on the screen at line 5.
                     SPRINTF (szBuf, "Field 1 value: %s", psz); 
                     DisplayOutput (pMe, 5, szBuf);
                  }
               }

               // Get the field information of the second field.
               if ((fieldType = IDBRECORD_NextField(pRec, &fieldName, &fieldLen))
                  == AEEDB_FT_WORD)
               {
                  // Get the raw data of the second field.
                  if ((bRetValue = IDBRECORD_GetFieldWord (pRec, &wordValue)) == TRUE)
                  {
                     // Print the data on the screen at line 6.
                     SPRINTF (szBuf, "Field 2 value: %u",wordValue); 
                     DisplayOutput (pMe, 6, szBuf);
                  }
               }

               // Get the field information of the thrid field.
               if ((fieldType = IDBRECORD_NextField(pRec, &fieldName, &fieldLen))
                  == AEEDB_FT_DWORD)
               {
                  // Get the raw data of the third field.
                  if ((bRetValue = IDBRECORD_GetFieldDWord (pRec, &dwordValue)) == TRUE)
                  {
                     // Print the data on the screen at line 7.
                     SPRINTF (szBuf, "Field 3 value: %u", dwordValue);  
                     DisplayOutput (pMe, 7, szBuf);
                  }
               }

               // Now let us update a field with new values in the second 
               // record. And retrieve it from the database and print it 
               // to screen.
               // Update the dbField2 array item corresponding to the first 
               // field of the second record.
               dbField2[0].fName = AEEDBFIELD_LASTNAME;
               dbField2[0].fType = AEEDB_FT_STRING;
               dbField2[0].pBuffer = (void *)&lastName;
               dbField2[0].wDataLen = STRLEN (lastName);

               // Invoke IDBRECORD_Update to update the new values on the record.
               if ((nRetVal = IDBRECORD_Update (pRec, dbField2, 1)) == SUCCESS)
               {
                  // Note: we don't need to reset the record since the update 
                  // would have caused the field index to be intialized to zero.

                  // Retrieve the first field of the record.
                  if ((fieldType = IDBRECORD_NextField(pRec, &fieldName, &fieldLen))
                     == AEEDB_FT_STRING)
                  {
                     // Get the string value of the field.
                     if ((psz = IDBRECORD_GetFieldString (pRec)) != NULL)
                     {
                        // Print the updated value on the screen at line 8.
                        SPRINTF (szBuf, "Updated value: %s", psz); 
                        DisplayOutput (pMe, 8, szBuf);
                     }
                  }
               }

               // Release the second record
               IDBRECORD_Release(pRec);
            }

            // IDATABASE_Release: closes the open database files, and frees 
            // any memory associated with the database.
            IDATABASE_Release (pIDatabase);
            IDBMGR_Remove (pIDBMgr, "db");   // Remove the database files.
            IDBMGR_Release (pIDBMgr);        // Release the DB manager
         }
         break;
      case USAGE_DBMGR_OPENDATABASE:
         {
            // 2. Usage: IDBMGR_OpenDatabase
            IDBMgr *pIDBMgr = NULL;
            IDatabase * pIDatabase = NULL;
            boolean bCreate = FALSE;

            // Create IDBMgr object. This step needs to be done only if
            // the IDBMgr object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_DBMGR, (void **)&pIDBMgr);

            if (pIDBMgr == NULL)
               return;

            // IDBMGR_OpenDatabase: This function opens the specified database. 
            // If database is already open, it returns NULL. Boolean flag 
            // bCreate specifies action to be taken if database does not exist. 
            // If the specified database does not exist, and bCreate is TRUE, 
            // this function creates a new database with minimum records and 
            // minimum record size parameters both set to 0.

            // This should fail since there is no database of the given name 
            // existing and the bCreate flag is false.
            if ((pIDatabase = IDBMGR_OpenDatabase (pIDBMgr, "bd0", bCreate))
               == NULL)
            {
               // Now set bCreate to TRUE (this should create the DB).
               bCreate = TRUE; 

               // Create database.
               if ((pIDatabase = IDBMGR_OpenDatabase (pIDBMgr, "db1", bCreate))
                  != NULL)
               {
                  DisplayOutput (pMe, -1, "Database open successful");

                  // IDATABASE_Release: closes the open database files, and 
                  // frees any memory associated with the database.
                  IDATABASE_Release (pIDatabase);

                  // Now remove the DB files.
                  IDBMGR_Remove (pIDBMgr, "db1");
               }
               else
               {
                  DisplayOutput (pMe, -1, "Database open failed");
               }
            }
            else
            {
               // Opened an already existing database.
               DisplayOutput (pMe, -1, "Opened an already existing Database.");

               // Release the Database
               IDATABASE_Release (pIDatabase);
               // Remove the DB file since it is not used again.
               IDBMGR_Remove (pIDBMgr, "db0");
            }

            // Release IDBMgr Interface. This step needs to be done
            // only if no further use of the IDBMgr object is needed.
            IDBMGR_Release (pIDBMgr);
         }
         break;
      case USAGE_DBMGR_OPENDATABASEEX:
         {
            // 3. Usage: IDBMGR_OpenDatabaseEx
            IDBMgr *pIDBMgr = NULL;
            IDatabase * pIDatabase = NULL;
            boolean bCreate = FALSE;
            // Mimimum size of the records in the database
            uint32  dwMinSize = 100;

            // Minimum number of records the database should hold
            uint16  wMinRecs = 10;

            // Create IDBMgr object. This step needs to be done only if
            // the IDBMgr object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_DBMGR, (void **)&pIDBMgr);

            if (pIDBMgr == NULL)
               return;

            // IDBMGR_OpenDatabaseEx: opens the specified database. If database 
            // is already open, it returns NULL. Boolean flag bCreate specifies 
            // action to be taken if database does not exist. If the specified 
            // database does not exist, and bCreate is TRUE, this function creates 
            // a new database.While creating the database, the minmum record size 
            // can be specified by dwMinRecSize, and the minimum number of records 
            // can be specified by wMinRecs.

            // This should fail since there is no database of the given name 
            // existing and the bCreate flag is false.
            if ((pIDatabase = IDBMGR_OpenDatabaseEx (pIDBMgr, "bd2", bCreate,
               dwMinSize, wMinRecs)) == NULL)
            {
               // Now set bCreate to TRUE (this should create the DB)
               bCreate = TRUE; 

               // Open database again.
               if ((pIDatabase = IDBMGR_OpenDatabaseEx (pIDBMgr, "db3", bCreate,
                  dwMinSize, wMinRecs)) != NULL)
               {
                  DisplayOutput (pMe, -1, "Database open successful");

                  // IDATABASE_Release closes the open database files, and 
                  // frees any memory associated with the database.
                  IDATABASE_Release (pIDatabase);

                  // Now remove the DB files.
                  IDBMGR_Remove (pIDBMgr, "db3");
               }
               else
               {
                  DisplayOutput (pMe, -1, "Database open failed");
               }
            }
            else
            {
               // Opened an already existing database.
               DisplayOutput (pMe, -1, "Opened an already existing Database.");

               // Release the Database
               IDATABASE_Release (pIDatabase);
               // Remove the DB file since it is not used again.
               IDBMGR_Remove (pIDBMgr, "db1");
            }

            // Release IDBMgr Interface. This step needs to be done
            // only if no further use of the IDBMgr object is needed.
            IDBMGR_Release (pIDBMgr);
         }
         break;
      case USAGE_DBMGR_REMOVE:
         {
            // 4. Usage: IDBMGR_Remove
            IDBMgr *pIDBMgr = NULL;
            IDatabase * pIDatabase = NULL;
            int nRetVal = EBADFILENAME;

            // Create IDBMgr object. This step needs to be done only if
            // the IDBMgr object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_DBMGR, (void **)&pIDBMgr);

            if (pIDBMgr == NULL)
               return;

            // Create a new database.
            if ((pIDatabase = IDBMGR_OpenDatabase (pIDBMgr, "db4", TRUE))
                != NULL)
            {
               // We were able to create the database successfully.
               //
               // IDATABASE_Release closes the open database files, and frees 
               // any memory associated with the database. 
               IDATABASE_Release (pIDatabase);

               // IDBMGR_Remove removes the database (and all the files 
               // associated with the DB).
               if ((nRetVal = IDBMGR_Remove (pIDBMgr, "db4")) == SUCCESS)
               {
                  DisplayOutput (pMe, -1, "DB removed successfully");
               }
               else
               {
                  DisplayOutput (pMe, -1, "DB Remove failed");
               }
            }

            // Release IDBMgr object. This step needs to be done
            // only if no further use of the IDBMgr object is needed.
            IDBMGR_Release (pIDBMgr);
         }
         break;
      case USAGE_DB_CREATERECORD:
         {
            // 5. Usage: IDATABASE_CreateRecord
            IDBMgr *pIDBMgr = NULL;
            IDatabase * pIDatabase = NULL;
            IDBRecord *pIDBRecord = NULL;
            int nNumfields = 3;
            AEEDBField dbField[3]; 
            uint32 homeNumber = 1234567;
            const char name [] = "FirstName LastName";
            const char address[] = "123 First Street, USA";

            // Data fill values used to create a database record. 
            // The parameter dbField is a three item array of type 
            // AEEDBField. Each array item corresponds to three fields 
            // of the record. 
            dbField[0].fName = AEEDBFIELD_FULLNAME;
            dbField[0].fType = AEEDB_FT_STRING;
            dbField[0].pBuffer = (void *)name;
            dbField[0].wDataLen = STRLEN (name);
            dbField[1].fName = AEEDBFIELD_ADDRESS;
            dbField[1].fType = AEEDB_FT_STRING;
            dbField[1].pBuffer = (void *)address;
            dbField[1].wDataLen = STRLEN (address);
            dbField[2].fName = AEEDBFIELD_HOME_PHONE;
            dbField[2].fType = AEEDB_FT_DWORD;
            dbField[2].pBuffer = (void *)&homeNumber;
            dbField[2].wDataLen = sizeof (uint32);
            
            // Create IDBMgr object. This step needs to be done only if
            // the IDBMgr object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_DBMGR, (void **)&pIDBMgr);

            if (pIDBMgr == NULL)
               return;

            // Create a new database.
            if ((pIDatabase = IDBMGR_OpenDatabase (pIDBMgr, "db5", TRUE)) != NULL)
            {        
               // IDATABASE_CreateRecord: creates a new database record with the fields 
               // specified by pDBFields.
               if ((pIDBRecord = IDATABASE_CreateRecord (pIDatabase, dbField, nNumfields))
                  != NULL)
               {
                  // Successfully created a database record.
                  DisplayOutput (pMe, -1, "Create DB: successful");

                  // Release record
                  IDBRECORD_Release (pIDBRecord);
               }
               else
               {
                  DisplayOutput (pMe, -1, "Create DB: Failed");
               }

               // IDATABASE_Release: closes the open database files, and frees 
               // any memory associated with the database.
               IDATABASE_Release (pIDatabase);
               IDBMGR_Remove (pIDBMgr, "db5");
            }

            // Release IDBMgr object. This step needs to be done
            // only if no further use of the IDBMgr object is needed.
            IDBMGR_Release (pIDBMgr);
         }
         break;
      case USAGE_DB_GETRECORDCOUNT:
         {
            // 6. Usage: IDATABASE_GetRecordCount
            IDBMgr    * pIDBMgr = NULL;
            IDatabase * pIDatabase = NULL;
            IDBRecord * pIDBRecord1 = NULL;
            IDBRecord * pIDBRecord2 = NULL;
            IDBRecord * pIDBRecord3 = NULL;
            int nNumfields = 1;
            AEEDBField dbField1[1];      // data for record 1
            AEEDBField dbField2[1];      // data for record 2
            AEEDBField dbField3[1];      // data for record 3
            uint32 recordCount = 0;
            const char name1 [] = "John";
            const char name2 [] = "Paul";
            const char name3 [] = "Jeff";
            char szBuf[30] = {0};

            // Data fill values used to create a database record. 
            // The parameter dbField1, dbField2, and dbField3 
            // corresponds to the three database records.
            //
            // Data fill the three database records
            dbField1[0].fName = AEEDBFIELD_FIRSTNAME;
            dbField1[0].fType = AEEDB_FT_STRING;
            dbField1[0].pBuffer = (void *)name1;
            dbField1[0].wDataLen = STRLEN (name1);
            dbField2[0].fName = AEEDBFIELD_FIRSTNAME;
            dbField2[0].fType = AEEDB_FT_STRING;
            dbField2[0].pBuffer = (void *)name2;
            dbField2[0].wDataLen = STRLEN (name2);
            dbField3[0].fName = AEEDBFIELD_FIRSTNAME;
            dbField3[0].fType = AEEDB_FT_STRING;
            dbField3[0].pBuffer = (void *)name3;
            dbField3[0].wDataLen = STRLEN (name3);

            // Create IDBMgr object. This step needs to be done only if
            // the IDBMgr object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_DBMGR, (void **)&pIDBMgr);

            if (pIDBMgr == NULL)
               return;

            // Create a database.
            if ((pIDatabase = IDBMGR_OpenDatabase (pIDBMgr, "db6", TRUE)) == NULL)
            {
               // Release IDBMgr object before returning.
               IDBMGR_Release (pIDBMgr);
               return;
            }

            // IDATABASE_GetRecordCount: returns the number of records in the 
            // database specified. This gives the number of records in the
            // database prior to creating any records in the database.
            recordCount = IDATABASE_GetRecordCount (pIDatabase);

            // Display the DB record count. This should be zero since this
            // database is newly created and no new records have been added.
            SPRINTF (szBuf, "Record Count (before): %u", recordCount);
            DisplayOutput(pMe, 2, szBuf);

            // Create three records
            pIDBRecord1 = IDATABASE_CreateRecord (pIDatabase, dbField1, nNumfields);
            pIDBRecord2 = IDATABASE_CreateRecord (pIDatabase, dbField2, nNumfields);
            pIDBRecord3 = IDATABASE_CreateRecord (pIDatabase, dbField3, nNumfields);

            // If anyone of the above records were not created, clean up and return.
            if (pIDBRecord1 == NULL || pIDBRecord2 == NULL || pIDBRecord3 == NULL)
            {
               // Release any record that that did get created.
               if (pIDBRecord1 != NULL)
                  IDBRECORD_Release (pIDBRecord1);
               if (pIDBRecord2 != NULL)
                  IDBRECORD_Release (pIDBRecord2);
               if (pIDBRecord3 != NULL)
                  IDBRECORD_Release (pIDBRecord3);

               // IDATABASE_Release releases any database records that are still in 
               // the database, closes the open database files, and frees any memory 
               // associated with the database.
               IDATABASE_Release (pIDatabase);
               IDBMGR_Remove (pIDBMgr, "db6");   // IDBMGR_Remove removes the database files.
               IDBMGR_Release (pIDBMgr);         // Release the DB manager
               return;
            }

            // Since we reached here we have been able to create three 
            // records in the database. We will first get the record count
            // in the database and then remove the DB Records and finally 
            // remove database as well.

            // IDATABASE_GetRecordCount: returns the number of records in the 
            // database specified.
            recordCount = IDATABASE_GetRecordCount (pIDatabase);

            // Display the DB record count. At this point the database 
            // has three records.
            SPRINTF (szBuf, "Record Count (after): %u", recordCount);
            DisplayOutput(pMe, 4, szBuf);

            // We need to release the records before removing the DB
            IDBRECORD_Release (pIDBRecord1);
            IDBRECORD_Release (pIDBRecord2);
            IDBRECORD_Release (pIDBRecord3);

            // IDATABASE_Release: closes the open database files, and frees 
            // any memory associated with the database.
            IDATABASE_Release (pIDatabase);
            IDBMGR_Remove (pIDBMgr, "db6");   // Removes the database (files).
            IDBMGR_Release (pIDBMgr);         // Release the DB manager
         }
         break;
      case USAGE_DB_RESET:
      case USAGE_DB_GETNEXTRECORD:
         {
            // 7. Usage: IDATABASE_Reset/IDATABASE_GetNextRecord
            IDBMgr    * pIDBMgr = NULL;
            IDatabase * pIDatabase = NULL;
            IDBRecord * pIDBRecord1 = NULL;
            IDBRecord * pIDBRecord2 = NULL;
            IDBRecord * pIDBRec1 = NULL;
            IDBRecord * pIDBRec2 = NULL;

            int nNumfields = 2;
            AEEDBField dbField1[2];      // data for record 1
            AEEDBField dbField2[2];      // data for record 2
            const char name1 [] = "John Smith";
            const char name2 [] = "Paul Smith";
            const char address1[] = "123 First St, USA";
            const char address2[] = "123 Second St, USA";
            char szBuf[50] = {0};
            AEEDBFieldType fieldType;
            AEEDBFieldName fieldName;
            uint16 fieldLen;
            byte * data = NULL;

            // Data fill values used to create a database record. 
            // The parameter dbField1 and dbField2 corresponds to 
            // the two database records. Each record contains two fields.
            //
            // Data fill the first record 
            dbField1[0].fName = AEEDBFIELD_FULLNAME;
            dbField1[0].fType = AEEDB_FT_STRING;
            dbField1[0].pBuffer = (void *)name1;
            dbField1[0].wDataLen = STRLEN (name1);
            dbField1[1].fName = AEEDBFIELD_ADDRESS;
            dbField1[1].fType = AEEDB_FT_STRING;
            dbField1[1].pBuffer = (void *)address1;
            dbField1[1].wDataLen = STRLEN (address1);

            // Data fill the second record 
            dbField2[0].fName = AEEDBFIELD_FULLNAME;
            dbField2[0].fType = AEEDB_FT_STRING;
            dbField2[0].pBuffer = (void *)name2;
            dbField2[0].wDataLen = STRLEN (name2);
            dbField2[1].fName = AEEDBFIELD_ADDRESS;
            dbField2[1].fType = AEEDB_FT_STRING;
            dbField2[1].pBuffer = (void *)address2;
            dbField2[1].wDataLen = STRLEN (address2);

            // Create IDBMgr object. This step needs to be done only if
            // the IDBMgr object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_DBMGR, (void **)&pIDBMgr);

            if (pIDBMgr == NULL)
               return;

            // Create a database.
            if ((pIDatabase = IDBMGR_OpenDatabase (pIDBMgr, "db7", TRUE)) == NULL)
            {
               // Release IDBMgr object and return. 
               IDBMGR_Release (pIDBMgr);
               return;
            }

            // Create two records
            pIDBRecord1 = IDATABASE_CreateRecord (pIDatabase, dbField1, nNumfields);
            pIDBRecord2 = IDATABASE_CreateRecord (pIDatabase, dbField2, nNumfields);

            // If anyone of the above records were not created, clean up and return.
            if (pIDBRecord1 == NULL || pIDBRecord2 == NULL)
            {
               // Release any record that did get created.
               if (pIDBRecord1 != NULL)
                  IDBRECORD_Release (pIDBRecord1);
               if (pIDBRecord2 != NULL)
                  IDBRECORD_Release (pIDBRecord2);

               // IDATABASE_Release: closes the open database files, and frees 
               // any memory associated with the database.
               IDATABASE_Release (pIDatabase);
               IDBMGR_Remove (pIDBMgr, "db7");   // Removes the database.
               IDBMGR_Release (pIDBMgr);         // Release the DB manager
               return;
            }

            // Since we reached here we have been able to create two 
            // records in the database. We should release the DB Record
            // interface pointers here since we don't plan to use them 
            // anymore.
            //
            // Note: Even though we release the DBRecord interface 
            //       pointers we still have access to the DB Records
            //       using the IDatabase pointer. Note the difference 
            //       between a DBRecord interface and a DBRecord itself.
            IDBRECORD_Release (pIDBRecord1);
            IDBRECORD_Release (pIDBRecord2);

            // This will reset the record Index of the database to 0.
            IDATABASE_Reset (pIDatabase);

            // IDATABASE_GetNextRecord: returns the next IDBRecord from
            // the database
            // Get the first record.
            pIDBRec1 = IDATABASE_GetNextRecord (pIDatabase);

            if (pIDBRec1 != NULL)
            {
               // Get first field of record. We know it's of type string.
               fieldType = IDBRECORD_NextField (pIDBRec1, &fieldName, &fieldLen);
               // Get the raw data of the field.
               data = IDBRECORD_GetField (pIDBRec1, &fieldName, &fieldType, &fieldLen);
               if (data != NULL)
               {
                  // Display data at line 2.
                  SPRINTF (szBuf, "Rec1-field 1: %s", (char *)data);
                  DisplayOutput (pMe, 1, szBuf);
               }

               // Get second field of record one and display it
               fieldType = IDBRECORD_NextField (pIDBRec1, &fieldName, &fieldLen);
               // Get the raw data of the field.
               data = IDBRECORD_GetField (pIDBRec1, &fieldName, &fieldType, &fieldLen);
               if (data != NULL)
               {
                  // Display data at line 3.
                  SPRINTF (szBuf, "Rec1-field 2: %s", (char *)data);
                  DisplayOutput (pMe, 2, szBuf);
               }

               // Release DB Record.
               IDBRECORD_Release(pIDBRec1);
            }

            // It should be noted that when the IDATABASE_GetNextRecord
            // function is used to retrieve a record, this record needs
            // to be released prior to another record being retrieved 
            // from the database. In effect only one database record should
            // be retrieved from the database at anygiven time.
            //
            // This makes sure that the reference counter of the database
            // record will be properly incremented and decremented, as 
            // a result releasing all the memory allocated for each 
            // record. 
            // Note: the IDBRECORD_Release function will decrement
            //       the reference count. If the reference count
            //       is zero it releases all the resources allocated 
            //       to it. For this mechanism to work the user needs
            //       to maintain only one outstanding dbrecord.

            // Get the second record
            pIDBRec2 = IDATABASE_GetNextRecord (pIDatabase);

            if (pIDBRec2 != NULL)
            {
               // Get first field of record two and display it
               fieldType = IDBRECORD_NextField (pIDBRec2, &fieldName, &fieldLen);
               // Get the raw data of the field.
               data = IDBRECORD_GetField (pIDBRec2, &fieldName, &fieldType, &fieldLen);
               if (data != NULL)
               {
                  // Diaplsy data at line 5
                  SPRINTF (szBuf, "Rec2-field 1: %s", (char *)data);
                  DisplayOutput (pMe, 4, szBuf);
               }

               // Get second field of record two and display it
               fieldType = IDBRECORD_NextField (pIDBRec2, &fieldName, &fieldLen);
               // get the raw data of the field.
               data = IDBRECORD_GetField (pIDBRec2, &fieldName, &fieldType, &fieldLen);
               if (data != NULL)
               {
                  // Display data at line 6.
                  SPRINTF (szBuf, "Rec2-field 2: %s", (char *)data);
                  DisplayOutput (pMe, 5, szBuf);
               }

               // Release DB record before the DB is removed
               IDBRECORD_Release (pIDBRec2);
            }

            // IDATABASE_Release: closes the open database files, and frees 
            // any memory associated with the database.
            IDATABASE_Release (pIDatabase);
            IDBMGR_Remove (pIDBMgr, "db7");   // Removes the database (files).
            IDBMGR_Release (pIDBMgr);         // Release the DB manager
         }
         break;
      case USAGE_DB_GETRECORDBYID:
         {
            // 8. Usage: IDATABASE_GetRecordByID
            IDBMgr    * pIDBMgr = NULL;
            IDatabase * pIDatabase = NULL;
            IDBRecord * pIDBRecord1 = NULL;
            IDBRecord * pIDBRecord2 = NULL;
            IDBRecord * pIDBRec1 = NULL;
            IDBRecord * pIDBRec2 = NULL;

            int nNumfields = 1;
            AEEDBField dbField1;      // data for record 1
            AEEDBField dbField2;      // data for record 2
            const char name1 [] = "John Smith";
            const char name2 [] = "Paul Smith";
            AEEDBFieldType fType;
            AEEDBFieldName fName;
            uint16 fLen;
            byte * data = NULL;;

            // Data fill values used to create a database record. 
            // The parameter dbField1 and dbField2 corresponds to 
            // the two database records. Each record contains one field.
            //
            // Data fill the first record
            dbField1.fName = AEEDBFIELD_FULLNAME;
            dbField1.fType = AEEDB_FT_STRING;
            dbField1.pBuffer = (void *)name1;
            dbField1.wDataLen = STRLEN (name1);
            // Data fill the second record
            dbField2.fName = AEEDBFIELD_FULLNAME;
            dbField2.fType = AEEDB_FT_STRING;
            dbField2.pBuffer = (void *)name2;
            dbField2.wDataLen = STRLEN (name2);

            // Create IDBMgr object. This step needs to be done only if
            // the IDBMgr object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_DBMGR, (void **)&pIDBMgr);

            if (pIDBMgr == NULL)
               return;

            // Create a database.
            if ((pIDatabase = IDBMGR_OpenDatabase (pIDBMgr, "db8", TRUE)) == NULL)
            {
               // Release IDBMgr object and return.
               IDBMGR_Release (pIDBMgr);
               return;
            }

            // Create two records
            pIDBRecord1 = IDATABASE_CreateRecord (pIDatabase, &dbField1, nNumfields);
            pIDBRecord2 = IDATABASE_CreateRecord (pIDatabase, &dbField2, nNumfields);

            // If anyone of the above records were not created, clean up and return.
            if (pIDBRecord1 == NULL || pIDBRecord2 == NULL)
            {
               // Release any record that that did get created.
               if (pIDBRecord1 != NULL)
                  IDBRECORD_Release (pIDBRecord1);
               if (pIDBRecord2 != NULL)
                  IDBRECORD_Release (pIDBRecord2);

               // IDATABASE_Release releases any database records that are still in 
               // the database, closes the open database files, and frees any memory 
               // associated with the database.
               IDATABASE_Release (pIDatabase);
               IDBMGR_Remove (pIDBMgr, "db8");   // IDBMGR_Remove removes the database files.
               IDBMGR_Release (pIDBMgr);         // Release the DB manager
               return;
            }

            // We should release the DBRecord interface pointers here 
            // since we don't plan to use them anymore.
            IDBRECORD_Release (pIDBRecord1);
            IDBRECORD_Release (pIDBRecord2);

            // This will reset the record Index to 0.
            IDATABASE_Reset (pIDatabase);

            // IDATABASE_GetRecordByID: returns a pointer to the record whose 
            // record ID is specified.
            pIDBRec1 = IDATABASE_GetRecordByID (pIDatabase, 0);

            // Get record 1 first field and display it
            fType = IDBRECORD_NextField (pIDBRec1, &fName, &fLen);
            // Get the raw data of the field
            data = IDBRECORD_GetField (pIDBRec1, &fName, &fType, &fLen);
            if (data != NULL)
            {
               // Display data at line 2 on the screen.
               DisplayOutput (pMe, 2, (char *)data);
            }

            // Now remove record 1.
            IDBRECORD_Release(pIDBRec1);

            pIDBRec2 = IDATABASE_GetRecordByID (pIDatabase, 1);

            // Get record 2 first field and display it
            fType = IDBRECORD_NextField (pIDBRec2, &fName, &fLen);
            // Get the raw data of the field
            data = IDBRECORD_GetField (pIDBRec2, &fName, &fType, &fLen);
            if (data != NULL)
            {
               // Display the data at line 3 on the screen.
               DisplayOutput (pMe, 3, (char *)data);
            }

            // Release record 2
            IDBRECORD_Release(pIDBRec2);

            // IDATABASE_Release: closes the open database files, and frees any memory 
            // associated with the database.
            IDATABASE_Release (pIDatabase);
            IDBMGR_Remove (pIDBMgr, "db8");   // IDBMGR_Remove removes the database files.
            IDBMGR_Release (pIDBMgr);         // Release the DB manager
         }
         break;
      case USAGE_DBREC_RESET:
         {
            // 9. Usage: IDBRECORD_Reset
            IDBMgr    * pIDBMgr = NULL;
            IDatabase * pIDatabase = NULL;
            IDBRecord * pIDBRecord = NULL;
            IDBRecord * pIDBRec = NULL;

            int nNumfields = 3;
            AEEDBField dbField[3];      // data for record 1
            const char firstName [] = "John";
            const char lastName  [] =  "Smith";
            const char address   [] = "123 First Street, USA";
            AEEDBFieldType fieldType;
            AEEDBFieldName fieldName;
            uint16 nFieldLen;

            // Data fill values used to create a database record. 
            // The parameter dbField corresponds to the only record 
            // to be created in the datbase. This record will have 
            // three fields.
            //
            // Data fill the first record data
            dbField[0].fName = AEEDBFIELD_FIRSTNAME;
            dbField[0].fType = AEEDB_FT_STRING;
            dbField[0].pBuffer = (void *)firstName;
            dbField[0].wDataLen = STRLEN (firstName);
            dbField[1].fName = AEEDBFIELD_LASTNAME;
            dbField[1].fType = AEEDB_FT_STRING;
            dbField[1].pBuffer = (void *)lastName;
            dbField[1].wDataLen = STRLEN (lastName);
            dbField[2].fName = AEEDBFIELD_ADDRESS;
            dbField[2].fType = AEEDB_FT_STRING;
            dbField[2].pBuffer = (void *)address;
            dbField[2].wDataLen = STRLEN (address);

            // Create IDBMgr object. This step needs to be done only if
            // the IDBMgr object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_DBMGR, (void **)&pIDBMgr);

            if (pIDBMgr == NULL)
               return;

            // Create a database.
            if ((pIDatabase = IDBMGR_OpenDatabase (pIDBMgr, "db9", TRUE)) == NULL)
            {
               // Release IDBMgr object and return.
               IDBMGR_Release (pIDBMgr);
               return;
            }

            // Create three records
            if ((pIDBRecord = IDATABASE_CreateRecord (pIDatabase, dbField, nNumfields))
               == NULL)
            {
               // IDATABASE_Release: closes the open database files, and frees 
               // any memory associated with the database.
               IDATABASE_Release (pIDatabase);
               IDBMGR_Remove (pIDBMgr, "db9");   // Removes the database.
               IDBMGR_Release (pIDBMgr);         // Release the DB manager
            }

            // Reset the database so that record index is zero.
            IDATABASE_Reset(pIDatabase);

            // Get the first record in the field.
            pIDBRec = IDATABASE_GetNextRecord(pIDatabase);

            // Get the first field type, name and length.
            fieldType = IDBRECORD_NextField(pIDBRec, &fieldName, &nFieldLen);
            // Get the second field type, name and length.
            fieldType = IDBRECORD_NextField(pIDBRec, &fieldName, &nFieldLen);
            // Get the third field type, name and length
            fieldType = IDBRECORD_NextField(pIDBRec, &fieldName, &nFieldLen);

            // The field pointer at this point should be pointing to the thrid 
            // (last) record. The Reset below should reset the pointer to 
            // point to the first record. 
            IDBRECORD_Reset(pIDBRec);

            // This should retrive the Type and Name of the first field again.
            fieldType = IDBRECORD_NextField(pIDBRec, &fieldName, &nFieldLen);

            // Let's check if it is the first field that we received after 
            // the IDBRECORD_Reset invoke.
            if (fieldName == AEEDBFIELD_FIRSTNAME && fieldType == AEEDB_FT_STRING)
            {
               // Display output in the middle of the screen
               DisplayOutput (pMe, -1, "Reset Successful");
            }
            else
            {
               // Display output in the middle of the screen
               DisplayOutput (pMe, -1, "Reset failed");
            }

            // Now release all the DB Records.
            IDBRECORD_Release (pIDBRec);
            IDBRECORD_Release (pIDBRecord);

            // IDATABASE_Release: closes the open database files, and frees 
            // any memory associated with the database.
            IDATABASE_Release (pIDatabase);
            IDBMGR_Remove (pIDBMgr, "db9");   // Remove the database.
            IDBMGR_Release (pIDBMgr);         // Release the DB manager
         }
         break;
      case USAGE_DBREC_UPDATE:
         {
            // 10. Usage: IDBRECORD_Update
            IDBMgr    * pIDBMgr = NULL;
            IDatabase * pIDatabase = NULL;
            IDBRecord * pIDBRecord = NULL;
            int nRetVal = AEE_DB_EBADREC;
            int nNumfields = 1;
            AEEDBField dbField[1];      // data for record 1
            const char firstName [] = "John";
            const char lastName  [] = "Smith";
            char szBuf[30] = {0};
            AEEDBFieldType fieldType;
            AEEDBFieldName fieldName;
            byte * data = NULL;
            uint16 nFieldLen;

            // Data fill values used to create a database record. 
            // The parameter dbField corresponds to the only record 
            // to be created in the datbase. This record will have 
            // three fields.
            //
            // Data fill the record
            dbField[0].fName = AEEDBFIELD_FIRSTNAME;
            dbField[0].fType = AEEDB_FT_STRING;
            dbField[0].pBuffer = (void *)firstName;
            dbField[0].wDataLen = STRLEN (firstName);

            // Create IDBMgr object. This step needs to be done only if
            // the IDBMgr object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_DBMGR, (void **)&pIDBMgr);

            if (pIDBMgr == NULL)
               return;

            // Create a database.
            if ((pIDatabase = IDBMGR_OpenDatabase (pIDBMgr, "db10", TRUE)) == NULL)
            {
               // Release IDBMgr object and return.
               IDBMGR_Release (pIDBMgr);
               return;
            }

            // Create a record
            if ((pIDBRecord = IDATABASE_CreateRecord (pIDatabase, dbField, nNumfields)) == NULL)
            {

               // IDATABASE_Release: closes the open database files, and frees any memory 
               // associated with the database.
               IDATABASE_Release (pIDatabase);
               IDBMGR_Remove (pIDBMgr, "db10");  // Remove the database.
               IDBMGR_Release (pIDBMgr);         // Release the DB manager
            }

            // Get the first field of the record
            fieldType = IDBRECORD_NextField(pIDBRecord, &fieldName, &nFieldLen);
            // Get the raw data of the first field
            data = IDBRECORD_GetField (pIDBRecord, &fieldName, &fieldType, &nFieldLen);

            if (data != NULL)
            {
               // Print the data of the first field at line 2.
               SPRINTF (szBuf, "Field Val: %s", (char *)data);
               DisplayOutput (pMe, 2, szBuf);
            }

            // Update the dbField with new values.
            dbField[0].fName = AEEDBFIELD_LASTNAME;
            dbField[0].fType = AEEDB_FT_STRING;
            dbField[0].pBuffer = (void *)lastName;
            dbField[0].wDataLen = STRLEN (lastName);
                        
            // Update the record with the new values
            if ((nRetVal = IDBRECORD_Update (pIDBRecord, dbField, 1)) == SUCCESS)
            {            
               // Get the first field of the record.
               fieldType = IDBRECORD_NextField(pIDBRecord, &fieldName, &nFieldLen);
               // Get the data of the first field.
               data = IDBRECORD_GetField (pIDBRecord, &fieldName, &fieldType, &nFieldLen);
               if (data != NULL)
               {
                  // Print the data of the first field at line 4 on the screen. 
                  // This should be the updated values.
                  SPRINTF (szBuf, "Updated Val: %s", (char *)data);
                  DisplayOutput (pMe, 4, szBuf);
               }
            }
            else
            {
               DisplayOutput (pMe, 4, "Update Failed.");
            }

            // Release the DB Record.
            IDBRECORD_Release (pIDBRecord);

            // IDATABASE_Release: closes the open database files, and frees 
            // any memory associated with the database.
            IDATABASE_Release (pIDatabase);
            IDBMGR_Remove (pIDBMgr, "db10");  // Remove the database.
            IDBMGR_Release (pIDBMgr);         // Release the DB manager
         }
         break;
      case USAGE_DBREC_REMOVE:
         {
            // 11. Usage: IDBRECORD_Remove
            IDBMgr    * pIDBMgr = NULL;
            IDatabase * pIDatabase = NULL;
            IDBRecord * pIDBRecord1 = NULL;
            IDBRecord * pIDBRecord2 = NULL;
            IDBRecord * pIDBRecord3 = NULL;
            IDBRecord * pIDBRec1 = NULL;
            IDBRecord * pIDBRec2 = NULL;
            IDBRecord * pIDBRec3 = NULL;

            int nRetVal = AEE_DB_EBADREC;
            int nNumfields = 1;
            AEEDBField dbField1[1];      // data for record 1
            AEEDBField dbField2[1];      // data for record 2
            AEEDBField dbField3[1];      // data for record 3
            uint32 recordCount = 0;
            const char name1 [] = "John Smith";
            const char name2 [] = "Paul Smith";
            const char name3 [] = "Jeff Smith";
            char szBuf[30] = {0};

            // Data fill values used to create a database record. 
            // The parameter dbField1, dbField2, and dbField3 
            // corresponds to the three database records.
            //
            // Data fill the three record data
            dbField1[0].fName = AEEDBFIELD_FULLNAME;
            dbField1[0].fType = AEEDB_FT_STRING;
            dbField1[0].pBuffer = (void *)name1;
            dbField1[0].wDataLen = STRLEN (name1);
            dbField2[0].fName = AEEDBFIELD_LASTNAME;
            dbField2[0].fType = AEEDB_FT_STRING;
            dbField2[0].pBuffer = (void *)name2;
            dbField2[0].wDataLen = STRLEN (name2);
            dbField3[0].fName = AEEDBFIELD_LASTNAME;
            dbField3[0].fType = AEEDB_FT_STRING;
            dbField3[0].pBuffer = (void *)name3;
            dbField3[0].wDataLen = STRLEN (name3);

            // Create IDBMgr object. This step needs to be done only if
            // the IDBMgr object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_DBMGR, (void **)&pIDBMgr);

            if (pIDBMgr == NULL)
               return;

            // Create a database.
            if ((pIDatabase = IDBMGR_OpenDatabase (pIDBMgr, "db11", TRUE)) == NULL)
            {
               // Release IDBMgr object. This step needs to be done
               // only if no further use of the IDBMgr object is needed.
               IDBMGR_Release (pIDBMgr);
               return;
            }

            // Create three records
            pIDBRecord1 = IDATABASE_CreateRecord (pIDatabase, dbField1, nNumfields);
            pIDBRecord2 = IDATABASE_CreateRecord (pIDatabase, dbField2, nNumfields);
            pIDBRecord3 = IDATABASE_CreateRecord (pIDatabase, dbField3, nNumfields);

            // If anyone of the above records were not created, clean up and return.
            if (pIDBRecord1 == NULL || pIDBRecord2 == NULL || pIDBRecord3 == NULL)
            {
               // Release any record that that did get created.
               if (pIDBRecord1 != NULL)
                  IDBRECORD_Release (pIDBRecord1);
               if (pIDBRecord2 != NULL)
                  IDBRECORD_Release (pIDBRecord2);
               if (pIDBRecord3 != NULL)
                  IDBRECORD_Release (pIDBRecord3);

               // IDATABASE_Release: closes the open database files, and frees 
               // any memory associated with the database.
               IDATABASE_Release (pIDatabase);
               IDBMGR_Remove (pIDBMgr, "db11");  // IDBMGR_Remove removes the database files.
               IDBMGR_Release (pIDBMgr);         // Release the DB manager
               return;
            }

            // We should release the DBRecord interface pointers here 
            // since we don't plan to use them anymore. We will use
            // the GetNextRecord function to get each of the 
            // three records and remove them one at a time.
            IDBRECORD_Release (pIDBRecord1);
            IDBRECORD_Release (pIDBRecord2);
            IDBRECORD_Release (pIDBRecord3);

            // Get record Count
            recordCount = IDATABASE_GetRecordCount (pIDatabase);

            SPRINTF (szBuf, "# of Records: %u", recordCount);
            DisplayOutput (pMe, 1, szBuf);

            // Reset the database
            IDATABASE_Reset(pIDatabase);

            // Get the first record from the database
            pIDBRec1 = IDATABASE_GetNextRecord(pIDatabase);

            // Remove record 1
            if ((pIDBRec1 != NULL) && ((nRetVal = IDBRECORD_Remove (pIDBRec1)) == SUCCESS))
            {
               DisplayOutput (pMe, 2, "Removed Record 1");
            }
            else
            {
               DisplayOutput (pMe, 2, "Failed removing record 1");
            }

            // Get the second record from the database and remove it.
            pIDBRec2 = IDATABASE_GetNextRecord(pIDatabase);

            // Remove record 2
            if ((pIDBRec2 != NULL) && ((nRetVal = IDBRECORD_Remove (pIDBRec2)) == SUCCESS))
            {
               DisplayOutput (pMe, 3, "Removed Record 2");
            }
            else
            {
               DisplayOutput (pMe, 3, "Failed removing record 2");
            }

            // Get the third record from the database and remove it.
            pIDBRec3 = IDATABASE_GetNextRecord(pIDatabase);

            // Remove record 3
            if ((pIDBRec3 != NULL) && ((nRetVal = IDBRECORD_Remove (pIDBRec3)) == SUCCESS))
            {
               DisplayOutput (pMe, 4, "Removed Record 3");
            }
            else
            {
               DisplayOutput (pMe, 4, "Failed removing record 3");
            }

            // Get record Count
            recordCount = IDATABASE_GetRecordCount (pIDatabase);

            SPRINTF (szBuf, "# of Records: %u", recordCount);
            DisplayOutput (pMe, 5, szBuf);  

            // IDATABASE_Release: closes the open database files, and frees 
            // any memory associated with the database.
            IDATABASE_Release (pIDatabase);
            IDBMGR_Remove (pIDBMgr, "db11");  // Remove the database.
            IDBMGR_Release (pIDBMgr);         // Release the DB manager
         }
         break;
      case USAGE_DBREC_GETID:
         {
            // 12. Usage: IDBRECORD_GetID
            IDBMgr    * pIDBMgr = NULL;
            IDatabase * pIDatabase = NULL;
            IDBRecord * pIDBRecord1 = NULL;
            IDBRecord * pIDBRecord2 = NULL;
            IDBRecord * pIDBRecord3 = NULL;
            IDBRecord * pIDBRec1 = NULL;
            IDBRecord * pIDBRec2 = NULL;
            IDBRecord * pIDBRec3 = NULL;

            int nNumfields = 1;
            AEEDBField dbField1[1];      // data for record 1
            AEEDBField dbField2[1];      // data for record 2
            AEEDBField dbField3[1];      // data for record 3
            const char name1 [] = "John Smith";
            const char name2 [] = "Paul Smith";
            const char name3 [] = "Jeff Smith";
            char szBuf[30] = {0};
            uint16 recId = 0;

            // Data fill values used to create a database record. 
            // The parameter dbField1, dbField2, and dbField3 
            // corresponds to the three database records.
            //
            // Data fill the three record data
            dbField1[0].fName = AEEDBFIELD_FULLNAME;
            dbField1[0].fType = AEEDB_FT_STRING;
            dbField1[0].pBuffer = (void *)name1;
            dbField1[0].wDataLen = STRLEN (name1);
            dbField2[0].fName = AEEDBFIELD_LASTNAME;
            dbField2[0].fType = AEEDB_FT_STRING;
            dbField2[0].pBuffer = (void *)name2;
            dbField2[0].wDataLen = STRLEN (name2);
            dbField3[0].fName = AEEDBFIELD_LASTNAME;
            dbField3[0].fType = AEEDB_FT_STRING;
            dbField3[0].pBuffer = (void *)name3;
            dbField3[0].wDataLen = STRLEN (name3);

            // Create IDBMgr object. This step needs to be done only if
            // the IDBMgr object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_DBMGR, (void **)&pIDBMgr);

            if (pIDBMgr == NULL)
               return;

            // Create a database.
            if ((pIDatabase = IDBMGR_OpenDatabase (pIDBMgr, "db12", TRUE)) == NULL)
            {
               // Release IDBMgr object. This step needs to be done
               // only if no further use of the IDBMgr object is needed.
               IDBMGR_Release (pIDBMgr);
               return;
            }

            // Create three records
            pIDBRecord1 = IDATABASE_CreateRecord (pIDatabase, dbField1, nNumfields);
            pIDBRecord2 = IDATABASE_CreateRecord (pIDatabase, dbField2, nNumfields);
            pIDBRecord3 = IDATABASE_CreateRecord (pIDatabase, dbField3, nNumfields);

            // If anyone of the above records were not created, clean up and return.
            if (pIDBRecord1 == NULL || pIDBRecord2 == NULL || pIDBRecord3 == NULL)
            {
               // Release any record that that did get created.
               if (pIDBRecord1 != NULL)
                  IDBRECORD_Release (pIDBRecord1);
               if (pIDBRecord2 != NULL)
                  IDBRECORD_Release (pIDBRecord2);
               if (pIDBRecord3 != NULL)
                  IDBRECORD_Release (pIDBRecord3);

               // IDATABASE_Release: closes the open database files, and frees 
               // any memory associated with the database.
               IDATABASE_Release (pIDatabase);
               IDBMGR_Remove (pIDBMgr, "db12");   // IDBMGR_Remove removes the database files.
               IDBMGR_Release (pIDBMgr);         // Release the DB manager
               return;
            }

            // Now release all the DB Records interface pointers 
            // (not to be confused with DB records), since we don't 
            // plan to use them anymore. 
            IDBRECORD_Release (pIDBRecord1);
            IDBRECORD_Release (pIDBRecord2);
            IDBRECORD_Release (pIDBRecord3);

            // Reset the database
            IDATABASE_Reset(pIDatabase);

            // Get the first record from the database
            if ((pIDBRec1 = IDATABASE_GetNextRecord(pIDatabase)) != NULL)
            {
               // IDBRECORD_GetID: gets the ID of the specified record. 
               recId = IDBRECORD_GetID (pIDBRec1);
               // Print the record Id on the screen at line 2
               SPRINTF (szBuf, "Record 1 ID: %u", recId);
               DisplayOutput (pMe, 2, szBuf);

               // Remove first record from the DB.
               IDBRECORD_Remove (pIDBRec1);
            }

            // Get the second record from the database
            if ((pIDBRec2 = IDATABASE_GetNextRecord(pIDatabase)) != NULL)
            {
               // Get the record ID
               recId = IDBRECORD_GetID (pIDBRec2);
               // Print the record ID on the screen at line 3
               SPRINTF (szBuf, "Record 2 ID: %u", recId);
               DisplayOutput (pMe, 3, szBuf);

               // Remove rec 2 from DB.
               IDBRECORD_Remove (pIDBRec2);
            }

            // Get the third record from the database
            if ((pIDBRec3 = IDATABASE_GetNextRecord(pIDatabase)) != NULL)
            {
               // Get the record ID
               recId = IDBRECORD_GetID (pIDBRec3);
               // Print the record ID on the screen at line 4
               SPRINTF (szBuf, "Record 3 ID: %u", recId);
               DisplayOutput (pMe, 4, szBuf);

               // Remove rec 3 from DB.
               IDBRECORD_Remove (pIDBRec3);
            }

            // Note: IDBRECORD_Remove implicitly releases the records as well.
            
            // IDATABASE_Release: closes the open database files, and frees 
            // any memory associated with the database.
            IDATABASE_Release (pIDatabase);
            IDBMGR_Remove (pIDBMgr, "db12");  // Remove the database.
            IDBMGR_Release (pIDBMgr);         // Release the DB manager
         }
         break;
      case USAGE_DBREC_NEXTFIELD:
      case USAGE_DBREC_GETFIELD:
         {
            // 13. Usage: IDBRECORD_NextField/IDBRECORD_GetField
            IDBMgr    * pIDBMgr = NULL;
            IDatabase * pIDatabase = NULL;
            IDBRecord * pIDBRecord = NULL;
            IDBRecord * pIDBRec = NULL;

            int nNumfields = 3;
            AEEDBField dbField[3];      // data for record 1
            const char firstName [] = "John";
            const char lastName  [] =  "Smith";
            const char address   [] = "123 First Street, USA";
            char szBuf[50] = {0};
            AEEDBFieldType fieldType;
            AEEDBFieldName fieldName;
            uint16 nFieldLen;
            byte *data = NULL;

            // Data fill values used to create a database record. 
            // The parameter dbField corresponds to the only record 
            // to be created in the datbase. This record will have 
            // three fields.
            //
            // Data fill the first record data
            dbField[0].fName = AEEDBFIELD_FIRSTNAME;
            dbField[0].fType = AEEDB_FT_STRING;
            dbField[0].pBuffer = (void *)firstName;
            dbField[0].wDataLen = STRLEN (firstName);
            dbField[1].fName = AEEDBFIELD_LASTNAME;
            dbField[1].fType = AEEDB_FT_STRING;
            dbField[1].pBuffer = (void *)lastName;
            dbField[1].wDataLen = STRLEN (lastName);
            dbField[2].fName = AEEDBFIELD_ADDRESS;
            dbField[2].fType = AEEDB_FT_STRING;
            dbField[2].pBuffer = (void *)address;
            dbField[2].wDataLen = STRLEN (address);

            // Create IDBMgr object. This step needs to be done only if
            // the IDBMgr object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_DBMGR, (void **)&pIDBMgr);

            if (pIDBMgr == NULL)
               return;

            // Create a database.
            if ((pIDatabase = IDBMGR_OpenDatabase (pIDBMgr, "db13", TRUE)) == NULL)
            {
               // Release IDBMgr object and return.
               IDBMGR_Release (pIDBMgr);
               return;
            }

            // Create a record with three fields
            if ((pIDBRecord = IDATABASE_CreateRecord (pIDatabase, dbField, nNumfields)) == NULL)
            {
               // IDATABASE_Release: closes the open database files, and frees 
               // any memory associated with the database.
               IDATABASE_Release (pIDatabase);
               IDBMGR_Remove (pIDBMgr, "db13");  // Remove the database.
               IDBMGR_Release (pIDBMgr);         // Release the DB manager
            }

            // Release DB Rec ptrs since we don't plan to use it again.
            IDBRECORD_Release (pIDBRecord);

            // Reset the database
            IDATABASE_Reset(pIDatabase);

            // We will get the first and only record in the database
            pIDBRec = IDATABASE_GetNextRecord(pIDatabase); 

            // Get the first field of the record.
            fieldType = IDBRECORD_NextField(pIDBRec, &fieldName, &nFieldLen); 
            // Get the raw data of the field
            data = IDBRECORD_GetField (pIDBRec, &fieldName, &fieldType, &nFieldLen);
            if (data != NULL)
            {
               // Print the data on the screen at line 2
               SPRINTF (szBuf, "Field 1 Value: %s", (char *)data);
               DisplayOutput (pMe, 1, szBuf);
            }

            // Get second field
            fieldType = IDBRECORD_NextField(pIDBRec, &fieldName, &nFieldLen);
            // Get the raw data of the field
            data = IDBRECORD_GetField (pIDBRec, &fieldName, &fieldType, &nFieldLen);
            if (data != NULL)
            {
               // Print the data on the screen at line 3
               SPRINTF (szBuf, "Field 2 Value: %s", (char *)data);
               DisplayOutput (pMe, 2, szBuf);
            }

            // Get third field
            fieldType = IDBRECORD_NextField(pIDBRec, &fieldName, &nFieldLen);
            // Get the raw data of the field
            data = IDBRECORD_GetField (pIDBRec, &fieldName, &fieldType, &nFieldLen);
            if (data != NULL)
            {
               // Display the data on the screen at line 4
               SPRINTF (szBuf, "Field 3 Value: %s", (char *)data);
               DisplayOutput (pMe, 3, szBuf);
            }

            // Try to get the fourth field, which WILL FAIL since there are 
            // only 3 fields in the record.
            fieldType = IDBRECORD_NextField(pIDBRec, &fieldName, &nFieldLen);
            
            if (fieldType == AEEDB_FT_NONE)
            {
               DisplayOutput (pMe, 5, "Field 4: None !");
            }

            // Now release the DB rec.
            IDBRECORD_Release (pIDBRec);

            // IDATABASE_Release: closes the open database files, and frees 
            // any memory associated with the database.
            IDATABASE_Release (pIDatabase);
            IDBMGR_Remove (pIDBMgr, "db13");  // Remove the database.
            IDBMGR_Release (pIDBMgr);         // Release the DB manager
         }
         break;
      case USAGE_DBREC_GETFIELDWORD:
      case USAGE_DBREC_GETFIELDDWORD:      
      case USAGE_DBREC_GETFIELDSTRING:
         {
            // 14. Usage: IDBRECORD_GetFieldWord/IDBRECORD_GetFieldDWord/IDBRECORD_GetFieldString
            IDBMgr    * pIDBMgr = NULL;
            IDatabase * pIDatabase = NULL;
            IDBRecord * pIDBRecord = NULL;
            IDBRecord * pIDBRec = NULL;

            int nNumfields = 3;
            AEEDBFieldType fieldType;
            AEEDBFieldName fieldName;
            uint16 nFieldLen;
            AEEDBField dbField[3];      // data for record 1
            uint16 category = 123;
            uint32 homePhone = 7654321;
            const char lastName [] = "Mr. Smith";

            boolean bRetValue = FALSE;
            char szBuf[50] = {0};
            uint16 wordValue = 0;
            uint32 dwordValue = 0;
            AECHAR * psz = NULL;
            int i;

            // Data fill values used to create a database record. 
            // The parameter dbField corresponds to the only record 
            // to be created in the datbase. This record will have 
            // three fields.
            //
            // data fill the record
            dbField[0].fName = AEEDBFIELD_LASTNAME;     
            dbField[0].fType = AEEDB_FT_STRING;          
            dbField[0].pBuffer = (void *)&lastName; 
            dbField[0].wDataLen = STRLEN (lastName);
            dbField[1].fName = AEEDBFIELD_CATEGORY;
            dbField[1].fType = AEEDB_FT_WORD;
            dbField[1].pBuffer = (void *)&category;
            dbField[1].wDataLen = sizeof (uint16);
            dbField[2].fName = AEEDBFIELD_HOME_PHONE;
            dbField[2].fType = AEEDB_FT_DWORD;
            dbField[2].pBuffer = (void *)&homePhone;
            dbField[2].wDataLen = sizeof (uint32);

            // Create IDBMgr object. This step needs to be done only if
            // the IDBMgr object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_DBMGR, (void **)&pIDBMgr);

            if (pIDBMgr == NULL)
               return;

            // Create a database.
            if ((pIDatabase = IDBMGR_OpenDatabase (pIDBMgr, "db14", TRUE)) == NULL)
            {
               // Release IDBMgr object and return.
               IDBMGR_Release (pIDBMgr);
               return;
            }

            // Create a record with three fields
            if ((pIDBRecord = IDATABASE_CreateRecord (pIDatabase, dbField, nNumfields)) == NULL)
            {
               // IDATABASE_Release: closes the open database files, and frees 
               // any memory associated with the database.
               IDATABASE_Release (pIDatabase);
               IDBMGR_Remove (pIDBMgr, "db14");  // Remove the database.
               IDBMGR_Release (pIDBMgr);         // Release the DB manager
            }

            // Release DB rec since we will not use it again.
            IDBRECORD_Release (pIDBRecord);

            // Reset the database
            IDATABASE_Reset(pIDatabase);

            // We will get the first and only record in the database
            pIDBRec = IDATABASE_GetNextRecord(pIDatabase);

            // Get the first three fields the record contains, and print the 
            // data of the field by retriving it using IDBRECORD_GetFieldString
            // IDBRECORD_GetFieldWord or IDBRECORD_GetFieldDWord functions.
            for (i = 0; i < 3; i++)
            {
               fieldType = IDBRECORD_NextField(pIDBRec, &fieldName, &nFieldLen);
               switch (fieldType)
               {
                  case AEEDB_FT_STRING:
                     // IDBRECORD_GetFieldString: returns a AECHAR * string for 
                     // a field of that type. Converts the resource string of 
                     // UNICODE/LATIN to the appropriate type for the compiled 
                     // device model.
                     if ((psz = IDBRECORD_GetFieldString (pIDBRec)) != NULL)
                     {
                        // Display output at line 2 on the screen
                        SPRINTF (szBuf, "Field %i value: %s", i+1, psz); 
                        DisplayOutput (pMe, 2, szBuf);
                     }
                     else
                     {
                        DisplayOutput (pMe, 2, "GetFieldDWord: failed");
                     }
                     break;
                  case AEEDB_FT_WORD:
                     // IDBRECORD_GetFieldWord: does an IDBRecord_GetField 
                     // operation on the specified database record, and checks 
                     // the returned field type. If the field type is AEE_FT_WORD, 
                     // this function sets wordValue to point to the buffer
                     // containing the word.
                     if ((bRetValue = IDBRECORD_GetFieldWord (pIDBRec, &wordValue)) == TRUE)
                     {
                        // Display output at line 3 on the screen
                        SPRINTF (szBuf, "Field %i value: %u", i+1, wordValue); 
                        DisplayOutput (pMe, 3, szBuf);
                     }
                     else
                     {
                        DisplayOutput (pMe, 3, "GetFieldWord: failed");
                     }
                     break;
                  case AEEDB_FT_DWORD:
                     // IDBRECORD_GetFieldDWord: does an IDBRecord_GetField 
                     // operation on the specified database record, and checks 
                     // the returned field type. If the field type is AEE_FT_DWORD, 
                     // this function sets dwordValue to point to the buffer
                     // containing the dword
                     if ((bRetValue = IDBRECORD_GetFieldDWord (pIDBRec, &dwordValue)) == TRUE)
                     {
                        // Display the output at line 4 on the screen
                        SPRINTF (szBuf, "Field %i value: %u", i+1, dwordValue); 
                        DisplayOutput (pMe, 4, szBuf);
                     }
                     else
                     {
                        DisplayOutput (pMe, 4, "GetFieldDWord: failed");
                     }
                     break;
               }
            }

            // Now release db rec.
            IDBRECORD_Release (pIDBRec);

            // IDATABASE_Release: closes the open database files, and frees 
            // any memory associated with the database.
            IDATABASE_Release (pIDatabase);
            IDBMGR_Remove (pIDBMgr, "db14");  // Remove the database.
            IDBMGR_Release (pIDBMgr);         // Release the DB manager
         }
         break;
      default:
         return;
   }
   
   // Display above event. 
   DisplayEvent (pMe, wParam);

   return;
}

/*===========================================================================

FUNCTION: DisplayEvent

DESCRIPTION
    This function is a display function which sets up the initial screen
    for the display output for a given interface function. This displays
    the function name at the top of the screen and displays the
    strings "Press Up/Down" and "keys to continue.." at the bottom of 
    the screen. In the case of functions that have no output it also 
    displays the string "No Output..." in the middile of the screen.

PROTOTYPE:
   void DisplayEvent (CIDBApp *pMe, uint16 wParam)

PARAMETERS:
   pMe:    [in]: Pointer to CIDBApp interface object
   wParam: [in]: API function type Id selected by user

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void DisplayEvent (CIDBApp *pMe, uint16 wParam)
{
   AECHAR szBuf[100];
   AEERect rc;

   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
      return;

   // Set display rectangle
   SETAEERECT (&rc, 0, 0, pMe->m_dInfo.cxScreen, pMe->m_dInfo.cyScreen);

   if (pMe->a.m_pIDisplay)
   {
      switch (wParam)
      {
         case USAGE_BASIC_USAGE:
            STR_TO_WSTR("Basic Usage:", szBuf, sizeof(szBuf));
            break;
         case USAGE_DBMGR_OPENDATABASE:
            STR_TO_WSTR("DBMgr: OpenDatabase", szBuf, sizeof(szBuf));
            break;
         case USAGE_DBMGR_OPENDATABASEEX:
            STR_TO_WSTR("DBMgr: OpenDatabaseEx", szBuf, sizeof(szBuf));
            break;
         case USAGE_DBMGR_REMOVE:
            STR_TO_WSTR("DBMgr: Remove", szBuf, sizeof(szBuf));
            break;
         case USAGE_DB_CREATERECORD:
            STR_TO_WSTR("DB: CreateRecord", szBuf, sizeof(szBuf));
            break;
         case USAGE_DB_GETRECORDCOUNT:
            STR_TO_WSTR("DB: GetRecordCount", szBuf, sizeof(szBuf));
            break;
         case USAGE_DB_RESET:
            STR_TO_WSTR("DB: Reset", szBuf, sizeof(szBuf));
            break;
         case USAGE_DB_GETNEXTRECORD:
            STR_TO_WSTR("DB: GetNextRecord", szBuf, sizeof(szBuf));
            break;
         case USAGE_DB_GETRECORDBYID:
            STR_TO_WSTR("DB: GetRecordById", szBuf, sizeof(szBuf));
            break;
         case USAGE_DBREC_RESET:
            STR_TO_WSTR("DBRec: Reset", szBuf, sizeof(szBuf));
            break;
         case USAGE_DBREC_UPDATE:
            STR_TO_WSTR("DBRec: Update", szBuf, sizeof(szBuf));
            break;
         case USAGE_DBREC_REMOVE:
            STR_TO_WSTR("DBRec: Remove", szBuf, sizeof(szBuf));
            break;
         case USAGE_DBREC_GETID:
            STR_TO_WSTR("DBRec: GetID", szBuf, sizeof(szBuf));
            break;
         case USAGE_DBREC_NEXTFIELD:
            STR_TO_WSTR("DBRec: NextFeild", szBuf, sizeof(szBuf));
            break;
         case USAGE_DBREC_GETFIELD:
            STR_TO_WSTR("DBRec: GetField", szBuf, sizeof(szBuf));
            break;
         case USAGE_DBREC_GETFIELDWORD:
            STR_TO_WSTR("DBRec: GetFieldWord", szBuf, sizeof(szBuf));
            break;
         case USAGE_DBREC_GETFIELDDWORD:
            STR_TO_WSTR("DBRec: GetFieldDWord", szBuf, sizeof(szBuf));
            break;
         case USAGE_DBREC_GETFIELDSTRING:
            STR_TO_WSTR("DBRec: GetFieldString", szBuf, sizeof(szBuf));
            break;
         default:
            STR_TO_WSTR("Unsupported fn", szBuf, sizeof(szBuf));
      }
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1, 5, 0, &rc,
		   IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);

      STR_TO_WSTR("Press Up/Down", szBuf, sizeof(szBuf));
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,5, rc.dy - 
         2*pMe->m_nLineHeight, &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
      STR_TO_WSTR("keys to continue..", szBuf, sizeof(szBuf));
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,5, rc.dy - 
         pMe->m_nLineHeight, &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
		IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);
   }
}

/*===========================================================================

FUNCTION: BuildMainMenu

DESCRIPTION
    This function builds the main menu when this applet is started.
    It displays a simple menu with menu items corresponding to 
    each of the interface functions.
    
PROTOTYPE:
   void BuildMainMenu(CIDBApp *pMe)

PARAMETERS:
   pMe: [in]: Pointer to AEEApplet struct

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void BuildMainMenu(CIDBApp *pMe)
{
	AEERect rc;
   AECHAR szBuf[40];

   if (pMe->m_pIMenu == NULL || pMe->a.m_pIShell == NULL)
      return;

	// Set Title
   STR_TO_WSTR("IDB Examples", szBuf, sizeof(szBuf));
	IMENUCTL_SetTitle(pMe->m_pIMenu, NULL, 0, szBuf);

	//Set the Rectangle for the Menu
   SETAEERECT (&rc, 0, 0, pMe->m_dInfo.cxScreen, pMe->m_dInfo.cyScreen);

	IMENUCTL_SetRect(pMe->m_pIMenu, &rc);  

	//Add invidual entries to the Menu

   // Add Basic Usage to menu
   STR_TO_WSTR("1. Basic Usage", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_BASIC_USAGE, szBuf, 0);

   // Add DBMgr-OpenDatabase Event to menu
   STR_TO_WSTR("2. DBMgr: OpenDatabase", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DBMGR_OPENDATABASE, szBuf, 0);

   // Add DBMgr-OpenDatabaseEx to menu
   STR_TO_WSTR("3. DBMgr: OpenDatabaseEx", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DBMGR_OPENDATABASEEX, szBuf, 0);
			
   // Add DBMgr-Remove to menu
   STR_TO_WSTR("4. DBMgr: Remove", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DBMGR_REMOVE, szBuf, 0);

   // Add DB: GetRecordCount to menu
   STR_TO_WSTR("5. DB: CreateRecord", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DB_CREATERECORD, szBuf, 0);

   // Add DB: GetRecordCount to menu
   STR_TO_WSTR("6. DB: GetRecordCount", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DB_GETRECORDCOUNT, szBuf, 0);

	// Add DB: Reset to menu
   STR_TO_WSTR("7. DB: Reset", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DB_RESET, szBuf, 0);

	// Add DB: GetNextRecord to menu
   STR_TO_WSTR("8. DB: GetNextRecord", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DB_GETNEXTRECORD, szBuf, 0);	

	// Add DB: GetRecordById to menu
   STR_TO_WSTR("9. DB: GetRecordById", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DB_GETRECORDBYID, szBuf, 0);	

   // Add DBRec: Reset to menu
   STR_TO_WSTR("10. DBRec: Reset", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DBREC_RESET, szBuf, 0);	

   // Add DBRec: Update to menu
   STR_TO_WSTR("11. DBREc: Update", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DBREC_UPDATE, szBuf, 0);	

	// Add DBRec: Remove to menu
   STR_TO_WSTR("12. DBRec: Remove", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DBREC_REMOVE, szBuf, 0);	

   // Add DBRec: GetID to menu
   STR_TO_WSTR("13. DBRec: GetID", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DBREC_GETID, szBuf, 0);	

   // Add DBRec: NextField to menu
   STR_TO_WSTR("14. DBRec: NextField", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DBREC_NEXTFIELD, szBuf, 0);	

   // Add DBRec: GetField to menu
   STR_TO_WSTR("15. DBRec: GetField", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DBREC_GETFIELD, szBuf, 0);	

   // Add DBRec: GetFieldWord to menu
   STR_TO_WSTR("16. DBRec: GetFieldWord", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DBREC_GETFIELDWORD, szBuf, 0);	

	// Add DBRec: GetFieldDWord to menu
   STR_TO_WSTR("17. DBRec: GetFieldDWord", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DBREC_GETFIELDDWORD, szBuf, 0);	

	// Add DBRec: GetFieldString to menu
   STR_TO_WSTR("18. DBRec: GetFieldString", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DBREC_GETFIELDSTRING, szBuf, 0);	

   // Activate menu
	IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);
}

/*===========================================================================

FUNCTION: DisplayOutput

DESCRIPTION
    This function displays an output string at a given line number on the
    screen. If the nline parameter is a negative value (-1) the string
    is displayed in the middle of the screen. If the "nline" value is larger
    than or equal to zero the "nline" value is multiplied by 15 and the 
    resulting value in pixels is set to the y-coordinate of the start of 
    the string display on the screen. If the string does not fit on one line
    the string wraps around to the next line (spaced rougly 10-15 pixels apart).
    By default 5 is used as the starting the x-coordinate of a displayed 
    string.

    How many characters that fit on one line is calculated for each line 
    that is wrapped around to the next line.

    Note: depending on the phone screen size and the fonts used for characters 
          the output might differ on different handsets (devices). Where some 
          handsets will have a smaller screen and large default fonts which will 
          cause partial overlapping of lines. This function does not try to address
          these issues (this is meant as a simple display function).
    
PROTOTYPE:
   void DisplayOutput(CIDBApp * pMe, int nline, char *pszStr)

PARAMETERS:
   pMe:     [in]: Contains a pointer to the CIDBApp class.
   nline:  [in]: Contains the line number to start displaying the text. 
        The line numbers are by default spaced 15 pixels apart along the y-axis.
   pszStr: [in]: The character string to be displayed on the screen.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void DisplayOutput(CIDBApp * pMe, int nline, char *pszStr)
{
   AECHAR * pszBuf = NULL;
   AECHAR * psz = NULL;
   int pixelWidth;
   AEEFont font = AEE_FONT_NORMAL;
   int pnFits = 0, dy;
   int totalCh = 0;

   if (pMe == NULL)
      return;

   // Allocate buffer to hold string.
   if ((pszBuf = (AECHAR *)MALLOC(200)) == NULL)
      return;

   // Convert to wide string (unicode)
   STR_TO_WSTR ((char *)pszStr, pszBuf, 200);

   // If nlines is zero then print this string starting around the middle of 
   // the screen. Or else multiply nlines by 10 to decide the y coordinate of
   // the start of the string.
   if (nline < 0) {
      dy = pMe->m_dInfo.cyScreen*2/5;
   }
   else{
      dy = nline * pMe->m_nLineHeight;
   }

   // psz keeps track of the point from which to write from the string buffer
   // in case the string does not fit one line and needs to wrap around in the
   // next line.
   psz = pszBuf;
      
   // Need to calculate the lotal string length to decide if any wrapping
   // around is needed.
   totalCh = STRLEN ((char *)pszStr);

   // Keep displaying text string on multiple lines if the string can't be displayed
   // on one single line. Lines are spaced 15 pixels apart.
   while ((totalCh > 0) && (*psz != NULL))
   { 
      // Get information on how many characters will fit in a line.
      // Give the pointer to the buffer to be displayed, and the number of
      // pixels along the x axis you want to display the string in (max number)
      // pnFits will have the max number of chars that will fit in the maxWidth
      // number of pixels (given string can't fit in one line), or the number of 
      // chars in the string (if it does fit in one line). pnWidth gives the
      // number of pixels that will be used to display pnFits number of chars.
      pixelWidth = IDISPLAY_MeasureTextEx(pMe->a.m_pIDisplay,
                      font, 
                      (AECHAR *) psz,  // Start of the buffer to display,
                      -1,
                      pMe->m_dInfo.cxScreen - 5, // maxWidth
                      &pnFits);         // Number of chars that will fit a line

      // If pnFits is zero there is something wrong in the input to above function. 
      // Normally this scenario should not occur. But, have the check anyway.
      if (pnFits == 0)
      {
         FREE (pszBuf);
         return;
      }

      IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, psz, pnFits, 5 /*start dx*/, 
         dy, 0 /* use default rectangle coordinates */, 0);

      psz += pnFits;      // move pointer to the next segment to be displayed
      totalCh -= pnFits;  // reduce the total number of characters to still display
      dy += pMe->m_nLineHeight; // Place next line charHeight pixels below the 
                                // previous line. 
      IDISPLAY_Update(pMe->a.m_pIDisplay);
      if (totalCh < pnFits)
         pnFits = totalCh;  // if total number is less than pnFits, adjust pnFits
   }

   FREE (pszBuf);
   return;   
} // End of DisplayOutput

