/*===========================================================================

FILE: idisplayusage.c

SERVICES: Sample applet using AEE

DESCRIPTION
  This file contains usage examples of IDisplay interface. IDisplay interface
  functions allow the user to create/operate on IDisplay objects. IDisplay 
  interface offers basic sound services.

PUBLIC CLASSES:  
   N/A


   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/

/*===============================================================================
                     INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "AEEModGen.h"
#include "AEEAppGen.h"
#include "AEEShell.h"
#include "AEEMenu.h"
#include "AEEStdLib.h"
#include "AEEUsageAppIDs.h"
#include "idisplay_res.h"
#include "AEEImage.h"

/*===========================================================================

                      PUBLIC DATA DECLARATIONS

===========================================================================*/

/*-------------------------------------------------------------------
            Type Declarations
-------------------------------------------------------------------*/
// Display app struct. This is the main struct for this applet. This will hold 
// all the data members that needs to be remembered throughout the life of
// the applet. 
// THE FIRST DATA MEMBER OF THIS STRUCT MUST BE AN AEEApplet OBJECT. 
// This is critical requirement which is related to an applet being
// dynamically loaded. The app developers should add their data members
// following the AEEApplet data member.
typedef struct _CIDisplayApp
{
	AEEApplet	a;        // Mandatory first AEEApplet data member
   IMenuCtl *  m_pIMenu;
   boolean     bOn;
   boolean     bgColor;

   // Device parameters
   int             m_nLineHeight;
   AEEDeviceInfo   m_dInfo;

}CIDisplayApp;

/*-------------------------------------------------------------------
            Function Prototypes
-------------------------------------------------------------------*/
// App Handle Event function
static boolean DisplayApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);

// App specific data alloc-init/free functions
static boolean IDisplay_InitAppData(IApplet* pMe);
static void IDisplay_FreeAppData(IApplet* pMe);

static void BuildMainMenu(CIDisplayApp *pMe);
static void IDisplayUsage (CIDisplayApp * pMe, uint16 wParam);
static void DisplayEvent (CIDisplayApp *pMe, uint16 wParam);

/*-------------------------------------------------------------------
            Global Constant Definitions
-------------------------------------------------------------------*/
// App Resource File
#define  APP_RES_FILE      "idisplay.bar"

// App specific constants
#define USAGE_BASIC_USAGE      100
#define USAGE_GETFONTMETRICS    101
#define USAGE_MEASURETEXTEX    102
#define USAGE_MEASURETEXT      103
#define USAGE_DRAWTEXT         104
#define USAGE_DRAWRECT         105
#define USAGE_BITBLT           106
#define USAGE_UPDATEEX         107
#define USAGE_BACKLIGHT        108
#define USAGE_SETCOLOR         109
#define USAGE_UPDATE           110
#define USAGE_GETSYMBOL        111
#define USAGE_DRAWFRAME        112
#define USAGE_FRAMEBUTTON      113
#define USAGE_FILLRECT         114
#define USAGE_CLEARSCREEN      115
#define USAGE_FRAMERECT        116
#define USAGE_FRAMESOLIDRECT   117
#define USAGE_INVERTRECT       118
#define USAGE_ERASERECT        119
#define USAGE_ERASERGN         120
#define USAGE_DRAWHLINE        121
#define USAGE_DRAWVLINE        122


/*===============================================================================
                     FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to IDisplay_InitAppData(). 
   This is done as follows: InitAppData() is called to initialize AppletData 
   instance. It is app developers responsibility to fill-in app data initialization 
   code of InitAppData(). App developer is also responsible to release memory 
   allocated for data contained in AppletData -- this can be done in 
   IDisplay_FreeAppData().

PROTOTYPE:
   int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell object. 

	pIModule: pin]: Contains pointer to the IModule object to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid IApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the app needs to be loaded and if AEEApplet_New() invocation was
     successful
  EFAILED: If the app does not need to be loaded or if errors occurred in 
     AEEApplet_New(). If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
{
   *ppObj = NULL;
		
   // We want to load this App. So, invoke AEEApplet_New().To it, pass the
   // address of the app-specific handle event function.
   if(ClsId == AEECLSID_DISPLAY_APP){
      if(AEEApplet_New(sizeof(CIDisplayApp), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)DisplayApp_HandleEvent,(PFNFREEAPPDATA)IDisplay_FreeAppData)
         == TRUE)
      {
         if (IDisplay_InitAppData((IApplet*)*ppObj) == TRUE)
         {
            return(AEE_SUCCESS);
         }
      }
   }
	return (EFAILED);
}

/*===========================================================================

FUNCTION DisplayApp_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean DisplayApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

	ecode: Specifies the Event sent to this applet

   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/
static boolean DisplayApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
   CIDisplayApp * pMe = (CIDisplayApp*)pi;

   switch (eCode) 
	{
       case EVT_APP_START:
         // Create the IMenu interface object. When the applet is started the 
         // main menu is set up. When the user selects a menu item from the main 
         // menu, an EVT_COMMAND event is received by this applet. The processing for 
         // EVT_COMMAND event is found below.
		   if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, (void **)&pMe->m_pIMenu)
            != SUCCESS)
         {
				return FALSE;
         }

         // Build the main menu
			BuildMainMenu(pMe);
			return(TRUE);
      case EVT_APP_STOP:
         return(TRUE);
      case EVT_KEY:
			if(pMe->m_pIMenu != NULL && IMENUCTL_HandleEvent(pMe->m_pIMenu, EVT_KEY, wParam, 0))
				return TRUE;
         else
         {
            // We would have come here because the pressed key was not handled 
            // by either the Display or the MenuCtl HandleEvent functions (the 
            // keys handled by the above controls are Up/Down/Select/Left/Right).
            //
            // Hence, we have reached here since 0-9, *, #, or CLR keys were
            // pressed. So redraw the menu and return true.
          //  IMENUCTL_Redraw (pMe->m_pIMenu);
            return FALSE;
         }
         //   return FALSE;
		case EVT_COMMAND:
         // When a menu item is selected this applet receives the
         // EVT_COMMAND event in the wParam field.
  		   switch(wParam)
         {
            case USAGE_BASIC_USAGE:
            case USAGE_GETFONTMETRICS:
            case USAGE_MEASURETEXTEX:
            case USAGE_MEASURETEXT:
            case USAGE_DRAWTEXT:
            case USAGE_DRAWRECT:
            case USAGE_BITBLT:
            case USAGE_UPDATEEX:
            case USAGE_BACKLIGHT:
            case USAGE_SETCOLOR:
            case USAGE_UPDATE:
            case USAGE_GETSYMBOL:
            case USAGE_DRAWFRAME:
            case USAGE_FRAMEBUTTON:
            case USAGE_FILLRECT:
            case USAGE_CLEARSCREEN:
            case USAGE_FRAMERECT:
            case USAGE_FRAMESOLIDRECT:
            case USAGE_INVERTRECT:
            case USAGE_ERASERECT:
            case USAGE_ERASERGN:
            case USAGE_DRAWHLINE:
            case USAGE_DRAWVLINE:
               // Call the usage function that exercises all the 
               // interface functions.
               IDisplayUsage (pMe, wParam);
               return TRUE;
		   	default:
			   	return FALSE;
         }
      default:
         break;
   }
   return FALSE;
}

/*===========================================================================

FUNCTION IDisplay_InitAppData

DESCRIPTION
   This function initializes app specific data allocates memory for app data 
   (AppletData) and sets it to pAppData of AEEApplet. It also function . 
   App developer needs to fill-in the initialization code.

PROTOTYPE:
	boolean IDisplay_InitAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains 
   information specific to this applet. It was initialized during the 
   AEEClsCreateInstance().

DEPENDENCIES
   Assumes pi is not NULL

RETURN VALUE
  TRUE: If the app has app data is allocated and initialized successfully
  FALSE: Either app data could not be allocated or initialzied

SIDE EFFECTS
  none
===========================================================================*/
static boolean IDisplay_InitAppData(IApplet* pi)
{
   int pnAscent;        // Stores the ascent in number of pixels
   int pnDescent;       // Stores the descent in number of pixels

   CIDisplayApp * pMe = (CIDisplayApp*)pi;

   // Initialize the MenuCtl pointer to NULL
   pMe->m_pIMenu = NULL;
   pMe->bOn = FALSE;
   pMe->bgColor = FALSE;

   // Get the font metrics info
   pMe->m_nLineHeight = IDISPLAY_GetFontMetrics (pMe->a.m_pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&pMe->m_dInfo);

   return TRUE;
}

/*===========================================================================

FUNCTION IDisplay_FreeAppData

DESCRIPTION
   Frees data contained in app data and memory for app data itself.
   App developer needs to free data contained in AppletData.

PROTOTYPE:
	void IDisplay_FreeAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains 
   information specific to this applet. It was initialized during the 
   AEEClsCreateInstance().

DEPENDENCIES
   Assumes pi is not NULL

RETURN VALUE
   None

SIDE EFFECTS
   None
===========================================================================*/
static void IDisplay_FreeAppData(IApplet* pi)
{
   CIDisplayApp * pMe = (CIDisplayApp*)pi;

   // Release main menu
   if (pMe->m_pIMenu != NULL)
   {
      IMENUCTL_Release (pMe->m_pIMenu);
      pMe->m_pIMenu = NULL;
   }
}

/*===============================================================================
                     APP HELPER FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: IDisplayUsage

DESCRIPTION
	This function encompasses all the usage examples of all the functions
   in code blocks switched using the BREW API function Id passed into this
   function. The code blocks will include almost all the variable declarations
   needed to demonstrate the usage of a given function. Within the code block
   all the needed interfaces are instantiated and released subsequent to their
   use.

   There are some exceptions to the above general rule, such as call-back functions,
   common helper function, or common display functions called from within the
   code block which are defined elsewhere in this file.

   There are some instances where an object instantiated within a code block is
   not released at the end of the code block, to be released at a later point
   (for example in a call-back function, in this case pointers to such objects
   are stored in the AEEApplet object to be released later).
   
PROTOTYPE:
   void IDisplayUsage (CIDisplayApp * pMe, uint16 wParam)

PARAMETERS:
	pMe: [in]: Contains a pointer to the CIDisplayApp struct.
   wParam: [in]: Contains the Id of the function type to be used (contains the
      Id corresponding to the menu item selected on the emulator).

DEPENDENCIES
  none

RETURN VALUE
  none

SIDE EFFECTS
  none
===========================================================================*/
static void IDisplayUsage (CIDisplayApp * pMe, uint16 wParam)
{
   AEERect rect;
   int lineHeight      = pMe->m_nLineHeight;
   IShell *pIShell     = pMe->a.m_pIShell;
   IDisplay *pIDisplay = pMe->a.m_pIDisplay;
   AEEDeviceInfo *di   = &pMe->m_dInfo;        // Device Info

   if (pMe == NULL || pIShell == NULL || pIDisplay == NULL || di == NULL)
      return;

   // Erase screen first for display output purposes.
   IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);

   switch (wParam)
   {
      case USAGE_BASIC_USAGE:
         {
            // 1. Usage: Basic IDisplay Usage
            AEERect rct;
            RGBVAL clrFrame, clrFill;
            int charHeight, pnAscent, pnDescent, yStrRoot = 0;
            int pixelWidth, nFits, nLineHeight;
            AECHAR szBuf [50] = {0};
            char szBuffer [50] = {0};
            AEEFrameType fType;
            uint32 dwFlags;

            // Following section gives usage examples of some of the
            // IDisplay functions. This section does not go into a lot of
            // details of each function call nor does it illustrate all the
            // possible variations of the functions outputs. To see detailed
            // function usage examples please see the corresponding usage
            // sections.
            // 
            // The steps that follow are outlined below:
            // 1. Invoke IDISPLAY_GetFontMetrics and get the character height,
            //    ascent, and decent of a NORMAL font character.
            // 2. Use the character height from step 1 to initialize couple
            //    of local variables that keep track of the locations to 
            //    place the next output of functions.
            // 3. Display the character height, ascent, and descent values 
            //    from Step 1. of the screen.
            // 4. Invoke IDISPLAY_MeasureTextEx to get the pixel width of the
            //    string print on the screen in Step 3 using LARGE font.
            // 5. Change the Text-color to Blue by invoking IDISPLAY_SetColor.
            // 6. Display the pixel width and number of characters that will
            //    fit in one screen width (nFits) below the text drawn in
            //    Step 3. The text will be BOLD font and will be in the 
            //    text color set in Step 5 (Blue).
            // 7. Change the Text-color to Red by invoking IDISPLAY_SetColor.
            // 8. Load resource string from the resource file.
            // 9. Display resource string on the screen. This string will 
            //    be displayed in BOLD font and in the color set in Step 7.
            //    (Red).
            // 10. Initialize parameters and display a rectangle by calling
            //     IDISPLAY_DrawRect. This step will draw a red color filled
            //     blue outlined rectangle.
            // 11. Set the Text-color to Blue by invoking IDISPLAY_SetColor.
            // 12. Set the Frame-color to red using IDISPLAY_SetColor function
            //     with the CLR_USER_FRAME as the second parameter.
            // 13. Draw a raised frame in Cyan color by invoking IDISPLAY_DrawFrame.
            // 14. Set the Frame-color to Green by invoking IDISPLAY_SetColor.
            // 15. Draw a rectangle filled with Red color using IDISPLAY_FillRect
            //     function.
            // 16. Set the Frame-color to Yellow by invoking IDISPLAY_SetColor.
            // 17. Draw a raise framed button filled in Green color using the
            //     IDISPLAY_FrameButton function.
            // 18. Erase a small rectangle region on the screen by invoking
            //     IDISPLAY_EraseRgn function.
            // 19. Draw a text string in the center of the screen using the 
            //     LARGE font.
            // 20. Draw a vertical line acorss the screen using the 
            //     IDISPLAY_DrawVLine function.
            // 21. Draw a horizontal line acorss the screen using the 
            //     IDISPLAY_DrawHLine function.
            // 22. Set the Text-color back to Black by invoking IDISPLAY_SetColor
            //     with CLR_USER_TEXT as the second parameter.
            // 23. Enable the backlight by invoking IDISPLAY_Backlight with
            //     second parameter specified as TRUE.
            // 24. Initialize a AEERect object with values of an area on the 
            //     screen and invoke IDISPLAY_InvertRect function. This will
            //     result in all the colors on the screen under the area of
            //     rectangle being inverted.
            // 25. Update all the drawing to the screen by invoking 
            //     IDISPLAY_Update function.

            // Step 1. Get font metrics for NORMAL font.
            charHeight = IDISPLAY_GetFontMetrics (pIDisplay, AEE_FONT_NORMAL,
                &pnAscent, &pnDescent);

            // Step 2. Initialize variables used for display purposes.
            yStrRoot = charHeight;
            nLineHeight = charHeight;

            // Step 3. Use IDISPLAY_DrawText to draw values from Step 1.
            SPRINTF (szBuffer, "Normal: %i/%i/%i", charHeight, pnAscent, pnDescent);
            STR_TO_WSTR (szBuffer, szBuf, sizeof (szBuf));
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, szBuf, -1, di->cxScreen/5,
               yStrRoot, 0, 0);

            // Step 4. Measure text info for the string displayed in step 3.
            //         using LARGE font.
            pixelWidth = IDISPLAY_MeasureTextEx(pIDisplay, AEE_FONT_LARGE,
                         szBuf, -1, di->cxScreen, &nFits);

            // Step 5. Set text color to Blue
            IDISPLAY_SetColor(pIDisplay,CLR_USER_TEXT,MAKE_RGB(0x0,0x0,0xff));

            // Step 6. Display values returned in Step 4.
            SPRINTF (szBuffer, "pixelW/nFits: %i/%i", pixelWidth, nFits);
            STR_TO_WSTR (szBuffer, szBuf, sizeof (szBuf));
            yStrRoot += nLineHeight; 
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_BOLD, szBuf, -1, 10, yStrRoot, 0, 0);

            // Step 7. Set text color to Red.
            IDISPLAY_SetColor(pIDisplay,CLR_USER_TEXT,MAKE_RGB(0xff,0,0));

            // Step 8. Load string from resource file.
            ISHELL_LoadResString(pMe->a.m_pIShell, APP_RES_FILE, IDS_DRAWTEXT, 
                                 szBuf, sizeof (szBuf));

            // Step 9. Draw the string loaded from the resource file.
            yStrRoot += nLineHeight;    // Calculate the next string y-axis location
            SETAEERECT (&rct, 0, yStrRoot, di->cxScreen, charHeight);
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_BOLD, szBuf, -1, 0, yStrRoot, 
               &rct, IDF_ALIGN_CENTER);

            // Step 10. Draw rectangle with filled with Red color and with a 
            //          frame in Blue.
            yStrRoot += nLineHeight;             
            SETAEERECT (&rct, 5, yStrRoot, di->cxScreen/2, nLineHeight); 
            clrFrame = MAKE_RGB (0, 0, 0xff);         // Blue color
            clrFill =  MAKE_RGB (0xff, 0, 0);         // Red color
            dwFlags = IDF_RECT_FILL | IDF_RECT_FRAME;
            // Draw rectangle with values initialized above
            IDISPLAY_DrawRect (pIDisplay, &rct, clrFrame, clrFill, dwFlags);

            // Step 11. Set text color ro Blue.
            IDISPLAY_SetColor(pIDisplay,CLR_USER_TEXT,MAKE_RGB(0x0,0x0,0xff));

            // Step 12. Set frame color to Red
            IDISPLAY_SetColor(pIDisplay,CLR_USER_FRAME,MAKE_RGB(0xff,0,0));

            // Step 13. Draw raised frame, color filled with Cyan color.
            yStrRoot += nLineHeight;
            SETAEERECT (&rct, di->cxScreen*2/3, yStrRoot, di->cxScreen/4, nLineHeight);
            fType = AEE_FT_RAISED;
            clrFill = MAKE_RGB (0, 0xff, 0xff);
            IDISPLAY_DrawFrame (pIDisplay, &rct, fType, clrFill);

            // Step 14. Change frame color to Blue.
            yStrRoot += nLineHeight;
            IDISPLAY_SetColor(pIDisplay,CLR_USER_FRAME,MAKE_RGB(0x0,0xff,0));

            // Step 15. Draw a rectangle filled with Red color
            SETAEERECT (&rct, 0, yStrRoot, di->cxScreen/4, nLineHeight);
            clrFill = MAKE_RGB (0xff, 0, 0);
            IDISPLAY_FillRect (pIDisplay, &rct, clrFill);

            // Step 16. Change frame color to Yellow.
            IDISPLAY_SetColor(pIDisplay,CLR_USER_FRAME,MAKE_RGB(0xff,0xff,0));

            // Step 17. Draw a lowered frame button filled with Green color.
            yStrRoot += nLineHeight;
            SETAEERECT (&rct, di->cxScreen/4, yStrRoot, di->cxScreen/2, nLineHeight);
            clrFill = MAKE_RGB (0, 0xff, 0);
            // Draw a raised button
            IDISPLAY_FrameButton (pIDisplay, &rct, FALSE, clrFill);

            // Step 18. Erase region on the screen
            IDISPLAY_EraseRgn (pIDisplay, di->cxScreen/2, di->cxScreen/2,
                               di->cxScreen/10, di->cyScreen/10);

            // Step 19. Draw a text string in LARGE font.
            STR_TO_WSTR ("Text String", szBuf, sizeof (szBuf));
            yStrRoot += nLineHeight;    // Calculate the next string y-axis location
            SETAEERECT (&rct, 0, yStrRoot, di->cxScreen, charHeight);
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_LARGE, szBuf, -1, 0, yStrRoot, 
               &rct, IDF_ALIGN_CENTER);

            // Step 20. Draw a vertical line on the screen
            IDISPLAY_DrawVLine(pIDisplay, di->cxScreen/5, di->cyScreen/5, 
               di->cyScreen/2);

            // Step 21. Draw a horizontal line on the screen.
            IDISPLAY_DrawHLine(pIDisplay, di->cxScreen/8, yStrRoot + nLineHeight/2, 
               di->cxScreen*3/4);

            // Step 22. Set text color back to Black
            IDISPLAY_SetColor(pIDisplay,CLR_USER_TEXT,RGB_BLACK);

            // Step 23. Set the backlight to ON.
            IDISPLAY_Backlight(pIDisplay, TRUE);

            // Step 24. Invert rectangle area on the screen
            SETAEERECT (&rct, di->cxScreen/3, nLineHeight, di->cxScreen/10, 
               di->cyScreen*3/4);
            IDISPLAY_InvertRect (pIDisplay, &rct);

            // Step 25. Update the display
            IDISPLAY_Update(pIDisplay);

         }
         break;
      case USAGE_GETFONTMETRICS:
         {
            // 2. Usage: IDISPLAY_GetFontMetrics
            int charHeight = 0;      // Stores the char height in pixels for given font
            int pnAscent = 0;        // Stores the ascent in number of pixels
            int pnDescent = 0;       // Stores the descent in number of pixels
            char szText[40] = {0};
            AECHAR szBuf[40] = {0};
            int yStrRoot = 0;
            
            // The following section uses the IDISPLAY_GetFontMetrics function to 
            // get the character Height/Ascent/Decent information and prints it to 
            // screen using the font used to get the values. 
            //
            // First the heading "Font: Hgt/Asnt/Dcnt:" is displayed on the screen. 
            // This string placed at the top of the screen. The character height 
            // returned from the IDISPLAY_GetFontMetrics function is used to place 
            // the text string on the screen. IDISPLAY_GetFontMetrics function is 
            // used to get the height, ascent, and descent of a character of fonts
            // Normal, Bold, and Large and each of the values are displayed on the
            // screen using the given font.

            // IDISPLAY_GetFontMetrics: retrieves information about the specified font. 
            // It retrieves information about the ascent and descent values for the 
            // specified font. All the values are in pixel numbers. For example, this 
            // function returns the height of a character in pixels.
            // Note: ascent+descent = total character height, but does not include 
            //       any leading spaces.
            charHeight = IDISPLAY_GetFontMetrics (pIDisplay, AEE_FONT_NORMAL,
                &pnAscent, &pnDescent);

            // Display a heading at the top of the screen. Character height is 
            // used to calculate where to place the string on the screen in the y-axis
            // direction.
            yStrRoot = charHeight;
            STR_TO_WSTR ("Font: Hight/Asnt/Dsnt:", szBuf, sizeof (szBuf));
            // Display the string on the screen. See IDISPLAY_DrawText function
            // usage for more details on this function.
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_BOLD, szBuf, -1, 0, yStrRoot, 0, 0);

            // Calculate the location to place the next string on the screen. This would
            // be two charHeights below the previously displayed string.
            yStrRoot += charHeight*2;

            // Display the Height/Ascent/Descent values for the NORMAL font on the screen.
            SPRINTF (szText, "Normal: %i/%i/%i", charHeight, pnAscent, pnDescent);
            STR_TO_WSTR (szText, szBuf, sizeof (szBuf));
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, szBuf, -1, 0, yStrRoot, 0, 0);

            // Calculate the location to place the next string on the screen. This would
            // be one charHeight pixels more than the previously displayed string.
            yStrRoot += charHeight;    // Next location to draw text

            // Invoke IDISPLAY_GetFontMetrics again to get the Height/Ascent/Descent
            // for a character displayed in BOLD font.
            charHeight = IDISPLAY_GetFontMetrics (pIDisplay, AEE_FONT_BOLD,
                &pnAscent, &pnDescent);

            // Display the values for BOLD font on the screen
            SPRINTF (szText, "Bold: %i/%i/%i", charHeight, pnAscent, pnDescent);
            STR_TO_WSTR (szText, szBuf, sizeof (szBuf));
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_BOLD, szBuf, -1, 0, yStrRoot, 0, 0);

            // Calculate the location to place the next string on the screen. This would
            // be one charHeight pixels more than the previously displayed string.
            yStrRoot += charHeight;
            
            // Invoke IDISPLAY_GetFontMetrics again to get the Height/Ascent/Descent
            // for a character displayed in LARGE font.
            charHeight = IDISPLAY_GetFontMetrics (pIDisplay, AEE_FONT_LARGE,
                &pnAscent, &pnDescent);

            // Display the values for LARGE font on the screen
            SPRINTF (szText, "Large: %i/%i/%i", charHeight, pnAscent, pnDescent);
            STR_TO_WSTR (szText, szBuf, sizeof (szBuf));
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_LARGE, szBuf, -1, 0, yStrRoot, 0, 0);

            // Update screen
            IDISPLAY_Update (pIDisplay);
         }
         break;
      case USAGE_MEASURETEXTEX:
         {
            // 3. Usage: IDISPLAY_MeasureTextEx
            int pixelWidth = 0;
            int nFits = 0;
            int yStrRoot, totalStrLen = 0;
            char szText [] = "MeasureTextEx String";
            AECHAR szBuf [50] = {0};
            AECHAR *pTmp = NULL;
            char szTmp1 [50] = {0};
            AECHAR szTmp2 [50] = {0};

            // Following section gives example usages of the IDISPLAY_MeasureTextEx
            // function. This function is invoked three times with three 
            // different fonts. Each time the last parameter "nFits" is used to 
            // check if the string will fit in one width of the screen and if not
            // the the section which did not fit on one line is prited on the next 
            // line on the screen. The string defined above in szText is used
            // for all the measurements and display.
            //
            // The steps that follow are outlined below:
            // 1. Invoke IDISPLAY_MeasureTextEx functions retrieve the pixel
            //    width the string will take, given NORMAL font.
            // 2. Display the pixel width of the string for NORMAL font
            // 3. Display the string in NORMAL font. 
            // 4. Use the "nFits" value to see how many characters of the string 
            //    fit in one width of the screen. If some of the characters does 
            //    not fit, display them on the next line (below the previously 
            //    displayed text). This is done for only two lines (even if the 
            //    text does not fully fit in two lines).
            // 5. Invoke IDISPLAY_MeasureTextEx functions retrieve the pixel
            //    width the string will take, given BOLD font.
            // 6. Display the pixel width of the string for BOLD font
            // 7. Display the string in BOLD font. 
            // 8. Use the "nFits" value to see how many characters of the string 
            //    fit in one width of the screen. If some of the characters does 
            //    not fit, display them on the next line (below the previously 
            //    displayed text). This is done for only two lines (even if the 
            //    text does not fully fit in two lines).
            // 9. Invoke IDISPLAY_MeasureTextEx functions retrieve the pixel
            //    width the string will take, given LARGE font.
            // 10. Display the pixel width of the string for LARGE font
            // 11. Display the string in LARGE font. 
            // 12. Use the "nFits" value to see how many characters of the string 
            //    fit in one width of the screen. If some of the characters does 
            //    not fit, display them on the next line (below the previously 
            //    displayed text). This is done for only two lines (even if the 
            //    text does not fully fit in two lines).

            // Convert to wide string format
            STR_TO_WSTR (szText, szBuf, sizeof (szBuf));
            totalStrLen = STRLEN (szText);

            // Step 1.
            //
            // IDISPLAY_MeasureTextEx: measures the width of a given text string
            // if drawn using the specified font. The return is the actual pixel 
            // width of the string. 
            //
            // nFits parameter will contain the number of character from the string 
            // in szBuf that will fit in di->cxScreen pixels across the screen.
            pixelWidth = IDISPLAY_MeasureTextEx(pIDisplay,
                      AEE_FONT_NORMAL, // Use NORMAL font
                      szBuf,           // Start of the buffer to display,
                      -1,              // How many char of the given string to be 
                                       // used in the measurement. In this case -1
                                       // will use the whole string.
                      di->cxScreen,     // maximum pixel width that can be used to 
                                       // draw string. Use screen width
                      &nFits);         // Number of chars that will fit a line


            // Step 2.
            //
            // Print the string width in pixels at the top of the screen.
            SPRINTF (szTmp1, "Width (Normal): %i", pixelWidth);
            STR_TO_WSTR (szTmp1, szTmp2, sizeof (szTmp2));
            yStrRoot = lineHeight;       // y coordinate on the screen to print
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, szTmp2, -1, 0, yStrRoot, 0, 0);

            // Step 3.
            //
            // Calculate the location to place the next string on the screen. This would
            // be one lineHeight pixels more than the previously displayed string.
            yStrRoot += lineHeight;
            // Display "nFits" number of char on the screen at "yStrRoot" on the y-axis
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, szBuf, nFits, 0, yStrRoot, 0, 0);

            // Step 4.
            //
            // Check if we have only part of the string on the screen. If so, print
            // the rest on the next line.
            //
            // Note: This section demonstrates how "nFits" can be used to figure out
            //       how many characters of a string fits on a given number of pixels
            //       (in this case the screen width).
            // Depending on the screen dimensions the string might fit on one or 
            // more lines. This example only prints the string on two lines if it does
            // go over one line.
            if (totalStrLen > nFits)
            {
               pTmp = szBuf + nFits;      // New location in the string to display from
               yStrRoot += lineHeight;   // Calculate the next string y-axis location
               IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, pTmp, (totalStrLen-nFits), 
                  0, yStrRoot, 0, 0);
            }

            // Step 5.
            //
            // Measure Text using the BOLD font
            pixelWidth = IDISPLAY_MeasureTextEx(pIDisplay, AEE_FONT_BOLD,
                         szBuf, -1, di->cxScreen, &nFits); 

            // Step 6.
            //
            // Print the string width in pixels on the screen.
            SPRINTF (szTmp1, "Width (Bold): %i", pixelWidth);
            STR_TO_WSTR (szTmp1, szTmp2, sizeof (szTmp2));
            yStrRoot += lineHeight;    // Calculate the next string y-axis location
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_BOLD, szTmp2, -1, 0, yStrRoot, 0, 0);

            // Step 7.
            //
            yStrRoot += lineHeight;    // Calculate the next string y-axis location
            // Display "nFits" number of char on the screen at "yStrRoot" on the y-axis
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_BOLD, szBuf, nFits, 0, yStrRoot, 0, 0);

            // Step 8.
            //
            // Check if we have only part of the string on the screen. If so, print
            // the rest on the next line.
            //
            // Depending on the screen dimensions the string might fit on one or 
            // more lines. This example only prints the string on two lines if it does
            // go over one line.
            if (totalStrLen > nFits)
            {
               pTmp = szBuf + nFits;    // New location in the string to display from
               yStrRoot += lineHeight;    // Calculate the next string y-axis location
               IDISPLAY_DrawText(pIDisplay, AEE_FONT_BOLD, pTmp, (totalStrLen-nFits), 
                  0, yStrRoot, 0, 0);
            }

            // Step 9.
            //
            // MeasureTextEx using the LARGE font
            pixelWidth = IDISPLAY_MeasureTextEx(pIDisplay, AEE_FONT_LARGE,
                         szBuf, -1, di->cxScreen, &nFits); 

            // Step 10.
            //
            // Print the string width in pixels on the screen.
            SPRINTF (szTmp1, "Width (Large): %i", pixelWidth);
            STR_TO_WSTR (szTmp1, szTmp2, sizeof (szTmp2));
            yStrRoot += lineHeight;     // Calculate the next string y-axis location
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_LARGE, szTmp2, -1, 0, yStrRoot, 0, 0);

            // Step 11.
            //
            // Display "nFits" number of char on the screen at "yStrRoot" on the y-axis
            yStrRoot += lineHeight + 2; // Add 2 more pixels since lineHeight calculated
                                      // using NORMAL font.                                
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_LARGE, szBuf, nFits, 0, yStrRoot, 0, 0);

            // Step 12.
            //
            // Check if we have only part of the string on the screen. If so, print
            // the rest on the next line.
            //
            // Depending on the screen dimensions the string might fit on one or 
            // more lines. This example only prints the string on two lines if it does
            // go over one line.
            if (totalStrLen > nFits)
            {
               yStrRoot += lineHeight + 2;  // Calculate the next string y-axis location
               pTmp = szBuf + nFits;      // New location in the string to display from
               IDISPLAY_DrawText(pIDisplay, AEE_FONT_LARGE, pTmp, (totalStrLen-nFits), 
                  0, yStrRoot, 0, 0);
            }

            // Update screen
            IDISPLAY_Update (pIDisplay);
         }
         break;
      case USAGE_MEASURETEXT:
         {
            // 4. Usage: IDISPLAY_MeasureText
            int pixelWidth = 0;
            int yStrRoot;
            char szText [30] = "String: MeasureText";
            AECHAR szBuf [30] = {0};
            char szTmp1 [30] = {0};
            AECHAR szTmp2 [30] = {0};

            // Following section gives example usages of the IDISPLAY_MeasureText
            // function. This function is invoked three times with three 
            // different fonts. The width in pixels returned is displayed on 
            // the screen for each font.
            //
            // The steps that follow are outlined below:
            // 1. Display on the screen the string that we are using in the
            //    IDISPLAY_MeasureText function invokes.
            // 2. Invoke IDISPLAY_MeasureText functions retrieve the pixel
            //    width the string will take, given NORMAL font.
            // 3. Display the width returned for NORMAL font.
            // 4. Invoke IDISPLAY_MeasureText functions retrieve the pixel
            //    width the string will take, given BOLD font.
            // 5. Display the width returned for BOLD font.
            // 6. Invoke IDISPLAY_MeasureText functions retrieve the pixel
            //    width the string will take, given LARGE font.
            // 7. Display the width returned for LARGE font.

            // Step 1.
            //
            // Convert to wide string format
            STR_TO_WSTR (szText, szBuf, sizeof (szBuf));
            // The first location along the y-axis to print string
            yStrRoot = lineHeight;
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_BOLD, szBuf, -1, 0, yStrRoot, 0, 0);

            // Step 2.
            //
            // IDISPLAY_MeasureText: measures the width of a given text string if 
            // drawn using the specified font
            pixelWidth = IDISPLAY_MeasureText(pIDisplay,
                      AEE_FONT_NORMAL,   // Use NORMAL font
                      szBuf);            // Start of the buffer to display,

            // Step 3.
            //
            // Print the string width in pixels at the top of the screen.
            SPRINTF (szTmp1, "Width (Normal): %i", pixelWidth);
            STR_TO_WSTR (szTmp1, szTmp2, sizeof (szTmp2));
            yStrRoot += lineHeight*2;       // y coordinate on the screen to print
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, szTmp2, -1, 0, yStrRoot, 0, 0);

            // Step 4.
            //
            // Measure Text using the BOLD font
            pixelWidth = IDISPLAY_MeasureText(pIDisplay, AEE_FONT_BOLD, szBuf);

            // Step 5.
            //
            // Print the string width in pixels on the screen.
            SPRINTF (szTmp1, "Width (Bold): %i", pixelWidth);
            STR_TO_WSTR (szTmp1, szTmp2, sizeof (szTmp2));
            yStrRoot += lineHeight;    // Calculate the next string y-axis location
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_BOLD, szTmp2, -1, 0, yStrRoot, 0, 0);

            // Step 6.
            //
            // MeasureText using the LARGE font
            pixelWidth = IDISPLAY_MeasureText(pIDisplay, AEE_FONT_LARGE, szBuf);

            // Step 7.
            //
            // Print the string width in pixels on the screen.
            SPRINTF (szTmp1, "Width (Large): %i", pixelWidth);
            STR_TO_WSTR (szTmp1, szTmp2, sizeof (szTmp2));
            yStrRoot += lineHeight;     // Calculate the next string y-axis location
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_LARGE, szTmp2, -1, 0, yStrRoot, 0, 0);
         }
         break;
      case USAGE_DRAWTEXT:
      case USAGE_UPDATEEX:
         {
            // 5. Usage: IDISPLAY_DrawText
            int xStrRoot = 0;
            int yStrRoot = 0;
            int nCopied = 0;
            AECHAR szBuf [30] = {0};
            AEERect prc;
            boolean bDefer;

            // First example is to display a string retrieved from the app
            // resource file. Here we use the ISHELL_LoadResString function
            // to retrieve a string defined in resource file. This function
            // returns the number of bytes coped. Zero is returned 
            // if the string was not found or if the buffer szBuf is NULL.
            nCopied = ISHELL_LoadResString(pMe->a.m_pIShell, APP_RES_FILE, 
                                     IDS_DRAWTEXT, szBuf, sizeof (szBuf));

            // Check if loading the resource string was successful.
            if (nCopied != 0)
            {
               // Initialize a rectangle which is located at the top of 
               // screen, with the height 2 pixels more than a character height,
               // and the width of the screen.
               SETAEERECT (&prc, 0, lineHeight, di->cxScreen, lineHeight+2);
               // Specify the location on the y-axis to draw the text. This
               // location is the root of the above specified rectangle.
               yStrRoot += lineHeight;

               // IDISPLAY_DrawText: draws the given text at the given location, using 
               // the given font and bounds the text to the clipping rectangle.
               // After calling this function, IDISPLAY_Update() must be called to 
               // update the screen. If there is a sequence of drawing operations 
               // being performed, it is sufficient to IDISPLAY_Update() once after 
               // all the drawing is done.
               IDISPLAY_DrawText(pIDisplay, 
                       AEE_FONT_BOLD,   // Font used to print string
                       szBuf,           // Buffer containing the string
                       -1,              // Number of characters to print from the string.
                       xStrRoot,        // x coordinate to place the root of string
                       yStrRoot,        // y coordinate to place the root of string
                       &prc,            // Clipping rectangle. If 0 is used the function 
                                        // automatically calculates the rectangle to 
                                        // fit the text.
                       IDF_ALIGN_CENTER);
            }
                                        // it exactly surround the text string. If 0
                                        // is used the function automatically 
                                        // calculates the rectangle to fit the text.
                                        // string. When -1 is used the DrawText
                                        // function calulates the string length.

            // The following section the IDISPLAY_DrawText function is
            // invoked with different flags and fonts set. All the text
            // string will be locally defined (not taken from resource file).
            //
            // The steps that follow are outlined below:
            // 1. Draw string "Left" on the left side of the screen, below
            //    the string from resource file. IDF_ALIGN_LEFT flag places
            //    the string in the left side of the rectangle.
            // 2. Draw string "Right" on the right side of the screen, below
            //    the string from resource file. IDF_ALIGN_RIGHT flag places
            //    the stringin the right side of of the rectangle.
            // 3. Draw string "Center" in the center of the screen, below 
            //    the above mentioned strings ("Left" and "right"). 
            //    IDF_ALIGN_CENTER flag causes the string be displayed in the
            //    middle of the specified rectangle.
            // 4. Set up a rectangle which is below the displayed "Center"
            //    string but spaning the left 1/3 of the screen and is three
            //    character heights in height. In this rectangle, string "Top",
            //    is drawn in LARGE font. The flags IDF_ALIGN_TOP  align the
            //    the text to the top of the specified rectangle. And 
            //    IDF_ALIGN_LEFT flag causes the string be aligned left.
            // 5. Set up a similar rectangle to the one specified in the previous
            //    step but is located in the middle 1/3 of the screen (aligned
            //    vertically). In this rectangle draw string "Middle" in NORMAL
            //    font. This string is drawn with flags set to IDF_ALIGN_MIDDLE, 
            //    IDF_ALIGN_CENTER, and IDF_TEXT_UNDERLINE. The IDF_ALIGN_MIDDLE
            //    flag causes the text string to be placed in the middle of the 
            //    specified rectangle aligned in the vertical plane.
            //    The IDF_ALIGN_CENTER flag places the string in the middle of the 
            //    rectangle in the horizontal plane. The IDF_TEXT_UNDERLINE flag
            //    causes the text to be underlined.
            // 6. Set up a rectangle that is the same spcifications as the 
            //    two previous steps, except that it is placed the the right of
            //    rectangle in the previous step. In this rectangle the string
            //    "Bottom" is drawn in BOLD font. The string is drawn with 
            //    the IDF_ALIGN_BOTTOM, IDF_ALIGN_RIGHT, and IDF_TEXT_INVERTED
            //    flags set. The IDF_ALIGN_BOTTOM flag places the string at the
            //    bottom on the specified rectangle, the IDF_ALIGN_RIGHT flag
            //    aligns the string to the right side, and the IDF_TEXT_INVERTED
            //    flag inverts the whole rectangle and the string in color.

            // 1. Draw "Left" string on the screen.
            // Calculate the y-axis location of the next text string
            yStrRoot += lineHeight;
            // Set a rectangle that is placed below the previously 
            // displayed text. The rectangle is of a character height
            // and spaning over the left half of the screen
            SETAEERECT (&prc, 0, yStrRoot, di->cxScreen/2, lineHeight);
            // Convert the text string to wide string format
            STR_TO_WSTR ("Left", szBuf, sizeof (szBuf));
            // Display text string "Left" aligned left in the rectangle 
            // specified above.
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, szBuf, -1, 
                       xStrRoot, yStrRoot, &prc, IDF_ALIGN_LEFT);

            // 2. Draw the "Right" string on the screen.
            // Convert the text to wide string format
            STR_TO_WSTR ("Right", szBuf, sizeof (szBuf));
            prc.x = di->cxScreen/2;
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, szBuf, -1,
                       xStrRoot, yStrRoot, &prc, IDF_ALIGN_RIGHT);

            // 3. Draw the string "Center" on the screen.
            // Calculate the y-axis location of the next text string
            yStrRoot += lineHeight + 1;
            // Change the location and the width of the rectangle.
            prc.x = 0;
            prc.y = yStrRoot;
            prc.dx = di->cxScreen;  // Use the whole width of the screen
            // Convert the text to wide string format
            STR_TO_WSTR ("Center", szBuf, sizeof (szBuf));
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, szBuf, -1,
                       xStrRoot, yStrRoot, &prc, IDF_ALIGN_CENTER);

            // 4. Draw string "Top" on the screen
            // Calculate the y-axis location of the next text string
            yStrRoot += lineHeight;
            // Set up a rectangle in the region below string "Center", 
            // aligned left, 1/3 in width as the screen width, and 
            // three lineHeights high.
            SETAEERECT (&prc, 0, yStrRoot, di->cxScreen/3, lineHeight*3);
            STR_TO_WSTR ("Top", szBuf, sizeof (szBuf));
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_LARGE, szBuf, -1,
                       xStrRoot, yStrRoot, &prc, IDF_ALIGN_TOP | IDF_ALIGN_LEFT);

            // 5. Draw atring "Middle" on the screen
            STR_TO_WSTR ("Middle", szBuf, sizeof (szBuf));
            // Set up a rectangle in the region below string "Center", 
            // 1/3 in width as the screen width, and three lineHeights high.
            SETAEERECT (&prc, di->cxScreen/3, yStrRoot, di->cxScreen/3, lineHeight*3);
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, szBuf, -1,
                       xStrRoot, yStrRoot, &prc, IDF_ALIGN_MIDDLE | 
                       IDF_ALIGN_CENTER | IDF_TEXT_UNDERLINE);

            // 6. Draw string "Bottom" on the screen
            // Set up a rectangle in the region below string "Center", 
            // aligned right on the screen, 1/3 in width as the screen width, 
            // and three lineHeights high.
            SETAEERECT (&prc, di->cxScreen*2/3, yStrRoot, di->cxScreen/3, lineHeight*3);
            STR_TO_WSTR ("Bottom", szBuf, sizeof (szBuf));
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_BOLD, szBuf, -1,
                       xStrRoot, yStrRoot, &prc, IDF_ALIGN_BOTTOM | 
                       IDF_ALIGN_RIGHT | IDF_TEXT_INVERTED);

            // IDISPLAY_UpdateEx: updates the screen. If the bDefer flag is 
            // set to true, the screen is refreshed (updated) immediately. 
            // If the bDefer flag is set to false, the update message is 
            // posted onto the queue of the UI task, thereby allowing all 
            // the drawings to be done before updating the screen.
            bDefer = TRUE;             // Update screen immediately
            IDISPLAY_UpdateEx (pIDisplay, bDefer);
         }
         break;
      case USAGE_DRAWRECT:
         {
            // 6. Usage: IDISPLAY_DrawRect
            AEERect rct = {0};
            RGBVAL clrFrame, clrFill;
            uint32 dwFlags;

            // Following section gives example usages of the IDISPLAY_DrawRect
            // function. This function is invoked five times with five
            // different combinations of flags and color values.
            //
            // The steps that follow are outlined below:
            // 1. Draw a rectangle at the top of the screen with a black
            //    outline.
            // 2. Draw a rectangle filled with Red color (using flag 
            //    IDF_RECT_FILL), without a outline frame. This rectangle 
            //    will be placed below the rectangle drawn in the previous step.
            // 3. Draw a rectangle with flag IDF_RECT_INVERT set. This rectangle 
            //    is placed partly over the rectangle in the previous step.
            //    This results in the section overlapping the Red rectangle
            //    being inverted (to cyan color). The rest of the area in 
            //    this rectangle is inverted from the background color (white)
            //    to black.
            // 4. Draw a rectangle using both flags IDF_RECT_FRAME and 
            //    IDF_RECT_FILL. This results in a rectangle which is filled
            //    in Green and a frame drawn in Blue. This rectangle is
            //    placed below the rectangle drawn in the previous step.
            // 5. Draw a rectangle which all the flags enabled. But, when
            //    IDF_RECT_INVERT flag is used the other two flags are 
            //    ignored. Hence, a rectangle which is an invertion of 
            //    the background color results. This rectangle is placed below
            //    the rectangle below the one drawn in the previous step.

            // To see the effects of the IDISPLAY_DrawRect function a device
            // should be chosen with a color display.
            // 

            // Step 1.
            //
            // The first rectangle drawn on the screen is placed at the top
            // of the screen. It is lineHeight pixels high, half the width
            // of the screen.
            SETAEERECT (&rct, 5, lineHeight, di->cxScreen/2, lineHeight); 
            clrFrame = MAKE_RGB (0, 0, 0);             // Black color
            clrFill =  MAKE_RGB (0xff, 0xff, 0xff) ;   // White color
            // Draw only the outline of the rectangle (rectangle frame).
            dwFlags = IDF_RECT_FRAME;

            // IDISPLAY_DrawRect: draws the given rectangle using the specified color 
            // and flags. After calling this function, IDISPLAY_Update() must be 
            // called to update the screen. If there is a sequence of drawing 
            // operations being performed, it is sufficient to IDISPLAY_Update() once 
            // after all the drawing is done.
            IDISPLAY_DrawRect (pIDisplay, &rct, clrFrame, clrFill, dwFlags);

            // Step 2.
            //
            // The next set of initializations and IDISPLAY_DrawRect results in 
            // a rectangle of height lineHeight and a width of half the screen 
            // width. Only IDF_RECT_FILL flag is enabled, hence, a blue color frame
            // will not be drawn even though clrFrame value is initialized to blue. 
            // The resulting rectangle is a rectangle filled with red color.
            // Note: the output on the screen only shows half of the rectangle
            //       drawn below. This is because the subsequent IDISPLAY_DrawRect
            //       function call inverts half of the rectangle in color.
            SETAEERECT (&rct, 5, lineHeight*2 + 2, di->cxScreen/2, lineHeight); 
            clrFrame = MAKE_RGB (0, 0, 0xff);         // Blue color
            clrFill =  MAKE_RGB (0xff, 0, 0);         // Red color
            // Fill the rectangle with the color specified in the clfFrame parameter
            dwFlags = IDF_RECT_FILL;

            // Draw rectangle with values initialized above
            IDISPLAY_DrawRect (pIDisplay, &rct, clrFrame, clrFill, dwFlags);

            // Step 3.
            //
            // Following initialization and the IDISPLAY_DrawRect function results
            // in the colors of the rectangle area specified by rct being inverted. 
            // The rct rectangle specifies an area that covers half of the red rectangle 
            // drawn in the above IDISPLAY_DrawRect and area to the right of the above 
            // drawn rectangle.
            // Note: when IDF_RECT_INVERT flag is used the clrFrame and clrFill 
            //       parameters are ignored. Hence, the clrFrame and clrFill 
            //       initialization below has no efect.
            SETAEERECT (&rct, di->cxScreen/4, lineHeight*2 + 2, di->cxScreen/2, lineHeight); 
            clrFrame = MAKE_RGB (0, 0xff, 0);         // Green color
            clrFill =  MAKE_RGB (0xff, 0, 0);         // Red color
            dwFlags = IDF_RECT_INVERT;

            // Draw rectangle with values initialized above
            IDISPLAY_DrawRect (pIDisplay, &rct, clrFrame, clrFill, dwFlags);

            // Step 4.
            //
            // Following initializations results in a rectangle of height lineHeight
            // pixels and half the screen width. This rectangle is drawn below the
            // rectangle drawn in the above IDISPLAY_DrawRect operation.
            // The flags initialized are the IDF_RECT_FRAME and IDF_RECT_FILL. Hence,
            // the rectangle will be filled with the color specified in clrFill 
            // parameter (Green color) and will have a frame in color specified 
            // by clrFrame (Blue color).
            SETAEERECT (&rct, 5, lineHeight*3 + 6, di->cxScreen/2, lineHeight); 
            clrFrame = MAKE_RGB (0, 0, 0xff);   // Blue color
            clrFill =  MAKE_RGB (0, 0xff, 0);   // Green color
            dwFlags = IDF_RECT_FRAME | IDF_RECT_FILL;

            // Draw rectangle with values initialized above
            IDISPLAY_DrawRect (pIDisplay, &rct, clrFrame, clrFill, dwFlags);

            // Step 5.
            //
            // And finally all three flags are used when drawing the rectangle.
            // Here because of IDF_RECT_INVERT the other flags are ignored resulting
            // in the inverting of background color in the specified rectangle.
            SETAEERECT (&rct, 5, lineHeight*4 + 8, di->cxScreen/2, lineHeight); 
            clrFrame = MAKE_RGB (0, 0xff, 0xff);      // Cyan color
            clrFill =  MAKE_RGB (0xff, 0xff, 0);      // Yellow color
            dwFlags = IDF_RECT_FRAME | IDF_RECT_FILL | IDF_RECT_INVERT;

            // Draw rectangle with values initialized above
            IDISPLAY_DrawRect (pIDisplay, &rct, clrFrame, clrFill, dwFlags);
         }
         break;
      case USAGE_BITBLT:
         {
            // 7. Usage: IDISPLAY_BitBlt
            AEEBmp pbmSource;
            int xDest = 30;
            int yDest = 40;
            int cxDest = di->cxScreen;
            int cyDest = di->cyScreen;
            int xSrc = 0;
            int ySrc = 0;
            AEERasterOp dwRopCode = AEE_RO_MASK;
            AEEImageInfo imageInfo;
            void *pBmp = NULL, *pDataBytes = NULL;
            boolean bVal = TRUE;

            // Load bitmap resource from the resource file.
            pbmSource = ISHELL_LoadResData (pIShell, APP_RES_FILE, 
                                     IDB_BITMAP, RESTYPE_IMAGE); // please do not use RESTYPE_BINARY

			//There is a header that needs to be removed before passing the data to convertbmp
			//the first byte specifies the size of the header and hence offset  we need to skip.
            pDataBytes = (byte *)pbmSource + *((byte *)pbmSource);


            pBmp = CONVERTBMP (pDataBytes, &imageInfo, &bVal);

            // IDISPLAY_BitBlt: performs a bit-block transfer of the data 
            // corresponding to a rectangle of pixels from the specified source 
            // bitmap into the given Display area. After calling this function, 
            // IDISPLAY_Update() must be called to update the screen. If there is 
            // a sequence of drawing operations being performed, it is sufficient 
            // to IDISPLAY_Update() once after all the drawing is done.
            IDISPLAY_BitBlt (pIDisplay, xDest, yDest, cxDest, cyDest, pBmp,
               xSrc, ySrc, dwRopCode);

            IDISPLAY_Update (pIDisplay);

            ISHELL_FreeResData (pIShell, pbmSource);
			if(bVal)//free only if realloc was done
				SYSFREE ( pBmp);
         }
         break;
      case USAGE_BACKLIGHT:
         {
            // 8. Usage: IDISPLAY_Backlight
            AECHAR szBuf [20] = {0};
  
            // Set the flag that stores the backlight on/off value.
            // If the backlight was on then off it. If the backlight
            // was off then on it.
            if (pMe->bOn == FALSE)
               pMe->bOn = TRUE;
            else
               pMe->bOn = FALSE;

            // IDISPLAY_Backlight: turns the backlight of the phone on or off 
            // depending on the parameter bOn. The behavior of the backlight 
            // is completely dependent on the specific handset. When this 
            // function is used to turn the baclight on, the OEM of the handset 
            // (i.e. handset manufacturer) decides how long it remains on 
            // before it is turned off again. Similarly, when this function is 
            // used to turn off the backlight, the OEM of the handset decides 
            // how long it remains off before it is turned on again.
            IDISPLAY_Backlight(pIDisplay, pMe->bOn);

            if (pMe->bOn)
            {
               STR_TO_WSTR ("Backlight ON", szBuf, sizeof (szBuf));
               IDISPLAY_DrawText(pIDisplay, AEE_FONT_BOLD, szBuf, -1, 
                       0, 0, &rect, IDF_ALIGN_MIDDLE | IDF_ALIGN_CENTER);
            }
            else
            {
               STR_TO_WSTR ("Backlight OFF", szBuf, sizeof (szBuf));
               IDISPLAY_DrawText(pIDisplay, AEE_FONT_BOLD, szBuf, -1, 
                       0, 0, &rect, IDF_ALIGN_MIDDLE | IDF_ALIGN_CENTER);
            }
         }
         break;
      case USAGE_SETCOLOR:
         {
            // 8. Usage: IDISPLAY_SetColor
            AECHAR szBuf [20] = {0};
            RGBVAL prevColor;
				int nTextWidth;
				AEERect rc;

            // Following section gives example usages of the IDISPLAY_SetColor
            // function. This function is invoked five times with five
            // different combinations of flags and color values.
            //
            // The steps that follow are outlined below:
            // 1. Invoke IDISPLAY_SetColor function to set the text color.
            //    This is done by using CLR_USER_TEXT as the second parameter
            //    a RGB color value in the third parameter field.
            // 2. Display a text string on the screen. This text string
            //    is drawn in Red. Update the display for the text to be flushed 
            //    to the screen.
            // 3. Invoke IDISPLAY_SetColor again to change the text color
            //    to Blue. 
            // 4. Display a text string on the screen. This string will
            //    be displayed in Blue color.
            // 5. Invoke IDISPLAY_SetColor again with second parameter 
            //    value of CLR_USER_BACKGROUND and the color of Yellow.
            //    This will in effect change the background color to 
            //    Yellow. This function can be invoked again to change
            //    the background color to white.
            // 6. Invoke IDISPLAY_SetColor with CLR_USER_LINE and a 
            //    color value of Red. This will in cause the frame lines
            //    are drawn to be in Red.
            // 7. Display a text string on the screen and draw a frame 
            //    around it. the frame will be in Red color.
            // 8. Invoke IDISPLAY_SetColor with the CLR_USER_TEXT to be 
            //    changed to system-title color by using CLR_SYS_TITLE_TEXT
            //    in the second parameter field.
            // 9. Draw a text string on the screen which will be in the
            //    color of the system-title. 

            // IDISPLAY_SetColor: sets the color of the specified user Item. 
            // User items are those listed in the top of the AEEClrItem 
            // enumeration. These items have a prefix CLR_USER. The  color 
            // of the system-items (i.e. the items in the bottom of the 
            // AEEClrItem enumeration which have a prefix CLR_SYS) cannot 
            // be changed. The color of an item can either be set to a 
            // specific RGB value or it can be set to the color of another 
            // item in the AEEClrItem enumeration.

            // Step 1.
            //
            // Set the text color to Red.
            prevColor = IDISPLAY_SetColor(pIDisplay,CLR_USER_TEXT,MAKE_RGB(0xff,0,0));

            // Step 2.
            //
            // The string printed below will be in Red color.
            STR_TO_WSTR ("Red Color Text", szBuf, sizeof (szBuf));
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_BOLD, szBuf, -1, 
                 5, lineHeight, 0, IDF_ALIGN_CENTER);

            IDISPLAY_Update (pIDisplay);

            // Step 3.
            //
            // Set the text color to Blue.
            IDISPLAY_SetColor(pIDisplay,CLR_USER_TEXT,MAKE_RGB(0x0,0x0,0xff));

            // Step 4.
            //
            // The string printed below will be in Blue color.
            STR_TO_WSTR ("Blue Color Text", szBuf, sizeof (szBuf));
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_BOLD, szBuf, -1, 
                 5, lineHeight*2, 0, IDF_ALIGN_CENTER); 

            // Step 5.
            //
            // Change the background color to Yellow.
            // By invoking this function again the uuser can change the 
            // background color back to white.
            if (pMe->bgColor == FALSE)
            {
               IDISPLAY_SetColor(pIDisplay,CLR_USER_BACKGROUND,MAKE_RGB(0xff,0xff,0));
               pMe->bgColor = TRUE;
            }
            else
            {
               // Reset the color background color back to white
               IDISPLAY_SetColor(pIDisplay,CLR_USER_BACKGROUND,RGB_WHITE);
               pMe->bgColor = FALSE;
            }

            // Step 6.
            //
            // Set the line color to Red. Hence. the frame will be drawn in 
				// Red color.
            IDISPLAY_SetColor(pIDisplay,CLR_USER_LINE,MAKE_RGB(0xff,0,0));

            // Step 7.
            //
            // Draw a text string on the screen. This string will be drawn
            // in blue color.
            STR_TO_WSTR ("Framed Text", szBuf, sizeof (szBuf));
				nTextWidth = IDISPLAY_MeasureText(pIDisplay, AEE_FONT_BOLD, szBuf);
				SETAEERECT(&rc, (pMe->m_dInfo.cxScreen - nTextWidth)/2, lineHeight*3, nTextWidth, lineHeight);
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_BOLD, szBuf, -1, 
                 5, lineHeight*3, 0, IDF_ALIGN_CENTER);
            // Draw a frame around text. This frame will be drawn
            // in red color.
				IDISPLAY_FrameRect(pIDisplay, &rc);

            // Step 8.
            // 
            // Replace the text color with the color of system-title. This is
            // done using by invoking SetColor function with CLR_USER_TEXT
            // and CLR_SYS_TITLE_TEXT in the second and third parameter fields
            // respectively.
            IDISPLAY_SetColor(pIDisplay,CLR_USER_TEXT,CLR_SYS_TITLE_TEXT);

            // This string will be printed in the system-color.
            STR_TO_WSTR ("Sys Title color Text", szBuf, sizeof (szBuf));
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_BOLD, szBuf, -1, 
                 5, lineHeight*4, 0, IDF_ALIGN_CENTER);
            
            IDISPLAY_Update (pIDisplay);
         }
         break;
      case USAGE_UPDATE:
         {
            // 9. Usage: IDISPLAY_Update
            AECHAR szBuf[] = {'T','e','x','t',' ','S','t','r','i','n','g','\0'};

            // The following section draws a text string on the screen
            // and updates it. Hence, the output will be flushed to the
            // screen.
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, szBuf, -1, 
                              di->cxScreen/4, lineHeight, 0, 0);
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, szBuf, -1, 
                              di->cxScreen/4, lineHeight*2, 0, 0);
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, szBuf, -1, 
                              di->cxScreen/4, lineHeight*3, 0, 0);
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, szBuf, -1, 
                              di->cxScreen/4, lineHeight*4, 0, 0);
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, szBuf, -1, 
                              di->cxScreen/4, lineHeight*5, 0, 0);

            // IDISPLAY_Update: updates the screen. The update message is posted 
            // onto the queue of the UI task, thereby allowing all the drawings 
            // to be done before updating the screen.
            IDISPLAY_Update (pIDisplay);
         }
         break;
      case USAGE_GETSYMBOL:
         {
            // 10. Usage: IDISPLAY_GetSymbol
            AECHAR aeChar[25];

            // The following section retrieve the AECHAR value for all the
            // symbols defined in AEE and prints them on the screen six characters
            // at a time on different lines.
            //
            // IDISPLAY_GetSymbol: returns the AECHAR value corresponding to 
            // the specified symbol value.
            aeChar[0] = IDISPLAY_GetSymbol (pIDisplay, AEE_SYM_ELLIPSES, AEE_FONT_NORMAL);
            aeChar[1] = IDISPLAY_GetSymbol (pIDisplay, AEE_SYM_AM, AEE_FONT_NORMAL);
            aeChar[2] = IDISPLAY_GetSymbol (pIDisplay, AEE_SYM_PM, AEE_FONT_NORMAL);
            aeChar[3] = IDISPLAY_GetSymbol (pIDisplay, AEE_SYM_KEY_LEFT, AEE_FONT_NORMAL);
            aeChar[4] = IDISPLAY_GetSymbol (pIDisplay, AEE_SYM_KEY_RIGHT, AEE_FONT_NORMAL);
            aeChar[5] = IDISPLAY_GetSymbol (pIDisplay, AEE_SYM_KEY_CLR, AEE_FONT_NORMAL);
            aeChar[6] = IDISPLAY_GetSymbol (pIDisplay, AEE_SYM_KEY_1, AEE_FONT_NORMAL);
            aeChar[7] = IDISPLAY_GetSymbol (pIDisplay, AEE_SYM_KEY_2, AEE_FONT_NORMAL);
            aeChar[8] = IDISPLAY_GetSymbol (pIDisplay, AEE_SYM_KEY_3, AEE_FONT_NORMAL);
            aeChar[9] = IDISPLAY_GetSymbol (pIDisplay, AEE_SYM_KEY_4, AEE_FONT_NORMAL);
            aeChar[10] = IDISPLAY_GetSymbol (pIDisplay, AEE_SYM_KEY_5, AEE_FONT_NORMAL);
            aeChar[11] = IDISPLAY_GetSymbol (pIDisplay, AEE_SYM_KEY_6, AEE_FONT_NORMAL);
            aeChar[12] = IDISPLAY_GetSymbol (pIDisplay, AEE_SYM_KEY_7, AEE_FONT_NORMAL);
            aeChar[13] = IDISPLAY_GetSymbol (pIDisplay, AEE_SYM_KEY_5, AEE_FONT_NORMAL);
            aeChar[14] = IDISPLAY_GetSymbol (pIDisplay, AEE_SYM_KEY_6, AEE_FONT_NORMAL);
            aeChar[15] = IDISPLAY_GetSymbol (pIDisplay, AEE_SYM_KEY_7, AEE_FONT_NORMAL);
            aeChar[16] = IDISPLAY_GetSymbol (pIDisplay, AEE_SYM_KEY_8, AEE_FONT_NORMAL);
            aeChar[17] = IDISPLAY_GetSymbol (pIDisplay, AEE_SYM_KEY_9, AEE_FONT_NORMAL);
            aeChar[18] = IDISPLAY_GetSymbol (pIDisplay, AEE_SYM_KEY_STAR, AEE_FONT_NORMAL);
            aeChar[19] = IDISPLAY_GetSymbol (pIDisplay, AEE_SYM_KEY_POUND, AEE_FONT_NORMAL);
            aeChar[20] = IDISPLAY_GetSymbol (pIDisplay, AEE_SYM_KEY_SEND, AEE_FONT_NORMAL);
            aeChar[21] = IDISPLAY_GetSymbol (pIDisplay, AEE_SYM_KEY_END, AEE_FONT_NORMAL);
            aeChar[22] = IDISPLAY_GetSymbol (pIDisplay, AEE_SYM_KEY_SELECT, AEE_FONT_NORMAL);
            aeChar[24] = '\0';

            // Display 6 characters on the top section of the screen. 
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, aeChar, 6, 
                              0, 2*lineHeight, 0, 0);
            // Display the next 6 characters on the next line
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, &aeChar[6], 6, 
                              0, 3*lineHeight, 0, 0);
            // Display the next 6 characters on the next line
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, &aeChar[12], 6, 
                              0, 4*lineHeight, 0, 0);
            // Display the last 4 character on the next line.
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, &aeChar[18], 4, 
                              0, 5*lineHeight, 0, 0);
         }
         break;
      case USAGE_DRAWFRAME:
         {
            // 11. Usage: IDISPLAY_DrawFrame
            AEERect rct;
            AEEFrameType fType;
            RGBVAL clrFill;

            // Following section gives example usages of the IDISPLAY_DrawFrame
            // function. This function is invoked ten times with ten
            // different combinations of flags and color values.
            //
            // The steps that follow are outlined below:
            // 1. Invoke IDISPLAY_DrawFrame with a frame type of AEE_FT_EMPTY
            //    and a fill color of Blue. This results in a blue colored 
            //    rectangle on the top left corner of the screen.
            // 2. Invoke IDISPLAY_DrawFrame with a frame type of AEE_FT_3D_EMPTY
            //    and a fill color of Green. This results in a green colored 
            //    rectangle on the center of the top of the screen.
            // 3. Invoke IDISPLAY_DrawFrame with a frame type of AEE_FT_RAISED
            //    and a fill color of Cyan. This results in a cyan colored 
            //    rectangle on the top right corner of the screen.
            // 4. Invoke IDISPLAY_DrawFrame with a frame type of AEE_FT_LOWERED
            //    and a fill color of Red. This results in a Red colored 
            //    rectangle on the top left corner of the screen, below the
            //    rectangle drawn in step 1.
            // 5. Invoke IDISPLAY_DrawFrame with a frame type of AEE_FT_BOX
            //    and a fill color of Purple. This results in a Purple colored 
            //    rectangle below the rectangle drawn in step 2.
            // 6. Invoke IDISPLAY_DrawFrame with a frame type of AEE_FT_INDENT
            //    and a fill color of Yellow. This results in a Yellow colored 
            //    rectangle below the rectangle drawn in step 3.
            // 7. Invoke IDISPLAY_DrawFrame with a frame type of 
            //    AEE_FT_TAB_BOTTOM_SEL and a fill color of White. This results 
            //    in a White colored rectangle below the rectangle drawn in 
            //    step 4.
            // 8. Invoke IDISPLAY_DrawFrame with a frame type of AEE_FT_TAB_BOTTOM
            //    and a fill color of Yellow. This results in a Yellow colored 
            //    rectangle below the rectangle drawn in step 5.
            // 9. Invoke IDISPLAY_DrawFrame with a frame type of AEE_FT_INDENT
            //    and a fill color of Yellow. This results in a Yellow colored 
            //    rectangle below the rectangle drawn in step 6.
            // 10. Invoke IDISPLAY_DrawFrame with a frame type of AEE_FT_INDENT
            //    and a fill color of Yellow. This results in a Yellow colored 
            //    rectangle below the rectangle drawn in step 8.

            // Step 1: Draw a blue empty frame
            SETAEERECT (&rct, 0, lineHeight, di->cxScreen/4, lineHeight);
            fType = AEE_FT_EMPTY;
            clrFill = MAKE_RGB (0, 0, 0xff);
            IDISPLAY_DrawFrame (pIDisplay, &rct, fType, clrFill);

            // Step 2: Draw a green empty 3D frame
            SETAEERECT (&rct, di->cxScreen/3, lineHeight, di->cxScreen/4, lineHeight);
            fType = AEE_FT_3D_EMPTY;
            clrFill = MAKE_RGB (0, 0xff, 0);
            IDISPLAY_DrawFrame (pIDisplay, &rct, fType, clrFill);

            // Step 3: Draw a cyan raised frame
            SETAEERECT (&rct, di->cxScreen*2/3, lineHeight, di->cxScreen/4, lineHeight);
            fType = AEE_FT_RAISED;
            clrFill = MAKE_RGB (0, 0xff, 0xff);
            IDISPLAY_DrawFrame (pIDisplay, &rct, fType, clrFill);

            // Step 4: Draw a red lowered frame
            SETAEERECT (&rct, 0, lineHeight*3, di->cxScreen/4, lineHeight);
            fType = AEE_FT_LOWERED;
            clrFill = MAKE_RGB (0xff, 0, 0);
            IDISPLAY_DrawFrame (pIDisplay, &rct, fType, clrFill);

            // Step 5: Draw a purple/pink box frame
            SETAEERECT (&rct, di->cxScreen/3, lineHeight*3, di->cxScreen/4, lineHeight);
            fType = AEE_FT_BOX;
            clrFill = MAKE_RGB (0xff, 0, 0xff);
            IDISPLAY_DrawFrame (pIDisplay, &rct, fType, clrFill);

            // Step 6: Draw a yellow indented frame
            SETAEERECT (&rct, di->cxScreen*2/3, lineHeight*3, di->cxScreen/4, lineHeight);
            fType = AEE_FT_INDENT;
            clrFill = MAKE_RGB (0xff, 0xff, 0);
            IDISPLAY_DrawFrame (pIDisplay, &rct, fType, clrFill);

            // Step 7: Draw a white tab bottom selected frame
            SETAEERECT (&rct, 0, lineHeight*5, di->cxScreen/4, lineHeight);
            fType = AEE_FT_TAB_BOTTOM_SEL;
            clrFill = MAKE_RGB (0xff, 0xff, 0xff);
            IDISPLAY_DrawFrame (pIDisplay, &rct, fType, clrFill);

            // Step 8: Draw a black tab bottom frame
            SETAEERECT (&rct, di->cxScreen/3, lineHeight*5, di->cxScreen/4, lineHeight);
            fType = AEE_FT_TAB_BOTTOM;
            clrFill = MAKE_RGB (0, 0, 0);
            IDISPLAY_DrawFrame (pIDisplay, &rct, fType, clrFill);

            // Step 9: Draw a blue top selected frame
            SETAEERECT (&rct, di->cxScreen*2/3, lineHeight*5, di->cxScreen/4, lineHeight);
            fType = AEE_FT_TAB_TOP_SEL;
            clrFill = MAKE_RGB (0, 0, 0xff);
            IDISPLAY_DrawFrame (pIDisplay, &rct, fType, clrFill);

            // Step 10: Draw a blue tab top frame
            SETAEERECT (&rct, di->cxScreen/3, lineHeight*7, di->cxScreen/4, lineHeight);
            fType = AEE_FT_TAB_TOP;
            clrFill = MAKE_RGB (0, 0, 0xff);
            IDISPLAY_DrawFrame (pIDisplay, &rct, fType, clrFill);
         }
         break;
      case USAGE_FRAMEBUTTON:
         {
            // 12. Usage: IDISPLAY_FrameButton
            AEERect rct;
            AEEFrameType fType;
            RGBVAL clrFill;

            // Following section illustrates example usages of the 
            // IDISPLAY_FrameButton function. This function is invoked 
            // two times with a lowered and raised frame button.
            //
            // The steps that follow are outlined below:
            // 1. Invoke IDISPLAY_FrameButton pressed 3D frame button
            // 2. Draw a raised 3D frame button. This button will be placed
            //    right below the lowered frame button drawn in the 
            //    previous step.

            // Initialize rectangle, frame type, and fill color
            SETAEERECT (&rct, di->cxScreen/4, lineHeight*2, di->cxScreen/3, lineHeight*3/2);
            fType = AEE_FT_EMPTY;
            clrFill = MAKE_RGB (0, 0, 0xff);

            // Step 1.
            //
            // IDISPLAY_FrameButton: draws a 3D framed button based upon the 
            // color resolution of the system. The function will draw a lowered 
            // button when bPressed is TRUE, and a raised button when the bPressed is FALSE.
            // Draw a pressed button
            IDISPLAY_FrameButton (pIDisplay, &rct, TRUE, clrFill);

            // Step 2.
            //
            // Initialize rectangle, frame type, and fill color
            SETAEERECT (&rct, di->cxScreen/4, lineHeight*5, di->cxScreen/3, lineHeight*3/2);
            fType = AEE_FT_EMPTY;
            clrFill = MAKE_RGB (0, 0, 0xff);
            // Draw a raised button
            IDISPLAY_FrameButton (pIDisplay, &rct, FALSE, clrFill);
         }
         break;
      case USAGE_FILLRECT:
         {
            // 13. Usage: IDISPLAY_FillRect
            AEERect rct;
            RGBVAL clrFill;

            // Following section illustrates example usages of the 
            // IDISPLAY_FillRect function. This function is invoked 
            // three times with three different colors.
            //
            // The steps of the following section is outlined below:
            // 1. Invoke IDISPLAY_FillRect to draw a Blue rectangle
            // 2. Invoke IDISPLAY_FillRect to draw a Green rectangle
            // 3. Invoke IDISPLAY_FillRect to draw a Red rectangle

            // Step 1: Draw Blue rectangle
            SETAEERECT (&rct, di->cxScreen/4, lineHeight, di->cxScreen/2, lineHeight);
            clrFill = MAKE_RGB (0, 0, 0xff);   // Blue color
            IDISPLAY_FillRect (pIDisplay, &rct, clrFill);

            // Step 2: Draw Green rectangle
            SETAEERECT (&rct, di->cxScreen/4, lineHeight*3, di->cxScreen/2, lineHeight);
            clrFill = MAKE_RGB (0, 0xff, 0);   // Green color
            IDISPLAY_FillRect (pIDisplay, &rct, clrFill);

            // Step 3: Draw Red rectangle
            SETAEERECT (&rct, di->cxScreen/4, lineHeight*5, di->cxScreen/2, lineHeight);
            clrFill = MAKE_RGB (0xff, 0, 0);   // Red color
            IDISPLAY_FillRect (pIDisplay, &rct, clrFill);
         }
         break;
      case USAGE_CLEARSCREEN:
         {
            // 14. Usage: IDISPLAY_ClearScreen
            AEERect rct;
            RGBVAL clrFill;
            AECHAR szBuf[] = {'T','e','x','t',' ','S','t','r','i','n','g','\0'};

            // Following section illustrates example usages of the 
            // IDISPLAY_ClearScreen function.
            //
            // The steps that follow are outlined below:
            // 1. Draw a text string on the screen.
            // 2. Draw a rectangle using IDISPLAY_FillRect.
            // 3. Update screen so that the output is flushed to screen.
            // 4. Invoke IDISPLAY_ClearScreen to clear the whole screen.
            //    This results in all the drawn text being cleared
            //    from screen. The user Will see text at the to and
            //    bottom of the screen corresponding to text drawn from the
            //    DisplayEvent function at the end of this function.


            // Step 1: Draw  text string on the screen.
            IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, szBuf, -1, 
                              di->cxScreen/4, lineHeight*2, 0, 0);

            // Step 2: Drawrectangle on the screen
            SETAEERECT (&rct, di->cxScreen/4, lineHeight*4, di->cxScreen/2, lineHeight);
            clrFill = MAKE_RGB (0, 0, 0xff);   // Blue color
            IDISPLAY_FillRect (pIDisplay, &rct, clrFill);

            // Step 3: Update the display
            IDISPLAY_Update (pIDisplay);

            // Step 4: Clear screen, in effect clearing the text, and rectangle
            //         drawn in the above steps.
            IDISPLAY_ClearScreen(pIDisplay);
         }
         break;
      case USAGE_FRAMERECT:
         {
            // 15. Usage: IDISPLAY_FrameRect
            AEERect rct;
   
            // Following section illustrates example usages of the 
            // IDISPLAY_FrameRect function.
            //
            // The steps that follow are outlined below:
            // 1. Invoke IDISPLAY_FrameRect to draw a rectangle 
            //    in currrent User-Frame color.
            // 2. Set User-Frame color to Red.
            // 3. Invoke IDISPLAY_FrameRect agian to draw another
            //    rectangle. This rectangle will be drawn in Red
            //    color and will be placed below the rectangle 
            //    drawn in step 1.

            // Initialize a rectangle
            SETAEERECT (&rct, di->cxScreen/4, lineHeight*2, di->cxScreen/2, lineHeight);

            // Step 1: 
            //
            // IDISPLAY_FrameRect: draws the borders of a rectangle. The color 
            // used for drawing the borders is the current color assigned to 
            // the item CLR_USER_FRAME.       
            IDISPLAY_FrameRect(pIDisplay, &rct);

            // Step 2: Change User-Frame color ro Red
            IDISPLAY_SetColor(pIDisplay,CLR_USER_FRAME,MAKE_RGB(0xff,0,0x0));

            SETAEERECT (&rct, di->cxScreen/4, lineHeight*4, di->cxScreen/2, lineHeight);

            // Step 3: Draw another rectangle which will be in the new 
            //         User-Frame color (Red).
            IDISPLAY_FrameRect(pIDisplay, &rct);
         }
         break;
      case USAGE_FRAMESOLIDRECT:
         {
            // 16. Usage: IDISPLAY_FrameSolidRect
            AEERect rct;
   
            // Following section illustrates example usages of the 
            // IDISPLAY_FrameSolidRect function.
            //
            // The steps that follow are outlined below:
            // 1. Invoke IDISPLAY_FrameSolidRect to draw a rectangle 
            //    in currrent User-Frame color.
            // 2. Set User-Frame color to Blue.
            // 3. Invoke IDISPLAY_FrameSolidRect agian to draw another
            //    rectangle. This rectangle will be drawn in Blue
            //    color and will be placed below the rectangle 
            //    drawn in step 1.
            // 4. Set User-Frame color to Green.
            // 5. Invoke IDISPLAY_FrameSolidRect agian to draw another
            //    rectangle. This rectangle will be drawn in Green
            //    color and will be placed below the rectangle 
            //    drawn in step 3.

            // Initialize rectangle
            SETAEERECT (&rct, di->cxScreen/4, lineHeight*2, di->cxScreen/2, lineHeight);

            // Step 1.
            //
            // IDISPLAY_FrameSolidRect: draws the borders of a rectangle and 
            // fills it with a color. The color used for drawing the borders 
            // is the current color assigned to the item CLR_USER_FRAME. The 
            // color used for filling the rectangle is the current color 
            // assigned to the item CLR_USER_BACKGROUND
            IDISPLAY_FrameSolidRect (pIDisplay, &rct);

            // Step 2: Set User-Frame color to Blue.
            IDISPLAY_SetColor(pIDisplay,CLR_USER_FRAME,MAKE_RGB(0,0,0xff));
            rct.y = lineHeight*4;

            // Step 3: Invoke IDISPLAY_FrameSolidRect to draw a rectangle in the
            //         new User-Frame color (Blue).
            IDISPLAY_FrameSolidRect (pIDisplay, &rct);

            // Step 4: Set User-Frame color to Green.
            IDISPLAY_SetColor(pIDisplay,CLR_USER_FRAME,MAKE_RGB(0,0xff,0));
            rct.y = lineHeight*6;

            // Step 5: Invoke IDISPLAY_FrameSolidRect to draw a rectangle in the
            //         new User-Frame color (Green).
            IDISPLAY_FrameSolidRect (pIDisplay, &rct);
         }
         break;
      case USAGE_INVERTRECT:
         {
            // 17. Usage: IDISPLAY_InvertRect
            AEERect rct;
   
            // Following section illustrates example usages of the 
            // IDISPLAY_InvertRect function. Here, we draw a rectangle
            // on the screen which will be inverted in color to the 
            // previous color that was on the screen within the specified
            // rectangle.
            //
            // The steps that follow are outlined below:
            // 1. Invoke IDISPLAY_DrawFrame to draw a tab-bottom-selected
            //    frame in cyan color.
            // 2. Initialize a rectangle which overlaps the rectangle
            //    drawn in the previous step.
            // 3. Invoke IDISPLAY_InvertRect on the rectangle initialized 
            //    in the previous step.

            // Initialize a rectangle
            SETAEERECT (&rct, di->cxScreen/4, di->cyScreen/4, di->cxScreen/4, di->cyScreen/5);

            // Step 1: Draw frame in cyan color
            IDISPLAY_DrawFrame (pIDisplay, &rct, AEE_FT_TAB_BOTTOM_SEL, 
               MAKE_RGB (0, 0xff, 0xff));

            // Step 2: Initialize rectangle
            SETAEERECT (&rct, di->cxScreen/4, di->cyScreen/4, di->cxScreen/2, 
               di->cyScreen/3);

            // Step 3:
            //
            // IDISPLAY_InvertRect: inverts the color in the given Rectangle. 
            // It inverts the color and re-fills the rectangle with that 
            // inverted color pattern.
            IDISPLAY_InvertRect (pIDisplay, &rct);
         }
         break;
      case USAGE_ERASERECT:
         {
            // 18. Usage: IDISPLAY_EraseRect
            AEERect rct;

            // Following section illustrates example usages of the 
            // IDISPLAY_EraseRect function. 
            //
            // The steps that follow are outlined below:
            // 1. Invoke IDISPLAY_DrawRect to draw a rectangle 
            //    filled in Red color and framed in Blue
            // 2. Initialize a rectangle that covers an area within
            //    rectangle that was drawn in the previous step.
            // 3. Erase the screen in the region of the rectangle 
            //    specified in the previous step.

            // Step 1: Draw red color filled rectangle.
            SETAEERECT (&rct, di->cxScreen/4, lineHeight*2, di->cxScreen/2, di->cyScreen/2); 
            IDISPLAY_DrawRect (pIDisplay, &rct, MAKE_RGB (0, 0, 0xff), 
               MAKE_RGB (0xff, 0, 0), IDF_RECT_FILL | IDF_RECT_FRAME);

            // Step 2: Set up the rectangle to erase
            SETAEERECT (&rct, di->cxScreen/3, lineHeight*3, di->cxScreen/4, di->cyScreen/4); 

            // Step 3:
            //
            // IDISPLAY_EraseRect: This function fills the given rectangle 
            // with the default background color
            IDISPLAY_EraseRect (pIDisplay, &rct);
         }
         break;
      case USAGE_ERASERGN:
         {
            // 19. Usage: IDISPLAY_EraseRgn
            AEERect rct;

            // Following section illustrates example usages of the 
            // IDISPLAY_EraseRgn function. 
            //
            // The steps that follow are outlined below:
            // 1. Draw a rectangle in the middle of the screen in Red color.
            // 2. Erase a section within the rectangle drawn in the step above.

            // Step 1. Draw red colored rectangle
            SETAEERECT (&rct, di->cxScreen/4, lineHeight*2, di->cxScreen/2, di->cyScreen/2); 
            IDISPLAY_DrawRect (pIDisplay, &rct, MAKE_RGB (0, 0, 0xff), 
               MAKE_RGB (0xff, 0, 0), IDF_RECT_FILL);

            // Step 2:
            //
            // IDISPLAY_EraseRgn: This function fills the region enclosed by 
            // the given coordinates with the default background color
            IDISPLAY_EraseRgn (pIDisplay, di->cxScreen/2, lineHeight*3,
                               di->cxScreen/3, di->cyScreen/4);
         }
         break;
      case USAGE_DRAWHLINE:
         {
            // 20. Usage: IDISPLAY_DrawHLine
            int16 startX = 0;
            int16 startY = di->cyScreen/4;
            int16 len = di->cxScreen/2;

            // Following section illustrates example usages of the 
            // IDISPLAY_DrawHLine function. This function draws three
            // horizontal lines at different locations on the screen.

            // IDISPLAY_DrawHLine: draws a horizontal line of the given length, 
            // starting from the given point. The line shall be Black in color.
            IDISPLAY_DrawHLine(pIDisplay, startX, startY, len)

            // Draw another horizontal line
            startX = di->cxScreen/4;
            startY = di->cyScreen/2;
            len = di->cxScreen/2;

            // Draw line
            IDISPLAY_DrawHLine(pIDisplay, startX, startY, len)

            // Draw another horizontal line
            startX = di->cxScreen/2;
            startY = di->cyScreen*3/4;
            len = di->cxScreen/2;

            // Draw line
            IDISPLAY_DrawHLine(pIDisplay, startX, startY, len)
         }
         break;
      case USAGE_DRAWVLINE:
         {
            // 21. Usage: IDISPLAY_DrawVLine
            int16 startX = di->cxScreen/4;
            int16 startY = di->cyScreen/4;
            int16 len = di->cyScreen/2;

            // Following section illustrates example usages of the 
            // IDISPLAY_DrawVLine function. This function draws three
            // vertical lines at different locations on the screen.

            // IDISPLAY_DrawVLine: draws a vertical line of the given length, 
            // starting from the given point.
            IDISPLAY_DrawVLine(pIDisplay, startX, startY, len)

            // Draw another vertical line
            startX = di->cxScreen/2;
            startY = di->cyScreen/2;
            len = di->cyScreen/4;

            // Draw line
            IDISPLAY_DrawVLine(pIDisplay, startX, startY, len)

            // Draw another vertical line
            startX = di->cxScreen*3/4;
            startY = di->cyScreen/3;
            len = di->cyScreen/3;

            // Draw line
            IDISPLAY_DrawVLine(pIDisplay, startX, startY, len)
         }
         break;
      default:
         return;
   }
   
   // Display above event. 
   DisplayEvent (pMe, wParam);

   return;
}

/*===========================================================================

FUNCTION: DisplayEvent

DESCRIPTION
    This function is a display function which sets up the initial screen
    for the display output for a given interface function. This displays
    the function name at the top of the screen and displays the
    strings "Press Up/Down" and "keys to continue.." at the bottom of 
    the screen. In the case of functions that have no output it also 
    displays the string "No Output..." in the middile of the screen.

PROTOTYPE:
   void DisplayEvent (CIDisplayApp *pMe, uint16 wParam)

PARAMETERS:
   pMe:    [in]: Pointer to CIDisplayApp interface object
   wParam: [in]: API function type Id selected by user

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void DisplayEvent (CIDisplayApp *pMe, uint16 wParam)
{
   AECHAR szBuf[100];
   AEERect rc; 

   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
      return;

   SETAEERECT (&rc, 0, 0, pMe->m_dInfo.cxScreen, pMe->m_dInfo.cyScreen);

   if (pMe->a.m_pIDisplay)
   {
      switch (wParam)
      {
         case USAGE_BASIC_USAGE:
            STR_TO_WSTR("Basic Usage:", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETFONTMETRICS:
            STR_TO_WSTR("GetFontMetrics", szBuf, sizeof(szBuf));
            break;
         case USAGE_MEASURETEXTEX:
            STR_TO_WSTR("MeasureTextEx", szBuf, sizeof(szBuf));
            break;
         case USAGE_MEASURETEXT:
            STR_TO_WSTR("MeasureText", szBuf, sizeof(szBuf));
            break;
         case USAGE_DRAWTEXT:
            STR_TO_WSTR("DrawText", szBuf, sizeof(szBuf));
            break;
         case USAGE_DRAWRECT:
            STR_TO_WSTR("DrawRect", szBuf, sizeof(szBuf));
            break;
         case USAGE_BITBLT:
            STR_TO_WSTR("BitBlt", szBuf, sizeof(szBuf));
            break;
         case USAGE_UPDATEEX:
            STR_TO_WSTR("UpdateEx", szBuf, sizeof(szBuf));
            break;
         case USAGE_BACKLIGHT:
            STR_TO_WSTR("Backlight", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETCOLOR:
            STR_TO_WSTR("SetColor", szBuf, sizeof(szBuf));
            break;
         case USAGE_UPDATE:
            STR_TO_WSTR("Update", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETSYMBOL:
            STR_TO_WSTR("GetSymbol", szBuf, sizeof(szBuf));
            break;
         case USAGE_DRAWFRAME:
            STR_TO_WSTR("DrawFrame", szBuf, sizeof(szBuf));
            break;
         case USAGE_FRAMEBUTTON:
            STR_TO_WSTR("FrameButton", szBuf, sizeof(szBuf));
            break;
         case USAGE_FILLRECT:
            STR_TO_WSTR("FillRect", szBuf, sizeof(szBuf));
            break;
         case USAGE_CLEARSCREEN:
            STR_TO_WSTR("ClearScreen", szBuf, sizeof(szBuf));
            break;
         case USAGE_FRAMERECT:
            STR_TO_WSTR("FrameRect", szBuf, sizeof(szBuf));
            break;
         case USAGE_FRAMESOLIDRECT:
            STR_TO_WSTR("FrameSolidRect", szBuf, sizeof(szBuf));
            break;
         case USAGE_INVERTRECT:
            STR_TO_WSTR("InvertRect", szBuf, sizeof (szBuf));
            break;
         case USAGE_ERASERECT:
            STR_TO_WSTR("EraseRect", szBuf, sizeof(szBuf));
            break;
         case USAGE_ERASERGN:
            STR_TO_WSTR("EraseRgn", szBuf, sizeof(szBuf));
            break;
         case USAGE_DRAWHLINE:
            STR_TO_WSTR("DrawHLine", szBuf, sizeof(szBuf));
            break;
         case USAGE_DRAWVLINE:
            STR_TO_WSTR("DrawVLine", szBuf, sizeof(szBuf));
            break;
         default:
            STR_TO_WSTR("Unsupported fn", szBuf, sizeof(szBuf));
      }
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,5, 0, &rc,
		   IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);

      STR_TO_WSTR("Press Up/Down", szBuf, sizeof(szBuf));
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,5, rc.dy - 
         2*(pMe->m_nLineHeight) , &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
      STR_TO_WSTR("keys to continue..", szBuf, sizeof(szBuf));
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,5, rc.dy - 
         pMe->m_nLineHeight, &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
		IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);
   }
}

/*===========================================================================

FUNCTION: BuildMainMenu

DESCRIPTION
    This function builds the main menu when this applet is started.
    It displays a simple menu with menu items corresponding to 
    each of the interface functions.
    
PROTOTYPE:
   void BuildMainMenu(CIDisplayApp *pMe)

PARAMETERS:
   pMe: [in]: Pointer to AEEApplet struct

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void BuildMainMenu(CIDisplayApp *pMe)
{
	AEERect rc;
   AECHAR szBuf[100];

   if (pMe == NULL || pMe->m_pIMenu == NULL || pMe->a.m_pIShell == NULL)
      return;

	// Set Title
   STR_TO_WSTR("IDisplay Examples", szBuf, sizeof(szBuf));
	IMENUCTL_SetTitle(pMe->m_pIMenu, NULL, 0, szBuf);

   // Set display rectangle
   SETAEERECT (&rc, 0, 0, pMe->m_dInfo.cxScreen, pMe->m_dInfo.cyScreen);
	IMENUCTL_SetRect(pMe->m_pIMenu, &rc);  

	//Add invidual entries to the Menu

   // Add Basic Usage to menu
   STR_TO_WSTR("1. Basic Usage", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_BASIC_USAGE, szBuf, 0);

   // Add GteFontMetrics to menu
   STR_TO_WSTR("2. GetFontMetrics", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETFONTMETRICS, szBuf, 0);

   // Add MeasureTextEx to menu
   STR_TO_WSTR("3. MeasureTextEx", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_MEASURETEXTEX, szBuf, 0);

   // Add MeasureText to menu
   STR_TO_WSTR("4. MeasureText", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_MEASURETEXT, szBuf, 0);

   // Add DrawText to menu
   STR_TO_WSTR("5. DrawText", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DRAWTEXT, szBuf, 0);

   // Add DrawRect to menu
   STR_TO_WSTR("6. DrawRect", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DRAWRECT, szBuf, 0);

   // Add BitBlt to menu
   STR_TO_WSTR("7 BitBlt", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_BITBLT, szBuf, 0);

   // Add UpdateEx to menu
   STR_TO_WSTR("8. UpdateEx", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_UPDATEEX, szBuf, 0);

   // Add Backlight to menu
   STR_TO_WSTR("9. Backlight", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_BACKLIGHT, szBuf, 0);

	// Add SetColor to menu
   STR_TO_WSTR("10. SetColor", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETCOLOR, szBuf, 0);

	// Add Update to menu
   STR_TO_WSTR("11. Update", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_UPDATE, szBuf, 0);	

	// Add GetSymbol to menu
   STR_TO_WSTR("12. GetSymbol", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETSYMBOL, szBuf, 0);	

	// Add DrawFrame to menu
   STR_TO_WSTR("13. DrawFrame", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DRAWFRAME, szBuf, 0);	

	// Add FrameButton to menu
   STR_TO_WSTR("14. FrameButton", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_FRAMEBUTTON, szBuf, 0);	

	// Add FillRect to menu
   STR_TO_WSTR("15. FillRect", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_FILLRECT, szBuf, 0);	

	// Add ClearScreen to menu
   STR_TO_WSTR("16. ClearScreen", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_CLEARSCREEN, szBuf, 0);	

	// Add FrameRect to menu
   STR_TO_WSTR("17. FrameRect", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_FRAMERECT, szBuf, 0);	

	// Add FrameSolidRect to menu
   STR_TO_WSTR("18. FrameSolidrect", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_FRAMESOLIDRECT, szBuf, 0);	

	// Add InvertRect to menu
   STR_TO_WSTR("19. InvertRect", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_INVERTRECT, szBuf, 0);	

	// Add EraseRect to menu
   STR_TO_WSTR("20. EraseRect", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_ERASERECT, szBuf, 0);	

	// Add EraseRgn to menu
   STR_TO_WSTR("21. EraseRgn", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_ERASERGN, szBuf, 0);	

	// Add DrawHLine to menu
   STR_TO_WSTR("22. DrawHLine", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DRAWHLINE, szBuf, 0);	

	// Add DrawVLine to menu
   STR_TO_WSTR("23. DrawVLine", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DRAWVLINE, szBuf, 0);	

   // Activate menu
	IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);
}
