/*===========================================================================

FILE: ifileusage.c

SERVICES: Sample applet using AEE

DESCRIPTION
  This file contains usage examples of the IFileMgr and IFile interfaces. The IFileMgr
  interface functions include the creation and deletion of files and directories. 
  IFile interface functions consist of operations performed on files opened by the IFileMgr 
  interface.  The following functions are illustrated in this usage example:

      -IFILEMGR_EnumInit
      -IFILEMGR_EnumNext
	  -IFILEMGR_GetFreeSpace
	  -IFILEMGR_GetInfo
	  -IFILEMGR_GetLastError
	  -IFILEMGR_MkDir
	  -IFILEMGR_OpenFile
	  -IFILEMGR_Remove
	  -IFILEMGR_Rename
	  -IFILEMGR_RmDir
	  -IFILEMGR_Test
      -IFILE_GetInfo
	  -IFILE_Read
	  -IFILE_Release
	  -IFILE_Seek
	  -IFILE_Truncate
	  -IFILE_Write

PUBLIC CLASSES:  
   N/A


   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/

/*===============================================================================
                     INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
// Mandatory includes
#include "AEEAppGen.h"	// AEEApplet declaration
#include "AEEUsageAppIDs.h" // Class IDs of usage apps (including this one)
#include "AEE.h"			// Standard AEE Declarations
#include "AEEShell.h"	// AEE Shell Services
#include "AEEDisp.h"		// AEE Display Services

// Services used by app
#include "AEEFile.h"		//	AEEFile Services
#include "AEEMenu.h"		// Menu Services
#include "AEEStdLib.h"   // AEE stdlib services

/*===========================================================================

                      PUBLIC DATA DECLARATIONS

===========================================================================*/

/*-------------------------------------------------------------------
            Type Declarations
-------------------------------------------------------------------*/
// IFile app struct. This is the main struct for this applet. This will hold 
// all the data members that need to be remembered throughout the life of
// the applet. 
// THE FIRST DATA MEMBER OF THIS STRUCT MUST BE AN AEEApplet OBJECT. 
// This is critical requirement which is related to an applet being
// dynamically loaded. The app developers should add their data members
// following the AEEApplet data member.
typedef struct _CIFileApp {
   AEEApplet	      a;
   IMenuCtl *  m_pIMenu;
   IFileMgr * m_pIFileMgr;
   AEERect     m_rc;
   int m_cLineNum; // screen line to be written to next
   int m_cMaxLine; // max line number of screen on which output can be written
} CIFileApp;

/*-------------------------------------------------------------------
            Function Prototypes
-------------------------------------------------------------------*/
// Mandatory function:
// App Handle Event function
static boolean FileApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);

// App specific data alloc-init/free functions
static boolean IFile_InitAppData(IApplet* pMe);
static void IFile_FreeAppData(IApplet* pMe);

static void BuildMainMenu(CIFileApp *pMe);
static void IFileUsage (CIFileApp *pMe, uint16 wParam);
static void DisplayEvent (CIFileApp *pMe, uint16 wParam);
static int DisplayOutput(IShell *pIShell, IDisplay *pIDisplay, int nline, char *pszStr);
static void WriteLine (CIFileApp *pMe, char *pszStr);

/*-------------------------------------------------------------------
            Global Constant Definitions
-------------------------------------------------------------------*/

// App specific constants
#define USAGE_BASIC_USAGE       100
#define USAGE_FMGR_ENUMINIT     101
#define USAGE_FMGR_ENUMNEXT     102
#define USAGE_FMGR_GETFREESPACE 103
#define USAGE_FMGR_GETINFO      104
#define USAGE_FMGR_GETLASTERROR 105
#define USAGE_FMGR_MDKIR        106
#define USAGE_FMGR_OPENFILE     107
#define USAGE_FMGR_REMOVE       108
#define USAGE_FMGR_RENAME       109
#define USAGE_FMGR_RMDIR        110
#define USAGE_FMGR_TEST         111
#define USAGE_FILE_GETINFO      112
#define USAGE_FILE_READ         113
#define USAGE_FILE_RELEASE      114
#define USAGE_FILE_SEEK         115
#define USAGE_FILE_TRUNCATE     116
#define USAGE_FILE_WRITE        117
#define USAGE_ERASE_SCREEN      118   // Special case to erase screen

// App Resource File
#define APP_RES_FILE "ifile.bar"

// Size of buffer used to hold text strings
#define TEXT_BUFFER_SIZE (100 * sizeof(AECHAR))

/*===============================================================================
                     FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to IFile_InitAppData(). This is done as follows:
   IFile_InitAppData() is called to initialize the applet's data. It is app developer's 
   responsibility to fill-in app data initialization code of IFile_InitAppData(). App developer
   is also responsible to release memory allocated for data -- this can be
   done in IFile_FreeAppData().

PROTOTYPE:
	boolean AEEClsCreateInstance(AEECLSID ClsId, IShell* pIShell, IModule* pIModule, 
										  void** ppObj)

PARAMETERS:
	ClsId: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell interface. 

	pIModule: [in]: Contains pointer to the IModule interface to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid AEEApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app needs to be loaded and if AEEApplet_New() invocation was successful
  FALSE: If the app does not need to be loaded or if errors occurred in AEEApplet_New().
  If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId, IShell* pIShell, IModule* po, void** ppObj)
{
   *ppObj = NULL;
		
   // We want to load this App. So, invoke AEEApplet_New().To it, pass the
   // address of the app-specific handle event function.
   if(ClsId == AEECLSID_FILE_APP){
      if (AEEApplet_New(sizeof(CIFileApp), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)FileApp_HandleEvent,(PFNFREEAPPDATA)IFile_FreeAppData) == TRUE)
      {
         if (IFile_InitAppData((IApplet*)*ppObj))
             return(AEE_SUCCESS);
      }
   }
	return (EFAILED);
}

/*===========================================================================

FUNCTION FileApp_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function.

PROTOTYPE:
	static boolean App_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet.

	ecode: Specifies the Event sent to this applet

    wParam, dwParam: Event-specific data.

DEPENDENCIES
    none

RETURN VALUE
    TRUE: If the app has processed the event
    FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/

static boolean FileApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
   CIFileApp * pMe = (CIFileApp*)pi;

   switch (eCode) 
	{

        case EVT_APP_START:
			// When this applet is started, try to create the main menu
			// that allows the user to select a usage example to execute.
			// If we cannot create an instance of the menu class, exit and
			// indicate that we have handled the event by returning TRUE
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, 
				  (void **)&pMe->m_pIMenu) != SUCCESS)
				return TRUE;

			// Succeeded in obtaining a menu instance pointer, so
			// construct the menu and display it on the screen
			BuildMainMenu(pMe);
			return(TRUE);

        case EVT_APP_STOP:
            return(TRUE);

        case EVT_KEY: 
				if (pMe->m_pIMenu)
				{
					if ((IMENUCTL_IsActive(pMe->m_pIMenu) == FALSE) &&
						 ((wParam == AVK_UP) || (wParam == AVK_DOWN)))
					{
						IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);
						IMENUCTL_Redraw(pMe->m_pIMenu);
					}
			      return IMENUCTL_HandleEvent(pMe->m_pIMenu, EVT_KEY, wParam, 0);
				}
				else
					return FALSE;

		case EVT_COMMAND:
  		    switch(wParam)
			{
				// The following commands are generated by user selections
				// from the main usage app menu.
			    case USAGE_BASIC_USAGE:
                case USAGE_FMGR_ENUMINIT:
                case USAGE_FMGR_ENUMNEXT:
                case USAGE_FMGR_GETFREESPACE:
                case USAGE_FMGR_GETINFO:
                case USAGE_FMGR_GETLASTERROR:
                case USAGE_FMGR_MDKIR:
                case USAGE_FMGR_OPENFILE:
                case USAGE_FMGR_REMOVE:
                case USAGE_FMGR_RENAME:
                case USAGE_FMGR_RMDIR:
                case USAGE_FMGR_TEST:
                case USAGE_FILE_GETINFO:
                case USAGE_FILE_READ:
                case USAGE_FILE_RELEASE:
                case USAGE_FILE_SEEK:
                case USAGE_FILE_TRUNCATE:
				case USAGE_FILE_WRITE:

				   // Deactivate main menu to turn off horizontal scrolling
				   // of long menu items
				   IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);

				   // Execute the usage example the user has selected
                   IFileUsage (pMe, wParam);
			       return TRUE;
			    default:
				    return FALSE;
			}
        default:
            break;
   }
   return TRUE;
}


/*===========================================================================

FUNCTION IFile_InitAppData

DESCRIPTION
   This function initializes app specific data.

PROTOTYPE:
	static boolean IFile_InitAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure.

DEPENDENCIES
    None.

RETURN VALUE
    TRUE: If the app has app data is allocated and initialized successfully
    FALSE: Either app data could not be allocated or initialized

SIDE EFFECTS
    None.
===========================================================================*/
static boolean IFile_InitAppData(IApplet* pi)
{
   CIFileApp * pMe = (CIFileApp*)pi;
   int charHeight, pnAscent, pnDescent;
   AEEDeviceInfo di;

   // Make sure the pointers we'll be using are valid
   if (pMe == NULL || pMe->a.m_pIShell == NULL)
	   return FALSE;

   pMe->m_pIMenu = NULL;
   pMe->m_pIFileMgr = NULL;

   // Determine the amount of available screen space
   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);

   // Determine the height of a line of text
   charHeight = IDISPLAY_GetFontMetrics (pMe->a.m_pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   // Number of available lines equals the available screen
   // space divided by the height of a line, minus 3 for the
   // lines we always print at the top and bottom of the screen
   pMe->m_cMaxLine = (di.cyScreen / charHeight) - 3;

   return TRUE;
}

/*===========================================================================

FUNCTION IFile_FreeAppData

DESCRIPTION
    Frees any data and interfaces used by this app.

PROTOTYPE:
	static void IFile_FreeAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure.

DEPENDENCIES
    None.

RETURN VALUE
    None.

SIDE EFFECTS
    None.
===========================================================================*/
static void IFile_FreeAppData(IApplet* pi)
{
   CIFileApp * pMe = (CIFileApp*)pi;

   if(pMe->m_pIMenu != NULL)
   {
	  IMENUCTL_Release(pMe->m_pIMenu);
      pMe->m_pIMenu = NULL;
   }

   if(pMe->m_pIFileMgr != NULL)
   {
	  IFILEMGR_Release(pMe->m_pIFileMgr);
      pMe->m_pIFileMgr = NULL;
   }
}

/*===============================================================================
                     APP HELPER FUNCTION DEFINITIONS
=============================================================================== */
/*===========================================================================

FUNCTION: IFileUsage

DESCRIPTION
    This function executes the usage example selected from the main menu by the
	user.  It takes as input the app data pointer and the ID of the example the
	user selected.  After clearing the screen, the selected example is executed.
	Each example is a self-contained code block that contains any local variables
	needed to execute the example.  Each code block also creates and releases any
	BREW interfaces that are required, and will exit if any errors are detected.  
	Once the example has been executed, this function prints a heading at the top 
	of the screen to indicate which example was selected, 
	and a message at the bottom of the screen instructing the user how to return to
	the main menu to execute another example.
    
PROTOTYPE:
   static void IFileUsage(CIFileApp * pMe, uint16 wParam)

PARAMETERS:
   pMe:   [in]: Contains a pointer to the usage app data structure.
   wParam: [in]: Identifier of the selected example (example IDs are specified when the
                 main usage app menu is created).

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void IFileUsage (CIFileApp * pMe, uint16 wParam)
{
   // Make sure the pointers we'll be using are valid
   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
	   return;

   // Erase screen first for display output purposes.
   DisplayEvent (pMe, USAGE_ERASE_SCREEN);
	IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
	IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);

   // Initialize line number where we will start the output 
   // of each test
   pMe->m_cLineNum = 1;

   switch (wParam)
   {
      case USAGE_BASIC_USAGE:
         {
            // 1. Usage: Basic File Usage

			// This example illustrates several of the functions in the IFileMgr
			// and IFile interfaces.  We first create a directory and two files within
			// the directory, using IFileMgr's EnumInit and EnumNext functions to verify
			// that the creates succeeded.  We then open one of the files for reading and
			// writing, and write three sentences to it with IFile's Write function.  We then
			// use IFile's Seek and Read functions to read back the second sentence we wrote
			// to the file and echo it to the screen.  We then clean up by deleting the directory
			// and files we created, using IFileMgr's Test function to verify that they no longer
			// exist.

  		    FileInfo fi;
			IFile * pIFile1, * pIFile2;
			char szFirstStr[]  = "First Sentence";
            char szSecondStr[] = "Second Sentence";
            char szThirdStr[]  = "Third Sentence";
			char szBuffer[30];

			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR,(void **)&pMe->m_pIFileMgr))
			    return;

			// Create a new directory.  It will be created as a sub-directory of
			// the directory that contains this app's files.
			if (IFILEMGR_MkDir(pMe->m_pIFileMgr, "xxxdir") != SUCCESS)
			{	
				IFILEMGR_Release(pMe->m_pIFileMgr);
            pMe->m_pIFileMgr = NULL;
				return;
			}

			// Create two new files in this directory, specifying the directory
			// and file names
			if ((pIFile1 = IFILEMGR_OpenFile(pMe->m_pIFileMgr, "xxxdir/xx1.fil", _OFM_CREATE)) == NULL)
			{	
				IFILEMGR_Release(pMe->m_pIFileMgr);
            pMe->m_pIFileMgr = NULL;
				return;
			}

            if ((pIFile2 = IFILEMGR_OpenFile(pMe->m_pIFileMgr, "xxxdir/xx2.fil", _OFM_CREATE)) == NULL)
			{	
				IFILEMGR_Release(pMe->m_pIFileMgr);
            pMe->m_pIFileMgr = NULL;
				IFILE_Release(pIFile1);
				return;
			}

			// To verify that the directory was created, list it
			// (TRUE parm means list directories only)
            IFILEMGR_EnumInit(pMe->m_pIFileMgr, "xxxdir", TRUE);
            while (IFILEMGR_EnumNext(pMe->m_pIFileMgr, &fi))
				WriteLine(pMe, fi.szName);

			// To verify that the files were created, list them
			// (FALSE parm means list files only)
            IFILEMGR_EnumInit(pMe->m_pIFileMgr, "xxxdir", FALSE);
            while (IFILEMGR_EnumNext(pMe->m_pIFileMgr, &fi))
				WriteLine(pMe, fi.szName);

			// Write the three test sentences to the file we just opened.  IFILE_Write
			// returns the number of bytes it wrote, which we check to make sure the 
			// write operations were successful.
			if (IFILE_Write(pIFile1, szFirstStr, sizeof(szFirstStr)) != sizeof(szFirstStr))
				WriteLine(pMe, "1st Write failed");
			if (IFILE_Write(pIFile1, szSecondStr, sizeof(szSecondStr)) != sizeof(szSecondStr))
				WriteLine(pMe, "2nd Write failed");
			if (IFILE_Write(pIFile1, szThirdStr, sizeof(szThirdStr)) != sizeof(szThirdStr))
				WriteLine(pMe, "3rd Write failed");

			// To verify that we wrote to the file successfully,
			// retrieve its size in bytes using GetInfo.
			IFILE_GetInfo(pIFile1, &fi);
			SPRINTF(szBuffer, "Wrote %d bytes", fi.dwSize); 
			WriteLine(pMe, szBuffer);

			// Read back the second sentence from the file.
			// First, we seek to the start position of the sentence
			// by hopping over the first sentence
			if (IFILE_Seek(pIFile1, _SEEK_START, sizeof(szFirstStr)) == SUCCESS)
			{
			    // Read the second sentence and echo it back to the screen
			    // (when successful, IFile_Read returns the number of bytes read)
			    if (IFILE_Read(pIFile1, szBuffer, sizeof(szSecondStr)) == sizeof(szSecondStr))
				    WriteLine(pMe, szBuffer);
			}

			// Clean up--first remove the two files we created
			// (must close them first--second file is still open from when 
			// we created it)
            IFILE_Release(pIFile1);
			if (IFILEMGR_Remove(pMe->m_pIFileMgr, "xxxdir/xx1.fil") != SUCCESS)
				WriteLine(pMe, "1st Remove Failed");

            IFILE_Release(pIFile2);
            if (IFILEMGR_Remove(pMe->m_pIFileMgr, "xxxdir/xx2.fil") != SUCCESS)
				WriteLine(pMe, "2nd Remove Failed");

			// The directory we created is now empty, so we can remove it
			// as well (note that we don't need to open and close directories)
			if (IFILEMGR_RmDir(pMe->m_pIFileMgr, "xxxdir") != SUCCESS)
				WriteLine(pMe, "Dir Remove Failed");

			// To verify that our removals really happened, we can test if the
			// directory and its files are still there
			if (IFILEMGR_Test(pMe->m_pIFileMgr, "xxxdir") == SUCCESS)
				WriteLine(pMe, "Dir Still There!");

			if (IFILEMGR_Test(pMe->m_pIFileMgr, "xxxdir/xx1.fil") == SUCCESS)
				WriteLine(pMe, "xxxdir/xx1.fil Still There!");

			if (IFILEMGR_Test(pMe->m_pIFileMgr, "xxxdir/xx2.fil") == SUCCESS)
				WriteLine(pMe, "xxxdir/xx2.fil Still There!");

            // We conclude by releasing the IFileMgr interface we created initially
			IFILEMGR_Release(pMe->m_pIFileMgr);
         pMe->m_pIFileMgr = NULL;
		 }
         break;

      case USAGE_FMGR_ENUMINIT:
         {
            // 2. Usage: IFILEMGR_EnumInit

			// IFILEMGR_EnumInit initializes an enumeration of all the files or 
			// directories in a given directory.  After calling IFILEMGR_Init, you can
			// then call IFILEMGR_EnumNext repeatedly to obtain information about each
			// file or directory being enumerated.  In this example, we use IFILEMGR_EnumInit
			// to initialize the enumeration of all files in this app's directory (not all of
			// them can be displayed on the screen).

		    FileInfo fi;

			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR,(void **)&pMe->m_pIFileMgr))
			return;

			// Initialize the enumeration of all files in the app directory.
			// The second parameter can be set to match all files with a given
			// extension, e.g. *.c.  Setting it to NULL matches all files in the
			// app directory.  You can also set it to the name of a directory in
			// the app directory, which will enumerate all files in that directory.
            if (IFILEMGR_EnumInit(pMe->m_pIFileMgr, "", FALSE) != SUCCESS)
			{
				IFILEMGR_Release(pMe->m_pIFileMgr);
            pMe->m_pIFileMgr = NULL;
				return;
			}

            while (IFILEMGR_EnumNext(pMe->m_pIFileMgr, &fi) == TRUE)
				WriteLine(pMe, fi.szName);

			IFILEMGR_Release(pMe->m_pIFileMgr);
         pMe->m_pIFileMgr = NULL;
         }
         break;

      case USAGE_FMGR_ENUMNEXT:
         {
            // 3. Usage: IFILEMGR_EnumNext

			// IFILEMGR_EnumNext is used together with IFILEMGR_EnumInit (see above) to
			// enumerate all the files or directories in a directory.  In this example,
			// we use the two functions to enumerate all the directories in this app's directory.
		    FileInfo fi;

			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR,(void **)&pMe->m_pIFileMgr))
			    return;

			// Initialize the enumeration of all directories in the app directory.
            if (IFILEMGR_EnumInit(pMe->m_pIFileMgr, "", TRUE) != SUCCESS)
			{
				IFILEMGR_Release(pMe->m_pIFileMgr);
            pMe->m_pIFileMgr = NULL;
				return;
			}

			// Now enumerate the directories and print the name of each one we
			// find.  IFILEMGR_EnumNext populates a FileInfo structure
			// for each directory it finds.  It returns FALSE when there are no
			// more directories to enumerate.
            while (IFILEMGR_EnumNext(pMe->m_pIFileMgr, &fi) == TRUE)
				WriteLine(pMe, fi.szName);

			IFILEMGR_Release(pMe->m_pIFileMgr);
         pMe->m_pIFileMgr = NULL;
         }
         break;

      case USAGE_FMGR_GETFREESPACE:
         {
            // 4. Usage: IFILEMGR_GetFreeSpace

			// IFILEMGR_GetFreeSpace returns the amount of free space available
			// in the file system.  Here we use this function to retrieve the
			// number of free and total file system bytes and echo the values
			// back to the screen.

			uint32 dwFreeSpace, dwTotalFileSystemSize;
			char szBuf[100];

  			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR,(void **)&pMe->m_pIFileMgr))
			    return;

			// Get file system information.  Note that the amount of free space is the
			// return value of this function, and the total file system size is passed
			// back in the function's second parameter.
            dwFreeSpace = IFILEMGR_GetFreeSpace(pMe->m_pIFileMgr, &dwTotalFileSystemSize);

			// Echo amount of free space
			WriteLine(pMe, "File Space Avail:");
		    SPRINTF(szBuf, "  %u bytes", dwFreeSpace); 
            WriteLine(pMe, szBuf);

			// Echo total file system size
			WriteLine(pMe, "Total File Sys Size:");
		    SPRINTF(szBuf, "  %u bytes", dwTotalFileSystemSize); 
            WriteLine(pMe, szBuf);

			IFILEMGR_Release(pMe->m_pIFileMgr);
         pMe->m_pIFileMgr = NULL;
         }
         break;

      case USAGE_FMGR_GETINFO:
         {
            // 5. Usage: IFILEMGR_GetInfo

			// IFILEMGR_GetInfo retrieves the name, size, creation timestamp and
			// attributes of a file, storing them in a FileInfo structure.  Here we
			// obtain this information for this application's resource file and echo
			// it back to the screen.

			FileInfo fi;
            char szBuf[100];

   			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR,(void **)&pMe->m_pIFileMgr))
			    return;

			// Retrieve information about this app's resource file.  IFILEMGR_GetInfo
			// populates the FileInfo structure pointed to by its second argument
            if (IFILEMGR_GetInfo(pMe->m_pIFileMgr, APP_RES_FILE, &fi) != SUCCESS)
			{
				IFILEMGR_Release(pMe->m_pIFileMgr);
            pMe->m_pIFileMgr = NULL;
				return;
			}

		    SPRINTF(szBuf, "Name: %s", fi.szName); 
			WriteLine(pMe, szBuf);

		    SPRINTF(szBuf, "Size: %d bytes", fi.dwSize); 
			WriteLine(pMe, szBuf);

            SPRINTF(szBuf, "Created: %d", fi.dwCreationDate); 
            WriteLine(pMe, szBuf);

            WriteLine(pMe, "Attributes:");			
 
			// If attribute variable is zero, this is a normal file;
			// otherwise, it is a read-only file
			if (fi.attrib == 0)
                WriteLine(pMe, "_FA_NORMAL");
			else
			{
			    if (fi.attrib && _FA_READONLY)
                    WriteLine(pMe, "  _FA_READONLY");
			}

			IFILEMGR_Release(pMe->m_pIFileMgr);
         pMe->m_pIFileMgr = NULL;
         }
         break;

      case USAGE_FMGR_GETLASTERROR:
         {
            // 6. Usage: IFILEMGR_GetLastError

			// IFILEMGR_GetLastError is used to obtain more detailed information
			// about why an IFileMgr function did not succeed in performing its task.
			// Here we attempt to open a non-existent file and then use 
			// IFILEMGR_GetLastError to pinpoint the cause of the failure of
			// IFILEMGR_OpenFile to open the file.
			IFile * pIFile = NULL;

   			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR,(void **)&pMe->m_pIFileMgr))
			    return;

			// Try to open a non-existent file for reading 
            pIFile = IFILEMGR_OpenFile(pMe->m_pIFileMgr, "xzxzx.bar", _OFM_READ);

			// Use IFILEMGR_GetLastError to figure out why the file open failed
			if (NULL == pIFile)
			{
				// When IFileMgr tries to open the file for reading and is unable 
				// to find it, it sets its last error detected to EFILENOEXISTS.
				// Print out a message to verify that this is true
				if (IFILEMGR_GetLastError(pMe->m_pIFileMgr) == EFILENOEXISTS)
                    WriteLine(pMe, "An EFILENOEXISTS error was detected");
			}
			else IFILE_Release(pIFile);
            
			IFILEMGR_Release(pMe->m_pIFileMgr);
         pMe->m_pIFileMgr = NULL;
         }
         break;

      case USAGE_FMGR_MDKIR:
         {
            // 7. Usage: IFILEMGR_MkDir

			// IFILEMGR_MkDir creates a new directory.  In this example, we 
			// use it to create a new directory and then use the directory-
			// enumeration functions to verify that the directory was created.
 		    FileInfo fi;

			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR,(void **)&pMe->m_pIFileMgr))
			    return;

			// Create a new directory
			if (IFILEMGR_MkDir(pMe->m_pIFileMgr, "x.dir") != SUCCESS)
			{
				IFILEMGR_Release(pMe->m_pIFileMgr);
            pMe->m_pIFileMgr = NULL;
				return;
			}

			// Display all directories to verify that the new one shows up
            if (IFILEMGR_EnumInit(pMe->m_pIFileMgr, "", TRUE) != SUCCESS)
			{
				IFILEMGR_RmDir(pMe->m_pIFileMgr, "x.dir");
				IFILEMGR_Release(pMe->m_pIFileMgr);
            pMe->m_pIFileMgr = NULL;
				return;
			}
				
            while (IFILEMGR_EnumNext(pMe->m_pIFileMgr, &fi))
				WriteLine(pMe, fi.szName);

			// Remove the directory we just created
			IFILEMGR_RmDir(pMe->m_pIFileMgr, "x.dir");

			IFILEMGR_Release(pMe->m_pIFileMgr);
         pMe->m_pIFileMgr = NULL;
         }
         break;

      case USAGE_FMGR_OPENFILE:
         {
            // 8. Usage: IFILEMGR_OpenFile

			// IFILEMGR_OpenFile opens a file for creating, reading, reading
			// and writing, or appending.  It returns a pointer to an IFile
			// instance that can be used to access the file.  Here we open this
			// application's resource file and then use IFile_GetInfo to verify
			// that we opened the file successfully.

			FileInfo fi;
			IFile * pIFile;
			char szBuf[100];

			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR,(void **)&pMe->m_pIFileMgr))
			    return;

			// Open this app's resource file for reading
			if ((pIFile = IFILEMGR_OpenFile(pMe->m_pIFileMgr, APP_RES_FILE, _OFM_READ)) != NULL)
			{
				// To verify that we opened the file successfully,
				// use the file pointer to retrieve its name and print it
				IFILE_GetInfo(pIFile, &fi);

				SPRINTF(szBuf, "Name of Opened File: %s", fi.szName); 
			    WriteLine(pMe, szBuf);
				IFILE_Release(pIFile);
			}
     
			IFILEMGR_Release(pMe->m_pIFileMgr);
         pMe->m_pIFileMgr = NULL;
         }
         break;

      case USAGE_FMGR_REMOVE:
         {
            // 9. Usage: IFILEMGR_Remove

			// IFILEMGR_Remove removes a file from the file system.  To illustrate the
			// use of this function, we create a file, close it, and then remove it with
			// IFILEMGR_Remove.  We enumerate the files before and after the remove to
			// verify that the remove succeeded.
			FileInfo fi;
			IFile * pIFile;

			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR,(void **)&pMe->m_pIFileMgr))
			    return;

			// Create a new file
			if ((pIFile = IFILEMGR_OpenFile(pMe->m_pIFileMgr, "xxxFOO.xxx", _OFM_CREATE)) != NULL)
			{
				// Verify that the file is present
                WriteLine(pMe, "Before Remove:");
                IFILEMGR_EnumInit(pMe->m_pIFileMgr, "*.xxx", FALSE);
                while (IFILEMGR_EnumNext(pMe->m_pIFileMgr, &fi))
				    WriteLine(pMe, fi.szName);

				// Before we remove the file, we must first close it by releasing
				// the interface pointer 
                IFILE_Release(pIFile);

				// Now remove the file
				IFILEMGR_Remove(pMe->m_pIFileMgr, "xxxFOO.xxx");

				// Verify that the file is no longer present
				WriteLine(pMe, "After Remove:");
                IFILEMGR_EnumInit(pMe->m_pIFileMgr, "*.xxx", FALSE);
                while (IFILEMGR_EnumNext(pMe->m_pIFileMgr, &fi))
				    WriteLine(pMe,fi.szName);
			}
     
			IFILEMGR_Release(pMe->m_pIFileMgr);
         pMe->m_pIFileMgr = NULL;
         }
         break;

      case USAGE_FMGR_RENAME:
         {
            // 10. Usage: IFILEMGR_Rename

			// IFILEMGR_Remove renamess a file from the file system.  To illustrate the
			// use of this function, we create a file, close it, and then rename it with
			// IFILEMGR_Rename.  We enumerate the files before and after the rename to
			// verify that the rename succeeded.
			FileInfo fi;
			IFile * pIFile;

			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR,(void **)&pMe->m_pIFileMgr))
			    return;

			// Create a new file
			if ((pIFile = IFILEMGR_OpenFile(pMe->m_pIFileMgr, "foo.xxx", _OFM_CREATE)) != NULL)
			{
				// Verify that the file is present
                WriteLine(pMe,"Before Rename:");
                IFILEMGR_EnumInit(pMe->m_pIFileMgr, "*.xxx", FALSE);
                while (IFILEMGR_EnumNext(pMe->m_pIFileMgr, &fi))
				    WriteLine(pMe, fi.szName);

				// Before we rename the file, we must first close it by releasing
				// the interface pointer 
                IFILE_Release(pIFile);

				// Now rename the file, specifying its old and new names as input
                if (IFILEMGR_Rename(pMe->m_pIFileMgr, "foo.xxx", "bar.xxx") != SUCCESS)
				{
				   IFILEMGR_Remove(pMe->m_pIFileMgr, "foo.xxx");
				   IFILEMGR_Release(pMe->m_pIFileMgr);
               pMe->m_pIFileMgr = NULL;
				   return;
				}

				// Verify that the file has been renamed
				WriteLine(pMe, "After Rename:");
                IFILEMGR_EnumInit(pMe->m_pIFileMgr, "*.xxx", FALSE);
                while (IFILEMGR_EnumNext(pMe->m_pIFileMgr, &fi))
				    WriteLine(pMe,fi.szName);

				// Now remove the file
				IFILEMGR_Remove(pMe->m_pIFileMgr, "bar.xxx");
			}
     
			IFILEMGR_Release(pMe->m_pIFileMgr);
         pMe->m_pIFileMgr = NULL;
         }
         break;

      case USAGE_FMGR_RMDIR:
         {
            // 11. Usage: IFILEMGR_RmDir

			// IFILEMGR_RmDir removes a directory (before a directory can be removed, it
			// cannot contain any files or directories).  In this example, we create a directory,
			// remove it, and list directories before and after the RmDir to verify that the
			// remove was successful.
 		    FileInfo fi;

			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR,(void **)&pMe->m_pIFileMgr))
			    return;

			// Create a new directory
			if (IFILEMGR_MkDir(pMe->m_pIFileMgr, "x.dir") != SUCCESS)
			{
		       IFILEMGR_Release(pMe->m_pIFileMgr);
             pMe->m_pIFileMgr = NULL;
			   return;
			}

			WriteLine(pMe, "After MkDir:");

			// Display all directories to verify that the new one shows up
            IFILEMGR_EnumInit(pMe->m_pIFileMgr, "*.dir", TRUE);
            while (IFILEMGR_EnumNext(pMe->m_pIFileMgr, &fi))
				WriteLine(pMe, fi.szName);

			// Remove the directory we just created
			if (IFILEMGR_RmDir(pMe->m_pIFileMgr, "x.dir") != SUCCESS)
			{
		       IFILEMGR_Release(pMe->m_pIFileMgr);
             pMe->m_pIFileMgr = NULL;
			   return;
			}

			WriteLine(pMe, "After RmDir:");

			// Display all directories to verify that the directory was removed
            IFILEMGR_EnumInit(pMe->m_pIFileMgr, "*.dir", TRUE);
            while (IFILEMGR_EnumNext(pMe->m_pIFileMgr, &fi))
				WriteLine(pMe, fi.szName);

			IFILEMGR_Release(pMe->m_pIFileMgr);
         pMe->m_pIFileMgr = NULL;
         }
         break;

      case USAGE_FMGR_TEST:
         {
            // 12. Usage: IFILEMGR_Test

			// IFILEMGR_Test checks if a particular file or directory exists in the file
			// system.  As examples of its use, we test for the existence of two files, one
			// that exists and one that does not.

    		if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR,(void **)&pMe->m_pIFileMgr))
			   return;

			// Test if the resource file for this app exists
			if(IFILEMGR_Test(pMe->m_pIFileMgr, APP_RES_FILE) == SUCCESS)
                WriteLine(pMe, "ifile.bar exists");
			else WriteLine(pMe, "ifile.bar does not exist");
						
			// Now test for a non-existent file
			if(IFILEMGR_Test(pMe->m_pIFileMgr, "xxyy.zz") != SUCCESS)
                WriteLine(pMe,"xxyy.zz does not exist");
			else WriteLine(pMe, "xxyy.zz exists");

			IFILEMGR_Release(pMe->m_pIFileMgr);
         pMe->m_pIFileMgr = NULL;
         }
         break;

      case USAGE_FILE_GETINFO:
         {
            // 13. Usage: IFILE_GetInfo

			// IFILE_GetInfo retrieves the name, size, creation timestamp and
			// attributes of a file, storing them in a FileInfo structure.  Here we
			// obtain this information for this application's resource file and echo
			// it back to the screen.
			FileInfo fi;
            char szBuf[100];
            IFile * pIFile;

   			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR,(void **)&pMe->m_pIFileMgr))
			    return;

			// Open the resource file for this app
			if ((pIFile = IFILEMGR_OpenFile(pMe->m_pIFileMgr, APP_RES_FILE, _OFM_READ)) != NULL)
			{
				// To verify that we opened the file successfully,
				// use the file pointer to retrieve its name and print it
				IFILE_GetInfo(pIFile, &fi);

		        SPRINTF(szBuf, "Name: %s", fi.szName); 
			    WriteLine(pMe, szBuf);

		        SPRINTF(szBuf, "Size: %d bytes", fi.dwSize); 
			    WriteLine(pMe, szBuf);

                SPRINTF(szBuf, "Created: %d", fi.dwCreationDate); 
                WriteLine(pMe, szBuf);

                WriteLine(pMe, "Attributes:");			
 
			    // If attribute variable is zero, this is a normal file;
			    // otherwise, it has the read-only attribu
			    if (fi.attrib == 0)
                    WriteLine(pMe,"  _FA_NORMAL");
			    else
				{
			        if (fi.attrib && _FA_READONLY)
                        WriteLine(pMe, "  _FA_READONLY");
				}
				IFILE_Release(pIFile);
			}
			IFILEMGR_Release(pMe->m_pIFileMgr);
         pMe->m_pIFileMgr = NULL;
         }
         break; 

      case USAGE_FILE_READ:
         {
            // 14. Usage: IFILE_Read

			// IFILE_Read reads data from a file and copies it into a buffer in
			// memory whose address is supplied as input.  To illustrate the use of
			// this function, we create a file, write three sentences to it, and then
			// use IFILE_Read to read back the sentences from the file.  The sentences
			// are then echoed back to the screen to verify that the read succeeded.

			IFile * pIFile;

			char szFirstStr[]  = "First Sentence";
            char szSecondStr[] = "Second Sentence";
            char szThirdStr[]  = "Third Sentence";
			char szBuffer[30];

  			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR,(void **)&pMe->m_pIFileMgr))
			    return;

			// Create the file to be read
			if ((pIFile = IFILEMGR_OpenFile(pMe->m_pIFileMgr, "xxyy.zz", _OFM_CREATE)) == NULL)
			{
		       IFILEMGR_Release(pMe->m_pIFileMgr);
             pMe->m_pIFileMgr = NULL;
			   return;
			}

			// First, we write the three sentences to the file we just opened
			IFILE_Write(pIFile, szFirstStr, sizeof(szFirstStr));
            IFILE_Write(pIFile, szSecondStr, sizeof(szSecondStr));
            IFILE_Write(pIFile, szThirdStr, sizeof(szThirdStr));

			// After the file writes above, the file pointer is at the end
			// of the file.  Move the file pointer to the beginning of the 
			// file so we can start reading from there.
			if (IFILE_Seek(pIFile, _SEEK_START, 0) == SUCCESS)
			{
				// Now read back each sentence and echo it on the screen.  Note that
				// the file pointer automatically advances to the end of each sentence
				// as we read it in.  The read function returns the number of bytes read,
				// which we check to make sure the read was successful.
                if (IFILE_Read(pIFile, szBuffer, sizeof(szFirstStr)) == sizeof(szFirstStr))
					WriteLine(pMe, szBuffer);

				if (IFILE_Read(pIFile, szBuffer, sizeof(szSecondStr)) == sizeof(szSecondStr))
					WriteLine(pMe, szBuffer);

				if (IFILE_Read(pIFile, szBuffer, sizeof(szThirdStr)) == sizeof(szThirdStr))
					WriteLine(pMe, szBuffer);
			}

            // Close the file
		    IFILE_Release(pIFile);

			// Remove the file
            IFILEMGR_Remove(pMe->m_pIFileMgr, "xxyy.zz");

			IFILEMGR_Release(pMe->m_pIFileMgr);
         pMe->m_pIFileMgr = NULL;
         }
         break;

      case USAGE_FILE_RELEASE:
         {
            // 15. Usage: IFILE_Release

			// IFILE_Release closes a file that was opened with IFILEMGR_OpenFile.
			// This example illustrates that each open of the file must be matched
			// with a separate close before a file can be removed.

			IFile * pIFile1;

 			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR,(void **)&pMe->m_pIFileMgr))
			    return;

			// Create the file 
			if ((pIFile1 = IFILEMGR_OpenFile(pMe->m_pIFileMgr, "xxyy.zz", _OFM_CREATE)) != NULL)
				WriteLine(pMe, "Created file...");
            else 
			{
			   WriteLine(pMe, "Failed to create...");
		       IFILEMGR_Release(pMe->m_pIFileMgr);
             pMe->m_pIFileMgr = NULL;
			   return;
			}

			// Now try to remove the file.  This will fail since we've opened the file
			//  but have not closed it 
            if (IFILEMGR_Remove(pMe->m_pIFileMgr, "xxyy.zz") == SUCCESS)
				WriteLine(pMe, "Removed file...");
			else WriteLine(pMe, "Remove failed...");

			// Close the file a second time so it can be removed
			IFILE_Release(pIFile1);

			// Try again to remove the file.  This time we'll succeed since we have closed
			// the file 
            if (IFILEMGR_Remove(pMe->m_pIFileMgr, "xxyy.zz") == SUCCESS)
				WriteLine(pMe, "Removed file...");
			else WriteLine(pMe, "Failed to remove file...");
			
			IFILEMGR_Release(pMe->m_pIFileMgr);
         pMe->m_pIFileMgr = NULL;
         }
         break;

      case USAGE_FILE_SEEK:
         {
            // 16. Usage: IFILE_Seek

			// IFILE_Seek is used to position the file cursor, which is the position
			// in the file that will be read or written by the next IFILE_Read or IFILE_Write
			// operation.  In this example, we write three sentences to a file and then use
			// IFILE_Seek in its various forms to move the cursor in the file so the sentences
			// can be read back and echoed to the screen.
			IFile * pIFile;
            uint32 nSeekPos;
			char szFirstStr[]  = "First Sentence";
            char szSecondStr[] = "Second Sentence";
            char szThirdStr[]  = "Third Sentence";
			char szBuffer[30];

  			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR,(void **)&pMe->m_pIFileMgr))
			    return;

			// Create the file to be read
			if ((pIFile = IFILEMGR_OpenFile(pMe->m_pIFileMgr, "xxyy.zz", _OFM_CREATE)) == NULL)
			{
		       IFILEMGR_Release(pMe->m_pIFileMgr);
             pMe->m_pIFileMgr = NULL;
			   return;
			}

			// First, we write the three test sentences to the file we just opened.  
			IFILE_Write(pIFile, szFirstStr, sizeof(szFirstStr));
			IFILE_Write(pIFile, szSecondStr, sizeof(szSecondStr));
			IFILE_Write(pIFile, szThirdStr, sizeof(szThirdStr));
                
			// We now use IFILE_Seek to position the file pointer to read
			// back the three sentences just written

			// After the file writes above, the file pointer is at the end
			// of the file.  Move the file pointer to the beginning of the 
			// file so we can start reading from there.
			if (IFILE_Seek(pIFile, _SEEK_START, 0) == SUCCESS)
            {
				// Read back the first sentence and echo it on the screen.
                IFILE_Read(pIFile, szBuffer, sizeof(szFirstStr));
				WriteLine(pMe, szBuffer);
			}

			// The file pointer now points to the beginning of the second sentence.
			// You can use IFILE_Seek with seek type _SEEK_CURRENT and move distance
			// zero to obtain the current file pointer's position.  To verify that 
			// this works, we get the current position and then seek from the start 
			// of the file back to it to read the second sentence
			nSeekPos = IFILE_Seek(pIFile, _SEEK_CURRENT, 0);
			if (IFILE_Seek(pIFile, _SEEK_START, nSeekPos) == SUCCESS)
			{
				IFILE_Read(pIFile, szBuffer, sizeof(szSecondStr));
				WriteLine(pMe, szBuffer);
			}
 
			// You can also seek backwards from the end of the file by using
			// IFILE_Seek with seek type _SEEK_END and a negative move distance.
			// We use this approach to read in the third sentence from the file.
			if (IFILE_Seek(pIFile, _SEEK_END, -(int32)(sizeof(szThirdStr))) == SUCCESS)
			{
				IFILE_Read(pIFile, szBuffer, sizeof(szThirdStr));
				WriteLine(pMe, szBuffer);
			}

            // Close the file
		    IFILE_Release(pIFile);

		    // Remove the file
            IFILEMGR_Remove(pMe->m_pIFileMgr, "xxyy.zz");

			IFILEMGR_Release(pMe->m_pIFileMgr);
         pMe->m_pIFileMgr = NULL;
         }
         break;

      case USAGE_FILE_TRUNCATE:
         {
            // 17. Usage: IFILE_Truncate

			// IFILE_Truncate reduces the size of a file, discarding a number
			// of bytes at the end of the file equal to the difference between
			// the file's original and truncated sizes.  In this example, we
			// create a file, write some data to it, and then truncate it, checking
			// its size before and after the truncation to verify that it succeeded.
			IFile * pIFile;
			FileInfo fi;
			char szFirstStr[]  = "First Sentence";
            char szSecondStr[] = "Second Sentence";
            char szThirdStr[]  = "Third Sentence";
			char szBuffer[30];

  			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR,(void **)&pMe->m_pIFileMgr))
			    return;

			// Create the file to be read
			if ((pIFile = IFILEMGR_OpenFile(pMe->m_pIFileMgr, "xxyy.zz", _OFM_CREATE)) != NULL)
			{
				// First, we write the three test sentences to the file we just opened.  
				IFILE_Write(pIFile, szFirstStr, sizeof(szFirstStr));
				IFILE_Write(pIFile, szSecondStr, sizeof(szSecondStr));
				IFILE_Write(pIFile, szThirdStr, sizeof(szThirdStr));
                
				// Get the initial size of the file
				IFILE_GetInfo(pIFile, &fi);

		        SPRINTF(szBuffer, "Size before: %d", fi.dwSize); 
			    WriteLine(pMe, szBuffer);

				// Now truncate the file so that it contains only the first two sentences
                if (IFILE_Truncate(pIFile, sizeof(szFirstStr) + sizeof(szSecondStr)) == SUCCESS)
				{
				    // Verify that the size of the file has changed correctly
				    // as a result of the file truncation
				    IFILE_GetInfo(pIFile, &fi);

		            SPRINTF(szBuffer, "Size after: %d", fi.dwSize); 
			        WriteLine(pMe, szBuffer);
				}

                // Close the file
				IFILE_Release(pIFile);

				// Remove the file
                IFILEMGR_Remove(pMe->m_pIFileMgr, "xxyy.zz");
			}

			IFILEMGR_Release(pMe->m_pIFileMgr);
         pMe->m_pIFileMgr = NULL;
         }
         break; 
		 
	  case USAGE_FILE_WRITE:
         {
            // 18. Usage: IFILE_Write

			// IFILE_Write writes data to a file.  In this example, we write three
			// sentences to a file and read each one back, echoing it to the screen
			// to verify that the writes were successful.

			IFile * pIFile;

			char szFirstStr[]  = "First Sentence";
            char szSecondStr[] = "Second Sentence";
            char szThirdStr[]  = "Third Sentence";
			char szBuffer[30];

  			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR,(void **)&pMe->m_pIFileMgr))
			    return;

			// Create the file to be read
			if ((pIFile = IFILEMGR_OpenFile(pMe->m_pIFileMgr, "xxyy.zz", _OFM_CREATE)) == NULL)
			{
		       IFILEMGR_Release(pMe->m_pIFileMgr);
             pMe->m_pIFileMgr = NULL;
			   return;
			}

			// First, we write the three sentences to the file we just opened.  IFILE_Write
			// returns the number of bytes it wrote, which we check to make sure the 
			// write operations were successful
			if (IFILE_Write(pIFile, szFirstStr, sizeof(szFirstStr)) != sizeof(szFirstStr))
				WriteLine(pMe, "1st Write Failed");
			if (IFILE_Write(pIFile, szSecondStr, sizeof(szSecondStr)) != sizeof(szSecondStr))
				WriteLine(pMe, "2nd Write Failed");
			if (IFILE_Write(pIFile, szThirdStr, sizeof(szThirdStr)) != sizeof(szThirdStr))
				WriteLine(pMe, "3rd Write Failed");
                
			// Now read back the written sentences from the file to make sure they
			// were written correctly

			// After the file writes above, the file pointer is at the end
			// of the file.  Move the file pointer to the beginning of the 
			// file so we can start reading from there.
			if (IFILE_Seek(pIFile, _SEEK_START, 0) == SUCCESS)
			{
			    // Now read back each sentence and echo it on the screen.
                if (IFILE_Read(pIFile, szBuffer, sizeof(szFirstStr)) == sizeof(szFirstStr))
			        WriteLine(pMe, szBuffer);

			    if (IFILE_Read(pIFile, szBuffer, sizeof(szSecondStr)) == sizeof(szSecondStr))
			        WriteLine(pMe, szBuffer);

			    if (IFILE_Read(pIFile, szBuffer, sizeof(szThirdStr)) == sizeof(szThirdStr))
			        WriteLine(pMe, szBuffer);
			}

            // Close the file
			IFILE_Release(pIFile);

			// Remove the file
            IFILEMGR_Remove(pMe->m_pIFileMgr, "xxyy.zz");

			IFILEMGR_Release(pMe->m_pIFileMgr);
         pMe->m_pIFileMgr = NULL;
         }
         break;

      default:
         return;
   }
            
   // Display above event. 
   DisplayEvent (pMe, wParam);

   return;
}

/*===========================================================================

FUNCTION: DisplayEvent

DESCRIPTION
    This function prints a heading at the top 
	of the screen to indicate which usage example was selected by the user, 
	It also prints a message on the bottom two lines of the screen instructing 
	the user to press the UP or DOWN key to return to
	the main menu to execute another example.
    
PROTOTYPE:
   static void DisplayEvent(CIFileApp * pMe, uint16 wParam)

PARAMETERS:
   pMe:   [in]: Contains a pointer to the usage app data structure.
   wParam: [in]: Identifier of the selected example (example IDs are specified when the
                 main usage app menu is created).  If the value USAGE_ERASE_SCREEN is supplied
				 instead, this function clears the screen.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void DisplayEvent (CIFileApp *pMe, uint16 wParam)
{
   AEERect qrc;
   AEEDeviceInfo di; // Device Info
   AECHAR * szBuf;
   AEERect rc; 

   // Make sure the pointers we'll be using are valid
   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
	   return;

   if ((szBuf = (AECHAR *) MALLOC(TEXT_BUFFER_SIZE)) == NULL)
	   return;

   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);
	qrc.x = 0;
	qrc.y = 0;
	qrc.dx = di.cxScreen;
	qrc.dy = di.cyScreen;
   rc.x = 5;
   rc.y = 5;
   rc.dx = di.cxScreen - 5;
   rc.dy = di.cyScreen - 5;

   // Specify text for heading based on example the user has selected
   switch (wParam)
   {
      case USAGE_ERASE_SCREEN:
         IDISPLAY_EraseRect(pMe->a.m_pIDisplay,&qrc);  //Clear the display
		 FREE(szBuf);
         return;
      case USAGE_BASIC_USAGE:
         STR_TO_WSTR("Basic Usage", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_FMGR_ENUMINIT:
         STR_TO_WSTR("EnumInit", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_FMGR_ENUMNEXT:
         STR_TO_WSTR("EnumNext", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_FMGR_GETFREESPACE:
         STR_TO_WSTR("GetFreeSpace", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_FMGR_GETINFO:
         STR_TO_WSTR("FMgr GetInfo", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_FMGR_GETLASTERROR:
         STR_TO_WSTR("GetLastError", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_FMGR_MDKIR:
         STR_TO_WSTR("MkDir", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_FMGR_OPENFILE:
         STR_TO_WSTR("OpenFile", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_FMGR_REMOVE:
         STR_TO_WSTR("Remove", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_FMGR_RENAME:
         STR_TO_WSTR("Rename", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_FMGR_RMDIR:
         STR_TO_WSTR("RmDir", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_FMGR_TEST:
         STR_TO_WSTR("Test", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_FILE_GETINFO:
         STR_TO_WSTR("File GetInfo", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_FILE_READ:
         STR_TO_WSTR("Read", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_FILE_RELEASE:
         STR_TO_WSTR("Release", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_FILE_SEEK:
         STR_TO_WSTR("Seek", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_FILE_TRUNCATE:
         STR_TO_WSTR("Truncate", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_FILE_WRITE:
         STR_TO_WSTR("Write", szBuf, TEXT_BUFFER_SIZE);
         break;
      default:
         STR_TO_WSTR("Unsupported fn", szBuf, TEXT_BUFFER_SIZE);
   }

   // Print the heading at the top of the screen
   IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,10, 5, &rc,
	  IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);

   // Print a message at the bottom of the screen telling the user
   // how to return to the main app menu
   STR_TO_WSTR("Press Up/Down", szBuf, TEXT_BUFFER_SIZE);
   IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,10, qrc.dy - 25, &rc,
		IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
   STR_TO_WSTR("keys to continue..", szBuf, TEXT_BUFFER_SIZE);
   IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,10, qrc.dy - 15, &rc,
		IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
   IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);

   FREE(szBuf);
}
/*===========================================================================

FUNCTION: BuildMainMenu

DESCRIPTION
    This function constructs the menu control that allows the user to
	select a usage example to execute.  The command ID for each menu item
	is passed to the usage example function above when the user selects the
	menu item for that example.
    
PROTOTYPE:
   static void BuildMainMenu(CIFileApp * pMe)

PARAMETERS:
   pMe:   [in]: Contains a pointer to the usage app data structure.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void BuildMainMenu(CIFileApp *pMe)
{
	AEERect qrc;
	AEEDeviceInfo di;
    AECHAR * szBuf;

	// Make sure the pointers we'll be using are valid
    if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->m_pIMenu == NULL)
	   return;

    if ((szBuf = (AECHAR *) MALLOC(TEXT_BUFFER_SIZE)) == NULL)
	   return;

	// Set Title
    STR_TO_WSTR("IFile Examples", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_SetTitle(pMe->m_pIMenu, NULL, 0, szBuf);

	//Set the Rectangle for the Menu
	ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);
	qrc.x = 0;
	qrc.y = 0;
	qrc.dx = di.cxScreen;
	qrc.dy = di.cyScreen;
	IMENUCTL_SetRect(pMe->m_pIMenu, &qrc);  

	//Add individual entries to the Menu

   // Add Basic Usage to menu
   STR_TO_WSTR("1. Basic Usage", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_BASIC_USAGE, szBuf, 0);

   // Add EnumInit to menu
   STR_TO_WSTR("2. EnumInit", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_FMGR_ENUMINIT, szBuf, 0);

   // Add EnumNext to menu
   STR_TO_WSTR("3. EnumNext", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_FMGR_ENUMNEXT, szBuf, 0);

   // Add GetFreeSpace to menu
   STR_TO_WSTR("4. GetFreeSpace", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_FMGR_GETFREESPACE, szBuf, 0);

   // Add FMgr GetInfo to menu
   STR_TO_WSTR("5. FMgr GetInfo", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_FMGR_GETINFO, szBuf, 0);

   // Add GetLastError to menu
   STR_TO_WSTR("6. GetLastError", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_FMGR_GETLASTERROR, szBuf, 0);

   // Add MkDir to menu
   STR_TO_WSTR("7. MkDir", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_FMGR_MDKIR, szBuf, 0);

   // Add OpenFile to menu
   STR_TO_WSTR("8. OpenFile", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_FMGR_OPENFILE, szBuf, 0);

   // Add Remove to menu
   STR_TO_WSTR("9. Remove", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_FMGR_REMOVE, szBuf, 0);

   // Add Rename to menu
   STR_TO_WSTR("10. Rename", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_FMGR_RENAME, szBuf, 0);

   // Add RmDir to menu
   STR_TO_WSTR("11. RmDir", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_FMGR_RMDIR, szBuf, 0);

   // Add Test to menu
   STR_TO_WSTR("12. Test", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_FMGR_TEST, szBuf, 0);

   // Add File GetInfo to menu
   STR_TO_WSTR("13. File GetInfo", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_FILE_GETINFO, szBuf, 0);
			
   // Add Read to menu
   STR_TO_WSTR("14. Read", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_FILE_READ, szBuf, 0);

   // Add Release to menu
   STR_TO_WSTR("15. Release", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_FILE_RELEASE, szBuf, 0);

   // Add Seek to menu
   STR_TO_WSTR("16. Seek", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_FILE_SEEK, szBuf, 0);

   // Add Truncate to menu
   STR_TO_WSTR("17. Truncate", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_FILE_TRUNCATE, szBuf, 0);	

   // Add Write to menu
   STR_TO_WSTR("18. Write", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_FILE_WRITE, szBuf, 0);	

   // Activate menu
	IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);

	FREE(szBuf);
}
/*===========================================================================

FUNCTION: DisplayOutput

DESCRIPTION
    This function displays an output string at a given line number on the
    screen. If the nline parameter is a negative value (-1) the string
    is displayed in the middle of the screen. If the "nline" value is larger
    than or equal to zero the "nline" value is multiplied by 15 and the 
    resulting value in pixels is set to the y-coordinate of the start of 
    the string display on the screen. If the string does not fit on one line
    the string wraps around to the next line (spaced rougly 10-15 pixels apart).
    By default 5 is used as the starting the x-coordinate of a displayed 
    string.

    How many characters that fit on one line is calculated for each line 
    that is wrapped around to the next line.

    Note: depending on the phone screen size and the fonts used for characters 
          the output might differ on different handsets (devices). Where some 
          handsets will have a smaller screen and large default fonts which will 
          cause partial overlapping of lines. This function does not try to address
          these issues (this is meant as a simple display function).
    
PROTOTYPE:
   int DisplayOutput(IShell *pIShell, IDisplay *pDisplay, int nline, char *pszStr)

PARAMETERS:
   pIShell:   [in]: Contains a pointer to the IShell interface.
   pIDisplay: [in]: Contains a pointer to the IDisplay interface.
   nline:     [in]: Contains the line number to start displaying the text. The line
        numbers are by default spaced 15 pixels apart along the y-axis.
   pszStr:    [in]: The character string to be displayed on the screen.

DEPENDENCIES
  None

RETURN VALUE
  Number of lines written to the screen.

SIDE EFFECTS
  None

===========================================================================*/
static int DisplayOutput(IShell *pIShell, IDisplay *pIDisplay, int nline, char *pszStr)
{
   AEEDeviceInfo di; // Device Info
   AECHAR * szBuf;     // a buffer that supports 200 char string
   AECHAR * psz = NULL;
   int charHeight = 0;      // Stores the char height in pixels for given font
   int pnAscent = 0;        // Stores the ascent in number of pixels
   int pnDescent = 0;       // Stores the descent in number of pixels
   int pixelWidth;
   AEEFont font = AEE_FONT_NORMAL;
   int pnFits = 0, dy;
   int totalCh = 0;
   int numLinesPrinted = 0;

   // Make sure the pointers we'll be using are valid
   if (pIShell == NULL || pIDisplay == NULL)
	   return 0;

   if ((szBuf = (AECHAR *) MALLOC(TEXT_BUFFER_SIZE)) == NULL)
	   return 0;

   // Get device information
   ISHELL_GetDeviceInfo(pIShell,&di);

   // Get the font metrics info
   charHeight = IDISPLAY_GetFontMetrics (pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   // Convert to wide string (unicode)
   STR_TO_WSTR ((char *)pszStr, szBuf, TEXT_BUFFER_SIZE);

   // If nlines is zero then print this string starting around the middle of 
   // the screen. Or else multiply nlines by charheight to decide the y coordinate of
   // the start of the string.
   if (nline < 0) {
      dy = di.cyScreen*2/5;
   }
   else{
      dy = nline * charHeight + 5;
   }

   // psz keeps track of the point from which to write from the string buffer
   // in case the string does not fit one line and needs to wrap around in the
   // next line.
   psz = szBuf;
      
   // Need to calculate the lotal string length to decide if any wrapping
   // around is needed.
   totalCh = STRLEN ((char *)pszStr);

   // Keep displaying text string on multiple lines if the string can't be displayed
   // on one single line. Lines are spaced 15 pixels apart.
   while ((totalCh > 0) && (*psz != NULL))
   { 
      // Get information on how many characters will fit in a line.
      // Give the pointer to the buffer to be displayed, and the number of
      // pixels along the x axis you want to display the string in (max number)
      // pnFits will have the max number of chars that will fit in the maxWidth
      // number of pixels (given string can't fit in one line), or the number of 
      // chars in the string (if it does fit in one line). pnWidth gives the
      // number of pixels that will be used to display pnFits number of chars.
      pixelWidth = IDISPLAY_MeasureTextEx(pIDisplay,
                      font, 
                      (AECHAR *) psz,  // Start of the buffer to display,
                      -1,
                      di.cxScreen - 5, // maxWidth
                      &pnFits);         // Number of chars that will fit a line

      // If pnFits is zero there is something wrong in the input to above function. 
      // Normally this scenario should not occur. But, have the check anyway.
      if (pnFits == 0)
	  {
		 FREE(szBuf);
         return 0;
	  }

      IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, psz, pnFits, 5 /*start dx*/, 
         dy, 0 /* use default rectangle coordinates */, 0);

      psz += pnFits;      // move pointer to the next segment to be displayed
      totalCh -= pnFits;  // reduce the total number of characters to still display
      dy += charHeight;   // Place next line charHeight pixels below the 
                          // previous line.
	  ++numLinesPrinted;

      IDISPLAY_Update(pIDisplay); //, TRUE);
      if (totalCh < pnFits)
         pnFits = totalCh;  // if total number is less than pnFits, adjust pnFits
   }

   FREE(szBuf);

   return numLinesPrinted;   
} // End of DisplayOutput

/*===========================================================================

FUNCTION: WriteLine

DESCRIPTION
    This function displays lines of text on the screen, taking into account
	the number of text lines printed so far and the number of lines available
	for output.

    If all available lines are exhausted, this function returns without doing
	anything.

    If the last available line of the screen is reached, this function prints
	"..." to indicate that some lines may not have been printed due to lack
	of space.

    Otherwise, this function prints the line on the screen by calling DisplayOutput,
	and updates the count of lines printed based on how many lines DisplayOutput
	used to print the text.
 
    
PROTOTYPE:
   static void WriteLine(CIFileApp * pMe,  char *pszStr)

PARAMETERS:
   pMe:   [in]: Contains a pointer to this app's data structure.
   pszStr:    [in]: The character string to be displayed on the screen.

DEPENDENCIES
  Assumes the pMe->m_cLineNum gets initialized to the starting line for text display
  on the screen, and that pMe->m_cMaxLine contains the last available line for 
  output on the screen.

RETURN VALUE
  None.

SIDE EFFECTS
  None

===========================================================================*/
static void WriteLine(CIFileApp * pMe, char *pszStr)
{
   
	// If all available lines used, return
	if (pMe->m_cLineNum > pMe->m_cMaxLine)
		return;

	// If we've reached last available line, print an
	// ellipsis indicator
	if (pMe->m_cLineNum == pMe->m_cMaxLine)
	{
		DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, pMe->m_cLineNum++, "...");
	    return;
	}

	// There are lines available for printing.  Print the string passed as
	// input and update the count of lines available
    pMe->m_cLineNum += DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, pMe->m_cLineNum, pszStr);
	return;

} // End of WriteLine

