/*===========================================================================

FILE: igraphicsusage.c

SERVICES: Sample applet using AEE

DESCRIPTION
  This file contains usage examples of IGPRAPHICS interface. IGRAPHICS interface
  functions allow the user to create/operate on IGraphics objects. IGraphics
  objects are used to invoke graphics related functionality.

  To see the effects of the operations of graphics at work a phone device with 
  a color display should be used.

PUBLIC CLASSES:  
   N/A


   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/

/*===============================================================================
                     INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "AEEModGen.h"
#include "AEEAppGen.h"
#include "AEEShell.h"
#include "AEEMenu.h"
#include "AEEStdLib.h"
#include "AEEUsageAppIDs.h"
#include "AEEGraphics.h"

// App resource header generated by BREW ResourceEditor.
#include "igraphics_res.h"

/*===========================================================================

                      PUBLIC DATA DECLARATIONS

===========================================================================*/

/*-------------------------------------------------------------------
            Type Declarations
-------------------------------------------------------------------*/
// Graphics struct. This is the main struct for this applet. This will hold 
// all the data members that needs to be remembered throughout the life of
// the applet. 
// THE FIRST DATA MEMBER OF THIS STRUCT MUST BE AN AEEApplet OBJECT. 
// This is critical requirement which is related to an applet being
// dynamically loaded. The app developers should add their data members
// following the AEEApplet data member.
typedef struct _CIGraphicsApp
{
	AEEApplet	a;          // Mandatory first AEEApplet data member
   IMenuCtl *  m_pIMenu;

   // Device parameters
   int             m_nLineHeight;
   AEEDeviceInfo   m_dInfo;

}CIGraphicsApp;

/*-------------------------------------------------------------------
            Function Prototypes
-------------------------------------------------------------------*/
// Mandatory function:
// App Handle Event function
static boolean GraphicsApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);

// App specific data alloc-init/free functions
static boolean IGraphics_InitAppData(IApplet* pMe);
static void IGraphics_FreeAppData(IApplet* pMe);

static void BuildMainMenu(CIGraphicsApp *pMe);
static void IGraphicsUsage (CIGraphicsApp * pMe, uint16 wParam);
void DisplayEvent (CIGraphicsApp *pMe, uint16 wParam, AECHAR* pStr);

/*-------------------------------------------------------------------
            Global Constant Definitions
-------------------------------------------------------------------*/
// App Resource File
#define  APP_RES_FILE      "igraphics.bar"

// App specific constants
#define USAGE_BASIC_USAGE         100
#define USAGE_SETBACKGROUND       101
#define USAGE_SETCOLOR            102
#define USAGE_SETFILLMODE         103
#define USAGE_SETFILLCOLOR        104
#define USAGE_SETPOINTSIZE        105
#define USAGE_SETCLIP             106
#define USAGE_SETWINDOW           107
#define USAGE_SETPAINTMODE        108
#define USAGE_GETBACKGROUND       109
#define USAGE_GETCOLORDEPTH       110
#define USAGE_GETCOLOR            111
#define USAGE_GETFILLMODE         112
#define USAGE_GETFILLCOLOR        113
#define USAGE_GETPOINTSIZE        114
#define USAGE_GETCLIP             115
#define USAGE_GETWINDOW           116
#define USAGE_GETPAINTMODE        117
#define USAGE_DRAWPOINT           118
#define USAGE_DRAWLINE            119
#define USAGE_DRAWRECT            120
#define USAGE_DRAWCIRCLE          121
#define USAGE_DRAWARC             122
#define USAGE_DRAWELLIPSE         123
#define USAGE_DRAWTRIANGLE        124
#define USAGE_DRAWPOLYGON         125
#define USAGE_DRAWPOLYLINE        126
#define USAGE_CLEARRECT           127
#define USAGE_ENABLEDOUBLEBUFFER  128
#define USAGE_SWAP                129
#define USAGE_TRANSLATE           130
#define USAGE_PAN                 131
#define USAGE_STRETCH                 132

#define START_ARCANGLE            30
#define ARC_ANGLE                 120

/*===============================================================================
                     FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to InitAppData(). This is done as follows:
   InitAppData() is called to initialize AppletData instance. It is app developers 
   responsibility to fill-in app data initialization code of InitAppData(). App developer
   is also responsible to release memory allocated for data contained in AppletData -- this can be
   done in FreeAppData().

PROTOTYPE:
   int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell interface. 

	pIModule: pin]: Contains pointer to the IModule interface to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid IApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the app needs to be loaded and if AEEApplet_New() invocation was
     successful
  EFAILED: If the app does not need to be loaded or if errors occurred in 
     AEEApplet_New(). If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
{
   *ppObj = NULL;
		
   // We want to load this App. So, invoke AEEApplet_New().To it, pass the
   // address of the app-specific handle event function.
   if(ClsId == AEECLSID_GRAPHICS_APP){
      if(AEEApplet_New(sizeof(CIGraphicsApp), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)GraphicsApp_HandleEvent,(PFNFREEAPPDATA)IGraphics_FreeAppData)
         == TRUE)
      {
         if (IGraphics_InitAppData((IApplet*)*ppObj) == TRUE)
         {
            return(AEE_SUCCESS);
         }
      }
   }
	return (EFAILED);
}

/*===========================================================================

FUNCTION GraphicsApp_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean GraphicsApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance().

	ecode: Specifies the Event sent to this applet

   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/

static boolean GraphicsApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
   CIGraphicsApp * pMe = (CIGraphicsApp *)pi;
   
   switch (eCode) 
	{
      case EVT_APP_START:
         // Create the IMenu interface object. When the applet is started the 
         // main menu is set up. When the user selects a menu item from the main 
         // menu, an EVT_COMMAND event is received by this applet. The processing for 
         // EVT_COMMAND event is found below.
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, (void **)&pMe->m_pIMenu)
            != SUCCESS)
         {
				return FALSE;
         }

         // Build the main menu
			BuildMainMenu(pMe);
			return(TRUE);
      case EVT_APP_STOP:
         return(TRUE);
      case EVT_KEY: 
         // We come here when a key is pressed on the handset keypad. Call 
         // the IMenuCtl handle event function to handle this key press.
         // If this key press corressponds to an Up/Down arrow 
         // key press, this event will be handled by the IMenuCtl object
         // in moving to the next menu item (in the given direction). 
         // If the key is neither Up/Down, the IMenuCtl the key press 
         // is not handled, hence, return FALSE.
			if(pMe->m_pIMenu != NULL && IMENUCTL_HandleEvent(pMe->m_pIMenu, EVT_KEY, wParam, 0))
				return TRUE;
         else
            return FALSE;
		case EVT_COMMAND:
         // When a menu item is selected this applet receives the
         // EVT_COMMAND event in the wParam field.
  		   switch(wParam)
         {
            case USAGE_BASIC_USAGE:
            case USAGE_SETBACKGROUND:
            case USAGE_SETCOLOR:
            case USAGE_SETFILLMODE:
            case USAGE_SETFILLCOLOR:
            case USAGE_SETPOINTSIZE:
            case USAGE_SETCLIP:
            case USAGE_SETWINDOW:
            case USAGE_SETPAINTMODE:
            case USAGE_GETBACKGROUND:
            case USAGE_GETCOLORDEPTH:
            case USAGE_GETCOLOR:
            case USAGE_GETFILLMODE:
            case USAGE_GETFILLCOLOR:
            case USAGE_GETPOINTSIZE:
            case USAGE_GETCLIP:
            case USAGE_GETWINDOW:
            case USAGE_GETPAINTMODE:
            case USAGE_DRAWPOINT:
            case USAGE_DRAWLINE:
            case USAGE_DRAWRECT:
            case USAGE_DRAWCIRCLE:
            case USAGE_DRAWARC:
            case USAGE_DRAWELLIPSE:
            case USAGE_DRAWTRIANGLE:
            case USAGE_DRAWPOLYGON:
            case USAGE_DRAWPOLYLINE:
            case USAGE_CLEARRECT:
            case USAGE_ENABLEDOUBLEBUFFER:
            case USAGE_SWAP:
            case USAGE_TRANSLATE:
            case USAGE_PAN:
				case USAGE_STRETCH:	
               // Call the usage function that exercises all the 
               // interface functions.
               IGraphicsUsage (pMe, wParam);
               return TRUE;
			   default:
				   return FALSE;
         }
      default:
         break;
   }
   return FALSE;
}



/*===========================================================================

FUNCTION IGraphics_InitAppData

DESCRIPTION
   This function initializes app specific dataallocates memory for app data 
   (AppletData) and sets it to pAppData of AEEApplet. It also function . 
   App developer needs to fill-in the initialization code.

PROTOTYPE:
	boolean IGraphics_InitAppData(IApplet * pi)

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains 
      information specific to this applet. It was initialized during the 
      AEEClsCreateInstance().

DEPENDENCIES
   Assumes pi is not NULL

RETURN VALUE
  TRUE: If the app has app data is allocated and initialized successfully
  FALSE: Either app data could not be allocated or initialzied

SIDE EFFECTS
  none
===========================================================================*/
static boolean IGraphics_InitAppData(IApplet * pi)
{
   int pnAscent;        // Stores the ascent in number of pixels
   int pnDescent;       // Stores the descent in number of pixels

   CIGraphicsApp * pMe = (CIGraphicsApp *)pi;

   // Initialize the MenuCtl pointer to NULL
   pMe->m_pIMenu = NULL;

   // Get the font metrics info
   pMe->m_nLineHeight = IDISPLAY_GetFontMetrics (pMe->a.m_pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&pMe->m_dInfo);

   return TRUE;
}

/*===========================================================================

FUNCTION IGraphics_FreeAppData

DESCRIPTION
   Frees data contained in app data and memory for app data itself.
   App developer needs to free data contained in AppletData.

PROTOTYPE:
	void IGraphics_FreeAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains 
      information specific to this applet. It was initialized during the 
      AEEClsCreateInstance().

DEPENDENCIES
   Assumes pi is not NULL

RETURN VALUE
   None

SIDE EFFECTS
  none
===========================================================================*/
static void IGraphics_FreeAppData(IApplet * pi)
{
   CIGraphicsApp * pMe = (CIGraphicsApp *)pi;

   if (pMe->m_pIMenu != NULL)
   {
      IMENUCTL_Release (pMe->m_pIMenu);
      pMe->m_pIMenu = NULL;
   }
}

/*===============================================================================
                     APP HELPER FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: IGraphicsUsage

DESCRIPTION
	This function encompasses all the usage examples of all the functions
   in code blocks switched using the BREW API function Id passed into this
   function. The code blocks will include almost all the variable declarations
   needed to demonstrate the usage of a given function. Within the code block
   all the needed interfaces are instantiated and released subsequent to their
   use.

   There are some exceptions to the above general rule, such as call-back functions,
   common helper function, or common display functions called from within the
   code block which are defined elsewhere in this file.

   There are some instances where an object instantiated within a code block is
   not released at the end of the code block, to be released at a later point
   (for example in a call-back function, in this case pointers to such objects
   are stored in the AEEApplet object to be released later).
   
PROTOTYPE:
   void IGraphicsUsage (CIGraphicsApp * pMe, uint16 wParam)

PARAMETERS:
	pMe: [in]: Contains a pointer to the CIGraphicsApp struct.
   wParam: [in]: Contains the Id of the function type to be used (contains the
      Id corresponding to the menu item selected on the emulator).

DEPENDENCIES
  none

RETURN VALUE
  none

SIDE EFFECTS
  none
===========================================================================*/
static void IGraphicsUsage (CIGraphicsApp * pMe, uint16 wParam)
{
   AEEDeviceInfo *di;        // Device Info
   int lineHeight = 0;       // Stores the char height in pixels for given font
   IShell *pIShell = pMe->a.m_pIShell;
   int cx, cy;
   AECHAR szFormat[50];
   AECHAR *pzBuff = NULL;
   pzBuff = (AECHAR *)MALLOC(200);
   *(pzBuff)= (AECHAR)NULL;

   lineHeight = pMe->m_nLineHeight;
   di = &pMe->m_dInfo;

   if (pMe == NULL || pIShell == NULL || pMe->a.m_pIDisplay == NULL || di == NULL)
       return;

   cx = pMe->m_dInfo.cxScreen;
   cy = pMe->m_dInfo.cyScreen;
   // Erase screen first for display output purposes.
   IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);

   switch (wParam)
   {
      case USAGE_BASIC_USAGE:
         {
            // 1. Usage Basic IGraphics Usage.
            IGraphics *pIGraphics = NULL;
            // Please see individual function usage sections below the
            // Basic-Usage section for details on the date types and 
            // their ranges for the parameters initialized below.
            AEERect rect;      // Used to display SetColor
            AEERect rect1;     // Used to display FillColor
            AEECircle circle;  // Used to draw circle
            AEEPoint point;  
            AEELine line;  
            AEEEllipse ellipse; 
            AEEPolygon polygon;
            AEEPoint points[6];
            AEEPolyline polyLine;

            // Init first rectangle
            rect.x = cx/10;
            rect.y = cy/5;
            rect.dx = cx/4;
            rect.dy = cy/3;
            // Init second rectangle
            rect1.x = cx/3;
            rect1.y = cy*3/5;
            rect1.dx = cx/3;
            rect1.dy = cy/6;
            // Init circle
            circle.cx = cx/2;
            circle.cy = cy/2;
            circle.r = cx/4;
            // Init point values
            point.x = cx/2;
            point.y = cy/2;
            // Init line values
            line.sx = cx/4;
            line.sy = cy/6;
            line.ex = cx*4/5;
            line.ey = cy*2/3;
            // Init ellipese values
            ellipse.cx = cx/4;
            ellipse.cy = cy*3/5;
            ellipse.wx = cx/6;
            ellipse.wy = cy/4;

            // Initialize polygon points
            points[0].x = cx*3/5;
            points[0].y = cy/3;
            points[1].x = cx*4/5;
            points[1].y = cy/8;
            points[2].x = cx -10;
            points[2].y = cy*2/5;
            points[3].x = cx -5;
            points[3].y = cy/2;
            points[4].x = cx*4/5;
            points[4].y = cy*3/4;

            polygon.len = 5;          // Initialize number of points
            polygon.points = points;

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {
               // To find out more about the parameters and their types of 
               // the following functions, please see the appropriate
               // function usage section.
               IGRAPHICS_SetColor(pIGraphics, 0xff, 0, 0, 0);
               IGRAPHICS_DrawRect (pIGraphics, &rect);

               // Use FillColor/FillMode/DrawCircle
               IGRAPHICS_SetFillColor (pIGraphics, 0, 0, 0xff, 0);
               IGRAPHICS_SetFillMode (pIGraphics, TRUE);
               IGRAPHICS_DrawCircle (pIGraphics, &circle);

               // Draw Point.
               IGRAPHICS_SetPointSize (pIGraphics, 7);
               IGRAPHICS_DrawPoint (pIGraphics, &point);

               // Draw Line
               IGRAPHICS_SetColor(pIGraphics, 0xff, 0, 0, 0);
               IGRAPHICS_DrawLine (pIGraphics, &line);

               // Draw Ellipse
               IGRAPHICS_SetFillColor (pIGraphics, 0, 0xff, 0, 0);
               IGRAPHICS_DrawEllipse (pIGraphics, &ellipse);

               // Draw Polygon
               IGRAPHICS_SetColor(pIGraphics, 0xff, 0, 0, 0);
               IGRAPHICS_DrawPolygon (pIGraphics, &polygon);

               // Initialize point values
               points[0].x = 5;
               points[0].y = cy*6/7;
               points[1].x = cx/4;
               points[1].y = cy/8;
               points[2].x = cx/2;
               points[2].y = cy/5;
               points[3].x = cx*3/4;
               points[3].y = cy*7/8;
               points[4].x = cx -5;
               points[4].y = cy*3/5;
               points[5].x = cx*3/5;
               points[5].y = cy/8;

               polyLine.len = 6;        // Initialize number of points
               polyLine.points = points;

               // Draw Poly Line
               IGRAPHICS_SetColor(pIGraphics, 0, 0, 0, 0);
               IGRAPHICS_DrawPolyline (pIGraphics, &polyLine);

               IGRAPHICS_SetFillColor (pIGraphics, 0xff, 0xff, 0, 0);
               IGRAPHICS_DrawRect (pIGraphics, &rect1);

               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_SETBACKGROUND:
         {
            // 2. Usage IGRAPHICS_SetBackground
            IGraphics * pIGraphics = NULL;
            RGBVAL rgbVal = 0;
            uint8 red = 0;
            uint8 green = 0;
            uint8 blue = 0xff;

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {
               rgbVal = IGRAPHICS_SetBackground(pIGraphics, red, green, blue);

               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_SETCOLOR:
         {
            // 3. Usage IGRAPHICS_SetColor
            IGraphics * pIGraphics = NULL;
            AEERect rect;
            RGBVAL rgbVal = 0;
            uint8 red = 0;
            uint8 green = 0;
            uint8 blue = 0xff;
            uint8 alpha = 1;

            // Init rectangle coordinates
            rect.x = cx/6;
            rect.y = cy/4;
            rect.dx = cx/2;
            rect.dy = cy/2;

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {
               // Set color
               rgbVal = IGRAPHICS_SetColor(pIGraphics, red, green, blue, alpha);

               // IGRAPHICS_DrawRect below is used only to illustrate the above 
               // functions effects on the graphics object. See 
               // IGRAPHICS_DrawRect usage for more details.
               IGRAPHICS_DrawRect (pIGraphics, &rect);

               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_SETFILLMODE:
         {
            // 4. Usage IGRAPHICS_SetFillMode
            IGraphics *pIGraphics = NULL;
            AEERect rect;
            boolean boolRetVal = FALSE;

            // Init rectangle coordinates
            rect.x = cx/6;
            rect.y = cy/4;
            rect.dx = cx/2;
            rect.dy = cy/2;

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {
               // Set the Fill Mode to True
               boolRetVal = IGRAPHICS_SetFillMode (pIGraphics, TRUE);

               // The functions below are used only to illustrate the above 
               // functions effects on the graphics object. See 
               // IGRAPHICS_SetFillColor/IGRAPHICS_DrawRect usage for more details.
               IGRAPHICS_SetFillColor (pIGraphics, 0xff, 0, 0, 0);
               IGRAPHICS_DrawRect (pIGraphics, &rect);

               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_SETFILLCOLOR:
         {
            // 5. Usage IGRAPHICS_SetFillColor
            IGraphics *pIGraphics = NULL;
            AEECircle circle; 
            uint8 red = 0;
            uint8 green = 0xff;
            uint8 blue = 0;
            uint8 alpha = 0;

            // Circle used for illustration purposes only
            circle.cx = cx/2; // Center x of the circle 
            circle.cy = cy/2; // Center y of the circle 
            circle.r = cx/4;  // Radius of circle

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {
               // Set Fill Color
               IGRAPHICS_SetFillColor (pIGraphics, red, green, blue, alpha);

               // Set FillMode/DrawCircle functions below are used only to 
               // illustrate the above functions effects on the graphics object. 
               // See IGRAPHICS_SetFillMode/IGRAPHICS_DrawCircle usage for details.
               IGRAPHICS_SetFillMode (pIGraphics, TRUE);
               IGRAPHICS_DrawCircle (pIGraphics, &circle);

               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_SETPOINTSIZE:
         {
            // 6. Usage IGRAPHICS_SetPointSize
            IGraphics *pIGraphics = NULL;
            uint8 retPointSize = 0;
            uint8 setPointSize = cx/20;
            AEEPoint point;

            point.x = cx/2; // x coordinate of point
            point.y = cy/2; // y coordinate of point

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {
               // Set point size
               retPointSize = IGRAPHICS_SetPointSize (pIGraphics, setPointSize);

               // The function below is used only to illustrate the above 
               // functions effects on the graphics object. See 
               // IGRAPHICS_DrawPoint usage for more details.
               IGRAPHICS_DrawPoint (pIGraphics, &point);

               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_SETCLIP:
         {
            // 7. Usage IGRAPHICS_SetClip
            IGraphics *pIGraphics = NULL;
            boolean boolRetVal = FALSE;
            AEEClip clip;
            AEECircle circle;
            uint8 nFlag;

            // Circle used for illustration purposes only
            circle.cx = cx/2; // Center x of the circle 
            circle.cy = cy/2; // Center y of the circle 
            circle.r = cx/4;  // Radius of circle
            
            clip.type = CLIPPING_RECT;
            clip.shape.rect.x = cx/13;
            clip.shape.rect.y = cy/6;
            clip.shape.rect.dx = cx/2;
            clip.shape.rect.dy = cy/3;

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {
               // Set flags
               nFlag = AEE_GRAPHICS_FRAME;

               // Set Clip
               boolRetVal = IGRAPHICS_SetClip (pIGraphics, &clip, nFlag);

               // Set FillMode/DrawCircle functions below are used only to 
               // illustrate the above functions effects on the graphics object. 
               // See IGRAPHICS_SetFillMode/IGRAPHICS_DrawCircle usage for details.
               IGRAPHICS_SetFillMode (pIGraphics, TRUE);
               IGRAPHICS_DrawCircle (pIGraphics, &circle);
               
               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_SETWINDOW:
         {
            // 8. Usage IGRAPHICS_SetWindow
            IGraphics *pIGraphics = NULL;
            boolean retVal = FALSE;
            AEERect rect;
            AEERect rect1;
            uint8 nFlag;

            // Init rectangle coordinates
            rect.x = cx/12;
            rect.y = cy/4;
            rect.dx = cx/2;
            rect.dy = cy*5/8;
            rect1.x = cx/6;
            rect1.y = cy/5;
            rect1.dx = cx/2;
            rect1.dy = cy/2;

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {
               // Set flags
               nFlag = AEE_GRAPHICS_FRAME;

               // Set Window
               retVal = IGRAPHICS_SetViewport (pIGraphics, &rect, nFlag);

               // The function below is used only to illustrate the
               // above functions effects on the graphics object. See 
               // See IGRAPHICS_DrawRect usage for more details.
               IGRAPHICS_DrawRect (pIGraphics, &rect1);

               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_SETPAINTMODE:
         {
            // 9. Usage IGRAPHICS_SetPaintMode
            IGraphics *pIGraphics = NULL;
            AEEPaintMode retPaintMode;
            AEEPaintMode inputPaintMode = AEE_PAINT_XOR;
            AEERect rect;

            // Init rectangle coordinates
            rect.x = cx/12;
            rect.y = cy/4;
            rect.dx = cx*5/8;
            rect.dy = cy/3;

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {
               retPaintMode = IGRAPHICS_SetPaintMode (pIGraphics, inputPaintMode);

               // The function below is used only to illustrate the
               // above functions effects on the graphics object. See 
               // See IGRAPHICS_DrawRect usage for more details.
               IGRAPHICS_SetFillMode (pIGraphics, TRUE);
               IGRAPHICS_SetFillColor (pIGraphics, 0, 0, 0xff, 0);
               IGRAPHICS_DrawRect (pIGraphics, &rect);

               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_GETBACKGROUND:
         {
            // 10. Usage IGRAPHICS_GetBackground
            IGraphics *pIGraphics = NULL;
            uint8 red = 0;
            uint8 green = 0;
            uint8 blue = 0;

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {
               // Get background
               IGRAPHICS_GetBackground (pIGraphics, &red, &green, &blue);
			   STR_TO_WSTR("r:%d,g:%d,b:%d", szFormat, sizeof(szFormat));

			   WSPRINTF(pzBuff, 200, szFormat, (int)red,(int)green,(int)blue);

               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_GETCOLORDEPTH:
         {
            // 11. Usage IGRAPHICS_GetColorDepth
            IGraphics *pIGraphics = NULL;
            short colorDepth;

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {
               // Get Color Depth
               colorDepth = IGRAPHICS_GetColorDepth (pIGraphics);
			   STR_TO_WSTR("depth=%d", szFormat, sizeof(szFormat));
			   WSPRINTF(pzBuff, 200, szFormat, (int)colorDepth);

               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_GETCOLOR:
         {
            // 12. Usage IGRAPHICS_GetColor
            IGraphics *pIGraphics = NULL;
            uint8 red = 0;
            uint8 green = 0;
            uint8 blue = 0;
            uint8 alpha = 0;

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {
               // Get Color
               IGRAPHICS_GetColor (pIGraphics, &red, &green, &blue, &alpha);
			   STR_TO_WSTR("r:%d,g:%d,b:5d,Alpha:%d", szFormat, sizeof(szFormat));
			   WSPRINTF(pzBuff, 200, szFormat, red, green,blue, alpha);

               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_GETFILLMODE:
         {
            // 13. Usage IGRAPHICS_GetFillMode
            IGraphics *pIGraphics = NULL;
            boolean retVal = FALSE;

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {
               // Get Fill Mode
               retVal = IGRAPHICS_GetFillMode (pIGraphics);
			   STR_TO_WSTR("fillMode=%d", szFormat, sizeof(szFormat));
			   WSPRINTF(pzBuff, 200, szFormat, (int)retVal);

               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_GETFILLCOLOR:
         {
            // 14. Usage IGRAPHICS_GetFillColor
            IGraphics *pIGraphics = NULL;
            uint8 red = 0;
            uint8 green = 0;
            uint8 blue = 0;
            uint8 alpha = 0;

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {
               // Get Fill Color
               IGRAPHICS_GetFillColor (pIGraphics, &red, &green, &blue, &alpha);
			   STR_TO_WSTR("r:%d,g:%d,b:%d,alpha:%d", szFormat, sizeof(szFormat));
			   WSPRINTF(pzBuff, 200, szFormat, red, green, blue, alpha);

               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_GETPOINTSIZE:
         {
            // 15. Usage IGRAPHICS_GetPointSize
            IGraphics *pIGraphics = NULL;
            uint8 pointSize = 0;

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {
               // Get current point size
               pointSize = IGRAPHICS_GetPointSize (pIGraphics);
			   STR_TO_WSTR("size:%d", szFormat, sizeof(szFormat));
			   WSPRINTF(pzBuff, 200, szFormat, (int)pointSize);

               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_GETCLIP:
         {
            // 16. Usage IGRAPHICS_
            IGraphics *pIGraphics = NULL;
            boolean retVal = FALSE;
            AEEClip clip;

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {
               // Get Clip
               retVal = IGRAPHICS_GetClip (pIGraphics, &clip);
			   STR_TO_WSTR("size:%d", szFormat, sizeof(szFormat));
			   WSPRINTF(pzBuff, 200, szFormat, (int)retVal);

               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_GETWINDOW:
         {
            // 17. Usage IGRAPHICS_GetWindow
            IGraphics *pIGraphics = NULL;
            boolean retVal = FALSE;
            AEERect rect;
            boolean framed = FALSE;

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {
               // Get Window
               retVal = IGRAPHICS_GetViewport (pIGraphics, &rect, &framed); 
               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_GETPAINTMODE:
         {
            // 18. Usage IGRAPHICS_GetPaintMode
            IGraphics *pIGraphics = NULL;
            AEEPaintMode paintMode;

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {
               // Get Paint Mode
               paintMode = IGRAPHICS_GetPaintMode (pIGraphics);

               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_DRAWPOINT:
         {
            // 19. Usage IGRAPHICS_DrawPoint
            IGraphics *pIGraphics = NULL;
            AEEPoint point;

            point.x = cx/2; // x coordinate of point
            point.y = cy/2; // y coordinate of point

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {
               // The function below is used only to increase the size
               // of the point drawn for better illustration. See
               // IGRAPHICS_SetPointSize usage for more details.
               IGRAPHICS_SetPointSize (pIGraphics, 5);

               // Draw Point
               IGRAPHICS_DrawPoint (pIGraphics, &point);

               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_DRAWLINE:
         {
            // 20. Usage IGRAPHICS_DrawLine
            IGraphics *pIGraphics = NULL;
            int retVal = 0;
            AEELine line;

            line.sx = cx/4;   // Starting x coordinate of line
            line.sy = cy/6;   // Starting y coordinate of line
            line.ex = cx*5/6; // Ending x coordinate of line
            line.ey = cy*5/8; // Ending y coordinate of line

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {
               // Draw Line
               retVal = IGRAPHICS_DrawLine (pIGraphics, &line);

               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_DRAWRECT:
         {
            // 21. Usage IGRAPHICS_DrawRect
            IGraphics *pIGraphics = NULL;
            AEERect rect;
            int retVal = 0;

            // Init rectangle coordinates
            rect.x = cx/6;    // Root x coordinate of rectangle
            rect.y = cy/4;    // Root y coordinate of rectangle
            rect.dx = cx/2;   // Rectangle x direction width (in pixels)
            rect.dy = cy/2;   // Rectangle y direction height (in pixels)

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {
               // Draw Rectangle
               retVal = IGRAPHICS_DrawRect (pIGraphics, &rect);

               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_DRAWCIRCLE:
         {
            // 22. Usage IGRAPHICS_DrawCircle
            IGraphics *pIGraphics = NULL;
            AEECircle circle;
            int retVal = 0;

            circle.cx = cx/2;  // Center x of the circle 
            circle.cy = cy/2;  // Center y of the circle 
            circle.r = cx/4;   // Radius of circle

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {
               // Draw Circle
               retVal = IGRAPHICS_DrawCircle (pIGraphics, &circle);

               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_DRAWARC:
         {
            // 23. Usage IGRAPHICS_DrawArc
            IGraphics *pIGraphics = NULL;
            AEEArc  arc;
            int retVal = 0;
            
            // Initialize arc coordinates
            arc.cx = cx/2;   // Center x of the reference circle  
            arc.cy = cy/2;   // Center y of the reference circle
            arc.r = cx/4;    // Radius of the reference circle
            arc.startAngle = START_ARCANGLE; // In degrees from 0 to 359
            arc.arcAngle = ARC_ANGLE;  // In degrees from 0 360

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {
               // Draw Arc
               retVal = IGRAPHICS_DrawArc (pIGraphics, &arc);

               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_DRAWELLIPSE:
         {
            // 24. Usage IGRAPHICS_DrawEllipse
            IGraphics *pIGraphics = NULL;
            AEEEllipse ellipse;
            int retVal = 0;

            // Initialize ellipse coordinates
            ellipse.cx = cx/2;  // Center of x ellipse
            ellipse.cy = cy/2;  // Center of y ellipse
            ellipse.wx = cx/5;  // Semimajor/minor axis along x-axis
            ellipse.wy = cy/4;  // Semimajor/minor axis along y-axis
                                // which is major or minor depends on the 
                                // relative values of wx and wy

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {
               // Draw Ellipse
               retVal = IGRAPHICS_DrawEllipse (pIGraphics, &ellipse);

               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_DRAWTRIANGLE:
         {
            // 25. Usage IGRAPHICS_DrawTriangle
            IGraphics *pIGraphics = NULL;
            AEETriangle triangle;
            int retVal = 0;

            triangle.x0 = cx/12;  // x coordinate of first point
            triangle.y0 = cy/6;   // y coordiante of first point
            triangle.x1 = cx*3/5; // x coordinate of second point
            triangle.y1 = cy/4;   // y coordinate of second point
            triangle.x2 = cx/3;   // x coordinate of third point
            triangle.y2 = cy*3/4; // y coordinate of third point

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {
               // Draw Triangle
               retVal = IGRAPHICS_DrawTriangle (pIGraphics, &triangle);

               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_DRAWPOLYGON:
         {
            // 26. Usage IGRAPHICS_DrawPolygon
            IGraphics *pIGraphics = NULL;
            AEEPolygon polygon;
            int retVal = 0;
            AEEPoint points[5];

            points[0].x = cx/12;
            points[0].y = cy/3;
            points[1].x = cx/4;
            points[1].y = cy/5;
            points[2].x = cx*5/6;
            points[2].y = cy/4;
            points[3].x = cx*7/8;
            points[3].y = cy*5/8;
            points[4].x = cx/2;
            points[4].y = cy*3/4;

            polygon.len = 5;
            polygon.points = points;

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {
               // Draw Polygon
               retVal = IGRAPHICS_DrawPolygon (pIGraphics, &polygon);

               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_DRAWPOLYLINE:
         {
            // 27. Usage IGRAPHICS_DrawPolyline
            IGraphics *pIGraphics = NULL;
            AEEPolyline polyLine;
            AEEPoint points[6];
            int retVal = 0;

            // Initialize points of polyline
            points[0].x = cx/12;
            points[0].y = cy/3;
            points[1].x = cx/4;
            points[1].y = cy/4;
            points[2].x = cx/2;
            points[2].y = cy/2;
            points[3].x = cx*3/4;
            points[3].y = cy/5;
            points[4].x = cx*6/7;
            points[4].y = cy*3/4;
            points[5].x = cx/3;
            points[5].y = cy*3/5;

            polyLine.len = 6;         // Number of points
            polyLine.points = points;

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {
               // Draw Poly Line
               retVal = IGRAPHICS_DrawPolyline (pIGraphics, &polyLine);

               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_CLEARRECT:
         {
            // 28. Usage IGRAPHICS_ClearRect
            IGraphics *pIGraphics = NULL;
            AEERect rect;
            AEERect rect1;

            // Initialize rectangle to draw
            rect.x = cx/6;
            rect.y = cy/5;
            rect.dx = cx*3/5;
            rect.dy = cy/2;

            // Initialize the rectangle to clear
            rect1.x = cx/6;
            rect1.y = cy/5;
            rect1.dx = cx*2/5;
            rect1.dy = cy/3;

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {
               // First fill a larger rectangle and clear a rectangle within it.
               // The functions below are used only to illustrate the above 
               // functions effects on the graphics object. See 
               // IGRAPHICS_SetFillMode/IGRAPHICS_SetFillColor/IGRAPHICS_DrawRect 
               // usage for more details.
               IGRAPHICS_SetFillMode (pIGraphics, TRUE);
               IGRAPHICS_SetFillColor (pIGraphics, 0xff, 0, 0, 0);
               IGRAPHICS_DrawRect (pIGraphics, &rect);

               // Clear Rectangle
               IGRAPHICS_ClearRect (pIGraphics, &rect1); 

               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_ENABLEDOUBLEBUFFER:
         {
            // 29. Usage IGRAPHICS_EnableDoubleBuffer
            IGraphics *pIGraphics = NULL;

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {

               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_SWAP:
         {
            // 30. Usage IGRAPHICS_Swap
            IGraphics *pIGraphics = NULL;

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {

               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_TRANSLATE:
         {
            // 31. Usage IGRAPHICS_Translate
            IGraphics *pIGraphics = NULL;
            AEERect rect;
            int16 x = cx/3;  // The origin will be translated by x pixels along X-axis.
            int16 y = cy/3;  // The origin will be translated by y pixels along Y-axis.

            rect.x = 0;
            rect.y = 0;
            rect.dx = cx*3/5;
            rect.dy = cy/2;

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {
               // This should translate the root coordinates to the given x, y
               // coordinates.
               IGRAPHICS_Translate(pIGraphics, x, y);

               // Draw rectangle. This is for illustrations purposes only.
               IGRAPHICS_DrawRect (pIGraphics, &rect);

               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      case USAGE_STRETCH: {
			// 32. Usage IGRAPHICS_STRETCH
			IGraphics *pIGraphics = NULL;
			AEEBmp pbmSource;
			int xDest = 0;
			int yDest = 0;
			int cxDest = pMe->m_dInfo.cxScreen;
			int cyDest = pMe->m_dInfo.cyScreen;
			int xSrc = 0;
			int ySrc = 0;
			AEERasterOp dwRopCode = AEE_RO_COPY;
			AEEImageInfo imageInfo;
			void *pBmp = NULL;
			byte *pDataBytes = NULL;
			boolean bVal = TRUE;
			

			// Create IGraphics object. This step needs to be done only if
			// the IGraphics object has not been created already. 
			ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);
			
			if (pIGraphics)
			{
				
				
            // Load bitmap resource from the resource file.
            pbmSource = ISHELL_LoadResData (pIShell, APP_RES_FILE, 
					IDB_BITMAP2, RESTYPE_IMAGE); // please do not use RESTYPE_BINARY
				
				//There is a header that needs to be removed before passing the data to convertbmp
				//the first byte specifies the size of the header and hence offset  we need to skip.
            pDataBytes = (byte *)pbmSource + *((byte *)pbmSource);

				IGRAPHICS_StretchBlt(pIGraphics,				
					0,(int16) (yDest+25+5),60,20,pDataBytes, dwRopCode);

				IGRAPHICS_StretchBlt(pIGraphics,				
					0,(int16) (yDest+47+2),20,35,pDataBytes, dwRopCode);				
				
            pBmp = CONVERTBMP (pDataBytes, &imageInfo, &bVal);
				
            // IDISPLAY_BitBlt: performs a bit-block transfer of the data 
            // corresponding to a rectangle of pixels from the specified source 
            // bitmap into the given Display area. After calling this function, 
            // IDISPLAY_Update() must be called to update the screen. If there is 
            // a sequence of drawing operations being performed, it is sufficient 
            // to IDISPLAY_Update() once after all the drawing is done.
            IDISPLAY_BitBlt (pMe->a.m_pIDisplay, xDest, yDest, cxDest, cyDest, pBmp,
               xSrc, ySrc, dwRopCode);
				
			     




            ISHELL_FreeResData (pMe->a.m_pIShell, pbmSource);
				if(bVal)//free only if realloc was done
					SYSFREE ( pBmp);
         }
			
			
			// Release IGraphics object. This step needs to be done
			// only if no further use of the IGraphics object needed.
			IGRAPHICS_Release (pIGraphics);
			FREE(pzBuff);
			IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);
			return;
			
			
			
			}
			break;
		case USAGE_PAN:
         {
            // 32. Usage IGRAPHICS_Pan
            IGraphics *pIGraphics = NULL;

            // Create IGraphics object. This step needs to be done only if
            // the IGraphics object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_GRAPHICS, (void **)&pIGraphics);

            if (pIGraphics)
            {
               // Release IGraphics object. This step needs to be done
               // only if no further use of the IGraphics object needed.
               IGRAPHICS_Release (pIGraphics);
            }
         }
         break;
      default:
         return;
    
   }
            
   // Display above event. 
   DisplayEvent (pMe, wParam, pzBuff);
   FREE(pzBuff);
 
   return;
}

/*===========================================================================

FUNCTION: DisplayEvent

DESCRIPTION
    This function is a display function which sets up the initial screen
    for the display output for a given interface function. This displays
    the function name at the top of the screen and displays the
    strings "Press Up/Down" and "keys to continue.." at the bottom of 
    the screen. In the case of functions that have no output it also 
    displays the string "No Output..." in the middile of the screen.
    
PROTOTYPE:
   void DisplayEvent (CIGraphicsApp * pMe, uint16 wParam)

PARAMETERS:
   pMe: [in]: Pointer to CIGraphicsApp interface object
   wParam:  [in]: API function type Id selected by user

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
void DisplayEvent (CIGraphicsApp * pMe, uint16 wParam, AECHAR* pStr)
{
   AECHAR szBuf[30];
   AEERect rc; 

   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
      return;

   // Set display rectangle
   SETAEERECT (&rc, 0, 0, pMe->m_dInfo.cxScreen, pMe->m_dInfo.cyScreen);
   
   if (pMe->a.m_pIDisplay)
   {
      switch (wParam)
      {
         case USAGE_BASIC_USAGE:
            STR_TO_WSTR("Basic Usage", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETBACKGROUND:
            STR_TO_WSTR("SetBackground", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETCOLOR:
            STR_TO_WSTR("SetColor", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETFILLMODE:
            STR_TO_WSTR("SetFillMode", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETFILLCOLOR:
            STR_TO_WSTR("SetFillColor", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETPOINTSIZE:
            STR_TO_WSTR("SetPointSize", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETCLIP:
            STR_TO_WSTR("SetClip", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETWINDOW:
            STR_TO_WSTR("SetWindow", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETPAINTMODE:
            STR_TO_WSTR("SetPaintMode", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETBACKGROUND:
            STR_TO_WSTR("GetBackground", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETCOLORDEPTH:
            STR_TO_WSTR("GetColorDepth", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETCOLOR:
            STR_TO_WSTR("GetColor", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETFILLMODE:
            STR_TO_WSTR("GetFillMode", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETFILLCOLOR:
            STR_TO_WSTR("GetFillMode", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETPOINTSIZE:
            STR_TO_WSTR("GetPointSize", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETCLIP:
            STR_TO_WSTR("GetClip", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETWINDOW:
            STR_TO_WSTR("GetWindow", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETPAINTMODE:
            STR_TO_WSTR("GetPaintMode", szBuf, sizeof(szBuf));
            break;
         case USAGE_DRAWPOINT:
            STR_TO_WSTR("DrawPoint", szBuf, sizeof(szBuf));
            break;
         case USAGE_DRAWLINE:
            STR_TO_WSTR("DrawLine", szBuf, sizeof(szBuf));
            break;
         case USAGE_DRAWRECT:
            STR_TO_WSTR("DrawRect", szBuf, sizeof(szBuf));
            break;
         case USAGE_DRAWCIRCLE:
            STR_TO_WSTR("DrawCircle", szBuf, sizeof(szBuf));
            break;
         case USAGE_DRAWARC:
            STR_TO_WSTR("DrawArc", szBuf, sizeof(szBuf));
            break;
         case USAGE_DRAWELLIPSE:
            STR_TO_WSTR("DrawEllipse", szBuf, sizeof(szBuf));
            break;
         case USAGE_DRAWTRIANGLE:
            STR_TO_WSTR("DrawTriangle", szBuf, sizeof(szBuf));
            break;
         case USAGE_DRAWPOLYGON:
            STR_TO_WSTR("DrawPolygon", szBuf, sizeof(szBuf));
            break;
         case USAGE_DRAWPOLYLINE:
            STR_TO_WSTR("DrawPolyline", szBuf, sizeof(szBuf));
            break;
         case USAGE_CLEARRECT:
            STR_TO_WSTR("CrearRect", szBuf, sizeof(szBuf));
            break;
         case USAGE_ENABLEDOUBLEBUFFER:
            STR_TO_WSTR("EnableDoubleBuffer", szBuf, sizeof(szBuf));
            break;
         case USAGE_SWAP:
            STR_TO_WSTR("Swap", szBuf, sizeof(szBuf));
            break;
         case USAGE_TRANSLATE:
            STR_TO_WSTR("Translate", szBuf, sizeof(szBuf));
            break;
         case USAGE_PAN:
            STR_TO_WSTR("Pan", szBuf, sizeof(szBuf));
            break;
         case USAGE_STRETCH:
            STR_TO_WSTR("SBlt", szBuf, sizeof(szBuf));
            break;
         default:
            STR_TO_WSTR("Unsupported fn", szBuf, sizeof(szBuf));
      }
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,5, 5, &rc,
		   IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);

      // Functions with no output
      switch (wParam)
      {
         case USAGE_SETBACKGROUND:
         case USAGE_GETBACKGROUND:
         case USAGE_GETCOLORDEPTH:
         case USAGE_GETCOLOR:
         case USAGE_GETFILLMODE:
         case USAGE_GETFILLCOLOR:
         case USAGE_GETPOINTSIZE:
         case USAGE_GETCLIP:
         case USAGE_GETWINDOW:
         case USAGE_GETPAINTMODE:
         case USAGE_ENABLEDOUBLEBUFFER:
         case USAGE_SWAP:
         case USAGE_TRANSLATE:
         case USAGE_PAN:
 
            // Place in the middle of screen
			 if(pStr[0] == NULL)
			 {
            STR_TO_WSTR("No Output...", szBuf, sizeof(szBuf));
 		      IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,10, 
               rc.dy/2 - 10, &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
			 }
			 else
			 {
			   IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL,pStr, -1,10, 
               rc.dy/2 - 10, &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
			 }
         default:
            break;
      }

      STR_TO_WSTR("Press Up/Down", szBuf, sizeof(szBuf));
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,5, rc.dy - 
         2*(pMe->m_nLineHeight) , &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
      STR_TO_WSTR("keys to continue..", szBuf, sizeof(szBuf));
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,5, rc.dy - 
         pMe->m_nLineHeight, &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
		IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);
   }
}

/*===========================================================================

FUNCTION: BuildMainMenu

DESCRIPTION
    This function builds the main menu when this applet is started.
    It displays a simple menu with menu items corresponding to 
    each of the interface functions.
    
PROTOTYPE:
   void BuildMainMenu(CIGraphicsApp *pMe)

PARAMETERS:
   pMe: [in]: Pointer to CIGraphicsApp struct

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void BuildMainMenu(CIGraphicsApp *pMe)
{
	AEERect rc;
   AECHAR szBuf[100];

   if (pMe->m_pIMenu == NULL || pMe->a.m_pIShell == NULL)
      return;

   // Set display rectangle
   SETAEERECT (&rc, 0, 0, pMe->m_dInfo.cxScreen, pMe->m_dInfo.cyScreen);

   // Set Title
   STR_TO_WSTR("IGraphics Examples", szBuf, sizeof(szBuf));
	IMENUCTL_SetTitle(pMe->m_pIMenu, NULL, 0, szBuf);

	//Set the Rectangle for the Menu
	IMENUCTL_SetRect(pMe->m_pIMenu, &rc);  

	//Add invidual entries to the Menu

   // Add Basic Usage to menu
   STR_TO_WSTR("1. Basic Usage", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_BASIC_USAGE, szBuf, 0);

   // Add SetBackground to menu
   STR_TO_WSTR("2. SetBackground", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETBACKGROUND, szBuf, 0);
			
   // Add SetColor to menu
   STR_TO_WSTR("3. SetColor", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETCOLOR, szBuf, 0);

   // Add SetFillMode to menu
   STR_TO_WSTR("4. SetFillMode", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETFILLMODE, szBuf, 0);

	// Add SetFillColor to menu
   STR_TO_WSTR("5. SetFillColor", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETFILLCOLOR, szBuf, 0);

	// Add SetPointSize to menu
   STR_TO_WSTR("6. SetPointSize", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPOINTSIZE, szBuf, 0);	

	// Add SetClip to menu
   STR_TO_WSTR("7. SetClip", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETCLIP, szBuf, 0);	

	// Add SetWindow to menu
   STR_TO_WSTR("8. SetWindow", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETWINDOW, szBuf, 0);	

	// Add SetPaintMode to menu
   STR_TO_WSTR("9. SetPaintMode", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPAINTMODE, szBuf, 0);	

   // Add GetBackground to menu
   STR_TO_WSTR("10. GetBackground", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETBACKGROUND, szBuf, 0);	

   // Add GetColorDepth to menu
   STR_TO_WSTR("11. GetColorDepth", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETCOLORDEPTH, szBuf, 0);	

   // Add GetColor to menu
   STR_TO_WSTR("12. GetColor", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETCOLOR, szBuf, 0);	

	// Add GetFillMode to menu
   STR_TO_WSTR("13. GetFillMode", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETFILLMODE, szBuf, 0);	

	// Add GetFillColor to menu
   STR_TO_WSTR("14. GetFillColor", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETFILLCOLOR, szBuf, 0);	

	// Add GetPointSize to menu
   STR_TO_WSTR("15. GetPointSize", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETPOINTSIZE, szBuf, 0);	

	// Add GetClip to menu
   STR_TO_WSTR("16. GetClip", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETCLIP, szBuf, 0);	

	// Add GetWindow to menu
   STR_TO_WSTR("17. GetWindow", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETWINDOW, szBuf, 0);	

	// Add GetPaintMode to menu
   STR_TO_WSTR("18. GetPaintMode", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETPAINTMODE, szBuf, 0);	

	// Add DrawPoint to menu
   STR_TO_WSTR("19. DrawPoint", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DRAWPOINT, szBuf, 0);	

	// Add DrawLine to menu
   STR_TO_WSTR("20. DrawLine", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DRAWLINE, szBuf, 0);	

	// Add DrawRect to menu
   STR_TO_WSTR("21. DrawRect", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DRAWRECT, szBuf, 0);	

	// Add DrawCircle to menu
   STR_TO_WSTR("22. DrawCircle", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DRAWCIRCLE, szBuf, 0);	

	// Add DrawArc to menu
   STR_TO_WSTR("23. DrawArc", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DRAWARC, szBuf, 0);	

	// Add DrawEllipse to menu
   STR_TO_WSTR("24. DrawEllipse", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DRAWELLIPSE, szBuf, 0);	

	// Add DrawTriangle to menu
   STR_TO_WSTR("25. DrawTriangle", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DRAWTRIANGLE, szBuf, 0);	

	// Add DrawPolygon to menu
   STR_TO_WSTR("26. DrawPolygon", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DRAWPOLYGON, szBuf, 0);	

	// Add DrawPolyline to menu
   STR_TO_WSTR("27. DrawPolyline", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DRAWPOLYLINE, szBuf, 0);	

	// Add ClearRect to menu
   STR_TO_WSTR("28. ClearRect", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_CLEARRECT, szBuf, 0);	

	// Add EnableDoubleBuffer to menu
   STR_TO_WSTR("29. EnableDoubleBuffer", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_ENABLEDOUBLEBUFFER, szBuf, 0);	

	// Add Swap to menu
   STR_TO_WSTR("30. Swap", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SWAP, szBuf, 0);	

	// Add Translate to menu
   STR_TO_WSTR("31. Translate", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_TRANSLATE, szBuf, 0);	

	// Add Pan to menu
   STR_TO_WSTR("32. Pan", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_PAN, szBuf, 0);	

   STR_TO_WSTR("33. SBlt", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_STRETCH, szBuf, 0);	

   // Activate menu
	IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);
}
