/*===========================================================================

FILE: iimageusage.c

SERVICES: Sample applet using AEE

DESCRIPTION
  This file contains usage examples of the IImage interface. IImage interface
  functions allow the user to display bitmap objects. 

PUBLIC CLASSES:  
   N/A


   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/

/*===============================================================================
                     INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
// Mandatory includes
#include "AEEAppGen.h"	// AEEApplet declaration
#include "AEEUsageAppIDs.h"  // Class IDs of usage apps (including this one)

#include "AEE.h"			// Standard AEE Declarations
#include "AEEShell.h"	// AEE Shell Services
#include "AEEDisp.h"		// AEE Display Services
#include "AEEStdLib.h"  // AEE StdLib Services

// Services used by app
#include "AEEFile.h"		//	AEEFile Services
#include "AEEMenu.h"		// Menu Services

// App resource header generated by BREW ResourceEditor.
#include "image_res.h"

/*===========================================================================

                      PUBLIC DATA DECLARATIONS

===========================================================================*/

/*-------------------------------------------------------------------
            Type Declarations
-------------------------------------------------------------------*/
// IImage app struct. This is the main struct for this applet. This will hold 
// all the data members that need to be remembered throughout the life of
// the applet. 
// THE FIRST DATA MEMBER OF THIS STRUCT MUST BE AN AEEApplet OBJECT. 
// This is a critical requirement which is related to an applet being
// dynamically loaded. The app developers should add their data members
// following the AEEApplet data member.
typedef struct _CIImageApp {
   AEEApplet	      a;
   IMenuCtl *  m_pIMenu;
   IImage *	  m_pIImage;
} CIImageApp;

/*-------------------------------------------------------------------
            Function Prototypes
-------------------------------------------------------------------*/
// Mandatory function:
// App Handle Event function
static boolean ImageApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);

// App specific data alloc-init/free functions
static boolean IImage_InitAppData(IApplet* pMe);
static void IImage_FreeAppData(IApplet* pMe);

static void BuildMainMenu(CIImageApp *pMe);
static void IImageUsage (CIImageApp *pMe, uint16 wParam);
static void DisplayEvent (CIImageApp *pMe, uint16 wParam);
static void DisplayOutput(CIImageApp * pMe, int nline, char *pszStr);

/*-------------------------------------------------------------------
            Global Constant Definitions
-------------------------------------------------------------------*/
// App Resource File
#define  APP_RES_FILE           "image.bar"

#define  SAMPLE_BITMAP_FILE     "sample.bmp"

// App specific constants
#define USAGE_BASIC_USAGE			  100
#define USAGE_DRAW                    101
#define USAGE_DRAWFRAME               102
#define USAGE_GETINFO                 103
#define USAGE_SETDATA                 104
#define USAGE_SETPARM_SIZE            105
#define USAGE_SETPARM_OFFSET          106
#define USAGE_SETPARM_CXFRAME         107
#define USAGE_SETPARM_ROP_OR          108
#define USAGE_SETPARM_ROP_NOT         109
#define USAGE_SETPARM_ROP_XOR         110
#define USAGE_SETPARM_ROP_COPY        111
#define USAGE_SETPARM_ROP_TRANSPARENT 112
#define USAGE_SETPARM_ROP_MASK        113
#define USAGE_SETPARM_ROP_MERGENOT    114
#define USAGE_SETPARM_ROP_MASKNOT     115
#define USAGE_SETPARM_NFRAMES         116
#define USAGE_SETPARM_RATE            117
#define USAGE_SETPARM_OFFSCREEN       118
#define USAGE_START                   119
#define USAGE_STOP                    120
#define USAGE_ERASE_SCREEN            121   // Special case to erase screen
#define USAGE_SETSTREAM               122

// Size of buffer used to hold text strings
#define TEXT_BUFFER_SIZE (100 * sizeof(AECHAR))

/*===============================================================================
                     FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to IImage_InitAppData(). This is done as follows:
   IImage_InitAppData() is called to initialize CIImageApp instance. It is app developer's 
   responsibility to fill-in app data initialization code in IImage_InitAppData(). App developer
   is also responsible to release memory allocated for data contained in CIImageApp -- this can be
   done in IImage_FreeAppData().

PROTOTYPE:
	boolean AEEClsCreateInstance(AEECLSID clsID, IShell* pIShell, IModule* pIModule, 
										  void** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell interface. 

	pIModule: [in]: Contains pointer to the IModule interface to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid IApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app needs to be loaded and if AEEApplet_New() invocation was successful
  FALSE: If the app does not need to be loaded or if errors occurred in AEEApplet_New().
  If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId, IShell* pIShell, IModule* po, void** ppObj)
{
   *ppObj = NULL;
		
   // We want to load this App. So, invoke AEEApplet_New().To it, pass the
   // address of the app-specific handle event function.
   if(ClsId == AEECLSID_IMAGE_APP){
      if(AEEApplet_New(sizeof(CIImageApp), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)ImageApp_HandleEvent,(PFNFREEAPPDATA)IImage_FreeAppData))
      {
         if (IImage_InitAppData((IApplet*)*ppObj))
             return(AEE_SUCCESS);
      }
   }
	return (EFAILED);
}

/*===========================================================================

FUNCTION ImageApp_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All apps must supply an Event Handler.

PROTOTYPE:
	boolean ImageApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the IApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

	ecode: Specifies the Event sent to this applet

    wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/

static boolean ImageApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
   CIImageApp * pMe = (CIImageApp*)pi;

   // Make sure the pointers we'll be using are valid
   if (pMe == NULL || pMe->a.m_pIShell == NULL)
	   return FALSE;

   switch (eCode) 
	{

        case EVT_APP_START:

			// Build the main menu from which the user
			// selects usage examples to execute
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, (void **)&pMe->m_pIMenu))
				return TRUE;

			BuildMainMenu(pMe);

			return(TRUE);

        case EVT_APP_STOP:

			// Exit--all data and interface freeing are done in
			// the IImage_FreeAppData function
            return(TRUE);

        case EVT_KEY: 

			// In the usage example for IIMAGE_Stop (example 21), we allow the user to
			// stop an animated bitmap by pressing the '*' key
			if (wParam == AVK_STAR && pMe->m_pIImage != NULL)
			{
				IIMAGE_Stop (pMe->m_pIImage);
				return TRUE;
			}

			// If any other key has been pressed, user wishes to return to 
			// main usage app menu -- if there is an animated bitmap image 
			// running, release it

			if (pMe->m_pIImage)
			{
				IIMAGE_Release (pMe->m_pIImage);
				pMe->m_pIImage = NULL;
			}
			// Re-activate main menu and pass it the key event
                
			IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);
			if(IMENUCTL_HandleEvent(pMe->m_pIMenu, EVT_KEY, wParam, 0))
				return TRUE;
            else return FALSE; 

		case EVT_COMMAND:
  		    switch(wParam)
			{
			    case USAGE_BASIC_USAGE:
                case USAGE_DRAW:
                case USAGE_DRAWFRAME:
                case USAGE_GETINFO:
                case USAGE_SETDATA:
                case USAGE_SETPARM_SIZE:
                case USAGE_SETPARM_OFFSET:
                case USAGE_SETPARM_CXFRAME:
                case USAGE_SETPARM_ROP_OR:
                case USAGE_SETPARM_ROP_NOT:
                case USAGE_SETPARM_ROP_XOR:
                case USAGE_SETPARM_ROP_COPY:
                case USAGE_SETPARM_ROP_TRANSPARENT:
                case USAGE_SETPARM_ROP_MASK:
                case USAGE_SETPARM_ROP_MERGENOT:
                case USAGE_SETPARM_ROP_MASKNOT:
                case USAGE_SETPARM_NFRAMES:
                case USAGE_SETPARM_RATE:
                case USAGE_SETPARM_OFFSCREEN:
                case USAGE_START:
                case USAGE_STOP:
                case USAGE_SETSTREAM:

				   // De-activate main menu to turn off
				   // scrolling menu items
				   IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);

				   // Execute the usage example the user has selected
                   IImageUsage (pMe, wParam);
			       return TRUE;

			    default:
				   return FALSE;
			}
        default:
            break;
   }
   return TRUE;
}



/*===========================================================================

FUNCTION IImage_InitAppData

DESCRIPTION
   This function initializes app-specific data.

PROTOTYPE:
	boolean IImage_InitAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

DEPENDENCIES
   Assumes pi is not NULL

RETURN VALUE
  TRUE: If the app data is initialized successfully
  FALSE: Otherwise

SIDE EFFECTS
  none
===========================================================================*/
static boolean IImage_InitAppData(IApplet* pi)
{
   CIImageApp * pMe = (CIImageApp*)pi;

   pMe->m_pIImage = NULL;

   return TRUE;
}

/*===========================================================================

FUNCTION IImage_FreeAppData

DESCRIPTION
   Frees any app-specific data and releases any interfaces the app may be using
   when it is terminated.

PROTOTYPE:
	static void IImage_FreeAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

DEPENDENCIES
   Assumes pi is not NULL

RETURN VALUE
  None.

SIDE EFFECTS
  None.
===========================================================================*/
static void IImage_FreeAppData(IApplet* pi)
{
   CIImageApp * pMe = (CIImageApp*)pi;

   // If there is an image interface allocated for an animated bitmap, release it
   if (pMe->m_pIImage != NULL)
   {
      IIMAGE_Release (pMe->m_pIImage);
      pMe->m_pIImage = NULL;
   }

   // Release the IMenuCtl instance used to display the main app menu
   if (pMe->m_pIMenu != NULL)
   {
      IMENUCTL_Release(pMe->m_pIMenu);
      pMe->m_pIImage = NULL;
   }
}

/*===============================================================================
                     APP HELPER FUNCTION DEFINITIONS
=============================================================================== */

static void IImageUsage (CIImageApp* pMe, uint16 wParam)
{
   AEEDeviceInfo di; // Device Info
   AEERect rect;
   AEERect rc; 
   AEEImageInfo aii;

   char szResFile[] = APP_RES_FILE;   // Resource File used by this applet

   IImage * pIImage;

   char szBuf[30];

   // Screen coordinates for example displays
   uint16 img1_x = 0, img1_y = 0, img2_x = 0, img2_y = 0, img3_x = 0, img3_y = 0;

   // Make sure the pointers we'll be using are valid
   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
	   return;

   // Get screen size information for display purposes.
   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);

   // Set rectangle coordinates for a rectangle in the middle of the screen.
   rect.x = 10;
   rect.y = di.cyScreen /3;
   rect.dx = di.cxScreen - 5;
   rect.dy = di.cyScreen/3;

   // Rectangle used for displaying text messages on screen
   rc.x = 5;
   rc.y = 5;
   rc.dx = di.cxScreen - 5;
   rc.dy = di.cyScreen - 5;

   // Erase screen first for display output purposes.
   DisplayEvent (pMe, USAGE_ERASE_SCREEN);

   switch (wParam)
   {
      case USAGE_BASIC_USAGE:
         {
            // 1. Usage: Basic Image Usage

			// This example illustrates the basic steps involved in drawing
			// non-animated and animated bitmaps.  We will first draw the non-animated
			// browser icon bitmap in the upper center of the screen, using a raster
			// operation to display it in reverse video.  We will them display the
			// animated browser icon in the bottom center.

			// Read in the bitmap that is to be drawn
			// NOTE: We don't use CreateInstance to get an
			// instance of the IImage interface -- the resource file access
			// function returns a pointer to an instance of IImage
			pIImage = ISHELL_LoadResImage(pMe->a.m_pIShell, szResFile, IDB_BROWSER_ICON);
			if(pIImage)
			{
				// Retrieve bitmap information, including its pixel dimensions
				// (these will be stored in aii.cx and aii.cy)
				IIMAGE_GetInfo(pIImage, &aii);

			    // Calculate coordinates for non-animated bitmap -- 
			    // we'll display it centered in the upper portion
			    // of the screen
			    img1_x = (di.cxScreen - aii.cx) / 2;
			    img1_y = 10 + ((di.cyScreen - 2 * aii.cy - 30) / 3);

				// Set the raster operation to NOT, which will complement
				// the pixels of the image
				IIMAGE_SetParm(pIImage, IPARM_ROP, AEE_RO_NOT, 0);

			    // Draw the non-animated bitmap at the specified coordinates
				IIMAGE_Draw(pIImage,img1_x,img1_y);

				// Don't need the bitmap any more, so release the interface 
				// (pixels will remain on the screen until something else overwrites them).
				IIMAGE_Release (pIImage);
            }
            else break;

			// Now read in the animated bitmap.  Notice that we use the IImage pointer
			// that is part of the applet data structure, since this pointer will be accessed
			// by other parts of the program (see code below).
			pMe->m_pIImage = ISHELL_LoadResImage(pMe->a.m_pIShell, szResFile, IDB_BROWSER_ICON_ANIM);
			if(pMe->m_pIImage)
			{
				// The animated browser icon has four frames (104 x 26 pixels, with each frame
				// 26 x 26).  You must specify the number of frames before you can start 
				// the animation.
				IIMAGE_SetParm(pMe->m_pIImage, IPARM_NFRAMES, 4, 0);

				// Begin displaying the animated bitmap.  Animation is implemented with timers:
				// a timer whose length is the inter-frame duration is started, and its callback
				// function displays one frame of the image and starts another timer with the 
				// same duration and callback function.
				// NOTE: The interface pointer must not be released until
				// the animation is to be stopped.  We do this when the user returns to
				// the main menu (EVT_KEY) or exits the test app (EVT_APP_STOP) (see code
				// in the ImageApp_HandleEvent and IImage_FreeAppData functions).  If you don't 
				// release the IImage interface, the animation will continue to be drawn on the 
				// screen after you've exited this application.		
				IIMAGE_GetInfo(pMe->m_pIImage, &aii);
				IIMAGE_Start(pMe->m_pIImage, img1_x, img1_y + aii.cy + 5);
			}

			// Update the display so the operations above will take effect
			IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);
         }
         break;

      case USAGE_DRAW:
         {
            // 2. Usage: IIMAGE_Draw

			// This function draws a (non-animated) bitmap on the screen.  In this example, we
			// retrieve an example bitmap from our resource file and use IIMAGE_Draw to draw it
			// on the screen.  Note that once the bitmap's pixels are drawn on the screen, we don't
			// need the IImage interface that contains it any more, so we can immediately free the
			// interface instance.

			// Read in the bitmap that is to be drawn
			pIImage = ISHELL_LoadResImage(pMe->a.m_pIShell, szResFile, IDB_LOWER_LEFT_BLACK);
			if(pIImage)
			{
				// Retrieve bitmap information, including its pixel dimensions
				// (these will be stored in aii.cx and aii.cy)
				IIMAGE_GetInfo(pIImage, &aii);

                // Calculate coordinates for bitmap -- 
			    // we'll display it centered on the screen
			    img1_x = (di.cxScreen - aii.cx) / 2;
			    img1_y = (di.cyScreen - aii.cy) / 2;

			    // Draw the bitmap at the specified coordinates
				IIMAGE_Draw(pIImage,img1_x,img1_y);

				// Don't need the bitmap any more, so release the interface 
				IIMAGE_Release (pIImage);
			}

			// Update the display so the operations above will take effect
			IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);
         }
         break;

      case USAGE_DRAWFRAME:
         {
            // 3. Usage: IIMAGE_DrawFrame

			// IIMAGE_DrawFrame draws a single frame of an animated image to the screen.
			// The displayed image frame is not animated.  Before calling this function, you
			// must set the image's NFRAMES or CXFRAME parameter to indicate that it is an
			// animated bitmap.  In this example, we use IIMAGE_DrawFrame to draw one frame 
			// of the animated browser icon bitmap.
			
			// Read in the animated browser icon bitmap
			pIImage = ISHELL_LoadResImage(pMe->a.m_pIShell, szResFile, IDB_BROWSER_ICON_ANIM);
			if(pIImage)
			{
				// Retrieve bitmap information, including its pixel dimensions
				// (these will be stored in aii.cx and aii.cy)
				IIMAGE_GetInfo(pIImage, &aii);

			    // Calculate coordinates for frame display -- 
			    // we'll display the bitmap centered on the screen
				// (adjust x coordinate since this is a 4-frame animated image)
			    img1_x = (di.cxScreen - (aii.cx / 4)) / 2;
			    img1_y = (di.cyScreen - aii.cy) / 2;

				// The animated browser icon has four frames -- must
				// specify the number of frames before we can draw a particular frame
				IIMAGE_SetParm(pIImage, IPARM_NFRAMES, 4, 0);

				// Draw frame number 1 (frame numbering starts at zero)
				IIMAGE_DrawFrame(pIImage, 1, img1_x, img1_y);

				// Don't need the bitmap any more, so release the interface 
				IIMAGE_Release (pIImage);
			}
			IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);
         }
         break;

      case USAGE_GETINFO:
         {
            // 4. Usage: IIMAGE_GetInfo

			// In this example, we'll use the IIMAGE_GetInfo function
			// to retrieve information about a bitmap.  Some of the
			// information retrieved about the bitmap will be displayed 
			// on the screen.

			// Read in the browser icon bitmap
			pIImage = ISHELL_LoadResImage(pMe->a.m_pIShell, szResFile, IDB_BROWSER_ICON);
			if(pIImage)
			{
				// Retrieve bitmap information using GetInfo
				IIMAGE_GetInfo(pIImage, &aii);

				// Echo the info that was returned 
				// by GetInfo in the AEEImageInfo structure

				// bitmap width in pixels
                SPRINTF(szBuf, "cx: %d cy: %d", aii.cx, aii.cy); 
                DisplayOutput(pMe, 2, szBuf);

				// number of colors (2 for monochrome diplay)
                SPRINTF(szBuf, "nColors: %d", aii.nColors); 
                DisplayOutput(pMe, 3, szBuf);

				// Don't need the bitmap any more, so release the interface 
				IIMAGE_Release (pIImage);
			}
			IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);
         }
         break;

  /*    case USAGE_SETDATA:
         {
            // 5. Usage: IIMAGE_SetData 

			// This example illustrates how bitmap files can be read into
			// memory directly, without storing them in BREW resource files.
            IFileMgr *  pIFileMgr;
            IFile *     pIFile;
            byte *      pBuff;
            FileInfo    fi;
			AEECLSID cls;

			// Create an instance of IImage so we can populate it with
		    // info about the .bmp file we will read into memory.
		    // NOTE:CreateInstance is not used directly for IImage--we must determine the
		    // class ID of the handler function for .bmp's and then
		    // create an instance of this BREW class
		    if (0 == (cls = ISHELL_GetHandler(pMe->a.m_pIShell, HTYPE_VIEWER, "image/bmp")))
				break;

            ISHELL_CreateInstance(pMe->a.m_pIShell, cls, (void **)(&pIImage));
            if(NULL == pIImage)
				break;

			// Create an instance of IFileMgr so we can access the .bmp file
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR, (void **)(&pIFileMgr));
			if (NULL == pIFileMgr)
			{
				IIMAGE_Release(pIImage);
				break;
            }

			// Open the browser icon bitmap file for reading
            if (NULL == (pIFile = IFILEMGR_OpenFile(pIFileMgr, "BrowserIcon.bmp", _OFM_READ)))
			{
				IIMAGE_Release(pIImage);
				IFILEMGR_Release(pIFileMgr);
				break;
            }

            // Determine the size of the bitmap file and allocate memory
			// to store its contents
			IFILE_GetInfo(pIFile, &fi);
            if (NULL == (pBuff = (byte *) MALLOC(fi.dwSize)))
			{
				IIMAGE_Release(pIImage);
				IFILE_Release(pIFile);
				IFILEMGR_Release(pIFileMgr);
				break;
            }

            // Read the contents of the .bmp file into the
		    // area allocated in memory for it
			if (IFILE_Read(pIFile, (void *)pBuff, fi.dwSize) == 0)
			{
				IIMAGE_Release(pIImage);
				FREE(pBuff);
				IFILE_Release(pIFile);
				IFILEMGR_Release(pIFileMgr);
				break;
            }				
			
            // Use SetData to associate the .bmp file read into memory
			// with the IImage instance we've created
			// (NOTE: the third parameter to SetData indicates if the .bmp
			// file has MIME header info, FALSE in this case)
			if(IIMAGE_SetData(pIImage, pBuff, FALSE))
			{
				// Retrieve bitmap information, including its pixel dimensions
				// (these will be stored in aii.cx and aii.cy)
				IIMAGE_GetInfo(pIImage, &aii);

                // Calculate coordinates for bitmap -- 
			    // we'll display it centered on the screen
			    img1_x = (di.cxScreen - aii.cx) / 2;
			    img1_y = (di.cyScreen - aii.cy) / 2;

			    // Now draw the image
				IIMAGE_Draw(pIImage,img1_x,img1_y);
			}
            else FREE(pBuff);

			// Don't need the bitmap any more, so release the interface 
		    // (this will also free the data allocated by MALLOC).
            IIMAGE_Release (pIImage);
            IFILE_Release(pIFile);
            IFILEMGR_Release(pIFileMgr);
         }
         break;
		*/
      case USAGE_SETPARM_SIZE:
         {
            // 6. Usage: IIMAGE_SetParm -- IPARM_SIZE

			// Setting IPARM_SIZE allows you to display a portion of a bitmap
			// by specifying new x and y pixel dimensions for it.  Since the 
			// bitmap's origin (upper left corner) is unaffected, reducing the 
			// x dimension truncates the right side of the image, and reducing
			// the y dimension truncates the bottom of the image.
			// In the example below, we reduce the x and y dimensions of a bitmap
			// by 25% each, displaying the original image at the top of the
			// screen and the truncated image at the bottom.

			// Read in the bitmap that is to be drawn
			pIImage = ISHELL_LoadResImage(pMe->a.m_pIShell, szResFile, IDB_LOWER_RIGHT_BLACK);
			if(pIImage)
			{
				// Retrieve bitmap information, including its pixel dimensions
				// (these will be stored in aii.cx and aii.cy)
				IIMAGE_GetInfo(pIImage, &aii);

				// Calculate coordinates for bitmap -- 
			    // we'll display it centered on the 
			    // upper portion of the screen 
			    img1_x = (di.cxScreen - aii.cx) / 2;
			    img1_y = 10 + ((di.cyScreen - 2 * aii.cy - 30) / 3);

			    // Draw original bitmap in upper center of screen
				IIMAGE_Draw(pIImage,img1_x,img1_y);

				// Now set the size of the bitmap to a smaller value --
				// approx 75% of its original value
				IIMAGE_SetParm(pIImage, IPARM_SIZE, aii.cx - (aii.cx / 4), aii.cy - (aii.cy / 4));

				// Draw the truncated bitmap in the lower center of the screen
                IIMAGE_Draw(pIImage,img1_x,img1_y + aii.cy + 5);

				// Don't need the bitmap any more, so release the interface 
				IIMAGE_Release (pIImage);
			}

			// Update the display so the operations above will take effect
			IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);

         }
         break; 
		 
      case USAGE_SETPARM_OFFSET:
         {
            // 7. Usage: IIMAGE_SetParm -- IPARM_OFFSET

			// Setting IPARM_OFFSET allows you to display a portion of a bitmap
			// by specifying a new origin (upper left corner) for it.  Since the 
			// bitmap's default origin is (0, 0), increasing the 
			// x origin truncates the left side of the image, and increasing
			// the y origin truncates the top of the image.
			// In the example below, we change the origin coordinates to 25% of the height 
			// and width of the image, displaying the original image at the top of the
			// screen and the truncated image at the bottom.

			// Read in the bitmap that is to be drawn
			pIImage = ISHELL_LoadResImage(pMe->a.m_pIShell, szResFile, IDB_LOWER_RIGHT_BLACK);
			if(pIImage)
			{
				// Retrieve bitmap information, including its pixel dimensions
				// (these will be stored in aii.cx and aii.cy)
				IIMAGE_GetInfo(pIImage, &aii);

			    // Calculate coordinates for bitmap -- 
			    // we'll display the original bitmap centered on the 
			    // upper portion of the screen initially
			    img1_x = (di.cxScreen - aii.cx) / 2;
			    img1_y = 10 + (di.cyScreen - 2 * aii.cy - 30) / 3;

			    // Draw original bitmap in upper center of screen
				IIMAGE_Draw(pIImage,img1_x,img1_y);

				// Now set the offset of the bitmap to a value
				// that differs from the default (0,0) --
				// "slice" 25% from top and left side of image
				IIMAGE_SetParm(pIImage, IPARM_OFFSET, aii.cx / 4, aii.cy / 4);

				// Draw the truncated bitmap in the lower center of the screen
                IIMAGE_Draw(pIImage,img1_x,img1_y + aii.cy + 5);

				// Don't need the bitmap any more, so release the interface 
				IIMAGE_Release (pIImage);
			}

			// Update the display so the operations above will take effect
			IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);
         }
         break;

      case USAGE_SETPARM_CXFRAME:
         {
            // 8. Usage: IIMAGE_SetParm -- IPARM_CXFRAME

			// To display an animated bitmap image, you must first specify the number
			// of frames it contains, either by setting IPARM_CXFRAME or IPARM_NFRAMES
			// (see example 9 below).  Frames are laid out along the x dimension of the
			// bitmap, so an animated bitmap with four 26 x 26 frames is 104 pixels wide and
			// 26 pixels high.  The specified frame width must evenly divide the total bitmap
			// width (if not, the function below sets the width to the closest multiple).

			// Read in the animated "bouncing squares" bitmap
			pMe->m_pIImage = ISHELL_LoadResImage(pMe->a.m_pIShell, szResFile, IDB_BOUNCING_SQUARES);
			if(pMe->m_pIImage)
			{
				// Retrieve bitmap information, including its pixel dimensions
				// (these will be stored in aii.cx and aii.cy)
				IIMAGE_GetInfo(pMe->m_pIImage, &aii);

			    // Calculate coordinates for animation display -- 
			    // we'll display the bitmap centered on the screen
				// (adjust x coordinate since this is an 8-frame animated image)
			    img1_x = (di.cxScreen - (aii.cx / 8)) / 2;
			    img1_y = (di.cyScreen - aii.cy) / 2;

				// The animated bouncing squares image has eight frames, each 10 pixels wide -- must
				// specify the frame width or number of frames (see example 9 below) 
				// before we can start the animation
				IIMAGE_SetParm(pMe->m_pIImage, IPARM_CXFRAME, 10, 0);

				// Begin displaying the animated bitmap.
				// NOTE: The interface pointer must not be released until
				// the animation is to be stopped.  We do this when the user returns to
				// the main menu (EVT_KEY) or stops the test app (EVT_APP_STOP) (see code
				// in the ImageApp_HandleEvent and IImage_FreeAppData functions).
				IIMAGE_Start(pMe->m_pIImage, img1_x, img1_y);
			}
			IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);
         }
         break;

      case USAGE_SETPARM_NFRAMES:
         {
            // 9. Usage: IIMAGE_SetParm -- IPARM_NFRAMES

		    // To display an animated bitmap image, you must first specify the number
			// of frames it contains, either by setting IPARM_CXFRAME (see example 8 above)
			// or IPARM_NFRAMES.  Frames are laid out along the x dimension of the
			// bitmap, so an animated bitmap with four 26 x 26 frames is 104 pixels wide and
			// 26 pixels high.  The specified frame width must evenly divide the total bitmap
			// width (if not, the function below sets the width to the closest multiple).

			// Read in the animated "bouncing squares" bitmap
			pMe->m_pIImage = ISHELL_LoadResImage(pMe->a.m_pIShell, szResFile, IDB_BOUNCING_SQUARES);
			if(pMe->m_pIImage)
			{
				// Retrieve bitmap information, including its pixel dimensions
				// (these will be stored in aii.cx and aii.cy)
				IIMAGE_GetInfo(pMe->m_pIImage, &aii);

			    // Calculate coordinates for animation display -- 
			    // we'll display the bitmap centered on the screen
				// (adjust x coordinate since this is an 8-frame animated image)
			    img1_x = (di.cxScreen - (aii.cx / 8)) / 2;
			    img1_y = (di.cyScreen - aii.cy) / 2;

				// The animated bouncing squares image has eight frames, each 10 pixels wide -- must
				// specify the frame width (see example 8 above) or number of frames 
				// before we can start the animation
				IIMAGE_SetParm(pMe->m_pIImage, IPARM_NFRAMES, 8, 0);

				// Begin displaying the animated bitmap.
				// NOTE: The interface pointer must not be released until
				// the animation is to be stopped.  We do this when the user returns to
				// the main menu (EVT_KEY) or stops the test app (EVT_APP_STOP) (see code
				// in the ImageApp_HandleEvent and IImage_FreeAppData functions).
				IIMAGE_Start(pMe->m_pIImage, img1_x, img1_y);
			}
			IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);
         }
         break;

      case USAGE_SETPARM_OFFSCREEN:
          {
              // 10. Usage: IIMAGE_SetParm -- IPARM_OFFSCREEN

			  // NOT IMPLEMENTED -- Setting this parameter not supported
			  // in this SDK release
          }
         break;

      case USAGE_SETPARM_RATE:
         {
            // 11. Usage: IIMAGE_SetParm -- IPARM_RATE

			// Bitmap animation is implemented by setting a timer whose expiration
			// causes one frame of the image to be drawn on the screen and the setting 
			// of another timer that triggers the drawing of the next frame (animation
			// continues until it is stopped with IIMAGE_Stop or the IImage interface
			// is released).  The animation rate is the duration of the inter-frame
			// timers; its default value is 150 ms.  Below, the animation rate is slowed
			// down by setting the timer value to 750 ms.

			// Read in the animated "bouncing squares" bitmap
			pMe->m_pIImage = ISHELL_LoadResImage(pMe->a.m_pIShell, szResFile, IDB_BOUNCING_SQUARES);
			if(pMe->m_pIImage)
			{
				// Retrieve bitmap information, including its pixel dimensions
				// (these will be stored in aii.cx and aii.cy)
				IIMAGE_GetInfo(pMe->m_pIImage, &aii);

			    // Calculate coordinates for animation display -- 
			    // we'll display the bitmap centered on the screen
				// (adjust x coordinate since this is an 8-frame animated image)
			    img1_x = (di.cxScreen - (aii.cx / 8)) / 2;
			    img1_y = (di.cyScreen - aii.cy) / 2;

				// The animated bouncing squares image has eight frames, each 10 pixels wide --
				// must specify the frame width or number of frames 
				// before we can start the animation -- we choose the latter
				IIMAGE_SetParm(pMe->m_pIImage, IPARM_NFRAMES, 8, 0);

				// Set the animation rate to a new value (default is 150 ms between successive
				// frames, so chosen value is five times slower; see examples 8 and 9)
                IIMAGE_SetParm(pMe->m_pIImage, IPARM_RATE, 750, 0);

				// Begin displaying the animated bitmap.
				// NOTE: The interface pointer must not be released until
				// the animation is to be terminated.  We do this when the user returns to
				// the main menu (EVT_KEY) or stops the test app (EVT_APP_STOP) (see code
				// in the HandleEvent function).
				IIMAGE_Start(pMe->m_pIImage, img1_x, img1_y);
			}
			IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);
         }
         break;

		 //     The next seven examples illustrate the use of raster operations.  These
		 // operations define how the pixels currently on the screen (the destination 
		 // bitmap) are to be logically combined with those of a new image that is to be drawn on
		 // that portion of the screen (the source bitmap).  The supported raster operations
		 // and the Boolean pixel operations they represent are as follows:
		 //
		 //    OR   -- src | dest
		 //    NOT  -- !src
		 //    XOR  -- src ^ dest
		 //    COPY -- src
		 //    TRANSPARENT -- src
		 //    MASK        -- src & dest
		 //    MASKNOT     -- !src & dest
		 //    MERGENOT    -- !src | dest
		 //
		 //     The flow of each raster operation example is as follows:
		 //
		 //        1 The destination bitmap is drawn in the upper left portion of the 
		 //          screen, and the source bitmap in the upper right portion (this
		 //          allows the user to see what each bitmap looks like before they
		 //          are combined with a raster operation).
		 //        2 The destination bitmap is drawn to a region in the center of the lower
		 //          portion of the screen.
		 //        3 Using IImage_SetParm, the raster operation to be illustrated is set.
		 //        4 The source bitmap is drawn to the same region of the lower screen used for
		 //          the destination bitmap in step 2.  This allows the user to see the results of
		 //          combining the source and destination images with the selected
		 //          raster operation.

         case USAGE_SETPARM_ROP_OR:
         case USAGE_SETPARM_ROP_NOT:
	     case USAGE_SETPARM_ROP_XOR:
	     case USAGE_SETPARM_ROP_COPY:
	     case USAGE_SETPARM_ROP_TRANSPARENT:
	     case USAGE_SETPARM_ROP_MASK:
	     case USAGE_SETPARM_ROP_MASKNOT:
	     case USAGE_SETPARM_ROP_MERGENOT:
         {
            // 12. Usage: IIMAGE_SetParm -- IPARM_ROP -- OR
			// 13. Usage: IIMAGE_SetParm -- IPARM_ROP -- NOT
            // 14. Usage: IIMAGE_SetParm -- IPARM_ROP -- XOR
			// 15. Usage: IIMAGE_SetParm -- IPARM_ROP -- COPY
			// 16. Usage: IIMAGE_SetParm -- IPARM_ROP -- TRANSPARENT
			// 17. Usage: IIMAGE_SetParm -- IPARM_ROP -- MASK
			// 18. Usage: IIMAGE_SetParm -- IPARM_ROP -- MERGENOT
			// 19. Usage: IIMAGE_SetParm -- IPARM_ROP -- MASKNOT

			// Read in the destination bitmap
			pIImage = ISHELL_LoadResImage(pMe->a.m_pIShell, szResFile, IDB_LOWER_LEFT_BLACK);
			if(pIImage)
			{
				// Retrieve dest bitmap information using GetInfo
				IIMAGE_GetInfo(pIImage, &aii);
				
				// Calculate coordinates for dest bitmap in upper left portion of screen
			    img1_x = (di.cxScreen - 2 * aii.cx) / 3;
			    img1_y = (((di.cyScreen / 2) - aii.cy) / 2) + 7;

                // Calculate coordinates for dest ROP src bitmap in center of lower portion of screen
			    img3_x = (di.cxScreen - aii.cx) / 2;
			    img3_y = (img1_y + di.cyScreen / 2) - 17;

			    // Draw dest bitmap in upper left portion of screen
				IIMAGE_Draw(pIImage,img1_x,img1_y);

				// Draw dest bitmap in lower portion of screen
				IIMAGE_Draw(pIImage,img3_x,img3_y);

				// Don't need the dest bitmap any more, so release the interface 
				IIMAGE_Release (pIImage);
			}
            else break;

			// Read in the source bitmap
			pIImage = ISHELL_LoadResImage(pMe->a.m_pIShell, szResFile, IDB_LOWER_RIGHT_BLACK);
			if(pIImage)
			{
				// Retrieve source bitmap information using GetInfo
				IIMAGE_GetInfo(pIImage, &aii);

				// Calculate coordinates for source bitmap in upper right portion of screen
			    img2_x = 2 * img1_x + aii.cx;
			    img2_y = img1_y;

			    // Draw src bitmap in upper right portion of screen
				IIMAGE_Draw(pIImage,img2_x,img2_y);

                // Set the raster operation that will be used to combine the pixels 
				// of the dest and src bitmaps when the src bitmap is written to lower
				// half of the screen
				switch(wParam)
				{
		           case USAGE_SETPARM_ROP_OR:
                       IIMAGE_SetParm(pIImage, IPARM_ROP, AEE_RO_OR, 0);
					   break;

                   case USAGE_SETPARM_ROP_NOT:
                       IIMAGE_SetParm(pIImage, IPARM_ROP, AEE_RO_NOT, 0);
					   break;

	               case USAGE_SETPARM_ROP_XOR:
				       IIMAGE_SetParm(pIImage, IPARM_ROP, AEE_RO_XOR, 0);
					   break;

	               case USAGE_SETPARM_ROP_COPY:
				       IIMAGE_SetParm(pIImage, IPARM_ROP, AEE_RO_COPY, 0);
					   break;

	               case USAGE_SETPARM_ROP_TRANSPARENT:
				       IIMAGE_SetParm(pIImage, IPARM_ROP, AEE_RO_TRANSPARENT, 0);
					   break;

	               case USAGE_SETPARM_ROP_MASK:
				       IIMAGE_SetParm(pIImage, IPARM_ROP, AEE_RO_MASK, 0);
					   break;

	               case USAGE_SETPARM_ROP_MASKNOT:
				       IIMAGE_SetParm(pIImage, IPARM_ROP, AEE_RO_MERGENOT, 0);
					   break;

	               case USAGE_SETPARM_ROP_MERGENOT:
				       IIMAGE_SetParm(pIImage, IPARM_ROP, AEE_RO_MASKNOT, 0);
					   break;
				}

				// Draw src bitmap in lower portion of screen (same region where
				// dest bitmap was drawn above).  The raster op set above will be used
				// to combine the pixels of the src and dest bitmaps.
				IIMAGE_Draw(pIImage,img3_x,img3_y);

				// Don't need the src bitmap any more, so release the interface 
				IIMAGE_Release (pIImage);
			}
            
			// Update the display so the operations above will take effect
			IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);
         }
         break;

      case USAGE_START:
         {
            // 20. Usage: IIMAGE_Start

			// Read in the animated browser icon bitmap
			pMe->m_pIImage = ISHELL_LoadResImage(pMe->a.m_pIShell, szResFile, IDB_BROWSER_ICON_ANIM);
			if(pMe->m_pIImage)
			{
				// Retrieve pixel dimensions of the animated bitmap
				IIMAGE_GetInfo(pMe->m_pIImage, &aii);

			    // Calculate coordinates for animation display -- 
			    // we'll display the bitmap centered on the screen
				// (adjust x coordinate to reflect that bitmap is 4 frames wide)
			    img1_x = (di.cxScreen - (aii.cx / 4)) / 2;
			    img1_y = (di.cyScreen - aii.cy) / 2;

				// The animated browser icon has four frames -- must
				// specify the number of frames before we can start the animation
				IIMAGE_SetParm(pMe->m_pIImage, IPARM_NFRAMES, 4, 0);

				// Begin displaying the animated bitmap.
				// NOTE: The interface pointer must not be released until
				// the animation is to be stopped.  We do this when the user returns to
				// the main menu (EVT_KEY) or stops the test app (EVT_APP_STOP) (see code
				// in the HandleEvent function)
				IIMAGE_Start(pMe->m_pIImage, img1_x, img1_y);
			}
			IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);
         }
         break;

		case USAGE_STOP:
         {
            // 21. Usage: IIMAGE_Stop

			// This example illustrates the use of the IIMAGE_Stop function.  
            // It is the same as example 21, except that the user can stop
			// the animation by pressing the '*' key.  Note that the actual
			// call to IIMAGE_Stop is in ImageApp_HandleEvent
			// (see code in that function for event EVT_KEY, wParam == AVK_STAR). 
			// NOTE: When the '*' key is pressed, the last displayed frame of the 
			// animated bitmap remains on the screen.
			int charHeight, pnAscent, pnDescent, nLines;

            // Print message instructing the user to press the '*' key
			// to stop the animation.  We want this message to appear at
			// the bottom of the screen, below the bitmap image.  First, 
			// calculate the number of lines on the screen so we can figure out
		    // where the bottom is.
		    charHeight = IDISPLAY_GetFontMetrics (pMe->a.m_pIDisplay, AEE_FONT_NORMAL,
                &pnAscent, &pnDescent);

            // Number of lines equals the available screen
            // space divided by the height of a line, minus 2 for
			// the message "Press up/down keys to exit" that appears at 
			// the bottom of the screen.
            nLines = (di.cyScreen / charHeight) - 2;

            // Print message instructing the user to press the '*' key
			// to stop the animation
            DisplayOutput(pMe, nLines - 2, "Press * key to");
            DisplayOutput(pMe, nLines - 1, "stop animation");

			// Read in the animated browser icon bitmap
			pMe->m_pIImage = ISHELL_LoadResImage(pMe->a.m_pIShell, szResFile, IDB_BROWSER_ICON_ANIM);
			if(pMe->m_pIImage)
			{
				// Retrieve pixel dimensions of the animated bitmap
				IIMAGE_GetInfo(pMe->m_pIImage, &aii);

			    // Calculate coordinates for animation display -- 
			    // we'll display the bitmap centered on the screen
				// (adjust x coordinate to reflect that bitmap is 4 frames wide)
			    img1_x = (di.cxScreen - (aii.cx / 4)) / 2;
			    img1_y = 10 + ((di.cyScreen - aii.cy - 50) / 2);

				// The animated browser icon has four frames -- must
				// specify the number of frames before we can start the animation
				IIMAGE_SetParm(pMe->m_pIImage, IPARM_NFRAMES, 4, 0);

				// Begin displaying the animated bitmap.
				// NOTE: The interface pointer must not be released until
				// the animation is to be terminated.  We do this when the user returns to
				// the main menu (up/down menu scroll (EVT_KEY)) or stops the test app 
				// (EVT_APP_STOP) (see code in the HandleEvent function)
				IIMAGE_Start(pMe->m_pIImage, img1_x, img1_y);
			}
			IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);

         }
         break;

      case USAGE_SETSTREAM:
         {
            // 22. Usage: IIMAGE_SetStream

			// This function draws a (non-animated) bitmap on the screen.  In this example, we
			// retrieve an example bitmap from our resource file and use IIMAGE_Draw to draw it
			// on the screen.  Note that once the bitmap's pixels are drawn on the screen, we don't
			// need the IImage interface that contains it any more, so we can immediately free the
			// interface instance.

            AEEImageInfo aii;
            int icx, icy;
            AEECLSID cls;
            FileInfo fi;
            AEEDeviceInfo di;

            IImage *pIImage = NULL;
            IFileMgr *pIFileMgr = NULL;
            byte* pBuff = NULL;
            IFile* pIFile = NULL;
            IMemAStream *pMemStream = NULL;

            ISHELL_GetDeviceInfo(pMe->a.m_pIShell, &di);

   			// Create an instance of IImage so we can populate it with
		    // info about the .bmp file we will read into memory.
		    // NOTE:CreateInstance is not used directly for IImage--we must determine the
		    // class ID of the handler function for .bmp's and then
		    // create an instance of this BREW class
		    cls = ISHELL_GetHandler(pMe->a.m_pIShell, HTYPE_VIEWER, "image/bmp");

            if (cls)
                ISHELL_CreateInstance(pMe->a.m_pIShell, cls, (void **)(&pIImage));

            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR, (void **)(&pIFileMgr));

            if(pIImage && pIFileMgr)
            {
                // Open the bitmap file for reading, save handle
                pIFile = IFILEMGR_OpenFile(pIFileMgr, SAMPLE_BITMAP_FILE, _OFM_READ);

                if (pIFile)
                {
                    // Determine the size of the bitmap file and allocate memory
                    // to store its contents
                    IFILE_GetInfo(pIFile, &fi);
                    pBuff = (byte *) MALLOC(fi.dwSize);

                    if (pBuff)
                    {
                        // Read .bmp into memory
                        if (IFILE_Read(pIFile, (void *)pBuff, fi.dwSize))
                        {
                            //Create MEMSTREAM and use it as stream for IImage.
                            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MEMASTREAM, (void **)(&pMemStream));
                            if (pMemStream)
                            {
                                IMEMASTREAM_Set(pMemStream, pBuff, fi.dwSize, 0, 0);
                                IIMAGE_SetStream(pIImage, (IAStream*)pMemStream);

                                // Retrieve bitmap information, including its pixel dimensions
                                // (these will be stored in aii.cx and aii.cy)
                                IIMAGE_GetInfo(pIImage, &aii);

                                // Display on centre of screen
                                icx = (di.cxScreen - aii.cx) / 2;
                                icy = (di.cyScreen - aii.cy) / 2;

                                IIMAGE_Draw(pIImage, icx, icy);
                                IDISPLAY_UpdateEx(pMe->a.m_pIDisplay, TRUE);
                            }
                        }
                    }
                }
            }
            if (pIFile)
                IFILE_Release(pIFile);
            if (pIImage)
                IIMAGE_Release(pIImage);
            if (pIFileMgr)
                IFILEMGR_Release(pIFileMgr);
            if (pMemStream)
                IMEMASTREAM_Release(pMemStream);
            if (pBuff)
                FREE(pBuff);
        }
        break;

    default:
         return;  
   }
            
   // Display above event. 
   DisplayEvent (pMe, wParam);
    
   return;
}

static void DisplayEvent (CIImageApp* pMe, uint16 wParam)
{
   AEERect qrc;
   AEEDeviceInfo di; // Device Info
   AECHAR * szBuf;
   AEERect rc; 

   // Make sure the pointers we'll be using are valid
   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
	   return;

   if ((szBuf = (AECHAR *) MALLOC(TEXT_BUFFER_SIZE * sizeof(AECHAR))) == NULL)
	   return;

   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);
   qrc.x = 0;
   qrc.y = 0;
   qrc.dx = di.cxScreen;
   qrc.dy = di.cyScreen;
   rc.x = 5;
   rc.y = 5;
   rc.dx = di.cxScreen - 5;
   rc.dy = di.cyScreen - 5;
   
   switch (wParam)
   {
      case USAGE_ERASE_SCREEN:
         IDISPLAY_EraseRect(pMe->a.m_pIDisplay,&qrc);  //Clear the display
		    FREE(szBuf);
            return;
      case USAGE_BASIC_USAGE:
         STR_TO_WSTR("Basic Usage", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_DRAW:
         STR_TO_WSTR("Draw", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_DRAWFRAME:
         STR_TO_WSTR("DrawFrame", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_GETINFO:
         STR_TO_WSTR("GetInfo", szBuf, TEXT_BUFFER_SIZE);
         break;
     // case USAGE_SETDATA:
       //  STR_TO_WSTR("SetData", szBuf, TEXT_BUFFER_SIZE);
      //   break;
      case USAGE_SETPARM_SIZE:
         STR_TO_WSTR("SetParm Size", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_SETPARM_OFFSET:
         STR_TO_WSTR("SetParm Offset", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_SETPARM_CXFRAME:
         STR_TO_WSTR("SetParm CxFrame", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_SETPARM_NFRAMES:
         STR_TO_WSTR("SetParm NFrames", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_SETPARM_OFFSCREEN:
         STR_TO_WSTR("SetParm Offscreen", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_SETPARM_RATE:
         STR_TO_WSTR("SetParm Rate", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_SETPARM_ROP_OR:
         STR_TO_WSTR("ROP OR", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_SETPARM_ROP_NOT:
         STR_TO_WSTR("ROP NOT", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_SETPARM_ROP_XOR:
         STR_TO_WSTR("ROP XOR", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_SETPARM_ROP_COPY:
         STR_TO_WSTR("ROP COPY", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_SETPARM_ROP_TRANSPARENT:
         STR_TO_WSTR("ROP TRANSPARENT", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_SETPARM_ROP_MASK:
         STR_TO_WSTR("ROP MASK", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_SETPARM_ROP_MERGENOT:
         STR_TO_WSTR("ROP MERGENOT", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_SETPARM_ROP_MASKNOT:
         STR_TO_WSTR("ROP MASKNOT", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_START:
         STR_TO_WSTR("Start Animation", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_STOP:
         STR_TO_WSTR("Stop Animation", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_SETSTREAM:
         STR_TO_WSTR("Set Stream", szBuf, TEXT_BUFFER_SIZE);
         break;
      default:
         STR_TO_WSTR("Unsupported fn", szBuf, TEXT_BUFFER_SIZE);
   }
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,10, 5, &rc,
		   IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);

   // Functions with no output
   switch (wParam)
   {
	  // case USAGE_BASIC_USAGE:
      // case USAGE_DRAW:
      // case USAGE_DRAWFRAME:
      // case USAGE_GETINFO:
      // case USAGE_SETDATA:
      // case USAGE_SETPARM_SIZE:
      // case USAGE_SETPARM_OFFSET:
      // case USAGE_SETPARM_CXFRAME:
      // case USAGE_SETPARM_ROP_OR:
      // case USAGE_SETPARM_ROP_NOT:
      // case USAGE_SETPARM_ROP_XOR:
      // case USAGE_SETPARM_ROP_COPY:
      // case USAGE_SETPARM_ROP_TRANSPARENT:
      // case USAGE_SETPARM_ROP_MASK:
      // case USAGE_SETPARM_ROP_MERGENOT:
      // case USAGE_SETPARM_ROP_MASKNOT:
      // case USAGE_SETPARM_NFRAMES:
      // case USAGE_SETPARM_RATE:
      case USAGE_SETPARM_OFFSCREEN:  // not supported in current release
      // case USAGE_START:
      // case USAGE_STOP:
         // Place in the middle of screen
         STR_TO_WSTR("No Output...", szBuf, TEXT_BUFFER_SIZE);
 		   IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,10, 
            qrc.dy/2 - 10, &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
      default:
         break;
   }

   STR_TO_WSTR("Press any", szBuf, TEXT_BUFFER_SIZE);
   IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,10, qrc.dy - 25, &rc,
	  IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
   STR_TO_WSTR("key to continue..", szBuf, TEXT_BUFFER_SIZE);
   IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,10, qrc.dy - 15, &rc,
	  IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
   IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);
   FREE(szBuf);
}

static void BuildMainMenu(CIImageApp *pMe)
{
	AEERect qrc;
	AEEDeviceInfo di;
    AECHAR * szBuf;

   // Make sure the pointers we'll be using are valid
   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->m_pIMenu == NULL)
	   return;

   if ((szBuf = (AECHAR *) MALLOC(TEXT_BUFFER_SIZE * sizeof(AECHAR))) == NULL)
	   return;

	// Set Title
    STR_TO_WSTR("IImage Examples", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_SetTitle(pMe->m_pIMenu, NULL, 0, szBuf);

	//Set the Rectangle for the Menu
	ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);
	qrc.x = 0;
	qrc.y = 0;
	qrc.dx = di.cxScreen;
	qrc.dy = di.cyScreen;
	IMENUCTL_SetRect(pMe->m_pIMenu, &qrc);  

	//Add individual entries to the Menu

    // Add Basic Usage to menu
    STR_TO_WSTR("1. Basic Usage", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_BASIC_USAGE, szBuf, 0);

    // Add Draw to menu
    STR_TO_WSTR("2. Draw", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DRAW, szBuf, 0);
			
    // Add DrawFrame to menu
    STR_TO_WSTR("3. DrawFrame", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DRAWFRAME, szBuf, 0);

    // Add GetInfo to menu
    STR_TO_WSTR("4. GetInfo", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETINFO, szBuf, 0);

	// Add SetData to menu
    //STR_TO_WSTR("5. SetData", szBuf, TEXT_BUFFER_SIZE);
	//IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETDATA, szBuf, 0);

	// Add SetParm Size to menu
    STR_TO_WSTR("5. SetParm SIZE", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPARM_SIZE, szBuf, 0);	

	// Add SetParm Offset to menu
    STR_TO_WSTR("6. SetParm Offset", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPARM_OFFSET, szBuf, 0);	

	// Add SetParm CxFrame to menu
    STR_TO_WSTR("7. SetP CxFrame", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPARM_CXFRAME, szBuf, 0);	

	// Add SetParm NFrames to menu
    STR_TO_WSTR("8. SetP NFrames", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPARM_NFRAMES, szBuf, 0);	

    // Add SetParm Offscreen to menu
    STR_TO_WSTR("9. SetP Offscreen", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPARM_OFFSCREEN, szBuf, 0);	

    // Add SetParm Rate to menu
    STR_TO_WSTR("10. SetParm Rate", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPARM_RATE, szBuf, 0);	

    // Add SetParm ROP OR to menu
    STR_TO_WSTR("11. ROP OR", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPARM_ROP_OR, szBuf, 0);	

	// Add SetParm ROP NOT to menu
    STR_TO_WSTR("12. ROP NOT", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPARM_ROP_NOT, szBuf, 0);	

	// Add SetParm ROP XOR to menu
    STR_TO_WSTR("13. ROP XOR", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPARM_ROP_XOR, szBuf, 0);	

	// Add SetParm ROP COPY to menu
    STR_TO_WSTR("14. ROP COPY", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPARM_ROP_COPY, szBuf, 0);	

	// Add SetParm ROP TRANSPARENT to menu
    STR_TO_WSTR("15. ROP TRANSP'T", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPARM_ROP_TRANSPARENT, szBuf, 0);	

	// Add SetParm ROP MASK to menu
    STR_TO_WSTR("16. ROP MASK", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPARM_ROP_MASK, szBuf, 0);	

	// Add SetParm ROP MERGENOT to menu
    STR_TO_WSTR("17. ROP MRGNOT", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPARM_ROP_MERGENOT, szBuf, 0);	

	// Add SetParm ROP MASKNOT to menu
    STR_TO_WSTR("18. ROP MASKNOT", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPARM_ROP_MASKNOT, szBuf, 0);	

	// Add Start to menu
    STR_TO_WSTR("19. Start", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_START, szBuf, 0);	

	// Add SetActiveDayMask to menu
    STR_TO_WSTR("20. Stop", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_STOP, szBuf, 0);
    
	// Add SetActiveDayMask to menu
    STR_TO_WSTR("21. Set Stream", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETSTREAM, szBuf, 0);	

    // Activate menu
	IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);

	FREE(szBuf);
}
/*===========================================================================

FUNCTION: DisplayOutput

DESCRIPTION
    This function displays an output string at a given line number on the
    screen. If the nline parameter is a negative value (-1) the string
    is displayed in the middle of the screen. If the "nline" value is larger
    than or equal to zero the "nline" value is multiplied by 15 and the 
    resulting value in pixels is set to the y-coordinate of the start of 
    the string display on the screen. If the string does not fit on one line
    the string wraps around to the next line (spaced rougly 10-15 pixels apart).
    By default 5 is used as the starting the x-coordinate of a displayed 
    string.

    How many characters that fit on one line is calculated for each line 
    that is wrapped around to the next line.

    Note: depending on the phone screen size and the fonts used for characters 
          the output might differ on different handsets (devices). Where some 
          handsets will have a smaller screen and large default fonts which will 
          cause partial overlapping of lines. This function does not try to address
          these issues (this is meant as a simple display function).
    
PROTOTYPE:
   void DisplayOutput(IShell *pIShell, IDisplay *pDisplay, int nline, char *pszStr)

PARAMETERS:
   pIShell:   [in]: Contains a pointer to the IShell interface.
   pIDisplay: [in]: Contains a pointer to the IDisplay interface.
   nline:     [in]: Contains the line number to start displaying the text. The line
        numbers are by default spaced 15 pixels apart along the y-axis.
   pszStr:    [in]: The character string to be displayed on the screen.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void DisplayOutput(CIImageApp * pMe, int nline, char *pszStr)
{
   AEEDeviceInfo di; // Device Info
   AECHAR * szBuf;      
   AECHAR * psz = NULL;
   int charHeight = 0;      // Stores the char height in pixels for given font
   int pnAscent = 0;        // Stores the ascent in number of pixels
   int pnDescent = 0;       // Stores the descent in number of pixels
   int pixelWidth;
   AEEFont font = AEE_FONT_NORMAL;
   int pnFits = 0, dy;
   int totalCh = 0;
   IShell *pIShell = pMe->a.m_pIShell;
   IDisplay *pIDisplay = pMe->a.m_pIDisplay;

   // Make sure the pointers we'll be using are valid
   if (pIShell == NULL || pIDisplay == NULL)
	   return;

   if ((szBuf = (AECHAR *) MALLOC(TEXT_BUFFER_SIZE * sizeof(AECHAR))) == NULL)
	   return;

   // Get device information
   ISHELL_GetDeviceInfo(pIShell,&di);

   // Get the font metrics info
   charHeight = IDISPLAY_GetFontMetrics (pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   // Convert to wide string (unicode)
   STR_TO_WSTR ((char *)pszStr, szBuf, TEXT_BUFFER_SIZE);

   // If nlines is zero then print this string starting around the middle of 
   // the screen. Or else multiply nlines by charheight to decide the y coordinate of
   // the start of the string.
   if (nline < 0) {
      dy = di.cyScreen*2/5;
   }
   else{
      dy = nline * charHeight;
   }

   // psz keeps track of the point from which to write from the string buffer
   // in case the string does not fit one line and needs to wrap around in the
   // next line.
   psz = szBuf;
      
   // Need to calculate the lotal string length to decide if any wrapping
   // around is needed.
   totalCh = STRLEN ((char *)pszStr);

   // Keep displaying text string on multiple lines if the string can't be displayed
   // on one single line. Lines are spaced 15 pixels apart.
   while ((totalCh > 0) && (*psz != NULL))
   { 
      // Get information on how many characters will fit in a line.
      // Give the pointer to the buffer to be displayed, and the number of
      // pixels along the x axis you want to display the string in (max number)
      // pnFits will have the max number of chars that will fit in the maxWidth
      // number of pixels (given string can't fit in one line), or the number of 
      // chars in the string (if it does fit in one line). pnWidth gives the
      // number of pixels that will be used to display pnFits number of chars.
      pixelWidth = IDISPLAY_MeasureTextEx(pIDisplay,
                      font, 
                      (AECHAR *) psz,  // Start of the buffer to display,
                      -1,
                      di.cxScreen - 5, // maxWidth
                      &pnFits);         // Number of chars that will fit a line

      // If pnFits is zero there is something wrong in the input to above function. 
      // Normally this scenario should not occur. But, have the check anyway.
      if (pnFits == 0)
	  {
		 FREE(szBuf);
         return;
      }

      IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, psz, pnFits, 5 /*start dx*/, 
         dy, 0 /* use default rectangle coordinates */, 0);

      psz += pnFits;      // move pointer to the next segment to be displayed
      totalCh -= pnFits;  // reduce the total number of characters to still display
      dy += charHeight;   // Place next line charHeight pixels below the 
                          // previous line.
      IDISPLAY_Update(pIDisplay); //, TRUE);
      if (totalCh < pnFits)
         pnFits = totalCh;  // if total number is less than pnFits, adjust pnFits
   }
   FREE(szBuf);
   return;   
} // End of DisplayOutput
