/*===========================================================================

FILE: ilicenseusage.c

SERVICES: Sample applet using AEE

DESCRIPTION
  This file contains usage examples of the ILicense interface. ILicense interface
  functions allow the user to control usage. 
  Use vaious MIF files to see various behaviors. Be careful, they will be mdoified.
  Always copy the MIF to the MIF directory and retain your original in the applet's
  directory. You also need ot rename it to ilicense.mif.
  Example Setup:
      Copy ilicense_10days.mif file to your MIF directory.
      Rename ilicense_10days.mif to ilicense.mif.
      Run as you normally would.
  You can do this for the other files, please DO NOT LEAVE multiple ilicense
  MIF files in the same MIF directory, as BREW may pick the wrong one since they
  have the same CLSID.

PUBLIC CLASSES:  
   N/A


   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/

/*===============================================================================
                     INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
// Mandatory includes
#include "AEEAppGen.h"	// AEEApplet declaration
#include "AEEUsageAppIDs.h"  // Class IDs of usage apps (including this one)

#include "AEE.h"			// Standard AEE Declarations
#include "AEEShell.h"	// AEE Shell Services
#include "AEEDisp.h"		// AEE Display Services
#include "AEEStdLib.h"  // AEE StdLib Services
#include "AEELicense.h" // AEE License Services
#include "AEEMenu.h"    // AEE Menu Services

/*===========================================================================

                      PUBLIC DATA DECLARATIONS

===========================================================================*/
#define  LICENSE_INCREMENT    100   // increment # of uses
#define  LICENSE_DECREMENT    101   // decrement # of uses
#define  LICENSE_SHOWUSES     102   // Displays # of uses
#define  LICENSE_ADDFIVE      103   // increment # of uses by 5
#define  LICENSE_REMFIVE      104   // decrement # of uses by 5
#define  LICENSE_EXPIRED      105   // Is Expired?
#define  LICENSE_DATE         106   // Date expires
#define  LICENSE_DAYS         107   // Days left until expiration
#define  LICENSE_MINUSE       108   // Minutes of use remaining
#define  LICENSE_GETUSE       109   // Get Info!


/*-------------------------------------------------------------------
            Type Declarations
-------------------------------------------------------------------*/
// IImage app struct. This is the main struct for this applet. This will hold 
// all the data members that need to be remembered throughout the life of
// the applet. 
// THE FIRST DATA MEMBER OF THIS STRUCT MUST BE AN AEEApplet OBJECT. 
// This is a critical requirement which is related to an applet being
// dynamically loaded. The app developers should add their data members
// following the AEEApplet data member.
typedef struct _CLicenseApp {
   AEEApplet	      a;
   ILicense *	      m_pLicense;
   IMenuCtl *        m_pIMenu;
} CLicenseApp;

/*-------------------------------------------------------------------
            Function Prototypes
-------------------------------------------------------------------*/
// Mandatory function:
// App Handle Event function
static boolean    LicenseApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);
static boolean    LicenseApp_CommandHandler(CLicenseApp * pMe, uint16 wParam, uint32 dwParam);
// App specific data alloc-init/free functions
static int        InitAppData(IApplet* pMe);
static void       FreeAppData(IApplet* pMe);

static void       BuildMenu(CLicenseApp * pMe);
static void       DisplayOutput(CLicenseApp * pMe, int nline, char *pszStr);

/*-------------------------------------------------------------------
            Global Constant Definitions
-------------------------------------------------------------------*/
// Size of buffer used to hold text strings
#define TEXT_BUFFER_SIZE (100 * sizeof(AECHAR))

/*===============================================================================
                     FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to InitAppData(). This is done as follows:
   InitAppData() is called to initialize CLicenseApp instance. It is app developer's 
   responsibility to fill-in app data initialization code in InitAppData(). App developer
   is also responsible to release memory allocated for data contained in CLicenseApp -- this can be
   done in FreeAppData().

PROTOTYPE:
	boolean AEEClsCreateInstance(AEECLSID clsID, IShell* pIShell, IModule* pIModule, 
										  void** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell interface. 

	pIModule: [in]: Contains pointer to the IModule interface to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid IApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app needs to be loaded and if AEEApplet_New() invocation was successful
  FALSE: If the app does not need to be loaded or if errors occurred in AEEApplet_New().
  If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId, IShell* pIShell, IModule* po, void** ppObj)
{
   int   nErr  = 0;
   *ppObj = NULL;
		
   // We want to load this App. So, invoke AEEApplet_New().To it, pass the
   // address of the app-specific handle event function.
   if(ClsId == AEECLSID_LICENSE_APP){
      if(AEEApplet_New(sizeof(CLicenseApp), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)LicenseApp_HandleEvent,(PFNFREEAPPDATA)FreeAppData))
      {
         nErr  = InitAppData((IApplet*)*ppObj);
         if( nErr )
         {
            // BREW doesn't know about us, clean us up the applet.
            IAPPLET_Release(*ppObj);
            *ppObj = NULL;
            return nErr;
         }
         return SUCCESS;
      }
   }
	return EFAILED;
}

/*===========================================================================

FUNCTION LicenseApp_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All apps must supply an Event Handler.

PROTOTYPE:
	boolean LicenseApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the IApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

	ecode: Specifies the Event sent to this applet

    wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/

static boolean LicenseApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
   CLicenseApp * pMe = (CLicenseApp*)pi;

   // Make sure the pointers we'll be using are valid
   if (pMe == NULL || pMe->a.m_pIShell == NULL)
	   return FALSE;

   switch (eCode) 
	{

        case EVT_APP_START:
           // Builds the menu and shows licensing info
           // This menu varies depending on the license
           // type that the user buys
           BuildMenu(pMe);
           return(TRUE);

        case EVT_APP_STOP:

			// Exit--all data and interface freeing are done in
			// the FreeAppData function
            return(TRUE);

        case EVT_KEY: 
           if( pMe->m_pIMenu )
           {
              if( IMENUCTL_HandleEvent(pMe->m_pIMenu, EVT_KEY, wParam, 0) )
                 return TRUE;
              else 
                 return FALSE; 
           }
           break;
        case EVT_COMMAND:
           return LicenseApp_CommandHandler(pMe, wParam, dwParam);
           
        default:
            break;
   }
   return TRUE;
}



/*===========================================================================

FUNCTION InitAppData

DESCRIPTION
   This function initializes app-specific data.

PROTOTYPE:
	int InitAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

DEPENDENCIES
   Assumes pi is not NULL

RETURN VALUE
  EFAILED:     General error
  EEXPIRED:    Expired app
  SUCCESS:     Succesful

SIDE EFFECTS
  none
===========================================================================*/
static int InitAppData(IApplet* pi)
{
   CLicenseApp *  pMe         = (CLicenseApp*)pi;
   uint32         dwLicRemain = 0;

   if( !pMe )
      return EFAILED;

	if( ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_LICENSE, (void **)(&pMe->m_pLicense)) )
	   return EFAILED;
   if( ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, (void **)(&pMe->m_pIMenu)) )
   {
      ILICENSE_Release( pMe->m_pLicense );
      return EFAILED;
   }
   if( ILICENSE_GetInfo(pMe->m_pLicense, &dwLicRemain) == LT_USES && dwLicRemain == 0 )
      return EEXPIRED;

   return SUCCESS;
}

/*===========================================================================

FUNCTION FreeAppData

DESCRIPTION
   Frees any app-specific data and releases any interfaces the app may be using
   when it is terminated.

PROTOTYPE:
	static void FreeAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

DEPENDENCIES
   Assumes pi is not NULL

RETURN VALUE
  None.

SIDE EFFECTS
  None.
===========================================================================*/
static void FreeAppData(IApplet* pi)
{
   CLicenseApp * pMe = (CLicenseApp*)pi;

   if( !pMe )
      return;

   // Release the ILicense instance used to track usage
   if( pMe->m_pLicense )
   {
      ILICENSE_Release(pMe->m_pLicense);
      pMe->m_pLicense = NULL;
   }

   // Release the IMenuCtl instance used to display the main app menu
   if( pMe->m_pIMenu )
   {
      IMENUCTL_Release(pMe->m_pIMenu);
      pMe->m_pIMenu = NULL;
   }
}

/*===============================================================================
                     APP HELPER FUNCTION DEFINITIONS
=============================================================================== */
// This function handles menu commands.
static boolean LicenseApp_CommandHandler(CLicenseApp * pMe, uint16 wParam, uint32 dwParam)
{
   char *         pszOutput   = NULL;
   AEELicenseType licType     = 0;
   uint32         dwLicRemain = 0;
   AEERect        rct;
   AEEDeviceInfo  di;

   pszOutput = (char *) MALLOC(TEXT_BUFFER_SIZE);
   if( !pszOutput || !pMe )
      return FALSE;

   ISHELL_GetDeviceInfo(pMe->a.m_pIShell, &di);
   SETAEERECT(&rct, 0, 0, di.cxScreen, 15);
   IDISPLAY_FillRect(pMe->a.m_pIDisplay, &rct, RGB_NONE);

   // These HAndle menu commands
   switch( wParam )
   {
   case LICENSE_GETUSE:
      licType  =  ILICENSE_GetInfo(pMe->m_pLicense, &dwLicRemain);
      if( dwLicRemain == -1 )
         DisplayOutput(pMe, 0, "License Does not expire");
      else
         BuildMenu(pMe);
      if( pszOutput )
         FREE( pszOutput );
      return TRUE;
   case LICENSE_EXPIRED: // Check expired?
      if( ILICENSE_GetInfo(pMe->m_pLicense, &dwLicRemain) == LT_USES )
      {
         if( dwLicRemain )
            DisplayOutput(pMe, 0, "License Valid");
         else
            DisplayOutput(pMe, 0, "License Expired");
      }
      else
      {
         if( ILICENSE_IsExpired(pMe->m_pLicense) )
            DisplayOutput(pMe, 0, "License Expired");
         else
            DisplayOutput(pMe, 0, "License Valid");
      }

      if( pszOutput )
         FREE( pszOutput );
      return TRUE;
   case LICENSE_SHOWUSES: // show uses remaining
      licType  =  ILICENSE_GetInfo(pMe->m_pLicense, &dwLicRemain);
      if( licType != LT_USES )
         SPRINTF( pszOutput, "Wrong Mode");
      else
         SPRINTF( pszOutput, "%d uses left", dwLicRemain);
      DisplayOutput(pMe, 0, pszOutput);
      if( pszOutput )
         FREE( pszOutput );
      return TRUE;
   case LICENSE_INCREMENT: // increment uses
      licType  =  ILICENSE_GetInfo(pMe->m_pLicense, &dwLicRemain);
      if( licType != LT_USES )
         SPRINTF( pszOutput, "Wrong Mode");
      else
      {
         ILICENSE_IncrementUsesRemaining(pMe->m_pLicense);
         ILICENSE_GetInfo(pMe->m_pLicense, &dwLicRemain);
         SPRINTF( pszOutput, "%d uses left", dwLicRemain);
      }
      DisplayOutput(pMe, 0, pszOutput);
      if( pszOutput )
         FREE( pszOutput );
      return TRUE;
   case LICENSE_DECREMENT: // decrement uses
      licType  =  ILICENSE_GetInfo(pMe->m_pLicense, &dwLicRemain);
      if( licType != LT_USES )
         SPRINTF( pszOutput, "Wrong Mode");
      else
      {
         if( dwLicRemain )
         {
            ILICENSE_DecrementUsesRemaining(pMe->m_pLicense);
            ILICENSE_GetInfo(pMe->m_pLicense, &dwLicRemain);
            SPRINTF( pszOutput, "%d uses left", dwLicRemain);
         }
         else
            SPRINTF( pszOutput, "0 uses, Can't Decrement");
      }
      DisplayOutput(pMe, 0, pszOutput);
      if( pszOutput )
         FREE( pszOutput );
      return TRUE;
   case LICENSE_ADDFIVE: // Add five uses
      licType  =  ILICENSE_GetInfo(pMe->m_pLicense, &dwLicRemain);
      if( licType != LT_USES )
         SPRINTF( pszOutput, "Wrong Mode");
      else
      {
         ILICENSE_SetUsesRemaining(pMe->m_pLicense, dwLicRemain + 5);
         ILICENSE_GetInfo(pMe->m_pLicense, &dwLicRemain);
         SPRINTF( pszOutput, "%d uses left", dwLicRemain);
      }
      DisplayOutput(pMe, 0, pszOutput);
      if( pszOutput )
         FREE( pszOutput );
      return TRUE;
   case LICENSE_REMFIVE: // decrement five uses
      licType  =  ILICENSE_GetInfo(pMe->m_pLicense, &dwLicRemain);
      if( licType != LT_USES )
         SPRINTF( pszOutput, "Wrong Mode");
      else
      {
         if( dwLicRemain >= 5 )
         {
            ILICENSE_SetUsesRemaining(pMe->m_pLicense, dwLicRemain - 5);
            ILICENSE_GetInfo(pMe->m_pLicense, &dwLicRemain);
            SPRINTF( pszOutput, "%d uses left", dwLicRemain);
         }
         else
            SPRINTF( pszOutput, "%d uses, can't remove 5", dwLicRemain);
      }
      DisplayOutput(pMe, 0, pszOutput);
      if( pszOutput )
         FREE( pszOutput );
      return TRUE;
   default:
      break;
   }
   if( pszOutput )
      FREE( pszOutput );
   return FALSE;

}
// Function builds a menu to show available options based on the license type
static void BuildMenu(CLicenseApp * pMe)
{
   AEEDeviceInfo  di;
   AEERect        rct;
   uint32         dwLicRemain;
   AEELicenseType licType;
   char *         pszOutput;
   AECHAR *       pszWOutput;
   JulianType     jt;

   // params..
   if( !pMe )
      return;
   // Buffers
   pszOutput   = (char *)MALLOC( TEXT_BUFFER_SIZE );
   if( !pszOutput )
      return;

   pszWOutput  = (AECHAR *)MALLOC( TEXT_BUFFER_SIZE*2 );
   if( !pszWOutput )
   {
      FREE( pszOutput );
      return;
   }

   // Get Info for device and make the menu fill the lower and leave
   // ~1 line at top for output
   ISHELL_GetDeviceInfo(pMe->a.m_pIShell, &di);
   SETAEERECT(&rct, 0, 15, di.cxScreen, di.cyScreen -15);
   IMENUCTL_SetRect(pMe->m_pIMenu, &rct);

   // Get License Type by Getting info...
   licType  =  ILICENSE_GetInfo(pMe->m_pLicense, &dwLicRemain);
   switch( licType )
   {
      
   case LT_NONE: // Non-expiring application
      STRCPY( pszOutput, "App doesn't Expire");
      DisplayOutput(pMe, 0, pszOutput);
      STR_TO_WSTR("Is Expired?", pszWOutput, TEXT_BUFFER_SIZE*2);
      IMENUCTL_AddItem(pMe->m_pIMenu, NULL, 0, LICENSE_EXPIRED, pszWOutput, 0);
      break;
      
   case LT_USES: // Usage based
      // Add Menu Items
      if( dwLicRemain == -1 )
      {
         STRCPY( pszOutput, "App doesn't Expire");
         DisplayOutput(pMe, 0, pszOutput);
         STR_TO_WSTR("Is Expired?", pszWOutput, TEXT_BUFFER_SIZE*2);
         IMENUCTL_AddItem(pMe->m_pIMenu, NULL, 0, LICENSE_EXPIRED, pszWOutput, 0);
      }
      else
      {
         STR_TO_WSTR("ShowUses", pszWOutput, TEXT_BUFFER_SIZE*2);
         IMENUCTL_AddItem(pMe->m_pIMenu, NULL, 0, LICENSE_SHOWUSES, pszWOutput, 0);
         STR_TO_WSTR("Increment", pszWOutput, TEXT_BUFFER_SIZE*2);
         IMENUCTL_AddItem(pMe->m_pIMenu, NULL, 0, LICENSE_INCREMENT, pszWOutput, 0);
         STR_TO_WSTR("Decrement", pszWOutput, TEXT_BUFFER_SIZE*2);
         IMENUCTL_AddItem(pMe->m_pIMenu, NULL, 0, LICENSE_DECREMENT, pszWOutput, 0);
         STR_TO_WSTR("Add Five", pszWOutput, TEXT_BUFFER_SIZE*2);
         IMENUCTL_AddItem(pMe->m_pIMenu, NULL, 0, LICENSE_ADDFIVE, pszWOutput, 0);
         STR_TO_WSTR("Remove Five", pszWOutput, TEXT_BUFFER_SIZE*2);
         IMENUCTL_AddItem(pMe->m_pIMenu, NULL, 0, LICENSE_REMFIVE, pszWOutput, 0);
         STR_TO_WSTR("Get Info", pszWOutput, TEXT_BUFFER_SIZE*2);
         IMENUCTL_AddItem(pMe->m_pIMenu, NULL, 0, LICENSE_GETUSE, pszWOutput, 0);
         STR_TO_WSTR("Is Expired?", pszWOutput, TEXT_BUFFER_SIZE*2);
         IMENUCTL_AddItem(pMe->m_pIMenu, NULL, 0, LICENSE_EXPIRED, pszWOutput, 0);
         SPRINTF( pszOutput, "%d uses left", dwLicRemain);
         // Show our uses left
         DisplayOutput(pMe, 0, pszOutput);
      }
      break;
      
   case LT_DAYS: // Days left based
      if( dwLicRemain == -1 )
      {
         STRCPY( pszOutput, "App doesn't Expire");
         DisplayOutput(pMe, 0, pszOutput);
         STR_TO_WSTR("Is Expired?", pszWOutput, TEXT_BUFFER_SIZE*2);
         IMENUCTL_AddItem(pMe->m_pIMenu, NULL, 0, LICENSE_EXPIRED, pszWOutput, 0);
      }
      else
      {
         GETJULIANDATE(dwLicRemain, &jt);
         SPRINTF(pszOutput, "Expires: %d/%02d/%02d", jt.wYear, jt.wMonth, jt.wDay);
         DisplayOutput(pMe, 0, pszOutput);
         STR_TO_WSTR("Get Info", pszWOutput, TEXT_BUFFER_SIZE*2);
         IMENUCTL_AddItem(pMe->m_pIMenu, NULL, 0, LICENSE_GETUSE, pszWOutput, 0);
         STR_TO_WSTR("Is Expired?", pszWOutput, TEXT_BUFFER_SIZE*2);
         IMENUCTL_AddItem(pMe->m_pIMenu, NULL, 0, LICENSE_EXPIRED, pszWOutput, 0);
      }
      break;

   case LT_DATE: // Expiration date type
      if( dwLicRemain == -1 )
      {
         STRCPY( pszOutput, "App doesn't Expire");
         DisplayOutput(pMe, 0, pszOutput);
         STR_TO_WSTR("Is Expired?", pszWOutput, TEXT_BUFFER_SIZE*2);
         IMENUCTL_AddItem(pMe->m_pIMenu, NULL, 0, LICENSE_EXPIRED, pszWOutput, 0);
      }
      else
      {
         GETJULIANDATE(dwLicRemain, &jt);
         SPRINTF(pszOutput, "Expires: %d/%02d/%02d %dh%dm%ds", jt.wYear, jt.wMonth, jt.wDay, jt.wHour, jt.wMinute, jt.wSecond);
         DisplayOutput(pMe, 0, pszOutput);
         STR_TO_WSTR("Get Info", pszWOutput, TEXT_BUFFER_SIZE*2);
         IMENUCTL_AddItem(pMe->m_pIMenu, NULL, 0, LICENSE_GETUSE, pszWOutput, 0);
         STR_TO_WSTR("Is Expired?", pszWOutput, TEXT_BUFFER_SIZE*2);
         IMENUCTL_AddItem(pMe->m_pIMenu, NULL, 0, LICENSE_EXPIRED, pszWOutput, 0);
      }
      break;

   case LT_MINUTES_OF_USE: // Minutes remaining type
      if( dwLicRemain == -1 )
      {
         STRCPY( pszOutput, "App doesn't Expire");
         DisplayOutput(pMe, 0, pszOutput);
         STR_TO_WSTR("Is Expired?", pszWOutput, TEXT_BUFFER_SIZE*2);
         IMENUCTL_AddItem(pMe->m_pIMenu, NULL, 0, LICENSE_EXPIRED, pszWOutput, 0);
      }
      else
      {
         SPRINTF(pszOutput, "%d Minutes Left", dwLicRemain);
         DisplayOutput(pMe, 0, pszOutput);
         STR_TO_WSTR("Get Info", pszWOutput, TEXT_BUFFER_SIZE*2);
         IMENUCTL_AddItem(pMe->m_pIMenu, NULL, 0, LICENSE_GETUSE, pszWOutput, 0);
         STR_TO_WSTR("Is Expired?", pszWOutput, TEXT_BUFFER_SIZE*2);
         IMENUCTL_AddItem(pMe->m_pIMenu, NULL, 0, LICENSE_EXPIRED, pszWOutput, 0);
      }
      break;
   }
   // Activate it
   IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);

   if( pszOutput )
      FREE( pszOutput );
   if( pszWOutput )
      FREE( pszWOutput );
}
/*===========================================================================

FUNCTION: DisplayOutput

DESCRIPTION
    This function displays an output string at a given line number on the
    screen. If the nline parameter is a negative value (-1) the string
    is displayed in the middle of the screen. If the "nline" value is larger
    than or equal to zero the "nline" value is multiplied by 15 and the 
    resulting value in pixels is set to the y-coordinate of the start of 
    the string display on the screen. If the string does not fit on one line
    the string wraps around to the next line (spaced rougly 10-15 pixels apart).
    By default 5 is used as the starting the x-coordinate of a displayed 
    string.

    How many characters that fit on one line is calculated for each line 
    that is wrapped around to the next line.

    Note: depending on the phone screen size and the fonts used for characters 
          the output might differ on different handsets (devices). Where some 
          handsets will have a smaller screen and large default fonts which will 
          cause partial overlapping of lines. This function does not try to address
          these issues (this is meant as a simple display function).
    
PROTOTYPE:
   void DisplayOutput(IShell *pIShell, IDisplay *pDisplay, int nline, char *pszStr)

PARAMETERS:
   pIShell:   [in]: Contains a pointer to the IShell interface.
   pIDisplay: [in]: Contains a pointer to the IDisplay interface.
   nline:     [in]: Contains the line number to start displaying the text. The line
        numbers are by default spaced 15 pixels apart along the y-axis.
   pszStr:    [in]: The character string to be displayed on the screen.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void DisplayOutput(CLicenseApp * pMe, int nline, char *pszStr)
{
   AEEDeviceInfo di; // Device Info
   AECHAR * szBuf;      
   AECHAR * psz = NULL;
   int charHeight = 0;      // Stores the char height in pixels for given font
   int pnAscent = 0;        // Stores the ascent in number of pixels
   int pnDescent = 0;       // Stores the descent in number of pixels
   int pixelWidth;
   AEEFont font = AEE_FONT_NORMAL;
   int pnFits = 0, dy;
   int totalCh = 0;
   IShell *pIShell = pMe->a.m_pIShell;
   IDisplay *pIDisplay = pMe->a.m_pIDisplay;

   // Make sure the pointers we'll be using are valid
   if (pIShell == NULL || pIDisplay == NULL)
	   return;

   if ((szBuf = (AECHAR *) MALLOC(TEXT_BUFFER_SIZE * sizeof(AECHAR))) == NULL)
	   return;

   // Get device information
   ISHELL_GetDeviceInfo(pIShell,&di);

   // Get the font metrics info
   charHeight = IDISPLAY_GetFontMetrics (pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   // Convert to wide string (unicode)
   STR_TO_WSTR ((char *)pszStr, szBuf, TEXT_BUFFER_SIZE * sizeof(AECHAR));

   // If nlines is zero then print this string starting around the middle of 
   // the screen. Or else multiply nlines by charheight to decide the y coordinate of
   // the start of the string.
   if (nline < 0) {
      dy = di.cyScreen*2/5;
   }
   else{
      dy = nline * charHeight;
   }

   // psz keeps track of the point from which to write from the string buffer
   // in case the string does not fit one line and needs to wrap around in the
   // next line.
   psz = szBuf;
      
   // Need to calculate the lotal string length to decide if any wrapping
   // around is needed.
   totalCh = STRLEN ((char *)pszStr);

   // Keep displaying text string on multiple lines if the string can't be displayed
   // on one single line. Lines are spaced 15 pixels apart.
   while ((totalCh > 0) && (*psz != NULL))
   { 
      // Get information on how many characters will fit in a line.
      // Give the pointer to the buffer to be displayed, and the number of
      // pixels along the x axis you want to display the string in (max number)
      // pnFits will have the max number of chars that will fit in the maxWidth
      // number of pixels (given string can't fit in one line), or the number of 
      // chars in the string (if it does fit in one line). pnWidth gives the
      // number of pixels that will be used to display pnFits number of chars.
      pixelWidth = IDISPLAY_MeasureTextEx(pIDisplay,
                      font, 
                      (AECHAR *) psz,  // Start of the buffer to display,
                      -1,
                      di.cxScreen - 5, // maxWidth
                      &pnFits);         // Number of chars that will fit a line

      // If pnFits is zero there is something wrong in the input to above function. 
      // Normally this scenario should not occur. But, have the check anyway.
      if (pnFits == 0)
	  {
		 FREE(szBuf);
         return;
      }

      IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, psz, pnFits, 5 /*start dx*/, 
         dy, 0 /* use default rectangle coordinates */, 0);

      psz += pnFits;      // move pointer to the next segment to be displayed
      totalCh -= pnFits;  // reduce the total number of characters to still display
      dy += charHeight;   // Place next line charHeight pixels below the 
                          // previous line.
      IDISPLAY_Update(pIDisplay); //, TRUE);
      if (totalCh < pnFits)
         pnFits = totalCh;  // if total number is less than pnFits, adjust pnFits
   }
   FREE(szBuf);
   return;   
} // End of DisplayOutput
