/*===========================================================================

FILE: imenuctlusage.c

SERVICES: Sample applet using AEE

DESCRIPTION
  This file contains usage examples of the IMenuCtl interface. IMenuCtl interface
  functions are used to create various types of menus from which device users
  can make selections.  The following functions are illustrated in this usage example:

      -IMENUCTL_HandleEvent
      -IMENUCTL_Redraw
      -IMENUCTL_SetActive
      -IMENUCTL_IsActive
      -IMENUCTL_SetRect
      -IMENUCTL_GetRect
      -IMENUCTL_SetProperties (includes one example per IMenuCtl Property)
      -IMENUCTL_GetProperties
      -IMENUCTL_Reset
      -IMENUCTL_SetTitle
      -IMENUCTL_AddItem
      -IMENUCTL_AddItemEx
      -IMENUCTL_GetItemData
      -IMENUCTL_DeleteItem
      -IMENUCTL_DeleteAll
      -IMENUCTL_SetSel
      -IMENUCTL_GetSel
      -IMENUCTL_EnableCommand
      -IMENUCTL_SetItemText
      -IMENUCTL_SetItemTime
      -IMENUCTL_GetItemTime
      -IMENUCTL_SetStyle

PUBLIC CLASSES:  
   N/A


   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/

/*===============================================================================
                     INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
// Mandatory includes
#include "AEEAppGen.h"	// AEEApplet declaration
#include "AEEUsageAppIDs.h" // Class IDs of usage apps (including this one)
#include "AEE.h"			// Standard AEE Declarations
#include "AEEShell.h"	// AEE Shell Services
#include "AEEDisp.h"		// AEE Display Services
#include "AEEStdLib.h"  // AEE StdLib Services

// Services used by app
#include "AEEMenu.h"		// Menu Services

// App resource header generated by BREW ResourceEditor.
#include "imenuctl_res.h"

/*===========================================================================

                      PUBLIC DATA DECLARATIONS

===========================================================================*/

/*-------------------------------------------------------------------
            Type Declarations
-------------------------------------------------------------------*/
// IMenuCtl app struct. This is the main struct for this applet. This will hold 
// all the data members that need to be remembered throughout the life of
// the applet. 
// THE FIRST DATA MEMBER OF THIS STRUCT MUST BE AN AEEApplet OBJECT. 
// This is critical requirement which is related to an applet being
// dynamically loaded. The app developers should add their data members
// following the AEEApplet data member.
typedef struct _CIMenuCtlApp {
   AEEApplet	      a;
   IMenuCtl *  m_pIMenu;   // Pointer to main app menu
   IMenuCtl *  m_pStdMenu; // Pointer to standard menu used in examples
   IMenuCtl *  m_pSKMenu;  // Pointer to softkey menu used in examples
   IMenuCtl *  m_pListMenu; // Pointer to list-control menu used in examples
   IMenuCtl *  m_pIconMenu; // Pointer to icon-view menu used in examples
   AEERect     m_rc;
} CIMenuCtlApp;

/*-------------------------------------------------------------------
            Function Prototypes
-------------------------------------------------------------------*/
// Mandatory function:
// App Handle Event function
static boolean MenuCtlApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);

// App specific data alloc-init/free functions
static boolean IMenuCtl_InitAppData(IApplet* pMe);
static void IMenuCtl_FreeAppData(IApplet* pMe);

static void BuildMainMenu(CIMenuCtlApp *pMe);
static void IMenuCtl_Cleanup(CIMenuCtlApp *pMe);
static void IMenuCtlUsage (CIMenuCtlApp *pMe, uint16 wParam);
static void DisplayEvent (CIMenuCtlApp *pMe, uint16 wParam);
static void DisplayOutput(IShell *pIShell, IDisplay *pIDisplay, int nline, char *pszStr);
/*-------------------------------------------------------------------
            Global Constant Definitions
-------------------------------------------------------------------*/
// App Resource File
#define  APP_RES_FILE      "imenuctl.bar"

// App specific constants
#define USAGE_BASIC_USAGE                100
#define USAGE_HANDLEEVENT                101
#define USAGE_REDRAW                     102
#define USAGE_SETACTIVE                  103
#define USAGE_ISACTIVE                   104
#define USAGE_SETRECT                    105
#define USAGE_GETRECT                    106
#define USAGE_SETPROPS_WRAPSCROLL        107
#define USAGE_SETPROPS_NO_ARROWS         108
#define USAGE_SETPROPS_NO_REDRAW         109
#define USAGE_SETPROPS_MAXSOFTKEYITEMS   110
#define USAGE_SETPROPS_CALENDAR          111
#define USAGE_SETPROPS_AUTOSCROLLTIME    112
#define USAGE_SETPROPS_ICON_TEXT_TOP     113
#define USAGE_SETPROPS_ICON_TEXT_NOSEL   114
#define USAGE_SETPROPS_ICON_SINGLE_FRAME 115
#define USAGE_SETPROPS_UNDERLINE_TITLE   116
#define USAGE_GETPROPERTIES              117
#define USAGE_RESET                      118
#define USAGE_SETTITLE                   119
#define USAGE_ADDITEM                    120
#define USAGE_ADDITEMEX                  121
#define USAGE_GETITEMDATA                122
#define USAGE_DELETEITEM                 123
#define USAGE_DELETEALL                  124
#define USAGE_SETSEL                     125
#define USAGE_GETSEL                     126
#define USAGE_ENABLECOMMAND              127
#define USAGE_SETITEMTEXT                128
#define USAGE_SETITEMTIME                129
#define USAGE_GETITEMTIME                130
#define USAGE_SETSTYLE                   131
#define USAGE_ERASE_SCREEN               132   // Special case to erase screen

// Size of buffer used to hold text strings
#define TEXT_BUFFER_SIZE (100 * sizeof(AECHAR))

/*===============================================================================
                     FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to IMenuCtl_InitAppData(). This is done 
   as follows: IMenuCtl_InitAppData() is called to initialize applet-specific data structure 
   instance. It is the developer's responsibility to fill-in app data initialization code of 
   IMenuCtl_InitAppData(). The app developer is also responsible to release memory allocated 
   for data contained in applet-specific data structure  -- this can be
   done in IMenuCtl_FreeAppData().

PROTOTYPE:
	boolean AEEClsCreateInstance(AEECLSID ClsId, IShell* pIShell, IModule* pIModule, 
										  void** ppObj)

PARAMETERS:
	ClsId: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell interface. 

	pIModule: [in]: Contains pointer to the IModule interface to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid AEEApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app needs to be loaded and if AEEApplet_New() invocation was successful
  FALSE: If the app does not need to be loaded or if errors occurred in AEEApplet_New().
  If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId, IShell* pIShell, IModule* po, void** ppObj)
{
   *ppObj = NULL;
		
   // We want to load this App. So, invoke AEEApplet_New().To it, pass the
   // address of the app-specific handle event function.
   if(ClsId == AEECLSID_MENUCTL_APP){
      if(AEEApplet_New(sizeof(CIMenuCtlApp), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)MenuCtlApp_HandleEvent,(PFNFREEAPPDATA)IMenuCtl_FreeAppData) == TRUE)
      {
         if (IMenuCtl_InitAppData((IApplet*)*ppObj))
             return(AEE_SUCCESS);
      }
   }
   return (EFAILED);
}

/*===========================================================================

FUNCTION MenuCtlApp_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean App_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the IApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

	ecode: Specifies the Event sent to this applet

   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/

static boolean MenuCtlApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
   CIMenuCtlApp * pMe = (CIMenuCtlApp*)pi;
   AECHAR szBuf[25];

   switch (eCode) 
   {
        case EVT_APP_START:
			// When this applet is started, try to create the main menu
			// that allows the user to select a usage example to execute.
			// If we cannot create an instance of the menu class, exit and
			// indicate that we have handled this event by returning TRUE
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, 
				  (void **)&pMe->m_pIMenu) != SUCCESS)
				return TRUE;

			// Succeeded in obtaining a menu instance pointer, so
			// construct the menu and display it on the screen
			BuildMainMenu(pMe);
			return(TRUE);

        case EVT_APP_STOP:
            return(TRUE);

        case EVT_CTL_TAB:
			// User has pressed the LEFT or RIGHT arrow keys to "escape" the
			// standard menu in Example 28 (EnableCommand), which becomes dead
			// when the user disables command sending.  When we receive the control-
			// tabbing event that the menu control generates when the user presses 
			// these keys, release the standard menu and return to the main menu.
			if (pMe->m_pStdMenu != NULL && IMENUCTL_IsActive(pMe->m_pStdMenu))
			{
                IMENUCTL_Release(pMe->m_pStdMenu);
                pMe->m_pStdMenu = NULL;
				IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);
				IMENUCTL_Redraw(pMe->m_pIMenu);
				return TRUE;
			}
			return FALSE;

        case EVT_KEY: 
				// Send the key event to each of the menus that may be
				// active in our examples.  Since only one menu is active
				// at a time, this will assure that the correct menu 
				// handles the event.

			   if (pMe->m_pSKMenu != NULL && IMENUCTL_HandleEvent(pMe->m_pSKMenu, EVT_KEY, wParam, 0))
					return TRUE;

			   if (pMe->m_pListMenu != NULL && IMENUCTL_HandleEvent(pMe->m_pListMenu, EVT_KEY, wParam, 0))
				   return TRUE;

			   if (pMe->m_pIconMenu != NULL && IMENUCTL_HandleEvent(pMe->m_pIconMenu, EVT_KEY, wParam, 0))
					return TRUE;

			   if (pMe->m_pStdMenu != NULL && IMENUCTL_HandleEvent(pMe->m_pStdMenu, EVT_KEY, wParam, 0))
				   return TRUE;

				if (pMe->m_pIMenu)
				{
					boolean bOtherMenuActive = FALSE;

					// Set main menu active only if no other menu is active
					if ((pMe->m_pSKMenu && IMENUCTL_IsActive(pMe->m_pSKMenu)) ||
						 (pMe->m_pListMenu && IMENUCTL_IsActive(pMe->m_pListMenu)) ||
						 (pMe->m_pIconMenu && IMENUCTL_IsActive(pMe->m_pIconMenu)) ||
						 (pMe->m_pStdMenu && IMENUCTL_IsActive(pMe->m_pStdMenu)))
						bOtherMenuActive = TRUE;

					if (!bOtherMenuActive && !IMENUCTL_IsActive(pMe->m_pIMenu))
					{
						IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);
						IMENUCTL_Redraw(pMe->m_pIMenu);
					}
			      return IMENUCTL_HandleEvent(pMe->m_pIMenu, EVT_KEY, wParam, 0);
				}
				else
			      return FALSE;

		case EVT_COMMAND:
  		    switch(wParam)
			{
				// User is returning to the standard menu of example
				// 1 from one of the other menus.  Deactivate all of
				// the others and re-activate the standard menu.
			    case IDS_RETURN_TO_MENU:
					 IMENUCTL_SetActive(pMe->m_pSKMenu, FALSE);
                IMENUCTL_SetActive(pMe->m_pListMenu, FALSE);
                IMENUCTL_SetActive(pMe->m_pIconMenu, FALSE);				 
					 IDISPLAY_ClearScreen(pMe->a.m_pIDisplay);
                IMENUCTL_SetActive(pMe->m_pStdMenu, TRUE);
					 IMENUCTL_Redraw(pMe->m_pStdMenu);
					 return TRUE;

				// User is returning to the main usage app
				// menu after running one of the tests.  Release
				// any menus that may have been created in a test
				// and re-activate the main app menu.
                case IDS_RETURN_TO_MAIN_MENU:
                    IMenuCtl_Cleanup(pMe);
                    return TRUE;

				// User has made a menu selection of one of the
				// items in a test menu.  Since these items do nothing
				// (only the 'return' menu choice is used), simply return
				// indicating that we've handled the event
				case IDS_ITEM_1:
                case IDS_ITEM_2:
	            case IDS_ITEM_3:
					return TRUE;

				// User has selected a softkey menu created in
				// one of the examples.  Activate it after de-activating
				// the menu from which it was selected.
				case IDS_SOFTKEY_MENU:
					IMENUCTL_SetActive(pMe->m_pStdMenu, FALSE);
					IDISPLAY_EraseRect(pMe->a.m_pIDisplay, &pMe->m_rc);
                    IMENUCTL_SetActive(pMe->m_pSKMenu, TRUE);
					return TRUE;

				// User has selected a list control menu created in
				// one of the examples.  Activate it after de-activating
				// the menu from which it was selected.
				case IDS_LISTCTL_MENU:
					IMENUCTL_SetActive(pMe->m_pStdMenu, FALSE);
					IDISPLAY_EraseRect(pMe->a.m_pIDisplay, &pMe->m_rc);
                    IMENUCTL_SetActive(pMe->m_pListMenu, TRUE);
					return TRUE;

				// User has selected an icon view menu created in
				// one of the examples.  Activate it after de-activating
				// the menu from which it was selected.
				case IDS_ICON_VIEW_MENU:
					IMENUCTL_SetActive(pMe->m_pStdMenu, FALSE);
                    IMENUCTL_SetActive(pMe->m_pIconMenu, TRUE);
					return TRUE;

			    // User has disabled command sending in example 28
				case IDS_DISABLE_CMD:
					IMENUCTL_EnableCommand(pMe->m_pStdMenu, FALSE);
					return TRUE;

			    // User is replacing item text in example 29
				case IDS_NEW_ITEM_TEXT:
					STR_TO_WSTR("**NewText**", szBuf, sizeof(szBuf));
					IMENUCTL_SetItemText(pMe->m_pIMenu, USAGE_SETITEMTEXT, 0, 0, szBuf);

					// Release the standard menu used for replacing the item's text
					if (pMe->m_pStdMenu != NULL)
					{
						IMENUCTL_Release(pMe->m_pStdMenu);
						pMe->m_pStdMenu = NULL;
					}
					IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);
					IMENUCTL_Redraw(pMe->m_pIMenu);
					return TRUE;

			    // User is restoring item text to its original value in example 29
				case IDS_ORIG_ITEM_TEXT:
					STR_TO_WSTR("29. SetItemText", szBuf, sizeof(szBuf));
					IMENUCTL_SetItemText(pMe->m_pIMenu, USAGE_SETITEMTEXT, 0, 0, szBuf);

					// Release the standard menu used for replacing the item's text
					if (pMe->m_pStdMenu != NULL)
					{
						IMENUCTL_Release(pMe->m_pStdMenu);
						pMe->m_pStdMenu = NULL;
					}
					IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);
					IMENUCTL_Redraw(pMe->m_pIMenu);
					return TRUE;

				// The following commands are generated by user selections
				// from the main usage app menu.
			    case USAGE_BASIC_USAGE:
                case USAGE_HANDLEEVENT:
                case USAGE_REDRAW:
                case USAGE_SETACTIVE:
                case USAGE_ISACTIVE:
                case USAGE_SETRECT:
                case USAGE_GETRECT:
                case USAGE_SETPROPS_WRAPSCROLL:
                case USAGE_SETPROPS_NO_ARROWS:
                case USAGE_SETPROPS_NO_REDRAW:
                case USAGE_SETPROPS_MAXSOFTKEYITEMS:
                case USAGE_SETPROPS_CALENDAR:
                case USAGE_SETPROPS_AUTOSCROLLTIME:
                case USAGE_SETPROPS_ICON_TEXT_TOP:
                case USAGE_SETPROPS_ICON_TEXT_NOSEL:
                case USAGE_SETPROPS_ICON_SINGLE_FRAME:
                case USAGE_SETPROPS_UNDERLINE_TITLE:
                case USAGE_GETPROPERTIES:
                case USAGE_RESET:
                case USAGE_SETTITLE:
                case USAGE_ADDITEM:
                case USAGE_ADDITEMEX:
                case USAGE_GETITEMDATA:
                case USAGE_DELETEITEM:
                case USAGE_DELETEALL:
                case USAGE_SETSEL:
                case USAGE_GETSEL:
                case USAGE_ENABLECOMMAND:
                case USAGE_SETITEMTEXT:
                case USAGE_SETITEMTIME:
                case USAGE_GETITEMTIME:
                case USAGE_SETSTYLE:
                case USAGE_ERASE_SCREEN:

				   // Deactivate main menu to turn off horizontal scrolling
				   // of long menu items
				   IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);

				   // Execute the usage example the user has selected
                   IMenuCtlUsage (pMe, wParam);

			       return TRUE;

			    default:
				    return FALSE;
			}
        default:
            break;
   }
   return TRUE;
}


/*===========================================================================

FUNCTION IMenuCtl_InitAppData

DESCRIPTION
   This function initializes data in the app-specific data structure.

PROTOTYPE:
	boolean IMenuCtl_InitAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

DEPENDENCIES
  None.

RETURN VALUE
  TRUE: If the app has app data is allocated and initialized successfully
  FALSE: Otherwise

SIDE EFFECTS
  None.
===========================================================================*/
static boolean IMenuCtl_InitAppData(IApplet* pi)
{
   CIMenuCtlApp * pMe = (CIMenuCtlApp*)pi;
   int cLineHeight;
   AEEDeviceInfo di;

   // Make sure the pointers we'll be using are valid
   if (pMe == NULL || pMe->a.m_pIShell == NULL)
	   return FALSE;

   pMe->m_pIMenu = NULL;

   // Calculate dimensions of screen rectangle--leave room at
   // top of screen for the example header, as well as a 5-pixel
   // border all around
   cLineHeight = 2 + IDISPLAY_GetFontMetrics(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, NULL, NULL);

   ISHELL_GetDeviceInfo(pMe->a.m_pIShell, &di);

   pMe->m_rc.x = 5;
   pMe->m_rc.y = cLineHeight + 5;
   pMe->m_rc.dx = di.cxScreen - 5;
   pMe->m_rc.dy = di.cyScreen - cLineHeight - 5;

   return TRUE;
}

/*===========================================================================

FUNCTION IMenuCtl_FreeAppData

DESCRIPTION
   Frees data and interfaces contained in app-specific data structure.

PROTOTYPE:
   static void IMenuCtl_FreeAppData(IApplet* pi);

PARAMETERS:
   pi [in]: Pointer to the IApplet structure. This structure contains information specific
   to this applet. It was initialized during the AEEClsCreateInstance() function.

DEPENDENCIES
   Assumes pMe is not NULL

RETURN VALUE
   None.

SIDE EFFECTS
   None.
===========================================================================*/
static void IMenuCtl_FreeAppData(IApplet* pi)
{
   CIMenuCtlApp * pMe = (CIMenuCtlApp*)pi;

   // Release any menu-control interfaces that were created
   // during usage app execution

   // Release main usage app menu
   if(pMe->m_pIMenu != NULL)
   {
	  IMENUCTL_Release(pMe->m_pIMenu);
      pMe->m_pIMenu = NULL;
   }

   // Release standard menu used in examples
   if (pMe->m_pStdMenu != NULL)
   {
	  IMENUCTL_Release(pMe->m_pStdMenu);
	  pMe->m_pStdMenu = NULL;
   }

   // Release softkey menu used in examples
   if (pMe->m_pSKMenu != NULL)
   {
      IMENUCTL_Release(pMe->m_pSKMenu);
	  pMe->m_pSKMenu = NULL;
   }

   // Release list-control menu used in examples
   if (pMe->m_pListMenu != NULL)
   {
	  IMENUCTL_Release(pMe->m_pListMenu);
	  pMe->m_pListMenu = NULL;
   }

   // Release icon-view menu used in examples
   if (pMe->m_pIconMenu != NULL)
   {
	  IMENUCTL_Release(pMe->m_pIconMenu);
	  pMe->m_pIconMenu = NULL;
   }
}

/*===============================================================================
                     APP HELPER FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: IMenuCtl_Cleanup

DESCRIPTION
    This function performs cleanup operations needed whenever the user completes
	execution of a usage example.  It is also called from within the usage examples
	themselves whenever a fatal error is detected, to assure that all allocated 
	interfaces have been freed and that the main menu has been reactivated.
    
PROTOTYPE:
   static void IMenuCtl_Cleanup()

PARAMETERS:
   None

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void IMenuCtl_Cleanup(CIMenuCtlApp * pMe)
{
   // If an instance of the standard menu in the usage examples
   // is still present, free it and clear the pointer in our app data structure
   if (pMe->m_pStdMenu != NULL)
   {
      IMENUCTL_Release(pMe->m_pStdMenu);
      pMe->m_pStdMenu = NULL;
   }

   // If an instance of the standard menu in the usage examples
   // is still present, free it and clear the pointer in our app data structure
   if (pMe->m_pSKMenu != NULL)
   {
      IMENUCTL_Release(pMe->m_pSKMenu);
      pMe->m_pSKMenu = NULL;
   }

   // If an instance of the standard menu in the usage examples
   // is still present, free it and clear the pointer in our app data structure
   if (pMe->m_pListMenu != NULL)
   {
      IMENUCTL_Release(pMe->m_pListMenu);
      pMe->m_pListMenu = NULL;
   }

   // If an instance of the standard menu in the usage examples
   // is still present, free it and clear the pointer in our app data structure
   if (pMe->m_pIconMenu != NULL)
   {
      IMENUCTL_Release(pMe->m_pIconMenu);
      pMe->m_pIconMenu = NULL;
   }

   // When the usage example was selected, the main menu was deactivated
   // to turn off horizontal scrolling of long menu-item text strings.  Here
   // we re-activate and re-draw the main menu.
   IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);
   IMENUCTL_Redraw(pMe->m_pIMenu);
}
/*===========================================================================

FUNCTION: IMenuCtlUsage

DESCRIPTION
    This function executes the usage example selected from the main menu by the
	user.  It takes as input the app data pointer and the ID of the example the
	user selected.  After clearing the screen, the selected example is executed.
	Each example is a self-contained code block that contains any local variables
	needed to execute the example.  Each code block also creates and releases any
	BREW interfaces that are required, and will exit if any errors are detected.  
	Once the example has been executed, this function prints a heading at the top 
	of the screen to indicate which example was selected, 
	and a message at the bottom of the screen instructing the user how to return to
	the main menu to execute another example.
    
PROTOTYPE:
   static void IMenuCtlUsage(CIMenuCtlApp * pMe, uint16 wParam)

PARAMETERS:
   pMe:   [in]: Contains a pointer to the usage app data structure.
   wParam: [in]: Identifier of the selected example (example IDs are specified when the
                 main usage app menu is created).

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void IMenuCtlUsage (CIMenuCtlApp * pMe, uint16 wParam)
{
   char szResFile[] = APP_RES_FILE;   // Resource File used by this applet

   // Make sure the pointers we'll be using are valid
   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
	   return;

   // Erase screen first for display output purposes.
   DisplayEvent (pMe, USAGE_ERASE_SCREEN);
	IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
	IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);

   switch (wParam)
   {
	  case USAGE_BASIC_USAGE:
         {
            // 1. Usage: Basic IMenuCtl Usage

			// This example illustrates the four types of menus that can be created with the
			// IMenuCtl interface.  It creates a standard menu from which menus of the other
			// three types (softkey, list control and icon view) can be displayed.  The example
			// shows how item are added to a menu with the IMENUCTL_AddItem and IMENUCTL_AddItemEx
			// functions, and how the rectangles of each type of menu can be set with 
			// IMENUCTL_SetRect.

			CtlAddItem pai;
            AEERect rc;
			int cLineHeight, cTextWidth;
			AECHAR szBuf[25];

			// Create the standard menu that will be displayed first
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, 
				  (void **)&pMe->m_pStdMenu) != SUCCESS)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}

            // Add items to standard menu
            IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_LISTCTL_MENU, IDS_LISTCTL_MENU, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_SOFTKEY_MENU, IDS_SOFTKEY_MENU, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_ICON_VIEW_MENU, IDS_ICON_VIEW_MENU, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_RETURN_TO_MAIN_MENU, 
				IDS_RETURN_TO_MAIN_MENU, NULL, NULL);

            IMENUCTL_SetRect(pMe->m_pStdMenu, &pMe->m_rc);

			// Create the list control menu
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_LISTCTL, 
				  (void **)&pMe->m_pListMenu) != SUCCESS)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}

            // Add items to the list control menu
            IMENUCTL_AddItem(pMe->m_pListMenu, szResFile, IDS_ITEM_1, IDS_ITEM_1, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pListMenu, szResFile, IDS_ITEM_2, IDS_ITEM_2, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pListMenu, szResFile, IDS_ITEM_3, IDS_ITEM_3, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pListMenu, szResFile, IDS_RETURN_TO_MENU, 
				IDS_RETURN_TO_MENU, NULL, NULL);

			// List control menus look better if their rectangles are just large
			// enough to hold the biggest item text string.

            // Calculate the width of the longest string in the menu
 			STR_TO_WSTR("Return to Menu", szBuf, sizeof(szBuf));
			cTextWidth = 2 + IDISPLAY_MeasureTextEx(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1, 100, NULL);

			// Calculate the height of a line of text
			cLineHeight = 2 + IDISPLAY_GetFontMetrics(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, NULL, NULL);

			// Place the list control menu in the middle of the screen
			rc.x = (pMe->m_rc.dx - cTextWidth) / 2;
			rc.y = (pMe->m_rc.dy - cLineHeight) / 2;
			rc.dx = cTextWidth;
			rc.dy = cLineHeight;

			// Set the value of the menu's rectangle
			IMENUCTL_SetRect(pMe->m_pListMenu, &rc);

			// Create the softkey menu
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOFTKEYCTL, 
				  (void **)&pMe->m_pSKMenu) != SUCCESS)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}

            // Add items to the softkey menu
            IMENUCTL_AddItem(pMe->m_pSKMenu, szResFile, IDS_ITEM_1, IDS_ITEM_1, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pSKMenu, szResFile, IDS_ITEM_2, IDS_ITEM_2, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pSKMenu, szResFile, IDS_ITEM_3, IDS_ITEM_3, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pSKMenu, szResFile, IDS_RETURN_TO_MENU, 
				IDS_RETURN_TO_MENU, NULL, NULL);

			// Create the icon view menu
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_ICONVIEWCTL, 
				  (void **)&pMe->m_pIconMenu) != SUCCESS)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}

            // Add items to the icon view menu.  Since this menu contains a bitmap
			// for each item, we must use AddItemEx.

			// First, we populate the fields of the CtlAddItem structure that
			// will be the same for all items.

			// Since we're getting our item text and bitmaps from the
			// resource file, we set these pointers to NULL
            pai.pText  = NULL;        // Text 
            pai.pImage = NULL;        // Image 

			// Both text and image are contained in our app's
			// resource file
            pai.pszResImage = szResFile;   // Resource for Image
            pai.pszResText  = szResFile;   // Resource for Text

			// Use the default font for all item text
            pai.wFont = 0;         // 0 - Default

			// We don't specify any double-word data for our items
            pai.dwData = NULL;

			// Now specify the item-specific fields in the CtlAddItem structure

			// Specify values for first item and add them to the icon-view menu
            pai.wText   = IDS_ITEM_1; 
            pai.wImage  = IDB_ITEM_1_BMP;
            pai.wItemID = IDS_ITEM_1;

            IMENUCTL_AddItemEx(pMe->m_pIconMenu, &pai);

			// Specify values for second item and add them to the icon-view menu
            pai.wText   = IDS_ITEM_2; 
            pai.wImage  = IDB_ITEM_2_BMP;
            pai.wItemID = IDS_ITEM_2;

            IMENUCTL_AddItemEx(pMe->m_pIconMenu, &pai);

			// Specify values for third item and add them to the icon-view menu
            pai.wText   = IDS_ITEM_3; 
            pai.wImage  = IDB_ITEM_3_BMP;
            pai.wItemID = IDS_ITEM_3;

            IMENUCTL_AddItemEx(pMe->m_pIconMenu, &pai);

			// Specify values for last item and add them to the icon-view menu
            pai.wText   = IDS_RETURN_TO_MENU; 
            pai.wImage  = IDB_RETURN_TO_MENU_BMP;
            pai.wItemID = IDS_RETURN_TO_MENU;

            IMENUCTL_AddItemEx(pMe->m_pIconMenu, &pai);

            IMENUCTL_SetRect(pMe->m_pIconMenu, &pMe->m_rc);

			// Now that all menus have been created, activate the standard menu, 
			// which will allow theuser to select either the list menu,
			// softkey menu, or icon view menu to display
            IMENUCTL_SetActive(pMe->m_pStdMenu, TRUE);
         }
         break;

      case USAGE_HANDLEEVENT:
         {
            // 2. Usage: IMENUCTL_HandleEvent

			// This example illustrates the use of IMENUCTL_HandleEvent to send
			// an EVT_CTL_SET_TITLE event to the main app usage menu to change its title.
			// The dwParam parameter gives the name of this app's resource file,
			// and wParam identifies the character string in the resource file
			// that will be used for the title.  Since the main menu is active when
			// we select this menu item, it will handle the event and modify
			// its title.
			IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);

			// Pass the EVT_CTL_SET_TITLE event to the menu.  If the HandleEvent function
			// returns TRUE, indicating that the event was handled, redraw the screen so the
			// new title will be displayed.
 			if (IMENUCTL_HandleEvent(pMe->m_pIMenu, EVT_CTL_SET_TITLE, IDS_MENU_TITLE, (uint32) szResFile))
                IMENUCTL_Redraw(pMe->m_pIMenu);
         }
         break;

      case USAGE_REDRAW:
         {
            // 3. Usage: IMENUCTL_Redraw

				// IMENUCTL_Redraw re-displays the contents of a menu on the screen.
				// To illustrate thus function, we first clear the screen and then re-draw
				// the main menu of this usage app.  We display the standard example heading
				// on the screen, which over-writes some of the menu text.  When you change
				// menu selection, the heading disappears, which shows that the menu redraws
				// itself after each change in selection.

				// First clear the screen
 				DisplayEvent(pMe, USAGE_ERASE_SCREEN);

				
            IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);

				// Now redraw the main menu.  IMENUCTL_Redraw returns TRUE is the menu is
				// redrawn successfully, and FALSE otherwise.
				IMENUCTL_Redraw(pMe->m_pIMenu);
         }
         break;

      case USAGE_SETACTIVE:
         {
            // 4. Usage: IMENUCTL_SetActive

			// IMENUCTL_SetActive is used to de-activate and activate menu controls.
			// Only an active menu can receive key-press events.  Upon becoming active,
			// the menu redraws itself (unless the MP_NO_REDRAW property is set; see example
			// 10 below).  

  			// Create the standard menu that will be displayed
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, 
				  (void **)&pMe->m_pStdMenu) != SUCCESS)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}

            // Add items to standard menu
            IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_ITEM_1, IDS_ITEM_1, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_ITEM_2, IDS_ITEM_2, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_ITEM_3, IDS_ITEM_3, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_RETURN_TO_MAIN_MENU, 
				IDS_RETURN_TO_MAIN_MENU, NULL, NULL);

            IMENUCTL_SetRect(pMe->m_pStdMenu, &pMe->m_rc);

			// Use SetActive to activate the standard menu we just created. 
			// It can now receive key input from the user.
            IMENUCTL_SetActive(pMe->m_pStdMenu, TRUE);
         }
         break;

      case USAGE_ISACTIVE:
         {
            // 5. Usage: IMENUCTL_IsActive

			// In this example, we illustrate the use of IMENUCTL_SetActive
			// by testing if the main app menu is active.  If so, a message
			// will be printed on the screen.  Note that the menu remains
			// active even though it doesn't appear on the screen (it still
			// processes the Up or Down key that you press to return to it).

			// Set the main menu active
            IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);

			// Clear the screen
            DisplayEvent(pMe, USAGE_ERASE_SCREEN);

			// Test if the menu is still active
			if (IMENUCTL_IsActive(pMe->m_pIMenu))
			{
				DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 2, "Main Menu is active");
                DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 3, "Press Up/Down");
                DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 4, "to continue");
			}
 
			// Notice that when you press Up or Down, the main menu re-displays
			// itself with either Item 4 or Item 6 selected.  This demonstrates that
			// it is active and received your key press.
         }
         break;

      case USAGE_SETRECT:
         {
            // 6. Usage: IMENUCTL_SetRect

			// The SetRect function lets you specify the rectangle that
			// will contain the menu (by default, the menu occupies the
			// entire screen).  Since list control menus are designed to
			// occupy a small portion of the screen real estate, we will
			// illustrate SetRect by displaying a list control menu in
			// as small a region of the screen as possible.
            int16 cTextWidth, cLineHeight;
			AEEDeviceInfo di;
            AECHAR szBuf[30];
			AEERect rc;

			// First, create the list control menu
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_LISTCTL, 
				  (void **)&pMe->m_pListMenu) != SUCCESS)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}

            // Add items to the list control menu
            IMENUCTL_AddItem(pMe->m_pListMenu, szResFile, IDS_ITEM_1, IDS_ITEM_1, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pListMenu, szResFile, IDS_ITEM_2, IDS_ITEM_2, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pListMenu, szResFile, IDS_ITEM_3, IDS_ITEM_3, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pListMenu, szResFile, IDS_RETURN_TO_MAIN_MENU, 
				IDS_RETURN_TO_MAIN_MENU, NULL, NULL);

			// Calculate the width of the longest string in the menu--make
			// the menu two pixels wider than this
 			STR_TO_WSTR("Return to Main Menu", szBuf, sizeof(szBuf));
			cTextWidth = 2 + IDISPLAY_MeasureTextEx(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1, 100, NULL);

			// Calculate the height of a line of text--add two pixels to be sure there's
			// enough vertical space to display the menu item text
			cLineHeight = 2 + IDISPLAY_GetFontMetrics(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, NULL, NULL);

			// Retrieve the dimensions of the device screen
			ISHELL_GetDeviceInfo(pMe->a.m_pIShell, &di);

			// Place the list control menu in the middle of the screen,
			// and set its width and height to the values calculated above
			rc.x = (di.cxScreen - cTextWidth) / 2;
			rc.y = (di.cyScreen - cLineHeight) / 2;
			rc.dx = cTextWidth;
			rc.dy = cLineHeight;

			// Set the value of the menu's rectangle
			IMENUCTL_SetRect(pMe->m_pListMenu, &rc);

			// Clear the region of the screen before drawing
			// the menu (if we don't, we'll still see the items
			// in the main app menu)
			IDISPLAY_EraseRect(pMe->a.m_pIDisplay, &pMe->m_rc);

			// Activate the list control menu
			// so it will be displayed on the screen
            IMENUCTL_SetActive(pMe->m_pListMenu, TRUE);
         }
         break;

      case USAGE_GETRECT:
         {
            // 7. Usage: IMENUCTL_GetRect
 
			// This example illustrates the use of the GetRect function.  We
			// use it to retrieve the rectangle of the main usage app menu and
			// and echo its dimensions to the screen.

			AEERect rc;
			char szBuf[30];

		    // Retrieve the rectangle dimensions of the main app menu
			// (these are set with SetRect in the function BuildMainMenu below)
			IMENUCTL_GetRect(pMe->m_pIMenu, &rc);

		    // Echo the info that was returned 
			// by GetRect in the AEERect structure

			// Set the main menu active
            IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);

			// Clear the screen
			DisplayEvent(pMe, USAGE_ERASE_SCREEN);

		    // x coordinate of rectangle origin in pixels
		    SPRINTF(szBuf, "x: %d", rc.x); 
            DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 2, szBuf);

		    // y coordinate of rectangle origin in pixels
		    SPRINTF(szBuf, "y: %d", rc.y); 
            DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 3, szBuf);

		    // rectangle width in pixels
 		    SPRINTF(szBuf, "dx: %d", rc.dx); 
            DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 4, szBuf);
 
			// rectangle height in pixels
		    SPRINTF(szBuf, "dy: %d", rc.dy); 
            DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 5, szBuf);

			DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 6, "Press Up/Down");
			DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 7, "to continue");

         }
         break;

      case USAGE_SETPROPS_WRAPSCROLL:
         {
            // 8. Usage: IMENUCTL_SetProperties -- MP_WRAPSCROLL

			// The intent of this property is to enable wrap-around scrolling,
			// so you can scroll down when the last menu item is selected and up
			// when the first is selected.  At present, however, this property
			// cannot be set by the user: wrap-scrolling is enabled for softkey and
			// list menus, and disabled for standard and icon view menus.

  			// Create the standard menu that will be displayed as a calendar
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, 
				  (void **)&pMe->m_pStdMenu) != SUCCESS)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}

            // Add items to standard menu
            IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_ITEM_1, IDS_ITEM_1, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_ITEM_2, IDS_ITEM_2, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_ITEM_3, IDS_ITEM_3, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_RETURN_TO_MAIN_MENU, 
				IDS_RETURN_TO_MAIN_MENU, NULL, NULL);

            IMENUCTL_SetRect(pMe->m_pStdMenu, &pMe->m_rc);

			// Set the property that causes the menu 
			// scrolling to wrap around
			IMENUCTL_SetProperties(pMe->m_pIMenu, MP_WRAPSCROLL);

			// Activate the standard menu
            IMENUCTL_SetActive(pMe->m_pStdMenu, TRUE);
         }
         break;

      case USAGE_SETPROPS_NO_ARROWS:
         {
            // 9. Usage: IMENUCTL_SetProperties -- MP_NO_ARROWS

			// When the MP_ICON_SINGLE_FRAME property is set, arrows are drawn
			// on either side of the item text to instruct the user to use the 
			// left and right arrow keys to scroll between menu items.  The property
			// MP_NO_ARROWS suppresses drawing of these arrows (this property has no
			// effect unless MP_ICON_SINGLE_FRAME is also set).  In this example, we
			// set the MP_ICON_SINGLE_FRAME and MP_NO_ARROWS properties for an icon-view
			// menu (see example 12 for the effect of setting only the property 
			// MP_ICON_SINGLE_FRAME).
  			CtlAddItem pai;

 			// Create the icon view menu
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_ICONVIEWCTL, 
				  (void **)&pMe->m_pIconMenu) != SUCCESS)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}

			// Set the property that suppresses the drawing of arrows
			// with the item text when the MP_ICON_SINGLE_FRAME property is set
			// NOTE: You must set this property BEFORE you add items to
			// your menu, or BEFORE setting its rectangle with IMENUCTL_SetRect
			IMENUCTL_SetProperties(pMe->m_pIconMenu, MP_ICON_SINGLE_FRAME | MP_NO_ARROWS);

            // Add items to the icon view menu.  Since this menu contains a bitmap
			// for each item, we must use AddItemEx.

			// First, we populate the fields of the CtlAddItem structure that
			// will be the same for all items.

			// Since we're getting our item text and bitmaps from the
			// resource file, we set these pointers to NULL
            pai.pText  = NULL;        // Text 
            pai.pImage = NULL;        // Image 

			// Both text and image are contained in our app's
			// resource file
            pai.pszResImage = szResFile;   // Resource for Image
            pai.pszResText  = szResFile;   // Resource for Text

			// Use the default font for all item text
            pai.wFont = 0;         // 0 - Default

			// We don't specify any double-word data for our items
            pai.dwData = NULL;

			// Now specify the item-specific fields in the CtlAddItem structure

			// Specify values for first item and add them to the icon-view menu
            pai.wText   = IDS_ITEM_1; 
            pai.wImage  = IDB_ITEM_1_BMP;
            pai.wItemID = IDS_ITEM_1;

            IMENUCTL_AddItemEx(pMe->m_pIconMenu, &pai);

			// Specify values for second item and add them to the icon-view menu
            pai.wText   = IDS_ITEM_2; 
            pai.wImage  = IDB_ITEM_2_BMP;
            pai.wItemID = IDS_ITEM_2;

            IMENUCTL_AddItemEx(pMe->m_pIconMenu, &pai);

			// Specify values for third item and add them to the icon-view menu
            pai.wText   = IDS_ITEM_3; 
            pai.wImage  = IDB_ITEM_3_BMP;
            pai.wItemID = IDS_ITEM_3;

            IMENUCTL_AddItemEx(pMe->m_pIconMenu, &pai);

			// Specify values for last item and add them to the icon-view menu
            pai.wText   = IDS_RETURN_TO_MAIN_MENU; 
            pai.wImage  = IDB_RETURN_TO_MENU_BMP;
            pai.wItemID = IDS_RETURN_TO_MAIN_MENU;

            IMENUCTL_AddItemEx(pMe->m_pIconMenu, &pai);

            IMENUCTL_SetRect(pMe->m_pIconMenu, &pMe->m_rc);

			// Activate the icon view menu
            IMENUCTL_SetActive(pMe->m_pIconMenu, TRUE);
         }
         break;

      case USAGE_SETPROPS_NO_REDRAW:
         {
            // 10. Usage: IMENUCTL_SetProperties -- MP_NO_REDRAW

			// Setting the property MP_NO_REDRAW prebents a menu from redrawing
			// itself when it is set active.  To illustrate this property, we create a
			// standard menu and set the MP_NO_REDRAW property for it.  When we 
			// set this menu active with IMENUCTL_SetActive, it is not immediately
			// drawn on the screen.

			// Create the standard menu example
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, 
				  (void **)&pMe->m_pStdMenu) != SUCCESS)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}

            // Add items to standard menu
            IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_ITEM_1, IDS_ITEM_1, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_ITEM_2, IDS_ITEM_2, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_ITEM_3, IDS_ITEM_3, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_RETURN_TO_MAIN_MENU, 
				IDS_RETURN_TO_MAIN_MENU, NULL, NULL);

            IMENUCTL_SetRect(pMe->m_pStdMenu, &pMe->m_rc);

			// Set the property that prevents the standard menu from
			// redrawing itself
			IMENUCTL_SetProperties(pMe->m_pStdMenu, MP_NO_REDRAW);

			// Now activate the standard menu
			IMENUCTL_SetActive(pMe->m_pStdMenu, TRUE);

			// Notice that the main app menu remains on the screen,
			// even though the standard menu was set active.  This is because the
			// standard menu does not redraw itself upon activation, since
			// we set the MP_NO_REDRAW property.  If you press UP or DOWN,
			// notice that the standard menu handles the key and displays 
			// itself on the screen.
         }
         break;

      case USAGE_SETPROPS_MAXSOFTKEYITEMS:
         {
            // 11. Usage: IMENUCTL_SetProperties -- MP_MAXSOFTKEYITEMS

			// Ordinarily, at most three items of a softkey menu can appear
			// on the screen at a time.  Setting this property allows a larger
			// number of items to appear (up to 12, if the device screen will allow it).
			// To illustrate the effect of this property, we create a softkey menu
			// with a large number of items, each having a one-character item text field
			// so that many of them will fit on the screen at once.

 			// Create the softkey menu
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOFTKEYCTL, 
				  (void **)&pMe->m_pSKMenu) != SUCCESS)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}

            // Add lots of items to the softkey menu
            IMENUCTL_AddItem(pMe->m_pSKMenu, szResFile, IDS_A, IDS_A, NULL, NULL);
            IMENUCTL_AddItem(pMe->m_pSKMenu, szResFile, IDS_B, IDS_B, NULL, NULL);
            IMENUCTL_AddItem(pMe->m_pSKMenu, szResFile, IDS_C, IDS_C, NULL, NULL);
            IMENUCTL_AddItem(pMe->m_pSKMenu, szResFile, IDS_D, IDS_D, NULL, NULL);
            IMENUCTL_AddItem(pMe->m_pSKMenu, szResFile, IDS_E, IDS_E, NULL, NULL);
            IMENUCTL_AddItem(pMe->m_pSKMenu, szResFile, IDS_F, IDS_F, NULL, NULL);
            IMENUCTL_AddItem(pMe->m_pSKMenu, szResFile, IDS_G, IDS_G, NULL, NULL);
            IMENUCTL_AddItem(pMe->m_pSKMenu, szResFile, IDS_H, IDS_H, NULL, NULL);
            IMENUCTL_AddItem(pMe->m_pSKMenu, szResFile, IDS_I, IDS_I, NULL, NULL);
            IMENUCTL_AddItem(pMe->m_pSKMenu, szResFile, IDS_J, IDS_J, NULL, NULL);
            IMENUCTL_AddItem(pMe->m_pSKMenu, szResFile, IDS_K, IDS_K, NULL, NULL);
            IMENUCTL_AddItem(pMe->m_pSKMenu, szResFile, IDS_L, IDS_L, NULL, NULL);
            IMENUCTL_AddItem(pMe->m_pSKMenu, szResFile, IDS_M, IDS_M, NULL, NULL);

			IMENUCTL_AddItem(pMe->m_pSKMenu, szResFile, IDS_RETURN_TO_MAIN_MENU, 
				IDS_RETURN_TO_MAIN_MENU, NULL, NULL);

            IMENUCTL_SetRect(pMe->m_pSKMenu, &pMe->m_rc);

			// Set the property that increases the maximum number
			// of softkey menu items that can appear on the screen at once
			IMENUCTL_SetProperties(pMe->m_pSKMenu, MP_MAXSOFTKEYITEMS);

            IMENUCTL_SetActive(pMe->m_pSKMenu, TRUE);
         }
         break;

      case USAGE_SETPROPS_CALENDAR:
         {
            // 12. Usage: IMENUCTL_SetProperties -- MP_CALENDAR

			// By setting the property MP_CALENDAR for a standard menu, you can use the
			// menu as a daily calendar by associating a time and duration with each menu item.  The item
			// text represents an appointment for that time, and the height of the item is
			// proportional to its duration.  When the property is set, the
			// menu control handles the number keys, which can be used to select the appointment
			// (menu item) for a specific time.  The menu item corresponding to the current time of day 
			// is highlighted with a small arrow, and the appointment time of day is printed to
			// the left of each item.
			// In this example, we create a calendar view menu and add some items to it, setting 
			// the time and duration of each item.

 			// Create the standard menu that will be displayed as a calendar
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, 
				  (void **)&pMe->m_pStdMenu) != SUCCESS)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}

            // Add items to standard menu and set their times.  Times are specified in minutes
			// with 0 = 12:00am, 60 = 1:00am, 120 = 2:00am, etc.
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_WORK_OUT, IDS_WORK_OUT, NULL, NULL);
            IMENUCTL_SetItemTime(pMe->m_pStdMenu, IDS_WORK_OUT, 360, 60);

			// We'll leave this item with a blank text string
            IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_NULL_1, IDS_NULL_1, NULL, NULL);
            IMENUCTL_SetItemTime(pMe->m_pStdMenu, IDS_NULL_1, 420, 120);

            // Add status meeting item to menu
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_STATUS_MEETING, IDS_STATUS_MEETING, NULL, NULL);
            IMENUCTL_SetItemTime(pMe->m_pStdMenu, IDS_STATUS_MEETING, 540, 60);

			// We'll leave this item with a blank text string
            IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_NULL_2, IDS_NULL_2, NULL, NULL);
            IMENUCTL_SetItemTime(pMe->m_pStdMenu, IDS_NULL_2, 600, 120);

            // Add lunch meeting item to menu
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_LUNCH, IDS_LUNCH, NULL, NULL);
            IMENUCTL_SetItemTime(pMe->m_pStdMenu, IDS_LUNCH, 720, 60);

			// We'll leave this item with a blank text string
            IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_NULL_3, IDS_NULL_3, NULL, NULL);
            IMENUCTL_SetItemTime(pMe->m_pStdMenu, IDS_NULL_3, 780, 120);

            // Add code review item to menu
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_CODE_REVIEW, IDS_CODE_REVIEW, NULL, NULL);
            IMENUCTL_SetItemTime(pMe->m_pStdMenu, IDS_CODE_REVIEW, 900, 60);

			// Add item to return to main menu
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_RETURN_TO_MAIN_MENU, 
				IDS_RETURN_TO_MAIN_MENU, NULL, NULL);
            IMENUCTL_SetItemTime(pMe->m_pStdMenu, IDS_RETURN_TO_MAIN_MENU, 960, 120);

            IMENUCTL_SetRect(pMe->m_pStdMenu, &pMe->m_rc);

			// Set the property that causes the menu to appear
			// in the format of a daily appointment calendar
			IMENUCTL_SetProperties(pMe->m_pStdMenu, MP_CALENDAR);

			// Activate the calendar view menu
            IMENUCTL_SetActive(pMe->m_pStdMenu, TRUE);
         }
         break;

      case USAGE_SETPROPS_AUTOSCROLLTIME:
         {
            // 13. Usage: IMENUCTL_SetProperties -- MP_AUTOSCROLLTIME

			// When the MP_AUTOSCROLLTIME property is set for a calendar view menu,
			// the appointment corresponding to the current time of day appears at
			// the top of the screen.  This property has no effect is MP_CALENDAR is
			// not also set.
			// In this example, we set MP_AUTOSCROLLTIME for the menu created in Example 12.

  			// Create the standard menu that will be displayed as a calendar
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, 
				  (void **)&pMe->m_pStdMenu) != SUCCESS)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}

            // Add items to standard menu and set their times
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_WORK_OUT, IDS_WORK_OUT, NULL, NULL);
            IMENUCTL_SetItemTime(pMe->m_pStdMenu, IDS_WORK_OUT, 360, 60);

			// We'll leave this item with a blank text string
            IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_NULL_1, IDS_NULL_1, NULL, NULL);
            IMENUCTL_SetItemTime(pMe->m_pStdMenu, IDS_NULL_1, 420, 120);

            // Add status meeting item to menu
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_STATUS_MEETING, IDS_STATUS_MEETING, NULL, NULL);
            IMENUCTL_SetItemTime(pMe->m_pStdMenu, IDS_STATUS_MEETING, 540, 60);

			// We'll leave this item with a blank text string
            IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_NULL_2, IDS_NULL_2, NULL, NULL);
            IMENUCTL_SetItemTime(pMe->m_pStdMenu, IDS_NULL_2, 600, 120);

            // Add lunch meeting item to menu
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_LUNCH, IDS_LUNCH, NULL, NULL);
            IMENUCTL_SetItemTime(pMe->m_pStdMenu, IDS_LUNCH, 720, 60);

			// We'll leave this item with a blank text string
            IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_NULL_3, IDS_NULL_3, NULL, NULL);
            IMENUCTL_SetItemTime(pMe->m_pStdMenu, IDS_NULL_3, 780, 120);

            // Add code review item to menu
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_CODE_REVIEW, IDS_CODE_REVIEW, NULL, NULL);
            IMENUCTL_SetItemTime(pMe->m_pStdMenu, IDS_CODE_REVIEW, 900, 60);

			// Add item to return to main menu
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_RETURN_TO_MAIN_MENU, 
				IDS_RETURN_TO_MAIN_MENU, NULL, NULL);
            IMENUCTL_SetItemTime(pMe->m_pStdMenu, IDS_RETURN_TO_MAIN_MENU, 960, 120);

            IMENUCTL_SetRect(pMe->m_pStdMenu, &pMe->m_rc);

			// Set the property that causes the menu to appear
			// in the format of a daily appointment calendar, and also
			// set the property that forces the current time's appointment to
			// be scrolled to the top of the screen
			IMENUCTL_SetProperties(pMe->m_pStdMenu, MP_CALENDAR | MP_AUTOSCROLLTIME);

			// Activate the standard menu 
            IMENUCTL_SetActive(pMe->m_pStdMenu, TRUE);
         }
         break;

      case USAGE_SETPROPS_ICON_TEXT_TOP:
         {
            // 14. Usage: IMENUCTL_SetProperties -- MP_ICON_TEXT_TOP

			// In icon view menus, the text string associated with the currently selected
			// menu item appears at the bottom of the screen.  When this property is set,
			// the text appears at the top of the screen instead.  To illustrate this 
			// behavior, we create an icon view menu and set the MP_ICON_TEXT_TOP property
			// for it.

 			CtlAddItem pai;

 			// Create the icon view menu
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_ICONVIEWCTL, 
				  (void **)&pMe->m_pIconMenu) != SUCCESS)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}

            // Add items to the icon view menu.  Since this menu contains a bitmap
			// for each item, we must use AddItemEx.

			// First, we populate the fields of the CtlAddItem structure that
			// will be the same for all items.

			// Since we're getting our item text and bitmaps from the
			// resource file, we set these pointers to NULL
            pai.pText  = NULL;        // Text 
            pai.pImage = NULL;        // Image 

			// Both text and image are contained in our app's
			// resource file
            pai.pszResImage = szResFile;   // Resource for Image
            pai.pszResText  = szResFile;   // Resource for Text

			// Use the default font for all item text
            pai.wFont = 0;         // 0 - Default

			// We don't specify any double-word data for our items
            pai.dwData = NULL;

			// Now specify the item-specific fields in the CtlAddItem structure

			// Specify values for first item and add them to the icon-view menu
            pai.wText   = IDS_ITEM_1; 
            pai.wImage  = IDB_ITEM_1_BMP;
            pai.wItemID = IDS_ITEM_1;

            IMENUCTL_AddItemEx(pMe->m_pIconMenu, &pai);

			// Specify values for second item and add them to the icon-view menu
            pai.wText   = IDS_ITEM_2; 
            pai.wImage  = IDB_ITEM_2_BMP;
            pai.wItemID = IDS_ITEM_2;

            IMENUCTL_AddItemEx(pMe->m_pIconMenu, &pai);

			// Specify values for third item and add them to the icon-view menu
            pai.wText   = IDS_ITEM_3; 
            pai.wImage  = IDB_ITEM_3_BMP;
            pai.wItemID = IDS_ITEM_3;

            IMENUCTL_AddItemEx(pMe->m_pIconMenu, &pai);

			// Specify values for last item and add them to the icon-view menu
            pai.wText   = IDS_RETURN_TO_MAIN_MENU; 
            pai.wImage  = IDB_RETURN_TO_MENU_BMP;
            pai.wItemID = IDS_RETURN_TO_MAIN_MENU;

            IMENUCTL_AddItemEx(pMe->m_pIconMenu, &pai);

            IMENUCTL_SetRect(pMe->m_pIconMenu, &pMe->m_rc);

			// Set the property that causes the text for the selected
			// item to appear at the top of the screen instead of
			// the bottom
			IMENUCTL_SetProperties(pMe->m_pIconMenu, MP_ICON_TEXT_TOP);

			// Activate the icon view menu we just created
            IMENUCTL_SetActive(pMe->m_pIconMenu, TRUE);
         }
         break;

      case USAGE_SETPROPS_ICON_TEXT_NOSEL:
         {
            // 15. Usage: IMENUCTL_SetProperties -- MP_ICON_TEXT_NOSEL

			// This property causes the text associated with an item in an
			// icon-view menu to appear normally, instead of in reverse video.
			// We illustrate the effect of this property by creating an icon-view
			// menu and setting the property (compare the appearance of this menu
			// with the icon-view menu in example 1).

			CtlAddItem pai;

 			// Create the icon view menu
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_ICONVIEWCTL, 
				  (void **)&pMe->m_pIconMenu) != SUCCESS)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}
			// Set the property that prevents the text for the selected
			// item from appearing in color
			IMENUCTL_SetProperties(pMe->m_pIconMenu, 0);

            // Add items to the icon view menu.  Since this menu contains a bitmap
			// for each item, we must use AddItemEx.

			// First, we populate the fields of the CtlAddItem structure that
			// will be the same for all items.

			// Since we're getting our item text and bitmaps from the
			// resource file, we set these pointers to NULL
            pai.pText  = NULL;        // Text 
            pai.pImage = NULL;        // Image 

			// Both text and image are contained in our app's
			// resource file
            pai.pszResImage = szResFile;   // Resource for Image
            pai.pszResText  = szResFile;   // Resource for Text

			// Use the default font for all item text
            pai.wFont = 0;         // 0 - Default

			// We don't specify any double-word data for our items
            pai.dwData = NULL;

			// Now specify the item-specific fields in the CtlAddItem structure

			// Specify values for first item and add them to the icon-view menu
            pai.wText   = IDS_ITEM_1; 
            pai.wImage  = IDB_ITEM_1_BMP;
            pai.wItemID = IDS_ITEM_1;

            IMENUCTL_AddItemEx(pMe->m_pIconMenu, &pai);

			// Specify values for second item and add them to the icon-view menu
            pai.wText   = IDS_ITEM_2; 
            pai.wImage  = IDB_ITEM_2_BMP;
            pai.wItemID = IDS_ITEM_2;

            IMENUCTL_AddItemEx(pMe->m_pIconMenu, &pai);

			// Specify values for third item and add them to the icon-view menu
            pai.wText   = IDS_ITEM_3; 
            pai.wImage  = IDB_ITEM_3_BMP;
            pai.wItemID = IDS_ITEM_3;

            IMENUCTL_AddItemEx(pMe->m_pIconMenu, &pai);

			// Specify values for last item and add them to the icon-view menu
            pai.wText   = IDS_RETURN_TO_MAIN_MENU; 
            pai.wImage  = IDB_RETURN_TO_MENU_BMP;
            pai.wItemID = IDS_RETURN_TO_MAIN_MENU;

            IMENUCTL_AddItemEx(pMe->m_pIconMenu, &pai);

			IMENUCTL_SetRect(pMe->m_pIconMenu, &pMe->m_rc);

			// Activate the icon view menu
            IMENUCTL_SetActive(pMe->m_pIconMenu, TRUE);
         }
         break;

      case USAGE_SETPROPS_ICON_SINGLE_FRAME:
         {
            // 16. Usage: IMENUCTL_SetProperties -- MP_ICON_SINGLE_FRAME

			// This property causes only the icon of the currently selected
			// menu item to be displayed for an icon-view menu (by default, 
			// multiple icons are displayed, and the frame of the selected item's
			// icon is highlighted).  Also, arrows are drawn on either side of the
			// item text string at the bottom of the screen.  In this example, we create
			// an icon-view menu and set the MP_ICON_SINGLE_FRAME property for it.
  			CtlAddItem pai;

 			// Create the icon view menu
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_ICONVIEWCTL, 
				  (void **)&pMe->m_pIconMenu) != SUCCESS)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}
			// Set the property that causes only the selected item in an 
			// icon-view menu to be displayed.
			// NOTE: You must set this property BEFORE you add items to
			// your menu, or BEFORE setting its rectangle with IMENUCTL_SetRect
			IMENUCTL_SetProperties(pMe->m_pIconMenu, MP_ICON_SINGLE_FRAME);

            // Add items to the icon view menu.  Since this menu contains a bitmap
			// for each item, we must use AddItemEx.

			// First, we populate the fields of the CtlAddItem structure that
			// will be the same for all items.

			// Since we're getting our item text and bitmaps from the
			// resource file, we set these pointers to NULL
            pai.pText  = NULL;        // Text 
            pai.pImage = NULL;        // Image 

			// Both text and image are contained in our app's
			// resource file
            pai.pszResImage = szResFile;   // Resource for Image
            pai.pszResText  = szResFile;   // Resource for Text

			// Use the default font for all item text
            pai.wFont = 0;         // 0 - Default

			// We don't specify any double-word data for our items
            pai.dwData = NULL;

			// Now specify the item-specific fields in the CtlAddItem structure

			// Specify values for first item and add them to the icon-view menu
            pai.wText   = IDS_ITEM_1; 
            pai.wImage  = IDB_ITEM_1_BMP;
            pai.wItemID = IDS_ITEM_1;

            IMENUCTL_AddItemEx(pMe->m_pIconMenu, &pai);

			// Specify values for second item and add them to the icon-view menu
            pai.wText   = IDS_ITEM_2; 
            pai.wImage  = IDB_ITEM_2_BMP;
            pai.wItemID = IDS_ITEM_2;

            IMENUCTL_AddItemEx(pMe->m_pIconMenu, &pai);

			// Specify values for third item and add them to the icon-view menu
            pai.wText   = IDS_ITEM_3; 
            pai.wImage  = IDB_ITEM_3_BMP;
            pai.wItemID = IDS_ITEM_3;

            IMENUCTL_AddItemEx(pMe->m_pIconMenu, &pai);

			// Specify values for last item and add them to the icon-view menu
            pai.wText   = IDS_RETURN_TO_MAIN_MENU; 
            pai.wImage  = IDB_RETURN_TO_MENU_BMP;
            pai.wItemID = IDS_RETURN_TO_MAIN_MENU;

            IMENUCTL_AddItemEx(pMe->m_pIconMenu, &pai);

            IMENUCTL_SetRect(pMe->m_pIconMenu, &pMe->m_rc);

			// Activate the icon view menu
            IMENUCTL_SetActive(pMe->m_pIconMenu, TRUE);
         }
         break;

      case USAGE_SETPROPS_UNDERLINE_TITLE:
         {
            // 17. Usage: IMENUCTL_SetProperties -- MP_UNDERLINE_TITLE

			// This property causes the title of a menu to be underlined.  We
			// illustrate its use by setting the property for the main usage
			// app menu, whose title will then appear underlined.

			IMENUCTL_SetProperties(pMe->m_pIMenu, MP_UNDERLINE_TITLE);

			// Set the main menu active
            IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);

            IMENUCTL_Redraw(pMe->m_pIMenu);

			// Clear the menu's properties--title underline will
			// disappear when you select another item from the main menu,
			// which will cause it to be redrawn
			IMENUCTL_SetProperties(pMe->m_pIMenu, 0);
         }
         break;

      case USAGE_GETPROPERTIES:
         {
            // 18. Usage: IMENUCTL_GetProperties

			// This example illustrates how to retrieve and test
			// the values of menu control properties.

			uint32 dwProps;

			// Set the main menu active
            IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);

			// Clear the screen
			DisplayEvent(pMe, USAGE_ERASE_SCREEN);

			// Retrieve the properties of the main app usage menu
			dwProps = IMENUCTL_GetProperties(pMe->m_pIMenu);

			// Initially, all properties of a control are unset--
			// let's verify this
			if (dwProps == 0)
			   DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 2, "No Props Set");

			// Now let's set a couple of properties.  Each property is set using
			// a bit mask.
			dwProps = dwProps | MP_NO_REDRAW | MP_CALENDAR;
            IMENUCTL_SetProperties(pMe->m_pIMenu, dwProps);

			// Retrieve the new properties variable and verify that the
			// desired properties are now set
            dwProps = IMENUCTL_GetProperties(pMe->m_pIMenu);

		    if (dwProps & MP_NO_REDRAW)
			   DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 3, "No Redraw Set");

		    if (dwProps & MP_CALENDAR)
			   DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 4, "Calendar Set");

			// Now let's turn off both properties (we could just set dwProps to
			// zero in this case, but the code below shows how to turn off just
			// one property without disturbing the others)
			dwProps = dwProps & ~MP_NO_REDRAW & ~MP_CALENDAR;
            IMENUCTL_SetProperties(pMe->m_pIMenu, dwProps);

			// Retrieve the new properties variable and verify that no
			// properties are now set
            dwProps = IMENUCTL_GetProperties(pMe->m_pIMenu);

			if (dwProps == 0)
			   DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 5, "No Props Set");
         }
         break;

      case USAGE_RESET:
         {
            // 19. Usage: IMENUCTL_Reset

			// Resetting a menu causes all of its items to be freed and its properties to be unset.
			// The resulting menu-control interface pointer can then be used to create a
			// completely different menu.  To illustrate its use, we create a menu, add items
			// to it, set some properties, and then reset it.  We next create a different menu using 
			// the same interface pointer and verify that none of the previous menu items or
			// properties remain.

  			// Create the standard menu that will be used
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, 
				  (void **)&pMe->m_pStdMenu) != SUCCESS)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}

            // Add items to standard menu
            IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_ENABLE_CMD, IDS_ENABLE_CMD, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_DISABLE_CMD, IDS_DISABLE_CMD, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_RETURN_TO_MAIN_MENU, 
				IDS_RETURN_TO_MAIN_MENU, NULL, NULL);

			// Add a title, and set a property (these should both get
			// erased when we reset the menu)
            IMENUCTL_SetTitle(pMe->m_pStdMenu, szResFile, IDS_MENU_TITLE, NULL);
            IMENUCTL_SetProperties(pMe->m_pStdMenu, MP_CALENDAR);

			// Now reset the menu interface pointer, which will clear all its
			// menu items, title, properties, etc.
			IMENUCTL_Reset(pMe->m_pStdMenu);

			// Now add a different set of items
            IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_ITEM_1, IDS_ITEM_1, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_ITEM_2, IDS_ITEM_2, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_ITEM_3, IDS_ITEM_3, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_RETURN_TO_MAIN_MENU, 
				IDS_RETURN_TO_MAIN_MENU, NULL, NULL);

            IMENUCTL_SetRect(pMe->m_pStdMenu, &pMe->m_rc);

			// Activate the standard menu. We should see the new menu items, 
			// with no title or calendar appearance
            IMENUCTL_SetActive(pMe->m_pStdMenu, TRUE);
         }
         break;

      case USAGE_SETTITLE:
         {
            // 20. Usage: IMENUCTL_SetTitle

			// This function is used to specify a title for a menu.  The title appears
			// in bold font, left-justified at the top of the menu's rectangle.
			// In this example, we create a standard menu and assign a title to it using
			// IMENUCTL_SetTitle (note that you can also set the menu's title by calling
			// its IMENUCTL_HandleEvent function with an EVT_CTL_SET_TITLE event).

 			// Create the standard menu
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, 
				  (void **)&pMe->m_pStdMenu) != SUCCESS)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}

            IMENUCTL_SetRect(pMe->m_pStdMenu, &pMe->m_rc);

			// Clear the screen
			DisplayEvent(pMe, USAGE_ERASE_SCREEN);

            // Add items to standard menu
            IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_ITEM_1, IDS_ITEM_1, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_ITEM_2, IDS_ITEM_2, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_ITEM_3, IDS_ITEM_3, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_RETURN_TO_MAIN_MENU, 
				IDS_RETURN_TO_MAIN_MENU, NULL, NULL);

			// Add a title to the menu (since we are reading the title
			// from the resource file, the last parameter is NULL--it could
			// contain a pointer to an AECHAR text string to be used for the title).
			// Note that IMENUCTL_SetTitle returns TRUE if the title was set successfully.
			if (IMENUCTL_SetTitle(pMe->m_pStdMenu, szResFile, IDS_MENU_TITLE, NULL))
                IMENUCTL_SetActive(pMe->m_pStdMenu, TRUE);
         }
         break;

      case USAGE_ADDITEM:
	  case USAGE_ADDITEMEX:
         {
            // 21. Usage: IMENUCTL_AddItem
            // 22. Usage: IMENUCTL_AddItemEx

			// These functions are used to add items to menus.  You use IMENUCTL_AddItem
			// for simple items that are displayed only with normal-font text.  
			// IMENUCTL_AddItemEx is used for menu items that contain bitmaps, or when you 
			// want to specify a non-default text font for the item.
			// In this example, we create a menu and add items to it using both functions.

			CtlAddItem pai;

			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, 
				  (void **)&pMe->m_pStdMenu) != SUCCESS)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}

            // Add items to standard menu

			// For the first item, we use AddItemEx, since we'd like the item
			// text to appear in bold font and contain a bitmap image as well as text.
			// To use IMENUCTL_AddItemEx,we must populate a CtlAddItem structure and pass
			// it as a parameter to IMENUCTL_AddItemEx.

			// Since we're getting our item text and bitmaps from the
			// resource file, we set these pointers to NULL
            pai.pText  = NULL;        // Text 
            pai.pImage = NULL;        // Image 

			// Both text and image are contained in our app's
			// resource file
            pai.pszResImage = szResFile;   // Resource for Image
            pai.pszResText  = szResFile;   // Resource for Text

			// Use bold font for the item text
            pai.wFont = AEE_FONT_BOLD;        

			// We don't specify any double-word data for our items
            pai.dwData = NULL;

			// Now specify the item-specific fields in the CtlAddItem structure
            pai.wText   = IDS_ITEM_1; 
            pai.wImage  = IDB_ITEM_1_BMP;
            pai.wItemID = IDS_ITEM_1;

			// Add the item to the menu--IMENUCTL_AddItemEx returns FALSE
			// if the item was not added successfully, and TRUE otherwise
            if (IMENUCTL_AddItemEx(pMe->m_pStdMenu, &pai) == FALSE)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}

			// For the remaining items, we don't want anything special, so we
			// can get by with the simpler AddItem function.  We read the item
			// text from a resource file (second and third parameters), so the
			// fifth parameter is NULL.  For the item ID (wParam in HandleEvent 
			// call for the EVT_COMMAND when item is selected), we use the item's
			// string identifier.  The last parameter (dwParam in HandleEvent call)
			// is set to NULL, since we don't supply a double-word data value with the item.
			if (IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_ITEM_2, 
				IDS_ITEM_2, NULL, NULL) == FALSE)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}
			if (IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_ITEM_3, 
				IDS_ITEM_3, NULL, NULL) == FALSE)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}
			if (IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_RETURN_TO_MAIN_MENU, 
				IDS_RETURN_TO_MAIN_MENU, NULL, NULL) == FALSE)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}

            IMENUCTL_SetRect(pMe->m_pStdMenu, &pMe->m_rc);

			// Add a title to the menu (since we are reading the title
			// from the resource file, the last parameter is NULL--it could
			// contain a pointer to an AECHAR text string to be used for the title)
			IMENUCTL_SetTitle(pMe->m_pStdMenu, szResFile, IDS_MENU_TITLE, NULL);

			// Activate the standard menu
            IMENUCTL_SetActive(pMe->m_pStdMenu, TRUE);
         }
         break;

      case USAGE_GETITEMDATA:
         {
            // 23. Usage: IMENUCTL_GetItemData

			// Each menu item can have a double-word data value associated with it
			// whose value is specified when the menu item is created.  To illustrate this
			// function, we add an item with a double-word data value to the main usage
			// app menu and then use IMENUCTL_GetItemData to retrieve the value of the data.

			uint32 dwItemData;
			char szBuf[25];

			// Add an item to the main menu, specifying a value for the item's
			// double-word data (last parameter of the AddItem function)
            IMENUCTL_AddItem(pMe->m_pIMenu, szResFile, IDS_ITEM_1, IDS_ITEM_1, NULL, 123456);

			// Set the main menu active
            IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);

			// Now retrieve the data item value.  If IMENUCTL_GetItemData fails
			// to get the data value, it returns FALSE, otherwise TRUE.
            if (IMENUCTL_GetItemData(pMe->m_pIMenu, IDS_ITEM_1, &dwItemData) == FALSE)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}

			// Echo the item value back to the screen

			// Clear the screen
			DisplayEvent(pMe, USAGE_ERASE_SCREEN);

		    SPRINTF(szBuf, "Item value: %d", dwItemData); 
            DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 2, szBuf);
			DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 3, "Press Up/Down");
			DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 4, "to continue");

			// Delete the menu item to restore the main menu to
			// its original state
			IMENUCTL_DeleteItem(pMe->m_pIMenu, IDS_ITEM_1);
         }
         break;

      case USAGE_DELETEITEM:
         {
            // 24. Usage: IMENUCTL_DeleteItem

			// This function deletes a previously added item from a menu.  
			// In the example below, we illustrate this function by creating
			// a menu with four items, deleting one of them, and then 
			// verifying that the deleted item no longer appears when we
			// display the menu.

   			// Create the standard menu that will be displayed
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, 
				  (void **)&pMe->m_pStdMenu) != SUCCESS)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}

            // Add items to standard menu
            IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_ITEM_1, IDS_ITEM_1, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_ITEM_2, IDS_ITEM_2, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_ITEM_3, IDS_ITEM_3, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_RETURN_TO_MAIN_MENU, 
				IDS_RETURN_TO_MAIN_MENU, NULL, NULL);

            IMENUCTL_SetRect(pMe->m_pStdMenu, &pMe->m_rc);

			// Now delete one of the items we just added.  The iten to 
			// be deleted is designated by its ID (i.e., the value returned
			// when the user selects the item).  If IMENUCTL_DeleteItem fails to
			// delete the item, it returns FALSE, otherwise TRUE.
            if (IMENUCTL_DeleteItem(pMe->m_pStdMenu, IDS_ITEM_2) == FALSE)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}

			// Activate the standard menu we just created. 
            IMENUCTL_SetActive(pMe->m_pStdMenu, TRUE);
         }
         break;

      case USAGE_DELETEALL:
         {
            // 25. Usage: IMENUCTL_DeleteAll

			// This function deletes all the items from a menu.  
			// In the example below, we illustrate this function by creating
			// a menu with four items, deleting all of them, and then 
			// adding back one item.  We can verify when the menu is displayed
			// that only the added-back item appears.

    		// Create the standard menu that will be displayed
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, 
				  (void **)&pMe->m_pStdMenu) != SUCCESS)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}

            // Add items to standard menu
            IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_ITEM_1, IDS_ITEM_1, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_ITEM_2, IDS_ITEM_2, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_ITEM_3, IDS_ITEM_3, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_RETURN_TO_MAIN_MENU, 
				IDS_RETURN_TO_MAIN_MENU, NULL, NULL);

            IMENUCTL_SetRect(pMe->m_pStdMenu, &pMe->m_rc);

			// Now delete all of the items we just added.
			// IMENUCTL_DeleteAll returns FALSE if it fails to do so,
			// and TRUE otherwise.
            if (IMENUCTL_DeleteAll(pMe->m_pStdMenu) == FALSE)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}

			// Add back one item--this should be the only one that appears
			// when we display the menu
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_RETURN_TO_MAIN_MENU, 
				IDS_RETURN_TO_MAIN_MENU, NULL, NULL);

			// Activate the standard menu we just created. 
            IMENUCTL_SetActive(pMe->m_pStdMenu, TRUE);
         }
         break;

      case USAGE_SETSEL:
         {
            // 26. Usage: IMENUCTL_SetSel

			// This function sets the identity of the currently selected menu item and
			// causes it to be displayed at the top of the screen.  As an example of this
			// function, we set the currently selected item in the main usage app menu 
			// to be the the next menu item in the list (number 27, the GetSel usage example).
            IMENUCTL_SetSel(pMe->m_pIMenu, USAGE_GETSEL);

			// Set the main menu active
            IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);
				IMENUCTL_Redraw(pMe->m_pIMenu);
         }
         break;

      case USAGE_GETSEL:
         {
            // 27. Usage: IMENUCTL_GetSel

			// This function retrieves the identity of the currently selected menu item.  
			// As an example of this function, we use it to obtain the currently selected 
			// item in the main usage app menu and echo it on the screen.

			// Set the main menu active
            IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);

 			// Clear the screen
            DisplayEvent(pMe, USAGE_ERASE_SCREEN);

			// Access the currently selected item for this menu
			if (IMENUCTL_GetSel(pMe->m_pIMenu) == USAGE_GETSEL)
			{
				DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 2, "USAGE_GETSEL");
                DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 3, "is selected");
                DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 4, "Press Up/Down");
                DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 5, "to continue");
			}

         }
         break;

      case USAGE_ENABLECOMMAND:
         {
            // 28. Usage: IMENUCTL_EnableCommand

			// This function is used to enable and disable the sending of EVT_COMMAND
			// events when the user presses the SELECT key.  By default, command sending
			// is enabled for menu controls.
			// To illustrate this function, we create a menu with two items: one that returns
			// you to the main usage app menu, and another that disables command sending.  You
			// can verify that once you select the Disable item, the Return to Main Menu item 
			// no longer works, since the EVT_COMMAND is not sent to our app any more
			// (you can return to the main menu by pressing the LEFT or RIGHT keys).

 			// Create the standard menu that will be used
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, 
				  (void **)&pMe->m_pStdMenu) != SUCCESS)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}

            // Add items to standard menu
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_DISABLE_CMD, IDS_DISABLE_CMD, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_RETURN_TO_MAIN_MENU, 
				IDS_RETURN_TO_MAIN_MENU, NULL, NULL);

            IMENUCTL_SetRect(pMe->m_pStdMenu, &pMe->m_rc);

			// De-activate the main menu so it won't receive any keys while command
		    // sending is disabled 
            IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);

 			// Clear the screen
            DisplayEvent(pMe, USAGE_ERASE_SCREEN);

			// Activate the standard menu
            IMENUCTL_SetActive(pMe->m_pStdMenu, TRUE);

			// Remind user how to get back to main menu
			// once command sending is disabled
         DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 5, "When disabled, press L/R Keys to return to Main Menu");

			// NOTE: See the HandleEvent function above for the actual call to
			// IMENUCTL_EnableCommand (EVT_COMMAND, wParam == IDS_DISABLE_CMD).
			// Note that once command sending is disabled, it cannot be re-enabled
			// from this menu (since this menu no longer accepts commands!!)--you must
			// exit the menu with the left or right arrow keys.
         }
         break;

      case USAGE_SETITEMTEXT:
         {
            // 29. Usage: IMENUCTL_SetItemText

			// This function allows you to change the text associated with a menu item.
			// As an example of how to use the function, we create a menu that can be used
			// to change the text string in the main usage app menu for this example, and then
			// change it back to its original value.

  			// Create the standard menu that will be used
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, 
				  (void **)&pMe->m_pStdMenu) != SUCCESS)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}

            // Add items to standard menu.  Selecting the "New Item Text" item will change
		    // the text in the main menu for this item, and selecting the "Orig Item Text" 
			// changes it back (see the HandleEvent function above for the actual calls
			// to IMENUCTL_SetItemText)
            IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_NEW_ITEM_TEXT, IDS_NEW_ITEM_TEXT, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_ORIG_ITEM_TEXT, IDS_ORIG_ITEM_TEXT, NULL, NULL);

			// Activate the standard menu
            IMENUCTL_SetActive(pMe->m_pStdMenu, TRUE);
         }
         break;

      case USAGE_SETITEMTIME:
      case USAGE_GETITEMTIME:
         {
            // 30. Usage: IMENUCTL_SetItemTime
            // 31. Usage: IMENUCTL_GetItemTime

			int16 nStartTime;
			uint16 nDuration;

  			// Create the standard menu that will be displayed as a calendar
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, 
				  (void **)&pMe->m_pStdMenu) != SUCCESS)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}

            // Add items to standard menu and set their times
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_WORK_OUT, IDS_WORK_OUT, NULL, NULL);
            IMENUCTL_SetItemTime(pMe->m_pStdMenu, IDS_WORK_OUT, 360, 60);

			// We'll leave this item with a blank text string
            IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_NULL_1, IDS_NULL_1, NULL, NULL);
            IMENUCTL_SetItemTime(pMe->m_pStdMenu, IDS_NULL_1, 420, 120);

            // Add status meeting item to menu
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_STATUS_MEETING, IDS_STATUS_MEETING, NULL, NULL);
            IMENUCTL_SetItemTime(pMe->m_pStdMenu, IDS_STATUS_MEETING, 540, 60);

			// We'll leave this item with a blank text string
            IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_NULL_2, IDS_NULL_2, NULL, NULL);
            IMENUCTL_SetItemTime(pMe->m_pStdMenu, IDS_NULL_2, 600, 120);

            // Add lunch meeting item to menu
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_LUNCH, IDS_LUNCH, NULL, NULL);
            IMENUCTL_SetItemTime(pMe->m_pStdMenu, IDS_LUNCH, 720, 60);

			// We'll leave this item with a blank text string
            IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_NULL_3, IDS_NULL_3, NULL, NULL);
            IMENUCTL_SetItemTime(pMe->m_pStdMenu, IDS_NULL_3, 780, 120);

            // Add code review item to menu
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_CODE_REVIEW, IDS_CODE_REVIEW, NULL, NULL);
            IMENUCTL_SetItemTime(pMe->m_pStdMenu, IDS_CODE_REVIEW, 900, 60);

			// Add item to return to main menu
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_RETURN_TO_MAIN_MENU, 
				IDS_RETURN_TO_MAIN_MENU, NULL, NULL);
            IMENUCTL_SetItemTime(pMe->m_pStdMenu, IDS_RETURN_TO_MAIN_MENU, 960, 120);

			IMENUCTL_SetRect(pMe->m_pStdMenu, &pMe->m_rc);

			// Set the property that causes the menu to appear
			// in the format of a daily appointment calendar (the item
			// time and duration are relevant only for calendar-view menus)
			IMENUCTL_SetProperties(pMe->m_pStdMenu, MP_CALENDAR);

			// Here is an example of how we can use IMENUCTL_GetItemTime to
			// retrieve a time value we set for an item.  This function returns
			// the item start time and copies the address of the item duration
			// into its third parameter.
            if ((nStartTime = IMENUCTL_GetItemTime(pMe->m_pStdMenu, 
				    IDS_CODE_REVIEW, &nDuration)) == -1)
				DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 2, "GetItemTime failed");

			// Activate the calendar view menu
            IMENUCTL_SetActive(pMe->m_pStdMenu, TRUE);
         }
         break;

      case USAGE_SETSTYLE:
         {
            // 32. Usage: IMENUCTL_SetStyle

			// This function is used to select special styles for the
			// currently select menu item, and for non-selected items.
			// These are specified by populating two instances of the
			// AEEItemStyle structure and passing them as parameters to
			// IMENUCTL_SetStyle.
			// To illustrate how to use this function, we create a menu and 
			// use IMENUCTL_SetStyle to modify the appearance of the menu items,
			// including increasing the amount of padding aound the item text
			// and using a different type of frame to surround each item.

			AEEItemStyle si, ni;

 			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, 
				  (void **)&pMe->m_pStdMenu) != SUCCESS)
			{
				IMenuCtl_Cleanup(pMe);
				return;
			}

            // Add items to standard menu
            IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_ITEM_1, IDS_ITEM_1, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_ITEM_2, IDS_ITEM_2, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_ITEM_3, IDS_ITEM_3, NULL, NULL);
			IMENUCTL_AddItem(pMe->m_pStdMenu, szResFile, IDS_RETURN_TO_MAIN_MENU, 
				IDS_RETURN_TO_MAIN_MENU, NULL, NULL);

			// Set the item style for the selected menu item

            // Frame type for the item
            si.ft = AEE_FT_TAB_TOP_SEL;       
			
            // X padding inside item (does not include frame size)
            si.xOffset = 8;   
			
            // Y padding inside item (does not include frame size)
            si.yOffset = 8;  
			
            // Raster operation for drawing images inside the item
            si.roImage = AEE_RO_COPY;    

			// Set the item style for normal menu items

            // Frame type for the item
            ni.ft = AEE_FT_TAB_TOP;       
			
            // X padding inside item (does not include frame size)
            ni.xOffset = 8;   
			
            // Y padding inside item (does not include frame size)
            ni.yOffset = 8;  
			
            // Raster operation for drawing images inside the item
            ni.roImage = AEE_RO_COPY; 

			IMENUCTL_SetRect(pMe->m_pStdMenu, &pMe->m_rc);

			// Now set the item styles we just created
			IMENUCTL_SetStyle(pMe->m_pStdMenu, &ni, &si);

			// Activate the standard menu we just created. 
            IMENUCTL_SetActive(pMe->m_pStdMenu, TRUE);
         }
         break;

      default:

		IMenuCtl_Cleanup(pMe);
		return;
   }
            
   // Display above event. 
   DisplayEvent (pMe, wParam);

   return;
}

/*===========================================================================

FUNCTION: DisplayEvent

DESCRIPTION
   This function prints a heading at the top 
   of the screen to indicate which usage example was selected by the user, 
   It also prints a message on the bottom two lines of the screen instructing 
   the user to press the UP or DOWN key to return to
   the main menu to execute another example.
    
PROTOTYPE:
   static void DisplayEvent(CIHeapApp * pMe, uint16 wParam)

PARAMETERS:
   pMe:   [in]: Contains a pointer to the usage app data structure.
   wParam: [in]: Identifier of the selected example (example IDs are specified when the
                 main usage app menu is created).  If the value USAGE_ERASE_SCREEN is supplied
				 instead, this function clears the screen.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void DisplayEvent (CIMenuCtlApp *pMe, uint16 wParam)
{
   AEERect qrc;
   AEEDeviceInfo di; // Device Info
   AECHAR * szBuf;
   AEERect rc; 

   // Make sure the pointers we'll be using are valid
   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
	   return;

   if ((szBuf = (AECHAR *) MALLOC(TEXT_BUFFER_SIZE)) == NULL)
	   return;

   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);
	qrc.x = 0;
	qrc.y = 0;
	qrc.dx = di.cxScreen;
	qrc.dy = di.cyScreen;
   rc.x = 5;
   rc.y = 5;
   rc.dx = di.cxScreen - 5;
   rc.dy = di.cyScreen - 5;

   switch (wParam)
   {
      case USAGE_ERASE_SCREEN:
         IDISPLAY_EraseRect(pMe->a.m_pIDisplay,&qrc);  //Clear the display
		 FREE(szBuf);
         return;
	  case USAGE_BASIC_USAGE:
         STR_TO_WSTR("Basic Usage", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_REDRAW:
			FREE(szBuf);
			return;
      case USAGE_SETACTIVE:
         STR_TO_WSTR("SetActive", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_ISACTIVE:
         STR_TO_WSTR("IsActive", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_SETRECT:
         STR_TO_WSTR("SetRect", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_GETRECT:
         STR_TO_WSTR("GetRect", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_SETPROPS_WRAPSCROLL:
         STR_TO_WSTR("SetP WrapScroll", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_SETPROPS_NO_ARROWS:
         STR_TO_WSTR("SetP NoArrows", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_SETPROPS_NO_REDRAW:
         STR_TO_WSTR("SetP NoRedraw", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_SETPROPS_MAXSOFTKEYITEMS:
         STR_TO_WSTR("SetP MaxSKItems", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_SETPROPS_CALENDAR:
         STR_TO_WSTR("SetP Calendar", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_SETPROPS_AUTOSCROLLTIME:
         STR_TO_WSTR("SetP AutoScrTime", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_SETPROPS_ICON_TEXT_TOP:
         STR_TO_WSTR("SetP IconTextTop", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_SETPROPS_ICON_TEXT_NOSEL:
         STR_TO_WSTR("SetP IconTxtNoSel", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_SETPROPS_ICON_SINGLE_FRAME:
         STR_TO_WSTR("SetP IconSnglFrm", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_GETPROPERTIES:
         STR_TO_WSTR("GetProperties", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_RESET:
         STR_TO_WSTR("Reset", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_SETTITLE:
         STR_TO_WSTR("SetTitle", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_ADDITEM:
         STR_TO_WSTR("AddItem", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_ADDITEMEX:
         STR_TO_WSTR("AddItemEx", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_GETITEMDATA:
         STR_TO_WSTR("GetItemData", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_DELETEITEM:
         STR_TO_WSTR("DeleteItem", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_DELETEALL:
         STR_TO_WSTR("DeleteAll", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_GETSEL:
         STR_TO_WSTR("GetSel", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_ENABLECOMMAND:
         STR_TO_WSTR("EnableCommand", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_SETITEMTIME:
         STR_TO_WSTR("SetItemTime", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_GETITEMTIME:
         STR_TO_WSTR("GetItemTime", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_SETSTYLE:
         STR_TO_WSTR("SetStyle", szBuf, TEXT_BUFFER_SIZE);
         break; 
	  // For the following four examples, headings interfere
	  // with other activity on the screen, so just exit without
	  // printing anything
      case USAGE_HANDLEEVENT:
      case USAGE_SETPROPS_UNDERLINE_TITLE:
	  case USAGE_SETSEL:
	  case USAGE_SETITEMTEXT:
		 FREE(szBuf);
         return;
      default:
         STR_TO_WSTR("Unsupported fn", szBuf, sizeof(szBuf));
   }

   IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,10, 5, &rc,
	  IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
   IDISPLAY_UpdateEx(pMe->a.m_pIDisplay, TRUE);
   FREE(szBuf);
}
/*===========================================================================

FUNCTION: BuildMainMenu

DESCRIPTION
    This function constructs the menu control that allows the user to
	select a usage example to execute.  The command ID for each menu item
	is passed to the usage example function above when the user selects the
	menu item for that example.
    
PROTOTYPE:
   static void BuildMainMenu(CIHeapApp * pMe)

PARAMETERS:
   pMe:   [in]: Contains a pointer to the usage app data structure.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void BuildMainMenu(CIMenuCtlApp *pMe)
{
	AEERect qrc;
	AEEDeviceInfo di;
    AECHAR * szBuf;

	// Make sure the pointers we'll be using are valid
    if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->m_pIMenu == NULL)
	   return;

   if ((szBuf = (AECHAR *) MALLOC(TEXT_BUFFER_SIZE)) == NULL)
	   return;

	// Set Title
    STR_TO_WSTR("IMenuCtl Examples", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_SetTitle(pMe->m_pIMenu, NULL, 0, szBuf);

	//Set the Rectangle for the Menu
	ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);
	qrc.x = 0;
	qrc.y = 0;
	qrc.dx = di.cxScreen;
	qrc.dy = di.cyScreen;
	IMENUCTL_SetRect(pMe->m_pIMenu, &qrc);  

	//Add individual entries to the Menu

   // Add Basic Usage to menu
   STR_TO_WSTR("1. Basic Usage", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_BASIC_USAGE, szBuf, 0);

   // Add HandleEvent to menu
   STR_TO_WSTR("2. HandleEvent", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_HANDLEEVENT, szBuf, 0);

   // Add Redraw to menu
   STR_TO_WSTR("3. Redraw", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_REDRAW, szBuf, 0);

   // Add SetActive to menu
   STR_TO_WSTR("4. SetActive", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETACTIVE, szBuf, 0);

   // Add IsActive to menu
   STR_TO_WSTR("5. IsActive", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_ISACTIVE, szBuf, 0);

   // Add SetRect to menu
   STR_TO_WSTR("6. SetRect", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETRECT, szBuf, 0);

   // Add GetRect to menu
   STR_TO_WSTR("7. GetRect", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETRECT, szBuf, 0);

   // Add SetP WrapScroll to menu
   STR_TO_WSTR("8. SetP WrapScroll", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPROPS_WRAPSCROLL, szBuf, 0);

   // Add SetP NoArrows to menu
   STR_TO_WSTR("9. SetP NoArrows", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPROPS_NO_ARROWS, szBuf, 0);

   // Add SetP NoRedraw to menu
   STR_TO_WSTR("10. SetP NoRedraw", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPROPS_NO_REDRAW, szBuf, 0);

   // Add SetP MaxSKItems to menu
   STR_TO_WSTR("11. SetP MaxSKItems", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPROPS_MAXSOFTKEYITEMS, szBuf, 0);

   // Add SetP Calendar to menu
   STR_TO_WSTR("12. SetP Calendar", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPROPS_CALENDAR, szBuf, 0);

   // Add SetP AutoScrTime to menu
   STR_TO_WSTR("13. SetP AutoScrTime", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPROPS_AUTOSCROLLTIME, szBuf, 0);

   // Add SetP IconTextTop to menu
   STR_TO_WSTR("14. SetP IconTextTop", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPROPS_ICON_TEXT_TOP, szBuf, 0);

   // Add SetP IconTxtNoSel to menu
   STR_TO_WSTR("15. SetP IconTxtNoSel", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPROPS_ICON_TEXT_NOSEL, szBuf, 0);

   // Add SetP IconSnglFrm to menu
   STR_TO_WSTR("16. SetP IconSnglFrm", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPROPS_ICON_SINGLE_FRAME, szBuf, 0);

   // Add SetP ULTitle to menu
   STR_TO_WSTR("17. SetP ULTitle", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPROPS_UNDERLINE_TITLE, szBuf, 0);

   // Add GetProperties to menu
   STR_TO_WSTR("18. GetProperties", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETPROPERTIES, szBuf, 0);

   // Add Reset to menu
   STR_TO_WSTR("19. Reset", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_RESET, szBuf, 0);

   // Add SetTitle to menu
   STR_TO_WSTR("20. SetTitle", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETTITLE, szBuf, 0);

   // Add AddItem to menu
   STR_TO_WSTR("21. AddItem", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_ADDITEM, szBuf, 0);

   // Add AddItemEx to menu
   STR_TO_WSTR("22. AddItemEx", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_ADDITEMEX, szBuf, 0);

   // Add GetItemData to menu
   STR_TO_WSTR("23. GetItemData", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETITEMDATA, szBuf, 0);

   // Add DeleteItem to menu
   STR_TO_WSTR("24. DeleteItem", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DELETEITEM, szBuf, 0);

   // Add DeleteAll to menu
   STR_TO_WSTR("25. DeleteAll", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_DELETEALL, szBuf, 0);

   // Add SetSel to menu
   STR_TO_WSTR("26. SetSel", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETSEL, szBuf, 0);

   // Add GetSel to menu
   STR_TO_WSTR("27. GetSel", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETSEL, szBuf, 0);

   // Add EnableCommand to menu
   STR_TO_WSTR("28. EnableCommand", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_ENABLECOMMAND, szBuf, 0);

   // Add SetItemText to menu
   STR_TO_WSTR("29. SetItemText", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETITEMTEXT, szBuf, 0);

   // Add SetItemTime to menu
   STR_TO_WSTR("30. SetItemTime", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETITEMTIME, szBuf, 0);

   // Add GetItemTime to menu
   STR_TO_WSTR("31. GetItemTime", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETITEMTIME, szBuf, 0);

   // Add SetStyle to menu
   STR_TO_WSTR("32. SetStyle", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETSTYLE, szBuf, 0);

   // Activate menu
   IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);

   FREE(szBuf);
}
/*===========================================================================

FUNCTION: DisplayOutput

DESCRIPTION
    This function displays an output string at a given line number on the
    screen. If the nline parameter is a negative value (-1) the string
    is displayed in the middle of the screen. If the "nline" value is larger
    than or equal to zero the "nline" value is multiplied by 15 and the 
    resulting value in pixels is set to the y-coordinate of the start of 
    the string display on the screen. If the string does not fit on one line
    the string wraps around to the next line (spaced rougly 10-15 pixels apart).
    By default 5 is used as the starting the x-coordinate of a displayed 
    string.

    How many characters that fit on one line is calculated for each line 
    that is wrapped around to the next line.

    Note: depending on the phone screen size and the fonts used for characters 
          the output might differ on different handsets (devices). Where some 
          handsets will have a smaller screen and large default fonts which will 
          cause partial overlapping of lines. This function does not try to address
          these issues (this is meant as a simple display function).
    
PROTOTYPE:
   void DisplayOutput(IShell *pIShell, IDisplay *pDisplay, int nline, char *pszStr)

PARAMETERS:
   pIShell:   [in]: Contains a pointer to the IShell interface.
   pIDisplay: [in]: Contains a pointer to the IDisplay interface.
   nline:     [in]: Contains the line number to start displaying the text. The line
        numbers are by default spaced 15 pixels apart along the y-axis.
   pszStr:    [in]: The character string to be displayed on the screen.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void DisplayOutput(IShell *pIShell, IDisplay *pIDisplay, int nline, char *pszStr)
{
   AEEDeviceInfo di; // Device Info
   AECHAR * szBuf;      
   AECHAR * psz = NULL;
   int charHeight = 0;      // Stores the char height in pixels for given font
   int pnAscent = 0;        // Stores the ascent in number of pixels
   int pnDescent = 0;       // Stores the descent in number of pixels
   int pixelWidth;
   AEEFont font = AEE_FONT_NORMAL;
   int pnFits = 0, dy;
   int totalCh = 0;

   // Make sure the pointers we'll be using are valid
   if (pIShell == NULL || pIDisplay == NULL)
	   return;

   if ((szBuf = (AECHAR *) MALLOC(TEXT_BUFFER_SIZE)) == NULL)
	   return;

   // Get device information
   ISHELL_GetDeviceInfo(pIShell,&di);

   // Get the font metrics info
   charHeight = IDISPLAY_GetFontMetrics (pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   // Convert to wide string (unicode)
   STR_TO_WSTR ((char *)pszStr, szBuf, TEXT_BUFFER_SIZE);

   // If nlines is zero then print this string starting around the middle of 
   // the screen. Or else multiply nlines by charheight to decide the y coordinate of
   // the start of the string.
   if (nline < 0) {
      dy = di.cyScreen*2/5;
   }
   else{
      dy = nline * charHeight;
   }

   // psz keeps track of the point from which to write from the string buffer
   // in case the string does not fit one line and needs to wrap around in the
   // next line.
   psz = szBuf;
      
   // Need to calculate the lotal string length to decide if any wrapping
   // around is needed.
   totalCh = STRLEN ((char *)pszStr);

   // Keep displaying text string on multiple lines if the string can't be displayed
   // on one single line. Lines are spaced 15 pixels apart.
   while ((totalCh > 0) && (*psz != NULL))
   { 
      // Get information on how many characters will fit in a line.
      // Give the pointer to the buffer to be displayed, and the number of
      // pixels along the x axis you want to display the string in (max number)
      // pnFits will have the max number of chars that will fit in the maxWidth
      // number of pixels (given string can't fit in one line), or the number of 
      // chars in the string (if it does fit in one line). pnWidth gives the
      // number of pixels that will be used to display pnFits number of chars.
      pixelWidth = IDISPLAY_MeasureTextEx(pIDisplay,
                      font, 
                      (AECHAR *) psz,  // Start of the buffer to display,
                      -1,
                      di.cxScreen - 5, // maxWidth
                      &pnFits);         // Number of chars that will fit a line

      // If pnFits is zero there is something wrong in the input to above function. 
      // Normally this scenario should not occur. But, have the check anyway.
      if (pnFits == 0)
         return;

      IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, psz, pnFits, 5 /*start dx*/, 
         dy, 0 /* use default rectangle coordinates */, 0);

      psz += pnFits;      // move pointer to the next segment to be displayed
      totalCh -= pnFits;  // reduce the total number of characters to still display
      dy += charHeight;   // Place next line charHeight pixels below the 
                          // previous line.
      IDISPLAY_Update(pIDisplay); //, TRUE);
      if (totalCh < pnFits)
         pnFits = totalCh;  // if total number is less than pnFits, adjust pnFits
   }

   FREE(szBuf);
   return;   
} // End of DisplayOutput
