/*===========================================================================

FILE: inetsocketusage.c

SERVICES: Sample applet using AEE

DESCRIPTION
  This file contains usage examples of INETMGR and ISOCKET interface. 
  INETMGR and ISOCKET interface functions allow the user to 
  create/operate on INetMgr and ISocket interfaceobjects. INetMgr and
  ISocket objects are used to invoke network connectivity related 
  functionality.

PUBLIC CLASSES:  
   N/A


   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/

/*===============================================================================
                     INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "AEEModGen.h"
#include "AEEAppGen.h"
#include "AEEShell.h"
#include "AEEMenu.h"
#include "AEEStdLib.h"
#include "AEEUsageAppIDs.h"
#include "AEENet.h"

/*===========================================================================

                      PUBLIC DATA DECLARATIONS

===========================================================================*/

/*-------------------------------------------------------------------
            Type Declarations
-------------------------------------------------------------------*/
// Net/Socket struct. This is the main struct for this applet. This will hold 
// all the data members that needs to be remembered throughout the life of
// the applet. 
// THE FIRST DATA MEMBER OF THIS STRUCT MUST BE AN AEEApplet OBJECT. 
// This is critical requirement which is related to an applet being
// dynamically loaded. The app developers should add their data members
// following the AEEApplet data member.
typedef struct _CINetSktApp
{
	AEEApplet	a;          // Mandatory first AEEApplet data member
   IMenuCtl *  m_pIMenu;
   ISocket *   m_pISocket;
   INetMgr *   m_pINetMgr;
		
   AEEDNSResult dnsresult;
   AEECallback  cbkLookup;
   INAddr       addrsLookup[8];

   uint16      APIFnType;    // Storage for API function type selected
                             // by user.
}CINetSktApp;

/*-------------------------------------------------------------------
            Function Prototypes
-------------------------------------------------------------------*/
// App Handle Event function
static boolean NetSktApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);
void GetHostByNameCB(void * pUser);

// App specific data alloc-init/free functions
static boolean INetSkt_InitAppData(IApplet* pMe);
static void INetSkt_FreeAppData(IApplet* pMe);

// Main usage function
boolean INetSocketUsage (CINetSktApp * pMe, uint16 wParam);

// Display related helper functions
void DisplayEvent (CINetSktApp *pMe, uint16 wParam);
void DisplayOutput(CINetSktApp *pMe, int nline, char *pszStr);

// IP Addredd convertion functions
INAddr ConvertToINAddr(char *psz);
void ConvertToIPAddr(INAddr inaddr, char *psz);
static void NetApp_DNSResult(void *pv);

// Misc functions
void BuildMainMenu(CINetSktApp *pMe);
void ReleaseNetAndSocket (CINetSktApp * pMe);

// Call-back related functions
void CommonConnectCB(void * pUser, int iError);
void CommonWriteCB (void *pUser);
void CommonReadCB (void *pUser);


/*-------------------------------------------------------------------
            Global Constant Definitions
-------------------------------------------------------------------*/
// App specific constants
#define USAGE_BASIC_USAGE           100
#define USAGE_NET_GETHOSTBYNAME     101
#define USAGE_NET_GETLASTERROR      102
#define USAGE_NET_OPENSOCKET        103
#define USAGE_NET_SETLINGER         104
#define USAGE_NET_NETSTATUS         105
#define USAGE_NET_GETMYIPADDRESS    106
#define USAGE_SOCKET_GETPEERNAME    107
#define USAGE_SOCKET_GETLASTERROR   108
#define USAGE_SOCKET_CONNECT        109
#define USAGE_SOCKET_BIND           110
#define USAGE_SOCKET_WRITE          111
#define USAGE_SOCKET_WRITEV         112
#define USAGE_SOCKET_READ           113
#define USAGE_SOCKET_READV          114
#define USAGE_SOCKET_SENDTO         115
#define USAGE_SOCKET_RECVFROM       116
#define USAGE_SOCKET_READBALE       117
#define USAGE_SOCKET_WRITEABLE      118
#define USAGE_SOCKET_CANCEL         119
#define USAGE_SOCKET_SHUTDOWN       120
#define USAGE_ERASE_SCREEN          121   // Special case to erase screen

// Port address to connect to.
#define TIMESERVER_PORT      37                // Server time port
#define TIMESERVER_HOST      "132.163.135.130" // IP address in a.b.c.d dot-notation
#define USAGE_TEST_HOST      "129.46.156.46"   // "satriani.qualcomm.com"
#define USAGE_TEST_PORT      7                 // Server echo port

// A few utility macros; 
#define ISDIGIT(c)  ( (unsigned) ((c) - '0') < 10)
#define ISALPHA(c)  ( (unsigned) ( ((c)|32) - 'a') < 26 )
#define ISALNUM(c)  ( ISDIGIT(c) || ISALPHA(c) )
#define min(a,b)  ((a) < (b) ? (a) : (b))
#define ARRAYSIZE(a)   (sizeof(a) / sizeof((a)[0]))


/*===============================================================================
                     FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to InitAppData(). This is done as follows:
   InitAppData() is called to initialize AppletData instance. It is app developers 
   responsibility to fill-in app data initialization code of InitAppData(). App developer
   is also responsible to release memory allocated for data contained in AppletData -- this can be
   done in FreeAppData().

PROTOTYPE:
   int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell interface. 

	pIModule: pin]: Contains pointer to the IModule interface to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid IApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the app needs to be loaded and if AEEApplet_New() invocation was
     successful
  EFAILED: If the app does not need to be loaded or if errors occurred in 
     AEEApplet_New(). If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
{
   *ppObj = NULL;
		
   // We want to load this App. So, invoke AEEApplet_New().To it, pass the
   // address of the app-specific handle event function.
   if(ClsId == AEECLSID_NETUSAGE_APP){
      if(AEEApplet_New(sizeof(CINetSktApp), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)NetSktApp_HandleEvent,(PFNFREEAPPDATA)INetSkt_FreeAppData)
         == TRUE)
      {
         if (INetSkt_InitAppData((IApplet*)*ppObj) == TRUE)
         {
            return(AEE_SUCCESS);
         }
      }
   }
	return (EFAILED);
}

/*===========================================================================

FUNCTION NetSktApp_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean App_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information 
      specific to this applet. It was initialized during the 
      AEEClsCreateInstance().
	ecode: Specifies the Event sent to this applet
   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/

static boolean NetSktApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
   CINetSktApp * pMe = (CINetSktApp*)pi;

   switch (eCode) 
	{
      case EVT_APP_START:
         // Create the IMenu interface object. When the applet is started the 
         // main menu is set up. When the user selects a menu item from the main 
         // menu, an EVT_COMMAND event is received by this applet. The processing for 
         // EVT_COMMAND event is found below.
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, (void **)&pMe->m_pIMenu)
            != SUCCESS)
         {
				return FALSE;
         }

			CALLBACK_Init(&pMe->cbkLookup, NetApp_DNSResult, pMe);

         // Display the main menu
			BuildMainMenu(pMe);
			return(TRUE);
      case EVT_APP_STOP:
         // Release the INetMgr and IScoket pointers in case they are not released.
         // This would happen if the user ends the applet following the execution
         // of some usage paths which differ the release to a later point.
		CALLBACK_Cancel(&pMe->cbkLookup);
        if (pMe->m_pISocket != NULL)
         {
            ISOCKET_Release (pMe->m_pISocket);
            pMe->m_pISocket = NULL;
         }
         if (pMe->m_pINetMgr != NULL)
         {
            INETMGR_Release (pMe->m_pINetMgr);
            pMe->m_pINetMgr = NULL;
         }
         // Check if the IMenu interface is released. If not, release it here.
         if (pMe->m_pIMenu != NULL)
         {
			   IMENUCTL_Release(pMe->m_pIMenu);
            pMe->m_pIMenu = NULL;
         }
         return(TRUE);
      case EVT_KEY: 
         // We come here when a key is pressed on the handset keypad. Call 
         // the IMenuCtl handle event function to handle this key press.
			if(pMe->m_pIMenu != NULL)
			{
				if ((IMENUCTL_IsActive(pMe->m_pIMenu) == FALSE) &&
					 ((wParam == AVK_UP) || (wParam == AVK_DOWN)))
				{
					IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);
					IMENUCTL_Redraw(pMe->m_pIMenu);
				}
				return IMENUCTL_HandleEvent(pMe->m_pIMenu, EVT_KEY, wParam, 0);
			}
         else
            return FALSE;
		case EVT_COMMAND:
         // When a menu item is selected this applet receives the
         // EVT_COMMAND event in the wParam field.
  		   switch(wParam)
         {
            case USAGE_BASIC_USAGE:
            case USAGE_NET_GETHOSTBYNAME:
            case USAGE_NET_GETLASTERROR:
            case USAGE_NET_OPENSOCKET:
            case USAGE_NET_SETLINGER:
            case USAGE_NET_NETSTATUS:
            case USAGE_NET_GETMYIPADDRESS:
            case USAGE_SOCKET_GETPEERNAME:
            case USAGE_SOCKET_GETLASTERROR:
            case USAGE_SOCKET_CONNECT:
            case USAGE_SOCKET_BIND:
            case USAGE_SOCKET_WRITE:
            case USAGE_SOCKET_WRITEV:
            case USAGE_SOCKET_READ:
            case USAGE_SOCKET_READV:
            case USAGE_SOCKET_SENDTO:
            case USAGE_SOCKET_RECVFROM:
            case USAGE_SOCKET_READBALE:
            case USAGE_SOCKET_WRITEABLE:
            case USAGE_SOCKET_CANCEL:
            case USAGE_SOCKET_SHUTDOWN:
               // Call the usage function that exercises all the 
               // interface functions.
               INetSocketUsage (pMe, wParam);
               return TRUE;
			   default:
				   return FALSE;
         }
      default:
         break;
   }
   return FALSE;
}



/*===========================================================================

FUNCTION INetSkt_InitAppData

DESCRIPTION
   This function initializes app specific data allocates memory for app data (AppletData) 
   and sets it to pAppData of AEEApplet. It also function. App developer needs to 
   fill-in the initialization code.

PROTOTYPE:
	boolean INetSkt_InitAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains information 
      specific to this applet. It was initialized during the AEEClsCreateInstance() .

DEPENDENCIES
   Assumes pi is not NULL

RETURN VALUE
  TRUE: If the app has app data is allocated and initialized successfully
  FALSE: Either app data could not be allocated or initialzied

SIDE EFFECTS
  none
===========================================================================*/
static boolean INetSkt_InitAppData(IApplet* pi)
{
   CINetSktApp * pMe = (CINetSktApp*)pi;

   // Initialize the MenuCtl pointer to NULL
   pMe->m_pIMenu = NULL;

   return TRUE;
}

/*===========================================================================

FUNCTION INetSkt_FreeAppData

DESCRIPTION
   Frees data contained in app data and memory for app data itself.
   App developer needs to free data contained in AppletData.

PROTOTYPE:
	boolean INetSkt_FreeAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

DEPENDENCIES
   Assumes pi is not NULL

RETURN VALUE
   None

SIDE EFFECTS
  none
===========================================================================*/
static void INetSkt_FreeAppData(IApplet* pi)
{
   CINetSktApp * pMe = (CINetSktApp*)pi;

   if (pMe->m_pIMenu != NULL)
   {
      IMENUCTL_Release (pMe->m_pIMenu);
      pMe->m_pIMenu = NULL;
   }
}

/*===============================================================================
                     APP HELPER FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: GetHostByNameCB

DESCRIPTION
   This function is the callback function of the INETMGR_GetHostByName function.
   The parameters include pointer to the user data, a pointer to an array of
   IP addresses supported by a given host, and the number of IP addresses in the
   array. 

   If the pointer to the IP address array in the second parameter is NULL, this 
   indicates an error has occured in getting the given hosts IP addresses.
   
PROTOTYPE:
   void GetHostNameCB(void * pUser, INAddr * pdwAddrs, int nAddrs)

PARAMETERS:
   pUser: [in]: Contains a pointer to the user data passed into the function.
      This void pointer needs to be dereferenced to retrieve the user data.
   pdwAddrs: [in]: Contains a pointer to an array of IP addresses for a given host.
   pdwAddrs: [out]: Contains the number of IP address entries found in the 
      "pwdAddrs" array.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
void GetHostByNameCB(void * pUser)
{
   CINetSktApp * pMe = (CINetSktApp*)pUser;
   char *szMsg = MALLOC(100);
   int i;
   if(!szMsg)
      return;


   switch(pMe->dnsresult.nResult)
   {
      case AEE_NET_BADDOMAIN:
         SPRINTF(szMsg,"Host name is mal-formed; not a valid host name");
         break;
      case AEE_NET_UNKDOMAIN: 
         SPRINTF(szMsg,"Unknown host, or one without IP addresses");
         break;
      case AEE_NET_ETIMEDOUT:
         SPRINTF(szMsg,"No response was seen within the maximum time limit");
         break;
      case EUNSUPPORTED:
         SPRINTF(szMsg,"No DNS servers are configured");
         break;
      case ENOMEMORY: 
         SPRINTF(szMsg,"Could not perform query due to allocation failure");
         break;
      default:
         if(pMe->dnsresult.nResult > 0)
            SPRINTF(szMsg,"Success.Obtained %d address(es)",pMe->dnsresult.nResult);
         else
            SPRINTF(szMsg,"Unknown Error");
      break;
   }

   DisplayOutput(pMe,2,szMsg);
   if(pMe->dnsresult.nResult > 0 && pMe->dnsresult.nResult <= AEEDNSMAXADDRS)      
   {
      for(i = 0; i < pMe->dnsresult.nResult; i++)
      {
         SPRINTF(szMsg,"Addr=: %x",pMe->dnsresult.addrs[i]);
         DisplayOutput(pMe,i+4,szMsg);
      }
   }

   FREE(szMsg);

   
   return;

}

/*===========================================================================

FUNCTION: CommonConnectCB

DESCRIPTION
   This function is a common connect callback function. This function is
   called by all the connection done in the INetSocketUsage() function.

   This function retrieves the API function Id from the users AEEApllet object 
   and uses it to switch to a code block corresponding to the function Id.
   
PROTOTYPE:
   void CommonConnectCB(void * pUser, int iError)

PARAMETERS:
   pUser: [in]: Contains a pointer to the user data passed into the function.
      This void pointer needs to be dereferenced to retrieve the user data.
   iError: [in]: Contains an error code indicating the success/failure of the
      connection.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
void CommonConnectCB(void * pUser, int iError)
{
   CINetSktApp * pMe = (CINetSktApp*)pUser;

   if (pMe == NULL)
      return;

   // Check if connect failed
   if (iError) 
   {
      // Release the INetMgr and ISocket pointers
      ReleaseNetAndSocket(pMe);
      return;
   }

   // Switch on the API Function Id 
   switch (pMe->APIFnType)
   {
      case USAGE_BASIC_USAGE:
         {
            // Call CommonWriteCB function to do the writting.
            // This function writes data to the echo-port of the 
            // remote host. It loops through registering a callback 
            // function, till it can write all the data.
            CommonWriteCB((void *)pMe);

            // Call CommonReadCB function to do the reading.
            // This function will read the same data that was written
            // above from the echo-port of the remote host. This function
            // loops through registering a callback function, till it's 
            // done with all the reading.
            CommonReadCB ((void*)pMe);
         }
         break;
      case USAGE_SOCKET_GETPEERNAME:
         {
            int iRetValue = AEE_NET_EBADF;
            INAddr inAddr;
            uint16 pwPort;
            char psz[50] = {0};

            // Connect succeded. Now invoke GetPeerName
            if ((iRetValue = ISOCKET_GetPeerName (pMe->m_pISocket, 
               &inAddr, &pwPort)) != AEE_NET_SUCCESS)
            {    
				//GetLastError should not be called here - This call gives valid
				// info only after some calls and GetPeerName is not one of them  
				
                DisplayOutput (pMe, -1, "GetPeerName Failed");
               // ReleaseNetAndSocket(pMe);
               // return;
            }
			else
			{
            // Convert IP address to char string in the a.b.c.d notation
            ConvertToIPAddr(inAddr, psz);
            // Display on the screen
            DisplayOutput (pMe, -1, psz);
			}
   
            ReleaseNetAndSocket(pMe);  // Release INetMgr and ISocket
         }
         break;
      case USAGE_SOCKET_CONNECT:
         // Connect Succeeded, just display and release resources
         DisplayOutput (pMe, -1, "Connect Succeeded!");
         ReleaseNetAndSocket(pMe);
         break;
      case USAGE_SOCKET_WRITE:
      case USAGE_SOCKET_WRITEABLE:
         {
            // Call CommonWriteCB function to do the writting.
            CommonWriteCB((void *)pMe);
         }
         break;
      case USAGE_SOCKET_WRITEV:
      case USAGE_SOCKET_READV:
         {
            int rv = 0;
            SockIOBlock iov[2];
            uint16 wiovcount = 2;  // Number of entries in IOV block

            iov[0].pbBuffer = (byte *)"data1";
            iov[0].wLen  = 5;
            iov[1].pbBuffer = (byte *)"data2";
            iov[1].wLen =5;


            // Write char string to socket.
            rv = ISOCKET_WriteV(pMe->m_pISocket, 
               iov, wiovcount);

            if (rv == AEE_NET_ERROR)
            {
               // Write Error.
               ReleaseNetAndSocket(pMe);
            }
            else if (rv > 0)
            {
               // Need to read to show that the Write succeeded.
               CommonReadCB ((void *)pMe);
            }
            else if (rv == AEE_NET_WOULDBLOCK)
            {
               // Try again when "writeable";  return and wait for callback
               ISOCKET_Writeable(pMe->m_pISocket, 
                                 CommonWriteCB, (void *)pMe);
               return;
            }
         }
         break;
      case USAGE_SOCKET_READ:
      case USAGE_SOCKET_READBALE:
         {
            // Call CommonReadCB function to do the reading.
            CommonReadCB ((void*)pMe);
         }
         break;
      default:
         // We shouldn't be coming here. In case we do then release 
         // the resources and return.
         ReleaseNetAndSocket(pMe);
         break;
   }
}

/*===========================================================================

FUNCTION: CommonWriteCB

DESCRIPTION
   This function is a common write callback function. This function is called
   when any data needs to be written to a socket connection. This function
   registers itself as a callback function when all the data is not written.

   The function retrieves the API function Id from the users AEEApllet object 
   and uses it to switch to a code block corresponding to the function Id.
   
PROTOTYPE:
   void CommonWriteCB (void *pUser)

PARAMETERS:
   pUser: [in]: Contains a pointer to the user data passed into the function.
      This void pointer needs to be dereferenced to retrieve the user data.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
void CommonWriteCB (void *pUser)
{
   CINetSktApp * pMe = (CINetSktApp*)pUser;

   switch (pMe->APIFnType)
   {
      case USAGE_BASIC_USAGE:
         {
            int  rv;
            char pszData[100] = "Basic Usage Data Written and Read from Remote Host echo-port..";
            char *psz = NULL;
            int  cbWrite;

            psz = pszData;
            cbWrite = STRLEN (pszData);

            // Loop till all the bytes are written.
            while (cbWrite > 0) {
               // Write data to the echo-port of the remote host
               rv = ISOCKET_Write(pMe->m_pISocket, 
                  (byte *)psz, (uint16)cbWrite);

               if (rv == AEE_NET_WOULDBLOCK) {
                  // No byte were written successfully. Register callback to 
                  // try again later.
                  ISOCKET_Writeable(pMe->m_pISocket, 
                     CommonWriteCB, (void *)pMe);
                  return;
               } else if (rv == AEE_NET_ERROR) {
                  // Write Error.
                  ReleaseNetAndSocket(pMe);
                  return;
               }
               // In case of parital write, loop and write the rest 
               cbWrite -= rv;
               psz += rv;
            }
         }
         break;
      case USAGE_SOCKET_WRITE:
      case USAGE_SOCKET_WRITEABLE:
         {
            int  rv;
            char pszData[100] = "Data Written and Read from Remote Host echo-port..";
            char *psz = NULL;
            int  cbWrite;

            psz = pszData;
            cbWrite = STRLEN (pszData);

            // Loop till all the bytes are written.
            while (cbWrite > 0) {
               // Write data to the echo-port of the remote host
               rv = ISOCKET_Write(pMe->m_pISocket, 
                  (byte *)psz, (uint16)cbWrite);

               if (rv == AEE_NET_WOULDBLOCK) {
                  // No byte were written successfully. Register callback to 
                  // try again later.
                  ISOCKET_Writeable(pMe->m_pISocket, 
                     CommonWriteCB, (void *)pMe);
                  return;
               } else if (rv == AEE_NET_ERROR) {
                  // Write Error.
                  ReleaseNetAndSocket(pMe);
                  return;
               }
               // In case of parital write, loop and write the rest 
               cbWrite -= rv;
               psz += rv;
            }
            // done writing

            // Call CommonReadCB to read the bytes echoed back from the remote host
            CommonReadCB((void *)pMe);
         }
         break;
      case USAGE_SOCKET_WRITEV:
         {
            int rv = 0;
            SockIOBlock iov[2];
            uint16 wiovcount = 2;  // Number of entries in IOV block

            iov[0].pbBuffer = (byte *)"data1";
            iov[0].wLen  = 5;
            iov[1].pbBuffer = (byte *)"data2";
            iov[1].wLen =5;

            // Write char string to socket.
            rv = ISOCKET_WriteV(pMe->m_pISocket, 
               iov, wiovcount);

            if (rv == AEE_NET_ERROR)
            {
                ReleaseNetAndSocket(pMe);
            }
            else if (rv == AEE_NET_WOULDBLOCK)
            {
               // Return and wait for callback to try again
               ISOCKET_Writeable(pMe->m_pISocket, 
                                 CommonWriteCB, (void *)pMe);
               return;
            }
            else if (rv > 0)
            {
               // Now Read the data written to the echo port of the remote
               // host and display on the screen.
               CommonReadCB ((void *)pMe);
            }
         }
         break;
      case USAGE_SOCKET_SENDTO:
      case USAGE_SOCKET_RECVFROM:
         {
            int32 iRetValue = 0;
            INAddr nodeINAddr;
            uint16 nPort;
//            int  rv;
            char pszSendData[100] = "UDP packet sent to and received from  echo-port of remote host...";
           // char pszRecvData[100] = {0};

            // Before we can pass on the address to connect the socket, 
            // we need to convert the address from a string to the INAddr format.
            nodeINAddr = ConvertToINAddr(USAGE_TEST_HOST);
            nPort = HTONS(USAGE_TEST_PORT);  // convert port

            // SendTo again 
            iRetValue = ISOCKET_SendTo (pMe->m_pISocket, (byte *)pszSendData,
                                        (uint16)STRLEN(pszSendData), 0, nodeINAddr, nPort);

            if (iRetValue == AEE_NET_ERROR)
            {
                  ReleaseNetAndSocket (pMe);
                  return;
            }

            if (iRetValue == AEE_NET_WOULDBLOCK) {
               // Return and wait for callback
               ISOCKET_Writeable (pMe->m_pISocket, CommonWriteCB, (void *)pMe);
               return;
            }
               
            if (iRetValue > 0)
            {
               // Register a callback function in case all the data is not read
               // from the ISOCKET_RecvFrom function. This callback function 
               // needs to be done prior to the calling of ISOCKETRecvFrom.
               ISOCKET_Readable(pMe->m_pISocket, CommonReadCB, (void*)pMe);

#if 0 // no read to read anything here ... CommonReadCB should take care of it
               
               // Receive connection-less data from the remote host 
               rv = ISOCKET_RecvFrom (pMe->m_pISocket, 
                  (byte *)pszRecvData, sizeof (pszRecvData), 0, &nodeINAddr, &nPort);

               if (rv == AEE_NET_WOULDBLOCK)
               {
                  // Here we don't have to register a callback function because
                  // a callback function was registered prior to calling ISOCKET_RecvFrom.
                  // So return and wait for the callback to get activated.
         	      return;
               } 
               else if (rv > 0)
               {
                  // Cancel the previously registered call-back function
                  // because all the data was read (received).
                  ISOCKET_Cancel (pMe->m_pISocket, 
                     CommonReadCB, (void*)pMe);

                  // Display output starting in the middle of the screen
                  DisplayOutput (pMe, -1, pszRecvData);   // Display Data
                  ReleaseNetAndSocket (pMe);
               }
               else
               {
                  // Release Network and Socket interfaces.
                  ReleaseNetAndSocket (pMe);
               }
#endif

            }
         }
         break;

		case USAGE_SOCKET_BIND:
		{
			int iRetValue = AEE_NET_EBADF;

         // Bind to local port. The second parameter needs to be 0 since it's
         // not used (the local host IP address is automatically retrieved by
         // the ISOCKET_Bind function.
         if ((iRetValue = ISOCKET_Bind (pMe->m_pISocket, AEE_INADDR_ANY, 1240))  
            == AEE_NET_WOULDBLOCK)
         {
				// Try again when "writeable";  return and wait for callback
				ISOCKET_Writeable(pMe->m_pISocket, CommonWriteCB, (void *)pMe);
			}
			else
			{
				if (iRetValue != AEE_NET_SUCCESS)
					DisplayOutput (pMe, -1, "Bind Failed.");
				else
				{
					DisplayOutput (pMe, -1, "Bind Succeeded.");
					ReleaseNetAndSocket (pMe);
				}
			}
			break;
		}

      default:
         break;
   }
}

/*===========================================================================

FUNCTION: CommonReadCB

DESCRIPTION
   This function is a common read callback function. This function is called
   when any data needs to be read from a socket connection. This function
   registers itself as a callback function when all the data is not read.

   The function retrieves the API function Id from the users AEEApllet object 
   and uses it to switch to a code block corresponding to the function Id.
   
PROTOTYPE:
   void CommonReadCB (void *pUser)

PARAMETERS:
   pUser: [in]: Contains a pointer to the user data passed into the function.
      This void pointer needs to be dereferenced to retrieve the user data.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
void CommonReadCB (void *pUser)
{
   CINetSktApp * pMe = (CINetSktApp*)pUser;
   ISocket *piSock = pMe->m_pISocket;

   switch (pMe->APIFnType)
   {
      case USAGE_BASIC_USAGE:
         {
            int32 rv;
            char szBuf[100] = {0};

            // Read data from the echo-port of the remote host.
            rv = ISOCKET_Read(piSock, (byte *)szBuf, sizeof(szBuf));

            if (rv == AEE_NET_WOULDBLOCK)
            {
               // WOULDBLOCK => there is no more data available at the moment
               // Register the callback to read the data later.
               ISOCKET_Readable(piSock, CommonReadCB, (void*)pMe); 
	            return;
            } 
            else if (rv == AEE_NET_ERROR) 
            {
               DisplayOutput (pMe, -1, "Read Error!");
               ReleaseNetAndSocket (pMe);
            }
            else if (rv > 0) // data has been read
            {
	         	// The data has been read from the socket. Display Data read.
               DisplayOutput (pMe, 4, szBuf);
               ReleaseNetAndSocket (pMe);
               // If we wanted to read more data, we could loop here or call Readable() 
               // here and return.
            }
            else  // rv == 0
            {
               // There is no more data to be received. 
               // The peer has shut down the connection.
               DisplayOutput (pMe, -1, "No Data!");
               ReleaseNetAndSocket (pMe);
            }
         }
         break;
      case USAGE_SOCKET_WRITE:
      case USAGE_SOCKET_WRITEABLE:
         {
            int32 rv;          // Return value from Read()
            char szBuf[100] = {0};

            // Read data corresponding to usage example for the WRITE functions.
            // We are reading data here to indicate the write functions success
            // in writing data to the echo-port of the remote host, and the data
            // has been read back to be displayed.
            rv = ISOCKET_Read(piSock, (byte*)szBuf, sizeof(szBuf));

            if (rv == AEE_NET_WOULDBLOCK)
            {
               // WOULDBLOCK => there is no more data available at the moment
               // Register the callback to read the data later.
               ISOCKET_Readable(piSock, CommonReadCB, (void*)pMe); 
	            return;
            } 
            else if (rv == AEE_NET_ERROR) 
            {
               // Read error.
               DisplayOutput (pMe, -1, "Read Error!");
               ReleaseNetAndSocket (pMe);
            }
            else if (rv > 0) // data has been read
            {
	         	// The data has been read from the socket. Display Data read.
               DisplayOutput (pMe, -1, szBuf);

               ReleaseNetAndSocket (pMe);
               // If we wanted to read more data, we could loop here or call Readable() 
               // here and return.
            }
            else  // rv == 0
            {
               // There is no more data to be received.
               // The peer has shut down the connection.
               DisplayOutput (pMe, -1, "No Data!");
               ReleaseNetAndSocket (pMe);
            }
         }
         break;
      case USAGE_SOCKET_READ:
      case USAGE_SOCKET_READBALE:
         {
            int rv;          // Return value frm Read()
	        uint32 uTime;    // Buffer to pass to socket-read function
            char szBuf[50] = {0};

            rv = ISOCKET_Read(piSock, (byte*)&uTime, sizeof(uint32));

            if (rv == AEE_NET_WOULDBLOCK)
            {
               // WOULDBLOCK => there is no more data available at the moment
               DisplayOutput (pMe, -1, "Waiting...      ");
               // Register callback to read all the data when more data is available.
               ISOCKET_Readable(piSock, CommonReadCB, (void*)pMe);
	            return;
            } 
            else if (rv == AEE_NET_ERROR) 
            {
               // Read error has occured.
               DisplayOutput (pMe, -1, "Read Error!");
               ReleaseNetAndSocket (pMe);
            }
            else if (rv > 0) // data has been read
            {
	         	// The data has been read from the socket. So, convert the data from
		        // network format to host format. 
         		uTime = NTOHL(uTime) ;
		
	         	// Display the data.
	         	// This is the time elapsed in seconds since Jan 1 1900.
               SPRINTF(szBuf, "Seconds elapsed since Jan 1, 1900  =  %lu", (long) uTime);
               DisplayOutput (pMe, -1, szBuf);

               // Release INetMgr and IScoket interface objects.
               ReleaseNetAndSocket (pMe);
               // If we wanted to read more data, we could loop here or call Readable() here
               // and return.
            }
            else  // rv == 0
            {
               // There is no more data to be received.
               // The peer has shut down the connection.
               DisplayOutput (pMe, -1, "No Data!");
               ReleaseNetAndSocket (pMe);
            }
         }
         break;
      case USAGE_SOCKET_READV:
      case USAGE_SOCKET_WRITEV:
         {
            int32 rv;
            char szBuf[100] = {0};     
            SockIOBlock iov[2] = {0};
            byte buf1[6] = {0};
            byte buf2[6] = {0};
            int16 iovcount = 2;

            // Temp: test to see if providing no bufferes to write is causing the prob. 
            iov[0].pbBuffer = buf1;
            iov[0].wLen = 5;
            iov[1].pbBuffer = buf2;
            iov[1].wLen = 5;

            // Need to register a call-back before invoking ReadV
            /*rv =*/ ISOCKET_Readable(piSock, CommonReadCB, (void*)pMe);

            // ReadV
            rv = ISOCKET_ReadV(piSock, iov, iovcount);

            if (rv == AEE_NET_WOULDBLOCK)
            {
               // WOULDBLOCK => there is no more data available at the moment
               DisplayOutput (pMe, -1, "Waiting...");
               ISOCKET_Readable(piSock, CommonReadCB, (void*)pMe);
	            return;
            } 
            else if (rv == AEE_NET_ERROR) 
            {
               // Connection hosed.
               DisplayOutput (pMe, -1, "Read Error!");
               ReleaseNetAndSocket (pMe);
            }
            else if (rv > 0) // data has been read
            {
         	   // Display the data.
               SPRINTF(szBuf, "Buff1 = %s", iov[0].pbBuffer);
               DisplayOutput (pMe, 6, szBuf);
               SPRINTF(szBuf, "Buff2 = %s", iov[1].pbBuffer);
               DisplayOutput (pMe, 7, szBuf);

               ReleaseNetAndSocket (pMe);
               // If we wanted to read more data, we could loop here or call 
               // Readable() here and return.
            }
            else  // rv == 0
            {
               // There is no more data to be received.
               // The peer has shut down the connection.
               DisplayOutput (pMe, -1, "No Data!");
               ReleaseNetAndSocket (pMe);
            }   
         }
         break;
      case USAGE_SOCKET_SENDTO:
      case USAGE_SOCKET_RECVFROM:
         {
            int rv;    // Return value from Read()
            char psz [100] = {0};
            INAddr addr;
            uint16 port;
 
            // Receive a UDP packet from the remote host echo-port
            rv = ISOCKET_RecvFrom (piSock, (byte *)psz, sizeof (psz), 0, &addr, &port);

            if (rv == AEE_NET_WOULDBLOCK)
            {
               // WOULDBLOCK => No data available now; try again later.
               DisplayOutput (pMe, -1, "Waiting...");
               // Register callback to read the all the data later.
               ISOCKET_Readable(piSock, CommonReadCB, (void*)pMe);
         	   return;
            } 
            else if (rv == AEE_NET_ERROR) 
            {
               // Read error.
               DisplayOutput (pMe, -1, "Read Error!");
               ReleaseNetAndSocket (pMe);
            }
            else if (rv > 0) // data has been read
            {
               DisplayOutput (pMe, -1, psz);   // Display Data
               ReleaseNetAndSocket (pMe);
               // If we wanted to read more data, we could loop here or call Readable() here
               // and return.
            }
            else  // rv == 0
            {
               // There is no more data to be received. The peer has shut down
               // the connection.
               DisplayOutput (pMe, -1, "No Data!");
               ReleaseNetAndSocket (pMe);
            }
         }
   }
}

/*===========================================================================

FUNCTION: INetSocketUsage

DESCRIPTION
	This function encompasses all the usage examples of all the functions
   in code blocks switched using the BREW API function Id passed into this
   function. The code blocks will include almost all the variable declarations
   needed to demonstrate the usage of a given function. Within the code block
   all the needed interfaces are instantiated and released subsequent to their
   use.

   There are some exceptions to the above general rule, such as call-back functions,
   common helper function, or common display functions called from within the
   code block which are defined elsewhere in this file.

   There are some instances where an object instantiated within a code block is
   not released at the end of the code block, to be released at a later point
   (for example in a call-back function, in this case pointers to such objects
   are stored in the AEEApplet object to be released later).
   
PROTOTYPE:
   boolean INetSocketUsage (AEEApplet * pMe, uint16 wParam)

PARAMETERS:
	pMe: [in]: Contains a pointer to the AEEApplet struct.

   wParam: [in]: Contains the Id of the function type to be used (contains the
      Id corresponding to the menu item selected on the emulator).

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the functions call was successful. FALSE otherwise

SIDE EFFECTS
  none
===========================================================================*/
boolean INetSocketUsage (CINetSktApp * pMe, uint16 wParam)
{
   AEEDeviceInfo di; // Device Info
   AEERect rct;
   // Get screen size information for display purposes.
   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);
   // Set rectangle coordinates for a rectangle in the middle of the screen.
   rct.x = 10;
   rct.y = di.cyScreen /3;
   rct.dx = di.cxScreen - 5;
   rct.dy = di.cyScreen/3;

   // Just in case some path did not release the socket and the NetMgr 
   // objects, check and clean it up. Examples of instances where the 
   // these pointers would not have been cleaned up are:
   // - When the PPP connection is closing. 
   // - When a callback function is canceled using ISOCKET_Cancel().
   //
   // In these cases releasing the INetMgr and ISocket objects before 
   // it has had an opportunity to clean up its allocated resources 
   // or tearing down connections to remote hosts take a little bit of 
   // time. As a result if the INeMgr and ISocket objects are released 
   // too early these connections and resources will not be cleaned up 
   // properly. In the case of connection releasing, INETMGR_NetStatus() 
   // function can be used to query the INetMgr object to see if the 
   // connection has reached the CLOSED state before releasing the 
   // objects.
   if (pMe->m_pISocket != NULL)
   {
      // If there some callback still pending we should release them.
      ISOCKET_Release (pMe->m_pISocket);
      pMe->m_pISocket = NULL;
   }
   if (pMe->m_pINetMgr != NULL)
   {
      INETMGR_Release(pMe->m_pINetMgr);
      pMe->m_pINetMgr = NULL;
   }

   // Erase screen first for display output purposes.
   DisplayEvent (pMe, USAGE_ERASE_SCREEN);
	IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
	IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);

   // The function call-back type is store inAPIFnType of the AEEApplet
   // data structure so that it can be used in the call-back functions.
   switch (pMe->APIFnType = wParam)
   {
      case USAGE_BASIC_USAGE:
         {
            // 1. Usage: Basic Usage
            INetMgr *pINetMgr = NULL;
            ISocket * pISocket = NULL;
            int iRetValue = AEE_NET_EBADF, lastError;
            INAddr nodeINAddr;
            uint16 nPort;

            // Create INetMgr object. This step needs to be done only if
            // the INetMgr object has not been created already. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_NET, (void **)&pINetMgr);

            if (pINetMgr)
            {
               // Open Socket
               pISocket = INETMGR_OpenSocket (pINetMgr, AEE_SOCK_STREAM); 

               if (!pISocket)
               {
                  // Release INetMgr object. This step needs to be done
                  // only if no further use of the INetMgr object is needed.
                  INETMGR_Release (pINetMgr);

                  return FALSE;
               }
               
               // Before we can pass on the address to connect the socket, 
               // we need to convert the address from a string to the INAddr format.
               nodeINAddr = ConvertToINAddr(USAGE_TEST_HOST);
               nPort = HTONS(USAGE_TEST_PORT);  // convert port (echo port)

               // Store INetMgr and ISocket pointers in AEEApplet data structure
               // (to be used in the call-back function).
               pMe->m_pISocket = pISocket;
               pMe->m_pINetMgr = pINetMgr;

               // To write data we need to connect to first a remote host.
               // ISOCKET_Write and ISOCKET_Writable function usages are found
               // in the CommonConnectCB callback function. 
               if ((iRetValue = ISOCKET_Connect (pISocket, nodeINAddr, nPort, 
                  CommonConnectCB, (void *)pMe))  != AEE_NET_SUCCESS)
               {
                  // If it's WOULDBLOCK, then it's still connecting. Don't cleanup.
                  if (iRetValue != AEE_NET_WOULDBLOCK)
                  {
                     // Get Last error
                     lastError = ISOCKET_GetLastError(pISocket);
                     // Release NetMgr & ISocket and return.
                     ReleaseNetAndSocket (pMe);
                     return FALSE;
                  }
               }

               // Note: The INetMgr and ISocket pointers are not released
               // here since they will be released when the CommonConnectCB 
               // callback function gets called. We store pointers to both in
               // m_pINetMgr and m_pISocket members of the Applet data structure
               // above.  

               // A common callback function is used for all the connects. Here
               // the API function type that is being addressed is stored in the
               // APIFnType data member of the Applet structure at the begining 
               // of this function (to be used in the callback function).

               // Invoke Display: used purely for display purposes.
               DisplayOutput (pMe, -1, "Connecting...");
            }
         }
         break;
      case USAGE_NET_GETHOSTBYNAME:
         {
            // 2. Usage INETMGR_GetHostByName
            INetMgr *pINetMgr = NULL;
            const char hostName [] = "www.yahoo.com";
            //int retVal = EFAILED;
            // Create INetMgr object. This step needs to be done only if
            // the INetMgr object has not been created already. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_NET, (void **)&pINetMgr);

            if (pINetMgr)
            {
               // Store a pointer to the NetMgr in the AEEApplet struct. This 
               // will be used in the GetHostByNameCB callback fucntions.
               pMe->m_pINetMgr = pINetMgr;

               //Initialize the CB structure
                CALLBACK_Init(&pMe->cbkLookup, GetHostByNameCB, pMe);

                INETMGR_GetHostByName(pINetMgr,&pMe->dnsresult, hostName, &pMe->cbkLookup);
                INETMGR_Release (pMe->m_pINetMgr);
            }
         }
         break;
      case USAGE_NET_GETLASTERROR:
         {
            // 3. Usage INETMGR_GetLastError
            INetMgr *pINetMgr = NULL;
            int lastError = 0;
            char szBuf [30] = {0};
            int invalidSockType = 0xff;
			ISocket * pISocket = NULL;

            // Create INetMgr object. This step needs to be done only if
            // the INetMgr object has not been created already. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_NET, (void **)&pINetMgr);

            if (pINetMgr)
            {
				// Open socket of an invalid type to induce an error
				pISocket = INETMGR_OpenSocket(pINetMgr, invalidSockType);
				
				if(pISocket != NULL)
				{
					// Release ISocket object. This step needs to be done
                    // only if no further use of the ISocket object is needed.
					ISOCKET_Release (pISocket);
				}

				// Get last error
				if ((lastError = INETMGR_GetLastError(pINetMgr))
                     != AEE_NET_SUCCESS)
                {
                    // Invoke Display: used for display purposes only.
                    SPRINTF (szBuf, "Error Id: %u", lastError);
                    DisplayOutput (pMe, -1, szBuf);
                }
                else
                {
                    DisplayOutput (pMe, -1, "GetLastError: failed");
                }

                // Release INetMgr object. This step needs to be done
                // only if no further use of the INetMgr object needed.
                INETMGR_Release (pINetMgr);
            }
         }
         break;
      case USAGE_NET_OPENSOCKET:
         {
            // 4. Usage INETMGR_OpenSocket
            INetMgr *pINetMgr = NULL;
            NetSocket socketType = AEE_SOCK_STREAM;
            ISocket * pISocket = NULL;
            int lastError;

            // Create INetMgr object. This step needs to be done only if
            // the INetMgr object has not been created already. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_NET, (void **)&pINetMgr);

            if (pINetMgr)
            {
               // Open Socket
               if ((pISocket = INETMGR_OpenSocket (pINetMgr, socketType)) != NULL)
               {
                  // Invoke Display: used for display purposes only.
                  DisplayOutput (pMe, -1, "Socket Open...");

                  // Release ISocket object. This step needs to be done
                  // only if no further use of the ISocket object is needed.
                  ISOCKET_Release (pISocket);
               }
               else
               {
                  // Get last error
                  lastError = INETMGR_GetLastError(pINetMgr);
               }

               // Release INetMgr object. This step needs to be done
               // only if no further use of the INetMgr object is needed.
               INETMGR_Release (pINetMgr);
            }
         }
         break;
      case USAGE_NET_SETLINGER:
         {
            // 5. Usage INETMGR_SetLinger
            INetMgr *pINetMgr = NULL;
            char lTime[100] = {0};
            uint16 lingerTime1 = 20;   // Linger time in seconds
            uint16 lingerTime2 = 10;   // Linger time in seconds
            uint16 prevLingerTime = 0;


            // Create INetMgr object. This step needs to be done only if
            // the INetMgr object has not been created already. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_NET, (void **)&pINetMgr);

            if (pINetMgr)
            {
               // Set Linger time (in seconds). We should get back the 
               // default linger timer of 30 seconds.
               prevLingerTime = INETMGR_SetLinger (pINetMgr, lingerTime1);
               SPRINTF (lTime, "Linger Time 1 = %u", prevLingerTime);
               DisplayOutput (pMe, 2, lTime);

               // Set linger time a second time. We should get back the previously
               // set time.
               prevLingerTime = INETMGR_SetLinger (pINetMgr, lingerTime2);
               SPRINTF (lTime, "Linger Time 2 = %u", prevLingerTime);
               DisplayOutput (pMe, 4, lTime);

               // Release INetMgr object. This step needs to be done
               // only if no further use of the INetMgr object is needed.
               INETMGR_Release (pINetMgr);
            }
         }
         break;
      case USAGE_NET_NETSTATUS:
         {
            // 6. Usage INETMGR_NetStatus
            INetMgr *pINetMgr = NULL;
            NetState netState = NET_INVALID_STATE;
            AEENetStats netStats;

            // Create INetMgr object. This step needs to be done only if
            // the INetMgr object has not been created already. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_NET, (void **)&pINetMgr);

            if (pINetMgr)
            {
               // Get Net Status
               netState = INETMGR_NetStatus (pINetMgr, &netStats);

               // Display the Net Status returned.
               switch (netState)
               {
                  case NET_INVALID_STATE:
                     DisplayOutput (pMe, -1, "Status: Invalid");  // Display status
                     break;
                  case NET_PPP_OPENING:
                     DisplayOutput (pMe, -1, "Status: Opening");  // Display status
                     break;
                  case NET_PPP_OPEN:
                     DisplayOutput (pMe, -1, "Status: Open");     // Display status
                     break;
                  case NET_PPP_CLOSING:
                     DisplayOutput (pMe, -1, "Status: Closing");  // Dispaly status
                     break;
                  case NET_PPP_CLOSED:
                     DisplayOutput (pMe, -1, "Status: Closed");   // Display status
                     break;
               }

               // Release INetMgr object. This step needs to be done
               // only if no further use of the INetMgr object is needed.
               INETMGR_Release (pINetMgr);
            }
         }
         break;
      case USAGE_NET_GETMYIPADDRESS:
         {
            // 7. Usage INETMGR_GetMyIPAddr
            INetMgr *pINetMgr = NULL;
            INAddr addr = 0;
            char psz[50], psz1[100];


            // Create INetMgr object. This step needs to be done only if
            // the INetMgr object has not been created already. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_NET, (void **)&pINetMgr);

            if (pINetMgr)
            {
               // local host IP address
               addr = INETMGR_GetMyIPAddr (pINetMgr);
               
               // Convert IP address to char string in the a.b.c.d notation
               ConvertToIPAddr(addr, psz);

               STRCPY (psz1, "IPAddr: ");    
               STRCAT (psz1, psz);         // Append the my IP address 
               // Invoke Display: used for display purposes only.
               DisplayOutput (pMe, -1, psz1);

               // Release INetMgr object. This step needs to be done
               // only if no further use of the INetMgr object is needed.
               INETMGR_Release (pINetMgr);
            }
         }
         break;
      case USAGE_SOCKET_GETPEERNAME:
         {
            // 8. Usage ISOCKET_GetPeerName
            INetMgr *pINetMgr = NULL;
            ISocket * pISocket = NULL;
            int iRetValue = AEE_NET_EBADF, lastError;
            INAddr nodeINAddr;
            uint16 nPort;

            // Create INetMgr object. This step needs to be done only if
            // the INetMgr object has not been created already. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_NET, (void **)&pINetMgr);

            if (pINetMgr)
            {
               // Open Socket
               pISocket = INETMGR_OpenSocket (pINetMgr, AEE_SOCK_STREAM); 

               if (!pISocket)
               {
                  // Release INetMgr object. This step needs to be done
                  // only if no further use of the INetMgr object is needed.
                  INETMGR_Release (pINetMgr);

                  return FALSE;
               }
               
            	// Before we can pass on the address to connect the socket, 
               // we need to convert the address from a string to the INAddr format.
               nodeINAddr = ConvertToINAddr(TIMESERVER_HOST);
               nPort = HTONS(TIMESERVER_PORT);  // convert port 

               // Store INetMgr and ISocket pointers in AEEApplet data structure
               // (to be used in the call-back function).
               pMe->m_pISocket = pISocket;
               pMe->m_pINetMgr = pINetMgr;

               // To get the peer name we need to first connect to a remote host.
               // ISCOKET_GetPeerName function usage is found in the CommonConnectCB
               // callback function. 
               if ((iRetValue = ISOCKET_Connect (pISocket, nodeINAddr, nPort, 
                  CommonConnectCB, (void *)pMe))  != AEE_NET_SUCCESS)
               {
                  // If it's WOULDBLOCK, then it's still connecting. Don't cleanup.
                  if (iRetValue != AEE_NET_WOULDBLOCK)
                  {
                     // Get Last error
                     lastError = ISOCKET_GetLastError(pISocket);
                     // Release NetMgr & ISocket and return.
                     ReleaseNetAndSocket (pMe);
                     return FALSE;
                  }
               }

               // Note: The INetMgr and ISocket pointers are not released
               // here since they will be released when the CommonConnectCB 
               // callback function gets called. We store pointers to both in
               // m_pINetMgr and m_pISocket members of the Applet data structure
               // above.  

               // A common callback function is used for all the connects. Here
               // the API function type that is being addressed is stored in the
               // APIFnType data member of the Applet structure at the begining 
               // of this function (to be used in the callback function).

               // Invoke Display: used purely for display purposes.
               DisplayOutput (pMe, -1, "Connecting...");

            }
         }
         break;
      case USAGE_SOCKET_GETLASTERROR:
         {
            // 9. Usage ISOCKET_GetLastError
            INetMgr *pINetMgr = NULL;
            ISocket * pISocket = NULL;
            int iRetValue = AEE_NET_EBADF, lastError;
            uint32 readVal;   // Buffer to pass to socket-read function

            // Create INetMgr object. This step needs to be done only if
            // the INetMgr object has not been created already. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_NET, (void **)&pINetMgr);

            if (pINetMgr)
            {
               // Open Socket
               pISocket = INETMGR_OpenSocket (pINetMgr, AEE_SOCK_STREAM); 

               if (!pISocket)
               {
                  // Release INetMgr object. This step needs to be done
                  // only if no further use of the INetMgr object is needed.
                  INETMGR_Release (pINetMgr);

                  return FALSE;
               }
               
               // This Read should fail because the socket is not connected.
               if ((iRetValue = ISOCKET_Read(pISocket, (byte*)&readVal, 
                  sizeof(uint32))) != AEE_NET_SUCCESS)
               {
                  if ((lastError = ISOCKET_GetLastError(pISocket))
                     == AEE_NET_ENOTCONN)
                  {
                     // Display error message
                     DisplayOutput (pMe, -1, "Err: Not Connected");
                  }
               }

               // Release ISocket object. This step needs to be done
               // only if no further use of the ISocket object is needed.
               ISOCKET_Release (pISocket);
            }

            // Release INetMgr object. This step needs to be done
            // only if no further use of the INetMgr object is needed.
            INETMGR_Release (pINetMgr);
         }
         break;
      case USAGE_SOCKET_CONNECT:
         {
            // 10. Usage INETMGR_GetMyIPAddr
            INetMgr *pINetMgr = NULL;
            ISocket * pISocket = NULL;
            int iRetValue = AEE_NET_EBADF, lastError;
            INAddr nodeINAddr;
            uint16 nPort;

            // Create INetMgr object. This step needs to be done only if
            // the INetMgr object has not been created already. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_NET, (void **)&pINetMgr);

            if (pINetMgr)
            {
               // Open socket
               pISocket = INETMGR_OpenSocket (pINetMgr, AEE_SOCK_STREAM); 

               if (!pISocket)
               {
                  // Release INetMgr object. This step needs to be done
                  // only if no further use of the INetMgr object is needed.
                  INETMGR_Release (pINetMgr);

                  return FALSE;
               }
               
            	// Before we can pass on the address to connect the socket, 
               // we need to convert the address from a string to the INAddr format.
               nodeINAddr = ConvertToINAddr(USAGE_TEST_HOST);
               nPort = HTONS(USAGE_TEST_PORT);  // convert port

               // Store INetMgr and ISocket pointers in AEEApplet data structure
               // (to be used in the call-back function).
               pMe->m_pISocket = pISocket;
               pMe->m_pINetMgr = pINetMgr;

               // Connect to the remote host.
               if ((iRetValue = ISOCKET_Connect (pISocket, nodeINAddr, nPort, 
                  CommonConnectCB, (void *)pMe))  != AEE_NET_SUCCESS)
               {
                  // If it's WOULDBLOCK, then it's still connecting. Don't cleanup.
                  if (iRetValue != AEE_NET_WOULDBLOCK)
                  {
                     // Get Last error
                     lastError = ISOCKET_GetLastError(pISocket);
                     // Release NetMgr & ISocket and return.
                     ReleaseNetAndSocket (pMe);
                     return FALSE;
                  }
               }

               // Note: The INetMgr and ISocket pointers are not released
               // here since they will be released when the CommonConnectCB 
               // callback function gets called. We store pointers to both in
               // m_pINetMgr and m_pISocket members of the Applet data structure
               // above.  

               // A common callback function is used for all the connects. Here
               // the API function type that is being addressed is stored in the
               // APIFnType data member of the Applet structure at the begining 
               // of this function (to be used in the callback function).

               // Invoke Display: used purely for display purposes.
               DisplayOutput (pMe, -1, "Connecting...");
            }
         }
         break;
      case USAGE_SOCKET_BIND:
         {
            // 11. Usage ISOCKET_Bind
            INetMgr *pINetMgr = NULL;
            ISocket * pISocket = NULL;
            int iRetValue = AEE_NET_EBADF;
            uint16 nPort = 1240;

            // Create INetMgr object. This step needs to be done only if
            // the INetMgr object has not been created already. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_NET, (void **)&pINetMgr);

            if (pINetMgr)
            {
               // Open Socket
               pISocket = INETMGR_OpenSocket (pINetMgr, AEE_SOCK_DGRAM); 
               pMe->m_pISocket = pISocket;
               pMe->m_pINetMgr = pINetMgr;

               if (!pISocket)
               {
                  // Release INetMgr object. This step needs to be done
                  // only if no further use of the INetMgr object is needed.
                  INETMGR_Release (pINetMgr);
                  return FALSE;
               }
               
               // Bind to local port. The second parameter needs to be 0 since it's
               // not used (the local host IP address is automatically retrieved by
               // the ISOCKET_Bind function.
               if ((iRetValue = ISOCKET_Bind (pISocket, AEE_INADDR_ANY, nPort))  
                  == AEE_NET_WOULDBLOCK)
               {
						// Try again when "writeable";  return and wait for callback
						ISOCKET_Writeable(pMe->m_pISocket, CommonWriteCB, (void *)pMe);
					}
					else
					{
						if (iRetValue != AEE_NET_SUCCESS)
							DisplayOutput (pMe, -1, "Bind Failed.");
						else
						{
							// Display output. This function is for display purposes only.
							DisplayOutput (pMe, -1, "Bind Succeeded.");
							ReleaseNetAndSocket (pMe);
						}
					}
					return TRUE;
				}
				return FALSE;

         }
         break;
      case USAGE_SOCKET_WRITE:
      case USAGE_SOCKET_WRITEABLE:
         {
            // 12. Usage ISOCKET_Write & ISOCKET_Writeable
            INetMgr *pINetMgr = NULL;
            ISocket * pISocket = NULL;
            int iRetValue = AEE_NET_EBADF, lastError;
            INAddr nodeINAddr;
            uint16 nPort;

            // Create INetMgr object. This step needs to be done only if
            // the INetMgr object has not been created already. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_NET, (void **)&pINetMgr);

            if (pINetMgr)
            {
               // Open Socket
               pISocket = INETMGR_OpenSocket (pINetMgr, AEE_SOCK_STREAM); 

               if (!pISocket)
               {
                  // Release INetMgr object. This step needs to be done
                  // only if no further use of the INetMgr object is needed.
                  INETMGR_Release (pINetMgr);

                  return FALSE;
               }
               
            	// Before we can pass on the address to connect the socket, 
               // we need to convert the address from a string to the INAddr format.
               nodeINAddr = ConvertToINAddr(USAGE_TEST_HOST);
               nPort = HTONS(USAGE_TEST_PORT);  // convert port

               // Store INetMgr and ISocket pointers in AEEApplet data structure
               // (to be used in the call-back function).
               pMe->m_pISocket = pISocket;
               pMe->m_pINetMgr = pINetMgr;

               // To write data we need to connect to first a remote host.
               // ISOCKET_Write and ISOCKET_Writable function usages are found
               // in the CommonConnectCB callback function. 
               if ((iRetValue = ISOCKET_Connect (pISocket, nodeINAddr, nPort, 
                  CommonConnectCB, (void *)pMe))  != AEE_NET_SUCCESS)
               {
                  // If it's WOULDBLOCK, then it's still connecting. Don't cleanup.
                  if (iRetValue != AEE_NET_WOULDBLOCK)
                  {
                     // Get Last error
                     lastError = ISOCKET_GetLastError(pISocket);
                     // Release NetMgr & ISocket and return.
                     ReleaseNetAndSocket (pMe);
                     return FALSE;
                  }
               }

               // Note: The INetMgr and ISocket pointers are not released
               // here since they will be released when the CommonConnectCB 
               // callback function gets called. We store pointers to both in
               // m_pINetMgr and m_pISocket members of the Applet data structure
               // above.  

               // A common callback function is used for all the connects. Here
               // the API function type that is being addressed is stored in the
               // APIFnType data member of the Applet structure at the begining 
               // of this function (to be used in the callback function).

               // Invoke Display: used purely for display purposes.
               DisplayOutput (pMe, -1, "Connecting...");
            }
         }
         break;
      case USAGE_SOCKET_READV:
      case USAGE_SOCKET_WRITEV:
         {
            // 13. Usage ISOCKET_ReadV and ISOCKET_WriteV
            INetMgr *pINetMgr = NULL;
            ISocket * pISocket = NULL;
            int iRetValue = AEE_NET_EBADF, lastError;
            INAddr nodeINAddr;
            uint16 nPort;

            // Create INetMgr object. This step needs to be done only if
            // the INetMgr object has not been created already. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_NET, (void **)&pINetMgr);

            if (pINetMgr)
            {
               // Open Socket
               pISocket = INETMGR_OpenSocket (pINetMgr, AEE_SOCK_STREAM); 

               if (!pISocket)
               {
                  // Release INetMgr object. This step needs to be done
                  // only if no further use of the INetMgr object is needed.
                  INETMGR_Release (pINetMgr);

                  return FALSE;
               }

            	// Before we can pass on the address to connect the socket, 
               // we need to convert the address from a string to the INAddr format.
               nodeINAddr = ConvertToINAddr(USAGE_TEST_HOST);
               nPort = HTONS(USAGE_TEST_PORT);  // convert port

               // Store INetMgr and ISocket pointers in AEEApplet data structure
               // (to be used in the call-back function).
               pMe->m_pISocket = pISocket;
               pMe->m_pINetMgr = pINetMgr;

               // To write and read data we need to connect to first a remote host.
               // ISOCKET_ReadV and ISOCKET_WriteV function usages are found 
               // in the CommonConnectCB callback function. 
               if ((iRetValue = ISOCKET_Connect (pISocket, nodeINAddr, nPort, 
                  CommonConnectCB, (void *)pMe))  != AEE_NET_SUCCESS)
               {
                  // If it's WOULDBLOCK, then it's still connecting. Don't cleanup.
                  if (iRetValue != AEE_NET_WOULDBLOCK)
                  {
                     // Get Last error
                     lastError = ISOCKET_GetLastError(pISocket);
                     // Release NetMgr & ISocket and return.
                     ReleaseNetAndSocket (pMe);
                     return FALSE;
                  }
               }

               // Note: The INetMgr and ISocket pointers are not released
               // here since they will be released when the CommonConnectCB 
               // callback function gets called. We store pointers to both in
               // m_pINetMgr and m_pISocket members of the Applet data structure
               // above.  

               // A common callback function is used for all the connects. Here
               // the API function type that is being addressed is stored in the
               // APIFnType data member of the Applet structure at the begining 
               // of this function (to be used in the callback function).

               // Invoke Display: used purely for display purposes.
               DisplayOutput (pMe, -1, "Connecting...");
            }
         }
         break;
      case USAGE_SOCKET_READ:
      case USAGE_SOCKET_READBALE:
         {
            // 14. Usage ISOCKET_Read & ISOCKET_Readable
            INetMgr *pINetMgr = NULL;
            ISocket * pISocket = NULL;
            int32 iRetValue = 0;
            INAddr nodeINAddr;
            uint16 nPort;
            int lastError;

            // Create INetMgr object. This step needs to be done only if
            // the INetMgr object has not been created already. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_NET, (void **)&pINetMgr);

            if (pINetMgr)
            {
               // Open socket
               pISocket = INETMGR_OpenSocket (pINetMgr, AEE_SOCK_STREAM); 

               if (!pISocket)
               {
                  // Release INetMgr object. This step needs to be done
                  // only if no further use of the INetMgr object is needed.
                  INETMGR_Release (pINetMgr);

                  return FALSE;
               }
               
            	// Before we can pass on the address to connect the socket, 
               // we need to convert the address from a string to the INAddr format.
               nodeINAddr = ConvertToINAddr(USAGE_TEST_HOST);
               nPort = HTONS(TIMESERVER_PORT);  // convert port

               // Store INetMgr and ISocket pointers in AEEApplet data structure
               // (to be used in the call-back function).
               pMe->m_pISocket = pISocket;
               pMe->m_pINetMgr = pINetMgr;

               // To read data we need to first connect to a remote host.
               // ISOCKET_Read and ISOCKET_Readable function usages are found 
               // in the CommonConnectCB callback function. 
               if ((iRetValue = ISOCKET_Connect (pISocket, nodeINAddr, nPort, 
                  CommonConnectCB, (void *)pMe))  != AEE_NET_SUCCESS)
               {
                  // If it's WOULDBLOCK, then it's still connecting. Don't cleanup.
                  if (iRetValue != AEE_NET_WOULDBLOCK)
                  {
                     // Get Last error
                     lastError = ISOCKET_GetLastError(pISocket);
                     // Release NetMgr & ISocket and return.
                     ReleaseNetAndSocket (pMe);
                     return FALSE;
                  }
               }

               // Note: The INetMgr and ISocket pointers are not released
               // here since they will be released when the CommonConnectCB 
               // callback function gets called. We store pointers to both in
               // m_pINetMgr and m_pISocket members of the Applet data structure
               // above.  

               // A common callback function is used for all the connects. Here
               // the API function type that is being addressed is stored in the
               // APIFnType data member of the Applet structure at the begining 
               // of this function (to be used in the callback function).

               // Invoke Display: used for display purposes only.
               DisplayOutput (pMe, -1, "Connecting...");
            }
         }
         break;
      case USAGE_SOCKET_SENDTO:
      case USAGE_SOCKET_RECVFROM:
         {
            // 15. Usage ISOCKET_SendTo
            INetMgr *pINetMgr = NULL;
            ISocket * pISocket = NULL;
            int32 iRetValue = 0;
            INAddr nodeINAddr;
            uint16 nPort;
            int  rv;
            char pszSendData[100] = "UDP packet sent to and received from  echo-port of remote host...";
            char pszRecvData[100] = {0};

            // Create INetMgr object. This step needs to be done only if
            // the INetMgr object has not been created already. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_NET, (void **)&pINetMgr);

            if (pINetMgr)
            {
               // Open socket
               pISocket = INETMGR_OpenSocket (pINetMgr, AEE_SOCK_DGRAM); 

               if (!pISocket)
               {
                  // Release INetMgr object. This step needs to be done
                  // only if no further use of the INetMgr object is needed.
                  INETMGR_Release (pINetMgr);

                  return FALSE;
               }
               
               // Store INetMgr and ISocket pointers in AEEApplet data structure
               // (to be used in the call-back function).
               pMe->m_pISocket = pISocket;
               pMe->m_pINetMgr = pINetMgr;

            	// Before we can pass on the address to connect the socket, 
               // we need to convert the address from a string to the INAddr format.
               nodeINAddr = ConvertToINAddr(USAGE_TEST_HOST);
               nPort = HTONS(USAGE_TEST_PORT);  // convert port

               // Register a callback function in case all the data is not
               // written by the ISOCKET_SentTo function. This registration needs
               // to be done prior to calling ISOCKET_SendTo.
               ISOCKET_Writeable (pMe->m_pISocket,
                  CommonWriteCB, (void *)pMe);

               if ((iRetValue = ISOCKET_SendTo (pMe->m_pISocket, 
                  (byte *)pszSendData,(uint16)STRLEN(pszSendData), 0, nodeINAddr, nPort)) == AEE_NET_ERROR)
               {
                  ReleaseNetAndSocket (pMe);
                  return FALSE;
               }
               
               if (iRetValue > 0)
               {
                  // Register a callback function in case all the data is not read
                  // from the ISOCKET_RecvFrom function. This callback function 
                  // needs to be done prior to the calling of ISOCKETRecvFrom.
                  ISOCKET_Readable(pMe->m_pISocket, 
                     CommonReadCB, (void*)pMe);
   
                  // Receive connection-less data from the remote host 
                  rv = ISOCKET_RecvFrom (pMe->m_pISocket, 
                     (byte *)pszRecvData, sizeof (pszRecvData), 0, &nodeINAddr, &nPort);

                  if (rv == AEE_NET_WOULDBLOCK)
                  {
                     // Here we don't have to register a callback function because
                     // a callback function was registered prior to calling ISOCKET_RecvFrom.
                     // So return and wait for the callback to get activated.
         	         return TRUE;
                  } 
                  else if (rv > 0)
                  {
                     // Cancel the previously registered call-back function
                     // because all the data was read (received).
                     ISOCKET_Cancel (pISocket, CommonReadCB, (void*)pMe);

                     // Display output starting at line 5.
                     DisplayOutput (pMe, 5, pszRecvData);   // Display Data
                     ReleaseNetAndSocket (pMe);
                  }
                  else
                  {
                     // Release Network and Socket objects.
                     ReleaseNetAndSocket (pMe);
                  }
               }
            }
         }
         break;
      case USAGE_SOCKET_CANCEL:
         {
            // 16. Usage ISOCKET_ReadV
            INetMgr *pINetMgr = NULL;
            ISocket * pISocket = NULL;
            int32 iRetValue = 0;
            INAddr nodeINAddr;
            uint16 nPort;
            int lastError;

            // Create INetMgr object. This step needs to be done only if
            // the INetMgr object has not been created already. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_NET, (void **)&pINetMgr);

            if (pINetMgr)
            {
               // Open socket
               pISocket = INETMGR_OpenSocket (pINetMgr, AEE_SOCK_STREAM); 

               if (!pISocket)
               {
                  // Release INetMgr object. This step needs to be done
                  // only if no further use of the INetMgr object is needed.
                  INETMGR_Release (pINetMgr);

                  return FALSE;
               }
               
            	// Before we can pass on the address to connect the socket, 
               // we need to convert the address from a string to the INAddr format.
               nodeINAddr = ConvertToINAddr(USAGE_TEST_HOST);
               // convert port 
               nPort = HTONS(USAGE_TEST_PORT);

               // Store INetMgr and ISocket pointers in AEEApplet data structure
               // (to be used in the call-back function).
               pMe->m_pISocket = pISocket;
               pMe->m_pINetMgr = pINetMgr;

               // First Connect
               if ((iRetValue = ISOCKET_Connect (pISocket, nodeINAddr, nPort, 
                  CommonConnectCB, (void *)pMe))  != AEE_NET_SUCCESS)
               {
                  // If it's WOULDBLOCK, then it's connecting. Don't cleanup.
                  if (iRetValue != AEE_NET_WOULDBLOCK)
                  {
                     // Get Last error
                     lastError = ISOCKET_GetLastError(pISocket);
                  }
               }

               // Cancel call-back function
               ISOCKET_Cancel (pISocket, 0, 0);
               DisplayOutput (pMe, -1, "Socket Canceled");

               // We need to let the call-back function cancellation 
               // take effect before releasing the INetMgr and ISocket pointers.
               // The clean up happens when the next function is invoked or 
               // the applet is exited.
            }
         }
         break;
      case USAGE_SOCKET_SHUTDOWN:
         // not implemented yet
         break;
      default:
         return FALSE;
    
   }
            
   // Display above event. 
   DisplayEvent (pMe, wParam);

   return TRUE;
}

/*===========================================================================

FUNCTION: DisplayEvent

DESCRIPTION
    This function is a display function which sets up the initial screen
    for the display output for a given interface function. This displays
    the function name at the top of the screen and displays the
    strings "Press Up/Down" and "keys to continue.." at the bottom of 
    the screen. In the case of functions that have no output it also 
    displays the string "No Output..." in the middile of the screen.
    
PROTOTYPE:
   void DisplayEvent (CINetSktApp * pMe, uint16 wParam)

PARAMETERS:
   pIShell: [in]: Pointer to IShell interface object
   wParam:  [in]: API function type Id selected by user

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
void DisplayEvent (CINetSktApp * pMe, uint16 wParam)
{
   AEEDeviceInfo di; // Device Info
   AECHAR szBuf[100];
   AEERect rc; 
   int charHeight = 0;      // Stores the char height in pixels for given font
   int pnAscent = 0;        // Stores the ascent in number of pixels
   int pnDescent = 0;       // Stores the descent in number of pixels

   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
      return;

   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);
   rc.x = 0;
   rc.y = 0;
   rc.dx = di.cxScreen;
   rc.dy = di.cyScreen;
   
   if (pMe->a.m_pIDisplay)
   {
      // Get the font metrics info
      charHeight = IDISPLAY_GetFontMetrics (pMe->a.m_pIDisplay, AEE_FONT_NORMAL,
         &pnAscent, &pnDescent);

      switch (wParam)
      {
         case USAGE_ERASE_SCREEN:
            IDISPLAY_EraseRect(pMe->a.m_pIDisplay,&rc);  //Clear the display
            return;
         case USAGE_BASIC_USAGE:
            STR_TO_WSTR("Basic Usage", szBuf, sizeof(szBuf));
            break;
         case USAGE_NET_GETHOSTBYNAME:
            STR_TO_WSTR("Net: GetHostByName", szBuf, sizeof(szBuf));
            break;
         case USAGE_NET_GETLASTERROR:
            STR_TO_WSTR("Net: GetLastError", szBuf, sizeof(szBuf));
            break;
         case USAGE_NET_OPENSOCKET:
            STR_TO_WSTR("Net: OpenSocket", szBuf, sizeof(szBuf));
            break;
         case USAGE_NET_SETLINGER:
            STR_TO_WSTR("Net: SetLinger", szBuf, sizeof(szBuf));
            break;
         case USAGE_NET_NETSTATUS:
            STR_TO_WSTR("Net: NetStatus", szBuf, sizeof(szBuf));
            break;
         case USAGE_NET_GETMYIPADDRESS:
            STR_TO_WSTR("Net: GetMyIPAddress", szBuf, sizeof(szBuf));
            break;
         case USAGE_SOCKET_GETPEERNAME:
            STR_TO_WSTR("Socket: GetPeerName", szBuf, sizeof(szBuf));
            break;
         case USAGE_SOCKET_GETLASTERROR:
            STR_TO_WSTR("Socket: GetLastError", szBuf, sizeof(szBuf));
            break;
         case USAGE_SOCKET_CONNECT:
            STR_TO_WSTR("Socket: Connect", szBuf, sizeof(szBuf));
            break;
         case USAGE_SOCKET_BIND:
            STR_TO_WSTR("Socket: Bind", szBuf, sizeof(szBuf));
            break;
         case USAGE_SOCKET_WRITE:
            STR_TO_WSTR("Socket: Write", szBuf, sizeof(szBuf));
            break;
         case USAGE_SOCKET_WRITEV:
            STR_TO_WSTR("Socket: WriteV", szBuf, sizeof(szBuf));
            break;
         case USAGE_SOCKET_READ:
            STR_TO_WSTR("Socket: Read", szBuf, sizeof(szBuf));
            break;
         case USAGE_SOCKET_READV:
            STR_TO_WSTR("Socket: ReadV", szBuf, sizeof(szBuf));
            break;
         case USAGE_SOCKET_SENDTO:
            STR_TO_WSTR("Socket: SendTo", szBuf, sizeof(szBuf));
            break;
         case USAGE_SOCKET_RECVFROM:
            STR_TO_WSTR("Socket: RecvFrom", szBuf, sizeof(szBuf));
            break;
         case USAGE_SOCKET_READBALE:
            STR_TO_WSTR("Socket: Readable", szBuf, sizeof(szBuf));
            break;
         case USAGE_SOCKET_WRITEABLE:
            STR_TO_WSTR("Socket: Writeable", szBuf, sizeof(szBuf));
            break;
         case USAGE_SOCKET_CANCEL:
            STR_TO_WSTR("Socket: Cancel", szBuf, sizeof(szBuf));
            break;
         case USAGE_SOCKET_SHUTDOWN:
            STR_TO_WSTR("Socket: ShutDown", szBuf, sizeof(szBuf));
            break;
         default:
            STR_TO_WSTR("Unsupported fn", szBuf, sizeof(szBuf));
      }
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1, 5, 5, &rc,
		   IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);

      // Functions with no output
      switch (wParam)
      {
          case USAGE_SOCKET_SHUTDOWN:
            // Place in the middle of screen
            STR_TO_WSTR("No Output...", szBuf, sizeof(szBuf));
 		      IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1, 10, 
               rc.dy/2 - 10, &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
         default:
            break;
      }

      STR_TO_WSTR("Press Up/Down", szBuf, sizeof(szBuf));
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1, 5, rc.dy - 
         2*(charHeight) , &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
      STR_TO_WSTR("keys to continue..", szBuf, sizeof(szBuf));
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1, 5, rc.dy - 
         charHeight, &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
		IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);
   }
}

/*===========================================================================

FUNCTION: BuildMainMenu

DESCRIPTION
    This function builds the main menu when this applet is started.
    It displays a simple menu with menu items corresponding to 
    each of the interface functions.
    
PROTOTYPE:
   void BuildMainMenu(AEEApplet *pMe)

PARAMETERS:
   pMe: [in]: Pointer to AEEApplet struct

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
void BuildMainMenu(CINetSktApp *pMe)
{
	AEERect qrc;
	AEEDeviceInfo di;
   AECHAR szBuf[100] = {0};

	// Set Title
   STR_TO_WSTR("INet-ISocket Funcs:", szBuf, sizeof(szBuf));
	IMENUCTL_SetTitle(pMe->m_pIMenu, NULL, 0, szBuf);

	//Set the Rectangle for the Menu
	ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);
	qrc.x = 0;
	qrc.y = 0;
	qrc.dx = di.cxScreen;
	qrc.dy = di.cyScreen;
	IMENUCTL_SetRect(pMe->m_pIMenu, &qrc);  

	//Add invidual entries to the Menu

   // Add Basic Usage to menu
   STR_TO_WSTR("1. Basic Usage", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_BASIC_USAGE, szBuf, 0);

   STR_TO_WSTR("2. Net: GetHostByName", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_NET_GETHOSTBYNAME, szBuf, 0);
			
   // Add Net-GetLastError to menu
   STR_TO_WSTR("3. Net: GetLastError", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_NET_GETLASTERROR, szBuf, 0);

   // Add Net-OpenSocket to menu
   STR_TO_WSTR("4. Net: OpenSocket", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_NET_OPENSOCKET, szBuf, 0);

	// Add Net-SetLinger to menu
   STR_TO_WSTR("5. Net: SetLinger", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_NET_SETLINGER, szBuf, 0);

	// Add Net-NetStatus to menu
   STR_TO_WSTR("6. Net: NetStatus", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_NET_NETSTATUS, szBuf, 0);	

	// Add Net-GetMyIPAddress to menu
   STR_TO_WSTR("7. Net: GetMyIPAddr", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_NET_GETMYIPADDRESS, szBuf, 0);	

	// Add Socket-GetPeerName to menu
   STR_TO_WSTR("8. SKT: GetPeerName", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SOCKET_GETPEERNAME, szBuf, 0);	

   // Add Socket-GetLastError to menu
   STR_TO_WSTR("9. SKT: GetLastError", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SOCKET_GETLASTERROR, szBuf, 0);	

   // Add Socket-Connect to menu
   STR_TO_WSTR("10. SKT: Connect", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SOCKET_CONNECT, szBuf, 0);	

   // Add Socket-Bind to menu
   STR_TO_WSTR("11. SKT: Bind", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SOCKET_BIND, szBuf, 0);	

	// Add Socket-Write to menu
   STR_TO_WSTR("12. SKT: Write", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SOCKET_WRITE, szBuf, 0);	

	// Add Socket-WriteV to menu
   STR_TO_WSTR("13. SKT: WriteV", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SOCKET_WRITEV, szBuf, 0);	

	// Add Socket-Read to menu
   STR_TO_WSTR("14. SKT: Read", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SOCKET_READ, szBuf, 0);	

	// Add Socket-ReadV to menu
   STR_TO_WSTR("15. SKT: ReadV", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SOCKET_READV, szBuf, 0);	

	// Add Socket-SendTo to menu
   STR_TO_WSTR("16. SKT: SendTo", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SOCKET_SENDTO, szBuf, 0);	

	// Add Socket-RecvFrom to menu
   STR_TO_WSTR("17. SKT: RecvFrom", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SOCKET_RECVFROM, szBuf, 0);	

	// Add Socket-Readable to menu
   STR_TO_WSTR("18. SKT: Readable", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SOCKET_READBALE, szBuf, 0);	

	// Add Socket-Writeable to menu
   STR_TO_WSTR("19. SKT: Writeable", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SOCKET_WRITEABLE, szBuf, 0);	

	// Add Socket-Cancel to menu
   STR_TO_WSTR("20. SKT: Cancel", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SOCKET_CANCEL, szBuf, 0);	

	// Add Socket-ShutDown to menu
   STR_TO_WSTR("21. SKT: ShutDown", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SOCKET_SHUTDOWN, szBuf, 0);	

   // Activate menu
	IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);
}

/*===========================================================================

FUNCTION: ConvertToINAddr

DESCRIPTION
    This function converts an IP address in the aaa.bbb.ccc.ddd character
    string format to INAddr format.
    
PROTOTYPE:
   INAddr ConvertToINAddr(char *psz)

PARAMETERS:
   psz: [in]: Character string containing the IP address in the a.b.c.d format.

DEPENDENCIES
  None

RETURN VALUE
  Returns the IP address in a 32 bit INAddr format.

SIDE EFFECTS
  None

===========================================================================*/
INAddr ConvertToINAddr(char *psz)
{
   INAddr ul = 0;
   int nByte = 0;
   char c;

   if(!psz)
      return 0;

   while (ISDIGIT(*psz)) {
      int n = 0;
      while ( ISDIGIT(c=*psz)) {
         n = n*10 + (c - '0');
         ++psz;
      }
      ((char*)&ul)[nByte++] = n;
 
      if (nByte == 4 || *psz != '.')
         break;
 
      ++psz;
   }
 
   if (nByte < 4 || ISALNUM(*psz))
      ul = 0xFFFFFFFF;    // Invalid address

   return ul;
}

/*===========================================================================

FUNCTION: ConvertToIPAddr

DESCRIPTION
    This function converts an IP address in the 32 bit INAddr format to the
    aaa.bbb.ccc.ddd character string format.
    
PROTOTYPE:
   void ConvertToIPAddr(INAddr inaddr, char *psz)

PARAMETERS:
   inAddr: [in]: Contains the 32 bit INAddr format IP address to be converted
   psz: [out]: Character string containing the IP address in the a.b.c.d format.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
void ConvertToIPAddr(INAddr inaddr, char *psz)
{
   int i;
   int x[4] = {0};
   char ch[30];
   char ch1[50];

   for (i = 0; i < 4; i++)
   {
      x[i] = (int)(inaddr & 0xff);
      inaddr = inaddr >> 8;     
   }


   for (i = 3; i >= 0; i--)
   {
      SPRINTF (ch,"%d",x[i]);
      if (i != 0)
        STRCAT (ch, ".");

      if (i == 3)
         STRCPY (ch1, ch);
      else
         STRCAT(ch1, ch);
   }

   STRCPY (psz, ch1);   
}

/*===========================================================================

FUNCTION: DisplayOutput

DESCRIPTION
    This function displays an output string at a given line number on the
    screen. If the nline parameter is a negative value (-1) the string
    is displayed in the middle of the screen. If the "nline" value is larger
    than or equal to zero the "nline" value is multiplied by 15 and the 
    resulting value in pixels is set to the y-coordinate of the start of 
    the string display on the screen. If the string does not fit on one line
    the string wraps around to the next line (spaced rougly 10-15 pixels apart).
    By default 5 is used as the starting the x-coordinate of a displayed 
    string.

    How many characters that fit on one line is calculated for each line 
    that is wrapped around to the next line.

    Note: depending on the phone screen size and the fonts used for characters 
          the output might differ on different handsets (devices). Where some 
          handsets will have a smaller screen and large default fonts which will 
          cause partial overlapping of lines. This function does not try to address
          these issues (this is meant as a simple display function).
    
PROTOTYPE:
   void DisplayOutput(CINetSktApp *pMe, int nline,  char *pszStr)

PARAMETERS:
   pMe:    [in]: Contains a pointer to the CINetSktApp class.
   nline:  [in]: Contains the line number to start displaying the text. The line
        numbers are by default spaced 15 pixels apart along the y-axis.
   pszStr: [in]: The character string to be displayed on the screen.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
void DisplayOutput(CINetSktApp * pi, int nline, char *pszStr)
{
   AEEDeviceInfo di; // Device Info
   AECHAR szBuf[200] = {0};      // a buffer that supports 200 char string
   AECHAR * psz = NULL;
   int pixelWidth;
   AEEFont font = AEE_FONT_NORMAL;
   int pnFits = 0, dy;
   int totalCh = 0;
   int charHeight = 0;      // Stores the char height in pixels for given font
   int pnAscent = 0;        // Stores the ascent in number of pixels
   int pnDescent = 0;       // Stores the descent in number of pixels
	AEERect rc;

   AEEApplet * pMe = (AEEApplet*)pi;

   // Get device information
   ISHELL_GetDeviceInfo(pMe->m_pIShell,&di);

   // Get the font metrics info
   charHeight = IDISPLAY_GetFontMetrics (pMe->m_pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   // Convert to wide string (unicode)
   STR_TO_WSTR ((char *)pszStr, szBuf, sizeof (szBuf));

   // If nlines is zero then print this string starting around the middle of 
   // the screen. Or else multiply nlines by charHeight to decide the y coordinate of
   // the start of the string.
   if (nline < 0) {
      dy = di.cyScreen*2/5;
   }
   else{
      dy = nline * charHeight;
   }

   // psz keeps track of the point from which to write from the string buffer
   // in case the string does not fit one line and needs to wrap around in the
   // next line.
   psz = szBuf;
      
   // Need to calculate the lotal string length to decide if any wrapping
   // around is needed.
   totalCh = STRLEN ((char *)pszStr);

   // Keep displaying text string on multiple lines if the string can't be displayed
   // on one single line. Lines are spaced 15 pixels apart.
   while ((totalCh > 0) && (*psz != NULL))
   { 
      // Get information on how many characters will fit in a line.
      // Give the pointer to the buffer to be displayed, and the number of
      // pixels along the x axis you want to display the string in (max number)
      // pnFits will have the max number of chars that will fit in the maxWidth
      // number of pixels (given string can't fit in one line), or the number of 
      // chars in the string (if it does fit in one line). pnWidth gives the
      // number of pixels that will be used to display pnFits number of chars.
      pixelWidth = IDISPLAY_MeasureTextEx(pMe->m_pIDisplay,
                      font, 
                      (AECHAR *) psz,  // Start of the buffer to display,
                      -1,
                      di.cxScreen - 5, // maxWidth
                      &pnFits);         // Number of chars that will fit a line

      // If pnFits is zero there is something wrong in the input to above function. 
      // Normally this scenario should not occur. But, have the check anyway.
      if (pnFits == 0)
         return;

		SETAEERECT(&rc, 0, dy, di.cyScreen, charHeight);
		IDISPLAY_EraseRect(pMe->m_pIDisplay, &rc);

      IDISPLAY_DrawText(pMe->m_pIDisplay, AEE_FONT_NORMAL, psz, pnFits, 5 /*start dx*/, 
         dy, 0 /* use default rectangle coordinates */, 0);

      psz += pnFits;      // move pointer to the next segment to be displayed
      totalCh -= pnFits;  // reduce the total number of characters to still display
      dy += charHeight;   // Place next line charHeight pixels below the 
                          // previous line.
      IDISPLAY_Update(pMe->m_pIDisplay);
      if (totalCh < pnFits)
         pnFits = totalCh;  // if total number is less than pnFits, adjust pnFits
   }

      return;   
} // End of DisplayOutput

/*===========================================================================

FUNCTION: ReleaseNetAndSocket

DESCRIPTION
	Cleans up INetMgr and ISocket objects.

===========================================================================*/
void ReleaseNetAndSocket (CINetSktApp * pMe)
{
   if (pMe->m_pISocket != NULL)
   {
      // Release ISocket Interface. This step needs to be done
      // only if no further use of the ISocket interface needed.
      ISOCKET_Release (pMe->m_pISocket);
      pMe->m_pISocket = 0;
   }
   if (pMe->m_pINetMgr != NULL)
   {
      // Release INetMgr Interface. This step needs to be done
      // only if no further use of the INetMgr interface is needed.
      INETMGR_Release (pMe->m_pINetMgr);
      pMe->m_pINetMgr = 0;
   }
}

static void NetApp_DNSResult(void *pv)
{
   CINetSktApp * pMe = (CINetSktApp*)pv;
   int nRes = pMe->dnsresult.nResult;
   int n;
   char buf[80];

   if (nRes > AEEDNSMAXADDRS) {

      SPRINTF(buf, "DNS error %d", nRes);

   } else {
   
      STRCPY(buf, "Results:");

      for (n=0; n < nRes; ++n) {
         // INAddr's are in host byte order!
         unsigned char *pc = (unsigned char*) & pMe->dnsresult.addrs[n];
         SPRINTF(buf + STRLEN(buf), " %d.%d.%d.%d", pc[0], pc[1], pc[2], pc[3]);
      }
   }

   DisplayOutput(pMe, -1, buf);
}
