/*===========================================================================

FILE: IRingerusage.c

SERVICES: Sample applet using AEE

DESCRIPTION
  This file contains usage examples of IRingerMgr interface. IRingerMgr interface
  functions allow the user to create/operate on IRingerMgr objects. IRingerMgr 
  interface offers basic ringer services.

PUBLIC CLASSES:  
   N/A


   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/

/*===============================================================================
                     INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "AEEModGen.h"
#include "AEEAppGen.h"
#include "AEEShell.h"
#include "AEEStdLib.h"
#include "AEEUsageAppIDs.h"
#include "iringer_res.h"
#include "AEEImage.h"
#include "AEEFile.h"
#include "AEERinger.h"
#include "AEEMenu.h"

/*===========================================================================

                      PUBLIC DATA DECLARATIONS

===========================================================================*/

/*-------------------------------------------------------------------
            Type Declarations
-------------------------------------------------------------------*/
// MemStream app struct. This is the main struct for this applet. This will hold 
// all the data members that needs to be remembered throughout the life of
// the applet. 
// THE FIRST DATA MEMBER OF THIS STRUCT MUST BE AN AEEApplet OBJECT. 
// This is critical requirement which is related to an applet being
// dynamically loaded. The app developers should add their data members
// following the AEEApplet data member.
typedef struct _CIRingerApp
{
	AEEApplet	a;        // Mandatory first AEEApplet data member
   boolean     bOn;
   boolean     bgColor;

   // Device parameters
   int             m_nLineHeight;
   AEEDeviceInfo   m_dInfo;
   IMenuCtl     *  m_pIMenu;
	boolean m_bSPActive;
   // ringer members 
   IRingerMgr *  m_pIRingerMgr;
   IFile *m_pIFile;
   IFileMgr *m_pIFileMgr;
   IMemAStream *m_pMemStream;
	byte *      m_pBuff;
}CIRingerApp;

void CleanInterfaces(CIRingerApp *pMe)
{
   // Release ringermgr object if it is not released
  if (pMe->m_pIRingerMgr != NULL)
  {
      // Deregister Notify so that 
      IRINGERMGR_RegisterNotify (pMe->m_pIRingerMgr, NULL, NULL);
      // Stop any songs being played at this time
      IRINGERMGR_Stop (pMe->m_pIRingerMgr);
      IRINGERMGR_Release (pMe->m_pIRingerMgr);
      pMe->m_pIRingerMgr = NULL;
  }

  if(pMe->m_pIFile != NULL)
  {
       IFILE_Release(pMe->m_pIFile);
	   pMe->m_pIFile = NULL;
  }

  if(pMe->m_pIFileMgr != NULL)
  {
		IFILEMGR_Release(pMe->m_pIFileMgr);
		pMe->m_pIFileMgr = NULL;
  }

  if(pMe->m_pMemStream != NULL)
  {
		IMEMASTREAM_Release(pMe->m_pMemStream);
		pMe->m_pMemStream = NULL;
  }
}

int PlayRingerFile(CIRingerApp *pMe, const char * filename);


int CreateRinger(CIRingerApp *pMe, const char * ringername, const char * inputfile, AEESoundPlayerFile format);


/*-------------------------------------------------------------------
            Function Prototypes
-------------------------------------------------------------------*/
// App Handle Event function
static boolean MemStreamApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);

// App specific data alloc-init/free functions
static boolean IRinger_InitAppData(IApplet* pMe);
static void IRinger_FreeAppData(IApplet* pMe);

static void IRingerUsage (CIRingerApp * pMe, uint16 wParam);
static void BuildMainMenu(CIRingerApp *pMe);

/*-------------------------------------------------------------------
            Global Constant Definitions
-------------------------------------------------------------------*/
// App Resource File
#define  APP_RES_FILE      "iringer.bar"

// App specific constants
#define USAGE_BASIC_USAGE			100
#define USAGE_REGISTERNOTIFY		101
#define USAGE_GETNUMBERFORMATS      102    
#define USAGE_GETFORMATS            103
#define USAGE_SETRINGER             104
#define USAGE_ENUMCATEGORYINIT      105   
#define USAGE_ENUMNEXTCATEGORY      106
#define USAGE_ENUMRINGERINIT        107 
#define USAGE_ENUMNEXTRINGER		108
#define USAGE_GETRINGERID			109
#define USAGE_GETRINGERINFO			110
#define USAGE_CREATE				111
#define USAGE_REMOVE				112
#define USAGE_PLAYEX				113
#define USAGE_PLAY					114     
#define USAGE_PLAYFILE				115  
#define USAGE_PLAYSTREAM			116
#define USAGE_STOP					117        

/*===============================================================================
                     FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to IRinger_InitAppData(). 
   This is done as follows: InitAppData() is called to initialize AppletData 
   instance. It is app developers responsibility to fill-in app data initialization 
   code of InitAppData(). App developer is also responsible to release memory 
   allocated for data contained in AppletData -- this can be done in 
   IRinger_FreeAppData().

PROTOTYPE:
   int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell object. 

	pIModule: pin]: Contains pointer to the IModule object to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid IApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the app needs to be loaded and if AEEApplet_New() invocation was
     successful
  EFAILED: If the app does not need to be loaded or if errors occurred in 
     AEEApplet_New(). If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
{
   *ppObj = NULL;
		
   // We want to load this App. So, invoke AEEApplet_New().To it, pass the
   // address of the app-specific handle event function.
   if(ClsId == AEECLSID_RINGER_APP){
      if(AEEApplet_New(sizeof(CIRingerApp), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)MemStreamApp_HandleEvent,(PFNFREEAPPDATA)IRinger_FreeAppData)
         == TRUE)
      {
         if (IRinger_InitAppData((IApplet*)*ppObj) == TRUE)
         {
            return(AEE_SUCCESS);
         }
      }
   }
	return (EFAILED);
}


//Ringer Mgr callback function
void RingerMgrCBFn(void * pUser, AEERingerEvent evt, uint32 dwParm, int nErr)
{

}



/*===========================================================================

FUNCTION MemStreamApp_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean MemStreamApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

	ecode: Specifies the Event sent to this applet

   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/
static boolean MemStreamApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
   CIRingerApp * pMe = (CIRingerApp*)pi;

   switch (eCode) 
	{
	   case EVT_COMMAND:
         // When a menu item is selected this applet receives the
         // EVT_COMMAND event in the wParam field.
  		   switch(wParam)
         {

		   case USAGE_BASIC_USAGE:
		   case USAGE_REGISTERNOTIFY:
		   case USAGE_GETNUMBERFORMATS :   
		   case USAGE_GETFORMATS :
		   case USAGE_SETRINGER  :
		   case USAGE_ENUMCATEGORYINIT :
		   case USAGE_ENUMRINGERINIT : 
		   case USAGE_ENUMNEXTRINGER:
		   case USAGE_GETRINGERID:
		   case USAGE_GETRINGERINFO:
		   case USAGE_CREATE:
		   case USAGE_REMOVE:
		   case USAGE_PLAYEX:
		   case USAGE_PLAY	:     
		   case USAGE_PLAYFILE:
		   case USAGE_PLAYSTREAM:
		   case USAGE_STOP	: 
			   IRingerUsage (pMe, wParam);
			return TRUE;

		   default: return FALSE; 

		   }
       case EVT_APP_START:
		  // Create the IMenu interface object. When the applet is started the 
         // main menu is set up. When the user selects a menu item from the main 
         // menu, an EVT_COMMAND event is received by this applet. The processing for 
         // EVT_COMMAND event is found below.
		   if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, (void **)&pMe->m_pIMenu)
            != SUCCESS)
         {
				return FALSE;
         }
		  

         // Build the main menu
			BuildMainMenu(pMe);

		   
			return(TRUE);
      case EVT_APP_STOP:
         return(TRUE);
      case EVT_KEY:
        CleanInterfaces(pMe);

         

         // The key is probably the Up/Down keys. In this case we should 
         // call the IMenuCtl object so that the Up/Down key presses are
         // handled by the MenuCtl.
         if((pMe->m_pIMenu != NULL) && IMENUCTL_HandleEvent(pMe->m_pIMenu, 
            EVT_KEY, wParam, 0))
         {
				return TRUE;
         }
         else
         {
            // We would have come here because the pressed key was not handled 
            // by either the RingerPlayer or the MenuCtl HandleEvent functions (the 
            // keys handled by the above controls are Up/Down/Select/Left/Right).
            //
            // Hence, we have reached here since 0-9, *, or # keys were
            // pressed. So redraw the main applet menu and return true.
            if (pMe->m_pIMenu)
               IMENUCTL_Redraw (pMe->m_pIMenu);
         }
         return TRUE;
         break;
   }
   return FALSE;
}

/*===========================================================================

FUNCTION IRinger_InitAppData

DESCRIPTION
   This function initializes app specific data allocates memory for app data 
   (AppletData) and sets it to pAppData of AEEApplet. It also function . 
   App developer needs to fill-in the initialization code.

PROTOTYPE:
	boolean IRinger_InitAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains 
   information specific to this applet. It was initialized during the 
   AEEClsCreateInstance().

DEPENDENCIES
   Assumes pi is not NULL

RETURN VALUE
  TRUE: If the app has app data is allocated and initialized successfully
  FALSE: Either app data could not be allocated or initialzied

SIDE EFFECTS
  none
===========================================================================*/
static boolean IRinger_InitAppData(IApplet* pi)
{
   CIRingerApp * pMe = (CIRingerApp*)pi;
   int pnAscent = 0;        // Stores the ascent in number of pixels
   int pnDescent = 0;       // Stores the descent in number of pixels

   // Initialize the MenuCtl and ringer pointers
   pMe->m_pIMenu = NULL;
   pMe->m_pIRingerMgr = NULL;
   pMe->m_bSPActive = FALSE;

   // Get the font metrics info
   pMe->m_nLineHeight = IDISPLAY_GetFontMetrics (pMe->a.m_pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

	pMe->m_pIFile = NULL;
	pMe->m_pIFileMgr = NULL;
	pMe->m_pMemStream = NULL;
	pMe->m_pBuff = NULL;

   return TRUE;
}

/*===========================================================================

FUNCTION IRinger_FreeAppData

DESCRIPTION
   Frees data contained in app data and memory for app data itself.
   App developer needs to free data contained in AppletData.

PROTOTYPE:
	void IRinger_FreeAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains 
   information specific to this applet. It was initialized during the 
   AEEClsCreateInstance().

DEPENDENCIES
   Assumes pi is not NULL

RETURN VALUE
   None

SIDE EFFECTS
   None
===========================================================================*/
static void IRinger_FreeAppData(IApplet* pi)
{
      CIRingerApp * pMe = (CIRingerApp*)pi;
	CleanInterfaces(pMe);
	if(pMe->m_pIMenu != NULL)
	{
	IMENUCTL_Release(pMe->m_pIMenu);
	pMe->m_pIMenu = NULL;
	}


}

/*===============================================================================
                     APP HELPER FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: IRingerUsage

DESCRIPTION
   
PROTOTYPE:
   void IRingerUsage (CIRingerApp * pMe, uint16 wParam)

PARAMETERS:

DEPENDENCIES
  none

RETURN VALUE
  none

SIDE EFFECTS
  none
===========================================================================*/

static void IRingerUsage (CIRingerApp * pMe, uint16 wParam)
{
  

	switch(wParam)
	{	
		 case USAGE_GETNUMBERFORMATS :
			 {
			 int NumberOfFormats = -1;
			 ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_RINGERMGR, (void **)&pMe->m_pIRingerMgr); 
			 if(pMe->m_pIRingerMgr == NULL)
					{
						return;

					}
			 NumberOfFormats = IRINGERMGR_GetNumberFormats(pMe->m_pIRingerMgr);
  			 CleanInterfaces(pMe);
			 }
			 break;

		 case USAGE_GETFORMATS :
			 {
			 AEESoundPlayerFile pwFormats[4];
			 int NumberOfFormats = -1;
			 ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_RINGERMGR, (void **)&pMe->m_pIRingerMgr); 
			 if(pMe->m_pIRingerMgr == NULL)
					{
						return;

					}
			 NumberOfFormats = IRINGERMGR_GetNumberFormats(pMe->m_pIRingerMgr);
			 IRINGERMGR_GetFormats(pMe->m_pIRingerMgr, pwFormats, NumberOfFormats);
  			 CleanInterfaces(pMe);
			 }
			 break;

		 case USAGE_SETRINGER  :
			 {
				AEERingerID ringerId ;
				CreateRinger(pMe, "ring3", "AEE_TONE_0.wav", AEE_SOUNDPLAYER_FILE_MIDI);
				
				ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_RINGERMGR, (void **)&pMe->m_pIRingerMgr); 
				if(pMe->m_pIRingerMgr == NULL)
					{
						return;

					}
				ringerId =	IRINGERMGR_GetRingerID(pMe->m_pIRingerMgr, "ring3.mid");
				IRINGERMGR_SetRinger(pMe->m_pIRingerMgr,1, ringerId);
				CleanInterfaces(pMe);
			 }
			 break;

		 case USAGE_ENUMCATEGORYINIT :
		 case USAGE_ENUMNEXTCATEGORY :
			 {
			  AEERingerCat catinfo;
			  boolean success = TRUE;

			  ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_RINGERMGR, (void **)&pMe->m_pIRingerMgr); 
			  if(pMe->m_pIRingerMgr == NULL)
					{
						return;
					}

			 IRINGERMGR_EnumCategoryInit(pMe->m_pIRingerMgr);
			 while(success)
			 {
			 success = IRINGERMGR_EnumNextCategory(pMe->m_pIRingerMgr,&catinfo);
			 };
			 }
			 break;


		 case USAGE_ENUMRINGERINIT :
		 case USAGE_ENUMNEXTRINGER:
			 {
			  
			  AEERingerInfo ringerinfo;
			  boolean success = TRUE;
//			  int ringerId;
			  
				
			  CreateRinger(pMe, "ring6", "AEE_TONE_0.wav", AEE_SOUNDPLAYER_FILE_MIDI);
			  CreateRinger(pMe, "ring7", "AEE_TONE_0.wav", AEE_SOUNDPLAYER_FILE_MIDI);
	          CreateRinger(pMe, "ring8", "AEE_TONE_0.wav", AEE_SOUNDPLAYER_FILE_MIDI);
     		  CreateRinger(pMe, "ring9", "AEE_TONE_0.wav", AEE_SOUNDPLAYER_FILE_MIDI);





			  ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_RINGERMGR, (void **)&pMe->m_pIRingerMgr); 
			  if(pMe->m_pIRingerMgr == NULL)
					{
						return;
					}

			  IRINGERMGR_EnumRingerInit(pMe->m_pIRingerMgr);

			   while(success)
			   {
				   success = IRINGERMGR_EnumNextRinger(pMe->m_pIRingerMgr, &ringerinfo);
			   }
             
			 }
				break;

		 case USAGE_GETRINGERID:
			 {
				AEERingerID ringerId ;
				CreateRinger(pMe, "ring4", "AEE_TONE_0.wav", AEE_SOUNDPLAYER_FILE_MIDI);
				
				ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_RINGERMGR, (void **)&pMe->m_pIRingerMgr); 
				if(pMe->m_pIRingerMgr == NULL)
					{
						return;

					}
				ringerId =	IRINGERMGR_GetRingerID(pMe->m_pIRingerMgr, "ring4.mid");
				CleanInterfaces(pMe);
			 }
			 break;

		 case USAGE_GETRINGERINFO:
			 {
				AEERingerID ringerId ;
				AEERingerInfo  info;
				CreateRinger(pMe, "ring4", "AEE_TONE_0.wav", AEE_SOUNDPLAYER_FILE_MIDI);
				
				ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_RINGERMGR, (void **)&pMe->m_pIRingerMgr); 
				if(pMe->m_pIRingerMgr == NULL)
					{
						return;

					}
				ringerId =	IRINGERMGR_GetRingerID(pMe->m_pIRingerMgr, "ring4.mid");
				IRINGERMGR_GetRingerInfo(pMe->m_pIRingerMgr, ringerId, &info);
				CleanInterfaces(pMe);
			 }
			 break;

		case USAGE_BASIC_USAGE:
		case USAGE_PLAYFILE:
			{
				PlayRingerFile(pMe, "AEE_TONE_0.wav");
			}
		break;

		case USAGE_REGISTERNOTIFY:
			{
            // Create IRingerMgr interface object. This step needs to be done only
            // if the IRingerMgr interface object have not already been created. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_RINGERMGR, (void **)&pMe->m_pIRingerMgr); 
            if (pMe->m_pIRingerMgr)
            {
               // Register the status callback funtion using 
               // IRINGERMRG_RegisterNotify(). 
               IRINGERMGR_RegisterNotify(pMe->m_pIRingerMgr, &RingerMgrCBFn, (void *)pMe);

               // Cancel the callback notification
               IRINGERMGR_RegisterNotify(pMe->m_pIRingerMgr, NULL, NULL);
               // Release the ringer object
			   CleanInterfaces(pMe);
            }
         }
		 break;


		 case USAGE_CREATE:
			 {
				CreateRinger(pMe, "ring0", "AEE_TONE_0.wav", AEE_SOUNDPLAYER_FILE_MP3);
				CreateRinger(pMe, "ring0", "AEE_TONE_0.wav", AEE_SOUNDPLAYER_FILE_MIDI);

			 }
		 break;

		 case USAGE_REMOVE:
			 {
				AEERingerID ringerId ;
				CreateRinger(pMe, "ring1", "AEE_TONE_0.wav", AEE_SOUNDPLAYER_FILE_MIDI);
				
				ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_RINGERMGR, (void **)&pMe->m_pIRingerMgr); 
				if(pMe->m_pIRingerMgr == NULL)
					{
						return;

					}
				ringerId =	IRINGERMGR_GetRingerID(pMe->m_pIRingerMgr, "ring1.mid");
				IRINGERMGR_Remove(pMe->m_pIRingerMgr, ringerId);
				CleanInterfaces(pMe);

			 }
		  break;

		 case USAGE_PLAYEX:
			 {
			   AEERingerID ringerId ;
			   CreateRinger(pMe, "ring2", "AEE_TONE_0.wav", AEE_SOUNDPLAYER_FILE_MP3);
				ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_RINGERMGR, (void **)&pMe->m_pIRingerMgr); 
				if(pMe->m_pIRingerMgr == NULL)
					{
						return;

					}
				ringerId =	IRINGERMGR_GetRingerID(pMe->m_pIRingerMgr, "ring2.mp3");
				IRINGERMGR_PlayEx(pMe->m_pIRingerMgr,ringerId,NULL,NULL,2);
			 }
			 break;
		 case USAGE_PLAY:
			 {
			   AEERingerID ringerId ;
			   CreateRinger(pMe, "ring2", "AEE_TONE_0.wav", AEE_SOUNDPLAYER_FILE_MP3);
				ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_RINGERMGR, (void **)&pMe->m_pIRingerMgr); 
				if(pMe->m_pIRingerMgr == NULL)
					{
						return;

					}
				ringerId =	IRINGERMGR_GetRingerID(pMe->m_pIRingerMgr, "ring2.mp3");
				IRINGERMGR_Play(pMe->m_pIRingerMgr, ringerId, 2);
			 }
			break;
		 case USAGE_PLAYSTREAM:
			 {
				 //At this time , this will not work because the ISoundPlayer does not implement streams yet.
				   FileInfo fi;
					// Create IRingerMgr interface object. This step needs to be done only
					// if the IRingerMgr interface object have not already been created. 
					ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_RINGERMGR, (void **)&pMe->m_pIRingerMgr); 
					if(pMe->m_pIRingerMgr == NULL)
					{
						return;

					}

					// Create an instance of IFileMgr so we can access the  file
				   ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR, (void **)(&pMe->m_pIFileMgr));
				   if (NULL == pMe->m_pIFileMgr)
					{
						return;
					}

							// Open the browser icon bitmap file for reading
				   if (NULL == (pMe->m_pIFile = IFILEMGR_OpenFile(pMe->m_pIFileMgr, "AEE_TONE_0.wav", _OFM_READ)))
					{
						IFILEMGR_Release(pMe->m_pIFileMgr);
						return;
					}

					// Determine the size of the bitmap file and allocate memory
					// to store its contents
					IFILE_GetInfo(pMe->m_pIFile, &fi);
					if (NULL == (pMe->m_pBuff = (byte *) MALLOC(fi.dwSize)))
					{
						IFILE_Release(pMe->m_pIFile);
						IFILEMGR_Release(pMe->m_pIFileMgr);
						return;
					}

					// Read the contents of the ringer file into the
					// area allocated in memory for it
					if (IFILE_Read(pMe->m_pIFile, (void *)pMe->m_pBuff, fi.dwSize) == 0)
					{
						FREE(pMe->m_pBuff);
						IFILE_Release(pMe->m_pIFile);
						IFILEMGR_Release(pMe->m_pIFileMgr);
						return;
					}
	
					ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MEMASTREAM, (void **)(&pMe->m_pMemStream));
					if (NULL == pMe->m_pMemStream)
					{
						FREE(pMe->m_pBuff);
						IFILE_Release(pMe->m_pIFile);
						IFILEMGR_Release(pMe->m_pIFileMgr);
						return;
					}
				//	IASTREAM_AddRef((IAStream*)pMe->m_pMemStream);
				//IASTREAM_AddRef((IAStream*)pIFile);
				//	pMe->m_pMemStream =pMemStream;
						
				//	IFILE_Release(pIFile);
				//	IFILEMGR_Release(pIFileMgr);
				//	pMe->m_pIRingerMgr = pIRingerMgr;

					IMEMASTREAM_Set(pMe->m_pMemStream,pMe->m_pBuff, fi.dwSize, 0, 0);
                    IRINGERMGR_PlayStream(pMe->m_pIRingerMgr, (IAStream*)pMe->m_pMemStream, 2);

			 }
		 break;


		 case USAGE_STOP	: 
			{
				IRingerMgr *pIRingerMgr = NULL;
				// Create an instance of IRingerMgr
				ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_RINGERMGR, (void **)(&pIRingerMgr));
				if (NULL == pIRingerMgr)
				{
					return;
				}
				pMe->m_pIRingerMgr = pIRingerMgr;
				IRINGERMGR_PlayFile(pIRingerMgr,"AEE_TONE_1.wav", 2);
				IRINGERMGR_Stop(pIRingerMgr);
			}

			 break;
	}

}



static void BuildMainMenu(CIRingerApp *pMe)
{
	AEERect qrc;
	AEEDeviceInfo di;
	AECHAR szBuf[50];

	if (pMe == NULL || pMe->m_pIMenu == NULL || pMe->a.m_pIShell == NULL)
      return;

	// Set Title
	STR_TO_WSTR("IRingerMgr Examples", szBuf, sizeof(szBuf));
	IMENUCTL_SetTitle(pMe->m_pIMenu, NULL, 0, szBuf);

	//Set the Rectangle for the Menu
	ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);
	qrc.x = 0;
	qrc.y = 0;
	qrc.dx = di.cxScreen;
	qrc.dy = di.cyScreen;
	IMENUCTL_SetRect(pMe->m_pIMenu, &qrc);  

	//Add invidual entries to the Menu

   // Add Basic Usage to menu
    STR_TO_WSTR("1. Basic Usage", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_BASIC_USAGE, szBuf, 0);

    STR_TO_WSTR("2. Create", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_CREATE, szBuf, 0);

    STR_TO_WSTR("3. Play", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_PLAY, szBuf, 0);

    STR_TO_WSTR("4. PlayEx",szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_PLAYEX, szBuf, 0);

    STR_TO_WSTR("5. PlayFile", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_PLAYFILE, szBuf, 0);

    STR_TO_WSTR("6. PlayStream", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_PLAYSTREAM, szBuf, 0);	

    STR_TO_WSTR("7. Stop", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_STOP, szBuf, 0);	

    STR_TO_WSTR("8. RegisterNotify", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_REGISTERNOTIFY, szBuf, 0);

    STR_TO_WSTR("9. GetNumberFormats", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETNUMBERFORMATS, szBuf, 0);

    STR_TO_WSTR("10. GetFormats.", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETFORMATS, szBuf, 0);

    STR_TO_WSTR("11. SetRinger", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETRINGER, szBuf, 0);

    STR_TO_WSTR("12. EnumCategoryInit", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_ENUMCATEGORYINIT, szBuf, 0);	

    STR_TO_WSTR("13. EnumNextCategory", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_ENUMNEXTCATEGORY, szBuf, 0);	

    STR_TO_WSTR("14. EnumRingerInit", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_ENUMRINGERINIT, szBuf, 0);	

    STR_TO_WSTR("15. EnumNextRinger", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_ENUMNEXTRINGER, szBuf, 0);	

    STR_TO_WSTR("16. GetRingerID", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETRINGERID, szBuf, 0);	

    STR_TO_WSTR("17. GetRingerInfo", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETRINGERINFO, szBuf, 0);	

    STR_TO_WSTR("18. Remove", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_REMOVE, szBuf, 0);	

   // Activate menu
	IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);
}



int PlayRingerFile(CIRingerApp *pMe, const char * filename)
{
				// Create an instance of IRingerMgr
	ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_RINGERMGR, (void **)(&pMe->m_pIRingerMgr));
	if (NULL == pMe->m_pIRingerMgr)
		{
		return 0;
		}
	IRINGERMGR_PlayFile(pMe->m_pIRingerMgr,filename, 2);
	return 1;
}


int CreateRinger(CIRingerApp *pMe, const char * ringername, const char * inputfile, AEESoundPlayerFile format)
{

   FileInfo fi;
   AECHAR rngName[15] ;
   if ( ringername )
	   {
		  STR_TO_WSTR(ringername, rngName, sizeof(rngName) );
	   }
   else
	   return 0;

	// Create IRingerMgr interface object. This step needs to be done only
	// if the IRingerMgr interface object have not already been created. 
	ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_RINGERMGR, (void **)&pMe->m_pIRingerMgr); 
	if(pMe->m_pIRingerMgr == NULL)
	{
		return 0;
	}

	// Create an instance of IFileMgr so we can access the  file
   ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR, (void **)(&pMe->m_pIFileMgr));
   if (NULL == pMe->m_pIFileMgr)
	{
		return 0;
	}

   if (NULL == (pMe->m_pIFile = IFILEMGR_OpenFile(pMe->m_pIFileMgr, inputfile, _OFM_READ)))
	{
		CleanInterfaces(pMe);
		return 0;
	}
// Determine the size of the bitmap file and allocate memory
	// to store its contents
	IFILE_GetInfo(pMe->m_pIFile, &fi);
	if (NULL == (pMe->m_pBuff = (byte *) MALLOC(fi.dwSize)))
	{
	CleanInterfaces(pMe);
		
		return 0;
	}

	// Read the contents of the ringer file into the
	// area allocated in memory for it
	if (IFILE_Read(pMe->m_pIFile, (void *)pMe->m_pBuff, fi.dwSize) == 0)
	{
		FREE(pMe->m_pBuff);
		CleanInterfaces(pMe);
		return 0;
	}

	ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MEMASTREAM, (void **)(&pMe->m_pMemStream));
	if (NULL == pMe->m_pMemStream)
	{
		FREE(pMe->m_pBuff);
		CleanInterfaces(pMe);
		return 0;
	}
//	IASTREAM_AddRef((IAStream*)pMe->m_pMemStream);
//IASTREAM_AddRef((IAStream*)pIFile);
//	pMe->m_pMemStream =pMemStream;
		
//	IFILE_Release(pIFile);
//	IFILEMGR_Release(pIFileMgr);
//	pMe->m_pIRingerMgr = pIRingerMgr;

	IMEMASTREAM_Set(pMe->m_pMemStream,pMe->m_pBuff, fi.dwSize, 0, 0);
    IRINGERMGR_Create(pMe->m_pIRingerMgr,(AECHAR*)rngName,  format, (IAStream*)pMe->m_pMemStream);
	CleanInterfaces(pMe);
	return 1;
}