/*===========================================================================

FILE: ishellsusage.c

SERVICES: Sample applet using AEE

DESCRIPTION
  This file contains usage examples of ISHELL interface. ISHELL interface
  functions allow the user to create/operate on IShell objects. IShell
  objects are used to invoke system level functionality.

PUBLIC CLASSES:  
   N/A


   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/

/*===============================================================================
                     INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "AEEModGen.h"
#include "AEEAppGen.h"
#include "AEEShell.h"
#include "AEEMenu.h"
#include "AEEStdLib.h"
#include "AEEUsageAppIDs.h"
#include "AEEGraphics.h"

// App resource header generated by BREW ResourceEditor.
#include "ishellusage_res.h"

/*===========================================================================

                      PUBLIC DATA DECLARATIONS

===========================================================================*/

/*-------------------------------------------------------------------
            Type Declarations
-------------------------------------------------------------------*/
// cfg class to demo use of ISHELL_SetPrefs and ISHELL_GetPrefs
typedef enum
{
   APP_STOPPED,
   APP_PAUSED,
   APP_PLAYING,
   APP_NEXTSCREEN

} STATE;

// The following structs set up the 
typedef struct _AppState
{
   STATE    nState;
   uint16   nScore;
   int16    nBalls;
   uint16   nScreens;

} AppState;

typedef struct _PrefsCfg 
{
   // Store the highest score from previous games
   uint16   highestScore;
   // The present games statistics
   AppState  st;

} PrefsCfg;

// Shell usage struct. This is the main struct for this applet. This will hold 
// all the data members that needs to be remembered throughout the life of
// the applet. 
// THE FIRST DATA MEMBER OF THIS STRUCT MUST BE AN AEEApplet OBJECT. 
// This is critical requirement which is related to an applet being
// dynamically loaded. The app developers should add their data members
// following the AEEApplet data member.
typedef struct _CIShellApp
{
	AEEApplet	a;        // Mandatory first AEEApplet data member
   IMenuCtl *  m_pIMenu;
   boolean     alarmActive;

   // Device parameters
   int             m_nLineHeight;
   AEEDeviceInfo   m_dInfo;

}CIShellApp;

/*-------------------------------------------------------------------
            Function Prototypes
-------------------------------------------------------------------*/
// Mandatory function:
// App Handle Event function
static boolean IShellApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);

// App specific data alloc-init/free functions
static boolean IShell_InitAppData(IApplet* pMe);
static void IShell_FreeAppData(IApplet* pMe);

static void BuildMainMenu(CIShellApp *pMe);
static void IShellUsage (CIShellApp * pMe, uint16 wParam);
static void DisplayEvent (CIShellApp * pMe, uint16 wParam);
static void DisplayOutput(IApplet * pMe, int nline, char *pszStr);
static void TimerCB (void *pi);
static void ResumeNotifyCB (void *pi);
static void LocationCB(void * pUser, AEEPositionInfo * pli, int nErr);

typedef enum _StartReason
{
	NORMAL,
	ALARM,
	SENDEVENT,
	POSTEVENT
}	StartReason;

// Shell support applet struct. This is the main struct for this applet. This will hold 
// all the data members that needs to be remembered throughout the life of the applet. 
// THE FIRST DATA MEMBER OF THIS STRUCT MUST BE AN AEEApplet OBJECT. 
// This is critical requirement which is related to an applet being
// dynamically loaded. The app developers should add their data members
// following the AEEApplet data member.
typedef struct _CIShellSupportApp
{
	AEEApplet	a;        // Mandatory first AEEApplet data member
	StartReason	r;
}CIShellSupportApp;

// Support app for Starting applet
// App Handle Event function
static boolean IShellSupportApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);
static boolean IShellSupport_InitAppData(IApplet* pMe);
static void IShellSupport_FreeAppData(IApplet* pMe) {}

/*-------------------------------------------------------------------
            Global Constant Definitions
-------------------------------------------------------------------*/
// App specific constants
#define USAGE_CREATEINSTANCE      101
#define USAGE_QUERYCLASS          102
#define USAGE_GETDEVICEINFO       103
#define USAGE_STARTAPPLET         104
#define USAGE_STARTAPPLETEX       105
#define USAGE_CLOSEAPPLET         106
#define USAGE_ACTIVEAPPLET        107
#define USAGE_ENUMAPPLETINIT      108
#define USAGE_ENUMNEXTAPPLET      109
#define USAGE_CANSTARTAPPLET      110
#define USAGE_SETTIMER            111
#define USAGE_CANCELTIMER         112
#define USAGE_GETTIMEREXPIRATION  113
#define USAGE_GETTIMEMS           114
#define USAGE_GETUPTIMEMS         115
#define USAGE_GETSECONDS          116
#define USAGE_GETJULIANDATE       117
#define USAGE_LOADRESSTRING       118
#define USAGE_LOADRESDATA         119
#define USAGE_LOADRESOBJECT       120
#define USAGE_FREERESDATA         121
#define USAGE_BEEP                122
#define USAGE_GETPREFS            123
#define USAGE_SETPREFS            124
#define USAGE_PROMPT              125
#define USAGE_MESSAGEBOXTEXT      126
#define USAGE_MESSAGEBOX          127
#define USAGE_SETALARM            128
#define USAGE_CANCELALARM         129
#define USAGE_ALARMSACTIVE        130
#define USAGE_GETHANDLER          131
#define USAGE_REGISTERHANDLER     132
#define USAGE_REGISTERNOTIFY      133
#define USAGE_NOTIFY              134
#define USAGE_RESUME              135
#define USAGE_FORCEEXIT           136
#define USAGE_GETRAND             137
#define USAGE_CREATEDIALOG        138
#define USAGE_GETACTIVEDIALOG     139
#define USAGE_ENDDIALOG           140
#define USAGE_SENDEVENT           141
#define USAGE_POSTEVENT           142
#define USAGE_HANDLEEVENT         143
#define USAGE_GETPOSITION         144
#define USAGE_BUSY                145
#define USAGE_LOADRESIMAGE        146
#define USAGE_LOADRESSOUND        147
#define USAGE_ERASE_SCREEN        148   // Special case to erase screen

// These constants are used in the 16 and 32 bit data fileds of
// ISHELL_SendEvent and ISHELL_PostEvent functions.
#define SEND_EVENT_16BITDATA      0
#define POST_EVENT_16BITDATA      1
#define HANDLE_EVENT_16BITDATA    2
#define COMMON_32BIT_EVENT_DATA   0


// Shell App config version
#define PREFS_CFG_VERSION         1

// App Resource File
#define  APP_RES_FILE             "ishellusage.bar"


/*===============================================================================
                     FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to IShell_InitAppData(). This is done as follows:
   IShell_InitAppData() is called to initialize AppletData instance. It is app developers 
   responsibility to fill-in app data initialization code of IShell_InitAppData(). App developer
   is also responsible to release memory allocated for data contained in AppletData -- this can be
   done in IShell_FreeAppData().

PROTOTYPE:
   int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell interface. 

	pIModule: pin]: Contains pointer to the IModule interface to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid IApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the app needs to be loaded and if AEEApplet_New() invocation was
     successful
  EFAILED: If the app does not need to be loaded or if errors occurred in 
     AEEApplet_New(). If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
{
   *ppObj = NULL;
		
   // We want to load this App. So, invoke AEEApplet_New() with the address of 
   // the app-specific handle event and the free app data function.
   if(ClsId == AEECLSID_SHELL_MAIN_APP){
      if(AEEApplet_New(sizeof(CIShellApp), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)IShellApp_HandleEvent,(PFNFREEAPPDATA)IShell_FreeAppData)
         == TRUE)
      {
         if (IShell_InitAppData((IApplet*)*ppObj) == TRUE)
            return(AEE_SUCCESS);
      }
   }
   if(ClsId == AEECLSID_SHELL_SUPPORT_APP){
      if(AEEApplet_New(sizeof(CIShellSupportApp), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)IShellSupportApp_HandleEvent,(PFNFREEAPPDATA)IShellSupport_FreeAppData)
         == TRUE)
      {
			if (IShellSupport_InitAppData((IApplet*)*ppObj) == TRUE)
				return(AEE_SUCCESS);
      }
   }
	return (EFAILED);
}

/*===========================================================================

FUNCTION IShellApp_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean IShellApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

	ecode: Specifies the Event sent to this applet

   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/
static boolean IShellApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
	CIShellApp * pMe = (CIShellApp*)pi;

	switch (eCode) 
	{
		case EVT_APP_START:
			// Create the IMenu interface object. When the applet is started the 
			// main menu is set up. When the user selects a menu item from the main 
			// menu, an EVT_COMMAND event is received by this applet. The processing for 
			// EVT_COMMAND event is found below.
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, (void **)&pMe->m_pIMenu)
            != SUCCESS)
			{
				return FALSE;
			}

			// Build main menu.
			BuildMainMenu(pMe);
			return(TRUE);

		case EVT_APP_STOP:
			return(TRUE);

		case EVT_APP_SUSPEND:
			// We have reached here when we started the shell-support applet.
			// At this point this applet is suspended and the support applet 
			// is started.
			// Clean any app-specific data before suspending (if needed).
			return TRUE;

		case EVT_APP_RESUME:
            if (pMe->m_pIMenu)
                IMENUCTL_Redraw(pMe->m_pIMenu);
            return TRUE;

		case EVT_DIALOG_INIT:
			 // This event is sent when this applet creates a dialog. We need to 
			 // return true here for the Dialog to be created (indicating this applet
			 // processed the dialog init event).
			 return TRUE;

		case EVT_ALARM:
			 // When an alarm is activated this applet receives this event. We need
			 // to return true here to indicate that this applet did process the Alarm
			 // event.
			return TRUE;

		case EVT_KEY:
			// This message is received here when a key is pressed on the handset keypad. 
         
			// Check is an alarm is active and deactivate it at this time. 
			// An alarm can be active if the user has pressed the activate alarm 
			// option in the main applet menu and does not wait till it is activated 
			// and move to another function. If this happens an alarm will still
			// be outstanding and will get activated when the apporpriate time comes.
			// So we should deactivate the alarm if the user moves on to another
			// function.
			if (pMe->alarmActive == TRUE)
			{
				uint16 nUserCode = 0;   // User code which distinguished this alarm
										// from another.

				// Since there is only one alarm that is activated in this example we
				// know the alarm information to deactivate it.
				ISHELL_CancelAlarm (pMe->a.m_pIShell, AEECLSID_SHELL_SUPPORT_APP, nUserCode);
				pMe->alarmActive = FALSE;
			}

			if (pMe->m_pIMenu)
			{
				if ((IMENUCTL_IsActive(pMe->m_pIMenu) == FALSE) && ((wParam == AVK_UP) || (wParam == AVK_DOWN)))
				{
					IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);
					IMENUCTL_Redraw(pMe->m_pIMenu);
				}
			   return IMENUCTL_HandleEvent(pMe->m_pIMenu, EVT_KEY, wParam, 0);
			}
			else 
				return FALSE;

		case EVT_COMMAND:
         // When a menu item is selected this applet receives the
         // EVT_COMMAND event in the wParam field.
  		   switch(wParam)
         {
            case USAGE_CREATEINSTANCE:
            case USAGE_QUERYCLASS:
            case USAGE_GETDEVICEINFO:
            case USAGE_STARTAPPLET:
            case USAGE_STARTAPPLETEX:
            case USAGE_CLOSEAPPLET:
            case USAGE_ACTIVEAPPLET:
            case USAGE_ENUMAPPLETINIT:
            case USAGE_ENUMNEXTAPPLET:
            case USAGE_CANSTARTAPPLET:
            case USAGE_SETTIMER:
            case USAGE_CANCELTIMER:
            case USAGE_GETTIMEREXPIRATION:
            case USAGE_GETTIMEMS:
            case USAGE_GETUPTIMEMS:
            case USAGE_GETSECONDS:
            case USAGE_GETJULIANDATE:
            case USAGE_LOADRESSTRING:
            case USAGE_LOADRESDATA:
            case USAGE_LOADRESOBJECT:
            case USAGE_FREERESDATA:
            case USAGE_BEEP:
            case USAGE_GETPREFS:
            case USAGE_SETPREFS:
            case USAGE_PROMPT:
            case USAGE_MESSAGEBOXTEXT:
            case USAGE_MESSAGEBOX:
            case USAGE_SETALARM:
            case USAGE_CANCELALARM:
            case USAGE_ALARMSACTIVE:
            case USAGE_GETHANDLER:
            case USAGE_REGISTERHANDLER:
            case USAGE_REGISTERNOTIFY:
            case USAGE_NOTIFY:
            case USAGE_RESUME:
            case USAGE_FORCEEXIT:
            case USAGE_GETRAND:
            case USAGE_CREATEDIALOG:
            case USAGE_GETACTIVEDIALOG:
            case USAGE_ENDDIALOG:
            case USAGE_SENDEVENT:
            case USAGE_POSTEVENT:
            case USAGE_HANDLEEVENT:
            case USAGE_GETPOSITION:
            case USAGE_BUSY:
            case USAGE_LOADRESIMAGE:
            case USAGE_LOADRESSOUND:
               {
                  // Call the usage function that exercises all the 
                  // interface functions.
                  IShellUsage (pMe, wParam);
                  return TRUE;
               }
            // The following three cases occur if the user has created a 
            // prompt (see under USAGE_PROMPT in the IShellUsage function)
            // and pressed respective keys cooresponding to the IDs given below.
            // Or if the prompt timer runs out, in which case the default ID 
            // is sent (in the case of this example the default Id IDS_NO).
            case IDS_YES:
               // Do prompt processing for YES option.
               DisplayOutput ((IApplet *)pMe, -1, "YES - Option has been selected.... Press Up/Down Arrows to go back to menu...");
               return TRUE;
            case IDS_NO:
               // Do prompt processing for NO option
               DisplayOutput ((IApplet *)pMe, -1, "NO - Option has been selected.... Press Up/Down Arrows to go back to menu...");
               return TRUE;
            case IDS_PASS:
               // Do prompt processing for PASS option.
               DisplayOutput ((IApplet *)pMe, -1, "PASS - Option has been selected.... Press Up/Down Arrows to go back to menu...");
               return TRUE;
            case HANDLE_EVENT_16BITDATA:
               // We have reached here from the invoke of the ISHELL_HandleEvent() function
               // in the IShellUsage() function below. 
               DisplayOutput ((IApplet *)pMe, -1, "In current applet HandleEvent.... Press Up/Down Arrows to go back to menu...");
               return TRUE;
			   default:
				   return FALSE;
         }
      default:
         break;
   }
   return FALSE;
}

/*===========================================================================

FUNCTION IShellSupportApp_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean IShellSupportApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

	ecode: Specifies the Event sent to this applet

   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/
static boolean IShellSupportApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
   CIShellSupportApp * pMe = (CIShellSupportApp*)pi;

   switch (eCode) 
	{
      case EVT_APP_START:
			switch (pMe->r)
			{
				case ALARM:
					DisplayOutput(pi,-1,"Alarm: received in second applet. Press CLR key to return to previous app....");
					break;

				case SENDEVENT:
               DisplayOutput (pi, -1, "SendEvent: received in second applet. Press CLR key to return to previous app....");
					break;

				case POSTEVENT:
               DisplayOutput (pi, -1, "PostEvent: received in second applet. Press CLR key to return to previous app....");
					break;

				default:
					DisplayOutput(pi,-1,"Inside new app. Press CLR key to return to previous app....");
					break;
			}
			return(TRUE);

      case EVT_APP_STOP:
         return TRUE;

      case EVT_KEY_PRESS:
      case EVT_KEY:
         // We need to return FALSE when a key is pressed, this will cause this
         // applet to be closed and control to be passed to the other applet.
         return FALSE;

		case EVT_ALARM:
			pMe->r = ALARM;
			ISHELL_StartApplet(pMe->a.m_pIShell,AEECLSID_SHELL_SUPPORT_APP);
         return TRUE;

      case EVT_USER:
         {
            // This applet has received a user-specified event.
            // Check the 16-bit data field to see which event is being received.
            // In this example the 32-bit data field is not used. Only the 16-bit
            // data field is used. The 16-bit data field will have SEND_EVENT_16BITDATA
            // for the event sent by ISHELL_SendEvent function and POST_EVENT_16BITDATA
            // for the event sent by ISHELL_PostEvent function in the Main-shell applet.
            switch (wParam)
            {
               case SEND_EVENT_16BITDATA:
						pMe->r = SENDEVENT;
						ISHELL_StartApplet(pMe->a.m_pIShell,AEECLSID_SHELL_SUPPORT_APP);
                  return TRUE;

               case POST_EVENT_16BITDATA:
						pMe->r = POSTEVENT;
						ISHELL_StartApplet(pMe->a.m_pIShell,AEECLSID_SHELL_SUPPORT_APP);
                  return TRUE;
            }
         }
         break;
      default:
         break;
   }
   return FALSE;
}

/*===========================================================================

FUNCTION IShell_InitAppData

DESCRIPTION
   This function initializes app specific data, allocates memory for app data.
   App developer needs to fill-in the initialization code.

PROTOTYPE:
	boolean IShell_InitAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

DEPENDENCIES
   Assumes pMe is not NULL

RETURN VALUE
  TRUE: If the app has app data is allocated and initialized successfully
  FALSE: Either app data could not be allocated or initialzied

SIDE EFFECTS
  none
===========================================================================*/
static boolean IShell_InitAppData(IApplet* pi)
{
   int pnAscent;        // Stores the ascent in number of pixels
   int pnDescent;       // Stores the descent in number of pixels

   CIShellApp * pMe = (CIShellApp*)pi;

   // Initialize the MenuCtl pointer to NULL
   pMe->m_pIMenu = NULL;

   // Se the alarm active flag to false.
   pMe->alarmActive = FALSE;

   // Get the font metrics info
   pMe->m_nLineHeight = IDISPLAY_GetFontMetrics (pMe->a.m_pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&pMe->m_dInfo);

   return TRUE;
}

/*===========================================================================

FUNCTION IShell_FreeAppData

DESCRIPTION
   Frees data contained in app data and memory for app data itself.
   App developer needs to free data contained in AppletData.

PROTOTYPE:
	boolean IShell_FreeAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

DEPENDENCIES
   Assumes pMe is not NULL

RETURN VALUE
   None

SIDE EFFECTS
   None
===========================================================================*/
static void IShell_FreeAppData(IApplet* pi)
{
   CIShellApp * pMe = (CIShellApp*)pi;

   if (pMe->m_pIMenu != NULL)
   {
      IMENUCTL_Release (pMe->m_pIMenu);
      pMe->m_pIMenu = NULL;
   }

   // Check is an alarm is active and deactivate it at this time. 
   // An alarm can be active if the user has pressed the activate alarm 
   // option in the main applet menu and does not wait till it is activated 
   // and move to another function. If this happens an alarm will still
   // be outstanding and will get activated when the apporpriate time comes.
   // So we should deactivate the alarm if the user moves on to another
   // function.
   if ((pMe != NULL) && pMe->alarmActive == TRUE)
   {
      uint16 nUserCode = 0;   // User code which distinguished this alarm
                              // from another.

      // Since there is only one alarm that is activated in this example we
      // know the alarm information to deactivate it.
      ISHELL_CancelAlarm (pMe->a.m_pIShell, AEECLSID_SHELL_SUPPORT_APP, nUserCode);
      pMe->alarmActive = FALSE;
   }
}

/*===========================================================================

FUNCTION IShellSupport_InitAppData

DESCRIPTION
   This function initializes app specific data, allocates memory for app data.
   App developer needs to fill-in the initialization code.

PROTOTYPE:
	boolean IShellSupport_InitAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

DEPENDENCIES
   Assumes pMe is not NULL

RETURN VALUE
  TRUE: If the app has app data is allocated and initialized successfully
  FALSE: Either app data could not be allocated or initialzied

SIDE EFFECTS
  none
===========================================================================*/
static boolean IShellSupport_InitAppData(IApplet* pi)
{
   CIShellSupportApp * pMe = (CIShellSupportApp*)pi;

	pMe->r = NORMAL;

   return TRUE;
}

/*===============================================================================
                     APP HELPER FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: IShellUsage

DESCRIPTION
	This function encompasses all the usage examples of all the functions
   in code blocks switched using the BREW API function Id passed into this
   function. The code blocks will include almost all the variable declarations
   needed to demonstrate the usage of a given function. Within the code block
   all the needed interfaces are instantiated and released subsequent to their
   use.

   There are some exceptions to the above general rule, such as call-back functions,
   common helper function, or common display functions called from within the
   code block which are defined elsewhere in this file.

   There are some instances where an object instantiated within a code block is
   not released at the end of the code block, to be released at a later point
   (for example in a call-back function, in this case pointers to such objects
   are stored in the AEEApplet object to be released later).
   
PROTOTYPE:
   boolean IShellUsage (CIShellApp * pMe, uint16 wParam)

PARAMETERS:
	pMe: [in]: Contains a pointer to the CIShellApp struct.
   wParam: [in]: Contains the Id of the function type to be used (contains the
      Id corresponding to the menu item selected on the emulator).

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the functions call was successful. FALSE otherwise

SIDE EFFECTS
  none
===========================================================================*/
static void IShellUsage (CIShellApp * pMe, uint16 wParam)
{
   AEEDeviceInfo di;        // Device Info
   AEERect rct;      
   int cx, cy;              // Screen width and height

   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
      return;

   // Get screen size information for display purposes.
   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);

   // Set rectangle coordinates for a rectangle in the middle of the screen.
   // The rectangle will have differenty coordinates depending on different screen 
   // dimensions.
   rct.x = 10;
   rct.y = di.cyScreen /3;
   rct.dx = di.cxScreen - 5;
   rct.dy = di.cyScreen/3;

   cx = di.cxScreen;        // Screen width
   cy = di.cyScreen;        // Screen height

   // Erase screen first for display output purposes.
   IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
	IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
	IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);

   switch (wParam)
   {
      case USAGE_CREATEINSTANCE:
         {
            // 2. Usage IShell_CreateInstance.
            IDisplay *pIDisplay = NULL;

            // ISHELL_CreateInstance function is used to create instances of
            // interfaces supported by the BREW. Each interface that is created
            // is identified by the AEECLSLD value associated with that interface.

            // The following example shows the creation of the IGraphics interface.
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_DISPLAY, (void **)&pIDisplay);

            if (pIDisplay)
            {
               DisplayOutput ((IApplet *)pMe, -1, "CreateInstance: Success");

               // Release IGraphics Interface. This step needs to be done
               // only if no further use of the IGraphics interface needed.
               IDISPLAY_Release (pIDisplay);
            }
            else
            {
               DisplayOutput ((IApplet *)pMe, -1, "CreateInstance: Failed");
            }
         }
         break;
      case USAGE_QUERYCLASS:
         {
            // 3. Usage ISHELL_QueryClass.
            AEEAppInfo appInfo = {0};
            char buffer[40] = "Query Class MIF: ";

            // This method queries the AEE to determine if the specified class
            // or applet is supported.
            // Invoke ISHELL_QueryClass. Here AEECLSID_DATECTL_APP corresponds 
            // to the DateCtl usage applet.
            if (ISHELL_QueryClass(pMe->a.m_pIShell, AEECLSID_DATECTL_APP, &appInfo))
            {
               // "appInfo" contains data related to the Applet that is
               // queried. In this example the DateCtl usage app.
               // Print the MIF file name retrieved from appInfo. See 
               // AEEAppInfo struct definition for more information.
               STRCAT (buffer, appInfo.pszMIF);
               DisplayOutput ((IApplet *)pMe, -1, buffer);
            }
            else
            {
               DisplayOutput ((IApplet *)pMe, -1, "Query Class: Not supported");
            }
         }
         break;
      case USAGE_GETDEVICEINFO:
         {
            // 4. Usage ISHELL_GetDeviceInfo.
            AEEDeviceInfo di;                 // Device Info
            char buff [30] = {0};

            // ISHELL_GetDeviceInfo returns information on the
            // device that this applet is running on.
            ISHELL_GetDeviceInfo (pMe->a.m_pIShell, &di);

            // The following is for display purposes only
            // Display some of the device information. See 
            // AEEDeviceInfo definition for more information
            // on other device related parameters contained 
            // in AEEDeviceInfo.
            SPRINTF (buff, "Screen width: %u", di.cxScreen);
            DisplayOutput ((IApplet *)pMe, 2, buff);
            SPRINTF (buff, "Screen Height: %u", di.cyScreen);
            DisplayOutput ((IApplet *)pMe, 4, buff);
         }
         break;
      case USAGE_STARTAPPLET:
      case USAGE_STARTAPPLETEX:
         {
            // 5. Usage ISHELL_StartApplet.
            int nRetValue = 0;

            // ISHELL_StartApplet: instructs the AEE to start the applet associated
            // with the specified 32-bit class ID. If the requested class is 
            // supported and can be started, the AEE loads and starts the 
            // applet. Note that this call returns immediately to the caller 
            // before starting the applet. The applet is started asynchronously. 
            // Hence, if the applet class ID being specified is not found, this 
            // function will still return TRUE but the applet will not be started.
            // 
            // When the applet is started or when it is resumed the display is 
            // cleared and the HandleEvent is sent the EVT_APP_START or 
            // EVT_APP_RESUME along with the AEEAppStart parameter block.

            // Start the Support applet. This will in effect suspend this 
            // applet (main applet) and start the support applet. When 
            // the the support applet exits, this applet will resume.
            if ((nRetValue = ISHELL_StartApplet (pMe->a.m_pIShell, AEECLSID_SHELL_SUPPORT_APP))
               != SUCCESS)
            {
               DisplayOutput ((IApplet *)pMe, -1, "StartApplet: Failed");
            }
         }
         break;
      case USAGE_CLOSEAPPLET:
         {
            // 6. Usage ISHELL_CloseApplet.
            boolean bReturnToIdle = FALSE;

            // The following string will only display for a moment before the
            // applet is closed and the main applet menu is displayed.
            DisplayOutput ((IApplet *)pMe, -1, "Closing Active Applet");

            // ISHELL_CloseApplet: instructs the shell to close the active applet.
            // The applet will be sent the EVT_APP_STOP message. It's Release method
            // will then be called.
            //
            // The bReturnToIdle parameter indicates whether the AEE should close
            // all other active applications and return the idle screen.  
            // This parameter is not supported on the BREW emulator.

            // This will close the active applet and return to the main menu.
            ISHELL_CloseApplet (pMe->a.m_pIShell, bReturnToIdle);
         }
         break;
      case USAGE_ACTIVEAPPLET:
         {
            // 7. Usage ISHELL_ActiveApplet.
            AEECLSID activeAppletId = 0;
            char szBuf[30] = {0};

            // ISHELL_ActiveApplet: returns the AEECLSID associated with the
            // currently running applet.
            activeAppletId = ISHELL_ActiveApplet (pMe->a.m_pIShell);

            // Display the current active applet Id in decimal.
            SPRINTF (szBuf, "Active Applet Id (Dec): %u", activeAppletId);
            DisplayOutput ((IApplet *)pMe, 2, szBuf);
         }
         break;
      case USAGE_ENUMAPPLETINIT:
      case USAGE_ENUMNEXTAPPLET:
         {
            // 7. Usage ISHELL_EnumAppletInit/ISHELL_EnumAppletNext
            AEEAppInfo ai;
            char buff[150] = {0};
            AEECLSID clsId;

            // ISHELL_EnumAppletInit: resets the shell's internal applet 
            // enumeration index.
            ISHELL_EnumAppletInit(pMe->a.m_pIShell);

            // Eumerate and display the MIF file names for all the applets 
            // on the screen seperated by a comma.
            STRCPY (buff, "Applet clsId/MIFs: ");

            // ISHELL_EnumNextApplet: returns information regarding the next applet.
            while((clsId = ISHELL_EnumNextApplet(pMe->a.m_pIShell, &ai)) != 0)
            {
               // Append the MIF file name to the string
               STRCAT (buff, ai.pszMIF);
               STRCAT (buff, ", ");   // put some space between the MIF file names
            }

            DisplayOutput ((IApplet *)pMe, 1, buff);
         }
         break;
      case USAGE_CANSTARTAPPLET:
         {
            // 8. Usage ISHELL_CanStartApplet
            boolean canStart = FALSE;

            // ISHELL_CanStartApplet: queries the shell to determine whether it is
            // safe to start an applet.

            // Invoke CanStartApplet to check if the Support applet can be started.
            if ((canStart = ISHELL_CanStartApplet (pMe->a.m_pIShell, 
               AEECLSID_SHELL_SUPPORT_APP)) == TRUE)
            {
               DisplayOutput ((IApplet *)pMe, -1, "Applet can be started..");
            }
            else
            {
               DisplayOutput ((IApplet *)pMe, -1, "Applet can not be started...");
            }
         }
         break;
      case USAGE_SETTIMER:
         {
            // 9. Usage ISHELL_SetTimer
            int retVal = EFAILED;
            int32 timeMs = 2000;       // Activate timer after this time

            // ISHELL_SetTimer: allows the caller to set a short-term timer. Upon
            // expiration, the specified callback function is called, passing it the 
            // specified user data pointer as it's first argument.
            //
            // Set timer so that when activated TimerCB function is called.
            // In this function just print the fact that the control has reached
            // the function and return.
            if ((retVal = ISHELL_SetTimer(pMe->a.m_pIShell, timeMs, TimerCB, (void *)pMe))
               == SUCCESS)
            {
               DisplayOutput ((IApplet *)pMe, -1, "Waiting for timer activation...");
            }
            else
            {
               DisplayOutput ((IApplet *)pMe, -1, "SetTimer: Failed !");
            }
         }
         break;
      case USAGE_CANCELTIMER:
         {
            // 9. Usage ISHELL_SetTimer
            int retVal = EFAILED;
            int32 timeMs = 2000;       // Activate timer after this time

            // Start timer first.
            if ((retVal = ISHELL_SetTimer(pMe->a.m_pIShell, timeMs, TimerCB, (void *)pMe))
               == SUCCESS)
            {
               // ISHELL_CancelTimer: cancels timers that have been set by ISHELL_SetTimer.

               // Cancel timer previously set.
               if ((retVal = ISHELL_CancelTimer (pMe->a.m_pIShell, TimerCB, (void *)pMe))
                  == SUCCESS)
               {
                  // Timer was canceled successfully.
                  DisplayOutput ((IApplet *)pMe, -1, "Timer canceled successfully..");
               }
               else
               {
                  // Timer cancel failed.
                  DisplayOutput ((IApplet *)pMe, -1, "CancelTimer: Failed");
               }
            }
            else
            {
               // Set timer failed.
               DisplayOutput ((IApplet *)pMe, -1, "SetTimer: Failed !");
            }
         }
         break;
      case USAGE_GETTIMEREXPIRATION:
         {
            // 10. Usage ISHELL_GetTimerExpiration
            int retVal = EFAILED;
            int32 timeMs = 2000;       // Activate timer after this time (in MS)
            uint32 remainingTime = 0;
            char buffer [50] = {0};

            // Start timer.
            if ((retVal = ISHELL_SetTimer(pMe->a.m_pIShell, timeMs, TimerCB, (void *)pMe))
               == SUCCESS)
            {
               // ISHELL_GetTimerExpiration: returns the remaining time before the timer 
               // associated with the user callback/data expires.
 
               // Get the remaining time before timer expiration.
               remainingTime = ISHELL_GetTimerExpiration (pMe->a.m_pIShell, 
                  TimerCB, (void *)pMe);

               // Display the time left before timer expiration.
               SPRINTF (buffer, "Expiration Time (MS): %u", remainingTime);
               DisplayOutput ((IApplet *)pMe, -1, buffer);

               // Now cancel the timer.
               ISHELL_CancelTimer (pMe->a.m_pIShell, TimerCB, (void *)pMe);
            }
            else
            {
               // Set timer failed.
               DisplayOutput ((IApplet *)pMe, -1, "SetTimer: Failed !");
            }
         }
         break;
      case USAGE_GETTIMEMS:
         {
            // 11. Usage ISHELL_GetTimeMS
            uint32 timeMs = 0;    // Stores the 32-bit millisecond value since midnight.
            char buffer [30] = {0};

            // ISHELL_GetTimeMS: returns the current time of day in milliseconds.

            // Get the current time of day in milliseconds.
            timeMs = ISHELL_GetTimeMS (pMe->a.m_pIShell);

            // Display result.
            SPRINTF (buffer, "Time of day (MS): %u", timeMs);
            DisplayOutput ((IApplet *)pMe, -1, buffer);
         }
         break;
      case USAGE_GETUPTIMEMS:
         {
            // 12. Usage ISHELL_GetUpTimeMS
            uint32 upTimeMs = 0;  // Stores the 32-bit millisecond value from the 
                                  // time the device was powered.
            char buffer [40] = {0};

            // ISHELL_GetUpTimeMS: returns the millisecond timer from the time the device 
            // was powered on.

            // Get the millisecond time from the time the device was powered on.
            upTimeMs = ISHELL_GetUpTimeMS(pMe->a.m_pIShell);

            // Display result
            SPRINTF (buffer, "Device UpTime (MS): %u", upTimeMs);
            DisplayOutput ((IApplet *)pMe, -1, buffer);
         }
         break;
      case USAGE_GETSECONDS:
        {
            // 13. Usage ISHELL_GetSeconds
            uint32 timeSeconds = 0;  // Stores number of seconds since 
            char buffer [50] = {0};

            // ISHELL_GetSeconds: returns the number of seconds, adjusted for 
            // local time, since phone reference time of Jan 6, 1980 00:00:00.

            // Get the number of seconds since phone reference time
            timeSeconds = ISHELL_GetSeconds(pMe->a.m_pIShell);

            // Display result
            SPRINTF (buffer, "Time since phone reference time (Sec): %u", timeSeconds);
            DisplayOutput ((IApplet *)pMe, -1, buffer);
         }
         break;
      case USAGE_GETJULIANDATE:
         {
            // 14. Usage ISHELL_GetJulianDate
            uint32 dwSeconds;      // Stores Seconds since Jan 6 1980.
            JulianType julianDate;
            char buffer [40] = {0};

            // If dwSeconds parameter is set to zero it uses the system time 
            // to fill the julian date. If dwSeconds value is not zero, this value
            // is used as an offset in second from the Jan 6th 1980 to derive the
            // Julian Date information. In this example we will use the system time.
            dwSeconds = 0;              // Use system time in this example.

            // ISHELL_GetJulianDate: fills a julian date structure based upon the 
            // input second value.

            // Get the number of seconds since midnight
            ISHELL_GetJulianDate(pMe->a.m_pIShell, dwSeconds, &julianDate);

            // Display results on line 1 through 7
            SPRINTF (buffer, "Year: %u", julianDate.wYear);
            DisplayOutput ((IApplet *)pMe, 1, buffer);
            SPRINTF (buffer, "Month: %u", julianDate.wMonth);
            DisplayOutput ((IApplet *)pMe, 2, buffer);
            SPRINTF (buffer, "Day: %u", julianDate.wDay);
            DisplayOutput ((IApplet *)pMe, 3, buffer);
            SPRINTF (buffer, "Hour: %u", julianDate.wHour);
            DisplayOutput ((IApplet *)pMe, 4, buffer);
            SPRINTF (buffer, "Minute: %u", julianDate.wMinute);
            DisplayOutput ((IApplet *)pMe, 5, buffer);
            SPRINTF (buffer, "Second: %u", julianDate.wSecond);
            DisplayOutput ((IApplet *)pMe, 6, buffer);
            SPRINTF (buffer, "Day of week: %u", julianDate.wWeekDay);
            DisplayOutput ((IApplet *)pMe, 7, buffer);
         }
         break;
      case USAGE_LOADRESSTRING:
         {
            // 15. Usage ISHELL_LoadResString
            char szResFile[] = APP_RES_FILE;   // Resource File used by this applet
            int nCharCount = 0;
            AECHAR pszText[50] = {0};
            char pszStr [50] = {0};

            // ISHELL_LoadResString: this method allows the caller to retrieve UNICODE 
            // strings stored in the specified resource file

            // Load string from resource file. IDS_STRING1 is defined in ishellusage_res.h
            // file. This file was generated using the BREW resource editor.
            nCharCount = ISHELL_LoadResString (pMe->a.m_pIShell, szResFile, IDS_STRING1,
               pszText, sizeof (pszText));

            // Convert the wide-string to char. This is done only to use a single
            // display function (DisplayOutput).
            WSTR_TO_STR (pszText, pszStr, sizeof (pszStr));
            DisplayOutput ((IApplet *)pMe, -1, pszStr);
         }
         break;
      case USAGE_LOADRESDATA:
      case USAGE_FREERESDATA:
         {
            // ISHELL_LoadResData returns int instead of void *
            // 16. Usage ISHELL_LoadResData
            char szResFile[] = APP_RES_FILE;   // Resource File used by this applet
            char pszStr [50] = {0};
            void * resData = NULL;
            // ISHELL_LoadResData: returns a void * data block associated with the
            // specified resource file, ID and type. The memory returned must be freed 
            // using the ISHELL_FreeResData call.

            AEEBmp pbmSource;
            int xDest = 30;
            int yDest = 40;
            int cxDest = di.cxScreen;
            int cyDest = di.cyScreen;
            int xSrc = 0;
            int ySrc = 0;
            AEERasterOp dwRopCode = AEE_RO_MASK;
            AEEImageInfo imageInfo;
            void *pBmp = NULL, *pDataBytes = NULL;
            boolean bVal = TRUE;

            // Load bitmap resource from the resource file.
            pbmSource = ISHELL_LoadResData (pMe->a.m_pIShell, APP_RES_FILE, 
                                     IDB_BITMAP1, RESTYPE_IMAGE); // please do not use RESTYPE_BINARY

			//There is a header that needs to be removed before passing the data to convertbmp
			//the first byte specifies the size of the header and hence offset  we need to skip.
            pDataBytes = (byte *)pbmSource + *((byte *)pbmSource);


            pBmp = CONVERTBMP (pDataBytes, &imageInfo, &bVal);

            // IDISPLAY_BitBlt: performs a bit-block transfer of the data 
            // corresponding to a rectangle of pixels from the specified source 
            // bitmap into the given Display area. After calling this function, 
            // IDISPLAY_Update() must be called to update the screen. If there is 
            // a sequence of drawing operations being performed, it is sufficient 
            // to IDISPLAY_Update() once after all the drawing is done.
            IDISPLAY_BitBlt (pMe->a.m_pIDisplay, xDest, yDest, cxDest, cyDest, pBmp,
               xSrc, ySrc, dwRopCode);

            IDISPLAY_Update (pMe->a.m_pIDisplay);

            ISHELL_FreeResData (pMe->a.m_pIShell, pbmSource);
			if(bVal)//free only if realloc was done
				SYSFREE ( pBmp);
            
         }
         break;
      case USAGE_LOADRESOBJECT:
         {
            // 17. Usage ISHELL_LoadResObject
            IImage *pIImage = NULL;
            AEEHandlerType hType = HTYPE_VIEWER;  // Resource handler type (image)
            char szResFile[] = APP_RES_FILE;      // Resource File used by this applet

            // ISHELL_LoadResObject: loads the specified resource and creates 
            // a handler that can be used on the resource Data. The type of handler
            // created is specified by the input parameter hType.

            // Load resource object of IImage type.
            pIImage = (IImage *)ISHELL_LoadResObject (pMe->a.m_pIShell, szResFile, IDB_BITMAP1, hType);

            // Display the loaded resource object on the screen.
            IIMAGE_Draw(pIImage, cx/2,cy/2);

            // Release the image object
            IIMAGE_Release (pIImage);
         }
         break;
      case USAGE_BEEP:
         {
            // 18. Usage ISHELL_Beep
            BeepType nBeep = BEEP_MSG;   // Beep type to play
            boolean bLoud = TRUE;        // Play loud beep
            boolean bRetVal = FALSE;     

            // ISHELL_Beep: provides a very simple interface to play 
            // system beeps and/or vibrate the handset

            // Play a loud beep
            if ((bRetVal = ISHELL_Beep (pMe->a.m_pIShell, nBeep, bLoud)) == TRUE)
            {
               DisplayOutput ((IApplet *)pMe, -1, "Beep was played");
            }
            else
            {
               DisplayOutput ((IApplet *)pMe, -1, "No Beep Played");
            }
         }
         break;
      case USAGE_GETPREFS:
         {
            // 19. Usage ISHELL_GetPrefs
            PrefsCfg cfg      = {0};
            int iRetValue = 0;
            char szBuf [50] = {0};

            // ISHELL_GetPrefs: provides a means of retrieving a structure
            // containing applet or class level preferences.
            if ((iRetValue = ISHELL_GetPrefs(pMe->a.m_pIShell, AEECLSID_SHELL_MAIN_APP,
                PREFS_CFG_VERSION, &cfg, sizeof(cfg))) == SUCCESS)
            {
               // Display some of the data retrieved from the preference 
               // configuration file.

               // Display the heighest score.
               SPRINTF (szBuf, "Highest Score: %u", cfg.highestScore);
               DisplayOutput ((IApplet *)pMe, 2, szBuf);

               // Display the present score
               SPRINTF (szBuf, "Present Score: %u", cfg.st.nScore);
               DisplayOutput ((IApplet *)pMe, 3, szBuf);
               
               // Display the state of the game
               if (cfg.st.nState == APP_STOPPED)
               {
                  DisplayOutput ((IApplet *)pMe, 4, "Presnt state: Stopped");
               }

               // Display the balls left
               SPRINTF (szBuf, "Balls left: %u", cfg.st.nBalls);
               DisplayOutput ((IApplet *)pMe, 5, szBuf);

               // Display the screens left
               SPRINTF (szBuf, "Screens left: %u", cfg.st.nScreens);
               DisplayOutput ((IApplet *)pMe, 6, szBuf);
            }
            else
            {
               // In this case there is no preference configuration file 
               // found for this applet. The user should run the SetPrefs 
               // option which will cause the shell to create a preference 
               // file and store some data in it. When the GetPrefs function
               // is run again these values will be retrieved and displayed 
               // on the screen.
               DisplayOutput ((IApplet *)pMe, 2, "No Prefs set...");
               DisplayOutput ((IApplet *)pMe, 3, "Please run SetPrefs and run GetPrefs option again");
            }
         }
         break;
      case USAGE_SETPREFS:
         {
            // 20. Usage ISHELL_SetPrefs
            PrefsCfg gCfg      = {0};
            int iRetValue = 0;

            // Data fill gCfg parameter so that it can be used to
            // set preferences of an applet.
            gCfg.highestScore  = 998;
            gCfg.st.nState   = APP_STOPPED;
            gCfg.st.nScore   = 600;
            gCfg.st.nBalls   = 1;
            gCfg.st.nScreens = 1;

            // ISHELL_SetPrefs: provides a means of storing a structure
            // containing applet or class level preferences.
            //
            // This function provides a means for an applet to store its own
            // preferences through the help of the Shell. The shell when 
            // invoked creates a database file with the preferences and 
            // corrolate it with the applet Id that is being passed in. 
            // This allows the applet to retrieve the preferences file at
            // a later time by using the ISHELL_GetPrefs function.
            //
            // An example of the use of preferences file would be a game applet,
            // where if the game is exited before finishing, the applet can store 
            // the state/scores of the game using the ISHELL_SetPrefs function and 
            // when the user wants to play the game again it can restore the previous 
            // state/scores of the game. Another example is when a user stores his/her 
            // preferences of a particular application such as a browser, email tool, 
            // editor, etc., they can be loaded each time the user uses the application.
            if ((iRetValue = ISHELL_SetPrefs(pMe->a.m_pIShell, AEECLSID_SHELL_MAIN_APP,
               PREFS_CFG_VERSION, &gCfg, sizeof(gCfg))) == SUCCESS)
            {
               DisplayOutput ((IApplet *)pMe, -1, "Prefs successfully set");
            }
         }
         break;
      case USAGE_PROMPT:
         {
            // 21. Usage ISHELL_Prompt
            boolean retVal = FALSE;
            AEEPromptInfo pi;
            // Button Id array filled from value in the resource file.
            // The buttons for the prompt will have the strings corresponding to 
            // the IDs given below.
            uint16 pBtnIDs[] = {IDS_YES, IDS_NO, IDS_SKIP};
            AECHAR szBuf1[20] = {0};
            AECHAR szBuf2[20] = {0};

            // Initialize the local buffers with text.
            STR_TO_WSTR("Prompt Title", szBuf1, sizeof(szBuf1));
            STR_TO_WSTR("Prompt Text.....", szBuf2, sizeof(szBuf2));

            pi.pszRes = APP_RES_FILE;     // Resource file name 
            pi.pTitle = szBuf1;           // Title text buffer
            // When the "pText" is NULL, the prompt butttons are placed in 
            // a default top-down menu. If "pText" is not NULL then the text
            // string is displayed along with the Title and the button keys are 
            // placed in a Soft-Key menu at the bottom of the screen.
            pi.pText = szBuf2;            // Text buffer

            // Title resource ID. This will not get used because "pTitle" is not NULL.
            pi.wTitleID =IDS_PROMPT_TITLE;

            // Text resource ID. This resource is not use because "pText" is not NULL.
            pi.wTextID = IDS_PROMPT_TEXT; // Text resource ID (not used)

            // Command to send when CLR is hit (default is first entry in the button Id array)
            pi.wDefBtn = IDS_NO;          // Command to send when CLR is hit (IDS_NO)
            pi.pBtnIDs = pBtnIDs;         // Button Id array 

            // The text and title are presented using a static text control (instance
            // of the IStatic interface). You can specify the control's properties here
            // (we choose to center the static text control's title and vertically center
            // the prompt text).
            pi.dwProps = ST_MIDDLETEXT | ST_CENTERTITLE;
            pi.fntTitle = AEE_FONT_BOLD;  // The font the Title will appear in 
            pi.fntText = AEE_FONT_NORMAL; // The font the text will appear in
            pi.dwTimeout = 0;          // Time in MS, for how long the user will be prompted.

            // ISHELL_Prompt: provides a mechanism for an application to
            // display a multi-selection prompt.  The text can be specified from a
            // resource file or directly passed in the AEEPromptInfo structure.  The
            // shell first examines the text pointers before attempting to load
            // the text from the resource file.

            // The buttons are specified by the values in the AEEPromptInfo->pBtnIDs
            // list.  For convenience, it is assumed that the IDs for the buttons 
            // are associated with both the text and command ID for the button.
            // Note that the prompt dialog will be automatically dismissed when any
            // of the button selections is made by the user or when the CLR key is 
            // pressed.  In this case, the EVT_COMMAND message is sent to the
            // application with the 16-bit extra parameter indicating the ID of 
            // the selection.

            // Invoke prompt using directly passed text.
            retVal = ISHELL_Prompt(pMe->a.m_pIShell, &pi);

            // We should return here so that the text from the Shell prompt
            // will not be over-written by the DisplayEvent function call at the
            // end of this function.
            return;
         }
      case USAGE_MESSAGEBOXTEXT:
         {
            // 22. Usage ISHELL_MessageBoxText
            boolean bRetVal = FALSE;
            AECHAR szTitleBuff[10] = {0};
            AECHAR szTextBuff[100] = {0};

            STR_TO_WSTR("Title", szTitleBuff, sizeof(szTitleBuff));
            STR_TO_WSTR("Text of the message box.. Press Up/Down keys to get back to IShell usage menu...",
               szTextBuff, sizeof(szTextBuff));

            // ISHELL_MessageBoxText: instructs the shell to display a message box
            // to the user.  A message box is a simple window with a title and text. 
            // The message box is dismissed via the END or CLR keys.

            // Display simple message box with locally defined strings
            if ((bRetVal = ISHELL_MessageBoxText(pMe->a.m_pIShell, szTitleBuff, szTextBuff))
               == TRUE)
            {
               // We should return here so that the text from the message box
               // will not be over-written by the DisplayEvent function call at the
               // end of this function.
               return;
            }
            else
            {
               // The dialog could not be created
               DisplayOutput ((IApplet *)pMe, -1, "MessageBox display: Failed.");
            }
         }
         break;
      case USAGE_MESSAGEBOX:
         {
            // 23. Usage ISHELL_MessageBox
            boolean bRetVal = FALSE;
            char szResFile[] = APP_RES_FILE;   // Resource File used by this applet
            
            // ISHELL_MessageBox: instructs the shell to display a message box
            // to the user.  A message box is a simple window with a title and text.

            // Display simple message box with resource from resource file.
            if ((bRetVal = ISHELL_MessageBox(pMe->a.m_pIShell, szResFile, IDS_MESSAGEBOX_TITLE, 
               IDS_MESSAGEBOX_TEXT)) == TRUE)
            {
               // We should return here so that the text from the message box
               // will not be over-written by the DisplayEvent function call at the
               // end of this function.
               return;
            }
            else
            {
               // The dialog could not be created
               DisplayOutput ((IApplet *)pMe, -1, "MessageBox display: Failed.");
            }
         }
         break;
      case USAGE_SETALARM:
         {
            // 24. Usage ISHELL_SetAlarm
            int nRetVal = EBADPARM; // Initialize return value.
            uint32 numMin = 1;      // Number of minutes before alarm is activated.
            uint16 nUserCode = 0;   // User code which distinguished this alarm
                                    // from another.

            // ISHELL_SetAlarm: allows the caller to set a long-term alarm for an applet.  
            // When the alarm expires, the applet will be loaded and passed an EVT_ALARM 
            // event with the specified 16-bit nUserCode as the user parameter.  
            // If the applet is not active at the time of the alarm, it will be loaded
            // but will not be sent an EVT_APP_START event.  If the applet wishes to be 
            // activated, it must call ISHELL_StartApplet. Note that more than one 
            // alarm can be set for an applet by specifying a different 16-bit alarm nUserCode.

            // Set Alarm to be activated in one minute. The alarm willbe posted
            // to the Shell-Support applet.
            if ((nRetVal = ISHELL_SetAlarm (pMe->a.m_pIShell, AEECLSID_SHELL_SUPPORT_APP, 
               nUserCode, numMin)) == SUCCESS)
            {
               // We need to make sure that the alarm is canceled if the user 
               // decides not to wait for 1 minute to see the alarm activate.
               // This flag will be used in the HandleEvent function of this applet
               // under EVT_KEY (when user presses any key) to deactivate alarm.
               pMe->alarmActive = TRUE;
               

               DisplayOutput ((IApplet *)pMe, -1, "Please wait 1 minute for alarm to activate.. ");
            }
            else
            {
               // Failed to set alarm.
               DisplayOutput ((IApplet *)pMe, -1, "Set Alarm: Failed. ");
            }
         }
         break;
      case USAGE_CANCELALARM:
         {
            // 25. Usage ISHELL_CancelAlarm
            int nRetVal = EBADPARM; // Initialize return value.
            uint32 numMin = 1;      // Number of minutes before alarm is activated.
            uint16 nUserCode = 0;   // User code which distinguished this alarm
                                    // from another.

            // First set an alarm.
            if ((nRetVal = ISHELL_SetAlarm (pMe->a.m_pIShell, AEECLSID_SHELL_SUPPORT_APP, 
               nUserCode, numMin)) == SUCCESS)
            {
               // ISHELL_CancelAlarm: cancels an alarm set via ISHELL_SetAlarm.  The 
               // proper class and 16-bit user code must be specified.

               // The alarm was set successfully. Now, cancel the alarm.
               if ((nRetVal = ISHELL_CancelAlarm (pMe->a.m_pIShell, AEECLSID_SHELL_SUPPORT_APP, 
                  nUserCode)) == SUCCESS)
               {
                  // We were able to cancel the alarm successfully.
                  DisplayOutput ((IApplet *)pMe, -1, "Alarm canceled successfully..");
               }
               else 
               {
                  // Alarm cancel failed. The alarm is still active.
                  // We need to make sure that the alarm is attempted to be canceled 
                  // when the user decides to mode to another function operation.
                  // This flag will be used in the HandleEvent function of this applet
                  // under EVT_KEY (when user presses any key) to deactivate alarm.
                  pMe->alarmActive = TRUE;
               }
            }
            else
            {
               // The alarm was not set successfully.
               DisplayOutput ((IApplet *)pMe, -1, "Set Alarm: Failed. ");
            }
         }
         break;
      case USAGE_ALARMSACTIVE:
         {
            // 26. Usage ISHELL_CancelAlarm
            boolean bRetVal = FALSE;

            // ISHELL_AlarmsActive: returns TRUE if any of ANNUN_ALARMCLOCK, ANNUN_COUNTDOWN
            // and ANNUN_STOPWATCH are set. This function will not check for any other
            // type of alarms. On Windows, this function always returns FALSE because
            // annunciators are not supported in the BREW Emulator.

            // Check if any alamrs are active. In this case it should fail.
            if ((bRetVal = ISHELL_AlarmsActive (pMe->a.m_pIShell)) == FALSE)
            {
               DisplayOutput ((IApplet *)pMe, -1, "No active alarms..");
            }
         }
         break;
      case USAGE_GETHANDLER:
         {
            // 27. Usage ISHELL_GetHandler
            AEEHandlerType hType = HTYPE_SOUND;  // Resource handler type (image)
            AEECLSID clsRetValue;
            // The string required to get the handler of the sound/midi player.
            const char szBuf[] = "snd/midi"; 

            // ISHELL_GetHandler: provides query access to the shell's database
            // of registered content viewers and protocol scheme handlers.  
            // This database is provided to allow browsers and content viewers 
            // to expose handlers for content or data protocols they may support.
            // The AEEHandler type indicates whether the input string is a viewer 
            // (i.e. image type) or a sound type. 

            // Get handler
            clsRetValue = ISHELL_GetHandler (pMe->a.m_pIShell, hType, szBuf);
            
            if (clsRetValue == AEECLSID_SOUNDPLAYER)
            {
               // We successfully retrieved the soundplayer handler.
               DisplayOutput ((IApplet *)pMe, -1, "Sound Player handler returned");
            }
         }
         break;
      case USAGE_REGISTERHANDLER:
         {
            // 28. Usage ISHELL_RegisterHandler
            int  iRetValue = 0;
            // Use "ftp" as a dummy service handler we are registering
            char szInputString[] = "ftp"; 
            // Use this applet class Id as the handler registration
            // class Id. This is used only for demonstration purposes only.
            AEECLSID   clsIdValue = AEECLSID_SHELL_MAIN_APP;

            // ISHELL_RegisterHandler: provides a mechanism for a content viewer or
            // protocol engine to register itself with the shell.  This allows other
            // component to share it's functionality when content or protocol types
            // are encountered. As the handler is specified by class, it can be loaded
            // dynamically on an as needed basis. In order to update a handler, the 
            // existing handler must be deleted from the database. This is done by calling 
            // ISHELL_RegisterHandler with a 0 class.  

            // Invoke ISHELL_RegisterHandler. Here we are registering a handler
            // for "ftp" and giving this applets class Id as the class Id for the
            // web service. 
            //
            // NOTE: This applet does not support "ftp" services. The handler registration
            //       is done for demonstration purposes only.
            iRetValue = ISHELL_RegisterHandler(pMe->a.m_pIShell, HTYPE_SOUND, 
				                               szInputString, clsIdValue);
			   
			switch (iRetValue)
			{
				case SUCCESS:
					DisplayOutput ((IApplet *)pMe, -1, "Registering handler succeeded..");
					break;

				case EALREADY:
					DisplayOutput ((IApplet *)pMe, -1, "Handler already set..");
					break;

				default:
					DisplayOutput ((IApplet *)pMe, -1, "Registering handler failed !.");
			}

         }
         break;
      case USAGE_REGISTERNOTIFY:
      case USAGE_NOTIFY:
         {
            // 29. Usage ISHELL_RegisterHandler/ISHELL_Notify
            boolean canStart = FALSE;

            //
            // ISHELL_RegisterNotify and ISHELL_Notify are implemented in the INotify 
            // Usage applet. Here, we will try to switch to the Notify applet if it's
            // loaded. Check if the notify applet can be loaded by invoking 
            // ISHELL_CanStartApplet and then if it can be loaded then start that 
            // applet by invoking ISHELL_StartApplet.
            // 
            // The usage examples of the above two functions are found in the
            // inotifyusage.c file. Please see this file for more deatils.
            //
            // Invoke CanStartApplet to check if the Notify applet can be started.
            if ((canStart = ISHELL_CanStartApplet (pMe->a.m_pIShell, 
               AEECLSID_NOTIFY_APP)) == TRUE)
            {
               // The Notify applet can be started. Start the Notify applet.
               ISHELL_StartApplet (pMe->a.m_pIShell, AEECLSID_NOTIFY_APP);
            }
            else
            {
               // INotify Applet can not be started.
               DisplayOutput ((IApplet *)pMe, 2, 
                  "Please see INotify Usage implementation for more details on ISHELL_RegisterHandler/ISHELL_Notify..");
            }
         }
         break;
      case USAGE_RESUME:
         {
            // 30. Usage ISHELL_Resume
            AEECallback callBack;

            // ISHELL_Resume: allows a callback to be registered with the Shell. It adds 
            // the callback to shell's list of pending operations. The shell will then 
            // invoke the callback function the next time the event loop is called. This 
            // allows an application or object to cooperatively multitask. If the callback 
            // has already been registered, it will be cancelled (de-registered) and then
            // re-registered.

            // "pfnCancel" need to be set to NULL by the caller. The Shell will fill 
            // in this field with the correct address on return.
            callBack.pfnCancel = (void *)NULL;
            callBack.pCancelData = NULL;
            callBack.pfnNotify = ResumeNotifyCB;
            callBack.pNotifyData = (void *)pMe;

            // Invoke ISHELL_Resume. This will pass control to the ResumeNotifyCB
            // function when the event loop is called, where a statement is 
            // displayed indicating the control has reached the ResumeNotifyCB
            // function.
            ISHELL_Resume (pMe->a.m_pIShell, &callBack);
         }
         break;
      case USAGE_FORCEEXIT:
         {
            // 31. Usage ISHELL_ForceExit
            boolean bRetVal = FALSE;

            // ISHELL_ForceExit: returns true if the applet must exit. This method 
            // is provided only for very special use by browsers or other 
            // applications that may do substantial processing before returning 
            // from the HandleEvent method.
            bRetVal = ISHELL_ForceExit (pMe->a.m_pIShell);

            if (bRetVal == TRUE)
            {
               // This applet should stop processing and exit.
               DisplayOutput ((IApplet *)pMe, -1, "Closing Active Applet");

               //
               // Save data and bring this applet to a state where it can be closed.
               //

               // This will close the active applet and return to the main menu.
               ISHELL_CloseApplet (pMe->a.m_pIShell, FALSE);
            }
            else
            {
               DisplayOutput ((IApplet *)pMe, -1, "Can continue processing..");
            }
         }
         break;
      case USAGE_GETRAND:
         {
            // 32. Usage ISHELL_GetRand
            byte buffer[5] = {0};       // Create buffer to store 5 randon numbers.
            int count;                 
            char buff[30] = {0};        // Char buffer for display string

            // ISHELL_GetRand: leverages the random number generator on the device.  
            // It fills an input buffer with an array of random values.

            // Get a buffer filled with random numbers.
            ISHELL_GetRand(pMe->a.m_pIShell, buffer, sizeof(buffer));

            // Print the 5 random numbers.
            for (count = 0; count < 5; count++)
            {
               // Append the count and number to the string to be displayed.
               SPRINTF (buff, "Random number %i: %u", count, buffer[count]); 
               // Display the random number on the "count+1" line one the screen.
               DisplayOutput((IApplet *)pMe, count + 1, buff);
            }
         }
         break;
      case USAGE_CREATEDIALOG:
      case USAGE_GETACTIVEDIALOG:
      case USAGE_ENDDIALOG:
         {
            // 33. Usage ISHELL_CreateDialog/ISHELL_GetActiveDialog/ISHELL_EndDialog
            boolean canStart = FALSE;

            //
            // ISHELL_CreateDialog, ISHELL_GetActiveDialog, and ISHELL_EndDialog are 
            // implemented in the IDialog Usage applet. 
            //
            // Here, we will try to switch to the IDialog usage applet if it is loaded:
            //    - check if the notify applet can be loaded by invoking 
            //      ISHELL_CanStartApplet.
            //    - if it can be loaded then start that applet by invoking ISHELL_StartApplet.
            // 
            // The usage examples of the above three functions are found in the
            // idialogusage.c file. Please see this file for more deatils.
            //
            // Invoke CanStartApplet to check if the Notify applet can be started.
            if ((canStart = ISHELL_CanStartApplet (pMe->a.m_pIShell, 
               AEECLSID_DIALOG_APP)) == TRUE)
            {
               // The Dialog applet can be started. Start the Dialog applet.
               ISHELL_StartApplet (pMe->a.m_pIShell, AEECLSID_DIALOG_APP);
            }
            else
            {
               // IDialog Applet can not be started.
               DisplayOutput ((IApplet *)pMe, 2, 
                  "Please see IDialog Usage implementation for more details on ISHELL_CreateDialog/ISHELL_GetActiveDialog/ISHELL_EndDialog..");
            }
         }
         break;
      case USAGE_SENDEVENT:
         {
            // 36. Usage ISHELL_SendEvent
            boolean bRetVal = FALSE;

            // ISHELL_SendEvent:
            //
            // The entire execution model of the AEE is based around a 
            // semi-cooperative event passing model. Under this model, 
            // events are sent to the active dialog or applet using the 
            // ISHELL_SendEvent method.  
            //
            // The SendEvent method allows for control over the destination
            // class of the applet. This function sends the event directly to
            // the destination app.
            //
            // Events can be sent to a specific applet by specifying the
            // target applet class.  If the applet is not currently running
            // it will be loaded and the event will be sent directly to it.
            // Under these conditions, the applet is started in the background.
            // EVT_APP_START/EVT_APP_STOP events will not be sent to the applet.
            //
            // Private events can be sent to an applet by defining the event
            // at or above EVT_USER and specifying the applet class. An error
            // is returned if events in the range of EVT_USER and above are
            // sent without an associated class ID.
            //
            // Note that if there is a dialog active (ISHELL_CreateDialog), 
            // the event will be passed to the dialog before being passed to 
            // the applet.  The event will only be passed to the applet if the 
            // dialog did not process the event.

            // Send event to support applet. The event sent is the EVT_USER event. 
            // The 16-bit data field is used to distiguish this event from the 
            // event sent by the ISHELL_PostEvent function.
            bRetVal = ISHELL_SendEvent (pMe->a.m_pIShell, AEECLSID_SHELL_SUPPORT_APP, 
               EVT_USER, SEND_EVENT_16BITDATA, COMMON_32BIT_EVENT_DATA);

            if (bRetVal == FALSE)
            {
               DisplayOutput ((IApplet *)pMe, -1, "Send Event failed");
            }
         }
         break;
      case USAGE_POSTEVENT:
         {
            // 37. Usage ISHELL_PostEvent
            boolean bRetVal = FALSE;

            // This function posts an asynchronous event to the specified app. 
            // This function is very similar to ISHELL_SendEvent(). The main 
            // difference is that this function posts the event to the app 
            // while SendEvent() immediately sends the event to the app. 
            // Event posting is provided for special cases where the caller either 
            // wishes to post a event from another task or wishes to defer the 
            // processing of the event until the next iteration of the event loop.  
            // This is useful in providing continued execution while allowing other 
            // events to be processed.
            //
            // Private events can be sent to an applet by defining the event
            // at or above EVT_USER and specifying the applet class. An error is 
            // returned if events in the range of EVT_USER and above are sent 
            // without an associated class ID.

            // Post Event to support applet. The event sent is the EVT_USER event. 
            // The 16-bit data field is used to distiguish this event from the 
            // event sent by the ISHELL_SendEvent function. 
            bRetVal = ISHELL_PostEvent (pMe->a.m_pIShell, AEECLSID_SHELL_SUPPORT_APP, 
               EVT_USER, POST_EVENT_16BITDATA, COMMON_32BIT_EVENT_DATA);

            if (bRetVal == FALSE)
            {
               DisplayOutput ((IApplet *)pMe, -1, "Post Event failed");
            }
         }
         break;
      case USAGE_HANDLEEVENT:
         {
            // 38. Usage ISHELL_HandleEvent
            boolean bRetVal = FALSE;
            AEEEvent event = EVT_COMMAND;   

            // ISHELL_HandleEvent: sends the specified event to the currently 
            // active app. It internally invokes ISHELL_SendEvent with classId 
            // set to 0. The event sent must be a standard AEE event 
            // (i.e. it must be < EVT_USER)

            // Send Event to current applet. This event will result in a message 
            // being printed on the screen indicating the HandleEvent function
            // of applet has been reached.
            bRetVal = ISHELL_HandleEvent (pMe->a.m_pIShell, event, 
               HANDLE_EVENT_16BITDATA, COMMON_32BIT_EVENT_DATA);
         }
         break;
      case USAGE_GETPOSITION:
         {
            //39. Usage ISHELL_GetLocation
            int nRetVal = 0;
            AEEPosAccuracy pa = AEE_ACCURACY_LOW;

            // ISHELL_GetLocation: provides access to the gpsOne location feature
            // on the handset.  The precision specified indicates how exact the 
            // location will be returned. The precision is also directly related 
            // to the time it will take to satisfy the request.
            //
            // Upon completion, the callback will be made to the user with the 
            // position or an appropriate error. 
            
            // NOTE: Currently, this function is not implemented. It returns EUNSUPPORTED.
            nRetVal = ISHELL_GetPosition (pMe->a.m_pIShell, pa, LocationCB, (void *)pMe);

            if (nRetVal == EUNSUPPORTED)
            {
               DisplayOutput ((IApplet *)pMe, -1, "GPS not supported");
            }
         }
         break;
      case USAGE_BUSY:
         {
            // 40. Usage ISHELL_Busy
            boolean bRetValue = FALSE;

            // ISHELL_Busy: returns true if the applet must exit.  This 
            // method is provided only for very special use by browsers or other 
            // applications that may do substantial processing before returning 
            // from the HandleEvent method.
            if ((bRetValue = ISHELL_Busy(pMe->a.m_pIShell)) == TRUE)
            {
               DisplayOutput ((IApplet *)pMe, -1, "Applet must exit");
               // The applet should clean up all the data and bring itself 
               // to a state where it can yeld control to the shell.
            }
            else
            {
               DisplayOutput ((IApplet *)pMe, -1, "Applet can proceed processing");
            }
         }
         break;
      case USAGE_LOADRESIMAGE:
         {
            // 41. Usage ISHELL_LoadResImage
            IImage *pIImage;
            char szResFile[] = APP_RES_FILE;   // Resource File used by this applet

            // ISHELL_LoadResImage: loads a bitmap resource from the given resource
            // file and returns a valid IImage interface pointer. This pointer 
            // can then be used for viewing the image.

            // Load resource image.
            pIImage = ISHELL_LoadResImage(pMe->a.m_pIShell, szResFile, IDB_BITMAP1);

            // Display the image in the middle of the screen (image root coordinates
            // will be placed in the middle of the screen).
            IIMAGE_Draw(pIImage,cx/2,cy/2);

            // Release the image object
            IIMAGE_Release (pIImage);

         }
         break;
      case USAGE_LOADRESSOUND:
         {
            // 42. Usage ISHELL_LoadResSound

            // This function is not implemented at present. The function takes a resource
            // file and a resource Id to load the sound resource. But, at present 
            // the resource editor does not support sound files.
         }
         break;
      default:
         return;
   }
            
   // Display above event.
   DisplayEvent (pMe, wParam);

   return;
}

/*===========================================================================

FUNCTION: DisplayEvent

DESCRIPTION
    This function is a display function which sets up the initial screen
    for the display output for a given interface function. This displays
    the function name at the top of the screen and displays the
    strings "Press Up/Down" and "keys to continue.." at the bottom of 
    the screen. In the case of functions that have no output it also 
    displays the string "No Output..." in the middile of the screen.
    
PROTOTYPE:
   void DisplayEvent (CIShellApp * pMe, uint16 wParam)

PARAMETERS:
   pMe:    [in]: Pointer to CIShellApp object
   wParam: [in]: API function type Id selected by user

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void DisplayEvent (CIShellApp * pMe, uint16 wParam)
{
   AECHAR szBuf[30];
   AEERect rc; 

   // In the case of "StartApplet" we pass control to another applet so what
   // we are printing below is not applicable. So, return without 
   // printing anything.
   if (wParam == USAGE_STARTAPPLET)
      return;

   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
      return;

   // Set display rectangle
   SETAEERECT (&rc, 0, 0, pMe->m_dInfo.cxScreen, pMe->m_dInfo.cyScreen);
   
   if (pMe->a.m_pIDisplay)
   {
      switch (wParam)
      {
         case USAGE_ERASE_SCREEN:
            IDISPLAY_EraseRect(pMe->a.m_pIDisplay,&rc);  //Clear the display
            return;
         case USAGE_CREATEINSTANCE:
            STR_TO_WSTR("CreateInstance", szBuf, sizeof(szBuf));
            break;
         case USAGE_QUERYCLASS:
            STR_TO_WSTR("QueryClass", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETDEVICEINFO:
            STR_TO_WSTR("GetDeviceInfo", szBuf, sizeof(szBuf));
            break;
         case USAGE_STARTAPPLET:
            STR_TO_WSTR("StartApplet", szBuf, sizeof(szBuf));
            break;
         case USAGE_STARTAPPLETEX:
            STR_TO_WSTR("StartAppletEx", szBuf, sizeof(szBuf));
            break;
         case USAGE_CLOSEAPPLET:
            STR_TO_WSTR("CloseApplet", szBuf, sizeof(szBuf));
            break;
         case USAGE_ACTIVEAPPLET:
            STR_TO_WSTR("ActiveApplet", szBuf, sizeof(szBuf));
            break;
         case USAGE_ENUMAPPLETINIT:
            STR_TO_WSTR("EnumAppletInit", szBuf, sizeof(szBuf));
            break;
         case USAGE_ENUMNEXTAPPLET:
            STR_TO_WSTR("EnumNextApplet", szBuf, sizeof(szBuf));
            break;
         case USAGE_CANSTARTAPPLET:
            STR_TO_WSTR("CanStartApplet", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETTIMER:
            STR_TO_WSTR("SetTimer", szBuf, sizeof(szBuf));
            break;
         case USAGE_CANCELTIMER:
            STR_TO_WSTR("CancelTimer", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETTIMEREXPIRATION:
            STR_TO_WSTR("GetTimerExpiration", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETTIMEMS:
            STR_TO_WSTR("GetTimeMS", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETUPTIMEMS:
            STR_TO_WSTR("GetUpTimeMS", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETSECONDS:
            STR_TO_WSTR("GetSeconds", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETJULIANDATE:
            STR_TO_WSTR("GetJulianDate", szBuf, sizeof(szBuf));
            break;
         case USAGE_LOADRESSTRING:
            STR_TO_WSTR("LoadResString", szBuf, sizeof(szBuf));
            break;
         case USAGE_LOADRESDATA:
            STR_TO_WSTR("LoadResData", szBuf, sizeof(szBuf));
            break;
         case USAGE_LOADRESOBJECT:
            STR_TO_WSTR("LoadResObject", szBuf, sizeof(szBuf));
            break;
         case USAGE_FREERESDATA:
            STR_TO_WSTR("FreeResData", szBuf, sizeof(szBuf));
            break;
         case USAGE_BEEP:
            STR_TO_WSTR("Beep", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETPREFS:
            STR_TO_WSTR("GetPrefs", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETPREFS:
            STR_TO_WSTR("SetPrefs", szBuf, sizeof(szBuf));
            break;
         case USAGE_PROMPT:
            STR_TO_WSTR("Prompt", szBuf, sizeof(szBuf));
            break;
         case USAGE_MESSAGEBOXTEXT:
            STR_TO_WSTR("MessageBoxText", szBuf, sizeof(szBuf));
            break;
         case USAGE_MESSAGEBOX:
            STR_TO_WSTR("MessageBox", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETALARM:
            STR_TO_WSTR("SetAlarm", szBuf, sizeof(szBuf));
            break;
         case USAGE_CANCELALARM:
            STR_TO_WSTR("CancelAlarm", szBuf, sizeof(szBuf));
            break;
         case USAGE_ALARMSACTIVE:
            STR_TO_WSTR("AlarmsActive", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETHANDLER:
            STR_TO_WSTR("GetHandler", szBuf, sizeof(szBuf));
            break;
         case USAGE_REGISTERHANDLER:
            STR_TO_WSTR("RegisterHandler", szBuf, sizeof(szBuf));
            break;
         case USAGE_REGISTERNOTIFY:
            STR_TO_WSTR("RegisterNotify", szBuf, sizeof(szBuf));
            break;
         case USAGE_NOTIFY:
            STR_TO_WSTR("Notify", szBuf, sizeof(szBuf));
            break;
         case USAGE_RESUME:
            STR_TO_WSTR("Resume", szBuf, sizeof(szBuf));
            break;
         case USAGE_FORCEEXIT:
            STR_TO_WSTR("ForceExit", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETRAND:
            STR_TO_WSTR("GetRand", szBuf, sizeof(szBuf));
            break;
         case USAGE_CREATEDIALOG:
            STR_TO_WSTR("CreateDialog", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETACTIVEDIALOG:
            STR_TO_WSTR("GetActiveDialog", szBuf, sizeof(szBuf));
            break;
         case USAGE_ENDDIALOG:
            STR_TO_WSTR("EndDialog", szBuf, sizeof(szBuf));
            break;
         case USAGE_SENDEVENT:
            STR_TO_WSTR("SendEvent", szBuf, sizeof(szBuf));
            break;
         case USAGE_POSTEVENT:
            STR_TO_WSTR("PostEvent", szBuf, sizeof(szBuf));
            break;
         case USAGE_HANDLEEVENT:
            STR_TO_WSTR("HandleEvent", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETPOSITION:
            STR_TO_WSTR("GetPosition", szBuf, sizeof(szBuf));
            break;
         case USAGE_BUSY:
            STR_TO_WSTR("Busy", szBuf, sizeof(szBuf));
            break;
         case USAGE_LOADRESIMAGE:
            STR_TO_WSTR("LoadResImage", szBuf, sizeof(szBuf));
            break;
         case USAGE_LOADRESSOUND:
            STR_TO_WSTR("LoadResSound", szBuf, sizeof(szBuf));
            break;
         default:
            STR_TO_WSTR("Unsupported fn", szBuf, sizeof(szBuf));
      }
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,5, 0, &rc,
		   IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);

      // Functions with no output
      switch (wParam)
      {
         case USAGE_LOADRESSOUND:
            // Place in the middle of screen
            STR_TO_WSTR("No Output...", szBuf, sizeof(szBuf));
 		      IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,10, 
               rc.dy/2 - pMe->m_nLineHeight, &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
         default:
            break;
      }

      STR_TO_WSTR("Press Up/Down", szBuf, sizeof(szBuf));
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,10, rc.dy - 
         2*(pMe->m_nLineHeight) , &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
      STR_TO_WSTR("keys to continue..", szBuf, sizeof(szBuf));
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,10, rc.dy - 
         pMe->m_nLineHeight, &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
      IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);
   }
}

/*===========================================================================

FUNCTION: BuildMainMenu

DESCRIPTION
    This function builds the main menu when this applet is started.
    It displays a simple menu with menu items corresponding to 
    each of the interface functions.
    
PROTOTYPE:
   void BuildMainMenu(CIShellApp *pi)

PARAMETERS:
   pMe: [in]: Pointer to AEEApplet struct

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void BuildMainMenu(CIShellApp *pMe)
{
	AEERect rc;
   AECHAR szBuf[50];

   if (pMe->m_pIMenu == NULL || pMe->a.m_pIShell == NULL)
      return;

   // Set display rectangle
   SETAEERECT (&rc, 0, 0, pMe->m_dInfo.cxScreen, pMe->m_dInfo.cyScreen);

   // Set Title
   STR_TO_WSTR("IShell Functions:", szBuf, sizeof(szBuf));
	IMENUCTL_SetTitle(pMe->m_pIMenu, NULL, 0, szBuf);

	//Set the Rectangle for the Menu
	IMENUCTL_SetRect(pMe->m_pIMenu, &rc);  

	//Add invidual entries to the Menu
  
   // Add CreateInstance to menu
   STR_TO_WSTR("1. CreateInstance", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_CREATEINSTANCE, szBuf, 0);
			
   // Add QueryClass to menu
   STR_TO_WSTR("2. QueryClass", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_QUERYCLASS, szBuf, 0);

   // Add GetDeviceInfo to menu
   STR_TO_WSTR("3. GetDeviceInfo", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETDEVICEINFO, szBuf, 0);

	// Add StartApplet to menu
   STR_TO_WSTR("4. StartApplet", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_STARTAPPLET, szBuf, 0);

	// Add StartAppletEx to menu
   STR_TO_WSTR("5. StartAppletEx", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_STARTAPPLETEX, szBuf, 0);	

	// Add CloseApplet to menu
   STR_TO_WSTR("6. CloseApplet", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_CLOSEAPPLET, szBuf, 0);	

	// Add ActiveApplet to menu
   STR_TO_WSTR("7. ActiveApplet", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_ACTIVEAPPLET, szBuf, 0);	

   // Add EnumAppletInit to menu
   STR_TO_WSTR("8. EnumAppletInit", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_ENUMAPPLETINIT, szBuf, 0);	

   // Add EnumNextApplet to menu
   STR_TO_WSTR("9. EnumNextApplet", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_ENUMNEXTAPPLET, szBuf, 0);	

   // Add CanStartApplet to menu
   STR_TO_WSTR("10. CanStartApplet", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_CANSTARTAPPLET, szBuf, 0);	

   // Add SetTimer to menu
   STR_TO_WSTR("11. SetTimer", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETTIMER, szBuf, 0);	

	// Add CancelTimer to menu
   STR_TO_WSTR("12. CancelTimer", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_CANCELTIMER, szBuf, 0);	

	// Add GetTimerExpiration to menu
   STR_TO_WSTR("13. GetTimerExpiration", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETTIMEREXPIRATION, szBuf, 0);	

	// Add GetTimeMS to menu
   STR_TO_WSTR("14. GetTimeMS", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETTIMEMS, szBuf, 0);	

	// Add GetUpTimeMS to menu
   STR_TO_WSTR("15. GetUpTimeMS", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETUPTIMEMS, szBuf, 0);	

	// Add GetSeconds to menu
   STR_TO_WSTR("16. GetSeconds", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETSECONDS, szBuf, 0);	

	// Add GetJulianDate to menu
   STR_TO_WSTR("17. GetJulianDate", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETJULIANDATE, szBuf, 0);	

	// Add LoadResString to menu
   STR_TO_WSTR("18. LoadResString", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_LOADRESSTRING, szBuf, 0);	

	// Add LoadResData to menu
   STR_TO_WSTR("19. LoadResData", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_LOADRESDATA, szBuf, 0);	

	// Add LoadResObject to menu
   STR_TO_WSTR("20. LoadResObject", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_LOADRESOBJECT, szBuf, 0);	

	// Add FreeResData to menu
   STR_TO_WSTR("21. FreeResData", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_FREERESDATA, szBuf, 0);	

	// Add Beep to menu
   STR_TO_WSTR("22. Beep", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_BEEP, szBuf, 0);	

	// Add GetPrefs to menu
   STR_TO_WSTR("23. GetPrefs", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETPREFS, szBuf, 0);	

	// Add SetPrefs to menu
   STR_TO_WSTR("24. SetPrefs", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPREFS, szBuf, 0);	

	// Add Prompt to menu
   STR_TO_WSTR("25. Prompt", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_PROMPT, szBuf, 0);	

   // Add MessageBoxText to menu
   STR_TO_WSTR("26. MessageBoxText", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_MESSAGEBOXTEXT, szBuf, 0);	

	// Add MessageBox to menu
   STR_TO_WSTR("27. MessageBox", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_MESSAGEBOX, szBuf, 0);	

	// Add SetAlarm to menu
   STR_TO_WSTR("28. SetAlarm", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETALARM, szBuf, 0);	

	// Add CancelAlarm to menu
   STR_TO_WSTR("29. CancelAlarm", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_CANCELALARM, szBuf, 0);	

	// Add AlarmsActive to menu
   STR_TO_WSTR("30. AlarmsActive", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_ALARMSACTIVE, szBuf, 0);	

	// Add GetHandler to menu
   STR_TO_WSTR("31. GetHandler", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETHANDLER, szBuf, 0);	

	// Add RegisterHandler to menu
   STR_TO_WSTR("32. RegisterHandler", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_REGISTERHANDLER, szBuf, 0);	

	// Add RegisterNotify to menu
   STR_TO_WSTR("33. RegisterNotify (switch applet)", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_REGISTERNOTIFY, szBuf, 0);	
   
	// Add Notify to menu
   STR_TO_WSTR("34. Notify (switch applet)", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_NOTIFY, szBuf, 0);	

	// Add Resume to menu
   STR_TO_WSTR("35. Resume", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_RESUME, szBuf, 0);	

	// Add ForceExit to menu
   STR_TO_WSTR("36. ForceExit", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_FORCEEXIT, szBuf, 0);	

	// Add GetRand to menu
   STR_TO_WSTR("37. GetRand", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETRAND, szBuf, 0);	

	// Add CreateDialog to menu
   STR_TO_WSTR("38. CreateDialog (switch applet)", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_CREATEDIALOG, szBuf, 0);	

	// Add GetActiveDialog to menu
   STR_TO_WSTR("39. GetActiveDialog (switch applet)", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETACTIVEDIALOG, szBuf, 0);	

	// Add EndDialog to menu
   STR_TO_WSTR("40. EndDialog (switch applet)", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_ENDDIALOG, szBuf, 0);	

	// Add SendEvent to menu
   STR_TO_WSTR("41. SendEvent", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SENDEVENT, szBuf, 0);	

	// Add PostEvent to menu
   STR_TO_WSTR("42. PostEvent", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_POSTEVENT, szBuf, 0);	

	// Add HandleEvent to menu
   STR_TO_WSTR("43. HandleEvent", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_HANDLEEVENT, szBuf, 0);	

	// Add GetPosition to menu
   STR_TO_WSTR("44. GetPosition", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETPOSITION, szBuf, 0);	

	// Add Busy to menu
   STR_TO_WSTR("45. Busy", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_BUSY, szBuf, 0);	

	// Add LoadResImage to menu
   STR_TO_WSTR("46. LoadResImage", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_LOADRESIMAGE, szBuf, 0);	

	// Add LoadResSound to menu
   STR_TO_WSTR("47. LoadResSound", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_LOADRESSOUND, szBuf, 0);	

   // Activate menu
	IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);
}

/*===========================================================================

FUNCTION: DisplayOutput

DESCRIPTION
    This function displays an output string at a given line number on the
    screen. If the nline parameter is a negative value (-1) the string
    is displayed in the middle of the screen. If the "nline" value is larger
    than or equal to zero the "nline" value is multiplied by 15 and the 
    resulting value in pixels is set to the y-coordinate of the start of 
    the string display on the screen. If the string does not fit on one line
    the string wraps around to the next line (spaced rougly 10-15 pixels apart).
    By default 5 is used as the starting the x-coordinate of a displayed 
    string.

    How many characters that fit on one line is calculated for each line 
    that is wrapped around to the next line.

    Note: depending on the phone screen size and the fonts used for characters 
          the output might differ on different handsets (devices). Where some 
          handsets will have a smaller screen and large default fonts which will 
          cause partial overlapping of lines. This function does not try to address
          these issues (this is meant as a simple display function).
    
PROTOTYPE:
   void DisplayOutput(IApplet * pi, int nline, char *pszStr)

PARAMETERS:
   pi:     [in]: Contains a pointer to the IApplet class.
   nline:  [in]: Contains the line number to start displaying the text. 
        The line numbers are by default spaced 15 pixels apart along the y-axis.
   pszStr: [in]: The character string to be displayed on the screen.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void DisplayOutput(IApplet * pi, int nline, char *pszStr)
{
   AEEDeviceInfo di;        // Device Info
   AECHAR szBuf[200] = {0}; // a buffer that supports 200 char string
   AECHAR * psz = NULL;
   int pixelWidth;
   AEEFont font = AEE_FONT_NORMAL;
   int pnFits = 0, dy;
   int totalCh = 0;
   int charHeight = 0;      // Stores the char height in pixels for given font
   int pnAscent = 0;        // Stores the ascent in number of pixels
   int pnDescent = 0;       // Stores the descent in number of pixels
	AEERect rc;

   AEEApplet * pMe = (AEEApplet*)pi;

   if (pMe == NULL)
      return;

   // Get device information
   ISHELL_GetDeviceInfo(pMe->m_pIShell,&di);

   // Get the font metrics info
   charHeight = IDISPLAY_GetFontMetrics (pMe->m_pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   // Convert to wide string (unicode)
   STR_TO_WSTR ((char *)pszStr, szBuf, sizeof (szBuf));

   // If nlines is less than zero then print this string starting around the middle of 
   // the screen. Or else multiply nlines by 10 to decide the y coordinate of
   // the start of the string.
   if (nline < 0) {
      dy = di.cyScreen*2/5;
   }
   else{
      dy = nline * charHeight;
   }

   // psz keeps track of the point from which to write from the string buffer
   // in case the string does not fit one line and needs to wrap around in the
   // next line.
   psz = szBuf;
      
   // Need to calculate the lotal string length to decide if any wrapping
   // around is needed.
   totalCh = STRLEN ((char *)pszStr);

   // Keep displaying text string on multiple lines if the string can't be displayed
   // on one single line. Lines are spaced 15 pixels apart.
   while ((totalCh > 0) && (*psz != NULL))
   { 
      // Get information on how many characters will fit in a line.
      // Give the pointer to the buffer to be displayed, and the number of
      // pixels along the x axis you want to display the string in (max number)
      // pnFits will have the max number of chars that will fit in the maxWidth
      // number of pixels (given string can't fit in one line), or the number of 
      // chars in the string (if it does fit in one line). pnWidth gives the
      // number of pixels that will be used to display pnFits number of chars.
      pixelWidth = IDISPLAY_MeasureTextEx(pMe->m_pIDisplay,
                      font, 
                      (AECHAR *) psz,  // Start of the buffer to display,
                      -1,
                      di.cxScreen - 5, // maxWidth
                      &pnFits);         // Number of chars that will fit a line

      // If pnFits is zero there is something wrong in the input to above function. 
      // Normally this scenario should not occur. But, have the check anyway.
      if (pnFits == 0)
         return;

		SETAEERECT(&rc, 0, dy, di.cxScreen, charHeight);
		IDISPLAY_EraseRect(pMe->m_pIDisplay, &rc);
      IDISPLAY_DrawText(pMe->m_pIDisplay, AEE_FONT_NORMAL, psz, pnFits, 5 /*start dx*/, 
         dy, 0 /* use default rectangle coordinates */, 0);

      psz += pnFits;      // move pointer to the next segment to be displayed
      totalCh -= pnFits;  // reduce the total number of characters to still display
      dy += charHeight;   // Place next line charHeight pixels below the 
                          // previous line. 
      IDISPLAY_Update(pMe->m_pIDisplay);
      if (totalCh < pnFits)
         pnFits = totalCh;  // if total number is less than pnFits, adjust pnFits
   }
   return;   
} // End of DisplayOutput

/*===========================================================================

Function:  NotifierClass_TimerCB()

Description:
		This function gets activated when the timer set in the 
      NotifierClass_SetMask() function expires. This timer expiration
      is used to mimic an event occuring on the Notifier-Service. And hence
      we inform the Shell by invoking the ISHELL_Notify() function that
      a previosly requested event has occursed.

Prototype:
	void NotifierClass_TimerCB (void *pi)

Parameters:
   po: pointer to user data

Return Value: None

Comments:  This function is a part of the Notifier class which implements
           a basic notification service.

Side Effects: None

==============================================================================*/
static void TimerCB (void *pi)
{
   CIShellApp *pMe = (CIShellApp *)pi;

   DisplayOutput ((IApplet *)pMe, -1, "We are in the Timer-callback function...");
   return;
}

/*===========================================================================

Function:  ResumeNotifyCB()

Description:
      This function is a resume-callback function registered with
      the shell. When the event loop is called the shell executes this 
      callback function. This callback mechanism allows an application or 
      object to cooperatively multitask. Hence, this callback needs to 
      restore the applets state and data and bring it to an operational state.

Prototype:
	void ResumeNotifyCB (void *pi)

Parameters:
   pi: pointer to user data

Return Value: None

Comments:  This function is a part of the Notifier class which implements
           a basic notification service.

Side Effects: None

==============================================================================*/
static void ResumeNotifyCB (void *pi)
{
   CIShellApp *pMe = (CIShellApp *)pi;

   // Display that the resume-callback has been called and return.
   DisplayOutput ((IApplet *)pMe, -1, "We are in the ResumeNotify-callback function...");
   return;
}

static void LocationCB(void * pUser, AEEPositionInfo * pli, int nErr)
{
   CIShellApp *pMe = (CIShellApp *)pUser;

   DisplayOutput ((IApplet *)pMe, -1, "In Location CB function...");
   return;
}
