/*===========================================================================

FILE: isoundusage.c

SERVICES: Sample applet using AEE

DESCRIPTION
  This file contains usage examples of ISound interface. ISound interface
  functions allow the user to create/operate on ISound objects. ISound 
  interface offers basic sound services.

PUBLIC CLASSES:  
   N/A


   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/

/*===============================================================================
                     INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "AEEModGen.h"
#include "AEEAppGen.h"
#include "AEEMenu.h"
#include "AEEStdLib.h"
#include "AEEUsageAppIDs.h"
#include "AEESound.h"

/*===========================================================================

                      PUBLIC DATA DECLARATIONS

===========================================================================*/

/*-------------------------------------------------------------------
            Type Declarations
-------------------------------------------------------------------*/
// Sound app struct. This is the main struct for this applet. This will hold 
// all the data members that needs to be remembered throughout the life of
// the applet. 
// THE FIRST DATA MEMBER OF THIS STRUCT MUST BE AN AEEApplet OBJECT. 
// This is critical requirement which is related to an applet being
// dynamically loaded. The app developers should add their data members
// following the AEEApplet data member.
typedef struct _CISoundApp
{
	AEEApplet	a;        // Mandatory first AEEApplet data member

   IMenuCtl *  m_pIMenu;

   ISound *             m_pISound;
   AEESoundToneData *   m_pToneData; // Tone-list
   uint16               m_nTones; // Number of tones in the list

   // Device parameters
   int             m_nLineHeight;
   AEEDeviceInfo   m_deviceInfo;
}CISoundApp;

/*-------------------------------------------------------------------
            Function Prototypes
-------------------------------------------------------------------*/
// App Handle Event function
static boolean SoundApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);

// App specific data alloc-init/free functions
static boolean ISound_InitAppData(IApplet* pMe);
static void ISound_FreeAppData(IApplet* pMe);

static void BuildMainMenu(CISoundApp *pMe);
static void ISoundUsage (CISoundApp * pMe, uint16 wParam);
static void DisplayEvent (CISoundApp *pMe, uint16 wParam);
static void DisplayOutput(CISoundApp * pMe, int nline, char *pszStr);
static void SoundCBFn( void * pUser, AEESoundCmd eCBType, AEESoundStatus eStatus, uint32 dwParam );

/*-------------------------------------------------------------------
            Global Constant Definitions
-------------------------------------------------------------------*/

// App specific constants
#define USAGE_BASIC_USAGE      100
#define USAGE_REGISTERNOTIFY   101
#define USAGE_SET              102
#define USAGE_GET              103
#define USAGE_SETDEVICE        104
#define USAGE_PLAYTONE         105
#define USAGE_PLAYTONELIST     106
#define USAGE_PLAYFREQTONE     107
#define USAGE_STOPTONE         108
#define USAGE_VIBRATE          109
#define USAGE_STOPVIBRATE      110
#define USAGE_SETVOLUME        111
#define USAGE_GETVOLUME        112

#define HIGHFREQ               600
#define LOWFREQ                200
#define PLAY_DURATION1         3000   // In milliseconds
#define PLAY_DURATION2         4000
#define PLAY_DURATION3         5000
#define PLAY_DURATION4         6000

/*===============================================================================
                     FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to ISound_InitAppData(). 
   This is done as follows: InitAppData() is called to initialize AppletData 
   instance. It is app developers responsibility to fill-in app data initialization 
   code of InitAppData(). App developer is also responsible to release memory 
   allocated for data contained in AppletData -- this can be done in 
   ISound_FreeAppData().

PROTOTYPE:
   int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell object. 

	pIModule: pin]: Contains pointer to the IModule object to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid IApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the app needs to be loaded and if AEEApplet_New() invocation was
     successful
  EFAILED: If the app does not need to be loaded or if errors occurred in 
     AEEApplet_New(). If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
{
   *ppObj = NULL;
		
   // We want to load this App. So, invoke AEEApplet_New().To it, pass the
   // address of the app-specific handle event function.
   if(ClsId == AEECLSID_SOUND_APP){
      if(AEEApplet_New(sizeof(CISoundApp), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)SoundApp_HandleEvent,(PFNFREEAPPDATA)ISound_FreeAppData)
         == TRUE)
      {
         if (ISound_InitAppData((IApplet*)*ppObj) == TRUE)
         {
            return(AEE_SUCCESS);
         }
      }
   }
	return (EFAILED);
}

/*===========================================================================

FUNCTION SoundApp_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean SoundApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

	ecode: Specifies the Event sent to this applet

   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/
static boolean SoundApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
   CISoundApp * pMe = (CISoundApp*)pi;

   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
      return FALSE;

   switch (eCode) 
	{
       case EVT_APP_START:
         // Create the IMenu interface object. When the applet is started the 
         // main menu is set up. When the user selects a menu item from the main 
         // menu, an EVT_COMMAND event is received by this applet. The processing for 
         // EVT_COMMAND event is found below.
		   if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, (void **)&pMe->m_pIMenu)
            != SUCCESS)
         {
				return FALSE;
         }

         // Build the main menu
			BuildMainMenu(pMe);
			return(TRUE);
      case EVT_APP_STOP:
         return(TRUE);
      case EVT_KEY:
         // Release the ISound object if it is not released yet.
         if (pMe->m_pISound != NULL)
         {
            // Deregister the notify function. Do this first so that
            // the subsequent ISOUND operations will not queue any
            // callbacks.
            ISOUND_RegisterNotify(pMe->m_pISound, NULL, NULL);
            // Release tone data on heap, if any
            if ( pMe->m_pToneData )
            {
               FREE(pMe->m_pToneData);
               pMe->m_pToneData = NULL;
               pMe->m_nTones = 0;
            }
            // Stop tone if any are playing
            ISOUND_StopTone (pMe->m_pISound);
            // If the vibrate is on stop it
            ISOUND_StopVibrate (pMe->m_pISound);
            // Release the sound object
            ISOUND_Release (pMe->m_pISound);
            pMe->m_pISound = NULL;
         }

         // We come here when a key is pressed on the handset keypad. Call 
         // the IMenuCtl handle event function to handle this key press.
			if(pMe->m_pIMenu != NULL)
			{
				if ((IMENUCTL_IsActive(pMe->m_pIMenu) == FALSE) &&
					 ((wParam == AVK_UP) || (wParam == AVK_DOWN)))
				{
					IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);
					IMENUCTL_Redraw (pMe->m_pIMenu);
				}
				return IMENUCTL_HandleEvent(pMe->m_pIMenu, EVT_KEY, wParam, 0);
			}
         else
            return FALSE;
		case EVT_COMMAND:
         // When a menu item is selected this applet receives the
         // EVT_COMMAND event in the wParam field.
  		   switch(wParam)
         {
            case USAGE_BASIC_USAGE:
            case USAGE_REGISTERNOTIFY:
            case USAGE_SET:
            case USAGE_GET:
            case USAGE_SETDEVICE:
            case USAGE_PLAYTONE:
            case USAGE_PLAYTONELIST:
            case USAGE_PLAYFREQTONE:
            case USAGE_STOPTONE:
            case USAGE_VIBRATE:
            case USAGE_STOPVIBRATE:
            case USAGE_SETVOLUME:
            case USAGE_GETVOLUME:
               // Call the usage function that exercises all the 
               // interface functions.
               ISoundUsage (pMe, wParam);
               return TRUE;
		   	default:
			   	return FALSE;
         }
      default:
         break;
   }
   return FALSE;
}



/*===========================================================================

FUNCTION ISound_InitAppData

DESCRIPTION
   This function initializes app specific data allocates memory for app data 
   (AppletData) and sets it to pAppData of AEEApplet. It also function . 
   App developer needs to fill-in the initialization code.

PROTOTYPE:
	boolean ISound_InitAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains 
   information specific to this applet. It was initialized during the 
   AEEClsCreateInstance().

DEPENDENCIES
   Assumes pi is not NULL

RETURN VALUE
  TRUE: If the app has app data is allocated and initialized successfully
  FALSE: Either app data could not be allocated or initialzied

SIDE EFFECTS
  none
===========================================================================*/
static boolean ISound_InitAppData(IApplet* pi)
{
   int pnAscent;        // Stores the ascent in number of pixels
   int pnDescent;       // Stores the descent in number of pixels

   CISoundApp * pMe = (CISoundApp*)pi;

   // Initialize the MenuCtl pointer to NULL
   pMe->m_pIMenu = NULL;

   // Initialize sound parameters
   pMe->m_pISound = NULL;
   pMe->m_nTones = 0;
   pMe->m_pToneData = NULL;

   // Get the font metrics info
   pMe->m_nLineHeight = IDISPLAY_GetFontMetrics (pMe->a.m_pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&pMe->m_deviceInfo);

   return TRUE;
}

/*===========================================================================

FUNCTION ISound_FreeAppData

DESCRIPTION
   Frees data contained in app data and memory for app data itself.
   App developer needs to free data contained in AppletData.

PROTOTYPE:
	void ISound_FreeAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains 
   information specific to this applet. It was initialized during the 
   AEEClsCreateInstance().

DEPENDENCIES
   Assumes pi is not NULL

RETURN VALUE
   None

SIDE EFFECTS
   None
===========================================================================*/
static void ISound_FreeAppData(IApplet* pi)
{
   CISoundApp * pMe = (CISoundApp*)pi;

   // Release main menu
   if (pMe->m_pIMenu != NULL)
   {
      IMENUCTL_Release (pMe->m_pIMenu);
      pMe->m_pIMenu = NULL;
   }
   // Release Sound object if it is not released
   if (pMe->m_pISound != NULL)
   {
      // Deregister the notify function
      ISOUND_RegisterNotify(pMe->m_pISound, NULL, NULL);
      // Stop tone if any are playing
      ISOUND_StopTone (pMe->m_pISound);
      // Release tone data on heap, if any
      if ( pMe->m_pToneData )
      {
         FREE(pMe->m_pToneData);
         pMe->m_pToneData = NULL;
         pMe->m_nTones = 0;
      }
      // If the vibrate is on stop it
      ISOUND_StopVibrate (pMe->m_pISound);
      // Release the sound object
      ISOUND_Release (pMe->m_pISound);
      pMe->m_pISound = NULL;
   }
}

/*===============================================================================
                     APP HELPER FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: ISoundUsage

DESCRIPTION
	This function encompasses all the usage examples of all the functions
   in code blocks switched using the BREW API function Id passed into this
   function. The code blocks will include almost all the variable declarations
   needed to demonstrate the usage of a given function. Within the code block
   all the needed interfaces are instantiated and released subsequent to their
   use.

   There are some exceptions to the above general rule, such as call-back functions,
   common helper function, or common display functions called from within the
   code block which are defined elsewhere in this file.

   There are some instances where an object instantiated within a code block is
   not released at the end of the code block, to be released at a later point
   (for example in a call-back function, in this case pointers to such objects
   are stored in the AEEApplet object to be released later).
   
PROTOTYPE:
   void ISoundUsage (CISoundApp * pMe, uint16 wParam)

PARAMETERS:
	pMe: [in]: Contains a pointer to the CISoundApp struct.
   wParam: [in]: Contains the Id of the function type to be used (contains the
      Id corresponding to the menu item selected on the emulator).

DEPENDENCIES
  none

RETURN VALUE
  none

SIDE EFFECTS
  none
===========================================================================*/
static void ISoundUsage (CISoundApp * pMe, uint16 wParam)
{
   if (!pMe)
      return;

   // Erase screen first for display output purposes.
   IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
	IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
	IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);

   switch (wParam)
   {
      case USAGE_BASIC_USAGE:
         {
            // 1. Usage: Basic ISound Usage
            AEESoundToneData toneData;

            // Create ISound interface object. This step needs to be done only
            // if the ISound interface object have not already been created. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUND, (void **)&pMe->m_pISound); 

            if (pMe->m_pISound)
            {
               // Register a notify function. Results of most of the sound 
               // operations will be sent to the client via the callback
               // function pointer. Also, when the operation is done, it notifies
               // the client through the same callback function.
               ISOUND_RegisterNotify(pMe->m_pISound, &SoundCBFn, (void *)pMe);

               // Tone ID.
               toneData.eTone = AEE_TONE_RING_A;
               // Tone duration in milliseconds. A value of zero plays until ISOUND_StopTone() is called.
               toneData.wDuration = PLAY_DURATION1;  

               // ISOUND_PlayTone: plays the specified tone, given a tone ID 
               // for the specified amount of time the tone.
               //
               // Note: the status of this operation is passed to the
               //       callback registered above. For each tone played 
               //       the callback function will receive:
               //         1) AEE_SOUND_STATUS_CB
               //         2) AEE_SOUND_SUCCESS
               //       For tone playback is completed or stopped, the callback receives: 
               //         1) AEE_SOUND_STATUS_CB
               //         2) AEE_SOUND_PLAY_DONE
               //
               ISOUND_PlayTone (pMe->m_pISound, toneData);

               // The sound object is not released here so that we give 
               // an opportunity for the PlayTone operation to finish
               // and the callback function to be called with the status of 
               // the operation.
               //
               // This object will be released when the user presses a key to 
               // move to another menu option or when the user exits the applet.
            }
         }
         break;
      case USAGE_REGISTERNOTIFY:
         {
            // 2. Usage: ISOUND_RegisterNotify
            ISound * pISound = NULL;

            // Create ISound interface object. This step needs to be done only
            // if the ISound interface object have not already been created. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUND, (void **)&pISound); 

            if (pISound)
            {
               // Register the status callback funtion using ISOUND_RegisterNotify(). 
               // ISound always communicates with applets thru this callback 
               // function. Callback function carries information regarding user 
               // data, status callback types (AEE_SOUND_STATUS_CB, 
               // AEE_SOUND_VOLUME_CB) and status (AEE_SOUND_SUCCESS, 
               // AEE_SOUND_PLAY_DONE, AEE_SOUND_FAILURE)
               ISOUND_RegisterNotify(pISound, &SoundCBFn, (void *)pMe);

               // Cancel the callback notification
               ISOUND_RegisterNotify(pISound, NULL, NULL);
               // Release the sound object
               ISOUND_Release (pISound);
            }
         }
         break;
      case USAGE_SET:
         {
            // 3. Usage: ISOUND_Set
            AEESoundInfo sndInfo;
            int nRetVal = EBADPARM;

            // Use the default values
            sndInfo.eDevice = AEE_SOUND_DEVICE_CURRENT;
            sndInfo.eMethod = AEE_SOUND_METHOD_BEEP;
            sndInfo.eAPath = AEE_SOUND_APATH_LOCAL;
            sndInfo.eEarMuteCtl = AEE_SOUND_MUTECTL_UNMUTED;
            sndInfo.eMicMuteCtl = AEE_SOUND_MUTECTL_UNMUTED;

            // Create ISound interface object. This step needs to be done only
            // if the ISound interface object have not already been created. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUND, (void **)&pMe->m_pISound); 

            if (pMe->m_pISound)
            {
               ISOUND_RegisterNotify(pMe->m_pISound, &SoundCBFn, (void *)pMe);

               // ISOUND_Set: sets the device attributes of ISound interface 
               // object. The device attributes include Device, Method, APath, 
               // Ear-piece Mute Control and Mic Mute Control.

               if ((nRetVal = ISOUND_Set (pMe->m_pISound, &sndInfo)) == SUCCESS)
               {
                  DisplayOutput (pMe, -1, "Set: successful");
               }

               // The audio device is not connected at this point.
               // A call the ISOUND_SetDevice function will connect the
               // device. When this operation has completed the callback
               // function will receive the status of the SetDevice.
               // See ISOUND_SetDevice for more information.

               // Cancel the callback notification
               ISOUND_RegisterNotify(pMe->m_pISound, NULL, NULL);
               // Release the sound object
               ISOUND_Release (pMe->m_pISound);
            }
         }
         break;
      case USAGE_GET:
         {
            // 4. Usage: ISOUND_Get
            ISound  *pISound = NULL;
            AEESoundInfo sndInfo = {0};

            // Create ISound interface object. This step needs to be done only
            // if the ISound interface object have not already been created. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUND, (void **)&pISound); 

            if (pISound)
            {
               ISOUND_RegisterNotify(pISound, &SoundCBFn, (void *)pMe);

               // ISOUND_Get: gets the device attributes of ISound 
               // interface object.
               ISOUND_Get (pISound, &sndInfo);

               // These are the expected default values that we are 
               // checking against.
               if (sndInfo.eDevice == AEE_SOUND_DEVICE_CURRENT)
               {
                  DisplayOutput (pMe, 2, "Device: Current");
               }
               if (sndInfo.eMethod == AEE_SOUND_METHOD_BEEP)
               {
                  DisplayOutput (pMe, 3, "Method: Beep");
               }
               if (sndInfo.eAPath == AEE_SOUND_APATH_LOCAL)
               {
                  DisplayOutput (pMe, 4, "APath: Local");
               }
               if (sndInfo.eEarMuteCtl == AEE_SOUND_MUTECTL_UNMUTED)
               {
                  DisplayOutput (pMe, 5, "Ear MuteCtl: Unmuted");
               }
               if (sndInfo.eMicMuteCtl == AEE_SOUND_MUTECTL_UNMUTED)
               {
                  DisplayOutput (pMe, 6, "Mic MuteCtl: Unmuted");
               }

               // Cancel the callback notification
               ISOUND_RegisterNotify(pISound, NULL, NULL);
               // Release the sound object
               ISOUND_Release (pISound);
            }
         }
         break;
      case USAGE_SETDEVICE:
         {
            // 5. Usage: ISOUND_SetDevice
            AEESoundInfo sndInfo;

            // Use the default values
            sndInfo.eDevice = AEE_SOUND_DEVICE_CURRENT;
            sndInfo.eMethod = AEE_SOUND_METHOD_RING;
            sndInfo.eAPath = AEE_SOUND_APATH_LOCAL;
            sndInfo.eEarMuteCtl = AEE_SOUND_MUTECTL_UNMUTED;
            sndInfo.eMicMuteCtl = AEE_SOUND_MUTECTL_UNMUTED;

            // Create ISound interface object. This step needs to be done only
            // if the ISound interface object have not already been created. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUND, (void **)&pMe->m_pISound); 

            if (pMe->m_pISound)
            {
               ISOUND_RegisterNotify(pMe->m_pISound, &SoundCBFn, (void *)pMe);

               // ISOUND_Set: sets the device attributes of ISound interface 
               // object. The device attributes include Device, Method, APath, 
               // Ear-piece Mute Control and Mic Mute Control.

               ISOUND_Set (pMe->m_pISound, &sndInfo);

               // The audio device is not connected at this point.
               // A call the ISOUND_SetDevice function will connect the
               // device. 
               //
               // ISOUND_SetDevice: commands ISound to connect to a specific audio 
               // device. There is always one, and only one, audio device 
               // connected at any particular time.
               ISOUND_SetDevice(pMe->m_pISound);

               // The result of the above operation will be sent to the client 
               // via the callback function pointer. The status can be one of 
               // the following values:
               // - AEE_SOUND_SUCCESS: successful connecting to the new sound device.
               // - AEE_SOUND_FAILURE: failed connecting to the new sound device.

               // We do not release the sound object here. So that we give 
               // SetDevice operation to occur and the callback function
               // to be called with the status of the operation. This 
               // object will be released when the user presses a key to move
               // to another menu option or when the user exits the applet.
            }
         }
         break;
      case USAGE_PLAYTONE:
         {
            // 6. Usage: ISOUND_PlayTone
            AEESoundToneData toneData;

            // Create ISound interface object. This step needs to be done only
            // if the ISound interface object have not already been created. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUND, (void **)&pMe->m_pISound); 

            if (pMe->m_pISound)
            {
               // Register a notify function. Results of most of the sound 
               // operations will be sent to the client via the callback
               // function pointer. Also, when the operation is done, it notifies
               // the client through the same callback function.
               ISOUND_RegisterNotify(pMe->m_pISound, &SoundCBFn, (void *)pMe);

               toneData.eTone = AEE_TONE_RING_B;
               toneData.wDuration = PLAY_DURATION4;     // in milliseconds

               // ISOUND_PlayTone: plays a tone given a tone ID for the 
               // specified amount of time.
               //
               // Note: the status of this operation is passed to the
               //       callback registered above. For the tone played 
               //       the callback function will receive:
               //         1) AEE_SOUND_STATUS_CB
               //         2) AEE_SOUND_SUCCESS
               //       When the tone playback is completed, the callback receives: 
               //         1) AEE_SOUND_STATUS_CB
               //         2) AEE_SOUND_PLAY_DONE
               ISOUND_PlayTone (pMe->m_pISound, toneData);


               // The sound object is not released nor the callback deregistered 
               // here so that it give an opportunity for the PlayTone operation 
               // to finish and the callback function to be called with the 
               // status of the operation.
            }
         }
         break;
      case USAGE_PLAYTONELIST:
         {
            // 7. Usage: ISOUND_PlayToneList

            // Create ISound interface object. This step needs to be done only
            // if the ISound interface object have not already been created. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUND, (void **)&pMe->m_pISound); 

            if (pMe->m_pISound)
            {
               // Register a notify function. Results of most of the sound 
               // operations will be sent to the client via the callback
               // function pointer. Also, when the operation is done, it notifies
               // the client through the same callback function.
               ISOUND_RegisterNotify(pMe->m_pISound, &SoundCBFn, (void *)pMe);

               // IMPORTANT: Allocate tone list data on heap. 
               // It must stay on heap:
               // (1) till ISOUND_PlayToneList() completes playing all the tones in the list or
               // (2) till ISOUND_StopTone() is called to stop the play list.

               pMe->m_nTones = 4;
               pMe->m_pToneData = MALLOC(sizeof(AEESoundToneData)*pMe->m_nTones);
               if ( pMe->m_pToneData )
               {
                  pMe->m_pToneData[0].eTone = AEE_TONE_RING_B;
                  pMe->m_pToneData[0].wDuration = PLAY_DURATION2; // In milliseconds

                  pMe->m_pToneData[1].eTone = AEE_TONE_RING_A;
                  pMe->m_pToneData[1].wDuration = PLAY_DURATION1;

                  pMe->m_pToneData[2].eTone = AEE_TONE_0;
                  pMe->m_pToneData[2].wDuration = PLAY_DURATION2;

                  pMe->m_pToneData[3].eTone = AEE_TONE_1;
                  pMe->m_pToneData[3].wDuration = PLAY_DURATION3;

                  // ISOUND_PlayToneList: plays a list of tones, given a tone ID 
                  // for the specified amount of time for each tone.
                  //
                  // Note: the status of this operation is passed to the
                  //       callback registered above. For each tone played 
                  //       the callback function will receive:
                  //         1) AEE_SOUND_STATUS_CB
                  //         2) AEE_SOUND_SUCCESS
                  //       For each tone that completes the callback receives: 
                  //         1) AEE_SOUND_STATUS_CB
                  //         2) AEE_SOUND_PLAY_DONE
                  ISOUND_PlayToneList (pMe->m_pISound, pMe->m_pToneData, pMe->m_nTones);
               } // if ( pMe->m_pToneData )

               // The sound object is not released here so that we give 
               // an opportunity for the PlayToneList operation to finish
               // and the callback function to be called with the status of 
               // the operation.
               //
               // This object will be released when the user presses a key to 
               // move to another menu option or when the user exits the applet.
            }
         }
         break;
      case USAGE_PLAYFREQTONE:
         {
            // 8. Usage: ISOUND_PlayFreqTone
            uint16 wHiFreq = HIGHFREQ;      // Higher frequcy of the DTMF pair
            uint16 wLoFreq = LOWFREQ;       // Lower frequcy of the DTMF pair
            uint16 wDuration = PLAY_DURATION3; // Tone play duration in milliseconds  

            // Create ISound interface object. This step needs to be done only
            // if the ISound interface object have not already been created. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUND, (void **)&pMe->m_pISound); 

            if (pMe->m_pISound)
            {
               // Register a notify function. Results of most of the sound 
               // operations will be sent to the client via the callback
               // function pointer. Also, when the operation is done, it notifies
               // the client through the same callback function.
               ISOUND_RegisterNotify(pMe->m_pISound, &SoundCBFn, (void *)pMe);


               // ISOUND_PlayFreqTone: plays a tone given a high frequecy and 
               // a low frequecy for the specified amount of time.
               // 
               // If the duration is set to 0, the tone will play until the 
               // ISOUND_StopTone call is issued to cancel it.
               //
               // Note: the status of this operation is passed to the
               //       callback registered above. For the tone played 
               //       the callback function will receive:
               //         1) AEE_SOUND_STATUS_CB
               //         2) AEE_SOUND_SUCCESS
               //       When the tone playback is completed, the callback receives: 
               //         1) AEE_SOUND_STATUS_CB
               //         2) AEE_SOUND_PLAY_DONE
               ISOUND_PlayFreqTone (pMe->m_pISound, wHiFreq, wLoFreq, wDuration);

               // The sound object is not released here so that we give 
               // an opportunity for the PlayFreqTone operation to finish
               // and the callback function to be called with the status of 
               // the operation.
               //
               // This object will be released when the user presses a key to 
               // move to another menu option or when the user exits the applet.
            }
         }
         break;      
      case USAGE_STOPTONE:
         {
            // 9. Usage: ISOUND_StopTone
            AEESoundToneData toneData;

            // Create ISound interface object. This step needs to be done only
            // if the ISound interface object have not already been created. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUND, (void **)&pMe->m_pISound); 

            if (pMe->m_pISound)
            {
               // Register a notify function. Results of most of the sound 
               // operations will be sent to the client via the callback
               // function pointer. Also, when the operation is done, it notifies
               // the client through the same callback function.
               ISOUND_RegisterNotify(pMe->m_pISound, &SoundCBFn, (void *)pMe);

               toneData.eTone = AEE_TONE_RING_A;
               toneData.wDuration = PLAY_DURATION3;     // in milliseconds

               // Start playing the tone
               ISOUND_PlayTone (pMe->m_pISound, toneData);

               // Note: ISOUND_StopTone operation is not invoked here since
               //       there will be no audible tone if we Stop the tone 
               //       right after we start playing the tone. The 
               //       ISOUND_StopTone implementation is found in the 
               //       HandleEvents function under EVT_KEY and EVT_STOP
               //       events.
               //
               // The result of the ISOUND_StopTone() will be sent to the client via the
               // callback function pointer. The status can be one of the following values:
               // - AEE_SOUND_SUCCESS: successful and accepted the stop command. If playback of a tone
               //   or tone list is active, then it also generates AEE_SOUND_PLAY_DONE callback
               // - AEE_SOUND_FAILURE: failed and did not accept the stop command

               // This example will play the tone for 5 seconds. The user can 
               // stop the tone by moving to another menu selection by using the 
               // Up/Down arrow keys or when the user exits the applet. At this 
               // time the ISOUND_StopTone is invoked and the ISOUND_Release 
               // function is used to release the sound object.
            }
         }
         break;
      case USAGE_VIBRATE:
      case USAGE_STOPVIBRATE:
         {
            // 10. Usage: ISOUND_Vibrate/ISOUND_StopVibrate
            uint16 w16Duration = PLAY_DURATION3;      // Time in milliseconds

            // Create ISound interface object. This step needs to be done only
            // if the ISound interface object have not already been created. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUND, (void **)&pMe->m_pISound); 

            if (pMe->m_pISound)
            {
               // Register a notify function. Results of most of the sound 
               // operations will be sent to the client via the callback
               // function pointer. Also, when the operation is done, it notifies
               // the client through the same callback function.
               ISOUND_RegisterNotify(pMe->m_pISound, &SoundCBFn, (void *)pMe);

               // ISOUND_Vibrate: causes the handset to vibrate for the specified 
               // amount to time or until ISOUND_StopVibrate() is called.
               // NOTE: ISOUND_Vibrate() and ISOUND_StopVibrate() does not invoke
               // the registered callback functions
               ISOUND_Vibrate(pMe->m_pISound, w16Duration);

               // Note: the ISOUND_StopVibrate function is not invoked here.
               //       it is invoked when the user selects to move to a 
               //       different menu item before the 5 second elapse in
               //       playing the tone or if the user decides to exit the 
               //       applet before the 5 seconds elapse in playing the tone.
               //
               // The sound object is not release at this point.
               // This example will play the tone for 10 seconds till the user 
               // moves to another menu selection by using the Up/Down arrow keys
               // or when the user exits the applet. At this time the 
               // ISOUND_StopVibrate is invoked and the ISOUND_Release function is
               // used to release the sound object.
            }
         }
         break;
      case USAGE_SETVOLUME:
         {
            // 10. Usage: ISOUND_SetVolume
            uint16 wVolume;
            AEESoundToneData toneData;

            // Create ISound interface object. This step needs to be done only
            // if the ISound interface object have not already been created. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUND, (void **)&pMe->m_pISound); 

            if (pMe->m_pISound)
            {
               // Register a notify function. Results of most of the sound 
               // operations will be sent to the client via the callback
               // function pointer. Also, when the operation is done, it notifies
               // the client through the same callback function.
               ISOUND_RegisterNotify(pMe->m_pISound, &SoundCBFn, (void *)pMe);

               toneData.eTone = AEE_TONE_RING_A;
               toneData.wDuration = PLAY_DURATION2;     // in milliseconds

               // Start playing tone
               ISOUND_PlayTone (pMe->m_pISound, toneData);

               // Set the volume to half the maximum volume of the device.
               // Note: The range of volume is from 0 (lowest) to 
               //       AEE_MAX_VOLUME (highest).
               wVolume = AEE_MAX_VOLUME/2;

               // ISOUND_SetVolume: sets the volume to be used for the 
               // device/method pair.
               ISOUND_SetVolume (pMe->m_pISound, wVolume);

               // The result of the above operation will be sent to the client 
               // via the callback function pointer. The status can be one of 
               // the following values:
               // AEE_SOUND_SUCCESS: successful setting the new volume level
               // AEE_SOUND_FAILURE: failed setting the new volume level; the old level still prevails

               // The sound object is not released here so that we give 
               // an opportunity for the PlayTone operation to finish
               // and the callback function to be called with the status of 
               // the operation.
               //
               // This object will be released when the user presses a key to 
               // move to another menu option or when the user exits the applet.
            }
         }
         break;
      case USAGE_GETVOLUME:
         {
            // 11. Usage: ISOUND_GetVolume

            // Create ISound interface object. This step needs to be done only
            // if the ISound interface object have not already been created. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUND, (void **)&pMe->m_pISound); 

            if (pMe->m_pISound)
            {
               // Register a notify function. Results of most of the sound 
               // operations will be sent to the client via the callback
               // function pointer. Also, when the operation is done, it notifies
               // the client through the same callback function.
               ISOUND_RegisterNotify(pMe->m_pISound, &SoundCBFn, (void *)pMe);

               // ISOUND_GetVolume: This function retrieves the current volume 
               // used for the device/method pair. 
               ISOUND_GetVolume (pMe->m_pISound);

               // Note: The volume level will be sent back to the client via 
               //       the callback function pointer. The result includes 
               //       the status as well as the current volume level.
               // - AEE_SOUND_SUCCESS: successful retrieving the current volume level. dwParam contains volume
               // - AEE_SOUND_FAILURE: failed retrieving the current volume level; the volume level will be reset to 0
 
               // In the callback function we first check if the GetVolume function
               // caused the callback to get invoked, and if so, the volume 
               // value returned is printed to screen.
               // 
               // The sound object is not released nor the callback deregistered
               // here so that we give an opportunity for the sound object 
               // operation to finish and the callback function to be called 
               // with the status of the operation and for the current volume 
               // level to be sent.
            }
         }
         break;
      default:
         return;
   }
   
   // Display above event. 
   DisplayEvent (pMe, wParam);

   return;
}

/*===========================================================================

FUNCTION SoundCBFn

DESCRIPTION
	This is the callback function that is registered with ISound interface.
   ISound communicates with the applet via this callback only.

PROTOTYPE:
   void SoundCBFn
   (
      AEE_SOUNDCBIndType  eCBType,
      AEESoundStatus eSPStatus,
      const void *              pClientData,
      void *                    dwParam
   );

PARAMETERS:
   eCBType: Type of ISound callback.
   eSPStatus: Status contained in callback. The status depends on the 
            eCBType.
   pClientData: A reference to correlate this transaction
   dwParam: Additional info specific to callback type and status type

   Refer to BREW Reference Guide for more details.

DEPENDENCIES
  none

RETURN VALUE
  none

SIDE EFFECTS
  none
===========================================================================*/
static void SoundCBFn( void * pUser, AEESoundCmd eCBType, AEESoundStatus eStatus, uint32 dwParam )
{
   CISoundApp* pMe = (CISoundApp*)pUser;
   char szBuf [30] = {0};
   AEESoundCmdData * pData = (AEESoundCmdData *) dwParam;

   if ( pMe == NULL )
      return;

   switch ( eStatus )
   {
      case AEE_SOUND_SUCCESS:
         {
            // If the callback was invoked as a result of the
            // GetVolume function print the volume value on the
            // screen.
            if (eCBType == AEE_SOUND_VOLUME_CB)
            {
               uint16 wVol = 0;
               if (pData)
                  wVol = pData->wVolume;
               SPRINTF (szBuf, "Current Vol: %u", wVol); 
               DisplayOutput (pMe, 2, szBuf);
            }
            else
            {
               DisplayOutput (pMe, 2, "Sound: Successful");
            }  
         }
         break;
      case AEE_SOUND_PLAY_DONE:
         DisplayOutput (pMe, 3, "Sound: Play done");
         break;

      case AEE_SOUND_FAILURE:
         DisplayOutput (pMe, 4, "Sound failure");
         break;

      default:
         DisplayOutput (pMe, -1, "Sound: Unknown status");
         break;

   }
}

/*===========================================================================

FUNCTION: DisplayEvent

DESCRIPTION
    This function is a display function which sets up the initial screen
    for the display output for a given interface function. This displays
    the function name at the top of the screen and displays the
    strings "Press Up/Down" and "keys to continue.." at the bottom of 
    the screen. In the case of functions that have no output it also 
    displays the string "No Output..." in the middile of the screen.

PROTOTYPE:
   void DisplayEvent (CISoundApp *pMe, uint16 wParam)

PARAMETERS:
   pMe:    [in]: Pointer to CISoundApp interface object
   wParam: [in]: API function type Id selected by user

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void DisplayEvent (CISoundApp *pMe, uint16 wParam)
{
   AEEDeviceInfo di;        // Device Info
   AECHAR szBuf[100];
   AEERect rc; 
   int charHeight = 0;      // Stores the char height in pixels for given font
   int pnAscent = 0;        // Stores the ascent in number of pixels
   int pnDescent = 0;       // Stores the descent in number of pixels

   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
      return;

   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);
   rc.x = 0;
   rc.y = 0;
   rc.dx = di.cxScreen;
   rc.dy = di.cyScreen;
   
   if (pMe->a.m_pIDisplay)
   {
      // Get the font metrics info
      charHeight = IDISPLAY_GetFontMetrics (pMe->a.m_pIDisplay, AEE_FONT_NORMAL,
         &pnAscent, &pnDescent);

      switch (wParam)
      {
         case USAGE_BASIC_USAGE:
            STR_TO_WSTR("Basic Usage:", szBuf, sizeof(szBuf));
            break;
         case USAGE_REGISTERNOTIFY:
            STR_TO_WSTR("RegisterNotify", szBuf, sizeof(szBuf));
            break;
         case USAGE_SET:
            STR_TO_WSTR("Set", szBuf, sizeof(szBuf));
            break;
         case USAGE_GET:
            STR_TO_WSTR("Get", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETDEVICE:
            STR_TO_WSTR("SetDevice", szBuf, sizeof(szBuf));
            break;
         case USAGE_PLAYTONE:
            STR_TO_WSTR("PlayTone", szBuf, sizeof(szBuf));
            break;
         case USAGE_PLAYTONELIST:
            STR_TO_WSTR("PlayToneList", szBuf, sizeof(szBuf));
            break;
         case USAGE_PLAYFREQTONE:
            STR_TO_WSTR("PlayFreqTone", szBuf, sizeof(szBuf));
            break;
         case USAGE_STOPTONE:
            STR_TO_WSTR("StopTone", szBuf, sizeof(szBuf));
            break;
         case USAGE_VIBRATE:
            STR_TO_WSTR("Vibrate", szBuf, sizeof(szBuf));
            break;
         case USAGE_STOPVIBRATE:
            STR_TO_WSTR("StopVibrate", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETVOLUME:
            STR_TO_WSTR("SetVolume", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETVOLUME:
            STR_TO_WSTR("GetVolume", szBuf, sizeof(szBuf));
            break;
         default:
            STR_TO_WSTR("Unsupported fn", szBuf, sizeof(szBuf));
      }
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,5, 0, &rc,
		   IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);

      // Functions with no output
      switch (wParam)
      {
         case USAGE_REGISTERNOTIFY:
            // Place in the middle of screen
            STR_TO_WSTR("No Output...", szBuf, sizeof(szBuf));
 		      IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,10, 
               rc.dy/2 - charHeight, &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
         default:
            break;
      }

      STR_TO_WSTR("Press UP/DOWN", szBuf, sizeof(szBuf));
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,5, rc.dy - 
         2*(charHeight) , &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
      STR_TO_WSTR("keys to continue..", szBuf, sizeof(szBuf));
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,5, rc.dy - 
         charHeight, &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
		IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);
   }
}

/*===========================================================================

FUNCTION: BuildMainMenu

DESCRIPTION
    This function builds the main menu when this applet is started.
    It displays a simple menu with menu items corresponding to 
    each of the interface functions.
    
PROTOTYPE:
   void BuildMainMenu(CISoundApp *pMe)

PARAMETERS:
   pMe: [in]: Pointer to AEEApplet struct

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void BuildMainMenu(CISoundApp *pMe)
{
	AEERect qrc;
	AEEDeviceInfo di;
   AECHAR szBuf[100];

   if (pMe == NULL || pMe->m_pIMenu == NULL || pMe->a.m_pIShell == NULL)
      return;

	// Set Title
   STR_TO_WSTR("ISound Examples", szBuf, sizeof(szBuf));
	IMENUCTL_SetTitle(pMe->m_pIMenu, NULL, 0, szBuf);

	//Set the Rectangle for the Menu
	ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);
	qrc.x = 0;
	qrc.y = 0;
	qrc.dx = di.cxScreen;
	qrc.dy = di.cyScreen;
	IMENUCTL_SetRect(pMe->m_pIMenu, &qrc);  

	//Add invidual entries to the Menu

   // Add Basic Usage to menu
   STR_TO_WSTR("1. Basic Usage", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_BASIC_USAGE, szBuf, 0);

   // Add RegisterNotify to menu
   STR_TO_WSTR("2. RegisterNotify", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_REGISTERNOTIFY, szBuf, 0);

   // Add Set to menu
   STR_TO_WSTR("3. Set", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SET, szBuf, 0);

   // Add Get to menu
   STR_TO_WSTR("4. Get", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GET, szBuf, 0);

   // Add Set Device to menu
   STR_TO_WSTR("5. SetDevice", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETDEVICE, szBuf, 0);

   // Add Play Tone to menu
   STR_TO_WSTR("6. PlayTone", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_PLAYTONE, szBuf, 0);

   // Add Play Tone List to menu
   STR_TO_WSTR("7 PlayToneList", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_PLAYTONELIST, szBuf, 0);

   // Add Play Freq Tone to menu
   STR_TO_WSTR("8. PlayFreqTone", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_PLAYFREQTONE, szBuf, 0);

   // Add Stop Tone to menu
   STR_TO_WSTR("9. StopTone", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_STOPTONE, szBuf, 0);

	// Add Vibrate to menu
   STR_TO_WSTR("10. Vibrate", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_VIBRATE, szBuf, 0);

	// Add Stop Vibrate to menu
   STR_TO_WSTR("11. StopVibrate", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_STOPVIBRATE, szBuf, 0);	

	// Add Set Volume to menu
   STR_TO_WSTR("12. SetVolume", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETVOLUME, szBuf, 0);	

	// Add Get Volume to menu
   STR_TO_WSTR("13. GetVolume", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETVOLUME, szBuf, 0);	

   // Activate menu
	IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);
}

/*===========================================================================

FUNCTION: DisplayOutput

DESCRIPTION
    This function displays an output string at a given line number on the
    screen. If the nline parameter is a negative value (-1) the string
    is displayed in the middle of the screen. If the "nline" value is larger
    than or equal to zero the "nline" value is multiplied by 15 and the 
    resulting value in pixels is set to the y-coordinate of the start of 
    the string display on the screen. If the string does not fit on one line
    the string wraps around to the nime line (spaced rougly 10-15 pixels apart).
    By default 5 is used as the starting the x-coordinate of a displayed 
    string.

    How many characters that fit on one line is calculated for each line 
    that is wrapped around to the nime line.

    Note: depending on the phone screen size and the fonts used for characters 
          the output might differ on different handsets (devices). Where some 
          handsets will have a smaller screen and large default fonts which will 
          cause partial overlapping of lines. This function does not try to address
          these issues (this is meant as a simple display function).
    
PROTOTYPE:
   void DisplayOutput(CISoundApp * pi, int nline, char *pszStr)

PARAMETERS:
   pi:     [in]: Contains a pointer to the CISoundApp class.
   nline:  [in]: Contains the line number to start displaying the time. 
        The line numbers are by default spaced 15 pixels apart along the y-axis.
   pszStr: [in]: The character string to be displayed on the screen.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void DisplayOutput(CISoundApp * pMe, int nline, char *pszStr)
{
   AEEDeviceInfo di;        // Device Info
   AECHAR * pszBuf = NULL;
   AECHAR * psz = NULL;
   int pixelWidth;
   AEEFont font = AEE_FONT_NORMAL;
   int pnFits = 0, dy;
   int totalCh = 0;
   int charHeight = 0;      // Stores the char height in pixels for given font
   int pnAscent = 0;        // Stores the ascent in number of pixels
   int pnDescent = 0;       // Stores the descent in number of pixels

   if (pMe == NULL)
      return;

   if ((pszBuf = (AECHAR *)MALLOC(200)) == NULL)
      return;

   // Get device information
   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);

   // Get the font metrics info
   charHeight = IDISPLAY_GetFontMetrics (pMe->a.m_pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   // Convert to wide string (unicode)
   STR_TO_WSTR ((char *)pszStr, pszBuf, 200);

   // If nlines is zero then print this string starting around the middle of 
   // the screen. Or else multiply nlines by 10 to decide the y coordinate of
   // the start of the string.
   if (nline < 0) {
      dy = di.cyScreen*2/5;
   }
   else{
      dy = nline * charHeight;
   }

   // psz keeps track of the point from which to write from the string buffer
   // in case the string does not fit one line and needs to wrap around in the
   // next line.
   psz = pszBuf;
      
   // Need to calculate the lotal string length to decide if any wrapping
   // around is needed.
   totalCh = STRLEN ((char *)pszStr);

   // Keep displaying text string on multiple lines if the string can't be displayed
   // on one single line. Lines are spaced 15 pixels apart.
   while ((totalCh > 0) && (*psz != NULL))
   { 
      // Get information on how many characters will fit in a line.
      // Give the pointer to the buffer to be displayed, and the number of
      // pixels along the x axis you want to display the string in (max number)
      // pnFits will have the max number of chars that will fit in the maxWidth
      // number of pixels (given string can't fit in one line), or the number of 
      // chars in the string (if it does fit in one line). pnWidth gives the
      // number of pixels that will be used to display pnFits number of chars.
      pixelWidth = IDISPLAY_MeasureTextEx(pMe->a.m_pIDisplay,
                      font, 
                      (AECHAR *) psz,  // Start of the buffer to display,
                      -1,
                      di.cxScreen - 5, // maxWidth
                      &pnFits);         // Number of chars that will fit a line

      // If pnFits is zero there is something wrong in the input to above function. 
      // Normally this scenario should not occur. But, have the check anyway.
      if (pnFits == 0)
      {
         FREE (pszBuf);
         return;
      }

      IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, psz, pnFits, 5, dy, 0, 0);

      psz += pnFits;      // move pointer to the next segment to be displayed
      totalCh -= pnFits;  // reduce the total number of characters to still display
      dy += charHeight;   // Place next line charHeight pixels below the 
                          // previous line. 
      IDISPLAY_Update(pMe->a.m_pIDisplay);
      if (totalCh < pnFits)
         pnFits = totalCh;  // if total number is less than pnFits, adjust pnFits
   }

   FREE (pszBuf);
   return;   
} // End of DisplayOutput

