/*===========================================================================

FILE: isoundplayer.c

SERVICES: Sample applet using AEE

DESCRIPTION
  This file contains usage examples of ISoundPlayer interface. 
  ISoundPlayer interface functions allow the user to create/operate 
  on ISoundPlayer objects. ISoundPlayer interface offers services
  play and operate on MP3 and MIDI sound files.

PUBLIC CLASSES:  
   N/A


   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/

/*===============================================================================
                     INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "AEEModGen.h"
#include "AEEAppGen.h"
#include "AEEMenu.h"
#include "AEEStdLib.h"
#include "AEEUsageAppIDs.h"
#include "AEESound.h"
#include "AEESoundPlayer.h"
#include "AEEFile.h"

/*===========================================================================

                      PUBLIC DATA DECLARATIONS

===========================================================================*/
/*-------------------------------------------------------------------
            Global Constant Definitions
-------------------------------------------------------------------*/

// App specific constants
#define USAGE_BASIC_USAGE        100
#define USAGE_REGISTERNOTIFY     101
#define USAGE_SET                102
#define USAGE_PLAY               103
#define USAGE_STOP               104
#define USAGE_REWIND             105
#define USAGE_FASTFORWARD        106
#define USAGE_PAUSE              107
#define USAGE_RESUME             108
#define USAGE_SETTEMPO           109
#define USAGE_SETTUNE            110
#define USAGE_GETTOTALTIME       111
#define USAGE_SETSOUNDDEVICE     112
#define USAGE_SETVOLUME          113
#define USAGE_GETVOLUME          114
#define USAGE_PLAYER_MENU        115
#define USAGE_EXIT_PLAYER_MENU   116
#define USAGE_CHANGE_SONG        117

#define MIDI_MAX_FILES  10
#define TIME_INMILLISECONDS      5000  // 5000 ms
#define TEMPFACTOR               300

#ifndef DIRECTORY_CHAR
#ifdef AEE_SIMULATOR
#define DIRECTORY_CHAR  '\\'
#define DIRECTORY_STR   "\\"
#else
#define DIRECTORY_CHAR  '/'
#define DIRECTORY_STR   "/"
#endif
#endif 


/*-------------------------------------------------------------------
            Type Declarations
-------------------------------------------------------------------*/
// SoundPlayer app struct. This is the main struct for this applet. This will hold 
// all the data members that needs to be remembered throughout the life of
// the applet. 
// THE FIRST DATA MEMBER OF THIS STRUCT MUST BE AN AEEApplet OBJECT. 
// This is critical requirement which is related to an applet being
// dynamically loaded. The app developers should add their data members
// following the AEEApplet data member.
typedef struct _CISoundPlayerApp
{
	AEEApplet	    a;   // Mandatory first AEEApplet data member
   IMenuCtl     *  m_pIMenu;

   // SoundPlayer members 
   IMenuCtl     *  m_pPlayerMenu;
   ISoundPlayer *  m_pISoundPlayer;
   char         *  m_szFileName;
   boolean         m_bSPActive;
   uint32          m_nPlayTime;
   // Device parameters
   int             m_nLineHeight;
   AEEDeviceInfo   m_deviceInfo;

}CISoundPlayerApp;

/*-------------------------------------------------------------------
            Function Prototypes
-------------------------------------------------------------------*/
// App Handle Event function
static boolean SoundPlayerApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);

// App specific data alloc-init/free functions
static boolean ISoundPlayer_InitAppData(IApplet* pMe);
static void ISoundPlayer_FreeAppData(IApplet* pMe);

static void BuildMainMenu(CISoundPlayerApp *pMe);
static void ISoundPlayerUsage (CISoundPlayerApp * pMe, uint16 wParam);
static void DisplayEvent (CISoundPlayerApp *pMe, uint16 wParam);
static void DisplayOutput(CISoundPlayerApp * pMe, int nline, char *pszStr);
static void SoundPlayerCBFn( void * pUser, AEESoundPlayerCmd eCBType, AEESoundPlayerStatus eStatus, uint32 dwParam );
static void BuildPlayerMenu(CISoundPlayerApp *pMe);
static void CleanupSP_RedrawMainMenu (CISoundPlayerApp * pMe);

/*===============================================================================
                     FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to ISoundPlayer_InitAppData(). 
   This is done as follows: InitAppData() is called to initialize AppletData 
   instance. It is app developers responsibility to fill-in app data initialization 
   code of InitAppData(). App developer is also responsible to release memory 
   allocated for data contained in AppletData -- this can be done in 
   ISoundPlayer_FreeAppData().

PROTOTYPE:
   int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell object. 

	pIModule: pin]: Contains pointer to the IModule object to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid IApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the app needs to be loaded and if AEEApplet_New() invocation was
     successful
  EFAILED: If the app does not need to be loaded or if errors occurred in 
     AEEApplet_New(). If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
{
   *ppObj = NULL;
		
   // We want to load this App. So, invoke AEEApplet_New().To it, pass the
   // address of the app-specific handle event function.
   if(ClsId == AEECLSID_SOUNDPLAYER_APP){
      if(AEEApplet_New(sizeof(CISoundPlayerApp), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)SoundPlayerApp_HandleEvent,(PFNFREEAPPDATA)ISoundPlayer_FreeAppData)
         == TRUE)
      {
         if (ISoundPlayer_InitAppData((IApplet*)*ppObj) == TRUE)
         {
            return(AEE_SUCCESS);
         }
      }
   }
	return (EFAILED);
}

/*===========================================================================

FUNCTION SoundPlayerApp_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean SoundPlayerApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

	ecode: Specifies the Event sent to this applet

   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/
static boolean SoundPlayerApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
   CISoundPlayerApp * pMe = (CISoundPlayerApp*)pi;
   
   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
      return FALSE;

   switch (eCode) 
	{
       case EVT_APP_START:
         // Create the IMenu interface object. When the applet is started the 
         // main menu is set up. When the user selects a menu item from the main 
         // menu, an EVT_COMMAND event is received by this applet. The processing for 
         // EVT_COMMAND event is found below.
		   if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, (void **)&pMe->m_pIMenu)
            != SUCCESS)
         {
				return FALSE;
         }

         // Build the main menu
			BuildMainMenu(pMe);
			return(TRUE);
      case EVT_APP_STOP:
         return(TRUE);
      case EVT_KEY:
         // We have come here because the user has pressed a key.
         // Check if the sound player object is not NULL and if the
         // sound player is not playing. Hence, we should release the
         // the soundplayer object here (a new sound player object is 
         // created in each sound-player usage example).
         if ((pMe->m_pISoundPlayer != NULL) && (pMe->m_bSPActive != TRUE))
         {
            ISOUNDPLAYER_RegisterNotify (pMe->m_pISoundPlayer, NULL, NULL);
            // Stop any songs being played at this time
            ISOUNDPLAYER_Stop (pMe->m_pISoundPlayer);
            ISOUNDPLAYER_Release (pMe->m_pISoundPlayer);
            pMe->m_pISoundPlayer = NULL;
            // Also set the m_nPlayTime to zero in case the user exited while 
            // a song is being played
            pMe->m_nPlayTime = 0;
         }

         // First check if the player-menu is active. Also, check if the
         // player menu object is not NULL. First give the player-menu to 
         // handle the event. If it does get handled then we return.
         // Or else go and check if the pressed key is clear. If the pressed
         // key is Clear, then we need to exit the player menu and redraw the
         // main menu. Call CleanupSP_RedrawMainMenu function to clean up
         // the player menu and redraw the main menu of the applet.
         if ((pMe->m_bSPActive == TRUE) && pMe->m_pPlayerMenu)
         {
            if (IMENUCTL_HandleEvent(pMe->m_pPlayerMenu, EVT_KEY, wParam, 0))
            {
               return TRUE;
            }

            // This means that the a key not handled by the player-menu has 
            // been pressed. We should ignore it unless it is the Clear key,
            // in which case move up to the previous menu (clean up all the 
            // resources before that).
            if (wParam == AVK_CLR)
               CleanupSP_RedrawMainMenu (pMe);
            return TRUE;
         }

         // We are here because of user key press, soundplayer is inactive, 
         // and the ISoundPlayer object has been cleanup (CB deregister, song
         // stopped, interface released, etc.).
         // 
         // The key is probably the Up/Down keys. In this case we should 
         // call the IMenuCtl object so that the Up/Down key presses are
         // handled by the MenuCtl.
         if((pMe->m_pIMenu != NULL) && IMENUCTL_HandleEvent(pMe->m_pIMenu, 
            EVT_KEY, wParam, 0))
         {
				return TRUE;
         }
         else
         {
            // We would have come here because the pressed key was not handled 
            // by either the SoundPlayer or the MenuCtl HandleEvent functions (the 
            // keys handled by the above controls are Up/Down/Select/Left/Right).
            //
            // Hence, we have reached here since 0-9, *, or # keys were
            // pressed. So redraw the main applet menu and return true.
            if (pMe->m_pIMenu)
               IMENUCTL_Redraw (pMe->m_pIMenu);
         }
         return TRUE;
		case EVT_COMMAND:
         // When a menu item is selected this applet receives the
         // EVT_COMMAND event in the wParam field.
  		   switch(wParam)
         {
            case USAGE_BASIC_USAGE:
            case USAGE_REGISTERNOTIFY:
            case USAGE_SET:
            case USAGE_PLAY:
            case USAGE_STOP:
            case USAGE_REWIND:
            case USAGE_FASTFORWARD:
            case USAGE_PAUSE:
            case USAGE_RESUME:
            case USAGE_SETTEMPO:
            case USAGE_SETTUNE:
            case USAGE_GETTOTALTIME:
            case USAGE_SETSOUNDDEVICE:
            case USAGE_SETVOLUME:
            case USAGE_GETVOLUME:
               // Call the usage function that exercises all the 
               // interface functions.
               ISoundPlayerUsage (pMe, wParam);
               return TRUE;
            case USAGE_PLAYER_MENU:
               BuildPlayerMenu (pMe);
               return TRUE;
            case USAGE_EXIT_PLAYER_MENU:
               CleanupSP_RedrawMainMenu (pMe);
               return TRUE;
		   	default:
			   	return FALSE;
         }
      default:
         break;
   }
   return FALSE;
}



/*===========================================================================

FUNCTION ISoundPlayer_InitAppData

DESCRIPTION
   This function initializes app specific data allocates memory for app data 
   (AppletData) and sets it to pAppData of AEEApplet. It also function . 
   App developer needs to fill-in the initialization code.

PROTOTYPE:
	boolean ISoundPlayer_InitAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains 
   information specific to this applet. It was initialized during the 
   AEEClsCreateInstance().

DEPENDENCIES
   Assumes pi is not NULL

RETURN VALUE
  TRUE: If the app has app data is allocated and initialized successfully
  FALSE: Either app data could not be allocated or initialzied

SIDE EFFECTS
  none
===========================================================================*/
static boolean ISoundPlayer_InitAppData(IApplet* pi)
{
   IFileMgr *  pIFileMgr = NULL;
   FileInfo    fi;
   int pnAscent = 0;        // Stores the ascent in number of pixels
   int pnDescent = 0;       // Stores the descent in number of pixels
   char *   szStart;

   CISoundPlayerApp * pMe = (CISoundPlayerApp*)pi;

   if (!pMe)
      return FALSE;

   // Initialize the MenuCtl and SoundPlayer pointers
   pMe->m_pIMenu = NULL;
   pMe->m_pPlayerMenu = NULL;
   pMe->m_pISoundPlayer = NULL;
   pMe->m_nPlayTime = 0;
   pMe->m_bSPActive = FALSE;
   pMe->m_szFileName = NULL;

   // Get the font metrics info
   pMe->m_nLineHeight = IDISPLAY_GetFontMetrics (pMe->a.m_pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&pMe->m_deviceInfo);

   // Enumerate all music files in the Music directory and add their names
   // to the FileNameArray. See IFile and IFileMgr interface usage applet 
   // for more details on how to use the File related functions.
   if ( ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR, (void **)&pIFileMgr) )
      return FALSE;

   // Initialize the file pointer to the first file in the "Music" directory.
   IFILEMGR_EnumInit(pIFileMgr, "Music", FALSE);

   IFILEMGR_EnumNext(pIFileMgr, &fi);

   // Remove duplicate app name in the path
   szStart = fi.szName; // STRCHR(fi.szName, (int)DIRECTORY_CHAR) + 1;

   if ( szStart )
   {
      pMe->m_szFileName = MALLOC(STRLEN(szStart) + 1);
      STRCPY(pMe->m_szFileName, szStart);
   }
   IFILEMGR_Release(pIFileMgr); 

   return TRUE;
}

/*===========================================================================

FUNCTION ISoundPlayer_FreeAppData

DESCRIPTION
   Frees data contained in app data and memory for app data itself.
   App developer needs to free data contained in AppletData.

PROTOTYPE:
	void ISoundPlayer_FreeAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains 
   information specific to this applet. It was initialized during the 
   AEEClsCreateInstance().

DEPENDENCIES
   Assumes pi is not NULL

RETURN VALUE
   None

SIDE EFFECTS
   None
===========================================================================*/
static void ISoundPlayer_FreeAppData(IApplet* pi)
{
//   int i = 0;
   CISoundPlayerApp * pMe = (CISoundPlayerApp*)pi;

   // Release main menu
   if (pMe->m_pIMenu != NULL)
   {
      IMENUCTL_Release (pMe->m_pIMenu);
      pMe->m_pIMenu = NULL;
   }

   // Release the player menu
   if (pMe->m_pPlayerMenu != NULL)
   {
      IMENUCTL_Release (pMe->m_pPlayerMenu);
      pMe->m_pPlayerMenu = NULL;
   }

   // Release SoundPlayer object if it is not released
   if (pMe->m_pISoundPlayer != NULL)
   {
      // Deregister Notify so that 
      ISOUNDPLAYER_RegisterNotify (pMe->m_pISoundPlayer, NULL, NULL);
      // Stop any songs being played at this time
      ISOUNDPLAYER_Stop (pMe->m_pISoundPlayer);
      ISOUNDPLAYER_Release (pMe->m_pISoundPlayer);
      pMe->m_pISoundPlayer = NULL;
   }

   if (pMe->m_szFileName)
      FREE (pMe->m_szFileName);
}

/*===============================================================================
                     APP HELPER FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: ISoundPlayerUsage

DESCRIPTION
	This function encompasses all the usage examples of all the functions
   in code blocks switched using the BREW API function Id passed into this
   function. The code blocks will include almost all the variable declarations
   needed to demonstrate the usage of a given function. Within the code block
   all the needed interfaces are instantiated and released subsequent to their
   use.

   There are some exceptions to the above general rule, such as call-back functions,
   common helper function, or common display functions called from within the
   code block which are defined elsewhere in this file.

   There are some instances where an object instantiated within a code block is
   not released at the end of the code block, to be released at a later point
   (for example in a call-back function, in this case pointers to such objects
   are stored in the AEEApplet object to be released later).
   
PROTOTYPE:
   void ISoundPlayerUsage (CISoundPlayerApp * pMe, uint16 wParam)

PARAMETERS:
	pMe: [in]: Contains a pointer to the CISoundPlayerApp struct.
   wParam: [in]: Contains the Id of the function type to be used (contains the
      Id corresponding to the menu item selected on the emulator).

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the functions call was successful. FALSE otherwise

SIDE EFFECTS
  none
===========================================================================*/
static void ISoundPlayerUsage (CISoundPlayerApp * pMe, uint16 wParam)
{
   AEEDeviceInfo *di = NULL;

   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
      return;

   di = &pMe->m_deviceInfo;

   // Erase screen first for display output purposes.
   IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);

   switch (wParam)
   {
      case USAGE_BASIC_USAGE:
         {
            // 1. Usage: Basic ISoundPlayer Usage
            AEESoundPlayerInput input;

            // Create ISoundPlayer interface object. This step needs to be done only
            // if the ISoundPlayer interface object have not already been created. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUNDPLAYER, (void **)&pMe->m_pISoundPlayer); 

            if (pMe->m_pISoundPlayer == NULL)
               return;

            // Note: each of the functions invoked below are not described in detail
            //       Please see the corresponding usage section for more details.

            // Register a notify function.
            ISOUNDPLAYER_RegisterNotify(pMe->m_pISoundPlayer, &SoundPlayerCBFn, pMe);

            // Set input
            input = SDT_FILE;

            // Set the first file in the array.
            ISOUNDPLAYER_Set(pMe->m_pISoundPlayer, input, pMe->m_szFileName);
               
            // Build main player menu. This will set up a menu from which 
            // the user can pick to play/stop/rewind/fast forward/pause/resume 
            // a song. The corresponding source code for each of the functions
            // in this menu are given below in the individual usage sections.
            BuildPlayerMenu (pMe);

            // The callback function is not deregistered here nor is
            // the sound player object released here. This is to allow
            // for the sound file to play and for the user to hear it.
            // 
            // The soundplayer object IS RELEASED when the user exits 
            // soundplayer menu and return to the main applet menu, or
            // when the user presses the CLR or END keys. See under event 
            // EVT_KEY in the HandleEvent function to see the release.
         }
         return;
      case USAGE_REGISTERNOTIFY:
         {
            // 2. Usage: ISOUNDPLAYER_RegisterNotify
            ISoundPlayer * pISoundPlayer = NULL;

            // Create ISoundPlayer interface object. This step needs to be done only
            // if the ISoundPlayer interface object have not already been created. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUNDPLAYER, (void **)&pISoundPlayer); 

            if (pISoundPlayer)
            {
               // Register the status callback funtion using 
               // ISOUNDPLAYER_RegisterNotify(). ISoundPlayer always communicates 
               // with applets through this callback function.
               ISOUNDPLAYER_RegisterNotify(pISoundPlayer, &SoundPlayerCBFn, (void *)pMe);

               // Cancel the callback notification
               ISOUNDPLAYER_RegisterNotify(pISoundPlayer, NULL, NULL);
               // Release the sound object
               ISOUNDPLAYER_Release (pISoundPlayer);
            }
         }
         break;
      case USAGE_SET:
         {
            // 3. Usage: ISOUNDPLAYER_Set
            ISoundPlayer * pISoundPlayer = NULL;
            AEESoundPlayerInput input = SDT_FILE;

            // Create ISoundPlayer interface object. This step needs to be done only
            // if the ISoundPlayer interface object have not already been created. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUNDPLAYER, (void **)&pISoundPlayer); 

            if (pISoundPlayer)
            {
               // Register the status callback funtion.
               ISOUNDPLAYER_RegisterNotify(pISoundPlayer, &SoundPlayerCBFn, (void *)pMe);


               // ISOUNDPLAYER_Set: sets the source of the MIDI/MP3 audio 
               // (File, buffer, or MIME buffer).
               //
               // Set the first file in the array.
               ISOUNDPLAYER_Set(pISoundPlayer, input, pMe->m_szFileName);

               // Cancel the callback notification
               ISOUNDPLAYER_RegisterNotify(pISoundPlayer, NULL, NULL);
               // Release the sound object
               ISOUNDPLAYER_Release (pISoundPlayer);
            }
         }
         break;
      case USAGE_PLAY:
         {
            // 4. Usage: ISOUNDPLAYER_Play

            // We need to check to see if the pMe->m_pISoundPlayer is already created.
            // This is possible if we are in the Sound-Player menu and we have invoked 
            // another function before calling this one.
            if (pMe->m_pISoundPlayer == NULL)
            {
               // Create ISoundPlayer interface object. This step needs to be done only
               // if the ISoundPlayer interface object have not already been created. 
               ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUNDPLAYER, (void **)&pMe->m_pISoundPlayer); 

               if (pMe->m_pISoundPlayer)
               {
                  // Register the status callback funtion.
                  ISOUNDPLAYER_RegisterNotify(pMe->m_pISoundPlayer, &SoundPlayerCBFn, (void *)pMe);
                  ISOUNDPLAYER_Set(pMe->m_pISoundPlayer, SDT_FILE, pMe->m_szFileName);
               }
               else
               {
                  return;
               }
            }

            // ISOUNDPLAYER_Play: issues a command to play a MIDI/MP3 audio 
            // from input source.
            ISOUNDPLAYER_Play (pMe->m_pISoundPlayer);

            // The callback function is not deregistered here nor is 
            // the sound player object released here. This is to allow
            // for the sound file to play and for the user to hear it. 
            // When the user decides to move to another menu selection 
            // or exit the applet the callback function is deregistered
            // and the sound player object is released.

            // Redraw the player menu
            if (pMe->m_pPlayerMenu)
               IMENUCTL_Redraw(pMe->m_pPlayerMenu);
         }
         return;
      case USAGE_STOP:
         {
            // 5. Usage: ISOUNDPLAYER_Stop

            // It's possible that the m_pISoundPlayer pointer is NULL. This
            // will occur when we enter the sound-player menu and invoke 
            // the STOP option while a song is not playing.
            if (pMe->m_pISoundPlayer && pMe->m_bSPActive)
            {
               // ISOUNDPLAYER_Stop: issues a command to stop the current 
               // MIDI/MP3 playback.
               ISOUNDPLAYER_Stop (pMe->m_pISoundPlayer);
            }

            // Redraw the player menu
            if (pMe->m_pPlayerMenu)
               IMENUCTL_Redraw(pMe->m_pPlayerMenu);
         }
         return;
      case USAGE_REWIND:
         {
            // 6. Usage: ISOUNDPLAYER_Rewind
            uint32 dwTime;

            // It's possible that the m_pISoundPlayer pointer is NULL. This
            // will occur when we enter the sound-player menu and invoke 
            // the REWIND option while a song is not playing.
            if (pMe->m_pISoundPlayer && pMe->m_bSPActive)
            {
               // Rewind 2 seconds of playback
               dwTime = TIME_INMILLISECONDS;   // in milliseconds

               // ISOUNDPLAYER_Rewind: issues a command to 
               // rewind the current MIDI/MP3 playback.
               ISOUNDPLAYER_Rewind (pMe->m_pISoundPlayer, dwTime);
            }

            // Redraw the player menu
            if (pMe->m_pPlayerMenu)
               IMENUCTL_Redraw(pMe->m_pPlayerMenu);
         }
         return;
      case USAGE_FASTFORWARD:
         {
            // 7. Usage: ISOUNDPLAYER_FastForward
            uint32 dwTime;

            // It's possible that the m_pISoundPlayer pointer is NULL. This
            // will occur when we enter the sound-player menu and invoke 
            // the FastFoward option while a song is not playing.
            if (pMe->m_pISoundPlayer && pMe->m_bSPActive)
            {
               // Fast forward 2 seconds of playback
               dwTime = TIME_INMILLISECONDS;   // in milliseconds

               // ISOUNDPLAYER_FastForward: issues a command to 
               // fast-forward the current MIDI/MP3 playback.
               ISOUNDPLAYER_FastForward (pMe->m_pISoundPlayer, dwTime);
            }

            // Redraw the player menu
            if (pMe->m_pPlayerMenu)
               IMENUCTL_Redraw(pMe->m_pPlayerMenu);
         }
         return;
      case USAGE_PAUSE:
         {
            // 8. Usage: ISOUNDPLAYER_Pause

            // It is possible that the m_pISoundPlayer pointer is NULL. This
            // will occur when we enter the sound-player menu and invoke 
            // the PAUSE option while a song is not playing.
            if (pMe->m_pISoundPlayer && pMe->m_bSPActive)
            {
               // ISOUNDPLAYER_Pause: issues a command to 
               // pause the current MIDI/MP3 playback.
               ISOUNDPLAYER_Pause (pMe->m_pISoundPlayer);
            }

            // Redraw the player menu
            if (pMe->m_pPlayerMenu)
               IMENUCTL_Redraw(pMe->m_pPlayerMenu);
         }
         return;
      case USAGE_RESUME:
         {
            // 9. Usage: ISOUNDPLAYER_Resume

            // It is possible that the m_pISoundPlayer pointer is NULL. This
            // will occur when we enter the sound-player menu and invoke 
            // the RESUME option while a song is not playing.
            if (pMe->m_pISoundPlayer && pMe->m_bSPActive)
            {
               // ISOUNDPLAYER_Resume: issues a command to 
               // resume the current MIDI/MP3 playback.
               ISOUNDPLAYER_Resume (pMe->m_pISoundPlayer);
            }

            // Redraw the player menu
            if (pMe->m_pPlayerMenu)
               IMENUCTL_Redraw(pMe->m_pPlayerMenu);
         }
         return;
      case USAGE_SETTEMPO:
         {
            // 10. Usage: ISOUNDPLAYER_SetTempo
            uint32 dwTempoFactor;

            // Create ISoundPlayer interface object. This step needs to be done only
            // if the ISoundPlayer interface object have not already been created. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUNDPLAYER, (void **)&pMe->m_pISoundPlayer); 

            if (pMe->m_pISoundPlayer == NULL)
               return;

            // Register the status callback funtion.
            ISOUNDPLAYER_RegisterNotify(pMe->m_pISoundPlayer, &SoundPlayerCBFn, (void *)pMe);
            ISOUNDPLAYER_Set(pMe->m_pISoundPlayer, SDT_FILE, pMe->m_szFileName);

            // Set the tempo 
            dwTempoFactor = TEMPFACTOR;

            // ISOUNDPLAYER_SetTemp: issues a command to modify the playback 
            // tempo of the current MIDI/MP3 playback as a percentage of 
            // the normal playback tempo.
            ISOUNDPLAYER_SetTempo (pMe->m_pISoundPlayer, dwTempoFactor);

            // Play a sound
            ISOUNDPLAYER_Play (pMe->m_pISoundPlayer);

            // The callback function is not deregistered here nor is 
            // the sound player object released here. This is to allow
            // for the sound file to play and for the user to hear it. 
            // When the user decides to move to another menu selection 
            // or exit the applet the callback function is deregistered
            // and the sound player object is released.
         }
         break;
      case USAGE_SETTUNE:
         {
            // 11. Usage: ISOUNDPLAYER_SetTune
            uint8 nStep;

            // Create ISoundPlayer interface object. This step needs to be done only
            // if the ISoundPlayer interface object have not already been created. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUNDPLAYER, (void **)&pMe->m_pISoundPlayer); 

            if (pMe->m_pISoundPlayer == NULL)
               return;

            // Register the status callback funtion.
            ISOUNDPLAYER_RegisterNotify(pMe->m_pISoundPlayer, &SoundPlayerCBFn, (void *)pMe);
            ISOUNDPLAYER_Set(pMe->m_pISoundPlayer, SDT_FILE, pMe->m_szFileName);

            // Set the tempo 
            nStep = 12;

            // ISOUNDPLAYER_SetTune: issues a command to modify the tune 
            // of the current MIDI/MP3 playback in half-step increments.
            ISOUNDPLAYER_SetTune (pMe->m_pISoundPlayer, nStep);

            // Play a sound
            ISOUNDPLAYER_Play (pMe->m_pISoundPlayer);

            // The callback function is not deregistered here nor is 
            // the sound player object released here. This is to allow
            // for the sound file to play and for the user to hear it. 
            // When the user decides to move to another menu selection 
            // or exit the applet the callback function is deregistered
            // and the sound player object is released.
         }
         break;
      case USAGE_GETTOTALTIME:
         {
            // 12. Usage: ISOUNDPLAYER_GetTotalTime

            // Create ISoundPlayer interface object. This step needs to be done only
            // if the ISoundPlayer interface object have not already been created. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUNDPLAYER, (void **)&pMe->m_pISoundPlayer); 

            if (pMe->m_pISoundPlayer == NULL)
               return;

            // Register the status callback funtion.
            ISOUNDPLAYER_RegisterNotify(pMe->m_pISoundPlayer, &SoundPlayerCBFn, (void *)pMe);
            ISOUNDPLAYER_Set(pMe->m_pISoundPlayer, SDT_FILE, pMe->m_szFileName);

            // ISOUNDPLAYER_GetTotalTime: issues a command to request 
            // the calculation of the total playback time of the 
            // specified MP3/MIDI input source.
            ISOUNDPLAYER_GetTotalTime (pMe->m_pISoundPlayer);

            // The output of the above function will go to the callback function.
            // See the callback function for the output processing for this function.
            // 
            // Note: Only one play or get-total-time command execuate at a time; 
            //       a play command will override any previous play or get-total-time 
            //       command. However, a get-total-time command will not interrupt a 
            //       play command.

            // The callback function is not deregistered here nor is 
            // the sound player object released here. This is to allow
            // for the sound file to play and for the user to hear it. 
            // When the user decides to move to another menu selection 
            // or exit the applet the callback function is deregistered
            // and the sound player object is released.
         }
         break;
      case USAGE_SETSOUNDDEVICE:
         {
            // 13. Usage: ISOUNDPLAYER_SetSoundDevice
            // The types defined below are in AEESound.h file.
            AEESoundDevice eDevice;
            AEESoundMuteCtl eEarMute;
            AEESoundMuteCtl eMicMute;

            // Create ISoundPlayer interface object. This step needs to be done only
            // if the ISoundPlayer interface object have not already been created. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUNDPLAYER, (void **)&pMe->m_pISoundPlayer); 

            if (pMe->m_pISoundPlayer == NULL)
               return;

            // Register the status callback funtion.
            ISOUNDPLAYER_RegisterNotify(pMe->m_pISoundPlayer, &SoundPlayerCBFn, (void *)pMe);
            ISOUNDPLAYER_Set(pMe->m_pISoundPlayer, SDT_FILE, pMe->m_szFileName);

            // Set the sound device, ear and micraphone mute control 
            eDevice = AEE_SOUND_DEVICE_CURRENT;
            eEarMute = AEE_SOUND_MUTECTL_UNMUTED;
            eMicMute = AEE_SOUND_MUTECTL_UNMUTED;

            // ISOUNDPLAYER_SetSoundDevice: commands the handset to connect to a 
            // specific audio device; there is always one, and only one, audio 
            // device connected at any particular time.
            ISOUNDPLAYER_SetSoundDevice (pMe->m_pISoundPlayer, eDevice, eEarMute, eMicMute);

            // Play a song
            //ISOUNDPLAYER_Play (pMe->m_pISoundPlayer);

            // The output of the above function will go to the callback function.
            // See the callback function for the output processing for this function.
            //
            // The callback function is not deregistered here nor is 
            // the sound player object released here. This is to allow
            // for the sound file to play and for the user to hear it. 
            // When the user decides to move to another menu selection 
            // or exit the applet the callback function is deregistered
            // and the sound player object is released.
         }
         break;
      case USAGE_SETVOLUME:
         {
            // 14. Usage: ISOUNDPLAYER_SetVolume
            uint16 wVolume;

            // Create ISoundPlayer interface object. This step needs to be done only
            // if the ISoundPlayer interface object have not already been created. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUNDPLAYER, (void **)&pMe->m_pISoundPlayer); 

            if (pMe->m_pISoundPlayer == NULL)
               return;

            // Register the status callback funtion.
            ISOUNDPLAYER_RegisterNotify(pMe->m_pISoundPlayer, &SoundPlayerCBFn, (void *)pMe);
            ISOUNDPLAYER_Set(pMe->m_pISoundPlayer, SDT_FILE, pMe->m_szFileName);

            // Set volume value to half the maximum value. 
            wVolume = AEE_MAX_VOLUME;///2;

            // ISOUNDPLAYER_SetVolume: This function sets the playback volume of 
            // the sound player object.
            ISOUNDPLAYER_SetVolume (pMe->m_pISoundPlayer, wVolume); 

            // Play a sound
            //ISOUNDPLAYER_Play (pMe->m_pISoundPlayer);

            // The playback will occur at the new volume level set above.

            // The callback function is not deregistered here nor is 
            // the sound player object released here. This is to allow
            // for the sound file to play and for the user to hear it. 
            // When the user decides to move to another menu selection 
            // or exit the applet the callback function is deregistered
            // and the sound player object is released.
         }
         break;
      case USAGE_GETVOLUME:
         {
            // 15. Usage: ISOUNDPLAYER_GetVolume

            // Create ISoundPlayer interface object. This step needs to be done only
            // if the ISoundPlayer interface object have not already been created. 
            ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUNDPLAYER, (void **)&pMe->m_pISoundPlayer); 

            if (pMe->m_pISoundPlayer == NULL)
               return;

            // Register the status callback funtion.
            ISOUNDPLAYER_RegisterNotify(pMe->m_pISoundPlayer, &SoundPlayerCBFn, (void *)pMe);
            ISOUNDPLAYER_Set(pMe->m_pISoundPlayer, SDT_FILE, pMe->m_szFileName);

            // ISOUNDPLAYER_GetVolume: This function gets the playback volume of 
            // the sound player object. The volume value is sent to the callback 
            // function registered above.
            ISOUNDPLAYER_GetVolume (pMe->m_pISoundPlayer); 

            // The callback function is not deregistered here nor is 
            // the sound player object released here. This is to allow
            // for the sound file to play and for the user to hear it. 
            // When the user decides to move to another menu selection 
            // or exit the applet the callback function is deregistered
            // and the sound player object is released.
         }
         break;
      default:
         return;
   }

   // Display above event. 
   DisplayEvent (pMe, wParam);

   return;
}

/*===========================================================================

FUNCTION SoundPlayerCBFn

DESCRIPTION
	This is the callback function that is registered with ISoundPlayer interface.
   ISoundPlayer communicates with the applet via this callback only.

PROTOTYPE:
   void SoundPlayerCBFn
   (
      AEE_SOUNDCBIndType  eCBType,
      AEESoundPlayerStatus eSPStatus,
      const void *              pClientData,
      void *                    dwParam
   );

PARAMETERS:
   eCBType: Type of ISoundPlayer callback.
   eSPStatus: Status contained in callback. The status depends on the 
            eCBType.
   pClientData: A reference to correlate this transaction
   dwParam: Additional info specific to callback type and status type

   Refer to BREW Reference Guide for more details.

DEPENDENCIES
  none

RETURN VALUE
  none

SIDE EFFECTS
  none
===========================================================================*/
static void SoundPlayerCBFn( void * pUser, AEESoundPlayerCmd eCBType, AEESoundPlayerStatus eStatus, uint32 dwParam )
{
   CISoundPlayerApp* pMe = (CISoundPlayerApp*)pUser;
   char szBuf [30] = {0};
	AEEDeviceInfo di;
   int charHeight = 0;      // Stores the char height in pixels for given font
   int pnAscent = 0;        // Stores the ascent in number of pixels
   int pnDescent = 0;       // Stores the descent in number of pixels
   int lowestLine = 0;
   AEESoundPlayerCmdData * pData = (AEESoundPlayerCmdData *) dwParam;

   if (pMe == NULL || pMe->m_pIMenu == NULL || pMe->a.m_pIShell == NULL)
      return;

   // Get the font metrics info
   charHeight = IDISPLAY_GetFontMetrics (pMe->a.m_pIDisplay, AEE_FONT_BOLD,
      &pnAscent, &pnDescent);

   // get device information
   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);

   // Calculate the lowest line on the screen. This will be used to display 
   // the status of some operations at the bottom of the screen.
   lowestLine = di.cyScreen/charHeight - 1;

   switch ( eStatus )
   {
      case AEE_SOUNDPLAYER_SUCCESS:
         {
            // If the callback was invoked as a result of the
            // GetVolume function print the volume value on the
            // screen.
            if (pData && eCBType == AEE_SOUNDPLAYER_VOLUME_CB)
            {
               SPRINTF (szBuf, "Current Vol: %u", pData->wVolume); 
               DisplayOutput (pMe, -1, szBuf);
            }
            else if ((eCBType == AEE_SOUNDPLAYER_PLAY_CB) && (pMe->m_bSPActive))
            {
               DisplayOutput (pMe, lowestLine, "Status: Playing");
            }
            else if ((eCBType == AEE_SOUNDPLAYER_STATUS_CB || eCBType == AEE_SOUNDPLAYER_SOUND_CB) && !pMe->m_bSPActive)
            {
               DisplayOutput (pMe, lowestLine - 4, "Status: Success");
            }
         }
         break;
      case AEE_SOUNDPLAYER_DONE:
         if (pData && eCBType == AEE_SOUNDPLAYER_TIME_CB)
         {
            SPRINTF (szBuf, "Total (ms): %u", (uint16)pData->dwTotalTime); 
            DisplayOutput (pMe, -1, szBuf);
         }
         else if (pMe->m_bSPActive)
         {
            DisplayOutput (pMe, lowestLine, "Status: Stopped");
            pMe->m_nPlayTime = 0;
         }
         break;
      case AEE_SOUNDPLAYER_AUDIO_SPEC:
         if (pMe->m_bSPActive)
         {
            DisplayOutput (pMe, lowestLine, "Status: Playing");
            pMe->m_nPlayTime = 0;
         }
         break;
      case AEE_SOUNDPLAYER_REWIND:
//         if (pMe->m_nPlayTime >= 5)
//            pMe->m_nPlayTime -= 5;
//         else
//            pMe->m_nPlayTime = 0;
         if (pMe->m_bSPActive)
         {
            pMe->m_nPlayTime = pData ? pData->dwElapsedTime/1000 : 0;
            SPRINTF (szBuf, "Play Time: %i", pMe->m_nPlayTime); 
            DisplayOutput (pMe, lowestLine - 1, szBuf);
            DisplayOutput (pMe, lowestLine, "Status: Rewind done");
         }
         break;
      case AEE_SOUNDPLAYER_FFORWARD:
//         pMe->m_nPlayTime += 5;
         if (pMe->m_bSPActive)
         {
            pMe->m_nPlayTime = pData ? pData->dwElapsedTime/1000 : 0;
            SPRINTF (szBuf, "Play Time: %i", pMe->m_nPlayTime); 
            DisplayOutput (pMe, lowestLine - 1, szBuf);
            DisplayOutput (pMe, lowestLine, "Status: FFoward done");
         }
         break;
      case AEE_SOUNDPLAYER_PAUSE:
         if (pMe->m_bSPActive)
         {
            pMe->m_nPlayTime = pData ? pData->dwElapsedTime/1000 : 0;
            SPRINTF (szBuf, "Play Time: %i", pMe->m_nPlayTime); 
            DisplayOutput (pMe, lowestLine - 1, szBuf);
            DisplayOutput (pMe, lowestLine, "Status: Paused");
         }
         break;
      case AEE_SOUNDPLAYER_RESUME:
         if (pMe->m_bSPActive)
         {
            pMe->m_nPlayTime = pData ? pData->dwElapsedTime/1000 : 0;
            SPRINTF (szBuf, "Play Time: %i", pMe->m_nPlayTime); 
            DisplayOutput (pMe, lowestLine - 1, szBuf);
            DisplayOutput (pMe, lowestLine, "Status: Resumed");
         }
         break;
      case AEE_SOUNDPLAYER_TEMPO:
         DisplayOutput (pMe, -1, "SetTempo: Success");
         break;
      case AEE_SOUNDPLAYER_TUNE:
         DisplayOutput (pMe, -1, "SetTune: Success");
         break;
      case AEE_SOUNDPLAYER_TICK_UPDATE:
          pMe->m_nPlayTime++;
          SPRINTF (szBuf, "Play Time: %i", pMe->m_nPlayTime); 
          if (pMe->m_bSPActive)
          {
             DisplayOutput (pMe, lowestLine - 1, szBuf);
          }
          else
          {
             // Display around 2/3 the height of the screen (The middle of
             // the screen the satus of the function will be displayed)
             DisplayOutput (pMe, lowestLine*2/3, szBuf);
          }
         break;
      case AEE_SOUNDPLAYER_FAILURE:
         STRCPY(szBuf, "Player: failure");
         if (pMe->m_bSPActive)
         {
            DisplayOutput (pMe, lowestLine, szBuf);
         }
         else
         {
            // Display around 2/3 the height of the screen (The middle of
            // the screen the satus of the function will be displayed)
            DisplayOutput (pMe, lowestLine*2/3, szBuf);
         }
         break;
      case AEE_SOUNDPLAYER_ABORTED:
         pMe->m_nPlayTime = 0;
         break;
      default:
         DisplayOutput (pMe, lowestLine, "Player: Unknown status");
         break;
   }
}

/*===========================================================================

FUNCTION: DisplayEvent

DESCRIPTION
    This function is a display function which sets up the initial screen
    for the display output for a given interface function. This displays
    the function name at the top of the screen and displays the
    strings "Press Up/Down" and "keys to continue.." at the bottom of 
    the screen. In the case of functions that have no output it also 
    displays the string "No Output..." in the middile of the screen.

PROTOTYPE:
   void DisplayEvent (CISoundPlayerApp *pMe, uint16 wParam)

PARAMETERS:
   pMe:    [in]: Pointer to CISoundPlayerApp interface object
   wParam: [in]: API function type Id selected by user

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void DisplayEvent (CISoundPlayerApp *pMe, uint16 wParam)
{
   AEEDeviceInfo di;        // Device Info
   AECHAR szBuf[100];
   AEERect rc; 
   int charHeight = 0;      // Stores the char height in pixels for given font
   int pnAscent = 0;        // Stores the ascent in number of pixels
   int pnDescent = 0;       // Stores the descent in number of pixels

   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
      return;

   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);
   rc.x = 0;
   rc.y = 0;
   rc.dx = di.cxScreen;
   rc.dy = di.cyScreen;
   
   if (pMe->a.m_pIDisplay)
   {
      // Get the font metrics info
      charHeight = IDISPLAY_GetFontMetrics (pMe->a.m_pIDisplay, AEE_FONT_NORMAL,
         &pnAscent, &pnDescent);

      switch (wParam)
      {
         case USAGE_BASIC_USAGE:
            STR_TO_WSTR("Basic Usage:", szBuf, sizeof(szBuf));
            break;
         case USAGE_REGISTERNOTIFY:
            STR_TO_WSTR("RegisterNotify", szBuf, sizeof(szBuf));
            break;
         case USAGE_SET:
            STR_TO_WSTR("Set", szBuf, sizeof(szBuf));
            break;
         case USAGE_PLAY:
            STR_TO_WSTR("Play", szBuf, sizeof(szBuf));
            break;
         case USAGE_STOP:
            STR_TO_WSTR("Stop", szBuf, sizeof(szBuf));
            break;
         case USAGE_REWIND:
            STR_TO_WSTR("Rewind", szBuf, sizeof(szBuf));
            break;
         case USAGE_FASTFORWARD:
            STR_TO_WSTR("FastForward", szBuf, sizeof(szBuf));
            break;
         case USAGE_PAUSE:
            STR_TO_WSTR("Pause", szBuf, sizeof(szBuf));
            break;
         case USAGE_RESUME:
            STR_TO_WSTR("Resume", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETTEMPO:
            STR_TO_WSTR("SetTempo", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETTUNE:
            STR_TO_WSTR("SetTune", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETTOTALTIME:
            STR_TO_WSTR("GetTotalTime", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETSOUNDDEVICE:
            STR_TO_WSTR("SetSoundDevice", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETVOLUME:
            STR_TO_WSTR("SetVolume", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETVOLUME:
            STR_TO_WSTR("GetVolume", szBuf, sizeof(szBuf));
            break;
         default:
            STR_TO_WSTR("Unsupported fn", szBuf, sizeof(szBuf));
      }
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,5, 0, &rc,
		   IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);

      // Functions with no output
      switch (wParam)
      {
         case USAGE_REGISTERNOTIFY:
            // Place in the middle of screen
            STR_TO_WSTR("No Output...", szBuf, sizeof(szBuf));
 		      IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,10, 
               rc.dy/2 - charHeight, &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
         default:
            break;
      }

      STR_TO_WSTR("Press Up/Down, 0-9, *, #,", szBuf, sizeof(szBuf));
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,5, rc.dy - 
         2*(charHeight) , &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
      STR_TO_WSTR("or clr keys to continue", szBuf, sizeof(szBuf));
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,5, rc.dy - 
         charHeight, &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
		IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);
   }
}



/*===========================================================================

FUNCTION: BuildPlayerMenu

DESCRIPTION
    This function builds the main menu when this applet is started.
    It displays a simple menu with menu items corresponding to 
    each of the interface functions.
    
PROTOTYPE:
   void BuildPlayerMenu(CISoundPlayerApp *pMe)

PARAMETERS:
   pMe: [in]: Pointer to AEEApplet struct

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void BuildPlayerMenu(CISoundPlayerApp *pMe)
{
	AEERect qrc;
	AEEDeviceInfo di;
   AECHAR szBuf[50];
   int charHeight = 0;      // Stores the char height in pixels for given font
   int pnAscent = 0;        // Stores the ascent in number of pixels
   int pnDescent = 0;       // Stores the descent in number of pixels

   if (pMe == NULL || pMe->m_pIMenu == NULL || pMe->a.m_pIShell == NULL)
      return;

   // Get the font metrics info
   charHeight = IDISPLAY_GetFontMetrics (pMe->a.m_pIDisplay, AEE_FONT_BOLD,
      &pnAscent, &pnDescent);

   if (pMe->m_pPlayerMenu != NULL)
      return;

   // Set up the player menu
   if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, (void **)&pMe->m_pPlayerMenu)
      != SUCCESS)
   {
		return;
   }

   // Set the player active
   pMe->m_bSPActive = TRUE;

   //Set the Rectangle for the Menu
   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);
   SETAEERECT (&qrc, 0, 0, di.cxScreen, di.cyScreen - (3*charHeight- (charHeight/2)));

   // Set main menu inactive
   IMENUCTL_SetActive(pMe->m_pIMenu,FALSE);

   IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);

	// Set Title
   STR_TO_WSTR("SoundPlayer Menu:", szBuf, sizeof(szBuf));
	IMENUCTL_SetTitle(pMe->m_pPlayerMenu, NULL, 0, szBuf);

   IMENUCTL_SetRect(pMe->m_pPlayerMenu, &qrc);  

   //Add invidual entries to the Menu
   STR_TO_WSTR("Play", szBuf, sizeof(szBuf));
   IMENUCTL_AddItem(pMe->m_pPlayerMenu, 0, 0, USAGE_PLAY, szBuf, 0);

   STR_TO_WSTR("Stop", szBuf, sizeof(szBuf));
   IMENUCTL_AddItem(pMe->m_pPlayerMenu, 0, 0, USAGE_STOP, szBuf, 0);

   STR_TO_WSTR("Rewind (5 secs)", szBuf, sizeof(szBuf));
   IMENUCTL_AddItem(pMe->m_pPlayerMenu, 0, 0, USAGE_REWIND, szBuf, 0);

   STR_TO_WSTR("FastForward (5 secs)", szBuf, sizeof(szBuf));
   IMENUCTL_AddItem(pMe->m_pPlayerMenu, 0, 0, USAGE_FASTFORWARD, szBuf, 0);

   STR_TO_WSTR("Pause", szBuf, sizeof(szBuf));
   IMENUCTL_AddItem(pMe->m_pPlayerMenu, 0, 0, USAGE_PAUSE, szBuf, 0);

   STR_TO_WSTR("Resume", szBuf, sizeof(szBuf));
   IMENUCTL_AddItem(pMe->m_pPlayerMenu, 0, 0, USAGE_RESUME, szBuf, 0);

   STR_TO_WSTR("Exit Player Menu", szBuf, sizeof(szBuf));
   IMENUCTL_AddItem(pMe->m_pPlayerMenu, 0, 0, USAGE_EXIT_PLAYER_MENU, szBuf, 0);

   IMENUCTL_SetActive(pMe->m_pPlayerMenu,TRUE);
}

/*===========================================================================

FUNCTION: BuildMainMenu

DESCRIPTION
    This function builds the main menu when this applet is started.
    It displays a simple menu with menu items corresponding to 
    each of the interface functions.
    
PROTOTYPE:
   void BuildMainMenu(CISoundPlayerApp *pMe)

PARAMETERS:
   pMe: [in]: Pointer to AEEApplet struct

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void BuildMainMenu(CISoundPlayerApp *pMe)
{
	AEERect qrc;
	AEEDeviceInfo di;
   AECHAR szBuf[50];

   if (pMe == NULL || pMe->m_pIMenu == NULL || pMe->a.m_pIShell == NULL)
      return;

	// Set Title
   STR_TO_WSTR("ISoundPlayer Examples", szBuf, sizeof(szBuf));
	IMENUCTL_SetTitle(pMe->m_pIMenu, NULL, 0, szBuf);

	//Set the Rectangle for the Menu
	ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);
	qrc.x = 0;
	qrc.y = 0;
	qrc.dx = di.cxScreen;
	qrc.dy = di.cyScreen;
	IMENUCTL_SetRect(pMe->m_pIMenu, &qrc);  

	//Add invidual entries to the Menu

   // Add Basic Usage to menu
   STR_TO_WSTR("1. Basic Usage", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_BASIC_USAGE, szBuf, 0);

   // Add RegisterNotify to menu
   STR_TO_WSTR("2. RegisterNotify", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_REGISTERNOTIFY, szBuf, 0);

   // Add Set to menu
   STR_TO_WSTR("3. Set", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SET, szBuf, 0);

   // Add Play to menu
   STR_TO_WSTR("4. Play/Stop/Rewind/etc.", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_PLAYER_MENU, szBuf, 0);

	// Add SetTempo to menu
   STR_TO_WSTR("5. SetTempo", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETTEMPO, szBuf, 0);

	// Add SetTune to menu
   STR_TO_WSTR("6. SetTune", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETTUNE, szBuf, 0);	

	// Add GetTotalTime to menu
   STR_TO_WSTR("7. GetTotalTime", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETTOTALTIME, szBuf, 0);	

	// Add SetSoundDevice to menu
   STR_TO_WSTR("8. SetSoundDevice", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETSOUNDDEVICE, szBuf, 0);	

	// Add Set Volume to menu
   STR_TO_WSTR("9. SetVolume", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETVOLUME, szBuf, 0);	

	// Add Get Volume to menu
   STR_TO_WSTR("10. GetVolume", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETVOLUME, szBuf, 0);	

   // Activate menu
	IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);
}

/*===========================================================================

FUNCTION: DisplayOutput

DESCRIPTION
    This function displays an output string at a given line number on the
    screen. If the nline parameter is a negative value (-1) the string
    is displayed in the middle of the screen. If the "nline" value is larger
    than or equal to zero the "nline" value is multiplied by 15 and the 
    resulting value in pixels is set to the y-coordinate of the start of 
    the string display on the screen. If the string does not fit on one line
    the string wraps around to the nime line (spaced rougly 10-15 pixels apart).
    By default 5 is used as the starting the x-coordinate of a displayed 
    string.

    How many characters that fit on one line is calculated for each line 
    that is wrapped around to the nime line.

    Note: depending on the phone screen size and the fonts used for characters 
          the output might differ on different handsets (devcies). Where some 
          handsets will have a smaller screen and large default fonts which will 
          cause partial overlapping of lines. This function does not try to address
          these issues (this is meant as a simple display function).
    
PROTOTYPE:
   void DisplayOutput(CISoundPlayerApp * pi, int nline, char *pszStr)

PARAMETERS:
   pi:     [in]: Contains a pointer to the CISoundPlayerApp class.
   nline:  [in]: Contains the line number to start displaying the time. 
        The line numbers are by default spaced 15 pixels apart along the y-axis.
   pszStr: [in]: The character string to be displayed on the screen.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void DisplayOutput(CISoundPlayerApp * pMe, int nline, char *pszStr)
{
   AEEDeviceInfo di;        // Device Info
   AECHAR * pszBuf = NULL;
   AECHAR * psz = NULL;
   int pixelWidth;
   AEEFont font = AEE_FONT_NORMAL;
   int pnFits = 0, dy;
   int totalCh = 0;
   int charHeight = 0;      // Stores the char height in pixels for given font
   int pnAscent = 0;        // Stores the ascent in number of pixels
   int pnDescent = 0;       // Stores the descent in number of pixels
	AEERect rc;

   if (pMe == NULL)
      return;

   if ((pszBuf = (AECHAR *)MALLOC(200)) == NULL)
      return;

   // Get device information
   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);

   // Get the font metrics info
   charHeight = IDISPLAY_GetFontMetrics (pMe->a.m_pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   // Convert to wide string (unicode)
   STR_TO_WSTR ((char *)pszStr, pszBuf, 200);

   // If nlines is zero then print this string starting around the middle of 
   // the screen. Or else multiply nlines by 10 to decide the y coordinate of
   // the start of the string.
   if (nline < 0) {
      dy = di.cyScreen*2/5;
   }
   else{
      dy = nline * charHeight;
   }

   // psz keeps track of the point from which to write from the string buffer
   // in case the string does not fit one line and needs to wrap around in the
   // next line.
   psz = pszBuf;
      
   // Need to calculate the lotal string length to decide if any wrapping
   // around is needed.
   totalCh = STRLEN ((char *)pszStr);

   // Keep displaying text string on multiple lines if the string can't be displayed
   // on one single line. Lines are spaced 15 pixels apart.
   while ((totalCh > 0) && (*psz != NULL))
   { 
      // Get information on how many characters will fit in a line.
      // Give the pointer to the buffer to be displayed, and the number of
      // pixels along the x axis you want to display the string in (max number)
      // pnFits will have the max number of chars that will fit in the maxWidth
      // number of pixels (given string can't fit in one line), or the number of 
      // chars in the string (if it does fit in one line). pnWidth gives the
      // number of pixels that will be used to display pnFits number of chars.
      pixelWidth = IDISPLAY_MeasureTextEx(pMe->a.m_pIDisplay,
                      font, 
                      (AECHAR *) psz,  // Start of the buffer to display,
                      -1,
                      di.cxScreen - 5, // maxWidth
                      &pnFits);         // Number of chars that will fit a line

      // If pnFits is zero there is something wrong in the input to above function. 
      // Normally this scenario should not occur. But, have the check anyway.
      if (pnFits == 0)
      {
         FREE (pszBuf);
         return;
      }
		SETAEERECT(&rc, 0, dy, di.cyScreen, charHeight);
		IDISPLAY_EraseRect(pMe->a.m_pIDisplay, &rc);

      IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, psz, pnFits, 5 /*start dx*/, 
         dy, 0 /* use default rectangle coordinates */, 0);

      psz += pnFits;      // move pointer to the next segment to be displayed
      totalCh -= pnFits;  // reduce the total number of characters to still display
      dy += charHeight;   // Place next line charHeight pixels below the 
                          // previous line. 
      IDISPLAY_Update(pMe->a.m_pIDisplay);
      if (totalCh < pnFits)
         pnFits = totalCh;  // if total number is less than pnFits, adjust pnFits
   }

   FREE (pszBuf);
   return;   
} // End of DisplayOutput

/*===========================================================================

FUNCTION: CleanupSP_RedrawMainMenu

DESCRIPTION
    This function Releases the ISoundPlayer interface and redraws
    the main menu of this applet.
    
PROTOTYPE:
   void CleanupSP_RedrawMainMenu (CISoundPlayerApp * pMe)

PARAMETERS:
   pMe:     [in]: Contains a pointer to the CISoundPlayerApp class.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void CleanupSP_RedrawMainMenu (CISoundPlayerApp * pMe)
{
   if (pMe)
   {
      if (pMe->m_pISoundPlayer)
      {
         // First deregister the notify. This way no callbacks 
         // will be posted for the rest of the ISoundPlayer operations.
         ISOUNDPLAYER_RegisterNotify (pMe->m_pISoundPlayer, NULL, NULL);
         // Stop any songs being played at this time
         ISOUNDPLAYER_Stop (pMe->m_pISoundPlayer);
         ISOUNDPLAYER_Release (pMe->m_pISoundPlayer);
         pMe->m_pISoundPlayer = NULL;
      }
                 
      // Now release the player menu.
      IMENUCTL_SetActive(pMe->m_pPlayerMenu,FALSE);
      IMENUCTL_Release (pMe->m_pPlayerMenu);
      // Stamp the player menu as released
      pMe->m_pPlayerMenu = NULL;

      // Stamp the player active flag to false.
      pMe->m_bSPActive = FALSE;
      // Just in case the user exited while playing
      pMe->m_nPlayTime = 0;

      // Reactivate the main menu
      IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);
   }
}

