/*===========================================================================

FILE: istaticusage.c

SERVICES: Sample applet using AEE

DESCRIPTION
  This file contains usage examples of the IStatic interface. IStatic interface
  functions allow the user to display static text controls, which consist of a 
  text message and title.  Usage examples are provided for each IStatic interface
  function (for the SetProperties function, a separate example is provided for
  each of the eight properties of a static text control to illustrate the effect
  of setting each property).

PUBLIC CLASSES:  
   N/A


   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/

/*===============================================================================
                     INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
// Mandatory includes
#include "AEEAppGen.h"	// AEEApplet declaration
#include "AEEUsageAppIDs.h" // Class IDs for usage apps (including this one)

#include "AEE.h"			// Standard AEE Declarations
#include "AEEShell.h"	// AEE Shell Services
#include "AEEDisp.h"		// AEE Display Services
#include "AEEStdLib.h"  // AEE StdLib Services

// Services used by app
#include "AEEMenu.h"		// Menu Services

// App resource header generated by BREW Resource Editor.
#include "static_res.h"

/*===========================================================================

                      PUBLIC DATA DECLARATIONS

===========================================================================*/

/*-------------------------------------------------------------------
            Type Declarations
-------------------------------------------------------------------*/
// IStatic app struct. This is the main struct for this applet. This will hold 
// all the data members that need to be remembered throughout the life of
// the applet. 
// THE FIRST DATA MEMBER OF THIS STRUCT MUST BE AN AEEApplet OBJECT. 
// This is critical requirement which is related to an applet being
// dynamically loaded. The app developers should add their data members
// following the AEEApplet data member.
typedef struct _CIStaticApp {
   AEEApplet	      a;
   AEERect m_rc; // Enclosing rectangle for static text control
   IMenuCtl * m_pIMenu;
   IStatic *  m_pIStatic;
   IStatic *  m_pIStatic1;
   IImage *	  m_pIImage;
   AECHAR *   m_pszText;
} CIStaticApp;

/*-------------------------------------------------------------------
            Function Prototypes
-------------------------------------------------------------------*/
// Mandatory function:
// App Handle Event function
static boolean StaticApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);

// App specific data alloc-init/free functions
static boolean IStatic_InitAppData(IApplet* pMe);
static void IStatic_FreeAppData(IApplet* pMe);

static void BuildMainMenu(CIStaticApp *pMe);
static void IStaticUsage (CIStaticApp *pMe, uint16 wParam);
static void DisplayEvent (CIStaticApp *pMe, uint16 wParam);
static void DisplayOutput(CIStaticApp *pMe, int nline, char *pszStr);

/*-------------------------------------------------------------------
            Global Constant Definitions
-------------------------------------------------------------------*/
// App Resource File
#define  APP_RES_FILE      "static.bar"

// App specific constants

// Dimensions of screen border and text at top
// and bottom of screen 
#define BORDER_WIDTH 10
#define TEXT_TOP_HEIGHT 10
#define TEXT_BOTTOM_HEIGHT 15

// Size of buffer allocated to hold text strings
#define TEXT_BUFFER_SIZE (400 * sizeof(AECHAR))

// App specific constants
#define USAGE_BASIC_USAGE		   100
#define USAGE_HANDLEEVENT          101
#define USAGE_REDRAW               102
#define USAGE_SETACTIVE            103
#define USAGE_ISACTIVE             104
#define USAGE_SETRECT              105
#define USAGE_GETRECT              106
#define USAGE_SETPROPS_ICONTEXT    107
#define USAGE_SETPROPS_TEXTALLOC   108
#define USAGE_SETPROPS_TITLEALLOC  109
#define USAGE_SETPROPS_NOSCROLL    110
#define USAGE_SETPROPS_MIDDLETEXT  111
#define USAGE_SETPROPS_UNDERLINE   112
#define USAGE_SETPROPS_CENTERTEXT  113
#define USAGE_SETPROPS_CENTERTITLE 114
#define USAGE_GETPROPS             115
#define USAGE_RESET                116
#define USAGE_SETTEXT              117
#define USAGE_ERASE_SCREEN         118   // Special case to erase screen

/*===============================================================================
                     FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to IStatic_InitAppData(). This is done 
   as follows: IStatic_InitAppData() is called to initialize the applet-specific data structure 
   instance. It is the developer's responsibility to fill-in app data initialization code of 
   IStatic_InitAppData(). App developer is also responsible to free interfaces and memory 
   allocated for data contained in applet-specific data structure -- this can be done in 
   IStatic_FreeAppData().

PROTOTYPE:
	int AEEClsCreateInstance(AEECLSID ClsId, IShell* pIShell, IModule* pIModule, 
										  void** ppObj)

PARAMETERS:
	ClsId: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell interface. 

	pIModule: [in]: Contains pointer to the IModule interface to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid IApplet structure. 

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the app needs to be loaded and if AEEApplet_New() invocation was successful
  EFAILED: If the app does not need to be loaded or if errors occurred in AEEApplet_New().
  If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId, IShell* pIShell, IModule* po, void** ppObj)
{
   *ppObj = NULL;
		
   // We want to load this App. So, invoke AEEApplet_New().To it, pass the
   // address of the app-specific handle event function.
   if(ClsId == AEECLSID_STATIC_APP){
      if(AEEApplet_New(sizeof(CIStaticApp), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)StaticApp_HandleEvent,(PFNFREEAPPDATA)IStatic_FreeAppData) == TRUE)
      {
         if (IStatic_InitAppData((IApplet*)*ppObj))
             return(AEE_SUCCESS);
      }
   }
   return (EFAILED);
}

/*===========================================================================

FUNCTION StaticApp_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function.

PROTOTYPE:
	boolean App_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

	eCode: Specifies the Event sent to this applet

    wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/
static boolean StaticApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
   CIStaticApp * pMe = (CIStaticApp*)pi;

   switch (eCode) 
	{

        case EVT_APP_START:
			// User has selected this applet to run.  Try to create the main
			// menu that allows the user to select a usage example to execute--
			// return if we fail to get an interface pointer for the menu
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, 
				(void **)&pMe->m_pIMenu) != SUCCESS)
				return TRUE;
     
			// Succeeded in getting a menu interface pointer, so display
			// the main menu
			BuildMainMenu(pMe);
			return(TRUE);

        case EVT_APP_STOP:
            return(TRUE);

        case EVT_KEY: 
			if (wParam == AVK_UP || wParam == AVK_DOWN)
			{
			   // If the user pressed the UP or DOWN keys,
			   // they wish to return to the main menu -- release
			   // any interfaces that may have been created
			   if (pMe->m_pIStatic != NULL)
			   {
				  ISTATIC_Release (pMe->m_pIStatic);
				  pMe->m_pIStatic = NULL;
			   }

			   if (pMe->m_pIStatic1 != NULL)
			   {
				  ISTATIC_Release (pMe->m_pIStatic1);
				  pMe->m_pIStatic1 = NULL;
			   }

			   if (pMe->m_pIImage != NULL)
			   {
				  IIMAGE_Release (pMe->m_pIImage);
				  pMe->m_pIImage = NULL;
			   }
			}

			// Re-activate the main menu
            IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);

			// Pass the event to the main menu
			if(IMENUCTL_HandleEvent(pMe->m_pIMenu, EVT_KEY, wParam, 0))
          	   return TRUE;
            else
            return FALSE; 

		case EVT_COMMAND:
  		    switch(wParam)
			{
			    case USAGE_BASIC_USAGE:
                case USAGE_HANDLEEVENT:
                case USAGE_REDRAW:
                case USAGE_SETACTIVE:
                case USAGE_ISACTIVE:
                case USAGE_SETRECT:
                case USAGE_GETRECT:
                case USAGE_SETPROPS_ICONTEXT:
                case USAGE_SETPROPS_TEXTALLOC:
                case USAGE_SETPROPS_TITLEALLOC:
                case USAGE_SETPROPS_NOSCROLL:
                case USAGE_SETPROPS_MIDDLETEXT:
                case USAGE_SETPROPS_UNDERLINE:
                case USAGE_SETPROPS_CENTERTEXT:
                case USAGE_SETPROPS_CENTERTITLE:
                case USAGE_GETPROPS:
                case USAGE_RESET:
                case USAGE_SETTEXT:
				{
				   // De-activate main menu to turn off horizontal scrolling
				   // of long menu items
				   IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);

				   // Execute the usage example the user has selected
                   IStaticUsage (pMe, wParam);
				}
			       return TRUE;

			    default:
				   return FALSE;
			}
        default:
            break;
   }
   return TRUE;
}



/*===========================================================================

FUNCTION IStatic_InitAppData

DESCRIPTION
    This function initializes data in the app-specific data structure.

PROTOTYPE:
	static boolean IStatic_InitAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

DEPENDENCIES
    None.

RETURN VALUE
    TRUE: If the app has app data is allocated and initialized successfully
    FALSE: Either app data could not be allocated or initialized

SIDE EFFECTS
    None.
===========================================================================*/
static boolean IStatic_InitAppData(IApplet* pi)
{
   CIStaticApp * pMe = (CIStaticApp*)pi;
   AEEDeviceInfo di;

   // Make sure the pointers we'll be using are valid
   if (pMe == NULL || pMe->a.m_pIShell == NULL)
	   return FALSE;

   // determine screen dimensions
   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);

   // Construct a rectangle that will contain the static text controls
   // used in the example--take into account the text printed at the top
   // and bottom of the screen for each example
   pMe->m_rc.x = BORDER_WIDTH;
   pMe->m_rc.y = BORDER_WIDTH;
   pMe->m_rc.dx = di.cxScreen - (2 * BORDER_WIDTH);
   pMe->m_rc.dy = di.cyScreen - (2 * BORDER_WIDTH) - TEXT_BOTTOM_HEIGHT;

   // Initialize pointers
   pMe->m_pIStatic = NULL;
   pMe->m_pIStatic1 = NULL;
   pMe->m_pIImage = NULL;
   pMe->m_pIMenu = NULL;

   // Allocate memory for text-string buffer
   if ((pMe->m_pszText = (AECHAR *) MALLOC(TEXT_BUFFER_SIZE)) == NULL)
	   return FALSE;

   return TRUE;
}

/*===========================================================================

FUNCTION IStatic_FreeAppData

DESCRIPTION
   Frees data and interfaces contained in applet-specific data structure.
 
PROTOTYPE:
	static void IStatic_FreeAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

DEPENDENCIES
   Assumes pi is not NULL

RETURN VALUE
  None.

SIDE EFFECTS
  none
===========================================================================*/
static void IStatic_FreeAppData(IApplet* pi)
{
   CIStaticApp * pMe = (CIStaticApp*)pi;

   // Free all interfaces that may still be present
   if (pMe->m_pIStatic != NULL)
   {
	  ISTATIC_Release (pMe->m_pIStatic);
   	  pMe->m_pIStatic = NULL;
   }

   if (pMe->m_pIStatic1 != NULL)
   {
	  ISTATIC_Release (pMe->m_pIStatic1);
	  pMe->m_pIStatic1 = NULL;
   }

   if (pMe->m_pIImage != NULL)
   {
	  IIMAGE_Release (pMe->m_pIImage);
	  pMe->m_pIImage = NULL;
   }

   if (pMe->m_pIMenu != NULL)
   {
	  IMENUCTL_Release(pMe->m_pIMenu);
      pMe->m_pIMenu = NULL;
   }

   // Free memory allocated for text-string buffer
   FREE(pMe->m_pszText);
}

/*===============================================================================
                     APP HELPER FUNCTION DEFINITIONS
=============================================================================== */
/*===========================================================================

FUNCTION: IStaticUsage

DESCRIPTION
    This function executes the usage example selected from the main menu by the
	user.  It takes as input the app data pointer and the ID of the example the
	user selected.  After clearing the screen, the selected example is executed.
	Each example is a self-contained code block that contains any local variables
	needed to execute the example.  Each code block also creates and releases any
	BREW interfaces that are required, and will exit if any errors are detected.  
	Once the example has been executed, this function prints a heading at the top 
	of the screen to indicate which example was selected, and a message at the 
	bottom of the screen instructing the user how to return to the main menu to 
	execute another example.
    
PROTOTYPE:
   static void IStaticUsage(CIStaticApp * pMe, uint16 wParam)

PARAMETERS:
   pMe:   [in]: Contains a pointer to the usage app data structure.
   wParam: [in]: Identifier of the selected example (example IDs are specified when the
                 main usage app menu is created).

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void IStaticUsage (CIStaticApp* pMe, uint16 wParam)
{  
   AEEDeviceInfo di; // Device Info
   AEERect rc, qrc; 

   char szResFile[] = APP_RES_FILE;   // Resource File used by this applet

   char szBuf[50];

   AECHAR pszTitle[20];

   // Make sure the pointers we'll be using are valid
   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
	   return;

   // Get screen size information for display purposes.
   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);

   // Rectangle used for displaying text messages on screen
   rc.x = 5;
   rc.y = 5;
   rc.dx = di.cxScreen - 5;
   rc.dy = di.cyScreen - 5;

   // Erase screen first for display output purposes.
   DisplayEvent (pMe, USAGE_ERASE_SCREEN);

   switch (wParam)
   {
      case USAGE_BASIC_USAGE:
         {
            // 1. Usage: Basic Static Text Control Usage

			// To illustrate the basic usage of the IStatic interface, we
			// will create two instances of static text controls, with each instance occupying
			// half the screen.  The top half of the screen will contain a control
			// that uses an animated bitmap instead of a text string as the control's
			// contents.  The bottom half of the screen will contain a scrolling text
			// message, with both the text and title centered (illustrating the setting
			// of multiple static text control properties).
			AEERect rc1, rc2;
			int nCharCount;

			// Compute rectangle dimensions for first control's rectangle
			// (top half of screen, plus a few more pixels to accommodate
			// the animated bitmap)
			rc1.x = pMe->m_rc.x;
			rc1.dx = pMe->m_rc.dx;
			rc1.y = pMe->m_rc.y;
			rc1.dy = (pMe->m_rc.dy / 2) + 5;

			// Compute rectangle dimensions for second control's rectangle
			// (bottom half of screen)
			rc2.x = pMe->m_rc.x;
			rc2.dx = pMe->m_rc.dx;
			rc2.y = rc1.y + rc1.dy;
			rc2.dy = rc1.dy - 10;

			// Display the first control, which will be an animated bitmap
			pMe->m_pIImage = ISHELL_LoadResImage(pMe->a.m_pIShell, szResFile, IDB_STATIC_TEXT_BITMAP);
			if(pMe->m_pIImage)
			{
				// The animated bitmap text has four frames (404 x 20 pixels, with each frame
				// 101 x 26).  You must specify the number of frames before you can start 
				// the animation.
				IIMAGE_SetParm(pMe->m_pIImage, IPARM_NFRAMES, 4, 0);

				// Slow down the animation rate so that each line can be read
				// (default rate is 150 ms between frames; we increase to 1000 ms)
                IIMAGE_SetParm(pMe->m_pIImage, IPARM_RATE, 1000, 0);

			    // Create an instance of an IStatic text control
			    if (ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_STATIC, (void **)&(pMe->m_pIStatic)) != SUCCESS)
				    return;

			    // Set the dimensions of the rectangle for the first text control
                ISTATIC_SetRect(pMe->m_pIStatic, &rc1);

			    // Set the property that specifies that the control text will
			    // instead be an animated bitmap
			    ISTATIC_SetProperties(pMe->m_pIStatic, ST_ICONTEXT);

			    // Set the values of the title and text strings for this control
				// (note that the third parameter to SetText is a pointer to an 
				// instance of IImage instead of a pointer to a text string)
			    ISTATIC_SetText(pMe->m_pIStatic, NULL, (AECHAR *) pMe->m_pIImage, AEE_FONT_BOLD, AEE_FONT_NORMAL);

			    // Display the text control
                ISTATIC_Redraw(pMe->m_pIStatic);
			}

			// Now display the second control, which will consist of a scrolling
			// text message (text and title both will be centered).  Note that we need
			// a separate instance of the IStatic interface for this second control.

			// Read in the title and text string from the resource file
			nCharCount = ISHELL_LoadResString(pMe->a.m_pIShell, szResFile, IDS_CENTERTEXT_TITLE, pszTitle, sizeof(pszTitle));
			nCharCount = ISHELL_LoadResString(pMe->a.m_pIShell, szResFile, IDS_CENTERTEXT_TEXT, pMe->m_pszText, TEXT_BUFFER_SIZE);

			// Create an instance of an IStatic text control for the second control
			if (ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_STATIC, (void **)&(pMe->m_pIStatic1)) != SUCCESS)
				return;

			// Set the dimensions of the rectangle for the second control
            ISTATIC_SetRect(pMe->m_pIStatic1, &rc2);
            
			// Set the properties that specify centering of control text lines and title
			ISTATIC_SetProperties(pMe->m_pIStatic1, ST_CENTERTEXT | ST_CENTERTITLE);
			
			// Set the values of the title and text strings for this control
			ISTATIC_SetText(pMe->m_pIStatic1, pszTitle, pMe->m_pszText, AEE_FONT_BOLD, AEE_FONT_NORMAL);

			// Display the text control
            ISTATIC_Redraw(pMe->m_pIStatic1);
         }
         break;

      case USAGE_HANDLEEVENT:
         {
            // 2. Usage: ISTATIC_HandleEvent

			// This function is not supported in the current release.  In future
			// releases, the UP and DOWN arrow keys will be used for manual scrolling
			// of long text messages.  In the current release, static text controls do not
			// handle any events.
         }
         break;

      case USAGE_REDRAW:
         {
            // 3. Usage: ISTATIC_Redraw

			// The Redraw function does the work of actually displaying the control on the
			// screen, and is usually the last function called when setting up a static text
			// control.  Before calling ISTATIC_Redraw, you must first call ISTATIC_SetText 
			// to specify the control title and text or animated bitmap.

			// Read in the title and long text string from the resource file
			int nCharCount = ISHELL_LoadResString(pMe->a.m_pIShell, szResFile, IDS_LONG_TEXT_TITLE, pszTitle, sizeof(pszTitle));
			nCharCount = ISHELL_LoadResString(pMe->a.m_pIShell, szResFile, IDS_LONG_TEXT_TEXT, pMe->m_pszText, TEXT_BUFFER_SIZE);

			// Create an instance of an IStatic text control
			if (ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_STATIC, (void **)&(pMe->m_pIStatic)) != SUCCESS)
				return;

			// Set the dimensions of the rectangle for the text control
            ISTATIC_SetRect(pMe->m_pIStatic, &(pMe->m_rc));

			// Set the values of the title and text strings for this control
			ISTATIC_SetText(pMe->m_pIStatic, pszTitle, pMe->m_pszText, AEE_FONT_BOLD, AEE_FONT_NORMAL);

			// Display the text control
            ISTATIC_Redraw(pMe->m_pIStatic);

         }
         break;

      case USAGE_SETACTIVE:
         {
            // 4. Usage: ISTATIC_SetActive

			// This function is not supported in the current release.

         }
         break;

      case USAGE_ISACTIVE:
         {
            // 5. Usage: ISTATIC_IsActive

			// This function is not supported in the current release.

         }
         break;

      case USAGE_SETRECT:
         {
            // 6. Usage: ISTATIC_SetRect

			// The rectangle of a static control defines the portion of the screen that
			// will contain the control's title and text.  For these usage examples, we
			// have created a rectangle that leaves space for the messages that the usage
			// example framework display at the top and bottom of the screen.  In this example,
			// we will create a new rectangle that is smaller at the top and bottom to show how
			// the default rectangle can be changed (see also the Basic Usage example, which splits
			// the default rectangle in two so that two static text controls can be displayed on
			// the screen at the same time).

			AEERect rc_new;

			// Read in the title and long text string from the resource file
			int nCharCount = ISHELL_LoadResString(pMe->a.m_pIShell, szResFile, IDS_SETRECT_TITLE, pszTitle, sizeof(pszTitle));
			nCharCount = ISHELL_LoadResString(pMe->a.m_pIShell, szResFile, IDS_SETRECT_TEXT, pMe->m_pszText, TEXT_BUFFER_SIZE);

			// Create an instance of an IStatic text control
			if (ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_STATIC, (void **)&(pMe->m_pIStatic)) != SUCCESS)
				return;

			// The default rectangle for these examples is in pMe->m_rc.  
			// Create a new rectangle that is 20 pixels smaller on the top and bottom
			rc_new.x = pMe->m_rc.x;
			rc_new.y = pMe->m_rc.y + 20;
			rc_new.dx = pMe->m_rc.dx;
			rc_new.dy = pMe->m_rc.dy - 40;

			// Set the dimensions of the rectangle for the text control
            ISTATIC_SetRect(pMe->m_pIStatic, &rc_new);

			// Set the values of the title and text strings for this control
			ISTATIC_SetText(pMe->m_pIStatic, pszTitle, pMe->m_pszText, AEE_FONT_BOLD, AEE_FONT_NORMAL);

			// Display the text control
            ISTATIC_Redraw(pMe->m_pIStatic);
         }
         break;

      case USAGE_GETRECT:
         {
            // 7. Usage: ISTATIC_GetRect

			// The GetRect function allows the dimensions of a control's rectangle to
			// be retrieved.  To illustrate the use of this function, we 
			// retrieve the contents of our default control rectangle, and then we
			// echo the rectangle dimensions back to the device screen.

			// Read in the title and long text string from the resource file
			int nCharCount = ISHELL_LoadResString(pMe->a.m_pIShell, szResFile, IDS_LONG_TEXT_TITLE, pszTitle, sizeof(pszTitle));
			nCharCount = ISHELL_LoadResString(pMe->a.m_pIShell, szResFile, IDS_LONG_TEXT_TEXT, pMe->m_pszText, TEXT_BUFFER_SIZE);

			// Create an instance of an IStatic text control
			if (ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_STATIC, (void **)&(pMe->m_pIStatic)) != SUCCESS)
				return;

			// Set the dimensions of the rectangle for the text control
            ISTATIC_SetRect(pMe->m_pIStatic, &(pMe->m_rc));
            
		    // Retrieve the rectangle dimensions just specified	
			// (GetRect copies the dimensions into the AEERect structure provided)
			ISTATIC_GetRect(pMe->m_pIStatic, &qrc);

		    // Echo the info that was returned 
			// by GetRect in the AEERect structure

		    // x coordinate of rectangle origin in pixels
		    SPRINTF(szBuf, "x: %d", qrc.x); 
            DisplayOutput(pMe, 2, szBuf);

		    // y coordinate of rectangle origin in pixels
		    SPRINTF(szBuf, "y: %d", qrc.y); 
            DisplayOutput(pMe, 3, szBuf);

		    // rectangle width in pixels
 		    SPRINTF(szBuf, "dx: %d", qrc.dx); 
            DisplayOutput(pMe, 4, szBuf);
 
			// rectangle height in pixels
		    SPRINTF(szBuf, "dy: %d", qrc.dy); 
            DisplayOutput(pMe, 5, szBuf);
         }
         break;  

	  // The next eight examples illustrate the use of the SetProperties function.
	  // There is one example for each static text control property to illustrate
	  // the effect of setting that property.
	  // The value passed to SetProperties in its second argument is assigned as the
	  // new value of the control's property bit vector.  To set a single property, use
	  //
	  //    ISTATIC_SetProperties(pIStatic, ST_ICONTEXT)
	  //
	  // To set multiple properties, you can OR their bit masks, e.g.
	  //
	  //    ISTATIC_SetProperties(pIStatic, ST_ICONTEXT | ST_TITLEALLOC)
	  //
	  // To set a property in cases where one or more properties may already be
	  // set, use GetProperties to get the current settings, e.g.
	  //
	  //     props = ISTATIC_GetProperties(pIStatic);
	  //     ISTATIC_SetProperties(pIStatic, props | ST_ICONTEXT)
		 
      case USAGE_SETPROPS_ICONTEXT:
         {
            // 8. Usage: ISTATIC_SetProperties -- ST_ICONTEXT

			// This example shows how an animated bitmap can be used as the
			// contents of a text control instead of a text string.  After
			// reading in the bitmap from the resource file and setting some
			// bitmap parameters so that the animation will display correctly,
			// we then set the IStatic ST_ICONTEXT property to signal to SetText
			// that an animated bitmap is being used.  When we call SetText, we pass
			// it a pointer to the IImage interface for the bitmap (since ST_ICONTEXT
			// is set, SetText will expect an IImage pointer instead of a pointer
			// to a text string as its third argument).
			// NOTE: When the IStatic interface is released, the memory for the title
			// string will be freed, but the IImage interface will NOT be released.
			// You must explicitly release the IImage pointer (see the StaticApp_HandleEvent 
			// function above).
			int nCharCount;

			// Retrieve the animated bitmap from the resource file for this usage example
			pMe->m_pIImage = ISHELL_LoadResImage(pMe->a.m_pIShell, szResFile, IDB_STATIC_TEXT_BITMAP);
			if(pMe->m_pIImage)
			{
				// The animated bitmap text has four frames (404 x 20 pixels, with each frame
				// 101 x 26).  You must specify the number of frames before you can start 
				// the animation.
				IIMAGE_SetParm(pMe->m_pIImage, IPARM_NFRAMES, 4, 0);

				// Slow down the animation rate so that each line can be read
				// (default rate is 150 ms between frames; we increase it to 1000 ms)
                IIMAGE_SetParm(pMe->m_pIImage, IPARM_RATE, 1000, 0);

			    // Read in the title from the resource file
			    nCharCount = ISHELL_LoadResString(pMe->a.m_pIShell, szResFile, IDS_ICONTEXT_TITLE, pszTitle, sizeof(pszTitle));

			    // Create an instance of an IStatic text control
			    if (ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_STATIC, (void **)&(pMe->m_pIStatic)) != SUCCESS)
				   return;

			    // Set the dimensions of the rectangle for the text control
                ISTATIC_SetRect(pMe->m_pIStatic, &(pMe->m_rc));

			    // Set the property that specifies that the control text will
			    // instead be an animated bitmap
			    ISTATIC_SetProperties(pMe->m_pIStatic, ST_ICONTEXT);

			    // Set the values of the title string and animated bitmap for this control
				// (because we set the ST_ICONTEXT property above, SetText will interpret
				// its third parameter as a pointer to an IImage interface)
			    ISTATIC_SetText(pMe->m_pIStatic, pszTitle, (AECHAR *) pMe->m_pIImage, AEE_FONT_BOLD, AEE_FONT_NORMAL);

			    // Display the text control
                ISTATIC_Redraw(pMe->m_pIStatic);
			}

			// Update the display so the operations above will take effect
			IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);
         }
         break;

      case USAGE_SETPROPS_TEXTALLOC:
         {
            // 9. Usage: ISTATIC_SetProperties -- ST_TEXTALLOC

			// Ordinarily, SetText allocates storage for the text string that
			// holds the static text control's contents.  This allocation would be wasteful
			// if SetText's caller had already allocated memory for the string.
			// By setting the ST_TEXTALLOC property, you can signal to SetText that
			// you've already obtained storage for the text string (SetText will then
			// avoid the unnecessary memory allocation).
			// NOTE: ISTATIC_Release always frees the memory for the text string, regardless
			// of whether or not ST_TEXTALLOC is set.

			// Pointer to allocated memory for control text
			AECHAR * pszControlText;

			// Read in the title and long text string from the resource file
			int nCharCount = ISHELL_LoadResString(pMe->a.m_pIShell, szResFile, IDS_TEXTALLOC_TITLE, pszTitle, sizeof(pszTitle));
			nCharCount = ISHELL_LoadResString(pMe->a.m_pIShell, szResFile, IDS_TEXTALLOC_TEXT, pMe->m_pszText, TEXT_BUFFER_SIZE);

			// Allocate memory for the control text string
			pszControlText = WSTRDUP(pMe->m_pszText);

			// Create an instance of an IStatic text control
			if (ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_STATIC, (void **)&(pMe->m_pIStatic)) != SUCCESS)
				return;

			// Set the dimensions of the rectangle for the text control
			// (NOTE: you must call SetRect before calling SetText!!!)
            ISTATIC_SetRect(pMe->m_pIStatic, &(pMe->m_rc));

			// Set the property that indicates that memory has
			// already been allocated for the text string, so that
		    // SetText need not do so
			ISTATIC_SetProperties(pMe->m_pIStatic, ST_TEXTALLOC);

			// Set the values of the title and text strings for this control
			// (notice that we use the pointer to the allocated memory for the control
			// text string).
			// (NOTE: The memory allocated for the text string will be freed when we reset 
			// or release the static text control pointed to by pMe->m_pIStatic)
			ISTATIC_SetText(pMe->m_pIStatic, pszTitle, pszControlText, AEE_FONT_BOLD, AEE_FONT_NORMAL);

			// Display the text control
            ISTATIC_Redraw(pMe->m_pIStatic);
         }
         break;

      case USAGE_SETPROPS_TITLEALLOC:
         {
            // 10. Usage: ISTATIC_SetProperties -- ST_TITLEALLOC

			// Ordinarily, SetText allocates storage for the text string that
			// holds the static text control's title.  This allocation would be wasteful
			// if SetText's caller had already allocated memory for the string.
			// By setting the ST_TITLEALLOC property, you can signal to SetText that
			// you've already obtained storage for the title text string (SetText will then
			// avoid the unnecessary memory allocation).
			// NOTE: ISTATIC_Release always frees the memory for the title string, regardless
			// of whether or not ST_TITLEALLOC is set.

			// Pointer to allocated memory for control title
			AECHAR * pszControlTitle;

			// Read in the title and long text string from the resource file
			int nCharCount = ISHELL_LoadResString(pMe->a.m_pIShell, szResFile, IDS_TITLEALLOC_TITLE, pszTitle, sizeof(pszTitle));
			nCharCount = ISHELL_LoadResString(pMe->a.m_pIShell, szResFile, IDS_TITLEALLOC_TEXT, pMe->m_pszText, TEXT_BUFFER_SIZE);

			// Allocate memory for the control title string
			pszControlTitle = WSTRDUP(pszTitle);

			// Create an instance of an IStatic text control
			if (ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_STATIC, (void **)&(pMe->m_pIStatic)) != SUCCESS)
				return;

			// Set the dimensions of the rectangle for the text control
			// (NOTE: you must call SetRect before calling SetText!!!)
            ISTATIC_SetRect(pMe->m_pIStatic, &(pMe->m_rc));

			// Set the property that indicates that memory has
			// already been allocated for the title string, so that
		    // SetText need not do so
			ISTATIC_SetProperties(pMe->m_pIStatic, ST_TITLEALLOC);

			// Set the values of the title and text strings for this control
			// (notice that we use the pointer to the allocated memory for the control
			// title string).
			// (NOTE: The memory allocated for the title string will be freed when we reset 
			// or release the static text control pointed to by pMe->m_pIStatic)
			ISTATIC_SetText(pMe->m_pIStatic, pszControlTitle, pMe->m_pszText, AEE_FONT_BOLD, AEE_FONT_NORMAL);

			// Display the text control
            ISTATIC_Redraw(pMe->m_pIStatic);
         }
         break;

      case USAGE_SETPROPS_NOSCROLL:
          {
            // 11. Usage: ISTATIC_SetProperties -- ST_NOSCROLL

		    // By default, a static text control will continuously scroll a text
			// message that is too large to fit completely within its rectangle.
			// The ST_NOSCROLL property disables this automatic scrolling.
			// (NOTE: In later releases, setting ST_NOSCROLL will allow the text
			// to be manually scrolled by the handset user with the UP and DOWN
			// keys.  Currently, ST_NOSCROLL is not very useful, since it prevents
			// long messages from being completely read.)

			// Read in the title and long text string from the resource file
			int nCharCount = ISHELL_LoadResString(pMe->a.m_pIShell, szResFile, IDS_NOSCROLL_TITLE, pszTitle, sizeof(pszTitle));
			nCharCount = ISHELL_LoadResString(pMe->a.m_pIShell, szResFile, IDS_LONG_TEXT_TEXT, pMe->m_pszText, TEXT_BUFFER_SIZE);

			// Create an instance of an IStatic text control
			if (ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_STATIC, (void **)&(pMe->m_pIStatic)) != SUCCESS)
				return;

			// Set the dimensions of the rectangle for the text control
            ISTATIC_SetRect(pMe->m_pIStatic, &(pMe->m_rc));

			// Set the property that disables auto-scrolling
			ISTATIC_SetProperties(pMe->m_pIStatic, ST_NOSCROLL);

			// Set the values of the title and text strings for this control
			ISTATIC_SetText(pMe->m_pIStatic, pszTitle, pMe->m_pszText, AEE_FONT_BOLD, AEE_FONT_NORMAL);

			// Display the text control
            ISTATIC_Redraw(pMe->m_pIStatic);
          }
         break;

      case USAGE_SETPROPS_MIDDLETEXT:
         {
            // 12. Usage: ISTATIC_SetProperties -- ST_MIDDLETEXT

			// The ST_MIDDLETEXT property vertically centers short messages on the screen,
			// so that they are displayed with an equal amount of blank space above and below
			// (by default, the message text begins at the top, immediately below the control
			// title).  This property has no effect for animated bitmaps (which are vertically
			// centered by default) or for long scrolling text messages.

			// Read in the title and text string from the resource file
			int nCharCount = ISHELL_LoadResString(pMe->a.m_pIShell, szResFile, IDS_MIDDLETEXT_TITLE, pszTitle, sizeof(pszTitle));
			nCharCount = ISHELL_LoadResString(pMe->a.m_pIShell, szResFile, IDS_MIDDLETEXT_TEXT, pMe->m_pszText, TEXT_BUFFER_SIZE);

			// Create an instance of an IStatic text control
			if (ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_STATIC, (void **)&(pMe->m_pIStatic)) != SUCCESS)
				return;

			// Set the dimensions of the rectangle for the text control
            ISTATIC_SetRect(pMe->m_pIStatic, &(pMe->m_rc));

			// Set the property that specifies vertical centering of text
			ISTATIC_SetProperties(pMe->m_pIStatic, ST_MIDDLETEXT);

			// Set the values of the title and text strings for this control
			ISTATIC_SetText(pMe->m_pIStatic, pszTitle, pMe->m_pszText, AEE_FONT_BOLD, AEE_FONT_NORMAL);

			// Display the text control
            ISTATIC_Redraw(pMe->m_pIStatic);
         }
         break;

      case USAGE_SETPROPS_UNDERLINE:
         {
            // 13. Usage: ISTATIC_SetProperties -- ST_UNDERLINE

			// The ST_UNDERLINE property specifies that the title of the control
			// is to be underlined (note that the horizontal line produced by
			// setting this property spans the entire width of the control rectangle,
			// not just the width of the control text).

			// Read in the title and text string from the resource file
			int nCharCount = ISHELL_LoadResString(pMe->a.m_pIShell, szResFile, IDS_UNDERLINE_TITLE, pszTitle, sizeof(pszTitle));
			nCharCount = ISHELL_LoadResString(pMe->a.m_pIShell, szResFile, IDS_UNDERLINE_TEXT, pMe->m_pszText, TEXT_BUFFER_SIZE);

			// Create an instance of an IStatic text control
			if (ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_STATIC, (void **)&(pMe->m_pIStatic)) != SUCCESS)
				return;

			// Set the dimensions of the rectangle for the text control
            ISTATIC_SetRect(pMe->m_pIStatic, &(pMe->m_rc));

			// Set the property that specifies underlining of control title
			ISTATIC_SetProperties(pMe->m_pIStatic, ST_UNDERLINE);

			// Set the values of the title and text strings for this control
			ISTATIC_SetText(pMe->m_pIStatic, pszTitle, pMe->m_pszText, AEE_FONT_BOLD, AEE_FONT_NORMAL);

			// Display the text control
            ISTATIC_Redraw(pMe->m_pIStatic);

         }
         break;

      case USAGE_SETPROPS_CENTERTEXT:
         {
            // 14. Usage: ISTATIC_SetProperties -- ST_CENTERTEXT

			// The ST_CENTERTEXT property specifies that each line of text
			// will be horizontally centered on the screen (by default, each
			// line is left-justified).  This property has no effect for animated
			// bitmaps (which are horizonally centered by default).

			// Read in the title and text string from the resource file
			int nCharCount = ISHELL_LoadResString(pMe->a.m_pIShell, szResFile, IDS_CENTERTEXT_TITLE, pszTitle, sizeof(pszTitle));
			nCharCount = ISHELL_LoadResString(pMe->a.m_pIShell, szResFile, IDS_CENTERTEXT_TEXT, pMe->m_pszText, TEXT_BUFFER_SIZE);

			// Create an instance of an IStatic text control
			if (ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_STATIC, (void **)&(pMe->m_pIStatic)) != SUCCESS)
				return;

			// Set the dimensions of the rectangle for the text control
            ISTATIC_SetRect(pMe->m_pIStatic, &(pMe->m_rc));
            
			// Set the property that specifies centering of control text lines
			ISTATIC_SetProperties(pMe->m_pIStatic, ST_CENTERTEXT);
			
			// Set the values of the title and text strings for this control
			ISTATIC_SetText(pMe->m_pIStatic, pszTitle, pMe->m_pszText, AEE_FONT_BOLD, AEE_FONT_NORMAL);

			// Display the text control
            ISTATIC_Redraw(pMe->m_pIStatic);
         }
         break;

      case USAGE_SETPROPS_CENTERTITLE:
         {
            // 15. Usage: ISTATIC_SetProperties -- ST_CENTERTITLE

			// The ST_CENTERTITLE property specifies that the control title
			// will be horizontally centered on the screen (by default, it
			// is left-justified).  

			// Read in the title and text string from the resource file
			int nCharCount = ISHELL_LoadResString(pMe->a.m_pIShell, szResFile, IDS_CENTERTITLE_TITLE, pszTitle, sizeof(pszTitle));
			nCharCount = ISHELL_LoadResString(pMe->a.m_pIShell, szResFile, IDS_CENTERTITLE_TEXT, pMe->m_pszText, TEXT_BUFFER_SIZE);

			// Create an instance of an IStatic text control
			if (ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_STATIC, (void **)&(pMe->m_pIStatic)) != SUCCESS)
				return;

			// Set the dimensions of the rectangle for the text control
            ISTATIC_SetRect(pMe->m_pIStatic, &(pMe->m_rc));

			// Set the property that specifies underlining of control title
			ISTATIC_SetProperties(pMe->m_pIStatic, ST_CENTERTITLE);

			// Set the values of the title and text strings for this control
			ISTATIC_SetText(pMe->m_pIStatic, pszTitle, pMe->m_pszText, AEE_FONT_BOLD, AEE_FONT_NORMAL);

			// Display the text control
            ISTATIC_Redraw(pMe->m_pIStatic);
         }
         break;

      case USAGE_GETPROPS:
         {
            // 16. Usage: ISTATIC_GetProperties

			// The GetProperties function retrieves the bit vector that holds
			// the control's property settings.  To illustrate the use of
			// this function, we set some control properties using SetProperties,
			// use GetProperties to access the property bit vector, and then test
			// each bit of the vector to determine which properties are set, echoing
			// the results to the device screen (note that all properties are unset
			// by default).

			// Variable that will hold properties bitmap returned 
			// by GetProperties
			uint32 props;

			uint16 nLineNum = 3;

			// Create an instance of an IStatic text control
			if (ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_STATIC, (void **)&(pMe->m_pIStatic)) != SUCCESS)
				return;

			// Set the dimensions of the rectangle for the text control
            ISTATIC_SetRect(pMe->m_pIStatic, &(pMe->m_rc));

			// Set a few properties of the control
			ISTATIC_SetProperties(pMe->m_pIStatic, ST_CENTERTITLE | ST_MIDDLETEXT | ST_UNDERLINE);

			// Now read back the property settings using GetProperties
			// (the value of the control's property bit vector is returned
			// by the call to GetProperties)
			props = ISTATIC_GetProperties(pMe->m_pIStatic);

			// Echo back to the screen the properties that are set
            DisplayOutput(pMe, 2, "Properties Set:");

			// Test if ICONTEXT property is set
			if (props & ST_ICONTEXT)
               DisplayOutput(pMe, nLineNum++, "   ICONTEXT");

			// Test if TEXTALLOC property is set
			if (props & ST_TEXTALLOC)
               DisplayOutput(pMe, nLineNum++, "   TEXTALLOC");

			// Test if TITLEALLOC property is set
			if (props & ST_TITLEALLOC)
               DisplayOutput(pMe, nLineNum++, "   TITLEALLOC");

			// Test if NOSCROLL property is set
			if (props & ST_NOSCROLL)
               DisplayOutput(pMe, nLineNum++, "   NOSCROLL");

			// Test if MIDDLETEXT property is set
			if (props & ST_MIDDLETEXT)
               DisplayOutput(pMe, nLineNum++, "   MIDDLETEXT");

			// Test if UNDERLINE property is set
			if (props & ST_UNDERLINE)
               DisplayOutput(pMe, nLineNum++, "   UNDERLINE");

			// Test if CENTERTEXT property is set
			if (props & ST_CENTERTEXT)
               DisplayOutput(pMe, nLineNum++, "   CENTERTEXT");

			// Test if CENTERTITLE property is set
			if (props & ST_CENTERTITLE)
               DisplayOutput(pMe, nLineNum++, "   CENTERTITLE");
         }
         break;

      case USAGE_RESET:
         {
            // 17. Usage: ISTATIC_Reset

			// Read in the first title and text string from the resource file
			int nCharCount = ISHELL_LoadResString(pMe->a.m_pIShell, szResFile, IDS_CENTERTEXT_TITLE, pszTitle, sizeof(pszTitle));
			nCharCount = ISHELL_LoadResString(pMe->a.m_pIShell, szResFile, IDS_CENTERTEXT_TEXT, pMe->m_pszText, TEXT_BUFFER_SIZE);

			// Create an instance of an IStatic text control
			if (ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_STATIC, (void **)&(pMe->m_pIStatic)) != SUCCESS)
				return;

			// Set the dimensions of the rectangle for the text control
            ISTATIC_SetRect(pMe->m_pIStatic, &(pMe->m_rc));
            
			// Set the property that specifies centering of control text lines
			ISTATIC_SetProperties(pMe->m_pIStatic, ST_CENTERTEXT);
			
			// Set the values of the title and text strings for this control
			ISTATIC_SetText(pMe->m_pIStatic, pszTitle, pMe->m_pszText, AEE_FONT_BOLD, AEE_FONT_NORMAL);

			// Now reset the control.  This will clear all the previous text, settings, etc.
			// associated with the control and allow us to re-use the interface with a
			// different set of contents
            ISTATIC_Reset(pMe->m_pIStatic); 

			// Read in the new title and text string from the resource file
			nCharCount = ISHELL_LoadResString(pMe->a.m_pIShell, szResFile, IDS_RESET_TITLE, pszTitle, sizeof(pszTitle));
			nCharCount = ISHELL_LoadResString(pMe->a.m_pIShell, szResFile, IDS_RESET_TEXT, pMe->m_pszText, TEXT_BUFFER_SIZE);
			
			// Set the dimensions of the rectangle for the text control
            ISTATIC_SetRect(pMe->m_pIStatic, &(pMe->m_rc));

			// Clear the properties associated with the control
			ISTATIC_SetProperties(pMe->m_pIStatic, 0);

			// Set the values of the title and text strings for this control
			ISTATIC_SetText(pMe->m_pIStatic, pszTitle, pMe->m_pszText, AEE_FONT_BOLD, AEE_FONT_NORMAL);
			
			// Display the text control
            ISTATIC_Redraw(pMe->m_pIStatic);
         }
         break;

      case USAGE_SETTEXT:
         {
            // 18. Usage: ISTATIC_SetText

            // The SetText function specifies the title text string and control 
			// contents (text string or animated bitmap) and the fonts to be used
			// for the title and text message.
			// Before calling SetText, you must have pointers to text strings that
			// contain the control's title and text message (if you are displaying an
			// animated bitmap instead of the control text, you must have a pointer to
			// an instance of an IImage interface that contains the bitmap; see example
			// above for ST_ICONTEXT property).

			// In this example, we read in the title and text strings from the resource file
			int nCharCount = ISHELL_LoadResString(pMe->a.m_pIShell, szResFile, IDS_LONG_TEXT_TITLE, pszTitle, sizeof(pszTitle));
			nCharCount = ISHELL_LoadResString(pMe->a.m_pIShell, szResFile, IDS_LONG_TEXT_TEXT, pMe->m_pszText, TEXT_BUFFER_SIZE);

			// Create an instance of an IStatic text control
			if (ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_STATIC, (void **)&(pMe->m_pIStatic)) != SUCCESS)
				return;

			// Set the dimensions of the rectangle for the text control
			// (NOTE: You must call SetRect before calling SetText, since SetText
			// uses information about the control's rectangle to determine how the
			// control will be displayed)
            ISTATIC_SetRect(pMe->m_pIStatic, &(pMe->m_rc));

			// Set the values of the title and text strings for this control (if we were using
			// an animated bitmap, the third parameter would be an IImage interface pointer).
			// Note that ISTATIC_SetText returns FALSE if it fails to set the text and title correctly.
			if (ISTATIC_SetText(pMe->m_pIStatic, pszTitle, pMe->m_pszText, AEE_FONT_BOLD, AEE_FONT_NORMAL) == FALSE)
				return;

			// Display the text control
            ISTATIC_Redraw(pMe->m_pIStatic);
         }
         break;

      default:
         return;  
   }      
   
   // Display above event. 
   DisplayEvent (pMe, wParam);

   return;
}
/*===========================================================================

FUNCTION: DisplayEvent

DESCRIPTION
    This function prints a heading at the top 
	of the screen to indicate which usage example was selected by the user, 
	It also prints a message on the bottom two lines of the screen instructing 
	the user to press the UP or DOWN key to return to
	the main menu to execute another example.
    
PROTOTYPE:
   static void DisplayEvent(CIStaticApp * pMe, uint16 wParam)

PARAMETERS:
   pMe:   [in]: Contains a pointer to the usage app data structure.
   wParam: [in]: Identifier of the selected example (example IDs are specified when the
                 main usage app menu is created).  If the value USAGE_ERASE_SCREEN is supplied
				 instead, this function clears the screen.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void DisplayEvent (CIStaticApp *pMe, uint16 wParam)
{
   AEEDeviceInfo di; // Device Info
   AEERect rc; 
   AEERect qrc;

   // Make sure the pointers we'll be using are valid
   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
	   return;

   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);

   qrc.x = 0;
   qrc.y = 0;
   qrc.dx = di.cxScreen;
   qrc.dy = di.cyScreen;

   rc.x = 5;
   rc.y = 5;
   rc.dx = di.cxScreen - 5;
   rc.dy = di.cyScreen - 5;

   switch (wParam)
   {
      case USAGE_ERASE_SCREEN:
         IDISPLAY_EraseRect(pMe->a.m_pIDisplay, &qrc);  //Clear the display
         return;
      case USAGE_BASIC_USAGE:
         STR_TO_WSTR("Basic Usage", pMe->m_pszText, TEXT_BUFFER_SIZE);
         break;
      case USAGE_HANDLEEVENT:
         STR_TO_WSTR("HandleEvent", pMe->m_pszText, TEXT_BUFFER_SIZE);
         break;
      case USAGE_REDRAW:
         STR_TO_WSTR("Redraw", pMe->m_pszText, TEXT_BUFFER_SIZE);
         break;
      case USAGE_SETACTIVE:
         STR_TO_WSTR("SetActive", pMe->m_pszText, TEXT_BUFFER_SIZE);
         break;
      case USAGE_ISACTIVE:
         STR_TO_WSTR("IsActive", pMe->m_pszText, TEXT_BUFFER_SIZE);
         break;
      case USAGE_SETRECT:
         STR_TO_WSTR("SetRect", pMe->m_pszText, TEXT_BUFFER_SIZE);
         break;
      case USAGE_GETRECT:
         STR_TO_WSTR("GetRect", pMe->m_pszText, TEXT_BUFFER_SIZE);
         break;
      case USAGE_SETPROPS_ICONTEXT:
         STR_TO_WSTR("SetP ICONTEXT", pMe->m_pszText, TEXT_BUFFER_SIZE);
         break;
      case USAGE_SETPROPS_TEXTALLOC:
         STR_TO_WSTR("SetP TEXTALLOC", pMe->m_pszText, TEXT_BUFFER_SIZE);
         break;
      case USAGE_SETPROPS_TITLEALLOC:
         STR_TO_WSTR("SetP TITLEALLOC", pMe->m_pszText, TEXT_BUFFER_SIZE);
         break;
      case USAGE_SETPROPS_NOSCROLL:
         STR_TO_WSTR("SetP NOSCROLL", pMe->m_pszText, TEXT_BUFFER_SIZE);
         break;
      case USAGE_SETPROPS_MIDDLETEXT:
         STR_TO_WSTR("SetP MIDDLETEXT", pMe->m_pszText, TEXT_BUFFER_SIZE);
         break;
      case USAGE_SETPROPS_UNDERLINE:
         STR_TO_WSTR("SetP UNDERLINE", pMe->m_pszText, TEXT_BUFFER_SIZE);
         break;
      case USAGE_SETPROPS_CENTERTEXT:
         STR_TO_WSTR("SetP CENTERTEXT", pMe->m_pszText, TEXT_BUFFER_SIZE);
         break;
      case USAGE_SETPROPS_CENTERTITLE:
         STR_TO_WSTR("SetP CENTERTITLE", pMe->m_pszText, TEXT_BUFFER_SIZE);
         break;
      case USAGE_GETPROPS:
         STR_TO_WSTR("GetProps", pMe->m_pszText, TEXT_BUFFER_SIZE);
         break;
      case USAGE_RESET:
         STR_TO_WSTR("Reset", pMe->m_pszText, TEXT_BUFFER_SIZE);
         break;
      case USAGE_SETTEXT:
         STR_TO_WSTR("SetText", pMe->m_pszText, TEXT_BUFFER_SIZE);
         break;
      default:
         STR_TO_WSTR("Unsupported fn", pMe->m_pszText, TEXT_BUFFER_SIZE);
   }
   
   // Since controls have their own titles, and to allow example displays to fit
   // on phone screen, we suppress printing of title in this example
   //IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,10, 5, &rc,
      //IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);

   // Functions with no output
   switch (wParam)
   {
	  // case USAGE_BASIC_USAGE:
      case USAGE_HANDLEEVENT:  // not supported this release
      // case USAGE_REDRAW:
      case USAGE_SETACTIVE: // not supported this release
      case USAGE_ISACTIVE:  // not supported this release
      // case USAGE_SETRECT:
      // case USAGE_GETRECT:
      // case USAGE_SETPROPS_ICONTEXT:
      // case USAGE_SETPROPS_TEXTALLOC:
      // case USAGE_SETPROPS_TITLEALLOC:
      // case USAGE_SETPROPS_NOSCROLL:
      // case USAGE_SETPROPS_MIDDLETEXT:
      // case USAGE_SETPROPS_UNDERLINE:
      // case USAGE_SETPROPS_CENTERTITLE:
      // case USAGE_GETPROPS:
      // case USAGE_RESET:  
      // case USAGE_SETTEXT:
         // Place in the middle of screen
         STR_TO_WSTR("No Output...", pMe->m_pszText, TEXT_BUFFER_SIZE);
 		 IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, pMe->m_pszText, -1,10, 
            qrc.dy/2 - 10, &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
      default:
         break;
   }

   STR_TO_WSTR("Press Up/Down", pMe->m_pszText, TEXT_BUFFER_SIZE);
   IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, pMe->m_pszText, -1,10, qrc.dy - 25, &rc,	   
	   IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
   STR_TO_WSTR("keys to continue..", pMe->m_pszText, TEXT_BUFFER_SIZE);
   IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, pMe->m_pszText, -1,10, qrc.dy - 15, &rc,
	  IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
   IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);
}
/*===========================================================================

FUNCTION: BuildMainMenu

DESCRIPTION
    This function constructs the menu control that allows the user to
	select a usage example to execute.  The command ID for each menu item
	is passed to the usage example function above when the user selects the
	menu item for that example.
    
PROTOTYPE:
   static void BuildMainMenu(CIStaticApp * pMe)

PARAMETERS:
   pMe:   [in]: Contains a pointer to the usage app data structure.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void BuildMainMenu(CIStaticApp *pMe)
{
   AEERect qrc;
   AEEDeviceInfo di;

   // Make sure the pointers we'll be using are valid
   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->m_pIMenu == NULL)
	   return;

   // Set Title
   STR_TO_WSTR("IStatic Examples", pMe->m_pszText, TEXT_BUFFER_SIZE);
   IMENUCTL_SetTitle(pMe->m_pIMenu, NULL, 0, pMe->m_pszText);

	// Set the Rectangle for the Menu
	ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);
	qrc.x = 0;
	qrc.y = 0;
	qrc.dx = di.cxScreen;
	qrc.dy = di.cyScreen;
	IMENUCTL_SetRect(pMe->m_pIMenu, &qrc);  

	// Add individual entries to the Menu

   // Add Basic Usage to menu
   STR_TO_WSTR("1. Basic Usage", pMe->m_pszText, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_BASIC_USAGE, pMe->m_pszText, 0);

   // Add Handle Event to menu
   STR_TO_WSTR("2. HandleEvent", pMe->m_pszText, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_HANDLEEVENT, pMe->m_pszText, 0);

   // Add Redraw to menu
   STR_TO_WSTR("3. Redraw", pMe->m_pszText, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_REDRAW, pMe->m_pszText, 0);
			
   // Add SetActive to menu
   STR_TO_WSTR("4. SetActive", pMe->m_pszText, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETACTIVE, pMe->m_pszText, 0);

   // Add IsActive to menu
   STR_TO_WSTR("5. IsActive", pMe->m_pszText, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_ISACTIVE, pMe->m_pszText, 0);

	// Add SetRect to menu
   STR_TO_WSTR("6. SetRect", pMe->m_pszText, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETRECT, pMe->m_pszText, 0);

	// Add GetRect to menu
   STR_TO_WSTR("7. GetRect", pMe->m_pszText, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETRECT, pMe->m_pszText, 0);	

	// Add SetProperties ICONTEXT to menu
   STR_TO_WSTR("8. SetP ICONTEXT", pMe->m_pszText, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPROPS_ICONTEXT, pMe->m_pszText, 0);	

	// Add SetProperties TEXTALLOC to menu
   STR_TO_WSTR("9. SetP TEXTALLOC", pMe->m_pszText, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPROPS_TEXTALLOC, pMe->m_pszText, 0);	

	// Add SetProperties TITLEALLOC to menu
   STR_TO_WSTR("10. SetP TITLEALLOC", pMe->m_pszText, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPROPS_TITLEALLOC, pMe->m_pszText, 0);	

   // Add SetProperties NOSCROLL to menu
   STR_TO_WSTR("11. SetP NOSCROLL", pMe->m_pszText, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPROPS_NOSCROLL, pMe->m_pszText, 0);	

   // Add SetProperties MIDDLETEXT to menu
   STR_TO_WSTR("12. SetP MIDDLETEXT", pMe->m_pszText, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPROPS_MIDDLETEXT, pMe->m_pszText, 0);	

   // Add SetProperties UNDERLINE to menu
   STR_TO_WSTR("13. SetP UNDERLINE", pMe->m_pszText, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPROPS_UNDERLINE, pMe->m_pszText, 0);	

   // Add SetProperties CENTERTEXT to menu
   STR_TO_WSTR("14. SetP CENTERTEXT", pMe->m_pszText, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPROPS_CENTERTEXT, pMe->m_pszText, 0);	

   // Add SetProperties CENTERTITLE to menu
   STR_TO_WSTR("15. SetP CENTERTITLE", pMe->m_pszText, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPROPS_CENTERTITLE, pMe->m_pszText, 0);	

	// Add GetProperties to menu
   STR_TO_WSTR("16. GetProps", pMe->m_pszText, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETPROPS, pMe->m_pszText, 0);	

	// Add Reset to menu
   STR_TO_WSTR("17. Reset", pMe->m_pszText, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_RESET, pMe->m_pszText, 0);	

	// Add SetText to menu
   STR_TO_WSTR("18. SetText", pMe->m_pszText, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETTEXT, pMe->m_pszText, 0);	

   // Activate menu
   IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);
}
/*===========================================================================

FUNCTION: DisplayOutput

DESCRIPTION
    This function displays an output string at a given line number on the
    screen. If the nline parameter is a negative value (-1) the string
    is displayed in the middle of the screen. If the "nline" value is larger
    than or equal to zero the "nline" value is multiplied by 15 and the 
    resulting value in pixels is set to the y-coordinate of the start of 
    the string display on the screen. If the string does not fit on one line
    the string wraps around to the next line (spaced rougly 10-15 pixels apart).
    By default 5 is used as the starting the x-coordinate of a displayed 
    string.

    How many characters that fit on one line is calculated for each line 
    that is wrapped around to the next line.

    Note: depending on the phone screen size and the fonts used for characters 
          the output might differ on different handsets (devices). Where some 
          handsets will have a smaller screen and large default fonts which will 
          cause partial overlapping of lines. This function does not try to address
          these issues (this is meant as a simple display function).
    
PROTOTYPE:
   void DisplayOutput(CIStaticApp *pMe, int nline, char *pszStr)

PARAMETERS:
   pMe   [in]: Contains a pointer to this app's data structure
   nline:     [in]: Contains the line number to start displaying the text. The line
                    numbers are by default spaced 15 pixels apart along the y-axis.
   pszStr:    [in]: The character string to be displayed on the screen.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void DisplayOutput(CIStaticApp *pMe, int nline, char *pszStr)
{
   IShell   * pIShell   = pMe->a.m_pIShell;
   IDisplay * pIDisplay = pMe->a.m_pIDisplay;
   AEEDeviceInfo di; // Device Info
   AECHAR * psz = NULL;
   int charHeight = 0;      // Stores the char height in pixels for given font
   int pnAscent = 0;        // Stores the ascent in number of pixels
   int pnDescent = 0;       // Stores the descent in number of pixels
   int pixelWidth;
   AEEFont font = AEE_FONT_NORMAL;
   int pnFits = 0, dy;
   int totalCh = 0;

   // Get device information
   ISHELL_GetDeviceInfo(pIShell,&di);

   // Get the font metrics info
   charHeight = IDISPLAY_GetFontMetrics (pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   // Convert to wide string (unicode)
   STR_TO_WSTR ((char *)pszStr, pMe->m_pszText, TEXT_BUFFER_SIZE);

   // If nlines is zero then print this string starting around the middle of 
   // the screen. Or else multiply nlines by charheight to decide the y coordinate of
   // the start of the string.
   if (nline < 0) {
      dy = di.cyScreen*2/5;
   }
   else{
      dy = nline * charHeight;
   }

   // psz keeps track of the point from which to write from the string buffer
   // in case the string does not fit one line and needs to wrap around in the
   // next line.
   psz = pMe->m_pszText;
      
   // Need to calculate the lotal string length to decide if any wrapping
   // around is needed.
   totalCh = STRLEN ((char *)pszStr);

   // Keep displaying text string on multiple lines if the string can't be displayed
   // on one single line. Lines are spaced 15 pixels apart.
   while ((totalCh > 0) && (*psz != NULL))
   { 
      // Get information on how many characters will fit in a line.
      // Give the pointer to the buffer to be displayed, and the number of
      // pixels along the x axis you want to display the string in (max number)
      // pnFits will have the max number of chars that will fit in the maxWidth
      // number of pixels (given string can't fit in one line), or the number of 
      // chars in the string (if it does fit in one line). pnWidth gives the
      // number of pixels that will be used to display pnFits number of chars.
      pixelWidth = IDISPLAY_MeasureTextEx(pIDisplay,
                      font, 
                      (AECHAR *) psz,  // Start of the buffer to display,
                      -1,
                      di.cxScreen - 5, // maxWidth
                      &pnFits);         // Number of chars that will fit a line

      // If pnFits is zero there is something wrong in the input to above function. 
      // Normally this scenario should not occur. But, have the check anyway.
      if (pnFits == 0)
         return;

      IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, psz, pnFits, 5 /*start dx*/, 
         dy, 0 /* use default rectangle coordinates */, 0);

      psz += pnFits;      // move pointer to the next segment to be displayed
      totalCh -= pnFits;  // reduce the total number of characters to still display
      dy += charHeight;   // Place next line charHeight pixels below the 
                          // previous line.
      IDISPLAY_Update(pIDisplay); //, TRUE);
      if (totalCh < pnFits)
         pnFits = totalCh;  // if total number is less than pnFits, adjust pnFits
   }

      return;   
} // End of DisplayOutput
