/*===========================================================================

FILE: itapiusage.c

SERVICES: Sample applet using AEE

DESCRIPTION
    This file contains usage examples of the ITapi interface. ITAPI is a simple 
    interface to the telephony layer in the device. This function provides the 
    following services:
        Retrieving status
        Placing voice calls
        Extracting SMS text from SMS messages
        Obtaining caller ID on incoming or in-progress calls

PUBLIC CLASSES:  
   N/A


   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/

/*===============================================================================
                     INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
// Mandatory includes
#include "AEEAppGen.h"	    // AEEApplet declaration
#include "AEEUsageAppIDs.h" // Class IDs of usage apps (including this one)
#include "AEE.h"			// Standard AEE Declarations
#include "AEEShell.h"	    // AEE Shell Services
#include "AEEDisp.h"		// AEE Display Services

// Services used by this app
#include "AEETapi.h"		//	AEETapi Services
#include "AEEMenu.h"		// Menu Services
#include "AEEStdLib.h"      // AEE StdLib services
#include "AEEFile.h"        // File Services
#include "itapiusage.bid"
/*===========================================================================

                      PUBLIC DATA DECLARATIONS

===========================================================================*/

/*-------------------------------------------------------------------
            Type Declarations
-------------------------------------------------------------------*/
// ITapi app struct. This is the main struct for this applet. This will hold 
// all the data members that need to be remembered throughout the life of
// the applet. 
// THE FIRST DATA MEMBER OF THIS STRUCT MUST BE AN AEEApplet OBJECT. 
// This is critical requirement which is related to an applet being
// dynamically loaded. The app developers should add their data members
// following the AEEApplet data member.
typedef struct _CITapiApp {
   AEEApplet	        a;
   IMenuCtl *           m_pIMenu;
   AEEDeviceInfo        di;
   char                 charbuffer[SMS_MAX_DATA+1];
   AECHAR*              aecharbuffer;
	boolean					m_bSMSMessageStart;
} CITapiApp;

/*-------------------------------------------------------------------
            Function Prototypes
-------------------------------------------------------------------*/
// Mandatory function:
// App Handle Event function
static boolean TapiApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);

// App specific data alloc-init/free functions
static boolean ITapi_InitAppData(IApplet* pMe);
static void ITapi_FreeAppData(IApplet* pMe);

static void BuildMainMenu(CITapiApp *pMe);
static void ITapiUsage (CITapiApp *pMe, uint16 wParam, uint32 dwParam);
static void DisplayEvent (CITapiApp *pMe, uint16 wParam);
static void DisplayOutput(IShell *pIShell, IDisplay *pIDisplay, int nline, char *pszStr);
static void StoreLastCallerID(CITapiApp* pMe);
static void FetchLastCallerID(CITapiApp* pMe);
/*-------------------------------------------------------------------
            Global Constant Definitions
-------------------------------------------------------------------*/

#define EVT_SMS_TEXT_ARRIVED    EVT_USER + 0x1
#define EVT_CALL_ARRIVED        EVT_USER + 0x2

// App specific constants
#define USAGE_EXTRACTSMSTEXT    100
#define USAGE_GETCALLERID       101
#define USAGE_GETSTATUS         102
#define USAGE_MAKEVOICECALL     103
#define USAGE_ERASE_SCREEN      104   // Special case to erase screen

// Size of buffer used to hold text strings
#define TEXT_BUFFER_SIZE (100 * sizeof(AECHAR))
#define CALLERID_BUFFER_SIZE    32

// Phone number to be called by ITAPI_MakeVoiceCall
#define DEMO_PHONE_NUMBER       "8588453759"

/*-------------------------------------------------------------------
            Global Data Definitions
-------------------------------------------------------------------*/


/*-------------------------------------------------------------------
            Static variable Definitions
-------------------------------------------------------------------*/


/*===============================================================================
                     FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to ITapi_InitAppData(). This is done as follows:
   ITapi_InitAppData() is called to initialize AppletData instance. It is app developers 
   responsibility to fill-in app data initialization code of ITapi_InitAppData(). App developer
   is also responsible to release memory allocated for data contained in AppletData -- this can be
   done in ITapi_FreeAppData().

PROTOTYPE:
	boolean AEEClsCreateInstance(AEECLSID ClsId, IShell* pIShell, IModule* pIModule, 
										  void** ppObj)

PARAMETERS:
	ClsId: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell interface. 

	pIModule: pin]: Contains pointer to the IModule interface to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid AEEApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app needs to be loaded and if AEEApplet_New() invocation was successful
  FALSE: If the app does not need to be loaded or if errors occurred in AEEApplet_New().
  If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId, IShell* pIShell, IModule* po, void** ppObj)
{
   *ppObj = NULL;
		
   // We want to load this App. So, invoke AEEApplet_New().To it, pass the
   // address of the app-specific handle event function.
   if(ClsId == AEECLSID_ITAPI_USAGE){
      if(AEEApplet_New(sizeof(CITapiApp), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)TapiApp_HandleEvent,(PFNFREEAPPDATA)ITapi_FreeAppData) == TRUE)
      {
         if(ITapi_InitAppData((IApplet*)*ppObj))
             return(AEE_SUCCESS);
      }
   }
	return (EFAILED);
}

/*===========================================================================

FUNCTION TapiApp_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. 

PROTOTYPE:
	static boolean App_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet.

	ecode: Specifies the Event sent to this applet

    wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/

static boolean TapiApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
   CITapiApp * pMe = (CITapiApp*)pi;

   switch (eCode) 
	{
        case EVT_NOTIFY:
            {
                ITAPI* pITapi = NULL;
                AEENotify* pNotify = (AEENotify*) dwParam;
                if (pNotify && (pNotify->cls == AEECLSID_TAPI))
                {
                    if ((pNotify->dwMask & NMASK_TAPI_STATUS) == NMASK_TAPI_STATUS) 
                    {
                        TAPIStatus* ptpst = (TAPIStatus*) pNotify->pData;
                        if (ptpst->state == PS_INCOMING)
                        {
			                // Create an instance of the ITapi interface
			                ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_TAPI,(void **)&pITapi);
                            if (pITapi)
                            {
                                if (ITAPI_GetCallerID(pITapi, pMe->aecharbuffer, CALLERID_BUFFER_SIZE))
                                    StoreLastCallerID(pMe);
                                // Free the instance of the ITapi interface
			                    ITAPI_Release(pITapi);
                            }
                        }
                    }
                }
            }
            return TRUE;

        case EVT_APP_MESSAGE:	  
				pMe->m_bSMSMessageStart = TRUE;				
            STRCPY(pMe->charbuffer, (char*)dwParam);
				ISHELL_StartApplet(pMe->a.m_pIShell, AEECLSID_ITAPI_USAGE);
				return TRUE;

        case EVT_APP_START:
			// When this applet is started, try to create the main menu
			// that allows the user to select a usage example to execute.
			// If we cannot create an instance of the menu class, exit and
			// indicate that we have handled the event by returning TRUE
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, 
				  (void **)&pMe->m_pIMenu) != SUCCESS)
				return TRUE;

			// Succeeded in obtaining a menu instance pointer, so
			// construct the menu and display it on the screen
			BuildMainMenu(pMe);
			if (pMe->m_bSMSMessageStart == TRUE)
         {
				AEERect qrc;
            SETAEERECT(&qrc, 0, 0, pMe->di.cxScreen, pMe->di.cyScreen);
            IDISPLAY_EraseRect(pMe->a.m_pIDisplay,&qrc);
            DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 5, pMe->charbuffer);
         }
			return(TRUE);

        case EVT_APP_STOP:
            return(TRUE);

        case EVT_APP_SUSPEND:
            return(TRUE);

        case EVT_APP_RESUME:
            if (pMe->m_pIMenu)
                IMENUCTL_Redraw(pMe->m_pIMenu);
            return TRUE;

        case EVT_KEY:
            IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);
            if (wParam != AVK_CLR)
                return IMENUCTL_HandleEvent(pMe->m_pIMenu, EVT_KEY, wParam, 0);
            else     
                return TRUE;

		case EVT_COMMAND:
  		    switch(wParam)
			{
				// The following commands are generated by user selections
				// from the main usage app menu.
                case USAGE_EXTRACTSMSTEXT:
                case USAGE_GETCALLERID:
                case USAGE_GETSTATUS:
                case USAGE_MAKEVOICECALL:
				   // De-activate main menu to turn off horizontal scrolling
				   // of long menu items
				   IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);

				   // Execute the usage example the user has selected
                   ITapiUsage (pMe, wParam, 0);
			       return TRUE;
			    default:
				    return FALSE;
			}
        default:
            break;
   }
   return FALSE;
}


/*===========================================================================

FUNCTION ITapi_InitAppData

DESCRIPTION
   This function initializes app-specific data.

PROTOTYPE:
	static boolean ITapi_InitAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains information specific
	to this applet.

DEPENDENCIES
   None.

RETURN VALUE
  TRUE: If the app has app data is allocated and initialized successfully
  FALSE: Either app data could not be allocated or initialized

SIDE EFFECTS
  none
===========================================================================*/
static boolean ITapi_InitAppData(IApplet* pi)
{
   CITapiApp * pMe = (CITapiApp*)pi;

   // Make sure the pointers we'll be using are valid
   if (pMe == NULL || pMe->a.m_pIShell == NULL)
	   return FALSE;

   pMe->m_pIMenu = NULL;

   ISHELL_GetDeviceInfo(pMe->a.m_pIShell, &(pMe->di));

   pMe->aecharbuffer = (AECHAR*)MALLOC(CALLERID_BUFFER_SIZE); 

	pMe->m_bSMSMessageStart = FALSE;

   return TRUE;
}

/*===========================================================================

FUNCTION ITapi_FreeAppData

DESCRIPTION
   Frees data contained in app data and memory for app data itself.

PROTOTYPE:
   static boolean ITapi_FreeAppData(IApplet* pi);

PARAMETERS:
   pi [in]: Pointer to the IApplet structure. This structure contains information specific
   to this applet.

DEPENDENCIES
   None.

RETURN VALUE
   None.

SIDE EFFECTS
   None.
===========================================================================*/
static void ITapi_FreeAppData(IApplet* pi)
{
   CITapiApp * pMe = (CITapiApp*)pi;

   if(pMe->m_pIMenu != NULL)
   {
	  IMENUCTL_Release(pMe->m_pIMenu);
      pMe->m_pIMenu = NULL;
   }
   if (pMe->aecharbuffer)
       FREE(pMe->aecharbuffer);
}

/*===============================================================================
                     APP HELPER FUNCTION DEFINITIONS
=============================================================================== */
/*===========================================================================

FUNCTION: ITapiUsage

DESCRIPTION
    This function executes the usage example selected from the main menu by the
	user.  It takes as input the app data pointer and the ID of the example the
	user selected.  After clearing the screen, the selected example is executed.
	Each example is a self-contained code block that contains any local variables
	needed to execute the example.  Each code block also creates and releases any
	BREW interfaces that are required, and will exit if any errors are detected.  
	Once the example has been executed, this function prints a heading at the top 
	of the screen to indicate which example was selected, 
	and a message at the bottom of the screen instructing the user how to return to
	the main menu to execute another example.
    
PROTOTYPE:
   static void ITapiUsage(CITapiApp * pMe, uint16 wParam, uint32 dwParam)

PARAMETERS:
   pMe:   [in]: Contains a pointer to the usage app data structure.
   wParam: [in]: Identifier of the selected example (example IDs are specified when the
                 main usage app menu is created).
   dwParam: [in]: Used in special case where ITapiUsage is called on SMS message
                  event.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void ITapiUsage (CITapiApp * pMe, uint16 wParam, uint32 dwParam)
{
   // Make sure the pointers we'll be using are valid
   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
	   return;

   // Erase screen first for display output purposes.
   DisplayEvent (pMe, USAGE_ERASE_SCREEN);
   switch (wParam)
   {
      case USAGE_EXTRACTSMSTEXT:
         {
             AEERect qrc;
             SETAEERECT(&qrc, 0, 0, pMe->di.cxScreen, pMe->di.cyScreen);
             IDISPLAY_EraseRect(pMe->a.m_pIDisplay,&qrc);
             DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 0, "Send //BREW:0x01008020:message to phone from http://www.msg.myvzw.com");
         }
         break;

     case USAGE_GETCALLERID:
         {
             FetchLastCallerID(pMe);
             if (*(pMe->aecharbuffer))
             {
                 DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 1, "Last Caller's ID:");
                 IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, pMe->aecharbuffer, CALLERID_BUFFER_SIZE, 0, 30, 0, 0);
                 IDISPLAY_Update(pMe->a.m_pIDisplay);
             }
             else
             {
                 DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 1, "No caller ID Found!");
             }
         }
         break; 

      case USAGE_GETSTATUS:
         {
			ITAPI * pITapi;
            TAPIStatus tpst;

			// Create an instance of the ITapi interface
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_TAPI, 
				  (void **)&pITapi) != SUCCESS)
				return;

            if (ITAPI_GetStatus(pITapi, &tpst) != SUCCESS)
            {
                DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 1, "Couldn't access device status");
            }
            else
            {
                STRCPY(pMe->charbuffer, "MOBILE ID:");
                STRCAT(pMe->charbuffer, tpst.szMobileID);
                DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 1, pMe->charbuffer);
                switch (tpst.state)
                {
                    case PS_OFFLINE:
                        STRCPY(pMe->charbuffer, "State:Offline");
                        break;

                    case PS_IDLE:
                        STRCPY(pMe->charbuffer, "State:Idle");
                        break;

                    case PS_INCOMING:
                        STRCPY(pMe->charbuffer, "State:Call Incoming");
                        break;

                    case PS_ORIG:
                        STRCPY(pMe->charbuffer, "State:Call Outgoing");
                        break;

                    case PS_CONVERSATION:
                        STRCPY(pMe->charbuffer, "State:Conversation");
                        break;
                }
                DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 2, pMe->charbuffer);
                tpst.bData ? STRCPY(pMe->charbuffer, "Data Mode On, "):STRCPY(pMe->charbuffer, "Data Mode Off, ");
                tpst.bDigital ? STRCAT(pMe->charbuffer, "Digital Service On, "):STRCAT(pMe->charbuffer, "Digital Service Off, "); 
                tpst.bRoaming ? STRCAT(pMe->charbuffer, "Roaming On, "):STRCAT(pMe->charbuffer, "Roaming Off, ");
                tpst.bCallEnded ? STRCAT(pMe->charbuffer, "Call Ended On, "):STRCAT(pMe->charbuffer, "Call Ended Off, ");
                tpst.bE911CallbackMode ? STRCAT(pMe->charbuffer, "911 Mode On, "):STRCAT(pMe->charbuffer, "911 Mode Off, ");
                tpst.bRestricted ? STRCAT(pMe->charbuffer, "Restricted"):STRCAT(pMe->charbuffer, "Not Restricted");
                DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 3, pMe->charbuffer);
            }
			// Free the instance of the ITapi interface
			ITAPI_Release(pITapi);
         }
         break;
		 
	  case USAGE_MAKEVOICECALL:
         {
			ITAPI * pITapi;

			// Create an instance of the ITapi interface
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_TAPI, 
				  (void **)&pITapi) != SUCCESS)
				return;

            ITAPI_MakeVoiceCall(pITapi, DEMO_PHONE_NUMBER, AEECLSID_ITAPI_USAGE);
			// Free the instance of the ITapi interface
			ITAPI_Release(pITapi);
         }
         break;

      default:
         return;
   }
            
   // Display heading at top of screen that indicates the example
   // selected, and display message at bottom instructing the user
   // to return to main menu by pressing UP or DOWN. 
   // DisplayEvent (pMe, wParam);

   return;
}

/*===========================================================================

FUNCTION: DisplayEvent

DESCRIPTION
    This function prints a heading at the top 
	of the screen to indicate which usage example was selected by the user, 
	It also prints a message on the bottom two lines of the screen instructing 
	the user to press the UP or DOWN key to return to
	the main menu to execute another example.
    
PROTOTYPE:
   static void DisplayEvent(CITapiApp * pMe, uint16 wParam)

PARAMETERS:
   pMe:   [in]: Contains a pointer to the usage app data structure.
   wParam: [in]: Identifier of the selected example (example IDs are specified when the
                 main usage app menu is created).  If the value USAGE_ERASE_SCREEN is supplied
				 instead, this function clears the screen.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void DisplayEvent (CITapiApp *pMe, uint16 wParam)
{
   AEERect qrc;
   AECHAR * szBuf;
   AEERect rc; 

   // Make sure the pointers we'll be using are valid
   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
	   return;

   if ((szBuf = (AECHAR *) MALLOC(TEXT_BUFFER_SIZE * sizeof(AECHAR))) == NULL)
	   return;

   SETAEERECT(&qrc, 0, 0, pMe->di.cxScreen, pMe->di.cyScreen);
   SETAEERECT(&rc, 5, 5, pMe->di.cxScreen - 5, pMe->di.cyScreen - 5);

   // Specify the text string to be used for the example heading
   switch (wParam)
   {
      case USAGE_ERASE_SCREEN:
         IDISPLAY_EraseRect(pMe->a.m_pIDisplay,&qrc);  //Clear the display
		 FREE(szBuf);
         return;
      case USAGE_EXTRACTSMSTEXT:
         STR_TO_WSTR("ExtractSMSText", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_GETCALLERID:
         STR_TO_WSTR("GetCallerID", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_GETSTATUS:
         STR_TO_WSTR("GetStatus", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_MAKEVOICECALL:
         STR_TO_WSTR("MakeVoiceCall", szBuf, TEXT_BUFFER_SIZE);
         break;
      default:
         STR_TO_WSTR("Unsupported fn", szBuf, TEXT_BUFFER_SIZE);
   }

   // Print the heading at the top of the screen
   IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,10, 5, &rc,
	  IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);

   // Print message at bottom of screen to instruct user to press UP
   // or DOWN to return to main menu
   STR_TO_WSTR("Press Up/Down", szBuf, TEXT_BUFFER_SIZE);
   IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,10, qrc.dy - 25, &rc,
		IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
   STR_TO_WSTR("keys to continue..", szBuf, TEXT_BUFFER_SIZE);
   IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,10, qrc.dy - 15, &rc,
		IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
   IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);

   FREE(szBuf);
}
/*===========================================================================

FUNCTION: BuildMainMenu

DESCRIPTION
    This function constructs the menu control that allows the user to
	select a usage example to execute.  The command ID for each menu item
	is passed to the usage example function above when the user selects the
	menu item for that example.
    
PROTOTYPE:
   static void BuildMainMenu(CITapiApp * pMe)

PARAMETERS:
   pMe:   [in]: Contains a pointer to the usage app data structure.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void BuildMainMenu(CITapiApp *pMe)
{
	AEERect qrc;
    AECHAR * szBuf;

	// Make sure the pointers we'll be using are valid
    if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->m_pIMenu == NULL)
	   return;

   if ((szBuf = (AECHAR *) MALLOC(TEXT_BUFFER_SIZE * sizeof(AECHAR))) == NULL)
	   return;

	// Set Title
    STR_TO_WSTR("ITapi Examples", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_SetTitle(pMe->m_pIMenu, NULL, 0, szBuf);

	//Set the Rectangle for the Menu
    SETAEERECT(&qrc, 0, 0, pMe->di.cxScreen, pMe->di.cyScreen);
	IMENUCTL_SetRect(pMe->m_pIMenu, &qrc);  

   // Add Extract SMS Text to menu
   STR_TO_WSTR("1. Extract SMS Text", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_EXTRACTSMSTEXT, szBuf, 0);

   // Add Get Caller ID to menu
   STR_TO_WSTR("2. Get Caller ID", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETCALLERID, szBuf, 0);

   // Add Get Status to menu
   STR_TO_WSTR("3. Get Status", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETSTATUS, szBuf, 0);

   // Add Make Voice Call to menu
   STR_TO_WSTR("4. Make Voice Call", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_MAKEVOICECALL, szBuf, 0);

   // Activate menu
   IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);

   FREE(szBuf);
}
/*===========================================================================

FUNCTION: DisplayOutput

DESCRIPTION
    This function displays an output string at a given line number on the
    screen. If the nline parameter is a negative value (-1) the string
    is displayed in the middle of the screen. If the "nline" value is larger
    than or equal to zero the "nline" value is multiplied by 15 and the 
    resulting value in pixels is set to the y-coordinate of the start of 
    the string display on the screen. If the string does not fit on one line
    the string wraps around to the next line (spaced rougly 10-15 pixels apart).
    By default 5 is used as the starting the x-coordinate of a displayed 
    string.

    How many characters that fit on one line is calculated for each line 
    that is wrapped around to the next line.

    Note: depending on the phone screen size and the fonts used for characters 
          the output might differ on different handsets (devices). Where some 
          handsets will have a smaller screen and large default fonts which will 
          cause partial overlapping of lines. This function does not try to address
          these issues (this is meant as a simple display function).
    
PROTOTYPE:
   void DisplayOutput(IShell *pIShell, IDisplay *pDisplay, int nline, char *pszStr)

PARAMETERS:
   pIShell:   [in]: Contains a pointer to the IShell interface.
   pIDisplay: [in]: Contains a pointer to the IDisplay interface.
   nline:     [in]: Contains the line number to start displaying the text. The line
        numbers are by default spaced 15 pixels apart along the y-axis.
   pszStr:    [in]: The character string to be displayed on the screen.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
void DisplayOutput(IShell *pIShell, IDisplay *pIDisplay, int nline, char *pszStr)
{
   AEEDeviceInfo di;
   AECHAR * szBuf;      // a buffer that supports 200 char string
   AECHAR * psz = NULL;
   int charHeight = 0;      // Stores the char height in pixels for given font
   int pnAscent = 0;        // Stores the ascent in number of pixels
   int pnDescent = 0;       // Stores the descent in number of pixels
   int pixelWidth;
   AEEFont font = AEE_FONT_NORMAL;
   int pnFits = 0, dy;
   int totalCh = 0;

   // Make sure the pointers we'll be using are valid
   if (pIShell == NULL || pIDisplay == NULL)
	   return;

   if ((szBuf = (AECHAR *) MALLOC(TEXT_BUFFER_SIZE * sizeof(AECHAR))) == NULL)
	   return;

   ISHELL_GetDeviceInfo(pIShell, &di);

   // Get the font metrics info
   charHeight = IDISPLAY_GetFontMetrics (pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   // Convert to wide string (unicode)
   STR_TO_WSTR ((char *)pszStr, szBuf, TEXT_BUFFER_SIZE);

   // If nlines is zero then print this string starting around the middle of 
   // the screen. Or else multiply nlines by charheight to decide the y coordinate of
   // the start of the string.
   if (nline < 0) {
      dy = di.cyScreen*2/5;
   }
   else{
      dy = nline * charHeight;
   }

   // psz keeps track of the point from which to write from the string buffer
   // in case the string does not fit one line and needs to wrap around in the
   // next line.
   psz = szBuf;
      
   // Need to calculate the lotal string length to decide if any wrapping
   // around is needed.
   totalCh = STRLEN ((char *)pszStr);

   // Keep displaying text string on multiple lines if the string can't be displayed
   // on one single line. Lines are spaced 15 pixels apart.
   while ((totalCh > 0) && (*psz != NULL))
   { 
      // Get information on how many characters will fit in a line.
      // Give the pointer to the buffer to be displayed, and the number of
      // pixels along the x axis you want to display the string in (max number)
      // pnFits will have the max number of chars that will fit in the maxWidth
      // number of pixels (given string can't fit in one line), or the number of 
      // chars in the string (if it does fit in one line). pnWidth gives the
      // number of pixels that will be used to display pnFits number of chars.
      pixelWidth = IDISPLAY_MeasureTextEx(pIDisplay,
                      font, 
                      (AECHAR *) psz,  // Start of the buffer to display,
                      -1,
                      di.cxScreen - 5, // maxWidth
                      &pnFits);         // Number of chars that will fit a line

      // If pnFits is zero there is something wrong in the input to above function. 
      // Normally this scenario should not occur. But, have the check anyway.
      if (pnFits == 0)
	  {
		 FREE(szBuf);
         return;
      }

      IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, psz, pnFits, 5 /*start dx*/, 
         dy, 0 /* use default rectangle coordinates */, 0);

      psz += pnFits;      // move pointer to the next segment to be displayed
      totalCh -= pnFits;  // reduce the total number of characters to still display
      dy += charHeight;   // Place next line charHeight pixels below the 
                          // previous line.
      IDISPLAY_Update(pIDisplay); //, TRUE);
      if (totalCh < pnFits)
         pnFits = totalCh;  // if total number is less than pnFits, adjust pnFits
   }
   FREE(szBuf);
   return;   
} // End of DisplayOutput

void StoreLastCallerID(CITapiApp* pMe)
{
    IFileMgr* pIFileMgr = NULL;
    IFile* pIFile = NULL;
    ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR, (void **)&pIFileMgr);
    if (pIFileMgr)
    {
        if (IFILEMGR_Test(pIFileMgr, "calleridfile") != SUCCESS)
        {
            pIFile = IFILEMGR_OpenFile(pIFileMgr, "calleridfile", _OFM_CREATE);
        }
        else
        {
            pIFile = IFILEMGR_OpenFile(pIFileMgr, "calleridfile", _OFM_READWRITE);
        }
        if (pIFile)
        {
            IFILE_Write(pIFile, pMe->aecharbuffer, CALLERID_BUFFER_SIZE);
            IFILE_Release(pIFile);
        }
        IFILEMGR_Release(pIFileMgr);
    }
}

void FetchLastCallerID(CITapiApp* pMe)
{
    IFileMgr* pIFileMgr = NULL;
    IFile* pIFile = NULL;
    ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR, (void **)&pIFileMgr);
    if (pIFileMgr)
    {
        if (IFILEMGR_Test(pIFileMgr, "calleridfile") == SUCCESS)
        {
            pIFile = IFILEMGR_OpenFile(pIFileMgr, "calleridfile", _OFM_READWRITE);
        }
        if (pIFile)
        {
            IFILE_Read(pIFile, pMe->aecharbuffer, CALLERID_BUFFER_SIZE);
            IFILE_Release(pIFile);
        }
        else
        {
            MEMSET(pMe->aecharbuffer, 0, CALLERID_BUFFER_SIZE);
        }
        IFILEMGR_Release(pIFileMgr);
    }
}