/*===========================================================================

FILE: itextctlusage.c

SERVICES: Sample applet using AEE

DESCRIPTION
  This file contains usage examples of ITextCtl interface. ITextCtl interface
  functions allow the user to create/operate on ITextCtl objects. ITextCtl
  objects are used to invoke Calender text related functionality.

PUBLIC CLASSES:  
   N/A


   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/

/*===============================================================================
                     INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "AEEModGen.h"
#include "AEEAppGen.h"
#include "AEEMenu.h"
#include "AEEStdLib.h"
#include "AEEUsageAppIDs.h"
#include "AEEText.h"

// App resource header generated by BREW ResourceEditor.
#include "itextctl_res.h"

/*===========================================================================

                      PUBLIC DATA DECLARATIONS

===========================================================================*/

/*-------------------------------------------------------------------
            Type Declarations
-------------------------------------------------------------------*/
// TextCtl app struct. This is the main struct for this applet. This will hold 
// all the data members that needs to be remembered throughout the life of
// the applet. 
// THE FIRST DATA MEMBER OF THIS STRUCT MUST BE AN AEEApplet OBJECT. 
// This is critical requirement which is related to an applet being
// dynamically loaded. The app developers should add their data members
// following the AEEApplet data member.
typedef struct _CITextCtlApp
{
	AEEApplet	a;        // Mandatory first AEEApplet data member
   IMenuCtl *  m_pIMenu;
   ITextCtl *  m_pIText;
   IMenuCtl *  m_pISoftKey;

   // Device parameters
   int             m_nLineHeight;
   AEEDeviceInfo   m_dInfo;

}CITextCtlApp;

/*-------------------------------------------------------------------
            Function Prototypes
-------------------------------------------------------------------*/
// Mandatory function:
// App Handle Event function
static boolean TextCtlApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);

// App specific data alloc-init/free functions
static boolean ITextCtl_InitAppData(IApplet* pMe);
static void ITextCtl_FreeAppData(IApplet* pMe);

static void BuildMainMenu(CITextCtlApp *pMe);
static void ITextCtlUsage (CITextCtlApp * pMe, uint16 wParam);
static void DisplayEvent (CITextCtlApp *pMe, uint16 wParam);
static void DisplayOutput(CITextCtlApp * pMe, int nline, char *pszStr);

/*-------------------------------------------------------------------
            Global Constant Definitions
-------------------------------------------------------------------*/
// App Resource File
#define  APP_RES_FILE      "itextctl.bar"

// App specific constants
#define USAGE_BASIC_USAGE       100
#define USAGE_HANDLEEVENT       101
#define USAGE_REDRAW            102
#define USAGE_SETACTIVE         103
#define USAGE_ISACTIVE          104
#define USAGE_SETRECT           105
#define USAGE_GETRECT           106
#define USAGE_SETPROPERTIES     107
#define USAGE_GETPROPERTIES     108
#define USAGE_RESET             109
#define USAGE_SETTITLE_LOCAL    110
#define USAGE_SETTITLE_RES      111
#define USAGE_SETTEXT           112
#define USAGE_GETTEXT           113
#define USAGE_GETTEXTPTR        114
#define USAGE_ENABLECOMMAND     115
#define USAGE_SETMAXSIZE        116
#define USAGE_SETSOFTKEYMENU    117

/*===============================================================================
                     FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to ITextCtl_InitAppData(). 
   This is done as follows: InitAppData() is called to initialize AppletData 
   instance. It is app developers responsibility to fill-in app data initialization 
   code of InitAppData(). App developer is also responsible to release memory 
   allocated for data contained in AppletData -- this can be done in 
   ITextCtl_FreeAppData().

PROTOTYPE:
   int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell interface. 

	pIModule: pin]: Contains pointer to the IModule interface to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid IApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the app needs to be loaded and if AEEApplet_New() invocation was
     successful
  EFAILED: If the app does not need to be loaded or if errors occurred in 
     AEEApplet_New(). If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
{
   *ppObj = NULL;
		
   // We want to load this App. So, invoke AEEApplet_New().To it, pass the
   // address of the app-specific handle event function.
   if(ClsId == AEECLSID_TEXTCTL_APP){
      if(AEEApplet_New(sizeof(CITextCtlApp), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)TextCtlApp_HandleEvent,(PFNFREEAPPDATA)ITextCtl_FreeAppData)
         == TRUE)
      {
         if (ITextCtl_InitAppData((IApplet*)*ppObj) == TRUE)
         {
            return(AEE_SUCCESS);
         }
      }
   }
	return (EFAILED);
}

/*===========================================================================

FUNCTION TextCtlApp_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean TextCtlApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

	ecode: Specifies the Event sent to this applet

   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/
static boolean TextCtlApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
   CITextCtlApp * pMe = (CITextCtlApp*)pi;

   switch (eCode) 
	{

      case EVT_APP_START:
         // Create the IMenu interface object. When the applet is started the 
         // main menu is set up. When the user selects a menu item from the main 
         // menu, an EVT_COMMAND event is received by this applet. The processing for 
         // EVT_COMMAND event is found below.
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, (void **)&pMe->m_pIMenu)
            != SUCCESS)
         {
				return FALSE;
         }

         // Build the main menu
			BuildMainMenu(pMe);
			return(TRUE);
      case EVT_APP_STOP:
         return(TRUE);
      case EVT_KEY: 
         // Call the TextCtl HandleEvent to see if this event would be 
         // handled by the TextCtl interface. When the Up or Down key is 
         // pressed we will fail the check below, release the ITextCtl object
         // and pass the controls to the menu control object.
         if ((pMe->m_pIText != NULL) && 
            ITEXTCTL_HandleEvent(pMe->m_pIText, EVT_KEY, wParam, dwParam))
         {
            // This event was handled by TextCtl. Return here since
            // this was handled by the text control object.
            return TRUE;
         }

			// Release soft key menu.
			if (pMe->m_pISoftKey != NULL)
			{
				IMENUCTL_Release (pMe->m_pISoftKey);
				pMe->m_pISoftKey = NULL;
			}

         // Release the ITextCtl object if it is not NULL.
         if (pMe->m_pIText != NULL)
         {
            // We have come here because the TextCtl object was 
            // not NULL. This means we would have been in an active edit
            // mode of the Text Control and the select key was pressed
            // (any other key press would have been handled by the Text
            // control.
            ITEXTCTL_Release (pMe->m_pIText);
            pMe->m_pIText = NULL;
            //return TRUE;
         }

         // We have come here when a key is pressed on the handset keypad. Call 
         // the IMenuCtl handle event function to handle this key press.
         // If this key press corressponds to an Up/Down arrow 
         // key press, this event will be handled by the IMenuCtl object
         // in moving to the next menu item (in the given direction). 
         // If the key is neither Up/Down, the IMenuCtl the key press 
         // is not handled, hence, return FALSE.
			  if (pMe->m_pIMenu)
			  {
					if ((IMENUCTL_IsActive(pMe->m_pIMenu) == FALSE) && ((wParam == AVK_UP) || (wParam == AVK_DOWN)))
					{
						IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);
						IMENUCTL_Redraw(pMe->m_pIMenu);
					}
			      return IMENUCTL_HandleEvent(pMe->m_pIMenu, EVT_KEY, wParam, 0);
			  }
			  else 
				  return FALSE;

		case EVT_COMMAND:
         // When a menu item is selected this applet receives the
         // EVT_COMMAND event in the wParam field.
  		   switch(wParam)
         {
            case USAGE_BASIC_USAGE:
            case USAGE_HANDLEEVENT:
            case USAGE_REDRAW:
            case USAGE_SETACTIVE:
            case USAGE_ISACTIVE:
            case USAGE_SETRECT:
            case USAGE_GETRECT:
            case USAGE_SETPROPERTIES:
            case USAGE_GETPROPERTIES:
            case USAGE_RESET:
            case USAGE_SETTITLE_LOCAL:
            case USAGE_SETTITLE_RES:
            case USAGE_SETTEXT:
            case USAGE_GETTEXT:
            case USAGE_GETTEXTPTR:
            case USAGE_ENABLECOMMAND:
            case USAGE_SETMAXSIZE:
            case USAGE_SETSOFTKEYMENU:
               // Call the usage function that exercises all the 
               // interface functions.
               ITextCtlUsage (pMe, wParam);
               return TRUE;
		   	default:
			   	return FALSE;
         }
      default:
         break;
   }
   return FALSE;
}



/*===========================================================================

FUNCTION ITextCtl_InitAppData

DESCRIPTION
   This function initializes app specific data allocates memory for app data 
   (AppletData) and sets it to pAppData of AEEApplet. It also function . 
   App developer needs to fill-in the initialization code.

PROTOTYPE:
	boolean ITextCtl_InitAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains 
   information specific to this applet. It was initialized during the 
   AEEClsCreateInstance().

DEPENDENCIES
   Assumes pi is not NULL

RETURN VALUE
  TRUE: If the app has app data is allocated and initialized successfully
  FALSE: Either app data could not be allocated or initialzied

SIDE EFFECTS
  none
===========================================================================*/
static boolean ITextCtl_InitAppData(IApplet* pi)
{
   int pnAscent;        // Stores the ascent in number of pixels
   int pnDescent;       // Stores the descent in number of pixels

   CITextCtlApp * pMe = (CITextCtlApp*)pi;

   // Initialize the MenuCtl pointer to NULL
   pMe->m_pIMenu = NULL;
   pMe->m_pIText = NULL;
   pMe->m_pISoftKey = NULL;

   // Get the font metrics info
   pMe->m_nLineHeight = IDISPLAY_GetFontMetrics (pMe->a.m_pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&pMe->m_dInfo);

   return TRUE;
}

/*===========================================================================

FUNCTION ITextCtl_FreeAppData

DESCRIPTION
   Frees data contained in app data and memory for app data itself.
   App developer needs to free data contained in AppletData.

PROTOTYPE:
	void ITextCtl_FreeAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains 
   information specific to this applet. It was initialized during the 
   AEEClsCreateInstance().

DEPENDENCIES
   Assumes pi is not NULL

RETURN VALUE
   None

SIDE EFFECTS
   None
===========================================================================*/
static void ITextCtl_FreeAppData(IApplet* pi)
{
   CITextCtlApp * pMe = (CITextCtlApp*)pi;

   // Release main menu
   if (pMe->m_pIMenu != NULL)
   {
      IMENUCTL_Release (pMe->m_pIMenu);
      pMe->m_pIMenu = NULL;
   }
   // Release soft key menu.
   if (pMe->m_pISoftKey != NULL)
   {
      IMENUCTL_Release (pMe->m_pISoftKey);
      pMe->m_pISoftKey = NULL;
   }
   // Release TextCtl object if it is not released
   if (pMe->m_pIText != NULL)
   {
      ITEXTCTL_Release (pMe->m_pIText);
      pMe->m_pIText = NULL;
   }
}

/*===============================================================================
                     APP HELPER FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: ITextCtlUsage

DESCRIPTION
	This function encompasses all the usage examples of all the functions
   in code blocks switched using the BREW API function Id passed into this
   function. The code blocks will include almost all the variable declarations
   needed to demonstrate the usage of a given function. Within the code block
   all the needed interfaces are instantiated and released subsequent to their
   use.

   There are some exceptions to the above general rule, such as call-back functions,
   common helper function, or common display functions called from within the
   code block which are defined elsewhere in this file.

   There are some instances where an object instantiated within a code block is
   not released at the end of the code block, to be released at a later point
   (for example in a call-back function, in this case pointers to such objects
   are stored in the AEEApplet object to be released later).
   
PROTOTYPE:
   void ITextCtlUsage (CITextCtlApp * pMe, uint16 wParam)

PARAMETERS:
	pMe: [in]: Contains a pointer to the CITextCtlApp struct.
   wParam: [in]: Contains the Id of the function type to be used (contains the
      Id corresponding to the menu item selected on the emulator).

DEPENDENCIES
  none

RETURN VALUE
  none

SIDE EFFECTS
  none
===========================================================================*/
static void ITextCtlUsage (CITextCtlApp * pMe, uint16 wParam)
{
   AEERect rect;
   IShell *pIShell = pMe->a.m_pIShell;

   // Set rectangle coordinates for a rectangle in the middle of the screen.
   SETAEERECT (&rect, pMe->m_dInfo.cxScreen/8, pMe->m_dInfo.cyScreen/5, 
      pMe->m_dInfo.cxScreen*3/4, pMe->m_dInfo.cyScreen/2);

   // Erase screen first for display output purposes.
   IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
	IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
	IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);

   switch (wParam)
   {
      case USAGE_BASIC_USAGE:
         {
            // 1. Usage: Basic ITextCtl Usage
            AECHAR szBuf[50] = {0};

            // Create ITextCtl object. This step needs to be done only if
            // the ITextCtl object has not been created already. 
            //
            // Create an instance of the ITextCtl object and store it in
            // the CITextCtlApp class data member. This is used to better
            // illustrate the ITextCtl object operations.
            ISHELL_CreateInstance(pIShell, AEECLSID_TEXTCTL, (void **)&pMe->m_pIText);

            if (pMe->m_pIText)
            {
               // Set title of the menu control object
               STR_TO_WSTR("Title ", szBuf, sizeof(szBuf));
               ITEXTCTL_SetTitle(pMe->m_pIText, NULL, NULL, szBuf);
               // Set maximum text size supported by the text control object
               ITEXTCTL_SetMaxSize(pMe->m_pIText, 100);

               // Set properties to have thetext control object have a
               // frame and to have a multi-line text edit space.
               ITEXTCTL_SetProperties(pMe->m_pIText,  TP_FRAME | TP_MULTILINE );
               ITEXTCTL_SetRect(pMe->m_pIText, &rect);

               // Set the initial text of the text control object.
               STR_TO_WSTR("Enter Text..", szBuf, sizeof(szBuf));
               ITEXTCTL_SetText(pMe->m_pIText, szBuf, sizeof (szBuf));

               // Set the text control object so that the user can edit it's
               // text content.
               ITEXTCTL_SetActive (pMe->m_pIText, TRUE);

               // The ITextCtl object is NOT released at this point. The ITextCtl 
               // object is released in the HandleEvents function under EVT_KEY. 
               // This gives the user the ability to edit text on an active 
               // ITextCtl object. The ITextCtl object is released when the user 
               // selects the Up/Down arrow to move to another main menu selection
               // or when the user exits the applet.
            }
         }
         break;
      case USAGE_HANDLEEVENT:
         {
            // 2. Usage: ITEXTCTL_HandleEvent
            AEEEvent  evt = EVT_KEY;       // TextCtl only supports this event
            uint16    wParam  = AVK_RIGHT;  // Supports SELECT, LEFT, RIGHT, UP, DOWN
            uint32    dwParam = 0;
            AECHAR    szBuf[50] = {0};

            // Create ITextCtl object. This step needs to be done only if
            // the ITextCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_TEXTCTL, (void **)&pMe->m_pIText);

            if (pMe->m_pIText)
            {
               // SetRect/SetText used for output display purposes only. For more 
               // details see ITEXTCTL_SetText/ITEXTCTL_SetRect usage. 
               ITEXTCTL_SetRect(pMe->m_pIText, &rect);
               STR_TO_WSTR("HandleEvent Text..", szBuf, sizeof(szBuf));
               ITEXTCTL_SetText(pMe->m_pIText, szBuf, sizeof (szBuf));

               // The text control needs to be active for the handle event function
               // to succeed. See ITEXTCTL_SetActive usage for more information.
               ITEXTCTL_SetActive (pMe->m_pIText, TRUE);

               // ITEXTCTL_HandleEvent: is used to handle the events received by 
               // text control object. If the text control object is in non 
               // edit mode, it processed only set title, set text and press of 
               // UP and DOWN key events. In text edit mode, it processes various 
               // events like key up, key down, key held, set title, set text, 
               // command event from soft key menu.

               // Invoke HandleEvent. 
               // This will move the cursor to the right by one character.
               // By default the cursor is positioned at the begining of the
               // text string. By invoking HandleEvents function with EVT_KEY event
               // and the key value of AVK_RIGHT the cursor will be moved one 
               // character to the right on the display.
               //
               // Note: a more typical use of ITEXTCTL_HandleEvent is found in 
               // the HandleEvent function of this applet under EVT_KEY event.
               if ((ITEXTCTL_HandleEvent(pMe->m_pIText, evt, wParam, dwParam)) == FALSE)
               {
                  // Do not need to return here. Fall through and invoke 
                  // DisplayEvents() function.
               }

               // The ITextCtl object is NOT released at this point. The ITextCtl 
               // object is released in the HandleEvents function under EVT_KEY. 
               // This gives the user the ability to edit text on an active 
               // ITextCtl object. The ITextCtl object is released when the user 
               // selects the Up/Down arrow to move to another main menu selection
               // or when the user exits the applet.
            }
         }
         break;
      case USAGE_REDRAW:
         {
            // 3. Usage: ITEXTCTL_Redraw
            ITextCtl * pITextCtl = NULL;
            boolean boolRetVal = FALSE;
            AECHAR    szBuf[50] = {0}; 
            AEERect   rect;

            // Create ITextCtl object. This step needs to be done only if
            // the ITextCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_TEXTCTL, (void **)&pITextCtl);

            if (pITextCtl)
            {
               // Set Text/Rect for output display purposes only. 
               // See ITEXTCTL_SetRect/ITEXTCTL_SetText usage for more details.
               SETAEERECT (&rect, 0, 2*pMe->m_nLineHeight, pMe->m_dInfo.cxScreen, 
                  pMe->m_dInfo.cyScreen*3/4);
               ITEXTCTL_SetRect(pITextCtl, &rect);
               STR_TO_WSTR("Redraw Text..", szBuf, sizeof(szBuf));
               ITEXTCTL_SetText(pITextCtl, szBuf, sizeof (szBuf));

               // ITEXTCTL_Redraw: instructs the text control object to redraw its 
               // contents. The text control object does not redraw its contents 
               // every time the underlying data behind the text control changes. 
               // This allows several data updates to occur while minimizing 
               // screen flashes. For example, several changes can be made to 
               // the contents of the text control object with no visible effect 
               // until Redraw method is called.

               // Redraw text control object
               if (!(boolRetVal = ITEXTCTL_Redraw (pITextCtl)))
               {
                  // Release Text control. This step only needs to be done
                  // if no further use of the ITextCtl object is needed.
                  ITEXTCTL_Release (pITextCtl);
                  return;
               }

               // Release ITextCtl object. This step needs to be done
               // only if no further use of the ITextCtl object is needed.
               ITEXTCTL_Release (pITextCtl);
            }
         }
         break;
      case USAGE_SETACTIVE:
         {
            // 3. Usage: ITEXTCTL_SetActive	
            AECHAR    szBuf[50] = {0}; 

            // Create ITextCtl object. This step needs to be done only if
            // the ITextCtl object has not been created already. 
            //
            // Create an instance of the ITextCtl object and store it in
            // the CITextCtlApp class data member. This is used to better
            // illustrate the ITextCtl object operations.
            ISHELL_CreateInstance(pIShell, AEECLSID_TEXTCTL, (void **)&pMe->m_pIText);
	
            if (pMe->m_pIText)
            {	
               // Set Text/Rect for output display purposes. 
               // See ITEXTCTL_SetRect/ITEXTCTL_SetText usage for more details.
               ITEXTCTL_SetRect(pMe->m_pIText, &rect);

               // Draw the text control object with a border.
               ITEXTCTL_SetProperties(pMe->m_pIText, TP_FRAME | TP_MULTILINE);
               STR_TO_WSTR("SetActive Text..", szBuf, sizeof(szBuf));
               ITEXTCTL_SetText(pMe->m_pIText, szBuf, sizeof (szBuf));

               // ITEXTCTL_SetActive: is used to make a text control object 
               // active. Only an active text control object handles the event 
               // sent to it. Inactive text control object just ignores the events.

               // Set text control object active. 
               ITEXTCTL_SetActive (pMe->m_pIText, TRUE);

               // The ITextCtl object is NOT released at this time. The ITextCtl 
               // object is released in the HandleEvents function under EVT_KEY. 
               // This gives the user the ability to edit text on an active 
               // ITextCtl object. The ITextCtl object is released when the user 
               // selects the Up/Down arrow to move to another main menu selection
               // or when the user exits the applet.
            }
         }
         break;
      case USAGE_ISACTIVE:
         {
            // 4. Usage: ITEXTCTL_IsActive
            boolean   boolRetVal;
            AECHAR    szBuf[50] = {0}; 
		
            // Create ITextCtl object. This step needs to be done only if
            // the ITextCtl object has not been created already.
            //
            // Create an instance of the ITextCtl object and store it in
            // the CITextCtlApp class data member. This is used to better
            // illustrate the ITextCtl interface operations.
            ISHELL_CreateInstance(pIShell, AEECLSID_TEXTCTL, (void **)&pMe->m_pIText);

            if (pMe->m_pIText)
            {
               // Set Text/Rect for output display purposes. 
               // See ITEXTCTL_SetRect/ITEXTCTL_SetText usage for more details.
               ITEXTCTL_SetRect(pMe->m_pIText, &rect);
               STR_TO_WSTR("IsActive Text..", szBuf, sizeof(szBuf));
               ITEXTCTL_SetText(pMe->m_pIText, szBuf, sizeof (szBuf));
               // First Set active
               ITEXTCTL_SetActive (pMe->m_pIText, TRUE);

               // Check if text control object is active. It should not be since
               // text control is not activated.
               if ((boolRetVal = ITEXTCTL_IsActive (pMe->m_pIText)) == FALSE)
               {
                  // Release ITextCtl object. This step needs to be done
                  // only if no further use of the ITextCtl object is needed.
                  ITEXTCTL_Release (pMe->m_pIText);
                  pMe->m_pIText = NULL;
                  return;
               }

               // The ITextCtl object is NOT released at this time. The ITextCtl 
               // object is released in the HandleEvents function under EVT_KEY. 
               // This gives the user the ability to edit text on an active 
               // ITextCtl object. The ITextCtl object is released when the user 
               // selects the Up/Down arrow to move to another menu selection.
            }
         }
         break;
      case USAGE_SETRECT:
         {
            // 5. Usage: ITEXTCTL_SetRect 
            ITextCtl * pITextCtl = NULL;
            AEERect rct;
            AECHAR    szBuf[50] = {0}; 

            rct.x = pMe->m_dInfo.cxScreen/2;
            rct.y = pMe->m_dInfo.cyScreen /2;
            rct.dx = pMe->m_dInfo.cxScreen - 20;
            rct.dy = pMe->m_dInfo.cyScreen/3;

            // Create ITextCtl object. This step needs to be done only if
            // the ITextCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_TEXTCTL, (void **)&pITextCtl); 

            if (pITextCtl)
            {
               // ITEXTCTL_SetRect: sets the rectangle coordinates of area on the
               // display where the text control object will be drawn.
               ITEXTCTL_SetRect(pITextCtl, &rct);

               // Set Text/Active for output display purposes. 
               // See ITEXTCTL_SetRect/ITEXTCTL_SetActive usage for more details.
               STR_TO_WSTR("SetRect Text..", szBuf, sizeof(szBuf));
               ITEXTCTL_SetText(pITextCtl, szBuf, sizeof (szBuf));

               // Release ITextCtl object. This step needs to be done
               // only if no further use of the ITextCtl object is needed.
               ITEXTCTL_Release (pITextCtl);
            }
         }
         break;
      case USAGE_GETRECT:
         {
            // 6. Usage: ITEXTCTL_GetRect
            ITextCtl *pITextCtl = NULL;
            AEERect   rc = {0};
 
            // Create ITextCtl object. This step needs to be done only if
            // the ITextCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_TEXTCTL, (void **)&pITextCtl);

            if (pITextCtl)
            {
               // Get text control object rectangle
               ITEXTCTL_GetRect(pITextCtl, &rc);

               // No display output...

               // Release ITextCtl object. This step needs to be done
               // only if no further use of the ITextCtl object is needed.
               ITEXTCTL_Release (pITextCtl);
            }
         }
         break;      
      case USAGE_SETPROPERTIES:
         {
            // 7. Usage:ITEXTCTL_SetProperties
            // Set the properties of the text control object so that
            // there is a border drawn around the text control object,
            // the text control object is multi-lined and enable T9 mode.
            uint32     nProperties = TP_FRAME | TP_MULTILINE | TP_T9_MODE;
            AECHAR    szBuf[50] = {0}; 
	
            // Create ITextCtl object. This step needs to be done only if
            // the ITextCtl object has not been created already. 
            //
            // Create an instance of the ITextCtl object and store it in
            // the CITextCtlApp class data member. This is used to better
            // illustrate the ITextCtl interface operations.
            ISHELL_CreateInstance(pIShell, AEECLSID_TEXTCTL, (void **)&pMe->m_pIText);

            if (pMe->m_pIText)
            {
      		   // Set properties of text control object 
               ITEXTCTL_SetProperties(pMe->m_pIText,  nProperties); 

               // Set the y axis coordinate to new value, so that the
               // control object rectangle is bigger.
               rect.dy = pMe->m_dInfo.cyScreen/2; 

               ITEXTCTL_SetRect(pMe->m_pIText, &rect);
               // Set Rect/Text/Active for output display purposes. For more details
               // see ITEXTCTL_SetText/ITEXTCTL_SetRect/ITEXTCTL_SetActive usage. 
               STR_TO_WSTR("SetProperties Text..", szBuf, sizeof(szBuf));
               ITEXTCTL_SetText(pMe->m_pIText, szBuf, sizeof (szBuf));
               // Make the control object active so that the user can edit text.
               ITEXTCTL_SetActive (pMe->m_pIText, TRUE);

               // The ITextCtl object is NOT released at this time. The ITextCtl 
               // object is released in the HandleEvents function under EVT_KEY. 
               // This gives the user the ability to edit text on an active 
               // ITextCtl object. The ITextCtl object is released when the user 
               // selects the Up/Down arrow to move to another menu selection.
            }
         }
         break;
      case USAGE_GETPROPERTIES:
         {
            // 8. Usage: ITEXTCTL_GetProperties
            ITextCtl * pITextCtl = NULL;
            uint32     nProperties = 0;       // Test value

     	      // Create ITextCtl object. This step needs to be done only if
            // the ITextCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_TEXTCTL, (void **)&pITextCtl);
  
            if (pITextCtl)
            {
               // Get Properties
               nProperties = ITEXTCTL_GetProperties(pITextCtl);	   

               // No output...

               // Release ITextCtl object. This step needs to be done
               // only if no further use of the ITextCtl object is needed.
               ITEXTCTL_Release (pITextCtl);
            }
         }
         break;
      case USAGE_RESET:
         {
            // 9. Usage: ITEXTCTL_Reset
            AECHAR  szBuf[50] = {0};
            
            // Create ITextCtl object. This step needs to be done only if
            // the ITextCtl object has not been created already. 
	         ISHELL_CreateInstance(pIShell, AEECLSID_TEXTCTL, (void **)&pMe->m_pIText);

	         if (pMe->m_pIText)
            {
               // Set Rect/Text/Active/Title for output display purposes only. 
               // For more details see ITEXTCTL_SetRect/ITEXTCTL_SetText/
               // ITEXTCTL_SetActive/ITEXTCTL_SetTitle usage. 
               ITEXTCTL_SetRect(pMe->m_pIText, &rect);

               // Set text to display.
               STR_TO_WSTR("Reset Text..", szBuf, sizeof(szBuf));
               ITEXTCTL_SetText(pMe->m_pIText, szBuf, sizeof (szBuf));

               // This will make the text control object active.
               ITEXTCTL_SetActive (pMe->m_pIText, TRUE);

               // Set title of the text control object.
               STR_TO_WSTR("TextCtl Title ", szBuf, sizeof(szBuf));
               ITEXTCTL_SetTitle(pMe->m_pIText, NULL, NULL, szBuf);

               // ITEXTCTL_Reset: instructs the text control to reset (free/delete) 
               // it's contents as well as to immediately leave active/focus mode.

		         // Reset text control object. This should reset the title
               // of the text control object and make the object inactive.
		         ITEXTCTL_Reset (pMe->m_pIText);

               // Redraw the text control object to see the effects of the
               // Reset operation. The title set above will not be displayed
               // since the reset released the title. Also the object is
               // in the inactive mode (no arrow key inputs are handled by the
               // text control).
               ITEXTCTL_Redraw (pMe->m_pIText);

               // The ITextCtl object is NOT released at this time. The ITextCtl 
               // object is released in the HandleEvents function under EVT_KEY. 
               // This gives the user the ability to edit text on an active 
               // ITextCtl object. The ITextCtl object is released when the user 
               // selects the Up/Down arrow to move to another menu selection.
            }
         }
         break;
      case USAGE_SETTITLE_LOCAL:
         {
            // 10. Usage: ITEXTCTL_SetTitle
            ITextCtl * pITextCtl = NULL;
            boolean    boolRetVal = FALSE;
            AECHAR szBuf [50];
		     
            // Create ITextCtl object. This step needs to be done only if
            // the ITextCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_TEXTCTL, (void **)&pITextCtl);

            if (pITextCtl)
            {		
               ITEXTCTL_SetRect(pITextCtl, &rect);
               ITEXTCTL_SetProperties(pITextCtl,  TP_FRAME);

               // Set title using locally defined strings.
               STR_TO_WSTR("TextCtl Title ", szBuf, sizeof(szBuf));
               if (!(boolRetVal = ITEXTCTL_SetTitle(pITextCtl, NULL, NULL, szBuf)))
               {
                  // Release Date control. This step only needs to be done
                  // if no further use of the ITextCtl object is needed.
                  ITEXTCTL_Release (pITextCtl);
                  return;
               }

               // Initialize some text for display.
               STR_TO_WSTR("SetTitle Text..", szBuf, sizeof(szBuf));
               ITEXTCTL_SetText(pITextCtl, szBuf, sizeof (szBuf));

               // Release ITextCtl object. This step needs to be done
               // only if no further use of the ITextCtl object is needed.
               ITEXTCTL_Release (pITextCtl);
            }
         }
         break;
      case USAGE_SETTITLE_RES:
         {
            // 10. Usage: ITEXTCTL_SetTitle
            ITextCtl * pITextCtl = NULL;
            boolean    boolRetVal = FALSE;
            char szResFile[] = APP_RES_FILE;   // Resource File used by this applet
            AECHAR szBuf [50];
		     
            // Create ITextCtl object. This step needs to be done only if
            // the ITextCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_TEXTCTL, (void **)&pITextCtl);

            if (pITextCtl)
            {		
               ITEXTCTL_SetRect(pITextCtl, &rect);
               ITEXTCTL_SetProperties(pITextCtl,  TP_FRAME);

               // Set title using string defined in resource file.
               if (!(boolRetVal = ITEXTCTL_SetTitle(pITextCtl, szResFile, IDS_TITLE, NULL)))
               {
                  // Release Date control. This step only needs to be done
                  // if no further use of the ITextCtl object is needed.
                  ITEXTCTL_Release (pITextCtl);
                  return;
               }

               // Initialize some text as well for display.
               STR_TO_WSTR("SetTitle Text..", szBuf, sizeof(szBuf));
               ITEXTCTL_SetText(pITextCtl, szBuf, sizeof (szBuf));

               // Release ITextCtl object. This step needs to be done
               // only if no further use of the ITextCtl object is needed.
               ITEXTCTL_Release (pITextCtl);
            }
         }
         break;
      case USAGE_SETTEXT:
         {
            // 11. Usage: ITEXTCTL_SetText
            ITextCtl * pITextCtl = NULL;
            boolean    boolRetVal = FALSE;
            AECHAR szBuf [50];
		     
            // Create ITextCtl object. This step needs to be done only if
            // the ITextCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_TEXTCTL, (void **)&pITextCtl);

            if (pITextCtl)
            {	
               // Set rectangle
               ITEXTCTL_SetRect(pITextCtl, &rect);

               // Set title
               STR_TO_WSTR("Text Control Text..", szBuf, sizeof(szBuf));
               if ((boolRetVal = ITEXTCTL_SetText(pITextCtl, szBuf, sizeof (szBuf)))
                  == FALSE)
               {
                  // Release Date control. This step only needs to be done
                  // if no further use of the ITextCtl object is needed.
                  ITEXTCTL_Release (pITextCtl);
                  return;
               }

               // Release ITextCtl object. This step needs to be done
               // only if no further use of the ITextCtl object is needed.
               ITEXTCTL_Release (pITextCtl);
            }
         }
         break;
      case USAGE_GETTEXT:
         {
            // 12. Usage: ITEXTCTL_GetText
            ITextCtl * pITextCtl = NULL;
            boolean    bRetVal = FALSE;
            char pszStr[30] = {0};
            AECHAR szBuf1[30] = {0};
            AECHAR szBuf2[30] = {0};

            // Create ITextCtl object. This step needs to be done only if
            // the ITextCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_TEXTCTL, (void **)&pITextCtl);

            if (pITextCtl)
            {		
               // Set rectangle to point to the a section of the top half 
               // of the screen. Also, have a frame around the text by
               // setting the properties and setting the control object active.
               rect.y = pMe->m_nLineHeight * 2;
               ITEXTCTL_SetRect(pITextCtl, &rect);
               ITEXTCTL_SetProperties(pITextCtl,  TP_FRAME);
               ITEXTCTL_SetActive (pITextCtl, TRUE);

               // Set text so that the text control object will have some
               // valid text when we retrieve text from it. Also, by
               // setting the properties and activating the text control
               // object it srounds the text control object text with 
               // a frame. This is done to distinguish it from the text
               // retrieved from the text control object (using ITEXTCTL_GetText
               // below) and printed to screen.
               STR_TO_WSTR("GetTextPtr Text..", szBuf1, sizeof(szBuf1));
               ITEXTCTL_SetText(pITextCtl, szBuf1, sizeof (szBuf1));

               // ITEXTCTL_GetText: is used to read text associated with the 
               // text control object in the given buffer subject to the maximum 
               // of specified characters.
               if ((bRetVal = ITEXTCTL_GetText (pITextCtl, szBuf2, sizeof (szBuf2)))
                  == TRUE)
               {
                  // Convert the wide-string to char. This is done only to 
                  // use a single display function.
                  WSTR_TO_STR (szBuf2, pszStr, sizeof (pszStr));

                  // Display the text retrieved. In the middle of the 
                  // screen. This text will not have a frame srounding it.
                  DisplayOutput (pMe, -1, pszStr);
               }
               else 
               {
                  DisplayOutput (pMe, -1, "GetText: Failed..");
               }

               // Release ITextCtl object. This step needs to be done
               // only if no further use of the ITextCtl object is needed.
               ITEXTCTL_Release (pITextCtl);
            }
         }
         break;      
      case USAGE_GETTEXTPTR:
         {
            // 13. Usage: ITEXTCTL_GetTextPtr
            ITextCtl * pITextCtl = NULL;
            char pszStr[30] = {0};
            AECHAR * pszText = NULL;
            AECHAR szBuf [30] = {0};

            // Create ITextCtl object. This step needs to be done only if
            // the ITextCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_TEXTCTL, (void **)&pITextCtl);

            if (pITextCtl)
            {
               // Set rectangle to point to the a section of the top half 
               // of the screen. Also, have a frame around the text by
               // setting the properties and setting the control object active.
               // Set rectangle
               // Set rectangle to point to the a section of the top half 
               // of the screen. Also, have a frame around the text by
               // setting the properties and setting the control object active.
               rect.y = pMe->m_nLineHeight * 2;
               ITEXTCTL_SetRect(pITextCtl, &rect);
               ITEXTCTL_SetProperties(pITextCtl,  TP_FRAME);
               ITEXTCTL_SetActive (pITextCtl, TRUE);

               // Set text so that the text control object will have some
               // valid text when we retrieve a pointer to it. Also, by
               // setting the properties and activating the text control
               // object it srounds the text control object text with 
               // a frame. This is done to distinguish it from the text
               // retrieved from the text control object (using 
               // ITEXTCTL_GetTextPtr below) and printed to screen.
               STR_TO_WSTR("GetTextPtr Text..", szBuf, sizeof(szBuf));
               ITEXTCTL_SetText(pITextCtl, szBuf, sizeof (szBuf));

               // ITEXTCTL_GetText: is used to read text associated with the 
               // text control object in the given buffer subject to the maximum 
               // of specified characters.
               if ((pszText = ITEXTCTL_GetTextPtr (pITextCtl))
                  != NULL)
               {
                  // Convert the wide-string to char. This is done only to 
                  // use a single display function.
                  WSTR_TO_STR (pszText, pszStr, sizeof (pszStr));

                  // Display text retrieved from the text control object.
                  DisplayOutput (pMe, -1, pszStr);
               }

               // Release ITextCtl object. This step needs to be done
               // only if no further use of the ITextCtl object is needed.
               ITEXTCTL_Release (pITextCtl);
            }
         }
         break;
      case USAGE_ENABLECOMMAND:
         {
            // 14. Usage: ITEXTCTL_EnableCommand
            ITextCtl * pITextCtl = NULL;
            uint16 nCmdId = EVT_KEY_PRESS;

            // Create ITextCtl object. This step needs to be done only if
            // the ITextCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_TEXTCTL, (void **)&pITextCtl);

            if (pITextCtl)
            {
               // ITEXTCTL_EnableCommand: is used to enable sending of specified 
               // command by the text control object to the shell object upon 
               // receiving the event generated by pressing center key.
               ITEXTCTL_EnableCommand (pITextCtl, TRUE, nCmdId);

               // No output for this function...

               // Release ITextCtl object. This step needs to be done
               // only if no further use of the ITextCtl object is needed.
               ITEXTCTL_Release (pITextCtl);
            }
         }
         break;
      case USAGE_SETMAXSIZE:
         {
            // 15. Usage: ITEXTCTL_SetMaxSize
            ITextCtl * pITextCtl = NULL;
            uint16 nMaxSize = 100;

            // Create ITextCtl object. This step needs to be done only if
            // the ITextCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_TEXTCTL, (void **)&pITextCtl);

            if (pITextCtl)
            {
               // ITEXTCTL_SetMaxSize: is used to set maximum text size supported 
               // by the text control object. If the size being set is more than 
               // the size already set, this leads to the freeing up of the 
               // memory associated with the previous size and allocation of the 
               // memory as per new size.
               ITEXTCTL_SetMaxSize (pITextCtl, nMaxSize);

               // No output for this function...

               // Release ITextCtl object. This step needs to be done
               // only if no further use of the ITextCtl object is needed.
               ITEXTCTL_Release (pITextCtl);
            }
         }
         break;      
      case USAGE_SETSOFTKEYMENU:
         {
            // 16. Usage: ITEXTCTL_SetSoftKeyMenu
            AECHAR szBuf[50] = {0};
            AEERect softKeyRect;

            // Create ITextCtl object. This step needs to be done only if
            // the ITextCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_TEXTCTL, (void **)&pMe->m_pIText);

            if (pMe->m_pIText)
            {
               // Create IMenuCtl object. This step needs to be done only if
               // the IMenuCtl object has not been created already. 
               ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOFTKEYCTL, (void **)&pMe->m_pISoftKey);

               if (!pMe->m_pISoftKey)
               {
                  ITEXTCTL_Release (pMe->m_pIText);
                  pMe->m_pIText = NULL;
                  return;
               }

               // Set the soft key menu rectangle coordinates.
               softKeyRect.x = 0;
               softKeyRect.y = pMe->m_dInfo.cyScreen - (pMe->m_nLineHeight * 4);
               softKeyRect.dx = pMe->m_dInfo.cxScreen;
               softKeyRect.dy = pMe->m_nLineHeight * 2;
               // Set Rectangle
               IMENUCTL_SetRect(pMe->m_pISoftKey, &softKeyRect);

               // Set title of the text control object
               STR_TO_WSTR("TextCtl Title ", szBuf, sizeof(szBuf));
               ITEXTCTL_SetTitle(pMe->m_pIText, NULL, NULL, szBuf);

               // Set the max size of the text supported by the text control object.
               ITEXTCTL_SetMaxSize(pMe->m_pIText, 100);

               // Set properties of the text control object to have a frame and 
               // to have a multi-lined text edit capability.
               ITEXTCTL_SetProperties(pMe->m_pIText, TP_FRAME | TP_MULTILINE );

               // Specify the area where the text control object is to be placed.
               rect.y = pMe->m_nLineHeight * 2;
               rect.dy = pMe->m_dInfo.cyScreen/2;
               ITEXTCTL_SetRect(pMe->m_pIText, &rect);


               // Initialize the softkey menu of the text control.
               ITEXTCTL_SetSoftKeyMenu (pMe->m_pIText, pMe->m_pISoftKey);                

               // Initialize the text field of the text control object.
               STR_TO_WSTR("SoftKey menu Text..", szBuf, sizeof(szBuf));
               ITEXTCTL_SetText(pMe->m_pIText, szBuf, sizeof (szBuf));

               // Set both the soft key and text control active.
               IMENUCTL_SetActive(pMe->m_pISoftKey,TRUE);
               ITEXTCTL_SetActive (pMe->m_pIText, TRUE);

               // The ITextCtl object is NOT released at this time. The ITextCtl 
               // object is released in the HandleEvents function under EVT_KEY. 
               // This gives the user the ability to edit text on an active 
               // ITextCtl object. The ITextCtl object is released when the user 
               // selects the Up/Down arrow to move to another menu selection.
            }
         }
         break;
      default:
         return;
   }
   
   // Display above event. 
   DisplayEvent (pMe, wParam);

   return;
}

/*===========================================================================

FUNCTION: DisplayEvent

DESCRIPTION
    This function is a display function which sets up the initial screen
    for the display output for a given interface function. This displays
    the function name at the top of the screen and displays the
    strings "Press Up/Down" and "keys to continue.." at the bottom of 
    the screen. In the case of functions that have no output it also 
    displays the string "No Output..." in the middile of the screen.

PROTOTYPE:
   void DisplayEvent (CITextCtlApp *pMe, uint16 wParam)

PARAMETERS:
   pMe:    [in]: Pointer to CITextCtlApp interface object
   wParam: [in]: API function type Id selected by user

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void DisplayEvent (CITextCtlApp *pMe, uint16 wParam)
{
   AECHAR szBuf[100];
   AEERect rc; 

   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
      return;

   // Set display rectangle
   SETAEERECT (&rc, 0, 0, pMe->m_dInfo.cxScreen, pMe->m_dInfo.cyScreen);

   if (pMe->a.m_pIDisplay)
   {
      switch (wParam)
      {
         case USAGE_BASIC_USAGE:
            STR_TO_WSTR("Basic Usage:", szBuf, sizeof(szBuf));
            break;
         case USAGE_HANDLEEVENT:
            STR_TO_WSTR("HandleEvent", szBuf, sizeof(szBuf));
            break;
         case USAGE_REDRAW:
            STR_TO_WSTR("Redraw", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETACTIVE:
            STR_TO_WSTR("SetActive", szBuf, sizeof(szBuf));
            break;
         case USAGE_ISACTIVE:
            STR_TO_WSTR("IsActive", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETRECT:
            STR_TO_WSTR("SetRect", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETRECT:
            STR_TO_WSTR("GetRect", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETPROPERTIES:
            STR_TO_WSTR("SetProperties", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETPROPERTIES:
            STR_TO_WSTR("GetProperties", szBuf, sizeof(szBuf));
            break;
         case USAGE_RESET:
            STR_TO_WSTR("Reset", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETTITLE_LOCAL:
            STR_TO_WSTR("SetTitle (Local)", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETTITLE_RES:
            STR_TO_WSTR("SetTitle (Resource)", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETTEXT:
            STR_TO_WSTR("SetText", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETTEXT:
            STR_TO_WSTR("GetText", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETTEXTPTR:
            STR_TO_WSTR("GetTextPtr", szBuf, sizeof(szBuf));
            break;
         case USAGE_ENABLECOMMAND:
            STR_TO_WSTR("EnableCommand", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETMAXSIZE:
            STR_TO_WSTR("SetMaxSize", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETSOFTKEYMENU:
            STR_TO_WSTR("SetSoftKeyMenu", szBuf, sizeof(szBuf));
            break;
         default:
            STR_TO_WSTR("Unsupported fn", szBuf, sizeof(szBuf));
      }
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,5, 5, &rc,
		   IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);

      // Functions with no output
      switch (wParam)
      {
         case USAGE_GETRECT:
         case USAGE_GETPROPERTIES:
         case USAGE_ENABLECOMMAND:
         case USAGE_SETMAXSIZE:
            // Place in the middle of screen
            STR_TO_WSTR("No Output...", szBuf, sizeof(szBuf));
 		      IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,10, 
               rc.dy/2 - pMe->m_nLineHeight, &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
         default:
            break;
      }

      STR_TO_WSTR("UP/DOWN to continue..", szBuf, sizeof(szBuf));
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,5, rc.dy - 
         pMe->m_nLineHeight, &rc, IDF_ALIGN_NONE | IDF_TEXT_TRANSPARENT);
		IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);
   }
}

/*===========================================================================

FUNCTION: BuildMainMenu

DESCRIPTION
    This function builds the main menu when this applet is started.
    It displays a simple menu with menu items corresponding to 
    each of the interface functions.
    
PROTOTYPE:
   void BuildMainMenu(CITextCtlApp *pMe)

PARAMETERS:
   pMe: [in]: Pointer to AEEApplet struct

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void BuildMainMenu(CITextCtlApp *pMe)
{
	AEERect rc;
   AECHAR szBuf[100];

   if (pMe->m_pIMenu == NULL || pMe->a.m_pIShell == NULL)
      return;

   // Set display rectangle
   SETAEERECT (&rc, 0, 0, pMe->m_dInfo.cxScreen, pMe->m_dInfo.cyScreen);

	// Set Title
   STR_TO_WSTR("ITextCtl Examples", szBuf, sizeof(szBuf));
	IMENUCTL_SetTitle(pMe->m_pIMenu, NULL, 0, szBuf);

	IMENUCTL_SetRect(pMe->m_pIMenu, &rc);  

	//Add invidual entries to the Menu

   // Add Basic Usage to menu
   STR_TO_WSTR("1. Basic Usage", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_BASIC_USAGE, szBuf, 0);

   // Add Handle Event to menu
   STR_TO_WSTR("2. HandleEvent", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_HANDLEEVENT, szBuf, 0);

   // Add Redraw to menu
   STR_TO_WSTR("3. Redraw", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_REDRAW, szBuf, 0);
			
   // Add SetActive to menu
   STR_TO_WSTR("4. SetActive", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETACTIVE, szBuf, 0);

   // Add IsActive to menu
   STR_TO_WSTR("5. IsActive", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_ISACTIVE, szBuf, 0);

	// Add SetRect to menu
   STR_TO_WSTR("6. SetRect", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETRECT, szBuf, 0);

	// Add GetRect to menu
   STR_TO_WSTR("7. GetRect", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETRECT, szBuf, 0);	

	// Add Set Properties to menu
   STR_TO_WSTR("8. SetProperties", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPROPERTIES, szBuf, 0);	

	// Add GetProperties to menu
   STR_TO_WSTR("9. GetProperties", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETPROPERTIES, szBuf, 0);	

	// Add Reset to menu
   STR_TO_WSTR("10. Reset", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_RESET, szBuf, 0);	

   // Add SetTitle to menu
   STR_TO_WSTR("11. SetTitle (Local)", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETTITLE_LOCAL, szBuf, 0);	

   // Add SetTitle to menu
   STR_TO_WSTR("12. SetTitle (Resource)", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETTITLE_RES, szBuf, 0);	

   // Add SetText to menu
   STR_TO_WSTR("13. SetText", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETTEXT, szBuf, 0);	

   // Add GetText to menu
   STR_TO_WSTR("14. GetText", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETTEXT, szBuf, 0);	

	// Add SetTextPtr to menu
   STR_TO_WSTR("15. GetTextPtr", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETTEXTPTR, szBuf, 0);	

	// Add EnableCommand to menu
   STR_TO_WSTR("16. EnableCommand", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_ENABLECOMMAND, szBuf, 0);	

	// Add SetMaxSize to menu
   STR_TO_WSTR("17. SetMaxSize", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETMAXSIZE, szBuf, 0);	

	// Add SetSoftKeyMenu to menu
   STR_TO_WSTR("18. SetSoftKeyMenu", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETSOFTKEYMENU, szBuf, 0);	

   // Activate menu
	IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);
}

/*===========================================================================

FUNCTION: DisplayOutput

DESCRIPTION
    This function displays an output string at a given line number on the
    screen. If the nline parameter is a negative value (-1) the string
    is displayed in the middle of the screen. If the "nline" value is larger
    than or equal to zero the "nline" value is multiplied by 15 and the 
    resulting value in pixels is set to the y-coordinate of the start of 
    the string display on the screen. If the string does not fit on one line
    the string wraps around to the next line (spaced rougly 10-15 pixels apart).
    By default 5 is used as the starting the x-coordinate of a displayed 
    string.

    How many characters that fit on one line is calculated for each line 
    that is wrapped around to the next line.

    Note: depending on the phone screen size and the fonts used for characters 
          the output might differ on different handsets (devices). Where some 
          handsets will have a smaller screen and large default fonts which will 
          cause partial overlapping of lines. This function does not try to address
          these issues (this is meant as a simple display function).
    
PROTOTYPE:
   void DisplayOutput(CITextCtlApp * pi, int nline, char *pszStr)

PARAMETERS:
   pi:     [in]: Contains a pointer to the CITextCtlApp class.
   nline:  [in]: Contains the line number to start displaying the text. 
        The line numbers are by default spaced 15 pixels apart along the y-axis.
   pszStr: [in]: The character string to be displayed on the screen.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void DisplayOutput(CITextCtlApp * pMe, int nline, char *pszStr)
{
   AECHAR * pszBuf = NULL;
   AECHAR * psz = NULL;
   int pixelWidth;
   AEEFont font = AEE_FONT_NORMAL;
   int pnFits = 0, dy;
   int totalCh = 0;

   if (pMe == NULL)
      return;

   // Allocate buffer to hold string.
   if ((pszBuf = (AECHAR *)MALLOC(200)) == NULL)
      return;

   // Convert to wide string (unicode)
   STR_TO_WSTR ((char *)pszStr, pszBuf, 200);

   // If nlines is zero then print this string starting around the middle of 
   // the screen. Or else multiply nlines by 10 to decide the y coordinate of
   // the start of the string.
   if (nline < 0) {
      dy = pMe->m_dInfo.cyScreen*2/5;
   }
   else{
      dy = nline * pMe->m_nLineHeight;
   }

   // psz keeps track of the point from which to write from the string buffer
   // in case the string does not fit one line and needs to wrap around in the
   // next line.
   psz = pszBuf;
      
   // Need to calculate the lotal string length to decide if any wrapping
   // around is needed.
   totalCh = STRLEN ((char *)pszStr);

   // Keep displaying text string on multiple lines if the string can't be displayed
   // on one single line. Lines are spaced 15 pixels apart.
   while ((totalCh > 0) && (*psz != NULL))
   { 
      // Get information on how many characters will fit in a line.
      // Give the pointer to the buffer to be displayed, and the number of
      // pixels along the x axis you want to display the string in (max number)
      // pnFits will have the max number of chars that will fit in the maxWidth
      // number of pixels (given string can't fit in one line), or the number of 
      // chars in the string (if it does fit in one line). pnWidth gives the
      // number of pixels that will be used to display pnFits number of chars.
      pixelWidth = IDISPLAY_MeasureTextEx(pMe->a.m_pIDisplay,
                      font, 
                      (AECHAR *) psz,  // Start of the buffer to display,
                      -1,
                      pMe->m_dInfo.cxScreen - 5, // maxWidth
                      &pnFits);         // Number of chars that will fit a line

      // If pnFits is zero there is something wrong in the input to above function. 
      // Normally this scenario should not occur. But, have the check anyway.
      if (pnFits == 0)
      {
         FREE (pszBuf);
         return;
      }

      IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, psz, pnFits, 5 /*start dx*/, 
         dy, 0 /* use default rectangle coordinates */, 0);

      psz += pnFits;      // move pointer to the next segment to be displayed
      totalCh -= pnFits;  // reduce the total number of characters to still display
      dy += pMe->m_nLineHeight; // Place next line charHeight pixels below the 
                                // previous line. 
      IDISPLAY_Update(pMe->a.m_pIDisplay);
      if (totalCh < pnFits)
         pnFits = totalCh;  // if total number is less than pnFits, adjust pnFits
   }

   FREE (pszBuf);
   return;   
} // End of DisplayOutput

