/*===========================================================================

FILE: itimectlusage.c

SERVICES: Sample applet using AEE

DESCRIPTION
  This file contains usage examples of ITimeCtl interface. ITimeCtl interface
  functions allow the user to create/operate on ITimeCtl objects. The time control 
  interface has three modes. Three are the clock mode, stop-watch mode, and the 
  count-down mode. Objects of the three modes are instantiated using the
  AEECLSID_CLOCKCTL, AEECLSID_STOPWATCHCTL, and AEECLSID_COUNTDOWNCTL class IDs.

PUBLIC CLASSES:  
   N/A


   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/

/*===============================================================================
                     INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "AEEModGen.h"
#include "AEEAppGen.h"
#include "AEEShell.h"
#include "AEEMenu.h"
#include "AEEStdLib.h"
#include "AEEUsageAppIDs.h"
#include "AEETime.h"

/*===========================================================================

                      PUBLIC DATA DECLARATIONS

===========================================================================*/

/*-------------------------------------------------------------------
            Type Declarations
-------------------------------------------------------------------*/
// TimeCtl app struct. This is the main struct for this applet. This will hold 
// all the data members that needs to be remembered throughout the life of
// the applet. 
// THE FIRST DATA MEMBER OF THIS STRUCT MUST BE AN AEEApplet OBJECT. 
// This is critical requirement which is related to an applet being
// dynamically loaded. The app developers should add their data members
// following the AEEApplet data member.
typedef struct _CITimeCtlApp
{
	AEEApplet	a;        // Mandatory first AEEApplet data member
   IMenuCtl *  m_pIMenu;
   ITimeCtl *  m_pITime;
   ITimeCtl *  m_pStopWatch;
   ITimeCtl *  m_pCountDown;

   // Device parameters
   int             m_nLineHeight;
   AEEDeviceInfo   m_dInfo;

}CITimeCtlApp;

/*-------------------------------------------------------------------
            Function Prototypes
-------------------------------------------------------------------*/
// Mandatory function:
// App Handle Event function
static boolean TimeCtlApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);

// App specific data alloc-init/free functions
static boolean ITimeCtl_InitAppData(IApplet* pMe);
static void ITimeCtl_FreeAppData(IApplet* pMe);

static void BuildMainMenu(CITimeCtlApp *pMe);
static void ITimeCtlUsage (CITimeCtlApp * pMe, uint16 wParam);
static void DisplayEvent (CITimeCtlApp *pMe, uint16 wParam);
static void DisplayOutput(CITimeCtlApp * pMe, int nline, char *pszStr);

/*-------------------------------------------------------------------
            Global Constant Definitions
-------------------------------------------------------------------*/
// App Resource File
#define  APP_RES_FILE      "itimectl.bar"

// App specific constants
#define USAGE_BASIC_USAGE       100
#define USAGE_HANDLEEVENT       101
#define USAGE_HANDLEEVENT_SW    102
#define USAGE_HANDLEEVENT_CD    103
#define USAGE_REDRAW            104
#define USAGE_SETACTIVE         105
#define USAGE_SETACTIVE_SW      106
#define USAGE_SETACTIVE_CD      107
#define USAGE_ISACTIVE          108
#define USAGE_SETRECT           109
#define USAGE_GETRECT           110
#define USAGE_SETPROPERTIES     111
#define USAGE_SETPROPERTIES_SW  112
#define USAGE_SETPROPERTIES_CD  113
#define USAGE_GETPROPERTIES     114
#define USAGE_RESET             115
#define USAGE_SETTIME           116
#define USAGE_GETTIME           117
#define USAGE_ENABLECOMMAND     118
#define USAGE_GETTIMESTRING     119
#define USAGE_SETINCREMENT      120
#define USAGE_SETEDITFIELD      121
#define USAGE_SETEDITFIELD_SW   122
#define USAGE_SETEDITFIELD_CD   123

/*-------------------------------------------------------------------
            Global Data Definitions
-------------------------------------------------------------------*/


/*-------------------------------------------------------------------
            Static variable Definitions
-------------------------------------------------------------------*/


/*===============================================================================
                     FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to ITimeCtl_InitAppData(). 
   This is done as follows: InitAppData() is called to initialize AppletData 
   instance. It is app developers responsibility to fill-in app data initialization 
   code of InitAppData(). App developer is also responsible to release memory 
   allocated for data contained in AppletData -- this can be done in 
   ITimeCtl_FreeAppData().

PROTOTYPE:
   int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell object. 

	pIModule: pin]: Contains pointer to the IModule object to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid IApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the app needs to be loaded and if AEEApplet_New() invocation was
     successful
  EFAILED: If the app does not need to be loaded or if errors occurred in 
     AEEApplet_New(). If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
{
   *ppObj = NULL;
		
   // We want to load this App. So, invoke AEEApplet_New().To it, pass the
   // address of the app-specific handle event function.
   if(ClsId == AEECLSID_TIMECLT_APP){
      if(AEEApplet_New(sizeof(CITimeCtlApp), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)TimeCtlApp_HandleEvent,(PFNFREEAPPDATA)ITimeCtl_FreeAppData)
         == TRUE)
      {
         if (ITimeCtl_InitAppData((IApplet*)*ppObj) == TRUE)
         {
            return(AEE_SUCCESS);
         }
      }
   }
	return (EFAILED);
}

/*===========================================================================

FUNCTION TimeCtlApp_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean TimeCtlApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

	ecode: Specifies the Event sent to this applet

   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/
static boolean TimeCtlApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
   CITimeCtlApp * pMe = (CITimeCtlApp*)pi;

   switch (eCode) 
	{
       case EVT_APP_START:
         // Create the IMenu interface object. When the applet is started the 
         // main menu is set up. When the user selects a menu item from the main 
         // menu, an EVT_COMMAND event is received by this applet. The processing for 
         // EVT_COMMAND event is found below.
		   if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, (void **)&pMe->m_pIMenu)
            != SUCCESS)
         {
				return FALSE;
         }

         // Build the main menu
			BuildMainMenu(pMe);
			return(TRUE);
      case EVT_APP_STOP:
         return(TRUE);
      case EVT_KEY:
         // Call the TimeCtl HandleEvent to see if this event would be 
         // handled by the TimeCtl object. When the Up or Down key is 
         // pressed we will fail the check below, release the ITimeCtl object
         // and pass the controls to the menu control object.
         // First, call handler of the regular time control object
         if (pMe->m_pITime != NULL
            && ITIMECTL_HandleEvent(pMe->m_pITime, EVT_KEY, wParam, 0))
         {
            // This event was handled by TimeCtl. Return here since
            // this was handled by the time control object.
            return TRUE;
         }

         // Call the handler of the Count-Down object
         if (pMe->m_pCountDown != NULL
            && ITIMECTL_HandleEvent(pMe->m_pCountDown, EVT_KEY, wParam, 0))
         {
            // This event was handled by TimeCtl. Return here since
            // this was handled by the time control object.
            return TRUE;
         }

         // Call the handler of the Stop-watch object.
         if (pMe->m_pStopWatch != NULL
            && ITIMECTL_HandleEvent(pMe->m_pStopWatch, EVT_KEY, wParam, 0))
         {
            // This event was handled by TimeCtl. Return here since
            // this was handled by the time control object.
            return TRUE;
         }

         // Release the ITimeCtl object if it is not NULL.
         if (pMe->m_pITime != NULL)
         {
            ITIMECTL_Release (pMe->m_pITime);
            pMe->m_pITime = NULL;
         }

         // Release the Count-Down object if it is not NULL.
         if (pMe->m_pCountDown != NULL)
         {
            ITIMECTL_Release (pMe->m_pCountDown);
            pMe->m_pCountDown = NULL;
         }

         // Release the Stop-Watch object if it is not NULL.
         if (pMe->m_pStopWatch != NULL)
         {
            ITIMECTL_Release (pMe->m_pStopWatch);
            pMe->m_pStopWatch = NULL;
         }

         // We come here when a key is pressed on the handset keypad. Call 
         // the IMenuCtl handle event function to handle this key press.
			if(pMe->m_pIMenu != NULL && IMENUCTL_HandleEvent(pMe->m_pIMenu, EVT_KEY, wParam, 0))
				return TRUE;
         else
         {
            // We would have come here because the pressed key was not handled 
            // by either the DateCtl or the MenuCtl HandleEvent functions (the 
            // keys handled by the above controls are Up/Down/Select/Left/Right).
            //
            // Hence, we have reached here since 0-9, *, #, or CLR keys were
            // pressed. If the key is "CLR" then redraw the main applet menu.
				// Or just return TRUE and ignore the key press
            if (wParam == AVK_CLR)
            {
					IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);
					IMENUCTL_Redraw (pMe->m_pIMenu);
            }
            return TRUE;
         }
		case EVT_COMMAND:
         // When a menu item is selected this applet receives the
         // EVT_COMMAND event in the wParam field.
  		   switch(wParam)
         {
            case USAGE_BASIC_USAGE:
            case USAGE_HANDLEEVENT:
            case USAGE_HANDLEEVENT_SW:
            case USAGE_HANDLEEVENT_CD:
            case USAGE_REDRAW:
            case USAGE_SETACTIVE:
            case USAGE_SETACTIVE_SW:
            case USAGE_SETACTIVE_CD:
            case USAGE_ISACTIVE:
            case USAGE_SETRECT:
            case USAGE_GETRECT:
            case USAGE_SETPROPERTIES:
            case USAGE_SETPROPERTIES_SW:
            case USAGE_SETPROPERTIES_CD:
            case USAGE_GETPROPERTIES:
            case USAGE_RESET:
            case USAGE_SETTIME:
            case USAGE_GETTIME:
            case USAGE_ENABLECOMMAND:
            case USAGE_GETTIMESTRING:
            case USAGE_SETINCREMENT:
            case USAGE_SETEDITFIELD:
            case USAGE_SETEDITFIELD_SW:
            case USAGE_SETEDITFIELD_CD:
               // Call the usage function that exercises all the 
               // interface functions.
               ITimeCtlUsage (pMe, wParam);
               return TRUE;
		   	default:
			   	return FALSE;
         }
      default:
         break;
   }
   return FALSE;
}



/*===========================================================================

FUNCTION ITimeCtl_InitAppData

DESCRIPTION
   This function initializes app specific data allocates memory for app data 
   (AppletData) and sets it to pAppData of AEEApplet. It also function . 
   App developer needs to fill-in the initialization code.

PROTOTYPE:
	boolean ITimeCtl_InitAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains 
   information specific to this applet. It was initialized during the 
   AEEClsCreateInstance().

DEPENDENCIES
   Assumes pi is not NULL

RETURN VALUE
  TRUE: If the app has app data is allocated and initialized successfully
  FALSE: Either app data could not be allocated or initialzied

SIDE EFFECTS
  none
===========================================================================*/
static boolean ITimeCtl_InitAppData(IApplet* pi)
{
   int pnAscent;        // Stores the ascent in number of pixels
   int pnDescent;       // Stores the descent in number of pixels

   CITimeCtlApp * pMe = (CITimeCtlApp*)pi;

   // Initialize the MenuCtl pointer to NULL
   pMe->m_pIMenu = NULL;
   pMe->m_pITime = NULL;
   pMe->m_pStopWatch = NULL;
   pMe->m_pCountDown = NULL;

   // Get the font metrics info
   pMe->m_nLineHeight = IDISPLAY_GetFontMetrics (pMe->a.m_pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&pMe->m_dInfo);

   return TRUE;
}

/*===========================================================================

FUNCTION ITimeCtl_FreeAppData

DESCRIPTION
   Frees data contained in app data and memory for app data itself.
   App developer needs to free data contained in AppletData.

PROTOTYPE:
	void ITimeCtl_FreeAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains 
   information specific to this applet. It was initialized during the 
   AEEClsCreateInstance().

DEPENDENCIES
   Assumes pi is not NULL

RETURN VALUE
   None

SIDE EFFECTS
   None
===========================================================================*/
static void ITimeCtl_FreeAppData(IApplet* pi)
{
   CITimeCtlApp * pMe = (CITimeCtlApp*)pi;

   // Release main menu
   if (pMe->m_pIMenu != NULL)
   {
      IMENUCTL_Release (pMe->m_pIMenu);
      pMe->m_pIMenu = NULL;
   }
   // Release TimeCtl object if it is not released
   if (pMe->m_pITime != NULL)
   {
      ITIMECTL_Release (pMe->m_pITime);
      pMe->m_pITime = NULL;
   }
   // Release Stop watch object if it is not released
   if (pMe->m_pStopWatch != NULL)
   {
      ITIMECTL_Release (pMe->m_pStopWatch);
      pMe->m_pStopWatch = NULL;
   }
   // Release count down object if it is not released
   if (pMe->m_pCountDown != NULL)
   {
      ITIMECTL_Release (pMe->m_pCountDown);
      pMe->m_pCountDown = NULL;
   }
}

/*===============================================================================
                     APP HELPER FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: ITimeCtlUsage

DESCRIPTION
	This function encompasses all the usage examples of all the functions
   in code blocks switched using the BREW API function Id passed into this
   function. The code blocks will include almost all the variable declarations
   needed to demonstrate the usage of a given function. Within the code block
   all the needed interfaces are instantiated and released subsequent to their
   use.

   There are some exceptions to the above general rule, such as call-back functions,
   common helper function, or common display functions called from within the
   code block which are defined elsewhere in this file.

   There are some instances where an object instantiated within a code block is
   not released at the end of the code block, to be released at a later point
   (for example in a call-back function, in this case pointers to such objects
   are stored in the AEEApplet object to be released later).
   
PROTOTYPE:
   void ITimeCtlUsage (CITimeCtlApp * pMe, uint16 wParam)

PARAMETERS:
	pMe: [in]: Contains a pointer to the CITimeCtlApp struct.
   wParam: [in]: Contains the Id of the function type to be used (contains the
      Id corresponding to the menu item selected on the emulator).

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the functions call was successful. FALSE otherwise

SIDE EFFECTS
  none
===========================================================================*/
static void  ITimeCtlUsage (CITimeCtlApp * pMe, uint16 wParam)
{
   AEERect rect;
   int lineHeight = pMe->m_nLineHeight;
   IShell *pIShell = pMe->a.m_pIShell;

   SETAEERECT (&rect, 0, 2*lineHeight, pMe->m_dInfo.cxScreen, pMe->m_dInfo.cyScreen/2);

   // Erase screen first for display output purposes.
   IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
	IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
	IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);

   switch (wParam)
   {
      case USAGE_BASIC_USAGE:
         {
            // 1. Usage: Basic ITimeCtl Usage
            int32     dwMSecs = 123456;
            ITimeCtl * pITimeCtl = NULL;
            ITimeCtl * pStopWatch = NULL;
            ITimeCtl * pCountDown = NULL;

            // Create ITimeCtl/pStopWatch/pCountDown objects. This step needs to be 
            // done only if the ITimeCtl/pStopWatch/pCountDown objects have not been 
            // created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_CLOCKCTL, (void **)&pITimeCtl);
            ISHELL_CreateInstance(pIShell, AEECLSID_STOPWATCHCTL, (void **)&pStopWatch);
            ISHELL_CreateInstance(pIShell, AEECLSID_COUNTDOWNCTL, (void **)&pCountDown);

            if (pITimeCtl && pStopWatch && pCountDown)
            {
               // Set initial rectangle coordinates
               SETAEERECT (&rect, 0, lineHeight, pMe->m_dInfo.cxScreen/2, lineHeight);

               // Each of the SetTime function invokes (subsequent to the SetRect 
               // functions) draws the time objects on the screen. Here, three different
               // rectangles are set up and the SetRect function is called so that 
               // each object will be drawn at the top half of the screen one after
               // another.
               ITIMECTL_SetRect(pITimeCtl, &rect);
               rect.y = lineHeight*2;
               ITIMECTL_SetRect(pStopWatch, &rect);
               rect.y = lineHeight*3;
               ITIMECTL_SetRect(pCountDown, &rect);

               // Set properties
               ITIMECTL_SetProperties(pITimeCtl,  TP_AUTOREDRAW | TP_NOEDIT_AMPM);
               ITIMECTL_SetProperties(pStopWatch,  TP_AUTOREDRAW | TP_NO_MSECONDS);
               ITIMECTL_SetProperties(pCountDown,  TP_AUTOREDRAW | TP_NO_SECONDS);

               // Set the time.
               ITIMECTL_SetTime(pITimeCtl, dwMSecs);
               ITIMECTL_SetTime(pStopWatch, dwMSecs);
               ITIMECTL_SetTime(pCountDown, dwMSecs);

               // Need to clear the middle of the screen since the above SetTime
               // by default draws the 
               // Change the rectangle coordinates and redraw again.
               SETAEERECT (&rect, 0, lineHeight*4, pMe->m_dInfo.cxScreen, lineHeight*3/2);
               ITIMECTL_SetRect(pITimeCtl, &rect);
               
               // ITIMECTL_Redraw: instructs the time control object to redraw it's 
               // contents. The time control object does not redraw its contents 
               // every time the underlying data behind the time control changes. 
               // This allows several data updates to occur while minimizing screen 
               // flashes. For example, several changes can be made to the contents 
               // of the time control object with no visible effect until the Redraw 
               // method is called.
               ITIMECTL_Redraw (pITimeCtl);

               // Change the rectangle coordinates and redraw again.
               SETAEERECT (&rect, 0, lineHeight*6 - (lineHeight/2), pMe->m_dInfo.cxScreen,
                  lineHeight*3/2);
               ITIMECTL_SetRect(pStopWatch, &rect);
               
               // Redraw stop watch object
               ITIMECTL_Redraw (pStopWatch);

               // Change the rectangle coordinates and redraw again.
               SETAEERECT (&rect, 0, lineHeight*7, pMe->m_dInfo.cxScreen,lineHeight*3/2);
               ITIMECTL_SetRect(pCountDown, &rect);
               
               // Redraw the count down object
               ITIMECTL_Redraw (pCountDown);

               // Release time control objects. This step needs to be done
               // only if no further use of the objects are needed.
               ITIMECTL_Release (pITimeCtl);
               ITIMECTL_Release (pStopWatch);
               ITIMECTL_Release (pCountDown);
            }
            else
            {
               // Atleast one of the objects did not get created. So release 
               // whichever got created
               if (pITimeCtl != NULL)
                  ITIMECTL_Release (pITimeCtl);
               if (pStopWatch != NULL)
                  ITIMECTL_Release (pStopWatch);
               if (pCountDown != NULL)
                  ITIMECTL_Release (pCountDown);
            }
         }
         break;
      case USAGE_HANDLEEVENT:
         {
            // 2. Usage: ITIMECTL_HandleEvent
            AEEEvent  evt = EVT_KEY;        // TimeCtl only supports EVT_KEY event
            uint16    wParam  = AVK_RIGHT;  // Supports SELECT, LEFT, RIGHT, UP, DOWN
            uint32    dwParam = 0;
            int32     dwMSecs = 123567;

            // Create ITimeCtl object. This step needs to be done only if
            // the ITimeCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_CLOCKCTL, (void **)&pMe->m_pITime);

            if (pMe->m_pITime)
            {
               // SetRect/SetTime used for output display purposes only. For more 
               // details see ITIMECTL_SetTime/ITIMECTL_SetRect usage. 
               ITIMECTL_SetRect(pMe->m_pITime, &rect);
               ITIMECTL_SetTime(pMe->m_pITime, dwMSecs);

               // The time control needs to be active for the handle event function
               // to succeed. See ITIMECTL_SetActive usage for more information.
               ITIMECTL_SetActive (pMe->m_pITime, TRUE);

               // ITIMECTL_HandleEvent: is used to handle the events received by 
               // time control object. If the time control object is in non 
               // edit mode, it processes only set title, set time and press of 
               // UP and DOWN key events. In time edit mode, it processes various 
               // events like key up, key down, key held, set title, set time, 
               // command event from soft key menu.

               // Invoke HandleEvent. 
               // This will move the cursor to the right by one character.
               // By default the cursor is positioned at the begining of the
               // time string. By invoking HandleEvents function with EVT_KEY event
               // and the key value of AVK_RIGHT the field selection will be moved 
               // to the second field (minute field) on the time control object.
               //  Note: by default the field selection is the first field (hour field).
               //
               // Note: a more practical or typical use of ITIMECTL_HandleEvent is 
               // found in the HandleEvent function of this applet under EVT_KEY event.
               if ((ITIMECTL_HandleEvent(pMe->m_pITime, evt, wParam, dwParam)) == FALSE)
               {
                  // Do not need to return here. Fall through and invoke 
                  // DisplayEvents() function.
               }

               // The ITimeCtl object is NOT released at this point. The ITimeCtl 
               // object is released in the HandleEvents function under EVT_KEY. 
               // This gives the user the ability to edit time on an active 
               // ITimeCtl object on the screen by pressing the up/down/left/right
               // keys. The ITimeCtl object is released when the user 
               // presses any key other than UP/DOWN/LEFT/RIGHT or SELECT.
            }
         }
         break;
      case USAGE_HANDLEEVENT_SW:
         {
          //  // 2. Usage: ITIMECTL_HandleEvent
          //  AEEEvent  evt = EVT_KEY;        // TimeCtl only supports EVT_KEY event
           // uint16    wParam  = AVK_RIGHT;  // Supports SELECT, LEFT, RIGHT, UP, DOWN
          //  uint32    dwParam = 0;
            int32     dwMSecs = 123567;

            // Create ITimeCtl object. This step needs to be done only if
            // the ITimeCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_STOPWATCHCTL, (void **)&pMe->m_pStopWatch);

            if (pMe->m_pStopWatch)
            {
               // SetRect/SetTime used for output display purposes only. For more 
               // details see ITIMECTL_SetTime/ITIMECTL_SetRect usage. 
               ITIMECTL_SetRect(pMe->m_pStopWatch, &rect);
               ITIMECTL_SetTime(pMe->m_pStopWatch, dwMSecs);

               // The time control needs to be active for the handle event function
               // to succeed. See ITIMECTL_SetActive usage for more information.
               ITIMECTL_SetActive (pMe->m_pStopWatch, TRUE);

               // ITIMECTL_HandleEvent: is used to handle the events received by 
               // time control object. If the time control object is in non 
               // edit mode, it processes only set title, set time and press of 
               // UP and DOWN key events. In time edit mode, it processes various 
               // events like key up, key down, key held, set title, set time, 
               // command event from soft key menu.

               // Invoke HandleEvent. 
               // This will move the cursor to the right by one character.
               // By default the cursor is positioned at the begining of the
               // time string. By invoking HandleEvents function with EVT_KEY event
               // and the key value of AVK_RIGHT the field selection will be moved 
               // to the second field (minute field) on the time control object.
               //  Note: by default the field selection is the first field (hour field).
               //
               // Note: a more correct use of ITIMECTL_HandleEvent is found in 
               // the HandleEvent function of this applet under EVT_KEY event.
               // The following four lines are commented out because of above reason.
               
               //if ((ITIMECTL_HandleEvent(pMe->m_pStopWatch, evt, wParam, dwParam)) == FALSE)
               //{
                  // Do not need to return here. Fall through and invoke 
                  // DisplayEvents() function.
               //}

               // The stop watch object is NOT released at this point. The stop watch
               // object is released in the HandleEvents function under EVT_KEY. 
               // This gives the user the ability to edit time on an active 
               // stop watch object on the screen by pressing the up/down/left/right
               // keys. The stop watch object is released when the user 
               // presses any key other than UP/DOWN/LEFT/RIGHT or SELECT.
            }
         }
         break;
      case USAGE_HANDLEEVENT_CD:
         {
            // 2. Usage: ITIMECTL_HandleEvent
            AEEEvent  evt = EVT_KEY;        // TimeCtl only supports EVT_KEY event
            uint16    wParam  = AVK_RIGHT;  // Supports SELECT, LEFT, RIGHT, UP, DOWN
            uint32    dwParam = 0;
            int32     dwMSecs = 123567;

            // Create ITimeCtl object. This step needs to be done only if
            // the ITimeCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_COUNTDOWNCTL, (void **)&pMe->m_pCountDown);

            if (pMe->m_pCountDown)
            {
               // SetRect/SetTime used for output display purposes only. For more 
               // details see ITIMECTL_SetTime/ITIMECTL_SetRect usage. 
               ITIMECTL_SetRect(pMe->m_pCountDown, &rect);
               ITIMECTL_SetTime(pMe->m_pCountDown, dwMSecs);

               // The time control needs to be active for the handle event function
               // to succeed. See ITIMECTL_SetActive usage for more information.
               ITIMECTL_SetActive (pMe->m_pCountDown, TRUE);

               // ITIMECTL_HandleEvent: is used to handle the events received by 
               // time control object. If the time control object is in non 
               // edit mode, it processes only set title, set time and press of 
               // UP and DOWN key events. In time edit mode, it processes various 
               // events like key up, key down, key held, set title, set time, 
               // command event from soft key menu.

               // Invoke HandleEvent. 
               // This will move the cursor to the right by one character.
               // By default the cursor is positioned at the begining of the
               // time string. By invoking HandleEvents function with EVT_KEY event
               // and the key value of AVK_RIGHT the field selection will be moved 
               // to the second field (minute field) on the time control object.
               //  Note: by default the field selection is the first field (hour field).
               //
               // Note: a more practical use of ITIMECTL_HandleEvent is found in 
               // the HandleEvent function of this applet under EVT_KEY event.
               if ((ITIMECTL_HandleEvent(pMe->m_pCountDown, evt, wParam, dwParam)) == FALSE)
               {
                  // Do not need to return here. Fall through and invoke 
                  // DisplayEvents() function.
               }

               // The ITimeCtl object is NOT released at this point. The ITimeCtl 
               // object is released in the HandleEvents function under EVT_KEY. 
               // This gives the user the ability to edit time on an active 
               // ITimeCtl object on the screen by pressing the up/down/left/right
               // keys. The ITimeCtl object is released when the user 
               // presses any key other than UP/DOWN/LEFT/RIGHT or SELECT.
            }
         }
         break;
      case USAGE_REDRAW:
         {
            // 3. Usage: ITIMECTL_Redraw
            int32     dwMSecs = 123456;
            ITimeCtl * pITimeCtl = NULL;
            ITimeCtl * pStopWatch = NULL;
            ITimeCtl * pCountDown = NULL;

            // Create ITimeCtl/pStopWatch/pCountDown objects. This step needs to be 
            // done only if the ITimeCtl/pStopWatch/pCountDown objects have not been 
            // created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_CLOCKCTL, (void **)&pITimeCtl);
            ISHELL_CreateInstance(pIShell, AEECLSID_STOPWATCHCTL, (void **)&pStopWatch);
            ISHELL_CreateInstance(pIShell, AEECLSID_COUNTDOWNCTL, (void **)&pCountDown);

            if (pITimeCtl && pStopWatch && pCountDown)
            {
               // Set initial rectangle coordinates
               SETAEERECT (&rect, 0, lineHeight, pMe->m_dInfo.cxScreen/2,lineHeight);

               // Each of the SetTime function invokes (subsequent to the SetRect 
               // functions) draws the time objects on the screen. Here, three different
               // rectangles are set up and the SetRect function is called so that 
               // each object will be drawn at the top half of the screen one after
               // another.
               ITIMECTL_SetRect(pITimeCtl, &rect);
               rect.y = lineHeight*2;
               ITIMECTL_SetRect(pStopWatch, &rect);
               rect.y = lineHeight*3;
               ITIMECTL_SetRect(pCountDown, &rect);

               // Set the time.
               ITIMECTL_SetTime(pITimeCtl, dwMSecs);
               ITIMECTL_SetTime(pStopWatch, dwMSecs);
               ITIMECTL_SetTime(pCountDown, dwMSecs);

               // Need to clear the middle of the screen since the above SetTime
               // by default draws the 
               // Change the rectangle coordinates and redraw again.
               SETAEERECT (&rect, 0, lineHeight*4, pMe->m_dInfo.cxScreen/2,lineHeight*3/2);
               ITIMECTL_SetRect(pITimeCtl, &rect);
               
               // ITIMECTL_Redraw: instructs the time control object to redraw it's 
               // contents. The time control object does not redraw its contents 
               // every time the underlying data behind the time control changes. 
               // This allows several data updates to occur while minimizing screen 
               // flashes. For example, several changes can be made to the contents 
               // of the time control object with no visible effect until the Redraw 
               // method is called.
               ITIMECTL_Redraw (pITimeCtl);

               // Change the rectangle coordinates and redraw again.
               SETAEERECT (&rect, 0, lineHeight*6 - (lineHeight/2), pMe->m_dInfo.cxScreen,
                  lineHeight*3/2);
               ITIMECTL_SetRect(pStopWatch, &rect);
               
               // Redraw stop watch object
               ITIMECTL_Redraw (pStopWatch);

               // Change the rectangle coordinates and redraw again.
               SETAEERECT (&rect, 0, lineHeight*7, pMe->m_dInfo.cxScreen, lineHeight*3/2);
               ITIMECTL_SetRect(pCountDown, &rect);
               
               // Redraw the count down object
               ITIMECTL_Redraw (pCountDown);

               // Release time control objects. This step needs to be done
               // only if no further use of the objects are needed.
               ITIMECTL_Release (pITimeCtl);
               ITIMECTL_Release (pStopWatch);
               ITIMECTL_Release (pCountDown);
            }
            else
            {
               // Atleast one of the objects did not get created. So release 
               // whichever got created
               if (pITimeCtl != NULL)
                  ITIMECTL_Release (pITimeCtl);
               if (pStopWatch != NULL)
                  ITIMECTL_Release (pStopWatch);
               if (pCountDown != NULL)
                  ITIMECTL_Release (pCountDown);
            }
         }
         break;
      case USAGE_SETACTIVE:
         {
            // 4. Usage: ITIMECTL_SetActive
            int32     dwMSecs = 12345678;

            // Create ITimeCtl object. This step needs to be done only if
            // the ITimeCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_CLOCKCTL, (void **)&pMe->m_pITime);

            if (pMe->m_pITime)
            {
               // SetRect/SetTime used for output display purposes only. For more 
               // details see ITIMECTL_SetTime/ITIMECTL_SetRect usage. 
               ITIMECTL_SetRect(pMe->m_pITime, &rect);
               ITIMECTL_SetTime(pMe->m_pITime, dwMSecs);

               // ITEXTCTL_SetActive: is used to make a time control object active. 
               // Only an active time control object handles the event sent to it. 
               // Inactive time control object just ignores the events.
               //
               // An active time control object gives the user the ability to 
               // send key press events to the active ITimeCtl object.               
               ITIMECTL_SetActive (pMe->m_pITime, TRUE);

               // The ITimeCtl object is NOT released at this point. The ITimeCtl 
               // object is released in the HandleEvents function under EVT_KEY. 
               // This gives the user the ability to edit time on an active 
               // ITimeCtl object on the screen by pressing the up/down/left/right
               // keys. The ITimeCtl object is released when the user 
               // presses any key other than UP/DOWN/LEFT/RIGHT or SELECT.
            }
         }
         break;
      case USAGE_SETACTIVE_SW:
         {
            // 4. Usage: ITIMECTL_SetActive
            int32     dwMSecs = 12345678;

            // Create ITimeCtl object. This step needs to be done only if
            // the ITimeCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_STOPWATCHCTL, (void **)&pMe->m_pStopWatch);

            if (pMe->m_pStopWatch)
            {
               // SetRect/SetTime used for output display purposes only. For more 
               // details see ITIMECTL_SetTime/ITIMECTL_SetRect usage. 
               ITIMECTL_SetRect(pMe->m_pStopWatch, &rect);
               ITIMECTL_SetTime(pMe->m_pStopWatch, dwMSecs);

               // ITEXTCTL_SetActive: is used to make a time control object active. 
               // Only an active time control object handles the event sent to it. 
               // Inactive time control object just ignores the events.
               //
               // An active time control object gives the user the ability to 
               // send key press events to the active ITimeCtl object.               
               ITIMECTL_SetActive (pMe->m_pStopWatch, TRUE);

               // The StopWatch object is NOT released at this point. The StopWatch 
               // object is released in the HandleEvents function under EVT_KEY. 
               // This gives the user the ability to edit time on an active 
               // StopWatch object on the screen by pressing the up/down/left/right
               // keys. The StopWatch object is released when the user 
               // presses any key other than UP/DOWN/LEFT/RIGHT or SELECT.
            }
         }
         break;
      case USAGE_SETACTIVE_CD:
         {
            // 4. Usage: ITIMECTL_SetActive
            int32     dwMSecs = 12345678;

            // Create ITimeCtl object. This step needs to be done only if
            // the ITimeCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_COUNTDOWNCTL, (void **)&pMe->m_pCountDown);

            if (pMe->m_pCountDown)
            {
               // SetRect/SetTime used for output display purposes only. For more 
               // details see ITIMECTL_SetTime/ITIMECTL_SetRect usage. 
               ITIMECTL_SetRect(pMe->m_pCountDown, &rect);
               ITIMECTL_SetTime(pMe->m_pCountDown, dwMSecs);

               // ITEXTCTL_SetActive: is used to make a time control object active. 
               // Only an active time control object handles the event sent to it. 
               // Inactive time control object just ignores the events.
               //
               // An active time control object gives the user the ability to 
               // send key press events to the active ITimeCtl object.               
               ITIMECTL_SetActive (pMe->m_pCountDown, TRUE);

               // The CountDown object is NOT released at this point. The CountDown 
               // object is released in the HandleEvents function under EVT_KEY. 
               // This gives the user the ability to edit time on an active 
               // CountDown object on the screen by pressing the up/down/left/right
               // keys. The CountDown object is released when the user 
               // presses any key other than UP/DOWN/LEFT/RIGHT or SELECT.
            }
         }
         break;
      case USAGE_ISACTIVE:
         {
            // 5. Usage: ITIMECTL_IsActive
            int32   dwMSecs = 12345678;
            boolean bRetvalue = FALSE; 
            ITimeCtl * pITimeCtl = NULL;
            ITimeCtl * pStopWatch = NULL;
            ITimeCtl * pCountDown = NULL;

            // Create ITimeCtl/pStopWatch/pCountDown objects. This step needs to be 
            // done only if the ITimeCtl/pStopWatch/pCountDown objects have not been 
            // created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_CLOCKCTL, (void **)&pITimeCtl);
            ISHELL_CreateInstance(pIShell, AEECLSID_STOPWATCHCTL, (void **)&pStopWatch);
            ISHELL_CreateInstance(pIShell, AEECLSID_COUNTDOWNCTL, (void **)&pCountDown);

            if (pITimeCtl && pStopWatch && pCountDown)
            {
               // Set initial rectangle coordinates
               SETAEERECT (&rect, 0, lineHeight, pMe->m_dInfo.cxScreen/2,lineHeight);

               ITIMECTL_SetRect(pITimeCtl, &rect);
               ITIMECTL_SetTime(pITimeCtl, dwMSecs);
               ITIMECTL_SetActive (pITimeCtl, TRUE);

               rect.y = lineHeight* 2;
               ITIMECTL_SetRect(pStopWatch, &rect);
               ITIMECTL_SetTime(pStopWatch, dwMSecs);
               ITIMECTL_SetActive (pStopWatch, TRUE);

               rect.y = lineHeight* 3;
               ITIMECTL_SetRect(pCountDown, &rect);
               ITIMECTL_SetTime(pCountDown, dwMSecs);
               ITIMECTL_SetActive (pCountDown, TRUE);

               // ITIMECTL_IsActive: returns the active/inactive state of the 
               // time control object.
               if (((bRetvalue = ITIMECTL_IsActive (pITimeCtl)) == TRUE) &&
                   ((bRetvalue = ITIMECTL_IsActive (pStopWatch)) == TRUE) && 
                   ((bRetvalue = ITIMECTL_IsActive (pCountDown)) == TRUE))
               {
                  DisplayOutput (pMe, 4, "TimeCtl: is active ..");
                  DisplayOutput (pMe, 5, "StopWatch: is active ..");
                  DisplayOutput (pMe, 6, "CountDown: is active ..");
               }

               // Release time control objects. This step needs to be done
               // only if no further use of the objects are needed.
               ITIMECTL_Release (pITimeCtl);
               ITIMECTL_Release (pStopWatch);
               ITIMECTL_Release (pCountDown);
            }
            else
            {
               // Atleast one of the objects did not get created. So release 
               // whichever got created
               if (pITimeCtl != NULL)
                  ITIMECTL_Release (pITimeCtl);
               if (pStopWatch != NULL)
                  ITIMECTL_Release (pStopWatch);
               if (pCountDown != NULL)
                  ITIMECTL_Release (pCountDown);
            }
         }
         break;
      case USAGE_SETRECT:
         {
            // 6. Usage: ITIMECTL_SetRect
            int32   dwMSecs = 12345678;
            AEERect rct;
            ITimeCtl * pITimeCtl = NULL;
            ITimeCtl * pStopWatch = NULL;
            ITimeCtl * pCountDown = NULL;

            // Create ITimeCtl/pStopWatch/pCountDown objects. This step needs to be 
            // done only if the ITimeCtl/pStopWatch/pCountDown objects have not been 
            // created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_CLOCKCTL, (void **)&pITimeCtl);
            ISHELL_CreateInstance(pIShell, AEECLSID_STOPWATCHCTL, (void **)&pStopWatch);
            ISHELL_CreateInstance(pIShell, AEECLSID_COUNTDOWNCTL, (void **)&pCountDown);

            if (pITimeCtl && pStopWatch && pCountDown)
            {
               SETAEERECT (&rect, pMe->m_dInfo.cxScreen, lineHeight, pMe->m_dInfo.cxScreen/2,
                  lineHeight);
               // ITIMECTL_SetRect: can be used to set the coordinates specified by 
               // rc as control rectangle of the time control object
               ITIMECTL_SetRect(pITimeCtl, &rct);

               // Use SetTime function to assign a time value to the TimeCtl object.
               // This method will draw the time on the screen in the previously set
               // rectangle coordinates.
               ITIMECTL_SetTime(pITimeCtl, dwMSecs);

               // Set the rectangle for the stop watch object.
               rct.y = lineHeight*3;
               ITIMECTL_SetRect(pStopWatch, &rct);
               ITIMECTL_SetTime(pStopWatch, dwMSecs);

               // Set the rectangle for the count down object.
               rct.y = lineHeight*5;
               ITIMECTL_SetRect(pCountDown, &rct);
               ITIMECTL_SetTime(pCountDown, dwMSecs);

               // Release time control objects. This step needs to be done
               // only if no further use of the objects are needed.
               ITIMECTL_Release (pITimeCtl);
               ITIMECTL_Release (pStopWatch);
               ITIMECTL_Release (pCountDown);
            }
            else
            {
               // Atleast one of the objects did not get created. So release 
               // whichever got created
               if (pITimeCtl != NULL)
                  ITIMECTL_Release (pITimeCtl);
               if (pStopWatch != NULL)
                  ITIMECTL_Release (pStopWatch);
               if (pCountDown != NULL)
                  ITIMECTL_Release (pCountDown);
            }
         }
         break;
      case USAGE_GETRECT:
         {
            // 7. Usage: ITIMECTL_GetRect
            AEERect rect1 = {0};
            AEERect rect2 = {0};
            AEERect rect3 = {0};
            char szBuf[30] = {0};
            ITimeCtl * pITimeCtl = NULL;
            ITimeCtl * pStopWatch = NULL;
            ITimeCtl * pCountDown = NULL;

            // Create ITimeCtl/pStopWatch/pCountDown objects. This step needs to be 
            // done only if the ITimeCtl/pStopWatch/pCountDown objects have not been 
            // created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_CLOCKCTL, (void **)&pITimeCtl);
            ISHELL_CreateInstance(pIShell, AEECLSID_STOPWATCHCTL, (void **)&pStopWatch);
            ISHELL_CreateInstance(pIShell, AEECLSID_COUNTDOWNCTL, (void **)&pCountDown);

            if (pITimeCtl && pStopWatch && pCountDown)
            {
               // ITIMECTL_GetRect: fills given pointer to rectangle with the 
               // coordinates of the bounding rectangle of the time control object.  
               // This is particularly useful after a control is created to determine 
               // it's optimal/default size and position.
               ITIMECTL_GetRect(pITimeCtl,  &rect1);
               ITIMECTL_GetRect(pStopWatch, &rect2);
               ITIMECTL_GetRect(pCountDown, &rect3);

               // Display the coordinates of the rectangle retrieved. These 
               // coordinates will be the default screen dimensions of the device.
               // Display default dimensions of the clock object
               SPRINTF (szBuf, "Clock: [%i, %i, %u, %u]", rect1.x, rect1.y, rect1.dx, rect1.dy);
               DisplayOutput (pMe, 1, szBuf);

               // Display default dimensions of the stop watch object
               SPRINTF (szBuf, "StopWatch: [%i, %i, %u, %u]", rect2.x, rect2.y, rect2.dx, rect2.dy);
               DisplayOutput (pMe, 3, szBuf);

               // Display default dimensions of the countdown object
               SPRINTF (szBuf, "CountDown: [%i, %i, %u, %u]", rect3.x, rect3.y, rect3.dx, rect3.dy);
               DisplayOutput (pMe, 5, szBuf);

               // Release time control objects. This step needs to be done
               // only if no further use of the objects are needed.
               ITIMECTL_Release (pITimeCtl);
               ITIMECTL_Release (pStopWatch);
               ITIMECTL_Release (pCountDown);
            }
            else
            {
               // Atleast one of the objects did not get created. So release 
               // whichever got created
               if (pITimeCtl != NULL)
                  ITIMECTL_Release (pITimeCtl);
               if (pStopWatch != NULL)
                  ITIMECTL_Release (pStopWatch);
               if (pCountDown != NULL)
                  ITIMECTL_Release (pCountDown);
            }
         }
         break;
      case USAGE_SETPROPERTIES:
         {
            // 8. Usage: ITIMECTL_SetProperties
           // AEERect rct = {0};
            int32   dwMSecs = 12345678;
            uint32 dwProps = TP_AUTOREDRAW | TP_NOEDIT_AMPM;

            // Create ITimeCtl object. This step needs to be done only if
            // the ITimeCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_CLOCKCTL, (void **)&pMe->m_pITime);

            if (pMe->m_pITime)
            {
               // SetRect/SetTime used for output display purposes only. For more 
               // details see ITIMECTL_SetTime/ITIMECTL_SetRect usage. 
               ITIMECTL_SetRect(pMe->m_pITime, &rect);
               ITIMECTL_SetTime(pMe->m_pITime, dwMSecs);

               // ITIMECTL_SetProperties: sets time control specific properties
               // Property TP_AUTOREDRAW when set redraws time control object
               // on SetActive invokes and TP_NOEDIT_AMPM disalows editing the 
               // AM/PM field.
               ITIMECTL_SetProperties(pMe->m_pITime,  dwProps);

               // Set active so that the user can use the UP/DOWN/LEFT/RIGHT
               // arrow keys to change time values 
               ITIMECTL_SetActive (pMe->m_pITime, TRUE);

               // The clock object is NOT released at this point. The clock
               // object is released in the HandleEvents function under EVT_KEY. 
               // This gives the user the ability to edit time on an active 
               // clock object. The clock object is released when the user 
               // presses any key other than UP/DOWN/LEFT/RIGHT or SELECT.
            }
         }
         break;
      case USAGE_SETPROPERTIES_SW:
         {
            // 8. Usage: ITIMECTL_SetProperties
            int32   dwMSecs = 12345678;
            uint32 dwProps = TP_AUTOREDRAW | TP_NO_MSECONDS;

            // Create pStopWatch object. This step needs to be done only if
            // the pStopWatch object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_STOPWATCHCTL, (void **)&pMe->m_pStopWatch);

            if (pMe->m_pStopWatch)
            {
               // SetRect/SetTime used for output display purposes only. For more 
               // details see ITIMECTL_SetTime/ITIMECTL_SetRect usage. 
               ITIMECTL_SetRect(pMe->m_pStopWatch, &rect);
               ITIMECTL_SetTime(pMe->m_pStopWatch, dwMSecs);

               // ITIMECTL_SetProperties: sets time control specific properties
               // Property TP_AUTOREDRAW when set redraws time control object
               // on SetActive invokes, and TP_NO_MSECONDS when set, does not show 
               // the millisecond field in the stop watch object.
               ITIMECTL_SetProperties(pMe->m_pStopWatch,  dwProps);

               // Set active so that the user can use the UP/DOWN/LEFT/RIGHT
               // arrow keys to change time values 
               ITIMECTL_SetActive (pMe->m_pStopWatch, TRUE);

               // The StopWatch object is NOT released at this point. The StopWatch 
               // object is released in the HandleEvents function under EVT_KEY. 
               // This gives the user the ability to edit time on an active 
               // StopWatch object on the screen by pressing the up/down/left/right
               // keys. The StopWatch object is released when the user 
               // presses any key other than UP/DOWN/LEFT/RIGHT or SELECT.
            }
         }
         break;
      case USAGE_SETPROPERTIES_CD:
         {
            // 8. Usage: ITIMECTL_SetProperties
            int32   dwMSecs = 12345678;
            uint32 dwProps = TP_NO_SECONDS | TP_AUTOREDRAW;

            // Create ITimeCtl object. This step needs to be done only if
            // the ITimeCtl object has not been created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_COUNTDOWNCTL, (void **)&pMe->m_pCountDown);

            if (pMe->m_pCountDown)
            {
               // SetRect/SetTime used for output display purposes only. For more 
               // details see ITIMECTL_SetTime/ITIMECTL_SetRect usage. 
               ITIMECTL_SetRect(pMe->m_pCountDown, &rect);
               ITIMECTL_SetTime(pMe->m_pCountDown, dwMSecs);

               // ITIMECTL_SetProperties: sets time control specific properties
               // Property TP_AUTOREDRAW when set redraws time control object
               // on SetActive invokes, and TP_NO_SECONDS, if set, does not show 
               // the seconds in the COUNTDOWN control.
               ITIMECTL_SetProperties(pMe->m_pCountDown,  dwProps);

               // Set active so that the user can use the UP/DOWN/LEFT/RIGHT
               // arrow keys to change time values 
               ITIMECTL_SetActive (pMe->m_pCountDown, TRUE);

               // The CountDown object is NOT released at this point. The CountDown 
               // object is released in the HandleEvents function under EVT_KEY. 
               // This gives the user the ability to edit time on an active 
               // CountDown object on the screen by pressing the up/down/left/right
               // keys. The CountDown object is released when the user 
               // presses any key other than UP/DOWN/LEFT/RIGHT or SELECT.
            }
         }
         break;
      case USAGE_GETPROPERTIES:
         {
            // 8. Usage: ITIMECTL_GetProperties
            uint32 dwProps1;
            uint32 dwProps2;
            uint32 dwProps3;
            ITimeCtl * pITimeCtl = NULL;
            ITimeCtl * pStopWatch = NULL;
            ITimeCtl * pCountDown = NULL;

            // Create ITimeCtl/pStopWatch/pCountDown objects. This step needs to be 
            // done only if the ITimeCtl/pStopWatch/pCountDown objects have not been 
            // created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_CLOCKCTL, (void **)&pITimeCtl);
            ISHELL_CreateInstance(pIShell, AEECLSID_STOPWATCHCTL, (void **)&pStopWatch);
            ISHELL_CreateInstance(pIShell, AEECLSID_COUNTDOWNCTL, (void **)&pCountDown);

            if (pITimeCtl && pStopWatch && pCountDown)
            {
               // ITIMECTL_GetProperties: returns the time control specific properties 
               // or flags. The properties retrieved will be the default properties
               // at object creation.
               dwProps1 = ITIMECTL_GetProperties(pITimeCtl);
               dwProps2 = ITIMECTL_GetProperties(pStopWatch);
               dwProps3 = ITIMECTL_GetProperties(pCountDown);

               // First check the clock object properties.
               // Check for Auto redraw property
               if (dwProps1 & TP_AUTOREDRAW)
               {
                  DisplayOutput (pMe, 1, "Clk: Auto-Redraw");
               }
               else
               {
                  DisplayOutput (pMe, 1, "Clk: No Auto-Redraw");
               }

               // Check for AM/PM editing flag
               if (dwProps1 & TP_NOEDIT_AMPM)
               {
                  DisplayOutput (pMe, 2, "Clk: No Edit AM/PM");
               }
               else
               {
                  DisplayOutput (pMe, 2, "Clk: Edit AM/PM");
               }

               // Check the stop watch object properties.
               // Check for Auto redraw property
               if (dwProps2 & TP_AUTOREDRAW)
               {
                  DisplayOutput (pMe, 3, "SW: Auto-Redraw");
               }
               else
               {
                  DisplayOutput (pMe, 3, "SW: No Auto-Redraw");
               }

               // Check for contro showing milliseconds field
               if (dwProps2 & TP_NO_MSECONDS)
               {
                  DisplayOutput (pMe, 4, "SW: No MS");
               }
               else
               {
                  DisplayOutput (pMe, 4, "SW: Have MS");
               }

               // Check the count down object properties.
               // Check for Auto redraw property
               if (dwProps3 & TP_AUTOREDRAW)
               {
                  DisplayOutput (pMe, 5, "CD: Auto-Redraw");
               }
               else
               {
                  DisplayOutput (pMe, 5, "CD: No Auto-Redraw");
               }

               // Check for count down control showing the seconds.
               if (dwProps3 & TP_NO_SECONDS)
               {
                  DisplayOutput (pMe, 6, "CD: No Seconds");
               }
               else
               {
                  DisplayOutput (pMe, 6, "CD: Have Secs");
               }

               // Release time control objects. This step needs to be done
               // only if no further use of the objects are needed.
               ITIMECTL_Release (pITimeCtl);
               ITIMECTL_Release (pStopWatch);
               ITIMECTL_Release (pCountDown);
            }
            else
            {
               // Atleast one of the objects did not get created. Release 
               // whichever got created
               if (pITimeCtl != NULL)
                  ITIMECTL_Release (pITimeCtl);
               if (pStopWatch != NULL)
                  ITIMECTL_Release (pStopWatch);
               if (pCountDown != NULL)
                  ITIMECTL_Release (pCountDown);
            }
         }
         break;
      case USAGE_RESET:
         {
            // 8. Usage: ITIMECTL_Reset
            int32   dwMSecs = 12345678;
            AEERect rc = {0};   
				
				SETAEERECT (&rc, 0, lineHeight*3, pMe->m_dInfo.cxScreen, lineHeight);
            // Create ITimeCtl/pStopWatch/pCountDown objects. This step needs to be 
            // done only if the ITimeCtl/pStopWatch/pCountDown objects have not been 
            // created already. 
            if (ISHELL_CreateInstance(pIShell, AEECLSID_CLOCKCTL, (void **)&pMe->m_pITime) == SUCCESS)
				{
               // Set the rectangle coordinates since the 
               ITIMECTL_SetRect(pMe->m_pITime, &rc);					
               ITIMECTL_SetTime(pMe->m_pITime, dwMSecs);
				}
            rc.y = lineHeight*4;
            if (ISHELL_CreateInstance(pIShell, AEECLSID_STOPWATCHCTL, (void **)&pMe->m_pStopWatch) == SUCCESS)
				{
               ITIMECTL_SetRect(pMe->m_pStopWatch, &rc);
               ITIMECTL_SetTime(pMe->m_pStopWatch, dwMSecs);
				}
            rc.y = lineHeight*5;
            if (ISHELL_CreateInstance(pIShell, AEECLSID_COUNTDOWNCTL, (void **)&pMe->m_pCountDown) == SUCCESS)
				{
               ITIMECTL_SetRect(pMe->m_pCountDown, &rc);
               ITIMECTL_SetTime(pMe->m_pCountDown, dwMSecs);
				}

            if (pMe->m_pITime && pMe->m_pStopWatch && pMe->m_pCountDown)
            {
               // ITIMECTL_Reset: instructs the time control to reset (free/delete) it's
               // contents as well as to immediately leave active/focus mode.
               // 
               // The user will see the time control objects drawn on the screen at 
               // the coordinates specified above. These time objects are drawn prior 
               // to the objects are reset, so they do show up on the screen. When the
               // objects are reset, they are taken out of active focus. As a result 
               // if the user presses Up/Down/Left/Right/Select keys they will not be 
               // handled by respective control object handler (and can not change 
               // any values).
               ITIMECTL_Reset (pMe->m_pITime);
               ITIMECTL_Reset (pMe->m_pStopWatch);
               ITIMECTL_Reset (pMe->m_pCountDown);

               // The three objects are NOT released at this point. The three
               // objects are released in the HandleEvents function under EVT_KEY. 
               // This gives the user the ability to edit time on an active 
               // CountDown object. The CountDown object is released when the user 
               // presses any key other than UP/DOWN/LEFT/RIGHT or SELECT.
               // Note: In this case the objects are not active because they were
               //       reset in this function. When the user presses any key the 
               //       control objects will be released.
            }
         }
         break;
      case USAGE_SETTIME:
         {
            // 8. Usage: ITIMECTL_SetTime/ITIMECTL_SetTimeEx
            int32     dwMSecs = 123456;
            ITimeCtl * pITimeCtl = NULL;
            ITimeCtl * pStopWatch = NULL;
            ITimeCtl * pCountDown = NULL;
            boolean bIncUpdate = TRUE;

            // Create ITimeCtl/pStopWatch/pCountDown objects. This step needs to be 
            // done only if the ITimeCtl/pStopWatch/pCountDown objects have not been 
            // created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_CLOCKCTL, (void **)&pITimeCtl);
            ISHELL_CreateInstance(pIShell, AEECLSID_STOPWATCHCTL, (void **)&pStopWatch);
            ISHELL_CreateInstance(pIShell, AEECLSID_COUNTDOWNCTL, (void **)&pCountDown);

            if (pITimeCtl && pStopWatch && pCountDown)
            {
               // Set initial rectangle coordinates
               SETAEERECT (&rect, 0, lineHeight, pMe->m_dInfo.cxScreen/2, lineHeight);

               // Each of the SetTime function invokes (subsequent to the SetRect 
               // functions) draws the time objects on the screen. Here, three different
               // rectangles are set up and the SetRect function is called so that 
               // each object will be drawn at the top half of the screen one after
               // another.
               ITIMECTL_SetRect(pITimeCtl, &rect);
               rect.y = lineHeight*2;
               ITIMECTL_SetRect(pStopWatch, &rect);
               rect.y = lineHeight*3;
               ITIMECTL_SetRect(pCountDown, &rect);

               // ITIMECTL_SetTime: can be used to set given time and redraw time string 
               // on phone screen.  It is the same as calling ITIMECTL_SetTimeEx with
               // the third parameter to be FALSE.
               ITIMECTL_SetTime(pITimeCtl, dwMSecs);
               ITIMECTL_SetTime(pStopWatch, dwMSecs);
               ITIMECTL_SetTime(pCountDown, dwMSecs);

               // Change the root y coordinate of each control so that it is printed
               // of subsequent lines to the above displayed lines (displayed due to 
               // setting time).
               rect.y = lineHeight*4;
               ITIMECTL_SetRect(pITimeCtl, &rect);
               rect.y = lineHeight*5;
               ITIMECTL_SetRect(pStopWatch, &rect);
               rect.y = lineHeight*6;
               ITIMECTL_SetRect(pCountDown, &rect);

               // ITIMECTL_SetTimeEx: can be used to set given time and redraw time 
               // string on phone screen. 
               ITIMECTL_SetTimeEx(pITimeCtl, dwMSecs, bIncUpdate);
               ITIMECTL_SetTimeEx(pStopWatch, dwMSecs, bIncUpdate);
               ITIMECTL_SetTimeEx(pCountDown, dwMSecs, bIncUpdate);

               // Release time control objects. This step needs to be done
               // only if no further use of the objects are needed.
               ITIMECTL_Release (pITimeCtl);
               ITIMECTL_Release (pStopWatch);
               ITIMECTL_Release (pCountDown);
            }
            else
            {
               // Atleast one of the objects did not get created. So release 
               // whichever got created
               if (pITimeCtl != NULL)
                  ITIMECTL_Release (pITimeCtl);
               if (pStopWatch != NULL)
                  ITIMECTL_Release (pStopWatch);
               if (pCountDown != NULL)
                  ITIMECTL_Release (pCountDown);
            }
         }
         break;
      case USAGE_GETTIME:
         {
            // 8. Usage: ITIMECTL_GetTime
            int32      nRetValue = 0;
            char szBuf [30] = {0};
            int32     dwMSecs = 123456;
            ITimeCtl * pITimeCtl = NULL;
            ITimeCtl * pStopWatch = NULL;
            ITimeCtl * pCountDown = NULL;

            // Create ITimeCtl/pStopWatch/pCountDown objects. This step needs to be 
            // done only if the ITimeCtl/pStopWatch/pCountDown objects have not been 
            // created already. 
            ISHELL_CreateInstance(pIShell, AEECLSID_CLOCKCTL, (void **)&pITimeCtl);
            ISHELL_CreateInstance(pIShell, AEECLSID_STOPWATCHCTL, (void **)&pStopWatch);
            ISHELL_CreateInstance(pIShell, AEECLSID_COUNTDOWNCTL, (void **)&pCountDown);

            if (pITimeCtl && pStopWatch && pCountDown)
            {
               // Set initial rectangle coordinates
               SETAEERECT (&rect, 0, lineHeight, pMe->m_dInfo.cxScreen/2, lineHeight);

               // Each of the SetTime function invokes (subsequent to the SetRect 
               // functions) draws the time objects on the screen. Here, three different
               // rectangles are set up and the SetRect function is called so that 
               // each object will be drawn at the top half of the screen one after
               // another.
               ITIMECTL_SetRect(pITimeCtl, &rect);
               rect.y = lineHeight*2;
               ITIMECTL_SetRect(pStopWatch, &rect);
               rect.y = lineHeight*3;
               ITIMECTL_SetRect(pCountDown, &rect);

               // ITIMECTL_SetTime: can be used to set given time and redraw time string 
               // on phone screen.  It is the same as calling ITIMECTL_SetTimeEx with
               // the third parameter to be FALSE.
               ITIMECTL_SetTime(pITimeCtl, dwMSecs);
               ITIMECTL_SetTime(pStopWatch, dwMSecs);
               ITIMECTL_SetTime(pCountDown, dwMSecs);

               // ITIMECTL_GetTime: gets the time value from the time control object. 
               // The time value is in milli seconds.
               // Get time from clock object
               nRetValue = ITIMECTL_GetTime(pITimeCtl);
               SPRINTF (szBuf, "CLK (ms): %i", nRetValue);
               DisplayOutput (pMe, 4, szBuf);

               // Get time from stop watch object
               nRetValue = ITIMECTL_GetTime(pStopWatch);
               SPRINTF (szBuf, "SW (ms): %i", nRetValue);
               DisplayOutput (pMe, 5, szBuf);

               // Get time from count down object
               nRetValue = ITIMECTL_GetTime(pCountDown);
               SPRINTF (szBuf, "CD (ms): %i", nRetValue);
               DisplayOutput (pMe, 6, szBuf);

               // Release time control objects. This step needs to be done
               // only if no further use of the objects are needed.
               ITIMECTL_Release (pITimeCtl);
               ITIMECTL_Release (pStopWatch);
               ITIMECTL_Release (pCountDown);
            }
            else
            {
               // Atleast one of the objects did not get created. So release 
               // whichever got created
               if (pITimeCtl != NULL)
                  ITIMECTL_Release (pITimeCtl);
               if (pStopWatch != NULL)
                  ITIMECTL_Release (pStopWatch);
               if (pCountDown != NULL)
                  ITIMECTL_Release (pCountDown);
            }
         }
         break;
      case USAGE_ENABLECOMMAND:
         {
            // 8. Usage: ITIMECTL_EnableCommand
            //
            // Enable command for all the modes (clock, stop-watch, count-down)
            // has the same behavior. Hence, there will only be one mode used to 
            // illustrate the EnableCommand.
            int32   dwMSecs = 12345678;
            AEERect rc = {0};
            boolean bEnable = TRUE;
            uint16 nCmdId = EVT_APP_START;

            // Create ITimeCtl object.
            ISHELL_CreateInstance(pIShell, AEECLSID_CLOCKCTL, (void **)&pMe->m_pITime);
			rc.y = lineHeight;

            if (pMe->m_pITime)
            {
               // Set rect
               SETAEERECT (&rect, 0, lineHeight, pMe->m_dInfo.cxScreen/2, lineHeight*2);

               // Set the rectangle coordinates
               ITIMECTL_SetRect(pMe->m_pITime, &rc);

               // Use SetTime function to assign a time value to the TimeCtl object.
               // This method will draw the time on the screen in the rectangle 
               // coordinates given above.
               ITIMECTL_SetTime(pMe->m_pITime, dwMSecs);

               // ITIMECTL_EnableCommand: is used to enable sending of specified 
               // command by the time control object to the shell object upon 
               // receiving the event generated by pressing center key.
               ITIMECTL_EnableCommand (pMe->m_pITime, bEnable, nCmdId);
               // The clock object is NOT released at this point. The clock 
               // object is released in the HandleEvents function under EVT_KEY. 
               // This gives the user the ability to edit time on an active 
               // clock object on the screen by pressing the up/down/left/right
               // keys. The clock object is released when the user presses any key 
               // other than UP/DOWN/LEFT/RIGHT or SELECT.
            }
         }
         break;
      case USAGE_GETTIMESTRING:
         {
            // 8. Usage: ITIMECTL_GetTimeString
            char   szBuf [30] = {0};
            AECHAR pszBuf [30] = {0};
            int32  dwMSecs = 123456;
            uint16 wFlags; 
            ITimeCtl * pITimeCtl = NULL;

            // Create ITimeCtl object.
            ISHELL_CreateInstance(pIShell, AEECLSID_CLOCKCTL, (void **)&pITimeCtl);

            if (pITimeCtl)
            {
               // ITIMECTL_GetTimeString: converts the time in milliseconds to a string 
               // in the specified format.
               //
               // The time strings returned from the ITIMECTL_GetTimeString function
               // has the same format for all the clock, stopwatch and count down modes.
               // Hence only, the clock mode is used to demonstrate this finction.
               //
               // Use milliseconds
               wFlags = GTS_MSECS;
               ITIMECTL_GetTimeString (pITimeCtl, dwMSecs, pszBuf, sizeof (pszBuf), wFlags);
               WSTR_TO_STR (pszBuf, szBuf, sizeof (pszBuf));
               DisplayOutput (pMe, 1, szBuf);

               // Use seconds
               wFlags = GTS_SECS;
               ITIMECTL_GetTimeString (pITimeCtl, dwMSecs, pszBuf, sizeof (pszBuf), wFlags);
               WSTR_TO_STR (pszBuf, szBuf, sizeof (pszBuf));
               DisplayOutput (pMe, 2, szBuf);

               // Use AM/PM
               wFlags = GTS_AMPM;
               ITIMECTL_GetTimeString (pITimeCtl, dwMSecs, pszBuf, sizeof (pszBuf), wFlags); 
               WSTR_TO_STR (pszBuf, szBuf, sizeof (pszBuf));
               DisplayOutput (pMe, 3, szBuf);

               // Release time control object. 
               ITIMECTL_Release (pITimeCtl);
            }
         }
         break;
      case USAGE_SETINCREMENT:
         {
            // 8. Usage: ITIMECTL_SetIncrement
            uint16 wMins = 5; 
            ITimeCtl * pITimeCtl = NULL;
            ITimeCtl * pStopWatch = NULL;
            ITimeCtl * pCountDown = NULL;

            // Create ITimeCtl/pStopWatch/pCountDown objects. This step needs to be 
            // done only if the ITimeCtl/pStopWatch/pCountDown objects have not been 
            // created already.
            ISHELL_CreateInstance(pIShell, AEECLSID_CLOCKCTL, (void **)&pITimeCtl);
            ISHELL_CreateInstance(pIShell, AEECLSID_STOPWATCHCTL, (void **)&pStopWatch);
            ISHELL_CreateInstance(pIShell, AEECLSID_COUNTDOWNCTL, (void **)&pCountDown);

            if (pITimeCtl && pStopWatch && pCountDown)
            {
               // ITIMECTL_SetIncrement: sets a new time increment value in minutes. 
               // The default is 1.
               // Here increment to 5 minutes.
               ITIMECTL_SetIncrement (pITimeCtl, wMins);
               ITIMECTL_SetIncrement (pStopWatch, wMins);
               ITIMECTL_SetIncrement (pCountDown, wMins);

               // Release time control objects. This step needs to be done
               // only if no further use of the objects are needed.
               ITIMECTL_Release (pITimeCtl);
               ITIMECTL_Release (pStopWatch);
               ITIMECTL_Release (pCountDown);
            }
            else
            {
               // Atleast one of the objects did not get created. So release 
               // whichever got created
               if (pITimeCtl != NULL)
                  ITIMECTL_Release (pITimeCtl);
               if (pStopWatch != NULL)
                  ITIMECTL_Release (pStopWatch);
               if (pCountDown != NULL)
                  ITIMECTL_Release (pCountDown);
            }
         }
         break;
      case USAGE_SETEDITFIELD:
         {
            // 8. Usage: ITIMECTL_SetEditField
            int32     dwMSecs = 123456;
            ITField field = ITF_HOUR;
            AEERect rc;

            // Create ITimeCtl object.
            ISHELL_CreateInstance(pIShell, AEECLSID_CLOCKCTL, (void **)&pMe->m_pITime);

            if (pMe->m_pITime)
            {
               SETAEERECT (&rect, 0, lineHeight*2, pMe->m_dInfo.cxScreen/2, lineHeight*2);
               // Set the rectangle coordinates
               ITIMECTL_SetRect(pMe->m_pITime, &rc);

               // Use SetTime function to assign a time value to the TimeCtl object.
               // This method will draw the time on the screen in the rectangle 
               // coordinates given above.
               ITIMECTL_SetTime(pMe->m_pITime, dwMSecs);

               // ITIMECTL_SetEditField: sets specified field for editing.
               ITIMECTL_SetEditField (pMe->m_pITime, field);

               // When active the above set field is the heighlighted field.
               ITIMECTL_SetActive (pMe->m_pITime, TRUE);

               // The clock object is NOT released at this point. The clock
               // object is released in the HandleEvents function under EVT_KEY. 
               // This gives the user the ability to edit time on an active 
               // clock object on the screen by pressing the up/down/left/right
               // keys. The clock object is released when the user 
               // presses any key other than UP/DOWN/LEFT/RIGHT or SELECT.
            }
         }
         break;
      case USAGE_SETEDITFIELD_SW:
         {
            // 8. Usage: ITIMECTL_SetEditField
            int32     dwMSecs = 123456;
            ITField field = ITF_MIN;
            AEERect rc;

            // Create ITimeCtl object.
            ISHELL_CreateInstance(pIShell, AEECLSID_STOPWATCHCTL, (void **)&pMe->m_pStopWatch);

            if (pMe->m_pStopWatch)
            {
               // Set rect
               SETAEERECT (&rect, 0, lineHeight*2, pMe->m_dInfo.cxScreen/2, lineHeight*2);

               // Set the rectangle coordinates
               ITIMECTL_SetRect(pMe->m_pStopWatch, &rc);

               // Use SetTime function to assign a time value to the TimeCtl object.
               // This method will draw the time on the screen in the rectangle 
               // coordinates given above.
               ITIMECTL_SetTime(pMe->m_pStopWatch, dwMSecs);

               // ITIMECTL_SetEditField: sets specified field for editing.
               ITIMECTL_SetEditField (pMe->m_pStopWatch, field);

               // When active the above set field is the heighlighted field.
               ITIMECTL_SetActive (pMe->m_pStopWatch, TRUE);

               // The stop watch object is NOT released at this point. The stop watch 
               // object is released in the HandleEvents function under EVT_KEY. 
               // This gives the user the ability to edit time on an active 
               // stop watch object on the screen by pressing the up/down/left/right
               // keys. The stop watch object is released when the user 
               // presses any key other than UP/DOWN/LEFT/RIGHT or SELECT.
            }
         }
         break;
      case USAGE_SETEDITFIELD_CD:
         {
            // 8. Usage: ITIMECTL_SetEditField
            int32     dwMSecs = 123456;
            ITField field = ITF_SEC;
            AEERect rc;

            // Create ITimeCtl object.
            ISHELL_CreateInstance(pIShell, AEECLSID_COUNTDOWNCTL, (void **)&pMe->m_pCountDown);

            if (pMe->m_pCountDown)
            {
               SETAEERECT (&rect, 0, lineHeight*2, pMe->m_dInfo.cxScreen/2, lineHeight*2);

               // Set the rectangle coordinates
               ITIMECTL_SetRect(pMe->m_pCountDown, &rc);

               // Use SetTime function to assign a time value to the TimeCtl object.
               // This method will draw the time on the screen in the rectangle 
               // coordinates given above.
               ITIMECTL_SetTime(pMe->m_pCountDown, dwMSecs);

               // ITIMECTL_SetEditField: sets specified field for editing.
               ITIMECTL_SetEditField (pMe->m_pCountDown, field);

               // When active the above set field is the heighlighted field.
               ITIMECTL_SetActive (pMe->m_pCountDown, TRUE);

               // The count down object is NOT released at this point. The count down 
               // object is released in the HandleEvents function under EVT_KEY. 
               // This gives the user the ability to edit time on an active 
               // count down object on the screen by pressing the up/down/left/right
               // keys. The count down object is released when the user 
               // presses any key other than UP/DOWN/LEFT/RIGHT or SELECT.
            }
         }
         break;
      default:
         return;
   }
   
   // Display above event. 
   DisplayEvent (pMe, wParam);

   return;
}

/*===========================================================================

FUNCTION: DisplayEvent

DESCRIPTION
    This function is a display function which sets up the initial screen
    for the display output for a given interface function. This displays
    the function name at the top of the screen and displays the
    strings "Press Up/Down" and "keys to continue.." at the bottom of 
    the screen. In the case of functions that have no output it also 
    displays the string "No Output..." in the middile of the screen.

PROTOTYPE:
   void DisplayEvent (CITimeCtlApp *pMe, uint16 wParam)

PARAMETERS:
   pMe:    [in]: Pointer to CITimeCtlApp interface object
   wParam: [in]: API function type Id selected by user

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void DisplayEvent (CITimeCtlApp *pMe, uint16 wParam)
{
   AECHAR szBuf[100];
   AEERect rc; 

   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
      return;
   
   // Set display rectangle
   SETAEERECT (&rc, 0, 0, pMe->m_dInfo.cxScreen, pMe->m_dInfo.cyScreen);

   if (pMe->a.m_pIDisplay)
   {
      switch (wParam)
      {
         case USAGE_BASIC_USAGE:
            STR_TO_WSTR("Basic Usage:", szBuf, sizeof(szBuf));
            break;
         case USAGE_HANDLEEVENT:
            STR_TO_WSTR("HandleEvent(clk)", szBuf, sizeof(szBuf));
            break;
         case USAGE_HANDLEEVENT_SW:
            STR_TO_WSTR("HandleEvent(sw)", szBuf, sizeof(szBuf));
            break;
         case USAGE_HANDLEEVENT_CD:
            STR_TO_WSTR("HandleEvent(cd)", szBuf, sizeof(szBuf));
            break;
         case USAGE_REDRAW:
            STR_TO_WSTR("Redraw(clk)", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETACTIVE:
            STR_TO_WSTR("SetActive(clk)", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETACTIVE_SW:
            STR_TO_WSTR("SetActive(sw)", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETACTIVE_CD:
            STR_TO_WSTR("SetActive(cd)", szBuf, sizeof(szBuf));
            break;
         case USAGE_ISACTIVE:
            STR_TO_WSTR("IsActive", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETRECT:
            STR_TO_WSTR("SetRect", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETRECT:
            STR_TO_WSTR("GetRect", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETPROPERTIES:
            STR_TO_WSTR("SetProperties(clk)", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETPROPERTIES_SW:
            STR_TO_WSTR("SetProperties(sw)", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETPROPERTIES_CD:
            STR_TO_WSTR("SetProperties(cd)", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETPROPERTIES:
            STR_TO_WSTR("GetProperties", szBuf, sizeof(szBuf));
            break;
         case USAGE_RESET:
            STR_TO_WSTR("Reset", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETTIME:
            STR_TO_WSTR("SetTime", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETTIME:
            STR_TO_WSTR("GetTime", szBuf, sizeof(szBuf));
            break;
         case USAGE_ENABLECOMMAND:
            STR_TO_WSTR("EnableCommand", szBuf, sizeof(szBuf));
            break;
         case USAGE_GETTIMESTRING:
            STR_TO_WSTR("GetTimeString", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETINCREMENT:
            STR_TO_WSTR("SetIncrement", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETEDITFIELD:
            STR_TO_WSTR("SetEditField(clk)", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETEDITFIELD_SW:
            STR_TO_WSTR("SetEditField(sw)", szBuf, sizeof(szBuf));
            break;
         case USAGE_SETEDITFIELD_CD:
            STR_TO_WSTR("SetEditField(cd)", szBuf, sizeof(szBuf));
            break;
         default:
            STR_TO_WSTR("Unsupported fn", szBuf, sizeof(szBuf));
      }
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,5, 0, &rc,
		   IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);

      // Functions with no output
      switch (wParam)
      {
         case USAGE_SETINCREMENT:
            // Place in the middle of screen
            STR_TO_WSTR("No Output...", szBuf, sizeof(szBuf));
 		      IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,10, 
               rc.dy/2 - pMe->m_nLineHeight, &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
         default:
            break;
      }

      STR_TO_WSTR("Press CLR to continue..", szBuf, sizeof(szBuf));
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,5, rc.dy - 
         pMe->m_nLineHeight, &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
		IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);
   }
}

/*===========================================================================

FUNCTION: BuildMainMenu

DESCRIPTION
    This function builds the main menu when this applet is started.
    It displays a simple menu with menu items corresponding to 
    each of the interface functions.
    
PROTOTYPE:
   void BuildMainMenu(CITimeCtlApp *pMe)

PARAMETERS:
   pMe: [in]: Pointer to AEEApplet struct

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void BuildMainMenu(CITimeCtlApp *pMe)
{
	AEERect rc;
   AECHAR szBuf[100];

   if (pMe == NULL || pMe->m_pIMenu == NULL || pMe->a.m_pIShell == NULL)
      return;

   // Set display rectangle
   SETAEERECT (&rc, 0, 0, pMe->m_dInfo.cxScreen, pMe->m_dInfo.cyScreen);

   // Set Title
   STR_TO_WSTR("ITimeCtl Examples", szBuf, sizeof(szBuf));
	IMENUCTL_SetTitle(pMe->m_pIMenu, NULL, 0, szBuf);

	//Set the Rectangle for the Menu
	IMENUCTL_SetRect(pMe->m_pIMenu, &rc);  

	//Add invidual entries to the Menu

   // Add Basic Usage to menu
   STR_TO_WSTR("1. Basic Usage", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_BASIC_USAGE, szBuf, 0);

   // Add Handle Event to menu
   STR_TO_WSTR("2. HandleEvent(clk)", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_HANDLEEVENT, szBuf, 0);

   // Add Handle Event to menu
   STR_TO_WSTR("3. HandleEvent(sw)", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_HANDLEEVENT_SW, szBuf, 0);

      // Add Handle Event to menu
   STR_TO_WSTR("4. HandleEvent(cd)", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_HANDLEEVENT_CD, szBuf, 0);

   // Add Redraw to menu
   STR_TO_WSTR("5. Redraw", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_REDRAW, szBuf, 0);

   // Add SetActive to menu
   STR_TO_WSTR("6. SetActive-clk", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETACTIVE, szBuf, 0);

   // Add SetActive to menu
   STR_TO_WSTR("7 SetActive-sw", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETACTIVE_SW, szBuf, 0);

   // Add SetActive to menu
   STR_TO_WSTR("8. SetActive-cd", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETACTIVE_CD, szBuf, 0);

   // Add IsActive to menu
   STR_TO_WSTR("9. IsActive", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_ISACTIVE, szBuf, 0);

	// Add SetRect to menu
   STR_TO_WSTR("10. SetRect", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETRECT, szBuf, 0);

	// Add GetRect to menu
   STR_TO_WSTR("11. GetRect", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETRECT, szBuf, 0);	

	// Add Set Properties to menu
   STR_TO_WSTR("12. SetProperties(clk)", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPROPERTIES, szBuf, 0);	

	// Add Set Properties to menu
   STR_TO_WSTR("13. SetProperties(sw)", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPROPERTIES_SW, szBuf, 0);	

	// Add Set Properties to menu
   STR_TO_WSTR("14. SetProperties(cd)", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETPROPERTIES_CD, szBuf, 0);	

	// Add GetProperties to menu
   STR_TO_WSTR("15. GetProperties", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETPROPERTIES, szBuf, 0);	

   // Add Reset to menu
   STR_TO_WSTR("16. Reset", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_RESET, szBuf, 0);	

   // Add SetTime to menu
   STR_TO_WSTR("17. SetTime", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETTIME, szBuf, 0);	

   // Add GetTime to menu
   STR_TO_WSTR("18. GetTime", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETTIME, szBuf, 0);	

   // Add EnableCommand to menu
   STR_TO_WSTR("19. EnableCommand", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_ENABLECOMMAND, szBuf, 0);	

	// Add SetTimeString to menu
   STR_TO_WSTR("20. GetTimeStr", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETTIMESTRING, szBuf, 0);	

	// Add SetIncrement to menu
   STR_TO_WSTR("21. SetIncrement", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETINCREMENT, szBuf, 0);	

	// Add SetEditField to menu
   STR_TO_WSTR("22. SetEditField(clk)", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETEDITFIELD, szBuf, 0);	

	// Add SetEditField to menu
   STR_TO_WSTR("23. SetEditField(sw)", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETEDITFIELD_SW, szBuf, 0);	

	// Add SetEditField to menu
   STR_TO_WSTR("24. SetEditField(cd)", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETEDITFIELD_CD, szBuf, 0);	

   // Activate menu
	IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);
}

/*===========================================================================

FUNCTION: DisplayOutput

DESCRIPTION
    This function displays an output string at a given line number on the
    screen. If the nline parameter is a negative value (-1) the string
    is displayed in the middle of the screen. If the "nline" value is larger
    than or equal to zero the "nline" value is multiplied by 15 and the 
    resulting value in pixels is set to the y-coordinate of the start of 
    the string display on the screen. If the string does not fit on one line
    the string wraps around to the nime line (spaced rougly 10-15 pixels apart).
    By default 5 is used as the starting the x-coordinate of a displayed 
    string.

    How many characters that fit on one line is calculated for each line 
    that is wrapped around to the nime line.

    Note: depending on the phone screen size and the fonts used for characters 
          the output might differ on different handsets (devices). Where some 
          handsets will have a smaller screen and large default fonts which will 
          cause partial overlapping of lines. This function does not try to address
          these issues (this is meant as a simple display function).
    
PROTOTYPE:
   void DisplayOutput(CITimeCtlApp * pi, int nline, char *pszStr)

PARAMETERS:
   pi:     [in]: Contains a pointer to the CITimeCtlApp class.
   nline:  [in]: Contains the line number to start displaying the time. 
        The line numbers are by default spaced 15 pixels apart along the y-axis.
   pszStr: [in]: The character string to be displayed on the screen.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void DisplayOutput(CITimeCtlApp * pMe, int nline, char *pszStr)
{
   AECHAR * pszBuf = NULL;
   AECHAR * psz = NULL;
   int pixelWidth;
   AEEFont font = AEE_FONT_NORMAL;
   int pnFits = 0, dy;
   int totalCh = 0;

   if (pMe == NULL)
      return;

   // Allocate buffer to hold string.
   if ((pszBuf = (AECHAR *)MALLOC(200)) == NULL)
      return;

   // Convert to wide string (unicode)
   STR_TO_WSTR ((char *)pszStr, pszBuf, 200);

   // If nlines is zero then print this string starting around the middle of 
   // the screen. Or else multiply nlines by 10 to decide the y coordinate of
   // the start of the string.
   if (nline < 0) {
      dy = pMe->m_dInfo.cyScreen*2/5;
   }
   else{
      dy = nline * pMe->m_nLineHeight;
   }

   // psz keeps track of the point from which to write from the string buffer
   // in case the string does not fit one line and needs to wrap around in the
   // next line.
   psz = pszBuf;
      
   // Need to calculate the lotal string length to decide if any wrapping
   // around is needed.
   totalCh = STRLEN ((char *)pszStr);

   // Keep displaying text string on multiple lines if the string can't be displayed
   // on one single line. Lines are spaced 15 pixels apart.
   while ((totalCh > 0) && (*psz != NULL))
   { 
      // Get information on how many characters will fit in a line.
      // Give the pointer to the buffer to be displayed, and the number of
      // pixels along the x axis you want to display the string in (max number)
      // pnFits will have the max number of chars that will fit in the maxWidth
      // number of pixels (given string can't fit in one line), or the number of 
      // chars in the string (if it does fit in one line). pnWidth gives the
      // number of pixels that will be used to display pnFits number of chars.
      pixelWidth = IDISPLAY_MeasureTextEx(pMe->a.m_pIDisplay,
                      font, 
                      (AECHAR *) psz,  // Start of the buffer to display,
                      -1,
                      pMe->m_dInfo.cxScreen - 5, // maxWidth
                      &pnFits);         // Number of chars that will fit a line

      // If pnFits is zero there is something wrong in the input to above function. 
      // Normally this scenario should not occur. But, have the check anyway.
      if (pnFits == 0)
      {
         FREE (pszBuf);
         return;
      }

      IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, psz, pnFits, 5 /*start dx*/, 
         dy, 0 /* use default rectangle coordinates */, 0);

      psz += pnFits;      // move pointer to the next segment to be displayed
      totalCh -= pnFits;  // reduce the total number of characters to still display
      dy += pMe->m_nLineHeight; // Place next line charHeight pixels below the 
                                // previous line. 
      IDISPLAY_Update(pMe->a.m_pIDisplay);
      if (totalCh < pnFits)
         pnFits = totalCh;  // if total number is less than pnFits, adjust pnFits
   }

   FREE (pszBuf);
   return;   
} // End of DisplayOutput

