/*===========================================================================

FILE: Resapp.c
  
SERVICES: Sample applet using AEE
    
DESCRIPTION
   This file contains the implementation of a simple example using the
   AEE services.

   This is to demonstrate using resources in an application. This application
   loads two strings from a resource file and prints the two strings on the
   top half of the screen. Using a color filled thin horizontal rectangle the 
   screen is devided in to two halfs. A bitmap of a cursor is drawn in the middle
   of the bottom half of the screen. This cursor bitmap can be moved in the 
   bottom half of the screen using Up/Down/Left/Right arrow keys.
      
PUBLIC CLASSES:  
   N/A
        
INITIALIZATION AND SEQUENCING REQUIREMENTS:
   The following explanation applies to this sample containing one applet which serves
   as a base for app developers to create their own applets using AEE Services:
   

   In the applet source file (like this one), include AEEAppGen.h.

   Mandatory Sections in Applet Source (this file):
   -----------------------------------------------
   Following Mandatory Sections are required for each applet source file. 
   (Search for "Mandatory" to identify these sections)

   Includes:     
      Copy this section as-is from the sample applet. It contains:
      AEEAppGen.h: For AEEApplet declaration

   Type Declarations:
		A data structure must be defined to hold the app specific data. In this structure,
		the first element must be of type AEEApplet.

   Functions: (For more details, see corresponding function description in this applet)

      App_HandleEvent(): is the Event Handler to this applet.
                         Copy the function outline from the sample applet and add app specific code.

      AEEClsCreateInstance(): creates an instance of the applet and initializes it. 
                              It is called by AEEModGen when applet is being created.

   
   Important Notes:
   ---------------
   1. DO NOT use any "static data" in the applet. Always use the functions exported by
	   AEEStdlib or by IHeap services to dynamically allocate data and make it a member of 
		the applet structure. 
   2. DO NOT include and link "standard C library". Use AEE Memory Services (in AEEHeap.h) and Standard Library macros(in AEEStdLib.h).
      For example, use MALLOC() to allocate memory, WSTRCPY() to make a copy of Unicode (wide) string.
   3. BREW is Unicode(wide string) compliant ONLY (no ISOLATIN1/ANSI) except for file names which are ISOLATIN1/ANSI.
      ALWAYS USE AECHAR instead of "char". Use string services provided in AEEStdLib.h for string manipulation.
   4. It is always a good idea to DEFINE RESOURCES using BREW ResourceEditor. Make Strings, Bitmaps, Dialogs, etc. 
      as resources. ResourceEditor saves resources as .bri file, generates resource header file 
      and compiles .bri into a .bar binary file, which can be used by the applet.

   Miscellanoeus Notes:
   -------------------
   1. Make sure that the class ID used for the app is the same as that defined corresponding in .MIF file
   2. Always make sure that compiled resource (.bar) file and corresponding
      resource header (a) reside in app directory and (b) are included in the applet code.
      Define a constant APP_RES_FILE containing the name of the compiled resource file (with .bar extension).

   More than one applet:
   --------------------
   If more than one applet needs to be defined, then do the following
   (1) Follow the above description for each applet




   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/
#include "AEEModGen.h"            // Module interface definitions
#include "AEEAppGen.h"            // Applet interface definitions
#include "AEEShell.h"             // Shell interface definitions
#include "AEEDisp.h"              // Display interface definitions
#include "AEEStdlib.h"            // Standard library definitions
#include "AEEImage.h"             // IImage interface definitions
#include "resapp_res.h"           // Resource ID definitions
#include "resapp.bid"
/*===========================================================================

  PUBLIC DATA DECLARATIONS
  
===========================================================================*/


/*-------------------------------------------------------------------
Type Declarations
-------------------------------------------------------------------*/

/*-------------------------------------------------------------------
      Type Declarations
-------------------------------------------------------------------*/
// Structure to contain app data. This is the Applet structure that will be allocated
// for the app. Define this structure if the app needs to store any data that needs
// to be remembered across different events or callbacks. The first element of this
// structure MUST be of type AEEApplet. This is extremely important. Following this
// element, store the app-specific data. Note that AEEApplet structure already exposes
// the most commonly needed interface pointers such as Shell, Display, etc. 
typedef struct _CResApp
{
   AEEApplet	a;    // Mandatory AEEApplet data member appears first
   int m_cxWidth;    // Stores the device screen width
   int m_cyHeight;   // Stores the device screen height
   int m_nCursorX;   // Stores the cursor bitmap x coordinate
   int m_nCursorY;   // Stores the cursor bitmap y coordinate
   IImage * m_pIImage;  // IImage interface pointer
}CResApp;


/*-------------------------------------------------------------------
Function Prototypes
-------------------------------------------------------------------*/
static boolean ResApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);
static boolean ResApp_InitAppData(IApplet* pMe);
static void ResApp_FreeAppData(IApplet* pMe);
static void CresApp_Move(CResApp * pMe, int xc, int yc);

/*-------------------------------------------------------------------
Global Constant Definitions
-------------------------------------------------------------------*/


/*-------------------------------------------------------------------
Global Data Definitions
-------------------------------------------------------------------*/
#define APP_RES_FILE  "resapp.bar"   // Resource file name


/*-------------------------------------------------------------------
            Static variable Definitions
-------------------------------------------------------------------*/


/*===============================================================================
FUNCTION DEFINITIONS
=============================================================================== */




/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time.

PROTOTYPE:
	int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell interface. 

	pIModule: pin]: Contains pointer to the IModule interface to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppApplet must point to a valid IBase object. 
	If the classID	represnts an applet, then ppObj must point to a valid AEEApplet structure.Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the class/app creation was successful. 
  EFAILED: Error occurred while creating the class/app. In this case, the app/class will
  not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
{
   *ppObj = NULL;
		
   if(ClsId == AEECLSID_RESAPP_APP){
      if(AEEApplet_New(sizeof(CResApp), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)ResApp_HandleEvent,(PFNFREEAPPDATA)ResApp_FreeAppData)
         == TRUE)
      {
         if (ResApp_InitAppData((IApplet*)*ppObj) == TRUE)
         {
            return(AEE_SUCCESS);
         }
      }
   }
	return (EFAILED);
}

/*===========================================================================

Function:  ResApp_InitAppData()

Description:
		This function initializes the app specific data.

Prototype:
	void ResApp_InitAppData(IApplet* po)

Parameters:
   IApplet* po: Pointer to the applet structure. This is of the type defined
	to store applet data. (CResApp*, in this case).

Return Value:
	TRUE: If successful
	FALSE: IF failed

Comments:  None

Side Effects: None

==============================================================================*/
static boolean ResApp_InitAppData(IApplet* pi)
{
   AEEDeviceInfo di;

   CResApp * pMe = (CResApp*)pi;

   pMe->m_pIImage = NULL;

   if (pMe->a.m_pIShell) {
      ISHELL_GetDeviceInfo(pMe->a.m_pIShell, &di);    // Get device info
        pMe->m_cxWidth = di.cxScreen;
        pMe->m_cyHeight = di.cyScreen;
   }

   pMe->m_nCursorX = pMe->m_cxWidth/2;
   pMe->m_nCursorY = pMe->m_cyHeight*2/3;

   // Load image resource and store in applet struct data member
   if ((pMe->m_pIImage = ISHELL_LoadResImage(pMe->a.m_pIShell, APP_RES_FILE, 
      IDB_CURSOR)) == NULL)
   {
      return FALSE;
   }

   return TRUE;
}

/*===========================================================================

Function:  ResApp_FreeAppData()

Description:
	This function frees the app data. This function is registered with the
	applet framework when the applet is created (inside AEEClsCreateInstance() function).
	This function is called by the app framework when the reference count of the 
	applet reaches zero. This function must free all the app data that has been
	allocated by this app. For ex: if their is data that was remembered when
	the app is suspended and resumed, those data must be freed here.

Prototype:
	void ResApp_FreeAppData(IApplet* po)

Parameters:
   IApplet* po: Pointer to the applet structure. This is of the type defined
	to store applet data. (CResApp*, in this case).

Return Value:
	None

Comments:  None

Side Effects: None

==============================================================================*/
static void ResApp_FreeAppData(IApplet* pi)
{
   CResApp * pMe = (CResApp*)pi;

   if (pMe->m_pIImage != NULL)
   {
      IIMAGE_Release (pMe->m_pIImage);
      pMe->m_pIImage = NULL;
   }
}

/*===========================================================================

FUNCTION ResApp_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean ResApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AppCreateInstance() function.

	ecode: Specifies the Event sent to this applet

   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/
static boolean ResApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
   AEERect rc;
   AECHAR szBuf[30] = {0};

   CResApp * pMe = (CResApp*)pi;

   switch (eCode) 
	{
      case EVT_APP_START:    // Applet start event
   	  case EVT_APP_RESUME:
      IDISPLAY_ClearScreen (pMe->a.m_pIDisplay); // Erase whole screen
         // Load string "Use arrow keys" from resource file.
         ISHELL_LoadResString(pMe->a.m_pIShell, APP_RES_FILE, IDS_STRING1, 
                                 szBuf, sizeof (szBuf));
         IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1, pMe->m_cxWidth/5, 
            pMe->m_cyHeight/8, 0, 0);

         // Load string "to move cursor" from resource file.
         ISHELL_LoadResString(pMe->a.m_pIShell, APP_RES_FILE, IDS_STRING2, 
                                 szBuf, sizeof (szBuf));
         IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1, pMe->m_cxWidth/5, 
            pMe->m_cyHeight/5, 0, 0);

         // Initialize rectangle
         SETAEERECT (&rc, 0, pMe->m_cyHeight/2 - 2, pMe->m_cxWidth, 2); 
         IDISPLAY_DrawRect (pMe->a.m_pIDisplay, &rc, 0, 1, IDF_RECT_FILL);
         IIMAGE_Draw (pMe->m_pIImage, pMe->m_nCursorX, pMe->m_nCursorY);

         IDISPLAY_Update(pMe->a.m_pIDisplay);
         return (TRUE);
      case EVT_KEY:	    // process key-down event
         switch (wParam)
         {
            case AVK_LEFT:
            case AVK_RIGHT:
               CresApp_Move(pMe,(wParam == AVK_RIGHT ? 1 : -1), 0);
                  break;
            case AVK_UP:
            case AVK_DOWN:
               CresApp_Move(pMe,0,(wParam == AVK_UP ? -1 : 1));
                  break;
            default:
               return(FALSE);
         }
			return(TRUE);
      case EVT_APP_STOP:
         return TRUE;
	  case EVT_APP_SUSPEND:
			return TRUE;
      default:
         break;
   }

   return FALSE;
}

// Helper function to move bitmap on the screen
/*===========================================================================

FUNCTION CresApp_Move

DESCRIPTION
	This function is a helper function to move bitmap on the screen.

PROTOTYPE:
	void CresApp_Move(CResApp * pMe, int xc, int yc)

PARAMETERS:
	pMe: Pointer to the CResApp structure.
	xc : The increment by which to move the cursor along the x-axis.
   yc : The increment by which to move the cursor along the y-axis.

DEPENDENCIES
  none

RETURN VALUE
  none

SIDE EFFECTS
  none
===========================================================================*/
static void CresApp_Move(CResApp * pMe, int xc, int yc)
{
   AEEImageInfo  iInfo;
   int min, max;
   int x = pMe->m_nCursorX;
   int y = pMe->m_nCursorY;

   IIMAGE_GetInfo (pMe->m_pIImage, &iInfo);     // Get image info

   // Erase previously displayed bitmap from the screen
   IDISPLAY_EraseRgn (pMe->a.m_pIDisplay, x, y, iInfo.cx, iInfo.cy);

   x += xc;
   y += yc;

   // Delimit the x & y coordinated to lower half of the screen
   min = 0;
   max = pMe->m_cxWidth - iInfo.cx;
   x = ((x < min) ? (min) : (x > max) ? max : (x)); 

   min = pMe->m_cyHeight/2;
   max = pMe->m_cyHeight - iInfo.cy;
   y = ((y < min) ? (min) : (y > max) ? max : (y)); 
   
   IIMAGE_Draw (pMe->m_pIImage, x, y);

   pMe->m_nCursorX = x;     // Store new x coordinate
   pMe->m_nCursorY = y;     // Store new y coordinate

   IDISPLAY_Update(pMe->a.m_pIDisplay);
}

