/*===========================================================================

FILE: security.c

SERVICES: Sample applet using AEE

DESCRIPTION
  This file contains usage examples of the 3 security related interface. 
  
	1) IHash interface - Supports hashing algorithms. Currently only MD5 is supported
		

	2) ICipher - Supports ArcFour, a symmetric stream cipher


	3) IRSA	- Supports RSA
	  
PUBLIC CLASSES:  
   N/A


   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/

/*===============================================================================
                     INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
// Mandatory includes
#include "AEEAppGen.h"	// AEEApplet declaration
#include "AEEUsageAppIDs.h" // Class IDs of usage apps (including this one)
#include "AEE.h"			// Standard AEE Declarations
#include "AEEShell.h"	// AEE Shell Services
#include "AEEDisp.h"		// AEE Display Services

// Services used by this app
#include "AEESecurity.h"		//	AEE security Services
#include "AEEMenu.h"		// Menu Services
#include "AEEStdLib.h"   // AEE StdLib services



/*===========================================================================

                      PUBLIC DATA DECLARATIONS

===========================================================================*/

/*-------------------------------------------------------------------
            Type Declarations
-------------------------------------------------------------------*/
// App struct. This is the main struct for this applet. This will hold 
// all the data members that need to be remembered throughout the life of
// the applet. 
// THE FIRST DATA MEMBER OF THIS STRUCT MUST BE AN AEEApplet OBJECT. 
// This is critical requirement which is related to an applet being
// dynamically loaded. The app developers should add their data members
// following the AEEApplet data member.
typedef struct _CSecurityApp {
   AEEApplet	      a;
   IMenuCtl *  m_pIMenu;
   AEERect     m_rc;
} CSecurityApp;

/*-------------------------------------------------------------------
            Function Prototypes
-------------------------------------------------------------------*/
// Mandatory function:
// App Handle Event function
static boolean SecurityApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);

// App specific data alloc-init/free functions
static boolean Security_InitAppData(IApplet* pMe);
static void Security_FreeAppData(IApplet* pMe);

static void BuildMainMenu(CSecurityApp *pMe);
static void SecurityUsage (CSecurityApp *pMe, uint16 wParam);
static void DisplayEvent (CSecurityApp *pMe, uint16 wParam);
static void DisplayOutput(IShell *pIShell, IDisplay *pIDisplay, int nline, char *pszStr);  
static void DataToString(const char* pszPrefix, const byte* pbData, int cbData, char* pszBuf);
/*-------------------------------------------------------------------
            Global Constant Definitions
-------------------------------------------------------------------*/

// App specific constants
#define USAGE_BASIC_USAGE  100
#define USAGE_BASIC_MD5	   101
#define USAGE_BASIC_ARC4	102
#define USAGE_BASIC_RSA	   103
#define USAGE_ERASE_SCREEN 108   // Special case to erase screen

// Size of buffer used to hold text strings
#define TEXT_BUFFER_SIZE (100 * sizeof(AECHAR))

/*-------------------------------------------------------------------
            Global Data Definitions
-------------------------------------------------------------------*/


/*-------------------------------------------------------------------
            Static variable Definitions
-------------------------------------------------------------------*/


/*===============================================================================
                     FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

  // todo: change this
   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to Security_InitAppData(). This is done as follows:
   IHAp_InitAppData() is called to initialize AppletData instance. It is app developers 
   responsibility to fill-in app data initialization code of Security_InitAppData(). App developer
   is also responsible to release memory allocated for data contained in AppletData -- this can be
   done in Security_FreeAppData().

PROTOTYPE:
	boolean AEEClsCreateInstance(AEECLSID ClsId, IShell* pIShell, IModule* pIModule, 
										  void** ppObj)

PARAMETERS:
	ClsId: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell interface. 

	pIModule: pin]: Contains pointer to the IModule interface to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid AEEApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app needs to be loaded and if AEEApplet_New() invocation was successful
  FALSE: If the app does not need to be loaded or if errors occurred in AEEApplet_New().
  If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId, IShell* pIShell, IModule* po, void** ppObj)
{
   *ppObj = NULL;
		
   // We want to load this App. So, invoke AEEApplet_New().To it, pass the
   // address of the app-specific handle event function.
   if(ClsId == AEECLSID_SECURITY_APP){
      if(AEEApplet_New(sizeof(CSecurityApp), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)SecurityApp_HandleEvent,(PFNFREEAPPDATA)Security_FreeAppData) == TRUE)
      {
         if(Security_InitAppData((IApplet*)*ppObj))
             return(AEE_SUCCESS);
      }
   }
	return (EFAILED);
}

/*===========================================================================

FUNCTION HahpApp_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. 

PROTOTYPE:
	static boolean App_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet.

	ecode: Specifies the Event sent to this applet

    wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/

static boolean SecurityApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
   CSecurityApp * pMe = (CSecurityApp*)pi;

   switch (eCode) 
	{

        case EVT_APP_START:
			// When this applet is started, try to create the main menu
			// that allows the user to select a usage example to execute.
			// If we cannot create an instance of the menu class, exit and
			// indicate that we have handled the event by returning TRUE
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, 
				  (void **)&pMe->m_pIMenu) != SUCCESS)
				return TRUE;

			// Succeeded in obtaining a menu instance pointer, so
			// construct the menu and display it on the screen
			BuildMainMenu(pMe);
			return(TRUE);

        case EVT_APP_STOP:
            return(TRUE);

        case EVT_KEY: 
			switch (wParam)
			{
			   default:
				  IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);
			      if (IMENUCTL_HandleEvent(pMe->m_pIMenu, EVT_KEY, wParam, 0))
				     return TRUE;
			      else return FALSE;
			}

		case EVT_COMMAND:
  		    switch(wParam)
			{
				// The following commands are generated by user selections
				// from the main usage app menu.
			    case USAGE_BASIC_USAGE:
             case USAGE_BASIC_MD5:
				 case USAGE_BASIC_ARC4:
				 case USAGE_BASIC_RSA:
                
					// De-activate main menu to turn off horizontal scrolling
				   // of long menu items
				   IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);

				   // Execute the usage example the user has selected
               SecurityUsage (pMe, wParam); 
			      return TRUE;
			    default:
				    return FALSE;
			}
        default:
            break;
   }
   return TRUE;
}


/*===========================================================================

FUNCTION IHasp_InitAppData

DESCRIPTION
   This function initializes app-specific data.

PROTOTYPE:
	static boolean Security_InitAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains information specific
	to this applet.

DEPENDENCIES
   None.

RETURN VALUE
  TRUE: If the app has app data is allocated and initialized successfully
  FALSE: Either app data could not be allocated or initialized

SIDE EFFECTS
  none
===========================================================================*/
static boolean Security_InitAppData(IApplet* pi)
{
   CSecurityApp * pMe = (CSecurityApp*)pi;

   // Make sure the pointers we'll be using are valid
   if (pMe == NULL || pMe->a.m_pIShell == NULL)
	   return FALSE;

   pMe->m_pIMenu = NULL;

   return TRUE;
}

/*===========================================================================

FUNCTION Security_FreeAppData

DESCRIPTION
   Frees data contained in app data and memory for app data itself.

PROTOTYPE:
   static boolean Security_FreeAppData(IApplet* pi);

PARAMETERS:
   pi [in]: Pointer to the IApplet structure. This structure contains information specific
   to this applet.

DEPENDENCIES
   None.

RETURN VALUE
   None.

SIDE EFFECTS
   None.
===========================================================================*/
static void Security_FreeAppData(IApplet* pi)
{
   CSecurityApp * pMe = (CSecurityApp*)pi;

   if(pMe->m_pIMenu != NULL)
   {
	  IMENUCTL_Release(pMe->m_pIMenu);
      pMe->m_pIMenu = NULL;
   }
}



/*===============================================================================
                     APP HELPER FUNCTION DEFINITIONS
=============================================================================== */
/*===========================================================================

FUNCTION: SecurityUsage

DESCRIPTION
    This function executes the usage example selected from the main menu by the
	user.  It takes as input the app data pointer and the ID of the example the
	user selected.  After clearing the screen, the selected example is executed.
	Each example is a self-contained code block that contains any local variables
	needed to execute the example.  Each code block also creates and releases any
	BREW interfaces that are required, and will exit if any errors are detected.  
	Once the example has been executed, this function prints a heading at the top 
	of the screen to indicate which example was selected, 
	and a message at the bottom of the screen instructing the user how to return to
	the main menu to execute another example.
    
PROTOTYPE:
   static void SecurityUsage(CSecurityApp * pMe, uint16 wParam)

PARAMETERS:
   pMe:   [in]: Contains a pointer to the usage app data structure.
   wParam: [in]: Identifier of the selected example (example IDs are specified when the
                 main usage app menu is created).

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void SecurityUsage (CSecurityApp * pMe, uint16 wParam)
{
	// Make sure the pointers we'll be using are valid
   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
	   return;

   // Erase screen first for display output purposes.
   DisplayEvent (pMe, USAGE_ERASE_SCREEN);

   switch (wParam)
   {
      case USAGE_BASIC_USAGE:
	   case USAGE_BASIC_MD5:
         {
			// 1. Usage: Basic Hash Usage
			// 2. Usage: IHASH_Update
         // 3. Usage: IHASH_GetResult
         // 4. Usage: IHASH_Restart

			// These functions are used to hash data. Currently only MD5 is supported
			// IHash_Update is used to update the hash context. IHash_Update can be called
			// multiple times to hash long or discontinous streams of data. To get the hashed
			// result call IHash_GetResult. To reinitialize the hash context, call IHash_Restart

			IHash* pIHash = NULL;

			byte szInput[20]	= {"Hash this"};					// Data to be hashed
			byte* pInput		= szInput;
			byte md5_digest[]	= {
							0x2f, 0xc0, 0x58, 0x12,					// MD5 hash for the above string
							0xb8, 0x73, 0xcc, 0xfd, 
							0xc1, 0xb8, 0x1d, 0x9c, 
							0xb8, 0x4c, 0xb7, 0xe2
			};

			byte szOutput[MD5_LENGTH+1];
			int  size = MD5_LENGTH;
			char szBuf[100];
			
			// Create an instance of the IHash interface
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MD5, 
				  (void **)&pIHash) != SUCCESS || pIHash == NULL)
			{
				SPRINTF(szBuf, "no IHash interface"); 
			   DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 2, szBuf);
				return;
			}

			// Display data to be hashed
			SPRINTF(szBuf, "Input: %s", szInput); 
	      DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 0, szBuf);
			
			// Hash the data.
			IHASH_Update(pIHash, pInput, STRLEN((const char*)szInput));
			
			// Get hashed result.
			if(IHASH_GetResult(pIHash, szOutput, &size) != SUCCESS)
				return;
	
			// Verify hash generated using previously computed result
			if(MEMCMP(szOutput, md5_digest, MD5_LENGTH))
				return;
			
			// IHash_Restart has to be called to reinitialize the hash context.
			IHASH_Restart(pIHash);
			
			// Now hash the same input string, but one byte at a time
			// Multiple calls to IHash_Update can be made to hash long 
			// or discontinous streams of data.
			for(; *pInput; pInput++)
				IHASH_Update(pIHash, pInput, 1);
			
			// Get hashed result.
			if(IHASH_GetResult(pIHash, szOutput, &size) != SUCCESS)
				return;
			
			// Verify hash generated using previously computed result
			if(MEMCMP(szOutput, md5_digest, MD5_LENGTH))
				return;
			
			// Display hashed data
			DataToString("MD5: ", szOutput, MD5_LENGTH, szBuf);
			DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 1, szBuf);
			
			// Display size of hashed data
			SPRINTF(szBuf, "Size: %d bytes", size); 
			DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 4, szBuf);

			// Release the interface pointer
			IHASH_Release(pIHash);

			break;
			}
		case USAGE_BASIC_ARC4:
			{
				ICipher* pICipher = NULL;
				
				byte szInput[]	= "arc4 encrypt";				// String to encrypt
				byte* pInput	= szInput;
				byte key [16]	= {
									0x29, 0x04, 0x19, 0x72,		// ArcFour key (symmetric)
									0xfb, 0x42, 0xba, 0x5f, 
									0xc7, 0x12, 0x77, 0x12,
									0xf1, 0x38, 0x29, 0xc9
				};

				byte szOutput[20];
				int len = sizeof(szOutput);
				char szBuf[100];

				// Create an instance of ICipher interface	
				if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_ARC4, 
				  (void **)&pICipher) != SUCCESS || pICipher == NULL)
				{
					SPRINTF(szBuf, "no ICipher interface"); 
					DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 2, szBuf);
					return;
				}

				// Display data to be hashed
				SPRINTF(szBuf, "Input: %s", szInput); 
				DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 0, szBuf);

				// Initialize ArcFour key
				if(ICIPHER_Init(pICipher, key, 16) != SUCCESS)
					return;

				// Encrypt
				if(ICIPHER_Cipher(pICipher, pInput, STRLEN((const char*)pInput), szOutput, &len) == EFAILED)
					return;

				// Display encrypted data
				DataToString("Arc4: ", szOutput, len, szBuf);
				DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 1, szBuf);

				// Reinitialize key
				if(ICIPHER_Init(pICipher, key, 16) != SUCCESS)
					return;

				// Decrypt
				if(ICIPHER_Cipher(pICipher, szOutput, len, szOutput, &len) == EFAILED)
					return;
				
				// Compare data to see if encrypt followed by decryption gets back the original data
				if(MEMCMP(szInput, szOutput, len))
					return;
				
				ICIPHER_Release(pICipher);

				break;
			}
		case USAGE_BASIC_RSA:
			{
				IRSA* pIRSA = NULL;
				char szBuf[256];
				// 512-bit modulus
				const unsigned char mod512[] = {
						0xa7, 0x84, 0x9d, 0x0e, 0xfe, 0x71, 0x58, 0x28, 
						0xa3, 0x96, 0xed, 0x35, 0x71, 0x54, 0xc1, 0x79, 
						0xcb, 0x62, 0x4c, 0xbe, 0x29, 0x5d, 0x54, 0x4e, 
						0xdd, 0xd2, 0x61, 0xf1, 0x08, 0x31, 0x25, 0x45, 
						0xa4, 0xf4, 0x1e, 0x61, 0xab, 0xf3, 0x1f, 0xc9, 
						0x4a, 0x30, 0x39, 0xd4, 0x8a, 0x53, 0xca, 0x30, 
						0x83, 0x4c, 0x51, 0x49, 0x47, 0x7e, 0xbc, 0xc0, 
						0x0d, 0x2b, 0xc4, 0x6a, 0xde, 0xd4, 0xc3, 0xf1};

				// 512-bit private key exponent
				unsigned char exp512[] = {
						0x6f, 0xad, 0xbe, 0x09, 0xfe, 0xf6, 0x3a, 0xc5,
						0xc2, 0x64, 0x9e, 0x23, 0xa0, 0xe3, 0x2b, 0xa6,
						0x87, 0x96, 0xdd, 0xd4, 0x1b, 0x93, 0x8d, 0x89,
						0xe9, 0x36, 0xeb, 0xf6, 0x05, 0x76, 0x18, 0xd8,
						0x04, 0x04, 0x6c, 0x4e, 0x57, 0x89, 0x4b, 0x4f,
						0x7f, 0x29, 0x06, 0xff, 0x0a, 0xb0, 0x71, 0x38,
						0x98, 0x9f, 0xbc, 0xae, 0x6b, 0xf9, 0x66, 0x23,
						0x94, 0x24, 0x80, 0x90, 0x32, 0x4b, 0x4e, 0xab
				};
				
				// public exponent
				unsigned char publicE = 0x03;

				byte szInput[64]	= "rsa encrypt";				// String to encrypt
				byte* pInput	= szInput;
				byte* pOutput	= NULL;
				int size;

				
				if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_RSA, 
				  (void **)&pIRSA) != SUCCESS || pIRSA == NULL)
				{
					SPRINTF(szBuf, "no IRSA interface"); 
					DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 2, szBuf);
					return;
				}
				
				// Set private key
				if(IRSA_Init(pIRSA, mod512, 64, exp512, 64) != SUCCESS)
					return;

				// RSA encrypt
				if(IRSA_RSA(pIRSA, pInput, 64, &pOutput, &size, 0) != SUCCESS)
					return;

				// Display data to be hashed
				SPRINTF(szBuf, "Input: %s", szInput); 
				DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 0, szBuf);

				// Display encrypted data
				DataToString("rsa: ", pOutput, size, szBuf);
				DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 1, szBuf);


				// Set public key
				if(IRSA_Init(pIRSA, mod512, 64, &publicE, sizeof(publicE)) != SUCCESS)
					return;

				// RSA decrypt
				if(IRSA_RSA(pIRSA, pOutput, size, &pOutput, &size, 0) != SUCCESS)
					return;
				
				// Compare data to see if encrypt followed by decryption gets back the original data
				if(MEMCMP(szInput, pOutput, size))
					return;

				IRSA_Release(pIRSA);

				break; 
			}

     
      default:
         return;
   }
            
   // Display heading at top of screen that indicates the example
   // selected, and display message at bottom instructing the user
   // to return to main menu by pressing UP or DOWN. 
   DisplayEvent (pMe, wParam);

   return;
}




/*===========================================================================

FUNCTION: DisplayEvent

DESCRIPTION
    This function prints a heading at the top 
	of the screen to indicate which usage example was selected by the user, 
	It also prints a message on the bottom two lines of the screen instructing 
	the user to press the UP or DOWN key to return to
	the main menu to execute another example.
    
PROTOTYPE:
   static void DisplayEvent(CIHashApp * pMe, uint16 wParam)

PARAMETERS:
   pMe:   [in]: Contains a pointer to the usage app data structure.
   wParam: [in]: Identifier of the selected example (example IDs are specified when the
                 main usage app menu is created).  If the value USAGE_ERASE_SCREEN is supplied
				 instead, this function clears the screen.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void DisplayEvent (CSecurityApp *pMe, uint16 wParam)
{
   AEERect qrc;
   AEEDeviceInfo di; // Device Info
   AECHAR * szBuf;
   AEERect rc; 

   // Make sure the pointers we'll be using are valid
   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
	   return;

   if ((szBuf = (AECHAR *) MALLOC(TEXT_BUFFER_SIZE * sizeof(AECHAR))) == NULL)
	   return;

   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);
	qrc.x = 0;
	qrc.y = 0;
	qrc.dx = di.cxScreen;
	qrc.dy = di.cyScreen;
   rc.x = 5;
   rc.y = 5;
   rc.dx = di.cxScreen - 5;
   rc.dy = di.cyScreen - 5;

   // Specify the text string to be used for the example heading
   switch (wParam)
   {
      case USAGE_ERASE_SCREEN:
         IDISPLAY_EraseRect(pMe->a.m_pIDisplay,&qrc);  //Clear the display
		 FREE(szBuf);
         return;
      case USAGE_BASIC_USAGE:
		case USAGE_BASIC_MD5:
         STR_TO_WSTR("MD5 Usage", szBuf, TEXT_BUFFER_SIZE);
         break; 
		case USAGE_BASIC_ARC4:
         STR_TO_WSTR("ArcFour Usage", szBuf, TEXT_BUFFER_SIZE);
         break; 
		case USAGE_BASIC_RSA:
         STR_TO_WSTR("RSA Usage", szBuf, TEXT_BUFFER_SIZE);
         break; 
      default:
         STR_TO_WSTR("Unsupported fn", szBuf, TEXT_BUFFER_SIZE);
   }

   // Print the heading at the top of the screen
   //IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,10, 5, &rc,
	//  IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);

   // Print message at bottom of screen to instruct user to press UP
   // or DOWN to return to main menu
   STR_TO_WSTR("Press Up/Down", szBuf, TEXT_BUFFER_SIZE);
   IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,10, qrc.dy - 25, &rc,
		IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
   STR_TO_WSTR("keys to continue..", szBuf, TEXT_BUFFER_SIZE);
   IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,10, qrc.dy - 15, &rc,
		IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
   IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);

   FREE(szBuf);
}
/*===========================================================================

FUNCTION: BuildMainMenu

DESCRIPTION
    This function constructs the menu control that allows the user to
	select a usage example to execute.  The command ID for each menu item
	is passed to the usage example function above when the user selects the
	menu item for that example.
    
PROTOTYPE:
   static void BuildMainMenu(CSecurityApp * pMe)

PARAMETERS:
   pMe:   [in]: Contains a pointer to the usage app data structure.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void BuildMainMenu(CSecurityApp *pMe)
{
	AEERect qrc;
	AEEDeviceInfo di;
    AECHAR * szBuf;

	// Make sure the pointers we'll be using are valid
    if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->m_pIMenu == NULL)
	   return;

   if ((szBuf = (AECHAR *) MALLOC(TEXT_BUFFER_SIZE * sizeof(AECHAR))) == NULL)
	   return;

	// Set Title
    STR_TO_WSTR("Security Interfaces: press Select key", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_SetTitle(pMe->m_pIMenu, NULL, 0, szBuf);

	//Set the Rectangle for the Menu
	ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);
	qrc.x = 0;
	qrc.y = 0;
	qrc.dx = di.cxScreen;
	qrc.dy = di.cyScreen;
	IMENUCTL_SetRect(pMe->m_pIMenu, &qrc);  

   // Add md5 to menu
   STR_TO_WSTR("1. MD5 Usage", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_BASIC_USAGE, szBuf, 0);

	// Add ArcFour to menu
   STR_TO_WSTR("2. ArcFour Usage", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_BASIC_ARC4, szBuf, 0);

	// Add rsa to menu
   STR_TO_WSTR("3. RSA Usage", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_BASIC_RSA, szBuf, 0);
   
      
   // Activate menu
   IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);

   FREE(szBuf);
}
/*===========================================================================

FUNCTION: DisplayOutput

DESCRIPTION
    This function displays an output string at a given line number on the
    screen. If the nline parameter is a negative value (-1) the string
    is displayed in the middle of the screen. If the "nline" value is larger
    than or equal to zero the "nline" value is multiplied by 15 and the 
    resulting value in pixels is set to the y-coordinate of the start of 
    the string display on the screen. If the string does not fit on one line
    the string wraps around to the next line (spaced rougly 10-15 pixels apart).
    By default 5 is used as the starting the x-coordinate of a displayed 
    string.

    How many characters that fit on one line is calculated for each line 
    that is wrapped around to the next line.

    Note: depending on the phone screen size and the fonts used for characters 
          the output might differ on different handsets (devices). Where some 
          handsets will have a smaller screen and large default fonts which will 
          cause partial overlapping of lines. This function does not try to address
          these issues (this is meant as a simple display function).
    
PROTOTYPE:
   void DisplayOutput(IShell *pIShell, IDisplay *pDisplay, int nline, char *pszStr)

PARAMETERS:
   pIShell:   [in]: Contains a pointer to the IShell interface.
   pIDisplay: [in]: Contains a pointer to the IDisplay interface.
   nline:     [in]: Contains the line number to start displaying the text. The line
        numbers are by default spaced 15 pixels apart along the y-axis.
   pszStr:    [in]: The character string to be displayed on the screen.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
void DisplayOutput(IShell *pIShell, IDisplay *pIDisplay, int nline, char *pszStr)
{
   AEEDeviceInfo di; // Device Info
   AECHAR * szBuf;      // a buffer that supports 200 char string
   AECHAR * psz = NULL;
   int charHeight = 0;      // Stores the char height in pixels for given font
   int pnAscent = 0;        // Stores the ascent in number of pixels
   int pnDescent = 0;       // Stores the descent in number of pixels
   int pixelWidth;
   AEEFont font = AEE_FONT_NORMAL;
   int pnFits = 0, dy;
   int totalCh = 0;

   // Make sure the pointers we'll be using are valid
   if (pIShell == NULL || pIDisplay == NULL)
	   return;

   if ((szBuf = (AECHAR *) MALLOC(TEXT_BUFFER_SIZE * sizeof(AECHAR))) == NULL)
	   return;

   // Get device information
   ISHELL_GetDeviceInfo(pIShell,&di);

   // Get the font metrics info
   charHeight = IDISPLAY_GetFontMetrics (pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   // Convert to wide string (unicode)
   STR_TO_WSTR ((char *)pszStr, szBuf, TEXT_BUFFER_SIZE);

   // If nlines is zero then print this string starting around the middle of 
   // the screen. Or else multiply nlines by charheight to decide the y coordinate of
   // the start of the string.
   if (nline < 0) {
      dy = di.cyScreen*2/5;
   }
   else{
      dy = nline * charHeight;
   }

   // psz keeps track of the point from which to write from the string buffer
   // in case the string does not fit one line and needs to wrap around in the
   // next line.
   psz = szBuf;
      
   // Need to calculate the lotal string length to decide if any wrapping
   // around is needed.
   totalCh = STRLEN ((char *)pszStr);

   // Keep displaying text string on multiple lines if the string can't be displayed
   // on one single line. Lines are spaced 15 pixels apart.
   while ((totalCh > 0) && (*psz != NULL))
   { 
      // Get information on how many characters will fit in a line.
      // Give the pointer to the buffer to be displayed, and the number of
      // pixels along the x axis you want to display the string in (max number)
      // pnFits will have the max number of chars that will fit in the maxWidth
      // number of pixels (given string can't fit in one line), or the number of 
      // chars in the string (if it does fit in one line). pnWidth gives the
      // number of pixels that will be used to display pnFits number of chars.
      pixelWidth = IDISPLAY_MeasureTextEx(pIDisplay,
                      font, 
                      (AECHAR *) psz,  // Start of the buffer to display,
                      -1,
                      di.cxScreen - 5, // maxWidth
                      &pnFits);         // Number of chars that will fit a line

      // If pnFits is zero there is something wrong in the input to above function. 
      // Normally this scenario should not occur. But, have the check anyway.
      if (pnFits == 0)
	  {
		 FREE(szBuf);
         return;
      }

      IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, psz, pnFits, 5 /*start dx*/, 
         dy, 0 /* use default rectangle coordinates */, 0);

      psz += pnFits;      // move pointer to the next segment to be displayed
      totalCh -= pnFits;  // reduce the total number of characters to still display
      dy += charHeight;   // Place next line charHeight pixels below the 
                          // previous line.
      IDISPLAY_Update(pIDisplay); //, TRUE);
      if (totalCh < pnFits)
         pnFits = totalCh;  // if total number is less than pnFits, adjust pnFits
   }
   FREE(szBuf);
   return;   
} // End of DisplayOutput

/*===========================================================================

FUNCTION: DataToString

DESCRIPTION
    Format the hashed data to a printable 

PROTOTYPE:
   static void DataToString(const char* pszPrefix, const byte* pbData, int cbData, char* pszBuf)

PARAMETERS:
   pszPrefix:   [in]: Prefix string
   pbData: [in]: Data to be formatted
	int cbData[in] : Size of data

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void DataToString(const char* pszPrefix, const byte* pbData, int cbData, char* pszBuf)
{
	int j = 0;
	int i = SPRINTF(pszBuf, pszPrefix ); 
	while(j < cbData) {
		 SPRINTF(pszBuf + i, "%x   ", ((int)pbData[j++]) & 0xff);
		i+=3;
	}

}