/*===========================================================================

FILE: Socket.c

SERVICES: Sample applet to demonstrate usage of INetMgr and ISocket Services. 

DESCRIPTION
  This file contains the implementation of a simple applet using the
  AEE NetMgr and Socket services.

  This file contains a sample program that reads from a socket. The socket that has
  been chosen to read from is one of the standard addresses that gives out the
  current time in seconds elapsed since Jan 1 1900. This address is at Node:
	"132.163.135.130" and Port: 37
  
PUBLIC CLASSES:  
   N/A

INITIALIZATION AND SEQUENCING REQUIREMENTS:
   The following explanation applies to this sample containing one applet which serves
   as a base for app developers to create their own applets using AEE Services:
   

   In the applet source file (like this one), include AEEAppGen.h.

   Mandatory Sections in Applet Source (this file):
   -----------------------------------------------
   Following Mandatory Sections are required for each applet source file. 
   (Search for "Mandatory" to identify these sections)

   Includes:     
      Copy this section as-is from the sample applet. It contains:
      AEEAppGen.h: For AEEApplet declaration

   Type Declarations:
		A data structure must be defined to hold the app specific data. In this structure,
		the first element must be of type AEEApplet.

   Functions: (For more details, see corresponding function description in this applet)

      App_HandleEvent(): is the Event Handler to this applet.
                         Copy the function outline from the sample applet and add app specific code.

      AEEClsCreateInstance(): creates an instance of the applet and initializes it. 
                              It is called by AEEModGen when applet is being created.

   
   Important Notes:
   ---------------
   1. DO NOT use any "static data" in the applet. Always use the functions exported by
	   AEEStdlib or by IHeap services to dynamically allocate data and make it a member of 
		the applet structure. 
   2. DO NOT include and link "standard C library". Use AEE Memory Services (in AEEHeap.h) and Standard Library macros(in AEEStdLib.h).
      For example, use MALLOC() to allocate memory, WSTRCPY() to make a copy of Unicode (wide) string.
   3. BREW is Unicode(wide string) compliant ONLY (no ISOLATIN1/ANSI) except for file names which are ISOLATIN1/ANSI.
      ALWAYS USE AECHAR instead of "char". Use string services provided in AEEStdLib.h for string manipulation.
   4. It is always a good idea to DEFINE RESOURCES using BREW ResourceEditor. Make Strings, Bitmaps, Dialogs, etc. 
      as resources. ResourceEditor saves resources as .bri file, generates resource header file 
      and compiles .bri into a .bar binary file, which can be used by the applet.

   Miscellanoeus Notes:
   -------------------
   1. Make sure that the class ID used for the app is the same as that defined corresponding in .MIF file
   2. Always make sure that compiled resource (.bar) file and corresponding
      resource header (a) reside in app directory and (b) are included in the applet code.
      Define a constant APP_RES_FILE containing the name of the compiled resource file (with .bar extension).

   More than one applet:
   --------------------
   If more than one applet needs to be defined, then do the following
   (1) Follow the above description for each applet

   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/


/*===============================================================================
                     INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "AEE.h"			// Standard AEE Declarations
#include "AEEShell.h"	// AEE Shell Services
#include "AEEDisp.h"		// AEE Display Services
#include "AEEStdLib.h"  // AEE StdLib Services
#include "AEEAppGen.h"	// AEEApplet structure and Applet services
#include "AEESampleAppIDs.h" // Sample App ID

// Services used by app
#include "AEENet.h"		//	AEENet Services

/*===========================================================================

                      PUBLIC DATA DECLARATIONS

===========================================================================*/

/*-------------------------------------------------------------------
            Type Declarations
-------------------------------------------------------------------*/
//Mandatory structure.
//Structure to contain app data. This is the Applet structure that will be allocated
// for the app. Define this structure if the app needs to store any data that needs
// to be remembered across different events or callbacks. The first element of this
// structure MUST be of type AEEApplet. This is extremely important. Following this
// element, store the app-specific data. Note that AEEApplet structure already exposes
// the most commonly needed interface pointers such as Shell, Display, etc. 
typedef struct _CSocketApp
{
	//First element of this structure must be AEEApplet.
	AEEApplet	      a ;

	//App specific data
   INetMgr *m_piNet;		// INetMgr interface ptr
   ISocket *m_piSock;	// ISocket interface ptr
   boolean demoInProgress; // demo in progress
}CSocketApp;


//A few utility macros; We shall eventually move this into the AEE Services.
#define ISDIGIT(c)  ( (unsigned) ((c) - '0') < 10)
#define ISALPHA(c)  ( (unsigned) ( ((c)|32) - 'a') < 26 )
#define ISALNUM(c)  ( ISDIGIT(c) || ISALPHA(c) )


#define min(a,b)  ((a) < (b) ? (a) : (b))

#define ARRAYSIZE(a)   (sizeof(a) / sizeof((a)[0]))

/*-------------------------------------------------------------------
            Function Prototypes
-------------------------------------------------------------------*/

// Mandatory function: App Handle Event function

static boolean SockApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);

// Socket operation:

static void Sockapp_Demo(CSocketApp *pme);
static void Sockapp_ConnectCB(void *cxt, int err);
static void Sockapp_ReadCB(void *cxt);
static void Sockapp_DemoCleanup(CSocketApp *pme);

// App Helper Functions

/*-------------------------------------------------------------------
            Global Constant Definitions
-------------------------------------------------------------------*/


/*-------------------------------------------------------------------
            Global Data Definitions
-------------------------------------------------------------------*/

// App specific constants

//Port address to connect to.
#define  TIMESERVER_PORT      37
#define  TIMESERVER_HOST      "132.163.135.130"   // IP address in a.b.c.d dot-notation


// Invalid Address
#define DOTADDR_NONE  0xFFFFFFFF


/*-------------------------------------------------------------------
            Static variable Definitions
-------------------------------------------------------------------*/

/*===============================================================================
                     FUNCTION DEFINITIONS
=============================================================================== */



/*===========================================================================

FUNCTION: xDrawTextA

DESCRIPTION
	Convenience function. Single-byte version of IDISPLAY_DrawText().

===========================================================================*/
static void xDrawTextA(IDisplay * pd,AEEFont fnt, const char * pszText, int nChars,
                       int x,int y,const AEERect * prcBackground,uint32 dwFlags)
{
   AECHAR wcBuf[80];

   if (nChars < 0)
      nChars = STRLEN(pszText);
   nChars = min(nChars, ARRAYSIZE(wcBuf));

   STR_TO_WSTR((char*)pszText, wcBuf, sizeof(wcBuf));

   IDISPLAY_DrawText(pd, fnt, wcBuf, nChars, x, y, prcBackground, dwFlags);
}



/*===========================================================================

FUNCTION: xDrawTextA

DESCRIPTION
	Convenience function.
   Temporary substitute for inet_addr (this functionality will be
   provided in AEE eventually).

   This function supports only a.b.c.d notation.  This generates an
   INAddr, which is defined to be an IP address in network byte order.

===========================================================================*/
static INAddr xConvertToINAddr(char *psz)
{
   INAddr ul = 0;
   int nByte = 0;
   char c;

   if(!psz)
      return 0;

   while (ISDIGIT(*psz)) {
      int n = 0;
      while ( ISDIGIT(c=*psz)) {
         n = n*10 + (c - '0');
         ++psz;
      }
      ((char*)&ul)[nByte++] = n;
 
      if (nByte == 4 || *psz != '.')
         break;
 
      ++psz;
   }
 
   if (nByte < 4 || ISALNUM(*psz))
      ul = DOTADDR_NONE;

   return ul;
}


/*===========================================================================

FUNCTION: xStatus

DESCRIPTION
	Convenience Function: Display a status line on the screen.   nLine indicates the line
   number.

===========================================================================*/

#define LINEHEIGHT   14   // hack
#define TOPLINE      LINEHEIGHT

static void xStatus(CSocketApp *pme, int nLine, const char *psz)
{
   AEEDeviceInfo di;
   AEERect rc;
   int nMaxLines;
   
	ISHELL_GetDeviceInfo(pme->a.m_pIShell,&di);
   nMaxLines = (di.cyScreen / LINEHEIGHT) - 2;
   if (nMaxLines < 1)
      nMaxLines = 1;
   
   // 0 always first; lines 1...n cycle
   // 0 clears all other lines

   rc.x = 0;
   rc.dx = di.cxScreen;
   
   if (nLine > 0)
      nLine = (nLine - 1) % nMaxLines + 1;

   rc.y = TOPLINE + (nLine * LINEHEIGHT);
   rc.dy = LINEHEIGHT;
   if (nLine == 0)
      rc.dy = di.cyScreen - rc.y;

   xDrawTextA(pme->a.m_pIDisplay,
              (nLine == 0 ? AEE_FONT_BOLD : AEE_FONT_NORMAL),
              psz, -1, rc.x, rc.y, &rc, 0);
   IDISPLAY_Update(pme->a.m_pIDisplay);
}

/*===========================================================================

Function:  Socket_InitAppData()

Description:
		This function initializes the app specific data.

Prototype:
	void Socket_InitAppData(IApplet* po)

Parameters:
   IApplet* po: Pointer to the applet structure. This is of the type defined
	to store applet data. (CSocketApp*, in this case).

Return Value:
	TRUE: If successful
	FALSE: IF failed

Comments:  None

Side Effects: None

==============================================================================*/
boolean Socket_InitAppData(IApplet* po)
{
	CSocketApp *pMe = (CSocketApp*)po;

	//Initialize Data Members
	pMe->m_piNet = 0;
	pMe->m_piSock = 0;
    pMe->demoInProgress = FALSE;

	return TRUE;

}

/*===========================================================================

Function:  Socket_FreeAppData()

Description:
	This function frees the app data. This function is registered with the
	applet framework when the applet is created (inside AEEClsCreateInstance() function).
	This function is called by the app framework when the reference count of the 
	applet reaches zero. This function must free all the app data that has been
	allocated by this app. For ex: if their is data that was remembered when
	the app is suspended and resumed, those data must be freed here.

Prototype:
	void Socket_FreeAppData(IApplet* po)

Parameters:
   IApplet* po: Pointer to the applet structure. This is of the type defined
	to store applet data. (CSocketApp*, in this case).

Return Value:
	None

Comments:  None

Side Effects: None

==============================================================================*/
void Socket_FreeAppData(IApplet* po)
{
	CSocketApp *pMe = (CSocketApp*)po;
   if (pMe->m_piSock) 
	{
      ISOCKET_Release(pMe->m_piSock);
      pMe->m_piSock = 0;
   }

   if (pMe->m_piNet) 
	{
      INETMGR_Release(pMe->m_piNet);
      pMe->m_piNet = 0;
   }
}

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time.

PROTOTYPE:
	int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell interface. 

	pIModule: pin]: Contains pointer to the IModule interface to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppApplet must point to a valid IBase object. 
	If the classID	represnts an applet, then ppObj must point to a valid AEEApplet structure.Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the class/app creation was successful. 
  EFAILED: Error occurred while creating the class/app. In this case, the app/class will
  not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
//Mandatory Function
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
{
	*ppObj = NULL;

   if(ClsId == AEECLSID_SOCKETAPP)
	{
		//Create the applet
      if(AEEApplet_New(sizeof(CSocketApp), ClsId, pIShell,po,(IApplet**)ppObj,(AEEHANDLER)SockApp_HandleEvent,(PFNFREEAPPDATA)Socket_FreeAppData))
		{
			//Initialize applet data
         if(Socket_InitAppData((IApplet*)*ppObj))
			{
				//Data initialized successfully
				return(AEE_SUCCESS);
			}
			else
			{
				//Release the applet. This will free the memory allocated for the applet when
				// AEEApplet_New was called.
				IAPPLET_Release((IApplet*)*ppObj);
				return EFAILED;
			}

      }//AEEApplet_New

   }// ClsId == AEECLSID_SOCKETAPP

   return(EFAILED);
}


/*===========================================================================

FUNCTION Sockapp_CheckNetStatus

DESCRIPTION
   This timer-based callback checks the current network status.

===========================================================================*/
static void Sockapp_CheckNetStatus(CSocketApp *pMe)
{
	INetMgr *piNet;
   if (ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_NET, (void**)(&piNet)) == SUCCESS) 
	{

      AEERect rc = {20, 0, 120, LINEHEIGHT};

      NetState ns = INETMGR_NetStatus(piNet,NULL);

      char *psz = (ns == NET_PPP_OPENING ? "Net: Dialing..." :
                   ns == NET_PPP_OPEN    ? "Net: UP" :
                   ns == NET_PPP_CLOSING ? "Net: Closing..." :
                   ns == NET_PPP_CLOSED  ? "-no net-" : "?" );

      xDrawTextA(pMe->a.m_pIDisplay,AEE_FONT_NORMAL,
                 psz, -1, 20, 0, &rc, 0);
      
      INETMGR_Release(piNet);
   }
}

/*===========================================================================

FUNCTION SockApp_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean App_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AppCreateInstance() function.

	ecode: Specifies the Event sent to this applet

   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/
//Mandatory Function
static boolean SockApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{
   CSocketApp * pme = (CSocketApp*)pi;

   switch (eCode) 
	{

   case EVT_APP_START:
      xStatus(pme, 0, "Socket Demo");
      xStatus(pme, 1, " 1 = Start");
      xStatus(pme, 2, " 2 = Stop");

      // Request INETMGR notifications.  This is only for the sake of the network
      // status display at the top of the screen.
      //
      ISHELL_RegisterNotify(pme->a.m_pIShell, AEECLSID_SOCKETAPP, AEECLSID_NET, NMASK_OPENED|NMASK_CLOSED);
      
      Sockapp_CheckNetStatus(pme);
      
      return(TRUE);

   case EVT_APP_STOP:

      // Release any acquired resources

      Sockapp_DemoCleanup(pme);
      
      ISHELL_CancelTimer(pme->a.m_pIShell, 0, pme);

      ISHELL_RegisterNotify(pme->a.m_pIShell, AEECLSID_SOCKETAPP, AEECLSID_NET, 0);

      xStatus(pme, 0, "Socket Stop");

      return(TRUE);

   case EVT_NOTIFY: 
	{

      // Check change in network status
      
      AEENotify *pnot = (AEENotify*)dwParam;

      if (pnot->cls == AEECLSID_NET) 
		{
         // Examine change in network status
         Sockapp_CheckNetStatus(pme);
         return (TRUE);
      }
   }

   case EVT_KEY:
      switch(wParam) 
		{

      case AVK_1:

         // Invoke Function to Read from a Socket.
         if (!pme->demoInProgress)
         {
             pme->demoInProgress = TRUE;
             Sockapp_Demo(pme);
         }
         return TRUE;

      case AVK_2:

         // Stop the operation without releasing the ISocket

         if (pme->m_piSock) 
         {
            xStatus(pme, 6, "**STOP**");
            ISOCKET_Cancel(pme->m_piSock, 0, 0);
         } 
         else 
         {
            xStatus(pme, 6, "(not active)");
         }
         Sockapp_DemoCleanup(pme);
         return(TRUE);
      }
   }
   
   return(FALSE);
}



/*===============================================================================
                     APP HELPER FUNCTION DEFINITIONS
=============================================================================== */



/*===========================================================================

FUNCTION Sockapp_Demo()

DESCRIPTION
	This function is invoked from the HandleEvent() in the main app. Its purpose
	is to demonstrate the usage of INetMgr and ISocket intrfaces to perform all the steps
	necessary to read from a socket.
	In order to read from a socket using the AEE services, the following steps need
	to be carried out:
	1. Get an Instance of INetMgr. This is done using ISHELL_CreateInstance()
	2. Open a socket. This is done usiong INETMGR_OpenSocket
	3. Perform a connect operation on the socket. Please look at the function 
	   SocketConnect() for details on how to connect to the socket
	4. Perform a read operation on the socket. Please look at the function
	   Sockapp_ReadCB() for details on how to do this
	5. Close the socket using ISOCKET_Release()

PARAMETERS:
	pme: Pointer to the AEEApplet structure. 

===========================================================================*/
static void Sockapp_Demo(CSocketApp *pme)
{
	INetMgr *piNet;
	ISocket *piSock;
   INAddr nodeINAddr;                           // IP address in network byte order

	//This is a sample function that tries to read from a socket:

   xStatus(pme, 0, "Demo");
   
	//Step 1: Create an instance of INetMgr
   if (ISHELL_CreateInstance(pme->a.m_pIShell, AEECLSID_NET, (void**)(&piNet)) != SUCCESS) 
	{
      xStatus(pme, 1, "INetMgr failed");
      Sockapp_DemoCleanup(pme);
		return; // failed
   }
   xStatus(pme, 1, "Got INetMgr");

	//Set the member into our applet structure
	pme->m_piNet = piNet;
    INETMGR_SetLinger(pme->m_piNet, 0);

	// Step 2: Open the socket. This shall be a TCP socket.
	piSock = INETMGR_OpenSocket(piNet, AEE_SOCK_STREAM);
	if (!piSock) 
	{
      xStatus(pme, 2, "Socket failed");
      Sockapp_DemoCleanup(pme);
		return;
   }
   xStatus(pme, 2, "Got Socket");

	//Set the member into our applet structure
	pme->m_piSock = piSock;

	//Step 3: Connect the Socket
   xStatus(pme, 3, "Connecting...");

	// Before we can pass on the address to connect the socket, we need to convert the
	// address from a string to the INAddr format.
   nodeINAddr = xConvertToINAddr(TIMESERVER_HOST);

   // Connect the socket. Remember to convert the port number from Host to Network system
	// format.  This is done using HTONS.
   ISOCKET_Connect(piSock, nodeINAddr, HTONS(TIMESERVER_PORT), Sockapp_ConnectCB, pme);
}


/*===========================================================================

FUNCTION Sockapp_ConnectCB()

DESCRIPTION

   This function is the callback that accepts the result code from Connect().

   It takes a 'void *' so it can be passed to Connect() without a cast.
   (Safer to cast a simple structure pointer than to cast a function pointer.)

PARAMETERS:
	pme: Pointer to the AEEApplet structure. 

===========================================================================*/
static void Sockapp_ConnectCB(void *cxt, int err)
{
   CSocketApp *pme = (CSocketApp*)cxt;

   if (err) 
   {
      xStatus(pme, 3, "Connect failed!");
      Sockapp_DemoCleanup(pme);
	  return;
   }
   xStatus(pme, 3, "Connected");

   // Success. Socket has been connected. Now read from the Socket.
   Sockapp_ReadCB((void*)pme);
}


/*===========================================================================

FUNCTION Sockapp_ReadCB()

DESCRIPTION

   This is the function that reads from the socket. It will be called after a
   successful connect, and after some data has been read it may schedule itself to
   be called again.

   It takes a 'void *' so it can be passed to the Readable() without a cast.
   (Safer to cast a simple structure pointer than to cast a function pointer.)

PARAMETERS:
	pme: Pointer to the AEEApplet structure. 

===========================================================================*/
static void Sockapp_ReadCB(void *cxt)
{
   CSocketApp *pme = (CSocketApp *)cxt;

   ISocket *piSock = pme->m_piSock;
   int rv;				   // Return value frm Read()
	uint32 uTime;			// Buffer to pass to socket-read function
   char szBuf[50];

	// Issue a read command on the socket. We know that the address that we have
	// connected to returns an uint32 value. Hence, we cast that to char* and send
	// it to the read function.
   
   rv = ISOCKET_Read(piSock, (byte*)&uTime, sizeof(uint32));

	// Read() return values fall into four categories of interest:
   //
	//   1. AEE_NET_WOULDBLOCK: This means, data is not yet available on the socket. 
	//   2. AEE_NET_ERROR: An eror occurred
   //   3. 0 (zero):  No more data (the peer has closed the connection).
	//   3. >0 : The actual data that was read
   //
   
   if (rv == AEE_NET_WOULDBLOCK)
   {
      // WOULDBLOCK => there is no more data available at the moment
      //
      // In this case, register a callback function using ISOCKET_Readable() and then
      // return from this function.  The callback will be called by the system at
      // a later time.  It is perfectly OK to give the current function itself
      // as the callback.   The callback can then proceed to call ISOCKET_Read()
      // to obtain data (or perhaps an error, end of file, etc.).

      xStatus(pme, 4, "Waiting...");
      ISOCKET_Readable(piSock, Sockapp_ReadCB, (void*)pme);
	  return;
   } 
   else if (rv == AEE_NET_ERROR) 
   {
      // Connection hosed.
      xStatus(pme, 4, "Read Error!");
   }
   else if (rv > 0) // data has been read
   {
      SPRINTF(szBuf, "Read: %d", rv);
      xStatus(pme, 4, szBuf);

		// The data has been read from the socket. So, convert the data from
		// host network format to host format. 
		uTime = NTOHL(uTime) ;
		
		// Display the data.
		// This is the time elapsed in seconds since Jan 1 1900.
      SPRINTF(szBuf, "Time = %lu", (long) uTime);
      xStatus(pme, 5, szBuf);

      // If we wanted to read more data, we could loop here or call Readable() here
      // and return.
   }
   else  // rv == 0
   {
      // This demo doesn't expect to see the end of the stream...
      xStatus(pme, 4, "No Data!");
   }

   // Release interfaced used for network activity  
   Sockapp_DemoCleanup(pme);
}


/*===========================================================================

FUNCTION Sockapp_DemoCleanup()

DESCRIPTION Free any resources allocated as part of the demo.  This
   can be called before the app exits, or before the demo is repeated.

===========================================================================*/
static void Sockapp_DemoCleanup(CSocketApp *pme)
{
   if (pme->m_piSock) 
	{
      ISOCKET_Release(pme->m_piSock);
      pme->m_piSock = 0;
   }

   if (pme->m_piNet) 
	{
      INETMGR_Release(pme->m_piNet);
      pme->m_piNet = 0;
   }

   pme->demoInProgress = FALSE;
}


/*========================End of File ================================================*/


