/*
  ===========================================================================
  
  FILE:  webber.c
  
  AUTHOR: Rob Walker <rwalker@qualcomm.com>
  
  SERVICES:  
    IWeb API sample applet.
  
  GENERAL DESCRIPTION:
    This BREW Applet illustrates usage of BREW's IWeb Shows usage of 
    BREW's IWeb interface.  It's a simple URL fetcher/shower.
    
    The IWeb interface is designed to speed development of web-protocol
    aware applications.  IWeb handles many mundane chores of the 
    protocol part of a web browser, such as HTTP keep-alives and HTTP
    proxying.
  
  REVISION HISTORY: 
    Thu Jul 05 13:54:14 2001: Created

  ===========================================================================
  ===========================================================================
    
               Copyright  1999-2001 QUALCOMM Incorporated 
                     All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
    
  ===========================================================================
  ===========================================================================
*/

/*
  ===========================================================================
  ===========================================================================
  ======================= follows stuff for every ===========================
  ========================== BREW applet ====================================
  ===========================================================================
  ===========================================================================
*/

#include "AEE.h"       /* Standard AEE declarations */
#include "AEEShell.h"  /* AEE Shell services, IShell declarations */
#include "AEEStdlib.h" /* AEE StdLib decls */

/*
  ===========================================================================
     Forward declaration, see function implementation for comments...
  ===========================================================================
*/
static int xModule_New(IShell *piShell, IModule **pp);


#ifdef AEE_SIMULATOR
/*
  ===========================================================================
  ===========================================================================
    IMPORTANT NOTE: g_pvtAEEStdLibEntry global variable is defined for 
      the BREW SDK ONLY! This variable should NOT BE:
   
         (1) overwritten 
         (2) USED DIRECTLY by BREW applets (use the macros in AEEStdLib, 
             instead).
   
     g_pvtAEEStdLibEntry is used as an entry point to AEEStdLib
  ===========================================================================
  ===========================================================================
*/
AEEHelperFuncs *g_pvtAEEStdLibEntry;
#endif /* #ifdef AEE_SIMULATOR */


/**
  || Function
  || --------
  || int AEEMod_Load(IShell *piShell, void *ph, IModule **ppMod)
  ||
  || Description
  || -----------
  || This is the module load function. It is invoked when the module is being
  || loaded. It creates an instance of the AEEMod structure and initializes
  || the data members appropriately.
  || 
  || If running under the AEE Simulator, the function is prefixed with the tag
  || __declspec(dllexport) so that this function is exported by the App DLL.
  ||
  || Parameters
  || ----------
  || IShell *piShell: BREW shell interface pointer
  || void *ph: useful in AEE_SIMULATOR only, really, is a pointer to the 
  ||            AEEStdLib helper function vtable
  || ppMod: [out] where a successfully instantiated IModule pointer goes
  ||
  || Returns
  || -------
  || SUCCESS on success
  || ENOMEMORY if the IModule could not be instantiated
  ||
  || Remarks
  || -------
  || *****IMPORTANT****** this function must appear *first* in the *first*
  || applet source module, since that's the way BREW finds this entry point 
  || on a handset.
  || 
  || Putting the function at the top of the source module takes care of
  || it being first in this module.  The Applet makefile for the handset
  || takes care of this source module being first in the Applet DLL.
  ||
  || Most applets should not have this code, they should use AEEModGen.c
  ||
  */

#if defined(BREW_MODULE) || defined(FLAT_BREW)
#define AEEMOD_LOAD module_main
#else
#define AEEMOD_LOAD AEEMod_Load
#endif

#ifdef AEE_SIMULATOR
__declspec(dllexport) /* AEEMod_Load has to be exported from a DLL */
#endif /* #ifdef AEE_SIMULATOR */

int AEEMOD_LOAD(IShell *piShell, void *ph, IModule **ppMod)
{

#ifdef AEE_SIMULATOR
/*
  ===========================================================================
  ===========================================================================
    IMPORTANT NOTE: g_pvtAEEStdLibEntry global variable is defined for 
      the BREW SDK ONLY! This variable should NOT BE:
   
         (1) overwritten 
         (2) USED DIRECTLY by BREW applets (use the macros in AEEStdLib, 
             instead).
   
     g_pvtAEEStdLibEntry is used as an entry point to AEEStdLib
  ===========================================================================
  ===========================================================================
*/
   g_pvtAEEStdLibEntry = (AEEHelperFuncs *)ph;
#else  /* #ifdef AEE_SIMULATOR */
   (void)ph; /* unused */
#endif /* #ifdef AEE_SIMULATOR */

   /* create and return a new module */
   return xModule_New(piShell,ppMod);
}

/*
  ===========================================================================
  ===========================================================================
  ======================== end stuff for every ==============================
  ========================== BREW applet ====================================
  ===========================================================================
  ===========================================================================
*/

/*
  ===========================================================================
  ===========================================================================
  ======================== begin Webber stuff ===============================
  ===========================================================================
  ===========================================================================
*/


#include "AEEClassIDs.h"  /* for creating IText, etc */
#include "AEEText.h"  /* AEE Text control */
#include "AEEWeb.h"   /* AEE Web interface */

/*
  || webber resource header
 */
#include "webber_res.h"

/* 
   || webber module unofficial ClassID, kinda spells "webber"
 */
#include "webber.bid"


/*
  || command define, kinda spells "web go"
 */
#define IDC_WEBGO              (0x0000eb60)


/**
  || Function
  || --------
  || static void ReleaseIf(IBase **ppif)
  ||
  || Description
  || -----------
  || releases the interface pointer, sets it to null
  ||
  || Parameters
  || ----------
  || IBase **ppif: interfacee pointer
  ||
  || Returns
  || -------
  || nothing
  ||
  || Remarks
  || -------
  ||
  */
static void ReleaseIf(IBase **ppif)
{
   if ((IBase *)0 != *ppif) {
      IBASE_Release(*ppif);
      *ppif = (IBase *)0;
   }
}


/*
  ===========================================================================
  ===========================================================================
   Textwin stuff, to be moved out of here soonly....
  ===========================================================================
  ===========================================================================
*/

#define TW_LINES    200
#define TW_COLS     22
#define TW_CHAR     char

/* Convert any difference/sum to a non-negative modulo (0...TW_LINES-1) */
#define TW_MODU(val)   (((val) + TW_LINES*2) % TW_LINES)

/* Convert any difference/sum to a signed modulo (-TW_LINES/2 ... TW_LINES/2-1) */
#define TW_MODS(val)   ((((val) + TW_LINES + TW_LINES/2) % TW_LINES) - TW_LINES/2)

/*
 || Convert between buffer positions and screen positions
 ||    Note that BUFTOWIN returns values in the range 0...TW_LINES => 
 ||        possibly off screen
 || 
 ||    The "+TW_LINES" is to avoid modulo of negative number (undefined in C)
*/
#define TW_BUFTOWIN(me, nline)     TW_MODU( nline - (me)->nTopLine )
#define TW_WINTOBUF(me, nline)     TW_MODU( nline + (me)->nTopLine )

#define TW_LINENUM(curpos) ((curpos)/TW_COLS)
#define TW_COLNUM(curpos)  ((curpos)%TW_COLS)

typedef struct TextWin 
{
   IDisplay *piDisplay;  /* draw with */
   AEERect   rc;         /* draw within */
                          
   int       nShowLines; /* no. of lines on screen at once */
   int       nTopLine;   /* top line displayed on screen */
   int       nCurPos;    /* cursor line (buffer relative) */
   int       nScrollTop; /* "top" of allowed scroll range (buffer-rel) */
   int       nScrollBtm; /* "bottom" of allowed scroll range (buffer-rel) */
           
   TW_CHAR   achLines[TW_LINES * TW_COLS]; /* char buffer */

} TextWin;

#define LINEHEIGHT 13  /* this really only works for Kyocera 3035..... */

static void TextWin_Clear(TextWin *me)
{
   MEMSET(me->achLines,0,sizeof(me->achLines));

   me->nCurPos = 0;
   me->nTopLine = 0;
   me->nScrollTop = 0;
   me->nScrollBtm = 0;

   IDISPLAY_FillRect(me->piDisplay, &me->rc, RGB_WHITE);
   IDISPLAY_Update(me->piDisplay);
}

static size_t TextWin_Sizeof(void)
{
   return sizeof(TextWin);
}

static void TextWin_CtorZ(TextWin *me, IDisplay *piDisplay)
{
   me->piDisplay = piDisplay;
   IDISPLAY_AddRef(piDisplay);
}

static void TextWin_SetRect(TextWin *me, AEERect *prc)
{
   me->rc = *prc;
   me->nShowLines = (me->rc.dy) / LINEHEIGHT;
}

static void TextWin_Dtor(TextWin *me)
{
   ReleaseIf((IBase **)&me->piDisplay);
}

static void IDisplay_DrawSz(IDisplay *me,AEEFont fnt, const char * pszText, 
                            int nChars /*ignored*/,
                            int x,int y,const AEERect * prcBackground,
                            uint32 dwFlags)
{
   int nLen = 0;
   AECHAR wszBuf[100];

   while (nLen <= ARRAY_SIZE(wszBuf) && pszText[nLen]) {
      wszBuf[nLen] = pszText[nLen];
      ++nLen;
   }

   IDISPLAY_DrawText(me, fnt, wszBuf, nLen, x, y, prcBackground, dwFlags);
}

static void TextWin_DrawLine(TextWin *me, int nLine)
{
   AEERect rcBG;
   char *psz = me->achLines + (TW_WINTOBUF(me, nLine) * TW_COLS);
   
   rcBG.x = me->rc.x;
   rcBG.dx = me->rc.dx;

   rcBG.y = (nLine * LINEHEIGHT) + me->rc.y;
   rcBG.dy = LINEHEIGHT;

   IDisplay_DrawSz(me->piDisplay, AEE_FONT_NORMAL, psz, -1, rcBG.x, rcBG.y, 
                   &rcBG, IDF_RECT_FILL);
}

/*
  ||  Redraw line(s)
  || 
  ||   nLine==-1  => redraw all
*/
static void TextWin_Draw(TextWin *me, int nLine)
{
   int nMax;

   if (nLine < 0) {
      nLine = 0;
      nMax = me->nShowLines;
   } else {
      nMax = nLine + 1;
   }

   for ( ; nLine < nMax; ++nLine) {
      TextWin_DrawLine(me, nLine);
      /* 
         bug catcher
         IDISPLAY_Update(me->piDisplay); 
      */
   }

   { /* fill in bottom */
      AEERect rc = me->rc;
      rc.y = (nLine * LINEHEIGHT) + me->rc.y;
      rc.dy = me->rc.dy - (rc.y - me->rc.y);
      IDISPLAY_FillRect(me->piDisplay, &rc, RGB_WHITE);
   }

   IDISPLAY_Update(me->piDisplay);
}


/*
  || Write rest of, or entire line at curspr 
*/
static void TextWin_Line(TextWin *me, const char *pcASCII, int cb)
{
   boolean bForceLF;
   int     nCurLine = TW_LINENUM(me->nCurPos);
   int     nOnScreen = TW_BUFTOWIN(me, nCurLine);

   if (pcASCII[cb - 1] == '\n') {
      cb--;
      bForceLF = TRUE;
   } else {
      bForceLF = FALSE;
   }
   
   /* copy text into buffer, natural-like */
   MEMMOVE(me->achLines + me->nCurPos, pcASCII, cb);
   me->nCurPos += cb;
   me->achLines[me->nCurPos] = 0;

   /*  advance to next line ? */
   if (bForceLF || (TW_COLNUM(me->nCurPos) >= TW_COLS-1)) {

      /* push scroll bottom down if we've written below the screen it 
         described */
      if (nCurLine == me->nScrollBtm + me->nShowLines) {
         if (++me->nScrollBtm + me->nShowLines == TW_LINES) {
            me->nScrollBtm -= TW_LINES;
         }
      }

      me->nCurPos = (++nCurLine) * TW_COLS;
      
      if (nCurLine >= TW_LINES) {
         nCurLine = 0;
         me->nCurPos = 0;
      }

      if (bForceLF) {
         me->achLines[me->nCurPos] = 0;
      }

      /* push scroll region down if we've just overwritten it */
      if (nCurLine == me->nScrollTop) {
         if (++me->nScrollTop == TW_LINES) {
            me->nScrollTop = 0;
         }

         if (++me->nScrollBtm + me->nShowLines == TW_LINES) {
            me->nScrollBtm -= TW_LINES;
         }
      }
   }

   /* expose last line */

   if (nOnScreen >= me->nShowLines) {
      me->nTopLine = TW_WINTOBUF(me, nOnScreen - me->nShowLines + 1);
      nOnScreen = -1;  /*  redraw all */
   }

   TextWin_Draw(me, nOnScreen);
}

static void TextWin_Append(TextWin *me, const char *pszASCII)
{
   int len = STRLEN(pszASCII);

   while (0 != len) {
      char *pc;
      int cb = len;

      if (TW_COLNUM(me->nCurPos) + cb >= TW_COLS) {
         cb = TW_COLS - TW_COLNUM(me->nCurPos) - 1;
      } else {
         cb = len;
      }

      if ((char *)0 != (pc = MEMCHR(pszASCII,'\n',cb))) {
         cb = pc - pszASCII + 1;
      }
      
      TextWin_Line(me, pszASCII, cb);

      pszASCII += cb;
      len -= cb;
   }
}

static void TextWin_Scroll(TextWin *me, int nDown)
{
   /* limit to scroll top */
   int n = TW_MODU(me->nTopLine - me->nScrollTop);

   if (nDown < -n) {
      nDown = -n;
   }

   /* limit to bottom */
   n = TW_MODU(me->nScrollBtm - me->nTopLine);
   if (nDown > n) {
      nDown = n;
   }

   me->nTopLine += nDown;

   TextWin_Draw(me, -1);
}




/*
  ===========================================================================
  ===========================================================================
    Webber stuff
  ===========================================================================
  ===========================================================================
*/

/*
  ===========================================================================
    defines, macros
  ===========================================================================
*/
#define WEBBER_USERAGENT  "Webber/0.9 (built on "__DATE__")\r\n"

/**
  || Defined
  || -------
  || INITURLBUF
  ||
  || Description
  || -----------
  || initial contents of a URL history buffer
  ||
  || Remarks
  || -------
  || for use with Webber, this must be defined with a NULL beginning and 
  ||   *double* NULL end
  || 
  || the FTP URLs in this list won't work unless you configure and deploy
  ||   an FTP web proxy
  ||
  || some of the IP-only URLs may become invalid because IP addresses 
  ||   change, some of the content may also become invalid.  Write your 
  ||   own tests for some of these.
  ||
  || the URLs included are:
  ||
  ||  http://63.194.129.162/hi.html - (www.rwalker.com) no DNS lookup,
  ||                                   does Keep-Alive, short document
  ||
  ||  http://204.71.200.68/ - (one of www.yahoo.com) no DNS, no Keep-Alive
  ||
  ||  http://www.rwalker.com/cgi-bin/hi.sh - long, slow document
  ||
  ||  http://www.x.com/ - just x.com
  ||
  ||  http://207.111.197.2,63.194.129.162/hi.html - failover test, 
  ||                        207.111.197.2 is a dead IP address, the second in
  ||                        the list is www.rwalker.com
  || 
  ||  http://www.rwalker.com/ - just www.rwalker.com
  || 
  ||  http://www.rwalker.com/cgi-bin/snoop.sh\001hi%20there - tests POST with 
  ||                       Keep-Alives (returns a buncha info from an 
  ||                       identification cgi, including user-agent). 
  ||                       The byte '\001' denotes separation between URL and
  ||                       POST data. POST data is already URL-encoded..
  ||
  ||  http://www.qualcomm.com/ - no Keep-Alive, is a server that requires
  ||                           User-Agent, tests User-Agent, default User-Agent
  ||
  ||  ftp://ftp.cdrom.com
  ||  ftp://rs.internic.net/domain/named.ca - FTP over HTTP proxy tests
  ||
  ||  http://www.rwalker.com/cgi-bin/fortune.sh?Barry - long funny document, 
  ||                          tests sense of humor
  ||
  */
#define INITURLBUF "\000"\
                   "http://63.194.129.162/hi.html\000"\
                   "http://204.71.200.68/\000"\
                   "http://www.rwalker.com/cgi-bin/hi.sh\000"\
                   "http://www.x.com/\000"\
                   "http://207.111.197.2,63.194.129.162/hi.html\000"\
                   "http://www.rwalker.com/\000"\
                   "http://www.rwalker.com/cgi-bin/snoop.sh\001hi%20there\000"\
                   "http://www.qualcomm.com/\000"\
                   "ftp://ftp.cdrom.com\000"\
                   "ftp://rs.internic.net/domain/named.ca\000"\
                   "http://www.rwalker.com/cgi-bin/fortune.sh?Barry\000"



/*
  ===========================================================================
    types
  ===========================================================================
*/
typedef struct Webber Webber;

/*
  A WebAction holds state necessary to perform 1 IWeb transaction
 */
typedef struct WebAction
{
   Webber      *pwbr; /* my parent */
   AEECallback  cb;   /* how to call me back */
              
   IWebResp    *piWResp;   /* the answer I get from IWeb */
   IGetLine    *piGetLine; /* the body of the response, if any */
   int          nLines;    /* count of lines in response */
   int          nBytes;    /* count of bytes in response */

   uint32       uStart;
   uint32       uRecvStart;

   IPeek       *pipPostData; /* POST stream, if any */
   char        *pszPostData; /* POST string */

} WebAction;

/*
  A Webber is the Applet implementation, holds everything
 */
struct Webber
{
   IApplet      ia;     /* my vtable, must be first so that casts to Webber
                           succeed */
   unsigned     uRef;   /* refcount */
               
   IModule     *pim;    /* my parent the IModule  must be AddRef'd, Released,
                           is how BREW keeps track of when a Module is in 
                           use */
   
   IWeb        *piWeb;   /* IWeb interface, my raison d'tre */
   IShell      *piShell; /* IShell interface, used for CreateInstance of 
                            service classes, mostly */
               
   AEERect      rc;      /* screen rectangle */

   WebAction    awa[1];  /* array of WebActions.  By changing the '1' to any 
                            number, you can attempt to kick off multiple Web 
                            transactions simultaneously, testing Keep-Alive,
                            multiple-POST Keep-Alives, NOWAITCONN, FORCENEW, 
                            and request queueing.
                            
                            All output goes to the main TextWin 
                            simultaneously */

   IDisplay    *piDisplay; /* where I draw */
   ITextCtl    *piText;    /* the text control used to get the URL */
   IMenuCtl    *piSoftKey; /* softkey menu, created for me and the Text 
                              control */
   
   boolean      bDump;     /* whether to print to the TextWin */
   TextWin     *ptw;       /* TextWin, manages scrolling text output */
   
   int          nMode;     /* mode of operation, one of the below, used
                              to help event dispatch */

#define WBRM_INPUT   0     /* in text input mode, piSoftKey and piText get
                              first crack at events */
#define WBRM_TEXTWIN 1     /* in scrolling text mode, I do all events */

   /* overkill, is room for history, plus above */
   AECHAR       wszzUrlBuf[sizeof(INITURLBUF)+512]; /* URL buffer, oversized, 
                                                       display directly from 
                                                       it.. */
   AECHAR      *pwszUrl;        /* current browsed URL, is pointer into 
                                   wszzUrlBuf, may be compressed (suitable for
                                   IWeb) */
   boolean      bUrlCompressed; /* keep track of whether I've modded UrlBuf */ 

};



/*
  || forward decls (vtable parts)
*/
static uint32 Webber_AddRef(IApplet *pif);
static uint32 Webber_Release(IApplet *pif);
static boolean Webber_HandleEvent(IApplet *pif, AEEEvent eCode, uint16 wParam, uint32 dwParam);


/**
  || Function
  || --------
  || static void str2wstr(const char *pSrc, int nlen, AECHAR *pDest)
  ||
  || Description
  || -----------
  || hokem function to convert single-char string to double-wide chars
  ||
  || Parameters
  || ----------
  || const char *pSrc: chars to expand
  || int nlen: number of pSrc bytes to expand
  || AECHAR *pDest: where to put 'em, must be at least nlen * sizeof(AECHAR) 
  ||                bytes in size
  ||
  || Returns
  || -------
  || nothing
  ||
  || Remarks
  || -------
  || doesn't do conversion, just expansion..
  ||
  */
static void str2wstr(const char *pSrc, int nlen, AECHAR *pDest)
{
   MEMMOVE(((char *)pDest) + nlen, pSrc, nlen); 
   /* might be a noop, move to end  */
   
   pSrc = ((char *)pDest) + nlen;

   while (nlen > 0) {
      *pDest++ = (AECHAR)(unsigned char)*pSrc++;
      nlen--;
   }
}

/**
  || Function
  || --------
  || static void wstr2str(const AECHAR *pSrc, int nlen, char *pDest)
  ||
  || Description
  || -----------
  || hokem function to convert double-wide chars to single chars 
  ||
  || Parameters
  || ----------
  || const char *pSrc: chars to expand
  || int nlen: number of pSrc AECHARs to convert
  || AECHAR *pDest: where to put 'em, must be at least nlen bytes in size
  ||
  || Returns
  || -------
  || nothing
  ||
  || Remarks
  || -------
  || doesn't do real conversion, just takes low byte 
  ||
  */
static void wstr2str(const AECHAR *pSrc, int nlen, char *pDest)
{
   /* shrinking, so no need to move */
   while (nlen > 0) {
      *pDest++ = (char)*pSrc++;
      nlen--;
   }
}


/**
  || Function
  || --------
  || static void Webber_CtorZ(Webber *me, IModule *pim, IDisplay *piDisplay,
  ||                          TextWin *ptw, ITextCtl *piText,
  ||                          IMenuCtl *piSoftKey,
  ||                          IShell *piShell, IWeb *piWeb)
  ||
  || Description
  || -----------
  || initializes a Webber object into zero-initialized memory
  ||
  || Parameters
  || ----------
  || Webber *me: new memory
  || IModule *pim: my owner, must be AddRef'd
  || IDisplay *piDisplay: where to draw
  || TextWin *ptw: new TextWin
  || ITextCtl *piText: new IText
  || IMenuCtl *piSoftKey: new softkey 
  || IShell *piShell: BREW Shell interface pointer
  || IWeb *piWeb: BREW Web interface pointer
  ||
  || Returns
  || -------
  || nothing, can't fail..
  ||
  || Remarks
  || -------
  ||
  */
static void Webber_CtorZ(Webber *me, IModule *pim, IDisplay *piDisplay,
                         TextWin *ptw, ITextCtl *piText, IMenuCtl *piSoftKey,
                         IShell *piShell, IWeb *piWeb)
{
   me->ia.pvt = (IAppletVtbl *)(me + 1);

   /* Initialize individual entries in the VTBL */
   me->ia.pvt->AddRef      = Webber_AddRef;
   me->ia.pvt->Release     = Webber_Release;
   me->ia.pvt->HandleEvent = Webber_HandleEvent;

   me->uRef = 1;

   /* take my module, must AddRef becuase BREW keeps track of Module usage
      with the IModule's refcount */
   me->pim = pim;
   IMODULE_AddRef(pim);

   /* take shell */
   me->piShell = piShell;
   ISHELL_AddRef(piShell);

   /* take display, we'll be black on white */
   me->piDisplay = piDisplay;
   IDISPLAY_AddRef(piDisplay);
   IDISPLAY_SetColor(me->piDisplay, CLR_USER_BACKGROUND, RGB_WHITE);

   /* take the web */
   me->piWeb = piWeb;
   IWEB_AddRef(piWeb);

   /* initialize the IWeb with a few options.. 
      
      unfortunately, the ARM compiler can't do multiple initializations 
      here, we have to do them by hand... */
   {
      WebOpt awo[10]; /* ***IMPORTANT**** grow this if you add more 
                         WebOpts here, or shrink it and call AddOpt() 
                         multiple times */
      int    i = 0;


      /* set the IWeb connect timeout to 10 seconds.  this also sets the 
         failover timeout, if unset, or set to 0, IWeb uses the system 
         default (30 seconds unless an OEM changes it) */
      awo[i].nId  = WEBOPT_CONNECTTIMEOUT;
      awo[i].pVal = (void *)10000;
      i++;

      /* test user-agent, uncomment this section to ship your own user-agent 
         string. if unset, IWeb will send a default.  If set to NULL, no 
         user agent header will be sent */

#if 0
      awo[i].nId  = WEBOPT_USERAGENT;
      awo[i].pVal = (void *)WEBBER_USERAGENT;
      i++;
#endif /* #if 0 */

      /* test nowaitconn, this only comes into effect if you build webber 
         with multiple WebActions (see the definition of struct Webber) */
      awo[i].nId  = WEBOPT_FLAGS;
      awo[i].pVal = (void *)WEBREQUEST_NOWAITCONN;
      i++;
      

      /* test forcenew, uncomment this section to try multiple simultaneous
         "forced" new connections. Forced new connections are not kept alive
         unless they are the first forced new connection to a host */
#if 0
      awo[i].nId  = WEBOPT_FLAGS;
      awo[i].pVal = (void *)WEBREQUEST_FORCENEWCONN;
      i++;
#endif /* #if 0 */


      /* turn off HTTP over HTTP proxying */
      awo[i].nId  = WEBOPT_PROXYSPEC;
      awo[i].pVal = (void *)"http:///";
      i++;

      /* turn on ALL proxying.  Proxyspecs are examined in WebOpt order, so
         in this list, with the above and below PROXYSPECs, everything
         except HTTP will be proxied through 
         http://webproxy.yourdomain.com:8080, (which you'll have to set up to 
         test, sorry....) */
      awo[i].nId  = WEBOPT_PROXYSPEC;
      awo[i].pVal = (void *)"*:///http://webproxy.yourdomain.com:8080";
      i++;

      /* this is absolutely necessary, do not remove, marks the end of the 
         array of WebOpts */
      awo[i].nId  = WEBOPT_END;
      
      /* add 'em */
      IWEB_AddOpt(me->piWeb,awo);
   }
   
   /* copy the INITURL list into my history buffer, expanding it for display */
   str2wstr(INITURLBUF, sizeof(INITURLBUF), me->wszzUrlBuf);

   /* first url */
   me->pwszUrl = me->wszzUrlBuf + 1;

   /* initialize all my WebActions to point to me */
   {
      int i;
      for (i = 0; i < ARRAY_SIZE(me->awa); i++) {
         me->awa[i].pwbr = me;
      }
   }
   
   /* take my TextWin */
   me->ptw = ptw; /* static mem */
   
   /* get screen size, tuck away */
   {
      AEEDeviceInfo di;

      ZEROAT(&di);

      di.wStructSize = sizeof(di);
      
      ISHELL_GetDeviceInfo(me->piShell, &di);
      me->rc.dx = di.cxScreen;
      me->rc.dy = di.cyScreen;
   }
   /* tell TextWin how big to be */
   TextWin_SetRect(me->ptw,&me->rc);

   /* take the Text Control */
   me->piText = piText;
   ITEXTCTL_AddRef(piText);


   /* take the Menu */
   me->piSoftKey = piSoftKey;
   IMENUCTL_AddRef(piSoftKey);

   /* add the 'Go' button */
   IMENUCTL_AddItem(me->piSoftKey, WEBBER_RES_FILE, IDS_GO,IDC_WEBGO, 0, 0);


   /* initialize the text control... set the text title */
   ITEXTCTL_SetTitle(me->piText, WEBBER_RES_FILE, IDS_URLTITLE, 0);

   /* tell the Text control where to draw */
   {
      AEERect rc;

      IMENUCTL_GetRect(me->piSoftKey,&rc);
      
      rc.dy = rc.y; /* softkey menu's at the bottom */
      rc.x  = me->rc.x;
      rc.y  = me->rc.y;
      rc.dx = me->rc.dx;

      ITEXTCTL_SetRect(me->piText,&rc);
   }
   ITEXTCTL_SetProperties(me->piText,
                          ITEXTCTL_GetProperties(me->piText) | 
                          TP_FRAME | TP_MULTILINE);
   ITEXTCTL_SetInputMode(me->piText, AEE_TM_LETTERS);

   /* don't let the Text Control overflow my history */
   ITEXTCTL_SetMaxSize(me->piText,(uint16)(ARRAY_SIZE(me->wszzUrlBuf) - 3));
   /* room for 3 \0 */

   /* give the menu to the text control (it adds a menu item for mode 
      switching) */
   ITEXTCTL_SetSoftKeyMenu(me->piText,me->piSoftKey);
}

/**
  || Function
  || --------
  || static void Webber_Dtor(Webber *me)
  ||
  || Description
  || -----------
  || fries contents of Webber, doesn't free memory
  ||
  || Parameters
  || ----------
  || Webber *me: the Webber
  ||
  || Returns
  || -------
  || nothing
  ||
  || Remarks
  || -------
  ||
  */
static void Webber_Dtor(Webber *me)
{
   ReleaseIf((IBase **)&me->piDisplay);
   ReleaseIf((IBase **)&me->piSoftKey);
   ReleaseIf((IBase **)&me->piText);
   ReleaseIf((IBase **)&me->piWeb);
   ReleaseIf((IBase **)&me->piShell);

   ReleaseIf((IBase **)&me->pim);
}

/**
  || Function
  || --------
  || static void Webber_Delete(Webber *me)
  ||
  || Description
  || -----------
  || deletes a Webber
  ||
  || Parameters
  || ----------
  || Webber *me: the Webber
  ||
  || Returns
  || -------
  || nothing
  ||
  || Remarks
  || -------
  ||
  */
static void Webber_Delete(Webber *me)
{
   Webber_Dtor(me);
   TextWin_Dtor(me->ptw);
   FREE(me);
}

/**
  || Function
  || --------
  || static int Webber_New(IModule *pim, IShell *piShell, IApplet **pp)
  ||
  || Description
  || -----------
  || makes a new Webber, the BREW applet class for this Module
  ||
  || Parameters
  || ----------
  || IModule *pim: my owner
  || IShell *piShell: the BREW shell
  || IApplet **pp: [out] where to stuff the new Applet (the new Webber)
  ||
  || Returns
  || -------
  || ENOMEMORY if allocation fails
  || EPRIVLEVEL if AEECLSID_WEB could not be CreateInstance()d
  ||
  || Remarks
  || -------
  ||
  */
static int Webber_New(IModule *pim, IShell *piShell, IApplet **pp)
{
   Webber   *me = 0;
   IDisplay *piDisplay = 0;
   IWeb     *piWeb = 0;
   ITextCtl *piText = 0;
   IMenuCtl *piSoftKey = 0;
   int       nErr;

   nErr = ISHELL_CreateInstance(piShell, AEECLSID_DISPLAY,(void **)&piDisplay);
   
   if (SUCCESS == nErr) {
      nErr = ISHELL_CreateInstance(piShell, AEECLSID_TEXTCTL,(void **)&piText);
   }

   if (SUCCESS == nErr) {
      nErr = ISHELL_CreateInstance(piShell, AEECLSID_SOFTKEYCTL,
                                   (void **)&piSoftKey);
      if (SUCCESS != nErr) {
         nErr = ISHELL_CreateInstance(piShell, AEECLSID_SOFTKEYCTL_10,
                                      (void **)&piSoftKey);
      }
   }

   if (SUCCESS == nErr) {
      nErr = ISHELL_CreateInstance(piShell, AEECLSID_WEB,(void **)&piWeb);
   }

   if (SUCCESS == nErr) {
      me = MALLOCREC_EX(Webber, sizeof(IAppletVtbl) + TextWin_Sizeof());
      if ((Webber *)0 == me) {
         nErr = ENOMEMORY;
      }
   }

   if (SUCCESS == nErr) {
      TextWin *ptw = (TextWin *)(((byte *)me) + sizeof(Webber) + 
                                 sizeof(IAppletVtbl));
      
      TextWin_CtorZ(ptw,piDisplay);
      Webber_CtorZ(me, pim, piDisplay, ptw, piText, piSoftKey, piShell, piWeb);
   } 
   
   ReleaseIf((IBase **)&piDisplay);
   ReleaseIf((IBase **)&piSoftKey);
   ReleaseIf((IBase **)&piText);
   ReleaseIf((IBase **)&piWeb);
   
   *pp = &me->ia;
   return nErr;
}


/**
  || Function
  || --------
  || static uint32 Webber_AddRef(IApplet *p)
  || static uint32 Webber_Release(IApplet *p)
  ||
  || Description
  || -----------
  || increment/decrement refcount on a Webber
  ||
  || Parameters
  || ----------
  || IApplet *p: a Webber
  ||
  || Returns
  || -------
  || new refcount
  ||
  || Remarks
  || -------
  || if refcount drops to 0, the Webber is deleted
  ||
  */
static uint32 Webber_AddRef(IApplet *p)
{
   Webber *me = (Webber *)p;
   return ++me->uRef;
}

static uint32 Webber_Release(IApplet *p)
{
   Webber *me = (Webber *)p;
   uint32 uRef = --me->uRef;

   if (uRef == 0) {
      Webber_Delete(me);
   }
   return uRef;
}


/**
  || Function
  || --------
  || static void WebAction_Header(void *p, const char *cpszName, 
  ||                              GetLine *pglVal)
  ||
  || Description
  || -----------
  || received header callback for a web transaction
  ||
  || Parameters
  || ----------
  || void *p: a WebAction (the subscriber)
  || const char *cpszName: the name of the web header (like "Content-Type")
  || GetLine *pglVal: the value of the header, like "text/html"
  ||
  || Returns
  || -------
  || nothing
  ||
  || Remarks
  || -------
  || cpszName is NULL in the case of continuation header line parts..
  ||
  */
static void WebAction_Header(void *p, const char *cpszName, GetLine *pglVal)
{
   WebAction *me = (WebAction *)p;

   
   if (me->pwbr->bDump) { /* if I'm dumping to screen...*/
      TextWin *ptw = me->pwbr->ptw;

      if ((char *)0 != cpszName) {
         TextWin_Append(ptw, cpszName);
         TextWin_Append(ptw, ":");
      }
      TextWin_Append(ptw, pglVal->psz);
      TextWin_Append(ptw, "\n");
   }
}

/**
  || Function
  || --------
  || static void WebAction_Status(void *p, WebStatus ws, void *pVal)
  ||
  || Description
  || -----------
  || web status callback for a Web transaction 
  ||
  || Parameters
  || ----------
  || void *p: a WebAction (the subscriber)
  || WebStatus ws: type of status
  || void *pVal: unused as of yet
  ||
  || Returns
  || -------
  || nothing
  ||
  || Remarks
  || -------
  ||
  */
static void WebAction_Status(void *p, WebStatus ws, void *pVal)
{
   char *pszStatus = 0;
   WebAction *me = (WebAction *)p;
   
   (void)pVal;

   switch (ws) {
   case WEBS_CANCELLED:
      pszStatus = "** cancelled...\n";
      break;
   case WEBS_GETHOSTBYNAME:
      pszStatus = "** finding host...\n";
      break;          
   case WEBS_CONNECT:
      pszStatus = "** connecting...\n";
      break;
   case WEBS_SENDREQUEST:
      pszStatus = "** sending...\n";
      break;
   case WEBS_READRESPONSE:
      pszStatus = "** receiving...\n";
      break;
   case WEBS_GOTREDIRECT:
      pszStatus = "** redirect...\n";
      break;
   case WEBS_CACHEHIT:
      pszStatus = "** cache hit...\n";
      break;
   }

   /* show that status! */
   if ((char *)0 != pszStatus) {
      TextWin_Append(me->pwbr->ptw, pszStatus);
   }
}

/**
  || Function
  || --------
  || static void WebAction_Stop(WebAction *me)
  ||
  || Description
  || -----------
  || halts a web transaction, wrapped/represented by a WebAction
  ||
  || Parameters
  || ----------
  || WebAction *me: the WebAction
  ||
  || Returns
  || -------
  || nothing
  ||
  || Remarks
  || -------
  ||
  */
static void WebAction_Stop(WebAction *me)
{
   /* this cancels any pending web transaction, or readable on the 
      response body.  if nothing is pending, this has no effect */
   CALLBACK_Cancel(&me->cb);
   
   /* then clean up, if necessary */
   if ((char *)0 != me->pszPostData) {
      *me->pszPostData = 1; /* write delimiter back in, if any */
      me->pszPostData = 0;
   }

   ReleaseIf((IBase **)&me->pipPostData);

   ReleaseIf((IBase **)&me->piGetLine); /* let body go */
   ReleaseIf((IBase **)&me->piWResp); /* let response go */

}

/**
  || Function
  || --------
  || static void Webber_Stop(Webber *me)
  ||
  || Description
  || -----------
  || stops any pending web transactions, uncompresses the URL just taken,
  ||   if necessary
  ||
  || Parameters
  || ----------
  || Webber *me: the Webber Applet
  ||
  || Returns
  || -------
  || nothing
  ||
  || Remarks
  || -------
  ||
  */
static void Webber_Stop(Webber *me)
{
   int i;
   for (i = 0; i < ARRAY_SIZE(me->awa); i++) {
      WebAction_Stop(&me->awa[i]);
   }

   /* this'll work, cuz the string was compressed inplace ... */
   if (me->bUrlCompressed) { /* NOT idempotent!! */
      str2wstr((const char *)me->pwszUrl,STRLEN((char *)me->pwszUrl) + 1, 
                 me->pwszUrl);
      me->bUrlCompressed = FALSE;
   }

}

/**
  || Function
  || --------
  || static void WebAction_ReadLines(void *p)
  ||
  || Description
  || -----------
  || consumes the body of a web response, line by line, is called when the
  ||  IGetLine that represents the body is readable
  ||
  || Parameters
  || ----------
  || void *p: the WebAction, is void * to meet callback signature requirements
  ||
  || Returns
  || -------
  || nothing
  ||
  || Remarks
  || -------
  ||
  */
static void WebAction_ReadLines(void *p)
{
   WebAction *me = (WebAction *)p;
   TextWin   *ptw = me->pwbr->ptw;
   int        rv;
   GetLine    gl;

   /* read a line */
   rv = IGETLINE_GetLine(me->piGetLine, &gl, IGETLINE_LF);
   
   /* if stream's not reached EOF */
   if (!IGETLINE_Exhausted(rv)) {

      /* and isn't stalled */
      if (rv != IGETLINE_WAIT) { 
         int nLastBytes = me->nBytes;

         me->nBytes += gl.nLen + IGETLINE_EOLSize(rv);
         
         if (!gl.bLeftover) {
            me->nLines += 1;
         }

         /* print data to screen? */
         if (me->pwbr->bDump) {
            TextWin_Append(ptw, gl.psz);
            TextWin_Append(ptw, "\n");
         } else {
            if (((me->nBytes & ~0x1ff) != (nLastBytes & ~0x1ff)) ||
                !nLastBytes) {
               TextWin_Append(ptw, "#");
            }
         }
         
      }

      /* wait for more data */
      CALLBACK_Init(&me->cb, WebAction_ReadLines, me);
      IGETLINE_Peekable(me->piGetLine, &me->cb);
      return;
   }

   /* else no more data */
   if (rv == IGETLINE_ERROR) {
      TextWin_Append(ptw, "\n** error\n");
   } else {
      TextWin_Append(ptw, "\n** end\n");
   }
   
   {
      char buf[96];
      uint32 uNow = GETUPTIMEMS();
      SPRINTF(buf, 
              "** %d bytes\n"
              "** %db/s\n"
              "** %dms et (recv)\n"
              "** %dms et (total)\n",
              me->nBytes, 
              me->nBytes * 1000 / MAX(1,uNow - me->uRecvStart),
              uNow - me->uRecvStart,
              uNow - me->uStart);
      TextWin_Append(ptw, buf);
   }

   /* am done, _Stop cleans up */
   WebAction_Stop(me);
}


/**
  || Function
  || --------
  || static void WebAction_GotResp(void *p)
  ||
  || Description
  || -----------
  || web transaction callback, fires when the response body is ready to
  ||  be read, or when an error has occurred
  ||
  || Parameters
  || ----------
  || void *p: the WebAction
  ||
  || Returns
  || -------
  || nothing
  ||
  || Remarks
  || -------
  ||
  */
static void WebAction_GotResp(void *p)
{
   WebAction *me = (WebAction *)p;
   TextWin   *ptw = me->pwbr->ptw;   
   WebRespInfo *pwri;
   char buf[32];

   SPRINTF(buf, "** got response...\n");
   TextWin_Append(ptw, buf);

   /* get information about how the web transaction went...
      me->piWResp is ***NEVER NULL***, even though the transaction may fail
      for wont of memory
   */
   pwri = IWEBRESP_GetInfo(me->piWResp);
   
   SPRINTF(buf, "** info code: %d\n", pwri->nCode);
   TextWin_Append(ptw, buf);
   
   /* body may be NULL */
   if ((ISource *)0 != pwri->pisMessage) {
      ISourceUtil *pisu;

      /* read the body by creating an IGetLine to help parse into lines
         
         This step is not actually necessary if you don't need line-by-line
         output.  In that case, use pisMessage directly.
       */
      ISHELL_CreateInstance(me->pwbr->piShell,AEECLSID_SOURCEUTIL,
                            (void **)&pisu);
      
      if ((ISourceUtil *)0 != pisu) {
         ISOURCEUTIL_GetLineFromSource(pisu, pwri->pisMessage, 1100, 
                                       &me->piGetLine);
         ISOURCEUTIL_Release(pisu);
      }
   }
   
   if ((IGetLine *)0 != me->piGetLine) {

      me->uRecvStart = GETUPTIMEMS(); /* record that I started reading */
      
      /* got an IGetLine, go ahead and read it */
      me->nBytes = 0;
      WebAction_ReadLines(me);

   } else {
      /* ug, tranaction failed or couldn't get an IGetLine, oh well, bail */
      TextWin_Append(ptw, "** no response\n");
      WebAction_Stop(me); /* stop cleans up */
   }
   
}

/**
  || Function
  || --------
  || static void WebAction_Start(WebAction *me, char *pszUrl)
  ||
  || Description
  || -----------
  || kicks off a web transaction
  ||
  || Parameters
  || ----------
  || WebAction *me: wrapper for the web transaction
  || char *pszUrl: the URL to go GET/POST
  ||
  || Returns
  || -------
  || nothing
  ||
  || Remarks
  || -------
  ||
  */
static void WebAction_Start(WebAction *me, char *pszUrl)
{
   ISourceUtil *pisu;

   /* look to see if there's POST data, this is totally non-standard, but 
      easy to put into tests.. */
   me->pszPostData = STRCHR(pszUrl, 1);
   
   /* if there's post data, construct a stream for IWeb to consume */
   if ((char *)0 != me->pszPostData) {
      *me->pszPostData = 0;
      if (SUCCESS == 
          ISHELL_CreateInstance(me->pwbr->piShell,AEECLSID_SOURCEUTIL, 
                                (void **)&pisu)) {
         
         ISOURCEUTIL_PeekFromMemory(pisu, me->pszPostData + 1, 
                                    STRLEN(me->pszPostData + 1), 0, 0,
                                    &me->pipPostData);
         ISOURCEUTIL_Release(pisu);
      }
   }

   /* initialize the callback, where I'll be called when the request
      completes */
   CALLBACK_Init(&me->cb, WebAction_GotResp, me);
   
   me->uStart = GETUPTIMEMS();
   /* start transaction, pass callbacks for web status, web headers
      the extra WEBOPT_HEADER is used to help test what's sent 
      (snoop.sh above shows all the headers) */
   if ((IPeek *)0 != me->pipPostData) {
      IWEB_GetResponse(me->pwbr->piWeb,
                       (me->pwbr->piWeb, &me->piWResp, &me->cb, pszUrl, 
                        WEBOPT_HANDLERDATA, me,
                        WEBOPT_HEADER, "X-Method: POST\r\n", /* for kicks */
                        WEBOPT_HEADERHANDLER, WebAction_Header, 
                        WEBOPT_STATUSHANDLER, WebAction_Status, 
                        WEBOPT_METHOD, "POST",
                        WEBOPT_BODY, me->pipPostData,
                        WEBOPT_CONTENTLENGTH, STRLEN(me->pszPostData + 1),
                        WEBOPT_END));
   } else {
      IWEB_GetResponse(me->pwbr->piWeb,
                       (me->pwbr->piWeb, &me->piWResp, &me->cb, pszUrl, 
                        WEBOPT_HANDLERDATA, me, 
                        WEBOPT_HEADER, "X-Method: GET\r\n",
                        WEBOPT_HEADERHANDLER, WebAction_Header, 
                        WEBOPT_STATUSHANDLER, WebAction_Status, 
                        WEBOPT_END));
   }
}


/**
  || Function
  || --------
  || static void Webber_Start(Webber *me)
  ||
  || Description
  || -----------
  || kicks off all of a Webber's WebActions
  ||
  || Parameters
  || ----------
  || Webber *me: the Applet
  ||
  || Returns
  || -------
  || nothing
  ||
  || Remarks
  || -------
  ||
  */
static void Webber_Start(Webber *me)
{
   TextWin_Clear(me->ptw);

   TextWin_Append(me->ptw, "** starting...\n");
   
   if (!me->bUrlCompressed) { /* NOT idempotent!! */
      /* compress in place ... */
      wstr2str(me->pwszUrl,WSTRLEN(me->pwszUrl) + 1,(char *)me->pwszUrl);
      me->bUrlCompressed = TRUE;
   }

   /* pass URL to each WebAction, they'll all output to TextWin */
   {
      int i;
      for (i = 0; i < ARRAY_SIZE(me->awa); i++) {
         WebAction_Start(&me->awa[i],(char *)me->pwszUrl);
      }
   }
}         



/**
  || Function
  || --------
  || static void Webber_InputMode(Webber *me)
  ||
  || Description
  || -----------
  || converts applet to Text control input mode (for URL input)
  ||
  || Parameters
  || ----------
  || Webber *me: the Applet
  ||
  || Returns
  || -------
  || nothing
  || 
  || Remarks
  || -------
  ||
  */
static void Webber_InputMode(Webber *me)
{
   Webber_Stop(me);

   me->nMode = WBRM_INPUT;
   IMENUCTL_SetActive(me->piSoftKey, FALSE);
   ITEXTCTL_SetActive(me->piText,TRUE);
   ITEXTCTL_SetText(me->piText,me->pwszUrl,WSTRLEN(me->pwszUrl));

}

/**
  || Function
  || --------
  || static void Webber_Go(Webber *me)
  ||
  || Description
  || -----------
  || reads URL from Text control, updates history and kicks off web 
  ||  transactions
  ||
  || Parameters
  || ----------
  || Webber *me: the Webber
  ||
  || Returns
  || -------
  || nothing
  ||
  || Remarks
  || -------
  ||
  */
static void Webber_Go(Webber *me)
{
   AECHAR *pwszUrl;
   int     nUrlLen;

   /* switch app mode */
   me->nMode = WBRM_TEXTWIN;
   
   /* set the controls inactive */
   ITEXTCTL_SetActive(me->piText,FALSE);
   IMENUCTL_SetActive(me->piSoftKey,FALSE);
         
   /* read URL */
   pwszUrl = ITEXTCTL_GetTextPtr(me->piText);

   nUrlLen = WSTRLEN(pwszUrl); /* including NULL */
   
   /* ASSERT(nUrlLen <= URLBUFLEN - 3); */

   /* move history to end, stuff new URL in front */
   MEMMOVE(me->wszzUrlBuf + nUrlLen + 1, me->wszzUrlBuf, 
           (ARRAY_SIZE(me->wszzUrlBuf) - nUrlLen - 1) * sizeof(AECHAR));
   MEMMOVE(me->wszzUrlBuf + 1, pwszUrl, nUrlLen * sizeof(AECHAR));
   me->wszzUrlBuf[ARRAY_SIZE(me->wszzUrlBuf) - 1] = 
      me->wszzUrlBuf[ARRAY_SIZE(me->wszzUrlBuf) - 2] = 0;
   
   /* set pointer to URL I'm gonna read */
   me->pwszUrl = me->wszzUrlBuf + 1;

   /* kickoff */
   Webber_Start(me);
}


/**
  || Function
  || --------
  || static boolean Webber_HandleEvent(IApplet *p, AEEEvent eCode, 
  ||                                   uint16 wParam, uint32 dwParam)
  ||
  || Description
  || -----------
  || main event dispatcher
  ||
  || Parameters
  || ----------
  || IApplet *p: the Webber
  || AEEEvent eCode: event code
  || uint16 wParam: unnecessarily small 1st parameter
  || uint32 dwParam: 2nd parameter
  ||
  || Returns
  || -------
  || TRUE if the app handled the event
  || FALSE otherwise
  ||
  || Remarks
  || -------
  ||
  */
static boolean Webber_HandleEvent(IApplet *p, AEEEvent eCode, uint16 wParam,
                                  uint32 dwParam)
{
   Webber *me = (Webber *)p;

   switch (eCode) {
   case EVT_APP_START:
      Webber_InputMode(me);
      return TRUE;

   case EVT_APP_STOP:
      Webber_Stop(me);
      IDISPLAY_EraseRect(me->piDisplay,&me->rc);  /* Clear the display */
      IDISPLAY_UpdateEx(me->piDisplay,TRUE);
      return TRUE;

   case EVT_COMMAND:
      switch (wParam) {
      case IDC_WEBGO:
         /* got the "Go" button */
         Webber_Go(me);
         return TRUE;

      default:
         /* got something else, since the Text Control shares the 
            menu with me, let him have the event */
         return ITEXTCTL_HandleEvent(me->piText,eCode,wParam,dwParam);
      }

   case EVT_KEY:
      switch (me->nMode) {
      case WBRM_INPUT: /* input mode? */

         /* if Menu wants it, we're done */
         if (IMENUCTL_HandleEvent(me->piSoftKey,eCode,wParam,dwParam)) {
            return TRUE;
         }
         
         /* now... this is complicated...

            The Text control may have a sub menu active, in which case, 
            it won't be "active", so I can override UP and DOWN */
         if (ITEXTCTL_IsActive(me->piText) && 
             ((wParam == AVK_UP) || (wParam == AVK_DOWN))) { 

            /* intercept these, fill Text control with previous and
               next history entries.. */

            if (wParam == AVK_UP) {
               AECHAR *pwsz = me->pwszUrl + WSTRLEN(me->pwszUrl) + 1;
               if ((AECHAR)0 != *pwsz) {
                  me->pwszUrl = pwsz;
               }
            } else if (me->pwszUrl != me->wszzUrlBuf + 1) {/* AVK_UP */
               AECHAR *pwc = me->pwszUrl - 2;

               while ((AECHAR)0 != *pwc) {
                  pwc--;
               }
               me->pwszUrl = pwc + 1;
            }
            
            ITEXTCTL_SetText(me->piText,me->pwszUrl,WSTRLEN(me->pwszUrl));
            
            return TRUE;
         }

         /* otherwise if Text control handles it (including it's sub-menu,
            we're done */
         if (ITEXTCTL_HandleEvent(me->piText,eCode,wParam,dwParam)) {
            return TRUE;
         }
         
         /* so BACK is the same as UP, more crazy overrides...
            you can mostly ignore any dispatching weirdness, I like 
            the "clr" button to do the same as "up" when I'm in the 
            softkey menu... */
         if (IMENUCTL_IsActive(me->piSoftKey) && (wParam == AVK_CLR)) {
            ITEXTCTL_SetActive(me->piText,TRUE);
            IMENUCTL_SetActive(me->piSoftKey,FALSE);
            return TRUE;
         }
         break;

      case WBRM_TEXTWIN: /* web transaction mode */
         switch (wParam) {
         case AVK_POUND:
            me->bDump = !me->bDump;
            return TRUE;

         case AVK_SELECT:
            Webber_Stop(me); /* re-do transactions */
            Webber_Start(me);
            return TRUE;

         case AVK_CLR:
            Webber_InputMode(me); /* go back to URL input mode */
            return TRUE;

         case AVK_UP:
            TextWin_Scroll(me->ptw, -1); /* scroll textwin */
            return TRUE;

         case AVK_DOWN:
            TextWin_Scroll(me->ptw, 1);
            return TRUE;

         }
         break;
      }
      
      break;
   }

   return FALSE;
}



/*
  ===========================================================================
  ===========================================================================
    xModule stuff
  ===========================================================================
  ===========================================================================
*/

/*
  ===========================================================================
    types
  ===========================================================================
*/
typedef struct xModule {
   IModule        im;      /* inline struct, has vtable */
   unsigned       uRef;    /* ref count */
} xModule;


/*
  ===========================================================================
   forward decls (vtable parts), see impls for documentation 
  ===========================================================================
*/
static uint32 xModule_AddRef(IModule *p);
static uint32 xModule_Release(IModule *p);
static int    xModule_CreateInstance(IModule *p, IShell *piShell, 
                                     AEECLSID idClass, void **pp);
static void   xModule_FreeResources(IModule *p, IHeap *ph, IFileMgr *pfm);


/**
  || Function
  || --------
  || static void xModule_Delete(xModule *me)
  ||
  || Description
  || -----------
  || deletes an xModule
  ||
  || Parameters
  || ----------
  || xModule *me: interface implementer to delete
  ||
  || Returns
  || -------
  || nothing
  ||
  || Remarks
  || -------
  ||
  */
static void xModule_Delete(xModule *me)
{
   /* not enough work here to warrant a Dtor */
   FREE(me);
}

/**
  || Function
  || --------
  || static void xModule_CtorZ(xModule *me)
  ||
  || Description
  || -----------
  || set up a new xModule in zero-initialized memory, sets up vtable
  ||
  || Parameters
  || ----------
  || xModule *me: new memory to pour myself into, is over-allocated
  ||
  || Returns
  || -------
  || nothing
  ||
  || Remarks
  || -------
  ||
  */
static void xModule_CtorZ(xModule *me)
{
   me->im.pvt = (IModuleVtbl *)(me + 1);

   /* Initialize individual entries in the VTBL */
   me->im.pvt->AddRef         = xModule_AddRef;
   me->im.pvt->Release        = xModule_Release;
   me->im.pvt->CreateInstance = xModule_CreateInstance;
   me->im.pvt->FreeResources  = xModule_FreeResources;

   me->uRef = 1; /* new object */
}


/**
  || Function
  || --------
  || static int xModule_New(IShell *piShell, IModule **pp);
  ||
  || Description
  || -----------
  || creates an xModule, which is the implementation class for 
  || IModule for this Applet
  ||
  || Parameters
  || ----------
  || IShell *piShell: BREW shell interface pointer
  || IModule **pp: [out] where a freshly created xModule/IModule pointer
  ||                goes
  ||
  || Returns
  || -------
  || SUCCESS on success
  || ENOMEMORY if an xModule could not be allocated
  ||
  || Remarks
  || -------
  ||
  */
static int xModule_New(IShell *piShell, IModule **pp)
{
   xModule *me = MALLOCREC_EX(xModule, sizeof(IModuleVtbl));

   if ((xModule *)0 == me) {
      *pp = 0;
      return ENOMEMORY;
   }

   *pp = &me->im;
   xModule_CtorZ(me);
   return SUCCESS;

}

/**
  || Function
  || --------
  || static uint32 xModule_AddRef(IModule *p)
  ||
  || Description
  || -----------
  || increments refcount on an xModule (IModule impl)
  ||
  || Parameters
  || ----------
  || IModule *p: the IModule
  ||
  || Returns
  || -------
  || new refcount
  ||
  || Remarks
  || -------
  ||
  */
static uint32 xModule_AddRef(IModule *p)
{
   xModule *me = (xModule *)p;
   return ++me->uRef;
}

/**
  || Function
  || --------
  || static uint32 xModule_Release(IModule *p)
  ||
  || Description
  || -----------
  || decrements refcount on an xModule (IModule impl), may delete
  ||  the xModule
  ||
  || Parameters
  || ----------
  || IModule *p: the IModule
  ||
  || Returns
  || -------
  || new refcount
  ||
  || Remarks
  || -------
  ||
  */
static uint32 xModule_Release(IModule *p)
{
   xModule *me = (xModule *)p;
   uint32 uRef = --me->uRef;

   if (uRef == 0) { 
      /* I'm gone */
      xModule_Delete(me);
   }
   return uRef;
}

/**
  || Function
  || --------
  || static int xModule_CreateInstance(IModule *p, IShell *piShell, 
  ||                                   AEECLSID idClass, void **pp)
  ||
  || Description
  || -----------
  || xModule's CreateInstance implementation
  ||
  || Parameters
  || ----------
  || IModule *p: module 
  || IShell *piShell: BREW shell interface
  || AEECLSID idClass: class id of the desired class
  || void **pp: where to stick the new interface pointer
  ||
  || Returns
  || -------
  || SUCCESS if a webber could be constructed
  || some other AEE error otherwise, see Webber_New()
  ||
  || Remarks
  || -------
  || in this Applet, this function only responds to the webber classid
  ||
  */
static int xModule_CreateInstance(IModule *p, IShell *piShell, 
                                  AEECLSID idClass, void **pp)
{
   if (idClass == AEECLSID_WEBBER) {
      return Webber_New(p,piShell,(IApplet **)pp);
   }
   
   *pp = 0;
   return ECLASSNOTSUPPORT; /* don't understand what you're asking for */
}


/**
  || Function
  || --------
  || static void xModule_FreeResources(IModule *p, IHeap *ph, IFileMgr *pfm)
  ||
  || Description
  || -----------
  || frees resources, hopefully
  ||
  || Parameters
  || ----------
  || IModule *p: module the BREW shell's asking to give a little back..
  || IHeap *ph: the BREW heap interface
  || IFileMgr *pfm: file manager interface...
  ||
  || Returns
  || -------
  || nothing
  ||
  || Remarks
  || -------
  || this function doesn't do anything, since xModule doesn't really have
  || any application smarts...
  ||
  */
static void xModule_FreeResources(IModule *p, IHeap *ph, IFileMgr *pfm)
{
   (void)p,(void)ph,(void)pfm;
}

