#============================================================================
#  Name:
#    webber_thumb.mak
#
#  Description:
#    Makefile to build the webber downloadable module with ADS 1.01
#
#   The following nmake targets are available in this makefile:
#
#     all           - make .elf and .mod image files (default)
#     clean         - delete object directory and image files
#     filename.o    - make object file
#
#   The above targets can be made with the following command:
#
#     nmake /f webber_thumb.mak [target]
#
# Assumptions:
#   1. The ARM ADS tools are installed in $(ARMHOME)
#   2. The BREW sdk source files AEEAppGen.c and AEEModGen.c are installed
#      in $(BREWSDKDIR)/src
#   3. The BREW sdk include files are installed in $(BREWSDKDIR)/inc
#   4. Optionally the AEE include files may be placed in $(BREWAEEDIR) instead
#      of $(BREWSDKDIR)/inc if BREWAEEDIR is suitably defined
#   5. The variable TARGET below has be set to the proper base name of the
#      module being built
#
#        Copyright  2000-2001 QUALCOMM Incorporated.
#               All Rights Reserved.
#            QUALCOMM Proprietary/GTDR
#
#----------------------------------------------------------------------------
#============================================================================

#----------------------------------------------------------------------------
# Target file name and type and path definitions
#----------------------------------------------------------------------------

TARGET     = webber          # Target name for output files and object dir

#-----------------------------------------------------------------------------
# BREW SDK location
#
# change this to location of the BREW sdk directory 
#  (e.g.Program Files/BREW v1.1)
#-----------------------------------------------------------------------------

BREWDIR     = ..\..

#----------------------------------------------------------------------------
# Software tool and environment definitions
#----------------------------------------------------------------------------

!IFNDEF ARMHOME
!ERROR ARMHOME must be defined as the ARM installation dir with Bin, Include & Lib subdirs
!ENDIF
#ARMHOME = C:\Program Files\ARM\ADSv1_1  	# ARM ADS home directory

ARMBIN = $(ARMHOME)\Bin  	# ARM ADS application directory
ARMINC = $(ARMHOME)\Include	# ARM ADS include file directory
ARMLIB = $(ARMHOME)\Lib		# ARM ADS library directory

ARMCC   = "$(ARMBIN)\armcc"     # ARM ADS 32-bit ISA C compiler
ARMCPP  = "$(ARMBIN)\armcpp"    # ARM ADS 32-bit ISA C++ compiler
TCC     = "$(ARMBIN)\tcc"       # ARM ADS 16-bit ISA (thumb) C compiler
TCPP    = "$(ARMBIN)\tcpp"      # ARM ADS 16-bit ISA (thumb) C++ compiler
LD      = "$(ARMBIN)\armlink"   # ARM ADS linker
FROMELF = "$(ARMBIN)\fromelf"   # ARM ADS utility to create bin file from image

!IF "$(MODE)" == "arm"
!MESSAGE building "$(TARGET)" in 32-bit ISA (ARM mode)

CC             = $(ARMCC)
FLAT_BREW      =
THUMBWRAP_OBJS = 
MODLOAD_ENTRY  = -first AEEMod_Load

!ELSE
!MESSAGE building "$(TARGET)" in 16-bit ISA (Thumb mode)

CC             = $(TCC)
FLAT_BREW      = -DFLAT_BREW
THUMBWRAP_OBJS = runtime.o modulert.l
MODLOAD_ENTRY  = 

!ENDIF

#----------------------------------------------------------------------------
# Target compile time symbol definitions
#----------------------------------------------------------------------------

DEFINES = -DDYNAMIC_APP $(FLAT_BREW)

#----------------------------------------------------------------------------
# Processor architecture options
#----------------------------------------------------------------------------

CPU = -littleend -cpu ARM7TDMI # ARM7TDMI target processor

#----------------------------------------------------------------------------
# ARM Procedure Call Standard (APCS) options
#----------------------------------------------------------------------------

APCS = -apcs /noswst/inter/ropi

#----------------------------------------------------------------------------
# C Compiler options
#----------------------------------------------------------------------------

COPT = -zi4 -zo -Wbns+a -fa

#----------------------------------------------------------------------------
# Compiler/assembler debug options
#----------------------------------------------------------------------------

#DBG = -g                        # Enable debug
DBG =                       	# No debug

#----------------------------------------------------------------------------
# Compiler optimization options
#----------------------------------------------------------------------------

OPT = -Ospace -O2               # Full compiler optimization for space
#OPT = -O0			# No optimization at all

#----------------------------------------------------------------------------
# Include file search path options
#----------------------------------------------------------------------------

INC = -I$(BREWDIR)\inc

#----------------------------------------------------------------------------
# Compiler flag definitions
#----------------------------------------------------------------------------

CFLAGS = $(CPU) $(APCS) $(COPT) $(DBG) $(OPT) $(DEFINES) $(INC)

#----------------------------------------------------------------------------
# Linker options
#----------------------------------------------------------------------------

LFLAGS = -remove -ropi -strict $(MODLOAD_ENTRY)

#----------------------------------------------------------------------------
#  define what goes into target
#----------------------------------------------------------------------------

TARGET_OBJS =  $(TARGET).o \
               $(THUMBWRAP_OBJS) \
               
TARGET_LIBS = 

#----------------------------------------------------------------------------
# Default target
#----------------------------------------------------------------------------

all : $(TARGET).mod  # the target app

#----------------------------------------------------------------------------
# Clean target
#----------------------------------------------------------------------------

# The object subdirectory, target image file, and target hex file are deleted.

clean :
        @echo ---------------------------------------------------------------
        @echo CLEAN
        -del /f $(TARGET).o
        -del /f $(TARGET).elf
        -del /f $(TARGET).mod
        @echo ---------------------------------------------------------------

#============================================================================
#                           DEFAULT SUFFIX RULES
#============================================================================

# The following are the default suffix rules used to compile all objects that
# are not specifically included in one of the module specific rules defined
# in the next section.

# The following macros are used to specify the output object file, MSG_FILE
# symbol definition and input source file on the compile line in the rules
# defined below.

.SUFFIXES :
.SUFFIXES : .o .c .cpp

#--------------------------------------------------------------------------
# C code inference rules
#----------------------------------------------------------------------------

.c.o:
        @echo ---------------------------------------------------------------
        @echo OBJECT $<
        $(CC) -ansic -c $(CFLAGS) -o $@ $<
        @echo ---------------------------------------------------------------

.cpp.o:
        @echo ---------------------------------------------------------------
        @echo OBJECT $<
        $(CPP) -cpp -c $(CFLAGS) -o $@ $<
        @echo ---------------------------------------------------------------

#============================================================================
#                           MODULE SPECIFIC RULES
#============================================================================


#----------------------------------------------------------------------------
# Target rules
#----------------------------------------------------------------------------

$(TARGET).mod : $(TARGET).elf
        @echo ---------------------------------------------------------------
        @echo TARGET $@
        $(FROMELF) -bin -o $@ $(TARGET).elf
!IF "$(MODE)" != "arm"
        perl zeropadbin.pl $@
!ENDIF


$(TARGET).elf : $(TARGET_OBJS)
        @echo ---------------------------------------------------------------
        @echo TARGET $@
        $(LD) -o $@ $(LFLAGS) $(TARGET_OBJS) $(TARGET_LIBS) -first AEEMod_Load

# ------------
# DEPENDENCIES

$(TARGET).o : $(TARGET).c
